#ifndef CAMERA3D_H
#define CAMERA3D_H

#include "Rectangle.h"
#include "Point3D.h"
#include "Matrix.h"


namespace Basic3D
{
    class Camera3D
    {
        public:
            Camera3D( Rectangle vp = Rectangle( 0.0, 0.0, 0.0, 0.0 ),
                      Rectangle window = Rectangle( -0.5, -0.5, 1.0, 1.0 ),
                      Vector3D vrp = Vector3D( 0.0, 0.0, -1.0 ),
                      Vector3D vpn = Vector3D( 0.0, 0.0, -1.0 ),
                      Vector3D vup = Vector3D( 0.0, 1.0, 0.0 ),
                      Vector3D eye = Vector3D( 0.0, 0.0, 1.5 ),
                      double fp = 0.5,
                      double bp = 1.5 )
                      : m_vp( vp ), m_window( window ),
                        m_vrp( vrp ), m_vpn( vpn ), m_vup( vup ), m_eye( eye ),
                        m_fp( fp ), m_bp( bp ), m_zoom ( 1.0 ) {}

            Matrix createViewMatrix();
            Matrix getViewMatrix() const { return m_view; }

            void setViewPort( Rectangle vp ) { m_vp = vp; }
            void setZoomLevel( double zoom ) { m_zoom = zoom > 0.0 ? zoom : 1.0; }
            void setEye( Vector3D eye ) { m_eye = eye; }

        protected:
        private:
            Rectangle   m_vp;           //viewport
            Rectangle   m_window;       //screen
            Vector3D    m_vrp;          //view reference point
            Vector3D    m_vpn;          //viewport normal vector
            Vector3D    m_vup;          //view up vector
            Vector3D    m_eye;          //eye position

            double      m_fp;           //todo
            double      m_bp;           //todo

            Matrix      m_view;
            double      m_zoom;

    };
}


#endif // CAMERA3D_H
