#ifndef TABLEEDITOR3D_H
#define TABLEEDITOR3D_H


#include "Surface3D.h"
#include "Camera3D.h"
#include "TableEditor3DDefs.h"
#include "../vemsgauge/GroupMember.h"
#include "../common/VemsMatrix.h"
#include "../common/types/Int2StringHash.h"
#include "../common/types/DoubleArray.h"
#include "../events/VEMSTableEditorEvents.h"
#include "../bcWidgets/BitmapCanvasParentBase.h"

//#define TABLE_EDITOR_3D_TEST

WX_DECLARE_OBJARRAY( wxColour, ArrayOfColours );

class bcLabel;
class FocusParentSinkerBase;
class GaugeGroupManager;
class GaugeProperties;
class SelectableWidgetBase;
class TextDesc;
class wxHotKeyManager;

using Basic3D::Camera3D;


class TableEditor3D : public wxControl, public GroupMember, public BitmapCanvasParentBase
{
    DECLARE_DYNAMIC_CLASS(TableEditor3D);

    public:
        TableEditor3D() {};
        TableEditor3D( wxWindow* parent, GaugeGroupManager * ggm, GaugeProperties &prop);
        virtual ~TableEditor3D();

        void        setDimension( size_t rows, size_t cols );
        void        setMarkedPosition( double x, double y );
        void        setTargetPosition( double z );
        void        setSelectedColor( wxColor c ) { m_selectedColor = c; }
        void        setMarkedColor( wxColor c ) { m_markedColor = c; }
        void        setRowLabelValue( int row, double value );
        void        setColLabelValue( int col, double value );
        void        setCellValue( int row, int col, double value );
        void        setCellsMinValue( double min );
        void        setCellsMaxValue( double max );
        void        setSmallStepValue( double smallStep ) { m_smallStep = smallStep; }
        void        setLargeStepValue( double largeStep ) { m_largeStep = largeStep; }
        int         GetRowLabelValue( int row );
        int         GetColLabelValue( int col );
        double      getCellValue( int row, int col );
        int         getNumberCols();
        int         getNumberRows();
        void        useTargetMarker( bool use ) { m_bUseTargetMarker = use; }
        void        SetTables( const wxArrayString& tables ) { m_tables = tables; }
        void        SetTableName( const wxString& tablename ) { m_tableName = tablename; }
        void        setRowHeaderMinValue( double v ) { m_rowHeaderMinValue = v; }
        void        setRowHeaderMaxValue( double v ) { m_rowHeaderMaxValue = v; }
        void        setColHeaderMinValue( double v ) { m_colHeaderMinValue = v; }
        void        setColHeaderMaxValue( double v ) { m_colHeaderMaxValue = v; }
        void        moveCursorToPosition( const wxPoint& position );
        void        setRowNumberDigits( size_t digits );
        void        setColNumberDigits( size_t digits );
        void        setTableNumberDigits( size_t digits );

        void        loadFromVemsMatrix( const VemsMatrix<double>& m );
        void        loadRowHeaderFromVemsMatrix( const VemsMatrix<double>& m );
        void        loadColHeaderFromVemsMatrix( const VemsMatrix<double>& m );

        const VemsMatrix<double>*  GetRowLabels() { return &m_rowHeaders; }
        const VemsMatrix<double>*  GetColLabels() { return &m_columnHeaders; }

        void AutoFollowLoad();

        void SetFocusParentSinker(FocusParentSinkerBase* fps) { m_focusSinker = fps; }

    private:
        //enum CursorMovingMode { CURSOR_MOVING_MODE_RELATIVE, CURSOR_MOVING_MODE_ABSOLUTE };
        enum CursorMove
        {
            CURSOR_MOVE_LEFT,
            CURSOR_MOVE_RIGTH,
            CURSOR_MOVE_UP,
            CURSOR_MOVE_DOWN,
        };

        void OnMouseLeftClick(wxMouseEvent& event);
        SelectableWidgetBase* GetSelectableWidgetBase();

        wxWindow* GetSelectFaceParent();
        void OnMouseLeftUp(wxMouseEvent& evt);
        void OnMouseMove(wxMouseEvent& evt);

        void OnSetFocus(wxFocusEvent &evt);
        void OnKillFocus(wxFocusEvent &evt);

    private:
        VemsMatrix<double>      m_table;
        VemsMatrix<double>      m_columnHeaders;
        VemsMatrix<double>      m_rowHeaders;
        VemsMatrix<uint8_t>     m_highlights;

        double                  m_tableMinValue;
        double                  m_tableMaxValue;

        Surface3D*              m_pSurface;
        wxBitmap*               m_pBackBuffer;
        bool                    m_needUpdateBackBuffer;

        Camera3D                m_camera;
        double                  m_xAngle;
        double                  m_yAngle;
        size_t                  m_rows;
        size_t                  m_cols;
        bool                    m_bHighlighted;         // more cell highlighted
        bool                    m_bShowHelp;

        Axis3D                  m_axis;

        //selected point params
        wxColor         m_selectedColor;
        wxColor         m_highlightedColor;
        size_t          m_selectedCol;
        size_t          m_selectedRow;
        int             m_highlightOriginRow;
        int             m_highlightOriginCol;

        //marked point params
        wxColor         m_markedColor;
        Point3D         m_markedPoint;
        double          m_markedX;
        double          m_markedY;
        double          m_markedZ;


        double          m_smallStep;
        double          m_largeStep;


        Point3D         m_min;      /// minimum coordinates of axes ( 0.0 - 1.0 )
        Point3D         m_max;      /// maximum coordinates of axes ( 0.0 - 1.0 )
        Point3D         m_scale;    /// scales of axes
        double          m_zoom;

        ///matrices
        Matrix          m_mTranslateToCenter;
        Matrix          m_transformationMatrix;

        // target point params
        Point3D         m_targetPoint;
        bool            m_bUseTargetMarker;

        /// bounds of table ( from ini )

        double          m_rowHeaderMinValue;
        double          m_rowHeaderMaxValue;
        double          m_colHeaderMinValue;
        double          m_colHeaderMaxValue;

        /// bounds of y-axis ( from user )
        double          m_userMinValue;
        double          m_userMaxValue;

        size_t          m_rowNumberDigits;
        size_t          m_colNumberDigits;
        size_t          m_tableNumberDigits;

        wxString        m_rowNumberFormat;
        wxString        m_colNumberFormat;
        wxString        m_tableNumberFormat;

        wxArrayString   m_tables;           // available tables from ini
        Int2StringHash  m_menuIDtoTableName;
        wxString        m_tableName;

        wxHotKeyManager*    m_hotkeyMgr;
        wxBitmap*           m_helpPanelBmp;
        bool                m_isResized;
        Vector3D            m_cameraEye;

        wxPoint             m_delta;

#ifdef TABLE_EDITOR_3D_TEST
        double          m_markerStepX;
        double          m_markerStepY;
#endif
        int             m_cursorMovingMode;

        bcLabel*        m_selectedLabel;
        bcLabel*        m_markedLabel;
        bcLabel*        m_markedLoadLabel;

        TextDesc*       m_selectedTextDesc;
        TextDesc*       m_markedTextDesc;
        TextDesc*       m_markedLoadTextDesc;

        FocusParentSinkerBase* m_focusSinker;

        int             m_selectedLabelAlign;
        int             m_markedLabelAlign;

        int             m_labelFontHeight;
        int             m_padding;
        int             m_labelTextPadding;
        int             m_maxLabelWidth;
        bool            m_areLabelsReady;
        bool            m_isAutoFollowLoad;

        void            initLabels();
        void            drawLabels();
        void            drawSelectedLabel();
        void            drawMarkedLabel();
        void            drawLoadLabel();
        void            calculateMaxLabelWidth();
        void            alignSingleSelected();
        void            alignSingleMarked();
        void            alignAllLabels();
        void            alignLabels();

        int             getHelpAlign();

        size_t          index( size_t row, size_t col );
        bool            calculateMarkedPosition();
        void            interpolate( double x, double y );
        int             findFirstGreater( double d, VemsMatrix<double>& a, bool searchFromEnd = false );
        void            increaseCellValue( int row, int col, double inc );
        void            increaseSelectedCells( double inc );
        void            moveSelectedAbsolute( CursorMove dir );
        void            moveSelectedRelative( CursorMove dir );
        void            moveSelected( CursorMove dir );
        void            moveAndHighlightSelected( CursorMove dir );
        void            moveAndUnhighlight( CursorMove dir );
        void            highlightAll();
        void            setDefault();
        void            resetHighlightOrigin();
        void            unhighlightAllCells();
        void            highlightArea( int startRow, int startCol, int endRow, int endCol );
        void            interpolateHighlighted();
        void            setMaxRangeY( double y );
        void            setMinRangeY( double y );
        void            showDialogRangeYMin();
        void            showDialogRangeYMax();
        void            sendSingleCellChangedEvent( int row, int col, double d );
        void            sendVEMSTableEditorEvent( TableEditorEvent::EventType type, int row = 0, int col = 0, double d = 0.0 );
        void            loadParams();
        void            saveParams();
        void            SelectLoad();
        void            displayValues(  wxDC& dc,
                                        const wxPoint& pos,
                                        const wxString& name,
                                        const DoubleArray& values,
                                        const ArrayOfColours& colors );

        void            normalizeIndex(int& idx, int arraySize );
        Point4D         createSurfacePoint( size_t row, size_t col );
        void            updateBackBuffer( wxDC& dc, const wxSize& size );
        void            updateSurface( wxDC& dc, const wxSize& size );
        void            updateHighlightedPoints( wxDC& dc, const wxSize& size );
        void            updateLoadMarker( wxDC& dc, const wxSize& size );
        void            updateAxes( wxDC& dc, const wxSize& size );
        void            renderText( wxDC& dc );
        void            renderHelp( wxDC& dc );
        void            setNeedUpdateBackBuffer();
        void            resetNeedUpdateBackBuffer();
        void            setRangeY();

        void            OnSize( wxSizeEvent& event );
        void            OnPaint( wxPaintEvent& event );
        void            OnKeyDown( wxKeyEvent& event );
        void            OnMouseRightClick( wxMouseEvent& event );
        void            OnPopupMenuChangeTable( wxCommandEvent& event );
        void            OnPopupMenu( wxCommandEvent& event );
        void            drawHelpPanel( wxDC& dc );
        double          getHelpTextHeight( int clientHeight, size_t lineCount, double margin, double lineSpacing );
        void            createHelpPanel( int clientHeight );
        void            moveCameraLeft();
        void            moveCameraRight();
        void            moveCameraUp();
        void            moveCameraDown();
        void            resetView();
        bool            isGroupMember();

		DECLARE_EVENT_TABLE();
};


#endif // TABLEEDITOR3D_H
