#include "AudioController.h"

#include <portaudiocpp/Exception.hxx>

#include "ringbuffer/RingBuffer.h"
#include "ringbuffer/RingBufferSync.h"
#include "ringbuffer/SyncObj.h"
#include "ringbuffer/ThreadControllerAudio.h"

#include "ringbuffer/AudioPlayRecordThread.h"
#include "ringbuffer/SignalProducerThread.h"
#include "ringbuffer/GUIConsumerThread.h"

#include "rtdatamanager/RTDataMgrFacade.h"
#include "rtdatamanager/RTDataInputSignalgen.h"

#include "logger.h"
#include "UnicodeHelper.h"

#include "configmanager/ConfigMgrAudio.h"
#include "configmanager/ConfigHelper2.h"

SignalGeneratorCallBack::SignalGeneratorCallBack(RingBufferSync* ringBufferSync){
        m_ringBufferSync = ringBufferSync;
        tableSize_ = 200;
        leftPhase_ = 0;
        rightPhase_ = 0;
        testGenerateSine();
}

SignalGeneratorCallBack::~SignalGeneratorCallBack()
{
    delete[] table_;

    System& paSystem = System::instance();
    paSystem.terminate();

}

/// only play
int SignalGeneratorCallBack::play(const void *inputBuffer, void *outputBuffer, unsigned long framesPerBuffer,
    const PaStreamCallbackTimeInfo *timeInfo, PaStreamCallbackFlags statusFlags)
{
    assert(outputBuffer != NULL);
    assert(m_ringBufferSync != NULL);

//		testSineInterleaved(outputBuffer, framesPerBuffer);

//        m_ringBufferSync->waitRingBufferNotEmpty();
    writeOutput(outputBuffer,framesPerBuffer);
    m_ringBufferSync->nextTailAndBroadcast();

    return paContinue;
}

/// only record
int SignalGeneratorCallBack::record(const void *inputBuffer, void *outputBuffer, unsigned long framesPerBuffer,
    const PaStreamCallbackTimeInfo *timeInfo, PaStreamCallbackFlags statusFlags)
{
    /// TODO: instead of ussing ringbuffer send event
    /// copy inputBuffer into event obj;
    /// send EventObj;
    //m_ringBufferSync->waitRingBufferNotFull();
    readInput(inputBuffer,framesPerBuffer);
    m_ringBufferSync->nextHeadAndBroadcast();
    return paContinue;
}

/// interleaved
void SignalGeneratorCallBack::writeOutput(void * outputBuffer,unsigned long framesPerBuffer)
{
    short *out =  static_cast<short *>(outputBuffer);
    short * buf = (short *)(m_ringBufferSync->m_ringBuffer->getTailPointer());
    //Logger::logf(LOG_MSG, _T("SignalGeneratorCallBack::writeOutput(): %i, %p, %i"), framesPerBuffer, buf, buf[0]);
    //memcpy(out, buf, 4*framesPerBuffer);

    for (unsigned int i = 0; i < framesPerBuffer; ++i)
    {
        out[2*i] = buf[2*i];
        out[2*i+1] = buf[2*i+1];
    }
};

void SignalGeneratorCallBack::readInput(const void * inputBuffer,unsigned long framesPerBuffer)
{
    const short* input =  static_cast<const short *>(inputBuffer);
    short* buf = (short *)(m_ringBufferSync->m_ringBuffer->getHeadPointer());

    for (unsigned int i = 0; i < framesPerBuffer; ++i)
    {
        buf[2*i] = input[2*i];
        buf[2*i+1] = input[2*i+1];
    }

};
/// Only Test functions:
void SignalGeneratorCallBack::nextPhase()
{
    leftPhase_ += 1;
    if (leftPhase_ >= tableSize_)
        leftPhase_ -= tableSize_;

    rightPhase_ += 3;
    if (rightPhase_ >= tableSize_)
        rightPhase_ -= tableSize_;
}

void SignalGeneratorCallBack::testGenerateSine()
{
    const double MY_PI = 3.14159265;
    table_ = new short[tableSize_];
    for (int i = 0; i < tableSize_; ++i){
        table_[i] = (short)( 32000.0 * (float)sin(((double)i/(double)tableSize_)*MY_PI*2.) );
        //table_[i] = 0.125f * (float)sin(((double)i/(double)tableSize)*PI*2.);
    }
}

/// interlaved
void SignalGeneratorCallBack::testSineInterleaved(void * outputBuffer,unsigned long framesPerBuffer)
{
    short *out = static_cast<short *>(outputBuffer);
    for (unsigned int i = 0; i < framesPerBuffer; ++i)
    {
        out[2*i] = table_[leftPhase_];
        out[2*i+1] = table_[rightPhase_];

        nextPhase();
    }
}

/// non interlaved;
void SignalGeneratorCallBack::testSineNonInterleaved(void * outputBuffer,unsigned long framesPerBuffer)
{
    short **out = static_cast<short **>(outputBuffer);
    for (unsigned int i = 0; i < framesPerBuffer; ++i)
    {
        out[0][i] = table_[leftPhase_];
        out[1][i] = table_[rightPhase_];

        nextPhase();
    }
}

/*
     /// non interleaved solution
     /// inputParameters.setSampleFormat(PA_SAMPLE_TYPE, false);
     void writeOutput(void * outputBuffer,unsigned long framesPerBuffer){
        short **out = static_cast<short **>(outputBuffer);
        short * buf = (short*) m_ringBufferSync->m_ringBuffer->getTailPointer();

		for (unsigned int i = 0; i < framesPerBuffer; ++i)
		{
			out[0][i] = buf[i];
			out[1][i] = buf[i+1];
		}

     };
*/


/////////////////////////////////////////////////////////////////////////////////////////////
AudioController::AudioController()
{
    //ctor
    //m_audioPlayRecordThread = NULL;
    //m_signalgenThread = NULL;
//    m_isStreamOK = false;

    m_signalProcessMgr = NULL;

    ConfigMgrAudio cfgMgr( ConfigHelper2::getConfig() );
    cfgMgr.load(&m_audioConfig);

    m_dataInput = NULL;
    listAvailableSampleRate();
    //setSampleRate (44100);

    initBuffersAndSyncObjs();
}

AudioController::~AudioController()
{

    //dtor
    stop();

    delete m_SignalGeneratorCallBack;
    delete m_threadCtrlSignalgen;
    delete m_ringBufferSync;

/*
Logger::logf(LOG_ERROR, _T("DTOR AudioController: DOR START "));

    ///m_guiThread->Delete();

    if( m_audioPlayRecordThread && m_audioPlayRecordThread->IsRunning() ){
        m_audioPlayRecordThread->stop();
        //m_audioPlayRecordThread->Exit();
        m_audioPlayRecordThread->Delete();
    }
Logger::logf(LOG_ERROR, _T("DTOR AudioController: DOR 10"));
    if( m_signalgenThread && m_signalgenThread->IsRunning() ){
        Logger::logf(LOG_MSG, _T("AudioController::m_signalgenThread() is running!") );
         m_signalgenThread->stop();
         m_signalgenThread->Delete();
    }
Logger::logf(LOG_ERROR, _T("DTOR AudioController: 20"));

    m_audioPlayRecordThread = NULL;
    m_signalgenThread = NULL;

    delete m_inputBuffer;
    delete m_outputBuffer;
    delete m_syncObjPlay;

    m_inputBuffer = NULL;
    m_outputBuffer = NULL;
    m_syncObjPlay = NULL;

    Logger::logf(LOG_ERROR, _T("DTOR AudioController: DOR END "));
*/

}

void AudioController::initStream()
{
        Logger::logf(LOG_MSG,_T("AudioController::initStream()"));

        initStream(getInputDevice(), getOutputDevice() );
}

void AudioController::initStream(int inputDeviceID, int outputDeviceID)
{
    Logger::logf(LOG_MSG,_T("AudioController::initStream(id,id)"));

    //stop();
    //if ( m_isStreamOK==true ) return;
    if (m_stream.isOpen()){

        Logger::logf(LOG_MSG,_T("AudioController::initStream isopen"));
        if (!m_stream.isStopped()){
            Logger::logf(LOG_MSG,_T("AudioController::initStream not isstopped (running)"));

            m_stream.stop();
        }
        Logger::logf(LOG_MSG,_T("AudioController::initStream stoped"));

        m_stream.close();
    }

    Logger::logf(LOG_MSG,_T("AudioController::initStream 10"));

    try{
        Logger::logf(LOG_MSG, _T("AudioController::initStreamParam()") );
        //portaudio::AutoSystem autoSys;
        System& paSystem = System::instance();
        Logger::logf(LOG_MSG,_T("AudioController::initStream 20"));
        //serialErrorMsgDialog("!!!!!!!!!!!!!!");
        DirectionSpecificStreamParameters inputParameters;// = DirectionSpecificStreamParameters ();

        if(inputDeviceID == -1){
            inputParameters.setDevice(paSystem.defaultInputDevice() );
            //char* defaultname = (char *)(paSystem.defaultInputDevice()).name();
            Logger::logf(LOG_MSG,_T("AudioController::initStream defaultname"));
        }else{
            inputParameters.setDevice(paSystem.deviceByIndex(inputDeviceID) );
            //char* devicename = (char*)paSystem.deviceByIndex(inputDeviceID).name();
            Logger::logf(LOG_MSG,_T("AudioController::initStream"));
        }
        Logger::logf(LOG_MSG,_T("AudioController::initStream 30"));
        // inputParameters.setDevice(paSystem.deviceByIndex(1) );
          //  inputParameters.setDevice(paSystem.defaultInputDevice() );
        /*
        printf( "Input device # %d.\n", inputParameters.device );
        printf( "Input LL: %g s\n", Pa_GetDeviceInfo( inputParameters.device )->defaultLowInputLatency );
        printf( "Input HL: %g s\n", Pa_GetDeviceInfo( inputParameters.device )->defaultHighInputLatency );
        */
        inputParameters.setNumChannels(NUM_CHANNELS);
        Logger::logf(LOG_MSG,_T("AudioController::initStream 40"));
        /// non interlaved
        //inputParameters.setSampleFormat(PA_SAMPLE_TYPE, false);
        /// interlaved
        inputParameters.setSampleFormat(PA_SAMPLE_TYPE, true);
        inputParameters.setSuggestedLatency(inputParameters.device().defaultHighInputLatency () );
        //Pa_GetDeviceInfo( inputParameters.device )->defaultHighInputLatency ;
        inputParameters.setHostApiSpecificStreamInfo(NULL);


        DirectionSpecificStreamParameters outputParameters = DirectionSpecificStreamParameters::null ();

        if(outputDeviceID == -1){
            outputParameters.setDevice(paSystem.defaultOutputDevice() );
            Logger::logf(LOG_MSG,_T("AudioController::initStream OutputDevice defaultname"));
        }else{
            outputParameters.setDevice(paSystem.deviceByIndex(outputDeviceID) );
            Logger::logf(LOG_MSG,_T("AudioController::initStream OutputDevice devicename"));

        }
     //outputParameters.setDevice(paSystem.deviceByIndex(1));

    //outputParameters.setDevice(paSystem.defaultOutputDevice() );
/*
    printf( "Output device # %d.\n", outputParameters.device );
    printf( "Output LL: %g s\n", Pa_GetDeviceInfo( outputParameters.device )->defaultLowOutputLatency );
    printf( "Output HL: %g s\n", Pa_GetDeviceInfo( outputParameters.device )->defaultHighOutputLatency );
 */
        outputParameters.setNumChannels(NUM_CHANNELS);
        outputParameters.setSampleFormat(PA_SAMPLE_TYPE, true);
        outputParameters.setSuggestedLatency(outputParameters.device().defaultHighInputLatency ());
    // = Pa_GetDeviceInfo( outputParameters.device )->defaultHighOutputLatency;
        outputParameters.setHostApiSpecificStreamInfo(NULL);

   // StreamParameters m_paStreamParam = StreamParameters();
        //m_paStreamParam.setInputParameters(inputParameters);
        m_paStreamParam.setInputParameters(portaudio::DirectionSpecificStreamParameters::null() );
//        m_paStreamParam.setOutputParameters(portaudio::DirectionSpecificStreamParameters::null() );
        m_paStreamParam.setOutputParameters(outputParameters);
        m_paStreamParam.setSampleRate(m_audioConfig.m_sampleRate);
        m_paStreamParam.setFramesPerBuffer(FRAMES_PER_BUFFER);
        Logger::logf(LOG_MSG,_T("AudioController::initStream all set sample rate= %i"), m_audioConfig.m_sampleRate);

        m_paStreamParam.clearFlags();
        Logger::logf(LOG_MSG,_T("AudioController::initStream flags clear"));

        //m_paStreamParam.setFlag(paClipOff);

        bool isStreamOK = m_paStreamParam.isSupported();
        Logger::logf(LOG_MSG, _T("AudioController::initStream(): stream is supported: %i"), isStreamOK );

        if ( isStreamOK==true ){
            m_stream.open(m_paStreamParam, *m_SignalGeneratorCallBack, &SignalGeneratorCallBack::play);
            Logger::logf(LOG_MSG,_T("AudioController::initStream stream opened"));

        }

    }
	catch (const portaudio::PaException &e)
	{
	    fprintf(stderr, "stderr AudioController::initStream(): Portaudio exception: %s", e.paErrorText());
	    fprintf(stdout, "stdout AudioController::initStream(): Portaudio exception: %s", e.paErrorText());
	    Logger::logf(LOG_MSG, _T("X AudioController::initStream(): Portaudio exception: %s"), e.paErrorText() );

	}
	catch (const portaudio::PaCppException &e)
	{
	    Logger::logf(LOG_MSG, _T("AudioController::initStream(): PortaudioCpp exception: %s"), e.what() );
	}
	catch (const std::exception &e)
	{
	    Logger::logf(LOG_MSG, _T("AudioController::initStream(): Generic exception: %s"), e.what() );
	}
	catch (...)
	{
        Logger::logf(LOG_MSG, _T("AudioController::initStream(): Unknown exception!"));
	}

    Logger::logf(LOG_MSG, _T("AudioController::initStream(): END!"));
}

void AudioController::processPulseTrainPacket(SIPRPacket &pulsePacket)
{
    //Logger::logf(LOG_MSG, _T("AudioController:processPulseTrainPacket"));
    m_dataInput->processPulseTrainPacket(pulsePacket);
}

void AudioController::initSignalgen(const wxString& type, int rpm, double amplitude, int sampleRate)
{
    //SignalGen* signalGen = new SignalGen();
    m_audioConfig.m_sampleRate = sampleRate;

    if ( m_dataInput != NULL ){
        Logger::logf(LOG_MSG, _T("AudioController: trigger type: %s"),type.GetData());
        m_dataInput->setTriggerType(type);
    }

    const char* typeC = UnicodeHelper::U2C(type);

    Logger::logf(LOG_MSG, _T("AudioController::initSignalgen:  rpm: %d"), rpm);
    Logger::logf(LOG_MSG, _T("AudioController::initSignalgen: amp: %f"), amplitude);
    Logger::logf(LOG_MSG, _T("AudioController::initSignalgen: wxtype: ")+ type);
    Logger::logf(LOG_MSG, _T("AudioController::initSignalgen: type: %s"), type.c_str() );

    const char * typeCstr = UnicodeHelper::U2C(type);
    m_signalGen.setTriggerType(typeCstr);
    m_signalGen.initBlock(rpm, amplitude/1000, m_audioConfig.m_sampleRate);
    delete typeCstr;
    delete typeC;
}

void AudioController::initBuffersAndSyncObjs()
{
    m_syncObjRecord = new SyncObj();
    m_inputBuffer  = new RingBuffer(16,  4 * FRAMES_PER_BUFFER);

    //m_ringBufferSync = new RingBufferSync(m_outputBuffer, m_syncObjPlay);
    m_ringBufferSync = new RingBufferSync(16, 4 * FRAMES_PER_BUFFER);

    m_SignalGeneratorCallBack =  new SignalGeneratorCallBack(m_ringBufferSync);
    m_threadCtrlSignalgen = new ThreadControllerAudio(m_ringBufferSync, &m_signalGen);
    //m_threadCtrlSignalgen->start();
}

void AudioController::initViewerRefreshThread()
{
    m_signalProcessMgr = new RTDataMgrFacade();

    m_dataInput = new RTDataInputSignalgen(m_inputBuffer, NULL, m_syncObjRecord );
    ((RTDataMgrFacade*)m_signalProcessMgr)->SetInput( m_dataInput );
    m_dataInput->setActive(true);
    m_dataInput->initSignalProcess(); //MUST call after  SetInput()
/*
    m_guiThread = new GUIConsumerThread(m_signalProcessMgr);
    if ( m_guiThread->Create() != wxTHREAD_NO_ERROR ){
        Logger::logf(LOG_MSG, _T("Can't create GUI Thread!"));
    }
    m_guiThread->Run();
*/
}


void AudioController::setAmplitude(double amplitude)
{
    m_signalGen.setAmplitude(amplitude/1000);
}

void AudioController::setRpm(int rpm)
{
    m_signalGen.setRpm(rpm);
}

void AudioController::setDeltaMaxClimb(double deltaMaxClimb)
{
    m_signalGen.setdeltaMaxClimb(deltaMaxClimb);
}

void AudioController::setSecTrigAngle(double d)
{
    m_signalGen.setSecTrigAngle(d);
}

void AudioController::muteSignals( int primarychannel, int secondarychannel )
{
    if ( primarychannel >= 0 )
    {
        m_signalGen.mute_prim_signal( (bool)primarychannel );
    }
    if ( secondarychannel >= 0 )
    {
        m_signalGen.mute_sec_signal( (bool) secondarychannel );
    }
}

void AudioController::start()
{
//    m_outputBuffer->init();
//    m_inputBuffer->init();
//    if( m_audioPlayRecordThread != NULL) return;
    Logger::logf(LOG_MSG, _T("AudioController::start(): START!"));

    if(m_threadCtrlSignalgen->isRunning() ) return;

    Logger::logf(LOG_MSG, _T("AudioController::start(): 10!"));
    if( ! m_stream.isOpen() ){
        initStream();
    }
    if( ! m_stream.isOpen() ){
        Logger::logf(LOG_MSG, _T("AudioController::start(): m_stream not supported!"));
        return;
    }

    Logger::logf(LOG_MSG, _T("AudioController::start(): 20!"));
    try{
        //portaudio::AutoSystem autoSys;
        //System& paSystem = System::instance();

        if(!streamStopped(m_stream) ){

            return;
        }
        m_threadCtrlSignalgen->start();
        wxThread::Sleep(10);
        m_stream.start();
        Logger::logf(LOG_MSG, _T("AudioController::start(): m_stream.start()!"));
    }
    catch (const portaudio::PaException &e)
	{
	    Logger::logf(LOG_MSG, _T("AudioController::start(): Portaudio exception: %d"), e.paError() );
	    m_threadCtrlSignalgen->stop();
	}
	catch (const portaudio::PaCppException &e)
	{
	    Logger::logf(LOG_MSG, _T("AudioController::start(): PortaudioCpp exception: %s"), e.what() );
	     m_threadCtrlSignalgen->stop();
	}
	catch (const std::exception &e)
	{
	    Logger::logf(LOG_MSG, _T("AudioController::start(): Generic exception: %s"), e.what() );
	     m_threadCtrlSignalgen->stop();
	}
	catch (...)
	{
        Logger::logf(LOG_MSG, _T("AudioController::start(): Unknown exception!"));
         m_threadCtrlSignalgen->stop();
	}
     Logger::logf(LOG_MSG, _T("AudioController::start(): END!"));

    //initAudioThread();
/*
    if(m_signalgenThread) m_signalgenThread->setEnable(true);
     wxThread::Sleep(10);
    if(m_audioPlayRecordThread) m_audioPlayRecordThread->setEnable(true);
*/

}

void AudioController::stop()
{
    Logger::logf(LOG_MSG, _T("AudioController::stop(): START!"));
     if( ! m_stream.isOpen() ){
        Logger::logf(LOG_MSG, _T("AudioController::stop(): m_stream not supported!"));
        return;
    }
    try{
        //portaudio::AutoSystem autoSys;
        //System& paSystem = System::instance();
        if(!streamStopped(m_stream) ){
            m_stream.stop();
            m_stream.close();
            //paSystem.terminate();
        }
    }
    catch (const portaudio::PaException &e)
	{
	    Logger::logf(LOG_MSG, _T("AudioController::stop(): Portaudio exception: %s"), e.paErrorText() );
	}
	catch (const portaudio::PaCppException &e)
	{
	    Logger::logf(LOG_MSG, _T("AudioController::stop(): PortaudioCpp exception: %s"), e.what() );
	}
	catch (const std::exception &e)
	{
	    Logger::logf(LOG_MSG, _T("AudioController::stop(): Generic exception: %s"), e.what() );
	}
	catch (...)
	{
        Logger::logf(LOG_MSG, _T("AudioController::start(): Unknown exception!"));
	}
     Logger::logf(LOG_MSG, _T("AudioController::stop(): END!"));


    m_threadCtrlSignalgen->stop();

/*
    if(m_audioPlayRecordThread == NULL) return;

    m_audioPlayRecordThread->stop();
    m_audioPlayRecordThread->Delete();

    m_audioPlayRecordThread = NULL;
*/

/*
    if(m_audioPlayRecordThread) m_audioPlayRecordThread->setEnable(false);
    wxThread::Sleep(50);
    if(m_signalgenThread) m_signalgenThread->setEnable(false);
*/
}

long AudioController::getOutputDevice() const
{
    return m_audioConfig.m_outputDevice;

    //return ConfigHelper::getAudioOutputDevice();
}

void AudioController::setOutputDevice(long device)
{

    //ConfigHelper::setAudioOutputDevice(device);
    m_audioConfig.m_outputDevice = device;
    ConfigMgrAudio cfgMgr( ConfigHelper2::getConfig() );
    cfgMgr.save(&m_audioConfig);

}

long AudioController::getInputDevice() const
{
     return m_audioConfig.m_inputDevice;
}

void AudioController::setInputDevice(long device)
{
    //ConfigHelper::setAudioOutputDevice(device);
    m_audioConfig.m_inputDevice = device;
    ConfigMgrAudio cfgMgr( ConfigHelper2::getConfig() );
    cfgMgr.save(&m_audioConfig);
}

void AudioController::setSampleRate(long sampleRate)
{
    //ConfigHelper::setAudioOutputDevice(device);
    m_audioConfig.m_sampleRate = sampleRate;
    ConfigMgrAudio cfgMgr( ConfigHelper2::getConfig() );
    cfgMgr.save(&m_audioConfig);
}

RTDataMgrBase* AudioController::getRTDataMgrBase()
{
    return m_signalProcessMgr;
}

int AudioController::getSelectedSampleRate() const
{
    for (unsigned int i=0; i < m_availableSampleRateArray->Count(); i++){
        if (m_audioConfig.m_sampleRate == m_availableSampleRateArray->Item(i) )
        return i;
    }
    return 0;
}

wxArrayString AudioController::getAvailableSampleRate() const
{
    wxArrayString rarr;
    for (unsigned int i=0; i<m_availableSampleRateArray->Count(); i++ ){
        wxString sampleItem;
        sampleItem<<m_availableSampleRateArray->Item(i);
        rarr.Add(sampleItem);
    }
    return rarr;

}

int AudioController::GetSampleRate(void) const
{
        return m_audioConfig.m_sampleRate;
}

void AudioController::listAvailableSampleRate()
{
    m_availableSampleRateArray = new IntArray();
    m_availableSampleRateArray->Add(44100);
    m_availableSampleRateArray->Add(48000);
    m_availableSampleRateArray->Add(96000);

}

bool AudioController::streamStopped(portaudio::MemFunCallbackStream<SignalGeneratorCallBack> &stream)
{
    try
    {
        //portaudio::AutoSystem autoSys;
        //System& paSystem = System::instance();
        return stream.isStopped();
    }
    catch (const portaudio::PaException &e)
    {
	    Logger::logf(LOG_MSG, _T("AudioController::streamStopped(): Portaudio exception:"));
	    Logger::logf(LOG_MSG, _T(" %d"), e.paError() );
        return 1;
    }

}
