#ifndef AUDIOCONTROLLER_H
#define AUDIOCONTROLLER_H

#include <portaudiocpp/StreamParameters.hxx>
#include <portaudiocpp/MemFunCallbackStream.hxx>
#include <portaudiocpp/PortAudioCpp.hxx>

#include "signalgen/signalgen.h"
#include "Observable.h"
#include "common/types/IntArray.h"
#include "vemsprotocol/Sipr.h"


//#define SAMPLE_RATE  (48000)
#define FRAMES_PER_BUFFER (1024)
#define NUM_CHANNELS    (2)
#define PA_SAMPLE_TYPE    portaudio::INT16
//#define PA_SAMPLE_TYPE  paInt16
#define SAMPLE_SILENCE  (0)

#include "ringbuffer/RingBuffer.h"
#include "ringbuffer/RingBufferSync.h"
#include "configmanager/AudioConfig.h"
#include "logger.h"

class AudioPlayRecordThread;
class SignalProducerThread;
class GUIConsumerThread;

class RingBuffer;
class SyncObj;
class ProcessBase;

class RTDataMgrBase;
class RTDataMgrFacade;

class RTDataInputSignalgen;

////////////////////////////////
class SignalGeneratorCallBack
{
public:
	SignalGeneratorCallBack(RingBufferSync* ringBufferSync);

	~SignalGeneratorCallBack();

    /// only play
	int play(const void *inputBuffer, void *outputBuffer, unsigned long framesPerBuffer,
		const PaStreamCallbackTimeInfo *timeInfo, PaStreamCallbackFlags statusFlags);

    /// only record
    int record(const void *inputBuffer, void *outputBuffer, unsigned long framesPerBuffer,
		const PaStreamCallbackTimeInfo *timeInfo, PaStreamCallbackFlags statusFlags);
    /// play and record

private:
    RingBufferSync* m_ringBufferSync;
    /// interleaved
    void writeOutput(void * outputBuffer,unsigned long framesPerBuffer);
    void readInput(const void * inputBuffer,unsigned long framesPerBuffer);

    /// Testing
    short  *table_;
	int tableSize_;
	int leftPhase_;
	int rightPhase_;

    void nextPhase();
    void testGenerateSine();
    /// interlaved
     void testSineInterleaved(void * outputBuffer,unsigned long framesPerBuffer);
    /// non interlaved;
    void testSineNonInterleaved(void * outputBuffer,unsigned long framesPerBuffer);
};

//////////////////////////////////
class AudioController //: public Observable
{
    public:
        AudioController();
        virtual ~AudioController();
        void initStream();

        void initSignalgen(const wxString& type, int rpm = 1000, double amplitude = 1.0, int sampleRate = 44100);
        void initViewerRefreshThread();

        void setAmplitude(double amplitude);
        void setRpm(int rpm);
        void setDeltaMaxClimb(double deltaMaxClimb);
        void setSecTrigAngle(double d);

        //
        // -1: do not change, 0: unmute it, 1: mute it
        void muteSignals( int primarychannel = -1, int secondarychannel = -1 );
        void setFluctuation ( uint8_t percent ) { m_signalGen.setFluctuatingScaler( percent );}

        void start();
        void stop();

        int getSelectedSampleRate() const;
        wxArrayString getAvailableSampleRate() const;
        void setSampleRate(long sampleRate);
        int GetSampleRate(void) const;

        long getOutputDevice() const;
        void setOutputDevice(long device);
        long getInputDevice() const;
        void setInputDevice(long device);
        RTDataMgrBase* getRTDataMgrBase();

        void processPulseTrainPacket(SIPRPacket &pulsePacket);

    protected:
    private:
        portaudio::MemFunCallbackStream<SignalGeneratorCallBack> m_stream;
//        bool m_isStreamOK;
        portaudio::StreamParameters m_paStreamParam;
        SignalGen m_signalGen;

        SignalGeneratorCallBack*    m_SignalGeneratorCallBack;
        ProcessBase *               m_threadCtrlSignalgen;
        RingBufferSync*             m_ringBufferSync;

        //AudioPlayRecordThread* m_audioPlayRecordThread;
        //SignalProducerThread* m_signalgenThread;
        //GUIConsumerThread*  m_guiThread;

        RTDataInputSignalgen* m_dataInput;
        RingBuffer* m_inputBuffer;
        SyncObj* m_syncObjRecord;
        RTDataMgrFacade* m_signalProcessMgr;
        IntArray* m_availableSampleRateArray;

        AudioConfig m_audioConfig;

        //const char* m_errorMsg;
        void initStream(int inputDeviceID, int outputDeviceID);
        void listAvailableSampleRate();
        bool streamStopped(portaudio::MemFunCallbackStream<SignalGeneratorCallBack> &stream);


        void initBuffersAndSyncObjs();

        portaudio::AutoSystem autoSys;

};



#endif // AUDIOCONTROLLER_H
