#ifndef CONFIGBASE_H
#define CONFIGBASE_H
#include "common/typedef.h"
#include <wx/wx.h>

class ConfigBase
{
    public:
          enum EntryType
    {
        Type_Unknown,
        Type_String,
        Type_Boolean,
        Type_Integer,
        Type_Float
    };
        ConfigBase();
        virtual ~ConfigBase();

        virtual bool save() = 0;
        virtual bool load() = 0;
        virtual bool flush() = 0;

        virtual void setPath(const wxString& strPath) = 0;
        virtual const wxString& getPath() = 0;

        virtual bool read(const wxString& key, wxString* str) = 0;
        virtual bool read(const wxString& key, wxString* str, const wxString& defVal) = 0;
        virtual wxString read(const wxString& key, const wxString& defVal) = 0;

        virtual bool read(const wxString& key, long* data) = 0;
        virtual bool read(const wxString& key, long* data, long defVal) = 0;
        virtual long read(const wxString& key, long defVal) = 0;

        virtual bool read(const wxString& key, double* data) = 0;
        virtual bool read(const wxString& key, double* data, double defVal) = 0;
        virtual double read(const wxString& key, double defVal) = 0;

        virtual bool read(const wxString& key, bool* data) = 0;
        virtual bool read(const wxString& key, bool* data, bool defVal) = 0;
        virtual bool read(const wxString& key, bool defVal) = 0;

        virtual wxString getValue(const wxString& path, const wxString& key, const wxString& defValue) = 0;
        virtual bool saveValue(const wxString& path, const wxString& key, const wxString& value) = 0;

        virtual bool write(const wxString& key, const wxString& value) = 0;
        virtual bool write(const wxString& key, int value) = 0;
        virtual bool write(const wxString& key, double value) = 0;
        virtual bool write(const wxString& key, bool value) = 0;

        virtual uint32_t getNumberOfEntries(bool bRecursive = false) = 0;
        virtual bool getFirstEntry(wxString& str, long& index) = 0;
        virtual bool getNextEntry(wxString& str, long& index) = 0;
        virtual EntryType getEntryType(const wxString& name) = 0;

        virtual bool hasEntry(const wxString& name) = 0;

    protected:
    private:

};

WX_DECLARE_HASH_MAP( int, ConfigBase*, wxIntegerHash, wxIntegerEqual, ConfigBaseHash);

#endif // CONFIGBASE_H
