#ifndef CONFIGFILE_H
#define CONFIGFILE_H

#include "ConfigBase.h"

#include <wx/fileconf.h>
#include <wx/wfstream.h>

class ConfigFile: public ConfigBase
{
    public:
        ConfigFile(const wxString& fileName);
        virtual ~ConfigFile();

        bool save();
        bool load();
        bool flush();

        void setPath(const wxString& strPath);
        const wxString& getPath();

        bool read(const wxString& key, wxString* str);
        bool read(const wxString& key, wxString* str, const wxString& defVal);
        wxString read(const wxString& key, const wxString& defVal);

        bool read(const wxString& key, long* data);
        bool read(const wxString& key, long* data, long defVal);
        long read(const wxString& key, long defVal);

        bool read(const wxString& key, double* data);
        bool read(const wxString& key, double* data, double defVal);
        double read(const wxString& key, double defVal);

        bool read(const wxString& key, bool* data);
        bool read(const wxString& key, bool* data, bool defVal);
        bool read(const wxString& key, bool defVal);

        wxString getValue(const wxString& path, const wxString& key, const wxString& defValue);
        bool saveValue(const wxString& path, const wxString& key, const wxString& value);

        bool write(const wxString& key, const wxString& value);
        bool write(const wxString& key, int value);
        bool write(const wxString& key, double value);
        bool write(const wxString& key, bool value);


        uint32_t getNumberOfEntries(bool bRecursive = false);
        bool getFirstEntry(wxString& str, long& index);
        bool getNextEntry(wxString& str, long& index);
        ConfigBase::EntryType getEntryType(const wxString& name);

        bool hasEntry(const wxString& name);

    protected:
    private:
        void checkAndCreateFile();
        wxString m_fileName;
        wxFileConfig* m_wxFileConfig;
		wxFileInputStream* m_wxFileInputStream;
};



#endif // CONFIGFILE_H
