#ifndef ECUTIMER_H
#define ECUTIMER_H

#include <wx/timer.h>
#include <wx/hashmap.h>

#include "RTDataMgrBase.h"
#include "ObserverBase.h"
#include "ringbuffer/RingBuffer.h"
#include "ringbuffer/RingBufferThreadTester.h"

class STDataMgrBase;
#include "Observable.h"

/*
BOOT_MODE =1
ECU_CONNECTED =2
ECU_DISABLED = 4
INPUT_SOURCE = 8+16
INPUT_FILE = 8
INPUT_ECU = 16

*/
enum EcuStates
{
  //  UNKNOWN = -1,
    DISABLED = 1,
    FILELOGPLAY = 3,
    NOTCONNECTED_RB,
    CONNECTED_RB,
    BOOTMODE
};

enum EcuIntervals
{

    INTERVAL_NOTCONNECTED = 1000,
#if defined (__WINCE_STANDARDSDK__)
    INTERVAL_CONNECTED = 120,
#else
    INTERVAL_CONNECTED = 90,
#endif
    INTERVAL_FILELOGPLAY = 100

};

WX_DECLARE_HASH_MAP(int, ObserverBase*, wxIntegerHash, wxIntegerEqual, ObserverHashMap);

class EcuTimer: public wxTimer, public Observable
{
    public:
        EcuTimer(RTDataMgrBase *rtdatamgrbase, STDataMgrBase* stDataMgr, MTConfigData* mtConfig,
        VemsProtocol* vp);
        virtual ~EcuTimer();

        void StartTimer();
        void Notify(void);

        void setCurrState();
        //void SetState(EcuStates state);
        wxString GetStateAsString();
        EcuStates GetStateID();
/*
        //void SetRTDataMgrBase(RTDataMgrBase *rtdatamgrbase);
        void registerObserver(ObserverBase * o);
        void unRegisterObserver(ObserverBase * o);
*/
        void setLocked(bool lock);
        bool isLocked() { return m_locked; };
        void setRawDataLog(bool b);
        bool isRawDataLog() { return m_ringBufferMgr.isRawLog(); };
        RingBuffer * getRingBuffer();
        FirmwareInfo* getFirmwareInfo() { return m_ringBufferMgr.GetFirmwareInfo(); };
        void setSTDataMgr( STDataMgrBase* stdataMgr){
            m_STDataMgrBase = stdataMgr;
            m_ringBufferMgr.setSTDataMgr(stdataMgr);
        }

        bool IsEcuConnected();

        bool GetFilePlay() { return m_filePlay; };
        void SetFilePlay(bool play) { m_filePlay = play; }

        void SetFilelogPlayInterval( long interval, bool resetTimer);

    protected:

    private:

        EcuStates       m_state;
        EcuStates       m_prevstate;
        EcuIntervals    m_interval;
        ObserverHashMap m_observerRegistry;

        RTDataMgrBase*  m_rtDataMgr;
        ObserverBase*   m_observer;
        RingBufferThreadTester m_ringBufferMgr;

        bool m_first;
        bool m_locked;

        bool m_filePlay;


        STDataMgrBase*  m_STDataMgrBase;
};



#endif // ECUTIMER_H
