#include "FirmwareMgrFrame.h"

#include "VemsTuneController.h"
#include "MainPanel.h"

#include "events/DeviceEvent.h"
#include "FirmwareInfo.h"
#include "DebugTabObserver.h"
#include "FirmwareMgrDialog.h"
#include "vemsprotocol.h"
#include "FirmwareInfoDialog.h"
#include "helper.h"


/** \class FirmwareMgrFrame
  *
  * \~english
  * \brief
  * \details
  *
  *
  * \~hungarian
  * \brief
  * \details
  *
  *
  */



int idFirmwareInfo          = wxNewId();
int idMenuVerify            = wxNewId();
int idMenuUpload            = wxNewId();
int idMenuVerifyConfig      = wxNewId();
int idMenuUploadConfig      = wxNewId();
int idMenuDownloadConfig    = wxNewId();
int idMenuClearLog          = wxNewId();

BEGIN_EVENT_TABLE(FirmwareMgrFrame,wxFrame)

    EVT_MENU(idFirmwareInfo, FirmwareMgrFrame::OnFirmwareInfo)
	EVT_MENU(idMenuVerify, FirmwareMgrFrame::OnVerifyFirmware)
	EVT_MENU(idMenuUpload, FirmwareMgrFrame::OnUploadFirmware)
	EVT_MENU(idMenuVerifyConfig, FirmwareMgrFrame::OnVerifyConfig)
	EVT_MENU(idMenuUploadConfig, FirmwareMgrFrame::OnUploadConfig)
	EVT_MENU(idMenuDownloadConfig, FirmwareMgrFrame::OnDownloadConfig)

    EVT_MENU(idMenuClearLog, FirmwareMgrFrame::OnClearLog)

END_EVENT_TABLE()

FirmwareMgrFrame::FirmwareMgrFrame(wxWindow* parent, wxWindowID id, const wxString& title, const wxPoint& pos, const wxSize& size, long style)

{
    //ctor
    Create(parent, id, title,  pos, size, style);


    wxMenuBar* mbar = new wxMenuBar();

    m_mainSizer = new wxBoxSizer(wxVERTICAL);


    m_fileMenu = new wxMenu(_(""));

    m_fileMenu->Append(idFirmwareInfo, _("&Firmware Info\t"), _("Firmware Info"));
    m_fileMenu->AppendSeparator();
    m_fileMenu->Append(idMenuVerify, _("&Verify Firmware\t"), _("Verify Firmware"));
    m_fileMenu->Append(idMenuUpload, _("&Upload Firmware\t"), _("Upload Firmware"));
    m_fileMenu->AppendSeparator();
    m_fileMenu->Append(idMenuVerifyConfig, _("&Verify Config\t"), _("Verify Config"));
    m_fileMenu->Append(idMenuUploadConfig, _("&Upload Config\t"), _("Upload Config"));
    m_fileMenu->AppendSeparator();
    m_fileMenu->Append(idMenuDownloadConfig, _("&Download Config\t"), _("Download Config"));
    m_fileMenu->AppendSeparator();
    m_fileMenu->Append(idMenuClearLog, _("&Clear Log"), _("Clear Log"));

    mbar->Append(m_fileMenu,_T("&File"));
    SetMenuBar(mbar);

    SetMenuState(ECUInfo::DEVICE_UNKNOWN);
}

FirmwareMgrFrame::~FirmwareMgrFrame()
{
    //dtor
}

void FirmwareMgrFrame::Init()
{

    m_panel = new MainPanel( this, 10, 20, 400, 300, m_vemsTuneCtrl); // new PanelStyle


    m_mainSizer->Add(m_panel,1,wxEXPAND);



	SetSizer(m_mainSizer);
	m_mainSizer->Fit(this);
	m_mainSizer->SetSizeHints(this);
}

void FirmwareMgrFrame::SetVemsTuneCtrl(VemsTuneController *ctrl)
{
    m_vemsTuneCtrl = ctrl;
    m_vemsTuneCtrl->Connect( wxEVT_DEVICE, DeviceEventHandler( FirmwareMgrFrame::OnDeviceEvent ),NULL, this);
}

void FirmwareMgrFrame::SetMenuState(int device)
{


    switch(device){
        case ECUInfo::DEVICE_V3:
            m_fileMenu->Enable(idFirmwareInfo, true);
            m_fileMenu->Enable(idMenuVerify, true);
            m_fileMenu->Enable(idMenuUpload, true);
            m_fileMenu->Enable(idMenuVerifyConfig, true);
            m_fileMenu->Enable(idMenuUploadConfig,true);
            m_fileMenu->Enable(idMenuDownloadConfig, true);
            break;
        case ECUInfo::DEVICE_ARM:
            m_fileMenu->Enable(idFirmwareInfo, true);
            m_fileMenu->Enable(idMenuVerify, true);
            m_fileMenu->Enable(idMenuUpload, true);
            m_fileMenu->Enable(idMenuVerifyConfig, true);
            m_fileMenu->Enable(idMenuUploadConfig,true);
            m_fileMenu->Enable(idMenuDownloadConfig, false);
            break;
        case ECUInfo::DEVICE_ROUND:
             m_fileMenu->Enable(idFirmwareInfo, true);
            m_fileMenu->Enable(idMenuVerify, true);
            m_fileMenu->Enable(idMenuUpload, true);
            m_fileMenu->Enable(idMenuVerifyConfig, false);
            m_fileMenu->Enable(idMenuUploadConfig,false);
            m_fileMenu->Enable(idMenuDownloadConfig, false);
            break;
        default:
            m_fileMenu->Enable(idFirmwareInfo, true);
            m_fileMenu->Enable(idMenuVerify, true);
            m_fileMenu->Enable(idMenuUpload, true);
            m_fileMenu->Enable(idMenuVerifyConfig, true);
            m_fileMenu->Enable(idMenuUploadConfig, true);
            m_fileMenu->Enable(idMenuDownloadConfig, true);
    }

}



void FirmwareMgrFrame::OnDeviceEvent(DeviceEvent& event)
{
    //bool enable = false;
    SetMenuState(event.device);
    //show();
    event.Skip();
}


void FirmwareMgrFrame::OnFirmwareInfo(wxCommandEvent& event)
{
    FirmwareInfo* fwInfo = m_vemsTuneCtrl->getFirmwareInfo();
    FirmwareInfoDialog(this, fwInfo);
}

//////////////////////////////////////////// START Firmware mgr events //////////////////////////////////
void FirmwareMgrFrame::OnVerifyFirmware(wxCommandEvent& event)
{
    DebugTabObserver debugTabObserver(m_panel->m_debug_text);
    FirmwareMgrDialog firmwareMgrDlg(this, m_vemsTuneCtrl, &debugTabObserver);
    firmwareMgrDlg.verify();

    //verifyUpdate(VERIFY);
}

void FirmwareMgrFrame::OnUploadFirmware(wxCommandEvent& event)
{
    DebugTabObserver debugTabObserver(m_panel->m_debug_text);
    FirmwareMgrDialog firmwareMgrDlg(this, m_vemsTuneCtrl, &debugTabObserver);
    firmwareMgrDlg.upload();

    //firmwareMgrDlg.quickFirmwareUpload();
}

void FirmwareMgrFrame::OnDownloadConfig(wxCommandEvent& event)
{
    DebugTabObserver debugTabObserver(m_panel->m_debug_text);
    FirmwareMgrDialog firmwareMgrDlg(this, m_vemsTuneCtrl, &debugTabObserver);
    firmwareMgrDlg.downloadConfig();
}

void FirmwareMgrFrame::OnUploadConfig(wxCommandEvent& event)
{
    DebugTabObserver debugTabObserver(m_panel->m_debug_text);
    FirmwareMgrDialog firmwareMgrDlg(this, m_vemsTuneCtrl, &debugTabObserver);
    firmwareMgrDlg.uploadConfig();

    //firmwareMgrDlg.quickFirmwareConfigUpload();
}


void FirmwareMgrFrame::OnVerifyConfig(wxCommandEvent& event)
{
    DebugTabObserver debugTabObserver(m_panel->m_debug_text);
    FirmwareMgrDialog firmwareMgrDlg(this, m_vemsTuneCtrl, &debugTabObserver);
    firmwareMgrDlg.verifyConfig();
}
//////////////////////////////////////////// END Firmware mgr events //////////////////////////////////


void FirmwareMgrFrame::OnClearLog(wxCommandEvent& event)
{

   Helper::clearLog();
}
