#include "FrameLayoutHandler.h"

#include "logger.h"


#include <wx/filename.h>
#include <wx/regex.h>
#include <wx/tokenzr.h>
#include <wx/wxsqlite3.h>

#include "toolbartools/TbTbGaugeGroup.h"
#include "toolbartools/TbTbECUState.h"
#include "toolbartools/TbTbIsLogging.h"
#include "toolbartools/TbTbInputSource.h"
#include "toolbartools/TbTbSerialMode.h"
#include "toolbartools/TbTbPlay.h"
#include "toolbartools/TbTbStop.h"
#include "toolbartools/TbTbPause.h"
#include "toolbartools/TbTbGPS.h"
#include "toolbartools/TbTbDevices.h"
#include "toolbartools/TbTbConfigValidate.h"
#include "toolbartools/TbTbStaticControl.h"
#include "toolbartools/TbTbEditMode.h"
#include "toolbartools/TbTbLogPlaySpeed.h"
#include "toolbartools/TbTbLogRepeat.h"
#include "DisplayHelper.h"
#include "VemsAuiManager.h"

#include "vemsgauge/VemsFlagWidget.h"

#include "ConfigBase.h"
#include "configmanager/MainGuiConfig.h"
#include "configmanager/ConfigHelper2.h"
#include "configmanager/ConfigMgrMainGui.h"

#include "FilePosSlider.h"
#include "panels/DebugPanel.h"

#include "wxXmlHelper.h"

#include "loghistory/LogHistoryMgrHelper.h"
#include "loghistory/LogHistoryMgr.h"


#include "wxSqliteHelper.h"

#include "gaugedesc/GaugeDescHelper.h"
#include "gui/VemsWelcomeScreen.h"

#include "events/VemsDeviceControllerEvent.h"
#include "events/CommunicationEvent.h"

#include "wxVariantHelper.h"

/** FrameLayoutHandler class.
  *
  *
  */

FrameLayoutHandler::FrameLayoutHandler(wxFrame *frame, RTDataMgrBase * rtDataMgr)
                :m_mainWindow (NULL)
                ,m_menu (NULL)
                ,m_perspective (wxEmptyString)
                ,m_isMenu (false)
{
    m_parentFrame = frame;
    m_auiMDIFrame = NULL;
    m_tabsMenu = NULL;
//    m_vemsTuneController = NULL;

//    m_auiMgr = new wxAuiManager(frame);
    m_auiMgr = new VemsAuiManager(frame);

    m_rtDataMgr = rtDataMgr;

//    m_auiMgr->Connect( wxEVT_MOVE, wxMoveEventHandler(FrameLayoutHandler::OnMove));

    m_auiMgr->SetFlags( 0
                      //  | wxAUI_MGR_TRANSPARENT_DRAG
                        //| wxAUI_MGR_DEFAULT
                       // | wxAUI_MGR_ALLOW_ACTIVE_PANE

                        | wxAUI_MGR_TRANSPARENT_HINT
                        | wxAUI_MGR_HINT_FADE
                        | wxAUI_MGR_NO_VENETIAN_BLINDS_FADE
                      );

    Connect(    wxID_ANY,
                wxEVT_VEMS_DEVICE_CONTROLLER,
                VemsDeviceControllerEventHandler( FrameLayoutHandler::OnVemsDeviceControllerEvent ),
                NULL, this );

    Connect(    wxID_ANY,
                wxEVT_COMMUNICATION,
                CommunicationEventHandler( FrameLayoutHandler::OnCommunicationEvent ),
                NULL, this );

    frame->PushEventHandler(this);



}

FrameLayoutHandler::~FrameLayoutHandler()
{
    //dtor
    delete m_auiMgr;

    //ClearToolBarTools();

}

bool FrameLayoutHandler::SaveSymbol(int itemid, const wxString& name)
{
    if(itemid == -1) return false;
    wxSQLite3Database db;
    db.Open( m_guiLayoutFile );
    db.ExecuteUpdate( wxString::Format(_T("update items set name='%s' where id=%i;"),name.c_str(), itemid) );
    db.Close();
    return true;
}

wxString FrameLayoutHandler::GetPerspective()
{

    m_perspective = m_auiMgr->SavePerspective();
    return m_perspective;

}


void FrameLayoutHandler::OnMove(wxMoveEvent & event)
{

    Logger::logf(LOG_MSG,_T("FrameLayoutHandler::OnMove()"));

}


/** @brief SetMainWindow: The Main Window is the default pane, not movable and the center of the parentframe
  *
  * @todo: document this function
  */
void FrameLayoutHandler::SetMainWindow(wxWindow *win)
{
    m_mainWindow = win;
}


void FrameLayoutHandler::OnMenu(wxCommandEvent& event)
{
    wxString ToolBarName = m_menu->GetLabel(event.GetId());


    bool IsShown = (m_menu->FindItem(event.GetId())->IsChecked());

    m_auiMgr->GetPane(ToolBarName).Show(IsShown);

    m_auiMgr->Update();

}

void FrameLayoutHandler::OnVemsDeviceControllerEvent( VemsDeviceControllerEvent& event )
{
    VemsToolBarToolBaseHash::iterator iter = m_toolBarTools.begin();

    for ( ; iter != m_toolBarTools.end(); iter++ )
    {
        VemsToolBarToolBase* tool = iter->second;
        tool->AddPendingEvent( event );
    }
}

void FrameLayoutHandler::OnCommunicationEvent( CommunicationEvent& event )
{
    VemsToolBarToolBaseHash::iterator iter = m_toolBarTools.begin();

    for ( ; iter != m_toolBarTools.end(); iter++ )
    {
        VemsToolBarToolBase* tool = iter->second;
        tool->AddPendingEvent( event );
    }
}

short FrameLayoutHandler::isUseSQL()
{
    wxFileName fn(m_guiLayoutFile);

    if( fn.GetExt() == _T("xml") )
        return 0;
    else if( fn.GetExt() == _T("db") || fn.GetExt() == _T("s3db") )
        return 1;
    else
        return -1;
}

bool FrameLayoutHandler::SetToolBarIconParam(const wxString &param, int itemid)
{
    if(itemid == -1) return false;
    wxSQLite3Database db;
    db.Open( m_guiLayoutFile );
    db.ExecuteUpdate( wxString::Format(_T("update items set param1='%s' where id=%i;"),param.c_str(), itemid) );
    db.Close();
    return true;
}

bool FrameLayoutHandler::SetToolBarIconParam(const wxString &tbname, const wxString &itemname, const wxString &oldparam, const wxString &newparam)
{


    bool res = false;

    for( wxXmlNode* ToolBarNode = m_guiLayoutXmlDoc.GetRoot()->GetChildren() ;ToolBarNode; ToolBarNode = ToolBarNode->GetNext() ) {
        if(ToolBarNode->GetName() != _T("ToolBar"))  continue;

        //Logger::logf(LOG_MSG,_T("FrameLayoutHandler::SetToolBarIconParam"));

        if( ToolBarNode->GetPropVal(_T("name"), wxEmptyString) != tbname) continue;

        for(wxXmlNode* ToolsNode = ToolBarNode->GetChildren(); ToolsNode; ToolsNode = ToolsNode->GetNext() ){
            if(ToolsNode->GetName() != _T("Item")){
                continue;
            }

            if( ToolsNode->GetPropVal(_T("type"), wxEmptyString) != _T("Icon") ) continue;
            if( ToolsNode->GetPropVal(_T("name"), wxEmptyString) != itemname ) continue;
            if( ToolsNode->GetPropVal(_T("param1"), wxEmptyString) != oldparam ) continue;

            res = wxXmlHelper::SetPropVal(ToolsNode, _T("param1"), newparam);
            break;
        }

        if(res) break;

    }

    if(res) {
        m_guiLayoutXmlDoc.Save(m_guiLayoutFile);
    }

    return res;

}

int FrameLayoutHandler::AddToolBarIconWithParam(int toolbar_id, const wxString &itemname, const wxString &param, const wxString& type)
{
    if(isUseSQL() != 1) return -1;

    wxSQLite3Database db;
    db.Open( m_guiLayoutFile );

    db.ExecuteUpdate( wxString::Format(_T("insert into items (type,name,param1,size,toolbar_id) values ('%s','%s','%s',0,%i);"),
                                              type.c_str(),itemname.c_str(),param.c_str(),toolbar_id) );

    int id = db.GetLastRowId().ToLong();
    db.Close();
    return id;
}

void FrameLayoutHandler::AddToolBarIconWithParam(const wxString &tbname, const wxString &itemname, const wxString &param)
{
    bool res = false;

    Logger::logf(LOG_MSG,_T("tbname=")+tbname+_T(", itemname=")+itemname + _T(", param=")+param);

    for( wxXmlNode* ToolBarNode = m_guiLayoutXmlDoc.GetRoot()->GetChildren() ;ToolBarNode; ToolBarNode = ToolBarNode->GetNext() ) {
        if(ToolBarNode->GetName() != _T("ToolBar"))  continue;

        if( ToolBarNode->GetPropVal(_T("name"), wxEmptyString) != tbname) continue;

        wxXmlNode * newitem = new wxXmlNode(wxXML_ELEMENT_NODE, _T("Item"));

        newitem->AddProperty(_T("type"),_T("Icon"));
        newitem->AddProperty(_T("name"),itemname);
        newitem->AddProperty(_T("param1"),param);

        ToolBarNode->AddChild(newitem);

        res = true;
        break;
    }

    if(res) {
        m_guiLayoutXmlDoc.Save(m_guiLayoutFile);
    }

    //return res;
}
//
//bool FrameLayoutHandler::LoadFromXML()
//{
//
//    if( !m_guiLayoutXmlDoc.Load( m_guiLayoutFile) )
//        return false;
//
//    if(m_guiLayoutXmlDoc.GetRoot()->GetName() != _T("guiLayout")){
//        return false;
//    }
//
//
//    wxXmlNode* ToolBarNode = m_guiLayoutXmlDoc.GetRoot()->GetChildren();
//
//    Logger::logf(LOG_MSG,_T("FrameLayoutHandler::Init() root cycle START"));
//
//    while(ToolBarNode){
//        if(ToolBarNode->GetName() != _T("ToolBar")){
//            ToolBarNode = ToolBarNode->GetNext();
//            continue;
//        }
//
//        long IconSizeW = 17;
//        long IconSizeH = 17;
//        long FlagSize = 80;
//
//        wxString ToolBarName = ToolBarNode->GetPropVal(_T("name"), wxEmptyString);
//        wxString ToolBarOri  = ToolBarNode->GetPropVal(_T("defaultOrientation"), wxEmptyString);
//        ToolBarNode->GetPropVal(_T("FlagSize"), wxEmptyString).ToLong(&FlagSize);
//        ToolBarNode->GetPropVal(_T("IconHeight"), wxEmptyString).ToLong(&IconSizeH);
//        ToolBarNode->GetPropVal(_T("IconWidth"), wxEmptyString).ToLong(&IconSizeW);
//
//        if( IconSizeW == -1 ) IconSizeW = IconSizeH;
//
//        AuiPaneToolBar *paneTB = new AuiPaneToolBar(m_parentFrame,ToolBarOri,ToolBarName);
//
//        wxXmlNode* ToolsNode = ToolBarNode->GetChildren();
//        wxToolBar *toolbar = paneTB->GetToolBar();
//
//        bool matchIcon = true;
//
//
//        if(ToolBarOri == TB_VERTICAL)
//            toolbar->SetToolBitmapSize(wxSize(FlagSize, (FlagSize/4)-5));
//        else if(ToolBarOri == TB_HORIZONTAL) {
//            if( IconSizeH <= 20 ) IconSizeH = 17;
//            FlagSize = IconSizeH * 4 + 5;
//            toolbar->SetToolBitmapSize(wxSize(IconSizeW, IconSizeH));
//        }
//
//        int id = wxNewId();
//        m_menu->AppendCheckItem( id, ToolBarName );
//        Connect( id,  wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(FrameLayoutHandler::OnMenu) );
//        m_menu->Check(id, true );
//
//        while(ToolsNode){
//            if(ToolsNode->GetName() != _T("Item")){
//                ToolsNode = ToolsNode->GetNext();
//                continue;
//            }
//
//            wxString iType = ToolsNode->GetPropVal(_T("type"), wxEmptyString);
//            wxString iName = ToolsNode->GetPropVal(_T("name"), wxEmptyString);
//            wxString ssize = ToolsNode->GetPropVal(_T("size"), _T("0"));
//            wxString param = ToolsNode->GetPropVal(_T("param1"), wxEmptyString);
//
//            long iSize;
//            ssize.ToLong(&iSize);
//
//            if(iType == _T("Flag")) {
//                //Logger::logf(LOG_MSG,_T("FrameLayoutHandler::Init() root iName=") + iName + _T(", pos=%i"),tPos);
//
//                GaugeProperties prop;
//                prop.symbolName = iName;
//                prop.width = FlagSize+3;
//
//                VemsFlagWidget *flag = new VemsFlagWidget( toolbar, wxID_ANY, GaugeDescHelper::GetGDMgr() ,prop );

//
//                m_rtDataMgr->RegisterViewer(flag);
//                flag->SetDataMgr(m_rtDataMgr);
//                flag->Show(true);
//
//
//                AddPaneSize(paneTB, wxSize( flag->GetSize().x, flag->GetSize().y+3 ), ToolBarOri);
//
//                toolbar->AddControl(flag);
//
//                /// WTF when toolbar is vertical
//  //              paneTB->m_size += wxSize(80,20);
//    //            toolbar->AddControl(new wxTextCtrl(toolbar,wxID_ANY,_T("Flag=")+iName));
//
//
//            }else if (iType == _T("Icon")) {
//                AddToolBarTool(toolbar,iName,-1,param);
//
//                AddPaneSize(paneTB, wxSize(toolbar->GetToolSize().x,toolbar->GetToolSize().y-1), ToolBarOri);
//
//                matchIcon = true;
//
//
//           }else if (iType == _T("Control")) {
//
//                AddControl(toolbar, iName, wxSize(iSize,IconSizeH));
//
//                AddPaneSize(paneTB, wxSize(iSize,IconSizeH-1), ToolBarOri);
//
//
//            }else if (iType == _T("None")) {
//                if(iName == _T("Separator")) {
//                    toolbar->AddSeparator();
//                    if(ToolBarOri == TB_VERTICAL)
//                        AddPaneSize(paneTB, wxSize(toolbar->GetToolSize().x,toolbar->GetToolSeparation()), ToolBarOri);
//                    else if(ToolBarOri == TB_HORIZONTAL)
//                        AddPaneSize(paneTB, wxSize(toolbar->GetToolSeparation()+1,toolbar->GetToolSize().y), ToolBarOri);
//                }
//
//            }
//
//
//            ToolsNode = ToolsNode->GetNext();
//
//        }
//
//        Logger::logf(LOG_MSG,_T("toolbar->GetToolSeparation()=%i"),toolbar->GetToolSeparation());
//
//
//        Logger::logf(LOG_MSG,_T("toolbar->GetToolsCount()=%i"),toolbar->GetToolsCount());
//
//        //if(!matchIcon) toolbar->AddTool(wxID_ANY,_T("debug"),wxBitmap());
//
//        toolbar->Realize();
//
//        SendObject(TN_AfterRealize, NULL);
//
//        //paneTB->UpdateSize();
//
//        paneTB->m_size += wxSize(2,2);
//
//        m_auiMgr->AddPane1(paneTB);
//
//        m_auiToolHash[ToolBarName] = paneTB;
//
//        ToolBarNode = ToolBarNode->GetNext();
//
//    }
//
//    return true;
//}


void FrameLayoutHandler::ClearFlags()
{
    VemsFlagWidgetSet flags;

    VemsFlagWidgetHash::iterator iter = m_flags.begin();

    for ( ; iter != m_flags.end(); ++iter ) {
        VemsFlagWidget* flag = iter->second;
        flags.insert( flag );
    }

    m_flags.clear();

    VemsFlagWidgetSet::iterator setIter = flags.begin();

    for ( ; setIter != flags.end(); ++setIter ) {
        VemsFlagWidget* flag = *setIter;

        if ( flag->IsRegistered() ) {
            flag->UnRegisterViewer();
        }

        wxToolBar* toolbar = dynamic_cast<wxToolBar*>(flag->GetParent());

        int flagId = flag->GetId();

        if ( toolbar ) {
            toolbar->DeleteTool( flagId );
        }

        Logger::logf( LOG_MSG, _T("remove flag: %d"), flagId );
    }
}

void FrameLayoutHandler::ClearToolBarTools()
{
    Logger::logf( LOG_MSG, _T("FrameLayoutHandler::ClearToolBarTools()") );

    VemsToolBarToolBaseHash::iterator iter = m_toolBarTools.begin();

    for ( ; iter != m_toolBarTools.end(); iter++ )
    {
        VemsToolBarToolBase* tool = iter->second;
        wxToolBar* toolbar = tool->GetToolBar();
        int toolid = tool->GetId();
        toolbar->DeleteTool( toolid );

        toolbar->Disconnect( toolid, wxEVT_COMMAND_TOOL_CLICKED, wxCommandEventHandler(VemsToolBarToolBase::OnTool), NULL, tool);
        toolbar->Disconnect( toolid, wxEVT_COMMAND_TOOL_RCLICKED, wxCommandEventHandler(VemsToolBarToolBase::OnToolR), NULL, tool);
        toolbar->Disconnect( toolid, wxEVT_COMMAND_TOOL_ENTER, wxCommandEventHandler(VemsToolBarToolBase::OnToolEnter), NULL, tool);


//        for(uint16_t a=0; a < m_vtbtNeedArray.GetCount(); a++){
//            VemsToolBarToolBase * tbn = m_vtbtNeedArray[a]->m_tool;
//
//            if( tbn == tool ) {
//                delete m_vtbtNeedArray[a];
//                m_vtbtNeedArray.RemoveAt( a );
//                break;
//            }
//        }

        delete tool;

        Logger::logf( LOG_MSG, _T("remove item: %d"), toolid );

    }

    m_toolBarTools.clear();
    WX_CLEAR_ARRAY( m_vtbtNeedArray );
}

void FrameLayoutHandler::ClearControls()
{
    FilePosSliderSet::iterator iter = m_controls.begin();

    for ( ; iter != m_controls.end(); ++iter ) {
        FilePosSlider* slider = *iter;
        //int sliderId = slider->GetId();
        LogHistoryMgrHelper::getInstance()->unRegisterObserver(slider);
        slider->unRegisterObserver(LogHistoryMgrHelper::getInstance());
        //toolbar->DeleteControl(sliderId);
    }

    m_controls.clear();
}

void FrameLayoutHandler::ClearToolBars()
{
    Logger::logf( LOG_MSG, _T("FrameLayoutHandler::ClearToolBars()") );

    AuiPaneToolBarHash::iterator iter = m_auiToolHash.begin();

    for ( ; iter != m_auiToolHash.end(); ++iter )
    {
        wxString name = iter->first;
        AuiPaneToolBar* auiToolbar = iter->second;
        wxToolBar* toolbar = auiToolbar->GetToolBar();
        m_auiMgr->DetachPane( toolbar );
        delete auiToolbar;
        int toolCount = toolbar->GetToolsCount();
        if ( toolCount ) {
            toolbar->ClearTools();
            Logger::logf( LOG_MSG, _T("destory ") + name );

        }


        toolbar->Destroy();

        Logger::logf( LOG_MSG, _T("\t ToolBarName: ") + name + _T(", tools: %d"), toolCount );
    }

    m_auiToolHash.clear();
}

void FrameLayoutHandler::Clear()
{
    Logger::logf( LOG_MSG, _T("FrameLayoutHandler::Clear() - BEGIN") );

    ClearFlags();
    ClearToolBarTools();
    ClearControls();
    ClearToolBars();

    Logger::logf( LOG_MSG, _T("FrameLayoutHandler::Clear() - END") );

    //m_auiMgr->Update();
}

void FrameLayoutHandler::EnableAllTool( bool enable )
{
    Logger::logf( LOG_MSG, _T("FrameLayoutHandler::EnableAllTool() - enable: %d"), enable );

    VemsToolBarToolBaseHash::iterator iter = m_toolBarTools.begin();

    for ( ; iter != m_toolBarTools.end(); iter++ )
    {
        VemsToolBarToolBase* tool = iter->second;
        wxToolBar* toolbar = tool->GetToolBar();
        int toolid = tool->GetId();

        toolbar->EnableTool( toolid, true );
        toolbar->Realize();
        Logger::logf( LOG_MSG, _T("FrameLayoutHandler::EnableAllTool() - toolid: %d"), toolid );
    }

    m_auiMgr->Update();
}

void FrameLayoutHandler::UpdateAUI()
{
    m_auiMgr->Update();
}

void FrameLayoutHandler::AddPaneSize(AuiPaneToolBar *paneTB, wxSize size, const wxString &ToolBarOri)
{
    if(ToolBarOri == TB_VERTICAL) {

        paneTB->m_size.y += size.y;
        paneTB->m_size.x = size.x;

    } else if(ToolBarOri == TB_HORIZONTAL) {

        paneTB->m_size.x += size.x;
        paneTB->m_size.y = size.y;

    }


//    Logger::logf(LOG_MSG,_T("newsize x=%i, y=%i, name=")+paneTB->GetName(),paneTB->m_size.x,paneTB->m_size.y);

}

bool FrameLayoutHandler::LoadFromSQL()
{
    wxArrayString arr;
    arr.Add(_T("items"));
    arr.Add(_T("toolbars"));

    if( !wxSqliteHelper::TablesCheck( arr, m_guiLayoutFile ) )
        return false;

    wxSQLite3Database db;
    db.Open( m_guiLayoutFile );

    wxSQLite3ResultSet tbres = db.ExecuteQuery( _T("select * from toolbars order by id asc;") );

    int idx = m_menu->FindItem(_T("Main"));
	if ( idx != wxNOT_FOUND )
		m_menu->Delete(idx);

    while( tbres.NextRow() ){

        wxString ToolBarName = tbres.GetString(_T("name"));
        wxString ToolBarOri  = tbres.GetBool(_T("isHorizontal")) == true ? TB_HORIZONTAL : TB_VERTICAL;
        long FlagSize = tbres.GetInt(_T("FlagSize"));
        long IconSizeW = tbres.GetInt(_T("IconWidth"));
        long IconSizeH = tbres.GetInt(_T("IconHeight"));

        //Logger::logf(LOG_MSG,_T("ToolBarOri=")+ToolBarOri);
        if( IconSizeW == -1 ) IconSizeW = IconSizeH;

        AuiPaneToolBar *paneTB = new AuiPaneToolBar(m_parentFrame,ToolBarOri,ToolBarName);

        wxToolBar *toolbar = paneTB->GetToolBar();

        bool matchIcon = false;

        if(ToolBarOri == TB_VERTICAL)
            toolbar->SetToolBitmapSize(wxSize(FlagSize, (FlagSize/4)-5));
        else if(ToolBarOri == TB_HORIZONTAL) {
            if( IconSizeH <= 20 ) IconSizeH = 17;
            FlagSize = IconSizeH * 4 + 5;
            toolbar->SetToolBitmapSize(wxSize(IconSizeW, IconSizeH));
        }

        int id = wxNewId();
        m_menu->AppendCheckItem( id, ToolBarName );
        Connect( id,  wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(FrameLayoutHandler::OnMenu) );
        m_menu->Check(id, true );

        int toolbar_id = tbres.GetInt(_T("id"));

        wxSQLite3ResultSet ires = db.ExecuteQuery( wxString::Format(_T("select * from items where toolbar_id=%i;"), toolbar_id ) );

        paneTB->ItemID = toolbar_id;

        while( ires.NextRow() ){

            wxString iType = ires.GetString(_T("type"));
            wxString iName = ires.GetString(_T("name"));
            wxString param = ires.GetString(_T("param1"));
            long iSize = ires.GetInt(_T("size"));
            int itemid = ires.GetInt(_T("id"));


            if(iType == _T("Flag")) {

                VemsFlagWidget* flag = AddFlag(toolbar, FlagSize, iName, itemid);

                AddPaneSize(paneTB, wxSize(flag->GetSize().x,flag->GetSize().y+3), ToolBarOri);

                flag->SetGroupName( ToolBarName );

                /// WTF when toolbar is vertical
  //              paneTB->m_size += wxSize(80,20);

            }else if (iType == _T("Icon")) {
                AddToolBarTool(toolbar,iName,-1,param,itemid);

                AddPaneSize(paneTB,wxSize(toolbar->GetToolSize().x,toolbar->GetToolSize().y-1), ToolBarOri);

                matchIcon = true;


           }else if (iType == _T("Control")) {

                AddPaneSize(paneTB, wxSize(iSize,IconSizeH), ToolBarOri);

                AddControl(toolbar, iName, wxSize(iSize,IconSizeH-1),itemid);


            }else if (iType == _T("None")) {
                if(iName == _T("Separator")) {
                    toolbar->AddSeparator();
                    if(ToolBarOri == TB_VERTICAL)
                        AddPaneSize(paneTB, wxSize(toolbar->GetToolSize().x,toolbar->GetToolSeparation()), ToolBarOri);
                    else if(ToolBarOri == TB_HORIZONTAL)
                        AddPaneSize(paneTB, wxSize(toolbar->GetToolSeparation()+1,toolbar->GetToolSize().y), ToolBarOri);

                }

            }


        }


        //Logger::logf(LOG_MSG,_T("toolbar->GetToolsCount()=%i"),toolbar->GetToolsCount());

        if(!matchIcon && ToolBarOri == TB_VERTICAL)
            toolbar->AddTool(wxID_ANY,_T("debug"),wxBitmap());

        toolbar->Realize();

        SendObject(TN_AfterRealize, NULL);

        paneTB->m_size += wxSize(2,2);

        //paneTB->UpdateSize();
        m_auiMgr->AddPane1(paneTB);

        m_auiToolHash[ToolBarName] = paneTB;


    }

    return true;
}

void FrameLayoutHandler::ExtendMenu(wxMenu* menu, wxObject* sender)
{
    if(sender->IsKindOf(CLASSINFO(VemsFlagWidget))) {
        m_flags.clear();
        VemsFlagWidget* flag = wxDynamicCast(sender,VemsFlagWidget);

        menu->AppendSeparator();

        int id = wxNewId();
        menu->Append(id, _T("Add a new flag"));
        Connect(id, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(FrameLayoutHandler::OnFlagMenu), NULL, this );
        m_flags[-1*id] = flag;

        id = wxNewId();
        menu->Append(id, _T("Remove"));
        Connect(id, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(FrameLayoutHandler::OnFlagMenu), NULL, this );
        m_flags[id] = flag;

    }
}

void FrameLayoutHandler::OnFlagMenu(wxCommandEvent& event)
{
    int id = event.GetId();

    if( m_flags.find(-1*id) != m_flags.end() ) {  /// add a new flag, to the toolbar
        VemsFlagWidget * flag = m_flags[-1*id];

        wxToolBar * toolbar = NULL;
        toolbar = dynamic_cast<wxToolBar*>(flag->GetParent());

        if( toolbar ) {

            wxString tbname = flag->GetGroupName();

            wxAuiPaneInfo info = m_auiMgr->GetPane( toolbar );

            AuiPaneToolBar * paneTB = m_auiToolHash[tbname];

            if(!paneTB) return;

            int itemid = AddToolBarIconWithParam( paneTB->ItemID, flag->GetSelectedSymbol() , _T(""), _T("Flag"));

            flag = AddFlag(toolbar, flag->GetSize().x-3, flag->GetSelectedSymbol(), itemid);

            toolbar->Realize();

            SendObject(TN_AfterRealize, NULL);

            paneTB->m_size.x += flag->GetSize().x;

            info.best_size.x = paneTB->m_size.x;

            m_auiMgr->DetachPane( toolbar );

            m_auiMgr->AddPane( toolbar, info );

            m_auiMgr->Update();

            m_parentFrame->Layout();

            flag->SetGroupName( tbname );

        }

    } else if( m_flags.find(id) != m_flags.end() ) {
        VemsFlagWidget * flag = m_flags[id];

        wxToolBar * toolbar = NULL;
        toolbar = dynamic_cast<wxToolBar*>(flag->GetParent());

        Logger::logf(LOG_MSG,_T("toolbar=%p"),toolbar);

        if( toolbar ) {

            wxString tbname = flag->GetGroupName();

            int itemid = flag->GetItemID();

            wxAuiPaneInfo info = m_auiMgr->GetPane( toolbar );

            AuiPaneToolBar * paneTB = m_auiToolHash[tbname];

            info.best_size.x -= flag->GetSize().x;

            if(paneTB)
                paneTB->m_size.x -= flag->GetSize().x;

            if ( flag->IsRegistered() ) {
                flag->UnRegisterViewer();
            }

            toolbar->DeleteTool( flag->GetId() );

            m_auiMgr->DetachPane( toolbar );

            m_auiMgr->AddPane( toolbar, info );

            m_auiMgr->Update();

            m_flags.erase(id);

            wxSQLite3Database db;
            db.Open( m_guiLayoutFile );
            db.ExecuteUpdate( wxString::Format(_T("delete from items where id=%i;"),itemid));
            db.Close();



        }

    }

}

VemsFlagWidget* FrameLayoutHandler::AddFlag(wxToolBar * toolbar, int FlagSize, const wxString& iName, int itemid)
{
    GaugeProperties prop;
    prop.symbolName = iName;
    prop.width = FlagSize+3;

    VemsFlagWidget *flag = new VemsFlagWidget( toolbar, wxID_ANY, GaugeDescHelper::GetGDMgr() ,prop );


    m_rtDataMgr->RegisterViewer(flag);
    flag->SetDataMgr(m_rtDataMgr);
    flag->SetGaugeBySymbolName( prop.symbolName );
    flag->InitGauge();
    flag->Show(true);
    flag->SetItemID(itemid);

    flag->SetSymbolSaver(this);
    flag->SetParentLayoutUpdater(this);

    flag->SetMenuExtendsHandler(this);

    toolbar->AddControl(flag);

    return flag;
}


bool FrameLayoutHandler::Init( const wxString& perspective, const wxString& layoutFile)
{
    //if ( m_device == device )
    m_perspective = perspective;
    m_guiLayoutFile = layoutFile;

    bool res = false;

    switch(isUseSQL()){
        case 1:
            res = LoadFromSQL();
            break;
        default:
            res = false;
            break;
    }

    if(!res) return res;

    m_auiMgr->SetDockSizeConstraint(1,1);
    LoadLayout();
    m_auiMgr->Update();

    return res;
}

int FrameLayoutHandler::AddControl(wxToolBar *toolbar, const wxString &name, const wxSize& size, int itemid)
{
    int id = wxNewId();
    if(name == _T("FilePosSlider")) {

        FilePosSlider *ctrl = new FilePosSlider(toolbar,id, 0, 0, 100, wxDefaultPosition, size);
        Logger::logf(LOG_MSG, _T("FLH:addcontrol()"));\
        LogHistoryMgrHelper::getInstance()->registerObserver(ctrl);
        ctrl->registerObserver(LogHistoryMgrHelper::getInstance());
        toolbar->AddControl(ctrl);
        ctrl->Enable(false);
        m_controls.insert( ctrl );
    }
    return id;
}

VemsToolBarToolBase* FrameLayoutHandler::AddNewToolBarTool(wxToolBar *toolbar, const wxString &tbname, const wxString &name, short pos, const wxString &param1)
{

    wxAuiPaneInfo info = m_auiMgr->GetPane( toolbar );

    AuiPaneToolBar * paneTB = m_auiToolHash[tbname];

    if(!paneTB) return NULL;

    int itemid = AddToolBarIconWithParam( paneTB->ItemID, name, param1);

    VemsToolBarToolBase* tbt = AddToolBarTool( toolbar, name, pos, param1, itemid);

    toolbar->Realize();

    SendObject(TN_AfterRealize, NULL);

    paneTB->m_size.x += toolbar->GetToolSize().x;

    info.best_size.x = paneTB->m_size.x;

    m_auiMgr->DetachPane( toolbar );

    m_auiMgr->AddPane( toolbar, info );

    //m_auiMgr->AddPane1( paneTB );

    m_auiMgr->Update();

    m_parentFrame->Layout();

    return tbt;
}

void FrameLayoutHandler::RemoveToolBarToolFromDB( int itemid )
{
    if(isUseSQL() != 1) return;

    wxSQLite3Database db;
    db.Open( m_guiLayoutFile );

    db.ExecuteUpdate( wxString::Format(_T("delete from items where id=%i;"),itemid));

    db.Close();
}

void FrameLayoutHandler::RefreshToolBar(wxToolBar *toolbar, const wxString &tbname)
{
    wxAuiPaneInfo info = m_auiMgr->GetPane( toolbar );

    //AuiPaneToolBar * paneTB = m_auiToolHash[tbname];

//    info.best_size.x -= toolbar->GetToolSize().x;
//    paneTB->m_size.x -= toolbar->GetToolSize().x;

    m_auiMgr->DetachPane( toolbar );

    m_auiMgr->AddPane( toolbar, info );

    m_auiMgr->Update();
}

void FrameLayoutHandler::RemoveToolBarTool(wxToolBar *toolbar, const wxString &tbname, int toolid )
{
    wxAuiPaneInfo info = m_auiMgr->GetPane( toolbar );

    AuiPaneToolBar * paneTB = m_auiToolHash[tbname];

    info.best_size.x -= toolbar->GetToolSize().x;

    paneTB->m_size.x -= toolbar->GetToolSize().x;

    toolbar->DeleteTool( toolid );

    m_auiMgr->DetachPane( toolbar );

    m_auiMgr->AddPane( toolbar, info );

    m_auiMgr->Update();

    VemsToolBarToolBase * tb = m_toolBarTools[toolid];

    m_toolBarTools.erase(toolid);

    for(uint16_t a=0; a < m_vtbtNeedArray.GetCount(); a++){
        VemsToolBarToolBase * tbn = m_vtbtNeedArray[a]->m_tool;

        if( tbn == tb ) {
            delete m_vtbtNeedArray[a];
            m_vtbtNeedArray.RemoveAt( a );
            break;
        }
    }

    delete tb;
}

void FrameLayoutHandler::RemoveToolBarTool(wxToolBar *toolbar, const wxString &tbname, int toolid, int itemid )
{
    RemoveToolBarTool( toolbar, tbname, toolid );
    RemoveToolBarToolFromDB( itemid );
}

VemsToolBarToolBase* FrameLayoutHandler::AddToolBarTool(wxToolBar *toolbar, const wxString &name, short pos, const wxString &param1, int itemid)
{
    int id = wxNewId();

    VemsToolBarToolBase *tb = NULL;


    if(name == _T("ECUState"))
        tb = new TbTbECUState(id);

    else if(name == _T("isLogging"))
        tb = new TbTbIsLogging(id);


    else if(name == _T("InputSource"))
        tb = new TbTbInputSource(id);

    else if(name == _T("SerialMode"))
        tb = new TbTbSerialMode(id);

    else if(name == _T("Play"))
        tb = new TbTbPlay(id);

    else if(name == _T("Stop"))
        tb = new TbTbStop(id);

    else if(name == _T("Pause"))
        tb = new TbTbPause(id);

    else if (name == _T("PlaySpeed") )
        tb = new TbTbLogPlaySpeed( id );

    else if ( name == _T("Repeat") )
        tb = new TbTbLogRepeat( id );

    else if(name == _T("GPS"))
        tb = new TbTbGPS(id);

    else if(name == _T("Devices"))
        tb = new TbTbDevices(id);

    else if(name == _T("GaugeGroup")){
        TbTbGaugeGroup* tbgg = new TbTbGaugeGroup(id, this);
        long tmp;
        param1.ToLong(&tmp);
        tbgg->SetGroupId(tmp);
        tb = tbgg;
     }

    else if(name == _T("ConfigValidate"))
        tb = new TbTbConfigValidate(id);

    else if(name == _T("StaticControl")) {
        TbTbStaticControl* stCtrl = new TbTbStaticControl(id,param1);
        m_rtDataMgr->RegisterViewer(stCtrl);
        ((RTDataViewerBase*)stCtrl)->SetDataMgr(m_rtDataMgr);
        tb = stCtrl;
    }


    else if(name == _T("EditMode"))
        tb = new TbTbEditMode(id);


    if(tb == NULL)  return NULL;

    tb->SetItemId( itemid );

    toolbar->Connect(id,wxEVT_COMMAND_TOOL_CLICKED, wxCommandEventHandler(VemsToolBarToolBase::OnTool), NULL, tb);
    toolbar->Connect(id,wxEVT_COMMAND_TOOL_RCLICKED, wxCommandEventHandler(VemsToolBarToolBase::OnToolR), NULL, tb);
    toolbar->Connect(id,wxEVT_COMMAND_TOOL_ENTER, wxCommandEventHandler(VemsToolBarToolBase::OnToolEnter), NULL, tb);

    tb->SetToolBar(toolbar);
    tb->SetId(id);

    tb->Init();

    TbNeedArray tmparr = tb->GetObjectIds();

    VtbtNeedObj *objtmp;

    for(uint16_t a=0; a<tmparr.GetCount(); a++) {
        objtmp = new VtbtNeedObj(tmparr[a],tb);
        m_vtbtNeedArray.Add(objtmp);
    }

    tb->SetHandler(GetAuiMgr());

    wxBitmap *tmp = tb->GetDefaultBmp();
    if(tmp == NULL)
        toolbar->AddTool(id, name,wxArtProvider::GetBitmap(wxART_MISSING_IMAGE,wxART_TOOLBAR, toolbar->GetToolBitmapSize()) );
    else
        toolbar->AddTool(id, name, *tmp , *tb->GetInactiveBitmap() );


    tb->InitHelpStrings();

    m_toolBarTools[id] = tb;

    return tb;

}

bool FrameLayoutHandler::SendObject(int id, wxObject *obj, const wxString &message)
{
    //Logger::logf(LOG_MSG,_T("FrameLayoutHandler::SendObject START"));

    for(uint16_t a=0; a < m_vtbtNeedArray.GetCount(); a++)
        if(m_vtbtNeedArray[a]->m_id == id)
            m_vtbtNeedArray[a]->m_tool->SendObject(id, obj, message);

    return true;
}

void FrameLayoutHandler::SendEvent( ToolBarToolsNeedId event, int value )
{
    for(uint16_t a=0; a < m_vtbtNeedArray.GetCount(); a++)
        if(m_vtbtNeedArray[a]->m_id == event)
            m_vtbtNeedArray[a]->m_tool->SendEvent( event, value );
}

void FrameLayoutHandler::OnTabClose(wxCloseEvent &evt)
{
    switch(evt.GetId()) {
        case idTabStartHere:
            if(m_tabsMenu) m_tabsMenu->Check(m_tabsMenu->FindItem(_T("Start here")), false);
            break;

        case idTabLog:
            if(m_tabsMenu) m_tabsMenu->Check(m_tabsMenu->FindItem(_T("Log")), false);
            break;
    }


    evt.Skip();
}

void FrameLayoutHandler::OnTabMenu(wxCommandEvent& event)
{
    switch(event.GetId()) {
        case idMenuTabStartHere:
            if(event.IsChecked())
                LoadWelcomeScreen();
            else
                CloseWelcomeScreen();
            break;

        case idMenuTabLog:
            if(event.IsChecked())
                LoadLogTab();
            else {
                wxWindow* tab = m_parentFrame->FindWindowById(idTabLog);
                if(tab) tab->Close();
            }
            break;
    }
}

void FrameLayoutHandler::CloseWelcomeScreen()
{
    wxWindow* tab = m_parentFrame->FindWindowById(idTabStartHere);
    if(tab) tab->Close();
}

void FrameLayoutHandler::LoadWelcomeScreen()
{
    if(!m_auiMDIFrame) return;

    wxString title = _T("Start here");

    wxAuiMDIChildFrame * chf = new wxAuiMDIChildFrame( m_auiMDIFrame, idTabStartHere, title);

    chf->Hide();

    chf->Connect(wxEVT_CLOSE_WINDOW, wxCloseEventHandler(FrameLayoutHandler::OnTabClose), NULL, this );

    wxBoxSizer * csizer = new wxBoxSizer(wxVERTICAL);

    wxScrolledWindow* scrollwin = new wxScrolledWindow(chf);

    wxBoxSizer* scrollwinSizer = new wxBoxSizer(wxVERTICAL);

    VemsWelcomeScreen* screenpanel = new VemsWelcomeScreen(scrollwin);

    screenpanel->SetEventHandler( m_auiMDIFrame );

    scrollwinSizer->Add(screenpanel,1,wxEXPAND);
    scrollwin->SetSizer(scrollwinSizer);

    scrollwin->SetScrollRate(10,10);

    csizer->Add(scrollwin , 1, wxEXPAND);

    chf->SetSizer(csizer);

    chf->Layout(); ///it's important to works good the sizers inner the wxAuiMDIChildFrame

    if(m_tabsMenu) {
        int id = m_tabsMenu->FindItem(title);
        if(id == wxNOT_FOUND) {
            id = idMenuTabStartHere;
            m_tabsMenu->AppendCheckItem(id,title);
            Connect(id,wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(FrameLayoutHandler::OnTabMenu), NULL, this);
}

        m_tabsMenu->Check(id,true);
    }

    //chf->Show();
}

void FrameLayoutHandler::LoadLogTab()
{
    if(!m_auiMDIFrame) return;

    wxString title = _T("Log");

    wxAuiMDIChildFrame * chf = new wxAuiMDIChildFrame( m_auiMDIFrame, idTabLog, title);
    chf->Connect(wxEVT_CLOSE_WINDOW, wxCloseEventHandler(FrameLayoutHandler::OnTabClose), NULL, this );

        wxBoxSizer * csizer = new wxBoxSizer(wxVERTICAL);

        m_debugPanel = new DebugPanel(chf);

        csizer->Add( m_debugPanel , 1, wxEXPAND);

        chf->SetSizer(csizer);
        csizer->Fit(chf);
        csizer->SetSizeHints(chf);

        chf->Layout();

    if(m_tabsMenu) {
        int id = m_tabsMenu->FindItem(title);
        if(id == wxNOT_FOUND) {
            id = idMenuTabLog;
            m_tabsMenu->AppendCheckItem(id,title);
            Connect(id,wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(FrameLayoutHandler::OnTabMenu), NULL, this);
        }

        m_tabsMenu->Check(id,true);
    }
}

void FrameLayoutHandler::LoadMainPanel()
{

    if(m_auiMDIFrame) {

        LoadLogTab();

    } else {
        wxAuiPaneInfo auiInfo2;

        auiInfo2.Dockable(1);
        auiInfo2.Floatable(1);
        auiInfo2.Dock();
        auiInfo2.CloseButton(false);
        auiInfo2.Centre();
        //auiInfo2.DefaultPane();
        auiInfo2.CaptionVisible(1);
        auiInfo2.PaneBorder(1);
        auiInfo2.Movable(1);
        auiInfo2.Caption(_T("Log"));

        auiInfo2.Name(_T("DebugCtrl"));

        m_debugPanel = new DebugPanel(m_parentFrame);

        m_auiMgr->AddPane(m_debugPanel,auiInfo2);
    }



}

void FrameLayoutHandler::ShowToolBars(bool isShow)
{

    for( AuiPaneToolBarHash::iterator it = m_auiToolHash.begin(); it != m_auiToolHash.end(); it++) {
        wxString ToolBarName = it->second->GetName();
        m_auiMgr->GetPane(ToolBarName).Show(isShow);
    }

    m_auiMgr->Update();
}

/** @brief SavePerspective
  *
  * @todo: document this function
  */
/*
void FrameLayoutHandler::SavePerspective( const wxRect& rect, bool isFullScreen, bool isMaximized, int mdiClientHeight )
{
    m_perspective = m_auiMgr->SavePerspective();


}
*/
void FrameLayoutHandler::LoadLayout()
{
    #define PANE    m_auiMgr->GetPane(it->first)

    //Logger::logf(LOG_MSG,_T("pers=")+m_perspective);

    wxSize dsize = DisplayHelper::GetDisplaySize();

    wxString newpers;// = m_perspective;


    wxRegEx exp1;
    exp1.Compile(_T("name=([^;]{1,});"));

    wxRegEx exp2;
    exp2.Compile(_T("bestw=[0-9]{1,};besth=[0-9]{1,};"));

    wxRegEx exp3;
    exp3.Compile(_T("dir=[0-9]{1};"));

    wxRegEx exp4;
    exp4.Compile(_T("pos=([0-9]{1,});"));


    wxArrayString arr = wxStringTokenize( m_perspective, _T("|") );

    for( size_t a = 0; a < arr.GetCount(); a++) {

        wxString panelayout = arr[a];

        bool res = exp1.Matches(panelayout);

        if(!res) {
            newpers += panelayout + _T("|");
            continue;
        }

        wxString panename = exp1.GetMatch(panelayout,1);

        if(m_auiToolHash.find(panename) == m_auiToolHash.end()) {
            newpers += panelayout + _T("|");
            continue;
        }


        wxSize size = m_auiToolHash[panename]->m_size;
        exp2.Replace( &panelayout, wxString::Format(_T("bestw=%i;besth=%i;"),size.x,size.y) );

        int dir = m_auiToolHash[panename]->GetAuiPaneInfo().dock_direction;
        exp3.Replace( &panelayout, wxString::Format(_T("dir=%i;"),dir) );

        if(exp4.Matches(panelayout)) {
            long pos = 0;
            exp4.GetMatch(panelayout,1).ToLong(&pos);
            if(pos > dsize.x)
                exp4.Replace( &panelayout, _T("pos=0;") );
        }

       // Logger::logf(LOG_MSG,_T("a=%i,x=%i,y=%i"),a,size.x,size.y);

        newpers += panelayout + _T("|");

    }

/*
    for(AuiPaneToolBarHash::iterator it = m_auiToolHash.begin(); it != m_auiToolHash.end(); it++) {
        wxAuiPaneInfo opane = it->second->GetAuiPaneInfo();


        exp.ReplaceAll( &m_perspective, _T("bestw=-1;besth=-1;") );

    }
*/
    m_auiMgr->LoadPerspective(newpers, false);


    for( AuiPaneToolBarHash::iterator it = m_auiToolHash.begin(); it != m_auiToolHash.end(); it++) {
        wxString ToolBarName = it->second->GetName();
        wxAuiPaneInfo& info = m_auiMgr->GetPane(ToolBarName);

        int id = m_menu->FindItem( info.caption );

        m_menu->Check(id,info.IsShown());

}


}

bool FrameLayoutHandler::IsCorruptedToolbarLayout()
{
    wxSize display_size = DisplayHelper::GetDisplaySize();

    for( AuiPaneToolBarHash::iterator it = m_auiToolHash.begin(); it != m_auiToolHash.end(); it++) {

        wxString ToolBarName = it->second->GetName();

        wxAuiPaneInfo& info = m_auiMgr->GetPane(ToolBarName);

        if( info.rect.x < 0 || info.rect.x > display_size.x ||
            info.rect.y < 0 || info.rect.y > display_size.y ||
            info.rect.width < 0 || info.rect.width > display_size.x ||
            info.rect.height < 0 || info.rect.height > display_size.y )  return true;

    }

    return false;
}

wxString FrameLayoutHandler::GetToolBarPositions()
{
    wxString msg;

    for( AuiPaneToolBarHash::iterator it = m_auiToolHash.begin(); it != m_auiToolHash.end(); it++) {
        wxString ToolBarName = it->second->GetName();
        wxAuiPaneInfo& info = m_auiMgr->GetPane(ToolBarName);
        msg += wxString::Format(_T("auipane(")+info.name+_T("):x=%i,y=%i,w=%i,h=%i\n"),info.rect.x,info.rect.y,info.rect.width,info.rect.height);
    }

    return msg;
}


void FrameLayoutHandler::ResetToolBarPositions()
{
    int c = 0;
    for( AuiPaneToolBarHash::iterator it = m_auiToolHash.begin(); it != m_auiToolHash.end(); it++) {
        wxString ToolBarName = it->second->GetName();
        wxAuiPaneInfo& info = m_auiMgr->GetPane(ToolBarName);
        info.Show(true);
        info.Position(0);
        info.Row(c);
        c++;
    }

    m_auiMgr->Update();
}

void FrameLayoutHandler::UpdateLayout(const wxVariantHash& params)
{
    Logger::logf(LOG_MSG,_T("FrameLayoutHandler::UpdateLayout() !!!!!!!!!!!!!!!"));

    const wxVariantHash::const_iterator it = params.find(_T("size_diff"));
    wxSizeEx diff(0,0);
    if(it != params.end()) diff << it->second;

    const wxVariantHash::const_iterator it2 = params.find(_T("sender_id"));
    long id = 0;
    if(it2 != params.end()) id = it2->second.GetLong();

    for(AuiPaneToolBarHash::iterator it = m_auiToolHash.begin(); it != m_auiToolHash.end(); it++) {

        AuiPaneToolBar * paneTB = it->second;

        wxString ToolBarName = paneTB->GetName();
        wxAuiPaneInfo& info = m_auiMgr->GetPane(ToolBarName);
        wxToolBar* toolbar = paneTB->GetToolBar();

        if(!toolbar) continue;

        wxControl * ctrl = toolbar->FindControl(id);

        if(!ctrl) continue;

        if(diff.x == 0) continue;

        Logger::logf(LOG_MSG,_T("diff.x=%i"),diff.x);



        Logger::logf(LOG_MSG,_T("info.best_size.x=%i, y=%i"),info.best_size.x,info.best_size.y);
        Logger::logf(LOG_MSG,_T("paneTB->m_size.x=%i, y=%i"),paneTB->m_size.x,paneTB->m_size.y);

        info.best_size.x -= diff.x;

        paneTB->m_size.x -= diff.x;

        toolbar->Realize();
//
//        m_auiMgr->DetachPane( toolbar );
//
//        m_auiMgr->AddPane( toolbar, info );
//
//        m_auiMgr->Update();
    }
}
