#include "MTConfigHelper.h"

#include <wx/dir.h>
#include <wx/xml/xml.h>

#include "fileparser/V3iniParser.h"

#include "MTConfigChecker.h"
#include "DirHelper.h"
#include "configmanager/ConfigHelper2.h"
#include "ConfigBase.h"
#include "logger.h"
#include "FirmwareInfo.h"
#include "project/ProjectConfigHelper.h"

#include <wx/filename.h>
#include "fileparser/parseheader.h"
#include "expression/RPNParser.h"
#include "vemsprotocol/ECUInfo.h"

#ifdef COMM_MODEL_WITH_GUI
#include "commmodel/ParserHelper.h"
wxArrayString MTConfigHelper::m_commodelAddresses;
#endif

#if defined(__WINCE_STANDARDSDK__)
    wxString MTConfigHelper::s_filename = _T("vemsv3pda.ini");
#else
    wxString MTConfigHelper::s_filename = _T("vemsv3.ini");
#endif

const wxEventType FILE_PARSED_EVENT = wxNewEventType();
const wxEventType PROPER_INIFILE_NOT_FOUND = wxNewEventType();

MTConfigData* MTConfigHelper::s_mtConfigData = NULL;
wxEvtHandler* MTConfigHelper::s_eventHandler = NULL;
//FirmwareVersion* MTConfigHelper::s_firmwareVersion = new FirmwareVersion();
ObserverBase*    MTConfigHelper::m_observer = NULL;
//wxString MTConfigHelper::s_currFileName = _T("");
//wxString MTConfigHelper::m_headerFile = wxEmptyString;
wxString MTConfigHelper::m_defaultConfigFile = wxEmptyString;

MTConfigData* MTConfigHelper::getMTConfigData()
{
    return s_mtConfigData;
}

MTConfigData* MTConfigHelper::createMTConfigData()
{
    return s_mtConfigData = new MTConfigData();
}

bool MTConfigHelper::parseMTConfigDataV2( const FirmwareInfo& fwInfo, MTConfigData* mtConfigData, bool _sendEvent )
{
    LOG
    const BoolHash& detetctedParams = fwInfo.getDetectedParams();
    BoolHash bh(detetctedParams);

    /*if ( bh[_T("COM_MODEL")] == false )
    {
        return  parseMTConfigData( fwInfo, mtConfigData, _sendEvent );
    }*/

#ifdef COMM_MODEL_WITH_GUI
    // communication model - parse global h, ini, map etc
    MTConfigData* currMtconfigData = mtConfigData;

    bool isEvent = false;

    if ( !mtConfigData || (mtConfigData == s_mtConfigData ) ) {
        isEvent = true;
        currMtconfigData = s_mtConfigData;
    }
    LOGP( _T("START") )

    wxString fwStr; fwStr = _T("firmware_") + fwInfo.getFirmwareVersionAsString();
    wxString dirname = DirHelper::GetPath( dPROGRAMROOT ) + _T("commmodel\\") +fwStr;// +  _T("\\my_make.16x14\\extras.ini");

    // parse ini
    wxString iniFilePath = fwInfo.getIniFile();

    if( iniFilePath.IsEmpty() ){
        if(isEvent && s_eventHandler){
            wxCommandEvent event( PROPER_INIFILE_NOT_FOUND, wxID_ANY );
            s_eventHandler->AddPendingEvent( event );
        }
        currMtconfigData->setValid(false);
        return false;
    }
    commonParse(&bh, currMtconfigData, iniFilePath);

    ParserHelper helper( dirname, currMtconfigData );

    // parse mymake, map, global.h,eventqueue.h
    helper.doParsing();

    FirmwareVersion fwv;
    wxString version = iniFilePath.BeforeLast( _T('.') ).AfterLast( _T('-') );
    FirmwareInfo::extractFirmwareVersion( version, fwv );
    currMtconfigData->setFileVersion( fwv.toString() );

    if ( isEvent && _sendEvent ) sendEvent();

    m_commodelAddresses = helper.getRequiredAddresses();
#endif
    return true;

}

bool MTConfigHelper::parseMTConfigData(const FirmwareInfo& fwInfo, MTConfigData* mtConfigData, bool _sendEvent )
{
    MTConfigData* currMtconfigData = mtConfigData;
    bool isEvent = false;
    if(!mtConfigData || (mtConfigData == s_mtConfigData) ){
        isEvent = true;
        currMtconfigData = s_mtConfigData;
    }

    Logger::logf( LOG_MSG, _T("MTConfigHelper::parseMTConfigData() START"));
    wxString iniFilePath = fwInfo.getIniFile();

    if( iniFilePath.IsEmpty() ){
        if(isEvent && s_eventHandler){
            wxCommandEvent event( PROPER_INIFILE_NOT_FOUND, wxID_ANY );
            s_eventHandler->AddPendingEvent( event );
        }
        currMtconfigData->setValid(false);
        return false;
    }

    const BoolHash& detetctedParams = fwInfo.getDetectedParams();
    BoolHash bh(detetctedParams);
    commonParse(&bh, currMtconfigData, iniFilePath);
    parseFirmwareHeader( fwInfo, currMtconfigData );

    FirmwareVersion fwv;
    wxString version = iniFilePath.BeforeLast( _T('.') ).AfterLast( _T('-') );
    FirmwareInfo::extractFirmwareVersion( version, fwv );
    currMtconfigData->setFileVersion( fwv.toString() );

//    MTConfigChecker checker( currMtconfigData );
//    checker.check();

    if ( isEvent && _sendEvent ) sendEvent();

    return true;
}

/// For ROUND use iniFile in v3gio.cfg
void MTConfigHelper::parseMTConfigDataByRound()
{
    BoolHash* bh = new BoolHash();
    ConfigBase* mainCfg = ConfigHelper2::getConfig();
    wxString fileName = mainCfg->getValue(_T("/devices/round"),_T("iniFile"),_T(""));
    commonParse(bh, s_mtConfigData, fileName);
    //s_mtConfigData->setValid(true);
    sendEvent();
}

void MTConfigHelper::commonParse(BoolHash* bh, MTConfigData* mtConfigData, const wxString& iniFileName)
{
    ProjectConfigHelper::getProjectParams(bh);

    V3iniParser parser(mtConfigData, bh );
    if ( m_observer ) { parser.registerObserver(m_observer); }

    mtConfigData->clear();
	Logger::logf( LOG_MSG, _T("MTConfigHelper::commonParse() inifile: ")+iniFileName);
    parser.parse( DirHelper::GetPath(dCONFIG) + iniFileName );
    fillGraph(mtConfigData->getSymbolMgr());
}

void MTConfigHelper::sendEvent()
{
    s_mtConfigData->setValid(true);
    if ( s_eventHandler )
    {
        wxCommandEvent event( FILE_PARSED_EVENT, wxID_ANY );
        s_eventHandler->AddPendingEvent( event );
    }
}

wxString MTConfigHelper::getDefaultConfigFile()
{

    if( !wxFileName::FileExists(m_defaultConfigFile) ){
        return wxEmptyString;
    }
    return m_defaultConfigFile;
}

void MTConfigHelper::cleanup()
{
	delete s_mtConfigData;
	s_mtConfigData = NULL;
}

bool MTConfigHelper::parseFirmwareHeader( const FirmwareInfo& fwInfo, MTConfigData* mtConfigData )
{
    MTConfigData* currMtconfigData = mtConfigData;

    if( !mtConfigData ) { currMtconfigData = s_mtConfigData; }

    wxString headerFile = fwInfo.getHeaderFile();
    m_defaultConfigFile = fwInfo.geDefaultConfigFile();

    SymbolTable& cfgSymbol = currMtconfigData->getSymbolMgr().getCfgSymbolTable();
    cfgSymbol.clear();
    Logger::logf( LOG_MSG, _T("MTConfigHelper::parseFirmwareHeader() headerfile: ") + headerFile );
    if( !wxFileName::FileExists(headerFile) ){
        Logger::logf( LOG_MSG, _T("MTConfigHelper::parseFirmwareHeader() NOT FOUND"));
        return false;
    }
    ParseHeader headerParser( cfgSymbol );
    headerParser.parse( headerFile );
    Logger::logf( LOG_MSG, _T("MTConfigHelper::parseFirmwareHeader() FOUND size: %i"), cfgSymbol.size() );

    currMtconfigData->getSymbolMgr().addConfigTableSymbolsAsScalar(cfgSymbol);

    return true;
}

void MTConfigHelper::fillGraph(SymbolMgr& symbolMgr)
{
    RPNParser rpn;

    const SymbolTable& expTable = symbolMgr.getExpTable();
    Graph& dag = symbolMgr.getDAG();

    SymbolTable::const_iterator it = expTable.begin();
    for(; it != expTable.end(); ++it )
    {
        Symbol* aSymbol = it->second;

        StringHashSet variables = rpn.getVariables( aSymbol->getExpr() );

        for( StringHashSet::const_iterator itSet = variables.begin(); itSet != variables.end(); ++itSet )
        {
            wxString name = *itSet;
            name = name.BeforeFirst('[');

            Symbol* vSymbol = symbolMgr.getSymbol( name );

            if( !vSymbol ) continue;

            if( vSymbol->isExpr() )
            {
                aSymbol->insertNode( vSymbol );
            }
        }
        dag.addNode( aSymbol );
    }
    dag.setupOrder();
}


bool MTConfigHelper::parseAIMConfig(MTConfigData* mtConfigData)
{
    MTConfigData* currMtconfigData = mtConfigData;

    if( !mtConfigData ) { currMtconfigData = s_mtConfigData; }

    BoolHash bh;

    wxString iniFileName = _T("vemsTune-aim.ini");

    V3iniParser parser(mtConfigData, &bh );
    if ( m_observer ) { parser.registerObserver(m_observer); }

    mtConfigData->clear();

	Logger::logf( LOG_MSG, _T("MTConfigHelper::parseAIMConfig() inifile: ")+iniFileName);

    parser.parse( DirHelper::GetPath(dCONFIG) + iniFileName );

    fillGraph(mtConfigData->getSymbolMgr());

    return true;
}
