#ifndef SYMBOL_H
#define SYMBOL_H

#include <wx/string.h>
#include <wx/arrstr.h>
#include <wx/hashmap.h>

#include "dag/GraphNode.h"

struct SymbolGroup
{
    wxString    name;
    int         offset;

    SymbolGroup( const wxString groupName, int groupOffset );
};

class Symbol : public GraphNode
{
public:
    Symbol();
    //Symbol(const Symbol& s);
    virtual ~Symbol();
    //set methods
    void setAlias( const wxString& name ) { m_alias = name; }
    void setData(double d) { m_data = d; }
    void setBasic( const wxString& name, const wxString& units,bool _const, int pageNo, int offset );
    void setFormat(double scale, double trans, double lo, double hi, int digits);
    void setType(bool isSigned, int sizeOf);
    void setArray(bool xOrder, int order, int dim1, int dim2);
    void setBit(int bitLo, int bitHi, int bitOfs, wxArrayString bitFields);
    void setExpr( const wxString& name, const wxString& expr );
    void setExprReverse( const wxString& expr ) { m_exprReverse = expr; }
    void setTarget( const wxString& target ) { m_target = target; }
    void setGroup( SymbolGroup* group ) { m_group = group; }
    void setChannel( int ch ) { m_channel = ch; }
    void setValid( bool isValid ) { m_isValid = isValid; }
    void setTitle( const wxString &title) { m_title = title; }
    void setGaugeType( const wxString &gt) { m_gaugeType = gt; }
    void setInputType( int type ) { m_inputType = type ; }
    void setConst( bool isConst ) { m_const = isConst; }
    void setExpressionsForTransformations( const wxString& fromRaw, const wxString& toRaw );

    //get methods
    wxString    getTitle() const { return m_title; }
    wxString    getGaugeType() const { return m_gaugeType; }

    wxString    getAlias() const { return m_alias; }
    wxString    getExpr() const { return m_expr; }
    wxString    getExprReverse() const { return m_exprReverse; }
    wxString    getTarget() const { return m_target; }
    double      getData() const { return m_data; }
    int         getPage() const { return m_pageNo; }
    wxString    getType();
    wxString    getUnit() const { return m_units; }
    int         getSize() const { return m_sizeOf;}
    int         getOffset() const;
    double      getScale() const { return m_scale; }
    double      getTranslation() const { return m_trans; }
    int         getBitLo() const { return m_bitLo; }
    int         getBitHi() const { return m_bitHi; }
    int         getBitMask() const;
    int         getBitOfs() const { return m_bitOfs; }
    int         getDigits() const { return m_digits; }
    wxArrayString getBits() const { return m_bitFields; }
    bool        getxOrder() const { return  m_xOrder; }
    int         getOrder() const { return m_order; }
    int         getShape0() const { return m_shape[0]; }
    int         getShape1() const { return m_shape[1]; }
    int         getChannel() const { return m_channel; }
    double      getHi() const { return m_hi; }
    double      getLo() const { return m_lo; }
    bool isValid() const { return m_isValid; }
    bool        hasGauge();
    int         getInputType() const { return m_inputType; }
    //
    void    setPage( int page ) { m_pageNo = page; }
    void    setOffset( int offset ) { m_offset = offset; }
    void    setArrayColSize( int size ) { m_shape[0] = size; }
    void    setArrayRowSize( int size ) { m_shape[1] = size; }
    void    setArrayDimension( int dim ) { m_order = dim; }
    int     getArrayColSize() const { return m_shape[0]; }
    int     getArrayRowSize() const { return m_shape[1]; }
    int     getArrayDimension() const { return m_order; }
    int     getArraySize() const { return m_shape[0] * m_shape[1]; }
    wxString getExprFromRaw() const { return m_exprFromRaw; }
    wxString getExprToRaw() const { return m_exprToRaw; }

    bool isConst() const { return m_const; }
    //bool isScalar() const { return m_order == 0 && !isExpr()?true:false; }
    bool isScalar() const { return !isExpr() && m_bitLo == -1 && m_order == 0; }
    bool isExpr() const { return m_expr != _T(""); }
    bool isBits() const { return m_bitLo != -1; }
    bool isArray() const { return m_order != 0; }
    bool isVemsCfgArray() const;
    bool isVemsCfgArrayAxis() const;
    bool isPecialArray(const wxString& rawName, wxString& name, long& row) const;

    bool useDefaultTransformation() const { return ( m_exprFromRaw.IsEmpty() && m_exprToRaw.IsEmpty() ); }
    static Symbol* createArray( const wxString& name, int x, int y, int page, int offset );
    static Symbol* createScalar( const wxString& name, int page, int offset, int type = -1 );
    //utility, test methods
    wxString toString();

    void setValidationExpr( const wxString& expr ) { m_validationExpr = expr; }
    wxString getValidationExpr() const { return m_validationExpr; }
private:
    double      m_data; //testing

    wxString    m_title;
    wxString    m_gaugeType;
    wxString    m_alias;
    wxString    m_units;
    bool        m_const; // True means in constants, false in output channels/variables.
    int         m_pageNo; // Zero-based index into tables.
    int         m_offset; // Offset of first byte in this table.
    int         m_sizeOf; // Number of bytes in one value, 1, 2 or 4 are valid.
    bool        m_signed; // Is data signed or unsigned?

    //--  User-units formatting information.  -----------------------------------
    double      m_scale;   // Scale of raw value.
    double      m_trans;   // Translation of raw value.
    int         m_digits;  // Display digits to right of decimal.
    //double      m_round;   // Fudge factor based upon digits count.
    double      m_lo;
    double      m_hi; // Limit values.
    //long        m_loRaw, m_hiRaw; // Limits in internal format.

    //--  Data for array types.  ------------------------------------------------
    bool        m_xOrder;   // Is table in x- or y-order?
    int         m_order;    // Tensor order, 0=scalar.
    int         m_shape[2]; // Size in up to two dimensions.

    //--  Data for bit fields.  -------------------------------------------------
    int         m_bitLo;    // Starting bit in a byte-base bitfield, zero-based.
    int         m_bitHi;    // Ending bit in bitfield.
    int         m_bitOfs;   // Offset for user value when extracted from bits.
                            // For instance, MS-I stores nCylinders in 0:3 as 1..16,
                            // so valueUser returns 0+1==1 for the first value.
    wxArrayString   m_bitFields;
    wxString        m_expr;
    wxString        m_exprReverse;      // reverse expression for const expressions
    wxString        m_target;       // reverse expression target for const expression
    SymbolGroup*    m_group;
    int             m_channel;

    bool            m_isValid;         //use this in the firmware config upload
    int             m_inputType;
    wxString        m_exprFromRaw;
    wxString        m_exprToRaw;
    wxString        m_validationExpr;

    wxString basicToString();
    wxString formatToString();
    wxString bitToString();
    wxString arrayToString();
    wxString exprToString();
};

//WX_DEFINE_ARRAY(Symbol*, SymbolOrder); // good idea?
WX_DECLARE_STRING_HASH_MAP( Symbol*, SymbolTable );
WX_DECLARE_STRING_HASH_MAP( SymbolGroup*, SymbolGroupHash );
WX_DEFINE_ARRAY( Symbol*, SymbolArray );

#endif
