#include "TCPClientHandler.h"
#include "SerialException.h"

#include "UnicodeHelper.h"
#include "logger.h"
#include "Math.h"

//#include <wx/ctb/timer.h>

BEGIN_EVENT_TABLE(TCPClientHandler, wxEvtHandler)

    EVT_SOCKET(isSocketClient, TCPClientHandler::OnSocket)

END_EVENT_TABLE()

TCPClientHandler::TCPClientHandler(const wxString &host, unsigned short port)
{
    m_sockClient = new wxSocketClient();

    m_address.Hostname( host );
    m_address.Service( port );
}

int TCPClientHandler::open()
{
    if (m_sockClient->IsConnected()) return 0;

    ///TODO SET TIMEOUT
    LOG
    m_sockClient->SetFlags( wxSOCKET_NONE );
    m_sockClient->Connect( m_address );
    LOG

    if ( !m_sockClient->IsConnected() ) {
        throw SerialException(SP_CANNOT_OPEN);
    }

    m_sockClient->SetFlags( wxSOCKET_NOWAIT ) ;

    return 0;
}

bool TCPClientHandler::IsOpen()
{
    return m_sockClient->IsConnected();
}

TCPClientHandler::~TCPClientHandler()
{
    m_sockClient->Close();
    delete m_sockClient;
}

void TCPClientHandler::OnSocket(wxSocketEvent& event)
{
    wxSocketBase *sock = event.GetSocket();

    //Logger::logf(LOG_MSG,_T("TCPClientHandler::OnSocket"));

    switch (event.GetSocketEvent())
    {
        case wxSOCKET_INPUT:
        {

            sock->SetNotify(wxSOCKET_LOST_FLAG);

            //char * c = new char[co];
            char c;//[100];

            sock->Read(&c, 1);
            {
                wxCriticalSectionLocker locker(m_criticialSection);
                m_buff.Add(c);
            }

            // Enable input events again.
//            sock->SetNotify(wxSOCKET_LOST_FLAG | wxSOCKET_INPUT_FLAG);

            //Logger::logf(LOG_MSG,_T("TCPClientHandler::OnSocket buffsize= %i"), m_buff.GetCount() );

            break;
        }
        case wxSOCKET_LOST:
        {
            Logger::logf(LOG_MSG,_T("TCPClientHandler::OnSocket wxSOCKET_LOST"));

//            sock->Destroy();
            break;
        }
        default:
            break;
    }

}

int TCPClientHandler::SendString(const wxString& str)
{
    const char * c = UnicodeHelper::U2C(str);
    int res = SendString( (uint8_t*)c );
    delete (char *)c;
    return res;
}

int TCPClientHandler::SendString(const uint8_t* chr)
{
    return SendString(chr,  strlen((char*)chr));
}

int TCPClientHandler::SendString(const uint8_t* str, size_t len)
{
    //Logger::logf(LOG_MSG,_T("TCPClientHandler::SendString: %s"),str);

    m_sockClient->Write( str, len);
    return m_sockClient->LastCount();
}

int TCPClientHandler::sendStringBlock(uint8_t* value, size_t len, size_t blockLen)
{
    int result = 0;
    size_t i = 0;
    size_t currBlock;
    for(i=0; i< len; i += blockLen){
        currBlock = len - i;
        if( currBlock > blockLen ){
            currBlock = blockLen;
        }
        result += SendString( (&(value[i])), currBlock);
    }
    return result;
}

int TCPClientHandler::Read(uint8_t* str, size_t len)
{
    m_sockClient->Read( str, len );

    if ( m_sockClient->LastCount() == 0 ) {
//        LOGP( wxString::Format( _T("m_sockClient->LastCount(): %d"), m_sockClient->LastCount() ) );
        wxThread::Sleep(1);
    }

    return m_sockClient->LastCount();
}

//int TCPClientHandler::Read(uint8_t* str, size_t len)
//{
//    int timeout = getTimeOut(len);
//
//    //timer t(timeout, &to, NULL);
//    //t.start();
//
//    //Logger::logf(LOG_MSG,_T("timeout=%i"),timeout);
//
//    wxCriticalSectionLocker locker(m_criticialSection);
//
//
//
//    while( (m_buff.GetCount() < len) ) {
////        LOG
//        wxThread::Sleep( 1 );
//        timeout--;
//        if( !timeout ) break;
//        //Logger::logf(LOG_MSG,_T("h"));
//    }
//
//    //Logger::logf(LOG_MSG,_T("TCPClientHandler::ReadString bufflen=%i, reqlen=%i"), m_buff.GetCount(), len);
//
//
////    LOGP( wxString::Format( _T("m_buff.GetCount(): %d"), m_buff.GetCount() ) );
//
//    if( m_buff.GetCount() == 0 ) {
//        return 0;
//    }
//
//    if( len > m_buff.GetCount() )
//        len = m_buff.GetCount();
//
//    for(size_t a=0; a < len ;a++){
//        str[a] = m_buff[a];
//    }
//
//    m_buff.RemoveAt(0, len);
//
//    return len;
//}

int TCPClientHandler::ReadString(uint8_t* str, size_t len)
{
    // int to = 0;

    int timeout = getTimeOut(len);

    //timer t(timeout, &to, NULL);
    //t.start();

    //Logger::logf(LOG_MSG,_T("timeout=%i"),timeout);

    while( (m_buff.GetCount() < len) ) {
        wxThread::Sleep( 1 );
        timeout--;
        if( !timeout ) break;
        //Logger::logf(LOG_MSG,_T("h"));
    }

    //Logger::logf(LOG_MSG,_T("TCPClientHandler::ReadString bufflen=%i, reqlen=%i"), m_buff.GetCount(), len);


    if( m_buff.GetCount() == 0 ) {
        return 0;
    }

    if( len > m_buff.GetCount() )
        len = m_buff.GetCount();

    for(size_t a=0; a < len ;a++){
        str[a] = m_buff[a];
    }

    m_buff.RemoveAt(0, len);

    return len;
}

int TCPClientHandler::ReadString(uint8_t* data, size_t len, int maxTryCount)
{
    return ReadString( data, len );
}

int TCPClientHandler::ReadUntilEOS(char ** readbuf, char * eosString , long timeout_in_ms)
{

    bool isend = false;
    size_t len = 0;

    for(size_t a=0;a<m_buff.GetCount();a++){

        for(size_t b=a;b<strlen(eosString);b++){
            if( m_buff[b] != eosString[b] ) {
                isend = false;
                break;
            }
            isend = true;
        }

        if(isend) break;

        (*readbuf)[a] = m_buff[a];
        len++;
    }

    return len;
}

void TCPClientHandler::setBaudRate( int baudRate )
{
}

int TCPClientHandler::getBaudRate()
{
    return -1;
}

wxString TCPClientHandler::getPortName() const
{
    wxString hostName = m_address.Hostname();
    wxString service = wxString::Format( _T("%d"), m_address.Service() );

    return hostName + _T(":") + service;
}

int TCPClientHandler::sendNumber(uint32_t number, int length)
{
    uint8_t str[4];
    Math::getBytes(number, str, length);
    return SendString(str, length);
}

int TCPClientHandler::sendNumber(uint16_t number)
{
    uint8_t str[2];

    str[0] = (number >> 8) &0xFF;
    str[1] = number & 0xFF;

    return SendString( str, 2);
}

void TCPClientHandler::close()
{
    m_sockClient->Close();
}

int TCPClientHandler::clear()
{
    m_buff.Clear();
    return 1;
}

int TCPClientHandler::getTimeOut(size_t len)
{
    return (int)(len * 1 );

    //return ((len*10)>>3) + 12; //9600 10000/baudRate

    //return ((len*10000)/m_baudrate) + 12;
}


/*
      char c[256];
      sock->Read( c, 256);

      //buff.AppendData( c, sock->LastCount() );

      m_serialDev->Write( c, sock->LastCount() );
      */
