#include "ThemesHelper.h"

#include <wx/xml/xml.h>

#include "DrawHelper.h"
#include "DirHelper.h"
#include "WidgetThemeTable.h"
#include "common/typedef.h"

wxString ThemesHelper::m_widgetThemeFile = _T("WidgetThemes.xml");
wxString ThemesHelper::m_defaultThemeName = _T("Gold");
WidgetThemeTableArray ThemesHelper::m_widgetThemes;

ThemesHelper::ThemesHelper()
{
    //ctor
}

ThemesHelper::~ThemesHelper()
{
    //dtor
}

bool ThemesHelper::parseWidgetThemeFile()
{



    wxXmlDocument doc;

    if( !doc.Load(DirHelper::GetPath(dCONFIG) + m_widgetThemeFile) )
        return false;

    if(doc.GetRoot()->GetName() != _T("WidgetThemes")){
        return false;
    }


    m_defaultThemeName = doc.GetRoot()->GetPropVal(_T("default"),_T("Gold"));       /// todo: default téma osztály bedrótozása

    wxXmlNode* ThemeNode  = doc.GetRoot()->GetChildren();

    while(ThemeNode){
        if(ThemeNode->GetName() != _T("Theme")){
            ThemeNode = ThemeNode->GetNext();
            continue;
        }

        WidgetThemeTable *theme = new WidgetThemeTable();


        theme->Name = ThemeNode->GetPropVal(_T("name"), wxEmptyString);




        //while(ThemeDetails){
        for(wxXmlNode* ThemeDetails = ThemeNode->GetChildren(); ThemeDetails ; ThemeDetails = ThemeDetails->GetNext() ){


            if(ThemeDetails->GetName() == _T("ScaleRatios")){

                ThemeDetails->GetPropVal(_T("borderwith"),_T("0.1")).ToDouble(&(theme->BorderWidthRatio));


            } else if(ThemeDetails->GetName() == _T("TextColors")){

                theme->TitleColor   = DrawHelper::ParseExtColorStr(ThemeDetails->GetPropVal(_T("title"),wxEmptyString));
                theme->DivTextColor = DrawHelper::ParseExtColorStr(ThemeDetails->GetPropVal(_T("div"),wxEmptyString));
                theme->ValueColor   = DrawHelper::ParseExtColorStr(ThemeDetails->GetPropVal(_T("value"),wxEmptyString));
                theme->SuffixColor  = DrawHelper::ParseExtColorStr(ThemeDetails->GetPropVal(_T("suffix"),wxEmptyString));



            }else if(ThemeDetails->GetName() == _T("FaceColors")){

                theme->BackgroundColor  = DrawHelper::ParseExtColorStr(ThemeDetails->GetPropVal(_T("background"),wxEmptyString));
//                theme->WarningColor     = DrawHelper::ParseExtColorStr(ThemeDetails->GetPropVal(_T("warning"),wxEmptyString));
//                theme->DangerColor      = DrawHelper::ParseExtColorStr(ThemeDetails->GetPropVal(_T("danger"),wxEmptyString));
                theme->PointerColor     = DrawHelper::ParseExtColorStr(ThemeDetails->GetPropVal(_T("pointer"),wxEmptyString));
                theme->DivColor         = DrawHelper::ParseExtColorStr(ThemeDetails->GetPropVal(_T("div"),wxEmptyString));
                theme->InnerButtonColor = DrawHelper::ParseExtColorStr(ThemeDetails->GetPropVal(_T("innerbutton"),wxEmptyString));
                theme->BorderColor      = DrawHelper::ParseExtColorStr(ThemeDetails->GetPropVal(_T("border"),wxEmptyString));
                theme->BorderBeginColor = DrawHelper::ParseExtColorStr(ThemeDetails->GetPropVal(_T("border_b"),wxEmptyString));
                theme->BorderEndColor   = DrawHelper::ParseExtColorStr(ThemeDetails->GetPropVal(_T("border_e"),wxEmptyString));


            }else if(ThemeDetails->GetName() == _T("VemsGaugeDigital")){

                long tmp;

                ThemeDetails->GetPropVal(_T("bmpborder"),_T("1")).ToLong(&tmp);
                theme->BmpBorder = tmp!=0?true:false;


            }else if(ThemeDetails->GetName() == _T("Showns")){

                long tmp;

                ThemeDetails->GetPropVal(_T("pointerAA"),_T("0")).ToLong(&tmp);
                theme->PointerAA = tmp!=0?true:false;
                ThemeDetails->GetPropVal(_T("border3d"),_T("1")).ToLong(&tmp);
                theme->Border3D = tmp!=0?true:false;
                ThemeDetails->GetPropVal(_T("innerbutton3d"),_T("1")).ToLong(&tmp);
                theme->InnerButton3D = tmp!=0?true:false;

            }else if(ThemeDetails->GetName() == _T("ColorDefine")){

                wxString name = ThemeDetails->GetPropVal(_T("name"),wxEmptyString);
                wxColor color = DrawHelper::ParseExtColorStr(ThemeDetails->GetPropVal(_T("value"),wxEmptyString));

                if(name == wxEmptyString && !color.IsOk() ) continue;

                theme->ColorSymbols[name] = color;

            }else if(ThemeDetails->GetName() == _T("ExpressionGrid")){
                theme->LinkTextColor = DrawHelper::ParseExtColorStr(ThemeDetails->GetPropVal(_T("link"),wxEmptyString));
                theme->VariableBackgroundColor = DrawHelper::ParseExtColorStr(ThemeDetails->GetPropVal(_T("variableBackground"),wxEmptyString));
                theme->InactiveBackgroundColor = DrawHelper::ParseExtColorStr(ThemeDetails->GetPropVal(_T("inactiveBackground"),wxEmptyString));
                theme->InactiveTextColor = DrawHelper::ParseExtColorStr(ThemeDetails->GetPropVal(_T("inactive"),wxEmptyString));
            }


        }



        ThemeNode = ThemeNode->GetNext();

        m_widgetThemes.Add(theme);

    }

    return true;

}

void ThemesHelper::GetThemeNameList( wxArrayString &arr )
{
    for(uint8_t c = 0; c < m_widgetThemes.GetCount(); c++) {
        arr.Add(m_widgetThemes[c]->Name);
    }
}

wxMenu * ThemesHelper::CreateThemeMenu(const WidgetThemeTableArray *themes, wxEvtHandler *target, wxObjectEventFunction func, const wxString& chkTheme, const wxString& grp, ThemeGroupPairHash *grph)
{

    wxMenu *tmenu = new wxMenu();

    for(uint8_t c = 0; c < themes->GetCount(); c++) {
        int id = wxNewId();
        tmenu->AppendRadioItem(id, themes->Item(c)->Name);

        if(chkTheme == themes->Item(c)->Name)
            tmenu->Check(id,true);

        target->Connect( id, wxEVT_COMMAND_MENU_SELECTED, func );

        if( grph == NULL ) continue;

        ThemeGroupPair *tgp = new ThemeGroupPair();
        tgp->Group = grp;
        tgp->Theme = themes->Item(c)->Name;
        (*grph)[id] = tgp;

        if(chkTheme == themes->Item(c)->Name)
            tgp->Checked = true;
        else
            tgp->Checked = false;

    }

    return tmenu;

}

void ThemesHelper::GetDefinedColornames(const WidgetThemeTableArray *arr, const wxString themename, wxArrayString &colornames)
{

    for(size_t a=0;a<arr->GetCount();a++){
        if( arr->Item(a)->Name != themename ) continue;

        for( wxColorHash::iterator it = arr->Item(a)->ColorSymbols.begin(); it != arr->Item(a)->ColorSymbols.end(); it ++ )
            colornames.Add(it->first);

        return;
    }
}
