#include "VemsMsgListCtrl.h"

#include "common/VemsMessage.h"
#include "configmanager/ConfigHelper2.h"

#include "logger.h"
#include "VemsGuideHtmlWindow.h"
#include "DirHelper.h"
#include <wx/imaglist.h>
#include "vemsdialog/UserDialogBase.h"


WX_DEFINE_OBJARRAY(VemsMessageArray);

BEGIN_EVENT_TABLE(VemsMsgListCtrl, wxFrame)

    EVT_LIST_ITEM_SELECTED(idListCtrl, VemsMsgListCtrl::OnItemSelected)

END_EVENT_TABLE()

VemsMsgListCtrl::VemsMsgListCtrl(UserDialogBase* userDialog, wxWindow * parent, int id, const wxString &title)
{
    long x;
    long y;
    long width;
    long height;
    wxString pers;

    ConfigBase* configBase = ConfigHelper2::getConfig(ConfigHelper2::CONFIG_LAYOUT);
    configBase->setPath(_T("/layout/vemsmsgctrl"));
    configBase->read(_T("x"),&x,50);
    configBase->read(_T("y"),&y,50);
    configBase->read(_T("width"),&width,50);
    configBase->read(_T("height"),&height,50);
    configBase->read(_T("perspective"),&pers, wxEmptyString);


    if(parent == NULL)
        parent = userDialog->GetDialogParent();

    Create( parent, id, title, wxPoint(x,y), wxSize(width, height), wxDEFAULT_FRAME_STYLE | wxFRAME_FLOAT_ON_PARENT );

    SetIcon(wxIcon(_T("vemsLogoIcon2")));

    m_shortMsgList = new wxListCtrl(this, idListCtrl, wxDefaultPosition, wxDefaultSize, wxLC_REPORT | wxLC_SINGLE_SEL);
    m_htmlMsg = new VemsGuideHtmlWindow(userDialog, this, wxID_ANY);

    m_auiMgr = new wxAuiManager(this, wxAUI_MGR_ALLOW_ACTIVE_PANE | wxAUI_MGR_TRANSPARENT_HINT | wxAUI_MGR_HINT_FADE | wxAUI_MGR_NO_VENETIAN_BLINDS_FADE );

    wxAuiPaneInfo info1;

    info1.Resizable( 1 );
    info1.CloseButton( 0 );
    info1.Centre();
    info1.Name(_T("msglistbox"));
    info1.Caption( _T("Message list box") );
    m_auiMgr->AddPane( m_shortMsgList, info1);

    info1.Name(_T("msgdetails"));
    info1.Caption( _T("Message details") );

    m_auiMgr->AddPane( m_htmlMsg, info1);
    m_auiMgr->LoadPerspective( pers );
    m_auiMgr->Update();

    Init();

}

VemsMsgListCtrl::~VemsMsgListCtrl()
{
    if ( IsIconized() ) return;

    ConfigBase* configBase = ConfigHelper2::getConfig(ConfigHelper2::CONFIG_LAYOUT);
    configBase->setPath(_T("/layout/vemsmsgctrl"));
    configBase->write(_T("x"),GetPosition().x);
    configBase->write(_T("y"),GetPosition().y);
    configBase->write(_T("width"),GetSize().x);
    configBase->write(_T("height"),GetSize().y);
    configBase->write(_T("perspective"), m_auiMgr->SavePerspective() );
    configBase->flush();
    configBase->save();
    //dtor
}

VemsMsgListCtrl* VemsMsgListCtrl::deleteAndCreate(UserDialogBase* userDialog, const wxString& name)
{
    wxWindow* win = FindWindowByName( name );
    if ( win != NULL ){
        Logger::logf(LOG_MSG, _T("VemsMsgListCtrl:findAndShow Destroy old one") );
        win->Destroy();
    }
    VemsMsgListCtrl * messageViewer = new VemsMsgListCtrl(userDialog, NULL, wxID_ANY, name );

    return messageViewer;
}

void VemsMsgListCtrl::Init()
{

    int w = m_shortMsgList->GetSize().x-4;

    wxListItem info1;
    wxListItem info2;
    wxListItem info3;

    info1.SetWidth( 25 );
    info1.SetAlign(wxLIST_FORMAT_LEFT);
    info1.SetText(_T("# "));

    info2.SetWidth( 150 );
    info2.SetAlign(wxLIST_FORMAT_LEFT);
    info2.SetText(_T("Name"));

    info3.SetWidth( (w-25-150) );
    info3.SetAlign(wxLIST_FORMAT_LEFT);
    info3.SetText(_T("Message"));

    m_shortMsgList->InsertColumn(0,info1);
    m_shortMsgList->InsertColumn(1,info2);
    m_shortMsgList->InsertColumn(2,info3);

    wxImageList * list = new wxImageList(16,16,true);

    wxBitmap bmp1(16,16);
    bmp1.LoadFile( DirHelper::GetPath(dICON) + _T("ul-warning-m.png"), wxBITMAP_TYPE_PNG );

    wxBitmap bmp2(16,16);
    bmp2.LoadFile( DirHelper::GetPath(dICON) + _T("ul-error-m.png"), wxBITMAP_TYPE_PNG );

    wxBitmap bmp3(16,16);
    bmp3.LoadFile( DirHelper::GetPath(dICON) + _T("ul-info-m.png"), wxBITMAP_TYPE_PNG );

    list->Add( bmp1 );
    list->Add( bmp2 );
    list->Add( bmp3 );

    m_imgHandler[VMT_WARNING] = 0;
    m_imgHandler[VMT_ERROR] = 1;
    m_imgHandler[VMT_INFO] = 2;

    m_shortMsgList->SetImageList( list, wxIMAGE_LIST_SMALL );

}

void VemsMsgListCtrl::show(const VemsMessage& message)
{
    m_msgList.Add( message );

    AddLine( message );

}

void VemsMsgListCtrl::AddLine( const VemsMessage& message )
{

    wxListItem item0, item1, item2;

    int i = m_shortMsgList->GetItemCount();

    item0.SetId( i );
    item1.SetId( i );
    item2.SetId( i );

    item0.SetColumn(0);
    item1.SetColumn(1);
    item2.SetColumn(2);


    item0.SetMask( wxLIST_MASK_IMAGE );
    //item0.SetText(wxString::Format(_T("%i"),i));
    item0.SetImage( m_imgHandler[ message.getType() ] );
    item1.SetText(message.getName());
    item2.SetText(message.getMsg());

    m_shortMsgList->InsertItem(item0);
    m_shortMsgList->SetItem(item1);
    m_shortMsgList->SetItem(item2);

}

void VemsMsgListCtrl::OnItemSelected(wxListEvent &evt)
{
    int idx = evt.GetIndex();
    //Logger::logf(LOG_MSG,_T("%i"),idx);

    m_htmlMsg->SetPage( m_msgList.Item( idx ).getHtmlMsg() );
}
