#include "VemsTuneCore.h"

#include "DirHelper.h"
#include "logger.h"

#include "MTConfigHelper.h"
#include "RawLogHelper.h"
#include "FirmwareInfo.h"

#include "loghistory/LogHistoryMgr.h"

#include "rtdatamanager/RTDataMgrFacade.h"
#include "rtdatamanager/RTDataInputSIPR.h"
#include "stdatamanager/STDataMgrFacade.h"


#include "gaugedesc/GaugeDescManager.h"
#include "gaugedesc/GaugeDescManagerSQL.h"
#include "gaugedesc/GaugeProperties.h"

#include "vemsgauge/GaugeType.h"
#include "vemsgauge/GaugeCarrierFrame.h"
#include "vemsgauge/GaugeGroupManager.h"

#include "vemsgauge/GaugeGroupManagerSQL.h"

#include "vemsdialog/UserDialogManager.h"
#include "vemsdialog/UserMenuRefresher.h"

#include "vemsprotocol/CommandRequestBase.h"
#include "vemsgauge/VemsWidgetBaseFactory.h"
#include "vemsgauge/TableEditorFactory.h"

VemsTuneCore::VemsTuneCore(wxVariantHash& properties)
    :m_mtConfigData (NULL)
    ,m_rtViewerModel (NULL)
    ,m_rtMgr (NULL)
    ,m_inputSIPR (NULL)
    ,m_stMgr (NULL)
    ,m_myModelSerial (NULL)
    ,m_deviceModel (NULL)
    ,m_logHistoryMgr (NULL)
    ,m_gaugeDescMgr (NULL)
    ,m_gaugeGroupManager (NULL)
    ,m_carrier (NULL)
    ,m_userDialogManager (NULL)
    ,m_pUserMenuRefresher (NULL)
{
    //ctor
    m_properties = properties;
}

VemsTuneCore::~VemsTuneCore()
{
    LOG
    //dtor
}


void VemsTuneCore::CleanUp()
{
    delete m_mtConfigData;
    delete m_rtViewerModel;
    //delete m_rtMgr;
    delete m_inputSIPR;
    delete m_stMgr;
    delete m_myModelSerial;
    delete m_deviceModel;
    delete m_logHistoryMgr;
    //delete m_gaugeGroupManager;
    delete m_gaugeDescMgr;
    //delete m_userDialogManager;
    //delete m_pUserMenuRefresher;

    LOG
}

void VemsTuneCore::OnCommand(wxCommandEvent& evt)
{

    if(evt.GetId() == 42) {
        //CleanUp();
        //delete this;
    }

}

void VemsTuneCore::Init()
{
    wxObject* obj = m_properties[_T("parent")].GetWxObjectPtr();
    wxWindow* parent = dynamic_cast<wxWindow*>(obj);
    if(!parent) return;

    if(m_properties.find(_T("logfile")) == m_properties.end()) return;

    wxString logfile = m_properties[_T("logfile")].GetString();

    m_mtConfigData = new MTConfigData();
    SymbolMgr* symbolMgr = &(m_mtConfigData->getSymbolMgr());

    m_rtViewerModel = new DoubleHash();

    m_rtMgr = new RTDataMgrFacade();

    m_inputSIPR = new RTDataInputSIPR( symbolMgr, m_rtViewerModel );

    m_rtMgr->registerInput( m_inputSIPR );
    m_rtMgr->selectInput( INPUT_TYPE_SIPR );

    m_stMgr = new STDataMgrFacade();

    m_myModelSerial = new STDataModelSerial( m_mtConfigData->getSymbolTable(), m_mtConfigData->getSIPRDescriptor() );
    m_stMgr->setModel( m_myModelSerial );

    m_deviceModel = new STDataModelSerial( m_mtConfigData->getSymbolTable(), m_mtConfigData->getSIPRDescriptor() );
    m_stMgr->setDeviceModel( m_deviceModel );

    m_logHistoryMgr = new LogHistoryMgr(m_rtMgr,m_stMgr,symbolMgr);

    m_gaugeDescMgr = new GaugeDescManagerSQL(m_mtConfigData);

    m_gaugeDescMgr->Init();

    m_gaugeGroupManager = new GaugeGroupManagerSQL(parent, new wxMenu(), m_mtConfigData, m_logHistoryMgr );
    m_gaugeGroupManager->SetGaugeDescMgr( m_gaugeDescMgr );
    m_gaugeDescMgr->SetGaugeGroupMgr( m_gaugeGroupManager );

    m_logHistoryMgr->loadHistoryFromLog( logfile );
    FirmwareInfo fwinfo;

    RawLogHelper::fillFirmwareInfo(fwinfo, m_logHistoryMgr->getHeaderHash());

    MTConfigHelper::parseMTConfigData( fwinfo, m_mtConfigData, false );


    m_logHistoryMgr->initST(*m_mtConfigData->getSIPRDescriptor() );


    m_gaugeDescMgr->LoadDescriptors( fwinfo.getDevice() );

    wxMenuBar* mbar = new wxMenuBar();

    m_carrier = new GaugeCarrierFrame(parent, wxID_ANY, logfile);

    m_userDialogManager = new UserDialogManager(    m_carrier,
                                                    m_mtConfigData,
                                                    m_stMgr,
                                                    m_rtMgr,
                                                    new CommandRequestDummy(),
                                                    mbar,
                                                    m_gaugeDescMgr,
                                                    fwinfo.getFirmwareVersionAsString(),
                                                    0 );



    VemsWidgetBaseFactory* vwFactory = new VemsWidgetBaseFactory();

    vwFactory->SetDataMgrObj( m_rtMgr );
    vwFactory->SetDataMgrObj( m_stMgr );
    vwFactory->SetGaugeDescManager( m_gaugeDescMgr );
    vwFactory->SetGaugeGroupManager( m_gaugeGroupManager );
    vwFactory->SetLogHistoryBase( m_logHistoryMgr );

    m_carrier->SetFactory( vwFactory );


    TableEditorFactory* tableFactory = new TableEditorFactory();

    tableFactory->SetRTDataMgrBase( m_rtMgr );
    tableFactory->SetSTDataMgrBase( m_stMgr );
    tableFactory->SetGaugeDescManager( m_gaugeDescMgr );
    tableFactory->SetGaugeGroupManager( m_gaugeGroupManager );
    tableFactory->SetMTConfigData( m_mtConfigData );

    m_carrier->SetFactory( tableFactory );


    m_pUserMenuRefresher = new UserMenuRefresher( m_userDialogManager, mbar );
    m_stMgr->RegisterViewer( m_pUserMenuRefresher );

//    m_carrier->SetGroupMgr( m_gaugeGroupManager );
    m_carrier->OnlyOneGauge( true );
//    m_carrier->SetDataMgrObj( m_rtMgr );
//    m_carrier->SetDataMgrObj( m_stMgr );
//    m_carrier->SetLogHistory( m_logHistoryMgr );



    m_carrier->SetShaped(false);

    m_carrier->SetMenuBar(mbar);

    m_carrier->IsUnregViewers = false;

    wxStatusBar * sbar = m_carrier->CreateStatusBar(3);
    sbar->SetStatusText( _T("Firmware version: ") + fwinfo.getFirmwareVersionAsString() , 1 );
    sbar->SetStatusText( _T("Config version: ") + m_mtConfigData->getFileVersion(), 2 );

    GaugeProperties prop;

    prop.x              = 0;
    prop.y              = 0;
    prop.width          = 800;
    prop.height         = 600;
    prop.gaugeType      = m_properties.find(_T("gaugetype")) == m_properties.end() ? GaugeType::MULTIGRAPH : m_properties[_T("gaugetype")].GetLong();
    prop.themeName      = _T("1");
    prop.descName       = _T("1");

    m_carrier->AddGauge( prop );

    m_carrier->ShowCarrier(true);

    m_logHistoryMgr->refreshLastPosition();

    m_carrier->Connect(42,wxEVT_COMMAND_ENTER, wxCommandEventHandler(VemsTuneCore::OnCommand), NULL, this);



}
