#include "WindowHelper.h"

#include "vemsdialog/WindowTipper.h"
#include "logger.h"
#include "vemsgauge/GaugeGroupManager.h"
#include "vemsgauge/GaugeCarrierBase.h"
#include "wxPointEx.h"
#include "wxSizeEx.h"

WindowHelper::WindowHelper(wxWindow * win, WindowTipper * winTip)
{
    //ctor
    m_winTip = winTip;
    m_targetWin = win;
    m_sideRatioRestriction = -1.0;

    m_startPos = win->GetPosition();
    m_startSize = win->GetSize();
    m_evt = NULL;
}

WindowHelper::~WindowHelper()
{
    //dtor
}

int WindowHelper::Action(wxKeyEvent &evt)
{
    m_evt = &evt;

    int code = evt.GetKeyCode();

    int inc = 0;
    int rinc = 1;
    int res = 0;

    if ( evt.ShiftDown() )
        inc = 1;

    if ( evt.AltDown() )
        inc = 10;

    int state = 1;

    if( evt.ShiftDown() && evt.AltDown() ) {
        state = 2;
        inc = 10;
        rinc = 1;
    }

    if( evt.ShiftDown() && evt.ControlDown() ) {
        state = 2;
        rinc = 1;
        inc = 1;
    }

    if(  0 ) {
        // teve
    } else if( code == WXK_UP && inc && state == 1 ){  // up
        MoveTarget(0,inc * -1);
        res = 1;

    } else if( code == WXK_DOWN && inc && state == 1  ){  // down
        MoveTarget(0,inc);
        res = 1;

    } else if( code == WXK_LEFT && inc && state == 1 ){  // left
        MoveTarget(inc * -1,0);
        res = 1;

    } else if( code == WXK_RIGHT && inc && state == 1 ){  // right
        MoveTarget(inc,0);
        res = 1;

    } else if( code == WXK_UP && state == 2 ){  // up

        SetSizeTarget(0,inc*rinc*-1);
        res = 2;

    } else if( code == WXK_DOWN && state == 2 ){  // down

        SetSizeTarget(0,inc*rinc);
        res = 2;

    } else if( code == WXK_LEFT && state == 2 ){  // left

        SetSizeTarget(inc*rinc*-1,0);
        res = 2;

    } else if( code == WXK_RIGHT && state == 2 ){  // right

        SetSizeTarget(inc*rinc,0);
        res = 2;

    } else {
        return 0;
    }

    return res;

}

bool WindowHelper::GroupAction(GaugeGroupManager* groupmgr, int lastActionResult, const wxString& grpName, bool isDropToCarrier)
{
    if(lastActionResult == 1) {

        wxPointEx rdelta = m_targetWin->GetPosition() - m_startPos;
        wxVariantHash hash;
        hash[_T("groupname")] = grpName;
        hash[_T("cmd")] =  _T("gauge_move");
        hash[_T("sender_id")] = (long)m_targetWin->GetId();
        hash[_T("deltapos")] << rdelta;
        hash[_T("target_id")] = _T("0");
        groupmgr->ProcessWidgetCommand(hash);

    } else if (lastActionResult == 2) {

        wxSizeEx rdelta = m_targetWin->GetSize() - m_startSize;
        wxVariantHash hash;
        hash[_T("groupname")] = grpName;
        hash[_T("cmd")] =  _T("gauge_size");
        hash[_T("sender_id")] = (long)m_targetWin->GetId();
        hash[_T("deltasize")] << rdelta;
        hash[_T("target_id")] = _T("0");
        groupmgr->ProcessWidgetCommand(hash);

    } else {

        if(isDropToCarrier && m_evt) {
            GaugeCarrierBase* carrier = groupmgr->GetGroupCarrier(grpName);
            if(carrier){
                wxEvtHandler* handler = dynamic_cast<wxEvtHandler*>(carrier);
                if(handler) {
                    handler->AddPendingEvent(*m_evt);
                }
            }
        }

        return false;
    }

    return true;
}

void WindowHelper::SetSizeTarget(int xdiff, int ydiff)
{

    wxWindow * win = m_targetWin;

    wxSize size = win->GetSize();

    wxSize newsize( size.x + xdiff , size.y + ydiff );

    if( m_sideRatioRestriction > 0 ) {

        if( xdiff != 0 )
            ydiff = (int)(xdiff / m_sideRatioRestriction);

        else if( ydiff != 0 )
            xdiff = (int)(ydiff * m_sideRatioRestriction);

        newsize.x = size.x + xdiff;
        newsize.y = size.y + ydiff;

        if(newsize.x / m_sideRatioRestriction != newsize.y) {
            if(abs(xdiff) > abs(ydiff))
                newsize.y = (int)(newsize.x / m_sideRatioRestriction);
            else
                newsize.x = (int)(newsize.y * m_sideRatioRestriction);
        }

    }

    //win->SetSize( size.x + xdiff , size.y + ydiff );
    win->SetSize( newsize );

    if(m_winTip) m_winTip->ShowMsg(wxString::Format(_T("width=%i\nheight=%i"),win->GetSize().x,win->GetSize().y));
}

void WindowHelper::MoveTarget(int xdiff, int ydiff)
{

    wxWindow * win = m_targetWin;

    win->Move( win->GetPosition().x + xdiff , win->GetPosition().y + ydiff );

    if(m_winTip) m_winTip->ShowMsg(wxString::Format(_T("xpos=%i\nypos=%i"),win->GetPosition().x,win->GetPosition().y));
}
