#include "wxAgg2.h"
#include "wxAgg2.h"

#include "agg_conv_stroke.h"
#include "agg_ellipse.h"
#include "agg_rounded_rect.h"
#include "agg_conv_dash.h"
#include "agg_conv_smooth_poly1.h"
#include "agg_vcgen_markers_term.h"
#include "agg_image_accessors.h"
#include "agg_span_pattern_rgba.h"
#include "agg_span_allocator.h"
#include "agg_span_gradient.h"
#include "agg_span_interpolator_linear.h"

#include "agg_gsv_text.h"
#include "agg_bounding_rect.h"

#include "agg_bounding_rect.h"
#include "agg_trans_perspective.h"
#include "agg_blur.h"

#include "aggHelper.h"

#include "../logger.h"

#include "../bcWidgets/BitmapCanvasWidgetBase.h"

template<class T> T Min(T a,T b) { if ( a > b ) return b; else return a; }
template<class T> T Max(T a,T b) { if ( a > b ) return a; else return b; }

typedef agg::gradient_x                                 gradient_func_type;
typedef agg::span_interpolator_linear<>                 interpolator_type;
typedef agg::span_allocator<agg::rgba8>                 span_allocator_type;
typedef agg::span_gradient<agg::rgba8,
                           interpolator_type,
                           gradient_func_type,
                           color_array_type>            span_gradient_type;
typedef agg::renderer_scanline_aa<ren_base,
                                  span_allocator_type,
                                  span_gradient_type>   renderer_gradient_type;

#define IS_INITIALIZED(x) if ( m_initialized == false ) return x;

#define M_GRADIENT_SIZE 100

#define COS_45 0.70710678118654752440084436210485

wxAgg2::wxAgg2( uint32_t width, uint32_t height, const wxColor& clearColor )
{
    Create( width, height, clearColor );
}

wxAgg2::wxAgg2( const wxSize& size, const wxColor& clearColor )
{
    Create( size.x, size.y, clearColor );
}

wxAgg2::wxAgg2( wxWindow* win, const wxColor& clearColor )
{
    if ( win )
        Create( win->GetSize().x, win->GetSize().y, clearColor );
}

wxAgg2::wxAgg2( wxBitmap* bitmap )
{
    if ( bitmap->IsOk() )
    {
        Create( bitmap->GetWidth(), bitmap->GetHeight() );

        if ( m_initialized )
        {

            wxImage tmpImg = bitmap->ConvertToImage();

            for ( int i = 0; i < bitmap->GetWidth() * bitmap->GetHeight(); ++i )
            {
                m_buffer[4 * i] = (tmpImg.GetData())[3 * i];
                m_buffer[4 * i + 1] = (tmpImg.GetData())[3 * i + 1];
                m_buffer[4 * i + 2] = (tmpImg.GetData())[3 * i + 2];
                m_buffer[4 * i + 3] = (tmpImg.GetAlpha())[i];
            }
        }
    }
}

wxAgg2::wxAgg2( BitmapCanvasWidgetBase* bcWidget, const wxColor& clearColor )
{
    if ( bcWidget == NULL )
    {
        m_initialized = false;
        return;
    }
    wxSize s = bcWidget->GetSize().Ceil();
    Create( s.x, s.y, clearColor );
}

wxAgg2::~wxAgg2()
{
    IS_INITIALIZED()

    delete m_renderer;
    delete m_pixf;
    delete m_rb;

    if ( m_buffer )
    {
        delete[] m_buffer;
        m_buffer = NULL;
    }

    if ( m_imgBuffer )
    {
        delete[] m_imgBuffer;
        m_imgBuffer = NULL;
    }

    if ( m_alphaBuffer )
    {
        delete[] m_alphaBuffer;
        m_alphaBuffer = NULL;
    }

    if ( m_patternBuffer )
    {
        delete[] m_patternBuffer;
        m_patternBuffer = NULL;
    }

    delete m_pen;
    delete m_brush;
    delete m_font;
    delete m_blur;

    delete m_states;
}

void wxAgg2::clear( const wxColor& color )
{
    IS_INITIALIZED()

    agg::rgba cRGBA = Agg::convertToRGBA( color );
    m_renderer->clear( cRGBA );
}

/** rendering **/

void wxAgg2::render( wxDC& dc, unsigned int xpos, unsigned int ypos )
{
    IS_INITIALIZED()

    getBuffers(wxRect(0,0,m_width,m_height));

    wxImage myImage( m_width, m_height, m_imgBuffer, true );

    myImage.SetAlpha( m_alphaBuffer, true );

    if( myImage.IsOk() )
    {
        #if defined(__UNIX__)
        wxBitmap myBitmap( myImage );
        #else
        wxBitmap myBitmap( myImage, 32 );
        #endif
        dc.DrawBitmap( myBitmap, xpos, ypos, true );
    }
}

wxBitmap* wxAgg2::renderToBitmap(bool isSupportAlphaChannel, const wxRect& renderBox)
{
    IS_INITIALIZED( NULL )

    wxRect rBox = renderBox;

    if(rBox == wxDefaultRect) rBox = wxRect(0, 0, m_width, m_height);

    rBox.Intersect(wxRect(0,0,m_width,m_height));

    getBuffers(rBox,isSupportAlphaChannel);

    wxImage  myImage( rBox.width, rBox.height, m_imgBuffer, true );

    if(isSupportAlphaChannel)
        myImage.SetAlpha( m_alphaBuffer, true );

    wxBitmap* myBitmap = new wxBitmap( myImage );

    return myBitmap;
}

wxImage* wxAgg2::renderToImage(bool isSupportAlphaChannel)
{
    IS_INITIALIZED( NULL )

    getBuffers(wxRect(0,0,m_width,m_height), isSupportAlphaChannel);

    wxImage *myImage = new wxImage( m_width, m_height, m_imgBuffer, true );

    if(isSupportAlphaChannel)
        myImage->SetAlpha( m_alphaBuffer, true );

    return myImage;
}

/// necessary to copy, if the image 'life' is longer then the canvas 'life'
wxImage* wxAgg2::renderCopyToImage(bool isSupportAlphaChannel)
{
    IS_INITIALIZED( NULL )

    wxImage *image = new wxImage( m_width, m_height, false );

    if(isSupportAlphaChannel) image->InitAlpha();

    unsigned char* data = image->GetData();
    unsigned char* alpha = NULL;

    if(isSupportAlphaChannel)
        alpha = image->GetAlpha();

    getBuffers(data, alpha);

    return image;
}

bool wxAgg2::renderToFile( const wxString& imgname, long bitmaptype )
{
    IS_INITIALIZED( false )

    if ( imgname == wxEmptyString )
        return false;

    switch ( bitmaptype )
    {
        case wxBITMAP_TYPE_BMP:
        case wxBITMAP_TYPE_JPEG:
        case wxBITMAP_TYPE_PNG:
        case wxBITMAP_TYPE_PCX:
        case wxBITMAP_TYPE_PNM:
        case wxBITMAP_TYPE_XPM:
        case wxBITMAP_TYPE_ICO:
        case wxBITMAP_TYPE_CUR: break;
        default: return false;
    }

    getBuffers(wxRect(0,0,m_width,m_height));

    wxImage  myImage( m_width, m_height, m_imgBuffer, true );

    myImage.SetAlpha( m_alphaBuffer, true );

    bool ret = myImage.SaveFile( imgname, bitmaptype );

    return ret;
}

void wxAgg2::drawBitmap(wxBitmap* bmp, const wxPoint& pos)
{
    wxImage* img = new wxImage( bmp->ConvertToImage() );
    drawImage( img, pos );
    delete img;
}

void wxAgg2::drawImage(wxImage* img, const wxPoint& pos)
{
    IS_INITIALIZED()

    if ( !img->IsOk() ) return;

    int target_w = m_width;
    int target_h = m_height;

    if( target_w < 0 || target_h < 0 ) return;

    if( pos.x > target_w || pos.y > target_h ) return;

    int img_w = img->GetWidth();
    int img_h = img->GetHeight();

    if( pos.x + img_w < 0 || pos.y + img_h < 0 ) return;

    int img_x, img_y, target_x, target_y, target_didx, img_aidx, img_didx;
    bool img_hashAlpha = img->HasAlpha();
    double divider = 1.0/255.0;

    double img_a_rat,target_a_rat,new_a_rat;

    unsigned char* img_data = img->GetData();
    unsigned char* img_alpha = img->GetAlpha();

    for(img_y = 0; img_y < img_h; img_y++) {

        target_y = pos.y + img_y;

        if(target_y < 0) continue;
        if(target_y >= target_h) break;

        for(img_x = 0; img_x < img_w; img_x++) {

            target_x = pos.x + img_x;

            if(target_x < 0) continue;
            if(target_x >= target_w) break;

            img_aidx = img_w * img_y + img_x;

            if(img_hashAlpha && img_alpha[img_aidx] == 0) continue;

            img_didx = img_aidx * 3;

            target_didx = ( target_x + target_w * target_y ) * 4;

            if(img_hashAlpha) {

                img_a_rat = img_alpha[img_aidx] * divider;

                target_a_rat = m_buffer[target_didx+3] * divider;

                new_a_rat = img_a_rat + target_a_rat * ( 1.0 - img_a_rat );

                m_buffer[target_didx] = (uint8_t)((img_data[img_didx] * img_a_rat + m_buffer[target_didx] * target_a_rat * (1.0 - img_a_rat)) / new_a_rat);
                m_buffer[target_didx+1] = (uint8_t)((img_data[img_didx+1] * img_a_rat + m_buffer[target_didx+1] * target_a_rat * (1.0 - img_a_rat)) / new_a_rat);
                m_buffer[target_didx+2] = (uint8_t)((img_data[img_didx+2] * img_a_rat + m_buffer[target_didx+2] * target_a_rat * (1.0 - img_a_rat)) / new_a_rat);
                m_buffer[target_didx+3] = (uint8_t)(new_a_rat * 255.0);

            } else {
                m_buffer[target_didx] = img_data[img_didx];
                m_buffer[target_didx+1] = img_data[img_didx+1];
                m_buffer[target_didx+2] = img_data[img_didx+2];
                m_buffer[target_didx+3] = 0xFF;
            }
        }
    }

}

void wxAgg2::setPen( const wxColour& pencolor )
{
    IS_INITIALIZED()

    m_pen->SetColor( pencolor );
}

void wxAgg2::setPen( const wxPen& pen )
{
    IS_INITIALIZED()

    m_pen->SetPen( pen );
}

void wxAgg2::setPen( const wxAggPen& pen )
{
    IS_INITIALIZED()

    m_pen->SetPen( pen );
}

void wxAgg2::setPenWidth( double width )
{
    IS_INITIALIZED()

    m_pen->SetWidth( width );
}

void wxAgg2::setTransparentPen()
{
    IS_INITIALIZED()

    m_pen->SetStyle( wxAGGPEN_TRANSPARENT );
}

double wxAgg2::getPenWidth() const
{
    IS_INITIALIZED( -1.0 )

    return m_pen->GetWidth();
}

wxAggPen* wxAgg2::getPen()
{
    IS_INITIALIZED( NULL )

    return m_pen;
}

void wxAgg2::setBrush( const wxColour& brushcolor )
{
    IS_INITIALIZED()

    m_brush->SetColor( brushcolor );
}

void wxAgg2::setBrush( const wxBrush& brush )
{
    IS_INITIALIZED()

    m_brush->SetBrush( brush );
}

void wxAgg2::setBrush( const wxAggBrush& brush )
{
    IS_INITIALIZED()

    m_brush->SetBrush( brush );
}

void wxAgg2::setTransparentBrush()
{
    IS_INITIALIZED()

    m_brush->SetStyle( wxAGGBRUSH_TRANSPARENT );
}

wxAggBrush* wxAgg2::getBrush()
{
    IS_INITIALIZED( NULL )

    return m_brush;
}

void wxAgg2::setBlur( const wxAggBlur& blur )
{
    *m_blur = blur;
}

void wxAgg2::disableBlur()
{
    *m_blur = wxAggNullBlur;
}

void wxAgg2::setDropShadow( double radius )
{
    m_blur->translate.x = 2*radius;
    m_blur->translate.y = 2*radius;
    m_blur->gaussianradius = radius;
    m_blur->color = wxColor( 128, 128, 128, 255 );
}

void wxAgg2::saveState()
{
    IS_INITIALIZED()

    m_states->push( new wxAggBrush( *m_brush ) );
    m_states->push( new wxAggPen( *m_pen ) );
    m_states->push( new wxAggFont( *m_font ) );
    m_states->push( new wxAggBlur( *m_blur ) );
}

void wxAgg2::restoreState()
{
    IS_INITIALIZED()

    try {
        wxAggObject* o = m_states->pop();

        wxAggBlur* blur = wxDynamicCast( o, wxAggBlur );
        if ( blur )
        {
            *m_blur = *blur;
            delete blur;
        }

        o = m_states->pop();

        wxAggFont* font = wxDynamicCast( o, wxAggFont );
        if ( font )
        {
            m_font->setFont( *font );
            delete font;
        }

        o = m_states->pop();

        wxAggPen* pen = wxDynamicCast( o, wxAggPen );
        if ( pen )
        {
            m_pen->SetPen( *pen );
            delete pen;
        }

        o = m_states->pop();
        wxAggBrush* brush = wxDynamicCast( o, wxAggBrush );
        if ( brush )
        {
            m_brush->SetBrush( *brush );
            delete brush;
        }
    }
    catch ( wxAggObjectStackException e )
    {
        Logger::logf(LOG_MSG, _T("wxAgg::restoreState() exception - ") + e.error() );
    }
}

/// -------------------------------------------------------------------------------------------------
///
/// DRAWING METHODS
///
/// -------------------------------------------------------------------------------------------------
void wxAgg2::drawLine( double x1, double y1, double x2, double y2 )
{
    IS_INITIALIZED()

    RealPointArray arr;

    wxRealPoint p1( x1, y1 ), p2( x2, y2 );

    arr.Add( p1 );
    arr.Add( p2 );

    drawLine( arr, false );
}

void wxAgg2::drawLine( wxRealPoint point1, wxRealPoint point2 )
{
    IS_INITIALIZED()

    RealPointArray arr;
    arr.Add( point1 ); arr.Add( point2 );

    drawLine( arr, false );
}

void wxAgg2::drawLine( const RealPointArray& coords, bool closeIt )
{
    IS_INITIALIZED()

    if ( m_pen->GetStyle() == wxAGGPEN_TRANSPARENT )
        return;

    if ( coords.GetCount() < 2 )
        return;

    if ( m_pen->GetStyle() == wxAGGPEN_SOLID )
    {
        agg::path_storage path;

        path.move_to( coords[0].x, coords[0].y );

        for( size_t a = 1; a < coords.GetCount(); ++a )
        {
            path.line_to( coords[a].x, coords[a].y );
        }

        if ( closeIt )
        {
            path.close_polygon();
        }

        agg::conv_stroke< agg::path_storage > stroke( path );
        stroke.line_join( agg::miter_join );

        stroke.line_cap( m_pen->GetCap() );
        stroke.width( m_pen->GetWidth() );

        m_ras.reset();

        applyClipping();

        m_ras.add_path( stroke );

        agg::render_scanlines_aa_solid( m_ras, m_sl, *m_renderer, agg::rgba8( m_pen->GetAggColor() ) );
    }
    else
    {
        /*typedef agg::path_base<
            agg::vertex_stl_storage<
                agg::pod_auto_vector<
                    agg::vertex_d, 50> > > path_storage_type;*/

        // path_storage_type path;
        agg::path_storage path;

        path.move_to( coords[0].x, coords[0].y );

        for( size_t a = 1; a < coords.GetCount(); ++a )
        {
            path.line_to( coords[a].x, coords[a].y );
        }

        if ( closeIt )
        {
            path.close_polygon();
        }

        m_ras.add_path(path);


        /*agg::conv_smooth_poly1<path_storage_type> smooth( path );
        smooth.smooth_value( 0.0 );

        agg::conv_curve<agg::conv_smooth_poly1<path_storage_type> > curve(smooth);
        agg::conv_dash<agg::conv_curve<agg::conv_smooth_poly1<path_storage_type> >, agg::vcgen_markers_term> dash(curve);
        agg::conv_stroke<agg::conv_dash<agg::conv_curve<agg::conv_smooth_poly1<path_storage_type> >, agg::vcgen_markers_term> > stroke(dash);*/

        agg::conv_curve<agg::path_storage> curve( path );
        agg::conv_dash<agg::conv_curve<agg::path_storage>, agg::vcgen_markers_term > dash( curve );
        agg::conv_stroke<agg::conv_dash<agg::conv_curve<agg::path_storage>, agg::vcgen_markers_term> > stroke( dash );

        stroke.line_cap( m_pen->GetCap() );
        stroke.width( m_pen->GetWidth() );

        double* dashes;
        int dashesNum = m_pen->GetDashes( &dashes );
        if ( dashes != NULL )
        {
            for ( int j = 0; j < dashesNum; ++j )
                dash.add_dash( dashes[2*j], dashes[2*j+1] );
            dash.dash_start( dashes[0] + dashes[1] / 2);
        }
        else
        {
            dash.add_dash( 10.0, 10.0 );
            dash.dash_start( 15 );
        }

        m_ras.reset();

        applyClipping();

        m_ras.add_path( stroke );
        agg::render_scanlines_aa_solid( m_ras, m_sl, *m_renderer, agg::rgba8( m_pen->GetAggColor() ) );
    }
}

///
/// draw polygons
///
void wxAgg2::drawPolygon( const RealPointArray& points, bool alignPatternToLeft )
{
    IS_INITIALIZED()

    if ( points.GetCount() < 3 )
        return;

    bool align = ( alignPatternToLeft && ( m_brush->GetStyle() > wxAGGBRUSH_TRANSPARENT ) );

    // fill polygon first
    if ( m_brush->GetStyle() != wxAGGBRUSH_TRANSPARENT )
    {
        double xmin = m_width;
        double ymin = m_height;
        double xmax = 0;
        double ymax = 0;

        agg::path_storage path;

        path.move_to( points[0].x, points[0].y );

        if ( align || m_brush->GetStyle() == wxAGGBRUSH_GRADIENT )
        {
            if ( xmin > points[0].x ) xmin = points[0].x;
            if ( ymin > points[0].y ) ymin = points[0].y;

            if ( xmax < points[0].x ) xmax = points[0].x;
            if ( ymax < points[0].y ) ymax = points[0].y;
        }

        for ( size_t i = 1; i < points.GetCount(); ++i )
        {
            path.line_to( points[i].x, points[i].y );

            if ( align || m_brush->GetStyle() == wxAGGBRUSH_GRADIENT )
            {
                if ( xmin > points[i].x ) xmin = points[i].x;
                if ( ymin > points[i].y ) ymin = points[i].y;

                if ( xmax < points[i].x ) xmax = points[i].x;
                if ( ymax < points[i].y ) ymax = points[i].y;
            }

        }

        path.close_polygon();

        m_ras.reset();

        applyClipping();

        // draw outlines - drop shadow...
        drawOutlines( path );

        // gradient brush
        if ( m_brush->GetStyle() == wxAGGBRUSH_GRADIENT )
        {
            m_ras.add_path( path );

            double xw_ = ( xmax - xmin ), yw_ = ( ymax - ymin );

            gradient_rendering( xmin + xw_ / 2.0, ymin + yw_ / 2.0, xw_ / 2.0, yw_ / 2.0 );
        }
        else
        {
            if ( m_brush->GetStyle() == wxAGGBRUSH_SOLID )
            {
                m_ras.add_path( path );
                agg::render_scanlines_aa_solid( m_ras, m_sl, *m_renderer, agg::rgba8( m_brush->GetAggColor() ) );
            }
            else
            {
                generatePattern();

                typedef agg::wrap_mode_repeat wrap_x_type;
                typedef agg::wrap_mode_repeat wrap_y_type;
                typedef agg::image_accessor_wrap<pixfmtT, wrap_x_type, wrap_y_type> img_source_type;
                typedef agg::span_pattern_rgba<img_source_type> span_gen_type;

                agg::span_allocator<agg::rgba8> sa;
                pixfmtT         img_pixf( m_pattern_rbuf );
                img_source_type img_src( img_pixf );

                unsigned int xoff = 0, yoff = 0;
                if ( align )
                {
                    xoff = m_pattern_w - (unsigned int) xmin % m_pattern_w;
                    yoff = m_pattern_h - (unsigned int) ymin % m_pattern_h;
                }
                span_gen_type   sg( img_src, xoff, yoff );

                m_ras.add_path( path );
                agg::render_scanlines_aa( m_ras, m_sl, *m_renderer, sa, sg );
            }
        }
    }

    // draw border
    if ( m_pen->GetStyle() != wxAGGPEN_TRANSPARENT )
    {
        drawLine( points, true );
    }
}

/// draw rectangles
void wxAgg2::drawRectangle( double x, double y, double width, double height )
{
    IS_INITIALIZED()

    // fill polygon
    if ( m_brush->GetStyle() != wxAGGBRUSH_TRANSPARENT )
    {
        agg::path_storage path;

        path.move_to( x, y );
        path.line_rel( width, 0 );
        path.line_rel( 0, height );
        path.line_rel( -width, 0 );
        path.close_polygon();

        m_ras.reset();

        applyClipping();

        // draw outlines - drop shadow...
        drawOutlines( path );

        // gradient brush
        if ( m_brush->GetStyle() == wxAGGBRUSH_GRADIENT )
        {
            m_ras.add_path( path );

            gradient_rendering( x + width / 2.0, y + height / 2.0, width / 2.0, height / 2.0 );
        }
        else
        {
            if ( m_brush->GetStyle() == wxAGGBRUSH_SOLID )
            {
                m_ras.add_path( path );
                agg::render_scanlines_aa_solid( m_ras, m_sl, *m_renderer, agg::rgba8( m_brush->GetAggColor() ) );
            }
            else
            {
                generatePattern();

                typedef agg::wrap_mode_repeat wrap_x_type;
                typedef agg::wrap_mode_repeat wrap_y_type;
                typedef agg::image_accessor_wrap<pixfmtT, wrap_x_type, wrap_y_type> img_source_type;
                typedef agg::span_pattern_rgba<img_source_type> span_gen_type;

                agg::span_allocator<agg::rgba8> sa;
                pixfmtT         img_pixf( m_pattern_rbuf );
                img_source_type img_src( img_pixf );

                unsigned int xoff = m_pattern_w - (unsigned int) x % m_pattern_w;
                unsigned int yoff = m_pattern_h - (unsigned int) y % m_pattern_h;

                span_gen_type   sg( img_src, xoff, yoff );

                m_ras.add_path( path );
                agg::render_scanlines_aa( m_ras, m_sl, *m_renderer, sa, sg );
            }
        }
    }

    // draw border
    if ( m_pen->GetStyle() != wxAGGPEN_TRANSPARENT )
    {
        RealPointArray arr;
        wxRealPoint p;
        p.x = x; p.y = y; arr.Add( p );
        p.x = x + width; p.y = y; arr.Add( p );
        p.x = x + width; p.y = y + height; arr.Add( p );
        p.x = x; p.y = y + height; arr.Add( p );

        drawLine( arr, true );
    }
}

void wxAgg2::drawRoundedRectangle( double x, double y, double width, double height, double radius )
{
    IS_INITIALIZED()

    double r = radius;
    double w = width;
    double h = height;

    if ( r > w/2 || r > h/2 )
        r = Min(w/2, h/2);

    agg::rounded_rect rect(x,y,x+w,y+h,r);
    rect.normalize_radius();


    // fill rectangle
    if ( m_brush->GetStyle() != wxAGGBRUSH_TRANSPARENT )
    {
        m_ras.reset();

        applyClipping();

        // draw outlines - drop shadow...
        drawOutlines( rect );

        // gradient brush
        if ( m_brush->GetStyle() == wxAGGBRUSH_GRADIENT )
        {
            m_ras.add_path( rect );
            gradient_rendering( x + width / 2.0, y + height / 2.0, width / 2.0, height / 2.0 );
        }
        else
        {
            if ( m_brush->GetStyle() == wxAGGBRUSH_SOLID )
            {
                m_ras.add_path( rect );

                agg::render_scanlines_aa_solid( m_ras, m_sl, *m_renderer, agg::rgba8( m_brush->GetAggColor() ) );
            }
            else
            {
                generatePattern();

                typedef agg::wrap_mode_repeat wrap_x_type;
                typedef agg::wrap_mode_repeat wrap_y_type;
                typedef agg::image_accessor_wrap<pixfmtT, wrap_x_type, wrap_y_type> img_source_type;
                typedef agg::span_pattern_rgba<img_source_type> span_gen_type;

                agg::span_allocator<agg::rgba8> sa;
                pixfmtT         img_pixf( m_pattern_rbuf );
                img_source_type img_src( img_pixf );

                unsigned int xoff = m_pattern_w - (unsigned int) x % m_pattern_w;
                unsigned int yoff = m_pattern_h - (unsigned int) y % m_pattern_h;

                span_gen_type   sg( img_src, xoff, yoff );

                m_ras.add_path( rect );

                agg::render_scanlines_aa( m_ras, m_sl, *m_renderer, sa, sg );
            }
        }
    }


    // draw border
    if ( m_pen->GetStyle() != wxAGGPEN_TRANSPARENT )
    {
        if ( m_pen->GetStyle() == wxAGGPEN_SOLID )
        {
            agg::conv_stroke<agg::rounded_rect> stroke(rect);

            stroke.width( m_pen->GetWidth() );
            stroke.line_cap( m_pen->GetCap() );

            m_ras.reset();

            applyClipping();

            m_ras.add_path( stroke );

            agg::render_scanlines_aa_solid( m_ras, m_sl, *m_renderer, agg::rgba8( m_pen->GetAggColor() ) );
        }
        else
        {


            agg::conv_curve<agg::rounded_rect> curve_( rect );
            agg::conv_dash<agg::conv_curve<agg::rounded_rect>, agg::vcgen_markers_term> dash(curve_);
            agg::conv_stroke<agg::conv_dash<agg::conv_curve<agg::rounded_rect> , agg::vcgen_markers_term> > stroke(dash);

            stroke.line_cap( m_pen->GetCap() );
            stroke.width( m_pen->GetWidth() );

            double* dashes;
            int dashesNum = m_pen->GetDashes( &dashes );
            if ( dashes != NULL )
            {
                for ( int j = 0; j < dashesNum; ++j )
                {
                    dash.add_dash( dashes[2*j], dashes[2*j+1] );
                }
                dash.dash_start( dashes[0] + dashes[1] / 2);
            }
            else
            {
                dash.add_dash( 10.0, 10.0 );
                dash.dash_start( 15 );
            }

            m_ras.reset();

            applyClipping();

            m_ras.add_path( stroke );
            agg::render_scanlines_aa_solid( m_ras, m_sl, *m_renderer, agg::rgba8( m_pen->GetAggColor() ) );
        }
    }

}

void wxAgg2::drawRoundedRectangle( const wxRealRect& rect, double radius )
{
    IS_INITIALIZED()

    drawRoundedRectangle( rect.x, rect.y, rect.w, rect.h, radius );
}

void wxAgg2::drawRotatedRectangle( double x, double y, double width, double height, double rot_degree )
{
    IS_INITIALIZED()

    double rad = rot_degree * agg::pi / 180.0;

    agg::path_storage path;

    path.move_to( x, y );
    path.line_rel( width, 0 );
    path.line_rel( 0, height );
    path.line_rel( -width, 0 );
    path.close_polygon();

    agg::trans_affine mtx;

    mtx *= agg::trans_affine_translation( -( x + width / 2 ) , -( y + height / 2 ) );
    mtx *= agg::trans_affine_rotation( rad );
    mtx *= agg::trans_affine_translation( x + width / 2 , y + height / 2 );

    path.transform( mtx );

    // fill rectangle
    if ( m_brush->GetStyle() != wxAGGBRUSH_TRANSPARENT )
    {
        m_ras.reset();

        applyClipping();

        drawOutlines( path );

        // gradient brush
        if ( m_brush->GetStyle() == wxAGGBRUSH_GRADIENT )
        {
            m_ras.add_path( path );

            gradient_rendering( x + width / 2 , y + height / 2, width/2.0 * cos( rad ), height/2.0* sin(rad) );
        }
        else
        {
            if ( m_brush->GetStyle() == wxAGGBRUSH_SOLID )
            {
                m_ras.add_path( path );
                agg::render_scanlines_aa_solid( m_ras, m_sl, *m_renderer, agg::rgba8( m_brush->GetAggColor() ) );
            }
            else
            {
                generatePattern();

                ///
                /// TODO
                ///   rotate pattern (?)
                ///

                typedef agg::wrap_mode_repeat wrap_x_type;
                typedef agg::wrap_mode_repeat wrap_y_type;
                typedef agg::image_accessor_wrap<pixfmtT, wrap_x_type, wrap_y_type> img_source_type;
                typedef agg::span_pattern_rgba<img_source_type> span_gen_type;

                agg::span_allocator<agg::rgba8> sa;
                pixfmtT         img_pixf( m_pattern_rbuf );
                img_source_type img_src( img_pixf );

                unsigned int xoff = m_pattern_w - (unsigned int) x % m_pattern_w;
                unsigned int yoff = m_pattern_h - (unsigned int) y % m_pattern_h;

                span_gen_type   sg( img_src, xoff, yoff );

                m_ras.add_path( path );
                agg::render_scanlines_aa( m_ras, m_sl, *m_renderer, sa, sg );
            }
        }
    }

    // border
    if ( m_pen->GetStyle() != wxAGGPEN_TRANSPARENT )
    {
        if ( m_pen->GetStyle() == wxAGGPEN_SOLID )
        {
            agg::conv_stroke< agg::path_storage > stroke( path );
            stroke.width( m_pen->GetWidth() );
            stroke.line_cap( m_pen->GetCap() );

            m_ras.reset();

            applyClipping();

            m_ras.add_path( stroke );
            agg::render_scanlines_aa_solid( m_ras, m_sl, *m_renderer, agg::rgba8( m_pen->GetAggColor() ) );
        }
        else
        {
            agg::conv_curve<agg::path_storage> curve( path );
            agg::conv_dash<agg::conv_curve<agg::path_storage>, agg::vcgen_markers_term> dash(curve);
            agg::conv_stroke<agg::conv_dash<agg::conv_curve<agg::path_storage> , agg::vcgen_markers_term> > stroke(dash);

            stroke.line_cap( m_pen->GetCap() );
            stroke.width( m_pen->GetWidth() );

            double* dashes;
            int dashesNum = m_pen->GetDashes( &dashes );
            if ( dashes != NULL )
            {
                for ( int j = 0; j < dashesNum; ++j )
                {
                    dash.add_dash( dashes[2*j], dashes[2*j+1] );
                }
                dash.dash_start( dashes[0] + dashes[1] / 2);
            }
            else
            {
                dash.add_dash( 10.0, 10.0 );
                dash.dash_start( 15 );
            }

            m_ras.reset();

            applyClipping();

            m_ras.add_path( stroke );
            agg::render_scanlines_aa_solid( m_ras, m_sl, *m_renderer, agg::rgba8( m_pen->GetAggColor() ) );
        }
    }
}

/// draw arc

void wxAgg2::drawArc( double centerx, double centery, double radius, double startAngleRad, double deltaAngleRad )
{
    IS_INITIALIZED()

    agg::bezier_arc arc( centerx, centery, radius, radius, startAngleRad, deltaAngleRad );

    agg::path_storage path;
    path.concat_path( arc, 0 );

    if ( m_brush->GetStyle() != wxAGGBRUSH_TRANSPARENT )
    {
        path.line_to( centerx, centery );
        path.close_polygon();
    }

    if ( m_brush->GetStyle() != wxAGGBRUSH_TRANSPARENT )
    {
        m_ras.reset();

        applyClipping();

        // draw outlines
        drawOutlines( path );

        agg::conv_curve<agg::path_storage> curve( path );

        // gradient brush
        if ( m_brush->GetStyle() == wxAGGBRUSH_GRADIENT )
        {
            m_ras.add_path( curve );

            gradient_rendering( centerx, centery, radius, radius );
        }
        else
        {
            if ( m_brush->GetStyle() == wxAGGBRUSH_SOLID )
            {
                m_ras.add_path( curve );
                agg::render_scanlines_aa_solid( m_ras, m_sl, *m_renderer, agg::rgba8( m_brush->GetAggColor() ) );
            }
            else
            {
                generatePattern();

                typedef agg::wrap_mode_repeat wrap_x_type;
                typedef agg::wrap_mode_repeat wrap_y_type;
                typedef agg::image_accessor_wrap<pixfmtT, wrap_x_type, wrap_y_type> img_source_type;
                typedef agg::span_pattern_rgba<img_source_type> span_gen_type;

                agg::span_allocator<agg::rgba8> sa;
                pixfmtT         img_pixf( m_pattern_rbuf );
                img_source_type img_src( img_pixf );

                unsigned int xoff = m_pattern_w - (unsigned int) (centerx - radius) % m_pattern_w;
                unsigned int yoff = m_pattern_h - (unsigned int) (centery - radius) % m_pattern_h;

                span_gen_type   sg( img_src, xoff, yoff );

                m_ras.add_path( curve );
                agg::render_scanlines_aa( m_ras, m_sl, *m_renderer, sa, sg );
            }
        }
    }

    if ( m_pen->GetStyle() != wxAGGPEN_TRANSPARENT )
    {
        if ( m_pen->GetStyle() == wxAGGPEN_SOLID )
        {
            agg::conv_stroke< agg::path_storage > stroke( path );
            stroke.width( m_pen->GetWidth() );
            stroke.line_cap( m_pen->GetCap() );

            m_ras.reset();

            applyClipping();

            m_ras.add_path( stroke );

            agg::render_scanlines_aa_solid( m_ras, m_sl, *m_renderer, agg::rgba8( m_pen->GetAggColor() ) );
        }
        else
        {
            agg::conv_curve<agg::path_storage> curve( path );
            agg::conv_dash<agg::conv_curve<agg::path_storage>, agg::vcgen_markers_term> dash(curve);
            agg::conv_stroke<agg::conv_dash<agg::conv_curve<agg::path_storage> , agg::vcgen_markers_term> > stroke(dash);

            stroke.line_cap( m_pen->GetCap() );
            stroke.width( m_pen->GetWidth() );

            double* dashes;
            int dashesNum = m_pen->GetDashes( &dashes );
            if ( dashes != NULL )
            {
                for ( int j = 0; j < dashesNum; ++j )
                {
                    dash.add_dash( dashes[2*j], dashes[2*j+1] );
                }
                dash.dash_start( dashes[0] + dashes[1] / 2);
            }
            else
            {
                dash.add_dash( 10.0, 10.0 );
                dash.dash_start( 15 );
            }

            m_ras.reset();

            applyClipping();

            m_ras.add_path( stroke );
            agg::render_scanlines_aa_solid( m_ras, m_sl, *m_renderer, agg::rgba8( m_pen->GetAggColor() ) );
        }
    }
}

void wxAgg2::drawArc( wxPoint center, double radius, double startAngle, double deltaAngle )
{
    IS_INITIALIZED()

    drawArc( center.x, center.y, radius, startAngle, deltaAngle );
}

void wxAgg2::drawArc( wxRealPoint center, double radius, double startAngle, double deltaAngle )
{
    IS_INITIALIZED()

    drawArc( center.x, center.y, radius, startAngle, deltaAngle );
}

void wxAgg2::drawFilledCircleSection( double centerx, double centery, double radius, double startAngleRad, double deltaAngleRad )
{
    IS_INITIALIZED()

    if ( m_brush->GetStyle() != wxAGGBRUSH_TRANSPARENT )
    {
        agg::bezier_arc arc( centerx, centery, radius, radius, startAngleRad, deltaAngleRad );

        agg::path_storage path;

        path.concat_path( arc, 0 );

        m_ras.reset();

        applyClipping();

        // draw outlines
        drawOutlines( path );

        agg::conv_curve<agg::path_storage> curve( path );

        // gradient brush
        if ( m_brush->GetStyle() == wxAGGBRUSH_GRADIENT )
        {
            m_ras.add_path( curve );

            gradient_rendering( centerx, centery, radius, radius );
        }
        else
        {
            if ( m_brush->GetStyle() == wxAGGBRUSH_SOLID )
            {
                m_ras.add_path( curve );
                agg::render_scanlines_aa_solid( m_ras, m_sl, *m_renderer, agg::rgba8( m_brush->GetAggColor() ) );
            }
            else
            {
                generatePattern();

                typedef agg::wrap_mode_repeat wrap_x_type;
                typedef agg::wrap_mode_repeat wrap_y_type;
                typedef agg::image_accessor_wrap<pixfmtT, wrap_x_type, wrap_y_type> img_source_type;
                typedef agg::span_pattern_rgba<img_source_type> span_gen_type;

                agg::span_allocator<agg::rgba8> sa;
                pixfmtT         img_pixf( m_pattern_rbuf );
                img_source_type img_src( img_pixf );

                unsigned int xoff = m_pattern_w - (unsigned int) (centerx - radius) % m_pattern_w;
                unsigned int yoff = m_pattern_h - (unsigned int) (centery - radius) % m_pattern_h;

                span_gen_type   sg( img_src, xoff, yoff );

                m_ras.add_path( curve );
                agg::render_scanlines_aa( m_ras, m_sl, *m_renderer, sa, sg );
            }
        }
    }
}

/// ellipse
void wxAgg2::drawEllipse( double centerx, double centery, double r1, double r2 )
{
    IS_INITIALIZED()

    agg::ellipse ellipse( centerx, centery, r1, r2 );

    if ( m_brush->GetStyle() != wxAGGBRUSH_TRANSPARENT )
    {
        m_ras.reset();

        applyClipping();

        // draw outlines
        drawOutlines( ellipse );

        // gradient brush
        if ( m_brush->GetStyle() == wxAGGBRUSH_GRADIENT )
        {
            m_ras.add_path( ellipse );

            gradient_rendering( centerx, centery, r1, r2 );
        }
        else
        {
            if ( m_brush->GetStyle() == wxAGGBRUSH_SOLID )
            {
                m_ras.add_path( ellipse );
                agg::render_scanlines_aa_solid( m_ras, m_sl, *m_renderer, agg::rgba8( m_brush->GetAggColor() ) );
            }
            else
            {
                generatePattern();

                typedef agg::wrap_mode_repeat wrap_x_type;
                typedef agg::wrap_mode_repeat wrap_y_type;
                typedef agg::image_accessor_wrap<pixfmtT, wrap_x_type, wrap_y_type> img_source_type;
                typedef agg::span_pattern_rgba<img_source_type> span_gen_type;

                agg::span_allocator<agg::rgba8> sa;
                pixfmtT         img_pixf( m_pattern_rbuf );
                img_source_type img_src( img_pixf );

                unsigned int xoff = m_pattern_w - (unsigned int) (centerx - r1) % m_pattern_w;
                unsigned int yoff = m_pattern_h - (unsigned int) (centery - r2) % m_pattern_h;

                span_gen_type   sg( img_src, xoff, yoff );

                m_ras.add_path( ellipse );
                agg::render_scanlines_aa( m_ras, m_sl, *m_renderer, sa, sg );
            }
        }
    }

    if ( m_pen->GetStyle() != wxAGGPEN_TRANSPARENT )
    {
        if ( m_pen->GetStyle() == wxAGGPEN_SOLID )
        {
            agg::conv_stroke< agg::ellipse > stroke( ellipse );
            stroke.width( m_pen->GetWidth() );
            stroke.line_cap( m_pen->GetCap() );

            m_ras.reset();

            applyClipping();

            m_ras.add_path( stroke );
            agg::render_scanlines_aa_solid( m_ras, m_sl, *m_renderer, agg::rgba8( m_pen->GetAggColor() ) );
        }
        else
        {
            agg::conv_curve<agg::ellipse> curve( ellipse );
            agg::conv_dash<agg::conv_curve<agg::ellipse>, agg::vcgen_markers_term> dash( curve );
            agg::conv_stroke<agg::conv_dash<agg::conv_curve<agg::ellipse>, agg::vcgen_markers_term> > stroke( dash );

            stroke.line_cap( m_pen->GetCap() );
            stroke.width( m_pen->GetWidth() );

            double *dashes;

            int dashesNum = m_pen->GetDashes( &dashes );

            if ( dashes != NULL )
            {
                for ( int j = 0; j < dashesNum; ++j )
                {
                    dash.add_dash( dashes[2*j], dashes[2*j+1] );
                }
                dash.dash_start( dashes[0] + dashes[1] / 2 );
            }
            else
            {
                dash.add_dash( 10.0, 10.0 );
                dash.dash_start( 15 );
            }

            m_ras.reset();

            applyClipping();

            m_ras.add_path( stroke );

            agg::render_scanlines_aa_solid( m_ras, m_sl, *m_renderer, agg::rgba8( m_pen->GetAggColor() ) );
        }
    }
}

void wxAgg2::drawEllipse( double centerx, double centery, double r1, double r2, double startAngleRad, double deltaAngleRad )
{
    IS_INITIALIZED()

    agg::bezier_arc arc( centerx, centery, r1, r2, startAngleRad, deltaAngleRad );

    agg::path_storage path;

    path.concat_path( arc, 0 );
    path.line_to( centerx, centery );
    path.close_polygon();

    if ( m_brush->GetStyle() != wxAGGBRUSH_TRANSPARENT )
    {
        m_ras.reset();

        applyClipping();

        // draw outlines
        drawOutlines( path );

        agg::conv_curve<agg::path_storage> curve( path );

        // gradient brush
        if ( m_brush->GetStyle() == wxAGGBRUSH_GRADIENT )
        {
            m_ras.add_path( curve );

            gradient_rendering( centerx, centery, r1, r2 );
        }
        else
        {
            if ( m_brush->GetStyle() == wxAGGBRUSH_SOLID )
            {
                m_ras.add_path( curve );
                agg::render_scanlines_aa_solid( m_ras, m_sl, *m_renderer, agg::rgba8( m_brush->GetAggColor() ) );
            }
            else
            {
                generatePattern();

                typedef agg::wrap_mode_repeat wrap_x_type;
                typedef agg::wrap_mode_repeat wrap_y_type;
                typedef agg::image_accessor_wrap<pixfmtT, wrap_x_type, wrap_y_type> img_source_type;
                typedef agg::span_pattern_rgba<img_source_type> span_gen_type;

                agg::span_allocator<agg::rgba8> sa;
                pixfmtT         img_pixf( m_pattern_rbuf );
                img_source_type img_src( img_pixf );

                unsigned int xoff = m_pattern_w - (unsigned int) (centerx - r1) % m_pattern_w;
                unsigned int yoff = m_pattern_h - (unsigned int) (centery - r2) % m_pattern_h;

                span_gen_type   sg( img_src, xoff, yoff );

                m_ras.add_path( curve );
                agg::render_scanlines_aa( m_ras, m_sl, *m_renderer, sa, sg );
            }
        }
    }

    if ( m_pen->GetStyle() != wxAGGPEN_TRANSPARENT )
    {
        if ( m_pen->GetStyle() == wxAGGPEN_SOLID )
        {
            agg::conv_stroke<agg::path_storage> stroke( path );
            stroke.line_cap( m_pen->GetCap() );
            stroke.width( m_pen->GetWidth() );

            m_ras.reset();
            applyClipping();
            m_ras.add_path( stroke );
            agg::render_scanlines_aa_solid( m_ras, m_sl, *m_renderer, agg::rgba8( m_pen->GetAggColor() ) );
        }
        else
        {
            agg::conv_curve<agg::path_storage> curve( path );
            agg::conv_dash<agg::conv_curve<agg::path_storage>, agg::vcgen_markers_term> dash( curve );
            agg::conv_stroke<agg::conv_dash<agg::conv_curve<agg::path_storage>, agg::vcgen_markers_term> > stroke( dash );

            stroke.width( m_pen->GetWidth() );
            stroke.line_cap( m_pen->GetCap() );

            double *dashes;

            int dashesNum = m_pen->GetDashes( & dashes );

            if ( dashes != NULL )
            {
                for ( int j = 0; j < dashesNum; ++j )
                {
                    dash.add_dash( dashes[2*j], dashes[2*j+1] );
                }
                dash.dash_start( dashes[0] + dashes[1] / 2 );
            }
            else
            {
                dash.add_dash( 10.0, 10.0 );
                dash.dash_start( 15 );
            }

            m_ras.reset();

            applyClipping();

            m_ras.add_path( stroke );

            agg::render_scanlines_aa_solid( m_ras, m_sl, *m_renderer, agg::rgba8( m_pen->GetAggColor() ) );
        }
    }
}

void wxAgg2::drawCircle( double centerx, double centery, double radius )
{
    IS_INITIALIZED()

    drawEllipse( centerx, centery, radius, radius );
}

void wxAgg2::drawCircle( wxRealPoint center, double radius )
{
    IS_INITIALIZED()

    drawEllipse( center.x, center.y, radius, radius );
}

void wxAgg2::drawGradientFilledCircle( const wxColour& begincolor, const wxColour& endcolor, double deg, double centerx, double centery, double radius )
{
    IS_INITIALIZED()

    /// TODO
    /// alpha?
    /// border?

    // The gradient objects declarations
    //----------------
    gradient_func_type  gradient_func;                   // The gradient function
    agg::trans_affine   gradient_mtx;                    // Affine transformer
    interpolator_type   span_interpolator(gradient_mtx); // Span interpolator
    //interpolator_type   span_interpolator_trans((gradient_mtx); // Span interpolator

    //interpolator_type   span_interpolator(); // Span interpolator
    span_allocator_type span_allocator;        // Span Allocator
    color_array_type    color_array;           // Gradient colors


    // Declare the gradient span itself.
    // The last two arguments are so called "d1" and "d2"
    // defining two distances in pixels, where the gradient starts
    // and where it ends. The actual meaning of "d1" and "d2" depands
    // on the gradient function.
    //----------------
    span_gradient_type span_gradient( span_interpolator, gradient_func, color_array, 0, 100 );

    // The gradient renderer
    //----------------
    renderer_gradient_type ren_gradient( *m_renderer, span_allocator, span_gradient );

    fillColorArray( color_array, Agg::convertToRGBA( begincolor ), Agg::convertToRGBA( endcolor ) );

    agg::ellipse ell( centerx, centery, radius, radius );

    deg = 180.0 - deg;

    double x = radius * cos( deg * agg::pi / 180.0 );
    double y = radius * sin( deg * agg::pi / 180.0 );

    m_ras.reset();

    applyClipping();

    // draw outlines
    drawOutlines( ell );

    m_ras.add_path( ell );

    gradient_mtx.reset();
    gradient_mtx *= agg::trans_affine_scaling( 2 * radius / 100.0 );
    gradient_mtx *= agg::trans_affine_rotation( deg * agg::pi / 180.0 );
    gradient_mtx *= agg::trans_affine_translation(  centerx - x , centery - y  );
    gradient_mtx.invert();

    agg::render_scanlines( m_ras, m_sl, ren_gradient );
}

/// font handling
bool wxAgg2::loadTTFont( const wxString& fontfilename, double height, double weight, double width )
{
    IS_INITIALIZED( false )

    return m_font->loadFont( fontfilename, height, weight, width );
}

void wxAgg2::setFontHeight( double h )
{
    IS_INITIALIZED()

    m_font->setHeight( h );
}

double wxAgg2::getFontHeight() const
{
    IS_INITIALIZED( -1.0 )

    return m_font->getHeight();
}

void wxAgg2::setFontWidth( double w )
{
    IS_INITIALIZED()

    m_font->setWidth( w );
}

double wxAgg2::getFontWidth() const
{
    IS_INITIALIZED( -1.0 )

    return m_font->getWidth();
}

/// text
void wxAgg2::drawText( const wxString& text, double x, double y, double w, double h, double s )
{
    IS_INITIALIZED()

    if ( m_pen->GetStyle() != wxAGGPEN_TRANSPARENT )
    {

        agg::gsv_text agg_text;
        agg_text.size( h );
        agg_text.text( text.mb_str( wxConvUTF8 ) );
        agg_text.start_point( x, y );
        agg_text.flip( true );


        agg::conv_stroke<agg::gsv_text> stroke( agg_text );
        stroke.width( s );

        m_ras.reset();
        applyClipping();
        m_ras.add_path( stroke );
        agg::render_scanlines_aa_solid( m_ras, m_sl, *m_renderer, agg::rgba8( m_pen->GetAggColor() ) );
    }
}

/* static */
double wxAgg2::getTextWidth( const wxString& text, double h )
{
    agg::gsv_text agg_text;
    agg_text.size( h );
    agg_text.text( text.mb_str( wxConvUTF8 ) );
    agg_text.start_point( 0, 0 );
    agg_text.flip( true );

    double x1, x2, y1, y2;
	agg::bounding_rect_single( agg_text, 0, &x1, &y1, &x2, &y2 );

    return  x2 - x1;
}

wxRealRect wxAgg2::drawTTFText( const wxString& text, double x, double y, double angleDeg )
{
    IS_INITIALIZED( wxNullRealRect )

    wxRealRect rect = drawTTFText( x, y, text, angleDeg );
    return rect;
}

wxRealRect wxAgg2::drawTTFText( double x, double y, const wxString& text, double angleDeg )
{
    IS_INITIALIZED( wxNullRealRect )

    if ( m_pen->GetStyle() == wxAGGPEN_TRANSPARENT )
        return wxNullRealRect;

    if ( m_font->isOk() == false )
        return wxNullRealRect;

    wxRealRect box;

    font_manager_type fman( m_font->getFontEngine() );

    agg::conv_curve<font_manager_type::path_adaptor_type> curves( fman.path_adaptor() );
    agg::conv_contour<agg::conv_curve<font_manager_type::path_adaptor_type> > contour( curves );

    m_ras.reset();

    applyClipping();

    bool kerning = true;

    curves.approximation_scale( 2.0 );
    contour.auto_detect_orientation( false );

    double height = m_font->getFontEngine().height();
    double fw = m_font->getFontWeight();

    contour.width( fw * height * 0.05 );

    m_ras.gamma( agg::gamma_power( 1.0 ) );

    // rotation
    agg::trans_affine mtx;
    mtx *= agg::trans_affine_rotation( agg::deg2rad( angleDeg ) );

    m_font->getFontEngine().transform( mtx );

    double x_ = x;
    double y_ = y;

    box.x = x;
    box.y = y;

    int maxh = 0;

    const wxChar* convertedTxt = text.c_str();

    for ( unsigned int j = 0; j < text.Len(); ++j )
    {
        unsigned int char_code = (unsigned int)convertedTxt[j];

        if ( char_code == '\n' )
        {
            x_ = x;
            y_ += 1.2 * maxh;
            continue;
        }

        const agg::glyph_cache* glyph = fman.glyph( char_code );

        if ( glyph )
        {
            if ( kerning )
            {
                fman.add_kerning( &x_, &y_ );
            }

            fman.init_embedded_adaptors( glyph, x_, y_ );

            if ( fabs( fw ) <= 0.01 )
            {
                m_ras.add_path( curves );
            }
            else
            {
                m_ras.add_path( contour );
            }

            // increment pen position
            x_ += glyph->advance_x;
            y_ += glyph->advance_y;

            int glyph_h = abs( glyph->bounds.y2 - glyph->bounds.y1 );
            if ( glyph_h > maxh )
                maxh = glyph_h;

            if ( x_ > (double) m_width - 0.5 )
                break;
            if ( y_ > (double) m_height - 0.5 )
                break;
        }
    }

    agg::render_scanlines_aa_solid( m_ras, m_sl, *m_renderer, agg::rgba8( m_pen->GetAggColor() ) );

    box.w = x_ - x + maxh * sin( agg::deg2rad( angleDeg ) ) + fw;
    box.h = y_ - y + height * cos( agg::deg2rad( angleDeg ) ) + fw;

    return box;
}

wxRealRect wxAgg2::getTTFTextBound( const wxString& text, double angleDeg, double x, double y )
{
    IS_INITIALIZED( wxNullRealRect )

    if ( !m_font->isOk() )
        return wxNullRealRect;

    wxRealRectArray tmp;

    return getTTFTextBound(text, tmp, angleDeg, x, y );
}

wxRealRect wxAgg2::getTTFTextBound( const wxString& text, wxRealRectArray& charRects, double angleDeg, double x, double y )
{
    IS_INITIALIZED( wxNullRealRect )

    if ( !m_font->isOk() )
        return wxNullRealRect;

    wxRealRect box;

    font_manager_type fman( m_font->getFontEngine() );

    bool kerning = true;

    double height = m_font->getFontEngine().height();

    m_font->getFontEngine().gamma( agg::gamma_power( 1.0 ) );

    agg::trans_affine mtx;

    mtx *= agg::trans_affine_rotation( agg::deg2rad( angleDeg ) );

    m_font->getFontEngine().transform( mtx );

    double x_ = x;
    double y_ = y;

    box.x = x;
    box.y = y;

    const wxChar * convertedTxt = text.c_str();

    int maxh = 0;

    for ( unsigned int j = 0; j < text.Len(); ++j )
    {
        unsigned int char_code = (unsigned int)convertedTxt[j];

        if ( char_code == '\n' )
        {
            x_ = x;
            y_ += 1.2 * maxh;
            continue;
        }

        const agg::glyph_cache* glyph = fman.glyph( char_code );

        if ( glyph )
        {
            if ( kerning )
            {
                fman.add_kerning( &x_, &y_ );
            }

            fman.init_embedded_adaptors( glyph, x_, y_ );

            wxRealRect cr;

            cr.x = x_;
            cr.y = y_;

            x_ += glyph->advance_x;
            y_ += glyph->advance_y;


            int glyph_h = abs( glyph->bounds.y2 - glyph->bounds.y1 );
            if ( glyph_h > maxh )
                maxh = glyph_h;

            cr.w = glyph->advance_x;
            cr.h = glyph_h;

            charRects.Add(cr);
        }
    }

    box.w = x_ - x + maxh * sin( agg::deg2rad( angleDeg ) ) + m_font->getFontWeight();
    box.h = y_ - y + height * cos( agg::deg2rad( angleDeg ) ) + m_font->getFontWeight();

    return box;
}


/// OTHER
/* static */
wxColor wxAgg2::gradient( const wxColor& bc, const wxColor& ec, double k )
{
    agg::rgba bcRGBA = Agg::convertToRGBA( bc );
    agg::rgba ecRGBA = Agg::convertToRGBA( ec );

    wxColor gc = Agg::convertToColor( bcRGBA.gradient( ecRGBA, k) );

    return gc;
}

void wxAgg2::drawInnerRectShadow( int width, int alpha, const wxRect& rect, const wxColor& color )
{
    IS_INITIALIZED()

    wxRealRect rrect( (double)rect.x + 0.5, (double)rect.y + 0.5, (double)rect.width - 1.0 , (double)rect.height - 1.0 );
    drawInnerRectShadow( width, alpha, rrect, color );
}

void wxAgg2::drawInnerRectShadow( int width, int alpha, const wxRealRect& rect, const wxColor& color )
{
    IS_INITIALIZED()

    int alphastep = alpha / width;

    double pw = m_pen->GetWidth();
    wxColor c1 = m_pen->GetWxColor();

    m_pen->SetWidth( 1.0 );


    // draw vertical lines
    for( int a = 0; a < width; ++a )
    {
        wxColor c( color.Red(), color.Green(), color.Blue(), alpha - ( a * alphastep ) );
        m_pen->SetColor( c );

        drawLine( rect.GetLeft(), rect.GetTop() + a, rect.GetRight(), rect.GetTop() + a );
    }

    // draw horizontal lines
    for( int a = 0; a < width; ++a )
    {
        wxColor c( color.Red(), color.Green(), color.Blue(), alpha - ( a * alphastep ) );
        m_pen->SetColor( c );

        drawLine(rect.GetLeft() + a, rect.GetTop(), rect.GetLeft()+a, rect.GetBottom() );
    }

    m_pen->SetWidth( pw );
    m_pen->SetColor(c1);
}

/// TESTING

void wxAgg2::aggTest1()
{
    IS_INITIALIZED()

    /// TODO
}

void wxAgg2::aggTest2()
{
    IS_INITIALIZED()

    /// TODO
}

/// ---------------------------------------------------------
/// private methods

void wxAgg2::Create( uint32_t width, uint32_t height, const wxColor& clearColor )
{
    int tmp_w   = width;
    int tmp_h   = height;

    if ( tmp_h <= 0 || tmp_w <= 0 )
    {
        m_initialized = false;
        return;
    }

    m_width     = width;
    m_height    = height;

    m_buffer    = NULL;
    m_rb        = NULL;
    m_pixf      = NULL;
    m_renderer  = NULL;
    m_pen       = NULL;
    m_brush     = NULL;
    m_font      = NULL;
    m_blur      = NULL;
    m_states    = NULL;

    try {
        m_buffer = new unsigned char[m_width * m_height * 4]; // rgba
        m_rb = new agg::rendering_buffer( m_buffer, m_width, m_height, m_width * 4 );

        m_pixf = new pixfmtT( *m_rb );
        m_renderer = new ren_base( *m_pixf );

        //m_renderer->clear( agg::rgba( 0, 0, 0, 0 ) );
        m_renderer->clear( Agg::convertToRGBA( clearColor ) );

        m_alphaBuffer = NULL;
        m_imgBuffer = NULL;

        m_clippingRect = wxDefaultRect;

        m_pen = new wxAggPen( wxDefaultAggPen );
        m_brush = new wxAggBrush( wxTransparentAggBrush );
        m_patternBuffer = NULL;

        m_pattern_w = 0;
        m_pattern_h = 0;

        m_font = new wxAggFont;

        m_blur = new wxAggBlur( 0, 0, wxColor( 0, 0, 0, 0 ) );

        m_states = new wxAggObjectStack;

        m_initialized = true;
    }
    catch ( std::bad_alloc& e )
    {
        if ( m_buffer ) delete[] m_buffer;
        if ( m_rb ) delete[] m_rb;
        if ( m_pixf ) delete m_pixf;
        if ( m_renderer ) delete m_renderer;
        if ( m_pen ) delete m_pen;
        if ( m_brush ) delete m_brush;
        if ( m_font ) delete m_font;
        if ( m_blur ) delete m_blur;
        if ( m_states ) delete m_states;

        m_initialized = false;
    }
}

void wxAgg2::getBuffers(const wxRect& renderBox, bool isSupportAlphaChannel)
{
    IS_INITIALIZED()

    if ( m_alphaBuffer != NULL && isSupportAlphaChannel )
    {
        delete[] m_alphaBuffer;
    }

    if ( m_imgBuffer != NULL )
    {
        delete[] m_imgBuffer;
    }

    if(isSupportAlphaChannel) m_alphaBuffer = new unsigned char[renderBox.width * renderBox.height];

    m_imgBuffer = new unsigned char[renderBox.width * renderBox.height * 3];

    long src_x = 0, src_y = 0, src_idx = 0, target_x = 0, target_y = 0, target_idx = 0;

    for(src_y = renderBox.GetTop(), target_y=0; src_y <= renderBox.GetBottom(); src_y++, target_y++ ) {
        for(src_x = renderBox.GetLeft(), target_x=0; src_x <= renderBox.GetRight(); src_x++, target_x++ ) {

            src_idx = src_y * m_width + src_x;
            target_idx = target_y * renderBox.width + target_x;

            memcpy( m_imgBuffer + 3 * target_idx, m_buffer + 4 * src_idx, 3 );

            if(isSupportAlphaChannel) m_alphaBuffer[target_idx] = m_buffer[4 * src_idx + 3];

        }
    }

}

void wxAgg2::getBuffers(unsigned char* rgb_buffer, unsigned char* a_buffer)
{
    IS_INITIALIZED()

    for ( size_t i = 0; i < m_width * m_height; ++i )
    {
        memcpy( rgb_buffer + 3*i, m_buffer + 4*i, 3 );

        if(a_buffer) a_buffer[i] = m_buffer[4*i + 3];
    }
}

void wxAgg2::applyClipping()
{
    IS_INITIALIZED()

    if ( m_clippingRect != wxDefaultRect )
    {
        m_ras.clip_box( m_clippingRect.x, m_clippingRect.y,
            m_clippingRect.GetRight(), m_clippingRect.GetBottom() );
    }
    else
    {
        m_ras.clip_box( 0, 0, m_width , m_height );
    }
}

void wxAgg2::generatePattern()
{
    IS_INITIALIZED()

    if ( m_brush->GetStyle() == wxAGGBRUSH_TRANSPARENT ||
        m_brush->GetStyle() == wxAGGBRUSH_SOLID )
        return;

    // image pattern
    if ( m_brush->GetStyle() == wxAGGBRUSH_BITMAP )
    {
        wxBitmap& bmp = m_brush->GetBitmap();
        if ( bmp.IsOk() == false )
            return;

        if ( m_patternBuffer )
            delete[] m_patternBuffer;

        unsigned int w = (unsigned int)( bmp.GetWidth() );
        unsigned int h = (unsigned int)( bmp.GetHeight() );

        m_patternBuffer = new agg::int8u[w*h*4];
        wxImage img = bmp.ConvertToImage();
        unsigned char* data = img.GetData();

        for ( size_t i = 0; i < w*h; ++i )
        for ( size_t i = 0; i < w*h; ++i )
            memcpy( m_patternBuffer + 4 * i, data + 3 * i, 3 );

        data = img.GetAlpha();
        for ( size_t i = 0; i < w*h; ++i )
            memcpy( m_patternBuffer + 4*i + 3, data + i, 1 );

        m_pattern_rbuf.attach( m_patternBuffer, w, h, w * 4 );

        m_pattern_w = w;
        m_pattern_h = h;

        return;
    }

    // simple patterns
    const unsigned int patternsize = 8;

    m_pattern_w = patternsize;
    m_pattern_h = patternsize;

    if ( m_patternBuffer )
        delete[] m_patternBuffer;

    m_patternBuffer = new agg::int8u[patternsize*patternsize*4];
    m_pattern_rbuf.attach( m_patternBuffer, patternsize, patternsize, patternsize*4 );
    pixfmtT pixf(m_pattern_rbuf);

    agg::renderer_base<pixfmtT> rb( pixf );
    agg::renderer_scanline_aa_solid<agg::renderer_base<pixfmtT> > rs( rb );

    rb.clear( agg::rgba( 0, 0, 0, 0 ) );

    agg::path_storage path;

    switch ( m_brush->GetStyle() )
    {
        case wxAGGBRUSH_BDIAGONAL_HATCH:
           path.move_to( 0, 0 );
           path.line_to( patternsize, patternsize );
           break;

        case wxAGGBRUSH_FDIAGONAL_HATCH:
            path.move_to( patternsize, 0 );
            path.line_to( 0, patternsize );
            break;

        case wxAGGBRUSH_HORIZONTAL_HATCH:
            path.move_to( 0, patternsize/2 );
            path.line_to( patternsize, patternsize/2 );
            break;

        case wxAGGBRUSH_VERTICAL_HATCH:
            path.move_to( patternsize/2, 0 );
            path.line_to( patternsize/2, patternsize );
            break;

        case wxAGGBRUSH_CROSSDIAG_HATCH:
            path.move_to( 0, 0 );
            path.line_to( patternsize, patternsize );
            path.move_to( patternsize, 0 );
            path.line_to( 0, patternsize );
            break;

        case wxAGGBRUSH_CROSS_HATCH:
            path.move_to( 0, patternsize/2 );
            path.line_to( patternsize, patternsize/2 );
            path.move_to( patternsize/2, 0 );
            path.line_to( patternsize/2, patternsize );
            break;

        default:
            break;
    }

    agg::conv_stroke< agg::path_storage > stroke( path );
    stroke.line_join( agg::miter_join );

    stroke.line_cap( agg::butt_cap );
    stroke.width( 1.0 );

    m_ras.reset();

    m_ras.add_path( stroke );

    agg::render_scanlines_aa_solid( m_ras, m_sl, rb, agg::rgba8( m_brush->GetAggColor() ) );
}

// A simple function to form the gradient color array
// consisting of 3 colors, "begin", "middle", "end"
// ---------------------------------------------------

void wxAgg2::fillColorArray( color_array_type& array, agg::rgba8 begin, agg::rgba8 end )
{
    unsigned int s = array.size();
    for ( unsigned int i = 0; i < s; ++i )
    {
        array[i] = begin.gradient( end, (double)i / (double) s );
    }
}

template<class T>
void wxAgg2::drawOutlines( T path )
{
    if ( m_blur->gaussianradius <= 0 )
        return;

    m_ras.reset();
    agg::conv_curve<T> shp(path);

    agg::trans_perspective shadow_persp;
    shadow_persp = shadow_persp.translate( m_blur->translate.x, m_blur->translate.y );

    agg::conv_transform<agg::conv_curve<T>, agg::trans_perspective> shadow_trans( shp, shadow_persp );

    // Render shadow
    m_ras.add_path( shadow_trans );
    agg::render_scanlines_aa_solid( m_ras, m_sl, *m_renderer, m_blur->color.getColor() );
    agg::rgba c = m_blur->color.getColor();

    // Calculate the bounding box and extend it by the blur radius
    agg::rect_d bbox;
    agg::bounding_rect_single( shadow_trans, 0, &bbox.x1, &bbox.y1, &bbox.x2, &bbox.y2 );

    bbox.x1 -= m_blur->gaussianradius*1.5;
    bbox.y1 -= m_blur->gaussianradius*1.5;
    bbox.x2 += m_blur->gaussianradius*1.5;
    bbox.y2 += m_blur->gaussianradius*1.5;

    // Create a new pixel renderer and attach it to the main one as a child image.
    // It returns true if the attachment suceeded. It fails if the rectangle
    // (bbox) is fully clipped.
    //------------------
    agg::rendering_buffer rbuf2;
    pixfmtT pixf2( rbuf2 );

    agg::recursive_blur<agg::rgba8, agg::recursive_blur_calc_rgba<> > recursive_blur;

    if( pixf2.attach( *m_pixf, int(bbox.x1), int(bbox.y1), int(bbox.x2), int(bbox.y2) ) )
    {
        // True Gaussian Blur
        recursive_blur.blur( pixf2, m_blur->gaussianradius );
    }
}

void wxAgg2::gradient_rendering( double centerx, double centery, double r1, double r2 )
{
    gradient_func_type  gradient_func;
    agg::trans_affine   gradient_mtx;
    interpolator_type   span_interpolator(gradient_mtx);
    span_allocator_type span_allocator;
    color_array_type    color_array;

    // 100 step
    span_gradient_type span_gradient( span_interpolator, gradient_func, color_array, 0, M_GRADIENT_SIZE );
    renderer_gradient_type ren_gradient( *m_renderer, span_allocator, span_gradient );
    fillColorArray( color_array, m_brush->GetAggColor(), m_brush->GetAggEndColor() );

    double deg = m_brush->GetGradientOrientation();
    double x = r1 * cos( deg * agg::pi / 180.0 );
    double y = r2 * sin( deg * agg::pi / 180.0 );

    //double wh = Max( r1, r2 );
    double wh = fabs(cos( deg * agg::pi / 180.0 )) < COS_45 ? r2 : r1;

    gradient_mtx.reset();
    gradient_mtx *= agg::trans_affine_scaling( wh * 2 / (double)M_GRADIENT_SIZE );
    gradient_mtx *= agg::trans_affine_rotation( deg * agg::pi / 180.0 );
    gradient_mtx *= agg::trans_affine_translation( centerx - x , centery - y );
    gradient_mtx.invert();

    agg::render_scanlines( m_ras, m_sl, ren_gradient );
}

