#include "wxAggBrush.h"
#include "aggHelper.h"

const wxAggBrush wxDefaultAggBrush = wxAggBrush( wxColor(255,255,255,255), wxAGGBRUSH_SOLID );
const wxAggBrush wxTransparentAggBrush = wxAggBrush( wxColor( 0, 0, 0, 0 ), wxAGGBRUSH_TRANSPARENT );

IMPLEMENT_DYNAMIC_CLASS( wxAggBrush, wxAggObject );

wxAggBrush::wxAggBrush()
{
    m_color = m_endcolor = agg::rgba( 1.0, 1.0, 1.0, 1.0 );
    m_style = wxAGGBRUSH_TRANSPARENT;

    m_graddegree = 0;

    m_bitmap = wxNullBitmap;
    m_isOk = true;
}

wxAggBrush::wxAggBrush( const wxColour& color, int style )
{
    m_color = Agg::convertToRGBA( color );
    m_endcolor = Agg::convertToRGBA( color );
    if ( style < 0 || style > 9 )
        m_style = wxAGGBRUSH_SOLID;
    else
        m_style = style;

    m_graddegree = 0;

    m_bitmap = wxNullBitmap;
    m_isOk = true;
}

wxAggBrush::wxAggBrush( const wxBrush& brush )
{
    m_color = Agg::convertToRGBA( brush.GetColour() );

    m_endcolor = Agg::convertToRGBA( brush.GetColour() );

    switch ( brush.GetStyle() )
    {
        case wxTRANSPARENT:     m_style = wxAGGBRUSH_TRANSPARENT; break;
        case wxBDIAGONAL_HATCH: m_style = wxAGGBRUSH_BDIAGONAL_HATCH; break;
        case wxCROSSDIAG_HATCH: m_style = wxAGGBRUSH_CROSSDIAG_HATCH; break;
        case wxFDIAGONAL_HATCH: m_style = wxAGGBRUSH_FDIAGONAL_HATCH; break;
        case wxCROSS_HATCH:     m_style = wxAGGBRUSH_CROSS_HATCH; break;
        case wxHORIZONTAL_HATCH:m_style = wxAGGBRUSH_HORIZONTAL_HATCH; break;
        case wxVERTICAL_HATCH:  m_style = wxAGGBRUSH_VERTICAL_HATCH; break;
        case wxSOLID:
        default:                m_style = wxAGGBRUSH_SOLID; break;
    }

    if ( brush.GetStyle() == wxSTIPPLE )
    {
        m_bitmap = wxBitmap(*brush.GetStipple());
    }
    else m_bitmap = wxNullBitmap;

    m_graddegree = 0;

    m_isOk = true;
}

wxAggBrush::wxAggBrush( const wxAggBrush& brush )
{
    m_color = brush.m_color;
    m_endcolor = brush.m_endcolor;
    m_graddegree = brush.m_graddegree;
    m_style = brush.m_style;
    m_bitmap = brush.m_bitmap;
    m_isOk = brush.m_isOk;
}

wxAggBrush::wxAggBrush( const wxBitmap& bitmap )
{
    m_color = agg::rgba( 0, 0, 0, 0 );
    m_endcolor = agg::rgba( 0, 0, 0, 0 );
    m_style = wxAGGBRUSH_BITMAP;
    m_bitmap = bitmap;
    m_graddegree = 0;
    m_isOk = true;
}

wxAggBrush::~wxAggBrush()
{
}

wxAggBrush wxAggBrush::operator = ( const wxAggBrush& brush )
{
    if ( &brush == this )
        return *this;

    m_color = brush.m_color;
    m_endcolor = brush.m_endcolor;
    m_graddegree = brush.m_graddegree;
    m_style = brush.m_style;
    m_bitmap = brush.m_bitmap;
    m_isOk = brush.m_isOk;

    return *this;
}

bool wxAggBrush::IsOk() const
{
    return m_isOk;
}

wxColour wxAggBrush::GetWxColor() const
{
    return Agg::convertToColor( m_color );
}

agg::rgba wxAggBrush::GetAggColor() const
{
    return m_color;
}

agg::rgba wxAggBrush::GetAggEndColor() const
{
    return m_endcolor;
}

wxBitmap& wxAggBrush::GetBitmap()
{
    return m_bitmap;
}

int wxAggBrush::GetStyle() const
{
    return m_style;
}

double wxAggBrush::GetGradientOrientation() const
{
    return m_graddegree;
}

void wxAggBrush::SetColor( const wxColour& color )
{
    m_color = Agg::convertToRGBA( color );
}

void wxAggBrush::SetColor( const agg::rgba& color )
{
    m_color = color;
}

void wxAggBrush::SetEndColor( const wxColour& color )
{
    m_endcolor = Agg::convertToRGBA( color );
}

void wxAggBrush::SetEndColor( const agg::rgba& color )
{
    m_endcolor = color;
}

void wxAggBrush::SetGradientOrientation( const double& degree )
{
    m_graddegree = degree;
}

void wxAggBrush::SetBitmap( const wxBitmap& bitmap )
{
    m_bitmap = bitmap;
}

void wxAggBrush::SetBrush( const wxBrush& brush )
{
    m_color = Agg::convertToRGBA( brush.GetColour() );
    switch ( brush.GetStyle() )
    {
        case wxTRANSPARENT:     m_style = wxAGGBRUSH_TRANSPARENT; break;
        case wxBDIAGONAL_HATCH: m_style = wxAGGBRUSH_BDIAGONAL_HATCH; break;
        case wxCROSSDIAG_HATCH: m_style = wxAGGBRUSH_CROSSDIAG_HATCH; break;
        case wxFDIAGONAL_HATCH: m_style = wxAGGBRUSH_FDIAGONAL_HATCH; break;
        case wxCROSS_HATCH:     m_style = wxAGGBRUSH_CROSS_HATCH; break;
        case wxHORIZONTAL_HATCH:m_style = wxAGGBRUSH_HORIZONTAL_HATCH; break;
        case wxVERTICAL_HATCH:  m_style = wxAGGBRUSH_VERTICAL_HATCH; break;
        case wxSOLID:
        default:                m_style = wxAGGBRUSH_SOLID; break;
    }

    if ( brush.GetStyle() == wxSTIPPLE )
    {
        m_bitmap = wxBitmap(*brush.GetStipple());
    }
    else m_bitmap = wxNullBitmap;

    m_graddegree = 0;

    m_isOk = true;
}

void wxAggBrush::SetBrush( const wxAggBrush& brush )
{
    m_color = brush.m_color;
    m_endcolor = brush.m_endcolor;
    m_graddegree = brush.m_graddegree;
    m_style = brush.m_style;
    m_bitmap = brush.m_bitmap;
    m_isOk = brush.m_isOk;
}

void wxAggBrush::SetStyle( int style )
{
    if ( style < 0 || style > 9 )
        m_style = wxAGGBRUSH_SOLID;
    else
        m_style = style;

    if ( style != wxAGGBRUSH_BITMAP && style != wxAGGBRUSH_TRANSPARENT )
        m_bitmap = wxNullBitmap;
}
