#include "wxAggColorTestFrame.h"

#include <wx/wx.h>
#include <wx/dcbuffer.h>

#include "wxAgg.h"
#include "wxAgg2.h"
#include "wxAggBrush.h"

#include "../DrawHelper.h"
#include "../logger.h"

BEGIN_EVENT_TABLE(ColorWindow, wxControl)

    EVT_SIZE(ColorWindow::OnSize)
    EVT_PAINT(ColorWindow::OnPaint)

END_EVENT_TABLE()


BEGIN_EVENT_TABLE(wxAggColorTestFrame, wxFrame)

    EVT_COMMAND_SCROLL(idSliderRed, wxAggColorTestFrame::OnScroll)
    EVT_COMMAND_SCROLL(idSliderGreen, wxAggColorTestFrame::OnScroll)
    EVT_COMMAND_SCROLL(idSliderBlue, wxAggColorTestFrame::OnScroll)
    EVT_COMMAND_SCROLL(idSliderAlpha, wxAggColorTestFrame::OnScroll)

END_EVENT_TABLE()

wxAggColorTestFrame::wxAggColorTestFrame(wxWindow* parent)
{
    //ctor
    Create(parent, wxID_ANY, _T("agg color test frame"));
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
    ClearBackground();

    m_sliderRed = new wxSlider(this, idSliderRed , 128, 0, 255);
    m_sliderGreen = new wxSlider(this, idSliderGreen , 128, 0, 255);
    m_sliderBlue = new wxSlider(this, idSliderBlue , 128, 0, 255);
    m_sliderAlpha = new wxSlider(this, idSliderAlpha , 128, 0, 255);

    m_colorWin = new ColorWindow(this);


    wxBoxSizer * mainsizer = new wxBoxSizer(wxHORIZONTAL);

    wxBoxSizer * setsizer = new wxBoxSizer(wxVERTICAL);

    setsizer->Add(new wxStaticText(this, wxID_ANY, _T("Red:")), 0, wxALL, 5);
    setsizer->Add(m_sliderRed, 0, wxALL, 5);

    setsizer->Add(new wxStaticText(this, wxID_ANY, _T("Green:")), 0, wxALL, 5);
    setsizer->Add(m_sliderGreen, 0, wxALL, 5);

    setsizer->Add(new wxStaticText(this, wxID_ANY, _T("Blue:")), 0, wxALL, 5);
    setsizer->Add(m_sliderBlue, 0, wxALL, 5);

    setsizer->Add(new wxStaticText(this, wxID_ANY, _T("Alpha:")), 0, wxALL, 5);
    setsizer->Add(m_sliderAlpha, 0, wxALL, 5);

    mainsizer->Add(m_colorWin, 1, wxEXPAND | wxALL, 5);

    mainsizer->Add(setsizer, 0, wxALL, 5);


    SetSizer(mainsizer);
    mainsizer->Fit(this);
    mainsizer->SetSizeHints(this);
}

wxAggColorTestFrame::~wxAggColorTestFrame()
{
    //dtor
}

void wxAggColorTestFrame::OnScroll(wxScrollEvent& evt)
{
    wxColor c( m_sliderRed->GetValue(), m_sliderGreen->GetValue(), m_sliderBlue->GetValue(), m_sliderAlpha->GetValue() );
    m_colorWin->SetColor(c);
}

ColorWindow::ColorWindow(wxWindow* parent)
{
    m_color = wxColor(255,0,0,128);
    Create(parent, wxID_ANY);
}

void ColorWindow::SetColor(const wxColor& color)
{
    m_color = color;
    Refresh(false);
}

void ColorWindow::OnSize(wxSizeEvent& evt)
{
    Refresh(false);
}

#include "wxAggBlur.h"

void ColorWindow::OnPaint(wxPaintEvent& evt)
{
    wxBufferedPaintDC dc(this);
    dc.Clear();

    int w = GetClientSize().x;
    int h = GetClientSize().y;

    wxColor cf(255,255,255);
    wxColor cb(158,158,158);

    dc.SetBackground( wxBrush(cb) );
    dc.SetBrush( wxBrush(cf) );
    dc.SetPen( wxPen(cf) );
    dc.Clear();

    int s = 10;

    for( int y=0; y<h; y+=s )
        for( int x=( (y%(2*s)) ? 0 : s ); x<w; x+=(2*s) )
            dc.DrawRectangle( x,y,s,s );

    //wxAgg aggCanvas(this);
    /*aggCanvas.setBrush(cb); aggCanvas.setPen(cb);aggCanvas.drawFilledRectangle(0,0,w,h,1.0);
    aggCanvas.setBrush(cf); aggCanvas.setPen(cf);
    int s = 10;

    for( int y=0; y<h; y+=s )
        for( int x=( (y%(2*s)) ? 0 : s ); x<w; x+=(2*s) )
            aggCanvas.drawFilledRectangle( x,y,s,s,1.0 );*/

    wxAgg2 canvas(this);
    wxAggBrush brush;
    brush.SetStyle( wxAGGBRUSH_SOLID );
    //brush.SetColor( wxColor( 0,0,0,255));
    //brush.SetEndColor( wxColor( 255,255,255,255));
    //canvas.setBrush( brush );
    //canvas.drawRectangle( 10, 10, 120, 220);

    brush.SetColor(wxColor(128,0,0,255)); brush.SetEndColor(wxColor(0,128,128,255));
    brush.SetGradientOrientation( 45 );

    canvas.setBlur( wxAggBlur(10,10,wxColor(20,20,20,255)));
    canvas.setBrush( brush );
    canvas.drawRectangle( 10, 10, 120, 220);

    canvas.drawCircle( 180, 120, 60 );
    canvas.render(dc);


    return;
/*    aggCanvas.setPen(m_color);
    aggCanvas.drawLine(0,0,w,h,5.0);

    wxBitmap * bmp = aggCanvas.renderToBitmap();

    dc.DrawBitmap(*bmp, 0, 0, 1);

    delete bmp;

    wxString str = wxString::Format(_T("Red: %i\nGreen: %i\nBlue: %i\nAlpha: %i"),m_color.Red(), m_color.Green(), m_color.Blue(), m_color.Alpha() );

    wxSize size = dc.GetMultiLineTextExtent(str);

    dc.DrawRectangle(4, 4, size.x+2, size.y+2);

    DrawHelper::DrawMultiLineText(&dc, str, wxPoint(5,5), wxALIGN_LEFT);*/

}
