#ifndef WXAGGPEN_H
#define WXAGGPEN_H

#include <wx/wx.h>

#include "agg_math_stroke.h"
#include "agg_color_rgba.h"
#include "aggHelper.h"

enum wxAggPenType {
    wxAGGPEN_SOLID,
    wxAGGPEN_TRANSPARENT,
    wxAGGPEN_DOT,
    wxAGGPEN_LONG_DASH,
    wxAGGPEN_SHORT_DASH,
    wxAGGPEN_DOT_DASH,
    wxAGGPEN_USER_DASH
};

class wxAggPen : public wxAggObject {
    public:
        wxAggPen();
        wxAggPen( const wxColour& color, double width = 1.0, int style = wxAGGPEN_SOLID );
        wxAggPen( const wxAggPen& pen );
        wxAggPen( const wxPen& pen );

        ~wxAggPen();

        bool                IsOk() const;

        agg::line_cap_e     GetCap() const;
        wxColour            GetWxColor() const;
        agg::rgba           GetAggColor() const;
        int                 GetStyle() const;
        double              GetWidth() const;
        int                 GetDashes( double** dashes );

        void                SetCap( agg::line_cap_e cap );
        void                SetColor( const wxColour& color );
        void                SetColor( agg::rgba color );
        void                SetStyle( int style );
        void                SetWidth( double width );
        void                SetDashes( int dashcount, double* dashes );

        void                SetPen( const wxColour& color, double width = 1.0, int style = wxAGGPEN_SOLID );
        void                SetPen( const wxAggPen& pen );
        void                SetPen( const wxPen& pen );



        wxAggPen&   operator = (const wxAggPen& pen);

    private:
        bool            m_isOk;

        agg::rgba       m_color;
        agg::line_cap_e m_cap;
        int             m_linestyle;
        double          m_linewidth;

        int             m_dashcount;
        double*         m_dashes;

        void            createDashes();

        DECLARE_DYNAMIC_CLASS( wxAggPen )
};

extern const wxAggPen wxDefaultAggPen;

#endif // WXAGGPEN_H
