#include "AIMCommunicationConfig.h"

#include <wx/statline.h>
#include <wx/animate.h>

#include "AIMDetectorEvent.h"
#include "../events/GeneralEvent.h"

#include "../logger.h"
#include "../DirHelper.h"
#include "../SerialPortHelper.h"

#include "../ConfigBase.h"
#include "../configmanager/ConfigHelper2.h"
#include "../configmanager/SerialConfigMgr.h"
#include "../configmanager/VemsDeviceControllerConfig.h"

#include "../VemsTuneController.h"

#include "../vemsprotocol/AIMDeviceController.h"
#include "../vemsprotocol/SIPRPacket.h"
#include "../vemsprotocol/SIPRTypes.h"

int idAIMCfgSaveButton      = wxNewId();
int idAIMCfgTestButton      = wxNewId();
int idAIMCfgCloseButton     = wxNewId();
int idAIMCfgTimer           = wxNewId();
int idAIMRefreshPortTimer   = wxNewId();

BEGIN_EVENT_TABLE( AIMCommunicationConfig, wxFrame )
    EVT_BUTTON( idAIMCfgSaveButton, AIMCommunicationConfig::OnSave )
    EVT_BUTTON( idAIMCfgTestButton, AIMCommunicationConfig::OnTest )
    EVT_BUTTON( idAIMCfgCloseButton, AIMCommunicationConfig::OnClose )

    EVT_TIMER( idAIMCfgTimer, AIMCommunicationConfig::OnTimer )
    EVT_TIMER( idAIMRefreshPortTimer, AIMCommunicationConfig::OnPortTimer )
    EVT_COMMUNICATION( wxID_ANY, AIMCommunicationConfig::OnCommEvent )
    //EVT_AIMDETECTOR( AIMCommunicationConfig::OnAIMDetectorEvent )
END_EVENT_TABLE()


AIMCommunicationConfig::AIMCommunicationConfig( wxWindow* parent ) :
    wxFrame( parent, wxID_ANY, _T("AIM Communication settings"),wxDefaultPosition, wxSize(450, 220),
        wxDEFAULT_FRAME_STYLE  | wxFRAME_FLOAT_ON_PARENT | wxSTAY_ON_TOP & ~wxRESIZE_BORDER )
{
    if ( parent )
    {
        parent->Disable();
    }

    m_evtHandler = dynamic_cast<wxEvtHandler*>(parent);

    SetBackgroundColour( wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE ) );
    ClearBackground();

    SetIcon( wxIcon( _T("vemsLogoIcon2") ) );

    m_aimcontroller = NULL;
    m_evtConnected = false;

    ConfigBase* configBase = ConfigHelper2::getConfig();
    SerialConfigMgr cfgMgr( configBase, _T("/aimsettings") );
    cfgMgr.load( &m_serialConfig );

    // get valid baudrates & convert them to string for the combobox
    IntArray baudRates = SerialPortHelper::getGPSSerialBaudRates();

    for ( size_t i = 0; i < baudRates.GetCount(); ++i )
    {
        m_baudRateS.Add( wxString::Format(_T("%d"), baudRates[i] ) );
    }

    getAvailablePorts();

    buildGUI();
    m_timer.SetOwner( this, idAIMCfgTimer );
    m_porttimer.SetOwner( this, idAIMRefreshPortTimer );
    m_porttimer.Start( 10000, false );
}

AIMCommunicationConfig::~AIMCommunicationConfig()
{
    m_timer.Stop();
    m_porttimer.Stop();

    if ( m_evtConnected )
        Disconnect( wxEVT_INPUT, InputEventHandler( AIMCommunicationConfig::OnSIPRInput ), NULL, this );

    if ( m_aimcontroller != NULL )
    {
        delete m_aimcontroller;
        m_aimcontroller = NULL;
    }
}

void AIMCommunicationConfig::getAvailablePorts()
{
    m_availPorts.Clear();
    m_availPorts = SerialPortHelper::GetCOMPortList();
    m_availPorts.Sort( sortFunction );

    //ConfigBase* configBase = ConfigHelper2::getConfig();
    //if ( m_vtc && m_vtc->isECUOnline() )
    /*{
        SerialConfigMgr cfgMgr( configBase, _T("/") );
        SerialConfig scfg;
        cfgMgr.load( &scfg );

        wxString pname = scfg.getPortName();

        int i = m_availPorts.Index( pname );

        if ( i != wxNOT_FOUND )
            m_availPorts.RemoveAt( i );
    }

    //if ( m_vtc && m_vtc->isGPSOnline() )
    {
        SerialConfigMgr cfgMgr( configBase, _T("/gps") );
        SerialConfig scfg;
        cfgMgr.load( &scfg );

        wxString pname = scfg.getPortName();

        int i = m_availPorts.Index( pname );

        if ( i != wxNOT_FOUND )
            m_availPorts.RemoveAt( i );
    }*/
}

void AIMCommunicationConfig::buildGUI()
{
    wxStaticBoxSizer* sizer = new wxStaticBoxSizer( wxVERTICAL, this, _T("AIM Communication Settings") );

    wxFlexGridSizer* fgs = new wxFlexGridSizer( 2 );

    wxStaticText* dis = new wxStaticText( this, -1, _T("AIM communication enabled:") );
    wxArrayString enArr; enArr.Add( _T("no") ); enArr.Add( _T("yes") );

    wxString msg;
    if ( m_serialConfig.isEnabled() == false )
    {
        msg = _T("no");
    }
    else
    {
        msg = _T("yes");
    }

    m_enabled = new wxComboBox( this, -1, msg, wxDefaultPosition, wxDefaultSize, enArr, wxCB_DROPDOWN | wxCB_READONLY );

    Connect( m_enabled->GetId(), wxEVT_COMMAND_COMBOBOX_SELECTED, wxCommandEventHandler( AIMCommunicationConfig::OnAIMEnabledCombo ) );

    fgs->Add( dis, 0, wxALL, 3 ); fgs->Add( m_enabled, 1, wxEXPAND | wxALL, 3 );

    wxStaticText* txt = new wxStaticText( this, -1, _T("COM port:") );

    wxString selected = m_serialConfig.getPortName();

    if ( m_availPorts.Index( selected ) == wxNOT_FOUND )
        selected = wxEmptyString;

    m_comports = new wxComboBox( this, wxID_ANY, selected, wxDefaultPosition, wxDefaultSize, m_availPorts, wxCB_DROPDOWN );
    Connect( m_comports->GetId(), wxEVT_COMMAND_COMBOBOX_SELECTED, wxCommandEventHandler( AIMCommunicationConfig::OnComboBox ) );

    fgs->Add( txt, 0, wxALL, 3 ); fgs->Add( m_comports, 1, wxEXPAND | wxALL, 3 );

    txt = new wxStaticText( this, -1, _T("Baud rate:") );
    m_baudRate = new wxComboBox( this, wxID_ANY, m_serialConfig.getBaudRateStr(), wxDefaultPosition, wxDefaultSize, m_baudRateS,
     wxCB_DROPDOWN | wxCB_READONLY );
    Connect( m_baudRate->GetId(), wxEVT_COMMAND_COMBOBOX_SELECTED, wxCommandEventHandler( AIMCommunicationConfig::OnComboBox ) );

    fgs->Add( txt, 0, wxALL, 3 ); fgs->Add( m_baudRate, 1, wxEXPAND | wxALL, 3 );
    fgs->Add( 10, 10 );

    m_testButton = new wxButton( this, idAIMCfgTestButton, _T("Test") );

    // m_animFName = DirHelper::GetPath( dGRAPHICS ) + _T("incomingdata.gif");
    // m_anim.LoadFile( m_animFName, wxANIMATION_TYPE_GIF );

    m_noaimbmp.LoadFile( DirHelper::GetPath( dGRAPHICS ) + _T("noaim.png"), wxBITMAP_TYPE_PNG );
    m_aimdatabmp.LoadFile( DirHelper::GetPath( dGRAPHICS ) + _T("aimdata.png"), wxBITMAP_TYPE_PNG );
    m_aimunknown.LoadFile( DirHelper::GetPath( dGRAPHICS ) + _T("aimunknown.png"), wxBITMAP_TYPE_PNG );
    m_aimtesting.LoadFile( DirHelper::GetPath( dGRAPHICS ) + _T("aimtesting.png"), wxBITMAP_TYPE_PNG );

    m_is_aim_bmp = new wxStaticBitmap( this, -1, m_aimunknown );

    // m_animctrl = new wxAnimationCtrl( this, wxID_ANY, m_anim, wxDefaultPosition, wxSize( 400, 20 ) );

    sizer->Add( fgs, 0, wxEXPAND | wxALL, 5 );
    sizer->Add( m_testButton, 0, wxBOTTOM | wxLEFT | wxRIGHT | wxALIGN_CENTER, 3 );

    sizer->Add( m_is_aim_bmp, 0, wxALL | wxALIGN_CENTER, 3 );

    //sizer->Add( new wxStaticText( this, -1, _T("Animation means test was successful") ), 0, wxALL, 3 );

    sizer->Add( new wxStaticLine( this, -1 ), 0, wxEXPAND | wxALL, 3 );

    wxBoxSizer* bs2 = new wxBoxSizer( wxHORIZONTAL );
    m_saveButton = new wxButton( this, idAIMCfgSaveButton, _T("Save && Close") );
    m_closeButton = new wxButton( this, idAIMCfgCloseButton, _T("Close w/o save") );

    bs2->Add( m_saveButton, 0, wxALL | wxALIGN_CENTER, 3 );
    bs2->Add( m_closeButton, 0, wxALL | wxALIGN_CENTER, 3 );

    sizer->Add( bs2, 0, wxEXPAND | wxALL | wxALIGN_CENTER, 5 );

    SetSizer( sizer );

    sizer->Fit( this );
    sizer->SetSizeHints( this );

    if ( !m_serialConfig.isEnabled() )
    {
        m_testButton->Enable( false );
        m_baudRate->Enable( false );
        m_comports->Enable( false );
    }
}

int AIMCommunicationConfig::sortFunction( const wxString& first, const wxString& second )
{
    if ( first.length() != second.length() )
        return first.length() - second.length();
    else return first.CmpNoCase( second );
}

void AIMCommunicationConfig::OnSave( wxCommandEvent& event )
{
    bool isEnabled = ( m_enabled->GetValue() == _T("yes") ) ? true : false;

    wxString baudrateStr = m_baudRate->GetValue();

    long baudRate;
    baudrateStr.ToLong( &baudRate );

    m_serialConfig.setEnabled( isEnabled );
    m_serialConfig.setPortName( m_comports->GetValue() );
    m_serialConfig.setBaudRate( baudRate );

    // save values to config file
    ConfigBase* configBase = ConfigHelper2::getConfig();

    SerialConfigMgr cfgMgr( configBase, _T("/aimsettings") );
    cfgMgr.save( &m_serialConfig );

    m_saved = true;

    int ret = ( isEnabled ) ? 1 : 2;

    if ( m_evtHandler )
    {
        GeneralEvent event( 512 );
        event.SetLong( ret );

        m_evtHandler->AddPendingEvent( event );
        Destroy();
    }

//    EndModal( ret );
}

void AIMCommunicationConfig::OnTest( wxCommandEvent& event )
{
    /*if ( m_evtConnected )
    {
        Disconnect( wxEVT_INPUT, InputEventHandler( AIMCommunicationConfig::OnSIPRInput ), NULL, this );
        m_evtConnected = false;
    }*/

    wxString selport = m_comports->GetValue();

    if ( selport == _T("") || ( m_availPorts.Index( selport ) == wxNOT_FOUND ) )
    {
        wxMessageBox(_T("Empty or non-existing port name"), _T("Error"), wxCANCEL | wxICON_ERROR );
        event.Skip();
        return;
    }

    stopController();

    m_is_aim_bmp->SetBitmap( m_aimtesting );

    m_timer.Start( 5000, true );

    m_aimreceived = false;

    setTestButton( false );
    //setAimBmp( false );

    VemsDeviceControllerConfig deviceControllerConfig;

    wxString br = m_baudRate->GetValue();

    deviceControllerConfig.port = m_comports->GetValue();
    br.ToLong( &deviceControllerConfig.baudrate );
    deviceControllerConfig.timeout = 40;
    deviceControllerConfig.interWriteDelay = 20;

    if ( m_evtConnected == false )
    {
        Connect( wxEVT_INPUT, InputEventHandler( AIMCommunicationConfig::OnSIPRInput ), NULL, this );
        m_evtConnected = true;
    }

    m_aimcontroller = new AIMDeviceController( deviceControllerConfig, this );

    m_aimcontroller->start();

    event.Skip();
}


void AIMCommunicationConfig::OnClose( wxCommandEvent& event )
{
//    EndModal( 0 );
    if ( m_evtHandler )
    {
        GeneralEvent event( 512 );
        event.SetLong( 0 );

        m_evtHandler->AddPendingEvent( event );
        Destroy();
    }
}

void AIMCommunicationConfig::OnAIMEnabledCombo( wxCommandEvent& event )
{
    if ( m_enabled->GetValue() == _T("yes") )
    {
        m_testButton->Enable( true );
        m_baudRate->Enable( true );
        m_comports->Enable( true );
        m_is_aim_bmp->SetBitmap( m_aimunknown );
    }
    else
    {
        if ( m_evtConnected )
        {
            Disconnect( wxEVT_INPUT, InputEventHandler( AIMCommunicationConfig::OnSIPRInput ), NULL, this );
            m_evtConnected = false;

            stopController();
        }

        m_testButton->Enable( false );
        m_baudRate->Enable( false );
        m_comports->Enable( false );
        m_is_aim_bmp->SetBitmap( m_aimunknown );
    }
}

void AIMCommunicationConfig::OnSIPRInput( InputEvent& event )
{
    SIPRPacket p = event.getPacket();
    if ( p.type() == SiprSensorType::SIPR_AIM )
    {
        if ( m_aimreceived == false )
        {
            setAimBmp( true );
            m_aimreceived = true;
            m_timer.Stop();
            setTestButton( true );
            //stopController();
        }
    }
}

void AIMCommunicationConfig::OnTimer( wxTimerEvent& event )
{
    if ( event.GetId() == idAIMCfgTimer )
    {
        setTestButton( true );
        m_is_aim_bmp->SetBitmap( m_noaimbmp );
    }
    //event.Skip();
}

void AIMCommunicationConfig::OnPortTimer( wxTimerEvent& event )
{
    if ( event.GetId() == idAIMRefreshPortTimer )
    {
        getAvailablePorts();
        wxString selected = m_comports->GetValue();

        Freeze();
        m_comports->Clear();
        m_comports->Append( m_availPorts );
        m_comports->SetValue( selected );
        Thaw();
    }
}

void AIMCommunicationConfig::stopController()
{
    if ( m_evtConnected )
    {
        Disconnect( wxEVT_INPUT, InputEventHandler( AIMCommunicationConfig::OnSIPRInput ), NULL, this );
        m_evtConnected = false;
    }

    if ( m_aimcontroller != NULL )
    {
        m_aimcontroller->stop();
        delete m_aimcontroller;
        m_aimcontroller = NULL;
    }
}

void AIMCommunicationConfig::OnComboBox( wxCommandEvent& event )
{
    Disconnect( wxEVT_INPUT, InputEventHandler( AIMCommunicationConfig::OnSIPRInput ), NULL, this );
    m_evtConnected = false;

    stopController();

    m_is_aim_bmp->SetBitmap( m_aimunknown );
    m_testButton->Enable();
    event.Skip();
}

void AIMCommunicationConfig::OnAIMDetectorEvent( AIMDetectorEvent& event )
{
    const wxString& s = event.GetString();

    if ( s == _T("TIMEOUT") )
    {
        m_timer.Stop();
        setTestButton( true );

        setAimBmp( false );

        wxMessageBox( _T("Can not open port / timeout"), _T("Error"), wxOK | wxICON_ERROR );
    }
}

void AIMCommunicationConfig::setTestButton( bool enable )
{
    if ( enable )
    {
        m_testButton->Enable(); m_testButton->SetLabel(_T("Test"));
    }
    else
    {
        m_testButton->Disable(); m_testButton->SetLabel(_T("Please wait"));
    }
}

void AIMCommunicationConfig::setAimBmp( bool incomingdata )
{
    if ( incomingdata )
    {
        m_is_aim_bmp->SetBitmap( m_aimdatabmp );
    }
    else
    {
        m_is_aim_bmp->SetBitmap( m_noaimbmp );
    }
}

void AIMCommunicationConfig::OnCommEvent( CommunicationEvent& event )
{
    if ( event.GetId() == CommunicationEvent::CANNOT_OPEN_PORT )
    {
        m_timer.Stop();
        m_is_aim_bmp->SetBitmap( m_aimunknown );
        setTestButton( true );
        wxMessageBox( _T("Cannot open port"), _T("Error"), wxCANCEL | wxICON_ERROR );
    }
}
