#include "AIMCommunicationThread.h"

#include "AIMDetectorEvent.h"
#include "../events/GeneralEvent.h"
#include "../vemsprotocol/AIMDeviceController.h"

#include "../logger.h"
#include "../common/types/IntArray.h"


AIMCommunicationThread::AIMCommunicationThread( SerialConfig& scfg, wxEvtHandler* evtHandler )
{
    m_evtHandler = evtHandler;

    m_cfg.port = scfg.getPortName();
    m_cfg.baudrate = scfg.getBaudRate();
    m_cfg.timeout = 40;
    m_cfg.interWriteDelay = 20;

    m_aimcontroller = NULL;
    m_aimcontroller = new AIMDeviceController( m_cfg, m_evtHandler );

    Connect( wxEVT_AIMDETECTOR, AIMDetectorEventHandler( AIMCommunicationThread::OnAimDetectEvent ) );

    m_isRunning = false;
}

AIMCommunicationThread::~AIMCommunicationThread()
{
    Disconnect( wxEVT_AIMDETECTOR, AIMDetectorEventHandler( AIMCommunicationThread::OnAimDetectEvent ) );
    if ( m_aimcontroller )
    {
        if ( m_isRunning )
        {
            m_aimcontroller->stop();
        }
        delete m_aimcontroller;
        m_aimcontroller = NULL;
    }
}

void AIMCommunicationThread::setSerialConfig( SerialConfig& scfg )
{
    m_cfg.port = scfg.getPortName();
    m_cfg.baudrate = scfg.getBaudRate();
    m_cfg.timeout = 40;
    m_cfg.interWriteDelay = 20;
}

void AIMCommunicationThread::start()
{
    if ( m_aimcontroller && m_isRunning == false )
    {
        m_aimcontroller->start();
        m_isRunning = true;
    }
    else
        restart();
}

void AIMCommunicationThread::restart()
{
    if ( m_aimcontroller )
        return;

    m_aimcontroller = new AIMDeviceController( m_cfg, m_evtHandler );
    m_aimcontroller->start();
    m_isRunning = true;
}

void AIMCommunicationThread::stop()
{
    if ( m_aimcontroller )
    {
        m_isRunning = false;
        m_aimcontroller->stop();
        delete m_aimcontroller;
        m_aimcontroller = NULL;
    }
}

void AIMCommunicationThread::OnAimDetectEvent( AIMDetectorEvent& event )
{
    if ( event.GetString() == _T("TIMEOUT") )
    {
        stop();
        GeneralEvent event(520);
        event.SetString(_T("timeout"));
        m_evtHandler->AddPendingEvent( event );
        m_isRunning = false;
    }
}

void AIMCommunicationThread::sendSiprAIM( SIPRPacket& packet )
{
    if ( m_aimcontroller )
        m_aimcontroller->SendSIPRPacket( packet );
}
