#include "AIMDisplayFrame.h"

#include <wx/statline.h>
#include <wx/dcbuffer.h>

#include "AimValuesPanel.h"
#include "AIMDisplayHelper.h"
#include "AIMFlagsPanel.h"
#include "AIMCommunicationThread.h"
#include "AIMCommunicationConfig.h"

#include "../DirHelper.h"
#include "../FirmwareVersion.h"
#include "../logger.h"
#include "../ConfigBase.h"
#include "../SymbolMgr.h"
#include "../SerialPortHelper.h"

#include "../common/SIPR2AIMConverter.h"
#include "../common/TxData.h"

#include "../configmanager/ConfigHelper2.h"
#include "../configmanager/SerialConfigMgr.h"

#include "../events/InputEvent.h"

#include "../rtdatamanager/RTDataMgrFacade.h"

#include "../vemsprotocol/SIPRPacket.h"
#include "../vemsprotocol/SIPRTypes.h"
#include "../vemsprotocol/CommandRequestBase.h"
#include "../vemsprotocol/VemsCommand.h"

// events
int idtopSelId      = wxNewId();
int idbottomSelId   = wxNewId();
int idAIMTestStop   = wxNewId();
int idAIMTestStart  = wxNewId();
int idAIMTestComm   = wxNewId();

AIMTestButtonPanel::AIMTestButtonPanel( wxWindow* parent, const wxSize& size, bool isTestEnabled )
{
    Create( parent, -1, wxDefaultPosition, size );
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
    ClearBackground();

    m_parent = parent;

    m_istestEnabled = isTestEnabled;

    int ypos = 90;

    Connect( idAIMTestComm, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( AIMDisplayPanel::onTestButtons ), 0, parent->GetParent() );

    m_eventConnected = false;

    m_startButton = new bcButton( this, idAIMTestStart, _T("Start AIM"), wxPoint( 0, ypos ), wxSize( 80, 25 ) );
    m_stopButton = new bcButton( this, idAIMTestStop, _T("Stop AIM"), wxPoint( 0, ypos + 30 ), wxSize( 80, 25 ) );
    m_aimcommButton = new bcButton( this, idAIMTestComm, _T("AIM Port"), wxPoint( 0, ypos + 60 ), wxSize( 80, 25 ) );

    m_txt = new bcLabel( this, -1, _T(""), wxPoint(0,0), wxSize(540, 80 ) );


    m_txt->Refresh();

    setTestEnabled( isTestEnabled );

    Connect( wxEVT_PAINT, wxPaintEventHandler( AIMTestButtonPanel::OnPaint ) );
    CreateBcParent( this );
}

void AIMTestButtonPanel::OnPaint( wxPaintEvent& event )
{
    wxBufferedPaintDC dc( this );
    dc.SetBackground( GetBackgroundColour() );
    dc.Clear();

    PaintWidgets(dc);
}

void AIMTestButtonPanel::setTestEnabled( bool isTestEnabled )
{
    LOG
    if ( isTestEnabled )
    {
        LOG
        if ( m_eventConnected == false )
        {
            Connect( idAIMTestStart, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( AIMDisplayPanel::onTestButtons ), 0, m_parent->GetParent() );
            Connect( idAIMTestStop, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( AIMDisplayPanel::onTestButtons ), 0, m_parent->GetParent() );
        }

        m_txt->SetLabel(_T(""));
        m_txt->Refresh();
        m_eventConnected = true;
    }
    else
    {
        LOG
        if ( m_eventConnected )
        {
            Disconnect( idAIMTestStart, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( AIMDisplayPanel::onTestButtons ), 0, m_parent->GetParent() );
            Disconnect( idAIMTestStop, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( AIMDisplayPanel::onTestButtons ), 0, m_parent->GetParent() );
        }

        m_txt->SetLabel ( _T("AIM Test is not enabled\nClick on the AIM Port button\nto enable it"));
        m_txt->Refresh();
    }
    m_istestEnabled = isTestEnabled;

    enableButtons( m_istestEnabled );
    Refresh( false );
}

void AIMTestButtonPanel::enableButtons( int b_enable )
{
    if ( b_enable & 1 ) {
        m_startButton->Enable();
        m_aimcommButton->Enable();
    }
    else {
        m_aimcommButton->Disable();
        m_startButton->Disable();
    }

    if ( b_enable & 2 )
    {
         m_stopButton->Enable();
    }
    else m_stopButton->Disable();

    if ( m_istestEnabled == false )
    {
        m_aimcommButton->Enable();
    }
}

BEGIN_EVENT_TABLE( AIMDisplayPanel, wxPanel )
    EVT_COMBOBOX( idtopSelId, AIMDisplayPanel::onTopSelected )
    EVT_COMBOBOX( idbottomSelId, AIMDisplayPanel::onBottomSelected )
    EVT_GENERAL( wxID_ANY, AIMDisplayPanel::onEvent )
    EVT_INPUT( wxID_ANY, AIMDisplayPanel::onInputEvent )
    EVT_CLOSE( AIMDisplayPanel::onClose )
    EVT_COMMUNICATION( wxID_ANY, AIMDisplayPanel::onCommunicationEvent )
END_EVENT_TABLE()

AIMDisplayPanel::AIMDisplayPanel( wxWindow* parent, STDataMgrBase* stdata_mgr, RTDataMgrBase* rtdatamgr, SymbolTable* symbolTable, SymbolMgr* symMgr, FirmwareVersion& fwversion, CommandRequestBase* requestMgr )
    : wxPanel( parent, wxID_ANY, wxDefaultPosition, wxDefaultSize )
{
    SetBackgroundColour( wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE ) );
    ClearBackground();

    getAimTestSettings();

    m_isAimTesting = false;
    m_aimthread = NULL;

    m_activeSlot = -1;

    m_demomodeVal = -1;

    /// INITIALIZATION
    m_customRTsRegistered = false;
    m_stDataMgr = stdata_mgr;
    m_symbols = symbolTable;
    m_rtDataMgr = rtdatamgr;
    m_symbolMgr = symMgr;
    m_reqMgr = requestMgr;

    m_isOnline = m_stDataMgr->isOnline();
    m_firstUpdate = true;


    if ( m_rtDataMgr )
        m_rtDataMgr->setMinRefreshInterval( 20 );

    // is it AIM version ?
    if ( m_stDataMgr && m_symbols )
    {
        SymbolTable::iterator it = m_symbols->find( _T("isAimType") );

        int aimv = 0;

        if ( it != m_symbols->end() )
            aimv = (int)(m_stDataMgr->getDataFromViewerModel(*(*m_symbols)[_T("isAimType")]));
        else {
            if ( fwversion.major == 0 ) aimv = 1;
            else aimv = 0;
        }

        m_isAIM = ( aimv == 1 );
    }

    // symbol names to register
    initRTs();

    m_statvcontroller = new StaticViewersController( parent, _T("Main Display Settings") );
    m_statvcontroller->setController( m_stDataMgr );
    m_statvcontroller->addViewer( this );

    /// OPEN THE DATABASE AND GET THE VAR.NAMES OF THE COMBOBOXES
    m_db.Open( DirHelper::GetPath( dCONFIG ) + _T("aimdsettings.db") );
    getDisplayValues();

    //
    wxSizer* mainsizer = new wxBoxSizer( wxVERTICAL );

    /// FLAG PANEL
    m_flagpanel = new AIMFlagsPanel( this );
    m_flagpanel->setFlags( AF_AIM_MODE, m_isAIM );

    mainsizer->Add( m_flagpanel, 0, wxEXPAND | wxALL, 3 );

    /// SLOT SELECTOR
    /// AND TOP/BOTTOM SELECTOR FOR EACH SLOT
    addSlotSelector( mainsizer );

    m_selectedSlot = 0;


    /// AIM VALUES PANEL (AGG DRAW)

    wxScrolledWindow* swin = new wxScrolledWindow( this, wxID_ANY, wxDefaultPosition, wxSize(780,380), wxScrolledWindowStyle );

    wxSizer* middlesizer = new wxBoxSizer( wxHORIZONTAL );

    m_aimvalues = new AIMValuesPanel( swin );

    m_aimvalues->registerObserver( this );

    m_aimvalues->setVarNames( m_topvaluesString, m_bottomvaluesString );

    middlesizer->Add( m_aimvalues, 0, wxEXPAND | wxALL, 3  );

    //
    middlesizer->Add( new wxStaticLine( swin, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxLI_VERTICAL ), 0, wxEXPAND | wxALL, 3 );


    /// ADD THE CUSTOM EDITOR PANELS
    addTopBottomSelector( swin, middlesizer );

    /// ADD THE TEST BUTTON - ONLY IN CASE OF AIM MODE
    if ( m_isAIM )
    {
        m_testPanel = new AIMTestButtonPanel( swin, wxSize(200,-1), m_serialConfig.isEnabled() );

        middlesizer->Add( m_testPanel, 0, wxEXPAND | wxALL, 3 );

        m_aimthread = new AIMCommunicationThread( m_serialConfig, this );

    }
    else
        m_aimthread = NULL;

    swin->SetSizer( middlesizer );

    swin->SetScrollRate( 10, 10 );


    middlesizer->Fit( swin );
    middlesizer->SetSizeHints( swin );

    //mainsizer->Add( middlesizer );
    mainsizer->Add( swin, 1, wxALL, 10 );

    //
    mainsizer->Add( new wxStaticLine( this, -1 ), 0, wxALL | wxEXPAND, 3 );

    /// REFRESH, SEND, BURN BUTTONS
    m_statcpanel = new StaticControlPanel( this, m_statvcontroller );

    mainsizer->Add( m_statcpanel, 0, wxEXPAND | wxALL, 10 );

    SetSizer( mainsizer );
    mainsizer->SetSizeHints( this );
    mainsizer->Fit( this );

    /// REGISTERING CONTROLLERS
    ((STDataViewerBase*)this)->SetDataMgr( m_stDataMgr );
    m_stDataMgr->RegisterViewer( this );

    ((RTDataViewerBase*)this)->SetDataMgr( m_rtDataMgr );
    m_rtDataMgr->RegisterViewer( this );

    require();
    update();

    m_initialized = false;
}

AIMDisplayPanel::~AIMDisplayPanel()
{
    unregisterViewer();
    unregisterSymbols();

    if ( m_customRTsRegistered )
        unregisterCustomsRTs();

    if ( m_statvcontroller )
    {
        m_statvcontroller->removeViewer(this);
        delete m_statvcontroller;
    }

    if ( m_rtDataMgr )
        m_rtDataMgr->UnRegisterViewer( this );

    if ( m_stDataMgr )
        m_stDataMgr->UnRegisterViewer( this );

    if ( m_aimthread )
    {
        if ( m_aimthread->isRunning() )
            m_aimthread->stop();

        delete m_aimthread;
}
}

void AIMDisplayPanel::initRTs()
{
    m_rtSymbolsToRegister.Add( _T("slotNumber") );
    m_rtSymbolsToRegister.Add( _T("egtf") ); // for fahrenheit
    m_symbolName2selection.clear();

    m_rtSymbolsToRegister.Add( _T("lambda") );  m_symbolName2selection[_T("lambda")] = _T("Lambda");
    m_selection2symbolName[_T("Lambda")] = _T("lambda"); m_symbolName2dotpos[_T("lambda")] = _T("X.XX");

    m_rtSymbolsToRegister.Add( _T("egt") ); m_symbolName2selection[_T("egt")] = _T("EGT");
    m_selection2symbolName[_T("EGT")] = _T("egt"); m_symbolName2dotpos[_T("egt")] = _T("No Dot");

    m_rtSymbolsToRegister.Add( _T("analin") ); m_symbolName2selection[_T("analin")] = _T("ANALOG");
    m_selection2symbolName[_T("ANALOG")] = _T("analin"); m_symbolName2dotpos[_T("analin")] = _T("No Dot");

    m_rtSymbolsToRegister.Add( _T("rpm") ); m_symbolName2selection[_T("rpm")] = _T("RPM");
    m_selection2symbolName[_T("RPM")] = _T("rpm"); m_symbolName2dotpos[_T("rpm")] = _T("No Dot");

    m_rtSymbolsToRegister.Add( _T("afr") ); m_symbolName2selection[_T("afr")] = _T("Afr");
    m_selection2symbolName[_T("Afr")] = _T("afr"); m_symbolName2dotpos[_T("afr")] = _T("XX.X");
}

///----------------------------------------------------
/// CREATE THE SLOT SELECTOR
/// AND THE TOP/BOTTOM VALUE SELECTOR FOR EACH SLOT
///----------------------------------------------------
void AIMDisplayPanel::addSlotSelector( wxSizer* sizer )
{
    // slot selector
    sizer->AddSpacer( 5 );
    wxSizer* slotselects = new wxFlexGridSizer( 8 );

    wxPanel* slotspanel = new wxPanel( this );

    for ( size_t i = 0; i < 8; ++i )
    {
        int id = wxNewId();
        wxString label = wxString::Format( _T("Slot%d"), i );
        m_slotSelectors[i] = new wxToggleButton( slotspanel, id, label, wxDefaultPosition, wxSize( 60, -1 ) );
        m_slotSelectorsIds[i] = id;
        Connect( id, wxEVT_COMMAND_TOGGLEBUTTON_CLICKED, wxCommandEventHandler( AIMDisplayPanel::onSlotSelect ) );
        slotselects->Add( m_slotSelectors[i], 0, wxLEFT | wxRIGHT | wxTOP, 2 );
    }

    // topval select
    for ( size_t i = 0; i < 8; ++i )
    {
        int id = wxNewId();
        m_modeSelectors[i] = new wxComboBox( slotspanel, id, _T(""), wxDefaultPosition, wxDefaultSize, m_topvaluesString, wxCB_DROPDOWN | wxCB_READONLY );
        slotselects->Add( m_modeSelectors[i], 0,  wxLEFT | wxRIGHT | wxTOP, 2 );
        m_modeSelectorsIds[i] = id;
        Connect( id, wxEVT_COMMAND_COMBOBOX_SELECTED, wxCommandEventHandler( AIMDisplayPanel::onModeSelect ) );
    }

    // bottomval select
    for ( size_t i = 0; i < 8; ++i )
    {
        int id = wxNewId();
        m_modeSelectors[8+i] = new wxComboBox( slotspanel, id, _T(""), wxDefaultPosition, wxDefaultSize, m_bottomvaluesString, wxCB_DROPDOWN | wxCB_READONLY );
        slotselects->Add( m_modeSelectors[8+i], 0,  wxLEFT | wxRIGHT | wxTOP, 2 );
        m_modeSelectorsIds[8+i] = id;
        Connect( id, wxEVT_COMMAND_COMBOBOX_SELECTED, wxCommandEventHandler( AIMDisplayPanel::onModeSelect ) );
    }

    slotspanel->SetSizer( slotselects );
    slotselects->Fit( slotspanel );
    slotselects->SetSizeHints( slotspanel );

    sizer->Add( slotspanel, 0, wxALL | wxEXPAND, 3 );
    sizer->Add( new wxStaticLine( this, -1 ), 0, wxALL | wxEXPAND, 5 );

}

///-----------------------------------------------------------
/// CREATE THE TOP/BOTTOM SELECTOR FOR THE SELECTED SLOT
/// AND THE CUSTOM VALUES EDITOR PANELS
///-----------------------------------------------------------

void AIMDisplayPanel::addTopBottomSelector( wxWindow* parent, wxSizer * sizer )
{
    // top display
    sizer->Add( new wxStaticLine( parent, -1 ), 0, wxALL | wxEXPAND, 3 );

    wxSizer* ssizer = new wxBoxSizer( wxVERTICAL );

    ///
    wxSizer* s1 = new wxBoxSizer( wxHORIZONTAL );
    wxStaticText* txt = new wxStaticText( parent, wxID_ANY, _T("Top display\n(affects only the gauge on the TOP):") );
    wxFont fnt = txt->GetFont(); fnt.SetWeight( wxFONTWEIGHT_BOLD  ); txt->SetFont( fnt );
    m_topSelector = new wxComboBox( parent, idtopSelId, m_topvaluesString[0], wxDefaultPosition, wxDefaultSize, m_topvaluesString,
        wxCB_DROPDOWN | wxCB_READONLY );

    s1->Add( txt, 0 , wxEXPAND | wxALL, 5 );
    s1->Add( m_topSelector, 0 , wxEXPAND | wxALL, 5 );

    m_customTopValuesPanel = new AIMCustomValuesEditorPanel( parent, 0, &m_db );
    m_customTopValuesPanel->registerObserver( this );

    ssizer->Add( s1, 0, wxEXPAND | wxALL, 3 );
    ssizer->Add( m_customTopValuesPanel, 0, wxEXPAND | wxALL, 3 );

    ssizer->Add( new wxStaticLine( parent, -1, wxDefaultPosition, wxDefaultSize, wxLI_HORIZONTAL ), 0, wxEXPAND | wxALL, 3 );

    // bottom display
    s1 = new wxBoxSizer( wxHORIZONTAL );
    txt = new wxStaticText( parent, wxID_ANY, _T("Bottom display\n(affects only the gauge on the TOP):") );
    fnt = txt->GetFont(); fnt.SetWeight( wxFONTWEIGHT_BOLD  ); txt->SetFont( fnt );
    m_bottomSelector = new wxComboBox( parent, idbottomSelId, m_bottomvaluesString[0], wxDefaultPosition, wxDefaultSize, m_bottomvaluesString,
        wxCB_DROPDOWN | wxCB_READONLY );

    s1->Add( txt, 0 , wxEXPAND | wxALL, 5 );
    s1->Add( m_bottomSelector, 0 , wxEXPAND | wxALL, 3 );

    m_customBottomValuesPanel = new AIMCustomValuesEditorPanel( parent, 1, &m_db );
    m_customBottomValuesPanel->registerObserver( this );

    ssizer->Add( s1, 0, wxEXPAND | wxALL, 3 );
    ssizer->Add( m_customBottomValuesPanel, 0, wxEXPAND | wxALL, 5 );

    sizer->Add( ssizer, 0, wxEXPAND | wxALL, 10 );
}

///------------------------------------------------------------------------------
/// OBSERVABLE:
///  SOMETHING CHANGED ON THE CUSTOM PANELS <-- AIMCustormValuesEditorPanel
///  OR THE BUTTON WAS PRESSED ( CHANGE ACTIVE SLOT) <-- AimValuesPanel
///------------------------------------------------------------------------------
void AIMDisplayPanel::notify( const wxString& s )
{
    if ( s == _T("top") )
    {
        m_writeCustomSettings = true;
        requestWrite();
    }
    if ( s == _T("bottom") )
    {
        m_writeCustomSettings = true;
        requestWrite();
    }

    if ( s == _T("button") )
    {
        m_activeSlot = ( m_activeSlot == 7 ) ? 0 : m_activeSlot + 1;

        wxString symName = _T("disp_state");
        Symbol* sym = NULL;
        SymbolTable::iterator it = m_symbols->find( symName );

        if ( it != m_symbols->end() )
        {
            sym = it->second;
        }
        if ( sym )
        {
            m_stDataMgr->setModelData( *sym, 0, 0, m_activeSlot );

            m_stDataMgr->requestWrite( *sym );

            m_stDataMgr->updateRegisteredViewers();
        }
    }
}

// notified when a text control is modified on the customaimvaluespanel
void AIMDisplayPanel::notify( int panel, void* win )
{
    if ( m_symbols == NULL || m_stDataMgr == NULL )
        return;

    wxTextCtrl* ctrl = (wxTextCtrl*) win;

    double dv = 0.0;
    wxString val = ctrl->GetValue();
    val.ToDouble(&dv);

    bool isTop = ( panel == 0 );
    AIMCustomValuesEditorPanel* p = ( panel == 0 ) ? m_customTopValuesPanel : m_customBottomValuesPanel;
    wxString selection = ( isTop ) ? m_topSelectedValue : m_bottomSelectedValue;

    wxString modeS = selection.Mid( 10, 1 );
    long mode; modeS.ToLong( &mode );

    Symbol* sym = NULL;

    if ( ctrl == p->m_channelId )
    {
        wxString symName = wxString::Format( _T("aim_channel%d"), mode );
        SymbolTable::iterator it = m_symbols->find( symName );
        if ( it != m_symbols->end() )
        {
            sym = it->second;
        }
    }
    if ( ctrl == p->m_multiplicator )
    {
        wxString symName = wxString::Format( _T("aim_a%d"), mode );
        SymbolTable::iterator it = m_symbols->find( symName );
        if ( it != m_symbols->end() )
        {
            sym = it->second;
        }
    }
    if ( ctrl == p->m_translation )
    {
        wxString symName = wxString::Format( _T("aim_b%d"), mode );
        SymbolTable::iterator it = m_symbols->find( symName );
        if ( it != m_symbols->end() )
        {
            sym = it->second;
        }
    }

    if ( sym )
    {
        m_stDataMgr->setModelData( *sym, 0, 0, dv );
        m_stDataMgr->requestWrite( *sym );
        m_stDataMgr->updateRegisteredViewers();
    }
}

/// -->STDataViewerBase virtual methods

// OK
void AIMDisplayPanel::update()
{
    if ( m_stDataMgr == NULL || m_symbols == NULL )
        return;

    // check whether round ini has been parsed (to prevent freeze)
    wxString symName = _T("egt_cal");
    SymbolTable::iterator sit = m_symbols->find( symName );
    if ( sit == m_symbols->end() )
    {
        LOGP(_T("V3?"))// not found -> V3
        return;
    }

    if ( m_isOnline == false || m_activeSlot < 0 )
    {
        SymbolTable::iterator sit = m_symbols->find( _T("disp_state") );
        if ( sit != m_symbols->end() )
        {
            if ( m_activeSlot >= 0 )
            {
                for ( int i = 0; i < 8;++i)
                {
                    wxFont fnt = m_slotSelectors[i]->GetFont();
                    fnt.SetWeight( wxFONTWEIGHT_NORMAL );
                    m_slotSelectors[i]->SetFont( fnt );
                }
            }

            int slot = (int) ( m_stDataMgr->getDataFromViewerModel( *(sit->second) ) );
            m_activeSlot = slot;
            //m_selectedSlot = slot;

            m_aimvalues->setSlots( m_selectedSlot, m_activeSlot );

            if ( m_firstUpdate )
            {
                m_selectedSlot = m_activeSlot;
                wxCommandEvent evt( wxEVT_COMMAND_TOGGLEBUTTON_CLICKED, m_slotSelectorsIds[m_selectedSlot] );
                AddPendingEvent( evt );
                m_firstUpdate = false;
            }

            wxFont fnt = m_slotSelectors[m_activeSlot]->GetFont();
            fnt.SetWeight( wxFONTWEIGHT_BOLD );
            m_slotSelectors[m_activeSlot]->SetFont( fnt );
        }
        //else m_activeSlot = 0;
    }

    /*if ( m_activeSlot < 0 || m_activeSlot > 7 )
        return;*/

    symName = _T("celsius");
    sit = m_symbols->find( symName );

    if ( sit != m_symbols->end() )
    m_isCelsius = ( (int)( m_stDataMgr->getDataFromViewerModel(*(*m_symbols)[symName]) ) == 0 );

    int demomode = -1;

    sit = m_symbols->find(_T("display_mask"));

    if ( sit != m_symbols->end() )
    {
        demomode = (int)( m_stDataMgr->getDataFromViewerModel(*(*m_symbols)[_T("display_mask")]) );
    }

    if ( m_demomodeVal != demomode && demomode != -1 )
    {
        m_demomodeVal = demomode;

        m_flagpanel->setFlags( AF_AIM_DEMO_MODE, m_demomodeVal );
    }

    int toproundtop = -1, toproundbottom = -1;
    int bottomroundtop = -1, bottomroundbottom = -1;

    // most top mode selectors
    for ( size_t i = 0; i < 8; ++i )
    {
        symName = wxString::Format( _T("disp_mode_t%d"), i );

        sit = m_symbols->find( symName );

        int val = -1;

        if ( sit != m_symbols->end() )
            val = (int)( m_stDataMgr->getDataFromViewerModel(*(*m_symbols)[symName]) );

        m_modeSelectors[i]->SetSelection( val );

        if ( (int)i == m_selectedSlot )
        {
            toproundtop = val;
            m_topSelector->SetSelection( toproundtop );
        }
        if ( (int)i == m_activeSlot )
        {
            bottomroundtop = val;
        }

        m_SlotTop[i] = val;

        //-----
        symName = wxString::Format( _T("disp_mode_b%d"), i );
        sit = m_symbols->find( symName );

        val = -1;

        if ( sit != m_symbols->end() )
        val = (int)( m_stDataMgr->getDataFromViewerModel(*(*m_symbols)[symName]) );

        m_modeSelectors[8+i]->SetSelection( val );

        if ( (int)i == m_selectedSlot )
        {
            toproundbottom = val;
            m_bottomSelector->SetSelection( toproundbottom );
        }
        if ( (int)i == m_activeSlot )
        {
            bottomroundbottom = val;
        }

        m_SlotBottom[i] = val;
    }

    wxString tops = m_topSelector->GetStrings().Item( bottomroundtop );
    wxString bottoms = m_bottomSelector->GetStrings().Item( bottomroundbottom );

    StringHash::iterator it = m_symbolName2selection.begin();

    m_activeSlotTopSymbolName = wxEmptyString;
    m_activeSlotBottomSymbolName = wxEmptyString;

    for ( ; it != m_symbolName2selection.end(); ++it )
    {
        if ( it->second == tops )
        {
            m_activeSlotTopSymbolName = it->first;
        }
        if ( it->second == bottoms )
        {
            m_activeSlotBottomSymbolName = it->first;
        }
    }

    // customs (a,b,dot)
    for ( size_t i = 0; i < 5; ++i )
    {
        symName = wxString::Format( _T("aim_channel%d"), i );

        sit = m_symbols->find( symName );
        if ( sit != m_symbols->end() )
        m_aimchannel[i] = (int)m_stDataMgr->getDataFromViewerModel(*(*m_symbols)[symName]);
        //
        symName = wxString::Format( _T("aim_a%d"), i );

        sit = m_symbols->find( symName );
        if ( sit != m_symbols->end() )
        m_aima[i] = m_stDataMgr->getDataFromViewerModel(*(*m_symbols)[symName]);

        //
        symName = wxString::Format( _T("aim_b%d"), i );

        sit = m_symbols->find( symName );
        if ( sit != m_symbols->end() )
        m_aimb[i] = m_stDataMgr->getDataFromViewerModel(*(*m_symbols)[symName]);

        //
        symName = wxString::Format( _T("aim_dot%d"), i );

        sit = m_symbols->find( symName );
        if ( sit != m_symbols->end() )
        m_aimdot[i] = (int)m_stDataMgr->getDataFromViewerModel(*(*m_symbols)[symName]);
    }

    updateCustomPanels();

    checkSymbolsToRegister();

    if ( toproundtop >= 0 && toproundbottom >= 0 && bottomroundbottom >= 0 && bottomroundtop >= 0 )
        m_aimvalues->setSelectedValues( toproundtop, toproundbottom, bottomroundtop, bottomroundbottom );

    m_aimvalues->redraw();
}

void AIMDisplayPanel::requestWrite()
{
    if ( m_symbols == NULL || m_stDataMgr == NULL )
        return;

    updateModel();

    SymbolTable::iterator it;

    // upper display
    wxString symName = wxString::Format( _T("disp_mode_t%d"), m_selectedSlot );
    it = m_symbols->find( symName );
    if ( it != m_symbols->end() )
    {
        Symbol* symbol = it->second;
        m_stDataMgr->requestWrite( *symbol );
    }

    if ( m_writeCustomSettings )
    {
        // custom settings
        int modeval = -1;
        if ( m_isAIM )
        {
            if ( m_topSelectedValue.StartsWith(_T("AIM Custom") ) )
            {
                wxString modeS = m_topSelectedValue.Mid( 10, 1 );
                long mode; modeS.ToLong( &mode );
                if ( mode == 0 || mode == 1 || mode == 4 )
                    modeval = mode;
            }
        }
        else
        {
            if ( m_topSelectedValue == _T("Blank C0") ) modeval = 0;
            if ( m_topSelectedValue == _T("Blank C1") ) modeval = 1;
            if ( m_topSelectedValue == _T("Blank C4") ) modeval = 4;
        }

        if ( modeval >= 0 && modeval <= 4 )
        {
            wxArrayString symbolS; symbolS.Add(_T("aim_channel")); symbolS.Add(_T("aim_a"));symbolS.Add(_T("aim_b"));
            symbolS.Add(_T("aim_dot"));

            for ( size_t i = 0; i < symbolS.GetCount(); ++i )
            {
                symName = symbolS[i]; symName <<  modeval;

                //if ( symName == _T("aim_dot4") )
                //    symName = _T("aim_dot10");

                it = m_symbols->find( symName );

                if ( it != m_symbols->end() )
                {
                    Symbol* symbol = it->second;
                    m_stDataMgr->requestWrite( *symbol );
                }
            }
        }
    }

    /////////////////////////////////////////////////////
    // lower display
    symName = wxString::Format( _T("disp_mode_b%d"), m_selectedSlot );
    it = m_symbols->find( symName );
    if ( it != m_symbols->end() )
    {
        Symbol* symbol = it->second;
        m_stDataMgr->requestWrite( *symbol );
    }

    if ( m_writeCustomSettings )
    {
        // custom settings
        int modeval = -1;
        if ( m_isAIM )
        {
            if ( m_bottomSelectedValue.StartsWith(_T("AIM Custom") ) )
            {
                wxString modeS = m_bottomSelectedValue.Mid( 10, 1 );
                long mode; modeS.ToLong( &mode );
                if ( mode == 2 || mode == 3 )
                    modeval = mode;
            }
        }
        else
        {
            if ( m_bottomSelectedValue == _T("Blank C2") ) modeval = 2;
            if ( m_bottomSelectedValue == _T("Blank C3") ) modeval = 3;
        }

        if ( modeval >= 0 && modeval <= 4 )
        {
            wxArrayString symbolS; symbolS.Add(_T("aim_channel")); symbolS.Add(_T("aim_a"));symbolS.Add(_T("aim_b"));
            symbolS.Add(_T("aim_dot"));

            for ( size_t i = 0; i < symbolS.GetCount(); ++i )
            {
                symName = symbolS[i]; symName <<  modeval;

                it = m_symbols->find( symName );

                if ( it != m_symbols->end() )
                {
                    //LOGP( symName )

                    Symbol* symbol = it->second;
                    m_stDataMgr->requestWrite( *symbol );
                }
            }
        }
    }

    m_stDataMgr->updateRegisteredViewers();
}

void AIMDisplayPanel::updateModel()
{
    if ( m_stDataMgr == NULL || m_symbols == NULL )
        return;

    SymbolTable::iterator it;

    // upper display
    wxString symName = wxString::Format( _T("disp_mode_t%d"), m_selectedSlot );
    it = m_symbols->find( symName );
    if ( it != m_symbols->end() )
    {
        Symbol* symbol = it->second;
        int sel = m_topSelector->GetSelection();
        m_stDataMgr->setModelData( *symbol, 0, 0, (double) sel );
    }

    // if custom -->
    if ( m_writeCustomSettings )
    {
        // custom settings
        int modeval = -1;
        if ( m_isAIM )
        {
            if ( m_topSelectedValue.StartsWith(_T("AIM Custom") ) )
            {
                wxString modeS = m_topSelectedValue.Mid( 10, 1 );
                long mode; modeS.ToLong( &mode );
                if ( mode == 0 || mode == 1 || mode == 4 )
                    modeval = mode;
            }
        }
        else
        {
            if ( m_topSelectedValue == _T("Blank C0") ) modeval = 0;
            if ( m_topSelectedValue == _T("Blank C1") ) modeval = 1;
            if ( m_topSelectedValue == _T("Blank C4") ) modeval = 4;
        }

        // valid mode?
        if ( modeval >= 0 && modeval <= 4 )
        {
            CustomSettings s = m_customTopValuesPanel->getCustomSettings();


            // channel
            symName = wxString::Format( _T("aim_channel%d"), modeval );
            SymbolTable::iterator it = m_symbols->find( symName );
            if ( it != m_symbols->end() )
            {
                Symbol* symbol = (*m_symbols)[symName];
                double ch; s.aimchId.ToDouble( &ch );
                m_stDataMgr->setModelData( *symbol, 0, 0, ch );
            }

            // a - multiplicator
            symName = wxString::Format( _T("aim_a%d"), modeval );
            it = m_symbols->find( symName );
            if ( it != m_symbols->end() )
            {
                Symbol* symbol = (*m_symbols)[symName];
                double a; s.multiplicator.ToDouble( &a );
                m_stDataMgr->setModelData( *symbol, 0, 0, a );
            }

            // b - translation
            symName = wxString::Format( _T("aim_b%d"), modeval );
            it = m_symbols->find( symName );
            if ( it != m_symbols->end() )
            {
                Symbol* symbol = (*m_symbols)[symName];
                double b; s.add.ToDouble( &b );
                m_stDataMgr->setModelData( *symbol, 0, 0, b );
            }

            // dotpos
/*            if ( modeval == 4 )
                modeval = 10;*/

            symName = wxString::Format( _T("aim_dot%d"), modeval );
            it = m_symbols->find( symName );
            if ( it != m_symbols->end() )
            {
                Symbol* symbol = (*m_symbols)[symName];
                m_stDataMgr->setModelData( *symbol, 0, 0, (double)s.dotSelection );
            }
        } // end valid mode?
    } // end top custom


    ////-----------------------------------------------------
    // lower display
    symName = wxString::Format( _T("disp_mode_b%d"), m_selectedSlot );
    it = m_symbols->find( symName );
    if ( it != m_symbols->end() )
    {
        Symbol* symbol = it->second;
        int sel = m_bottomSelector->GetSelection();
        m_stDataMgr->setModelData( *symbol, 0, 0, (double) sel );
    }

    // if custom -->
    if ( m_writeCustomSettings )
    {
        // custom settings
        int modeval = -1;
        if ( m_isAIM )
        {
            if ( m_bottomSelectedValue.StartsWith(_T("AIM Custom") ) )
            {
                wxString modeS = m_bottomSelectedValue.Mid( 10, 1 );
                long mode; modeS.ToLong( &mode );
                if ( mode == 2 || mode == 3 )
                    modeval = mode;
            }
        }
        else
        {
            if ( m_bottomSelectedValue == _T("Blank C2") ) modeval = 2;
            if ( m_bottomSelectedValue == _T("Blank C3") ) modeval = 3;
        }

        // valid mode?
        if ( modeval >= 0 && modeval <= 4 )
        {
            CustomSettings s = m_customBottomValuesPanel->getCustomSettings();

            // channel
            symName = wxString::Format( _T("aim_channel%d"), modeval );
            SymbolTable::iterator it = m_symbols->find( symName );
            if ( it != m_symbols->end() )
            {
                Symbol* symbol = (*m_symbols)[symName];
                double ch; s.aimchId.ToDouble( &ch );
                m_stDataMgr->setModelData( *symbol, 0, 0, ch );
            }

            // a - multiplicator
            symName = wxString::Format( _T("aim_a%d"), modeval );
            it = m_symbols->find( symName );
            if ( it != m_symbols->end() )
            {
                Symbol* symbol = (*m_symbols)[symName];
                double a; s.multiplicator.ToDouble( &a );
                m_stDataMgr->setModelData( *symbol, 0, 0, a );
            }

            // b - translation
            symName = wxString::Format( _T("aim_b%d"), modeval );
            it = m_symbols->find( symName );
            if ( it != m_symbols->end() )
            {
                Symbol* symbol = (*m_symbols)[symName];
                double b; s.add.ToDouble( &b );
                m_stDataMgr->setModelData( *symbol, 0, 0, b );
            }

            symName = wxString::Format( _T("aim_dot%d"), modeval );
            it = m_symbols->find( symName );
            if ( it != m_symbols->end() )
            {
                //Logger::logf(LOG_MSG, symName + _T(" => s.dotselection=%d"), s.dotSelection);
                Symbol* symbol = (*m_symbols)[symName];
                double dotsel = (double)(s.dotSelection);
                m_stDataMgr->setModelData( *symbol, 0, 0, dotsel );
            }
        } // end valid mode?
    } // end bottom custom
}

/// <--STDataViewerBase virtual methods

void AIMDisplayPanel::refreshRound( DoubleHash* table, int& top, int& bottom, bool isBottom )
{
    wxString symName;

    StringHash::iterator it2;

    DoubleHash::iterator it;

    bool fixedTop = m_isFixedTop[top];
    bool fixedBottom = m_isFixedBottom[bottom];

    if ( fixedTop )
    {
        if ( isBottom )
        {
            it = table->find( m_activeSlotTopSymbolName );
            if ( it != table->end() )
            {
                m_aimvalues->setTopDisplayFormat( m_symbolName2dotpos[it->first], true );
                m_aimvalues->setTopValue( it->second, true );
            }
        }
        else
        {
            it2 = m_selection2symbolName.find( m_topSelectedValue );

            if ( it2 != m_selection2symbolName.end() )
            {
                it = table->find( it2->second );

                if ( it != table->end() )
                {
                    m_aimvalues->setTopDisplayFormat( m_symbolName2dotpos[it->first], false );
                    m_aimvalues->setTopValue( it->second, false );
                }
            }
        }
    }
    else
    {
        if ( m_isCustomTop[top] == false ) // it is an AIM_XX
        {
            if ( m_isAIM )
            {
                if ( m_isAimTesting == false )
                {
                m_aimvalues->setTopValue( _T("---"), isBottom );
                }
            else
                {
                    CustomSettings s = getDefAIMValues( m_topSelector->GetString( top ) );

                    wxString pf = ( isBottom ) ? _T("bt") : _T("tt");
                    wxString symName = _T("aim_custom_c") + s.aimchId + _T("_") + pf;
                    double val = (*table)[symName];
                    wxString dispVal = formatAIMValue( val, true, s.dotSelection );
                    m_aimvalues->setTopValue( dispVal, isBottom );

                }
            }
            else
                m_aimvalues->setTopValue( _T("  . "), isBottom );
        }
        else // custom_xx
        {
            int mode = -1;

            // custom0
            if ( top == 3 )
                mode = 0;
            // custom1
            if ( top == 4 )
                mode = 1;
            // custom4
            if ( top == 7 )
                mode = 4;

            wxString dispVal;

            if ( mode >= 0 )
            {
                double a = m_aima[mode];
                int channel = m_aimchannel[mode];
                int dotpos = (int)m_aimdot[mode];

                // it is a "dot only"
                if ( a == 0 )
                {
                    // get the position of dot

                    switch ( dotpos )
                    {
                        case 0: dispVal = _T("  . "); break;
                        case 1: dispVal = _T(" .  "); break;
                        case 3:
                        default:
                            dispVal = _T(" ");
                    }
                }
                else
                {
                    if ( m_isAimTesting )
                    {
                        wxString pf = ( isBottom ) ? _T("bt") : _T("tt");
                        wxString symName = wxString::Format( _T("aim_custom_c%d_") + pf, channel );
                        double val = (*table)[symName];
                        dispVal = formatAIMValue( val, true, dotpos );
                    }
                    else
                    dispVal = _T("---");
            }

            }
            else
                dispVal = _T(" ");

            m_aimvalues->setTopValue( dispVal, isBottom );
        }
    }

    //---------------------------------------------------------------------------------------------------
    // bottom round (active slot)
    if ( fixedBottom )
    {
        if ( isBottom )
        {
            it = table->find( m_activeSlotBottomSymbolName );

            if ( it != table->end() )
            {
                m_aimvalues->setBottomDisplayFormat( m_symbolName2dotpos[it->first], true );

                if ( it->first == _T("egt") )
                {
                    wxString egtS;
                    egtS = ( m_isCelsius ) ? formatEGT( it->second ) : formatEGT ( (*table)[_T("egtf")] );
                    m_aimvalues->setBottomValue( egtS, true );
                }
                else
                    m_aimvalues->setBottomValue( it->second, true );
            }
        }
        else
        {
            it2 = m_selection2symbolName.find( m_bottomSelectedValue );
            if ( it2 != m_selection2symbolName.end() )
            {
                it = table->find( it2->second );

                if ( it != table->end() )
                {
                    m_aimvalues->setBottomDisplayFormat( m_symbolName2dotpos[it->first], false );
                    if ( it->first == _T("egt") )
                    {
                        wxString egtS;
                        egtS = ( m_isCelsius ) ? formatEGT( it->second ) : formatEGT ( (*table)[_T("egtf")] );
                        m_aimvalues->setBottomValue( egtS, false );
                    }
                    else
                        m_aimvalues->setBottomValue( it->second, false );
                }
            }
        }
    }
    else
    {
        if ( m_isCustomBottom[bottom] == false ) // aim_xx
        {
            if ( m_isAIM )
            {
                if ( m_isAimTesting == false )
                {
                m_aimvalues->setBottomValue( _T("----"), isBottom );
                }
            else
                {
                    CustomSettings s = getDefAIMValues( m_bottomSelector->GetString( bottom ) );

                    wxString pf = ( isBottom ) ? _T("bb") : _T("tb");
                    wxString symName = _T("aim_custom_c") + s.aimchId + _T("_") + pf;
                    double val = (*table)[symName];
                    wxString dispVal = formatAIMValue( val, false, s.dotSelection );
                    m_aimvalues->setBottomValue( dispVal, isBottom );
                }
            }

            else
                m_aimvalues->setBottomValue( _T("   . "), isBottom );
        }
        else // custom_xx
        {
            int mode = -1;

            // custom2
            if ( bottom == 6 )
                mode = 2;

            // custom3
            if ( bottom == 7 )
                mode = 3;

            wxString dispVal;

            if ( mode >= 0 )
            {
                double a = m_aima[mode];
                int channel = m_aimchannel[mode];
                int dotpos = (int)m_aimdot[mode];

                if ( a == 0 )
                {
                    switch ( dotpos )
                    {
                        case 0: dispVal = _T("   . "); break;
                        case 1: dispVal = _T("  .  "); break;
                        case 2: dispVal = _T(" .   "); break;
                        case 3:
                        default:
                            dispVal = _T(" ");
                    }
                }
                else
                {
                    if ( m_isAimTesting )
                    {
                        wxString pf = ( isBottom ) ? _T("bb") : _T("tb");
                        wxString symName = wxString::Format( _T("aim_custom_c%d_") + pf, channel );
                        double val = (*table)[symName];

                        dispVal = formatAIMValue( val, false, dotpos );
            }
                    else
                        dispVal = _T("----");
                }
            }

            m_aimvalues->setBottomValue( dispVal, isBottom );
        }
    }
}

/// -->RTDataViewerBase virtual methods
void AIMDisplayPanel::refresh( DoubleHash* table )
{

    if ( m_isOnline )
    {
    // get active slot
    DoubleHash::iterator it = table->find(_T("slotNumber"));

    if ( it != table->end() )
    {
        int slot = (int)it->second;

        if ( m_activeSlot != slot )
        {
            if ( m_activeSlot == -1 )
            {
                wxCommandEvent evt( wxEVT_COMMAND_TOGGLEBUTTON_CLICKED, m_slotSelectorsIds[slot] );
                //m_slotSelectors[slot]->SetValue( true );
                AddPendingEvent( evt );
            }
            wxFont fnt = m_slotSelectors[m_activeSlot]->GetFont();
            fnt.SetWeight( wxFONTWEIGHT_NORMAL );
            m_slotSelectors[m_activeSlot]->SetFont( fnt );

            fnt.SetWeight( wxFONTWEIGHT_BOLD );
            m_slotSelectors[slot]->SetFont( fnt );
            m_activeSlot = slot;

                m_aimvalues->setSlots( m_selectedSlot, m_activeSlot );

                int trt, trb, brt, brb;
                trt = m_SlotTop[m_selectedSlot];
                trb = m_SlotBottom[m_selectedSlot];
                brt = m_SlotTop[m_activeSlot];
                brb = m_SlotBottom[m_activeSlot];

                m_aimvalues->setSelectedValues( trt, trb, brt, brb );

                update();
        }
    }
    }

    // initialize if neccessary
    if ( m_initialized == false )
    {
        update();
        m_initialized = true;
    }

    // values of the active slot
    int tval = m_SlotTop[m_activeSlot];
    int bval = m_SlotBottom[m_activeSlot];

    refreshRound( table, tval, bval, true );

    // <--- end of refreshing lower round

    // --> try refresh upper round
    tval = m_SlotTop[m_selectedSlot];
    bval = m_SlotBottom[m_selectedSlot];

    refreshRound( table, tval, bval, false );

    m_aimvalues->setSlots( m_selectedSlot, m_activeSlot );

    m_aimvalues->redraw();
}

void AIMDisplayPanel::require()
{
    if ( m_rtDataMgr )
    {
        for ( size_t i = 0; i < m_rtSymbolsToRegister.GetCount(); ++i )
        {
            m_rtDataMgr->registerSymbol( m_rtSymbolsToRegister[i] );
        }
    }

}

void AIMDisplayPanel::unregisterSymbols()
{
    if ( m_rtDataMgr )
    {
        for ( size_t i = 0; i < m_rtSymbolsToRegister.GetCount(); ++i )
        {
            m_rtDataMgr->unregisterSymbol(  m_rtSymbolsToRegister[i] );
        }
    }
}

/// <--RTDataViewerBase virtual methods

void AIMDisplayPanel::onSlotSelect( wxCommandEvent& event )
{
    m_slotSelectors[m_selectedSlot]->SetValue( false );

    int id = event.GetId();
    for ( size_t i = 0; i < 8; ++i )
    {
        if ( id == m_slotSelectorsIds[i] )
        {
            m_selectedSlot = i;
            break;
        }
    }

    if ( m_slotSelectors[m_selectedSlot]->GetValue() == false )
        m_slotSelectors[m_selectedSlot]->SetValue( true );

    if ( m_symbols == NULL || m_stDataMgr == NULL )
        return;

    SymbolTable::iterator it;

    bool needsrefresh = false;
    // upper display
    wxString symName = wxString::Format( _T("disp_mode_t%d"), m_selectedSlot );
    it = m_symbols->find( symName );

    if ( it != m_symbols->end() )
    {
        int val = (int)( m_stDataMgr->getDataFromViewerModel(*(*m_symbols)[symName]) );

        if ( val != m_topSelector->GetSelection() )
        {
            m_topSelector->SetSelection( val );
            needsrefresh = true;
        }
    }
    // lower display
    symName = wxString::Format( _T("disp_mode_b%d"), m_selectedSlot );
    it = m_symbols->find( symName );

    if ( it != m_symbols->end() )
    {
        int val = (int)( m_stDataMgr->getDataFromViewerModel(*(*m_symbols)[symName]) );

        if ( val != m_bottomSelector->GetSelection() )
        {
            m_bottomSelector->SetSelection( val );
            needsrefresh = true;
        }
    }
    m_topSelectedValue = m_topSelector->GetValue();
    m_bottomSelectedValue = m_bottomSelector->GetValue();

    m_aimvalues->setSelectedValues( m_topSelector->GetSelection(), m_bottomSelector->GetSelection() );

    // update right display if neccessary
    if ( needsrefresh )
    {
        updateCustomPanels();
    }
    checkSymbolsToRegister();
    m_aimvalues->redraw();
}

void AIMDisplayPanel::updateCustomPanels()
{
    enableCustomPanels();

    //
    wxString top = m_topSelectedValue;
    wxString bottom = m_bottomSelectedValue;
    wxString topSym = _T(""), bottomSym = _T("");

    StringHash::iterator it = m_symbolName2selection.begin();
    for ( ; it != m_symbolName2selection.end(); ++it )
    {
        if ( it->second == top ) topSym = it->first;
        if ( it->second == bottom ) bottomSym = it->first;
    }

    ////------------------ UPDATE TOP CUSTOM PANEL --------------------
    // is it fixed? (Lambda, ANALOG, Afr)
    if ( !topSym.IsEmpty() )
    {
        CustomSettings s;
        s.add = _T("-"); s.aimchId=_T("-");s.conversion=_T("-"); s.multiplicator=_T("-"); s.dot = m_symbolName2dotpos[topSym];
        m_customTopValuesPanel->setCustomSettings( s );
    }
        // AIM VALUES (aim mode) OR BLANK
    else
    {
        if ( m_isAIM )
        {
            if ( top.StartsWith(_T("AIM") ) && ( top.StartsWith(_T("AIM Custom") ) == false ) )
            {
                CustomSettings s = getDefAIMValues( top );
                m_customTopValuesPanel->setCustomSettings( s );

                //
                m_aimvalues->setTopValue(_T("---"), false);
            }
            else
            {
                wxString modeS = top.Mid( 10, 1 ); long mode; modeS.ToLong(&mode);
                CustomSettings s;

                s.aimchId << m_aimchannel[mode];
                s.multiplicator = wxString::Format( _T("%.2f"), m_aima[mode] );
                s.add = wxString::Format( _T("%.2f"), m_aimb[mode] );
                s.dotSelection = m_aimdot[mode];
                s.dot = _T("");

                // is it blank?
                wxString displayVal = _T("---");
                if ( m_aima[mode] == 0 )
                {
                    switch ( s.dotSelection )
                    {
                        case 0:  displayVal = _T("  . ");break;
                        case 1: displayVal = _T(" .  "); break;
                        default: displayVal = _T(" ");break;
                    }
                }
                m_customTopValuesPanel->setCustomSettings( s );
                if ( displayVal )
                    m_aimvalues->setTopValue( displayVal, false );
            }
        } // if aim
        else // not aim
        {
            if ( top == _T("Blank.X") )
            {
                CustomSettings s;
                s.add = _T("-"); s.aimchId=_T("-");s.conversion=_T("-"); s.multiplicator=_T("-"); s.dot = _T("XX.X");
                m_customTopValuesPanel->setCustomSettings( s );
                m_aimvalues->setTopValue( _T("  . "), false );
            }
            else
            {
                int mode = -1;
                if ( top == _T("Blank C0") ) mode = 0;
                if ( top == _T("Blank C1") ) mode = 1;
                if ( top == _T("Blank C4") ) mode = 4;

                int dotpos = m_aimdot[mode];

                CustomSettings s;

                s.add = _T("-"); s.aimchId=_T("-");s.conversion=_T("-"); s.multiplicator=_T("-");

                wxString displayVal;

                switch ( dotpos )
                {
                    case 0:  displayVal = _T("  . "); s.dot = _T("XX.X"); break;
                    case 1: displayVal = _T(" .  "); s.dot = _T("X.XX"); break;
                    default: displayVal = _T(" "); s.dot = _T("No Dot"); break;
                }
                m_customTopValuesPanel->setCustomSettings( s );
                m_aimvalues->setTopValue( displayVal,false );
            }
        }
    }


    ////------------------ UPDATE BOTTOM CUSTOM PANEL --------------------
    // is it fixed? (EGT, ANALOG, RPM)
    if ( !bottomSym.IsEmpty() )
    {
        CustomSettings s;
        s.add = _T("-"); s.aimchId=_T("-");s.conversion=_T("-"); s.multiplicator=_T("-"); s.dot = m_symbolName2dotpos[bottomSym];
        m_customBottomValuesPanel->setCustomSettings( s );
    }
    else
    {
        if ( m_isAIM )
        {
            if ( bottom.StartsWith(_T("AIM") ) && ( bottom.StartsWith(_T("AIM Custom") ) == false ) )
            {
                CustomSettings s = getDefAIMValues( bottom );
                m_customBottomValuesPanel->setCustomSettings( s );
                m_aimvalues->setBottomValue(_T("----"), false);
            }
            else
            {
                wxString modeS = bottom.Mid( 10, 1 ); long mode; modeS.ToLong(&mode);
                CustomSettings s;

                s.aimchId << m_aimchannel[mode];

                s.multiplicator = wxString::Format(_T("%.2f"), m_aima[mode] );

                s.add = wxString::Format(_T("%.2f"), m_aimb[mode] );

                s.dotSelection = m_aimdot[mode]; s.dot = _T("");

                // is it blank?
                wxString displayVal = _T("----");
                if ( m_aima[mode] == 0 )
                {
                    switch ( s.dotSelection )
                    {
                        case 0:  displayVal = _T("   . ");break;
                        case 1: displayVal = _T("  .  "); break;
                        case 2: displayVal = _T(" .   "); break;
                        default: displayVal = _T(" ");break;
                    }
                }
                m_customBottomValuesPanel->setCustomSettings( s );
                if ( displayVal )
                    m_aimvalues->setBottomValue( displayVal, false );
            }
        } // if aim
        else // not aim
        {
            if ( bottom == _T("Blank.X") )
            {
                CustomSettings s;
                s.add = _T("-"); s.aimchId=_T("-");s.conversion=_T("-"); s.multiplicator=_T("-"); s.dot = _T("XXX.X");
                m_customBottomValuesPanel->setCustomSettings( s );
                m_aimvalues->setBottomValue( _T("   . "), false );
            }
            else
            {
                int mode = -1;
                if ( bottom == _T("Blank C2") ) mode = 2;
                if ( bottom == _T("Blank C3") ) mode = 3;

                int dotpos = m_aimdot[mode];

                CustomSettings s;

                s.add = _T("-"); s.aimchId=_T("-");s.conversion=_T("-"); s.multiplicator=_T("-");

                wxString displayVal;

                switch ( dotpos )
                {
                    case 0: displayVal = _T("   . "); s.dot = _T("XXX.X"); break;
                    case 1: displayVal = _T("  .  "); s.dot = _T("XX.XX"); break;
                    case 2: displayVal = _T(" .   "); s.dot = _T("X.XXX"); break;
                    default: displayVal = _T(" "); s.dot = _T("No Dot"); break;
                }

                m_customBottomValuesPanel->setCustomSettings( s );

                m_aimvalues->setBottomValue( displayVal, false );
            }
        }
    }
}

void AIMDisplayPanel::enableCustomPanels()
{
    if ( m_isCustomTop[ m_topSelector->GetSelection() ] )
        m_customTopValuesPanel->Enable();
    else
        m_customTopValuesPanel->Disable();

    if ( m_isCustomBottom[ m_bottomSelector->GetSelection() ] )
        m_customBottomValuesPanel->Enable();
    else
        m_customBottomValuesPanel->Disable();
}

void AIMDisplayPanel::onTopSelected( wxCommandEvent& event )
{
    m_topSelectedValue = m_topSelector->GetValue();

    m_writeCustomSettings = false;
    requestWrite();

    if ( m_stDataMgr )
        m_stDataMgr->updateRegisteredViewers();

}

void AIMDisplayPanel::onBottomSelected( wxCommandEvent& event )
{
    m_bottomSelectedValue = m_bottomSelector->GetValue();

    m_writeCustomSettings = false;
    requestWrite();

    if ( m_stDataMgr )
        m_stDataMgr->updateRegisteredViewers();
}

void AIMDisplayPanel::onModeSelect( wxCommandEvent& event )
{
    int id = event.GetId();

    int selection = -1;
    int selectedValue = 0;

    for ( size_t i = 0; i < 16; ++i )
    {
        if ( m_modeSelectorsIds[i] == id )
        {
            selection = i;
            selectedValue = m_modeSelectors[i]->GetSelection();
            break;
        }
    }

    if ( selection < 0 )
    {
        event.Skip();
        return;
    }

    wxString symName;
    if ( selection < 8 )
    {
        symName = wxString::Format( _T("disp_mode_t%d"), selection );

        if ( selection == m_selectedSlot)
        m_topSelectedValue = m_modeSelectors[selection]->GetValue();
    }
    else
    {
        symName = wxString::Format( _T("disp_mode_b%d"), selection - 8 );

        if ( selection == ( m_selectedSlot + 8 ) )
        m_bottomSelectedValue = m_modeSelectors[selection]->GetValue();
    }

    Symbol* sym = NULL;

    SymbolTable::iterator it = m_symbols->find( symName );

    if ( it != m_symbols->end() )
    {
        sym = it->second;
    }
    if ( sym )
    {
        m_stDataMgr->setModelData( *sym, 0, 0, (double) selectedValue );

        m_stDataMgr->requestWrite( *sym );

        m_stDataMgr->updateRegisteredViewers();
    }
    update();
}


void AIMDisplayPanel::getDisplayValues()
{
    try {
        int isaim = ( m_isAIM ) ? 1 : 0;

        wxString q = wxString::Format( _T("SELECT name, iscustom, isfixedvar FROM aimmodes WHERE isbottom=0 AND isaim=%d ORDER BY id ASC"), isaim );
        wxSQLite3ResultSet res = m_db.ExecuteQuery( q );

        int i=0;
        while ( res.NextRow() )
        {
            wxString str = res.GetString(_T("name"));
            m_isCustomTop[i] = res.GetBool(_T("iscustom"));
            m_isFixedTop[i++] = res.GetBool( _T("isfixedvar"));
            m_topvaluesString.Add( str );
        }

        q = wxString::Format( _T("SELECT name, iscustom, isfixedvar FROM aimmodes WHERE isbottom=1 AND isaim=%d ORDER BY id ASC"), isaim );
        res = m_db.ExecuteQuery( q );

        i = 0;
        while ( res.NextRow() )
        {
            wxString str = res.GetString(_T("name"));
            m_isCustomBottom[i] = res.GetBool(_T("iscustom"));
            m_isFixedBottom[i++] = res.GetBool( _T("isfixedvar"));
            m_bottomvaluesString.Add( str );
        }
    }
    catch ( wxSQLite3Exception e )
    {
        LOGP( e.GetMessage() )
    }
}

CustomSettings AIMDisplayPanel::getDefAIMValues( const wxString& val )
{
    CustomSettings s;

    if ( val == _T("AIM RPM") )
    {
        s.aimchId = _T("1"); s.multiplicator = _T("1");
        s.add = _T("0"); s.conversion = _T("y = x");
        s.dot = _T("No Dot"); s.dotSelection = 3;
    }
    else if ( val == _T("AIM WHEELSPEED") )
    {
        s.aimchId = _T("5"); s.multiplicator = _T("0.1");
        s.add = _T("0"); s.conversion = _T("y = x/10");
        s.dot = _T("X.XX"); s.dotSelection = 1;
    }
    else if ( val == _T("AIM Lambda") )
    {
        s.aimchId = _T("105"); s.multiplicator = _T("0.1");
        s.add = _T("0"); s.conversion = _T("y = x/1000");
        s.dot = _T("X.XX"); s.dotSelection = 1;
    }
    else if ( val == _T("AIM EGT") )
    {
        s.aimchId = _T("101"); s.multiplicator = _T("0.1");
        s.add = _T("-100"); s.conversion = _T("y = x/10 - 100");
        s.dot = _T("No Dot"); s.dotSelection = 3;
    }
    else if ( val == _T("AIM MAP") )
    {
        s.aimchId = _T("69"); s.multiplicator = _T("1");
        s.add = _T("0"); s.conversion = _T("y = x");
        s.dot = _T("XXX.X"); s.dotSelection = 0;
    }
    else {
        s.aimchId = _T("-"); s.multiplicator = _T("-");
        s.add = _T("-"); s.conversion = _T("-");
        s.dot = _T("-");
    }

    return s;
}

wxString AIMDisplayPanel::formatEGT( const double& egtval )
{
    if ( egtval < 1000.0 )
    {
        wxString egtS = wxString::Format( _T("%03d"), (int)egtval );
        if ( m_isCelsius )
            egtS += _T("c");
        else
            egtS += _T("F");
        return egtS;
    }
    else
    {
        return wxString::Format(_T("%04d"), (int)egtval );
    }
}

bool AIMDisplayPanel::isCustom( const wxString& name )
{
    if ( m_isAIM )
    {
        if ( name.StartsWith(_T("AIM Custom") ) )
            return true;
        else return false;
    }
    else
    {
        if ( name.StartsWith(_T("Blank C") ) )
            return true;
        else return false;
    }
}

bool AIMDisplayPanel::isAIM( const wxString& name )
{
    if ( m_isAIM )
    {
        if ( name.StartsWith(_T("AIM") ) )
        {
            if ( name.StartsWith(_T("AIM Custom") ) )
                return false;
            else
            return true;
        }

        else
            return false;
    }
    else
    {
        if ( name.StartsWith(_T("Blank.") ) )
            return true;
        else
            return false;
    }
}

void AIMDisplayPanel::onTestButtons( wxCommandEvent& event )
{
    int id = event.GetId();

    if ( m_isAIM == false )
        return;

    if (  m_aimthread == NULL )
        return;

    if ( id == idAIMTestComm )
    {
        AIMCommunicationConfig* cfgdlg = new AIMCommunicationConfig( this );
        cfgdlg->Show();
        /*int ret = cfgdlg.ShowModal();
        if ( ret > 0 )
        {
            // setting have been changed
            if ( m_aimthread )
            {
                delete m_aimthread;
            }
            getAimTestSettings();

            m_aimthread = new AIMCommunicationThread( m_serialConfig, this );
            if ( ret == 1 ) // enabled
            {
                m_testPanel->setTestEnabled( true );
            }
            if ( ret == 2 )
            {
                m_testPanel->setTestEnabled( false );
            }
        }*/
    }

    if ( id == idAIMTestStart )
    {
        AIMDisplayHelper::clearChannels();

        for ( unsigned int i = 0; i < m_requiredChannels.GetCount(); ++i )
        {
            AIMDisplayHelper::addChannel( m_requiredChannels[i] );
        }
        getAimTestSettings();

        wxArrayString validPorts = SerialPortHelper::GetCOMPortList();
        wxString portname = m_serialConfig.getPortName();
        bool validport = ( validPorts.Index( portname ) != wxNOT_FOUND );

        if ( portname == _T("") || validport == false )
        {
            wxMessageBox(_T("The AIM communication port is invalid\nClick on the AIM Port button\nto select the proper port"),
                _T("Information"), wxOK | wxICON_INFORMATION );
            return;
        }

        m_aimthread->setSerialConfig( m_serialConfig );

        m_isAimTesting = true;

        m_aimthread->start();
        m_testPanel->enableButtons( 2 );
    }
    if ( id == idAIMTestStop )
    {
        m_isAimTesting = false;
        m_aimthread->stop();
        m_testPanel->enableButtons( 1 );
    }
}

void AIMDisplayPanel::getAimTestSettings()
{
    ConfigBase* configBase = ConfigHelper2::getConfig();
    SerialConfigMgr cfgMgr( configBase, _T("/aimsettings") );
    cfgMgr.load( &m_serialConfig );
}

void AIMDisplayPanel::onEvent( GeneralEvent& event )
{
    if ( event.GetId() == 520 )
    {
        if ( event.GetString() == _T("timeout") )
        {
            wxMessageBox(_T("Can not open port/timeout"),_T("Error") );
        }
    }

    if ( event.GetId() == 512 )
    {
        int retvalue = event.GetLong();

        Enable();

        if ( retvalue > 0 )
        {
            // setting have been changed
            if ( m_aimthread )
            {
                delete m_aimthread;
            }
            getAimTestSettings();

            m_aimthread = new AIMCommunicationThread( m_serialConfig, this );
            if ( retvalue == 1 ) // enabled
            {
                m_testPanel->setTestEnabled( true );
            }
            if ( retvalue == 2 )
            {
                m_testPanel->setTestEnabled( false );
            }
        }
    }
}

void AIMDisplayPanel::onCommunicationEvent( CommunicationEvent& event )
{
    if ( event.GetId() == CommunicationEvent::CANNOT_OPEN_PORT )
    {
        m_testPanel->enableButtons( 1 );
        m_isAimTesting = false;
        m_aimthread->stop();
        wxString msg = _T("Cannot open port\nIf you think the port settings should be OK (the test was succesful), ");
        msg += _T("please remove and re-plug USB/RS232 adapter");

        wxMessageBox( msg, _T("Error"), wxCANCEL | wxICON_ERROR );
    }
}

void AIMDisplayPanel::onInputEvent( InputEvent& event )
{

    if ( m_isAIM == false )
    {
        event.Skip();
        return;
    }

    SIPRPacket p = event.getPacket();

    if ( p.type() == SiprSensorType::SIPR_AIM )
    {
        if (((RTDataMgrFacade*)m_rtDataMgr)->process( p ) )
            m_rtDataMgr->filteredNotifyViewers();

        // create aim packet
        uint8_t aim_p[5];
        SIPR2AIMConverter::Convert( p, aim_p );

        TxData data( aim_p, 5 );
        VemsCommand* cmd = new RawCommand( data );
        m_reqMgr->request( cmd );
    }
}

void AIMDisplayPanel::onClose( wxCloseEvent& event )
{
    if ( GetParent() )
        GetParent()->Close();
}

void AIMDisplayPanel::checkSymbolsToRegister()
{
    if ( m_isAIM == false )
        return;

    unregisterCustomsRTs();

    m_customSymbols.clear();
    wxArrayString aims;

    int modes[4];
    modes[0] = m_selectedSlot; modes[1] = m_selectedSlot + 8;
    modes[2] = m_activeSlot; modes[3] = m_activeSlot + 8;

    wxArrayString smodes; smodes.Add(_T("tt")); smodes.Add(_T("tb")); smodes.Add(_T("bt")); smodes.Add(_T("bb"));

    m_requiredChannels.Clear();

    for ( size_t i = 0; i < 4; ++i )
    {
        int& mode = modes[i];
        const wxString& str = m_modeSelectors[mode]->GetValue();
        if ( isAIM( str ) )
        {
            CustomSettings s = getDefAIMValues( str );

            wxString symName = str.Lower();

            SymbolTable::iterator it = m_customSymbols.find( symName );

            if ( it == m_customSymbols.end() )
            {
                Symbol* sym = new Symbol;
                long offset; s.aimchId.ToLong( &offset );
                double a,b; s.multiplicator.ToDouble(&a); s.add.ToDouble(&b);

                wxString nsymName = wxString::Format( _T("aim_custom_c%d_"), offset ) + smodes[i];

                m_requiredChannels.Add( offset );

                sym->setBasic( nsymName, _T(""), false, 255, offset );
                sym->setInputType( SiprSensorType::SIPR_AIM );
                sym->setType( false, 2 );

                sym->setFormat( a, b, 0, 0, 0 );
                sym->setGaugeType( _T("") );
                sym->setTitle( _T("") );

                m_customSymbols[nsymName] = sym;
            }
        }
        if ( isCustom( str ) )
        {
            wxString cmodes = str.Right(1); long cmode; cmodes.ToLong( &cmode );

            wxString symName = wxString::Format(_T("aim_channel%d"), cmode );
            int channel = 0;

            SymbolTable::iterator it = m_symbols->find( symName );
            if ( it != m_symbols->end() )
                channel = (int) m_stDataMgr->getDataFromViewerModel(*(*m_symbols)[symName]);

            wxString nsymName = wxString::Format(_T("aim_custom_c%d_"), channel );

            nsymName += smodes[i];

            it = m_customSymbols.find( nsymName );

            if ( it == m_customSymbols.end() )
            {
                Symbol* sym = new Symbol;

                double a = 0, b = 0;

                symName = wxString::Format(_T("aim_a%d"), cmode );

                SymbolTable::iterator it2 = m_symbols->find( symName );

                if ( it2 != m_symbols->end() )
                    a = m_stDataMgr->getDataFromViewerModel(*(*m_symbols)[symName]);

                symName = wxString::Format(_T("aim_b%d"), cmode );

                it2 = m_symbols->find( symName );

                if ( it2 != m_symbols->end() )
                    b = m_stDataMgr->getDataFromViewerModel(*(*m_symbols)[symName]);

                sym->setBasic( nsymName, _T(""), false, 255, channel );
                sym->setInputType( SiprSensorType::SIPR_AIM );
                sym->setType( false, 2 );

                sym->setFormat( a, b, 0, 0, 0 );
                sym->setGaugeType( _T("") );
                sym->setTitle( _T("") );

                m_customSymbols[nsymName] = sym;

                m_requiredChannels.Add( channel );
            }
        }
    }

    SymbolTable::iterator it = m_customSymbols.begin();

    for ( ; it != m_customSymbols.end(); ++it )
    {
        m_symbolMgr->addOrReplaceSymbol( it->second );

        //LOGP( _T("symboltable registered-->") + it->first )
    }

    registerCustomsRTs();

    AIMDisplayHelper::clearChannels();
    for ( size_t i = 0; i < m_requiredChannels.GetCount(); ++i )
    {
        AIMDisplayHelper::addChannel( m_requiredChannels[i] );
    }
}

void AIMDisplayPanel::registerCustomsRTs()
{
    SymbolTable::iterator it = m_customSymbols.begin();

    for ( ; it != m_customSymbols.end(); ++it )
    {
        wxString& symn = it->first;
        // m_rtDataMgr->registerSymbol( symn );

        DoubleHash& dh = m_rtDataMgr->getRequiredDH_RW();
        dh[symn] = 0.0f;

        //LOGP( symn + _T(" registered to rts") )
    }

    m_customRTsRegistered = true;
}

void AIMDisplayPanel::unregisterCustomsRTs()
{
    if ( m_customRTsRegistered == false )
        return;

    SymbolTable::iterator it = m_customSymbols.begin();

    for ( ; it != m_customSymbols.end(); ++it )
    {
        wxString& symn = it->first;
        m_rtDataMgr->unregisterSymbol( symn );
        //LOGP( symn + _T(" unregistered to rts") )
    }

    m_customRTsRegistered = false;
}

inline wxString AIMDisplayPanel::formatAIMValue( double& val, bool isTopRow, int dotpos )
{
    ///
    /// TODO: negative numbers
    ///
    int value = (int)val;
    bool valid = true;

    wxString formatted;
    wxString hexarr[6] = { _T("A"), _T("b"), _T("c"), _T("d"), _T("E"), _T("F") };

    if ( isTopRow )
    {
        // negative value
        if ( value < 0 )
        {
            int absval = (int)( abs( value ) );
            if ( absval < 100 )
                formatted = wxString::Format(_T("-%02d"), absval );
            else {
                int firstD = absval / 10, rem = absval % 10;
                if ( firstD >= 16 )
                {
                    valid = false; formatted = _T("-FU");
                }
                else
                {
                    formatted = _T("-") + hexarr[firstD - 10] + wxString::Format( _T("%d"), rem );
                }
            }
        }
        // positive value
        else {
            if ( value < 1000 ) formatted = wxString::Format( _T("%03d"), value );
            else
            {
                int firstD = value / 100, rem = value % 100;
                if ( firstD >= 16 ) {
                    valid = false; formatted = _T("FFU");
                }
                else
                {
                    formatted = hexarr[ firstD - 10 ] + wxString::Format( _T("%02d"), rem );
                }
            }
        }
    }
    else
    {
        // negative number
        if ( value < 0 )
        {
            int absval = (int)( abs( value ) );

            if ( absval < 1000 )
                formatted = wxString::Format( _T("-%03d"), absval );
            else
            {
                int firstD = absval / 100, rem = absval % 100;
                if ( firstD >= 16 )
                {
                    valid = false;
                    formatted = _T("-FFU");
                }
                else
                {
                    formatted = _T("-") + hexarr[firstD-10] + wxString::Format(_T("%02d"), rem );
                }
            }
        }
        // positive number
        else
        {
            if ( value < 10000 ) formatted = wxString::Format( _T("%04d"), value );
            else
            {
                int firstD = value / 1000, rem = value % 1000;
                if ( firstD >= 16 ) {
                    valid = false;; formatted = _T("FFFU");
                }
                else
                {
                    formatted = hexarr[ firstD - 10 ] + wxString::Format( _T("%03d"), rem );
                }
            }
        }
    }
    // overflow
    if ( !valid )
        return formatted;

    // place dot
    if ( isTopRow )
    {
        switch ( dotpos )
        {
            case 0: formatted = formatted.Left( 2 ) + _T(".") + formatted.Right( 1 ); break;
            case 1: formatted = formatted.Left( 1 ) + _T(".") + formatted.Right( 2 ); break;
            case 3:
            default: break;
        }
    }
    else
    {
        switch ( dotpos )
        {
            case 0: formatted = formatted.Left( 3 ) + _T(".") + formatted.Right( 1 ); break;
            case 1: formatted = formatted.Left( 2 ) + _T(".") + formatted.Right( 2 ); break;
            case 2: formatted = formatted.Left( 1 ) + _T(".") + formatted.Right( 3 ); break;
            case 3:
            default: break;
        }
    }

    return formatted;
}
