#ifndef AIMDISPLAYFRAME_H
#define AIMDISPLAYFRAME_H

#include <wx/wx.h>
#include <wx/tglbtn.h>

#include "../events/GeneralEvent.h"
#include "../events/CommunicationEvent.h"

#include "AIMCustomValuesEditorPanel.h"

#include "../common/types/DoubleHash.h"
#include "../common/types/IntArray.h"
#include "../common/types/Int2StringHash.h"
#include "../common/types/StringHash.h"
#include <wx/wxsqlite3.h>
#include "../rtdatamanager/RTDataViewerBase.h"
#include "../rtdatamanager/RTDataMgrBase.h"
#include "../stdatamanager/STDataViewerBase.h"
#include "../stdatamanager/STDataMgrBase.h"
#include "../vemsdialog/StaticViewersController.h"
#include "../vemsdialog/StaticControlPanel.h"
#include "../ObserverBase.h"
#include "../Symbol.h"
#include "../serialconfig.h"

#include "../bcWidgets/BitmapCanvasParentBase.h"
#include "../bcWidgets/bcButton.h"
#include "../bcWidgets/bcLabel.h"

class SymbolMgr;
class AIMValuesPanel;
class AIMFlagsPanel;
class FirmwareVersion;
class AIMCommunicationThread;
class InputEvent;
class CommandRequestBase;

class AIMTestButtonPanel : public wxPanel, public BitmapCanvasParentBase
{
    public:
        AIMTestButtonPanel( wxWindow* parent, const wxSize& size, bool isTestEnabled );

    public:
        void enableButtons( int b_enable );
        void setTestEnabled( bool isTestEnabled );

    private:
        void OnPaint( wxPaintEvent& event );

        wxWindow*   m_parent;
        bcLabel*    m_txt;
        bool        m_eventConnected;
        bcButton*   m_startButton;
        bcButton*   m_stopButton;
        bcButton*   m_aimcommButton;
        bool        m_istestEnabled;
};

class AIMDisplayPanel : public wxPanel, public STDataViewerBase, public RTDataViewerBase, public ObserverBase
{
    public:
        AIMDisplayPanel( wxWindow* parent, STDataMgrBase * stdata_mgr, RTDataMgrBase* rtdatamgr, SymbolTable* symbolTable,
            SymbolMgr* symMgr, FirmwareVersion& fwversion, CommandRequestBase* requestMgr );
        ~AIMDisplayPanel();

    public:
        // stdataviewerbase virtual methods
        void            update();
        void            requestRead() {}
        void            updateModel();
        void            requestWrite();

        StaticViewersController*  getController() { return m_statvcontroller; }

        // observerbase virtual methods
        void            notify( int panel, void* win );
        void            notify( const wxString& s );

        // rtdataviewerbase virtual methods
        void            SetValue(double value) {}
        void            refresh(DoubleHash* table);
        void            require();
        void            unregisterSymbols();
        void            SetValueIdx(int idx) {}

        //
        void            onTestButtons( wxCommandEvent& event );

    protected:

    private:
        bool                        m_isAIM;
        bool                        m_isOnline;
        bool                        m_firstUpdate;

        int                         m_activeSlot;
        wxString                    m_activeSlotTopSymbolName;
        wxString                    m_activeSlotBottomSymbolName;

        wxArrayString               m_rtSymbolsToRegister;
        StringHash                  m_symbolName2selection;
        StringHash                  m_selection2symbolName;
        StringHash                  m_symbolName2dotpos;
        Int2StringHash              m_val2symbolName; // 0 -> egt, 1 -> analin, 2 -> rpm etc

        wxArrayString               m_aimRtSymbols;

        Int2StringHash              m_channelId2Selection; //

        bool                        m_isFixedTop[8];
        bool                        m_isFixedBottom[8];
        bool                        m_isCustomTop[8];
        bool                        m_isCustomBottom[8];

        int                         m_SlotTop[8];
        int                         m_SlotBottom[8];

        double                      m_aima[5];
        double                      m_aimb[5];
        int                         m_aimdot[5];
        int                         m_aimchannel[5];

        //

        SymbolTable *               m_symbols;
        SymbolMgr*                  m_symbolMgr;
        StaticViewersController *   m_statvcontroller;
        StaticControlPanel *        m_statcpanel;
        CommandRequestBase*         m_reqMgr;

        wxToggleButton*             m_slotSelectors[8];
        int                         m_slotSelectorsIds[8];

        wxComboBox*                 m_modeSelectors[16]; // 2*8
        int                         m_modeSelectorsIds[16];
        //

        wxComboBox*                 m_topSelector;
        wxString                    m_topSelectedValue;
        wxComboBox*                 m_bottomSelector;
        wxString                    m_bottomSelectedValue;

        wxArrayString               m_topvaluesString;
        wxArrayString               m_bottomvaluesString;

        wxSQLite3Database           m_db;
        AIMFlagsPanel*              m_flagpanel;
        AIMValuesPanel*             m_aimvalues;
        AIMCustomValuesEditorPanel* m_customTopValuesPanel;
        AIMCustomValuesEditorPanel* m_customBottomValuesPanel;
        AIMTestButtonPanel*         m_testPanel;

        int                         m_selectedSlot;

        int                         m_demomodeVal;

        bool                        m_writeCustomSettings;

        bool                        m_isCelsius;

        bool                        m_initialized;

        SerialConfig                m_serialConfig;
        bool                        m_isAimTesting;
        AIMCommunicationThread*     m_aimthread;

        SymbolTable                 m_customSymbols;
        bool                        m_customRTsRegistered;

        IntArray                    m_requiredChannels;

    private:
        // gui creation
        void            addSlotSelector( wxSizer* sizer );
        void            addTopBottomSelector( wxWindow* parent, wxSizer * sizer );

        void            onSlotSelect( wxCommandEvent& event );
        void            onModeSelect( wxCommandEvent& event );
        void            onTopSelected( wxCommandEvent& event );
        void            onBottomSelected( wxCommandEvent& event );

        void            onEvent( GeneralEvent& event );
        void            onInputEvent( InputEvent& event );
        void            onClose( wxCloseEvent& event );
        void            onCommunicationEvent( CommunicationEvent& event );

        void            refreshRound( DoubleHash* table, int& top, int& bottom, bool isBottom );

        void            updateCustomPanels();

        // helpers
        void            getDisplayValues();
        void            getAimTestSettings();
        void            enableCustomPanels();
        void            initRTs();
        wxString        formatEGT( const double& egtval );

        void            redrawValues( int caller );

        void            checkSymbolsToRegister();

        inline bool     isCustom( const wxString& name );
        //inline bool     isCustom( int val );
        inline bool     isAIM( const wxString& name );
        //inline bool     isAIM( int val );

        CustomSettings  getDefAIMValues( const wxString& val );

        void            registerCustomsRTs();
        void            unregisterCustomsRTs();

        inline wxString formatAIMValue( double& val, bool isTopRow, int dotpos );

        DECLARE_EVENT_TABLE()

};

#endif // AIMDISPLAYFRAME_H
