#include "AIMFlagsPanel.h"

#include <wx/dcbuffer.h>
#include "../agg/wxAgg2.h"
#include "../DirHelper.h"
#include "../logger.h"

BEGIN_EVENT_TABLE( AIMFlagsPanel, wxPanel )
    EVT_PAINT( AIMFlagsPanel::OnPaint )
END_EVENT_TABLE()

AIMFlagsPanel::AIMFlagsPanel(wxWindow* parent) : wxPanel( parent, wxID_ANY, wxDefaultPosition, wxSize( -1, 25) )
{
    m_flags[AF_AIM_MODE] = 0;
    m_flags[AF_AIM_DEMO_MODE] = 255;

    Refresh();
}

void AIMFlagsPanel::OnPaint( wxPaintEvent& event )
{
    wxBufferedPaintDC dc( this );
    wxAgg2 canvas( this, wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE) );

    canvas.setBrush( wxDefaultAggBrush );

    wxColor flagoff( 0, 0, 0 ), flagon( 0, 160, 0 ), rectcolor( 130, 130, 130 ), white(255,255,255), black(0,0,0);
    canvas.loadTTFont( DirHelper::GetPath(dFONTS) + _T("arial.ttf"), 12, 0.0 );

    //
    wxString txtOut = _T("AIM Version ");
    canvas.setPen( rectcolor );
    if ( m_flags[AF_AIM_MODE] == 1 )
    {
        canvas.setBrush( flagon );
        txtOut += _T("ON");
    }

    else
    {
        canvas.setBrush( flagoff );
        txtOut += _T("OFF");
    }

    canvas.drawRectangle( 5, 5, 115, 20 ); canvas.setPen( white ); canvas.drawTTFText( txtOut, 10, 18 );

    txtOut = _T("Demo mode ");
    canvas.setPen( rectcolor );

    int flags = m_flags[AF_AIM_DEMO_MODE];

    if ( flags > 127 ) flags -= 128;

    wxColor txtColor = white;
    if (  flags == 127 )
    {
        canvas.setBrush( flagoff );
        txtOut += _T("OFF");
    }
    else
    {
        canvas.setBrush( wxColor( 255, 201, 14 ) );
        txtOut += _T("ON (");
        if ( flags == 0 ) txtOut += _T("all outputs forced)");
        else
        {
            //Logger::logf(LOG_MSG, _T("forc=%d"), flags );
            txtOut += _T("forcing segments: ");
            if ( ( flags & 1 ) != 1 ) txtOut += _T("0,");
            if ( ( flags & 2 )!= 2 ) txtOut += _T("1,");
            if ( ( flags & 4 ) != 4 ) txtOut += _T("2,");
            if ( ( flags & 8 ) != 8 ) txtOut += _T("3,");
            if ( ( flags & 16 )!= 16 ) txtOut += _T("4,");
            if ( ( flags & 32 ) != 32 ) txtOut += _T("5,");
            if ( ( flags & 64 ) != 64 ) txtOut += _T("6,");
            txtOut = txtOut.RemoveLast() + _T(")");
            txtColor = black;
        }
    }

    wxRealRect r = canvas.getTTFTextBound( txtOut );
    canvas.drawRectangle( 125, 5, r.w + 10, 20 ); canvas.setPen( txtColor ); canvas.drawTTFText( txtOut, 130, 18 );

    canvas.setPen( wxColor( 0, 0, 0 ) );
    txtOut = _T("Press F1 for show/hide the help");
    canvas.drawTTFText( txtOut, 140 + r.w + 10, 18 );

    canvas.render( dc );
}

void AIMFlagsPanel::setFlags( unsigned int flag_type, unsigned int flag_value )
{
    m_flags[flag_type] = flag_value;

    Refresh( false );
}
