#ifndef LEDDISPLAY_H
#define LEDDISPLAY_H

#include "../wxRealRect.h"
#include "../common/types/RealPointArray.h"
#include "../common/types/String2IntHash.h"
#include "../common/types/wxRegionArray.h"

class wxAgg2;

// A LED DIGIT
//
//          a (1)
//         -------
//        |       |
// f (32) |       | b (2)
//        | g(64) |
//         -------
//        |       |
// e (16) |       | c (4)
//        |       |
//         -------   (dot)(128)
//          d (8)

class LEDDisplay
{
    public:
        LEDDisplay();
        LEDDisplay( double height, int ledcolor = RED_LED );
        LEDDisplay( double height, const wxColor& activesegmentcolor,  double inclinationDeg = 0, const wxColor& inactivesegmentcolor = wxColor(0,0,0,0) );
        ~LEDDisplay();

    public:
        void        setCanvas( wxAgg2* canvas );
        void        drawText( const wxString& str, double x, double y );
        void        drawText( const wxChar& ch, double x, double y );

        void        draw( unsigned char segments, double x, double y );

        wxRealRect  getLEDSize() const;

        //8 wxRegions abcdefg(dot) representing the display - if more chars are displayed,
        // only the last is returned
        //wxRegionArray getSegmentRegions() { return m_regions; }
        wxPoint**   getSegmentRegions() { return m_regions; }

        enum LEDDisplayColor { GREEN_LED, RED_LED };

    private:
        double                  m_digitHeight;
        wxColor                 m_activecolor;
        wxColor                 m_inactivecolor;
        wxAgg2*                 m_canvas;

        wxRealPoint             m_segmenta[4];
        wxRealPoint             m_segmentb[5];
        wxRealPoint             m_segmentc[5];
        wxRealPoint             m_segmentd[4];
        wxRealPoint             m_segmente[5];
        wxRealPoint             m_segmentf[5];
        wxRealPoint             m_segmentg[6];
        wxRealPoint             m_segmentdot;
        double                  m_dotradius;
        double                  m_digitwidth;
        double                  m_inclination;

        double                  m_xpos, m_ypos;
        double                  m_xs, m_ys;

        static String2IntHash   m_charTable;

        //wxRegionArray           m_regions;
        wxPoint**               m_regions;

    private:
        void    init();

        void    calcCoordinates();

        void    translateSegment( RealPointArray& pts, wxRealPoint* segment, int pointnum, wxRealPoint& transl );
        double  inclineTr( double angleRad, double h );

        void    convertAndAddRegion( RealPointArray& pts, unsigned int segment );

        void    drawSegments( unsigned char segments, bool showDot );

        /// do not forget call this at least once!
        static void fillCharTable();

};

#endif // LEDDISPLAY_H
