#include "BitmapCanvasParentBase.h"

#include "../logger.h"
#include "../customwidgets/wxEventCatcher.h"

BitmapCanvasParentBase::BitmapCanvasParentBase(wxWindow* window)
{
    //ctor
    CreateBcParent(window);
}

void BitmapCanvasParentBase::CreateBcParent(wxWindow* window)
{
    m_lastZIndex = 0;
    m_window = window;
    m_focusedWidget = NULL;
    m_hoveredWidget = NULL;
    ClearRefreshRect();

    new wxEventCatcher<BitmapCanvasParentBase>(window, this);

    m_isSendLeaveEvt = true;
}

BitmapCanvasParentBase::~BitmapCanvasParentBase()
{
    //dtor
    BitmapCanvasWidgetBaseArray tmpA;

    for(BitmapCanvasWidgetBaseHash::iterator it = m_widgets.begin(); it != m_widgets.end(); it++) {
        tmpA.Add( it->second );
    }

    for(size_t a=0; a<tmpA.GetCount();a++) {  // because the BitmapCanvasWidgetBase modifies the m_widgets hash
        delete tmpA[a];
    }
}

void BitmapCanvasParentBase::OnEvent(wxEvent& evt)
{
    bcProcessEvent(evt);
}

void BitmapCanvasParentBase::UpdateWindow()
{
    if(!m_window) return;
    m_window->Update();
}

void BitmapCanvasParentBase::RefreshWindow(const wxRect& rect)
{
    if(!m_window) return;

    AddRefreshRect(rect);

    m_window->RefreshRect( m_refreshRect, false );

    ClearRefreshRect();
//    m_window->Refresh(false);
}

void BitmapCanvasParentBase::RefreshWindow(int widgetid)
{
    if(!m_window) return;

    if(widgetid == -1) {
        m_window->Refresh(false);
        return;
    }

    if( m_widgets.find(widgetid) == m_widgets.end() ) return;

    BitmapCanvasWidgetBase* widget = m_widgets[widgetid];

    if(widget && widget->IsShown()) {
        AddRefreshRect(widget->GetRect());
        m_window->RefreshRect( m_refreshRect, false );
        ClearRefreshRect();
    }

}

void BitmapCanvasParentBase::AddRefreshRect(const wxRect& rect)
{
    m_refreshRect.Union(rect);
}

void BitmapCanvasParentBase::ClearRefreshRect()
{
    m_refreshRect = wxRect(0,0,0,0);
}

void BitmapCanvasParentBase::bcProcessEvent(wxEvent& evt)
{
    //LOG
    //wxEventType event_type = evt.GetEventType();

    //int id = evt.GetId();

    int evtType = evt.GetEventType();

    int lastPosType = 0;

    if(evtType == wxEVT_LEFT_DOWN) lastPosType = 1;


    bool isSkip = true;

    if(evt.IsKindOf(CLASSINFO(wxMouseEvent))) {

        wxMouseEvent mevt = (wxMouseEvent&)evt;

        wxPoint pos = mevt.GetPosition();

        if( pos.x != 0 || pos.y != 0 )  {

            bool isDroppedLeaveEvtForWidget = false;
            bool isDroppedMouseEvtForWidget = false;

            for(BitmapCanvasWidgetBaseHash::iterator it = m_widgets.begin(); it != m_widgets.end(); it++) {

                BitmapCanvasWidgetBase * widget = it->second;

                int wid = widget->GetId();

                mevt.SetId(wid);

                if( !widget->IsShown() || widget->GetAlpha() < 1 || !widget->IsEnabled() ) continue;

                wxRect wrect = widget->GetRect();

                if(wrect.Contains( pos ) == false) {
                    if( wrect.Contains( m_lastMousePos[lastPosType] ) ) {
                        if(evtType == wxEVT_MOTION) {
                            SendMouseEvent(wxEVT_LEAVE_WINDOW, mevt, widget);
                            isDroppedLeaveEvtForWidget = true;
                            m_hoveredWidget = NULL;
                        } else if(evtType == wxEVT_LEFT_DOWN) {
                            SetWidgetFocus(NULL);
                        }
                    }
                    continue;
                }

                if(wrect.Contains( m_lastMousePos[lastPosType] ) == false) {
                    if(evtType == wxEVT_MOTION) {
                        if(m_isSendLeaveEvt) {
                            wxMouseEvent levent(wxEVT_LEAVE_WINDOW);
                            levent.m_linesPerAction = 42;
                            m_window->AddPendingEvent(levent);
                        }
                        SendMouseEvent(wxEVT_ENTER_WINDOW, mevt, widget);
                        m_hoveredWidget = widget;
                    } else if(evtType == wxEVT_LEFT_DOWN) {
                        SetWidgetFocus(widget);
                    }
                }

                wxMouseEvent devt( mevt );
                devt.m_x = pos.x - wrect.GetLeft();
                devt.m_y = pos.y - wrect.GetTop();

                widget->AddPendingEvent( devt );

                isSkip = false;
                isDroppedMouseEvtForWidget = true;
            }

            if(isDroppedLeaveEvtForWidget && !isDroppedMouseEvtForWidget) {
                wxMouseEvent levent(wxEVT_ENTER_WINDOW);
                m_window->AddPendingEvent(levent);
            }

            //Logger::logf(LOG_MSG,_T("m_lastMousePos.x=%i, y=%i, pos.x=%i,y=%i"),m_lastMousePos.x,m_lastMousePos.y,pos.x,pos.y);

            m_lastMousePos[lastPosType] = pos;
        }

        if(evtType == wxEVT_LEAVE_WINDOW) {

            if(m_hoveredWidget && mevt.m_linesPerAction != 42) { // special case
                SendMouseEvent(wxEVT_LEAVE_WINDOW, mevt, m_hoveredWidget);
                //m_hoveredWidget = NULL;
            }

        }


    } else if(evt.IsKindOf(CLASSINFO(wxKeyEvent))) {

        if(m_focusedWidget && m_focusedWidget->IsEnabled()) {
            wxKeyEvent kevt = (wxKeyEvent&)evt;
            m_focusedWidget->AddPendingEvent(kevt);
            isSkip = false;
        }

    } else {


    }

    evt.Skip(isSkip);

}

void BitmapCanvasParentBase::GetWidgetList(BitmapCanvasWidgetBaseArray& arr)
{
    for(BitmapCanvasWidgetBaseHash::iterator it = m_widgets.begin(); it != m_widgets.end(); it++) {
        arr.Add(it->second);
    }
}

wxSize BitmapCanvasParentBase::GetCanvasSize()
{
    if(!m_window) return wxDefaultSize;

    return m_window->GetClientSize();
}

void BitmapCanvasParentBase::AddWidget(BitmapCanvasWidgetBase* widget)
{
    int id = widget->GetId();
    m_widgets[ id ] = widget;
    WidgetAddCallBack(widget);
}

void BitmapCanvasParentBase::RemoveWidget(BitmapCanvasWidgetBase* widget)
{
    int id = widget->GetId();

    if( m_widgets.find(id) == m_widgets.end() ) return;

    m_widgets.erase(id);
}

void BitmapCanvasParentBase::SendMouseEvent(wxEventType type, wxMouseEvent& evt, BitmapCanvasWidgetBase* widget)
{

    wxPoint pos = evt.GetPosition();

    wxRect wrect = widget->GetRect();

    wxMouseEvent deevt( type );
    deevt.m_x = pos.x - wrect.GetLeft();
    deevt.m_y = pos.y - wrect.GetTop();
    widget->AddPendingEvent( deevt );
}

void BitmapCanvasParentBase::SendMouseEvent(wxEventType type, wxMouseEvent& evt, wxWindow* window)
{

}

void BitmapCanvasParentBase::PaintWidgets(wxDC& dc)
{
    wxRect dcRect(wxPoint(0,0),dc.GetSize());

    BitmapCanvasWidgetBaseArray widgetA;

    for(BitmapCanvasWidgetBaseHash::iterator it = m_widgets.begin(); it != m_widgets.end(); it++) {

        BitmapCanvasWidgetBase * widget = it->second;

        if( !widget->IsShown() ) continue;

        wxRect wrect = widget->GetRect();

        if( !dcRect.Intersects( wrect ) ) continue;

        wxBitmap* bmp = widget->GetBitmap();

        if(!bmp || !bmp->IsOk()) continue;

        widgetA.Add(widget);
    }

    widgetA.Sort( &BitmapCanvasWidgetBase::zindex_compare );

    for(size_t a=0; a < widgetA.GetCount(); a++){

        BitmapCanvasWidgetBase * widget = widgetA[a];

        wxBitmap* bmp = widget->GetBitmap( widget->IsEnabled() );
        wxRect wrect = widget->GetRect();
        if(!bmp || !bmp->IsOk()) continue;

        dc.DrawBitmap(*bmp, wrect.GetLeft(), wrect.GetTop() );
    }

}

int BitmapCanvasParentBase::GetNewZindex()
{
    return m_lastZIndex++;
}

void BitmapCanvasParentBase::SetWidgetFocus(BitmapCanvasWidgetBase* widget)
{
    if(widget) { // set focus to widget if possible

        if(!widget->IsEnabled() || !widget->IsShown() || !widget->Focusable()) return;

        wxFocusEvent evt(wxEVT_SET_FOCUS, widget->GetId());
        widget->AddPendingEvent(evt);

        if(m_focusedWidget) {
            wxFocusEvent evt1(wxEVT_KILL_FOCUS, m_focusedWidget->GetId());
            m_focusedWidget->AddPendingEvent(evt1);
        } else {
            wxFocusEvent evt2(wxEVT_KILL_FOCUS, m_window->GetId());
            m_window->AddPendingEvent(evt2);
        }

    } else { // kill focus from the last focused widget

        if(m_focusedWidget) {
            wxFocusEvent evt3(wxEVT_KILL_FOCUS, m_focusedWidget->GetId());
            m_focusedWidget->AddPendingEvent(evt3);
            wxFocusEvent evt4(wxEVT_SET_FOCUS, m_window->GetId());
            m_window->AddPendingEvent(evt4);
        }

    }

    m_focusedWidget = widget;
}

