#ifndef BITMAPCANVASPARENTBASE_H
#define BITMAPCANVASPARENTBASE_H

#include "BitmapCanvasWidgetBase.h"

WX_DEFINE_ARRAY(BitmapCanvasWidgetBase*, BitmapCanvasWidgetBaseArray);

WX_DECLARE_HASH_MAP( int, BitmapCanvasWidgetBase*, wxIntegerHash, wxIntegerEqual, BitmapCanvasWidgetBaseHash );

WX_DECLARE_HASH_MAP( int, wxPoint, wxIntegerHash, wxIntegerEqual, Int2wxPointHash);

class BitmapCanvasParentBase
{
    public:
        BitmapCanvasParentBase() { m_window = NULL; m_lastZIndex = 0; };
        BitmapCanvasParentBase(wxWindow* window);
        virtual ~BitmapCanvasParentBase();

        void bcProcessEvent(wxEvent& evt);

        void OnEvent(wxEvent& evt);

        void RefreshWindow(int widgetid = -1);
        void RefreshWindow(const wxRect& rect);
        void UpdateWindow();

        wxSize GetCanvasSize();

        void GetWidgetList(BitmapCanvasWidgetBaseArray& arr);

        void SetWidgetFocus(BitmapCanvasWidgetBase* widget);

        void AddRefreshRect(const wxRect& rect);
        void ClearRefreshRect();

    protected:

        /// call this function in the constructor of the target class
        void CreateBcParent(wxWindow* window);

        /// call this function in the OnPaint of the target class
        void PaintWidgets(wxDC& dc);

        bool m_isSendLeaveEvt;

        virtual void WidgetAddCallBack(BitmapCanvasWidgetBase* widget) {};

    private:

        void AddWidget(BitmapCanvasWidgetBase* widget);
        void RemoveWidget(BitmapCanvasWidgetBase* widget);

        void SendMouseEvent(wxEventType type, wxMouseEvent& evt, BitmapCanvasWidgetBase* widget);
        void SendMouseEvent(wxEventType type, wxMouseEvent& evt, wxWindow* window);

        int GetNewZindex();

        BitmapCanvasWidgetBaseHash  m_widgets;
        wxWindow*                   m_window;
        Int2wxPointHash             m_lastMousePos;
        BitmapCanvasWidgetBase*     m_focusedWidget;
        BitmapCanvasWidgetBase*     m_hoveredWidget;

        wxRect m_refreshRect;

        int m_lastZIndex;

        friend class BitmapCanvasWidgetBase;
};



#endif // BITMAPCANVASPARENTBASE_H

