#include "BitmapCanvasSystemSettings.h"

#include "BitmapCanvasWidgetBase.h"
#include "../wxVariantHelper.h"

Int2wxVariantIntHash BitmapCanvasSystemSettings::m_properties;
int BitmapCanvasSystemSettings::m_currProfile = 1;

void BitmapCanvasSystemSettings::Init(int prof)
{
    m_currProfile = 1;
    if(prof != -1) m_currProfile = prof;

    wxVariantIntHash& profile = m_properties[m_currProfile];

    profile[BCBS_BACKGROUND_COLOR]              = wxVariantHelper::ToVariant(wxColor(255,255,255,255));
    profile[BCBS_DISABLED_BACKGROUND_COLOR]     = wxVariantHelper::ToVariant(wxColor(140,140,140,255));
    profile[BCBS_DISABLED_TEXT_COLOR]           = wxVariantHelper::ToVariant(wxColor(190,190,190,255));
    profile[BCBS_FOREGROUND_COLOR]              = wxVariantHelper::ToVariant(wxColor(0,0,0,255));
    profile[BCBS_SELECTION_COLOR]               = wxVariantHelper::ToVariant(wxColor(0,102,153,255));
    profile[BCBS_BORDER_COLOR]                  = wxVariantHelper::ToVariant(wxColor(0,0,0,255));
    profile[BCBS_BORDER_HIGHLIGHT_COLOR]        = wxVariantHelper::ToVariant(wxColor(255,255,255,255));
    profile[BCBS_BORDER_WIDTH]                  = 1.0;
    profile[BCBS_BORDER_RADIUS]                 = 2.0;
    profile[BCBS_PADDDING]                      = 2.0;
    profile[BCBS_FONT_FILE]                     = _T("arial.ttf");
    profile[BCBS_TEXT_HEIGHT]                   = 12.0;
    profile[BCBS_TEXT_WEIGHT]                   = 0.0;
    profile[BCBS_TEXT_COLOR]                    = wxVariantHelper::ToVariant(wxColor(0,0,0,255));
    profile[BCBS_TEXT_GRAVEN_COLOR]             = wxVariantHelper::ToVariant(wxColor(255,255,255,255));
    profile[BCBS_TEXT_SELECTION_COLOR]          = wxVariantHelper::ToVariant(wxColor(255,255,255,255));
    profile[BCBS_VEMS_COLOR]                    = wxVariantHelper::ToVariant(wxColor(171,8,8,255));
    profile[BCBS_SHADOW_COLOR]                  = wxVariantHelper::ToVariant(wxColor(0,0,0,180));


    profile[BCBS_DYNAMIC_CONTENT_COLOR]         = profile[BCBS_VEMS_COLOR];

}

void BitmapCanvasSystemSettings::SetProfileId(int id)
{
    m_currProfile = id;
}

#define PROPCHECK(retval) if(m_properties.find(m_currProfile) == m_properties.end() || m_properties[m_currProfile].find(key) == m_properties[m_currProfile].end() ) return retval;

wxString BitmapCanvasSystemSettings::GetString(int key)
{
    PROPCHECK(wxEmptyString)

    return m_properties[m_currProfile][key].GetString();
}

wxPoint BitmapCanvasSystemSettings::GetPoint(int key)
{
    PROPCHECK(wxDefaultPosition)

    return wxVariantHelper::GetPointEx(m_properties[m_currProfile][key]);
}

wxRealPoint BitmapCanvasSystemSettings::GetRealPoint(int key)
{
    PROPCHECK(wxRealPoint(0,0))

    return wxVariantHelper::GetRealPointEx(m_properties[m_currProfile][key]);
}

wxRealSize BitmapCanvasSystemSettings::GetRealSize(int key)
{
    PROPCHECK(wxDefaultRealSize)

    return wxVariantHelper::GetRealSize(m_properties[m_currProfile][key]);
}

wxSize BitmapCanvasSystemSettings::GetSize(int key)
{
    PROPCHECK(wxDefaultSize)

    return wxVariantHelper::GetSizeEx(m_properties[m_currProfile][key]);
}

wxColor BitmapCanvasSystemSettings::GetColor(int key)
{
    PROPCHECK(wxNullColour)

    return wxVariantHelper::GetColor(m_properties[m_currProfile][key]);
}

double BitmapCanvasSystemSettings::GetDouble(int key)
{
    PROPCHECK(-1.0)

    return m_properties[m_currProfile][key].GetDouble();
}

long BitmapCanvasSystemSettings::GetLong(int key)
{
    PROPCHECK(-1)

    return m_properties[m_currProfile][key].GetLong();
}

bool BitmapCanvasSystemSettings::GetBool(int key)
{
    PROPCHECK(false)

    return m_properties[m_currProfile][key].GetBool();
}

