#ifndef BCBUTTON_H
#define BCBUTTON_H

#include "BitmapCanvasWidgetBase.h"

enum bcButtonStyles {
    bcBUTTON_ALIGN_LEFT                     = 0x0000,
    bcBUTTON_ALIGN_TOP                      = 0x0000,
    bcBUTTON_ALIGN_CENTER                   = 0x0001,
    bcBUTTON_ALIGN_RIGHT                    = 0x0002,
    bcBUTTON_ALIGN_MIDDLE                   = 0x0004,
    bcBUTTON_ALIGN_BOTTOM                   = 0x0008,
    bcBUTTON_AUTOSIZE                       = 0x0010,
    bcBUTTON_NO_BORDER                      = 0x0020,
    bcBUTTON_TOGGLE                         = 0x0040,
    bcBUTTON_BORDER_HIGHLIGHT               = 0x0080,
    bcBUTTON_NO_DRAW_TEXT_WHEN_BITMAP_LABEL = 0x0100,
    bcBUTTON_BITMAP_LABEL_ABOVE_TEXT_LABEL  = 0x0200,
    bcBUTTON_TEXT_GRAVEN                    = 0x0400,
    bcBUTTON_TEXT_PUSH_AFTER_BITMAP_BYX     = 0x0800,
    bcBUTTON_HOVER_OUTER_GLOW               = 0x1000,

    bcBUTTON_ALIGN_CENTER_VERTICAL          = bcBUTTON_ALIGN_MIDDLE,
    bcBUTTON_ALIGN_CENTER_HORIZONTAL        = bcBUTTON_ALIGN_CENTER,

    bcBUTTON_DEFAULT_STYLE                  = bcBUTTON_ALIGN_CENTER_HORIZONTAL |
                                              bcBUTTON_ALIGN_CENTER_VERTICAL |
                                              bcBUTTON_TEXT_GRAVEN,

    bcBUTTON_DEFAULT_COMPOSITE_STYLE        = bcBUTTON_ALIGN_CENTER_VERTICAL |
                                              bcBUTTON_TEXT_GRAVEN |
                                              bcBUTTON_TEXT_PUSH_AFTER_BITMAP_BYX |
                                              bcBUTTON_AUTOSIZE
};

enum bcButtonImageStates{
    bcBUTTON_IMAGE_NORMAL = 0,
    bcBUTTON_IMAGE_HOVER,
    bcBUTTON_IMAGE_PRESSED,
    bcBUTTON_IMAGE_DISABLED
};

WX_DECLARE_HASH_MAP( int, wxImage, wxIntegerHash, wxIntegerEqual, Int2wxImageHash );

class bcButton: public BitmapCanvasWidgetBase
{
    DECLARE_DYNAMIC_CLASS(bcButton);

    public:
        bcButton() { m_parent = NULL; }
        bcButton(BitmapCanvasParentBase* parent, int id, const wxString& label = wxEmptyString, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = bcBUTTON_DEFAULT_STYLE);
        bcButton(BitmapCanvasParentBase* parent, int id, const wxImage& label, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = bcBUTTON_DEFAULT_STYLE);
        bcButton(BitmapCanvasParentBase* parent, int id, const wxString& label, const wxImage& imglabel, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = bcBUTTON_DEFAULT_COMPOSITE_STYLE);

        virtual ~bcButton();

        virtual void DrawWidget();
        virtual void Create(BitmapCanvasParentBase* parent, wxVariantIntHash& properties);

        void SetLabel(const wxString& label);
        wxString GetLabel() { return m_label; };

        void SetLabel(const wxBitmap& label, bcButtonImageStates state = bcBUTTON_IMAGE_NORMAL, const wxPoint& pos = wxDefaultPosition);
        void SetImgLabel(const wxImage& label, bcButtonImageStates state = bcBUTTON_IMAGE_NORMAL, const wxPoint& pos = wxDefaultPosition);

        void SetGroupId(long id);
        long GetGroupId() { return m_groupId; };

        void SetBitmapPosition(const wxPoint& pos);
        wxPoint GetBitmapPosition() { return m_imgPos; }

        void SetToggleValue(bool state);
        bool GetToggleValue();

        void SetTextOffset(const wxRealSize& offset) { SetTextOffset(offset.w,offset.h); };
        void SetTextOffset(double x, double y);
        wxRealSize GetTextOffset() { return m_textOffset; };

        wxRect GetTextBox();

        enum bcButtonProperties {
            BCBP_GRADIENT_LIGHTNESS_DIFF,
            BCBP_GLOW_RADIUS,
            BCBP_GLOW_COLOR
        };

        wxRealSize GetFitSize();

        wxString GetState();

    protected:

        void OnLeftDown(wxMouseEvent& evt);
        void OnLeftUp(wxMouseEvent& evt);
        void OnEnter(wxMouseEvent& evt);
        void OnLeave(wxMouseEvent& evt);

        void OnSetFocus(wxFocusEvent& evt);
        void OnKillFocus(wxFocusEvent& evt);

        void OnKeyDown(wxKeyEvent& evt);

        bool IsAutoSize();
        bool IsBitmap();
        bool IsText();

        bool IsStateChanged();

        void CheckGroupButtons(bool isCheckingSelf);

    private:

        void DoPress(bool isup);

        enum bcButtonStates {
            BCB_NORMAL      = 0x00,
            BCB_HIGHLIGHTED = 0x01,
            BCB_PRESSED     = 0x02,
            BCB_TOGGLED     = 0x04,
            BCB_CLICK_START = 0x08
        };

        wxString    m_label;
        int         m_state;
        int         m_prev_state;
        wxPoint     m_imgPos;
        wxRealSize  m_textOffset;
        long        m_groupId;

        Int2wxImageHash m_imgLabelStates;

        DECLARE_EVENT_TABLE();

};

#endif // BCBUTTON_H

