#include "bcCheckBox.h"

#include "../agg/wxAgg2.h"

#include "../DirHelper.h"
#include "../logger.h"

#include "../DrawHelper.h"
#include "BitmapCanvasSystemSettings.h"

BEGIN_EVENT_TABLE(bcCheckBox, BitmapCanvasWidgetBase)

    EVT_LEFT_DOWN(bcCheckBox::OnLeftDown)
    EVT_LEFT_UP(bcCheckBox::OnLeftUp)

END_EVENT_TABLE()

IMPLEMENT_DYNAMIC_CLASS(bcCheckBox, BitmapCanvasWidgetBase);

bcCheckBox::bcCheckBox(BitmapCanvasParentBase* parent, int id, const wxString& text, const wxPoint& pos, const wxSize& size, long style)
    :BitmapCanvasWidgetBase(parent,id,pos,size,bcDefaultZindex,style)
{
    //ctor
    if( size == wxDefaultSize ) m_properties[BCWB_SIZE] << wxRealSize(100,18);

    m_state = BCCB_NORMAL;
    m_label = text;
    m_style = style;

    DrawWidget();

}

bcCheckBox::~bcCheckBox()
{
    //dtor
}
/// TODO: rethink and implement
void bcCheckBox::Create(BitmapCanvasParentBase* parent, wxVariantIntHash& properties)
{

}

void bcCheckBox::OnLeftDown(wxMouseEvent& evt)
{
    m_state |= BCCB_HOVER;

    Refresh();
}

void bcCheckBox::OnLeftUp(wxMouseEvent& evt)
{
    m_state &= ~BCCB_HOVER;

    m_state ^= BCCB_CHECKED;

    Refresh();
}

void bcCheckBox::SetValue(bool isChecked)
{
    bool oval = GetValue();

    if(oval == isChecked) return;

    m_state ^= BCCB_CHECKED;

    Refresh();
}

bool bcCheckBox::GetValue()
{
    return m_state & BCCB_CHECKED ? true : false;
}

void bcCheckBox::PaintCheckMarker(wxAgg2& aggCanvas, const wxRealRect& box)
{
    wxRealRect cb = box;
    cb.Deflate(2);

    wxAggPen* pen = aggCanvas.getPen();
    wxAggBrush* brush = aggCanvas.getBrush();

    pen->SetStyle(wxAGGPEN_TRANSPARENT);
    brush->SetStyle(wxAGGBRUSH_GRADIENT);

    wxColor checkcolor  = GetSysColor( BCBS_DYNAMIC_CONTENT_COLOR );
    wxColor checkcolor2 = DrawHelper::ModLuminance( checkcolor, -0.2 );

    brush->SetColor(checkcolor);
    brush->SetEndColor(checkcolor2);

    double gradori = (m_style & BCCBS_INVERTED3D) ? 90 : 270;

    brush->SetGradientOrientation( m_state & BCCB_HOVER ? gradori : gradori+180);

    if(m_style & BCCBS_CHECK) {

        double ref_h1 = 0.44664;
        double ref_h2 = 0.612648;
        double ref_h3 = 0.743083;
        double ref_w1 = 0.237886;
        double ref_w2 = 0.431718;

        RealPointArray points;

        points.Add( wxRealPoint( cb.x               , cb.y + cb.h*ref_h2 ) );
        points.Add( wxRealPoint( cb.x + cb.w*ref_w2 , cb.GetBottom()     ) );
        points.Add( wxRealPoint( cb.GetRight()      , cb.y+cb.h*ref_h1   ) );
        points.Add( wxRealPoint( cb.GetRight()      , cb.GetTop()        ) );
        points.Add( wxRealPoint( cb.x + cb.w*ref_w2 , cb.y + cb.h*ref_h3 ) );
        points.Add( wxRealPoint( cb.x + cb.w*ref_w1 , cb.y + cb.h*ref_h1 ) );

        aggCanvas.drawPolygon(points, true);

    } else if(m_style & BCCBS_RECT) {

        wxRealRect cbr = cb.GetInnerBox();
        cbr.w -= 1;
        cbr.h -= 1;
        aggCanvas.drawRectangle( cbr );

    } else if(m_style & BCCBS_CUSTOM) {

        aggCanvas.drawPolygon(m_customCheckPoints);

    } else if(m_style & BCCBS_CROSS || m_style & BCCBS_CROSS2) {

        double ref0 = 0.0;
        double ref1 = 0.2;
        double ref2 = 0.3;
        double ref3 = 0.5;
        double ref4 = 0.7;
        double ref5 = 0.8;
        double ref6 = 1.0;

        wxRealRect cbc = cb.GetInnerBox();
        cbc.w -= 1;
        cbc.h -= 1;

        RealPointArray points;

        if(m_style & BCCBS_CROSS2) points.Add( wxRealPoint( cbc.x+cbc.w*ref0, cbc.y+cbc.h*ref0 ) ); // 1
        points.Add( wxRealPoint( cbc.x+cbc.w*ref1, cbc.y+cbc.h*ref0 ) ); // 2
        points.Add( wxRealPoint( cbc.x+cbc.w*ref3, cbc.y+cbc.h*ref2 ) ); // 3
        points.Add( wxRealPoint( cbc.x+cbc.w*ref5, cbc.y+cbc.h*ref0 ) ); // 4
        if(m_style & BCCBS_CROSS2) points.Add( wxRealPoint( cbc.x+cbc.w*ref6, cbc.y+cbc.h*ref0 ) ); // 5
        points.Add( wxRealPoint( cbc.x+cbc.w*ref6, cbc.y+cbc.h*ref1 ) ); // 6
        points.Add( wxRealPoint( cbc.x+cbc.w*ref4, cbc.y+cbc.h*ref3 ) ); // 7
        points.Add( wxRealPoint( cbc.x+cbc.w*ref6, cbc.y+cbc.h*ref5 ) ); // 8
        if(m_style & BCCBS_CROSS2) points.Add( wxRealPoint( cbc.x+cbc.w*ref6, cbc.y+cbc.h*ref6 ) ); // 9
        points.Add( wxRealPoint( cbc.x+cbc.w*ref5, cbc.y+cbc.h*ref6 ) ); // 10
        points.Add( wxRealPoint( cbc.x+cbc.w*ref3, cbc.y+cbc.h*ref4 ) ); // 11
        points.Add( wxRealPoint( cbc.x+cbc.w*ref1, cbc.y+cbc.h*ref6 ) ); // 12
        if(m_style & BCCBS_CROSS2) points.Add( wxRealPoint( cbc.x+cbc.w*ref0, cbc.y+cbc.h*ref6 ) ); // 13
        points.Add( wxRealPoint( cbc.x+cbc.w*ref0, cbc.y+cbc.h*ref5 ) ); // 14
        points.Add( wxRealPoint( cbc.x+cbc.w*ref2, cbc.y+cbc.h*ref3 ) ); // 15
        points.Add( wxRealPoint( cbc.x+cbc.w*ref0, cbc.y+cbc.h*ref1 ) ); // 16

        aggCanvas.drawPolygon(points,true);

    }

}

void bcCheckBox::SetCustomCheckPoints(const RealPointArray& points)
{
    m_customCheckPoints = points;
    Refresh();
}

void bcCheckBox::DrawWidget()
{

    wxRealSize size = GetSize();

    wxAgg2 aggCanvas(size.Ceil());

    wxAggPen* pen = aggCanvas.getPen();
    wxAggBrush* brush = aggCanvas.getBrush();

    double padding = 2;
    double box_border = 1;

    wxRealRect box( padding + box_border/2, padding + box_border/2, 0, size.h - 2*padding - box_border );
    box.w = box.h;

    pen->SetColor( GetSysColor(BCBS_FOREGROUND_COLOR) );
    pen->SetWidth( box_border );
    brush->SetStyle(wxAGGBRUSH_GRADIENT);

    double bg_grad_diff = -0.3;
    double gradori = (m_style & BCCBS_INVERTED3D) ? 270 : 90;

    wxColor bgcolor  = GetSysColor( BCBS_BACKGROUND_COLOR );
    wxColor bgcolor2 = DrawHelper::ModLuminance( bgcolor, bg_grad_diff  );

    brush->SetColor(bgcolor);
    brush->SetEndColor(bgcolor2);
    brush->SetGradientOrientation( m_state & BCCB_HOVER ? gradori : gradori+180);


    aggCanvas.drawRectangle( box );

    if(m_state & BCCB_CHECKED) {

        PaintCheckMarker(aggCanvas, box);

    }

    pen->SetStyle( wxAGGPEN_SOLID );


    wxString fontfile = DirHelper::GetPath(dFONTS) + GetSysString(BCBS_FONT_FILE);
    if(aggCanvas.loadTTFont(fontfile, GetSysDouble(BCBS_TEXT_HEIGHT), 0 ) == false) return;

    wxRealRect textbox = aggCanvas.getTTFTextBound(m_label);

    wxRealPoint textpos(box.GetRight() + 2*padding, size.h - (size.h-textbox.h)/2 - padding);

    if(m_style & BCCBS_TEXT_GRAVEN) {
        pen->SetColor( GetSysColor(BCBS_TEXT_GRAVEN_COLOR) );
        aggCanvas.drawTTFText(m_label, textpos.x+1,textpos.y+1 );
    }

    pen->SetColor( GetSysColor(BCBS_TEXT_COLOR) );
    aggCanvas.drawTTFText(m_label, textpos.x,textpos.y );

    render(aggCanvas);
}
