#include "bcSlider.h"

#include "BitmapCanvasSystemSettings.h"

#include "../agg/wxAgg2.h"

#include "../DirHelper.h"
#include "../logger.h"

BEGIN_EVENT_TABLE(bcSlider, BitmapCanvasWidgetBase)

    EVT_LEFT_DOWN(bcSlider::OnLeftDown)

    EVT_ENTER_WINDOW(bcSlider::OnEnter)
    EVT_LEAVE_WINDOW(bcSlider::OnLeave)

    EVT_MOTION(bcSlider::OnMouseMove)

END_EVENT_TABLE()


IMPLEMENT_DYNAMIC_CLASS(bcSlider, BitmapCanvasWidgetBase);

bcSlider::bcSlider(BitmapCanvasParentBase* parent, int id, int minValue, int maxValue, int currValue, int style, const wxPoint& pos, const wxSize& size)
    :BitmapCanvasWidgetBase(parent,id,pos,size)
{
    //ctor
    if( size == wxDefaultSize ) m_properties[BCWB_SIZE] << wxRealSize(100,100);

    m_properties[BCS_MIN_VALUE] = (long)minValue;
    m_properties[BCS_MAX_VALUE] = (long)maxValue;
    m_properties[BCS_CURRENT_VALUE] = (long)currValue;

    m_hlValue = wxNullVariant;

    DrawWidget();
}
/// TODO: rethink and implement
void bcSlider::Create(BitmapCanvasParentBase* parent, wxVariantIntHash& properties)
{

}

//bcSlider::bcSlider(BitmapCanvasParentBase* parent, wxVariantIntHash& properties)
//{
//    CreateBcWidget(parent,properties);
//    DrawWidget();
//}

bcSlider::~bcSlider()
{
    //dtor
}

void bcSlider::OnLeave(wxMouseEvent& evt)
{
    m_hlValue = wxNullVariant;
    Refresh();
}

void bcSlider::OnEnter(wxMouseEvent& evt)
{
    SetCursor( wxCURSOR_HAND );
}

int bcSlider::GetValue(const wxPoint& mousePos)
{
    wxRealSize size = GetSize();
    size.DecBy(1);

    int min = GetPropLong(BCS_MIN_VALUE);
    int max = GetPropLong(BCS_MAX_VALUE);
    //int cur = GetPropLong(BCS_CURRENT_VALUE);

    double padding = GetPropDouble(BCBS_PADDDING,3);

    double step_count = fabs((double) max-min+1 );

    double slider_height = size.h - 2*padding;

    int v = (int)(( ((double)mousePos.y-padding) / slider_height) * step_count);

    v = max - v;

    return v;
}

void bcSlider::OnMouseMove(wxMouseEvent& evt)
{
    //SetCursor( wxCURSOR_HAND );
    long hl_val = GetValue(evt.GetPosition());
    m_hlValue = hl_val;
    Refresh();
}

void bcSlider::OnLeftDown(wxMouseEvent& evt)
{
    SetValue( GetValue(evt.GetPosition()) );
}

void bcSlider::ChangeValue(int val)
{
    m_properties[BCS_CURRENT_VALUE] = (long)val;
    Refresh();
}

void bcSlider::SetValue(int val)
{
    int min = GetPropLong(BCS_MIN_VALUE);
    int max = GetPropLong(BCS_MAX_VALUE);

    if( val < min ) val = min;
    if( val > max ) val = max;

    ChangeValue(val);

    wxScrollEvent evt( wxEVT_SCROLL_CHANGED, GetId() );
    evt.SetPosition( val );
    DropEvent(evt);
}

void bcSlider::SetMinValue(int val)
{
    m_properties[BCS_MIN_VALUE] = (long)val;
    Refresh();
}

void bcSlider::SetMaxValue(int val)
{
    m_properties[BCS_MAX_VALUE] = (long)val;
    Refresh();
}

void bcSlider::DrawWidget()
{
    wxAgg2 aggCanvas( this );

    if(!aggCanvas.isOk()) return;

    aggCanvas.getBrush()->SetStyle( wxAGGBRUSH_SOLID );

    aggCanvas.setPen( GetSysColor(BCBS_BORDER_COLOR));
    aggCanvas.setBrush( GetSysColor(BCBS_BACKGROUND_COLOR)  );

    wxRealSize size = GetSize();
    size.DecBy(1);

    wxAggPen* aggpen = aggCanvas.getPen();
    aggpen->SetWidth(GetSysDouble(BCBS_BORDER_WIDTH));

    aggCanvas.drawRoundedRectangle(0.5, 0.5, size.w, size.h, 3 );

    int min = GetPropLong(BCS_MIN_VALUE);
    int max = GetPropLong(BCS_MAX_VALUE);
    int cur = GetPropLong(BCS_CURRENT_VALUE);

    double padding = GetPropDouble(BCBS_PADDDING,3);
    double cellSpace = GetPropDouble(BCS_CELL_SPACE,1);

    double cell_minWidth = size.w * GetPropDouble(BCS_CELL_MIN_WIDTH_REF,0.2);
    double cell_widthStep = (size.w - cell_minWidth - 2*padding) / fabs((double) max-min+1 );

    double cellDist = (size.h - (2 * padding)) / fabs((double) max-min+1 );

    wxColor inactiveColor = GetPropColor(BCS_INACTIVE_COLOR,wxColor(0,100,0,220));
    wxColor activeColor = GetPropColor(BCS_ACTIVE_COLOR,wxColor(210,0,0,220));

    aggpen->SetStyle(wxAGGPEN_TRANSPARENT);

    for( int a = max, c=0; a >= min; a--,c++ ) {

        if(a == cur)
            aggCanvas.setBrush(activeColor);
        else
            aggCanvas.setBrush(inactiveColor);

        if( m_hlValue != wxNullVariant && a == m_hlValue.GetLong() ) aggCanvas.setBrush(activeColor);

        aggCanvas.drawRectangle( padding, padding + c * cellDist, size.w-2*padding - c * cell_widthStep , cellDist-cellSpace );
    }

    wxString valstr = wxString::Format(_T("%i"),cur);

    wxString fontfile = DirHelper::GetPath(dFONTS) + GetSysString(BCBS_FONT_FILE);

    aggpen->SetStyle(wxAGGPEN_SOLID);
    aggpen->SetColor( GetSysColor(BCBS_TEXT_COLOR) );
    aggCanvas.loadTTFont(fontfile, GetSysDouble(BCBS_TEXT_HEIGHT), 0);
    wxRealRect valBbox = aggCanvas.getTTFTextBound(valstr);
    aggCanvas.drawTTFText( size.w - padding - valBbox.w, size.h - padding, valstr);


    render(aggCanvas);

}
