#ifndef BCSTATICLINE_H
#define BCSTATICLINE_H

#include "BitmapCanvasWidgetBase.h"

enum bcStaticLineStyles {
    bcSL_HORIZONTAL     = 0x001,
    bcSL_VERTICAL       = 0x002,
    bcSL_ALIGN_LEFT     = 0x004,
    bcSL_ALIGN_BOTTOM   = bcSL_ALIGN_LEFT,
    bcSL_ALIGN_RIGHT    = 0x008,
    bcSL_ALIGN_TOP      = bcSL_ALIGN_RIGHT,
    bcSL_ALIGN_CENTER   = 0x010,
    bcSL_TEXT_GRAVEN    = 0x020,
    bcSL_TEXT_LABEL_FIRST   = 0x040,

    bcSL_DEFAULT_STYLE  = bcSL_HORIZONTAL | bcSL_ALIGN_CENTER | bcSL_TEXT_GRAVEN | bcSL_TEXT_LABEL_FIRST
};



#define USE_LABEL 1 /// 2 not implemented yet

class bcStaticLine: public BitmapCanvasWidgetBase
{
    DECLARE_DYNAMIC_CLASS(bcStaticLine);

    public:
        bcStaticLine() {}
        bcStaticLine(BitmapCanvasParentBase* parent, int id = wxID_ANY, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = bcSL_HORIZONTAL);
        virtual ~bcStaticLine();

        virtual void DrawWidget();
        virtual void Create(BitmapCanvasParentBase* parent, wxVariantIntHash& properties);
        virtual bool Focusable();

#if USE_LABEL
        void SetLabel(const wxString& label);
        wxString GetLabel() const;

        /// note: if labelpos between 0 and 1 the labelpos is a ratio, if larger then 1 is positon in pixels
        void SetLabelPos(double pos_or_ratio);

        void SetLabelPadding(double padding) { m_labelPadding = padding; Refresh(); };
        double GetLabelPadding() const { return m_labelPadding; };
#endif
#if USE_LABEL == 2

        void SetImgLabel(const wxImage& label);
        const wxImage& GetImgLabel() const;

        enum bcStaticLineLabelOris {
            ORI_AUTO = -1,
            ORI_0    = 0,
            ORI_90   = 1,
            ORI_180  = 2,
            ORI_270  = 3
        };

        void SetLabelOrinentation(int ori = ORI_AUTO);

#endif

    protected:

        wxRect GetTextBox();

    private:

        double      m_labelPos;
        int         m_labelRotate;
        wxString    m_label;
        wxImage     m_imgLabel;
        double      m_labelOri;
        double      m_labelPadding;

};

#endif // BCSTATICLINE_H
