#include "bcTextCtrl.h"

#include "../agg/wxAgg2.h"

#include "../DirHelper.h"
#include "../logger.h"
#include "../wxHSLColor.h"

#include "BitmapCanvasParentBase.h"
#include "BitmapCanvasSystemSettings.h"

BEGIN_EVENT_TABLE(bcTextCtrl,BitmapCanvasWidgetBase)

    EVT_ENTER_WINDOW(bcTextCtrl::OnEnter)
    EVT_LEAVE_WINDOW(bcTextCtrl::OnLeave)

    EVT_SET_FOCUS(bcTextCtrl::OnSetFocus)
    EVT_KILL_FOCUS(bcTextCtrl::OnKillFocus)

    EVT_TIMER(idTimer, bcTextCtrl::OnTimer)
    EVT_TIMER(idProcTimer, bcTextCtrl::OnProcessocTimer)

    EVT_LEFT_DOWN(bcTextCtrl::OnLeftDown)
    EVT_LEFT_UP(bcTextCtrl::OnLeftUp)
    EVT_MOTION(bcTextCtrl::OnMouseMove)


    //EVT_KEY_DOWN(bcTextCtrl::OnKeyDown)

END_EVENT_TABLE()

bcTextCtrl::bcTextCtrl(BitmapCanvasParentBase* parent, int id, const wxString& text, const wxPoint& pos, const wxSize& size, long style, const wxValidator& validator)
    :BitmapCanvasWidgetBase(parent,id,pos,size,style)
    ,m_cursorTimer(this, idTimer)
    ,m_processorChechker(this, idProcTimer)
{
    //ctor
    if( size == wxDefaultSize ) m_properties[BCWB_SIZE] << wxRealSize(80,20);
    m_isCursor = false;


    m_text = text;

    m_processor = new wxTextCtrl( GetWindowParent(), wxID_ANY, text, wxDefaultPosition, wxSize(1,1), style, validator );
    m_processor->Hide();

    m_processor->Connect(wxEVT_COMMAND_TEXT_UPDATED, wxCommandEventHandler(bcTextCtrl::OnProcessorUpdated), NULL, this );
    m_processor->Connect(wxEVT_SET_FOCUS, wxFocusEventHandler(bcTextCtrl::OnProcessorSetFocus), NULL, this );
    m_processor->Connect(wxEVT_KILL_FOCUS, wxFocusEventHandler(bcTextCtrl::OnProcessorKillFocus), NULL, this );
    //m_processor->Connect(wxEVT_COMMAND_TEXT_ENTER, wxCommandEventHandler(bcTextCtrl::OnProcessorTextEnter), NULL, this );
    //m_processor->Connect(wxEVT_KEY_DOWN, wxKeyEventHandler(bcTextCtrl::OnKeyDown), NULL, this );

    m_cursorFlashTime = 500;




    m_startMouseSelPos = -1;

    DrawWidget();

    m_processorChechker.Start(10);

    m_procInsertIdx = m_processor->GetInsertionPoint();
    long tmp1,tmp2;
    m_processor->GetSelection(&tmp1,&tmp2);
    m_procSelFromIdx = tmp1;
    m_procSelToIdx = tmp2;

    m_procState = CHG_NO;


}

bcTextCtrl::~bcTextCtrl()
{
    //dtor
    m_processorChechker.Stop();
    m_cursorTimer.Stop();
}
/// TODO: rethink and implement
void bcTextCtrl::Create(BitmapCanvasParentBase* parent, wxVariantIntHash& properties)
{

}

void bcTextCtrl::OnProcessorTextEnter(wxCommandEvent& evt)
{
    AddPendingEvent(evt);
}

void bcTextCtrl::OnProcessocTimer(wxTimerEvent& evt)
{
    int  procInsertIdx = m_processor->GetInsertionPoint();
    long procSelToIdx;
    long procSelFromIdx;

    m_processor->GetSelection(&procSelFromIdx,&procSelToIdx);

    if(m_procInsertIdx != procInsertIdx) m_procState |= CHG_INSPT;
    if(m_procSelFromIdx != procSelFromIdx) m_procState |= CHG_SELFROM;
    if(m_procSelToIdx != procSelToIdx) m_procState |= CHG_SELTO;

    if(m_procState != CHG_NO) {
        m_lastProcState = m_procState;
        m_procInsertIdx = procInsertIdx;
        m_procSelFromIdx = procSelFromIdx;
        m_procSelToIdx = procSelToIdx;
        m_isCursor = true;
        m_cursorTimer.Start(m_cursorFlashTime);
        Refresh();
        m_procState = CHG_NO;
        //LOG
    }
}

void bcTextCtrl::OnProcessorSetFocus(wxFocusEvent& evt)
{
    if(!IsFocused()) {
        SetFocus();
        //Logger::logf(LOG_MSG,_T("bcTextCtrl::OnProcessorSetFocus error!!!!!!!!"));
        //m_processor->GetParent()->SetFocus();
    }
}

void bcTextCtrl::OnProcessorKillFocus(wxFocusEvent& evt)
{
    if(IsFocused()) {
        m_parent->SetWidgetFocus(NULL);
    }
}

void bcTextCtrl::OnLeftUp(wxMouseEvent& evt)
{
    m_processor->ProcessEvent(evt);
    evt.Skip();
    return;
}

void bcTextCtrl::OnLeftDown(wxMouseEvent& evt)
{
    int curmpos = XYToPosition(evt.GetPosition());

    m_processor->ProcessEvent(evt);
    evt.Skip();
    return;

    if(curmpos >= 0) {

        //int insp = m_processor->GetInsertionPoint();

        if(evt.ShiftDown()) {
//            long from = wxMin(insp,curmpos);
//            long to = wxMax(insp,curmpos);
//            long ori_from,ori_to;
//            m_processor->GetSelection(&ori_from,&ori_to);
//
//            Logger::logf(LOG_MSG,_T("i=%i,of=%i,ot=%i,f=%i,t=%i"),insp,ori_from,ori_to,from,to);
//
//            if(curmpos > ori_to) {
//                from = ori_to;
//                to = curmpos;
//            } else if(curmpos < ori_from) {
//                from = curmpos;
//                to = ori_to;
//            }

            long from = wxMin(m_startMouseSelPos,curmpos);
            long to = wxMax(m_startMouseSelPos,curmpos);

            m_processor->SetSelection(from,to);


        } else {
            m_startMouseSelPos = curmpos;
            m_processor->SetInsertionPoint(curmpos);

        }

    }
}

void bcTextCtrl::OnMouseMove(wxMouseEvent& evt)
{
    m_processor->ProcessEvent(evt);
    evt.Skip();
    return;
}

void bcTextCtrl::OnKeyDown(wxKeyEvent& evt)
{
    evt.Skip();
    LOG
    Refresh();
}

void bcTextCtrl::OnEnter(wxMouseEvent& evt)
{
    SetCursor( wxCURSOR_IBEAM );
}

void bcTextCtrl::OnLeave(wxMouseEvent& evt)
{
    SetCursor( wxNullCursor );
}

void bcTextCtrl::OnTimer(wxTimerEvent& evt)
{
    m_isCursor = !m_isCursor;
    Refresh();
}

void bcTextCtrl::OnProcessorUpdated(wxCommandEvent& evt)
{
    if(!IsFocused()) return;

    m_text = m_processor->GetValue();

    m_cursorTimer.Start(m_cursorFlashTime);
    m_isCursor = true;

    Refresh();
}

void bcTextCtrl::OnSetFocus(wxFocusEvent& evt)
{
    m_processor->SetFocus();
    m_cursorTimer.Start(m_cursorFlashTime);

    m_state |= BCTC_HIGHLIGHTED;
    Refresh();
}

void bcTextCtrl::OnKillFocus(wxFocusEvent& evt)
{
    m_state = BCTC_NORMAL;
    m_cursorTimer.Stop();
    m_isCursor = false;
    Refresh();
}

void bcTextCtrl::SetValue(const wxString& text)
{
    m_text = text;
    Refresh();
}

long bcTextCtrl::XYToPosition(int x, int y)
{
    double font_size = GetSysDouble(BCBS_TEXT_HEIGHT);

    wxAgg2 aggCanvas(2,2);

    wxString fontfile = DirHelper::GetPath(dFONTS) + GetSysString(BCBS_FONT_FILE);
    if(aggCanvas.loadTTFont(fontfile, font_size, 0 ) == false) return -1;

    double padding = GetSysDouble(BCBS_PADDDING);
    double border_width = GetSysDouble(BCBS_BORDER_WIDTH);
    int shadow_width = 3;

    wxRealPoint textPos(padding + border_width + shadow_width, font_size + shadow_width );

    wxRealRectArray charRects;

    wxRealRect bbox = aggCanvas.getTTFTextBound(m_text, charRects, 0, textPos.x, textPos.y);

    if(x > bbox.GetRight()) return m_text.Len();

    if(x < bbox.GetLeft()) return 0;

    for(size_t a=0; a<charRects.GetCount();a++){
        wxRealRect& char_rect = charRects[a];
        wxString c = m_text[a];

        double x_center = char_rect.x + char_rect.w / 2;

        if(x >= char_rect.x && x <= x_center)
            return a;
        else if(x <= char_rect.GetRight() && x >= x_center)
            return a+1;
        else
            continue;
    }

    return -1;
}

void bcTextCtrl::DrawWidget()
{

    wxAgg2 aggCanvas(this);
    bool isCursor = m_isCursor;
    bool isSelected = false;

    wxRealSize size = GetSize();

    double padding = GetSysDouble(BCBS_PADDDING);
    double border_width = GetSysDouble(BCBS_BORDER_WIDTH);
    double border_radius = GetSysDouble(BCBS_BORDER_RADIUS);
    int shadow_width = 0;

    if(!aggCanvas.isOk()) return;

    wxColor fgcolor = GetSysColor(BCBS_FOREGROUND_COLOR);

    wxAggBrush* brush = aggCanvas.getBrush();
    brush->SetStyle( wxAGGBRUSH_GRADIENT );

    wxColor bgcolor = GetSysColor(BCBS_BACKGROUND_COLOR);
    wxHSLColor bghsl(bgcolor);
    bghsl.OffsetLum(-0.3);
    brush->SetColor(bghsl.ToRGB());
    brush->SetEndColor(bgcolor);
    brush->SetGradientOrientation( 90 );


    wxAggPen * pen = aggCanvas.getPen();
    pen->SetColor( GetSysColor(BCBS_BORDER_COLOR) );
    pen->SetWidth( border_width );

    aggCanvas.drawRoundedRectangle( border_width/2, border_width/2, size.w-border_width, size.h-border_width, border_radius);

    wxRealRect srect = GetRRect();
    srect.x = 0;
    srect.y = 0;
    srect.Deflate(border_width);

    pen->SetStyle(wxAGGPEN_SOLID);

    //aggCanvas.drawInnerRectShadow(shadow_width,150,srect.GetInnerBox());

    double font_size = GetSysDouble(BCBS_TEXT_HEIGHT);

    wxString fontfile = DirHelper::GetPath(dFONTS) + GetSysString(BCBS_FONT_FILE);
    if(aggCanvas.loadTTFont(fontfile, font_size, 0 ) == false) return;

    //wxRealPoint textPos(padding + border_width + shadow_width, font_size + shadow_width );
    wxPoint textPos = wxRealPointEx(padding + border_width + shadow_width, (size.h-font_size)/2 + font_size*0.8 ).Ceil();

    wxRealRectArray charRects;

    wxRealRect bbox = aggCanvas.getTTFTextBound(m_text, charRects, 0, textPos.x, textPos.y);

    size_t curPosIdx = m_procInsertIdx;

    wxRealRect selectionRect(0,border_width+padding+1,0,size.h-border_width*2-padding*2);

//    Logger::logf(LOG_MSG,_T("m_text='")+m_text+_T("'"));
//    for(size_t a=0; a<charRects.GetCount();a++){
//        wxString c = m_text[a];
//        wxString s = charRects[a].GetAsString(1);
//        Logger::logf(LOG_MSG,c+_T(":")+s);
//    }

    if(m_procSelFromIdx != m_procSelToIdx && charRects.GetCount()) {

        aggCanvas.saveState();

        double selfrom_px, selto_px;
        isSelected = true;

        selfrom_px = (size_t)m_procSelFromIdx < charRects.GetCount() ? charRects[m_procSelFromIdx].GetLeft() : charRects[charRects.GetCount()-1].GetRight();
        selto_px = (size_t)m_procSelToIdx < charRects.GetCount() ? charRects[m_procSelToIdx].GetLeft() : charRects[charRects.GetCount()-1].GetRight();

        brush->SetStyle( wxAGGBRUSH_SOLID );
        brush->SetColor( GetSysColor(BCBS_SELECTION_COLOR) );
        pen->SetStyle( wxAGGPEN_TRANSPARENT );
        selectionRect.x = selfrom_px;
        selectionRect.w = selto_px-selfrom_px;
        aggCanvas.drawRectangle( selectionRect );

        aggCanvas.restoreState();

        curPosIdx = (m_procState & CHG_SELFROM || m_lastProcState & CHG_SELFROM) ? m_procSelFromIdx : m_procSelToIdx;

        wxString sel_1_str = m_text.Mid(0,m_procSelFromIdx);
        wxString sel_2_str = m_text.Mid(m_procSelFromIdx,m_procSelToIdx-m_procSelFromIdx);
        wxString sel_3_str = m_text.Mid(m_procSelToIdx);

        pen->SetStyle( wxAGGPEN_SOLID );

        if(sel_1_str.Len()) {
            pen->SetColor( fgcolor );
            aggCanvas.drawTTFText(textPos.x,textPos.y,sel_1_str,0);
        }
        if(sel_2_str.Len()) {
            pen->SetColor( GetSysColor(BCBS_TEXT_SELECTION_COLOR) );
            aggCanvas.drawTTFText(selfrom_px,textPos.y,sel_2_str,0);
        }
        if(sel_3_str.Len()) {
            pen->SetColor( fgcolor );
            aggCanvas.drawTTFText(selto_px,textPos.y,sel_3_str,0);
        }


    } else {
        pen->SetColor( fgcolor );
        aggCanvas.drawTTFText(textPos.x,textPos.y,m_text,0);
    }

    if(isCursor) {

        double cursorpos = textPos.x;

        if(charRects.GetCount()){

            if(curPosIdx < charRects.GetCount())
                cursorpos = charRects[curPosIdx].GetLeft();
            else
                cursorpos = charRects[charRects.GetCount()-1].GetRight();

        }

        cursorpos = ceil(cursorpos) + 0.5;
        pen->SetColor(fgcolor);
        aggCanvas.drawLine(cursorpos, selectionRect.GetTop(), cursorpos, selectionRect.GetBottom() );
    }

    render(aggCanvas);
}
