#include "bcToolTip.h"

#include "../logger.h"

#include "../DrawHelper.h"

BEGIN_EVENT_TABLE(bcToolTip, bcLabel)

    EVT_TIMER(idTimer,bcToolTip::OnTimer)

    EVT_LEAVE_WINDOW(bcToolTip::OnLeave)
    EVT_ENTER_WINDOW(bcToolTip::OnLeave)

END_EVENT_TABLE()

bcToolTip::bcToolTip(BitmapCanvasParentBase* parent, const wxString& text)
    :bcLabel(parent,wxID_ANY,text,wxDefaultPosition,wxDefaultSize,BCLS_AUTO_SIZE,false)
    ,m_timer(this,idTimer)
{
    //ctor

    m_isDelayedShow = true;

    long zidx = GetPropLong(BCWB_Z_INDEX) + 10000;
    SetProperty(BCWB_Z_INDEX,zidx);


    //m_isTransBg = true;
    wxColor bgcolor = wxSystemSettings::GetColour(wxSYS_COLOUR_INFOBK);

    DrawHelper::SetColorA(bgcolor,210);

    SetProperty(BCBS_BACKGROUND_COLOR,bgcolor,false);

    TextDesc desc(_T("dummy"));
    desc.Color = wxSystemSettings::GetColour(wxSYS_COLOUR_INFOTEXT);
    desc.Style = bcLabel_GRAVEN;

    WriteLabel(desc);
    SetAlpha(0,false);
}

bcToolTip::~bcToolTip()
{
    //dtor
}

void bcToolTip::SetTipMessage(const wxString& msg)
{
    TextDesc desc = GetLabelDesc();
    desc.Text = msg;
    SetLabel(desc);
}

void bcToolTip::DoShowTip()
{
    SlideAlpha(100);
}

void bcToolTip::DoHideTip()
{
    SlideAlpha(0);
}

void bcToolTip::ShowTip(const wxPoint& pos, int showlenms, int delayedms)
{
    wxWindow* win = GetWindowParent();
    wxSize parentSize = win->GetClientSize();
    wxRealSize currSize = GetSize();

    wxPoint target_pos = pos;

    target_pos.x = wxMin( target_pos.x, parentSize.x-(int)currSize.w );
    target_pos.y = wxMin( target_pos.y, parentSize.y-(int)currSize.h );

    SetProperty(BCWB_POSITION, target_pos);

    m_timer.Start(delayedms ? delayedms : showlenms,wxTIMER_ONE_SHOT);
    m_isDelayedShow = delayedms;

    if(!m_isDelayedShow)  DoShowTip();

    m_showLen = showlenms;
}

void bcToolTip::HideTip()
{
    DoHideTip();
    m_timer.Stop();
}

void bcToolTip::OnLeave(wxMouseEvent& evt)
{
    HideTip();
}

void bcToolTip::OnTimer(wxTimerEvent &evt)
{
    if(m_isDelayedShow) {
        m_timer.Start(m_showLen,wxTIMER_ONE_SHOT);
        m_isDelayedShow = false;
        DoShowTip();
    } else {
        DoHideTip();
    }
}
