#include "GaugeFrame.h"

#include <wx/xml/xml.h>
#include <wx/tokenzr.h>

#include "../DirHelper.h"
#include "../RawLogHelper.h"
#include "../logger.h"
#include "../UserLog.h"
#include "../Math.h"
#include "ParserHelper.h"
//#include "UserLogWindow.h"

#include "../MTConfigData.h"
#include "../RawLogHelper.h"
#include "../FirmwareInfo.h"
#include "../Symbol.h"
#include "../MTConfigHelper.h"

#include "../loghistory/LogHistoryMgr.h"

#include "../rtdatamanager/RTDataMgrFacade.h"
#include "../rtdatamanager/RTDataInputSIPR.h"
#include "../stdatamanager/STDataMgrFacade.h"


#include "../gaugedesc/GaugeDescManager.h"
#include "../gaugedesc/GaugeDescManagerSQL.h"
#include "../gaugedesc/GaugeProperties.h"

#include "../vemsgauge/GaugeType.h"
#include "../vemsgauge/GaugeCarrierPanel.h"
#include "../vemsgauge/GaugeGroupManager.h"
#include "../vemsgauge/GroupMember.h"

#include "../vemsgauge/GaugeGroupManagerSQL.h"

#include "../vemsprotocol/CommandRequestBase.h"
#include "../vemsprotocol/VemsDeviceController.h"
#include "../vemsprotocol/SIPRFileLogger.h"
#include "../vemsprotocol/STDataModelWriter.h"

#include "../vemsgauge/VemsWidgetBaseFactory.h"

#include "../common/types/wxVariantHashHash.h"

#include "../events/InputEvent.h"

#include "../vemsprotocol/SIPRTypes.h"
#include "../ObserverBase.h"
#include "ParserHelper.h"

int idMenuSaveLayout    = wxNewId();

BEGIN_EVENT_TABLE( GaugeFrame, wxFrame )
    EVT_CLOSE( GaugeFrame::onClose )
    EVT_RIGHT_DOWN ( GaugeFrame::onMouseRightClick )
    EVT_MENU( idMenuSaveLayout, GaugeFrame::onMenu )
END_EVENT_TABLE()

GaugeFrame::GaugeFrame( wxWindow* win, MTConfigData * mtconfigdata, VemsDeviceController* controller, wxArrayString& requiredAddrs, const wxString& layoutname ) :
    wxFrame( win, wxID_ANY, _T("Gauge frame"), wxDefaultPosition, wxSize( 800, 600 ) )
{
    m_popupmenu = NULL;
    m_parent = win;
    m_layoutname = layoutname;

    SetBackgroundColour(*wxBLACK);
    ClearBackground();

    m_siprFileLogger = new SIPRFileLogger( 512 );

    m_controller = controller;

    // const FirmwareInfo& fwinfo = m_controller->getFirmwareInfo();

    m_mtConfigData = mtconfigdata;
    m_symbolMgr = &(m_mtConfigData->getSymbolMgr());

    m_aimmode = false;

    /*
    // parsing
    wxString fwStr; fwStr = _T("firmware_") + fwinfo.getFirmwareVersionAsString();
    wxString fname = DirHelper::GetPath( dPROGRAMROOT ) + _T("commmodel\\") +fwStr +  _T("\\my_make.16x14\\extras.ini");

    parseIni( fname );

    fname = DirHelper::GetPath( dPROGRAMROOT ) + _T("commmodel\\") +fwStr +  _T("\\my_make.16x14\\vems.map");

    if ( !parseMap( fname ) )
        LOGP( _T("MAP file parse error!") )

    fname = DirHelper::GetPath( dPROGRAMROOT ) + _T("commmodel\\") +fwStr +  _T("\\my_make.16x14\\global.h");

    parseGlobalH( fname );

    fname = DirHelper::GetPath( dPROGRAMROOT ) + _T("commmodel\\") +fwStr +  _T("\\my_make.16x14\\eventqueue.h");

    parseEventqueueH( fname );
    */
    // init managers & loghistory
    init();

    // init GUI
    initGauges();

    /*for ( SymbolTable::iterator it = m_symbolTable->begin(); it != m_symbolTable->end();++it)
    {
        if ( it->first.StartsWith(_T("etime") ) )
            LOGP( it->first )
    }*/

    // connect the input evt handler
    Connect( wxEVT_INPUT, InputEventHandler( GaugeFrame::onSIPREvent ), NULL, this );
    m_evtDisconnected = false;

    // start logging & start the comm.controller client
    startLog();
    m_controller->readOffsetSIPR( requiredAddrs, this );

    m_packetCounter = 0;
    m_sw.Start();

    m_onlineMode = true;
}

GaugeFrame::GaugeFrame( wxWindow *win, MTConfigData * mtconfigdata, const wxString& layoutname )
    :wxFrame( win, wxID_ANY, _T("Gauge frame"), wxDefaultPosition, wxSize( 800, 600 ) )
{
    m_popupmenu = NULL;
    m_parent = win;
    m_layoutname = layoutname;

    SetBackgroundColour(*wxBLACK);
    ClearBackground();

    m_siprFileLogger = NULL;

    //m_siprFileLogger = new SIPRFileLogger( 512 );

    m_controller = NULL;

    m_mtConfigData = mtconfigdata;
    m_symbolMgr = &(m_mtConfigData->getSymbolMgr());

    m_aimmode = true;

    init();

    // init GUI
    initGauges();

    /*for ( SymbolTable::iterator it = m_symbolTable->begin(); it != m_symbolTable->end();++it)
    {
        if ( it->first.StartsWith(_T("etime") ) )
            LOGP( it->first )
    }*/

    // connect the input evt handler
    Connect( wxEVT_INPUT, InputEventHandler( GaugeFrame::onSIPREvent ), NULL, this );
    m_evtDisconnected = false;


    m_packetCounter = 0;
    m_sw.Start();

    m_onlineMode = true;
}

GaugeFrame::GaugeFrame( wxWindow* win, const wxString& layoutname ) :
    wxFrame( win, wxID_ANY, _T("Gauge frame"), wxDefaultPosition, wxSize( 800, 600 ) )
{
    m_aimmode = false;

    m_popupmenu = NULL;
    m_parent = win;
    m_controller = NULL;
    m_layoutname = layoutname;

    SetBackgroundColour(*wxBLACK);
    ClearBackground();

    m_onlineMode = false;

    m_siprFileLogger = NULL;

    m_evtDisconnected = true;
}

GaugeFrame::~GaugeFrame()
{
    saveGaugesLayout(); // automatically save layout

    if ( m_controller )
        m_controller->stop();

    if ( m_siprFileLogger )
    {
        m_siprFileLogger->enableLog(false);
        delete m_siprFileLogger;
    }

    if ( !m_evtDisconnected )
    {
        m_evtDisconnected = true;
        Disconnect( wxEVT_INPUT, InputEventHandler( GaugeFrame::onSIPREvent ), NULL, this );
    }
}

void GaugeFrame::saveGauges( const wxString& layoutname )
{
    saveGaugesLayout( layoutname );
}

void GaugeFrame::init()
{
    m_rtViewerModel = new DoubleHash();

    m_rtMgr = new RTDataMgrFacade();

    m_inputSIPR = new RTDataInputSIPR( m_symbolMgr, m_rtViewerModel );

    m_rtMgr->registerInput( m_inputSIPR );
    m_rtMgr->selectInput( INPUT_TYPE_SIPR );

    m_stMgr = new STDataMgrFacade();

    m_myModelSerial = new STDataModelSerial( m_mtConfigData->getSymbolTable(), m_mtConfigData->getSIPRDescriptor() );
    m_stMgr->setModel( m_myModelSerial );

    m_deviceModel = new STDataModelSerial( m_mtConfigData->getSymbolTable(), m_mtConfigData->getSIPRDescriptor() );
    m_stMgr->setDeviceModel( m_deviceModel );

    m_logHistoryMgr = new LogHistoryMgr( m_rtMgr, m_stMgr, m_symbolMgr );

    m_logHistoryMgr->initST(*m_mtConfigData->getSIPRDescriptor() );
}

void GaugeFrame::initGauges()
{

    m_gaugeDescMgr = new GaugeDescManagerSQL( m_mtConfigData );

    m_gaugeDescMgr->Init();
    m_gaugeDescMgr->LoadDescriptors( DirHelper::GetPath( dPROGRAMROOT ) + _T("commmodel\\GaugeDescriptors.db") );


    m_gaugeGroupManager = new GaugeGroupManagerSQL( m_parent, new wxMenu(), m_mtConfigData, m_logHistoryMgr );
    m_gaugeGroupManager->SetGaugeDescMgr( m_gaugeDescMgr );
    m_gaugeDescMgr->SetGaugeGroupMgr( m_gaugeGroupManager );
    m_carrier = new GaugeCarrierPanel( this );

    loadGaugesLayout();

    VemsWidgetBaseFactory* vwFactory = new VemsWidgetBaseFactory();

    vwFactory->SetDataMgrObj( m_rtMgr );
    vwFactory->SetDataMgrObj( m_stMgr );
    vwFactory->SetGaugeDescManager( m_gaugeDescMgr );
    vwFactory->SetGaugeGroupManager( m_gaugeGroupManager );
    vwFactory->SetLogHistoryBase( m_logHistoryMgr );

    m_carrier->SetFactory( vwFactory );

    addGauges();

    wxBoxSizer* sizer = new wxBoxSizer( wxVERTICAL );
    sizer->Add( m_carrier, 0, wxEXPAND );

    m_carrier->SetWindowStyleFlag(wxBORDER_NONE);
    m_carrier->ShowCarrier(true);

    SetSize( wxSize( 800, 600 ) );
    SetSizer( sizer );
    sizer->Fit( this );

    CreateStatusBar( 1 );
    SetStatusText( _T("Update frequency: N/A") );

    Show();
}

///--------------------------------------------------------------------------
void GaugeFrame::setOnlineMode( bool online )
{
    if ( m_onlineMode == online )
        return;

    m_onlineMode = online;

    if ( m_onlineMode )
    {
        // ...
    }
    else
    {
        m_controller->stop();
    }
}

void GaugeFrame::loadLogFile( const wxString& filename )
{
    Hide();
    setOnlineMode( false );
    Disconnect( wxEVT_INPUT, InputEventHandler( GaugeFrame::onSIPREvent ), NULL, this );

//--
    m_mtConfigData = MTConfigHelper::getMTConfigData();//new MTConfigData;
    m_symbolMgr = &(m_mtConfigData->getSymbolMgr());

    init();

    m_logHistoryMgr->loadHistoryFromLog( filename );

    StringHash sh = m_logHistoryMgr->getHeaderHash();

    wxString fwStr; fwStr = _T("firmware_") + sh[_T("Version")];
    wxString dirname = DirHelper::GetPath( dPROGRAMROOT ) + _T("commmodel\\") +fwStr;

    FirmwareInfo fwinfo;

    RawLogHelper::fillFirmwareInfo(fwinfo, m_logHistoryMgr->getHeaderHash());

    MTConfigHelper::parseMTConfigDataV2( fwinfo, m_mtConfigData, false );

    ParserHelper helper( dirname, m_mtConfigData );

    // parse mymake, map, global.h,eventqueue.h
    helper.doParsing();

    // init GUI
    initGauges();

    Logger::logf( LOG_MSG, _T("MDX Markers:%d"), m_logHistoryMgr->getMdxFrameMarkersCount() );

    observerNotify( m_logHistoryMgr->getMdxFrameMarkersCount() - 1, this );

    // observerNotify( m_logHistoryMgr->getHistoryCount()-1, this );
}

void GaugeFrame::refreshIdx( int i )
{
    IntArray indices; m_logHistoryMgr->getMdxSiprIndices( i, indices );
    for ( size_t j = 0; j < indices.GetCount(); ++j )
    {
        // Logger::logf( LOG_MSG, _T("refresh:%d"), indices[j] );
        m_logHistoryMgr->refresh( indices[j], false );
    }
}

void GaugeFrame::startLog()
{
    m_siprFileLogger->enableLog(true);

    const FirmwareInfo& fwInfo = m_controller->getFirmwareInfo();

    wxString baseFileName = fwInfo.getSerialNumber() + _T(".vemslog");
    wxString logdir = DirHelper::GetPath(dLOG);
    wxString fileName = DirHelper::CreateNewFileName(baseFileName,logdir,true,true);

    wxString header = RawLogHelper::getVemslogHeader(fwInfo, 3);

    m_siprFileLogger->setHeader(header);
    m_siprFileLogger->setFile(fileName);

    SIPRDescriptor* siprDesc = m_mtConfigData->getSIPRDescriptor();

    STDataModelWriter::logPages( m_myModelSerial, m_siprFileLogger, siprDesc );

    LOGP( _T("header: ") + header );
    LOGP( _T("fileName: ") + fileName );

    UserLog::uLog( ULOG_OK, fileName );

    wxArrayString lines = wxStringTokenize( header, _T("\n") );
    for ( size_t i = 0; i < lines.GetCount(); ++i ){
        UserLog::uLog( ULOG_OK, lines[i] );
    }
}

void GaugeFrame::stopLog()
{
    m_siprFileLogger->enableLog( false );
}

void GaugeFrame::saveGaugesLayout( wxString layoutname )
{
    // get properties
    wxWindowHashInt &hash = m_carrier->GetIDControlHash();

    m_gprops.Clear();

    for( wxWindowHashInt::iterator it = hash.begin(); it != hash.end(); it++ )
    {
        wxWindow * ctrl = it->second;

        if(!ctrl) continue;
        GroupMember * gm = NULL;

        if( gm = dynamic_cast<GroupMember*> (ctrl) )
        {
            GaugeProperties gp = gm->GetGaugeProperties();
            m_gprops.Add( gp );
        }
    }

    wxXmlDocument doc;
    doc.SetFileEncoding( _T("ISO-8859-1") );

    if ( layoutname == _T("") )
    {
        if ( m_aimmode )
            layoutname = _T("aimgauges.xml");
        else
            layoutname = _T("gauges.xml");
    }


    wxXmlNode* root = new wxXmlNode( NULL, wxXML_ELEMENT_NODE, _T("gauges") );

    wxXmlNode* child = new wxXmlNode( root, wxXML_ELEMENT_NODE, _T("gauge") );

    root->SetChildren( child );

    for ( size_t i = 0; i < m_gprops.GetCount(); ++i )
    {
        if ( !child )
        {
            child = new wxXmlNode( root, wxXML_ELEMENT_NODE, _T("gauge") );
            root->SetNext( child );
        }

        // set properties
        GaugeProperties prop = m_gprops[i];

        // type
        wxString gtype;
        switch ( prop.gaugeType )
        {
            case GaugeType::DIGITAL:
                gtype = _T("digital"); break;

            case GaugeType::EXPRESSION_GRID:
                gtype = _T("grid"); break;

            case GaugeType::FLAG:
                gtype = _T("flag"); break;

            case GaugeType::GRAPH:
                gtype = _T("graph"); break;

            case GaugeType::ROUND:
            default: gtype = _T("round");
        }

        wxString xs, ys, ws, hs;
        xs << prop.x; ys << prop.y; ws << prop.width; hs << prop.height;
        child->AddProperty( _T("type"), gtype );
        child->AddProperty( _T("x"), xs ); child->AddProperty( _T("y"), ys );
        child->AddProperty( _T("width"), ws ); child->AddProperty( _T("height"), hs );
        child->AddProperty( _T("descname"), prop.descName ); child->AddProperty( _T("symbol"), prop.symbolName );

        child = NULL;
    }
    doc.SetRoot( root );
    wxString fname =  DirHelper::GetPath(dPROGRAMROOT) + _T("commmodel\\") + layoutname;
    doc.Save( fname );
}

void GaugeFrame::loadGaugesLayout()
{
    m_gprops.Clear();

    wxXmlDocument doc;
    doc.Load( DirHelper::GetPath(dPROGRAMROOT) + _T("commmodel\\") + m_layoutname );

    if ( doc.IsOk() == false )
        return;

    if ( doc.GetRoot()->GetName() != wxT("gauges") )
        return;

    wxXmlNode *child = doc.GetRoot()->GetChildren();
    while ( child )
    {
        if ( child->GetName() == wxT("gauge") )
        {
            wxString gtype = child->GetPropVal(wxT("type"), _T("round") );
            wxString gxs = child->GetPropVal(wxT("x"), _T("0") );
            long gx = 0; gxs.ToLong(&gx);
            wxString gys = child->GetPropVal(wxT("y"), _T("0") );
            long gy = 0; gys.ToLong(&gy);
            wxString gws = child->GetPropVal(wxT("width"), _T("0") );
            long gwidth = 0; gws.ToLong(&gwidth);
            wxString ghs = child->GetPropVal(wxT("height"), _T("0") );
            long gheight = 0; ghs.ToLong(&gheight);
            wxString gdname = child->GetPropVal(wxT("descname"), _T("1") );

            wxString symName = child->GetPropVal(wxT("symbol"), _T("calc_mdx_tps") );

            // Logger::logf(LOG_MSG, gtype + _T(",") + gdname + _T(",%d,%d,%d,%d"), gx,gy,gwidth,gheight);

            GaugeProperties prop;

            prop.x              = gx;
            prop.y              = gy;
            prop.width          = gwidth;
            prop.height         = gheight;
            prop.themeName      = _T("Gold");
            prop.descName       = gdname;
            prop.symbolName     = symName;

            if ( gtype == _T("round") )
                prop.gaugeType = GaugeType::ROUND;
            else if ( gtype == _T("digital") )
                prop.gaugeType = GaugeType::DIGITAL;
            else if ( gtype == _T("grid") )
                prop.gaugeType = GaugeType::EXPRESSION_GRID;
            else if ( gtype == _T("flag") )
                prop.gaugeType = GaugeType::FLAG;
            else if ( gtype == _T("graph") )
                prop.gaugeType = GaugeType::GRAPH;
            else
                prop.gaugeType = GaugeType::ROUND;

            prop.extraParams[_T("showBorder")] = false;
            m_gprops.Add( prop );
        }

        child = child->GetNext();
    }
}

void GaugeFrame::addGauges()
{
    // m_carrier->SetGroupMgr( m_gaugeGroupManager );

    m_carrier->OnlyOneGauge( true );
    //m_carrier->SetDataMgrObj( m_rtMgr );
    //m_carrier->SetDataMgrObj( m_stMgr );
    //m_carrier->SetLogHistory( m_logHistoryMgr );

    m_carrier->SetShaped(false);

    for ( size_t i = 0; i < m_gprops.GetCount(); ++i )
    {
        m_carrier->AddGauge( m_gprops[i] );
    }
}

void GaugeFrame::onSIPREvent( InputEvent& siprevent )
{
    SIPRPacket p = siprevent.getPacket();

    //Logger::logf(LOG_MSG,_T("pc: %i"),m_packetCounter);

    if ( p.type() == SiprSensorType::SIPR_READ_MEMORY_32 )
    {
        m_logHistoryMgr->process(p);

    } else if( p.type() == SiprSensorType::SIPR_AIM ) {



        m_logHistoryMgr->process(p);

    }

    if(m_siprFileLogger) m_siprFileLogger->process( p );

    ++m_packetCounter;

    updateStatusBar();
}

void GaugeFrame::onClose( wxCloseEvent& event )
{
    if ( !m_evtDisconnected )
    {
        m_evtDisconnected = true;
        Disconnect( wxEVT_INPUT, InputEventHandler( GaugeFrame::onSIPREvent ), NULL, this );
    }
    event.Skip();
}

void GaugeFrame::updateStatusBar()
{
    long elapsed = m_sw.Time() / 1000;

    double fps = m_packetCounter * 1.0 / elapsed;

    wxString fpsS = wxString::Format( _T("Update frequency: %.1f packets/s"), fps );
    SetStatusText( fpsS );
}

void GaugeFrame::onMenu( wxCommandEvent& event )
{
    int id = event.GetId();

    if ( id == idMenuSaveLayout )
    {
        saveGaugesLayout();
    }
}

void GaugeFrame::onMouseRightClick( wxMouseEvent& event )
{
    if ( m_popupmenu == NULL )
    {
        m_popupmenu = new wxMenu;

        m_popupmenu->Append( idMenuSaveLayout, _T("Save gauges layout") );
    }


	const wxPoint& pos = event.GetPosition();
    PopupMenu( m_popupmenu, pos.x, pos.y );
}
