#include "DeviceDetector.h"
#include "DeviceDetectorListener.h"
#include "../../vemsprotocol/recognizers/DeviceRecognizer.h"
#include "../../vemsprotocol/recognizers/StringRecognizer.h"
#include "../../common/TxData.h"
#include "../../logger.h"


DeviceDetector::DeviceDetector( DeviceDetectorListener* listener )
{

    m_listener = listener;

    m_state         = SEND;

    m_device        = ECUInfo::DEVICE_UNKNOWN;
    m_mode          = ECUInfo::UNKNOWN;

    DeviceRecognizer* deviceRecognizer;
    StringRecognizer* stringRecognizer = new StringRecognizer( _T("VEMS Genboard v3") );
    deviceRecognizer = new DeviceRecognizer( stringRecognizer, ECUInfo::DEVICE_V3, ECUInfo::MEGATUNE );
    m_recognizers.Add( deviceRecognizer );

    stringRecognizer = new StringRecognizer( _T("Yes, menu and comm worx") );
    deviceRecognizer = new DeviceRecognizer( stringRecognizer, ECUInfo::DEVICE_V3, ECUInfo::MAN );
    m_recognizers.Add( deviceRecognizer );

    stringRecognizer = new StringRecognizer( _T("AVREFI1") );
    deviceRecognizer = new DeviceRecognizer( stringRecognizer, ECUInfo::DEVICE_V3, ECUInfo::BOOT );
    m_recognizers.Add( deviceRecognizer );

    stringRecognizer = new StringRecognizer( _T("VEMS Round WBO2 v") );
    deviceRecognizer = new DeviceRecognizer( stringRecognizer, ECUInfo::DEVICE_ROUND, ECUInfo::MEGATUNE );
    m_recognizers.Add( deviceRecognizer );

    stringRecognizer = new StringRecognizer( _T("MenuPingx") );
    deviceRecognizer = new DeviceRecognizer( stringRecognizer, ECUInfo::DEVICE_ROUND, ECUInfo::MAN );
    m_recognizers.Add( deviceRecognizer );

    stringRecognizer = new StringRecognizer( _T("AVR2") );
    deviceRecognizer = new DeviceRecognizer( stringRecognizer, ECUInfo::DEVICE_ROUND, ECUInfo::BOOT );
    m_recognizers.Add( deviceRecognizer );
}

DeviceDetector::~DeviceDetector()
{
}

void DeviceDetector::write( uint8_t* buffer, int& size )
{
    switch ( m_state )
    {
        case SEND:
        {
            TxData command( _T("Smxc.") );
            size = command.size;

            memcpy( buffer, command.data, command.size );

            m_timeoutTimer.start( m_timeout + TIMEOUT );

            m_state = WAIT;

        } break;

        default:
            break;
    }
}

void DeviceDetector::read( uint8_t* buffer, int size )
{
    switch ( m_state )
    {
        case WAIT:
        {
            if ( size == 0 )
            {
                if ( !m_timeoutTimer.isTimedout() ) {
                    wxThread::Sleep( m_sleep );
                } else {
                    m_state = DEVICE_NOT_FOUND;
                }
            }
            else if ( size > 0 )
            {
                for ( int i = 0; i < size; ++i )
                {
                    for ( size_t j = 0; j < m_recognizers.GetCount(); ++j )
                    {
                        DeviceRecognizer* recognizer = m_recognizers[j];

                        if ( recognizer->feed( buffer[i] ) )
                        {
                            m_device = recognizer->getDevice();
                            m_mode = recognizer->getMode();

                            m_timeoutCount = 0;
                            m_state = DEVICE_FOUND;
                        }
                    }
                }
            }
        } break;

        default:
            break;
    }
}

void DeviceDetector::finish()
{
    if ( m_state == DEVICE_FOUND ) {
        m_listener->OnDeviceFound( m_device, m_mode );
    } else {
        m_listener->OnDeviceNotFound();
    }
}

bool DeviceDetector::isFinished()
{
    return ( ( m_state == DEVICE_FOUND ) || ( m_state == DEVICE_NOT_FOUND ) );
}
