#ifndef DEVICEMODESELECTOR_H
#define DEVICEMODESELECTOR_H


#include "PortScannerListener.h"
#include "BaudRateSelectorListener.h"
#include "TimeoutListener.h"
#include "../CommunicationControllerClient.h"
#include "../CommunicationClientEventHandler.h"
#include "../../vemsprotocol/ECUInfo.h"
#include "../../common/types/IntArray.h"

#include <wx/dynarray.h>


struct ModeSelectCmd
{
    ECUInfo::EcuMode    sourceMode;
    ECUInfo::EcuMode    targetMode;
    wxString            command;

    ModeSelectCmd( ECUInfo::EcuMode sourceMode, ECUInfo::EcuMode targetMode, const wxString& command )
    {
        this->sourceMode    = sourceMode;
        this->targetMode    = targetMode;
        this->command       = command;
    }
};


WX_DECLARE_OBJARRAY( ModeSelectCmd, ModeSelectCmdArray );


class DeviceModeSelectorListener;
class TimeoutListener;
class PortScannerEvent;
class CommunicationBase;


class DeviceModeSelector
    : public CommunicationControllerClient
    , public PortScannerListener
    , public BaudRateSelectorListener
    , public TimeoutListener
{
    public:
        DeviceModeSelector( DeviceModeSelectorListener* listener,
                            TimeoutListener* timeoutListener,
                            CommunicationBase* commBase,
                            const IntArray& baudrates,
                            ECUInfo::EcuMode sourceMode,
                            ECUInfo::EcuMode targetMode,
                            ECUInfo::EcuMode* selectedMode = NULL );

        virtual ~DeviceModeSelector();

        /// CommunicationControllerClient methods
        bool startProcess();
        void write( uint8_t* buffer, int& size );
        void read( uint8_t* buffer, int size );
        void finish();
        bool isFinished();

        /// PortScannerListener methods
        void OnDeviceFound( PortScannerEvent& event );
        void OnDeviceNotFound();

        /// BaudRateSelectorListener methods
        void OnBaudrateChanged( size_t baudrate );

        /// TimeoutListener methods
        void OnTimeout();

    private:
        enum State { SEND_CHANGE_MODE, CLIENT_MODE, CLEAR, FINISHED, TIMEDOUT };
        enum { TIMEOUT = 100 };
        enum { SELECT_TRY_COUNT = 5 };

    private:
        DeviceModeSelectorListener* m_listener;
        TimeoutListener*            m_timeoutListener;
        ECUInfo::EcuMode    m_sourceMode;
        ECUInfo::EcuMode    m_targetMode;
        ECUInfo::EcuMode*   m_selectedMode;

        ECUInfo::Device     m_detectedDevice;
        ECUInfo::EcuMode    m_detectedMode;

        State               m_state;

        ModeSelectCmdArray  m_modeSelectCommands;

        CommunicationControllerClient*  m_client;

        size_t              m_timeoutCount;
        size_t              m_clearCount;
        size_t              m_clearCountMax;

        size_t              m_detectTryCount;
        size_t              m_detectTryCountMax;

        int                 m_sourceBaudrate;

        CommunicationBase*  m_commBase;
        IntArray            m_baudrates;


    private:
        void sendChangeMode( uint8_t* buffer, int& size );
        void sendClient( uint8_t* buffer, int& size );
        void waitClient( uint8_t* buffer, int size );
        void clear( uint8_t* buffer, int size );
};

#endif // DEVICEMODESELECTOR_H
