#ifndef MEGATUNE_H
#define MEGATUNE_H


#include <wx/wx.h>
#include <wx/thread.h>
#include "../CommunicationControllerClient.h"
#include "../../vemsprotocol/MegatuneConfig.h"
#include "../../vemsprotocol/VemsCommand.h"
#include "../../common/typedef.h"
#include "../../common/TxData.h"
#include "../../common/types/IntSet.h"
#include "../../UpdateFrequencyMeter.h" ///TODO - move to DeviceController
#include "../../triggerlog/TriggerFrameFSM.h"

class SIPRDescriptor;
class TimeoutListener;


class PageRequest
{
    public:
        PageRequest()
        {
            active = false;
        }

        virtual ~PageRequest() {}

        bool    active;
        IntSet  requestedPages;
};

WX_DECLARE_LIST( PageRequest, PageRequestList );


class Megatune : public CommunicationControllerClient
{
    public:
        Megatune(   TimeoutListener* timeoutListener,
                    wxEvtHandler* eventHandler,
                    const MegatuneConfig& config,
                    SIPRDescriptor* descriptor );

        virtual ~Megatune();

        /// CommunicationControllerClient methods
        void write( uint8_t* buffer, int& size );
        void read( uint8_t* buffer, int size );
        void finish();
        bool isFinished();
        size_t getInterWriteDelay();
        void request( VemsCommand* command );

        void enableRealtime();

    private:
        enum State
        {
            REQUEST,
            WAIT_RT,
            WAIT_PAGE,
            CLEAR,
            HANDLE_CRC_ERROR,
            END,
        };

        enum { MAX_CRC_ERROR_COUNT_PER_SEC = 5 };

        uint8_t     m_buffer[512];
        uint8_t*    m_rxBuffer;
        int         m_rxDataLength;
        int         m_expectedRxSize;

        int         m_sentReadRTCmdCount;
        int         m_realtimeErrorCount;

        uint16_t    m_crc;
        uint16_t    m_rxCrc;

        uint8_t     m_timePacketBuffer[4];

        UpdateFrequencyMeter        m_updateFrequencyMeter;
        UpdateFrequencyMeter        m_crcErrorMeter;
        wxLongLong start;

        State                       m_state;
        TimeoutListener*            m_timeoutListener;

        MegatuneConfig              m_config;

        TxData                      m_command;
        VemsCommandList             m_queue;

        SIPRDescriptor*             m_descriptor;
        int                         m_page;

        wxDateTime                  m_startTime;

        size_t                      m_clearCount;
        wxEvtHandler*               m_eventHandler;

        bool                        m_isReadRealtimeEnabled;
        wxCriticalSection           m_criticialSection;

        bool                        m_sendAck;
        PageRequestList             m_readPageRequests;
        TriggerFrameFSMCRC          m_triggerFrameFSM;
    private:
        bool readPage( int pageId );
        bool writePage( int pageID, int pageOffset, uint8_t value );
        void burn();
        void readRealtime();
        void executeStringCommand( const wxString& command, size_t sleepAfterWrite, bool sendAck );
        void executeRawCommand( const TxData& data );

        void sendNextCommand( uint8_t* buffer, int& size );
        bool execute( VemsCommand* command );

        void waitRt( uint8_t* buffer, int size );
        void waitRtRead(  uint8_t* buffer, int size );
        void waitRtReadTriggerFrame( uint8_t* buffer, int size );
        void waitRtReadCheckCrc();
        void waitRtReadNoCrc();
        void updateRealtimeFrequency();
        void waitRtEmptyRead();

        void waitPage( uint8_t* buffer, int size );
        void waitPageRead( uint8_t* buffer, int size );
        void waitPageEmptyRead();

        void handleCrcError( uint8_t* buffer, int size );
        void clear();

        void sendInputEvent( const SIPRPacket& siprPacket );
        void sendRtPacket();
        void sendRtPacket(const IntArray& frame);
        void sendRtTimePacket();
        void sendUpdateFrequencyPacket( int updateFrequency );
        void sendPagePacket();
        void sendAckPacket();
        void sendRTCRCErrorPacket();
        void sendRTTimeoutPacket();
        void sendRefreshStartEvent();
        void sendRefreshStopEvent();

        void handlePageRequest();
        void activatePageRequest();
        bool isTriggerFrame();
};

#endif // MEGATUNE_H
