#include "PasswordChecker.h"
#include "PasswordCheckerListener.h"
#include "../CommunicationClientEventHandler.h"
#include "../../vemsprotocol/PasswordHelper.h"
#include "../../vemsprotocol/recognizers/RegExRecognizer.h"
#include "../../vemsprotocol/recognizers/StringRecognizer.h"
#include "../../common/TxData.h"
#include "../../logger.h"
#include "../../DirHelper.h"
#include "../../Math.h"


#include <wx/textfile.h>
#include <wx/filename.h>


PasswordChecker::PasswordChecker( PasswordCheckerListener* listener )
{
    m_listener = listener;
    m_state = SEND_P;
    m_timeoutCount = 0;
    m_result = NO_PASSWORD_PROTECTION;

    m_recognizerP = new RegExRecognizer( _T("BAD pw\r\n\\w{1}\\d{1}.\\d{1}_\\w{1}(\\d{6})") );
    m_recognizerPW = new StringRecognizer( _T("pwOK") );
    m_password = wxEmptyString;
}

PasswordChecker::~PasswordChecker()
{
    delete m_recognizerP;
    delete m_recognizerPW;
}

void PasswordChecker::write( uint8_t* buffer, int& size )
{
    switch ( m_state )
    {
        case SEND_P:
            sendP( buffer, size );
            break;

        case SEND_PW:
            sendPw( buffer, size );
            break;

        default:
            break;
    }
}

void PasswordChecker::read( uint8_t* buffer, int size )
{
    switch ( m_state )
    {
        case WAIT_P:
            waitP( buffer, size );
            break;

        case WAIT_PW:
            waitPw( buffer, size );
            break;

        default:
            break;
    }
}

void PasswordChecker::finish()
{
    switch ( m_result )
    {
        case NO_PASSWORD_PROTECTION:
        case PASSWORD_ACCEPTED:
            m_listener->OnDeviceNotProtected();
            break;

        case INVALID_PASSWORD:
        case PASSWORD_NOT_FOUND:
            m_listener->OnDeviceProtected();
            break;

        case TIMEOUT:
//            eventType = CommunicationClientEvent::TIMEOUT;
            break;
    }
}

bool PasswordChecker::isFinished()
{
    return ( m_state == FINISHED );
}

void PasswordChecker::sendP( uint8_t* buffer, int& size )
{
    TxData command = TxData( _T(".") );

    size = command.size;
    memcpy( buffer, command.data, command.size );

    m_state = WAIT_P;
}

void PasswordChecker::sendPw( uint8_t* buffer, int& size )
{
    TxData command = TxData( m_password );

    size = command.size;
    memcpy( buffer, command.data, command.size );

    m_state = WAIT_PW;
}

void PasswordChecker::waitP( uint8_t* buffer, int size )
{
    if ( size > 0 )
    {
        for ( int i = 0; i < size; ++i )
        {
            if ( m_recognizerP->feed( buffer[i] ) ) {
                TxData* data = m_recognizerP->getReceivedData();
                wxString serial = wxString::From8BitData(  data->data, data->size );
                delete data;

                m_password = findPassword( serial );

                if ( m_password.IsEmpty() ) {
                    m_result = PASSWORD_NOT_FOUND;
                    m_state = FINISHED;
                } else {
                    m_state = SEND_PW;
                }
            }
        }
    }
    else
    {
        ++m_timeoutCount;
        wxThread::Sleep( m_sleep );

        if ( m_timeoutCount >= m_timeout ) {
            m_result = NO_PASSWORD_PROTECTION;
            m_state = FINISHED;
        }
    }
}

void PasswordChecker::waitPw( uint8_t* buffer, int size )
{
    if ( size > 0 )
    {
        ///TODO HANDLE BAD PW
        for ( int i = 0; i < size; ++i )
        {
            if ( m_recognizerPW->feed( buffer[i] ) ) {
                m_result = PASSWORD_ACCEPTED;
                m_state = FINISHED;
            }
        }
    }
    else
    {
        ++m_timeoutCount;
        wxThread::Sleep( m_sleep );

        if ( m_timeoutCount >= m_timeout ) {
            m_result = INVALID_PASSWORD;
            m_state = FINISHED;
        }
    }
}

wxString PasswordChecker::findPassword( const wxString& serialNumber )
{
    wxString crcHex;
    PasswordHelper::checkPassword(serialNumber, &crcHex );
    return crcHex + _T(".");
}


