#include "SDCardFileSummary.h"

#include "../../common/TxData.h"
#include "../../events/SDCardEvent.h"
#include "../../Math.h"
#include "../../logger.h"

#include "SDCardFileHelper.h"

using namespace SDCardFile;

SDCardFileSummary::SDCardFileSummary( const MMCFilesDesc& desc, wxEvtHandler* evthandler )
{
    m_evtHandler = evthandler;
    m_nextBlock = true;

    m_startIdx = m_blockIdx = desc.StartAddress;
    m_endIdx = desc.EndAddress;

    m_buffer = new unsigned char[514+2];
    m_buflen = 0;

    m_state = GET_STATUS;

    m_dhash = new wxVariantHash;

    uint32_t len = m_endIdx - m_blockIdx;
    m_inc = len / desc.SampleCnt;

    if ( m_inc == 0 )
        m_inc = 1;

    (*m_dhash)[_T("Max RPM")] = 0L;
    (*m_dhash)[_T("AVG RPM")] = 0L;
    (*m_dhash)[_T("Max MAP")] = 0L;
    (*m_dhash)[_T("AVG MAP")] = 0L;
    (*m_dhash)[_T("ALS On")] = 0L;
    (*m_dhash)[_T("Launch On")] = 0L;

    m_realSampleCnt = 0;

    m_errorCnt = 0;

    m_firstBlock = true;

//    m_timeoutCounter = 0;

    m_msscounter = 0;
    m_msgcounter = 0;
    LOG

}

SDCardFileSummary::~SDCardFileSummary()
{
    delete[] m_buffer;
}

void SDCardFileSummary::Abort()
{
    m_aborted = true;
    m_state = ABORTED;
}

void SDCardFileSummary::write( uint8_t* buffer, int& size )
{
    m_timeout = TIMEOUT_VAL;
    m_interWriteDelay = 0;


    switch ( m_state )
    {
        case GET_SECTOR:
            getSector( buffer, size );
            break;

        case GET_STATUS:
            getStatus( buffer, size );
            break;

        default:
            break;
    }
}

void SDCardFileSummary::read( uint8_t* buffer, int size )
{
    switch ( m_state )
    {
        case CHECK_SECTOR:
            checkSector( buffer, size );
            break;

        case CHECK_STATUS:
            checkStatus( buffer, size );
            break;

        default:
            break;
    }
}

bool SDCardFileSummary::isFinished()
{
    return ( m_state == FINISHED || m_state == TIMEOUT || m_state == ABORTED || m_state == STATUS_FAILED );
}

void SDCardFileSummary::finish()
{
    SDCardEvent event;

    if ( m_state == TIMEOUT )
    {
        event.SetType( SDCardEventTypes::TIMEOUT );
    }
    else if ( m_state == ABORTED )
    {
        event.SetType( SDCardEventTypes::INTERRUPTED );
    }
    else if ( m_state == STATUS_FAILED )
    {
        event.SetType( SDCardEventTypes::MSx_FAILED );
    }
    else
    {
        event.SetType( SDCardEventTypes::SUMMARY_FINISHED );

        wxVariant v;
        v = m_dhash;

        event.SetVariant( v );
    }

    m_evtHandler->AddPendingEvent( event );
}

void SDCardFileSummary::getStatus( uint8_t* buffer, int& size )
{
    TxData cmd( _T("mss") );

    size = cmd.size;
    memcpy( buffer, cmd.data, cmd.size );

    m_timeoutTimer.start(m_timeout);

    LOG

    m_state = CHECK_STATUS;
}

void SDCardFileSummary::checkStatus( uint8_t* buffer, int size )
{
    if ( size == 0 )
    {

//        ++m_timeoutCounter;
        wxThread::Sleep( m_sleep );
//        LOGP( wxString::Format( _T("elapsed time: %d"), m_timeoutTimer.getElapsedTime() ) );
//        if ( m_timeoutCounter > m_timeout )
        if ( m_timeoutTimer.isTimedout() )
        {
            LOG
            m_state = TIMEOUT;
        }
        return;
    }
//    else m_timeoutCounter = 0;

    if ( size > 0 )
    {
        LOG
        char c = buffer[0];

        if ( c == '0' )
        {
            LOGP( wxString::Format( _T("first: %d"), m_timeoutTimer.getElapsedTime() ) );
            m_state = GET_SECTOR;
            m_msscounter = 0;
        }
        if ( c == 'u' )
        {
            LOGP( wxString::Format( _T("first: %d"), m_timeoutTimer.getElapsedTime() ) );
            ++m_msscounter;

            m_state = GET_STATUS;

            wxThread::Sleep( 100 );

            if ( m_msscounter > 10 )
            {
                LOGP(_T("ERROR: mss still responding 'u' after 10 tries") )

                m_state = STATUS_FAILED;
            }
        }
    }
}

///
void SDCardFileSummary::getSector( uint8_t* buffer, int& size )
{
    if ( m_nextBlock )
    {
        memset( m_buffer, 0, 514 + 2 );
        m_buflen = 0;

        uint32_t blockIdx = m_blockIdx & 0xffffff;

        wxString hIdx = wxString::Format( _T("%06X"), blockIdx );
        wxString cmd = _T("msg") + hIdx;

        TxData command = TxData( cmd );
        size = command.size;

        double percent = (double)( m_blockIdx - m_startIdx ) * 100.0 / (double)( m_endIdx - m_startIdx );
        SDCardEvent event;
        event.SetType( SDCardEventTypes::SUMMARY_PROGRESS );
        wxVariant v; v = percent;
        event.SetVariant( v );
        m_evtHandler->AddPendingEvent( event );

        memcpy( buffer, command.data, command.size );

        m_nextBlock = false;

//        m_timeoutCounter = 0;
        m_timeoutTimer.start(m_timeout);

        m_state = CHECK_SECTOR;
    }
}

void SDCardFileSummary::checkSector( uint8_t* buffer, int size )
{
    // timeout?
    if ( size == 0 )
    {
//        ++m_timeoutCounter;
        wxThread::Sleep( m_sleep );
//        LOGP( wxString::Format( _T("elapsed time: %d"), m_timeoutTimer.getElapsedTime() ) );
//        if ( m_timeoutCounter > m_timeout )
        if ( m_timeoutTimer.isTimedout() )
        {
            ++m_msgcounter;
            if ( m_msgcounter > MAX_MSG_COUNT )
            {
                m_state = TIMEOUT;
                return;
            }
            else
            {
                m_state = GET_STATUS;
                m_nextBlock = true;
//                m_timeoutCounter = 0;
            }
        }
        return;
    }
//    else m_timeoutCounter = 0;

    memcpy( m_buffer + m_buflen, buffer, size );
    m_buflen += size;

    if ( m_buflen == 1 ) {
        LOGP( wxString::Format( _T("first: %d"), m_timeoutTimer.getElapsedTime() ) );
    }

    if ( m_buflen >= 514 )
    {
        bool isOk = isCRCValid( m_buffer, 514 );
        bool isValid = isDataValid( m_buffer, 514 );

        if ( isOk && isValid && ( m_errorCnt < 4 ) )
        {
            LOGP( wxString::Format( _T("elapsed time: %d"), m_timeoutTimer.getElapsedTime() ) );
            // skip first block
            if ( m_firstBlock == false )
            {
                m_errorCnt = 0;
                LOG

                ++m_realSampleCnt;

                long rpm = m_buffer[14] * 256 + m_buffer[15];
                long map = (m_buffer[4] * 256 + m_buffer[5]) / 4;

                if ( (*m_dhash)[_T("Max RPM")].GetLong() < rpm )
                    (*m_dhash)[_T("Max RPM")] = (long) rpm;

                if ( (*m_dhash)[_T("Max MAP")].GetLong() < map )
                    (*m_dhash)[_T("Max MAP")] = (long) map;

                (*m_dhash)[_T("AVG RPM")] = (*m_dhash)[_T("AVG RPM")].GetLong() + rpm;
                (*m_dhash)[_T("AVG MAP")] = (*m_dhash)[_T("AVG MAP")].GetLong() + map;


                if ( m_buffer[47] & 128 == 128 )
                    (*m_dhash)[_T("ALS On")] = (*m_dhash)[_T("ALS On")].GetLong() + 1;

                if ( m_buffer[47] & 64 == 64 )
                    (*m_dhash)[_T("Launch On")] = (*m_dhash)[_T("Launch On")].GetLong() + 1;
            }
        }
        else if ( m_errorCnt < 4 )
        {
            ++m_errorCnt;
            m_nextBlock = true;
            m_state = GET_STATUS;
            return;
        }

        m_blockIdx += m_inc;
        m_firstBlock = false;

        if ( m_blockIdx > m_endIdx )
        {
            // finished
            (*m_dhash)[_T("SampleCount")] = (long)m_realSampleCnt;

            (*m_dhash)[_T("AVG RPM")] = (double)( (*m_dhash)[_T("AVG RPM")].GetLong() ) / (double)m_realSampleCnt;
            (*m_dhash)[_T("AVG MAP")] = (double)( (*m_dhash)[_T("AVG MAP")].GetLong() ) / (double)m_realSampleCnt;
            (*m_dhash)[_T("ALS On")] = (double)( (*m_dhash)[_T("ALS On")].GetLong() ) * 100.0 / (double)m_realSampleCnt;
            (*m_dhash)[_T("Launch On")] = (double)( (*m_dhash)[_T("Launch On")].GetLong() ) * 100.0 / (double)m_realSampleCnt;

            m_state = FINISHED;
        }
        else
        {
            m_nextBlock = true;
            m_state = GET_SECTOR;
        }
    }
}
