#include "ConfigFrameHelper.h"

#include <wx/dir.h>

#include "ConfigHelper2.h"

#include "../FirmwareInfo.h"
#include "../gui/GeneralConfigFrame.h"
#include "../ConfigFile.h"
#include "../logger.h"
#include "../DirHelper.h"
#include "../SerialPortHelper.h"

void ConfigFrameHelper::EnvironmentConfigFrame(wxWindow * parent)
{
    ConfigBase* config  = ConfigHelper2::getConfig();
    GeneralConfigFrame* frame = new GeneralConfigFrame( parent, wxID_ANY, _T("Environment Config"), config );
    frame->addGroup();
    frame->addTextCtrl( _T("/environment/ExtLogViewer") );
	wxArrayString choices;
    choices.Add(_T("1"));
    choices.Add(_T("2"));
	frame->addEntrySelectComboBox( _T("/graphics_quality_factor"),choices);
    frame->selectMainGroup();
    frame->addWarningText();
    frame->addSaveButton();
    frame->CenterOnScreen();
    frame->Show( true );
}

void ConfigFrameHelper::ProtocolConfigFrame(wxWindow * parent)
{
    ConfigBase* config  = ConfigHelper2::getConfig();

    GeneralConfigFrame* frame = new GeneralConfigFrame( parent, wxID_ANY, _T("Default Protocol"), config );

    frame->addGroup( _T("Default protocol") );
    wxArrayString choices;
    choices.Add(_T("MEGATUNE"));
    choices.Add(_T("SIPR"));
    //frame->addEntrySelectComboBox(_T("protocol"), choices );
    frame->addEntrySelectComboBox(  _T("protocol"), choices, wxEmptyString, _T("/devices") );

    frame->addGroup( _T("Default read realtime command") );
    choices.Clear();
    choices.Add(_T("SIPR_RT_MT_NORMAL"));
    choices.Add(_T("SIPR_RT_MT_EXT"));
    //frame->addEntrySelectComboBox(_T("protocol"), choices );
    frame->addEntrySelectComboBox(  _T("read_realtime"), choices, wxEmptyString, _T("/devices") );

    frame->selectMainGroup();

    //frame->addWarningText();
    frame->addSaveButton();
    frame->CenterOnScreen();
    frame->Show( true );
}

void ConfigFrameHelper::GPSConfigFrame(wxWindow * parent)
{
    wxWindow* win = wxFindWindowByName( _T("GPS Communication"), parent );
    if ( win )
    {
        win->SetFocus();
        return;
    }
    ConfigBase* config  = ConfigHelper2::getConfig();
    GeneralConfigFrame* frame = new GeneralConfigFrame( parent, wxID_ANY, _T("GPS Communication"), config );

    wxArrayString ports  = SerialPortHelper::GetCOMPortList();

    /// removed - see ECUConfigPanel
    /*
    frame->addGroup( _T("ECU") );

    frame->addEntrySelectComboBox(_T("/port"), ports);
    frame->addTextCtrl( _T("/timeout") );

    frame->addGroup( _T("SD Card Log") );
    //frame->addTextCtrl( _T("baudrate"), wxEmptyString, _T("/sdcard") );
    wxArrayString choices;
    choices.Add(_T("57600"));
    choices.Add(_T("115200"));
    frame->addEntrySelectComboBox(_T("/sdcard/baudrate"), choices);
    */
    ///
    frame->addGroup( _T("GPS") );

    //frame->addTextCtrl( _T("/gps/port") );
    frame->addEntrySelectComboBox(_T("/gps/port"), ports);

    frame->addWarningText( _T("Experiences show that ports over COM9 might not work") );


    wxArrayString baudRateA;
    baudRateA.Add( _T("4800") );
    baudRateA.Add( _T("9600") );
    baudRateA.Add( _T("19200") );
    baudRateA.Add( _T("38400") );
    baudRateA.Add( _T("57600") );
    baudRateA.Add( _T("115200") );

    frame->addEntrySelectComboBox( _T("/gps/baudrate"), baudRateA);

    frame->selectMainGroup();
    frame->addWarningText();
    frame->addSaveButton();
    frame->CenterOnScreen();
    frame->Show( true );

}

void ConfigFrameHelper::IniFileConfigFrame(wxWindow * parent)
{
    ConfigBase* config  = ConfigHelper2::getConfig();
    GeneralConfigFrame* frame = new GeneralConfigFrame( parent, wxID_ANY, _T("Inifile Config"), config );
    frame->addGroup( _T("Devices") );
    frame->addComboBox( _T("/devices/"), wxEmptyString, _T("v3") );
    frame->addGroup();
    //frame->addTextCtrl( _T("iniFile"), wxEmptyString, _T("/devices/") );
    wxArrayString choices;
    wxString configDirName = DirHelper::GetPath( dCONFIG, true );
    wxString fileSpec = _T("vemsTune*.ini");
    wxDir::GetAllFiles( configDirName, &choices, fileSpec );
    for( size_t i = 0; i < choices.GetCount(); ++i )
    {
        choices.Item( i ) = wxFileNameFromPath( choices.Item( i ) );
    }
    Logger::logf( LOG_MSG, _T("MainFrame::OnIniFileConfig()") + configDirName );
    frame->addEntrySelectComboBox(  _T("iniFile"), choices, wxEmptyString, _T("/devices/") );
    frame->selectMainGroup();
    frame->addWarningText();
    frame->addSaveButton();
    frame->CenterOnScreen();
    frame->Show( true );
}

void ConfigFrameHelper::EditConfigFrame(wxWindow * parent, FirmwareInfo * fwinfo)
{
    ConfigBase* config  = ConfigHelper2::getConfig();
    GeneralConfigFrame* frame = new GeneralConfigFrame( parent, wxID_ANY, _T("Parameters Config"), config );
    //frame->addGroup( _T("Projects") );
    //wxString currSession = _T("");//Helper::getCurSession();
    //frame->addComboBox( _T("/sessions/"), wxEmptyString, currSession );
    frame->addGroup( _T("Firmware Parameters") );
    frame->addTextCtrlGroup( wxEmptyString, _T("/defaultparams/"), _T("/sessions/"), _T("params/") );

    frame->addGroup( _T("Detected Firmware Parameters") );
    //frame->addTextCtrlGroup( m_vemsTuneController->getFirmwareInfo()->getDetectedParams() );
    frame->addTextCtrlGroup( fwinfo->getDetectedParams() );

    frame->selectMainGroup();
    frame->addWarningText();
    frame->addSaveButton();
    frame->CenterOnScreen();

    frame->Show( true );

}

void ConfigFrameHelper::TorqueConfigFrame(wxWindow * parent)
{
    ConfigBase* config  = ConfigHelper2::getConfig();
    GeneralConfigFrame* frame = new GeneralConfigFrame( parent, wxID_ANY, _T("Torque Config"), config );
    frame->addGroup( _T("Projects") );
    wxString currSession = _T("");//Helper::getCurSession();
    frame->addComboBox( _T("/sessions/"), wxEmptyString, currSession );
    frame->addGroup( _T("TorqueConfig") );
    frame->addTextCtrlGroup( wxEmptyString, _T("/defaultTorqueConfig/"), _T("/sessions/"), _T("torqueConfig/") );
    frame->addGroup( _T("TorqueConfig / DrivetTrain") );
    frame->addTextCtrlGroup( wxEmptyString, _T("/defaultTorqueConfig/drivetrain/"), _T("/sessions/"), _T("torqueConfig/drivetrain/") );
    frame->selectMainGroup();
    frame->addWarningText();
    frame->addSaveButton();
    frame->CenterOnScreen();
    frame->Show( true );
}
