#include "DeviceConfigHelper.h"

#include <wx/regex.h>
#include <wx/filename.h>

#include "../DisplayHelper.h"
#include "../vemsprotocol/ECUInfo.h"
#include "../ConfigBase.h"
#include "../DirHelper.h"

wxSize DeviceConfigHelper::m_refDisplaySize = wxDefaultSize;

DeviceConfigHelper::DeviceConfigHelper()
{
    //ctor
}

DeviceConfigHelper::~DeviceConfigHelper()
{
    //dtor
}

wxString DeviceConfigHelper::getLayoutFile(ConfigBase* configBase, int device )
{
    return getDeviceFile(configBase, device, _T("layoutFile") );
}

wxString DeviceConfigHelper::getDeviceFile(ConfigBase* configBase, int device, const wxString& key )
{
    switch ( device )
    {
        case ECUInfo::DEVICE_V3:
            return  configBase->getValue( _T("/devices/v3"), key, wxEmptyString );

        case ECUInfo::DEVICE_ROUND:
            return  configBase->getValue( _T("/devices/round"), key, wxEmptyString );

        default:
            return  configBase->getValue( _T("/"), key, wxEmptyString );
    }
}



wxString DeviceConfigHelper::GetGroupFile(ConfigBase* configBase, int device )
{
    switch ( device )
    {
        case ECUInfo::DEVICE_V3:
            return configBase->getValue( _T("/devices/v3"), _T("groupFile"), wxEmptyString );

        case ECUInfo::DEVICE_ROUND:
            return configBase->getValue( _T("/devices/round"), _T("groupFile"), wxEmptyString );

        default:
            return wxEmptyString;
    }


}


wxSize DeviceConfigHelper::ParseGaugesDBFile(const wxString& str)
{
    wxRegEx exp2;
    exp2.Compile(_T("gauges\\-([0-9]{2,})x([0-9]{2,}).db"));
    bool res2 = exp2.Matches(str);
    if(res2) {
        long w=1,h=1;
        exp2.GetMatch(str,1).ToLong(&w);
        exp2.GetMatch(str,2).ToLong(&h);
        return wxSize(w,h);
    }
    return wxDefaultSize;
}

wxString DeviceConfigHelper::getDevicePath(int device)
{
    wxString devPath;
    switch ( device )
    {
        case ECUInfo::DEVICE_V3:
            devPath = _T("/devices/v3");
            break;
        case ECUInfo::DEVICE_ROUND:
            devPath = _T("/devices/round");
            break;
        default:
            devPath = _T("/devices");
    }
    return devPath;
}


wxString DeviceConfigHelper::GetGaugesFile(ConfigBase* configBase, int device )
{
    wxString devpath = getDevicePath(device);
    wxString defValue = _T("gaugegroups.db");
    const wxString dgfnofile = _T("nofile");

    //configBase->setPath(_T("/"));
    wxString debug_gf = configBase->getValue(_T("/"), _T("debug_gaugesfile"), dgfnofile);

    if( debug_gf != dgfnofile && wxFileName::FileExists(DirHelper::GetPath(dCONFIG)+debug_gf) ) {
        return debug_gf;
    }

    wxSize ds = DisplayHelper::GetDisplaySize();
    double displayratio = (double)(ds.x) / (double)(ds.y);

    configBase->setPath(devpath + _T("/defaultgaugesfiles") );

    wxString str;
    long dummy;

    wxRegEx exp;
    exp.Compile(_T("r_([0-9]{1,})_([0-9]{1,})"));

    bool bCont = configBase->getFirstEntry(str, dummy);

    double min_ar_diff = 100.0;
    wxString min_ar_str;

    for(;bCont;bCont = configBase->getNextEntry(str, dummy)){

        long x=1,y=1;
        bool res = exp.Matches(str);
        if(!res) continue;

        exp.GetMatch(str,1).ToLong(&x);
        exp.GetMatch(str,2).ToLong(&y);

        double cr = (double)x/(double)y;

        double ar_diff = fabs(cr - displayratio);

        if( min_ar_diff > ar_diff ) {
            min_ar_diff = ar_diff;
            //min_ar_str = configBase->read(str, &defValue);
            configBase->read(str, &min_ar_str, defValue);
        }

        if( ar_diff <= 0.01 ) {

            //wxString dbfile = configBase->read(str, (wxString*)&defValue);
            wxString dbfile;
            configBase->read(str, &dbfile, defValue);

            m_refDisplaySize = ParseGaugesDBFile( dbfile );

            return dbfile;
        }

    }

    if( min_ar_str != wxEmptyString ) {

        m_refDisplaySize = ParseGaugesDBFile( min_ar_str );

        return min_ar_str;
    }

    m_refDisplaySize = wxDefaultSize;

    return wxEmptyString;
}

