#include "controllerdescriptor.h"

ControllerDescriptor::ControllerDescriptor()
    :interWriteDelay (1)
    ,writeBlocks (true)
    ,bigEnd (true)
    ,nPages (6)
    ,burnCommand (_T("B"))
    ,pageActivationDelay (10)
    ,m_ochGetCommand (_T("A"))
{}

ControllerDescriptor::~ControllerDescriptor()
{
    //dtor
}


int ControllerDescriptor::getPageSize(int pageNo) const
{
    int realPageNo = getPageActivate(pageNo);

    if((realPageNo < 0) || ( realPageNo > pageSize.GetCount())){ return -1; }

    return pageSize[realPageNo];
}


int ControllerDescriptor::getPageIdx(int page) const
{
    //return pageNo;
    if( pageActivate.GetCount() <= page) { return -1; }
    return pageActivate.Item(page);

}

int ControllerDescriptor::getPageActivate(int pageNo) const
{
    //return pageNo;

    if( pageActivate.Index(pageNo) == wxNOT_FOUND ) { return -1; }
    return pageActivate.Index(pageNo);

}

wxString ControllerDescriptor::getPageReadCommand(int pageNo) const
{
    int realPageIdx = getPageActivate(pageNo);

    if((realPageIdx < 0) || ( realPageIdx > pageSize.GetCount())){ return _T(""); }


    return pageReadCommand[realPageIdx];
}

wxString ControllerDescriptor::getPageValueWrite(int pageNo) const
{
    int realPageNo = getPageActivate(pageNo);

    if((realPageNo < 0) || ( realPageNo > pageSize.GetCount())){ return _T(""); }

    return pageValueWrite[realPageNo];
}

void ControllerDescriptor::setPageSize(const ArrayInt& ps)
{
    if(ps.GetCount() != nPages) { }
    else{ pageSize = ps; }
}

//test only functions
wxString ControllerDescriptor::pageSizeToString()
{
     wxString str = _T("PageSize : ");

     for(size_t i = 0; i < pageSize.GetCount(); i++)
     {
         str.Append(wxString::Format(_T("%d : %d, "),(int)i, (int)pageSize.Item(i) ));
     }

     return str;
}

wxString ControllerDescriptor::pageActivateToString()
{
     wxString str = _T("PageActivate : ");

     for(size_t i = 0; i < pageActivate.GetCount(); i++)
     {
         str.Append(wxString::Format(_T("%d : %d, "),(int)i, pageActivate[i] ));
     }

     return str;
}

wxString ControllerDescriptor::pageReadCommandToString()
{
     wxString str = _T("pageReadCommand : ");

     for(size_t i = 0; i < pageActivate.GetCount(); i++)
     {
         str.Append(wxString::Format(_T("%d : %s, "),(int)i, (pageReadCommand.Item(i)).c_str() ));
     }

     return str;
}

wxString ControllerDescriptor::pageValueWriteToString()
{
     wxString str = _T("pageValueWrite : ");

     for(size_t i = 0; i < pageActivate.GetCount(); i++)
     {
         str.Append(wxString::Format(_T("%d : %s, "),(int)i, (pageValueWrite.Item(i)).c_str() ));
     }

     return str;
}

wxString ControllerDescriptor::pageChunkWriteToString()
{
     wxString str = _T("pageChunkWrite : ");

     for(size_t i = 0; i < pageActivate.GetCount(); i++)
     {
         str.Append(wxString::Format(_T("%d : %s, "),(int)i, (pageChunkWrite.Item(i)).c_str() ));
     }

     return str;
}

void ControllerDescriptor::clear()
{
    pageActivationDelay = 10;
    interWriteDelay     = 1;
    writeBlocks = true;
    bigEnd = true;
    nPages = 6;

    burnCommand.Clear();
    pageSize.Clear();
    pageActivate.Clear();
    pageReadCommand.Clear();
    pageValueWrite.Clear();
    pageChunkWrite.Clear();
}
