#ifndef AGGTOOLTIP_H
#define AGGTOOLTIP_H

#include <wx/wx.h>
#include "../common/types/wxVariantIntHash.h"
#include "../common/types/DoubleArray.h"

WX_DECLARE_STRING_HASH_MAP(wxVariantIntHash, wxVariantIntHashHash);

#define DefaultAggToolTipProfile _T("DefaultProfile")

enum AggToolTip_PropertyTypes {
    ATT_TEXT_CONTENT = 0,
    ATT_TEXT_ISWRAP,
    ATT_TEXT_COLOR,
    ATT_TEXT_HEIGHT,
    ATT_TEXT_STROKE,
    ATT_TEXT_PADDING,
    ATT_TEXT_ALIGN,
    ATT_BACKGROUNDCOLOR,
    ATT_BORDER_WIDTH,
    ATT_BORDER_COLOR,
    ATT_ARROW_ALIGMENT,
    ATT_ARROW_SIZE,         /// You can set a ratio for this value, like this: "42%". In this case the arrow size is: TEXT_HEIGHT * 0.42
    ATT_ARROW_PXPOSITION,
    ATT_ARROW_REFPOSITION,
    ATT_ROUND_RADIUS,
    ATT_ROUND_TYPE,
    ATT_BOX_MARGIN,
    ATT_SPECIAL_NO_RENDER
};

enum AggToolTip_ArrowAlign {
    ATT_AA_TOP = 0,
    ATT_AA_RIGHT = 1,
    ATT_AA_BOTTOM = 2,
    ATT_AA_LEFT = 3,
    ATT_AA_NONE = 4
};

enum AggToolTip_TextAlign {
    ATT_TA_LEFT = 0,
    ATT_TA_CENTER = 1,
    ATT_TA_RIGHT = 2
};

enum AggToolTip_RoundTypes {
    ATT_RT_ARC,
    ATT_RT_LINE,
    ATT_RT_NONE
};

class AggToolTip
{
    public:
        AggToolTip();
        virtual ~AggToolTip();

        void SetProperty(int prop, const wxVariant& value);
        void SetColorProp(int prop, const wxColor& value);

        void SetProperties(wxVariantIntHash hash);

        wxVariant GetProperty(int prop, const wxVariant& defvalue = wxNullVariant);

        wxBitmap* DrawBox(const wxVariant& value = wxNullVariant, int key = ATT_TEXT_CONTENT);

        wxPoint GetArrowPoint();



        double CalcContentWidth(wxArrayString& lines, DoubleArray& line_widths);
        wxSize CalcSize();

        wxString GetCurrentProfile() { return m_currentProfile; }
        void SetCurrentProfile(const wxString& name) { m_currentProfile = name; }

        void InitProfile(const wxString &name);

    protected:

        bool GetBool(int key);
        long GetLong(int key);
        double GetDouble(int key);
        wxString GetStr(int key);
        wxColor GetColor(int key);

        double GetArrowSize();

    private:

        wxString                m_currentProfile;

        wxVariantIntHashHash    m_profileProps;

};

#endif // AGGTOOLTIP_H
