#include "ControlColorContentValidator.h"

#include "../logger.h"

#include "../events/EditorValueChangedEvent.h"

ControlColorContentValidator::ControlColorContentValidator(wxWindow * targetwin, const wxString& value, const wxString& nullstr, bool isPending)
{
    //ctor
    CreateValidator(targetwin,value,nullstr,isPending);
}

ControlColorContentValidator::~ControlColorContentValidator()
{
    //dtor
}

void ControlColorContentValidator::CreateValidator(wxWindow * targetwin, const wxString& value, const wxString& nullstr, bool isPending)
{
    m_targetWin = targetwin;

    m_isPending = isPending;
    m_isChanged = false;

    m_nullStr = nullstr;

    m_normalFont = m_targetWin->GetFont();
    m_normalColor= m_targetWin->GetBackgroundColour();

    m_nullColor = wxColor(255,255,0x99);
    m_nullFont  = m_targetWin->GetFont();

    m_changedColor = wxColor(255,130,130);

    m_nullFont.SetStyle(wxFONTSTYLE_ITALIC);

    if( value == nullstr || value == wxEmptyString ){

        m_targetWin->SetBackgroundColour(m_nullColor);
        m_targetWin->SetFont(m_nullFont);

        CCCV_ChangeValue(m_nullStr);
    }


    m_targetWin->ClearBackground();
}

void ControlColorContentValidator::ChangeToNull()
{
    CCCV_SetValue(m_nullStr);
    ResetChange();
}

void ControlColorContentValidator::CCCV_SetValue(const wxString& val)
{
    if( m_targetWin == NULL ) return;

    wxTextCtrl * textctrl = wxDynamicCast(m_targetWin,wxTextCtrl);
    if(textctrl) {
        textctrl->SetValue(val);
        return;
    }

    wxComboBox * combo = wxDynamicCast(m_targetWin,wxComboBox);
    if(combo) {
        combo->SetValue(val); ///?
        return;
    }

    wxStaticText * text = wxDynamicCast(m_targetWin,wxStaticText);
    if(text) {
        return text->SetLabel(val);
    }
}

void ControlColorContentValidator::CCCV_ChangeValue(const wxString& val)
{
    if( m_targetWin == NULL ) return;

    wxTextCtrl * textctrl = wxDynamicCast(m_targetWin,wxTextCtrl);
    if(textctrl) {
        textctrl->ChangeValue(val);
        return;
    }

    wxComboBox * combo = wxDynamicCast(m_targetWin,wxComboBox);
    if(combo) {
        combo->SetValue(val); ///?
        return;
    }

    wxStaticText * text = wxDynamicCast(m_targetWin,wxStaticText);
    if(text) {
        return text->SetLabel(val);
    }
}

wxString ControlColorContentValidator::CCCV_GetValue()
{
    if( m_targetWin == NULL ) return wxEmptyString;

    wxTextCtrl * textctrl = wxDynamicCast(m_targetWin,wxTextCtrl);
    if(textctrl) {
        return textctrl->GetValue();
    }

    wxComboBox * combo = wxDynamicCast(m_targetWin,wxComboBox);
    if(combo) {
        return combo->GetValue();
    }

    wxStaticText * text = wxDynamicCast(m_targetWin,wxStaticText);
    if(text) {
        return text->GetLabel();
    }

    return wxEmptyString;
}

 void ControlColorContentValidator::ResetChange(bool updateStyle)
{
    m_isChanged = false;

    if(updateStyle)
    SetStyleEx();
}

void ControlColorContentValidator::ChangeValueEx(const wxString& val, bool isStyle)
{
    if(isStyle) m_isChanged = true;
    CCCV_ChangeValue( val.Len() ? val : m_nullStr );
    SetStyleEx();
}

void ControlColorContentValidator::SetStyleEx()
{
    if( m_targetWin == NULL ) return;

    wxString str = CCCV_GetValue();
    if( str.Len() == 0 || str == m_nullStr ) {
        m_targetWin->SetBackgroundColour(m_isChanged?m_changedColor:m_nullColor);
        m_targetWin->SetFont(m_nullFont);
    } else {
        m_targetWin->SetBackgroundColour(m_isChanged?m_changedColor:m_normalColor);
        m_targetWin->SetFont(m_normalFont);
    }

    m_targetWin->Refresh(true);
}


bool ControlColorContentValidator::IsNull()
{
    return CCCV_GetValue() == m_nullStr ? true : false;
}

void ControlColorContentValidator::DoOnUpdated(bool updateStyle)
{
    if( m_targetWin == NULL ) return;

    m_isChanged = true;

    if(updateStyle) SetStyleEx();

    wxCommandEvent event(EditorValueChangedEvent);
    event.SetInt( EVCE_TEXT );

    wxWindow * win = m_targetWin->GetParent();

    if(win) {
        if(m_isPending)
            win->AddPendingEvent(event);
        else
            win->ProcessEvent(event);
    }
}

void ControlColorContentValidator::DoSetFocus()
{
    if( m_targetWin == NULL ) return;

    wxString value = CCCV_GetValue();

    if( value == m_nullStr || value == wxEmptyString ){

        m_targetWin->SetBackgroundColour(m_isChanged?m_changedColor:m_normalColor);
        m_targetWin->SetFont(m_normalFont);
        CCCV_ChangeValue(wxEmptyString);
    }

    //m_targetWin->ClearBackground();
}

void ControlColorContentValidator::DoKillFocus()
{
    if( m_targetWin == NULL ) return;

    wxString value = CCCV_GetValue();

    if( value == m_nullStr || value == wxEmptyString ){

        m_targetWin->SetBackgroundColour(m_isChanged?m_changedColor:m_nullColor);
        m_targetWin->SetFont(m_nullFont);
        CCCV_ChangeValue(m_nullStr);
    }


    //m_targetWin->ClearBackground();

}
