#include "SymbolChooserPanel.h"

#include "../logger.h"
#include "../DrawHelper.h"
#include "../gaugedesc/VemsFlagDescManager.h"
#include "../gaugedesc/GaugeDescManager.h"

BEGIN_EVENT_TABLE(SymbolChooserPanel, wxPanelWithCtrlExBase)

    EVT_BUTTON(idSelectButton,SymbolChooserPanel::OnButton)

    //EVT_SIZE(SymbolChooserPanel::OnSize)

END_EVENT_TABLE()

IMPLEMENT_DYNAMIC_CLASS(SymbolChooserPanel,wxPanelWithCtrlExBase);

SymbolChooserPanel::SymbolChooserPanel(wxWindow* parent, GaugeDescManager* mgr)
    :m_label (NULL)
    ,m_updateSizeTarget (NULL)
{
    //ctor
    m_mgr = mgr;
    m_selectMenu = NULL;

    Create(parent, wxID_ANY);

    m_label = new wxStaticText(this, wxID_ANY, _T(""),wxDefaultPosition, wxDefaultSize, wxALIGN_CENTRE | wxST_NO_AUTORESIZE | wxBORDER_SIMPLE);

    m_label->SetBackgroundColour( wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW) );

    CreateValidator(m_label,_T("a"));

    wxBitmapButton* sbutt = new wxBitmapButton(this,idSelectButton, DrawHelper::GetBitmap(_T("settings-16.png")) );

    wxBoxSizer* mainsizer = new wxBoxSizer(wxHORIZONTAL);

    mainsizer->Add( m_label, 1, wxRIGHT | wxTOP | wxBOTTOM | wxALIGN_CENTER_VERTICAL , 2 );
    mainsizer->Add( sbutt, 0, wxLEFT | wxTOP | wxBOTTOM , 2 );

    SetSizer(mainsizer);
    mainsizer->SetSizeHints(this);

    Connect(wxEVT_SIZE, wxSizeEventHandler(SymbolChooserPanel::OnSize));
}

SymbolChooserPanel::~SymbolChooserPanel()
{
    //dtor
}

void SymbolChooserPanel::SetValue(const wxVariant& value)
{
    SetSymbolName( value.GetString() );
}

wxVariant SymbolChooserPanel::GetValue()
{
    return GetSymbolName();
}

void SymbolChooserPanel::SetSymbolName(const wxString& name, bool isDropEvt)
{
    m_currentName = name;

    wxString title = m_mgr->GetGaugeSymbolTitle( name );

    m_label->SetLabel( title == wxEmptyString ? name : title );

    if(isDropEvt)
        DoOnUpdated();
    else
        SetStyleEx();
}

void SymbolChooserPanel::OnPopupMenuSelectSymbol(wxCommandEvent& evt)
{
    int id = evt.GetId();

    if( m_menuId2symbolname.find(id) == m_menuId2symbolname.end() ) return;

    wxString name = m_menuId2symbolname[id];

    //check
    if(m_currentName == name) return;

    SetSymbolName( name );

    //UpdateSize(true);
}

void SymbolChooserPanel::UpdateSize(bool isSend)
{
    if(!m_label) return;

    wxSize s = DrawHelper::GetTextExtent(m_label->GetLabel(), m_label->GetFont());

    s.IncBy(10,3);

    m_label->SetMinSize(s);

    wxSizer* mainsizer = GetSizer();

    if(mainsizer) {
        mainsizer->Layout();
        wxSize minsize = mainsizer->ComputeFittingWindowSize(this);
        SetSizeHints(minsize);

        wxSize size = GetSize();

        if(size.x < minsize.x || size.y < minsize.y) {
            wxSize newsize;
            newsize.x = wxMax(minsize.x,size.x);
            newsize.y = wxMax(minsize.y,size.y);
            SetSize(newsize);
        }

    }

    if(m_updateSizeTarget && isSend) {
        wxSizer* sizer = m_updateSizeTarget->GetSizer();
        if(sizer) {
            sizer->Layout();
            m_updateSizeTarget->Refresh(false);
        }
    }

}

void SymbolChooserPanel::OnSize(wxSizeEvent& evt)
{
    evt.Skip();

    UpdateSize();
}

void SymbolChooserPanel::OnButton(wxCommandEvent& evt)
{
    wxMenu popupMenu;

    wxSortedArrayString grparr;

    m_mgr->GetSymbolGroupListM(grparr);

    int cid=0;

    for(size_t a=0; a < grparr.GetCount(); a++){

        wxString grpname = grparr[a];

        wxMenu * sgmenu = new wxMenu();

        SymbolGaugeDescArray arr;

        m_mgr->GetSymbolDescs(grpname, arr);

        for(size_t i = 0; i < arr.GetCount(); i++ ) {

            SymbolGaugeDesc * desc = arr[i];

            if(i%20 == 0) sgmenu->Break();

            int id = wxNewId();

            sgmenu->AppendCheckItem(id, desc->Title );

            m_menuId2symbolname[id] = desc->Name;

            if( desc->Title == m_label->GetLabel() ) cid = id;

            Connect( id, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(SymbolChooserPanel::OnPopupMenuSelectSymbol) );

        }

        popupMenu.AppendSubMenu(sgmenu,grpname);
    }

    wxMenu* smenu = new wxMenu();

    wxVariantHashArray& flagGroups = m_mgr->GetVemsFlagDescMgr()->GetGroups();
    VemsFlagDescHash& fhash = m_mgr->GetVemsFlagDescMgr()->GetAllDesc();

    for(size_t a=0;a<flagGroups.GetCount();a++){
        wxVariantHash& group = flagGroups[a];
        smenu->AppendSubMenu(new wxMenu(), group[_T("title")].GetString());
    }

    wxArrayString flags;
    wxVariantHash flag2group;

    for(VemsFlagDescHash::iterator it = fhash.begin(); it != fhash.end(); it++ ) {
        flags.Add(it->first);
        flag2group[it->first] = it->second.Properties[_T("group_title")].GetString();
    }

    flags.Sort();

    for(size_t a=0; a<flags.GetCount(); a++) {

        wxString name = flags[a];

        int id = wxNewId();

        wxMenu* gmenu = smenu->FindItem(smenu->FindItem(flag2group[name].GetString()))->GetSubMenu();
        if(!gmenu) continue;

        if(gmenu->GetMenuItemCount() % 20 == 0) gmenu->Break();

        gmenu->AppendCheckItem(id, name );

        m_menuId2symbolname[id] = name;

        if( name == m_label->GetLabel() ) cid = id;

        Connect( id, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(SymbolChooserPanel::OnPopupMenuSelectSymbol) );
    }

    popupMenu.AppendSubMenu(smenu,_T("Flags"));


    popupMenu.Check(cid, true);

    PopupMenu(&popupMenu);
}
