#include "Graph.h"
#include "../logger.h"
#include "../common/typedef.h"

//#include <iostream>
Graph::Graph()
{
    //ctor
    m_askingNodeSet = NULL;
}

Graph::~Graph()
{
    //dtor
}


/**
* for all nodes {
*    if no edge INTO (node with empty hash) porder = 0
*    else porder = PORDER_UNKNOWN
* }
*/
void Graph::InitializeNodes()
{
    for(uint16_t cnt = 0;cnt < m_nodeArray.GetCount();cnt++){
        bool isroot = !m_nodeArray[cnt]->hasDepending();
        if( isroot ){
            m_nodeArray[cnt]->setPorder(0);
        }else{
            m_nodeArray[cnt]->setPorder(PORDER_UNKNOWN);
        }
    }
}

/**
* Iterate over nodes untill porder is found for all.
* do{
*  all_known = true;
*  for all nodes {
*   if( node != PORDER_UNKNOWN ) continue;
*   if( porder is known for all nodes in nodehash ) porder = NOW_READY;
*    else  all_known = false; // game not over yet
*  }
   set the finel poreder
   for(uint16_t cnt = 0;cnt < m_nodeArray.GetCount();cnt++){
            int node = m_nodeArray[cnt]->getPorder();
            if( node == NOW_READY ){
                m_nodeArray[cnt]->setPorder(porder_now);
            }
    }
*  porder_now ++;
* } while (all_known == false);
*/

void Graph::setupOrder()
{
    InitializeNodes();
    int porder_now = 1;
    bool all_known = true ;
    do{
        all_known = true;
        for(uint16_t cnt = 0;cnt < m_nodeArray.GetCount();cnt++){
            int node = m_nodeArray[cnt]->getPorder();
            if( node != PORDER_UNKNOWN ){
                continue;
            }
            bool allKnowInNodHash = m_nodeArray[cnt]->isAlledgeKnown(porder_now);
            if( allKnowInNodHash ){
                m_nodeArray[cnt]->setPorder(porder_now);
            }else {
                all_known = false; // game not over yet
            }
        }
        porder_now ++;
     } while (all_known == false);
}

void Graph::addNode(GraphNode* graphNodePtr)
{
     m_nodeArray.Add(graphNodePtr);

}

void Graph::removeNode(GraphNode* graphNodePtr)
{
    //GraphNode *delgraphNodePtr;
    //uint16_t idx = m_nodeArray.Index(graphNodePtr);
    //delgraphNodePtr = m_nodeArray[idx];
    //delete delgraphNodePtr;
    m_nodeArray.Remove(graphNodePtr);
}

void Graph::getMaxGraphLevel(const GraphNodeSet&  askingGraphNodeSet, int& idx) const
{
    GraphNodeSet::const_iterator IT = askingGraphNodeSet.begin();
	for(; IT !=askingGraphNodeSet.end(); IT++) {
        GraphNode* ptr = *IT;
        if ( ! ptr )
            continue;
        uint16_t idx_now = ptr->getPorder();
        if(idx < idx_now){
            idx = idx_now;
        }
    }
}

GraphNodeSetArray* Graph::getNodeDedepending(const GraphNodeSet&  askingGraphNodeSet) const
{
    int idx = 0;
    /**search max idx*/
    getMaxGraphLevel(askingGraphNodeSet, idx);

    GraphNodeSetArray *graphDedependingNodeSetArray = new GraphNodeSetArray();
    GraphNodeSet  downlevelGraphNodeSet;
    GraphNodeSet *levelGraphNodeSet;
    //GraphNodeSet *takeinGraphNodeSet;

    for(uint16_t cnt = 0; cnt <=idx; cnt++){
        levelGraphNodeSet = new GraphNodeSet();
        graphDedependingNodeSetArray->Add(levelGraphNodeSet);
    }
    /**take the elems of askingGraphNodeSet in good place of graphDedependingNodeSetArray*/
    GraphNodeSet::const_iterator IT = askingGraphNodeSet.begin();
    for(; IT !=askingGraphNodeSet.end(); IT++) {
        GraphNode* ptr = *IT;
        if ( ! ptr )
            continue;
        int idx_now = ptr->getPorder();
        (*graphDedependingNodeSetArray)[idx_now]->insert(ptr);
    }

    for( ; idx != 0; idx--){
        levelGraphNodeSet = (*graphDedependingNodeSetArray)[idx];
        /**get all elems  current level*/
        GraphNode *ptr1;
        for(GraphNodeSet::iterator it1 = levelGraphNodeSet->begin(); it1 !=levelGraphNodeSet->end(); it1++) {
            ptr1 = *it1;
            downlevelGraphNodeSet = ptr1->getDependNodeSet();
            /**set all elems  down level*/
            GraphNode *ptr2;
            for(GraphNodeSet::iterator it2 = downlevelGraphNodeSet.begin(); it2 !=downlevelGraphNodeSet.end(); it2++) {
                ptr2 = *it2;
                int idx2 = ptr2->getPorder();
                (*graphDedependingNodeSetArray)[idx2]->insert(ptr2);
            }
        }

    }
    return graphDedependingNodeSetArray;
}

void Graph::Test()
{
    GraphNode *graphNodePtr;
	uint16_t cnt;
    for( cnt = 0; cnt < 10 ;cnt++){
        graphNodePtr = new GraphNode();
        m_nodeArray.Add(graphNodePtr);
    }
    m_nodeArray[5]->insertNode(m_nodeArray[1]);
    m_nodeArray[9]->insertNode(m_nodeArray[5]);
    m_nodeArray[9]->insertNode(m_nodeArray[1]);
    m_nodeArray[9]->insertNode(m_nodeArray[2]);
    m_nodeArray[9]->insertNode(m_nodeArray[3]);
    m_nodeArray[9]->insertNode(m_nodeArray[4]);
    m_nodeArray[9]->insertNode(m_nodeArray[6]);
    m_nodeArray[8]->insertNode(m_nodeArray[9]);
/*
    setupOrder();
    for(int cnt = 0; cnt < 10 ;cnt++){
        int value = m_nodeArray[cnt]->getPorder();
        printf("cnt = %d value = %d\n",cnt,value);

    }
*/
    GraphNodeSet  askingGraphNodeSet;
    askingGraphNodeSet.insert(m_nodeArray[8]);
    askingGraphNodeSet.insert(m_nodeArray[5]);
    //askingGraphNodeSet.insert(m_nodeArray[8]);
    GraphNodeSetArray  *test = getNodeDedepending(askingGraphNodeSet);


    printDepending(*test);
/*
    for( cnt = 0; cnt < test->GetCount();cnt++){
        int value = (*test)[cnt]->size();
        printf("cnt = %d value = %d\n",cnt,value);
    }
*/

}

void Graph::clear()
{
    m_nodeArray.Clear();
}


void Graph::printDepending(const GraphNodeSetArray& nodeSetA) const
{
    for( unsigned int i = 0; i < nodeSetA.GetCount(); i++){
        int size = nodeSetA.Item(i)->size();
        Logger::logf(LOG_MSG, _T("Graph::printDepending() idx: %d size: %d"),i,size);
        GraphNodeSet::iterator it = nodeSetA.Item(i)->begin(), e = nodeSetA.Item(i)->end();
        for ( ; it != e; ++it )
            Logger::logf( LOG_MSG, _T("--> name=") + (*it)->getName() );
    }
}
