#ifndef EEPROMCFGSAVERBASE_H
#define EEPROMCFGSAVERBASE_H

#include "../globalhparser/VariableDescriptor.h"
#include "../common/types/String2IntHash.h"

class GeneralEvent;
class SymbolMgr;
class SymbolTable;
class wxTextFile;
class HeaderParser;
class bcProgressDialog;

class EepromCfgSaverBase : public wxEvtHandler
{
    public:
        EepromCfgSaverBase();
        EepromCfgSaverBase( SymbolMgr* symbolMgr );

        virtual ~EepromCfgSaverBase();

    public:
        enum { EEPROM_SAVE_OK = 0, EEPROM_SAVE_ERROR, EEPROM_SAVE_PROGRESS };

    public:
        void    parseGlobalH( const wxString& headerfile, const wxString& mymakefile );

        void    parseMapFile( const wxString& mapfile );

        void    setSymbolMgr( SymbolMgr* symbolMgr ) { m_symbolMgr = symbolMgr; }

        void    save( bool createProgressDialog, bcProgressDialog* dlg = NULL );

    protected:
        unsigned int            m_size;
        unsigned char           m_eepromdata[4096];

    protected:
        virtual void        getEepromData() = 0;
        virtual wxString    toString() = 0;

    private:
        String2StructVarHash    m_parsedstructures;
        String2IntHash          m_structAddresses;
        SymbolMgr*              m_symbolMgr;

        HeaderParser*           m_headerparser;

        bcProgressDialog*       m_progDialog;
        bool                    m_selfCreatedPrgDlg;


    private:
        void    _saveintern();
        void    saveToCfgFile( SymbolTable* symTable );
        void    addHeader( wxTextFile& f );

        void    onGeneralEvent( GeneralEvent& event );

        DECLARE_EVENT_TABLE()
};

#endif // EEPROMCFGSAVERBASE_H
