#ifndef GENERALEVENT_H
#define GENERALEVENT_H

#include <wx/wx.h>
#include "../common/types/wxVariantHash.h"

extern const wxEventType wxEVT_GENERAL;    /// without this MinGW gives undifened referenced linker error

#define EVT_GENERAL(id, func) \
    wx__DECLARE_EVT1(wxEVT_GENERAL, id, GeneralEventHandler(func))

class GeneralEvent : public wxEvent
{
    public:
        GeneralEvent( int id = 0 );
        virtual wxEvent *Clone() const { return new GeneralEvent( *this ); }

        long            GetLong() const { return m_lvalue; }
        wxString        GetString() const { return m_svalue; }
        double          GetDouble() const { return m_dvalue; }
        wxVariant       GetVariant() const { return m_vvalue; }
        wxVariantHash   GetVariantHash() const  { return m_vhvalue; }
        void*           GetVoidPtr() const { return m_pvalue; }

        void            SetLong( const long& value ) { m_lvalue = value; }
        void            SetString( const wxString& value ) { m_svalue = value; }
        void            SetDouble( const double& value ) { m_dvalue = value; }
        void            SetVariant( const wxVariant& value ) { m_vvalue = value; }
        void            SetVariantHash( const wxVariantHash& value ) { m_vhvalue = value; }
        void            SetVoidPtr( void* value ) { m_pvalue = value; }

    public:
        DECLARE_DYNAMIC_CLASS(GeneralEvent)

    private:
        long            m_lvalue;
        wxString        m_svalue;
        double          m_dvalue;
        wxVariant       m_vvalue;
        wxVariantHash   m_vhvalue;
        void*           m_pvalue;
};

typedef void (wxEvtHandler::*GeneralEventFunction)(GeneralEvent&);

#define GeneralEventHandler(func) \
    (wxObjectEventFunction)(wxEventFunction)wxStaticCastEvent(GeneralEventFunction, &func)

#endif // GENERALEVENT_H
