#ifndef VEMSDEVICECONTROLLEREVENT_H
#define VEMSDEVICECONTROLLEREVENT_H

#include <wx/event.h>

#include "../vemsprotocol/DeviceInfo.h"

extern const wxEventType wxEVT_VEMS_DEVICE_CONTROLLER;    /// without this MinGW gives undifened referenced linker error

#define EVT_VEMS_DEVICE_CONTROLLER(id, func) \
    wx__DECLARE_EVT1(wxEVT_VEMS_DEVICE_CONTROLLER, id, VemsDeviceControllerEventHandler(func))

enum VemsDeviceControllerEventType
{
    VDC_EVENT_UNDEFINED = -1,
    VDC_EVENT_DEVICE_FOUND,
    VDC_EVENT_DEVICE_NOT_FOUND,
    VDC_EVENT_DEVICE_CHECKING_PASSWORD,
    VDC_EVENT_DEVICE_PROTECTED,
    VDC_EVENT_DEVICE_NOT_PROTECTED,
    VDC_EVENT_DEVICE_FIRMWAREINFO_FETCHING,
    VDC_EVENT_DEVICE_FIRMWAREINFO_FETCHED,
    VDC_EVENT_DEVICE_FIRMWAREDATA_SYNCING,
    VDC_EVENT_DEVICE_FIRMWAREDATA_SYNCED,
    VDC_EVENT_DEVICE_CONNECTED,
    VDC_EVENT_DEVICE_DISCONNECTED,
    VDC_EVENT_DEVICE_RECONNECTED,
    VDC_EVENT_DEVICE_CANNOT_CONNECT,
    VDC_EVENT_STARTED,
    VDC_EVENT_STOPPED,
    VDC_EVENT_PAGE_SYNCED,
    VDC_EVENT_DEVICE_BAUDRATE_CHANGED,
    VDC_EVENT_CANNOT_OPEN_PORT,
    VDC_EVENT_START_RESUMING,
    VDC_EVENT_RESUMED,
    VDC_EVENT_DEVICE_MODE_CHANGED,
    VDC_EVENT_LEAVING_BOOT_MODE,
};


class VemsDeviceControllerEvent : public wxEvent
{
    public:
        VemsDeviceControllerEvent( int id = 0 ) : wxEvent( id, wxEVT_VEMS_DEVICE_CONTROLLER ) {}
        virtual wxEvent *Clone() const { return new VemsDeviceControllerEvent(*this); }
        DECLARE_DYNAMIC_CLASS(VemsDeviceControllerEvent)

        void setDeviceInfo( const DeviceInfo& deviceInfo ) { m_deviceInfo = deviceInfo; }
        void setDevice( ECUInfo::Device device ) { m_deviceInfo.device = device; }
        void setDeviceMode( ECUInfo::EcuMode deviceMode ) { m_deviceInfo.deviceMode = deviceMode; }
        void setPortName( const wxString& portName ) { m_deviceInfo.setPortName( portName ); }
        void setBaudrate( size_t baudrate ) { m_deviceInfo.setBaudrate( baudrate ); }
        void setScanningFinished( bool finished ) { m_isScanningFinished = finished; }

        DeviceInfo getDeviceInfo() const { return m_deviceInfo; }
        ECUInfo::Device     getDevice() const { return m_deviceInfo.device; }
        ECUInfo::EcuMode    getDeviceMode() const { return m_deviceInfo.deviceMode; }

        wxString    getPortName() const { return m_deviceInfo.getPortName(); }
        size_t      getBaudrate() const { return m_deviceInfo.getBaudrate(); }
        bool        isScanningFinished() const { return m_isScanningFinished; }

        bool isV3() const { return m_deviceInfo.isV3(); }
        bool isRound() const { return m_deviceInfo.isRound(); }

        bool isBootMode() const { return m_deviceInfo.isBootMode(); }
        bool isManMode() const { return m_deviceInfo.isManMode(); }
        bool isMegatuneMode() const { return m_deviceInfo.isMegatuneMode(); }

        wxString toString() const;

    private:
        DeviceInfo  m_deviceInfo;
        bool        m_isScanningFinished;
};

typedef void (wxEvtHandler::*VemsDeviceControllerEventFunction)(VemsDeviceControllerEvent&);

#define VemsDeviceControllerEventHandler(func) \
    (wxObjectEventFunction)(wxEventFunction)wxStaticCastEvent(VemsDeviceControllerEventFunction, &func)

#endif // VEMSDEVICECONTROLLEREVENT_H
