#include "ExpressionFvInterpolate2D.h"
#include "VariableData.h"
#include "../VemsTuneHelper.h"
#include "../logger.h"
#include "../common/VemsMatrixHelper.h"

ExpressionFvInterpolate2D::ExpressionFvInterpolate2D()
{
    setType( FUNCTION );
}


void ExpressionFvInterpolate2D::interpret( RPNStack& rpnStack )
{
    RPNStackItem* valueX = rpnStack.pop();
    RPNStackItem* tableY = rpnStack.pop();
    RPNStackItem* tableX = rpnStack.pop();

    if ( valueX->getType() != RPNStackItem::NUMBER || tableY->getType() != RPNStackItem::STRING ||
            tableX->getType() != RPNStackItem::STRING )
    {
        delItem( &valueX, &tableX, &tableY, NULL );
        throw ExpError::VariableException( _T("function interpolate2d parameter mismatch") );
    }

    if ( !VemsTuneHelper::staticVariables )
    {
        delItem( &valueX, &tableX, &tableY, NULL );
        Logger::logf( LOG_MSG, _T("-----ExpressionFvInterpolate2D::interpret() - VemsTuneHelper::staticVariables is NULL!!!") );
        return;
    }

    VemsMatrix<double> rows;

    if( !VemsTuneHelper::staticVariables->getValue( tableY->getString(), rows ) )
    {
		wxString varname = tableY->getString();
        delItem( &valueX, &tableX, &tableY, NULL );
        throw ExpError::VariableException( _T("variable not found : ") + varname );
    }

    //VemsMatrixHelper::print( rows );

    VemsMatrix<double> cols;

    if( !VemsTuneHelper::staticVariables->getValue( tableX->getString(), cols ) )
    {
		wxString varname = tableX->getString();
        delItem( &valueX, &tableX, &tableY, NULL );
        throw ExpError::VariableException( _T("variable not found : ") + varname );
    }

    //VemsMatrixHelper::print( cols );

    double result = VemsMatrixHelper::interpolate( rows, cols, valueX->getNumber() );

    //Logger::logf( LOG_MSG, _T("-----ExpressionFvInterpolate2D::interpret() - x: %f, y: %f"), valueX->getNumber(), result );

    RPNStackItem* item = new RPNStackItem();
    item->setNumber( result );
    rpnStack.push( item );

    delItem( &valueX, &tableX, &tableY, NULL );
}
