#include "FirmwareConfigParser.h"
#include "parserutil.h"

#include "../Math.h"
#include "../Utils.h"
#include "../logger.h"
#include "../RawLogHelper.h"

#include "../firmwareconfig/FirmwareConfigData.h"

#include <wx/regex.h>

FirmwareConfigParser::FirmwareConfigParser( FirmwareConfigData* fcData )
{
    m_ptrFcData = fcData;
    //m_ptrFcData->m_configLineCount = 0;

}

void FirmwareConfigParser::parseLine()
{
    parseHeader();

    ///parse header
    if ( m_line.StartsWith( _T("#") ) )
    {
        wxString s = m_line.AfterFirst( _T('#') ).Trim( false );
        wxString key = s.BeforeFirst( _T(':') );
        wxString value = s.AfterFirst( _T(':') );
        m_ptrFcData->header[key] = value;
        return;
    }

    wxString key, value;

    if ( ParserUtil::getKeyAndValue( m_line, key, value ) != 0 ) { return; }

    if ( getSymbolName( key ).Length() == 1  ) { parseTableLine(); }
    else { parseConfigLine( key, value ); }

    return;
}

wxString FirmwareConfigParser::getSymbolName( const wxString& key )
{
    return key.Contains( _T("[") ) ? key.BeforeFirst( _T('[') ) : key;
}


void FirmwareConfigParser::parseHeader()
{
    //Logger::logf( LOG_MSG, _T("-----FirmwareConfigParser::parseHeader() - START") );
    /// # Version: 1.1.27
    //wxString versionRegExp = _T("Version.*(\\d+\\.\\d+\\.\\d+)");
    wxRegEx version( _T("Version:.*(\\d+\\.\\d+\\.\\d+)"), wxRE_ICASE | wxRE_ADVANCED );
    wxRegEx version2( _T("Version:.*(\\d+\\.\\d+)"), wxRE_ICASE | wxRE_ADVANCED );
    //reVersion.Compile( versionRegExp,   );

    if( version.IsValid() || version2.IsValid() )
    {
        //Logger::logf( LOG_MSG, _T("-----FirmwareConfigParser::parseHeader() - version regexp is valid") );
        if ( version.Matches( m_line ) )
        {
            wxString fwver = version.GetMatch( m_line, 1 );
            m_ptrFcData->setFWVersion( fwver );
            //Logger::logf( LOG_MSG, _T("FirmwareConfigParser::parseHeader() - v: ") + m_ptrFcData->getFWVersion() );
        }
        else if ( version2.Matches( m_line ) )
        {
            wxString fwver = version2.GetMatch( m_line, 1 );
            fwver += _T(".0");
            m_ptrFcData->setFWVersion( fwver );
        }
    }
}


void FirmwareConfigParser::parseConfigLine( const wxString& key, const wxString& value )
{

    if(!key.Contains(_T("[G]")) ){
        m_ptrFcData->addScalar( key, Math::hex2int( value ), m_lineNumber );
    }else{
        wxString keyPreffix = key.BeforeFirst( _T('[') );
        m_ptrFcData->addScalar( keyPreffix + _T("[10]"), Math::hex2int( value ), m_lineNumber );
    }
    m_ptrFcData->m_configLineCount++;
}

void FirmwareConfigParser::parseTableLine()
{
    wxArrayString tokens;
    Utils::tokenizeToArray( m_line, _T("="), tokens );

    //invalid table line
    if ( tokens.GetCount() != 2 ) { return; }

    wxString strKey         = tokens[ 0 ];
    wxString strTableValues = tokens[ 1 ];

    tokens.Clear();
    Utils::tokenizeToArray( strTableValues, _T(" "), tokens );

    wxString strTableName;
    wxString strRowIndex;

    //get symbol name and row index
    wxString strRegExp = _T("^\\s*([^\\[]+)\\[(\\w+)\\]");
    ParserUtil::getCommonNameValue( strKey, strRegExp, strTableName, strRowIndex );

    //set table size and data
    if(strRowIndex == _T("r")){
        strTableName +=_T("_r");
        strRowIndex = _T("0");
    }else if(strRowIndex == _T("k")){
        strTableName +=_T("_k");
        strRowIndex = _T("0");
    }
    /*
    if(strTableName == _T("j") ){
        if( tokens.GetCount() == 12){   ///12x12
            if( strRowIndex ==_T("r") ) strRowIndex = _T("C");
            if( strRowIndex ==_T("k") ) strRowIndex = _T("D");
        }else{/// VE should always 16x14
            if( strRowIndex ==_T("r") ) strRowIndex = _T("E");
            if( strRowIndex ==_T("k") ) strRowIndex = _T("F");
        }
    }

    if(strTableName == _T("n") ){   /// Spark always 12x12
        if( strRowIndex ==_T("r") ) strRowIndex = _T("C");
        if( strRowIndex ==_T("k") ) strRowIndex = _T("D");
    }
    if(strTableName == _T("l") ){   /// Lambda always 8x8
        if( strRowIndex ==_T("r") ) strRowIndex = _T("8");
        if( strRowIndex ==_T("k") ) strRowIndex = _T("9");
    }
    */
    size_t uRowIndex    = Math::hex2int( strRowIndex );
    if(strTableName == _T("h")){
        strTableName = _T("h") + wxString::Format(_T("%i"), uRowIndex);
        m_ptrFcData->addTableLine( strTableName, 0, tokens );
    }else{
        m_ptrFcData->addTableLine( strTableName, uRowIndex, tokens );
    }
}

void FirmwareConfigParser::parseFiles( const wxArrayString& files )
{
    for ( size_t i = 0; i < files.GetCount(); ++i )
    {
        parse( files[i] );
    }
}


void FirmwareConfigParser::parseHeaderOnly( const wxString& fileName )
{
    if ( openFile( fileName ) ) { return; }

    do {
        parseNextLine();
    } while ( m_line.StartsWith( _T("#") ) );

    closeFile();
}
