#include "MapFileParser.h"

#include <wx/regex.h>

void MapFileParser::parseLine()
{
    wxString& str = m_line;

    // Match: /^(\S+)\s+0x([a-fA-F\d]{1,3})\s+/
    wxRegEx regexp(_T("^(\\S+)\\s+0x([a-fA-F\\d]{1,3})\\s+"), wxRE_ADVANCED );
    if ( regexp.Matches( str ) )
    {
        wxString key = regexp.GetMatch( str, 1 );
        wxString size_hex = _T("0x") + regexp.GetMatch( str, 2 );

        MapSymbol symbol;

        // convert size from hex to dec
        size_hex.ToLong( &symbol.size, 16 );
        symbol.name = key;

        m_symbols[key] = symbol;
    }
    else
    {
        // /0x00800([a-fA-F\d]{3})\s+(\S+)/
        regexp.Compile( _T("0x00800([a-fA-F\\d]{3})\\s+(\\S+)"), wxRE_ADVANCED );

        if ( regexp.Matches( str ) )
        {
            wxString key = regexp.GetMatch( str, 2 );
            wxString addr_hex = _T("0x") + regexp.GetMatch( str, 1 );

            MapSymbolHash::iterator it = m_symbols.find( key );

            if ( key.Find( _T("mmcbuff") ) != wxNOT_FOUND )
            {
                long int addr = -1;

                addr_hex.ToLong( &addr, 16 );

                MapSymbol symbol( 514, addr, key );

                m_symbols[key] = symbol;
            }
            // get size of the sybol
            else if ( it != m_symbols.end() )
            {
                long int addr = -1;

                addr_hex.ToLong( &addr, 16 );

                m_symbols[key].address = addr;
            }
            else
            {
                // something wrong in map file
            }
        }
        // try 64bit
        else
        {
            // /0x00800([a-fA-F\d]{3})\s+(\S+)/
            regexp.Compile( _T("0x0000000000800([a-fA-F\\d]{3})\\s+(\\S+)"), wxRE_ADVANCED );
            if ( regexp.Matches( str ) )
            {
                wxString key = regexp.GetMatch( str, 2 );
                wxString addr_hex = _T("0x") + regexp.GetMatch( str, 1 );

                MapSymbolHash::iterator it = m_symbols.find( key );

                if ( key.Find( _T("mmcbuff") ) != wxNOT_FOUND )
                {
                    long int addr = -1;

                    addr_hex.ToLong( &addr, 16 );

                    MapSymbol symbol( 514, addr, key );

                    m_symbols[key] = symbol;
                }
                // get size of the sybol
                else if ( it != m_symbols.end() )
                {
                    long int addr = -1;

                    addr_hex.ToLong( &addr, 16 );

                    m_symbols[key].address = addr;
                }
                else
                {
                    // something wrong in map file
                }
            }
            else
            {
                // neglected line
            }
        }
    }
}
