#ifndef PARSER_H
#define PARSER_H

#include "parserexception.h"
#include "parserbase.h"
#include "V3iniTokenizer.h"

class BoolHash;
class CondTree;
class Symbol;
class MTConfigData;

const wxString MT_PARAMS                = _T("Params");
const wxString MT_MEGATUNE              = _T("MegaTune");
const wxString MT_CONSTANTS             = _T("Constants");
const wxString MT_MENU                  = _T("Menu");
const wxString MT_TABLEMAPPER           = _T("TableMapper");
const wxString MT_TABLEEDITOR           = _T("TableEditor");
const wxString MT_CURVEEDITOR           = _T("CurveEditor");
const wxString MT_SUPER_CURVE_EDITOR    = _T("SuperCurve");
const wxString MT_GAUGECONFIG           = _T("GaugeConfigurations");
const wxString MT_FRONTPAGE             = _T("FrontPage");
const wxString MT_RUNTIME               = _T("RunTime");
const wxString MT_TUNING                = _T("Tuning");
const wxString MT_USERDEFINED           = _T("UserDefined");
const wxString MT_OUTPUTCHANNELS        = _T("OutputChannels");
const wxString MT_DATALOG               = _T("Datalog");
const wxString MT_GPSDATA               = _T("GPSData");
const wxString MT_AIMDATA               = _T("AIMData");
const wxString MT_SIPR_DESCRIPTOR       = _T("SIPR");
const wxString MT_SIPR_DATA             = _T("SIPRData");
const wxString MT_SYMBOL_ALIAS          = _T("SymbolAlias");
const wxString MT_SAVING_ORDER          = _T("ConfigTablesSavingOrder");
const wxString MT_USER_DATA             = _T("UserData");
const wxString MT_EXPRESSION_GRID       = _T("ExpressionGrid");
const wxString MT_INTERPOLATION_WIDGET  = _T("InterpolationWidget");


class V3iniParser;

class V3iniParserPriv
{
    public: V3iniParserPriv(MTConfigData* mtConfigData, BoolHash* conditionTable);

    virtual ~V3iniParserPriv();

    private:
        MTConfigData*   m_MTConfigData;
        BoolHash*       m_conditionTable;

        long*            m_currentPage;
        wxString*        m_currentSection;
        CondTree*        m_currentCT;
        bool*            m_currentCondValue;
        wxString*        m_logMsg;
        wxArrayString*   m_errorLog;
		bool			m_hasOwnConditionTable;

        friend class V3iniParser;
};

class V3iniParser :public ParserBase
{
    public:
        V3iniParser(MTConfigData* data, BoolHash* boolHash);
        V3iniParser(V3iniParserPriv* priv);

        virtual ~V3iniParser();
        void parseLine();
    private:
         V3iniParser(const V3iniParser&);
        V3iniParser& operator=(const V3iniParser&);

        bool m_del;
        V3iniParserPriv* m_priv;

        MTConfigData& getMTConfig(){ return *m_priv->m_MTConfigData; }
        BoolHash*     getCondTable(){ return m_priv->m_conditionTable; }
        long&         getPage(){ return *m_priv->m_currentPage; }
        wxString&     getSection(){ return *m_priv->m_currentSection; }
        CondTree*     getCondTree(){ return m_priv->m_currentCT; }
        bool&         getCondValue(){ return *m_priv->m_currentCondValue; }
        wxString&     getLogMsg(){ return *m_priv->m_logMsg; }
        wxArrayString&   getErrorLog(){ return *m_priv->m_errorLog; }

        void        setCondTree(CondTree* ct){ m_priv->m_currentCT = ct; }
        long        lookupOffset(const wxString& str);

    /*
        MTConfigData*   m_MTConfigData;
        BoolHash*       m_conditionTable;
        long            m_currentPage;
        wxString        m_currentSection;
        CondTree*       m_currentCT;
        bool            m_currentCondValue;
        wxString        m_logMsg;
        wxArrayString   m_errorLog;
*/




        void parseParams( const Tokenizer& t );
        void parseMegatune(const Tokenizer& t);
        void parseConstants(const Tokenizer& t);
        //symbol spec
        void parseTypeSpec(Symbol* symbol, const wxString& typeField);
        void parseArraySpec(Symbol* symbol, const wxString& arrayspec );
        void parseBitSpec(Symbol* symbol, const Tokenizer& t);

        void parseMenu(const Tokenizer& t);
        void parseTableMapper( const Tokenizer& t );
        void parseTableEditor( const Tokenizer& t );
        void parseCurveEditor( const Tokenizer& t );
        void parseSuperCurveEditor( const Tokenizer& t );
//        void parseGaugeConfig(const Tokenizer& t);
        void parseFrontPage();
        void parseRunTime();
        void parseTuning();
        void parseUserDefined(const Tokenizer& t);
        void parseGPSData( const Tokenizer& t );
        void parseAIMData( const Tokenizer& t );
        void parseSIPRDescriptor( const Tokenizer& t );
        void parseSIPRData( const Tokenizer& t );
        void parseOutputChannels(const Tokenizer& t);
        void parseDatalog(const Tokenizer& t);
        void parseSymbolAlias( const Tokenizer& t );
        void parseConfigTablesSavingOrder( const Tokenizer& t );
        void parseUserData( const Tokenizer& t );
        void parseExpressionGrid( const Tokenizer& t );
        void parseInterpolationWidget( const Tokenizer& t );

        //utility methods
        int tokenizeToArray(wxString& text, const wxString& token, wxArrayString& a);
        int convertTokenTypeToInterpolationWidgetFieldType( type tokenType );
};

#endif
