#include "trigger.h"

#include "../common/typedef.h"

#define _BV(x) (1<<x)

#define isvalid_rpm_period() (((uint16_t)engine.rpm_period) != 0xffff)

Trigger::Trigger():
IGN_BALANCE (256)
{
    //ctor

}

Trigger::~Trigger()
{
    //dtor
}

#define DEC_MODULO(x)  if( x-- == 0 ) x = totalteeth-1;
#define INC_MODULO(x)  if( ++x >= totalteeth) x = 0;
/**
    Calculate trigger tooth N, and the angle from tooth N to spark 'trig2spark'.

    Example:
    Say trigger tooth is defined (in config) as 64 degrees BTDC (ign_tdcdelay=128)
    and an advance of 21 degrees is requested.
    The proposed tooth will be 4 tooth after the trigger tooth and 3 degrees remains 64-21=43
    Also calculate dwell-tooth and delay.

    Care is needed with (modulo 256 arithmetics):
    tooth_wheel_twidth1 =0x60 decimal  96
    tooth_wheel_twidth2 =0xC0 decimal 192
	// Since 26 * 96 + 2 * 96 * 2 = 2880  (that is 720 * 4)
	reset_engphase_after = 0x40 = decimal 64 = (2880 / 256 -11) * 256

	TODO: precision problem with 360 / 135 = 2.667 degree (use more bits)
		360 / 135 * 64 = 170.67
		360 / 135 * 256 = 682.67
  */
void Trigger::triggertooth(uint16_t adv,const FirmwareTriggerConfig& config, Engine& engine)
{
        this->config = config;
        this->engine = engine;
            uint8_t trigtooth; // 8 bit
		uint16_t toothw16;
// for many conditions we only care about lower byte:
#define toothw8 (toothw16 & 0xFF)
            uint16_t delaydeg; // 16 bit,  1/4 crankdeg
            uint16_t delay, dwell; // 4 usec
            uint16_t trig2dwelltime = 0;
	    uint16_t toothtime = 0; // 16 bit, 4 usec
	    uint8_t totalteeth, catt; // total number of teeth (including missing), config.another_trigger_tooth

		delaydeg = config.ign_tdcdelay;
		delaydeg <<= 1;	// going 1/4 crankdeg resolution
		delaydeg += IGN_BALANCE;

            if(adv > delaydeg){
                delaydeg = 0; // argh, the user configured higher advance than ign_tdcdelay
            } else {
                // normal branch
                delaydeg -= adv;
            }

            // with trigtooth =0
            // resulting trigtooth would be relative (after trigger_tooth[n])
	    // we could drop config.trigger_tooth but have huge problems with avoiding missing tooth
            trigtooth = config.trigger_tooth; // might be 0
            // trigtooth-- might underflow below (negative result), no problem.

		toothw16 = config.toothw16;
		totalteeth = get_total_tooth();
            // count the number of teeth in delaydeg
	    if( toothw16 ){ // beware of infloop
            	while (delaydeg > toothw16) {
             	   delaydeg -= toothw16;
             	   INC_MODULO(trigtooth);
            	}
	    }

            // take care of the missing tooth:
	    catt = config.another_trigger_tooth;
	if((config.primary_trigger & _BV(ptrig_coil)) == 0) { // only for missing tooth trigger :
            while( avoid_tooth( trigtooth, catt ) ){
                    // go back to avoid missing-tooth and correct tdcdelaydeg:
                    delaydeg += toothw16;
                    DEC_MODULO( trigtooth );
            }
	}

            // convert angle to 4 usec time
	delay = qdegree_to_4usec( delaydeg, toothw8 );
	if( (!isvalid_rpm_period()) || (delay == RPM_TOO_LOW )){
	    	// oops, flag error condition:
		cli();
	    	engine.proposed_spark_phase =
			engine.proposed_dwell_phase = INVALID_TRIGGER_TOOTH;
		sei();
		// we do NOT set proposed_trig2spark and trig2dwell
		// but we let vems.c handle the switchoff, so NO:
		// ign_deactivate_all();
		// TODO: maybe tweak engine.sparkat ? like engine.sparkat = etime.now ?
		// or do that conditionally, only if isCoilActive()
		// and sparkat is more than 5 msec to the future (eg. 20 msec in the past == 242 msec future)
#ifdef BENCHMARK
		benchstats.wheel_still++;
#endif  // BENCHMARK
		return;
	    }

	    cli(); // ATOMICITY !! update both variables at the same time !
		engine.proposed_spark_phase = trigtooth;
            	engine.proposed_trig2spark = delay;
            sei();

            // up to now we were playing angle (quarterdegrees), for spark-angle.
	    // ================================== DWELL calc: ========================

	    // toothtime should NEVER be RPM_TOO_LOW, right ?
	    // TODO: rethink for coil-type 4 cyl
            toothtime = qdegree_to_4usec( toothw16, toothw8 );

            dwell = engine.dwell;

		//  dwell should be limited according to periods between dwells on same channel
		//  dwell = limit_dwell(engine.dwell);
            if( (dwell < delay) || (toothtime == 0)){
                // same teeth as used for spark
		cli();  // atomicity !! both vars !!!
                	engine.proposed_dwell_phase = trigtooth;
                	engine.proposed_trig2dwell = delay-dwell;  // using 4 usec resolution,
		sei();
		// TODO: we could set a flag near proposed_spark_phase
		// that flags it's the same tooth as dwell.
		// Or better to just detect that at dwelltime ?
		return;
            } else {
                dwell -= delay;
		// toothtime checked above (non-0) so no infloop
                while( dwell >= toothtime ){
                   	dwell -= toothtime;
                   	DEC_MODULO( trigtooth );
                }

                // go back one, draw it and you'll see why:
                DEC_MODULO( trigtooth );
                trig2dwelltime = toothtime - dwell;
		if((config.primary_trigger & _BV(ptrig_coil)) == 0) { // only for missing tooth trigger :

	// TODO: for auditrigger at high RPM (>=4000), also avoid
	// dwell-tooth that wants to be same as spark-tooth:
	// if ( engine.rpm >= 40 && auditrigger_configenabled()
	// 	&& trigtooth == engine.proposed_spark_phase )
                	// take care of the missing tooth:
			while( avoid_tooth( trigtooth, catt ) ){
                        	trig2dwelltime += toothtime;
                        	DEC_MODULO( trigtooth );
			}
		}

		if( trigtooth == engine.proposed_spark_phase ){
			// dwell and spark timed from same tooth:
			// take care, dwell should come earlier:
			if( trig2dwelltime >= engine.proposed_trig2spark ){
				// something is very wrong, better than nothing:
#ifdef BENCHMARK
				benchstats.unknownevent += 0x10;
#endif
				trig2dwelltime = 0;
			}
		}

		cli(); // Atomicity ! both vars !!!
	    		engine.proposed_dwell_phase = trigtooth;
                	engine.proposed_trig2dwell = trig2dwelltime; // 4usec resolution, instead of angle;
		sei();
            }
}

/**
Maybe superfluously precise calc when doing "delaydeg -= toothw;" in triggertooth()

Therefore we cheat, and slightly change degree from orig 0.25 crankdeg.

for audi 135  tooth use 0B (decimal 11) :
precise 2.6667 degrees will be used (not 2.75). delaydeg unit 3%  smaller than 0.25 deg

This could be handled in MegaTune: deviate in n[] table, and ignadv logging (knock and idle-ignadv adjust are not worth to compensate, no significant difference there).
We could neglect the difference: 29 degree instead of 30 degree advance: not a big deal, even same MegaTune would work, we could mention this in the docs (but noone would notice).

for volvo 142 tooth use 0A (decimal 10) :
precise 2.5352 degrees will be used (not 2.5) . delaydeg unit 1.4% higher than 0.25 deg

qdegree_to_4usec() is also modified.
In the case when we mult >1.0 value here, qdegree_to_4usec() multiplies <1.0
so the final result is the same.

*/
uint16_t Trigger::delaydeg_rescale(uint16_t delaydeg, uint8_t toothw )
{
	uint16_t t16;
	if( audi135( toothw )){
		// delaydeg *= 2.75 / 2.6667  which is *33/32
		t16 = ( delaydeg < 63549) ? (mult16_8( delaydeg, 33 ) >> 5) : 65535;
		return t16;
	}

	if( audi142( toothw )){
		// delaydeg *= 2.5 / 2.5352  which is *  252.45 /256
		t16 = mult16_8( delaydeg, 252 ) >> 8;
		return t16;
	}

	return delaydeg;
}


uint8_t Trigger::audi135( uint8_t toothw )
{
	return ( toothw == 11 ? 1 : 0);
}

uint8_t Trigger::audi142( uint8_t toothw )
{
	return ( toothw == 10 ? 1 : 0);
}

uint8_t Trigger::modulo_down( uint8_t tooth_cnt, uint8_t modulus )
{
	if (modulus){
		// "modulo while":
		while( tooth_cnt >= modulus){
			tooth_cnt -= modulus;
		}
	// tooth_cnt range:  0 <= tooth_cnt < c_another_trigger_tooth
	}
	return tooth_cnt;
}

/**
 returns true if this tooth is to be avoided:
	tooth == catt - 1
	or tooth == cat - 2 ( for N-2 wheel)
	or tooth == 2 for a 15-1 5 cyl (catt=6), because missing tooth14 == 2 !!

	where:
	catt = config.another_trigger_tooth

	TODO: make avoid_tooth return always 0 for auditrigger
	TODO: coiltype ?
*/
uint8_t Trigger::avoid_tooth(uint8_t tooth, uint8_t catt)
{
	// works for any "coiltype" (none-missing), including AUDITRIGGER:
	if(config.primary_trigger & _BV(ptrig_coil)) {
		return 0;
	}

    tooth = modulo_down( tooth, catt );
	if((tooth == catt - 1)
                || ((tooth == (catt - 2)) && ((config.primary_trigger & _BV(trigger_multitooth_Nminus1)) == 0 ))
                || ( catt==6 && tooth == 2 && config.tooth_wheel==14 )
		){
			return	1;
		};
		return 0;
}

/** quarterdegrees => 4 usec conversion, with low-RPM support
 */
uint16_t Trigger::qdegree_to_4usec( uint16_t quarterdegrees, uint8_t toothw)
{
	uint16_t t16;
	// consider NCYL:
	t16 = degree2time( quarterdegrees, 182 );
	t16 = periodscaler_to_4usec( t16, engine.rpm_period );
	return t16;
}

/** calculate the time needed for the crank to rotate d/4 degrees
 * Input limited to max 4095 (d < 4096)
 * degree2time is CONFUSING NAME since output is not time, but a
 * scaler. See usage below.
 * magic_const is 182 normally (when d is in exactly 0.25 crankdeg
 * Use lower than 182 magic_const if d is higher
 * (because d unit is smaller than 0.25 crankdeg (such a audi135())
 */
uint16_t Trigger::degree2time(uint16_t d, uint8_t magic_const)
{
	uint8_t ncyl2;
	uint16_t t16;

	// usage: time = engine.rpm_period * degree2time(n) / 4096;
	// note: t16 is max 510 (/4 = 127.5 crankdegree).
	// the rpm_period depends on ncyl (180 degree for 4 cyl, 90 deg for 8 cyl)
	// for 8 cyl t16=360 (90 degree) scales the period with *= 1.0 (return 4096)
	// for 4 cyl t16=360 (90 degree) scales the period with *= 0.5 (return 2048)
	// reference period value is 0xffff: RPM=57.22 for 8cyl

	// not ncyl, but 2 * ncyl
	ncyl2 = ((config.config11 & 0xF0) >> 3) + 2;
	// reusing variable:
	t16 = mult16_8(d, ncyl2);	// max 510 * 15 * 2
	return mult16_8(t16, magic_const )>>8;
}

void Trigger::cli()
{

}

void Trigger::sei()
{

}

////////////////////////////////////// FROM GLOBAL.H ////////////////////////////////////////////////

/**
 returns the number of teeth, together with the missing.
 multiplied by 2 if camsync is used.
 15 - 1 with camsync : 30
 60 - 2 with camsync: 120
 60 - 2 without camsync: 60
 When ptrig_coil then it is understood that we have a reset on the secondary
 trigger and config.tooth_wheel is understood to be the number of teeth
 between resets (normally taken from the cam) in this case we don't need *2
 In the future, config.tooth_wheel will need to be set for coiltype as well ?
 TODO: rethink for AUDITRIGGER:
 	135 * 2 modulo 256 ?
	or 142 * 2 modulo 256 (volvo 142 wheel) ?
*/
uint8_t Trigger::get_total_tooth()
{
	uint8_t cpt, t8 = config.tooth_wheel;
#ifdef AUDITRIGGER
	if( t8 == 135 ){
		return t8; // no +1, and no *2 even if camsync used
	}
#endif
	cpt = config.primary_trigger;
	if( cpt & _BV(ptrig_coil) ) {
		// no <<= 1 (even with camsync!)
		return t8;
	}

	if( cpt & _BV(trigger_multitooth_Nminus1) ) {
		// N-1:
		t8 ++;
	} else { // N-2
		t8 += 2;
	}

	if ( config.secondary_trigger & _BV(strig_cam_sync) ){
		// camsync configured, so *2: t8 <<= 1; t8 += t8;
		t8 <<= 1;
	}

	return t8;
}

/////////////////// ARITHM.C ////////////////////
uint32_t Trigger::mult16_8(uint16_t a, uint8_t b)
{
    return a*b;
}


// rpmperiodscaler => 4 usec time conversion, with low-RPM support
// usage: t16 = degree2time( quarterdegrees );
// time_4usec = periodscaler_to_4usec( t16, engine.rpm_period );
// See qdegree_to_4usec() practical wrapper.
//
// In some cases, t16 is precalculated (from mainloop) and stored,
// but this final step must be calculated FAST (possibly from interrupt)
// On ARM, this will look nicer ;-)
// return value for the bogus case:

// downscaling constants:
#define DSCL (2)
#define PDSCL (1<<DSCL)
static inline uint32_t mult16_16( uint16_t a, uint16_t b)
{
        return a*b;
}

// return only 16 bits
static inline uint16_t div4096( uint32_t a)
{
        return a>>12;
}

static inline uint8_t lsr24( uint32_t a)
{
        return a>>24;
}

uint16_t Trigger::periodscaler_to_4usec( uint16_t rpmperiodscaler, uint32_t engine_rpm_period ){
        uint32_t t32;
        uint16_t per, delay;

        per = engine_rpm_period >> 16;
        if((per>>8) == 0){ // rpm_period fits into 24 bits
                if((uint8_t)per == 0){  // we like the period that fits into 16 bits
                        per = (uint16_t)engine_rpm_period;
                        t32 = mult16_16(per, rpmperiodscaler );
                        if(lsr24(t32) < 16){
                                delay = div4096(t32);
                                return delay;
                        }
                }else if((uint8_t)per < PDSCL){
                        // does NOT fit into 16 bits.
                        // but give it another chance via DownSCaLing
                        per = (uint16_t)( engine_rpm_period >> DSCL);
                        t32 = mult16_16(per, rpmperiodscaler);
                        if(lsr24(t32) < (16>>DSCL)){
                                // at this low RPM,
                                // div4096(t32) << DSCL would be OK too (and faster)
                                delay = div4096(t32 << DSCL);   // still acceptable
                                return delay;
                        }
                }
        }

        // ouch, this cannot be good:
        return RPM_TOO_LOW;
}
