#include "firmwareconfig.h"

#include <wx/textfile.h>

#include "fileparser/parserutil.h"
#include "logger.h"
#include "Utils.h"
#include "Math.h"

/**TEST*/
#include "fileparser/FirmwareConfigParser.h"
#include "firmwareconfig/FirmwareManager.h"

FirmwareConfig::FirmwareConfig()
    : m_parseHeader( m_symbolTable )
{
    m_replaceCount = 0;

    m_symbolTable[ _T("h") ] = Symbol::createArray( _T("h"), 8, 3, 1, 0 );
    m_symbolTable[ _T("j") ] = Symbol::createArray( _T("j"), 16, 16, 2, 0 );
    m_symbolTable[ _T("r") ] = Symbol::createArray( _T("r"), 16, 1, 2, 144 );
    m_symbolTable[ _T("k") ] = Symbol::createArray( _T("k"), 16, 1, 2, 156 );
    m_symbolTable[ _T("l") ] = Symbol::createArray( _T("l"), 16, 16, 3, 0 );
    m_symbolTable[ _T("n") ] = Symbol::createArray( _T("n"), 16, 16, 4, 0 );
    m_symbolTable[ _T("t") ] = Symbol::createArray( _T("t"), 8, 1, 6, 0 );
    m_symbolTable[ _T("b") ] = Symbol::createArray( _T("b"), 16, 1, 6, 8 );
    m_symbolTable[ _T("e") ] = Symbol::createArray( _T("e"), 16, 1, 7, 0 );
    m_symbolTable[ _T("y") ] = Symbol::createArray( _T("y"), 16, 1, 8, 0 );
    m_symbolTable[ _T("g") ] = Symbol::createArray( _T("g"), 16, 1, 9, 0 );

    /** TEST ONLY!!!*/
    /*
    wxString strHeaderFile = _T("D:\\vems\\firm1.0.78_12x12\\global.h");
    wxArrayString configFilesA;
    configFilesA.Add( _T("config\\config.txt") );
    configFilesA.Add( _T("config\\tables.txt") );
    FirmwareManager fm;
    fm.parseConfig( strHeaderFile, configFilesA );
    fm.print();
    Logger::logf( LOG_MSG, _T("FirmwareManager::getParsingSummary() : \n") + fm.getParsingSummary() );
    Logger::logf( LOG_MSG, _T("FirmwareManager::getParsingDetails() : \n") + fm.getParsingDetails() );
    fm.uploadConfig();
    fm.verifyModel();
    */
}

FirmwareConfig::~FirmwareConfig()
{
    WX_CLEAR_HASH_MAP(SymbolFileDescH, m_symbolFileDescH);
}

int FirmwareConfig::init()
{
    configArray.Clear();
    configHash.clear();

    //mttConfigA.Clear();
    //mttTableA.Clear();

    undefinedSymbolA.Clear();
    extraSymbolA.Clear();

    return 0;
}

//First run the headerparse!!
int FirmwareConfig::parseHeaderFile( const wxString& strFile )
{
    m_parseHeader.init();

    int result = m_parseHeader.parse( strFile );

    Logger::logf( LOG_MSG, _T("FirmwareConfig::parseHeaderFile()") );
    for ( SymbolTable::const_iterator IT = m_symbolTable.begin(); IT != m_symbolTable.end(); ++IT )
    {
        wxString s = IT->first;

        int page    = IT->second->getPage();
        int offset  = IT->second->getOffset();
        if ( page == 9 )
        {
            Logger::logf( LOG_MSG, _T("\t") + s.Pad( 30 - s.length(), _T(' '), false ) + _T(";  p = %d; o = %d "), page, offset );
        }
    }

    return  result;
}

int FirmwareConfig::parseFile(const wxString& strFile)
{
    ParserUtil::addProcessInfo(START_parseFile);
    if ( strFile.Length()  < 1){ return FC_INVALID_FILENAME;}

    wxTextFile file;
    if ( ! file.Open(strFile) ){ return FC_CANNOT_OPEN_FILE; }
    wxString line;
    m_fileName = strFile;
    bool firstline = true;
    line = file.GetFirstLine();
    do
    {
        if ( firstline ) firstline = false;
        else line = file.GetNextLine();
        //Logger::logf(LOG_MSG, _T("FirmwareConfig::parseFile() line: ")+line );
        m_lineNumber = file.GetCurrentLine();
        parseLine(line);
        //sprintf(stdout, "key: %s value \n", );
        //wxVLogDebug("key: %s value: %s \n", key.c_str(), value.c_str());
        //fprintf(stdout,"key: %s value: %s \n", key.c_str(), value.c_str());

    }
    while ( !file.Eof() );

    calculateTablesOffset();
    /**TEST*/
    //FirmwareConfigParser parser;
    //parser.parse( strFile );
    //parser.printScalarsToFile( _T("config//ConfigDebug.txt") );
    //parser.printTablesToFile( _T("config//TablesDebug.txt") );

    return 0;
}

int FirmwareConfig::parseLine(wxString& line)
{
    if(line.StartsWith(_T("#"))) {return 0;} //Ignore Comment
    wxString key, value;

    if ( ParserUtil::getKeyAndValue(line, key, value) == 0)
    {
    //decide config or table
    wxString symbol = getSymbol(key);
        if(symbol.size() == 1){
            parseTableLine(line);
        }else{
            parseConfigLine(line);
        }
    }
    return 0;
}

wxString FirmwareConfig::getSymbol(wxString& key)
{
    wxString tmp = _T("[");
    if( ! key.Contains(tmp) ) {return key; }//symbol

    return key.BeforeFirst('['); // symbol[2], l[A],
}

//arrayDef: h[0], row: array of values
//key=h[0,0] = value0;
//key=h[1,0] = valueC;
void FirmwareConfig::add2CommpareHash(StringHash& h, const wxString& arrayDef, const wxArrayString & row)
{
    char bracket = ']';
    wxString base = arrayDef.BeforeFirst(bracket);
    //Logger::log(LOG_MSG, _T("Key: ")+base);
    for(uint16_t i=0; i< row.Count(); i++)
    {
        wxString key = base+(wxString::Format(_T(",%X]"),i) );
        wxString value = row.Item(i);
        if(h.find(key) != h.end() ){
//            Logger::log(LOG_MSG, _T("Duplicate key: ")+key);
        }else{
            configkeyArray.Add(key);
        }
        h[key] = value;
        //configkeySet.insert(key);

    }
}

int FirmwareConfig::parseConfigLine(wxString& line)
{
         wxString key, value;
        ParserUtil::getKeyAndValue(line, key, value);
        configArray.Add(key);
        configHash[key] = value;

        if ( m_parseHeader.getHeaderArray().Index(key)  == wxNOT_FOUND)
        {
            extraSymbolA.Add(key);
        }

        if(symbolKeyValueHash.find(key) != symbolKeyValueHash.end() ){
            //Logger::log(LOG_MSG, _T("Duplicate key: ")+key);
        }else{
             configkeyArray.Add(key);
        }
        symbolKeyValueHash[key] = value;

        SymbolFileDesc* symbolFileDesc = new SymbolFileDesc(m_fileName, m_lineNumber);
        m_symbolFileDescH[key] = symbolFileDesc;

        return 0;
}

int FirmwareConfig::parseTableLine(wxString& line)
{
    if(line.StartsWith(_T("#"))) {return 0;} //Ignore Comment

    wxArrayString tokens;
    wxString msg;
    Utils::tokenizeToArray(line, wxT("="), tokens);

    if (tokens.GetCount() != 2)
    {
        ParserUtil::addProcessInfo(FC_INVALID_TABLE_LINE, line.c_str());
        return FC_INVALID_TABLE_LINE;
    }
    wxString arrayDef = tokens.Item(0); // h[0]
    wxString valueList = tokens.Item(1); // 13, 3F, ....

    //symbolKeyValueHash[arrayDef] = valueList;

    wxString symbol;
    wxString indexStr;

    wxArrayString row;
    Utils::tokenizeToArray(valueList, wxT(" "), row);

    add2CommpareHash(symbolKeyValueHash, arrayDef, row); //arrayDef: h[0], row:

    SymbolFileDesc* symbolFileDesc = new SymbolFileDesc( m_fileName, m_lineNumber );
    m_symbolFileDescH[ arrayDef ] = symbolFileDesc;

    //symbol, index, list
    //l[0]=37 37 37 37 37 37 37 37 37 37 37 37
    //symbol = l
    //indexStr = 0;
    wxString regexp;
    regexp = _T("^\\s*([^\\[]+)\\[(\\w+)\\]");
    ParserUtil::getCommonNameValue(arrayDef, regexp, symbol, indexStr);

   // msg.Printf(_T("symbol:|%s|indexStr:|%s|"),  symbol.c_str(), indexStr.c_str() );
    //Logger::log(LOG_MSG, msg );

    int  res = addRow2Table(symbol, indexStr, row);

    return res;
}

// hex string: indexStr
int FirmwareConfig::addRow2Table(const wxString& key, const wxString& indexStr, const wxArrayString& stringA)
{
    size_t rowLength = stringA.Count();
    uint16_t rowIndex = Math::hex2int(indexStr); //
    Symbol* ptrSymbol = m_symbolTable[ key ];

    //if found a new table, allocate memory for raw data
    if ( m_tableSizeH.find( key ) == m_tableSizeH.end() )
    {
        m_tableSizeH[ key ] = 0;
        m_tableDataH[ key ] = new char[256];

        //
        ptrSymbol->setArrayColSize( rowLength );
        ptrSymbol->setArrayRowSize( 0 );
        ptrSymbol->setArrayDimension( 1 );
    }

    int arrayRowSize = ptrSymbol->getArrayRowSize();
    ptrSymbol->setArrayRowSize( ++arrayRowSize );
    if ( ptrSymbol->getArrayRowSize() > 1 ) { ptrSymbol->setArrayDimension( 2 ); }

    //set table size
    m_tableSizeH[ key ] += rowLength;

    //fill table data
    uint8_t* data = (uint8_t*)m_tableDataH[ key ];
    size_t offset = rowIndex * rowLength;

    //Logger::logf( LOG_MSG, _T("FirmwareConfig::addRow2Table() - table : ") + key + _T("; offset : %d, addres : %p"), offset, data );

    for ( size_t i = 0; i < rowLength; ++i )
    {
        wxString hexValue = stringA.Item( i ).Mid( 0, 2 );
        uint8_t value = Math::hex2int( hexValue );
        data[ offset + i ] = value;
    }




/*
    if( colIndex == 0){ // first Row init expected rowLength
         ftd->currRows = rowLength;
    }

    if(ftd->currRows != rowLength){
        addProcessInfo(FC_ALTERING_ROW_COUNT, ftd->currRows, rowLength, key.c_str()  );
        return FC_ALTERING_ROW_COUNT;
    }
*/
/*
    wxArrayString * table = ftd->tableA;
    //size_t tableLength = table->Count();

    if ( colIndex > ftd->maxCols)
    {
        ParserUtil::addProcessInfo(FC_HIGH_COL_COUNT, ftd->maxCols, colIndex, key.c_str() );
        return FC_HIGH_COL_COUNT;
    }

    size_t i;
    for (i=0; i< ftd->currRows; i++)
    {
        wxString item = stringA.Item(i);
        table->Add(item);
    }
    */
    return FC_OK;
}

void FirmwareConfig::calculateTablesOffset()
{
    if ( m_tableSizeH.find( _T("j") ) == m_tableSizeH.end() ) { return; }

    size_t jTableSize = m_tableSizeH[ _T("j") ];

    m_symbolTable[ _T("r") ]->setOffset( jTableSize );

    m_symbolTable[ _T("k") ]->setOffset( jTableSize + m_tableSizeH[ _T("r") ] );
/*
    for( String2IntHash::const_iterator IT = m_tableSizeH.begin(); IT != m_tableSizeH.end(); ++IT )
    {
        Logger::logf( LOG_MSG, _T("FirmwareConfig::calculateTablesOffset() - table : ") + IT->first + _T(" ; size : %d"), IT->second );
    }
  */

    SymbolTable::const_iterator IT = m_symbolTable.begin();
    for ( ; IT != m_symbolTable.end(); ++IT )
    {
        Symbol* ptrSymbol = IT->second;
        if ( !ptrSymbol->isArray() ) { continue; }
        Logger::logf( LOG_MSG, _T("\t %s; %d x %d; d : %d; p : %d; o : %d"),
                      ptrSymbol->getName().c_str(),
                      ptrSymbol->getArrayColSize(),
                      ptrSymbol->getArrayRowSize(),
                      ptrSymbol->getArrayDimension(),
                      ptrSymbol->getPage(),
                      ptrSymbol->getOffset() ) ;
    }
}


void FirmwareConfig::printKeyValueHash()
{
    mttA.Clear();
    wxString tmp; tmp = _T("Manmtt");
    mttA.Add(tmp);

    wxString command;
    for( uint16_t i=0; i < configkeyArray.Count(); i++ )
    {
        wxString key = configkeyArray.Item(i);
        if(symbolKeyValueHash.find(key) == symbolKeyValueHash.end() ){
            Logger::log(LOG_MSG, _T("FirmwareConfig::createMtt() key not found: ") + key);
            continue;
        }
        wxString value = symbolKeyValueHash[key];

        wxString symbol = getSymbol(key);
        if(symbol.size() == 1){
            command = getMTTTableCommand(key, value);
        }else{
            command = getMTTConfigCommand(key, value);
        }
        //Logger::log(LOG_MSG, key+_T("=")+ value + _T(" (") +command+ _T(")") );

        if(! command.IsEmpty()) mttA.Add(command);
    }
    tmp = _T("mcs");
    mttA.Add(tmp);
    tmp = _T("bye");
    mttA.Add(tmp);
}

void FirmwareConfig::createMtt(bool isLog)
{
    mttA.Clear();
    wxString tmp; tmp = _T("Manmtt");
    mttA.Add(tmp);
    wxString command;

    wxFile *binFile;

    if(isLog){
        binFile = new wxFile();
        wxString fileName = _T("c:\\key-value.txt");
        binFile->Open(fileName, wxFile::write);
    }
    //fprintf(stdout, " Key value test: \n");

    for( uint16_t i=0; i < configkeyArray.Count(); i++ )
    {
        wxString key = configkeyArray.Item(i);
        if(symbolKeyValueHash.find(key) == symbolKeyValueHash.end() ){
            Logger::log(LOG_MSG, _T("FirmwareConfig::printKeyValueHash() key not found: ") + key);
            continue;
        }
        wxString value = symbolKeyValueHash[key];

        wxString symbol = getSymbol(key);
        if(symbol.size() == 1){
            command = getMTTTableCommand(key, value);
        }else{
            command = getMTTConfigCommand(key, value);
        }
        //fprintf(stdout, "%s=%s (%s)\n", key.c_str(), value.c_str(), command.c_str() );
        //Logger::log(LOG_MSG, key+_T("=")+ value + _T(" (") +command+ _T(")") );
        if(isLog){
            binFile->Write(key +_T("=")+ value+ _T(" (") +command +_T(")") _T("\n"));
        }

        if(! command.IsEmpty()) mttA.Add(command);
    }

    if(isLog){
        binFile->Close();
    }
     tmp = _T("mcs"); mttA.Add(tmp);
     tmp = _T("bye"); mttA.Add(tmp);
}

wxString FirmwareConfig::getMTTConfigCommand(wxString& key, const wxString& value)
{
    int index = m_parseHeader.getHeaderArray().Index(key);
    if(index == wxNOT_FOUND){ return wxEmptyString; }
    wxString tmp;
    tmp.Printf(_T("g%02Xc"), index);
    tmp += value;
    return tmp;

}

wxString FirmwareConfig::getMTTTableCommand(wxString& key, const wxString& value)
{
    wxString symbol = getSymbol(key);
/*
    if( tdHash.find(symbol) == tdHash.end() ){
        Logger::log(LOG_MSG, _T("Invalid table name: ")+ symbol);
        return wxEmptyString;
    }
*/
//    FirmwareTableDesc * ftd = tdHash[key];
    int index = getTableIndex(key);

    wxString tmp;
    tmp.Printf(_T("mmttg%02X"), index);
    tmp.Append(symbol);
    tmp.Append(value.Mid(0,2));
    //tmp.Append(_T("mm"));
    return tmp;
}

int FirmwareConfig::getTableIndex(wxString& key)
{
    wxString symbol = getSymbol(key);
//    FirmwareTableDesc * ftd = tdHash[symbol];
//    size_t rowOffset = ftd->currRows;

    wxString colStrHex;
    wxString rowStrHex;
    ParserUtil::getArrayColumnAndRow(key, colStrHex, rowStrHex);


    int col = Math::hex2int(colStrHex);
    long row = Math::hex2int(rowStrHex); //rowStr.ToLong(&row);

    int index = 16*col + row;
    // int index = rowOffset *col + (int)row;

    return index;

}

/**
Man mode
*/
/*
int FirmwareConfig::createMTTBufferTable()
{
    wxString tmp, msg;
    tmp = _T("Mantt");
    mttTableA.Add(tmp);
    unsigned int i;

    FirmwareTableDesc * ftd;
    TDHash::iterator it;


    for ( it = tdHash.begin(); it != tdHash.end(); ++it )
    {
        wxString key = it->first;
        ftd = tdHash[key];
        size_t rowOffset = ftd->currRows;
        wxArrayString * table =ftd->tableA;
       // msg.Printf(_T("Row offset for table %s: %d"), key.c_str(), rowOffset);
       // Logger::log(LOG_MSG, msg);
        size_t row = 0;
        size_t col = 0;
        for (i= 0; i< table->Count(); i++)
        {
            wxString value = table->Item(i).Mid(0,2);

            if (row == rowOffset )
            {
                col++;
                row=0;
            }
            int index = 16*col + row;

            tmp.Printf(_T("g%02X"), index);
            tmp.Append(key);
            tmp.Append(value);

            mttTableA.Add(tmp);
            row++;
        }
    }

    tmp = _T("mcs");
    mttTableA.Add(tmp);
    tmp = _T("bye");
    mttTableA.Add(tmp);

    return FC_OK;
}
*/
wxArrayString * FirmwareConfig::getExtraSymbol()
{
    return &extraSymbolA;
}

wxArrayString * FirmwareConfig::getUndefinedSymbol()
{
    undefinedSymbolA.Clear();
    size_t size = m_parseHeader.getHeaderArray().GetCount();
    unsigned int i;
    for (i=0; i< size; i++)
    {
        wxString symbol = m_parseHeader.getHeaderArray().Item(i);

        if(configHash.find(symbol) == configHash.end() ) //wxString value = configHash[symbol];
        //if (value.IsEmpty())
        {
            undefinedSymbolA.Add(symbol);
        }
    }
    return &undefinedSymbolA;
}

void FirmwareConfig::configAutoUpgrade(wxArrayString* undefSymbols)
{
    int i;
    for(i=0; i < undefSymbols->Count(); i++){
        wxString undefKey = undefSymbols->Item(i);
        if(undefKey == _T("shiftcut_conf_input") ){
            if(configHash.find(_T("shiftcut_conf")) != configHash.end() &&
               configHash.find(_T("shiftcut_channel ")) != configHash.end() ){
                wxString shiftcut_confHex = configHash[_T("shiftcut_conf")];
                wxString shiftcut_channelHex = configHash[_T("shiftcut_channel")];
                uint8_t shiftcut_conf = Math::hex2int(shiftcut_confHex);
                uint8_t shiftcut_channel = Math::hex2int(shiftcut_channelHex);
                uint8_t calcValue = shiftcut_conf + shiftcut_channel;

                configHash[_T("shiftcut_conf_input")] = wxString::Format(_T("%02X"), calcValue);
            }else{
                ///TODO: cannot calculate!!
            }
        }
    }
}
/*
int FirmwareConfig::createMTTBuffer()
{
    wxString tmp;
    tmp = _T("Mantt");
    mttConfigA.Add(tmp);

    size_t size = parseHeader.getHeaderArray()->GetCount();
    unsigned int i;
    for (i=0; i< size; i++)
    {
        wxString symbol = parseHeader.getHeaderArray()->Item(i);
        wxString value = configHash[symbol];
        if (value.IsEmpty())
        {
            //msg =_T("WARNING: did not find a value for " + symbol +" in config.txt");
            //undefinedSymbolA.Add(symbol);
            ParserUtil::addProcessInfo(FC_UNDEFINED_SYMBOL, symbol.c_str() );
        }
        else
        {
            tmp.Printf(_T("g%02Xc"), i);
            tmp += value;
            mttConfigA.Add(tmp);
        }
    }
    tmp = _T("mcs");
    mttConfigA.Add(tmp);
    tmp = _T("bye");
    mttConfigA.Add(tmp);

    return FC_OK;
}
*/
wxArrayString& FirmwareConfig::getMTTA(){
        return mttA;
}

void FirmwareConfig::Message()
{
    size_t size = undefinedSymbolA.GetCount();
    unsigned int i;
    for (i = 0; i < size; i++)
    {
        wxString symbol = undefinedSymbolA.Item(i);
        wxString msg =_T("WARNING: \"") + symbol + _T("\" is missing from config.txt (missing value)");
        Logger::log(LOG_MSG, msg );
        ///Helper::Message(msg);
    }

    size = extraSymbolA.GetCount();
    for (i = 0; i < size; i++)
    {
        wxString symbol = extraSymbolA.Item(i);
        wxString msg =_T("WARNING: \"") + symbol + _T("\" in config.txt not exist in global.h (extra value)");
        Logger::log(LOG_MSG, msg );
        ///Helper::Message(msg);
    }

    Logger::log(LOG_ERROR,_T("Firmware::configMgr() END! ") );
}



/*
void FirmwareConfig::addProcessInfo(firmwareConfig_codes code, wxString msg)
{
    logType level = LOG_MSG; //geLevel(FC_INVALID_FILENAME);
    Logger::log(level, msg);
}
*/



/*
tableName-->page-->readCommand

tableName==c for config

variable read, write via binary mode (_NOT_ Man mode). 3 byte sequence:
<readcommand><offset><value>
<writecommand><offset><value>
instead of "mttg<offsethex>c<valuehex>"
ins
for config, offset comes from variable name, eg. primep=0, wbo2_nernstdc_target=0x89
configparse knowns variable offset from the variable name

we also have a binary command for "mcs" (B), so we don't have to go to Man mode

For readback, mcd will be used in the future too.
*/
/*
wxString getReadCommand(const wxString& tableName){
    c=>...
    j=>...
    return readCommandHash[tableName];

}
*/
/**
c=>...
j=>...
*/
/*
wxString getWriteCommand(const wxString& name)
{
    return writeCommandHash[name];
}
*/
StringHash& FirmwareConfig::getConfigHash()
{
        return configHash;
}

StringHash& FirmwareConfig::getSymbolKeyValueHash()
{
    return symbolKeyValueHash;
}

DoubleHash& FirmwareConfig::getConfigValueHash()
{
//    m_configDoubleHash.Clear();
     for(StringHash::iterator it = configHash.begin(); it != configHash.end(); it++){
        wxString key = it->first;
        wxString value = it->second;
        //Logger::logf(LOG_MSG,_T("FirmwareConfig::getConfigValueHash() ") + key+_T("=") + value);
        m_configDoubleHash[key] = (double) Math::hex2int(value);
        /*
        if(key == _T("warmup_clt_range[0]")){
            Logger::logf(LOG_MSG,_T("FirmwareConfig::getConfigValueHash() ") + key+_T("=") + value);
            Logger::logf(LOG_MSG,_T("FirmwareConfig::getConfigValueHash() ") + key+_T("=%f"),m_configDoubleHash[key]);
        }
        */

     }

    return m_configDoubleHash;
}

/*
wxArrayString& FirmwareConfig::getConfigArray()
{
        return configArray;
}
*/

void  FirmwareConfig::replaceKey(const wxString& oldKey, const wxString& newKey, int value)
{
    if(value < 0){
        Logger::logf(LOG_WARNING,_T("FirmwareConfig::replaceKey() value < 0") );
           value = 0;
    }
    if(value > 255){
        Logger::logf(LOG_WARNING,_T("FirmwareConfig::replaceKey() value > 255") );
        value = 255;
    }

    wxString vStr = wxString::Format(_T("%02X"), value);
    configHash.erase(oldKey);
    configHash[newKey] = vStr;

    int idx = configArray.Index(oldKey);
    if(idx  == wxNOT_FOUND)
    {
        Logger::logf(LOG_ERROR,_T("FirmwareConfig::replaceKey() key: ")+oldKey + _T(" NOT FOUND") );
        return;
    }

    configArray.RemoveAt(idx);
    configArray.Insert(newKey ,idx);
}

void FirmwareConfig::resetReplaceCount()
{
    m_replaceCount=0;
}

void FirmwareConfig::addReplaceCount(int i)
{
  m_replaceCount+=i;
}

void FirmwareConfig::saveConfig(const wxString& strFile, bool force)
{
    wxTextFile file;
    if ( ! file.Open(strFile) ){
        file.Create(strFile);
    }else{

        file.Clear();
    }

   // if(force || m_replaceCount > 0){
    Logger::logf(LOG_WARNING,_T("FirmwareConfig::save() replaceCount: %d"),  m_replaceCount);
     m_replaceCount = 0;
        for(uint16_t i=0; i < configArray.GetCount(); i++){
            wxString key = configArray.Item(i);
            wxString value = configHash[key];
            wxString line = key+_T("=")+value;
            file.AddLine(line);
        }
    //}

    file.Write();
    file.Close();

}

void FirmwareConfig::writeSymbols( DoubleHash* symbolHash, bool isTable )
{
    //we need sybols by fileNames;
    wxArrayStringH filesH;

    DoubleHash::iterator it;
    for(it = symbolHash->begin(); it != symbolHash->end(); it++ ){
        wxString symbol = it->first;

        if(m_symbolFileDescH.find(symbol) == m_symbolFileDescH.end() ){
            Logger::logf(LOG_MSG, _T("FirmwareConfig::writeSymbols() symbol: ")+ symbol+
            _T(" NOT FOUND"));
            continue;
        }

        SymbolFileDesc * symbolFileDesc = m_symbolFileDescH[symbol];

        wxString fileName = symbolFileDesc->getFileName();
        if(filesH.find(fileName) == filesH.end() ){
            filesH[fileName] = new wxArrayString();
        }

        wxArrayString * symbolA = filesH[fileName];
        symbolA->Add(symbol);
    }

    //////////////////////////
    wxArrayStringH::iterator it2;
    for(it2 = filesH.begin(); it2 != filesH.end(); it2++ ){
        wxString fileName = it2->first;
        wxArrayString * symbolA = it2->second;
        wxTextFile file;
        if ( ! file.Open(fileName) ){
             Logger::logf(LOG_MSG, _T("FirmwareConfig::writeSymbols() cannot open file: ")+ fileName);
            continue;
        }
        // write symbols to file
        for(uint32_t i = 0; i < symbolA->GetCount(); i++)
        {
            wxString symbol = symbolA->Item(i);
            SymbolFileDesc* symbolFileDesc = m_symbolFileDescH[ symbol ];

            wxString& lineStr = file.GetLine(symbolFileDesc->getLineNumber() );
            double valueD = (*symbolHash)[symbol];
            uint8_t value = (uint8_t)valueD;

            if( !isTable )
            {
                lineStr = wxString::Format( symbol + _T("=%2X"), value );
            }
            else
            {
                writeTableValue( lineStr, symbol, value );
            }
        }
        file.Write();
        file.Close();
    }

}


void FirmwareConfig::writeTableValue( wxString& line, const wxString& symbol, uint8_t value )
{
    wxArrayString tokens;
    wxString values = line.AfterFirst( _T('=') );
    Utils::tokenizeToArray( values, _T(" "), tokens );

    wxString head = line.BeforeFirst( _T('=') );

    long col = 0;
    symbol.AfterFirst( _T(',') ).BeforeFirst( _T(']') ).ToLong( &col );

    tokens.Item( col ) = wxString::Format( _T("%2X"), value );

    wxString testLine = wxEmptyString;
/*
    line.Empty();
    line.Append( head + _T("=") );

    for( uint32_t i = 0; i < tokens.GetCount(); i++ )
    {
        line.Append( tokens.Item(i) ) ;
        if( i != tokens.GetCount() )
        {
            line.Append( _T(" ") );
        }
    }
*/
    testLine.Append( head + _T("=") );

    for( uint32_t i = 0; i < tokens.GetCount(); i++ )
    {
        testLine.Append( tokens.Item(i) ) ;
        if( i != tokens.GetCount() )
        {
            testLine.Append( _T(" ") );
        }
    }

    Logger::logf( LOG_WARNING, _T("FirmwareConfig::writeTableValue ") + testLine );
}

