#include "ConfigReplacerMgr.h"

#include "ConfigCriteriaData.h"
#include "ConfigCriteria.h"
#include "ConfigCriteriaRange.h"
#include "FirmwareConfigData.h"

#include "../common/VemsMessage.h"

#include "../logger.h"

ConfigReplacerMgr::ConfigReplacerMgr(ConfigCriteriaData* criteriaData)
{
    //ctor
    m_criteriaData = criteriaData;
    //reset();
}

ConfigReplacerMgr::~ConfigReplacerMgr()
{
    //dtor
}

void ConfigReplacerMgr::registerVemsMessageViewer(VemsMessageViewerBase* vemsMessageViewer){
    m_viewerA.Add(vemsMessageViewer);
}

void ConfigReplacerMgr::replace(FirmwareConfigData& fcd, const wxArrayString& missingKeys,  wxArrayString& remainedA)
{
    if ( missingKeys.IsEmpty() ) { return; }

    VariableDataDH vd;
    fcd.fillVariableData(vd);
    m_rpn.setVariableData(&vd);
    setVariableData(vd, m_criteriaData->variableHash);

    wxString calcNotFoundHtml = _T("Missing keys that there are no calculations:<br>");
    wxString calcIgnoredHtml = _T("Missing keys that there are calculations but ignored (out of range):<br>");
    wxString calcProcessedHtml = _T("Missing keys that there are calculations:<br>");

    int replacedScalarCount = 0;
    int calcNotFound = 0;
    int calcIgnored = 0;

    //SymbolTable& replaceHash =  m_criteriaData.replaceHash;
    for ( size_t i = 0; i < missingKeys.GetCount(); i++ )
    {
        wxString missingKey = missingKeys.Item(i);

        SymbolTable::iterator IT =  m_criteriaData->replaceHash.find(missingKey);
        if(IT ==  m_criteriaData->replaceHash.end() ){
            calcNotFoundHtml.Append( missingKey + _T("<br>"));
            ++calcNotFound;
        }else{
            Symbol* replaceSymbol = IT->second;
            int v = (int) m_rpn.calculate( replaceSymbol->getExpr() );
            if ( v < 0 || v > 255 ){
                //fcd.removeScalar( replace->getOldKey() );
                calcIgnoredHtml.Append(missingKey +
                wxString::Format(_T(": %i"),v) +_T(" (")+replaceSymbol->getExpr()+_T(")<br>") );
                ++calcIgnored;
            }else{
                fcd.addScalar( replaceSymbol->getName(), v );

                calcProcessedHtml.Append(missingKey+
                wxString::Format(_T(": %i"),v) +_T(" (")+replaceSymbol->getExpr()+_T(")")+_T("<br>") );
                ++replacedScalarCount;
                remainedA.Remove(missingKey);
            }
        }
    }

    Logger::logf( LOG_MSG,_T("ConfigReplacerMgr::show(): errorCount:%d"), replacedScalarCount);



    if ( replacedScalarCount )
    {
        wxString replaceSummary = wxString::Format( _T("Config replace summary<br>"));
        wxString summaryHtml;
        summaryHtml.Append( wxString::Format(_T("Number of calculated variables: :%i<br>"),replacedScalarCount));

        if(calcNotFound > 0){
            summaryHtml.Append( wxString::Format(_T("Number of missing calculation rules:%i<br>"),calcNotFound));
        }
        if(calcIgnored > 0){
            summaryHtml.Append( wxString::Format(_T("Number of ignored calculations:%i<br>"),calcIgnored));
        }
        replaceSummary + calcProcessedHtml +calcIgnoredHtml +calcNotFoundHtml;

        VemsMessage vemsMsg1(VMT_INFO, replaceSummary, replaceSummary, replaceSummary);
        show(vemsMsg1);
/*
        VemsMessage vemsMsg2(VMT_INFO, _T("Calulated values details"), _T("Config replace details"), calcProcessedHtml);
        show(vemsMsg2);

        VemsMessage vemsMsg3(VMT_INFO, _T("Ignored values details"), _T("Config replace details"), calcIgnoredHtml);
        show(vemsMsg3);

        VemsMessage vemsMsg4(VMT_INFO, _T("Missing values details"), _T("Config replace details"), calcNotFoundHtml);
        show(vemsMsg4);
*/

    }

}

void ConfigReplacerMgr::setVariableData(VariableDataDH& vd, SymbolTable& variableHash)
{
    SymbolTable::const_iterator IT = variableHash.begin();
    for ( ; IT != variableHash.end(); ++IT )
    {
        Symbol* symbol = IT->second;
        double value = m_rpn.calculate( symbol->getExpr() );
        vd.addValue( IT->first, value );
    }
}

void ConfigReplacerMgr::show(const VemsMessage& vemsMsg)
{
    for(size_t i = 0; i < m_viewerA.GetCount(); ++i){
        VemsMessageViewerBase* viewer = m_viewerA.Item(i);
        viewer->show(vemsMsg);
    }
}
