#include "FWConfigUploadDialog.h"


#include <wx/filename.h>
#include <wx/progdlg.h>
#include <wx/textfile.h>

#include "FirmwareManager.h"
#include "FirmwareConfigDifferences.h"
#include "../VemsTuneController.h"
#include "../ObserverBase.h"
#include "../UserLog.h"
#include "../logger.h"

#include "../DirHelper.h"

#include "../unidlg2test.h"
#include "../vemsprotocol/VemsCommand.h"
#include "../vemsprotocol/SIPRTypes.h"
#include "../vemsprotocol/CommunicationController.h"
#include "../vemsprotocol/CommunicationDispatcher.h"
#include "../project/ProjectManager.h"
#include "../project/ProjectHelper.h"
//#include "../common/VemsMessageHashHelper.h"
#include "../common/ConfigSwitchTargetTypes.h"

#include "../bcWidgets/bcProgressDialog.h"

FWConfigUploadDialog::FWConfigUploadDialog( wxWindow* parent,
                                            CommunicationDispatcher* dispatcher,
                                            VemsTuneController* vemsController,
                                            ObserverBase* observer ) :
    CommunicationClientBase( vemsController->getECUController(), dispatcher )
{
    // external data
    m_parent                = parent;
    m_vemsController        = vemsController;
    m_observer              = observer;

    // internal data
    m_pProgressDialog       = NULL;
    m_pUploadManager        = new FWConfigUploadManager(vemsController->getUserDialog(), m_vemsController->getFirmwareInfo() );
    m_iLineCount            = 0;
    m_bIsFilteredUpload     = false;
    m_currentConfigType     = CFG_UNKNOWN;
    m_uploadConfigType      = CFG_BOTH;
}


FWConfigUploadDialog::~FWConfigUploadDialog()
{
    if ( m_pUploadManager ) { delete m_pUploadManager; }
    if ( m_pProgressDialog ) { m_pProgressDialog->Destroy(); m_pProgressDialog = NULL; }
}


void FWConfigUploadDialog::process()
{
    easyUploadConfig();
}


bool FWConfigUploadDialog::checkHeaderFile()
{
    FirmwareInfo* fwInfo = m_vemsController->getFirmwareInfo();
    wxString headerFile = fwInfo->getHeaderFile();
    if ( headerFile.IsEmpty() )
    {
        wxString fwVersion = fwInfo->getFirmwareVersionAsString();

        //StringHash hash;
        //hash[_T("firmware_version")] = fwVersion;
        //wxString msg = VemsMessageHashHelper::GetVemsMsg( _T("header_not_found"), &hash )->getMsg();

        wxString msg = _T("Descriptor file has been not found for firmware ")+ fwVersion +_T("\n");
        msg +=_T("This means that you should download firmware: ")+ fwVersion+ _T("by VemsTune!\n\n");
        msg +=_T("1. Go to menu Firmware/Firmware web tool")+ fwVersion+ _T("\n");
        msg +=_T("2. Select the given firmware version and press download!\n");
        msg +=_T("3. When it has been downloaded press right mouse button on it and click Install!\n");
        msg +=_T("Now the descriptor files should be available for VemsTune!\n");

        wxMessageBox( msg );

        stop();
        return false;
    }
    return true;
}

bool FWConfigUploadDialog::openConfigFile(wxString& configFile )
{
    int device = m_pController->getDevice();

    if ( device != ECUInfo::DEVICE_V3 && device != ECUInfo::DEVICE_ROUND )
    {
        UserLog::vLog( _T("ecu_not_found_1") );
        //stop();
        return false;
    }

    //m_pController->request( new EnableReadRealTimeCommand( false ) );

    /// Header file set!
    bool res = checkHeaderFile();
    if( !res )return false;

    wxString filter =  _T("Vems config file (*.vemscfg)|*.vemscfg");
    filter += _T("|mcdmct (*.txt)|*.txt");
    filter += _T("|All files (*.*)|*.*");
    wxString message = _T("Select a config file");
    wxString dir = DirHelper::GetPath(dLOG);

    //wxString configFile;

    //wxFileDialog dlg(NULL, message, ConfigHelper::GetDialogPath( _T("easyUploadConfig_fw") , dir) ,wxEmptyString, filter, wxCHANGE_DIR);

     wxFileDialog dlg(NULL, message,dir ,wxEmptyString, filter, wxCHANGE_DIR);

    if ( dlg.ShowModal() == wxID_OK )
    {
        configFile = dlg.GetPath();
        //wxFileName fn(configFile);
        //ConfigHelper::SaveDialogPath( _T("easyUploadConfig_fw"), fn.GetPath() );
    }

    if( configFile.IsEmpty() )
    {
        //stop();
        return false;
    }

    m_cfgfilename = configFile;

    return true;
}

void FWConfigUploadDialog::uploadConfigFile( const wxString& filename )
{
    if ( !m_pDispatcher->isIdle() )
    {
        return;
    }

    m_bIsIdle = false;
    connect();

    int device = m_pController->getDevice();

    if ( device != ECUInfo::DEVICE_V3 && device != ECUInfo::DEVICE_ROUND )
    {
        UserLog::vLog( _T("ecu_not_found_1") );
        stop();
        return;
    }

    /// Header file set!
    bool res = checkHeaderFile();
    if( !res )return;

    wxArrayString configFiles;
    configFiles.Add( filename );

    m_pUploadManager->clear();

    uploadConfig( configFiles );
    verifyConfig(false);
}


void FWConfigUploadDialog::filteredConfigUpload()
{
    Logger::logf( LOG_MSG, _T("FWConfigUploadDialog::filteredConfigUpload() - START") );

    if ( !lockController() )
    {
        wxMessageBox( _T("Controller is busy!") );
        return;
    }

    /// open config file
    wxString configFile;

    if ( !openConfigFile( configFile ) )
    {
        unlockController();
        return;
    }

    wxArrayString configFiles;
    configFiles.Add( configFile );


    /// open filter file
    wxString filter =_T("mcdmct (*.txt)|*.txt");
    filter += _T("|Vems config file (*.vemscfg)|*.vemscfg");
    filter += _T("|All files (*.*)|*.*");
    wxString message = _T("Select a filter file");
    wxString dir = DirHelper::GetPath(dCONFIGFILTERS);

    wxString filterFile;

    wxFileDialog dlg( NULL, message, dir, wxEmptyString, filter, wxCHANGE_DIR );

    if ( dlg.ShowModal() != wxID_OK )
    {
        unlockController();
        return;
    }

    filterFile = dlg.GetPath();

    ///parse filter file
    StringSet filters;

    parseConfigFilterFile( filterFile, filters );

    // if filters.
    m_bIsFilteredUpload = true;

    Logger::logf( LOG_MSG, _T("FWConfigUploadDialog::filteredConfigUpload()  -found %d keys\n"), filters.size() );

    StringSet::const_iterator it =  filters.begin();
    for( ; it != filters.end(); ++it )
    {
         Logger::logf( LOG_MSG, _T("na - ") + *it + _T("\n") );
    }

    m_pUploadManager->clear();
    m_pUploadManager->parseConfigFromFiles( configFiles );

    wxArrayString missing;

    m_pUploadManager->applyFilter( filters, missing );




    if ( !missing.IsEmpty() )
    {
        wxString htmlLog;

        if ( missing.GetCount() == filters.size() )
        {
            htmlLog += _T("All filter keys missing!<br>");
            m_observer->notify( htmlLog );
            reset();
            unlockController();
            return;
        }

        htmlLog = _T("Missing filter keys found!<br>");

        for ( size_t i = 0; i < missing.GetCount(); ++i )
        {

            htmlLog += _T("&nbsp;&nbsp;&nbsp;&nbsp;") + missing[i] + _T("<br>");

        }

        m_observer->notify( htmlLog );
    }


    m_pProgressDialog = new bcProgressDialog(   _T("Write config in progress!"),
                                                _T(""),
                                                100,
                                                NULL,
                                                wxPD_APP_MODAL | wxPD_AUTO_HIDE | wxPD_SMOOTH |
                                                wxPD_ELAPSED_TIME | wxPD_ESTIMATED_TIME | wxPD_REMAINING_TIME );
    m_pProgressDialog->Pulse();

    m_pUploadManager->uploadConfig( m_pController, m_currentConfigType, m_uploadConfigType );

    verifyConfig(false);

    //unlockController();
}


void FWConfigUploadDialog::easyUploadConfig()
{
    wxString configFile;

    if ( !openConfigFile( configFile ) )
    {
        stop();
        return;
    }

    m_pController->request( new EnableReadRealTimeCommand( false ) );

    wxArrayString configFiles;
    configFiles.Add( configFile );

    m_pUploadManager->clear();

    uploadConfig( configFiles );
    //stop();
    verifyConfig(false);
}


void FWConfigUploadDialog::uploadConfig( const wxArrayString& configFiles )
{
    bool error = m_pUploadManager->createUploadConfig( configFiles );

    /// create report to html log
    wxString htmlLog;
    htmlLog += _T("-------Upload config--------<br>");

    /// add parsed table info
    const wxArrayString& tables = m_pUploadManager->parsedTables;

    for ( size_t i = 0; i< tables.GetCount(); ++i )
    {
        htmlLog += _T("&nbsp;&nbsp;&nbsp;&nbsp;") + tables[i] + _T("<br>");
    }

    /// add detailed parsing info
    const FirmwareConfigDifferences& diff = m_pUploadManager->configsVsHeader;

    if ( diff.isEmpty() )
    {
        htmlLog += _T("Files parsing was successful!<br>");
    }
    else
    {
        htmlLog += _T("File parsing summary<br>");
        htmlLog += diff.summary( _T("<br>") );
        htmlLog += _T("File parsing details<br>");
        htmlLog += diff.details( _T("&nbsp;&nbsp;&nbsp;&nbsp;"), _T("<br>") );
        htmlLog += m_pUploadManager->menuLinks;
    }

    m_observer->notify( htmlLog );

    /// create message box info
    //wxString message = m_pUploadManager->getUploadSummary();
    wxString message;

    if ( diff.isEmpty() )
    {
        message += _T("Files parsing was successful!\n");
    }
    else
    {
        message += _T("File parsing summary\n");
        message += diff.summary( _T("\n") );
    }

    wxString configTableSize = _T("12x12");
    if ( !m_pUploadManager->isFile12x12() ) { configTableSize = _T("16x14"); }

    wxString ecuTableSize = _T("12x12");
    if ( !m_vemsController->is12x12() ) { ecuTableSize = _T("16x14"); }

    message += _T("Config table size:") + configTableSize + _T("\n") +
        _T("ECU table size:") + ecuTableSize + _T("\n");

    if ( error )
    {
        message += _T("More details can be found in debug tab.");
    }

    wxMessageBox( message, _T("Upload config") );

    if ( m_vemsController->getDevice() != ECUInfo::DEVICE_V3 && m_vemsController->getDevice() != ECUInfo::DEVICE_ROUND )
    {
        wxMessageBox( _T("ECU not connected!"), _T("Upload config"), wxICON_ERROR | wxOK );
        stop();
        return;
    }

    m_pProgressDialog = new bcProgressDialog(   _T("Write config in progress!"),
                                                _T(""),
                                                100,
                                                NULL,
                                                wxPD_APP_MODAL | wxPD_AUTO_HIDE | wxPD_SMOOTH |
                                                wxPD_ELAPSED_TIME | wxPD_ESTIMATED_TIME | wxPD_REMAINING_TIME );
    m_pProgressDialog->Pulse();

    m_pUploadManager->uploadConfig( m_pController, m_currentConfigType, m_uploadConfigType );
}


void FWConfigUploadDialog::verifyConfig( bool selectManMode )
{
    m_pUploadManager->verifyConfig();

    m_pController->save();
    if ( selectManMode ) {
    m_pController->request( new SetStateCommand( ECUInfo::MAN ) );
    }
    m_pController->request( new DownloadAllCommand() );
}


void FWConfigUploadDialog::showFirmwareConfigResult( int result )
{
    /*
        wxMessageDialog * msgDlg;
        //wxMessageDialog(this, _T("Firmvare config is identical!"),
        //_T("Result"),wxOK |wxICON_ERROR, wxDefaultPosition);
        if(result == 0){
            msgDlg = new wxMessageDialog( m_parent, _T("Firmware config is identical!"),
            _T("Firmware config verify"), wxOK, wxDefaultPosition);
              msgDlg->ShowModal();
        }else if(result < 0){
            serialErrorMsgDialog();
        }else{
             msgDlg = new wxMessageDialog( m_parent, _T("Firmware config is different!\nMore details can be found on the Debug tab."),
            _T("Firmware config verify"), wxOK |wxICON_ERROR, wxDefaultPosition);
              msgDlg->ShowModal();
        }
*/
        //delete msgDlg;
}


void FWConfigUploadDialog::OnSIPR( InputEvent& event )
{
    SIPRPacket packet = event.getPacket();

    switch ( packet.type() )
    {
        case SIPR_ACK:
        {
            if ( m_pController->getState() == ECUInfo::MAN || m_pController->getState() == ECUInfo::MEGATUNE )
            {
                //Logger::logf( LOG_MSG, _T("FWConfigUploadDialog::OnSIPR() - MAN") );
                event.Skip();
                //m_pController->AddPendingEvent( event );
            }

            if ( m_pProgressDialog )
            {
//                m_pProgressDialog->Pulse();
            }

        } break;
        case SIPR_CONFIG_LINE:
        {
            if ( !m_iLineCount )
            {
                m_pProgressDialog->SetTitle( _T("Download config in progress!") );
            }

            wxString line( (const char*)packet.data(), wxConvUTF8 );
            m_pUploadManager->processConfigLine( line );

            if ( m_pProgressDialog )
            {
//                m_pProgressDialog->Pulse( wxString::Format( _T("Line: %d"), ++m_iLineCount ) );
            }
        } break;
        case SIPR_TABLES_END:
        {
            FirmwareConfigDifferences* fcDiff = m_pUploadManager->compare();

            ///create html log
            wxString htmlLog;
            wxString message;

            if ( fcDiff->isEmpty() ||
                    ( m_bIsFilteredUpload && fcDiff->extraScalars.IsEmpty() && fcDiff->extraTables.IsEmpty() ) )
            {
                htmlLog += _T("Verify config was successful!<br>");
                message += _T("Verify config was successful!\n");
            }
            else
            {
                if ( m_bIsFilteredUpload )
                {
                    for ( size_t i = 0; i < fcDiff->extraScalars.GetCount(); ++i )
                    {
                        htmlLog += _T("&nbsp;&nbsp;&nbsp;&nbsp;") + fcDiff->extraScalars[i] + _T("<br>");
                    }
                    //fcDiff->extraTables();
                }
                else
                {
                    htmlLog += ( _T("Verify config summary<br>") );
                    htmlLog += fcDiff->summary( _T("<br>") );
                    htmlLog += _T("Verify config details<br>");
                    htmlLog += fcDiff->details( _T("&nbsp;&nbsp;&nbsp;&nbsp;"), _T("<br>") );
                    message += fcDiff->summary( _T("\n") );
                    message += _T("\n More details can be found in debug tab.");
                }
            }

            m_observer->notify( htmlLog );

            reset();

            wxMessageBox( message, _T("Verify config") );

            //m_pController->request( new SetStateCommand( ECUInfo::MEGATUNE ) );
            m_pController->restore();
            m_pController->request( new ReadAllPageCommand() );
            if ( m_pController->isReadSensors() )
            {
                m_pController->request( new EnableReadRealTimeCommand( true ) );
            }

            stop();

        } break;
        case SIPR_CONFIG_ERROR:
        case SIPR_TIMEOUT:
        {
            reset();

            //m_pController->request( new SetStateCommand( ECUInfo::MEGATUNE ) );
            m_pController->restore();
            if ( m_pController->isReadSensors() )
            {
                m_pController->request( new EnableReadRealTimeCommand( true ) );
            }
            stop();

        } break;
    }
}


void FWConfigUploadDialog::reset()
{
    m_pUploadManager->clear();

    m_iLineCount = 0;
    m_bIsFilteredUpload = false;

    if ( m_pProgressDialog )
    {
        m_pProgressDialog->Destroy();
        m_pProgressDialog = NULL;
    }
}


void FWConfigUploadDialog::uploadConfigFromProject()
{
    Unidlg2Test dlg( m_parent );

    dlg.setDialogType( _T("config_upload") );
    dlg.SetLabel( _T("Upload Config Files") );
    wxString btn = _T("Upload");
    dlg.setButtonOK( btn );

    Project *prj = ProjectHelper::GetPrjMgr()->GetCurrPrj();

    dlg.setProjectConfigSaver( prj );
    dlg.loadProject( prj->CreateFirmwareConfigItems( dlg.getTypes() ) );

    if ( dlg.ShowModal() == wxID_OK )
    {
        wxString        headerFile  = dlg.getHeaderFile();
        wxArrayString   configFiles = dlg.getConfigFiles();

        m_pUploadManager->clear();
        uploadConfig( configFiles );
        verifyConfig(false);
    }
    else
    {
        unlockController();
    }
}



void FWConfigUploadDialog::verifyConfigFromProject()
{
    Unidlg2Test dlg( m_parent );

    dlg.setDialogType( _T("config") );
    dlg.SetLabel( _T("Verify Config Files") );
    wxString tmp = _T("Verify");
    dlg.setButtonOK(tmp);

    Project *prj = ProjectHelper::GetPrjMgr()->GetCurrPrj();

    dlg.setProjectConfigSaver(prj);
    dlg.loadProject( prj->CreateFirmwareConfigItems(dlg.getTypes()) );

    if ( dlg.ShowModal() == wxID_OK )
    {
        wxString headerFile         = dlg.getHeaderFile();
        wxArrayString configFiles   = dlg.getConfigFiles();

        //m_pUploadManager->clear();
        reset();
        m_pUploadManager->createUploadConfig( configFiles );

        m_pProgressDialog = new bcProgressDialog(   _T("Download config in progress!"),
                                                    _T(""),
                                                    100,
                                                    NULL,
                                                    wxPD_APP_MODAL | wxPD_AUTO_HIDE | wxPD_SMOOTH |
                                                    wxPD_ELAPSED_TIME | wxPD_ESTIMATED_TIME | wxPD_REMAINING_TIME );
        m_pProgressDialog->Pulse();

        verifyConfig();
    }
    else
    {
        unlockController();
    }
}

void FWConfigUploadDialog::verifyConfigFromFile()
{
    wxString configFile;

    if ( !openConfigFile( configFile ) )  {
        stop();
        return;
    }

    m_pController->request( new EnableReadRealTimeCommand( false ) );

    wxArrayString configFiles;
    configFiles.Add( configFile );

    reset();
    m_pUploadManager->createUploadConfig( configFiles );

    // uploadConfig( headerFile, configFiles );
    // stop();
    m_pProgressDialog = new bcProgressDialog(   _T("Download config in progress!"),
                                                _T(""),
                                                100,
                                                NULL,
                                                wxPD_APP_MODAL | wxPD_AUTO_HIDE | wxPD_SMOOTH |
                                                wxPD_ELAPSED_TIME | wxPD_ESTIMATED_TIME | wxPD_REMAINING_TIME );
    m_pProgressDialog->Pulse();

    verifyConfig();
}


void FWConfigUploadDialog::parseConfigFilterFile( const wxString& name, StringSet& keys )
{
    wxTextFile filterFile( name );

    if ( !filterFile.Open( wxConvISO8859_1 ) ) return;

    wxString line = filterFile.GetFirstLine();

    for ( ; !filterFile.Eof(); line = filterFile.GetNextLine() )
    {
        if ( line.Find( _T('=') ) == wxNOT_FOUND )
        {
            keys.insert( line );
        }
        else
        {
            keys.insert( line.BeforeFirst( _T('=') ) );
        }
    }

    filterFile.Close();
}

