#ifndef DIFFGAUGEDESCMANAGER_H
#define DIFFGAUGEDESCMANAGER_H

#include <wx/wxsqlite3.h>
#include "../common/types/wxVariantHash.h"
#include "../common/types/wxVariantHashArray.h"

class DiffGaugeDesc
{
    public:
        DiffGaugeDesc() {};
        DiffGaugeDesc(long id, const wxString& name, double minval, double maxval);
        DiffGaugeDesc(wxSQLite3ResultSet& profileLineResult);

        int GetId();
        wxString GetName();
        double GetMinValue();
        double GetMaxValue();

        wxVariant GetProp(const wxString& key);
        bool SetProp(const wxString& key, const wxVariant& val);

        void AddSymbolProps(wxVariantHash hash) { m_symbols.Add(hash); };

        wxVariantHashArray& GetSymbols() { return m_symbols; }
        wxVariantHash& GetSymbol(int id);

    private:

        wxVariantHash       m_properties;
        wxVariantHashArray  m_symbols;

        wxVariantHash       m_dummy;
};

WX_DECLARE_HASH_MAP(int, DiffGaugeDesc, wxIntegerHash, wxIntegerEqual, Int2DiffGaugeDescHash);

class DiffGaugeDescManager
{
    public:
        DiffGaugeDescManager();
        virtual ~DiffGaugeDescManager();

        bool LoadFromFile(const wxString& filename);

        DiffGaugeDesc& GetProfile(int id);
        DiffGaugeDesc& GetProfile(const wxString& name);
        bool IsProfileExists(const wxString& name);
        bool IsProfileExists(int id);

        wxString GetLastDbFile() { return m_lastDatabaseFile; }

        Int2DiffGaugeDescHash& GetProfiles() { return m_profiles; };

    protected:

        bool CheckDB();

    private:

        wxSQLite3Database       m_db;

        Int2DiffGaugeDescHash   m_profiles;

        DiffGaugeDesc           m_defaultDesc;

        wxString                m_lastDatabaseFile;
};

#endif // DIFFGAUGEDESCMANAGER_H

