#ifndef GAUGEDESCMANAGER_H
#define GAUGEDESCMANAGER_H

#include "../vemsgauge/vemsgaugedesc.h"
#include "../common/types/IntArray.h"


class GaugeGroupManager;
class WidgetThemeTableArray;
class MTConfigData;
class MultiGraphDescManager;
class VemsFlagDescManager;
class DiffGaugeDescManager;
class MultiSymbolGaugeDescMgr;

class SymbolGaugeDesc
{
    public:
        SymbolGaugeDesc() {};
        SymbolGaugeDesc(const wxString &name, const wxString &type,const wxString &title,const wxString &grpname){
            Type = type;
            Name = name;
            Title = title;
            Groupname = grpname;
        }

        static int compare_function(SymbolGaugeDesc **A,SymbolGaugeDesc **B);

        wxString Type;
        wxString Name;
        wxString Title;
        wxString Groupname;
};

WX_DECLARE_STRING_HASH_MAP( SymbolGaugeDesc*, String2SymbolGaugeDescHash );
WX_DECLARE_HASH_MAP(int, SymbolGaugeDesc*, wxIntegerHash, wxIntegerEqual, SymbolGaugeDescIntHash );
WX_DEFINE_ARRAY(SymbolGaugeDesc* ,SymbolGaugeDescArray);

WX_DECLARE_STRING_HASH_MAP(SymbolGaugeDescArray ,SymbolGaugeDescArrayHash);
WX_DECLARE_OBJARRAY(SymbolGaugeDescArray ,SymbolGaugeDescArrayArray);

class GaugeDescManager
{
    public:
		GaugeDescManager(){};
        GaugeDescManager(MTConfigData* mtConfigData);
        virtual ~GaugeDescManager();

        virtual void LoadDescriptors( int device );
        virtual bool LoadDescriptors(const wxString &filename) = 0;
        virtual bool SaveGaugeDescriptors(const wxString &filename) = 0;
        virtual bool SaveGaugeDescriptor(VEMSGaugeDesc * desc) = 0;
        virtual bool AddGaugeDescriptor(VEMSGaugeDesc * desc) = 0;
        virtual wxString GetType() = 0;
        virtual short ManageColoredArea( short command, ColoredArea * ca, int desc_id = -1 ) = 0;

        virtual void GetTypeList(wxArrayString &arr) = 0;
        virtual void GetSymbolGroupList(wxArrayString &arr) = 0;
        virtual bool AddSymbolDesc(const wxString &name, const wxString &type, const wxString &title, const wxString &grpname) = 0;

        int GetLastError() { return m_lastError; };
        wxString GetLastErrorStr() { return m_lastErrorStr; };

        virtual wxVariant GetGaugeDescFieldBySymbolName( const wxString &symbolname, const wxString &fieldname ) = 0;
        virtual wxVariant GetGaugeDescFieldByAnyField( const wxString &fieldname, const wxVariant& value, const wxString &resultfieldname ) = 0;
        virtual wxVariant GetSymbolFieldBySymbolName( const wxString &symbolname, const wxString &fieldname ) = 0;

        virtual bool GetColoredAreasBySymbolName( const wxString &symbolname, ColoredAreaRefArray& result) = 0;
        virtual bool GetColoredAreaByAnyField( const wxString &fieldname, const wxVariant& value, ColoredAreaRefArray& result) = 0;

        virtual int GetGaugeSettings(const wxString gaugeType, int gauge_id, wxVariantHash &datas) = 0;
        virtual int SaveGaugeSettings(const wxString gaugeType, int gauge_id, wxVariantHash &datas) = 0;

        wxString GetGaugeSymbolType(const wxString &symbolname);
        wxString GetGaugeSymbolTitle(const wxString &symbolname);

        VEMSGaugeDescArray * GetVEMSGaugeDescArray() { return &m_gaugeTable; };
        void Init();

        const WidgetThemeTableArray* GetWidgetThemes();
        void SetGaugeGroupMgr( GaugeGroupManager * mgr ) { m_gaugeGroupMgr = mgr; };
        GaugeGroupManager * GetGaugeGrpMgr() { return m_gaugeGroupMgr; };

        void LoadGDDs(const wxString &filename);
        GDDescArray * GetGDDescArray() { return &m_gaugeTableDesc; };

        void GetSymbolDescs(SymbolGaugeDescArray &arr);
        void GetGrouppedSymbolDescs(SymbolGaugeDescArrayHash &hash);
        size_t GetSymbolGroupListM(wxSortedArrayString &arr); // get from model
        size_t GetSymbolDescs(const wxString& grpname, SymbolGaugeDescArray &arr);


        MTConfigData* getMTConfigData();

        bool CheckSymbolName(const wxString &symbolname);


        MultiGraphDescManager* GetMultiGraphDescMgr() { return m_mgMgr; };

        void CheckConfigFile(const wxString& file, const wxString& deffile);

        VemsFlagDescManager* GetVemsFlagDescMgr() { return m_flagDescMgr; };
        //DiffGaugeDescManager* GetDiffGaugeDescMgr() { return m_diffGaugeMgr; };
        MultiSymbolGaugeDescMgr* GetDiffGaugeDescMgr() { return m_diffGaugeMgr; };
        MultiSymbolGaugeDescMgr* GetColsGaugeDescMgr() { return m_colsGaugeMgr; };

    protected:

        void ClearLastError() { m_lastError = 0; m_lastErrorStr = wxEmptyString; }
        void SetLastError(int code, const wxString& str = wxEmptyString) { m_lastError = code; m_lastErrorStr = str; }

        void InitDiffGaugeDesc();
        void InitColsGaugeDesc();

        VEMSGaugeDescArray          m_gaugeTable;

        String2SymbolGaugeDescHash  m_symbolDesc;
        SymbolGaugeDescArrayHash    m_symbolDescGroup;

        MultiGraphDescManager*      m_mgMgr;

        VemsFlagDescManager*        m_flagDescMgr;
        //DiffGaugeDescManager*       m_diffGaugeMgr;

        MultiSymbolGaugeDescMgr*    m_diffGaugeMgr;
        MultiSymbolGaugeDescMgr*    m_colsGaugeMgr;

        MTConfigData*               m_mtConfigData;

        int                         m_lastError;
        wxString                    m_lastErrorStr;

        wxString                m_gaugeSettingsDBfile;

    private:

        GaugeGroupManager   *   m_gaugeGroupMgr;
        GDDescArray             m_gaugeTableDesc;


        bool    m_isLockGauges;

        void GetGaugeTypesFromStr(const wxString& str, IntArray& arr, const wxString& delimiter = _T(";"));

};

#endif // GAUGEDESCMANAGER_H

