#include "GaugeDescManagerSQL.h"

#include <wx/filename.h>

#include "../common/VemsMessageHashHelper.h"
#include "../common/FunctionMeter.h"
#include <wx/wxsqlite3.h>
#include "../logger.h"
#include "../UserLog.h"
#include "../DirHelper.h"
#include "../wxVariantHelper.h"

#include "../wxSQLColumnDesc.h"
#include "../wxSqliteHelper.h"


#include "../common/types/StringHash.h"

#define chkdbfile(x) if( !wxFileName::FileExists(m_dbFilename) ) return x;

#define chkdb(retval) if( ! (m_db.IsOpen()) ) return retval;

#define chkgsdb(retval) if( ! (m_gsdb.IsOpen()) ) return retval;

GaugeDescManagerSQL::GaugeDescManagerSQL(MTConfigData* mtConfigData)
    :GaugeDescManager(mtConfigData)
{
    //ctor

    m_gaugeSettingsDBfile = DirHelper::GetPath( dCONFIG ) + _T("GaugesSettings.db");

    m_gsdb.Open(m_gaugeSettingsDBfile);
}

GaugeDescManagerSQL::~GaugeDescManagerSQL()
{
    //dtor
}

bool GaugeDescManagerSQL::GetColoredAreaByAnyField( const wxString &fieldname, const wxVariant& value, ColoredAreaRefArray& result)
{
    chkdb(false)




/**
select * from colored_areas inner join descriptors on colored_areas.desc_id = descriptors.id
 inner join symbols on symbols.type=descriptors.type where symbols.name='rpm' and descriptors.id = colored_areas.desc_id ;

*/

    wxSQLite3ResultSet cares;

    wxString qs = wxString::Format(_T("select * from colored_areas where desc_id = \
                                            ( select id from descriptors where %s = '%s' limit 1 ) order by min asc")
                                                , fieldname.c_str(), value.MakeString().c_str() );

    try {
        cares = m_db.ExecuteQuery( qs );
    } catch ( wxSQLite3Exception& e ) {
        LOGP(_T("qs=")+qs);
        Logger::logf(LOG_MSG,_T("exception: ") + e.GetMessage() );
        SetLastError(e);
        return false;
    }

    while( cares.NextRow() ){
        result.Add( ColoredArea(cares.GetDouble(_T("min")),cares.GetDouble(_T("max")),cares.GetString(_T("colorname")),cares.GetInt(_T("id"))) );
    }


    return true;
}


bool GaugeDescManagerSQL::GetColoredAreasBySymbolName( const wxString &symbolname, ColoredAreaRefArray& result)
{
    chkdb(false)




/**
select * from colored_areas inner join descriptors on colored_areas.desc_id = descriptors.id
 inner join symbols on symbols.type=descriptors.type where symbols.name='rpm' and descriptors.id = colored_areas.desc_id ;

*/

    wxSQLite3ResultSet cares;

    wxString qs = wxString::Format(_T("select * from colored_areas where desc_id = \
                                            ( select descriptors.id from descriptors inner join symbols on \
                                            symbols.type=descriptors.type where symbols.name='%s' limit 1 ) order by min asc")
                                                , symbolname.c_str());

    try {
        cares = m_db.ExecuteQuery( qs );
    } catch ( wxSQLite3Exception& e ) {
        LOGP(_T("qs=")+qs);
        Logger::logf(LOG_MSG,_T("exception: ") + e.GetMessage() );
        SetLastError(e);
        return false;
    }

    while( cares.NextRow() ){
        result.Add( ColoredArea(cares.GetDouble(_T("min")),cares.GetDouble(_T("max")),cares.GetString(_T("colorname")),cares.GetInt(_T("id"))) );
    }


    return true;
}

wxVariant GaugeDescManagerSQL::GetGaugeDescFieldByAnyField( const wxString &fieldname, const wxVariant& value, const wxString &resultfieldname )
{
    chkdb(wxNullVariant)

    wxSQLite3ResultSet res;

    wxString qs = wxString::Format(_T("select %s from descriptors inner join symbols \
                            on symbols.type=descriptors.type and descriptors.%s='%s' order by descriptors.id limit 1;"),
                                resultfieldname.c_str(),fieldname.c_str(), wxVariantHelper::MakeString(value).c_str() );

    try {

        res = m_db.ExecuteQuery( qs );

    } catch ( wxSQLite3Exception& e ) {
        LOGP(_T("qs=")+qs);
        Logger::logf(LOG_MSG,_T("exception: ") + e.GetMessage() );
        SetLastError(e);
        return wxNullVariant;
    }



    wxVariant data;

    int a = 0;

    switch( res.GetColumnType(a) ){
        case WXSQLITE_INTEGER:
            data = (long)(res.GetInt(a)); break;
        case WXSQLITE_FLOAT:
            data = res.GetDouble(a); break;
        case WXSQLITE_TEXT:
            data = res.GetString(a); break;
        default:
            break;
    }

//    LOGP(_T("s=")+symbolname+_T(", f=")+fieldname + _T(", data=")+data.MakeString());

    return data;
}

void GaugeDescManagerSQL::SetLastError(wxSQLite3Exception& e)
{
    m_lastError = e.GetErrorCode();
    m_lastErrorStr = e.GetMessage();
}


wxVariant GaugeDescManagerSQL::GetSymbolFieldBySymbolName( const wxString &symbolname, const wxString &fieldname )
{
    chkdb(wxNullVariant)

    wxString qs = wxString::Format(_T("select %s from symbols where name='%s' limit 1;"),fieldname.c_str(),symbolname.c_str());

    wxVariant data;
    wxSQLite3ResultSet res;

    try{
        res = m_db.ExecuteQuery( qs );
        ClearLastError();
    } catch (wxSQLite3Exception& e) {
        LOGP(_T("qs=")+qs);
        Logger::logf(LOG_MSG,_T("exception: ") + e.GetMessage() );
        SetLastError(e);
        return wxNullVariant;
    }


    int a = 0;

    switch( res.GetColumnType(a) ){
        case WXSQLITE_INTEGER:
            data = (long)(res.GetInt(a)); break;
        case WXSQLITE_FLOAT:
            data = res.GetDouble(a); break;
        case WXSQLITE_TEXT:
            data = res.GetString(a); break;
        default:
            break;
    }

//    LOGP(_T("s=")+symbolname+_T(", f=")+fieldname + _T(", data=")+data.MakeString());

    return data;
}

wxVariant GaugeDescManagerSQL::GetGaugeDescFieldBySymbolName( const wxString &symbolname, const wxString &fieldname )
{
    chkdb(wxNullVariant)

    wxString qs = wxString::Format(_T("select %s from descriptors inner join symbols on symbols.type=descriptors.type \
                            where symbols.name='%s' order by descriptors.id limit 1;"),fieldname.c_str(),symbolname.c_str());

    wxVariant data;
    wxSQLite3ResultSet res;

    try{
        res = m_db.ExecuteQuery( qs );
        ClearLastError();
    } catch (wxSQLite3Exception& e) {
        LOGP(_T("qs=")+qs);
        Logger::logf(LOG_MSG,_T("exception: ") + e.GetMessage() );
        SetLastError(e);
        return wxNullVariant;
    }


    int a = 0;

    switch( res.GetColumnType(a) ){
        case WXSQLITE_INTEGER:
            data = (long)(res.GetInt(a)); break;
        case WXSQLITE_FLOAT:
            data = res.GetDouble(a); break;
        case WXSQLITE_TEXT:
            data = res.GetString(a); break;
        default:
            break;
    }

//    LOGP(_T("s=")+symbolname+_T(", f=")+fieldname + _T(", data=")+data.MakeString());

    return data;
}

bool GaugeDescManagerSQL::AddGaugeDescriptor(VEMSGaugeDesc * desc)
{
    chkdb(false)

    wxString dname = desc->Properties[_T("name")].GetString();

    wxSQLite3ResultSet res = m_db.ExecuteQuery( _T("select id from descriptors where name='") + dname + _T("';") );

    if( !res.IsNull(0) ) {

        UserLog::uLog( ULOG_ERROR, _T("<b>The '") + dname + _T("' descriptor name is alredy exist!</b></font><br>Please choose an other one!") );
        return false;
    }

    wxString qs = _T("insert into descriptors");
    wxString fields = _T(" (");
    wxString values = _T(" values (");

    for( wxVariantHash::iterator it = desc->Properties.begin(); it != desc->Properties.end(); it++ ){
        if(it->first == _T("id")) continue;

        fields += it->first + _T(",");
        values += _T("'") + it->second.MakeString() + _T("',");
    }

    fields.RemoveLast();
    values.RemoveLast();

    fields += _T(") ");
    values += _T(");");

    qs += fields + values;

//    Logger::logf(LOG_MSG,_T("qs=")+qs);

    m_db.ExecuteUpdate( qs );

    int did = m_db.GetLastRowId().ToLong();

    qs = _T("insert into colored_areas (min,max,colorname,desc_id) values (");

    wxString nf = desc->Properties[_T("numformat")].GetString();

    m_db.Begin();

    for( size_t a=0; a < desc->coloredA.GetCount(); a++){
        ColoredArea * ca = desc->coloredA[a];
        wxString datas = wxString::Format( _T("'") + nf + _T("','") + nf + _T("','") + ca->ColorName + _T("','%i');") , ca->minValue, ca->maxValue, did );
        m_db.ExecuteUpdate( qs + datas );
    }

    m_db.Commit();

    m_gaugeTable.Add( desc );

    return true;
}

bool GaugeDescManagerSQL::LoadDescriptors(const wxString &filename)
{
    WX_CLEAR_ARRAY( m_gaugeTable );
    WX_CLEAR_HASH_MAP( String2SymbolGaugeDescHash, m_symbolDesc );

    FunctionMeter fm(_T("GaugeDescManagerSQL::LoadDescriptors"));

    wxFileName fn(filename);

    Logger::logf(LOG_MSG,_T("GaugeDescManagerSQL::LoadDescriptors fn.GetExt()=")+fn.GetExt() +_T(" db name: ")+filename);

    if( fn.GetExt() != _T("db") && fn.GetExt() != _T("s3db") ) {
        StringHash hash;
        hash[_T("filename")] = filename;
        hash[_T("type")] = _T("gauge descriptor");
        UserLog::vLog( VemsMessageHashHelper::GetVemsMsg(_T("db_file_bad_ext"),&hash) );
        return false;
    }

    m_dbFilename = filename;

    m_db.Open( filename );
    if( !m_db.IsOpen() )
        return false;

//    Logger::logf(LOG_MSG,_T("GaugeDescManagerSQL::LoadDescriptors() db open OK") );
    wxSQLite3ResultSet res = m_db.ExecuteQuery( _T("select * from descriptors order by name asc;") );
//    Logger::logf(LOG_MSG,_T("GaugeDescManagerSQL::LoadDescriptors() execute query") );
    VEMSGaugeDesc * gd;

    while( res.NextRow() ){

        gd = new VEMSGaugeDesc();

        for(int a = 0; a < res.GetColumnCount(); a++) {

            switch( res.GetColumnType(a) ){
                case WXSQLITE_INTEGER:
                    gd->Properties[ res.GetColumnName(a) ] = (long)(res.GetInt(a)); break;
                case WXSQLITE_FLOAT:
                    gd->Properties[ res.GetColumnName(a) ] = res.GetDouble(a); break;
                case WXSQLITE_TEXT:
                    gd->Properties[ res.GetColumnName(a) ] = res.GetString(a); break;
                default:
                    break;
            }
        }

        wxSQLite3ResultSet cares = m_db.ExecuteQuery( wxString::Format(_T("select * from colored_areas where desc_id=%i order by min asc;"),res.GetInt(_T("id"))) );

        while( cares.NextRow() ){
            gd->coloredA.Add( new ColoredArea(cares.GetDouble(_T("min")),cares.GetDouble(_T("max")),cares.GetString(_T("colorname")),cares.GetInt(_T("id"))) );
        }

        m_gaugeTable.Add( gd );

    }

    //wxString qs = _T("select * from symbols;") );
    wxString qs = _T("select name,type,symbols.title,symbolgroups.title as groupname from \
                     symbols inner join symbolgroups on symbolgroups.id = symbols.group_id order by groupname asc;");

    wxSQLite3ResultSet sres = m_db.ExecuteQuery( qs );

    while( sres.NextRow() ){
        SymbolGaugeDesc * desc = new SymbolGaugeDesc(sres.GetString(_T("name")),sres.GetString(_T("type")),sres.GetString(_T("title")),sres.GetString(_T("groupname")) );
        m_symbolDesc[ sres.GetString(_T("name")) ] = desc;
    }


    Logger::logf(LOG_MSG,_T("GaugeDescManagerSQL::LoadDescriptors() END") );
    return true;
}

bool GaugeDescManagerSQL::SaveGaugeDescriptors(const wxString &filename)
{
    /// maybe not necassary
    return true;
}

bool GaugeDescManagerSQL::SaveGaugeDescriptor(VEMSGaugeDesc * desc)
{
    chkdb(false)

    wxString qs = _T("update descriptors set ");

    for( wxVariantHash::iterator it = desc->Properties.begin(); it != desc->Properties.end(); it++ ){
        if(it->first == _T("id")) continue;
        qs += _T(" ") + it->first + _T(" = '") + it->second.MakeString() + _T("',");
    }

    qs.RemoveLast();

    qs += wxString::Format( _T(" where id = %i;"), desc->Properties[_T("id")].GetLong() );

//    Logger::logf(LOG_MSG,_T("qs=")+qs);

    m_db.ExecuteUpdate( qs );


    qs = _T("update colored_areas set ");

    wxString nf = desc->Properties[_T("numformat")].GetString();

    m_db.Begin();

    for( size_t a=0; a < desc->coloredA.GetCount(); a++){
        ColoredArea * ca = desc->coloredA[a];
        wxString datas = wxString::Format( _T("min='") + nf + _T("', max='") + nf + _T("', colorname='") + ca->ColorName + _T("' where id=%i;") , ca->minValue, ca->maxValue, ca->itemID );
        m_db.ExecuteUpdate( qs + datas );
    }

    m_db.Commit();

    return true;
}


short GaugeDescManagerSQL::ManageColoredArea( short command, ColoredArea * ca, int desc_id)
{
    chkdb(0)

    short res = 0;

    wxString qs;

    switch( command ){
        case wxID_ADD:
            qs = _T("insert into colored_areas (min,max,colorname,desc_id) values (");
            qs += wxString::Format( _T("'%f','%f','") + ca->ColorName + _T("','%i');") , ca->minValue, ca->maxValue, desc_id );
            m_db.ExecuteUpdate( qs );
            ca->itemID = m_db.GetLastRowId().ToLong();
            break;

        case wxID_DELETE:
            m_db.ExecuteUpdate( wxString::Format(_T("delete from colored_areas where id=%i;"), ca->itemID) );
            break;

        default:
            res = -2;
            break;
    }


    return res;
}

void GaugeDescManagerSQL::GetTypeList(wxArrayString &arr)
{
    chkdb();


    //wxSQLite3ResultSet res =  m_db.ExecuteQuery( _T("select distinct type from symbols order by type asc;") );
    wxSQLite3ResultSet res =  m_db.ExecuteQuery( _T("select distinct type from descriptors order by type asc;") );

    while( res.NextRow() ){
        arr.Add( res.GetString(0) );
    }


}

void GaugeDescManagerSQL::GetSymbolGroupList(wxArrayString &arr)
{
    chkdb();

    wxSQLite3ResultSet res =  m_db.ExecuteQuery( _T("select title from symbolgroups order by title asc;") );

    while( res.NextRow() ){
        arr.Add( res.GetString(0) );
    }
}

bool GaugeDescManagerSQL::AddSymbolDesc(const wxString &name, const wxString &type, const wxString &title, const wxString &grpname)
{
    chkdb(false);

    int gid = m_db.ExecuteScalar( wxString::Format(_T("select id from symbolgroups where title = '%s';"),grpname.c_str()) );

    wxString qs = wxString::Format(_T("insert into symbols (name,type,title,group_id) values('%s','%s','%s',%i);"),name.c_str(),type.c_str(),title.c_str(),gid);

    try{
        m_db.ExecuteUpdate( qs );
        ClearLastError();
    } catch (wxSQLite3Exception &ex) {
        UserLog::uLog(ULOG_ERROR, ex.GetMessage());

        SetLastError(ex);
        return false;
    }


    return true;
}

bool GaugeDescManagerSQL::CreateGSTable(wxSQLite3Database *db, const wxString &tablename, wxVariantHash &fields)
{
    if(tablename.Len() < 1 || fields.size() < 1) return false;

    wxString qs = _T("CREATE TABLE ") + tablename + _T(" (gauge_id INTEGER UNIQUE NULL, ");

    for(wxVariantHash::iterator it = fields.begin(); it != fields.end() ; it++ ){
        wxString fieldtype = wxSqliteHelper::ConvertWxTypesToSQLTypes( it->second.GetType() );
        if( fieldtype == _T("unknown") )
            continue;

        qs += it->first + _T(" ") + fieldtype + _T(", ");
    }

    qs.RemoveLast(2);

    qs += _T(");");

    Logger::logf(LOG_MSG,_T("CreateGSTable qs=")+qs);

    bool res = true;

    try{
        db->ExecuteUpdate( qs );
    } catch (wxSQLite3Exception &exc) {
        Logger::logf(LOG_MSG,_T("GaugeDescManagerSQL::CreateGSTable() ") + exc.GetMessage() );
        res = false;
    }

    return res;
}


int GaugeDescManagerSQL::GetGaugeSettings(const wxString gaugeType, int gauge_id, wxVariantHash &datas)
{
    chkgsdb(-1);


    if( !(m_gsdb.TableExists( gaugeType )) )
        return -2;

    wxString qs = wxString::Format(_T("select * from ")+gaugeType+_T(" where gauge_id=%i;"), gauge_id);

    wxSQLite3ResultSet res = m_gsdb.ExecuteQuery( qs );

//    Logger::logf(LOG_MSG,_T("qs = ")+qs);

    if(res.IsNull(0))
        return -3;


    for(int a = 0; a < res.GetColumnCount(); a++) {
//        Logger::logf(LOG_MSG,_T("type=") +res.GetDeclaredColumnType(a) + _T(", i=%i"),res.GetColumnType(a) );

        switch( res.GetColumnType(a) ){
            case WXSQLITE_INTEGER:
                datas[ res.GetColumnName(a) ] = (long)(res.GetInt(a)); break;
            case WXSQLITE_FLOAT:
                datas[ res.GetColumnName(a) ] = res.GetDouble(a); break;
            case WXSQLITE_TEXT:
                datas[ res.GetColumnName(a) ] = res.GetString(a); break;
            default:
             //   if( res.GetDeclaredColumnType(a) == _T("boolean") ) datas[ res.GetColumnName(a) ] = res.GetBool(a);
                break;
        }


    }

    return 0;
}

int GaugeDescManagerSQL::SaveGaugeSettings(const wxString gaugeType, int gauge_id, wxVariantHash &datas)
{
    if(gauge_id < 1) return -1;

    chkgsdb(-1);

    wxString qs;

    if( !(m_gsdb.TableExists( gaugeType )) ) {
        if(CreateGSTable( &m_gsdb, gaugeType, datas ) == false) return -1;
    }

    wxSQLite3ResultSet res = m_gsdb.ExecuteQuery(wxString::Format(_T("select * from ")+gaugeType+_T(" where gauge_id=%i;"), gauge_id));

    bool bres = res.IsNull(0);

    //Logger::logf(LOG_MSG,_T("SaveGaugeSettings type=")+gaugeType+_T(",r=%i, id=%i"),bres,gauge_id);

    if( bres ){
        qs = _T("insert into ") + gaugeType;
        wxString fields = _T(" (gauge_id,");
        wxString values = wxString::Format(_T(" values ('%i',"),gauge_id);

        for(wxVariantHash::iterator it = datas.begin(); it != datas.end() ; it++ ){
            wxSqliteHelper::ColumnExists( &m_gsdb, gaugeType, it->first, true, wxSqliteHelper::ConvertWxTypesToSQLTypes( it->second.GetType() ) );

            fields += it->first + _T(",");
            values += _T("'") + it->second.MakeString() + _T("',");
        }

        fields.RemoveLast();
        values.RemoveLast();
        fields += _T(") ");
        values += _T(");");

        qs += fields + values;

        m_gsdb.ExecuteUpdate( qs );

        //Logger::logf(LOG_MSG,_T("insert qs = ")+qs);

    } else {
        qs = _T("update ") + gaugeType + _T(" set ");

        for(wxVariantHash::iterator it = datas.begin(); it != datas.end() ; it++ ){
            wxSqliteHelper::ColumnExists( &m_gsdb, gaugeType, it->first, true, wxSqliteHelper::ConvertWxTypesToSQLTypes( it->second.GetType() ) );

            qs += it->first + _T("='") + it->second.MakeString() + _T("', ");
        }

        qs.RemoveLast( 2 );

        m_gsdb.ExecuteUpdate( qs + wxString::Format(_T(" where gauge_id=%i;"),gauge_id));

        //Logger::logf(LOG_MSG,_T("update qs = ")+qs);
    }

    return 0;
}



