#include "MultiGraphDesc.h"

#include <wx/arrimpl.cpp>
WX_DEFINE_OBJARRAY(MultiGraphDescNodeArray)


MultiGraphDesc::MultiGraphDesc(MultiGraphDesc& m)
{
    Set(&m);
}

void MultiGraphDesc::Set(MultiGraphDesc* m)
{

    Cells.Clear();
    m_removedNodeIDs.Clear();
    m_removedSymbolIDs.Clear();

    WX_APPEND_ARRAY(Cells,m->Cells);

    WX_APPEND_ARRAY(m_removedSymbolIDs,m->m_removedSymbolIDs);
    WX_APPEND_ARRAY(m_removedNodeIDs,m->m_removedNodeIDs);


    ProfileID = m->ProfileID;
    m_itemID = m->m_itemID;

    IsChanged = false;
}

void MultiGraphDesc::GetSymbolNameList(wxArrayString &arr)
{
    for(size_t a=0;a<Cells.GetCount();a++)
        for(size_t b=0;b<Cells[a].Symbols.GetCount();b++)
            arr.Add(Cells[a].Symbols[b][_T("name")].GetString());

}

wxString MultiGraphDesc::GetLastSymbol()
{
    int cell_count = Cells.GetCount();

    if(cell_count < 1) return wxEmptyString;

    for(int a=cell_count-1; a >= 0; a--) {

        int cellitemcount = Cells[a].Symbols.GetCount();

        if( cellitemcount < 1 ) continue;

        wxVariantHash &hash = Cells[a].Symbols[cellitemcount-1];
        return hash[_T("name")];

    }

    return wxEmptyString;
}

wxString MultiGraphDesc::GetFirstSymbol()
{
    size_t cell_count = Cells.GetCount();

    if(cell_count == 0) return wxEmptyString;

    for(size_t a=0; a < cell_count; a++) {

        if( Cells[a].Symbols.GetCount() == 0 ) continue;

        wxVariantHash &hash = Cells[a].Symbols[0];
        return hash[_T("name")];

    }

    return wxEmptyString;
}

wxVariantHash* MultiGraphDesc::FindBy(const wxVariant& value, const wxString& fieldname, const wxVariant& except_value, const wxString& except_fieldname)
{
    for(size_t a=0;a<Cells.GetCount();a++) {

        for(size_t b=0;b<Cells[a].Symbols.GetCount();b++) {
            wxVariantHash& hash = Cells[a].Symbols[b];

            if( hash.find(fieldname) == hash.end() ) continue;

            if(except_value != wxNullVariant){
                if(hash.find(except_fieldname) == hash.end()) continue;
                if(hash[except_fieldname] == except_value) continue;
            }

            if( hash[fieldname] == value ) return &hash;
        }
    }

    return NULL;
}

long MultiGraphDesc::GetNewItemID()
{
    m_itemID--;
    return m_itemID;
}
