#include "MultiGraphDescManager.h"

#include <wx/wx.h>
#include <wx/fileconf.h>
#include <wx/wfstream.h>


#include "../logger.h"
#include "../DirHelper.h"
#include "../DrawHelper.h"

#include "../wxSqliteHelper.h"

#define chkdescdb(retval) if( ! (m_descDB.IsOpen()) ) return retval;
#define chkthemedb(retval) if( ! (m_themeDB.IsOpen()) ) return retval;

MultiGraphDescManager::MultiGraphDescManager()
{
    //ctor


}

MultiGraphDescManager::~MultiGraphDescManager()
{
    //dtor
    m_descDB.Close();
    m_themeDB.Close();
}

void MultiGraphDescManager::CheckThemeDB()
{
    ColumnDescArrayHash desc;

    ColumnDescArray arr1;
    arr1.Add( ColumnDesc(_T("id"), _T("INTEGER"), wxNullVariant, true, true ) );
    arr1.Add( ColumnDesc(_T("title"), _T("VARCHAR(64)")) );
    desc[_T("profiles")] = arr1;

    ColumnDescArray arr2;
    arr2.Add( ColumnDesc(_T("id"), _T("INTEGER"), wxNullVariant, true, true ) );
    arr2.Add( ColumnDesc(_T("name"), _T("VARCHAR(64)")) );
    arr2.Add( ColumnDesc(_T("title"), _T("VARCHAR(64)")) );
    desc[_T("properties")] = arr2;

    ColumnDescArray arr3;
    arr3.Add( ColumnDesc(_T("prop_id"), _T("INTEGER")) );
    arr3.Add( ColumnDesc(_T("profile_id"), _T("INTEGER")) );
    arr3.Add( ColumnDesc(_T("tvalue"), _T("VARCHAR(32)")) );
    desc[_T("tvalues")] = arr3;

    wxSqliteHelper::DatabaseCheck(m_mvgThemeFile,desc);

    int profc = m_themeDB.ExecuteScalar(_T("select count(*) from profiles;"));
    int propc = m_themeDB.ExecuteScalar(_T("select count(*) from properties;"));
    int valc  = m_themeDB.ExecuteScalar(_T("select count(*) from tvalues;"));

    if( profc < 1 ) wxSqliteHelper::ExecuteFromFile(&m_themeDB, DirHelper::GetPath(dDEFAULTSTATEMENTS) + _T("MultiGraphThemeDefProfiles.sql") );
    if( propc < 1 ) wxSqliteHelper::ExecuteFromFile(&m_themeDB, DirHelper::GetPath(dDEFAULTSTATEMENTS) + _T("MultiGraphThemeDefProps.sql") );
    if( valc < 1 )  wxSqliteHelper::ExecuteFromFile(&m_themeDB, DirHelper::GetPath(dDEFAULTSTATEMENTS) + _T("MultiGraphThemeDefValues.sql") );
}

void MultiGraphDescManager::CheckDescDB()
{
    ColumnDescArrayHash desc;

    ColumnDescArray arr1;
    arr1.Add( ColumnDesc(_T("id"), _T("INTEGER"), wxNullVariant, true, true ) );
    arr1.Add( ColumnDesc(_T("title"), _T("VARCHAR(255)")) );
    arr1.Add( ColumnDesc(_T("profile_id"), _T("INTEGER")) );
    arr1.Add( ColumnDesc(_T("ordernum"), _T("INTEGER") ) );
    arr1.Add( ColumnDesc(_T("flagdisplayratio"), _T("FLOAT")) );
    desc[_T("nodes")] = arr1;

    ColumnDescArray arr2;
    arr2.Add( ColumnDesc(_T("id"), _T("INTEGER"), wxNullVariant, true, true ) );
    arr2.Add( ColumnDesc(_T("title"), _T("VARCHAR(255)")) );
    desc[_T("profiles")] = arr2;

    ColumnDescArray arr3;
    arr3.Add( ColumnDesc(_T("id"), _T("INTEGER"), wxNullVariant, true, true ) );
    arr3.Add( ColumnDesc(_T("name"), _T("VARCHAR(255)")) );
    arr3.Add( ColumnDesc(_T("color"), _T("VARCHAR(255)")) );
    arr3.Add( ColumnDesc(_T("type"), _T("VARCHAR(255)") ) );
    arr3.Add( ColumnDesc(_T("limit_min"), _T("FLOAT")) );
    arr3.Add( ColumnDesc(_T("limit_max"), _T("FLOAT") ) );
    arr3.Add( ColumnDesc(_T("overlimitcolor"), _T("VARCHAR(255)")) );
    arr3.Add( ColumnDesc(_T("customtitle"), _T("VARCHAR(255)") ) );
    arr3.Add( ColumnDesc(_T("node_id"), _T("INTEGER")) );
    arr3.Add( ColumnDesc(_T("ordernum"), _T("INTEGER") ) );
    arr3.Add( ColumnDesc(_T("value_min"), _T("FLOAT")) );
    arr3.Add( ColumnDesc(_T("value_max"), _T("FLOAT") ) );
    arr3.Add( ColumnDesc(_T("gaugedesc_id"), _T("INTEGER") ) );
    arr3.Add( ColumnDesc(_T("scale"), _T("FLOAT") ) );
    desc[_T("symbols")] = arr3;

    wxSqliteHelper::DatabaseCheck(m_mvgDescFile,desc);

    int profc = m_descDB.ExecuteScalar(_T("select count(*) from profiles;"));
    int nodec = m_descDB.ExecuteScalar(_T("select count(*) from nodes;"));
    int symbc = m_descDB.ExecuteScalar(_T("select count(*) from symbols;"));

    if(profc < 1 || nodec < 1 || symbc < 1 )
        wxSqliteHelper::ExecuteFromFile(&m_themeDB, DirHelper::GetPath(dDEFAULTSTATEMENTS) + _T("MultiGraphDescDef.sql") );
}

void MultiGraphDescManager::InitMgr(const wxString& descDBFileName, const wxString& themeDBFileName)
{
    wxString fn = descDBFileName;
    if(fn == wxEmptyString)
        fn = DirHelper::GetPath(dCONFIG) + _T("MultiGraphDesc.db");

    m_mvgDescFile = fn;
    m_descDB.Open( m_mvgDescFile );

    wxString fn2 = themeDBFileName;

    if(fn2 == wxEmptyString)
        fn2 = DirHelper::GetPath(dCONFIG) + _T("MultiGraphTheme.db");

    m_mvgThemeFile = fn2;
    m_themeDB.Open( m_mvgThemeFile );

    CheckDescDB();
    CheckThemeDB();

    CreateMGDescStruct();
    LoadMultiGraphDesc();
    LoadMultiGraphThemes();
}

MultiGraphDesc* MultiGraphDescManager::GetMultiGraphDesc(size_t id)
{
    if( m_multiGraphDescHash.size() == 0 ) return new MultiGraphDesc();

    if( m_multiGraphDescHash.find(id) == m_multiGraphDescHash.end() ) return m_multiGraphDescHash.begin()->second;

    return m_multiGraphDescHash[id];
}

void MultiGraphDescManager::SetMultiGraphDesc(MultiGraphDesc* desc, size_t id)
{
    m_multiGraphDescHash[id] = desc;
}

MultiGraphTheme* MultiGraphDescManager::GetMultiGraphTheme(size_t id)
{
    if( m_multiGraphThemeHash.size() == 0 ) return new MultiGraphTheme();

    if( m_multiGraphThemeHash.find(id) == m_multiGraphThemeHash.end() ) return m_multiGraphThemeHash.begin()->second;

    return m_multiGraphThemeHash[id];
}

void MultiGraphDescManager::SetMultiGraphTheme(MultiGraphTheme* theme, size_t id)
{
    m_multiGraphThemeHash[id] = theme;
}

int MultiGraphDescManager::InsertSymbol(wxVariantHash& hash, int node_id)
{
    chkdescdb(-1);

    GUIDescStructItemArray& dsiarr = GetMGDescStruct();

    wxString sqs1 = _T("insert into symbols (");
    wxString sqs2 = _T(" values (");

    for(size_t c = 0; c < dsiarr.GetCount(); c++) {
        wxString pn = dsiarr[c].Name;

        if(pn == _T("id")) continue;
        if(pn == _T("node_id")) {
            sqs1 += pn + _T(",");
            sqs2 += wxString::Format(_T("%i,"),node_id);
            continue;
        }

        if( hash.find(pn) != hash.end() ) {
            sqs1 += pn + _T(",");

            switch(dsiarr[c].Type){
                case GDSI_COLOR:{
                    wxColor color;
                    color << hash[pn];
                    sqs2 += _T("'") + DrawHelper::GetColorAsString( color, wxC2S_HTML_SYNTAX ) + _T("',");
                    }break;
                case GDSI_NUMBER:
                case GDSI_SYMBOLDESC:
                    sqs2 += hash[pn].MakeString() + _T(",");
                    break;
                case GDSI_SYMBOLNAME:
                case GDSI_TEXT:
                    sqs2 += _T("'") + hash[pn].GetString() + _T("',");
            }
        }
    }

    sqs1.RemoveLast();
    sqs1 += _T(")");

    sqs2.RemoveLast();
    sqs2 += _T(");");

    int symbol_id = -1;

    try {
        m_descDB.ExecuteUpdate( sqs1 + sqs2 );
        symbol_id = m_descDB.GetLastRowId().ToLong();
    } catch(wxSQLite3Exception& se) {
        Logger::logf(LOG_MSG,_T("InsertSymbol sql error: ")+se.GetMessage());
    }

    return symbol_id;

}

int MultiGraphDescManager::InsertNode(MultiGraphDescNode &nodeDesc, int profile_id)
{
    chkdescdb(-1);

    size_t cac = nodeDesc.Symbols.GetCount();

    wxVariantHash& nHash = nodeDesc.Properties;

    wxString nqs = wxString::Format(_T("insert into nodes (title,ordernum,profile_id,flagdisplayratio) values ('%s',%i,%i,%f) "),
            nHash[_T("title")].GetString().c_str(),nHash[_T("ordernum")].GetLong(),profile_id,nHash[_T("flagdisplayratio")].GetDouble());

    m_descDB.ExecuteUpdate(nqs);

    int node_id = m_descDB.GetLastRowId().ToLong();

    for( size_t b = 0; b < cac; b++ ) {

        InsertSymbol( nodeDesc.Symbols[b], node_id );

    }

    return node_id;
}

MultiGraphDesc* MultiGraphDescManager::SaveAsMultiGraphDesc(size_t id, const wxString& title)
{
    chkdescdb(NULL);

    Logger::logf(LOG_MSG,_T("MultiGraphDescManager::SaveAsMultiGraphDesc title=")+title);

    MultiGraphDesc* desc = GetMultiGraphDesc(id);

    if(!desc) return NULL;

    m_descDB.Begin();

    m_descDB.ExecuteUpdate(wxString::Format(_T("insert into profiles (title) values ('%s');"),title.c_str()));

    int profile_id = m_descDB.GetLastRowId().ToLong();

    for( size_t a = 0; a < desc->Cells.GetCount(); a++ ) {

        InsertNode(desc->Cells[a],profile_id);

    }

    m_descDB.Commit();

    MultiGraphDesc * newDesc = LoadMultiGraphDescProfile(profile_id);

    if(newDesc) {
        desc->Title = title;
        m_multiGraphDescHash[profile_id] = newDesc;
    }


    return newDesc;
}

void MultiGraphDescManager::UpdateSymbol(wxVariantHash &hash)
{
    chkdescdb();

    int symbol_id = hash[_T("id")].GetLong();

    GUIDescStructItemArray& dsiarr = GetMGDescStruct();

    wxString qs = _T("update symbols set ");

    for(size_t c = 0; c < dsiarr.GetCount(); c++) {
        wxString pn = dsiarr[c].Name;

        if( hash.find(pn) == hash.end() ) {
            qs += pn + _T(" = NULL, ");
        } else {
            qs += pn + _T(" = ");
            switch(dsiarr[c].Type){
                case GDSI_COLOR:{
                    wxColor color;
                    color << hash[pn];
                    qs += _T("'") + DrawHelper::GetColorAsString( color, wxC2S_HTML_SYNTAX ) + _T("', ");
                    }break;
                case GDSI_NUMBER:
                case GDSI_SYMBOLDESC:
                    qs += hash[pn].MakeString() + _T(", ");
                    break;
                case GDSI_SYMBOLNAME:
                case GDSI_TEXT:
                    qs += _T("'") + hash[pn].GetString() + _T("', ");
            }
        }
    }

    qs.RemoveLast(2);

    qs += wxString::Format(_T(" where id=%i;"),symbol_id);

    m_descDB.ExecuteUpdate(qs);
}

void MultiGraphDescManager::UpdateNode(MultiGraphDescNode &nodeDesc)
{
    chkdescdb();

    size_t cac = nodeDesc.Symbols.GetCount();

    int node_id = nodeDesc.Properties[_T("id")].GetLong();

    for( size_t b = 0; b < cac; b++ ) {
        wxVariantHash &hash = nodeDesc.Symbols[b];

        if( hash[_T("id")].GetLong() < 0 ) {

            InsertSymbol(hash, node_id);

        } else {

            UpdateSymbol(hash);

        }
    }

    wxString nqs = _T("update nodes set ");
    wxVariantHash& nodehash = nodeDesc.Properties;
    for(wxVariantHash::iterator it = nodehash.begin(); it != nodehash.end(); it++) {
        nqs += it->first + _T(" = '") + it->second.MakeString() + _T("', ");
    }
    nqs.RemoveLast(2);
    nqs += wxString::Format(_T(" where id = %i;"),node_id);

    m_descDB.ExecuteUpdate(nqs);
}

void MultiGraphDescManager::SaveMultiGraphDesc(size_t id)
{
    chkdescdb();

    MultiGraphDesc* desc = GetMultiGraphDesc(id);

    if(!desc) return;

    m_descDB.Begin();

    for(size_t a = 0; a < desc->Cells.GetCount(); a++) {

        MultiGraphDescNode &nodeDesc = desc->Cells[a];

        if(nodeDesc.Properties[_T("id")].GetLong() < 0) {

            InsertNode(nodeDesc,desc->ProfileID);

        } else {

            UpdateNode(nodeDesc);

        }

    }

    IntArray& rsarr = desc->GetRemovedSymbols();

    for(size_t a=0; a < rsarr.GetCount(); a++) {
        m_descDB.ExecuteUpdate(wxString::Format(_T("delete from symbols where id=%i;"),rsarr[a]));
    }

    rsarr.Clear();

    IntArray& rnarr = desc->GetRemovedNodes();

    for(size_t a=0; a < rnarr.GetCount(); a++) {
        m_descDB.ExecuteUpdate(wxString::Format(_T("delete from nodes where id=%i;"),rnarr[a]));
        m_descDB.ExecuteUpdate(wxString::Format(_T("delete from symbols where node_id=%i;"),rnarr[a]));
    }

    rnarr.Clear();

    m_descDB.Commit();

    desc->IsChanged = false;
}

void MultiGraphDescManager::SaveMultiGraphTheme(size_t id)
{
    chkthemedb();

    MultiGraphTheme* theme = GetMultiGraphTheme(id);

    if(!theme) return;

    if( theme->IsChanged == false ) return;

    wxVariantHash& hash = theme->Properties;

    m_themeDB.Begin();

    for(wxVariantHash::iterator it = hash.begin(); it != hash.end(); it++ ) {

        wxString val = theme->MakeString( it->first );
        int pid = m_themePropNameToId[ it->first ];

        wxString qs = _T("update tvalues set tvalue = '")+val+_T("' where ");

        m_themeDB.ExecuteUpdate(qs+wxString::Format(_T(" prop_id = %i and profile_id = %i;"),pid,id));

    }

    m_themeDB.Commit();

    theme->IsChanged = false;
}


MultiGraphTheme* MultiGraphDescManager::SaveAsMultiGraphTheme(size_t id, const wxString& title)
{
    chkthemedb(NULL);

    Logger::logf(LOG_MSG,_T("MultiGraphDescManager::SaveAsMultiGraphTheme title=")+title);

    MultiGraphTheme* theme = GetMultiGraphTheme(id);

    if(!theme) return NULL;

    m_themeDB.Begin();

    m_themeDB.ExecuteUpdate(wxString::Format(_T("insert into profiles (title) values ('%s');"),title.c_str()));

    int profile_id = m_themeDB.GetLastRowId().ToLong();

    for(wxVariantHash::iterator it = theme->Properties.begin(); it != theme->Properties.end(); it++  ) {

        wxString val = theme->MakeString( it->first );
        int propid = m_themePropNameToId[ it->first ];

        wxString qs = wxString::Format(_T("insert into tvalues (prop_id,profile_id,tvalue) values (%i,%i,'%s');"),propid,profile_id,val.c_str());

        m_themeDB.ExecuteUpdate(qs);

    }

    m_themeDB.Commit();

    MultiGraphTheme * newTheme = LoadMultiGraphThemeProfile(profile_id);

    if(newTheme) {
        m_multiGraphThemeHash[profile_id] = newTheme;
    }

    return newTheme;
}


bool MultiGraphDescManager::LoadMultiGraphDesc()
{
    chkdescdb(false);

    WX_CLEAR_HASH_MAP( MultiGraphDescHash, m_multiGraphDescHash );

    wxString qs = wxString::Format(_T("select * from profiles;"));

    wxSQLite3ResultSet res = m_descDB.ExecuteQuery(qs);

    Logger::logf(LOG_MSG,_T("MultiGraphDescManager::LoadMultiGraphDesc"));

    while( res.NextRow() ) {
        int id = res.GetInt(_T("id"));

        MultiGraphDesc * desc = LoadMultiGraphDescProfile(id);

        if(desc) {
            desc->Title = res.GetString(_T("title"));
            m_multiGraphDescHash[id] = desc;
        }
    }

    return true;
}


MultiGraphDesc* MultiGraphDescManager::LoadMultiGraphDescProfile(int profilID)
{
    chkdescdb(NULL);
    MultiGraphDesc * multiGraphDesc = new MultiGraphDesc();

    wxString qs = wxString::Format(_T("select * from nodes where profile_id = %i order by ordernum;"), profilID );
    multiGraphDesc->ProfileID = profilID;


    wxSQLite3ResultSet res = m_descDB.ExecuteQuery(qs);

//    Logger::logf(LOG_MSG,_T("MultiGraphDescManager::LoadMultiGraphDescProfile"));

    while( res.NextRow() ) {

        int node_id = res.GetInt(_T("id"));

        MultiGraphDescNode node;
        node.Properties[_T("id")] = (long)node_id;
        node.Properties[_T("title")] = res.GetString(_T("title"));
        node.Properties[_T("ordernum")] = res.GetString(_T("ordernum"));
        node.Properties[_T("profile_id")] = res.GetString(_T("profile_id"));
        node.Properties[_T("flagdisplayratio")] = res.GetDouble(_T("flagdisplayratio"));

        wxString nqs = wxString::Format( _T("select * from symbols where node_id=%i order by ordernum;"), node_id );

        wxSQLite3ResultSet nres = m_descDB.ExecuteQuery( nqs );

        while( nres.NextRow() ) {

            wxVariantHash hash;

            hash[_T("id")] = (long)nres.GetInt(_T("id"));
            hash[_T("node_id")] = (long)nres.GetInt(_T("node_id"));

            hash[_T("name")] = nres.GetString(_T("name"));
            hash[_T("ordernum")] = (long)nres.GetInt(_T("ordernum"));

            wxColor color = DrawHelper::ParseExtColorStr( nres.GetString(_T("color")) );

            hash[_T("color")] << color;
/*
            if( nres.IsNull(_T("overlimitcolor")) ) {
                hash[_T("overcolor")] << DrawHelper::InvertColor( color, CIT_DEFAULT, CIF_NO_ALPHA);
            } else {
                hash[_T("overcolor")] << DrawHelper::ParseExtColorStr( nres.GetString(_T("overlimitcolor")) );
                hash[_T("overlimitcolor")] = hash[_T("overcolor")];
            }
*/
            if( !nres.IsNull(_T("overlimitcolor")) ) hash[_T("overlimitcolor")] << DrawHelper::ParseExtColorStr( nres.GetString(_T("overlimitcolor")) );

            if( !nres.IsNull(_T("limit_max")) ) hash[_T("limit_max")] = nres.GetDouble(_T("limit_max"));

            if( !nres.IsNull(_T("limit_min")) ) hash[_T("limit_min")] = nres.GetDouble(_T("limit_min"));

            if( !nres.IsNull(_T("type")) ) hash[_T("type")] = nres.GetString(_T("type"));

            if( !nres.IsNull(_T("value_max")) ) hash[_T("value_max")] = nres.GetDouble(_T("value_max"));

            if( !nres.IsNull(_T("value_min")) ) hash[_T("value_min")] = nres.GetDouble(_T("value_min"));

            if( !nres.IsNull(_T("customtitle")) ) hash[_T("customtitle")] = nres.GetString(_T("customtitle"));

            if( !nres.IsNull(_T("gaugedesc_id")) ) hash[_T("gaugedesc_id")] = (long)nres.GetInt(_T("gaugedesc_id"));

            if( !nres.IsNull(_T("scale")) ) hash[_T("scale")] = nres.GetDouble(_T("scale"));

            node.Symbols.Add(hash);

        }

        multiGraphDesc->Cells.Add(node);
    }

    return multiGraphDesc;
}

void MultiGraphDescManager::GetMultiGraphThemeProfiles(wxVariantHashArray& profiles)
{
    chkthemedb();
    wxSQLite3ResultSet res = m_themeDB.ExecuteQuery(_T("select * from profiles order by title asc;"));

    while( res.NextRow() ) {
        wxVariantHash hash;
        hash[_T("id")] = (long)res.GetInt(_T("id"));
        hash[_T("title")] = res.GetString(_T("title"));
        profiles.Add(hash);
    }
}

void MultiGraphDescManager::GetMultiGraphDescProfiles(wxVariantHashArray& profiles)
{
    chkdescdb();
    wxSQLite3ResultSet res = m_descDB.ExecuteQuery(_T("select * from profiles order by title asc;"));

    while( res.NextRow() ) {
        wxVariantHash hash;
        hash[_T("id")] = (long)res.GetInt(_T("id"));
        hash[_T("title")] = res.GetString(_T("title"));
        profiles.Add(hash);
    }
}

wxString MultiGraphDescManager::GetMultiGraphDescProfileTitle(size_t id)
{
    chkdescdb(wxEmptyString);

    wxSQLite3ResultSet res = m_descDB.ExecuteQuery(wxString::Format(_T("select title from profiles where id=%i;"),id));

    if( res.Eof() ) return wxEmptyString;

    return res.GetString(0);
}

void MultiGraphDescManager::SetMultiGraphDescProfileTitle(size_t id, const wxString& title)
{
    chkdescdb();
    m_descDB.ExecuteUpdate(wxString::Format(_T("update profiles set title='%s' where id=%i;"),title.c_str(), id));
}


wxString MultiGraphDescManager::GetMultiGraphThemeProfileTitle(size_t id)
{
    chkthemedb(wxEmptyString);
    wxSQLite3ResultSet res = m_themeDB.ExecuteQuery(wxString::Format(_T("select title from profiles where id=%i;"),id));

    if( res.Eof() ) return wxEmptyString;

    return res.GetString(0);
}

void MultiGraphDescManager::SetMultiGraphThemeProfileTitle(size_t id, const wxString& title)
{
    chkthemedb();
    m_themeDB.ExecuteUpdate(wxString::Format(_T("update profiles set title='%s' where id=%i;"),title.c_str(), id));
}


void MultiGraphDescManager::CreateMGDescStruct()
{
    m_MGdescStruct.Clear();
    m_MGdescStruct.Add(GUIDescStructItem(_T("name"), GDSI_SYMBOLNAME,true,false,_T("Symbol name"),_T("Select a symbol name from the menu.\nIf changes the symbol desc, maybe need to reopen the editor.")));
    m_MGdescStruct.Add(GUIDescStructItem(_T("gaugedesc_id"), GDSI_SYMBOLDESC,true,true,_T("Symbol desc"),_T("Select a symbol descriptor name from the menu. (Can be NULL. If NULL, use the first descriptor)\nIf changes the symbol name, maybe need to reopen the editor")));
    m_MGdescStruct.Add(GUIDescStructItem(_T("color"), GDSI_COLOR,true,false,_T("Line color"),_T("The symbol color. Available alpha chanel: rgba(red,green,blue,alpha)\nexample: rgba(255,128,0,200).")));
    m_MGdescStruct.Add(GUIDescStructItem(_T("scale"), GDSI_NUMBER,true,true,_T("Symbol magnify"),_T("In the graph, you can magnify the symbol. If this not null, this value will be use. (Can be NULL.)")));
    m_MGdescStruct.Add(GUIDescStructItem(_T("type"), GDSI_TEXT,false,true,_T("Symbol type"),_T("Flag or other (this case set to NULL this field). (Can be NULL.)")));
    m_MGdescStruct.Add(GUIDescStructItem(_T("limit_min"), GDSI_NUMBER,true,true,_T("Warning limit min"),_T("The bottom limit value. If the value lower then 'limit min',\nthe color line switch to Over limit color. (Can be NULL.)")));
    m_MGdescStruct.Add(GUIDescStructItem(_T("limit_max"), GDSI_NUMBER,true,true,_T("Warning limit max"),_T("The top limit value. If the value greater then 'limit max',\nthe color line switch to Over limit color. (Can be NULL.)")));
    m_MGdescStruct.Add(GUIDescStructItem(_T("overlimitcolor"), GDSI_COLOR,true,true,_T("Over limit color"),_T("When the value greater or lower the (not NULL) 'limit min' or 'limit max' value\nthis color will be use instead of symbol color.(Can be NULL.)\nIf this value is NULL, the program calculate automatically this color from symbol color (inverting).")));
    m_MGdescStruct.Add(GUIDescStructItem(_T("customtitle"), GDSI_TEXT,true,true,_T("Custom symbol title"),_T("The symbol title come from other descriptor database. If you like to overwrite this title set this field to anything. (Can be NULL.)")));
    m_MGdescStruct.Add(GUIDescStructItem(_T("value_min"), GDSI_NUMBER,false,true));
    m_MGdescStruct.Add(GUIDescStructItem(_T("value_max"), GDSI_NUMBER,false,true));

//    m_MGdescStruct.Add(GUIDescStructItem(_T("id"), GDSI_NUMBER,false,false));
    m_MGdescStruct.Add(GUIDescStructItem(_T("node_id"), GDSI_NUMBER,false,false));
    m_MGdescStruct.Add(GUIDescStructItem(_T("ordernum"), GDSI_NUMBER,false,false));

}

void MultiGraphDescManager::LoadMultiGraphThemes()
{
    chkthemedb();

    wxString qs = _T("select * from profiles;");
    wxSQLite3ResultSet res = m_themeDB.ExecuteQuery(qs);

    WX_CLEAR_HASH_MAP( MultiGraphThemeHash, m_multiGraphThemeHash );

    while( res.NextRow() ){

        int prof_id = res.GetInt(_T("id"));

        MultiGraphTheme * theme = LoadMultiGraphThemeProfile(prof_id);

        if(theme == NULL) continue;

        m_multiGraphThemeHash[prof_id] = theme;

    }

    m_mgThemePropDesc.Clear();

    wxSQLite3ResultSet pres = m_themeDB.ExecuteQuery(_T("select * from properties order by title asc;"));

    while( pres.NextRow() ){
       m_mgThemePropDesc.Add( MultiGraphThemeProperty(pres.GetInt(_T("id")),pres.GetString(_T("name")),pres.GetString(_T("title"))) );
       m_themePropNameToId[pres.GetString(_T("name"))] = pres.GetInt(_T("id"));
    }

}


MultiGraphTheme* MultiGraphDescManager::LoadMultiGraphThemeProfile(int profile_id)
{
    chkthemedb(NULL);

    wxString qs = wxString::Format(_T("select properties.name as name, tvalues.tvalue as value from tvalues \
                                      inner join properties on properties.id = tvalues.prop_id where profile_id = %i;"),profile_id);

    wxSQLite3ResultSet res = m_themeDB.ExecuteQuery(qs);

    MultiGraphTheme * theme = NULL;

    theme = new MultiGraphTheme();
    theme->Properties[_T("id")] = (long)profile_id;

    while( res.NextRow() ){

        theme->SetByString(res.GetString(_T("name")),res.GetString(_T("value")));

    }

    return theme;
}

void MultiGraphDescManager::DelMultiGraphDescProfile(size_t id)
{
    chkdescdb();

    if( m_multiGraphDescHash.find(id) == m_multiGraphDescHash.end() ) return;

    m_descDB.ExecuteUpdate(wxString::Format(_T("delete from symbols where node_id in (select id from nodes where profile_id=%i);"),id));
    m_descDB.ExecuteUpdate(wxString::Format(_T("delete from nodes where profile_id=%i;"),id));
    m_descDB.ExecuteUpdate(wxString::Format(_T("delete from profiles where id=%i;"),id));

    MultiGraphDesc * cd = m_multiGraphDescHash[id];
    delete cd;
    m_multiGraphDescHash.erase(id);
}

void MultiGraphDescManager::DelMultiGraphThemeProfile(size_t id)
{
    chkthemedb();

    if( m_multiGraphThemeHash.find(id) == m_multiGraphThemeHash.end() ) return;

    m_themeDB.ExecuteUpdate(wxString::Format(_T("delete from profiles where id=%i;"),id));
    m_themeDB.ExecuteUpdate(wxString::Format(_T("delete from tvalues where profile_id=%i;"),id));

    MultiGraphTheme * ct = m_multiGraphThemeHash[id];
    delete ct;
    m_multiGraphThemeHash.erase(id);
}
