#include "MultiGraphTheme.h"

#include "../DrawHelper.h"

#include <wx/arrimpl.cpp>
WX_DEFINE_OBJARRAY(MultiGraphThemePropertyArray);


MultiGraphTheme::MultiGraphTheme(MultiGraphTheme& m)
{
    Properties = wxVariantHash(m.Properties);
    IsChanged = false;
}

MultiGraphTheme::~MultiGraphTheme()
{
    //dtor
}

wxColor MultiGraphTheme::GetColor(const wxString& key, const wxColor& defval)
{
    if( Properties.find(key) == Properties.end() ) return defval;

    wxColor color;

    color << Properties[key];

    return color;
}

double MultiGraphTheme::GetDouble(const wxString& key, double defval)
{
    if( Properties.find(key) == Properties.end() ) return defval;

    return Properties[key].GetDouble();
}

int MultiGraphTheme::GetInt(const wxString& key, int defval)
{
    if( Properties.find(key) == Properties.end() ) return defval;

    return Properties[key].GetLong();
}

wxString MultiGraphTheme::GetString(const wxString& key, const wxString& defval)
{
    if( Properties.find(key) == Properties.end() ) return defval;

    return Properties[key].GetString();
}

wxString MultiGraphTheme::MakeString(const wxString& key)
{
    if( Properties.find(key) == Properties.end() ) return wxEmptyString;

    if( key.Find(_T("color")) != wxNOT_FOUND ) {
        wxColor color;
        color << Properties[key];
        return DrawHelper::GetColorAsString(color);
    }

    return Properties[key].MakeString();
}

void MultiGraphTheme::SetByString(const wxString& key, const wxString& value)
{
    if(key.Find(_T("color")) != wxNOT_FOUND) {
        wxColor c = DrawHelper::ParseExtColorStr( value );
        Properties[key] << c;
        return;
    }

    double tmp;
    if(value.ToDouble(&tmp)) {
        Properties[key] = tmp;
    } else {
        Properties[key] = value;
    }
}
