#ifndef GPSCOMMUNICATIONCONTROLLER_H
#define GPSCOMMUNICATIONCONTROLLER_H

#include <wx/wx.h>

#include "../common/types/IntArray.h"

#include "../communication/CommunicationClientEventHandler.h"

#include "GPSCommunicationEvent.h"
#include "../Interruptable.h"
#include "../serialconfig.h"

class CommunicationBase;
class CommunicationController2;
class CommunicationControllerClient;
class CommunicationEvent;

class GPSCommunicationController : public wxEvtHandler, public CommunicationClientEventHandler, public Interruptable
{
    public:
        GPSCommunicationController( wxEvtHandler* evtHandler );

        virtual ~GPSCommunicationController();

        // CommunicationClientEventHandler abstract method
        void    handleEvent( CommunicationClientEvent::Type eventType );

        void    detectGPS();
        void    testGPSPort( const wxString& comport, int baudrate );

        void    Abort();

        void    start();
        void    stop();
        bool    isRunning();
        int     getGpsState();

    private:
        CommunicationBase*              m_commbase;
        CommunicationController2*       m_controller;
        CommunicationControllerClient*  m_communicationClient;

        wxArrayString                   m_availablePorts;
        IntArray                        m_availableBaudrates;

        wxEvtHandler*                   m_evtHandler;

        enum State { IDLE, TESTING, DETECTING } m_state;

        signed char                     m_currentPortIdx;
        signed char                     m_currentBrIdx;

        unsigned int                    m_searchCounter;

        SerialConfig                    m_serialConfig;

    private:

        void        reset( const wxString& comport, int baudrate );

        void        OnGPSEVent( GPSCommunicationEvent& event );

        void        searchNext();

        void        OnCommunicationEvent( CommunicationEvent& event );
        void        sendOfflineEvent();
        void        sendOnlineEvent();

        DECLARE_EVENT_TABLE()


};

#endif // GPSCOMMUNICATIONCONTROLLER_H
