#include "AddECUDirFrame.h"
#include <wx/dir.h>
#include <wx/regex.h>
#include <wx/filename.h>
#include <wx/imagpng.h>

#include "../DirHelper.h"
#include "../LogDirManager.h"

#include "../logger.h"
#include "DT.h"

///

/// ScrolledWindow

AddECUDirScrolled::AddECUDirScrolled( wxWindow* parent ) :
    wxScrolledWindow( parent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxScrolledWindowStyle )
{
    SetBackgroundColour( wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE ) );
    ClearBackground();

    wxArrayString tmp;
    DT traverser( tmp );
    wxDir dir( DirHelper::GetPath( dWORKINGDIR ) );
    dir.Traverse( traverser );

    // subdir name format (name)-(number)
    wxRegEx regN(wxT("^([[:alnum:]_.-]+)-([a-zA-z]?[0-9]+)$"));

    for (unsigned int i = 0; i < tmp.GetCount(); ++i)
    {
        // tmp[i] is a fullpath
        wxString subdirName = tmp[i].AfterLast( wxFileName::GetPathSeparator() );

        if ( regN.Matches(subdirName) )
        {
            m_dirs.Add( subdirName );
        }
    }

    // sort them and create gui
    m_dirs.Sort();
    buildGUI();

    // add scrollbars
    SetScrollRate( 5, 10 );
}

void AddECUDirScrolled::buildGUI()
{
    wxBoxSizer* sizer = new wxBoxSizer( wxVERTICAL );

    wxString descr = _T("The following directories are found in the working directory. Select/unselect ECUs to\nadd/remove to/from the database\n");
    descr += _T("The default state of the checkbox indicates wheter a directory already exists\n in the database. If you uncheck it, it will be removed from the database.");

    wxStaticText* txt = new wxStaticText( this, wxID_ANY, descr );

    sizer->Add( txt, 0, wxEXPAND | wxALL, 5 );


    wxStaticBoxSizer * sbs = new wxStaticBoxSizer( wxVERTICAL, this, _T("") );
    wxFlexGridSizer* gs = new wxFlexGridSizer( 2 );

    // header
    wxStaticText* h1 = new wxStaticText( this, wxID_ANY, _T("Db") );
    h1->SetToolTip( _T("Indicates that this directory should be included in the database") );
    wxStaticText* h2 = new wxStaticText( this, wxID_ANY, _T("ECU Name / Serial # / Other information") );
    h2->SetToolTip( _T("List of directories. Check warning icon if any is shown") );
    gs->Add( h1, 0, wxALL|wxEXPAND, 5 ); gs->Add( h2, 0, wxALL|wxEXPAND, 5 );

    m_checkboxes = new wxCheckBox*[m_dirs.GetCount()];

    for ( size_t i = 0; i < m_dirs.GetCount(); ++i )
    {
        int id = wxNewId();

        m_checkboxes[i] = new wxCheckBox( this, id, wxEmptyString );

        Connect( id, wxEVT_COMMAND_CHECKBOX_CLICKED, wxCommandEventHandler( AddECUDirScrolled::OnCheckBox ) );
        gs->Add( m_checkboxes[i], 0, wxALL | wxALIGN_CENTER, 5 );
        m_chboxId[id] = i;

        wxString name = m_dirs[i].BeforeLast('-'), serialnum = m_dirs[i].AfterLast('-');

        // check wheter exists such s/n in db
        wxString dbecuname = LogDirManager::getNameBySerialNumber( serialnum );

        wxString statict = m_dirs[i];

        wxBoxSizer* s = new wxBoxSizer( wxHORIZONTAL );

        wxStaticText* txt3 = new wxStaticText( this, -1, statict );
        s->Add( txt3, 0, wxLEFT, 5 );

        if ( dbecuname != wxEmptyString )
        {
            // it already exists in db
            if ( dbecuname == name )
            {
                m_checkboxes[i]->SetValue( true );
            }
            else
            {
                wxString iconfilename = DirHelper::GetPath(dICON) + _T("ul-warning-m.png");
                wxStaticBitmap* icon = new wxStaticBitmap( this, wxID_ANY, *( new wxBitmap( iconfilename, wxBITMAP_TYPE_PNG ) ) );
                icon->SetToolTip( _T("WARNING: An ECU with the same serial number already exists.\nSelecting this causes removing the other record from the database.") );
                s->Add( icon, 0, wxLEFT, 5 );
            }
        }
        gs->Add( s, 0, wxALL, 5 );
    }

    sbs->Add( gs, 1, wxALL | wxEXPAND, 5 );

    sizer->Add( sbs, 0, wxLEFT | wxRIGHT| wxBOTTOM | wxEXPAND, 20 );

    SetSizer( sizer );
    sizer->Fit( this );
    sizer->SetSizeHints( this );
}

void AddECUDirScrolled::OnCheckBox( wxCommandEvent& event )
{
    int id = event.GetId();
    int dirnr = m_chboxId[id];

    if ( m_checkboxes[dirnr]->GetValue() == false )
    {
        event.Skip();
        return;
    }

    wxString ecunr = m_dirs[dirnr].AfterLast('-');

    // check for serial # collisions
    for ( int i = 0; i < (int)m_dirs.GetCount(); ++i )
    {
        if ( i == dirnr ) continue;

        // unselect ECU with the same serial number
        if ( m_dirs[i].EndsWith(ecunr) )
        {
            m_checkboxes[i]->SetValue( false );
        }
    }

    event.Skip();
}

void AddECUDirScrolled::updateDb()
{
    unsigned int added = 0, deleted = 0;

    for ( size_t i = 0; i < m_dirs.GetCount(); ++i )
    {
        wxString dirname = m_dirs[i];
        wxString ecuName = dirname.BeforeLast('-');
        wxString ecuserialnr = dirname.AfterLast('-');

        // delete it if exists
        if ( m_checkboxes[i]->GetValue() == false)
        {
            if ( LogDirManager::getNameBySerialNumber(ecuserialnr) == wxEmptyString )
            {
                continue; //
            }
            else
            {
                wxString dbecuname = LogDirManager::getNameBySerialNumber( ecuserialnr );
                if ( dbecuname == ecuName )
                    if ( LogDirManager::deleteSerialNumber( ecuserialnr ) )
                        deleted++;
            }
        }
        // insert
        else
        {
            wxString dbecuname = LogDirManager::getNameBySerialNumber( ecuserialnr );
            // it doesnot exist yet
            if ( dbecuname == wxEmptyString )
            {
                if ( LogDirManager::insertSerialNumber( ecuserialnr, ecuName, 0 ) )
                    added++;
            }
            else
            {
                if ( dbecuname == ecuName) continue; // already in db with this name
                else{
                    if ( LogDirManager::deleteSerialNumber(ecuserialnr) )
                        deleted++;
                    if ( LogDirManager::insertSerialNumber(ecuserialnr, ecuName, 0) )
                        added++;
                }
            }
        }
    }

    wxMessageBox(wxString::Format(_T("%d directory added, %d directory removed\nDo not forget to select the proper ECU from the database!"), added, deleted), _T("Database updated"),
        wxOK | wxICON_INFORMATION);

}

///

/// the dialog

BEGIN_EVENT_TABLE( AddECUDirDialog, wxDialog )
    EVT_BUTTON( wxID_CANCEL, AddECUDirDialog::OnCancel )
    EVT_BUTTON( wxID_OK, AddECUDirDialog::OnOK )
END_EVENT_TABLE()

AddECUDirDialog::AddECUDirDialog( wxWindow* parent )
{
    Create( parent, wxID_ANY, _T("Add directories to ECU database manually"), wxDefaultPosition, wxSize(480,320),
        wxDEFAULT_DIALOG_STYLE );

    m_scrolled = new AddECUDirScrolled( this );
    wxBoxSizer * sizer = new wxBoxSizer( wxVERTICAL );
    sizer->Add( m_scrolled, 1, wxALL|wxEXPAND,5 );

    wxSizer* buttons = CreateButtonSizer(wxOK|wxCANCEL);

	// change label of the OK button
    wxButton * okbutton = (wxButton*)( (buttons->GetItem((size_t)1))->GetWindow());
    if ( okbutton != NULL )
    {
        okbutton->SetLabel(_T("Apply and save"));
    }

    sizer->Add( buttons, 0, wxALL|wxALIGN_CENTER, 5 );
    SetSizer( sizer );
    sizer->Fit( this );
    sizer->SetSizeHints( this );

    // set default dialog size
    SetSize( 480,320 );
}

AddECUDirDialog::~AddECUDirDialog()
{
    //dtor
}

void AddECUDirDialog::OnCancel( wxCommandEvent& event )
{
    delete m_scrolled;
    event.Skip();
}

void AddECUDirDialog::OnOK( wxCommandEvent& event )
{
    m_scrolled->updateDb();

    delete m_scrolled;
    event.Skip();
}


///

/// Select ECU from DB list

BEGIN_EVENT_TABLE( SelectECUFromDbDialog, wxDialog )

    EVT_BUTTON( wxID_OK, SelectECUFromDbDialog::OnOK )

END_EVENT_TABLE()

SelectECUFromDbDialog::SelectECUFromDbDialog( wxWindow * parent )
{
    m_selectedECUName = _T("");

    Create( parent, wxID_ANY, _T("Select ECU from database"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE );

    wxBoxSizer* sizer = new wxBoxSizer( wxVERTICAL );

    m_eculist = LogDirManager::GetECUList();

    wxStaticBoxSizer* bsizer = new wxStaticBoxSizer( wxVERTICAL, this, _T("Select ECU from the dropdown list") );

    m_combobox = new wxComboBox( this, wxID_ANY, LogDirManager::selectedECU(),
        wxDefaultPosition, wxDefaultSize, m_eculist, wxCB_READONLY | wxCB_SORT );

    bsizer->Add( m_combobox, 0, wxEXPAND | wxALL, 5 );

    wxSizer* buttons = CreateButtonSizer( wxOK | wxCANCEL );
    bsizer->Add( buttons, 0, wxEXPAND | wxALL, 5 );

    sizer->Add( bsizer, 0, wxALL | wxEXPAND, 15 );
    SetSizer( sizer );
    sizer->Fit( this );
    sizer->SetSizeHints( this );
    m_selectedECUName = m_combobox->GetValue();
}

SelectECUFromDbDialog::~SelectECUFromDbDialog()
{
}

void SelectECUFromDbDialog::OnOK( wxCommandEvent& event )
{
    m_selectedECUName = m_combobox->GetValue();
    event.Skip();
}
