#include "CfgUploadDlg.h"

#include <wx/filename.h>

#include "../DirHelper.h"
#include "../logger.h"

BEGIN_EVENT_TABLE( RoundCfgUploadHelperDlg, wxDialog )
    EVT_BUTTON( idEarlierCfg, RoundCfgUploadHelperDlg::OnButton )
    EVT_BUTTON( idDefFwCfg, RoundCfgUploadHelperDlg::OnButton )
    EVT_BUTTON( idCancelCfg, RoundCfgUploadHelperDlg::OnButton )
END_EVENT_TABLE()

RoundCfgUploadHelperDlg::RoundCfgUploadHelperDlg( wxWindow* parent )
    : wxDialog( parent, wxID_ANY, _T("Warning"), wxDefaultPosition, wxDefaultSize )
{
    wxPanel* msgPanel = new wxPanel( this );
    msgPanel->SetBackgroundColour( wxColor( 255, 255, 255 ) );
    msgPanel->ClearBackground();

    /// msg
    wxBitmap bmp( DirHelper::GetPath(dICON) + _T("ul-warning-xl.png"), wxBITMAP_TYPE_PNG );

    wxStaticBitmap* icon = new wxStaticBitmap( msgPanel, wxID_ANY, bmp );

    wxString msg = _T("Config has not been saved before firmware upload,\nbecause files are missing from the appropriate directory.\n");

    msg += _T("You can download them via the Firmware Webtool\n\nYou can now continue to upload an earlier config (NOT RECOMMENDED!) or ");
    msg += _T("you can upload the default firmware config. The default firmware configs can be found in the 'defaultfirmwareconfigs'");
    msg += _T(" directory of the VemsTune installation.\n\n");
    msg += _T("Please choose!");

    wxStaticText* text = new wxStaticText( msgPanel, wxID_ANY, msg );
    text->Wrap( 300 );

    wxSizer* msgSizer = new wxFlexGridSizer( 2 );
    msgSizer->Add( icon, 0, wxALL, 15 );
    msgSizer->Add( text, 0, wxALL, 5 );

    msgPanel->SetSizer( msgSizer );
    msgSizer->Fit( msgPanel );
    msgSizer->SetSizeHints( msgPanel );

    /// buttons

    wxSizer* buttonSizer = new wxBoxSizer( wxHORIZONTAL );

    wxButton* button;
    button = new wxButton( this, idEarlierCfg, _T("Upload an\nearlier config"), wxDefaultPosition, wxSize( 120, 40 ) );
    buttonSizer->Add( button, 0, wxALL, 3 );
    button = new wxButton( this, idDefFwCfg, _T("Upload a default\nfirmware config"), wxDefaultPosition, wxSize( 120, 40 ) );
    buttonSizer->Add( button, 0, wxALL, 3 );
    button = new wxButton( this, idCancelCfg, _T("Skip config upload"), wxDefaultPosition, wxSize( 120, 40 ) );
    buttonSizer->Add( button, 0, wxALL, 3 );

    /// sizers
    wxSizer* mainSizer = new wxBoxSizer( wxVERTICAL );
    mainSizer->Add( msgPanel, 0, wxEXPAND | wxALL, 2 );
    mainSizer->Add( buttonSizer, 0, wxEXPAND | wxALIGN_RIGHT | wxALL, 3 );

    SetSizer( mainSizer );
    mainSizer->Fit( this );
    mainSizer->SetSizeHints( this );
}

void RoundCfgUploadHelperDlg::OnButton( wxCommandEvent& event )
{
    int id = event.GetId();

    switch ( id )
    {
        case idEarlierCfg: EndModal( 1 ); break;
        case idDefFwCfg: EndModal( 2 ); break;
        case idCancelCfg: EndModal( 3 ); break;

        default:
            break;
    }
}

int idCfgUplBrowseButton = wxNewId();

BEGIN_EVENT_TABLE( CfgUploadDlg, wxDialog )
    EVT_BUTTON( idCfgUplBrowseButton, CfgUploadDlg::onBrowseButtons )
END_EVENT_TABLE()

CfgUploadDlg::CfgUploadDlg( wxWindow * parent, wxString& filename ) : wxDialog( parent, -1, _T("Select config file to upload"), wxDefaultPosition )
{
    wxBoxSizer* sizer = new wxBoxSizer( wxVERTICAL );

    wxStaticText* txt = new wxStaticText( this, -1, _T("Select a config file to upload. Press now OK to upload the latest config file\nto the ECU or you can browse for another file and then press OK\nYou can skip this step by pressing Cancel") );

    sizer->Add(txt, 0, wxEXPAND | wxALL, 5 );

    wxBoxSizer* filebsizer = new wxBoxSizer( wxHORIZONTAL );
    m_fileNameCtrl = new wxTextCtrl( this, -1, filename, wxDefaultPosition, wxSize( 380, -1 ), wxTE_READONLY );

    wxButton* browseButton = new wxButton( this, idCfgUplBrowseButton, _T("Browse") );

    filebsizer->Add( m_fileNameCtrl, 0, wxEXPAND | wxALL, 3 );
    filebsizer->Add( browseButton, 0, wxALL, 3 );

    sizer->Add( filebsizer, 0, wxEXPAND | wxALL, 5 );

    sizer->Add( CreateButtonSizer( wxCANCEL | wxOK ), 1, wxEXPAND | wxALL, 5 );

    SetSizer( sizer );
    sizer->SetSizeHints( this );

    m_fileNameCtrl->SetInsertionPoint( 1 );
}

void CfgUploadDlg::onBrowseButtons( wxCommandEvent& event )
{
    wxString dir = wxEmptyString;
    wxString fname = m_fileNameCtrl->GetValue();

    if ( fname != wxEmptyString )
    {
        wxFileName fn( fname );
        dir = fn.GetPath();
    }
    if ( dir == wxEmptyString )
    {
        dir = DirHelper::GetPath( dLOG );
    }

    wxFileDialog dlg( this, _T("Choose vemscfg file"), dir, fname, _T("VEMS Config files (*.vemscfg)|*.vemscfg"), wxFD_OPEN | wxFD_FILE_MUST_EXIST );

    if ( dlg.ShowModal() == wxID_OK )
    {
        fname = dlg.GetPath();

        m_fileNameCtrl->SetValue( fname );
    }

    m_fileNameCtrl->ShowPosition( 0 );
}
