#include "ConfigSwitchUploadDialog.h"
#include <wx/statline.h>

#include "../logger.h"
#include "../DirHelper.h"
#include "../LogDirManager.h"
#include "../firmwareconfig/FWConfigUploadDialog.h"


BEGIN_EVENT_TABLE( ConfigSwitchUploadDialog, wxDialog )
    EVT_BUTTON( ID_OPEN_CONFIG_A, ConfigSwitchUploadDialog::OnOpenConfigA )
    EVT_BUTTON( ID_OPEN_CONFIG_B, ConfigSwitchUploadDialog::OnOpenConfigB )
    EVT_RADIOBUTTON( ID_RADIO_SAME, ConfigSwitchUploadDialog::OnSelectConfigB )
    EVT_RADIOBUTTON( ID_RADIO_B, ConfigSwitchUploadDialog::OnSelectConfigB )
    EVT_BUTTON( ID_UPLOAD, ConfigSwitchUploadDialog::OnUpload )
END_EVENT_TABLE()


ConfigSwitchUploadDialog::ConfigSwitchUploadDialog(   wxWindow* parent,
                                        wxWindowID id,
                                        const wxString& title,
                                        const wxPoint& pos,
                                        const wxSize& size,
                                        long style,
                                        const wxString& name )
{
    m_uploadDialog = NULL;

    Create( parent, id, _T("ConfigSwitch Upload"), pos, size, style, name );

    ///
    wxSize textSize = wxDefaultSize;
    textSize.SetWidth( 400 );

    ///ConfigA
    wxTextCtrl* configAText = new wxTextCtrl( this, ID_CONFIG_A );
    configAText->SetInitialSize( textSize );
    wxButton* openConfigAButton   = new wxButton( this, ID_OPEN_CONFIG_A, _T("Open") );
    wxStaticBoxSizer* configASizer = new wxStaticBoxSizer( wxHORIZONTAL, this, _T("ConfigA") );

    configASizer->AddSpacer( 13 );
    configASizer->Add( configAText, 0 , wxEXPAND | wxLEFT | wxRIGHT, 5 );
    configASizer->Add( openConfigAButton );

    ///ConfigB
    wxRadioButton*  sameAsRadioButton = new wxRadioButton( this, ID_RADIO_SAME, _T("Same as Config A") );

    wxRadioButton*  selectConfigBRadioButton = new wxRadioButton( this, ID_RADIO_B, _T("") );
    wxTextCtrl* configBText = new wxTextCtrl( this, ID_CONFIG_B );
    configBText->SetInitialSize( textSize );
    wxButton* openConfigBButton = new wxButton( this, ID_OPEN_CONFIG_B, _T("Open") );

    wxBoxSizer* openConfigBSizer = new wxBoxSizer( wxHORIZONTAL );
    openConfigBSizer->Add( selectConfigBRadioButton, 0 , wxRIGHT, 5 );
    openConfigBSizer->Add( configBText, 0 , wxRIGHT | wxEXPAND, 5 );
    openConfigBSizer->Add( openConfigBButton );

    wxStaticBoxSizer* configBSizer = new wxStaticBoxSizer( wxVERTICAL, this, _T("ConfigB") );
    configBSizer->Add( sameAsRadioButton, 0, wxBOTTOM, 5 );
    configBSizer->Add( openConfigBSizer );

    ///Upload
    wxButton* uploadButton = new wxButton( this, ID_UPLOAD, _T("Upload") );

    ///Main
    wxBoxSizer* mainSizer = new wxBoxSizer( wxVERTICAL );

    mainSizer->Add( configASizer, 0, wxALL | wxEXPAND, 5 );
    mainSizer->Add( configBSizer, 0, wxALL | wxEXPAND, 5 );
    mainSizer->Add( new wxStaticLine( this ), 0, wxEXPAND | wxALL, 1 );
    mainSizer->Add( uploadButton, 0, wxALIGN_RIGHT | wxALL, 5 );

    SetSizer( mainSizer );
    SetInitialSize();

    ///Init
    sameAsRadioButton->SetValue(true);
    configBText->Disable();
    openConfigBButton->Disable();
}

void ConfigSwitchUploadDialog::OnOpenConfigA( wxCommandEvent& event )
{
    wxFileDialog fdialog( this );

    if ( LogDirManager::checkForUsedECU() > 0 ) {
        fdialog.SetPath( DirHelper::GetPath(dLOG) );
    } else {
        fdialog.SetPath( DirHelper::GetPath(dDEFAULTCONFIGS));
    }

    fdialog.SetWildcard( _T("VemsCfg files (*.vemscfg)|*.vemscfg;|Any files (*.*)|*.*") );

    if( fdialog.ShowModal() != wxID_OK ) { return; }

    wxString cfgFile = fdialog.GetDirectory() + DirHelper::Separ() + fdialog.GetFilename();

    wxTextCtrl* configAText = (wxTextCtrl*) wxWindow::FindWindowById( ID_CONFIG_A );

    configAText->SetValue( cfgFile );
}

void ConfigSwitchUploadDialog::OnOpenConfigB( wxCommandEvent& event )
{
    wxFileDialog fdialog( this );

    if ( LogDirManager::checkForUsedECU() > 0 ) {
        fdialog.SetPath( DirHelper::GetPath(dLOG) );
    } else {
        fdialog.SetPath( DirHelper::GetPath(dDEFAULTCONFIGS));
    }

    fdialog.SetWildcard( _T("VemsCfg files (*.vemscfg)|*.vemscfg;|Any files (*.*)|*.*") );

    if( fdialog.ShowModal() != wxID_OK ) { return; }

    wxString cfgFile = fdialog.GetDirectory() + DirHelper::Separ() + fdialog.GetFilename();

    wxTextCtrl* configBText = (wxTextCtrl*) wxWindow::FindWindowById( ID_CONFIG_B );

    configBText->SetValue( cfgFile );
}

void ConfigSwitchUploadDialog::OnSelectConfigB( wxCommandEvent& event )
{
    wxTextCtrl* configBText = (wxTextCtrl*) wxWindow::FindWindowById( ID_CONFIG_B );
    wxButton* openConfigBButton = (wxButton*) wxWindow::FindWindowById( ID_OPEN_CONFIG_B );

    int eventId = event.GetId();

    switch ( eventId )
    {
        case ID_RADIO_SAME:
            configBText->Disable();
            openConfigBButton->Disable();
            break;
        case ID_RADIO_B:
            configBText->Enable();
            openConfigBButton->Enable();
            break;
    }
}

void ConfigSwitchUploadDialog::OnUpload( wxCommandEvent& event )
{
    wxTextCtrl* configAText = (wxTextCtrl*) wxWindow::FindWindowById( ID_CONFIG_A );
    wxString configA = configAText->GetValue();

    if ( m_uploadDialog && !configA.IsEmpty() ) {
        m_uploadDialog->uploadConfigFile( configA );
    }

    EndModal( event.GetId() );
}
