#include "ErrorReportDialog.h"

#include <wx/filedlg.h>

#include "../bcWidgets/bcParentPanel.h"
#include "../bcWidgets/bcStaticBitmap.h"
#include "../bcWidgets/bcWrapper.h"
#include "../bcWidgets/bcProgressBar.h"
#include "../bcWidgets/bcButton.h"
#include "../bcWidgets/bcLabel.h"
#include "../bcWidgets/bcCheckBox.h"
#include "../bcWidgets/bcRadioButton.h"
#include "../bcWidgets/bcStaticLine.h"

#include "../bcWidgets/bcLinkLabel.h"

#include "../DrawHelper.h"
#include "../SystemInfoPacker.h"
#include "../logger.h"

#include "../FileHelper.h"

#include "../common/IconSetsHelper.h"

BEGIN_EVENT_TABLE(ErrorReportDialog,wxDialog)

    EVT_BUTTON(idButtonCreateReportPack, ErrorReportDialog::OnButton)

    EVT_BUTTON(idButtonOFControlBrowse, ErrorReportDialog::OnButton)
    EVT_BUTTON(idButtonOFControlRemove, ErrorReportDialog::OnButton)
    EVT_BUTTON(idButtonOFControlClear,ErrorReportDialog::OnButton)

END_EVENT_TABLE()

ErrorReportDialog::ErrorReportDialog(wxWindow* parent)
{
    //ctor
    Create(parent, wxID_ANY,_T("Error reporting"));

    BuildGUI();
}

ErrorReportDialog::~ErrorReportDialog()
{
    //dtor
}

void ErrorReportDialog::BuildGUI()
{
    wxBoxSizer* mainsizer = new wxBoxSizer(wxVERTICAL);
    bcStaticLine* statline;

// create dialog head
    bcParentPanel* headPanel = new bcParentPanel(this);
    headPanel->SetBackgroundColour(wxColor(99,0,0));

    bcStaticBitmap* headImage = new bcStaticBitmap(headPanel,wxID_ANY, DrawHelper::GetImage(_T("vems-errorrep.png")));
    wxBoxSizer* headSizer = new wxBoxSizer(wxVERTICAL);
    headSizer->Add( new bcWrapper(headImage), 1, wxALL | wxEXPAND, 10 );
    headPanel->SetSizer(headSizer);

    mainsizer->Add(headPanel,0,wxEXPAND,0);

// create dialog body
    bcParentPanel* bodyPanel = new bcParentPanel(this);
    wxBoxSizer* bodySizer = new wxBoxSizer(wxVERTICAL);

    m_progressBar = new bcProgressBar(bodyPanel);
    bcButton* cbutton = new bcButton(bodyPanel, idButtonCreateReportPack, _T("Create report"));
    m_fbLabel = new bcLabel(bodyPanel, wxID_ANY, _T(" "));
    bcCheckBox* cbcfgfile = new bcCheckBox(bodyPanel,wxID_ANY,_T("Vems log/config file"));
    cbcfgfile->SetValue(true);
    statline = new bcStaticLine(bodyPanel);
    statline->SetLabel(_T("Select files"));
    statline->SetLabelPos(15);

    bodySizer->Add( new bcWrapper(statline), 0, wxEXPAND | wxLEFT | wxRIGHT, 5 );
    bodySizer->Add( new bcWrapper(cbcfgfile), 0, wxALL | wxEXPAND, 5 );

    bcLinkLabel* otherfileslink = new bcLinkLabel(bodyPanel, bcLinkInfo(_T("Other files...")),idOtherToggle);
    bodySizer->Add( new bcWrapper(otherfileslink), 0, wxLEFT|wxRIGHT, 5 );

// other files

    m_otherFilesSizer = new wxBoxSizer(wxHORIZONTAL);
    m_otherFilesList = new wxListBox(bodyPanel, wxID_ANY, wxDefaultPosition, wxDefaultSize, 0, 0, wxLB_SORT|wxLB_EXTENDED);

    wxBoxSizer* otherfilecontrolsizer = new wxBoxSizer(wxVERTICAL);
    bcButton* ofcBrowse = new bcButton(bodyPanel,idButtonOFControlBrowse,IconSetsHelper::GetImage(IS_FOLDER));
    bcButton* ofcRemove = new bcButton(bodyPanel,idButtonOFControlRemove,IconSetsHelper::GetImage(IS_X));
    bcButton* ofcClear = new bcButton(bodyPanel,idButtonOFControlClear,IconSetsHelper::GetImage(IS_RECYCLE_EMPTY));

    ofcBrowse->SetImgLabel( IconSetsHelper::GetImage(IS_WHITE_RAISED,IS_FOLDER,IS_16), bcBUTTON_IMAGE_PRESSED );
    ofcBrowse->SetStyleFlag(bcBUTTON_HOVER_OUTER_GLOW);
    ofcRemove->SetImgLabel( IconSetsHelper::GetImage(IS_WHITE_RAISED,IS_X,IS_16), bcBUTTON_IMAGE_PRESSED );
    ofcRemove->SetStyleFlag(bcBUTTON_HOVER_OUTER_GLOW);
    ofcClear->SetImgLabel( IconSetsHelper::GetImage(IS_WHITE_RAISED,IS_RECYCLE_EMPTY,IS_16), bcBUTTON_IMAGE_PRESSED );
    ofcClear->SetStyleFlag(bcBUTTON_HOVER_OUTER_GLOW);

    otherfilecontrolsizer->Add(new bcWrapper(ofcBrowse),0, wxALL, 0 );
    otherfilecontrolsizer->Add(new bcWrapper(ofcRemove),0, wxALL, 0 );
    otherfilecontrolsizer->Add(new bcWrapper(ofcClear),0, wxALL, 0 );

    m_otherFilesSizer->Add(m_otherFilesList, 1, wxLEFT | wxBOTTOM | wxEXPAND, 5);
    m_otherFilesSizer->Add(otherfilecontrolsizer);

    bodySizer->Add(m_otherFilesSizer, 0, wxEXPAND);

    m_otherFilesSizer->Show(false);

    m_message = new wxTextCtrl(bodyPanel,wxID_ANY,_T(""), wxDefaultPosition, wxSize(-1,100), wxTE_MULTILINE | wxTE_PROCESS_TAB);

    statline = new bcStaticLine(bodyPanel);
    statline->SetLabel(_T("Message"));
    statline->SetLabelPos(15);
    bodySizer->Add( new bcWrapper(statline), 0, wxEXPAND | wxLEFT | wxRIGHT, 5 );

    bodySizer->Add( m_message, 1, wxEXPAND | wxALL, 5 );


    statline = new bcStaticLine(bodyPanel);
    statline->SetLabel(_T("Prepare package"));
    statline->SetLabelPos(15);
    bodySizer->Add( new bcWrapper(statline), 0, wxEXPAND | wxLEFT | wxRIGHT, 5 );

    bodySizer->Add(new bcWrapper(cbutton), 0, wxALL, 5);

    bodySizer->Add(new bcWrapper(m_fbLabel), 0, wxALL, 5);
    bodySizer->Add(new bcWrapper(m_progressBar), 0, wxALL|wxEXPAND, 5);

    bodyPanel->SetSizer(bodySizer);

    mainsizer->Add(bodyPanel,0,wxEXPAND|wxALL,5);

    SetSizer(mainsizer);
    mainsizer->SetSizeHints(this);

    otherfileslink->Connect(wxEVT_BCLINK, bcLinkEventHandler(ErrorReportDialog::OnOtherToggle), NULL, this );
}

void ErrorReportDialog::OnOtherToggle(bcLinkEvent& evt)
{
    m_otherFilesSizer->Show( !m_otherFilesSizer->IsShown(m_otherFilesList) );
    m_otherFilesSizer->Layout();
    GetSizer()->Layout();
    GetSizer()->SetSizeHints(this);
}

void ErrorReportDialog::OnButton(wxCommandEvent& evt)
{
    switch(evt.GetId()){

        case idButtonCreateReportPack: {
                m_fbLabel->SetLabel(_T("Getting system informations..."));
                m_progressBar->SetStyle(BCPBS_INFINITY);
                m_progressBar->StartAnim();
                wxString pack = SystemInfoPacker::GetPack();
                m_fbLabel->SetLabel(_T(" "));
                m_progressBar->StopAnim();
                m_progressBar->SetStyle(BCPBS_DEFAULT);
            }break;

        case idButtonOFControlBrowse: {
            wxFileDialog dlg(this, _T("Choose a file"), _T(""), _T(""), _T("*.*"), wxFD_MULTIPLE | wxFD_OPEN);
            if(dlg.ShowModal() == wxID_OK) {
                wxArrayString files;
                dlg.GetFilenames(files);
                m_otherFilesList->Append(files);
            }
            }break;


        case idButtonOFControlRemove: {
            wxArrayInt sel;
            m_otherFilesList->GetSelections(sel);
            for(int a=sel.GetCount()-1;a >= 0;a--){
                m_otherFilesList->Delete( sel[a] );
            }
            }break;

        case idButtonOFControlClear:
            m_otherFilesList->Clear();
            break;
    }
}

void ErrorReportDialog::SetInfo(const wxString& key, const wxVariant& data)
{
    m_info[key] = data;
}

