#ifndef FILEOPENSQL_H
#define FILEOPENSQL_H

#include <wx/wx.h>
#include <wx/wxsqlite3.h>
#include "../common/typedef.h"
#include "../Observable.h"

class SearchDescr {
public:
    SearchDescr() { minval = -1; maxval = -1; }
    SearchDescr( long minv, long maxv = -1 ) : minval(minv), maxval(maxv) {}
    long minval;
    long maxval;
};

WX_DECLARE_STRING_HASH_MAP( SearchDescr*, SDHash );

class FileInfo {
public:
    wxString        fileName;
    unsigned long   fileSize;
    unsigned long   fileDate;
    wxString        fileType;
    uint8_t         fw_major;
    uint8_t         fw_minor;
    uint8_t         fw_build;

    int             rpmMax;
    int             mapMax;
    int             triggerErr;
    int             hasGPSData;

    bool    isLogFile() { return ( fileType == _T("VEMSLOG file") ); }
};

enum  {
    VF_VEMS_LOG = 1,
    VF_VEMS_CFG = 2,
    VF_VEMS_TLOG = 4,
    VF_VEMS_ALL = VF_VEMS_LOG | VF_VEMS_CFG | VF_VEMS_TLOG
};

enum FO_order { FO_OB_FNAME, FO_OB_SIZE, FO_OB_DATE, FO_OB_TYPE, FO_OB_FIRMWARE,
                FO_OB_RPM, FO_OB_MAP, FO_OB_TRIGERR, FO_OB_GPSDATA };

class FD_ColumnInfo {
    public:
        int         id;
        wxString    name;
        bool        visible;
        int         order;
        float       column_width;
};

WX_DEFINE_ARRAY( FileInfo*, FileInfoA );
WX_DEFINE_ARRAY( FD_ColumnInfo*, ColumnInfoA );

class FileDialogSettings {
    public:
        static ColumnInfoA getColumns( bool onlyVisible = true );
        static void setColumn( const wxString& columnname, bool visible = true, int order = -1 );
        static wxSize getDlgSize();
        static void setDlgSize( const wxSize& size );

        static void getSorting( uint8_t& column, bool& ascending );
        static void setSorting( uint8_t column, bool ascending );
    private:
        static wxSQLite3Database    m_db;
        static void    openDb();
        static void    checkDb();
};

class FileOpenSQL : public Observable
{
    public:
        FileOpenSQL();
        FileOpenSQL( const wxString& path );

        void                setPath( const wxString& path );
        FileInfoA           getFiles( int files = VF_VEMS_ALL, FO_order orderby = FO_OB_FNAME, bool ascending = true );
        static FileInfoA    searchFiles( const SDHash& search_criteria );

    protected:
    private:
        wxString             m_path;
        wxSQLite3Database    m_db;

        void    openDb();
        void    checkDb();
        bool    enumFiles();

        bool    fileExists( const wxString& file );
        void    removeFile( const wxString& filename );
};

class FileAnalyzeSQL {

    public:
        static bool addData( const wxString& filename, FileInfo info );
        static bool isAnalyzed( const wxString& filename );
        static void updateGPSdata( const wxString& filename, bool hasGPS );

    private:
        static wxString             m_path;
        static wxSQLite3Database    m_db;

        static void    openDb();
        static void    checkDb();
        static bool    fileExists( const wxString& file );
};


#endif // FILEOPENSQL_H
