#include "FirmwareUploadDialog.h"
#include "../logger.h"
#include "../vemsprotocol/VemsDeviceControllerBase.h"
#include "../vemsprotocol/FirmwareData.h"
#include "../events/VemsDeviceControllerEvent.h"
#include "../events/FirmwareUploadEvent.h"


IMPLEMENT_CLASS( FirmwareUploadDialog, wxProgressDialog )


FirmwareUploadDialog::FirmwareUploadDialog(   wxWindow* parent,
                                            VemsDeviceControllerBase* deviceController,
                                            FirmwareData* firmwareData )
    : wxProgressDialog( _T(""),
                        _T("Page index: "), ///TODO
                        firmwareData->getPages(),
                        parent,
                        //wxPD_CAN_ABORT |
                        wxPD_APP_MODAL |
                        wxPD_AUTO_HIDE |
                        wxPD_ELAPSED_TIME |
                        wxPD_SMOOTH |
                        wxPD_ESTIMATED_TIME |
                        wxPD_REMAINING_TIME )
{
    m_deviceController = deviceController;

    wxString deviceStr = m_deviceController->getDeviceAsString();
    wxString title = _T("ECU ( ") + deviceStr + _T(" ) flash upload progress!");

    SetTitle( title );

    /// FimwareUploadEvent
    m_deviceController->Connect( FirmwareUploadEvent::ERASING_PAGES_SUCCEEDED, wxEVT_FIRMWARE_UPLOAD, FirmwareUploadEventHandler( FirmwareUploadDialog::onFirmwarePagesErasingSucceeded ), NULL, this );
    m_deviceController->Connect( FirmwareUploadEvent::PAGE_UPLOADED, wxEVT_FIRMWARE_UPLOAD, FirmwareUploadEventHandler( FirmwareUploadDialog::onFirmwarePageUploaded ), NULL, this );
    m_deviceController->Connect( FirmwareUploadEvent::UPLOADING_PAGES_SUCCEEDED, wxEVT_FIRMWARE_UPLOAD, FirmwareUploadEventHandler( FirmwareUploadDialog::onFirmwareUploadingSucceeded ), NULL, this );
    m_deviceController->Connect( FirmwareUploadEvent::UPLOADING_PAGES_FAILED, wxEVT_FIRMWARE_UPLOAD, FirmwareUploadEventHandler( FirmwareUploadDialog::onFirmwareUploadingFailed ), NULL, this );

    Connect( VDC_EVENT_DEVICE_DISCONNECTED, wxEVT_VEMS_DEVICE_CONTROLLER, VemsDeviceControllerEventHandler( FirmwareUploadDialog::onDeviceDisconnected ), NULL, this );

    m_pagesCount    = firmwareData->getPages();
    m_message      = wxString::Format( _T("Page index: %d / "), m_pagesCount );

    SetSize( 300, 150 );
    SetFocus();

//    m_deviceController->uploadFirmware( firmwareData );
}

FirmwareUploadDialog::~FirmwareUploadDialog()
{
    m_deviceController->Disconnect( FirmwareUploadEvent::ERASING_PAGES_SUCCEEDED, wxEVT_FIRMWARE_UPLOAD, FirmwareUploadEventHandler( FirmwareUploadDialog::onFirmwarePagesErasingSucceeded ), NULL, this );
    m_deviceController->Disconnect( FirmwareUploadEvent::PAGE_UPLOADED, wxEVT_FIRMWARE_UPLOAD, FirmwareUploadEventHandler( FirmwareUploadDialog::onFirmwarePageUploaded ), NULL, this );
    m_deviceController->Disconnect( FirmwareUploadEvent::UPLOADING_PAGES_SUCCEEDED, wxEVT_FIRMWARE_UPLOAD, FirmwareUploadEventHandler( FirmwareUploadDialog::onFirmwareUploadingSucceeded ), NULL, this );
    m_deviceController->Disconnect( FirmwareUploadEvent::UPLOADING_PAGES_FAILED, wxEVT_FIRMWARE_UPLOAD, FirmwareUploadEventHandler( FirmwareUploadDialog::onFirmwareUploadingFailed ), NULL, this );

    Disconnect( VDC_EVENT_DEVICE_DISCONNECTED, wxEVT_VEMS_DEVICE_CONTROLLER, VemsDeviceControllerEventHandler( FirmwareUploadDialog::onDeviceDisconnected ), NULL, this );

}


void FirmwareUploadDialog::onDeviceDisconnected( VemsDeviceControllerEvent& event )
{
    Update( m_pagesCount );

    wxMessageDialog dialog( this, _T("Device disconnected!"),
                            _T("Firmware upload"), wxOK, wxDefaultPosition );
    if ( dialog.ShowModal() == wxID_OK ) {

        Destroy();
    }
    //m_deviceController->removeEventHandler( this );

}

void FirmwareUploadDialog::onFirmwareUploadingStarted( FirmwareUploadEvent& event )
{
}

void FirmwareUploadDialog::onFirmwarePagesErasingSucceeded( FirmwareUploadEvent& event )
{
}

void FirmwareUploadDialog::onFirmwareUploadingPagesStarted( FirmwareUploadEvent& event )
{
}

void FirmwareUploadDialog::onFirmwarePageUploaded( FirmwareUploadEvent& event )
{
    size_t page = event.getFirmwarePageIdx();

    wxString message = m_message + wxString::Format( _T("%d"), page );
    Update( page, message );
}

void FirmwareUploadDialog::onFirmwareUploadingSucceeded( FirmwareUploadEvent& event )
{
    Update( m_pagesCount );

    wxMessageDialog dialog( this, _T("Firmware upload was successful!"),
                            _T("Firmware upload"), wxOK, wxDefaultPosition );
    if ( dialog.ShowModal() == wxID_OK ) {
        m_deviceController->start();
        Destroy();
    }
}

void FirmwareUploadDialog::onFirmwareUploadingFailed( FirmwareUploadEvent& event )
{
}
