#ifndef GPSCONFIGDIALOG_H
#define GPSCONFIGDIALOG_H

#include <wx/wx.h>
#include "../ObserverBase.h"
#include "../Observable.h"
#include "../ProgressBar.h"
#include "../common/ProcessBase.h"
#include "../common/types/wxVariantHash.h"
#include "../common/types/IntArray.h"
#include "../GPSNMEA/NMEAParse.h"
#include "../serialconfig.h"

class VemsTuneController;
class CommunicationBase;

class GPSDetectProcess : public ProcessBase, public Observable
{
    public:
        GPSDetectProcess( wxArrayString comports, wxArrayString baudrates );

        bool            process();
        void            reset() {}
        void            stop();
    private:
        unsigned short  m_t;
        bool            m_running;
        bool            m_stopped;
        wxArrayString   m_comports;
        IntArray        m_baudrates;

        void            finished();

        wxString        readPort( CommunicationBase* cbase );
        wxString        parseGPSString( const wxString& str );
};


class GPSConfigDialog : public wxDialog, public ObserverBase
{
    public:
        GPSConfigDialog( wxWindow* parent, VemsTuneController* vtc);
        ~GPSConfigDialog();

        void    notify( int i );
        void    notify( wxVariantHash& hash );

    protected:

    private:
        VemsTuneController*     m_vtc;
        wxComboBox*             m_comports;
        wxComboBox*             m_baudRate;
        wxComboBox*             m_enabled;

        //wxString                m_currport, m_currbrate, m_isEnabled;
        SerialConfig            m_serialConfig;

        ProgressBar*            m_progressbar;

        wxStaticText*           m_txtDetect;

        wxArrayString           m_baudRateS;

        //ConfigBase*             m_config;
        GPSDetectProcess*       m_process;
        wxButton                *m_testButton, *m_detectButton, *m_saveButton,
                                *m_closeButton, *m_cancelButton;

        wxStaticBitmap*         m_statBmp;
        wxStaticText*           m_result;
        wxBitmap                m_bmp;
        bool                    m_found;
        bool                    m_saved;
        bool                    m_wasECUOnline;



        void    buildGUI();
        void    getComports( wxArrayString& ports );

        void    OnSave( wxCommandEvent& event );
        void    OnTest( wxCommandEvent& event );
        void    OnDetect( wxCommandEvent& event );
        void    OnClose( wxCommandEvent& event );
        void    OnCancelButton( wxCommandEvent& event );

        void    OnGPSEnabledCombo( wxCommandEvent& event );

        void    enableButtons( bool enable = true );

        DECLARE_EVENT_TABLE()
};

#endif // GPSCONFIGDIALOG_H
