#include "GeneralConfigFrame.h"

#include "../ConfigBase.h"
#include "../logger.h"


GeneralConfigFrame::GeneralConfigFrame( wxWindow* parent, wxWindowID id, const wxString& title, ConfigBase* config )
    : wxFrame( parent, id, title )
{
    SetIcon(wxIcon(_T("vemsLogoIcon2")));
    m_configBase = config;
    m_combo = NULL;
    this->initGUI();
}

GeneralConfigFrame::~GeneralConfigFrame()
{
    //dtor
}

void GeneralConfigFrame::initGUI()
{
    m_mainPanel = new wxPanel( this );
    m_mainPanel->SetSizer( new wxBoxSizer( wxVERTICAL ) );
    m_currentSizer = m_mainPanel->GetSizer();
}

/**
 \brief :   Add a StaticBox with label to the frame and set as current
            default : no label
*/
void GeneralConfigFrame::addGroup( const wxString& label )
{
    wxStaticBox* box = new wxStaticBox( m_mainPanel, wxID_ANY, label );
    wxFont font = box->GetFont();
    font.SetWeight( wxFONTWEIGHT_BOLD );
    box->SetFont( font );
    box->SetForegroundColour( *wxBLACK );

    m_currentSizer = new wxStaticBoxSizer( box , wxVERTICAL );
    m_mainPanel->GetSizer()->Add( m_currentSizer, 0, wxEXPAND | wxALL, 5 );
}

/**
    \brief : Select Main and set as current
*/
void GeneralConfigFrame::selectMainGroup()
{
    m_currentSizer = m_mainPanel->GetSizer();
}

/**
    \brief :    Add a ComboBox to the current group
                Fill ComboBox with path's groupnames
*/
void GeneralConfigFrame::addComboBox( const wxString& path, const wxString& label, const wxString& defaultValue )
{
    wxArrayString groupArray;
//    Helper::getGroupArray( path, &groupArray );

    const int ID = wxNewId();

    m_combo = new wxComboBox( m_mainPanel,
                              ID,
                              _T(""),
                              wxDefaultPosition,
                              wxDefaultSize,
                              groupArray,
                              wxCB_READONLY,
                              wxDefaultValidator,
                              path );

    if( label != wxEmptyString )
    {
        wxStaticText* text = new wxStaticText( m_mainPanel, wxID_ANY, label );
        wxBoxSizer* lineSizer = new wxBoxSizer( wxHORIZONTAL );
        lineSizer->Add( text, 1, wxALIGN_CENTER_VERTICAL, 0 );
        lineSizer->Add( m_combo, 1, wxALIGN_CENTER_VERTICAL, 0 );
        m_currentSizer->Add( lineSizer, 0, wxRIGHT | wxEXPAND | wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL, 5 );
    }
    else
    {
        m_currentSizer->Add( m_combo, 1, wxRIGHT|wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5 );
    }

    m_combo->SetValue( defaultValue );

    this->Connect( ID, wxEVT_COMMAND_TEXT_UPDATED, wxCommandEventHandler( GeneralConfigFrame::OnComboBox ) );

    setOptimalSize();
}

/**
 \brief :   Add a TextCtrl to the current sizer
 \details :
 \param entry - If control not connected to a combo box entry = full entry path ( /.../.../entryName )
                Else entry = entryName
*/
void GeneralConfigFrame::addTextCtrl( const wxString& entry,
                                      const wxString& defaultPath,
                                      const wxString& comboPath,
                                      const wxString& comboSubPath )
{
    wxString entryPath = wxEmptyString;
    wxString entryName = wxEmptyString;

    if( comboPath == wxEmptyString )
    {
        entryPath = entry;
        entryName = entry.AfterLast( _T('/') );
    }
    else
    {
        const wxString comboSelection = ( m_combo ? m_combo->GetValue() : _T("") );
        entryPath = comboPath + comboSelection + _T("/") + comboSubPath + entry;
        entryName = entry;
    }

    //Logger::logf( LOG_MSG, _T("GeneralConfigFrame::addTextCtrl() - entryPath") + entryPath );

    wxString value = wxEmptyString;

    if( !m_configBase->read( entryPath, &value ) ) { m_configBase->read( defaultPath + entry, &value ); }

    wxConfigTextCtrl* textCtrl = new wxConfigTextCtrl( m_mainPanel, wxID_ANY, value );
    textCtrl->setEntryName( entryPath );
    textCtrl->setSubPath( comboSubPath );
    textCtrl->setDefaultPath( defaultPath );
    wxStaticText* text = new wxStaticText( m_mainPanel, wxID_ANY, entryName );
    wxBoxSizer* lineSizer = new wxBoxSizer( wxHORIZONTAL );
    lineSizer->Add( text, 1, wxALIGN_CENTER_VERTICAL, 0 );
    lineSizer->Add( textCtrl, 1, wxALIGN_CENTER_VERTICAL, 0 );
    //m_currentSizer->Add( lineSizer, 0, wxRIGHT | wxEXPAND | wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL, 5 );
    m_currentSizer->Add( lineSizer, 0, wxALL | wxEXPAND | wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL, 1 );

    m_textCtrlA.Add( textCtrl );

    if( comboPath != wxEmptyString )
    {
        m_viewers.Add( textCtrl );
    }

    setOptimalSize();
}

//frame->addTextCtrlGroup( wxEmptyString, _T("/defaultparams/"), _T("/sessions/"), _T("params/") );
/**
 \brief :

*/
void GeneralConfigFrame::addTextCtrlGroup( const wxString& path,
                                           const wxString& defaultPath,
                                           const wxString& comboPath,
                                           const wxString& comboSubPath )
{
    wxString oldPath = m_configBase->getPath();

    wxString groupPath = path;

    if( comboPath != wxEmptyString )
    {
        const wxString comboSelection = ( m_combo ? m_combo->GetValue() : _T("") );
        groupPath = comboPath + comboSelection + _T("/") + comboSubPath;
    }

    m_configBase->setPath( groupPath );

    //Logger::logf( LOG_MSG, _T("GeneralConfigFrame::addTextCtrlGroup() - groupPath = ") + groupPath );

    if( !m_configBase->getNumberOfEntries() )
    {
        m_configBase->setPath( defaultPath );
    }

    long dummy      = 0;
    wxString entry  = wxEmptyString;
    wxString value  = wxEmptyString;
    wxConfigTextCtrl* textCtrl  = NULL;
    wxStaticText* staticText    = NULL;

    bool bCont = m_configBase->getFirstEntry( entry, dummy );

    while( bCont )
    {
        m_configBase->read( entry, &value );

        staticText  = new wxStaticText( m_mainPanel, wxID_ANY, entry );
        textCtrl    = new wxConfigTextCtrl( m_mainPanel, wxID_ANY, value );
        textCtrl->setEntryName( groupPath + entry );
        textCtrl->setSubPath( comboSubPath );
        textCtrl->setDefaultPath( defaultPath );
        wxBoxSizer* lineSizer = new wxBoxSizer( wxHORIZONTAL );
        lineSizer->Add( staticText, 1, wxALIGN_CENTER_VERTICAL, 0 );
        lineSizer->Add( textCtrl, 1, wxALIGN_CENTER_VERTICAL, 0 );
        //m_currentSizer->Add( lineSizer, 0, wxRIGHT | wxEXPAND | wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL, 5 );
        m_currentSizer->Add( lineSizer, 0, wxALL | wxEXPAND | wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL, 1 );

        m_textCtrlA.Add( textCtrl );

        if( comboPath != wxEmptyString )
        {
            m_viewers.Add( textCtrl );
        }

        bCont = m_configBase->getNextEntry( entry, dummy );
    }

    m_configBase->setPath( oldPath );

    setOptimalSize();
}

void GeneralConfigFrame::addTextCtrlGroup( const BoolHash& boolHash)
{
    BoolHash::const_iterator IT = boolHash.begin();

    for(;IT != boolHash.end(); IT++)
    {
        bool value = IT->second;
        wxString valueStr = wxString::Format(_T("%i"),value);
        wxStaticText* staticText  = new wxStaticText( m_mainPanel, wxID_ANY, IT->first );
        wxTextCtrl* textCtrl    = new wxTextCtrl( m_mainPanel, wxID_ANY, valueStr );
        textCtrl->Enable(false);

        wxBoxSizer* lineSizer = new wxBoxSizer( wxHORIZONTAL );
        lineSizer->Add( staticText, 1, wxALIGN_CENTER_VERTICAL, 0 );
        lineSizer->Add( textCtrl, 1, wxALIGN_CENTER_VERTICAL, 0 );
        //m_currentSizer->Add( lineSizer, 0, wxRIGHT | wxEXPAND | wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL, 5 );
        m_currentSizer->Add( lineSizer, 0, wxALL | wxEXPAND | wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL, 1 );
    }
    setOptimalSize();
}

void GeneralConfigFrame::addEntrySelectComboBox( const wxString& entry,
                                                 const wxArrayString choices,
                                                 const wxString& defaultPath,
                                                 const wxString& comboPath,
                                                 const wxString& comboSubPath )
{
    wxString entryPath = wxEmptyString;
    wxString entryName = wxEmptyString;

    if( comboPath == wxEmptyString )
    {
        entryPath = entry;
        entryName = entry.AfterLast( _T('/') );
    }
    else
    {
        const wxString comboSelection = ( m_combo ? m_combo->GetValue() : _T("") );
        entryPath = comboPath + comboSelection + _T("/") + comboSubPath + entry;
        entryName = entry;
    }

    wxString value = wxEmptyString;

    if( !m_configBase->read( entryPath, &value ) ) { m_configBase->read( defaultPath + entry, &value ); }

    //wxConfigTextCtrl* textCtrl = new wxConfigTextCtrl( m_mainPanel, wxID_ANY, value );
    wxConfigComboBox* combo = new wxConfigComboBox( m_mainPanel, wxID_ANY,
                                                    value,
                                                    wxDefaultPosition,
                                                    wxDefaultSize,
                                                    choices );

    combo->setEntryName( entryPath );
    combo->setSubPath( comboSubPath );
    combo->setDefaultPath( defaultPath );
    wxStaticText* text = new wxStaticText( m_mainPanel, wxID_ANY, entryName );
    wxBoxSizer* lineSizer = new wxBoxSizer( wxHORIZONTAL );
    lineSizer->Add( text, 0, wxALIGN_CENTER_VERTICAL | wxRIGHT, 5 );
    lineSizer->Add( combo, 1, wxALIGN_CENTER_VERTICAL, 0 );
    //m_currentSizer->Add( lineSizer, 0, wxRIGHT | wxEXPAND | wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL, 5 );
    m_currentSizer->Add( lineSizer, 0, wxALL | wxEXPAND | wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL, 1 );

    m_textCtrlA.Add( combo );

    if( comboPath != wxEmptyString )
    {
        m_viewers.Add( combo );
    }

    setOptimalSize();
}


/**
 \brief : Add a red warning text to the current group
*/
void GeneralConfigFrame::addWarningText( const wxString& warningText )
{
    wxStaticText* text = new wxStaticText( m_mainPanel, wxID_ANY, warningText );
    wxFont font = text->GetFont();
    font.SetWeight( wxFONTWEIGHT_BOLD );
    text->SetFont( font );
    text->SetForegroundColour( *wxRED );

    m_currentSizer->Add( text, 1, wxALL | wxALIGN_CENTER_HORIZONTAL, 5 );

    setOptimalSize();
}

void GeneralConfigFrame::addSaveButton( wxAlignment align )
{
   	const int ID = wxNewId();
   	wxButton* saveButton = new wxButton( m_mainPanel, ID, _T("Save") );
   	m_mainPanel->GetSizer()->Add( saveButton, 0, align | wxALL, 5 );
   	this->Connect( ID, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( GeneralConfigFrame::OnSaveButton ) );
    setOptimalSize();
}

void GeneralConfigFrame::setOptimalSize()
{
    m_mainPanel->Fit();
    SetClientSize( m_mainPanel->GetSize() );
    SetMinSize( GetSize() );
}

void GeneralConfigFrame::OnComboBox( wxCommandEvent& event )
{
    wxString path = ( ( wxComboBox* )( event.GetEventObject() ) )->GetName();
    wxString comboSelection = ( ( wxComboBox* )( event.GetEventObject() ) )->GetValue();

    IConfigCtrl* textCtrl = NULL;
    wxString entryPath  = wxEmptyString;    //full entry path
    wxString entryName  = wxEmptyString;
    wxString entryValue = wxEmptyString;

    for( size_t i = 0; i < m_viewers.GetCount(); ++i )
    {
        textCtrl = m_viewers[ i ];

        entryName = textCtrl->getEntryName().AfterLast( _T('/') );
        entryPath = path + comboSelection + _T("/") + textCtrl->getSubPath() + entryName;
        if( !m_configBase->read( entryPath, &entryValue ) ) { m_configBase->read( textCtrl->getDefaultPath() + entryName, &entryValue ); }
        textCtrl->setEntryValue( entryValue );
        textCtrl->setEntryName( entryPath );
        Logger::logf( LOG_MSG, _T("GeneralConfigFrame::OnComboBox() - entryPath = ") + entryPath );
    }
}

/**
 \brief :   Save all TextCtrl values
*/
void GeneralConfigFrame::OnSaveButton( wxCommandEvent& event )
{
    wxString entry = wxEmptyString;
    wxString value = wxEmptyString;

    for ( size_t i = 0; i < m_textCtrlA.GetCount(); ++i )
    {
        entry = m_textCtrlA[ i ]->getEntryName();
        value = m_textCtrlA[ i ]->getEntryValue();

        m_configBase->write( entry, value );
    }

    m_configBase->save();
	Close();
}
