#include "IgnitionOutputPanel.h"

#include "../logger.h"
#include "../DirHelper.h"
#include "../stdatamanager/STDataMgrBase.h"
#include "../gaugedesc/OutputChannelSQL.h"
#include "../vemsprotocol/CommandRequestBase.h"
#include "../vemsprotocol/VemsCommand.h"
#include "../common/types/IntSet.h"


int idIgnitionDualMode = wxNewId();
int idIgnitionOutputNum = wxNewId();
int idTestingCombo = wxNewId();

BEGIN_EVENT_TABLE( IgnitionOutputPanel, wxPanel )

    EVT_COMBOBOX( idIgnitionDualMode, IgnitionOutputPanel::OnDualMode )
    EVT_COMBOBOX( idIgnitionOutputNum, IgnitionOutputPanel::OnOutputNum )
    EVT_COMBOBOX( idTestingCombo, IgnitionOutputPanel::OnTestingMode )
    EVT_CLOSE( IgnitionOutputPanel::OnClose)

END_EVENT_TABLE()

IgnitionOutputPanel::IgnitionOutputPanel(wxWindow * parent, STDataMgrBase * stdata_mgr, SymbolTable * symbols,
CommandRequestBase* requestMgr, wxArrayString& symbolsToMark )
{
    m_symbolsToMark = symbolsToMark;

    m_stDataMgr = stdata_mgr;
    m_symboltable = symbols;
    m_requestMgr = requestMgr;

    m_statvcontroller = new StaticViewersController( parent, _T("Ignition outputs visual") );
    m_statvcontroller->setController(m_stDataMgr);
    m_statvcontroller->addViewer(this);

    getValidSymbols();

    buildGUI(parent);
}

IgnitionOutputPanel::~IgnitionOutputPanel()
{
    unregisterViewer();

    m_statvcontroller->removeViewer(this);

    delete m_statvcontroller;
}

void IgnitionOutputPanel::buildGUI(wxWindow* parent)
{
    Create(parent, wxID_ANY);

    wxString iconfilename = DirHelper::GetPath(dICON) + _T("ul-warning-m.png");
    m_warningbmp = new wxBitmap( iconfilename, wxBITMAP_TYPE_PNG );

    SetBackgroundColour( wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE ) );
    ClearBackground();

    //wxBoxSizer* m_mainsizer = new wxBoxSizer( wxVERTICAL );
    m_mainsizer  = new wxBoxSizer( wxVERTICAL );

    wxStaticText * text = new wxStaticText( this, -1, _T("Must not be changed with running engine") );
    wxFont font = text->GetFont();
    font.SetWeight( wxFONTWEIGHT_BOLD );
    text->SetFont( font );
    text->SetForegroundColour( wxColor(255,0,0 ) );

    wxStaticText* text2 = new wxStaticText( this, -1, _T("Ignition drivers (traversed backwards). The red arrow shows the firing order.") );
    text2->SetFont( font );

    m_mainsizer->Add( text, 0, wxALL | wxEXPAND, 5 );
    m_mainsizer->Add( text2, 0, wxALL | wxEXPAND, 5 );

    wxBoxSizer* middleSizer = new wxBoxSizer( wxHORIZONTAL );

    ///

    wxStaticBoxSizer* groupSizer = new wxStaticBoxSizer( wxVERTICAL, this, _T("") );

    wxFlexGridSizer* grid = new wxFlexGridSizer( 1, 5, 3 );

    wxStaticText* text3 = new wxStaticText( this, -1, _T("Dual Output Mode"));

    wxArrayString dualarr;
    dualarr.Add(_T("Disabled")); dualarr.Add(_T("Enabled"));

    m_dualMode = new wxComboBox( this, idIgnitionDualMode, _T(""), wxDefaultPosition, wxDefaultSize,
        dualarr, wxCB_DROPDOWN | wxCB_READONLY );

    grid->Add( text3, 0, wxALL, 2 );
    grid->Add( m_dualMode, 0, wxBOTTOM | wxRIGHT | wxLEFT, 2 );

    ///
    wxStaticText* text4 = new wxStaticText( this, -1, _T("Ignition outputs in use in h[2]") );

    wxArrayString outarr;
    outarr.Add(_T("00 only")); outarr.Add(_T("01..00")); outarr.Add(_T("02..00"));
    outarr.Add(_T("03..00")); outarr.Add(_T("04..00")); outarr.Add(_T("05..00"));
    outarr.Add(_T("06..00")); outarr.Add(_T("07..00"));

    m_outputNum = new wxComboBox( this, idIgnitionOutputNum, _T(""), wxDefaultPosition, wxDefaultSize,
        outarr, wxCB_DROPDOWN | wxCB_READONLY );

    grid->Add( text4, 0, wxBOTTOM | wxRIGHT | wxLEFT, 2 );
    grid->Add( m_outputNum, 0, wxBOTTOM | wxRIGHT | wxLEFT, 2 );

    ///

    wxStaticText* text5 = new wxStaticText( this, -1, _T("Testing mode") );
    wxArrayString tmodes;
    tmodes.Add( _T("OFF") );
    tmodes.Add( _T("ON") );

    m_testingMode = new wxComboBox( this, idTestingCombo, _T("OFF"), wxDefaultPosition, wxDefaultSize, tmodes, wxCB_DROPDOWN | wxCB_READONLY );

    grid->Add( text5, 0, wxBOTTOM | wxRIGHT | wxLEFT, 2 );
    grid->Add( m_testingMode, 0, wxBOTTOM | wxRIGHT | wxLEFT, 2 );

    groupSizer->Add( grid );

    ///
    wxFlexGridSizer* grid2 = new wxFlexGridSizer(6,5,5);

    for ( size_t i = 0; i < 8; ++i )
    {
        int id = wxNewId(), id2 = wxNewId();
        m_outputCombo[i] = new wxComboBox( this, id, _T(""), wxDefaultPosition, wxDefaultSize,
            0, NULL, wxCB_DROPDOWN | wxCB_READONLY | wxCB_SORT );

        long style = ( i == 0 ) ? wxRB_GROUP : 0;

        m_showPins[i] = new wxRadioButton( this, id2, _T(""), wxDefaultPosition, wxDefaultSize, style );
        m_showPins[i]->SetValue( false );

        Connect( id, wxEVT_COMMAND_COMBOBOX_SELECTED,
            wxCommandEventHandler(IgnitionOutputPanel::OnOutputCombo), 0, this );

        Connect( id2, wxEVT_COMMAND_RADIOBUTTON_SELECTED,
            wxCommandEventHandler(IgnitionOutputPanel::OnRadio), 0, this );

        wxStaticText* t = new wxStaticText( this, -1, wxString::Format(_T("%d"), i) );

        int bId = wxNewId();
        m_testButtons[i] = new wxButton( this, bId, _T("Test") );
        m_testButtons[i]->Show( false );
        m_hexIdTxt[i] = new wxTextCtrl( this, -1, _T(""), wxDefaultPosition, wxSize(32, 20) );
        m_hexIdTxt[i]->Show( false );
        m_hexIdTxt[i]->Enable( false );

        m_conflictBMP[i] = new wxStaticBitmap( this, wxID_ANY, wxNullBitmap );

        Connect( bId, wxEVT_COMMAND_BUTTON_CLICKED,
            wxCommandEventHandler(IgnitionOutputPanel::OnTestButton), 0, this );

        grid2->Add( m_showPins[i], 0, wxLEFT | wxRIGHT | wxBOTTOM | wxEXPAND, 5 );
        grid2->Add( t, 0, wxLEFT | wxRIGHT | wxBOTTOM | wxEXPAND, 5 );
        grid2->Add( m_outputCombo[i], 1, wxLEFT | wxRIGHT | wxBOTTOM | wxEXPAND, 5 );
        grid2->Add( m_conflictBMP[i], 0, wxBOTTOM | wxEXPAND, 5 );
        grid2->Add( m_hexIdTxt[i], 0, wxLEFT | wxRIGHT | wxBOTTOM | wxEXPAND, 5 );
        grid2->Add( m_testButtons[i], 0, wxLEFT | wxRIGHT | wxBOTTOM | wxEXPAND, 5 );


        m_comboIds.Add( id );
        m_radioIds.Add( id2 );
        m_testIds.Add( bId );
    }

    grid2->AddGrowableCol(2);

    middleSizer->Add( grid2, 1, wxALL | wxEXPAND, 5 );

    wxBitmap bmp( DirHelper::GetPath( dICON ) + _T("red_arrow.png"), wxBITMAP_TYPE_PNG );
    wxStaticBitmap* red_arrow = new wxStaticBitmap( this, wxID_ANY, bmp );

    middleSizer->Add( red_arrow, 0, wxRIGHT | wxALIGN_TOP, 4 );

    middleSizer->Add( groupSizer, 0, wxALL, 0 );

    m_mainsizer->Add( middleSizer, 0, wxALL | wxEXPAND, 3 );

    /// econoseal panels

    wxBoxSizer* ecosizer = new wxBoxSizer(wxHORIZONTAL);
    wxStaticBoxSizer *b18sizer = new wxStaticBoxSizer(wxHORIZONTAL, this, _T("Econoseal 18 pinout"));
    wxStaticBoxSizer* b36sizer = new wxStaticBoxSizer(wxHORIZONTAL, this, _T("Econoseal 36 pinout"));

    m_p36 = new EconosealPanel(this, 36);
    m_p18 = new EconosealPanel(this, 18);

    b18sizer->Add(m_p18, 0, wxEXPAND | wxALL, 10);
    b36sizer->Add(m_p36, 0, wxEXPAND | wxALL, 10);

    ecosizer->Add(b18sizer, 0, wxEXPAND | wxALL, 10);
    ecosizer->Add(b36sizer, 0, wxEXPAND | wxALL, 10);

    m_mainsizer->Add( ecosizer, 0, wxEXPAND | wxALIGN_CENTER | wxALIGN_CENTER_HORIZONTAL, 5 );

    m_statcpanel = new StaticControlPanel( this, m_statvcontroller );

    m_mainsizer->Add( m_statcpanel, 0, wxEXPAND | wxALL, 10 );

    SetSizer( m_mainsizer );

    m_mainsizer->Fit( this );
    m_mainsizer->SetSizeHints( this );
}

/// symbols: exist in DB and also in symboltable
void IgnitionOutputPanel::getValidSymbols()
{
    wxObjectArray arr = OutputChannelSQL::getSymbols();

    m_validSymbols.Clear();

    for ( size_t i = 0; i < arr.GetCount(); ++i )
    {
        wxString symname = ((dbsymbol*)arr[i])->name;

        SymbolTable::const_iterator it = m_symboltable->find(symname);

        if ( it != m_symboltable->end() )
        {
            m_validSymbols.Add( symname );
        }
    }
    arr.Clear();
}

void IgnitionOutputPanel::update()
{
    ///
    SymbolTable::iterator s_it;
    ///

    /// iac stepper ?
    IntSet iacvals;
    s_it = m_symboltable->find( _T("iac_type") );
    if ( s_it != m_symboltable->end() )
    {
        int val = (uint8_t)m_stDataMgr->getDataFromViewerModel( *(s_it->second), 0, 0 );

        // stepper is set
        if ( val == 3 )
        {
            for ( size_t i = 1; i <= 4; ++i )
            {
                wxString symN = wxString::Format( _T("iac_step_seq_%d"), i );

                s_it = m_symboltable->find( symN );

                if ( s_it != m_symboltable->end() )
                {
                    uint8_t val = (uint8_t)m_stDataMgr->getDataFromViewerModel( *(s_it->second), 0, 0 );

                    if ( val > 127 )
                        val -= 128;

                    // steppers: A => 0x36, B => 0x46, C => 0x56, D => 0x66, E => 0x76
                    val = ( 3 + val ) * 16 + 6;
                    iacvals.insert( val );
                }
            }
        }
    }

    /// dual solenoid
    // is dual solenoid mode set?
    int dual_solenoid_channel_out = -1;
    int dual_solenoid_channels[2] = { -1, -1 };
    s_it = m_symboltable->find( _T("iac_dualsol") );
    if ( s_it != m_symboltable->end() )
    {
        int val = (uint8_t)m_stDataMgr->getDataFromViewerModel( *(s_it->second), 0, 0 );
        if ( val == 1 )
        {
            s_it = m_symboltable->find( _T("iac_sol_channel_names") );
            dual_solenoid_channel_out = (uint8_t)m_stDataMgr->getDataFromViewerModel( *(s_it->second), 0, 0 );

            bool shouldCheckit = true;

            if ( dual_solenoid_channel_out == 0x0f ) // 0x0f is disabled
                shouldCheckit = false;
            if ( dual_solenoid_channel_out != 0x02 && dual_solenoid_channel_out != 0x06 ) // 2 is I259, 6 is P259
                shouldCheckit = false;

            if ( shouldCheckit ) // 0x0f is disabled
            {
                s_it = m_symboltable->find( _T("iac_sol_channel_bits_dual") );
                int dualval = (uint8_t)m_stDataMgr->getDataFromViewerModel( *(s_it->second), 0, 0 );

                if ( dualval < 7 )
                {
                    //0,i1 ; 1,i2..., 6,i7
                    dual_solenoid_channels[0] = 16 * dualval + dual_solenoid_channel_out;
                    dual_solenoid_channels[1] = 16 * ( dualval + 1 ) + dual_solenoid_channel_out + 128; // inverted
                }
                else
                {
                    // i0,1; i1,2... i6,7
                    dualval -= 7;
                    dual_solenoid_channels[0] = 16 * dualval + dual_solenoid_channel_out + 128; // inverted
                    dual_solenoid_channels[1] = 16 * ( dualval + 1) + dual_solenoid_channel_out;
                }
            }
        }
    }
    if ( dual_solenoid_channels[0] > 127 ) dual_solenoid_channels[0] -= 128;
    if ( dual_solenoid_channels[1] > 127 ) dual_solenoid_channels[1] -= 128;
    // end of dual solenoid mode.
    ///--------


    // is dual mode enabled
    bool isDual = ((uint8_t)m_stDataMgr->getDataFromViewerModel(*(*m_symboltable)[_T("ign_out_dual")]))!=0?true:false;

    if ( isDual )
        m_dualMode->SetSelection( 1 );
    else
        m_dualMode->SetSelection( 0 );

    // ignchmax
    uint8_t ignchmax = (uint8_t) m_stDataMgr->getDataFromViewerModel(*(*m_symboltable)[_T("ignchmax")]);
    m_outputNum->SetSelection( ignchmax );

    m_hexIds.Clear();

    wxArrayString::const_iterator it = m_validSymbols.begin();

    for ( ; it != m_validSymbols.end(); ++it )
    {
        uint8_t val = (uint8_t) m_stDataMgr->getDataFromViewerModel(*(*m_symboltable)[*it]);

        // if inverted ->
        if ( val > 127 )
            val -= 128;

        if ( m_hexIds.Index(val) == wxNOT_FOUND )
            m_hexIds.Add( val );
    }

    if ( dual_solenoid_channels[0] >= 0 )
    {
        if ( m_hexIds.Index( dual_solenoid_channels[0] ) == wxNOT_FOUND )
            m_hexIds.Add( dual_solenoid_channels[0] );
    }
    if ( dual_solenoid_channels[1] >= 0 )
    {
        if ( m_hexIds.Index( dual_solenoid_channels[1] ) == wxNOT_FOUND )
            m_hexIds.Add( dual_solenoid_channels[1] );
    }

    if ( iacvals.size() > 0 )
    {
        IntSet::const_iterator is_it = iacvals.begin();

        for ( ; is_it != iacvals.end(); ++is_it )
        {
            uint8_t val = *is_it;

            if ( m_hexIds.Index( val ) == wxNOT_FOUND )
                m_hexIds.Add( val );
        }
    }

    IntArray ignouts = OutputChannelSQL::getIgnitionOutputs( isDual );

    IntArray::const_iterator it2 = ignouts.begin();

    wxArrayString combolist;
    combolist.Clear();

    m_out2channel.clear();
    m_channel2str.clear();

    if ( isDual == false )
    {
        for ( ; it2 != ignouts.end(); ++it2 )
        {
            int val = *it2; // channel & type

            if ( m_hexIds.Index( val ) == wxNOT_FOUND )
            {
                OutputType* ot = OutputChannelSQL::getOutputType( val );

                if ( ot != NULL )
                {
                    wxString stepperStr = _T("");

                    // is it a stepper?

                    if ( ot->getType_id() == 6 )
                        stepperStr = wxString::Format( _T(" %c"), 65 + ( ot->getChannel_id() -3 ) );

                    // format: title channel (xA) type, connector
                    wxString comboStr = ot->title;

                    if ( ot->getType_id() == 6 )
                        comboStr += stepperStr;
                    else
                        comboStr += wxString::Format( _T(" %d"), ot->getChannel_id() );

                    comboStr += _T(" (") + ot->current + _T("A) ");
                    comboStr += ot->type;
                    comboStr += wxString::Format( _T(", EC%d"), ot->connector_type ) + _T(" pin ") + ot->pin;

                    combolist.Add( comboStr );

                    int chval = ot->channel;

                    m_out2channel[comboStr] = chval;
                    m_channel2str[chval] = comboStr;

                    delete ot;
                }
            }
        }
    } // isDual == false
    else
    {
        for ( ; it2 != ignouts.end(); it2 += 2 )
        {
            int val1 = *it2, val2 = *( it2 + 1);

            if ( ( m_hexIds.Index(val1) == wxNOT_FOUND) && ( m_hexIds.Index(val2) == wxNOT_FOUND ) )
            {
                OutputType* ot1 = OutputChannelSQL::getOutputType( val1 );
                OutputType* ot2 = OutputChannelSQL::getOutputType( val2 );

                if ( ot1 != NULL && ot2 != NULL )
                {
                    wxString stepperStr = _T("");

                    if ( ot1->getType_id() == 6 )
                        stepperStr = wxString::Format( _T(" %c+%c"), 65 + ( ot2->getChannel_id() - 3 ),
                            65 + ( ot1->getChannel_id() - 3 ) );

                    wxString comboStr = ot1->title;

                    // stepper?
                    if ( ot1->getType_id() == 6 )
                        comboStr += stepperStr;
                    else
                        comboStr += wxString::Format( _T(" %d+%d"), ot2->getChannel_id(),
                            ot1->getChannel_id() );

                    comboStr += _T(" (") + ot1->current + _T("A) ");
                    comboStr += ot1->type + wxString::Format( _T(", EC%d "), ot1->connector_type );
                    comboStr += _T(" pin ") + ot2->pin + _T(", ") + ot1->pin;

                    combolist.Add( comboStr );

                    int chval = 256 * ot1->channel + ot2->channel;

                    m_out2channel[comboStr] = chval;
                    m_channel2str[ot1->channel] = comboStr;

                    delete ot1;
                    delete ot2;
                }
            }
        }
    }

    // update gui

    Freeze();

    // refresh combolist
    for ( size_t i = 0; i < 8; ++i )
    {
        m_outputCombo[i]->Clear();
        m_outputCombo[i]->Append( combolist );
    }

    // enable comboboxes according to ignchmax
    for ( size_t i = 0; i < 8; ++i )
        m_outputCombo[i]->Enable ( (i <= ignchmax) );

    setCombos();

    Thaw();

}

void IgnitionOutputPanel::setCombos()
{
    bool isDual = ( m_dualMode->GetSelection() == 1 );

    wxString symname;

    for ( size_t i = 0; i < 8; ++i )
    {
        symname = ( isDual ) ? wxString::Format( _T("h2d%d"), i ): wxString::Format( _T("h2%d"), i );

        double val = m_stDataMgr->getDataFromViewerModel( *(*m_symboltable)[symname] );

        m_hexIdTxt[i]->SetValue( wxString::Format( _T("%02x"), (uint8_t) val ) );

        uint8_t t_id, ch_id;
        int channel = -1;

        if ( OutputChannelSQL::getIgnitionMapping( isDual, (uint8_t) val, t_id, ch_id ) )
        {
            channel = 16 * ch_id + t_id;

            wxString cstr = m_channel2str[channel];

            m_outputCombo[i]->SetValue( cstr );
        }

        if ( m_outputCombo[i]->GetValue() == wxEmptyString )
        {
            if ( OutputChannelSQL::isValidType( channel ) )
            {
                m_outputCombo[i]->Append( str_conflict_value );
                m_outputCombo[i]->SetValue( str_conflict_value );
                m_outputCombo[i]->SetForegroundColour( wxColor( 255, 0, 0 ) );
                m_conflictBMP[i]->SetBitmap( *m_warningbmp );
                m_conflictBMP[i]->SetToolTip( _T("Press 'Validate config' button for further details") );
            }
            else
            {
                m_outputCombo[i]->Append( str_invalid_value );
                m_outputCombo[i]->SetValue( str_invalid_value );
                m_outputCombo[i]->SetForegroundColour( wxColor( 255, 0, 0 ) );
                m_conflictBMP[i]->SetBitmap( wxNullBitmap );
                m_conflictBMP[i]->SetToolTip( _T("") );
            }
        }
        else
        {
            m_outputCombo[i]->SetForegroundColour( wxSystemSettings::GetColour( wxSYS_COLOUR_WINDOWTEXT ) );
            m_conflictBMP[i]->SetBitmap( wxNullBitmap );
            m_conflictBMP[i]->SetToolTip( _T("") );
        }

        if ( m_showPins[i]->GetValue() )
        {
            showPins( i );
        }
    }
}

void IgnitionOutputPanel::updateModel()
{
    bool isDual = ( m_dualMode->GetSelection() == 1 );

    double dualvalue = ( isDual ) ? 1.0 : 0.0;

    // update dualmode
    Symbol symbol = *(*m_symboltable)[_T("ign_out_dual")];

    m_stDataMgr->setModelData( symbol, 0, 0, dualvalue );

    // update output channel numbers
    int selection = ( m_outputNum->GetSelection() );
    symbol = *(*m_symboltable)[_T("ignchmax")];

    m_stDataMgr->setModelData( symbol, 0, 0, (double) selection );

    for ( size_t i = 0; i < 8; ++i )
    {
        wxString selection = m_outputCombo[i]->GetValue();

        if ( selection == wxEmptyString || selection == str_invalid_value || selection == str_conflict_value )
        {
            continue;
        }

        int channel = m_out2channel[selection];

        if ( isDual ) channel = channel / 256;

        uint8_t channel_id = channel >> 4;
        uint8_t type_id = channel & 15;

        int h2val = OutputChannelSQL::getIgnh2x( channel_id, type_id, isDual );

        if ( h2val != -1 )
        {
            wxString symname = ( isDual ) ? wxString::Format( _T("h2d%d"), i ) : wxString::Format( _T("h2%d"), i );
            symbol = *(*m_symboltable)[symname];
            m_stDataMgr->setModelData( symbol, 0, 0, (double) h2val );
        }
    }
}

void IgnitionOutputPanel::requestWrite()
{
    bool isDual = ( m_dualMode->GetSelection() == 1 );
    updateModel();

    // write dualmode
    Symbol symbol = *(*m_symboltable)[_T("ign_out_dual")];
    m_stDataMgr->requestWrite( symbol );

    // write outp. channel numbers
    symbol = *(*m_symboltable)[_T("ignchmax")];
    m_stDataMgr->requestWrite( symbol );

    // write combos
    for ( size_t i = 0; i < 8; ++i )
    {
        wxString symname = ( isDual ) ? wxString::Format(_T("h2d%d"), i) : wxString::Format(_T("h2%d"), i );
        symbol = *(*m_symboltable)[symname];

        m_stDataMgr->requestWrite( symbol );
    }

}

void IgnitionOutputPanel::OnDualMode( wxCommandEvent& event )
{
    requestWrite();
    m_stDataMgr->updateRegisteredViewers();
    event.Skip();
}

void IgnitionOutputPanel::OnOutputNum( wxCommandEvent& event )
{
    requestWrite();
    m_stDataMgr->updateRegisteredViewers();
    event.Skip();
}

void IgnitionOutputPanel::OnOutputCombo( wxCommandEvent& event )
{
    requestWrite();
    m_stDataMgr->updateRegisteredViewers();

    event.Skip();
}

void IgnitionOutputPanel::OnRadio( wxCommandEvent& event )
{
    int id = event.GetId();

    m_p18->ClearAllPin();
    m_p36->ClearAllPin();

    int pos = m_radioIds.Index(id);

    if ( pos == wxNOT_FOUND || pos < 0 || pos > 7 )
    {
        event.Skip();
        return;
    }

    showPins( pos );
}

void IgnitionOutputPanel::showPins( int id )
{
    wxString combosel = m_outputCombo[id]->GetValue();

    m_p18->ClearAllPin();
    m_p36->ClearAllPin();

    int channel = m_out2channel[combosel];

    int channel1 = -1, channel2 = -1;
    long pin1 = -1, pin2 = -1;

    if ( channel > 255 )
    {
        channel1 = channel / 256;
        channel2 = channel % 256;
    }
    else
    {
        channel1 = channel;
    }

    OutputType* ot = OutputChannelSQL::getOutputType( channel1 );

    int connector_type = 0;

    if ( ot != NULL )
    {
        connector_type = ot->connector_type;
        ot->pin.ToLong( &pin1 );
        delete ot;
    }

    if ( channel2 != -1 )
    {
        ot = OutputChannelSQL::getOutputType( channel2 );

        if ( ot != NULL )
        {
            ot->pin.ToLong( &pin2 );
        }

        delete ot;
    }

    EconosealPanel* ecopanel = NULL;

    if ( connector_type == 18 )
        ecopanel = m_p18;
    if ( connector_type == 36 )
        ecopanel = m_p36;

    if ( ecopanel != NULL )
    {
        ecopanel->SetPinColor( pin1, wxColor( 255, 0,0 ) );

        if ( pin2 > 0 )
        {
            ecopanel->SetPinColor( pin2, wxColor( 255, 0, 0 ) );
        }
    }
}

void IgnitionOutputPanel::OnTestingMode( wxCommandEvent& event )
{
    bool enableTesting = ( m_testingMode->GetValue() == _T("ON") );

    // ignchmax
    uint8_t ignchmax = (uint8_t) m_stDataMgr->getDataFromViewerModel(*(*m_symboltable)[_T("ignchmax")]);

    for ( size_t i = 0; i < 8; ++i )
    {
        if ( i <= ignchmax )
            m_testButtons[i]->Show( enableTesting );
        else
            m_testButtons[i]->Show( false );

        m_hexIdTxt[i]->Show( enableTesting );
    }

    m_mainsizer->RecalcSizes();

    m_mainsizer->Fit(this);
    m_mainsizer->SetSizeHints(this);

    wxWindow* parent = GetParent();
    wxSizer* sizer = parent->GetSizer();
    sizer->RecalcSizes();
    sizer->Fit(parent);
    sizer->SetSizeHints(parent);

}

void IgnitionOutputPanel::OnTestButton( wxCommandEvent& event )
{
    int id = event.GetId();

    int buttonNr = m_testIds.Index( id );

    if ( buttonNr == wxNOT_FOUND )
        return;

    wxString command = wxString::Format(_T("Manmdn%02Xbye"), buttonNr);
    VemsCommand* vemsCommand = new StringCommand(command);

    //Logger::logf( LOG_MSG, _T("IgnitionOutputPanel::OnTestButton() - command: ") + command );
    m_requestMgr->request(vemsCommand);
}

void IgnitionOutputPanel::OnClose( wxCloseEvent & event )
{
    if ( GetParent() ) { GetParent()->Close(); }
}
