#include "OutputChannels.h"
#include "../logger.h"
#include "../DirHelper.h"
#include "../gaugedesc/OutputChannelSQL.h"
#include "../Symbol.h"
#include "../common/types/wxObjectArray.h"
#include "../stdatamanager/STDataMgrBase.h"
#include "../Math.h"
#include "../vemsprotocol/CommandRequestBase.h"
#include "../vemsprotocol/VemsCommand.h"
#include "../customwidgets/InfoWindow.h"
#include "../common/types/IntSet.h"

#include "../common/FunctionMeter.h"

#include <wx/tokenzr.h>

const wxString dbName = wxT("outputchannels.s3db");

const int IDSHOW_START = 1000;
const int IDCOMBO_START = 1100;
const int IDHEXID_START = 1200;
const int IDINV_START = 1300;

int idOutputChTestingMode = wxNewId();

BEGIN_EVENT_TABLE(OutputChannelsCfg, wxPanel)

    EVT_CLOSE( OutputChannelsCfg::onClose)
    EVT_COMBOBOX( idOutputChTestingMode, OutputChannelsCfg::onTestingMode )

END_EVENT_TABLE();

OutputChannelsCfg::OutputChannelsCfg(wxWindow * parent, STDataMgrBase * stdata_mgr,
    CommandRequestBase* requestMgr,
    SymbolTable * symbols,
    wxArrayString& symbolsToMark )
{
    m_symbolsToMark = symbolsToMark;

    m_num_combos = 0;
    m_stDataMgr = stdata_mgr;
    m_requestMgr = requestMgr;
    m_symbols = symbols;
    m_injgrouppinsStr = _T("");
    m_idleSolenoidChannelNo = -1;

    cacheOT();
    getAllOutputtypes();

    m_statvcontroller = new StaticViewersController( parent, _T("Assign output channels") );
    m_statvcontroller->setController(m_stDataMgr);
    m_statvcontroller->addViewer(this);

    m_injgroupused = 0;

    buildGUI(parent);

    cacheOT();
//    update();
}

OutputChannelsCfg::~OutputChannelsCfg()
{
    unregisterViewer();
    m_statvcontroller->removeViewer(this);

    if ( m_conflictfound || m_invalidvaluesfound)
    {
        InfoWindow::Display( NULL, _T("Output channels"), _T("There were invalid settings and/or conflicts"),
            IW_FADE_IN | IW_MOVE | IW_WARNING );
    }

    delete m_statvcontroller;

    cleanUp();
    delete[] m_outputTypeCache;
}

void OutputChannelsCfg::cacheOT()
{
    m_outputTypeCache = new OutputType*[256];
    for ( unsigned i = 0; i < 256; ++i )
    {
        m_outputTypeCache[i] = OutputChannelSQL::getOutputType(i);
    }
}

OutputType* OutputChannelsCfg::getOTFromCache( int val )
{
    if ( val < 0 || val > 255 )
        return NULL;
    else
    {
        if ( m_outputTypeCache[val] == NULL )
            return NULL;
        return new OutputType(*m_outputTypeCache[val]);
    }
}

//
void OutputChannelsCfg::getAllOutputtypes()
{
    m_out2channel.clear();
    m_channel2str.clear();

    m_outputtypestrings.Add(str_disabled_value);

    m_out2channel[str_disabled_value] = 255; // 0xff
    m_channel2str[0xff] = str_disabled_value;
    m_hex2indexHash[_T("ff")] = 0;

    uint8_t i;

    m_channels = OutputChannelSQL::getChannels();

    for ( i = 0; i < m_channels.GetCount(); ++i )
    {
        //OutputType * outputtype = OutputChannelSQL::getOutputType( m_channels[i] );
        OutputType* outputtype = getOTFromCache( m_channels[i] );

        // stepper
        wxString stepperStr = wxEmptyString;

        if ( outputtype->getType_id() == 6 )
            stepperStr = wxString::Format( _T(" %c"), 65 + ( outputtype->getChannel_id() - 3 ) );

        wxString comboStr = outputtype->title;

        if ( outputtype->getType_id() == 6 )
            comboStr += stepperStr;
        else
            comboStr += wxString::Format( _T(" %d"), outputtype->getChannel_id() );

        comboStr += _T(" (") + outputtype->current + _T("A) ") + outputtype->type;

        if (outputtype->getType_id() == 0)
            comboStr += wxT(", InjGroup");
        else
            comboStr += wxString::Format(wxT(", EC%d"), outputtype->connector_type) + wxT(" pin ") + outputtype->pin;


        m_outputtypestrings.Add(comboStr);

        m_out2channel[comboStr] = m_channels[i];
        m_channel2str[m_channels[i]] = comboStr;
        m_channel2str[m_channels[i]+128] = comboStr; // inverted

        m_hex2indexHash[wxString::Format(_T("%02x"), m_channels[i])] = i+1;
        m_hex2indexHash[wxString::Format(_T("%02x"), m_channels[i]+128)] = i+1; // inverted

        delete outputtype;
    }

    m_outputtypestrings.Add( str_invalid_value );

    m_out2channel[str_invalid_value] = -1;
    m_channel2str[-1] = str_invalid_value;
    m_hex2indexHash[_T("-2")] = m_channels.GetCount()+2;

    m_outputtypestrings.Add( str_conflict_value );

    m_out2channel[str_conflict_value] = -2;
    m_channel2str[-2] = str_conflict_value;
    m_hex2indexHash[_T("-3")] = m_channels.GetCount()+3;
}

void OutputChannelsCfg::buildGUI(wxWindow* parent)
{
    Create(parent, wxID_ANY);
    parent->Connect( wxEVT_CLOSE_WINDOW, wxCloseEventHandler( OutputChannelsCfg::onClose ), 0, this );

    wxString iconfilename = DirHelper::GetPath(dICON) + _T("ul-warning-m.png");
    m_warningbmp = new wxBitmap( iconfilename, wxBITMAP_TYPE_PNG );

    wxScrolledWindow* scpanel = new wxScrolledWindow( this );

    SetBackgroundColour( wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE ) );
    ClearBackground();

    scpanel->SetBackgroundColour( wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE ) );
    scpanel->ClearBackground();

    wxBoxSizer* sizer = new wxBoxSizer( wxVERTICAL );

    // warning txt
    wxStaticText * wtext = new wxStaticText( this, -1, _T("Must not be changed with running engine") );
    wxFont font = wtext->GetFont();
    font.SetWeight( wxFONTWEIGHT_BOLD );
    wtext->SetFont( font );
    wtext->SetForegroundColour( wxColor(255,0,0 ) );

    sizer->Add( wtext, 0, wxTOP | wxLEFT, 5 );

    /// testing mode combobox
    wxBoxSizer* tModeSizer = new wxBoxSizer( wxHORIZONTAL );
    wxStaticText* testTxt = new wxStaticText( this, wxID_ANY, _T("Switch testing mode") );
    wxArrayString arr; arr.Add( _T("OFF") ); arr.Add( _T("ON") );
    m_testingMode = new wxComboBox( this, idOutputChTestingMode, _T("OFF"),
        wxDefaultPosition, wxDefaultSize, arr, wxCB_DROPDOWN | wxCB_READONLY );

    tModeSizer->Add( testTxt, 0, wxTOP | wxLEFT | wxRIGHT, 5 );
    tModeSizer->Add( m_testingMode, 0, wxTOP | wxLEFT | wxRIGHT, 5 );

    sizer->Add( tModeSizer, 0, wxALL, 5 );

    ///

    // only visible items
    wxObjectArray symbols = OutputChannelSQL::getSymbols( true );

    for ( size_t i = 0; i < symbols.GetCount(); ++i )
    {
        dbsymbol* sym = (dbsymbol*) symbols[i];
        wxString symname = sym->name;

        if ( symname == _T("iac_sol_channel") )
            m_idleSolenoidChannelNo = i;

        SymbolTable::const_iterator it = m_symbols->find( symname );

        if ( it != m_symbols->end() )
        {
            m_dbsymbols.Add( sym );
        }
    }

    m_num_combos = m_dbsymbols.GetCount();

    if ( m_num_combos < 1 )
        return;

    m_showButtons = new wxRadioButton*[m_num_combos];
    m_HexIds = new wxTextCtrl*[m_num_combos];
    m_inverted = new wxCheckBox*[m_num_combos];
    m_outputtypes = new wxComboBox*[m_num_combos];
    m_conflictBMP = new wxStaticBitmap*[m_num_combos];
    m_isconflicted = new bool[m_num_combos];
    m_labels = new wxStaticText*[m_num_combos];

    wxFlexGridSizer* sizer2 = new wxFlexGridSizer( 8 );

    /// header
    wxStaticText** headers = new wxStaticText*[5];
    m_headerText = new wxStaticText*[3];

    wxFont font2(8, wxFONTFAMILY_DEFAULT, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_BOLD);
    headers[0] = new wxStaticText( scpanel, -1, _T("") ); // Show pins
    headers[1] = new wxStaticText( scpanel, -1, _T("Channel name") );
    headers[2] = new wxStaticText( scpanel, -1, _T("Output type") );
    headers[3] = new wxStaticText( scpanel, -1, _T("") );
    headers[4] = new wxStaticText( scpanel, -1, _T("Inverted") );
    m_headerText[0] = new wxStaticText( scpanel, -1, _T("Hex") );
    m_headerText[1] = new wxStaticText( scpanel, -1, _T("Signal") );
    m_headerText[2] = new wxStaticText( scpanel, -1, _T("Test") );

    for ( size_t i = 0; i < 5; ++i )
    {
        headers[i]->SetFont( font2 );
        sizer2->Add( headers[i], 0, wxALL, 5 );
    }

    for ( size_t i = 0; i < 3; ++i )
    {
        m_headerText[i]->SetFont( font2 );
        m_headerText[i]->Show( false );
        sizer2->Add( m_headerText[i], 0, wxALL, 5 );
    }

    m_testCheckBoxes = new wxCheckBox*[m_num_combos];
    m_testButtons = new wxToggleButton*[m_num_combos];
    m_testBIds.Clear();

    for ( size_t i = 0; i < m_num_combos; ++i )
    {
        m_isconflicted[i] = false;

        if (  i == 0 )
        {
            m_showButtons[0] = new wxRadioButton( scpanel, IDSHOW_START, wxEmptyString,
                wxDefaultPosition, wxDefaultSize, wxRB_GROUP );
            m_showButtons[0]->SetValue( true );
        }
        else
        {
            m_showButtons[i] = new wxRadioButton( scpanel, IDSHOW_START + i, wxEmptyString );
        }

        m_outputtypes[i] = new wxComboBox( scpanel, IDCOMBO_START + i, str_disabled_value,
            wxDefaultPosition, wxDefaultSize, m_outputtypestrings, wxCB_READONLY | wxCB_SORT );

        m_HexIds[i] = new wxTextCtrl( scpanel, IDHEXID_START + i, _T(""),
            wxDefaultPosition, wxSize( 16, 18 ), wxTE_READONLY );

        m_conflictBMP[i] = new wxStaticBitmap( scpanel, wxID_ANY, wxNullBitmap );

        m_HexIds[i]->Show( false );

        m_inverted[i] = new wxCheckBox( scpanel, IDINV_START + i, wxEmptyString );

        wxString label = ((dbsymbol*)(m_dbsymbols[i]))->fullname;

        wxString symName = ((dbsymbol*)(m_dbsymbols[i]))->name;

        m_labels[i] = new wxStaticText( scpanel, wxID_ANY, label );

        if ( m_symbolsToMark.Index( symName ) != wxNOT_FOUND )
        {
            m_labels[i]->SetForegroundColour( *wxRED );
        }

        int b_id = wxNewId();
        m_testButtons[i] = new wxToggleButton( scpanel, b_id, _T("OFF"),
            wxDefaultPosition, wxSize(40,18) );
        m_testButtons[i]->Enable( false );
        m_testBIds.Add( b_id );

        int c_id = wxNewId();
        m_testCheckBoxes[i] = new wxCheckBox( scpanel, c_id, _T("") );
        m_cboxIds.Add( c_id );

        m_testButtons[i]->Show( false );
        m_testCheckBoxes[i]->Show( false );

        //---

        sizer2->Add( m_showButtons[i], 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 3 );
        sizer2->Add( m_labels[i], 0, wxEXPAND | wxALL, 3 );

        sizer2->Add( m_outputtypes[i], 0,  wxALL, 3 );
        sizer2->Add( m_conflictBMP[i], 0, wxALL, 3 );

        sizer2->Add( m_inverted[i], 0,  wxALL | wxALIGN_CENTER_HORIZONTAL, 3 );
        sizer2->Add( m_HexIds[i], 0, wxEXPAND | wxALL, 3 );

        sizer2->Add( m_testButtons[i], 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 3 );
        sizer2->Add( m_testCheckBoxes[i], 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 3 );

        scpanel->Connect( IDCOMBO_START + i, wxEVT_COMMAND_COMBOBOX_SELECTED ,
                wxCommandEventHandler(OutputChannelsCfg::onComboBox), 0, this );
        scpanel->Connect( IDSHOW_START + i, wxEVT_COMMAND_RADIOBUTTON_SELECTED,
                wxCommandEventHandler(OutputChannelsCfg::onRadioButton), 0, this );
        scpanel->Connect( IDINV_START + i, wxEVT_COMMAND_CHECKBOX_CLICKED,
                wxCommandEventHandler(OutputChannelsCfg::onCheckbox), 0, this );

        scpanel->Connect( b_id, wxEVT_COMMAND_TOGGLEBUTTON_CLICKED,
                wxCommandEventHandler( OutputChannelsCfg::onTestButton ), 0, this );
        scpanel->Connect( c_id, wxEVT_COMMAND_CHECKBOX_CLICKED,
                wxCommandEventHandler( OutputChannelsCfg::onTestCheckBox ), 0, this );

        m_oldselections.Add( str_disabled_value );
    }

    //sizer2->AddGrowableCol( 2 );

    scpanel->SetSizer( sizer2 );
    sizer2->Fit( scpanel );
    sizer2->SetSizeHints( scpanel );

    sizer->Add( scpanel, 1, wxEXPAND|wxALL, 5 );

    scpanel->SetScrollRate( 5, 10 );

    wxBoxSizer* ecosizer = new wxBoxSizer( wxHORIZONTAL );
    wxStaticBoxSizer *b18sizer = new wxStaticBoxSizer( wxHORIZONTAL, this, _T("Econoseal 18 pinout") );
    wxStaticBoxSizer *b36sizer = new wxStaticBoxSizer( wxHORIZONTAL, this, _T("Econoseal 36 pinout") );

    m_p36 = new EconosealPanel( this, 36 );
    m_p18 = new EconosealPanel( this, 18 );

    b18sizer->Add( m_p18, 0, wxEXPAND | wxALL, 10);
    b36sizer->Add( m_p36, 0, wxEXPAND | wxALL, 10 );

    ecosizer->Add( b18sizer, 0, wxEXPAND | wxALL, 10 );
    ecosizer->Add( b36sizer, 0, wxEXPAND | wxALL, 10 );

    sizer->Add( ecosizer, 0, wxEXPAND | wxALIGN_CENTER | wxALIGN_CENTER_HORIZONTAL, 0 );

    m_statcpanel = new StaticControlPanel( this, m_statvcontroller );

    sizer->Add( m_statcpanel, 0, wxEXPAND | wxALL, 10 );

    this->SetSizer( sizer );
    sizer->Fit( this );
    sizer->SetSizeHints( this );
}

void OutputChannelsCfg::onTestButton( wxCommandEvent& event )
{
    int id = event.GetId();

    int idx = m_testBIds.Index( id );
    if ( idx == wxNOT_FOUND || idx < 0 || idx >= (int)m_num_combos )
    {
        event.Skip();
        return;
    }

    uint8_t hexValue =  (uint8_t) (m_oldHexvalues[idx] & 255);

    if ( m_testButtons[idx]->GetValue() ){
        m_testButtons[idx]->SetLabel( _T("ON") );
        hexValue += 0x80;
    }
    else {
        m_testButtons[idx]->SetLabel( _T("OFF") );
    }
    wxString command = wxString::Format( _T("Manmdh%02xbye"), hexValue);

    Logger::logf(LOG_MSG, _T("Test button %d pressed"), idx );
    Logger::logf(LOG_MSG, _T("Test command: ")+ command );

    VemsCommand* vemsCommand = new StringCommand(command);
    m_requestMgr->request(vemsCommand);

    if ( m_testButtons[idx]->GetValue() == false ){

}
}

void OutputChannelsCfg::onTestCheckBox( wxCommandEvent& event )
{
    int id = event.GetId();

    int idx = m_cboxIds.Index( id );

    if ( idx == wxNOT_FOUND || idx < 0 || idx >= (int)m_num_combos )
    {
        event.Skip();
        return;
    }

    if ( m_testCheckBoxes[idx]->GetValue() )
    {
        m_testButtons[idx]->Enable();
        m_oldHexvalues[idx] = Math::hex2int( m_HexIds[idx]->GetValue() );

        dbsymbol* dbsym = (dbsymbol*) m_dbsymbols[idx];
        Symbol symbol = *(*m_symbols)[dbsym->name];

        m_stDataMgr->setModelData( symbol, 0, 0, 255.0 );
        m_stDataMgr->requestWrite( symbol, 0, 0 );
/*        m_stDataMgr->updateRegisteredViewers();
        m_statcpanel->Disable();*/
    }
    else
    {
        m_testButtons[idx]->Enable( false );

        // write original value to the model
        wxString hexvalue = wxString::Format( _T("%02x"), m_oldHexvalues[idx] );
        m_HexIds[idx]->SetValue( hexvalue );

        dbsymbol* dbsym = (dbsymbol*) m_dbsymbols[idx];
        Symbol symbol = *(*m_symbols)[dbsym->name];

        m_stDataMgr->setModelData( symbol, 0, 0, (double) m_oldHexvalues[idx] );
        m_stDataMgr->requestWrite( symbol, 0, 0 );
/*        m_stDataMgr->updateRegisteredViewers();
        m_statcpanel->Disable();*/
    }
}

void OutputChannelsCfg::onClose(wxCloseEvent & evt)
{
    if ( m_testingMode->GetValue() == _T("ON") )
    {
        wxMessageBox( _T("Please before closing this dialog\nswitch off Testing mode!"), _T("Information"), wxICON_INFORMATION | wxOK );
        evt.Veto();
        return;
    }
    GetParent()->Disconnect( wxEVT_CLOSE_WINDOW, wxCloseEventHandler( OutputChannelsCfg::onClose ), 0, this );
    // onClose -> restore original values
    if ( m_oldHexvalues.GetCount() != 0 )
    {
        for ( size_t i = 0; i < m_oldHexvalues.GetCount(); ++i )
        {
            wxString hexvalue = wxString::Format( _T("%02x"), m_oldHexvalues[i] );
            m_HexIds[i]->SetValue( hexvalue );
            requestWrite();
            m_stDataMgr->updateRegisteredViewers();
        }
    }

    //
    cleanUp();

    if ( GetParent() )
        GetParent()->Close();
}

void OutputChannelsCfg::onTestingMode( wxCommandEvent& event )
{
    if ( m_testingMode->GetValue() == _T("ON") )
    {
        m_statcpanel->Disable();

        m_oldHexvalues.Clear();

        for ( size_t i = 0; i < 3; ++i )
            m_headerText[i]->Show( true );

        for ( size_t i = 0; i < m_num_combos; ++i )
        {
            // saving old hex values
            int value = Math::hex2int( m_HexIds[i]->GetValue() );
            m_oldHexvalues.Add( value );

            // show buttons
            m_testButtons[i]->Show( true );
            m_testCheckBoxes[i]->Show( true );
            m_HexIds[i]->Show( true );

            // enable button when valid value is selected
            wxString txt = m_outputtypes[i]->GetValue();
            if ( txt == str_invalid_value || txt == str_disabled_value )
            {
                m_testButtons[i]->Enable( false );
                m_testCheckBoxes[i]->Enable( false );
            }
            else
            {
                m_testButtons[i]->Enable( m_testCheckBoxes[i]->GetValue() );
                m_testCheckBoxes[i]->Enable( true );
            }

            // disable comboboxes, inverted checkbox
            m_outputtypes[i]->Enable( false );
            m_inverted[i]->Enable( false );
        }
    }
    else
    {
        m_statcpanel->Enable();
        for ( size_t i = 0; i < 3; ++i )
            m_headerText[i]->Show( false );

        for ( size_t i = 0; i < m_num_combos; ++i )
        {
            m_testButtons[i]->Show( false );
            m_testCheckBoxes[i]->Show( false );

            m_testButtons[i]->SetValue( false ); m_testButtons[i]->SetLabel( _T("OFF") );
            m_testCheckBoxes[i]->SetValue( false );
            m_HexIds[i]->Show( false );

            // disable comboboxes, inverted checkbox
            m_outputtypes[i]->Enable( true );
            m_inverted[i]->Enable( true );

            /// TODO
            /// restore original values
            wxString hexvalue = wxString::Format( _T("%02x"), m_oldHexvalues[i] );
            m_HexIds[i]->SetValue( hexvalue );
        }
        // update model!!
        requestWrite();
        m_stDataMgr->updateRegisteredViewers();

        m_oldHexvalues.Clear();
    }

    wxSizer* sizer;

    wxWindow* scPanel = m_outputtypes[0]->GetParent();
    sizer = scPanel->GetSizer();
    sizer->RecalcSizes();
    sizer->Fit(scPanel);
    sizer->SetSizeHints(scPanel);

    sizer = GetSizer();
    sizer->RecalcSizes();
    sizer->Fit( this );
    sizer->SetSizeHints( this );



    wxWindow* parent = GetParent();
    sizer = parent->GetSizer();
    sizer->RecalcSizes();
    sizer->Fit(parent);
    sizer->SetSizeHints(parent);
}

void OutputChannelsCfg::requestWrite()
{
    updateModel();

    if ( true )
    {
        for ( size_t i = 0; i < m_dbsymbols.GetCount(); ++i )
        {
            dbsymbol* dbsym = (dbsymbol*) m_dbsymbols[i];

            Symbol symbol = *(*m_symbols)[dbsym->name];
            m_stDataMgr->requestWrite(symbol, 0,0);
        }
    }
//    else
//    {
//        wxMessageBox(_T("Cannot write invalid value"), _T("Error"));
//    }
    }

void OutputChannelsCfg::updateModel()
{
    size_t i;

    //m_invalidvaluesfound = false;

    for ( i = 0; i < m_num_combos; ++i)
    {
        wxString hexvalue = m_HexIds[i]->GetValue();
        unsigned int value = Math::hex2int(hexvalue);

        dbsymbol* dbsym = (dbsymbol*) m_dbsymbols[i];
        Symbol symbol = *(*m_symbols)[dbsym->name];

        m_stDataMgr->setModelData( symbol, 0, 0, (double) value);
    }
}

void OutputChannelsCfg::showPins(int ecpanel, const wxString& str)
{
    m_p18->ClearAllPin();
    m_p36->ClearAllPin();
    wxArrayString pins = wxStringTokenize(str, _T("|"));

    if ( pins.GetCount() == 0)
    {
        return;
    }

    EconosealPanel * panel = (ecpanel == 36) ? m_p36 : m_p18;
    uint8_t i;
    for ( i = 0; i < pins.GetCount(); ++i )
    {
        wxString pinstr = pins[i]; if ( pinstr == _T("") ) continue;
        long pin_num = 0;
        if (pinstr.ToLong(&pin_num))
        {
            panel->SetPinColor(pin_num, wxColor(255,0,0));
        }
    }
}

void OutputChannelsCfg::showPins( int channel )
{
    m_p18->ClearAllPin();
    m_p36->ClearAllPin();


    //Logger::logf(LOG_MSG, _T("Channel=%d"), channel );

    if ( channel == 255 ) return;

    if ( channel >= 128 )
        channel -= 128;

    // OutputType* ot = OutputChannelSQL::getOutputType( channel );
    OutputType* ot = getOTFromCache( channel );
    if ( ot != NULL )
    {
        long pin = 0;
        ot->pin.ToLong( &pin );

        int con_type = ot->connector_type;
        if ( con_type == 18 || con_type == 36 )
        {
            EconosealPanel* panel = ( con_type == 18 ) ? m_p18 : m_p36;
            panel->SetPinColor( pin, wxColor( 255, 0, 0 ) );
        }

        delete ot;
    }
}

void OutputChannelsCfg::getInjGroupPins()
{
    uint8_t i;

    if ( m_injgrouppinsStr == wxEmptyString )
        m_injgrouppinsStr = OutputChannelSQL::getInjGroupPins();

    wxArrayString injpins = wxStringTokenize(m_injgrouppinsStr, _T("|"));
    m_injgrouppins.Clear();

    for ( i = 0; i < 8; ++i )
    {
        uint8_t val ;
        wxString symname = wxString::Format(_T("h0%d"),i);
        val= (uint8_t) m_stDataMgr->getDataFromViewerModel(*(*m_symbols)[symname]);

        wxString tmpstr = wxEmptyString;

        if (val & 128) tmpstr += ( injpins[7] + _T("|"));
        if (val & 64) tmpstr += ( injpins[6] + _T("|"));
        if (val & 32) tmpstr += ( injpins[5] + _T("|"));
        if (val & 16) tmpstr += ( injpins[4] + _T("|"));
        if (val & 8) tmpstr += ( injpins[3] + _T("|"));
        if (val & 4) tmpstr += ( injpins[2] + _T("|"));
        if (val & 2) tmpstr += ( injpins[1] + _T("|"));
        if (val & 1) tmpstr += ( injpins[0] + _T("|"));

        m_injgrouppins.Add(tmpstr);
    }
}

void OutputChannelsCfg::onRadioButton(wxCommandEvent& event)
{
    m_p18->ClearAllPin();
    m_p36->ClearAllPin();

    int id = event.GetId() - IDSHOW_START;

    int channel = Math::hex2int ( m_HexIds[id]->GetValue() );

    if ( channel == 255 || channel == -1 ) // invalid or disabled
    {
        return;
    }

    // OutputType * ot = OutputChannelSQL::getOutputType( channel );
    OutputType* ot = getOTFromCache( channel );

    if ( ot == NULL || ot->isDisabled())
    {
        event.Skip();
        return;
    }

    // injgroup
    if ( ot->getType_id() == 0 )
    {
            getInjGroupPins();
            showPins(36, m_injgrouppins[ot->getChannel_id()]);
    }
    else
    {
        showPins( channel );
    }

    delete ot;
}

void OutputChannelsCfg::onCheckbox(wxCommandEvent& event)
{
    int id = event.GetId() - IDINV_START;

    wxComboBox * combobox = m_outputtypes[id];

    int channel = m_out2channel[combobox->GetValue()];

    if ( channel == 255 || channel == -1 ) // invalid or disabled
    {
        return;
    }

    //OutputType * ot = OutputChannelSQL::getOutputType( channel );
    OutputType * ot = getOTFromCache( channel );

    if ( ot == NULL || ot->isDisabled())
    {
        event.Skip();
        return;
    }

    wxString hexvalue = m_HexIds[id]->GetValue();

    unsigned int value = Math::hex2int(hexvalue);
    value ^= 128;

    m_HexIds[id]->SetValue( wxString::Format( _T("%02x"), value ) );

    requestWrite();
    m_stDataMgr->updateRegisteredViewers();

    delete ot;
}

void OutputChannelsCfg::cleanUp()
{
    m_ignusedchannels.Clear();
    m_oldHexvalues.Clear();
}

void OutputChannelsCfg::update()
{
//    FUNCTIME
    m_invalidvaluesfound = false;
    size_t i;

    Freeze();

    // injgroup alternateCyl
    uint8_t alt_cyl = 255;

    /////////////////////////////////
    /// new code

    wxArrayString* arr = new wxArrayString[m_num_combos];

    for ( size_t i = 0; i < m_num_combos; ++i )
        arr[i] = m_outputtypestrings;

    for (i = 0; i< m_dbsymbols.GetCount(); ++i)
    {
        dbsymbol* sym = (dbsymbol*)m_dbsymbols[i];
        uint8_t val = (uint8_t) m_stDataMgr->getDataFromViewerModel(*(*m_symbols)[sym->name]);

        // is inverted?
        bool inverted = ((val & 128) == 128) ? true : false;
        m_inverted[i]->SetValue(inverted);
        m_HexIds[i]->SetValue(wxString::Format(_T("%02x"), val));
    }

    // reset combobox list
    for ( size_t i = 0; i < m_num_combos; ++i )
    {
        m_outputtypes[i]->Clear();
        m_isconflicted[i] = false;
        //m_outputtypes[i]->Append( arr );
    }

    ///
    SymbolTable::iterator s_it;
    ///

    // iac_type == stepper?
    IntSet iac_stepper_used;

    s_it = m_symbols->find( _T("iac_type") );
    if ( s_it != m_symbols->end() )
    {
        int val = (uint8_t) m_stDataMgr->getDataFromViewerModel( *(s_it->second), 0, 0 );
        if ( val == 3 ) // this is stepper
        {
            for ( size_t i = 1; i <= 4; ++i )
            {
                s_it = m_symbols->find( wxString::Format( _T("iac_step_seq_%d"), i ) );
                if ( s_it != m_symbols->end() )
                {
                    uint8_t stepval = (uint8_t) m_stDataMgr->getDataFromViewerModel( *(s_it->second), 0, 0 );
                    // A => 36, B=46, C=56, D=66, E=76
                    stepval = ( stepval + 3 ) * 16 + 6;
                    iac_stepper_used.insert( stepval );
                }
            }
        }
    }

    // is dual solenoid mode set?
    int dual_solenoid_channel_out = -1;
    int dual_solenoid_channels[2] = { -1, -1 };
    s_it = m_symbols->find( _T("iac_dualsol") );
    if ( s_it != m_symbols->end() )
    {
        int val = (uint8_t)m_stDataMgr->getDataFromViewerModel( *(s_it->second), 0, 0 );
        if ( val == 1 )
        {
            s_it = m_symbols->find( _T("iac_sol_channel_names") );
            dual_solenoid_channel_out = (uint8_t)m_stDataMgr->getDataFromViewerModel( *(s_it->second), 0, 0 );

            if ( dual_solenoid_channel_out != 0x0f ) // 0x0f is disabled
            {
                s_it = m_symbols->find( _T("iac_sol_channel_bits_dual") );
                int dualval = (uint8_t)m_stDataMgr->getDataFromViewerModel( *(s_it->second), 0, 0 );

                if ( dualval < 7 )
                {
                    //0,i1 ; 1,i2..., 6,i7
                    dual_solenoid_channels[0] = 16 * dualval + dual_solenoid_channel_out;
                    dual_solenoid_channels[1] = 16 * ( dualval + 1 ) + dual_solenoid_channel_out + 128; // inverted
                }
                else
                {
                    // i0,1; i1,2... i6,7
                    dualval -= 7;
                    dual_solenoid_channels[0] = 16 * dualval + dual_solenoid_channel_out + 128; // inverted
                    dual_solenoid_channels[1] = 16 * ( dualval + 1) + dual_solenoid_channel_out;
                }

                Logger::logf(LOG_MSG, _T("dualsolchan=%d, val1=%02x, val2=%02x"), dual_solenoid_channel_out,
                    dual_solenoid_channels[0], dual_solenoid_channels[1]);
            }
        }
    }
    // end of dual solenoid mode.

    IntSet::iterator intset_it;

    for ( size_t i = 0; i < m_num_combos; ++i )
    {

        m_outputtypes[i]->SetForegroundColour(wxSystemSettings::GetColour( wxSYS_COLOUR_WINDOWTEXT ));

        // iac_stepper?
        if ( iac_stepper_used.size() > 0 )
        {
            int cval = Math::hex2int( m_HexIds[i]->GetValue() );

            if ( cval > 127 )
                cval -= 128;


            intset_it = iac_stepper_used.begin();

            for ( ; intset_it != iac_stepper_used.end(); ++intset_it )
            {
                uint8_t ival = *intset_it;

                if ( ival == cval )
                {
                    m_isconflicted[i] = true;
                }
                else
                    m_isconflicted[i] |= false;

                wxString str = m_channel2str[ival];
                int pos = arr[i].Index( str );
                if ( pos != wxNOT_FOUND )
                {
                    arr[i].RemoveAt( pos );
                }
            }
        }


        // dual solenoid
        if ( dual_solenoid_channels[0] != -1 && dual_solenoid_channels[1] != -1 && i != m_idleSolenoidChannelNo )
        {
            int val1 = dual_solenoid_channels[0]; if ( val1 > 127 ) val1 -= 128;
            int val2 = dual_solenoid_channels[1]; if ( val2 > 127 ) val2 -= 128;

            int cval = Math::hex2int( m_HexIds[i]->GetValue() );

            if ( cval == val1 || cval == val2 )
            {
                m_isconflicted[i] = true;
            }
            else m_isconflicted[i] |= false;

            wxString str = m_channel2str[val1];
            int pos = arr[i].Index( str );
            if ( pos != wxNOT_FOUND )
            {
                arr[i].RemoveAt( pos );
            }

            str = m_channel2str[val2];
            pos = arr[i].Index( str );
            if ( pos != wxNOT_FOUND )
            {
                arr[i].RemoveAt( pos );
            }
        }

        // injgroups
        SymbolTable::const_iterator it = m_symbols->find(_T("alternateCyl"));
        if ( it != m_symbols->end() )
        {
            alt_cyl = (uint8_t) m_stDataMgr->getDataFromViewerModel( *(*m_symbols)[_T("alternateCyl")] );
            for ( size_t j = 0; j <= alt_cyl; ++j )
            {
                wxString hexid = wxString::Format( _T("%d0"),j );
                int val1 = Math::hex2int(hexid), val2 = val1 ^ 128;

                int cval = Math::hex2int(m_HexIds[i]->GetValue());

                if ( cval == val1 || cval == val2 )
                {
                    //LOG
                    m_isconflicted[i] = true;
                    // mark as conflict & continue
//                    m_outputtypes[i]->SetForegroundColour( wxColor( 255, 0, 0 ) );
//                    continue;
                }
                else m_isconflicted[i] |= false;

                int channel = Math::hex2int( hexid ); if ( channel > 128 ) channel-= 128;
                wxString str = m_channel2str[channel];
                int pos = arr[i].Index( str );
                if ( pos != wxNOT_FOUND )
                {
                    arr[i].RemoveAt( pos );
                }
            }
        }

        // ignition outputs
        uint8_t ignoutputs= (uint8_t) m_stDataMgr->getDataFromViewerModel(*(*m_symbols)[_T("ignchmax")]);

        bool isDual = ( (uint8_t)m_stDataMgr->getDataFromViewerModel(*(*m_symbols)[_T("ign_out_dual")]) )!=0?true:false;

        uint8_t chId;
        uint8_t otype; // I259 or S259

        for ( size_t j = 0; j <= ignoutputs; ++j )
        {
            wxString h2str = ( isDual ) ? wxString::Format(_T("h2d%d"), j) : wxString::Format(_T("h2%d"), j);
            uint8_t pinval = (uint8_t) m_stDataMgr->getDataFromViewerModel(*(*m_symbols)[h2str]);
            if ( OutputChannelSQL::getIgnitionMapping( isDual, pinval, otype, chId) )
            {
                if ( isDual )
                {
                    int chval1 = ( 16* ( chId -1 ) + otype ), chval2 = (16 * chId + otype);
                    wxString hexid = wxString::Format(_T("%02x"), chval1 );
                    wxString hexid1 = wxString::Format(_T("%02x"), chval2 );

                    int cval1 = Math::hex2int( m_HexIds[i]->GetValue() ), cval2;
                    if ( cval1 > 128 ) cval2 = cval1 - 128;
                    else cval2 = cval1 + 128;

                    //if ( hexid == m_HexIds[i]->GetValue() || hexid1 == m_HexIds[i]->GetValue())
                    if ( cval1 == chval1 || cval1 == chval2 || cval2 == chval1 || cval2 == chval2 )
                    {
                        m_isconflicted[i] = true;
                        // mark as conflict & continue
//                        m_outputtypes[i]->SetForegroundColour( wxColor( 255, 0, 0 ) );
//                        continue;
                    }
                    else m_isconflicted[i] |= false ;
                    int cvalrem_1 = ( chval1 > 128 ) ? ( chval1 - 128 ) : chval1;
                    int cvalrem_2 = ( chval2 > 128 ) ? ( chval2 - 128 ) : chval2;
                    //wxString str = m_channel2str[chval1];
                    wxString str = m_channel2str[cvalrem_1];
                    int pos = arr[i].Index( str );
                    if ( pos != wxNOT_FOUND )
                        arr[i].RemoveAt( pos );
                    //str = m_channel2str[chval2];
                    str = m_channel2str[cvalrem_2];
                    pos = arr[i].Index( str );
                    if ( pos != wxNOT_FOUND )
                        arr[i].RemoveAt( pos );

                }
                else
                {
                    int chval = 16 * chId + otype;
                    wxString hexid = wxString::Format( _T("%02x"), chval );

                    int cval = Math::hex2int( m_HexIds[i]->GetValue() ); if ( cval > 128 ) cval -= 128;

                    //if ( hexid == m_HexIds[i]->GetValue() )
                    if ( cval == chval )
                    {
                        // mark as conflict & continue
//                        m_outputtypes[i]->SetForegroundColour( wxColor( 255, 0, 0 ) );
//                        continue;
                        m_isconflicted[i] = true;
                    }
                    else m_isconflicted[i] |= false;
                    wxString str = m_channel2str[chval];

                    int pos = arr[i].Index( str );
                    if ( pos != wxNOT_FOUND )
                        arr[i].RemoveAt( pos );
                }
            }
        }// for

        // other symbols
        for ( size_t k = 0; k < m_dbsymbols.GetCount(); ++k )
        {
            if ( i == k ) continue;
            dbsymbol* sym = (dbsymbol*)m_dbsymbols[k];
            uint8_t val = (uint8_t) m_stDataMgr->getDataFromViewerModel(*(*m_symbols)[sym->name]);
            if ( val > 128 ) val -= 128;
            int cval = Math::hex2int( m_HexIds[i]->GetValue() ); if ( cval > 128 ) cval -= 128;
            if ( cval == val )
            {
                m_isconflicted[i] = true; m_isconflicted[k] = true;
            }
        }

        // remove current values from other combos

        wxString hexid = m_HexIds[i]->GetValue();

        int channel = Math::hex2int( hexid ); if ( channel > 128 ) channel -= 128;
        wxString str = m_channel2str[channel];

        if ( str == str_disabled_value || str == str_disabled_value || str == str_conflict_value )
            ;
        else
        {
            for ( size_t j = 0; j < m_num_combos; ++j )
            {
                if ( i == j )
                    continue;
                int pos = arr[j].Index( str );
                if ( pos != wxNOT_FOUND )
                    arr[j].RemoveAt( pos );
            }
        }
    }

    m_conflictfound = false;

    size_t j;

    for ( size_t i = 0; i < m_num_combos; ++i )
    {
        for ( j = 0; j < arr[i].GetCount();++j )
        {
            m_outputtypes[i]->AppendString(arr[i][j]);
        }

        wxString hexid = m_HexIds[i]->GetValue();

        int channel = Math::hex2int( hexid );

        if ( (channel & 15) == 15 )
            m_outputtypes[i]->SetValue( str_disabled_value );
        else
        {
            wxString str = m_channel2str[channel];
            // set current sel
            m_outputtypes[i]->SetValue( str );
        }

        if ( m_outputtypes[i]->GetValue() == _T("") )
        {
            if ( m_isconflicted[i] )
            {
                m_labels[i]->SetForegroundColour( *wxRED );
                m_outputtypes[i]->SetValue( str_conflict_value );
                m_outputtypes[i]->SetForegroundColour(wxColor(255,0,0));

                m_conflictBMP[i]->SetBitmap( *m_warningbmp );
                m_conflictBMP[i]->SetToolTip( _T("Press 'Validate config' button for further details") );

                int pos = m_outputtypes[i]->FindString( str_invalid_value );

                if ( pos != wxNOT_FOUND )
                    m_outputtypes[i]->Delete( pos );

                m_conflictfound = true;
            }
            else
            {
                m_labels[i]->SetForegroundColour( *wxRED );
                m_outputtypes[i]->SetValue( str_invalid_value );
                m_invalidvaluesfound = true;
                m_outputtypes[i]->SetForegroundColour(wxColor(255,0,0));

                m_conflictBMP[i]->SetBitmap( wxNullBitmap );
                m_conflictBMP[i]->SetToolTip( _T("") );

                int pos = m_outputtypes[i]->FindString( str_conflict_value );

                if ( pos != wxNOT_FOUND )
                    m_outputtypes[i]->Delete( pos );
            }
        }
        else
        {
            // m_outputtypes[i]->SetForegroundColour(wxSystemSettings::GetColour( wxSYS_COLOUR_WINDOWTEXT ));
            int pos = m_outputtypes[i]->FindString( str_invalid_value );

            if ( pos != wxNOT_FOUND )
                m_outputtypes[i]->Delete( pos );

            pos = m_outputtypes[i]->FindString( str_conflict_value );

            if ( pos != wxNOT_FOUND )
                m_outputtypes[i]->Delete( pos );

            m_conflictBMP[i]->SetBitmap( wxNullBitmap );
            m_conflictBMP[i]->SetToolTip( _T("") );

            m_labels[i]->SetForegroundColour( wxSystemSettings::GetColour( wxSYS_COLOUR_WINDOWTEXT ) );
        }

        // show pins
        if ( m_showButtons[i]->GetValue() )
        {
            int val = Math::hex2int( hexid );
            // OutputType* ot = OutputChannelSQL::getOutputType( val );
            OutputType* ot = getOTFromCache( channel );
            if ( ot == NULL || ot->isDisabled()) continue;
            if (ot->getType_id() == 0) // injfet
            {
                getInjGroupPins();
                showPins( 36, m_injgrouppins[ot->getChannel_id()] );
            }
            else
            {
                showPins( val );
            }
            delete ot;
        }
    }

    Thaw();
}

void OutputChannelsCfg::onComboBox( wxCommandEvent& event )
{
//    FUNCTIME

    unsigned int id = event.GetId() - IDCOMBO_START;
    wxComboBox * combobox = m_outputtypes[id];

    wxString selvalue = combobox->GetValue();

    if ( m_oldselections[id] != str_disabled_value && m_oldselections[id] != str_invalid_value
        && m_oldselections[id] != str_conflict_value )
        for ( size_t i = 0; i < m_num_combos; ++i )
        {
            if ( i == id ) continue;
            else if ( m_outputtypes[i]->FindString( m_oldselections[i]) == wxNOT_FOUND )
                m_outputtypes[i]->Append( m_oldselections[id] );
        }

    // remove INVALID from list if exists
    int tmp = combobox->FindString(str_invalid_value);
    if ( tmp != wxNOT_FOUND )
        combobox->Delete( tmp );

    // remove CONFLICT from list if exists
    tmp = combobox->FindString(str_conflict_value);
    if ( tmp != wxNOT_FOUND )
        combobox->Delete( tmp );

    // set selection outputtype to m_outcombobox
    int val = m_out2channel[combobox->GetValue()];
    //OutputType* ot = OutputChannelSQL::getOutputType( val );
    OutputType* ot = getOTFromCache( val );

    m_HexIds[id]->SetLabel(wxString::Format(_T("%02x"), ot->channel));
    //m_HexIds[id]->SetValue(wxString::Format(_T("%02x"), ot->channel));

    m_inverted[id]->SetValue(false);

    //m_oldselections[id] = combobox->GetValue();

    if ( combobox->GetForegroundColour() == wxColour(255,0,0))
        combobox->SetForegroundColour(wxSystemSettings::GetColour( wxSYS_COLOUR_WINDOWTEXT ));

    if (m_showButtons[id]->GetValue())
    {
        if (ot->getType_id() == 0) // injfet
        {
            getInjGroupPins();
            showPins(36, m_injgrouppins[ot->getChannel_id()]);
        }
        else
        {
            showPins( val );
        }
    }

    delete ot;

    requestWrite();
    m_stDataMgr->updateRegisteredViewers();
//    Logger::logf(LOG_MSG, _T("Selected %d, count=%d"),m_outputtypes[id]->GetSelection(),
//        m_outputtypes[id]->GetCount());

    // combobox refresh in update()
    event.Skip();
}

void OutputChannelsCfg::SwitchOffAllTests()
{
//    for ( int i = 0; i < m_testBIds.GetCount(); ++i )
//    {
//        int bId = m_testBIds[i];
//    }
}

/// deprecated code
/*
// not needed due to 'new code'

void OutputChannelsCfg::refreshCombos(unsigned short comboid)
{
    size_t i;
    wxString hexid = m_HexIds[comboid]->GetValue();
    wxString value = m_outputtypes[comboid]->GetValue();
    if (m_showButtons[comboid]->GetValue())
    {
        m_p18->ClearAllPin(); m_p36->ClearAllPin();
    }

    if ( value == str_invalid_value ) return;
    if ( value == str_disabled_value )
    {
        // TODO check for conflicts?
        if ( m_outputtypes[comboid]->GetForegroundColour() == wxColor(255,0,0) )
            m_outputtypes[comboid]->SetForegroundColour(wxSystemSettings::GetColour( wxSYS_COLOUR_WINDOWTEXT ));
        return;
    }
    value = m_outputtypestrings[m_hex2indexHash[hexid]];

    bool hasConflict = false;

    for ( i = 0; i < m_num_combos; ++i )
    {
        if ( comboid == i ) continue;
        int p = m_outputtypes[i]->FindString( value );

        hasConflict |= checkConflict(comboid, i);

        if ( p != wxNOT_FOUND && checkConflict(comboid, i) == false )
        {
            m_outputtypes[i]->Delete(p);
            continue;
        }
        if ( checkConflict(comboid, i) )
        {
            if ( m_outputtypes[i]->FindString(value) == wxNOT_FOUND )
                m_outputtypes[i]->Append(value);
            m_oldselections[i] = value;
            m_outputtypes[i]->SetValue(value);
            m_outputtypes[i]->SetForegroundColour(wxColor(255,0,0));
        }
    }

    // if there exists conflict -> set the foreground color of the combobox to red!
    if ( hasConflict )
    {
        m_outputtypes[comboid]->SetForegroundColour(wxColor(255,0,0));
    }
    else
    {
        if ( m_outputtypes[comboid]->GetForegroundColour() == wxColor(255,0,0) )
            m_outputtypes[comboid]->SetForegroundColour(wxSystemSettings::GetColour( wxSYS_COLOUR_WINDOWTEXT ));
    }

    // show pins
    if (m_showButtons[comboid]->GetValue())
    {
        int val = Math::hex2int( hexid );
        OutputType* ot = OutputChannelSQL::getOutputType( val );
        if ( ot == NULL || ot->isDisabled()) return;
        if (ot->getType_id() == 0) // injfet
        {
            getInjGroupPins();
            showPins( 36, m_injgrouppins[ot->getChannel_id()] );
        }
        else
        {
            showPins( val );
        }
        delete ot;
    }
}

*/

/*
bool OutputChannelsCfg::checkConflict(unsigned short id1, unsigned short id2)
{
    uint8_t val1 = Math::hex2int(m_HexIds[id1]->GetValue()), val2 = Math::hex2int(m_HexIds[id2]->GetValue());

    if ((( val1 & 15) == 15) || ((val2 & 15) == 15)) // one of them is disabled
        return false;

    if ( val1 > 127) val1 -= 128; if ( val2 > 127 ) val2 -= 128;
    return (val1 == val2);
}
*/

/*
void OutputChannelsCfg::checkIgnitionOutputPins()
{
    uint8_t ignoutputs= (uint8_t) m_stDataMgr->getDataFromViewerModel(*(*m_symbols)[_T("ignchmax")]);

    bool isDual = (bool)( (uint8_t)m_stDataMgr->getDataFromViewerModel(*(*m_symbols)[_T("ign_out_dual")]) );
//    Logger::logf(LOG_MSG, _T("isDual = %d"), isDual);

    uint8_t chId;
    uint8_t otype; // I259 or S259

    /// ADD
    int i = m_ignusedchannels.GetCount() - 1;
    //for ( int i = m_ignusedchannels.GetCount()-1; i >= 0; --i )
    while ( i >= 0 )
    {
        wxString storedHexId = m_ignusedchannels[i];
        bool used = false;
        for ( size_t j = 0; j <= ignoutputs; ++j )
        {
            wxString h2str = ( isDual ) ? wxString::Format(_T("h2d%d"), j) : wxString::Format(_T("h2%d"), j);
            uint8_t pinval = (uint8_t) m_stDataMgr->getDataFromViewerModel(*(*m_symbols)[h2str]);
            if ( OutputChannelSQL::getIgnitionMapping( isDual, pinval, otype, chId) )
            {
                wxString hexid = wxString::Format(_T("%02x"), (16 * chId + otype));

                if ( ! isDual )
                {
                    if ( hexid == storedHexId )
                    {
                        used = true; break;
                    }
                }

                else
                {
                    wxString hexid1 = wxString::Format(_T("%02x"), (16 * ( chId - 1) + otype));
                    if ( hexid == storedHexId || hexid1 == storedHexId )
                    {
                        used = true; break;
                    }

                }
            }
        }

        // remove from ign.outp.channels and make this output selectable in comboboxes
        if ( ! used )
        {
            if ( isDual == false )
            {
                m_ignusedchannels.RemoveAt(i);
                wxString addS = m_outputtypestrings[m_hex2indexHash[storedHexId]];
                for ( size_t m = 0; m < m_dbsymbols.GetCount(); ++m )
                    if ( m_outputtypes[m]->FindString(addS) == wxNOT_FOUND )
                        m_outputtypes[m]->Append( addS );
            }
            else
            {
                wxString hId, hId1;
                hId = m_ignusedchannels[i]; if ( i > 0 ) hId1 = m_ignusedchannels[i-1];
                m_ignusedchannels.RemoveAt(i);
                if ( i > 0 )
                    m_ignusedchannels.RemoveAt(i-1);
                wxString addS, addS1;
                addS = m_outputtypestrings[m_hex2indexHash[hId]];
                if ( i> 0)
                    addS1 = m_outputtypestrings[m_hex2indexHash[hId1]];
                for ( size_t m = 0; m < m_dbsymbols.GetCount(); ++m )
                {
                    if ( m_outputtypes[m]->FindString(addS) == wxNOT_FOUND )
                        m_outputtypes[m]->Append( addS );
                    if ( i > 0 && ( m_outputtypes[m]->FindString(addS1) == wxNOT_FOUND ) )
                        m_outputtypes[m]->Append( addS1 );
                }
                --i;
            }
        }
        --i;
    }

    /// REMOVE
    // nondual case
    if ( isDual == false )
    {
        for ( size_t i = 0; i <= ignoutputs; ++i )
        {
            wxString h2str = wxString::Format(_T("h2%d"), i);
            uint8_t pinval = (uint8_t) m_stDataMgr->getDataFromViewerModel(*(*m_symbols)[h2str]);

            if ( OutputChannelSQL::getIgnitionMapping( false, pinval, otype, chId) )
            {
                wxString hexid = wxString::Format(_T("%02x"), (16 * chId + otype));

                for ( size_t m = 0; m < m_dbsymbols.GetCount(); ++m )
                {
                    wxComboBox * combo = m_outputtypes[m];
                    int pos = combo->FindString(m_outputtypestrings[m_hex2indexHash[hexid]]);
                    if ( pos != wxNOT_FOUND)
                        combo->Delete(pos);
                }

                if ( m_ignusedchannels.Index( hexid ) == wxNOT_FOUND )
                    m_ignusedchannels.Add( hexid );
            }
        }
    }
    // dual case
    else
    {
        for ( size_t i = 0; i <= ignoutputs; ++i )
        {
            wxString h2str = wxString::Format(_T("h2d%d"), i);
            uint8_t pinval = (uint8_t) m_stDataMgr->getDataFromViewerModel(*(*m_symbols)[h2str]);

            if ( OutputChannelSQL::getIgnitionMapping( true, pinval, otype, chId) )
            {
                wxString hexid = wxString::Format(_T("%02x"), (16 * ( chId - 1 ) + otype));
                wxString hexid1 = wxString::Format(_T("%02x"), (16 * chId + otype));

                for ( size_t m = 0; m < m_dbsymbols.GetCount(); ++m )
                {
                    wxComboBox * combo = m_outputtypes[m];
                    int pos = combo->FindString(m_outputtypestrings[m_hex2indexHash[hexid]]);
                    if ( pos != wxNOT_FOUND)
                        combo->Delete(pos);

                    pos = combo->FindString(m_outputtypestrings[m_hex2indexHash[hexid1]]);
                    if ( pos != wxNOT_FOUND)
                        combo->Delete(pos);
                }

                m_ignusedchannels.Add( hexid );
                m_ignusedchannels.Add( hexid1 );
            }
        }
    }
}
*/

/*

bool OutputChannelsCfg::isValidType(int channel)
{
    if ( channel < 0 || channel > 255 )
        return false;

    if ( channel > 127 ) // inverted
        channel -= 128;

    if ( ( channel & 0x0F ) == 0x0F ) //disabled
        return true;

    if ( m_channels.Index(channel) != wxNOT_FOUND )
        return true;
    else
        return false;
}

*/

/*
void OutputChannelsCfg::update()
{
    size_t i;

    Freeze();

    // injgroup alternateCyl
    uint8_t alt_cyl = 255;

    /// OLD CODE

    SymbolTable::const_iterator it = m_symbols->find(_T("alternateCyl"));

    if ( it != m_symbols->end() )
    {
        alt_cyl = (uint8_t) m_stDataMgr->getDataFromViewerModel( *(*m_symbols)[_T("alternateCyl")] );

        for ( i = 0; i < m_dbsymbols.GetCount(); ++i )
        {
            wxComboBox * combo = m_outputtypes[i];

            int j;

            for ( j = 0; j <= alt_cyl; ++j )
            {
                wxString hexid = wxString::Format( _T("%d0"),j );
                int pos = combo->FindString( m_outputtypestrings[m_hex2indexHash[hexid]] );

                if ( pos != wxNOT_FOUND )
                    combo->Delete(pos);
            }

            for ( j = alt_cyl+1; j <= 7; ++j )
            {
                wxString hexid = wxString::Format( _T("%d0"),j );
                int pos = combo->FindString(m_outputtypestrings[m_hex2indexHash[hexid]]);
                if ( pos == wxNOT_FOUND)
                    combo->Append(m_outputtypestrings[m_hex2indexHash[hexid]]);
            }
        }
    }

    /// ignition output collision detect

    checkIgnitionOutputPins();

    ///


    for (i = 0; i< m_dbsymbols.GetCount(); ++i)
    {
        dbsymbol* sym = (dbsymbol*)m_dbsymbols[i];
        uint8_t val = (uint8_t) m_stDataMgr->getDataFromViewerModel(*(*m_symbols)[sym->name]);

        // is inverted?
        bool inverted = ((val & 128) == 128) ? true : false;
        m_inverted[i]->SetValue(inverted);
        m_HexIds[i]->SetLabel(wxString::Format(_T("%02x"), val));

        wxComboBox * combobox = m_outputtypes[i];
        OutputType * ot = OutputChannelSQL::getOutputType(val);

        // invalid?
        if ( ot == NULL || isValidType (ot->channel) == false )
        {
            combobox->SetForegroundColour(wxColor(255,0,0));
            if (combobox->FindString(str_invalid_value) == wxNOT_FOUND)
                combobox->Append(str_invalid_value);
            combobox->SetValue(str_invalid_value);
            m_oldselections[i] = combobox->GetValue();
            if ( ot != NULL ) { delete ot; ot = NULL; }
            continue;
        }

        int p = m_outputtypes[i]->FindString(str_invalid_value);
        if ( p != wxNOT_FOUND ) m_outputtypes[i]->Delete(p);

        // disabled
        if (ot != NULL && ot->isDisabled())
        {
            combobox->SetValue(str_disabled_value);
            for ( size_t j = 0; j < m_num_combos; ++j)
            {
                if ( m_outputtypes[j]->FindString(m_oldselections[i]) == wxNOT_FOUND)
                    m_outputtypes[j]->Append(m_oldselections[i]);
            }

            m_oldselections[i] = combobox->GetValue();
            if ( ot != NULL ) { delete ot; ot = NULL; }
            continue;
        }

        // if alt_cyl & injgroup
        if ( alt_cyl != 255 && ot->getType_id() == 0)
        {
            if ( ot->getChannel_id() <= alt_cyl)
            {
                // set it to disabled
                m_inverted[i]->SetValue(true); m_HexIds[i]->SetLabel(_T("ff"));
            }
        }

        delete ot; ot = NULL;
        int id = m_hex2indexHash[m_HexIds[i]->GetValue()];
        combobox->SetValue( m_outputtypestrings[id] );

        for ( size_t j = 0; j < m_num_combos; ++j)
        {
            if ( m_outputtypes[j]->FindString(m_oldselections[i]) == wxNOT_FOUND )
                m_outputtypes[j]->Append( m_oldselections[i] );
        }

        m_oldselections[i] = combobox->GetValue();
    }

    for ( i = 0; i < m_num_combos; ++i )
    {
        refreshCombos(i);
    }

    Thaw();
}
*/
