#include "SDCardDialog.h"

#include <wx/statline.h>

#include "../stdatamanager/STDataModelBase.h"
#include "../vemsprotocol/SIPRFileLogger.h"
#include "../vemsprotocol/SIPRPacket.h"
#include "../vemsprotocol/SIPRTypes.h"
#include "../vemsprotocol/STDataModelWriter.h"
#include "../vemsprotocol/VemsDeviceControllerBase.h"
#include "../vemsprotocol/SDCardController.h"
#include "../common/types/DoubleHash.h"
#include "../DirHelper.h"
#include "../FirmwareInfo.h"
#include "../RawLogHelper.h"
#include "../logger.h"
#include "../mtconfigdata.h"
#include "../UserLog.h"
#include "../SDImageHandler.h"

#include "../customwidgets/InfoWindow.h"
#include "../configmanager/ConfigHelper2.h"
#include "../ConfigBase.h"
#include "../communication/clients/SDCardFileHelper.h"
#include "../triggerlog/TriggerFrameFSM.h"
#include "../triggerlog/TriggerFrame.h"
#include "../common/VemsContext.h"
/**

    Search panel

**/
SDCardSearchPanel::SDCardSearchPanel( wxWindow* parent )
{
    // current state: not playing
    m_state = 0;

    // create GUI
    Create( parent );
    wxSizer* sizer = new wxStaticBoxSizer( wxVERTICAL, this, _T("Step 1.") );

    wxString searchText = _T("Searching files, please wait!\nIf the progress bar is not shown in a few seconds,\nclose this dialog and try again!");

    m_searchTxt = new wxStaticText( this, -1, searchText );
    wxFont fnt = m_searchTxt->GetFont();
    fnt.SetWeight( wxFONTWEIGHT_BOLD );
    m_searchTxt->SetFont( fnt );

    wxAnimation anim;
	anim.LoadFile( DirHelper::GetPath( dGRAPHICS ) + _T("progress.gif"), wxANIMATION_TYPE_GIF );

    // when animation is not playing: background image is a solid rectangle
    wxColor c = wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE );
    wxImage img(400,20); img.SetRGB( wxRect(0,0,400,20), c.Red(), c.Green(), c.Blue()); wxBitmap nbmp(img);


    m_anim = new wxAnimationCtrl( this, wxID_ANY, anim, wxDefaultPosition, wxSize( 400, 20 ) );
    m_anim->SetInactiveBitmap( nbmp );

    sizer->Add( m_searchTxt, 0, wxALL, 5 );
    sizer->Add( m_anim, 0, (wxALL & ~wxTOP) | wxEXPAND | wxALIGN_CENTER, 5 );

    SetSizer( sizer );
    sizer->Fit( this );
    sizer->SetSizeHints( this );
}

void SDCardSearchPanel::setState( unsigned int i, int cardstate )
{
    m_state = i;
    switch ( i )
    {
        // searching, start animation
        case 1: m_searchTxt->SetLabel(_T("Searching...") );
            m_anim->Play(); break;

        // searching is finished, stop the animation
        // display result message
        case 2:
        {
            wxSizer* sizer = GetSizer();

            m_anim->Stop(); sizer->Detach( m_anim );
            sizer->RecalcSizes();
            wxSizer* resultS = new wxBoxSizer( wxHORIZONTAL );

            wxString bmpN;
            if ( cardstate == 0 ) bmpN = _T("ok");
            else bmpN = _T("stop");

            bmpN = DirHelper::GetPath( dICON ) + _T("ul-") + bmpN + _T("-m.png");

            wxBitmap bmp( bmpN, wxBITMAP_TYPE_PNG );
            wxStaticBitmap* sb = new wxStaticBitmap( this, wxID_ANY, bmp );

            resultS->Add( sb, 0, wxALL, 5 );

            wxString txt;
            switch ( cardstate )
            {
                case SDCardFile::SDFS_OK: txt = _T("Done, please select logs from below"); break;
                case SDCardFile::SDFS_EMPTY: txt = _T("Card seems to be empty"); break;
                case SDCardFile::SDFS_NOTVALIDFS: txt = _T("Card does not contain valid VEMS data"); break;
                default: txt = _T("Unknown error");
            }

            wxStaticText* st = new wxStaticText( this, wxID_ANY, txt, wxDefaultPosition, wxSize( 300, 30 ) );
            resultS->Add( st, 0, wxALL | wxEXPAND, 5 );
            //resultS->RecalcSizes();
            resultS->Fit( st );
            resultS->SetSizeHints( st );

            sizer->Add( resultS, 0, wxALL | wxEXPAND, 5 );

            // if error occured -> display a close button
            if ( cardstate != 0 )
            {
                wxButton* closeButton = new wxButton( this, wxID_ANY, _T("Close") );
                Connect( closeButton->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( SDCardDialog::OnCloseButton ),
                    NULL, GetParent() );
                sizer->Add( closeButton, 0, wxALL | wxALIGN_CENTRE_HORIZONTAL, 3 );
            }

//            sizer->RecalcSizes();
            sizer->Fit( this );
            sizer->SetSizeHints( this );
        }
        break;

        default:  break;
    }
}

/**

    Download panel

**/

BEGIN_EVENT_TABLE( SDCardDownloadPanel, wxPanel )
    EVT_BUTTON( idSelectAllButton, SDCardDownloadPanel::OnButton )
END_EVENT_TABLE()

SDCardDownloadPanel::SDCardDownloadPanel( wxWindow* parent )
{
    m_isAllSelected = false;

    buildGUI( parent );
}

void SDCardDownloadPanel::UpdateFiles( const MMCFilesDescA& files )
{
    // add files to the checklistbox
    Freeze();

    for ( size_t i = 0; i < files.GetCount(); ++i )
    {
        MMCFilesDesc& desc = files[i];

        uint32_t time = desc.Time;

        uint32_t min = time / 60;
        uint32_t sec = time % 60;

        wxString apprS = _T("");
        if ( desc.isApprox )
        {
            apprS = _T("~");
        }

        // file type (raw,bytestuffed)
        wxString typeS = wxEmptyString;
        #ifdef VEMSDEBUG
        typeS = wxString::Format( _T("[%d]"), desc.DataType );
        #endif

        // i. mm:ss
        wxString row =  wxString::Format( _T("%d. "), i + 1 );
        row += apprS;
        row += wxString::Format( _T("%02d:%02d"), min, sec );
        row += typeS;

        m_fileListBox->Append( row );
    }

    Thaw();
}

void SDCardDownloadPanel::buildGUI( wxWindow* parent )
{
    Create( parent );

    m_isAllSelected = false;

    m_summaryGrid = new wxGrid( this, idSummaryGrid );
    m_summaryGrid->CreateGrid( 0, 2 );

    m_fileListBox = new wxCheckListBox( this, idFileListGrid, wxDefaultPosition, wxSize( 200, 165 ) );

    m_summaryGrid->DisableDragRowSize();
    m_summaryGrid->SetColLabelValue( 0, _T("Prop.") );
    m_summaryGrid->SetColLabelValue( 1, _T("Value") );
    m_summaryGrid->EnableEditing( false );

    wxStaticLine* hline = new wxStaticLine( this, wxID_ANY );

    m_saveselbutton = new wxButton( this, idSaveButton, _T("Save checked files") );
    Connect( idSaveButton, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(SDCardDialog::OnSaveSelected), NULL, parent);

    m_selallbutton = new wxButton( this, idSelectAllButton, _T("Check/uncheck all") );

    m_summbutt = new wxButton( this, idGetSummaryButt, _T("Create summary") );
    Connect( idGetSummaryButt, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( SDCardDialog::OnCreateSummary ), NULL, parent );

    m_summaryProgress = new ProgressBar( this, wxID_ANY, wxDefaultPosition, wxSize( -1, 20 ) );

    wxStaticText* samplecnttxt = new wxStaticText( this, wxID_ANY, _T("Sample count") );
    wxFont fnt = samplecnttxt->GetFont();
    fnt.SetWeight( wxFONTWEIGHT_BOLD );
    samplecnttxt->SetFont( fnt );

    m_sampleCntCtrl = new wxTextCtrl( this, wxID_ANY, _T("5") );

    wxTextValidator validator( wxFILTER_NUMERIC );
    m_sampleCntCtrl->SetValidator( validator );
    m_sampleCntCtrl->SetMaxLength( 3 );

    wxBoxSizer *mainsizer = new wxStaticBoxSizer( wxVERTICAL, this, _T("Step 2. - Select files & save") );

    // grid level, files, summary
    wxSizer* gs = new wxBoxSizer( wxHORIZONTAL );
    gs->Add( m_fileListBox, 0, wxALL, 5 );

    wxSizer* ss = new wxBoxSizer( wxVERTICAL );
    ss->Add( samplecnttxt, 0, wxBOTTOM | wxLEFT | wxRIGHT | wxEXPAND, 3 );
    ss->Add( m_sampleCntCtrl, 0, wxALL | wxEXPAND, 3 );
    ss->Add( m_summbutt, 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 3 );
    ss->Add( m_summaryProgress, 0, wxALL | wxEXPAND, 3 );
    ss->Add( hline, 0, wxALL | wxEXPAND, 3 );
    ss->Add( m_selallbutton, 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 3 );
    ss->Add( m_saveselbutton, 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 3 );

    gs->Add( ss, 1, wxALL | wxEXPAND, 5 );
    gs->Add( m_summaryGrid, 0, wxALL | wxALIGN_RIGHT, 5 );

    mainsizer->Add( gs, 0, wxALL | wxEXPAND, 5 );

    m_summaryProgress->SetWindowStyle( wxBORDER_SUNKEN );
    m_summaryProgress->SetProgressStyle( PB_PERCENTTEXT_INNER );
    m_summaryProgress->Update( 0 );

    // saving
    wxSizer* sboxs = new wxStaticBoxSizer( wxVERTICAL, this, _T("Saving progress") );
    m_logProgress = new ProgressBar( this, wxID_ANY, wxDefaultPosition, wxSize( -1, 20 ) );
    m_overallProgress = new ProgressBar( this, wxID_ANY, wxDefaultPosition, wxSize( -1, 20 ) );

    wxButton* cancel = new wxButton( this, idCancelButton, _T("Cancel") );
    m_closeButton = new wxButton( this, idMMCCloseButton, _T("Close") );
    Connect( idCancelButton, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( SDCardDialog::OnCancelButton ), NULL, parent );
    Connect( idMMCCloseButton, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( SDCardDialog::OnCloseButton ), NULL, parent );

    wxStaticText* txt, *txt2;
    txt = new wxStaticText( this, wxID_ANY, _T("Current log progress:") );
    txt2 = new wxStaticText( this, wxID_ANY, _T("Overall progress") );

    sboxs->Add( txt, 0, wxEXPAND | wxALL, 5 );
    sboxs->Add( m_logProgress, 0, wxEXPAND | wxALL, 5 );
    sboxs->Add( txt2, 0, wxEXPAND | wxALL, 5 );
    sboxs->Add( m_overallProgress, 0, wxEXPAND | wxALL, 5 );

    wxSizer* timeSizer = new wxBoxSizer( wxHORIZONTAL );
    wxBitmap wbmp( DirHelper::GetPath(dICON) + _T("sw.png"), wxBITMAP_TYPE_PNG );
    wxStaticBitmap* sbmp = new wxStaticBitmap( this, wxID_ANY, wbmp );
    timeSizer->Add( sbmp, 0, wxALL | wxALIGN_RIGHT, 5 );
    wxSizer* tSizer2 = new wxBoxSizer( wxVERTICAL );
    m_elapsedTime = new wxStaticText( this, wxID_ANY, _T("") );
    //m_remainingTime = new wxStaticText( this, wxID_ANY, _T("") );
    m_estimatedTime = new wxStaticText( this, wxID_ANY, _T("") );
    setProgressTime( 0, -1 );

    tSizer2->Add( m_elapsedTime, 0, wxALIGN_LEFT | wxALL, 2 );
    tSizer2->Add( m_estimatedTime, 0, wxALIGN_LEFT | wxALL, 2 );
    timeSizer->Add( tSizer2, 0, wxALL | wxEXPAND, 3 );

    sboxs->Add( timeSizer, 0, wxALL | wxALIGN_CENTER, 2 );

    // buttons
    wxSizer* bs = new wxBoxSizer( wxHORIZONTAL );
    bs->Add( cancel, 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 5 );
    bs->Add( m_closeButton, 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 5 );

    sboxs->Add( bs, 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 5 );

    m_logProgress->SetProgressStyle( PB_PERCENTTEXT_INNER );
    m_logProgress->Update( 0 );
    m_overallProgress->SetProgressStyle( PB_PERCENTTEXT_INNER );
    m_overallProgress->Update( 0 );

    mainsizer->Add( sboxs, 1, wxEXPAND | wxLEFT | wxRIGHT, 5 );
    mainsizer->AddSpacer( 10 );

    ///
    SetSizer( mainsizer );
    mainsizer->Fit( this );
    mainsizer->SetSizeHints( this );
}

void SDCardDownloadPanel::OnButton( wxCommandEvent& event )
{
    switch ( event.GetId() )
    {
        // select/unselect all button
        case idSelectAllButton:
        {
            m_isAllSelected = !m_isAllSelected;

            for ( size_t i = 0; i < m_fileListBox->GetCount(); ++i )
            {
                m_fileListBox->Check( i, m_isAllSelected );
            }

            break;
        }

        default: event.Skip();
    }
}

void SDCardDownloadPanel::enableButtons( bool enable )
{
    m_saveselbutton->Enable( enable );
    m_selallbutton->Enable( enable );
    m_summbutt->Enable( enable );
    m_closeButton->Enable( enable );
}

void SDCardDownloadPanel::showSummary( const wxVariantHash& data )
{
    m_summaryGrid->ClearGrid();

    int d = m_summaryGrid->GetNumberRows() - data.size();

    // add rows to/delete rows from grid if neccessary
    if ( d < 0 )
    {
        m_summaryGrid->AppendRows( abs( d ) );
    }
    else
    {
        m_summaryGrid->DeleteRows( m_summaryGrid->GetNumberRows() - d, d );
    }

    wxVariantHash::const_iterator it = data.begin();

    for ( unsigned int row = 0; it != data.end(); ++it, ++row )
    {
        m_summaryGrid->SetCellValue( row, 0, it->first );
        // output format %.2f
        wxString val = it->second.MakeString();

        int decPos = val.Find('.');
        if ( decPos != wxNOT_FOUND )
        {
            if ( (int)( val.Length() ) > decPos + 2 )
            {
                val = val.Truncate( decPos + 2 );
            }
        }

        m_summaryGrid->SetCellValue( row, 1, val );
    }

    m_summaryGrid->AutoSize();
}

void SDCardDownloadPanel::setProgressTime( long elapsed, long estimated )
{
    unsigned int h = elapsed / 3600; elapsed %= 3600;
    unsigned int m = elapsed / 60; unsigned int s = elapsed % 60;

    wxString elStr;

    if ( h > 0 )
    {
        elStr = wxString::Format( _T("%d:%02d:%02d"), h, m, s );
    }
    else
    {
        elStr = wxString::Format( _T("%02d:%02d"), m, s );
    }

    elStr = _T("Elapsed time: ") + elStr;

    wxString esStr;

    if ( estimated < 0 )
    {
        esStr = _T("unknown");
    }
    else
    {
        h = estimated / 3600; estimated %= 3600;
        m = estimated / 60; s = estimated % 60;
        if ( h > 0 ) esStr = wxString::Format( _T("%d:%02d:%02d"), h, m, s );
        else esStr = wxString::Format( _T("%02d:%02d"), m, s );
    }

    esStr = _T("ETA: ") + esStr;

    m_elapsedTime->SetLabel( elStr );
    m_estimatedTime->SetLabel( esStr );
}

///
/// SDCard Dialog
///
BEGIN_EVENT_TABLE( SDCardDialog, wxDialog )
    EVT_SDCARD( SDCardDialog::OnSDCardEvent )
    EVT_CLOSE( SDCardDialog::OnClose )
END_EVENT_TABLE()

SDCardDialog::SDCardDialog( wxWindow* parent, FirmwareInfo* fwinfo, STDataModelBase* model, MTConfigData* mtconfig, VemsDeviceControllerBase* controller )
{
    m_controller = controller;
    m_sdhandler  = NULL;

    m_fwinfo = dynamic_cast<FirmwareInfo*>( VemsContext::getObject( _T("FirmwareInfo") ) );

    // should not happen
    if ( m_fwinfo == NULL )
    {
        m_fwinfo     = fwinfo;
    }


    m_model      = model;
    m_mtconfig   = mtconfig;

    m_resetState = false;

    m_currentJob = IDLE;

    m_oldevtHandler = parent;

    buildGUI( parent );

    Connect( wxEVT_SDCARD, SDCardEventHandler( SDCardDialog::OnSDCardEvent ), NULL, this );

    if ( m_controller )
    {
//        m_controller->searchingSDFilesGUI( this );
        m_controller->stop( VDC_FLAG_NOEVENT );
        m_sdCardController = new SDCardController( m_controller, this );
        m_sdCardController->search();
    }
    else
    {
        LOGP( _T("VemsDeviceControllerBase is NULL!!") )
    }

    m_isCancelled = false;
}

SDCardDialog::SDCardDialog( wxWindow* parent, FirmwareInfo* fwinfo, STDataModelBase* model, MTConfigData* mtconfig, SDImageHandler* sdhandler )
{
    m_controller = NULL;
    m_sdhandler  = sdhandler;

    m_fwinfo = dynamic_cast<FirmwareInfo*>( VemsContext::getObject( _T("FirmwareInfo") ) );

    // should not happen
    if ( m_fwinfo == NULL )
    {
        m_fwinfo     = fwinfo;
    }

    m_model      = model;
    m_mtconfig   = mtconfig;

    m_resetState = false;

    m_currentJob = IDLE;

    m_oldevtHandler = parent;

    buildGUI( parent );

    if ( m_sdhandler )
    {
        m_sdhandler->setEvtHandler( this );
        m_sdhandler->searchFiles();
    }
    else
    {
        LOGP( _T("SDImageHandler is NULL!!") )
    }

    m_isCancelled = false;
}

void SDCardDialog::onBaudRateChanged( VemsDeviceControllerEvent& event )
{
//    if ( m_resetState == false )
//    {
//        m_controller->searchingSDFilesGUI( this );
//    }
//    else
//    {
//        Destroy();
//        m_controller->Disconnect( VDC_EVENT_DEVICE_BAUDRATE_CHANGED, wxEVT_VEMS_DEVICE_CONTROLLER, VemsDeviceControllerEventHandler( SDCardDialog::onBaudRateChanged ), NULL, this );
//        m_controller->start();
//    }
}

SDCardDialog::~SDCardDialog()
{
    if ( m_controller )
    {
        m_controller->Disconnect( wxEVT_SDCARD, SDCardEventHandler( SDCardDialog::OnSDCardEvent ), NULL, this );

        m_controller->Disconnect( VDC_EVENT_DEVICE_DISCONNECTED, wxEVT_VEMS_DEVICE_CONTROLLER, VemsDeviceControllerEventHandler( SDCardDialog::onDeviceDisconnected ), NULL, this );

        //m_controller->removeEventHandler( this );
    }
}

void SDCardDialog::OnClose( wxCloseEvent& event )
{
    if ( m_controller )
    {
        ResetOriginalState();
//        m_controller->registerHandler( m_oldevtHandler );
//        m_controller->changeBaudRate( 19200 );
//        m_controller->start();
        m_controller->resume();
    }

    else if ( m_sdhandler )
    {
        delete m_sdhandler;
        Destroy();
    }
}

void SDCardDialog::buildGUI( wxWindow* parent )
{
    Create( parent, wxID_ANY, _T("SD Card overview"), wxDefaultPosition );

    // SetBackgroundColour( wxColor(212,208,200) );
    SetBackgroundColour( wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE ) );
    ClearBackground();

    wxBoxSizer* mainsizer = new wxBoxSizer( wxVERTICAL );

    // Searching panel
    wxSizer* ps1 = new wxBoxSizer( wxVERTICAL );
    m_searchPanel = new SDCardSearchPanel( this );
    ps1->Add( m_searchPanel, 0, wxALL | wxEXPAND, 5 );

    // download panel

    wxSizer* ps2 = new wxBoxSizer( wxVERTICAL );
    m_downlPanel = new SDCardDownloadPanel( this );
    ps2->Add( m_downlPanel, 0, wxALL | wxEXPAND, 5 );
    m_downlPanel->Show( false );

    mainsizer->Add( ps1, 1, wxALL | wxEXPAND, 5 );
    mainsizer->Add( ps2, 0, wxALL | wxEXPAND, 5 );

    SetSizer( mainsizer );
    mainsizer->Fit( this );
    mainsizer->SetSizeHints( this );
}

void SDCardDialog::OnSDCardEvent( SDCardEvent& event )
{
    /// DIFFERENT FW
    if ( event.GetType() == SDCardEventTypes::FIRMWARE_VERSION )
    {
        InfoWindow::Display( NULL, _T("Different firmwares"), _T("V3 firmware version and SD Card\nfirmware version are different. Data may be invalid"),
                    IW_FADE_IN | IW_MOVE | IW_WARNING );
    }
    /// TIMEOUT

    if ( event.GetType() == SDCardEventTypes::TIMEOUT )
    {
        wxMessageBox( _T("Timeout. Check cables and connection then try again!"), _T("Error"), wxOK | wxICON_HAND );

        if ( m_currentJob == SEARCH )
        {
            Close();
        }
        else
        {
            m_downlPanel->setProgressTime( 0, -1 );
            m_downlPanel->m_overallProgress->Update( 0 );
            m_downlPanel->m_logProgress->Update( 0 );
            m_downlPanel->m_summaryProgress->Update( 0 );
            m_downlPanel->enableButtons( true );
        }
    }

    ///
    /// ABORT
    ///

    if ( event.GetType() == SDCardEventTypes::INTERRUPTED )
    {
        wxMessageBox( _T("Process aborted"), _T("Information"), wxOK );

        m_downlPanel->m_overallProgress->Update( 0 );
        m_downlPanel->m_logProgress->Update( 0 );
        m_downlPanel->m_summaryProgress->Update( 0 );

        m_downlPanel->enableButtons( true );

        m_currentJob = IDLE;
    }

    ///
    /// SEARCHING
    ///
    if ( event.GetType() == SDCardEventTypes::SEARCHING_PROGRESS )
    {
        if ( m_searchPanel->getState() != 1 ) m_searchPanel->setState( 1 );

        m_currentJob = SEARCH;
    }
    if ( event.GetType() == SDCardEventTypes::SEARCHING_FINISHED )
    {
        int cardState = event.GetInt();

        m_searchPanel->setState( 2, cardState );

        if ( cardState == 0 )
        {
            m_downlPanel->Show( true );
            m_filesDesc = event.GetFiles();
            m_downlPanel->UpdateFiles( m_filesDesc );
            wxSizer* p2s = m_downlPanel->GetSizer();
            p2s->RecalcSizes();
            p2s->Layout();
            p2s->Fit( m_downlPanel );
            p2s->SetSizeHints( m_downlPanel );
        }

        wxSizer* mainsizer = GetSizer();
        mainsizer->Layout();
        mainsizer->RecalcSizes();
        mainsizer->Fit( this );
        mainsizer->SetSizeHints( this );

        m_currentJob = IDLE;
    }

    ///
    /// DOWNLOADING
    ///
    if ( event.GetType() == SDCardEventTypes::DOWNLOADING_START )
    {
        MMCFilesDesc desc = m_filesDesc[m_fileindices[m_counter]];

        m_currentJob = DOWNLOAD;

        m_startTime = wxDateTime::Now();

        // v3
        if ( m_controller )
        {
            //m_controller->downloadSDFile( desc );
            m_sdCardController->download( desc );

        }
        // image file
        else if ( m_sdhandler )
        {
            m_sdhandler->Reset();

            m_sdhandler->downloadSDFile( desc );
        }
    }
    if ( event.GetType() == SDCardEventTypes::DOWNLOADING_PROGRESS )
    {
        double percent = event.GetVariant().GetDouble();

        wxSafeYield( this );
        m_downlPanel->m_logProgress->Update( percent );

        double ovrall = (double)m_timeCtr + (double)(m_filesDesc[m_fileindices[m_counter]].Time) * percent / 100.0;

        ovrall = ovrall * 100.0 / m_timeAll;

        m_downlPanel->m_overallProgress->Update( ovrall );

        if ( m_sw.Time() > 1000 )
        {
            long elapsed = ( wxDateTime::Now() - m_progTime ).GetSeconds().ToLong();

            //long remaining = (long)( (double)elapsed / ovrall * ( 100.0 - ovrall ) );
            long estimated = (long)( (double)elapsed / ovrall * 100.0 );

            m_downlPanel->setProgressTime( elapsed, estimated );

            m_sw.Start();
        }
    }
    if ( event.GetType() == SDCardEventTypes::DOWNLOADING_FINISHED )
    {
        unsigned char* data = (unsigned char*)(event.GetVariant().GetVoidPtr());
        size_t datalen = (size_t)event.GetInt();
        Logger::logf(LOG_MSG, _T("SDCard DataLen=%d"),datalen);

        switch( event.GetDataType() ){
            case SDCardFile::SDLOG_TR_FRAME:
                SaveTFDataToFile(data, datalen );
                break;
            case SDCardFile::SDLOG_RAW:
            default:
                SaveDataToFile( data, datalen );
                break;
        }
        delete [] data;

        m_downlPanel->m_fileListBox->Check( m_fileindices[m_counter], false );

        m_currentJob = IDLE;

        // next log

        m_timeCtr += m_filesDesc[m_fileindices[m_counter]].Time;

        ++m_counter;

        if ( m_counter < m_numselected )
        {
            SDCardEvent event; event.SetType( SDCardEventTypes::DOWNLOADING_START );
            AddPendingEvent( event );
        }
        else
        {
            m_downlPanel->enableButtons( true );

            wxString msg;

            if ( m_counter == 1 )
            {
                msg = _T("1 log has been created");
            }
            else
            {
                msg = wxString::Format( _T("%d logs have been created"), m_counter );
            }

            SDCardResultDialog dlg( this, msg, DirHelper::GetPath( dLOG ) );
            dlg.ShowModal();

            // msg += _T("\nSee the userlog for more details");

            //wxMessageBox( msg, _T("Information"), wxOK | wxICON_INFORMATION );
            m_downlPanel->m_isAllSelected = false;
        }
    }

    ///
    /// SD FILE SUMMARY

    if ( event.GetType() == SDCardEventTypes::SUMMARY_START )
    {
        m_downlPanel->m_summaryProgress->Update( 0 );

        int sel = event.GetInt();

        if ( sel < 0 || sel > (int)( m_filesDesc.size() ) )
        {
            return;
        }

        MMCFilesDesc& desc = m_filesDesc[sel];

        m_currentJob = SUMMARY;

        if ( m_controller )
        {
            m_sdCardController->createSummary( desc );
        }
        else
        {
            m_sdhandler->Reset();

            m_sdhandler->createSDSummary( desc );
        }
    }

    if ( event.GetType() == SDCardEventTypes::SUMMARY_PROGRESS )
    {
        double percent = event.GetVariant().GetDouble();

        wxSafeYield( this );

        m_downlPanel->m_summaryProgress->Update( percent );
    }

    if ( event.GetType() == SDCardEventTypes::SUMMARY_FINISHED )
    {
        m_downlPanel->m_summaryProgress->Update( 100.0 );
        m_downlPanel->enableButtons( true );

        wxVariantHash* data = (wxVariantHash*)( event.GetVariant().GetVoidPtr() );
        m_downlPanel->showSummary( *data );

        delete data;

        m_currentJob = IDLE;
    }
}

void SDCardDialog::onDeviceDisconnected( VemsDeviceControllerEvent& event )
{
    LOG

    wxMessageBox( _T("Device disconneted"), _T("Error"), wxOK | wxICON_ERROR );

    EndModal( -1 );

    event.Skip();
}

void SDCardDialog::OnSaveSelected( wxCommandEvent& event )
{
    m_downlPanel->m_summaryProgress->Update( 0 );

    wxCheckListBox* box = m_downlPanel->m_fileListBox;

    m_fileindices.Clear();

    m_numselected = 0;
    m_timeAll = 0;
    m_timeCtr = 0;
    m_counter = 0;

    for ( size_t i = 0; i < box->GetCount(); ++i )
    {
        if ( box->IsChecked(i) )
        {
            m_fileindices.Add( i );
            ++m_numselected;
            m_timeAll += m_filesDesc[i].Time;
        }
    }

    if ( m_numselected == 0 )
    {
        wxMessageBox( _T("No files were selected"), _T("Error"), wxOK );
    }
    else
    {
        SaveSelectedFiles();
    }
}

void SDCardDialog::SaveSelectedFiles()
{
    m_downlPanel->enableButtons( false );
    m_downlPanel->setProgressTime( 0, -1 );

    m_sw.Start();

    m_progTime = wxDateTime::Now();

    SDCardEvent event;
    event.SetType( SDCardEventTypes::DOWNLOADING_START );
    AddPendingEvent( event );
}

void SDCardDialog::OnCreateSummary( wxCommandEvent& event )
{
    wxCheckListBox* box = m_downlPanel->m_fileListBox;

    int selection = box->GetSelection();
    if ( selection == wxNOT_FOUND )
    {
        return;
    }

    long smpVal = 0;

    const wxString& smpl = m_downlPanel->m_sampleCntCtrl->GetValue();

    smpl.ToLong( &smpVal );

    if ( smpVal <= 0 )
    {
        return;
    }

    m_filesDesc[selection].SampleCnt = smpVal;

    m_downlPanel->enableButtons( false );

    SDCardEvent sdevent;
    sdevent.SetType( SDCardEventTypes::SUMMARY_START );
    sdevent.SetInt( selection );
    AddPendingEvent( sdevent );
}

void SDCardDialog::OnCloseButton( wxCommandEvent& event )
{
    if ( m_controller )
    {
        //ResetOriginalState();
        Destroy();
        m_controller->resume();
    }
    else if ( m_sdhandler )
    {
        delete m_sdhandler;
        Destroy();
    }
}

void SDCardDialog::OnCancelButton( wxCommandEvent& event )
{
    m_isCancelled = true;

    if ( m_controller )
    {
        m_sdCardController->abort();
    }

    else if ( m_sdhandler )
    {
        m_sdhandler->Abort();
    }
}

wxString SDCardDialog::initSIPRFileLogger(SIPRFileLogger& siprFileLogger)
{
    siprFileLogger.enableLog( true );

    wxString header = RawLogHelper::getVemslogHeader( *m_fwinfo, 2 );
    siprFileLogger.setHeader( header );

    wxString sernum = m_fwinfo->getSerialNumber();

    wxString prefix = wxString::Format( _T("-%03d"), m_fileindices[m_counter] + 1 );

    wxString fileName = DirHelper::CreateNewFileName( sernum + prefix + _T("-SDCard.vemslog"),
        DirHelper::GetPath( dLOG ), true, true );

    siprFileLogger.setFile( fileName );

    if ( m_model != NULL && m_mtconfig != NULL )
    {
        STDataModelWriter::logPages( m_model, &siprFileLogger, m_mtconfig->getSIPRDescriptor() );
    }
    else
    {
        Logger::logf( LOG_MSG, _T("Either STDataModelSerial or MTConfigData is NULL!") );
    }

    return fileName;
}

void SDCardDialog::initSIPRData(uint8_t* siprdata)
{
    siprdata[0] = 0;
    siprdata[1] = 0;
    siprdata[2] = SiprSensorType::SIPR_RT_MT_NORMAL;
}

void SDCardDialog::SaveTFDataToFile( unsigned char* data, size_t len )
{
    // Logger::logf(LOG_MSG, _T("data=%p, datalen=%d"), data, len);

    if ( data == NULL || len < 1 )
    {
        return;
    }

    SIPRFileLogger siprFileLogger(512);
    wxString fileName = initSIPRFileLogger(siprFileLogger);

    uint8_t siprdata[512+3];
    initSIPRData(siprdata);

    uint8_t counter = 0;
    uint8_t buffer[512+2];

    for ( size_t i = 0; i < len; i += 512 )
    {
        memcpy( buffer, data + i, 512 );

        TriggerFrameFSMCRC trFSM;

        for ( size_t j = 0; j < 512; ++j )
        {
            if ( !trFSM.parse( buffer[j] ) )
            {
                continue;
            }

            int trType = trFSM.getType();

            const IntArray& rawData = trFSM.getFrame();

            if ( trType != TriggerFrame::EVENT_REALTIME )
            {
                int ssize = rawData.GetCount()+1;

                if ( ssize > 255 )
                {
                    LOGP( wxString::Format( _T("Cannot convert to SIPR Packet, size: %d"), ssize ) )
                    continue;
                }

                uint8_t* packetData = new uint8_t[ssize];

                packetData[0] = SIPR_TRIGGERFRAME;
                trFSM.intArrayToByte( &(packetData[1]) );

                SIPRPacket p( 0, SIPR_LOG_MARKER, ssize, packetData );
                siprFileLogger.process( p );
                //Logger::logf( LOG_MSG, _T("SaveTFDataToFile: unknown tr type: %i!"), trType );
                delete[] packetData;
                continue;
            }

            int rtSize = rawData.GetCount();

            if ( rtSize > ( 255 - 3 ) )
            {
                Logger::logf( LOG_MSG, _T("SaveTFDataToFile: tr type too long: %i!"), rtSize );
            }

            trFSM.intArrayToByte( &(siprdata[3]) );

            SIPRPacket p( counter, SIPR_READ_SENSORS, rtSize + 3, siprdata );

            siprFileLogger.process( p );

            ++counter;
        }
    }
    // additionally export whole buffer to triggerlog file (triggerframes)
    ExportTFDataToLogV3( data, len );

    // create ulog message
    wxTimeSpan elapTime = wxDateTime::Now() - m_startTime;
    wxString tStr = elapTime.Format( _T("%H:%M:%S") );
    wxString message = _T("SDCard log has been saved to: ") + fileName + _T("\nElapsed time: ") + tStr;
    UserLog::uLog( ULOG_OK, message );
}

void SDCardDialog::SaveDataToFile( unsigned char* data, size_t len )
{
    // Logger::logf(LOG_MSG, _T("data=%p, datalen=%d"), data, len);
    if ( data == NULL || len < 1 )
    {
        return;
    }

    SIPRFileLogger siprFileLogger(512);
    wxString fileName = initSIPRFileLogger(siprFileLogger);

    uint8_t siprdata[512+3];
    initSIPRData(siprdata);

    uint8_t counter = 0;
    uint8_t buffer[512+2];

    for ( size_t i = 0; i < len; i += 512 )
    {
        memcpy( buffer, data + i, 512 );

        uint8_t _size = buffer[511];
//        Logger::logf(LOG_MSG, _T("size=%d"), _size);
        if ( !_size )
        {
            continue;
        }

        size_t blockCount = (int)( floorf( ( 514 - 3 ) / _size ) );
        unsigned int bfrom = 0;

        for ( size_t j = 0; j < blockCount; ++j )
        {
            bfrom = j * _size;

            memcpy( siprdata + 3, buffer + bfrom, _size );

            SIPRPacket p( counter, SIPR_READ_SENSORS, _size + 3, siprdata );

            siprFileLogger.process( p );

            ++counter;
        }
    }

    wxTimeSpan elapTime = wxDateTime::Now() - m_startTime;
    wxString tStr = elapTime.Format( _T("%H:%M:%S") );
    wxString message = _T("SDCard log has been saved to: ") + fileName + _T("\nElapsed time: ") + tStr;
    UserLog::uLog( ULOG_OK, message );
}

void SDCardDialog::ExportTFDataToLogV3( unsigned char* data, size_t len )
{
    wxString header = RawLogHelper::getVemslogHeader( *m_fwinfo, 3, true );

    wxString sernum = m_fwinfo->getSerialNumber();

    wxString prefix = wxString::Format( _T("-%03d"), m_fileindices[m_counter] + 1 );

    wxString fileName = DirHelper::CreateNewFileName( sernum + prefix + _T("-SDCard.triggerlog"),
        DirHelper::GetPath( dLOG ), true, true );

    wxFile file;

    file.Create( fileName, true );

    char header_c[512];
    memset( header_c, 0, 512 );
    for ( size_t i = 0; i < header.Length(); ++i )
    {
        header_c[i] = header[i];
    }

    file.Write( header_c, 512 );

    file.Write( data, len );

    file.Flush();

    file.Close();
}

void SDCardDialog::ResetOriginalState()
{
    m_resetState = true;
    m_controller->Disconnect( VDC_EVENT_DEVICE_DISCONNECTED, wxEVT_VEMS_DEVICE_CONTROLLER, VemsDeviceControllerEventHandler( SDCardDialog::onDeviceDisconnected ), NULL, this );
    Hide();
    m_controller->changeBaudRate( m_oldBaudrate );
}

/**
    SDCardResultDialog

    Shows the result of sd card save task
**/

int idSDCardOpenFolder = wxNewId();

BEGIN_EVENT_TABLE( SDCardResultDialog, wxDialog )
    EVT_BUTTON( wxID_OK, SDCardResultDialog::OnButtonClick )
    EVT_BUTTON( idSDCardOpenFolder, SDCardResultDialog::OnButtonClick )
END_EVENT_TABLE()

SDCardResultDialog::SDCardResultDialog( wxWindow* parent, const wxString& message, const wxString& saved_path ) :
    wxDialog( parent, wxID_ANY, _T("Information"), wxDefaultPosition, wxDefaultSize )
{

    m_savedPath = saved_path;

    wxSizer* sizer = new wxBoxSizer( wxVERTICAL );

    // text & icon
    wxPanel* panel = new wxPanel( this );
    panel->SetBackgroundColour( *wxWHITE );
    panel->ClearBackground();
    wxBitmap infobmp( DirHelper::GetPath( dICON ) + _T("ul-info-xl.png"), wxBITMAP_TYPE_PNG );

    wxStaticBitmap* bmp = new wxStaticBitmap( panel, wxID_ANY, infobmp );

    wxString msg = message + _T("\nFiles has been saved to: ") + saved_path;
    msg += _T("\nSee the userlog for more details");

    wxStaticText* txt = new wxStaticText( panel, wxID_ANY, msg );

    wxSizer* infosizer = new wxBoxSizer( wxHORIZONTAL );
    infosizer->Add( bmp, 0, wxEXPAND | wxALL, 5 );
    infosizer->Add( txt, 0, wxEXPAND | wxALL, 5 );
    infosizer->Add( 10, 10, 0 );

    wxSizer* panelsizer = new wxBoxSizer( wxVERTICAL );
    panelsizer->AddSpacer( 10 );
    panelsizer->Add( infosizer, 0, wxEXPAND | wxALL, 3 );
    panelsizer->AddSpacer( 10 );

    panel->SetSizer( panelsizer );
    panelsizer->SetSizeHints( panel );
    panelsizer->SetSizeHints( panel );

    // buttons

    wxSizer* buttonsizer = new wxBoxSizer( wxHORIZONTAL );

    wxButton* button = NULL;
    #ifdef __WXMSW__
    button = new wxButton( this, idSDCardOpenFolder, _T("Open containing folder") );
    buttonsizer->Add( button, 0, wxEXPAND | wxALL | wxALIGN_RIGHT, 5 );
    #endif

    button = new wxButton( this, wxID_OK, _T("Close dialog") );
    buttonsizer->Add( button, 0, wxEXPAND | wxALL | wxALIGN_RIGHT, 5 );

    sizer->Add( panel, 1, wxEXPAND | wxBOTTOM, 8 );
    sizer->Add( buttonsizer, 0, wxALL | wxALIGN_RIGHT, 5 );

    SetSizer( sizer );
    sizer->SetSizeHints( this );
    sizer->Fit( this );
}

void SDCardResultDialog::OnButtonClick( wxCommandEvent& event )
{
    int id = event.GetId();

    if ( id == wxID_OK )
    {
        EndModal( 0 );
        event.Skip();
    }
    else if ( id == idSDCardOpenFolder )
    {
        #ifdef __WXMSW__
        if ( m_savedPath != wxEmptyString )
        {
            // get path to the explorer.exe
            wxString cmd = wxGetOSDirectory() + DirHelper::Separ() + _T("explorer.exe ");
            cmd += m_savedPath;

            if ( m_savedPath != wxEmptyString )
            {
                wxExecute( cmd );
            }
        }
        #endif

        event.Skip();
    }
}
