#include "SFloodDialog.h"

#include <wx/statline.h>

#include "../common/ProcessBase.h"
#include "../common/VemsMessageHashHelper.h"


int idSFloodStart = wxNewId();
int idSFloodStop  = wxNewId();
int idSFloodClose = wxNewId();
const wxEventType SFLOOD_EVENT = wxNewEventType();

BEGIN_EVENT_TABLE( SFloodDialog, wxDialog )
    EVT_BUTTON( idSFloodStart, SFloodDialog::OnButton )
    EVT_BUTTON( idSFloodStop, SFloodDialog::OnButton )
    EVT_BUTTON( idSFloodClose, SFloodDialog::OnButton )
    EVT_BUTTON( idSFloodClose, SFloodDialog::OnButton )
END_EVENT_TABLE()

SFloodDialog::SFloodDialog( wxWindow* parent, ProcessBase* processBase ) :
    wxDialog( parent, wxID_ANY, _T("Round emergency revival dialog"), wxDefaultPosition, wxDefaultSize )
{
    m_processBase = processBase;

    VemsMessage vemsMsg = VemsMessageHashHelper::GetVemsMsg(_T("round_sflood_steps") );
    wxString steps = vemsMsg.getHtmlMsg();

    wxBoxSizer* mainsizer = new wxBoxSizer( wxVERTICAL );

    wxFont font( 14, wxFONTFAMILY_DEFAULT, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_BOLD );
    m_text = new wxStaticText( this, wxID_ANY, _T("Status: "), wxDefaultPosition, wxSize( 400, -1 ) );
    m_text->SetFont( font );
    mainsizer->Add( m_text, 0, wxALL, 8 );
    wxStaticText* steps_text = new wxStaticText( this, wxID_ANY, steps, wxDefaultPosition, wxDefaultSize );
    mainsizer->Add( steps_text, 0, wxALL, 1);


    wxStaticLine* line = new wxStaticLine( this, wxID_ANY, wxDefaultPosition, wxSize( 360, -1 ) );

    mainsizer->Add( line, 0, wxALL | wxEXPAND | wxALIGN_CENTER, 5 );

    m_startButton = new wxButton( this, idSFloodStart, _T("Start") );
    m_stopButton = new wxButton( this, idSFloodStop, _T("Stop") );
    m_closeButton = new wxButton( this, idSFloodClose, _T("Close") );

    wxBoxSizer* buttonSizer = new wxBoxSizer( wxHORIZONTAL );
    buttonSizer->Add( m_startButton, 0, wxALL, 5 );
    buttonSizer->Add( m_stopButton, 0, wxALL, 5 );
    buttonSizer->Add( m_closeButton, 0, wxALL, 5 );

    mainsizer->Add( buttonSizer, 1, wxALL | wxALIGN_CENTER, 5 );

    SetSizer( mainsizer );
    mainsizer->Fit( this );
    mainsizer->SetSizeHints( this );

    m_stopButton->Disable();

    Connect( wxID_ANY, SFLOOD_EVENT, wxCommandEventHandler(SFloodDialog::OnNotify) );

    //Disconnect( toolid, SFLOOD_EVENT, wxCommandEventHandler(VemsToolBarToolBase::OnTool), NULL, tool);

     SetStatusText( _T("Stopped") );
}

void SFloodDialog::OnButton( wxCommandEvent& event )
{
    int id = event.GetId();

    // start button pressed
    if ( id == idSFloodStart )
    {
        m_stopButton->Enable();
        m_startButton->Disable();

        m_processBase->start();
        SetStatusText( _T("Searching...") );

        /// TODO
        /// ...

    }

    // stop button pressed
    if ( id == idSFloodStop )
    {
        m_startButton->Enable();
        m_stopButton->Disable();
        m_processBase->stop();
        SetStatusText( _T("Stopped") );

        /// TODO
        /// ...

    }

    if ( id == idSFloodClose )
    {
        m_processBase->stop();
        if ( IsModal() )
            EndModal( 0 );
        else
            Hide();
    }
}

void SFloodDialog::SetStatusText( const wxString& text )
{
    m_text->SetLabel( _T("Status: ") + text );
}

/// This is inwoked from thread so needs to send an event to update the GUI
void SFloodDialog::notify( int i )
{
    if(i == 1){
        wxCommandEvent event( SFLOOD_EVENT, wxID_ANY );
        AddPendingEvent( event );
    }
}

void SFloodDialog::OnNotify( wxCommandEvent& command)
{
    SetStatusText( _T("ROUND BOOT DETECTED!") );
    m_startButton->Enable();
    m_stopButton->Disable();

}
