#include "SaveConfigErrorDialog.h"

#include <wx/statline.h>
#include <wx/artprov.h>

#include "../common/types/StringHash.h"
#include "../common/VemsMessageHashHelper.h"
#include "../logger.h"


BEGIN_EVENT_TABLE( SaveConfigErrorDialog, wxDialog )
    EVT_BUTTON( ID_WEBTOOL, SaveConfigErrorDialog::OnButton )
    EVT_BUTTON( ID_DOWNLOAD_CONFIG, SaveConfigErrorDialog::OnButton )
    EVT_BUTTON( ID_DOWNLOAD_CONFIG_TO, SaveConfigErrorDialog::OnButton )
    EVT_BUTTON( ID_CANCEL, SaveConfigErrorDialog::OnButton )
END_EVENT_TABLE()


SaveConfigErrorDialog::SaveConfigErrorDialog(   wxWindow* parent,
                                                wxWindowID id,
                                                const wxString& title,
                                                const wxString& firmwareVersionAsString,
                                                bool online,
                                                const wxPoint& pos,
                                                const wxSize& size,
                                                long style,
                                                const wxString& name )
{
    //offline_save_config_str
    //online_save_config_str
    Create( parent, id, _T("Save Config Warning"), pos, size, style, name );

    SetSizer( new wxBoxSizer( wxVERTICAL ) );

    wxBoxSizer* pButtonLineSizer = new wxBoxSizer( wxHORIZONTAL );

    wxButton* webtoolButton   = new wxButton( this, ID_WEBTOOL, _T("Firmware web tool") );
    pButtonLineSizer->Add( webtoolButton, 0, wxALL, 5 );

    if ( online )
    {
        wxButton* downloadConfigButton = new wxButton( this, ID_DOWNLOAD_CONFIG, _T("Config Download") );
        pButtonLineSizer->Add( downloadConfigButton, 0, wxALL, 5 );

        wxButton* downloadConfigToButton = new wxButton( this, ID_DOWNLOAD_CONFIG_TO, _T("Config Download To ...") );
        pButtonLineSizer->Add( downloadConfigToButton, 0, wxALL, 5 );
    }

    wxButton* cancelButton = new wxButton( this, ID_CANCEL, _T("Cancel") );
    pButtonLineSizer->AddStretchSpacer();
    pButtonLineSizer->Add( cancelButton, 0, wxALL, 5 );

    wxBoxSizer* pMsgLineSizer = new wxBoxSizer( wxHORIZONTAL );

    wxBitmap bitmap = wxArtProvider::GetIcon(wxART_WARNING, wxART_MESSAGE_BOX);
    wxStaticBitmap *icon = new wxStaticBitmap(this, wxID_ANY, bitmap);
    pMsgLineSizer->Add( icon, 0, wxALIGN_CENTER | wxALL , 5 );

    wxString messageName = online ? _T("online_save_config_str") : _T("offline_save_config_str");
    StringHash hash;
    hash[_T("firmware_version")] = firmwareVersionAsString;
    VemsMessage message = VemsMessageHashHelper::GetVemsMsg( messageName ,&hash );
    wxString messageStr = message.getHtmlMsg();
    wxStaticText* text = new wxStaticText( this, wxID_ANY, messageStr );

    pMsgLineSizer->Add( text, 1, wxALIGN_CENTER | wxALL, 5 );

    GetSizer()->Add( pMsgLineSizer, 0, 0, 0 );
    GetSizer()->Add( new wxStaticLine( this ), 0, wxEXPAND | wxALL, 5 );
    GetSizer()->Add( pButtonLineSizer, 0, wxEXPAND, 0 );

    SetInitialSize();
}

void SaveConfigErrorDialog::OnButton( wxCommandEvent& event )
{
    EndModal( event.GetId() );
}

