#include "TableEditor3DPreferencesFrame.h"

#include "../logger.h"
#include "../3d/TableEditor3DDefs.h"
#include "../3d/TableEditor3DConfigHelper.h"

int CURSOR_MOVING_MODE_RADIO_BOX_ID = wxNewId();
int SELECTED_ALIGN_RADIO_BOX_ID     = wxNewId();
int MARKED_ALIGN_RADIO_BOX_ID       = wxNewId();
int LABEL_FONT_SIZE_ID              = wxNewId();

BEGIN_EVENT_TABLE( TableEditor3DPreferencesFrame, wxFrame )
    EVT_BUTTON(  wxID_SAVE, TableEditor3DPreferencesFrame::OnSaveButton )
END_EVENT_TABLE()

TableEditor3DPreferencesFrame::TableEditor3DPreferencesFrame( wxWindow* parent )
    : wxFrame( parent, wxID_ANY, _T("TableEditor3D Preferences") )
{
    buildGUI();
    loadParams();
}

TableEditor3DPreferencesFrame::~TableEditor3DPreferencesFrame()
{

}

void TableEditor3DPreferencesFrame::buildGUI()
{
    wxPanel* mainPanel = new wxPanel( this );

    mainPanel->SetSizer( new wxBoxSizer( wxVERTICAL ) );

    wxArrayString cursorMovingModes;
    cursorMovingModes.Add( _T("Relative (Arrow keys follow rotation)") );
    cursorMovingModes.Add( _T("Absolute (Arrow keys do not follow rotation)") );

    wxRadioBox* modeSelectionRadioBox = new wxRadioBox( mainPanel, CURSOR_MOVING_MODE_RADIO_BOX_ID, _T("Cursor moving mode"),
                                                    wxDefaultPosition, wxDefaultSize, cursorMovingModes,
                                                    cursorMovingModes.GetCount(), wxRA_SPECIFY_ROWS );

    mainPanel->GetSizer()->Add( modeSelectionRadioBox, 0, wxALL , 5 );

    wxBoxSizer* labelAlignSizer = new wxBoxSizer(  wxHORIZONTAL );


    wxArrayString labelAligns = TE3D_Align::getLabels();

    wxRadioBox* selectedAlignRadioBox = new wxRadioBox( mainPanel, SELECTED_ALIGN_RADIO_BOX_ID, _T("Selected Label Align"),
                                                    wxDefaultPosition, wxDefaultSize, labelAligns,
                                                    labelAligns.GetCount(), wxRA_SPECIFY_ROWS );

    labelAlignSizer->Add( selectedAlignRadioBox, 0, wxALL , 5 );

    wxRadioBox* markedAlignRadioBox = new wxRadioBox( mainPanel, MARKED_ALIGN_RADIO_BOX_ID, _T("Marked Label Align"),
                                                    wxDefaultPosition, wxDefaultSize, labelAligns,
                                                    labelAligns.GetCount(), wxRA_SPECIFY_ROWS );

    labelAlignSizer->Add( markedAlignRadioBox, 0, wxALL , 5 );


    wxStaticBoxSizer* labelFontSizeBox = new wxStaticBoxSizer( wxVERTICAL, mainPanel, _T("Label Font Size") );

    wxComboBox* labelFontSizeCombo = new wxComboBox( mainPanel, LABEL_FONT_SIZE_ID, TE3D_Font::getDefaultSizeStr(),
                                                        wxDefaultPosition, wxDefaultSize,
                                                        TE3D_Font::getValidSizesStrings(), wxCB_READONLY );

    labelFontSizeBox->Add( labelFontSizeCombo, 0, wxALL , 5 );
    labelAlignSizer->Add( labelFontSizeBox, 0, wxALL , 5 );

    mainPanel->GetSizer()->Add( labelAlignSizer );

    wxButton* saveButton = new wxButton( mainPanel, wxID_SAVE, _T("Save") );
    mainPanel->GetSizer()->Add( saveButton, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5 );

    this->CenterOnScreen();
    this->Show( true );
}

void TableEditor3DPreferencesFrame::findAndShow( wxWindow* parent )
{
    wxWindow* win = wxFindWindowByName(_T("TableEditor3D Preferences"),parent);

    if ( !win ) {
        new TableEditor3DPreferencesFrame( parent );
    } else {
        win->SetFocus();
    }
}

void TableEditor3DPreferencesFrame::loadParams()
{
    TableEditor3DParams params;
    TableEditor3DConfigHelper::loadGlobalParams( params );

    /// cursor moving mode
    wxRadioBox* cursorMovingModeRadioBox = (wxRadioBox*)wxWindow::FindWindowById( CURSOR_MOVING_MODE_RADIO_BOX_ID , this );
    wxString cursorMovingMode = TE3D_CursorMovingMode::idToLabelString(  params.cursorMovingMode );
    cursorMovingModeRadioBox->SetStringSelection( cursorMovingMode );

    /// selected label
    wxRadioBox* selectedAlignRadioBox = (wxRadioBox*)wxWindow::FindWindowById( SELECTED_ALIGN_RADIO_BOX_ID , this );
    wxString selectedLabelAlign = TE3D_Align::idToLabelString( params.selectedLabelAlign );
    selectedAlignRadioBox->SetStringSelection( selectedLabelAlign );

    /// marked label
    wxRadioBox* markedAlignRadioBox = (wxRadioBox*)wxWindow::FindWindowById( MARKED_ALIGN_RADIO_BOX_ID, this );
    wxString markedLabelAlign = TE3D_Align::idToLabelString( params.markedLabelAlign );
    markedAlignRadioBox->SetStringSelection( markedLabelAlign );

    /// label font size
    wxComboBox* labelFontSizeCombo = (wxComboBox*)wxWindow::FindWindowById( LABEL_FONT_SIZE_ID, this );
    wxString labelFontSize = wxString::Format( _T("%d"), params.labelFontSize );
    labelFontSizeCombo->SetValue( labelFontSize );
}

void TableEditor3DPreferencesFrame::saveParams()
{
    TableEditor3DParams params;

    /// cursor moving mode
    wxRadioBox* cursorMovingModeRadioBox = (wxRadioBox*)wxWindow::FindWindowById( CURSOR_MOVING_MODE_RADIO_BOX_ID , this );
    wxString cursorMovingMode =  cursorMovingModeRadioBox->GetStringSelection();
    params.cursorMovingMode = TE3D_CursorMovingMode::stringToId( cursorMovingMode );

    /// selected label
    wxRadioBox* selectedAlignRadioBox = (wxRadioBox*)wxWindow::FindWindowById( SELECTED_ALIGN_RADIO_BOX_ID , this );
    wxString selectedLabelAlign = selectedAlignRadioBox->GetStringSelection();
    params.selectedLabelAlign = TE3D_Align::stringToId( selectedLabelAlign );

    /// marked label
    wxRadioBox* markedAlignRadioBox = (wxRadioBox*)wxWindow::FindWindowById( MARKED_ALIGN_RADIO_BOX_ID, this );
    wxString markedLabelAlign = markedAlignRadioBox->GetStringSelection();
    params.markedLabelAlign = TE3D_Align::stringToId( markedLabelAlign );

    /// label font size
    wxComboBox* labelFontSizeCombo = (wxComboBox*)wxWindow::FindWindowById( LABEL_FONT_SIZE_ID, this );
    wxString labelFontSizeStr = labelFontSizeCombo->GetValue();
    long labelFontSize;
    labelFontSizeStr.ToLong( &labelFontSize );
    params.labelFontSize  = (int)labelFontSize;

    TableEditor3DConfigHelper::saveGlobalParams( params );
}

void TableEditor3DPreferencesFrame::OnSaveButton( wxCommandEvent& event )
{
    saveParams();
}
