#include "HotkeyHelper.h"

#include "wxHotKeyManager.h"

#include "../agg/wxAgg2.h"
#include "../DirHelper.h"
#include "../wxVariantHelper.h"

#define chkprop(key,default,func) properties.find(key) == properties.end() ? (default) : properties[(key)].func();
#define chkpropcolor(key,default) properties.find(key) == properties.end() ? (default) : wxVariantHelper::GetColor(properties[(key)]);

wxBitmap* HotkeyHelper::DrawHotkeyHelp(wxHotKeyManager* mgr, wxVariantHash properties)
{

    HotKeyDescArray arr;

    mgr->GetHotKeyList(arr);

    wxString titleStr = wxEmptyString, hkeyStr = wxEmptyString;

    double maxTitleWidth = 0.0, maxHKWidth = 0.0;

    wxAgg2 aggCanvas( 1042, 1042 );

    wxString fontfile = chkprop(_T("fontFile"),DirHelper::GetPath(dFONTS) + _T("arial.ttf"),GetString)

    double text_height = chkprop(_T("textHeight"),12L,GetLong)

    aggCanvas.loadTTFont( fontfile, text_height, 0 );

    for(size_t a = 0; a < arr.GetCount(); a++) {
        double tw = aggCanvas.getTTFTextBound( arr[a].Title, 0, 0 ).w;
        double hw = aggCanvas.getTTFTextBound( arr[a].HotKey, 0, 0 ).w;

        if( maxTitleWidth < tw ) maxTitleWidth = tw;
        if( maxHKWidth < hw ) maxHKWidth = hw;
    }



    double padding = chkprop(_T("padding"),2.0,GetDouble);
    wxColor border_color = chkpropcolor(_T("borderColor"),wxColor(0,0,0,255));
    double border_width = chkprop(_T("borderWidth"),2.0,GetDouble);
    double border_radius = chkprop(_T("borderRadius"),3.0,GetDouble);

    wxColor font_color = chkpropcolor(_T("textColor"),wxColor(0,0,0,255));
    wxColor bgcolor = chkpropcolor(_T("backgroundColor"),wxColor(255,255,255,200));

    wxAggPen* pen = aggCanvas.getPen();
    wxAggBrush* brush = aggCanvas.getBrush();

    pen->SetColor(border_color);
    pen->SetWidth(border_width);
    brush->SetColor(bgcolor);
    brush->SetStyle(wxAGGBRUSH_SOLID);


    wxRealRect rsize(0,0, 4*padding + 2*border_width + maxTitleWidth + maxHKWidth, 2*padding+2*border_width+ arr.GetCount()*(text_height) );
    rsize.Inflate(3*padding+2*border_width, 2*padding+2*border_width);


    rsize.SetTopLeft(0,0);

    wxRealRect bgrect = rsize.GetOuterBox();
    bgrect.Inflate(border_width/2);

    aggCanvas.drawRoundedRectangle( bgrect, border_radius );

    pen->SetColor( font_color );

    for( size_t a=0; a < arr.GetCount(); a++ ) {

        double ty = (a+1) * text_height * 1.15 + border_width + padding;

        aggCanvas.drawTTFText( arr[a].Title, border_width + padding, ty);
        aggCanvas.drawTTFText( arr[a].HotKey, border_width + padding + maxTitleWidth + 2*padding, ty);

    }

    return aggCanvas.renderToBitmap(true,rsize.GetOuterBox());
}
