#ifndef INTELHEX_H
#define INTELHEX_H

#include "common/typedef.h"

#define MAX_LINE_LENGTH 16
#define MAX_LINE_FULL_LENGTH 32

#define lo8(x)   (x & 0xFF)
#define hi8(x)   ((x >> 8) & 0xFF)


#define LINEBUFSIZE 128
#define INITIAL_INVALID_PAGEIDX -1

// 256 page bytes;
#define PAGE_SIZE_BITS (8)
#define PAGESIZEBYTES (1<<PAGE_SIZE_BITS)
#define PAGESIZEWORDS (PAGESIZEBYTES>>1)

// 128KByte = 2^17
// 1Page=256Byte= 2^8
// FlashPageNUM= 2^9
#define FLASH_PAGE_NUM 512

struct line_t {
	uint32_t addressB;
	uint8_t rawBytes[ MAX_LINE_FULL_LENGTH ];
	uint8_t rawLength;
	// uint8_t dataStartIdx; // constant ...
	// uint8_t dataEndIdx;
};

// TODO: malloc only as much as necessary
struct pagebuffer_t {
	uint16_t extended_address;	// TODO:
	uint32_t base_address;	// Byte address!!
	uint8_t buf[512];
	uint16_t len;
	uint16_t checksum;
	uint8_t already_printed;
};

class IntelHex
{
    public:
        IntelHex();
        virtual ~IntelHex();
        void chomp( char * str );
        int isComment( char * str );
        char * strclone( char * line );
        uint8_t to_hex_char(uint8_t value);
        uint8_t hexchr_val( uint8_t hexchr );
        int getLineBytes (char * line, struct line_t *lineBuf );
        uint8_t isChecksumError( struct line_t *lineBuf );
        uint32_t getLineAddress( struct line_t *lineBuf );
        int parseLine(uint8_t * line, struct line_t *lineBuf);
        void printPage( struct pagebuffer_t *pageBuffer);
        void initPageBuffer(struct pagebuffer_t * pageBuffer);
        void initAddresOffset();
        void fillPageBuffer(struct pagebuffer_t *pageBuffer, struct line_t *lineBuf);
        uint16_t getPageIdx(uint32_t addressB);
    protected:
    private:
        uint32_t addressOffset;


};



#endif // INTELHEX_H

