#ifndef CALCMODELANALYZER_H
#define CALCMODELANALYZER_H

#include <wx/wx.h>
#include <wx/grid.h>
#include <wx/progdlg.h>

#include "LogHistoryBase.h"
#include "LogHistoryMgr.h"

#include "../common/ProcessBase.h"
#include "../common/types/Int2DoubleHash.h"
#include "../rtdatamanager/RTDataAnalyzerBase.h"
#include "../Observable.h"
#include "../ObserverBase.h"
#include "../Symbol.h"

class CalcSymbol {
public:
    wxString    name;
    double      minlimit;
    double      maxlimit;
};

class DiffT {
public:
    int     rtidx;
    double  value;
};

WX_DECLARE_OBJARRAY( CalcSymbol, CalcSymbolA );
WX_DECLARE_OBJARRAY( DiffT, DiffTA );

WX_DECLARE_STRING_HASH_MAP( DiffTA, String2DiffTAHash );


class CalcModelAnalyzer : public RTDataAnalyzerBase
{
    public:
        CalcModelAnalyzer( SymbolTable* symTable );
        virtual ~CalcModelAnalyzer();

        String2DiffTAHash   getResult() { return m_diffs; }

    protected:
        void    processData();

    private:
        CalcSymbolA         m_symbols;
        String2DiffTAHash   m_diffs;
};

class CalcModelAnalyzerProcess : public ProcessBase, public Observable, public ObserverBase {
    public:
        CalcModelAnalyzerProcess();
        ~CalcModelAnalyzerProcess();

        bool                process();
        void                reset() {}
        void                start();
        String2DiffTAHash   getResult() { return m_diffs; }

        void notify( int i, void* classType );

        wxString            getLogFileName();

    private:
        String2DiffTAHash   m_diffs;

        RTDataAnalyzerBase* m_analyzer;
        LogHistoryMgr*      m_mgr;
        bool                m_isrunning;

        void    finished();
};

class CalcModelAnalyzerViewer : public wxFrame, public ObserverBase {
    public:
        CalcModelAnalyzerViewer( wxWindow* parent, const wxString& caption );
        ~CalcModelAnalyzerViewer();

        void    setLogHistory( LogHistoryBase* loghistory );

        void calculate();

        void notify( int i );
        void notify( int i, int idx );
        void notify( const wxString& str );

    private:
        wxComboBox*                 m_symbolnames;
        wxGrid*                     m_table;
        CalcModelAnalyzerProcess*   m_process;
        String2DiffTAHash           m_result;

        wxMenu*                     m_popupMenu;

        LogHistoryBase*             m_lh;

        wxProgressDialog*           m_dlg;

        void    OnSelectSymbol( wxCommandEvent& event );
        void    OnRightClick( wxMouseEvent& event );
        void    OnDoubleClickCell( wxGridEvent& event );

        void    Save( wxCommandEvent& event );

        DECLARE_EVENT_TABLE()
};

#endif // CALCMODELANALYZER_H
