#include "ExpressionPanel.h"

#include "ExpressionSQL.h"
#include "../logger.h"

const int idComboId  = wxNewId();

const int idValidBtt = wxNewId();
const int idSaveBtt = wxNewId();
const int idDelexpr = wxNewId();

const int idSymbols = wxNewId();
const int idAddsymb = wxNewId();

class PageBase : public wxPanel
{
    public:
        PageBase(wxWindow* parent, wxWindowID id = wxID_ANY, const wxPoint& pos = wxDefaultPosition,
            const wxSize& size = wxDefaultSize, long style = wxTAB_TRAVERSAL, const wxString& name = _T("panel"));
        wxComboBox*     getComboBox() { return m_combobox; }
        wxTextCtrl*     getTextCtrl() { return m_textctrl; }
        virtual void    refreshCombo() = 0;
    protected:
        wxComboBox*     m_combobox;
        wxTextCtrl*     m_textctrl;
        virtual void    buildGUI() = 0;
        virtual void    OnComboBox( wxCommandEvent& event ) = 0;

        DECLARE_EVENT_TABLE()

    private:
};

PageBase::PageBase(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style, const wxString& name)
    : wxPanel(parent, id, pos, size, style, name)
{
}

BEGIN_EVENT_TABLE( PageBase, wxPanel )
    EVT_COMBOBOX ( idComboId, PageBase::OnComboBox )
END_EVENT_TABLE();

/// FilterPage

class FilterPage : public PageBase
{
    public:
        FilterPage(wxWindow* parent, wxWindowID id = wxID_ANY, const wxPoint& pos = wxDefaultPosition,
            const wxSize& size = wxDefaultSize, long style = wxTAB_TRAVERSAL, const wxString& name = _T("panel"));
        void refreshCombo();
    private:
        void buildGUI();
        void OnComboBox( wxCommandEvent& event );
};

FilterPage::FilterPage(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style, const wxString& name) :
    PageBase(parent, id, pos, size, style, name)
{
    buildGUI();
    refreshCombo();
}

void FilterPage::buildGUI()
{
    wxFlexGridSizer* sizer = new wxFlexGridSizer(2,5,5);
    wxStaticText* txt = new wxStaticText(this, -1, _T("Filter name"));
    m_combobox = new wxComboBox(this, idComboId, _T(""), wxDefaultPosition, wxDefaultSize, 0, NULL, wxCB_SORT | wxCB_DROPDOWN );

    wxStaticText* txt1 = new wxStaticText(this, -1, _T("Filter expression"));
    m_textctrl = new wxTextCtrl(this, -1, _T(""), wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER | wxTE_MULTILINE | wxTE_BESTWRAP );

    sizer->Add(txt, 0, wxALL, 5); sizer->Add( m_combobox, 1, wxALL | wxEXPAND, 5);
    sizer->Add(txt1, 0, wxALL, 5); sizer->Add( m_textctrl, 1, wxALL | wxEXPAND, 5);
    sizer->AddGrowableCol(1);
    sizer->AddGrowableRow(1);

    SetSizer(sizer);

    sizer->Fit(this);
    sizer->SetSizeHints(this);
}

void FilterPage::refreshCombo()
{
    StringHashA arr = ExpressionSQL::getFilters(true);
    wxArrayString names;
    for ( size_t i = 0; i < arr.GetCount(); ++i )
    {
        StringHash* h = arr[i];
        names.Add((*h)[_T("name")]);
    }
    WX_CLEAR_ARRAY( arr );
    m_combobox->Clear();
    m_combobox->Append( names );
}

void FilterPage::OnComboBox( wxCommandEvent& event )
{
    wxString result = ExpressionSQL::getFilterByName( m_combobox->GetValue() );
    if ( result != wxEmptyString )
        m_textctrl->SetValue( result );
}

/// ExpressionPage

class ExpressionPage : public PageBase
{
    public:
        ExpressionPage(wxWindow* parent, wxWindowID id = wxID_ANY, const wxPoint& pos = wxDefaultPosition,
            const wxSize& size = wxDefaultSize, long style = wxTAB_TRAVERSAL, const wxString& name = _T("panel"));
        void            refreshCombo();
    private:
        void buildGUI();
        void OnComboBox( wxCommandEvent& event );
};

ExpressionPage::ExpressionPage(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style, const wxString& name) :
    PageBase(parent, id, pos, size, style, name)
{
    buildGUI();
    refreshCombo();
}

void ExpressionPage::buildGUI()
{
    wxFlexGridSizer* sizer = new wxFlexGridSizer(2,5,5);
    wxStaticText* txt = new wxStaticText(this, -1, _T("Expression name"));
    m_combobox = new wxComboBox(this, idComboId, _T(""), wxDefaultPosition, wxDefaultSize, 0, NULL, wxCB_SORT | wxCB_DROPDOWN );

    wxStaticText* txt1 = new wxStaticText(this, -1, _T("Expression"));
    m_textctrl = new wxTextCtrl(this, -1, _T(""), wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER | wxTE_MULTILINE | wxTE_BESTWRAP );

    sizer->Add(txt, 0, wxALL, 5); sizer->Add( m_combobox, 1, wxALL | wxEXPAND, 5);
    sizer->Add(txt1, 0, wxALL, 5); sizer->Add( m_textctrl, 1, wxALL | wxEXPAND, 5);
    sizer->AddGrowableCol(1);
    sizer->AddGrowableRow(1);

    SetSizer(sizer);

    sizer->Fit(this);
    sizer->SetSizeHints(this);
}

void ExpressionPage::refreshCombo()
{
    StringHashA arr = ExpressionSQL::getExpressions(true);
    wxArrayString names;
    for ( size_t i = 0; i < arr.GetCount(); ++i )
    {
        StringHash* h = arr[i];
        names.Add((*h)[_T("name")]);
    }
    WX_CLEAR_ARRAY( arr );
    m_combobox->Clear();
    m_combobox->Append( names );
}

void ExpressionPage::OnComboBox( wxCommandEvent& event )
{
    wxString result = ExpressionSQL::getExpressionByName( m_combobox->GetValue() );
    if ( result != wxEmptyString )
        m_textctrl->SetValue( result );
}

// END OF PAGES
////////////////////////////////////////////////////////////////////////////


BEGIN_EVENT_TABLE( ExpressionPanel, wxPanel )

    EVT_BUTTON ( idValidBtt, ExpressionPanel::OnValidate )
    EVT_BUTTON ( idSaveBtt, ExpressionPanel::OnSave )
    EVT_BUTTON ( idAddsymb, ExpressionPanel::OnAdd )
    EVT_BUTTON ( idDelexpr, ExpressionPanel::OnDelete )

END_EVENT_TABLE()

ExpressionPanel::ExpressionPanel(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size,
    long style, const wxString& name) : wxPanel( parent, id, pos, size, style, name)
{
    m_notebook = new wxNotebook( this, -1 );
    m_filterpg = new FilterPage( m_notebook, -1 );
    m_exprpg = new ExpressionPage( m_notebook, -1 );

    m_notebook->AddPage( m_filterpg, _T("Filters"), true );
    m_notebook->AddPage( m_exprpg, _T("Expressions") );

    wxBoxSizer* mainsizer = new wxBoxSizer( wxVERTICAL );

    wxBoxSizer* symsiz = new wxBoxSizer( wxHORIZONTAL );

    wxStaticText* txt = new wxStaticText( this, -1, _T("Available symbols:") );
    wxArrayString symbols = ExpressionSQL::getSymbols();
    m_symbolcombo = new wxComboBox( this, idSymbols, _T(""), wxDefaultPosition, wxDefaultSize, symbols, wxCB_READONLY | wxCB_SORT );
    wxButton* addbutton = new wxButton( this, idAddsymb, _T("Add to editor"));

    symsiz->Add( txt, 0, wxEXPAND | wxALL, 5);
    symsiz->Add( m_symbolcombo, 0, wxEXPAND | wxALL, 5);
    symsiz->Add( addbutton, 0, wxALL, 5);

    // buttons
    wxBoxSizer* btnsizer = new wxBoxSizer( wxHORIZONTAL );
    wxButton* validatebutt = new wxButton(this, idValidBtt, _T("Validate"));
    wxButton* save = new wxButton(this, idSaveBtt, _T("Save"));
    wxButton* delbutt = new wxButton(this, idDelexpr, _T("Delete"));
    btnsizer->Add( validatebutt, 0, wxEXPAND | wxALL| wxALIGN_CENTER, 5);
    btnsizer->Add( save, 0, wxEXPAND | wxALL| wxALIGN_CENTER, 5);
    btnsizer->Add( delbutt, 0, wxEXPAND | wxALL| wxALIGN_CENTER, 5);

    mainsizer->Add( m_notebook, 1, wxEXPAND | wxALL, 5 );
    mainsizer->Add( symsiz, 0, wxEXPAND | wxALL, 5);
    mainsizer->Add( btnsizer, 0, wxEXPAND | wxALL | wxALIGN_CENTER, 5);


    SetSizer(mainsizer);

    mainsizer->Fit(this);
    mainsizer->SetSizeHints(this);
}

void ExpressionPanel::OnValidate( wxCommandEvent& event )
{
    event.Skip();
}

void ExpressionPanel::OnAdd( wxCommandEvent& event )
{
    if ( m_symbolcombo->GetValue() != wxEmptyString )
    {
        PageBase* page = (PageBase*)m_notebook->GetCurrentPage();
        page->getTextCtrl()->WriteText( m_symbolcombo->GetValue() );
    }
    event.Skip();
}

void ExpressionPanel::OnSave( wxCommandEvent& event )
{
    bool isFilter = ( m_notebook->GetSelection() == 0);

    PageBase* page = (PageBase*)m_notebook->GetCurrentPage();

    wxString ename = page->getComboBox()->GetValue();

    if ( ename == wxEmptyString )
    {
        wxMessageBox( _T("Enter a name first"), _T("Error"), wxOK | wxICON_EXCLAMATION );
        event.Skip();
    }
    else{
        wxString value = page->getTextCtrl()->GetValue();

        if ( isFilter )
        {
            if ( ExpressionSQL::saveFilter(ename, value) )
            {
                wxMessageBox( _T("Filter has been saved as ") + ename, _T("Information"), wxOK );
                if ( page->getComboBox()->FindString(ename) == wxNOT_FOUND )
                    page->getComboBox()->Append( ename );
            }
            else
            {
                wxMessageBox(_T("Filter has not been saved"), _T("Error"), wxOK | wxICON_HAND );
            }
        }
        else
        {
            if ( ExpressionSQL::saveExpression(ename, value) )
            {
                wxMessageBox( _T("Expression has been saved as ") + ename, _T("Information"), wxOK );
                if ( page->getComboBox()->FindString(ename) == wxNOT_FOUND )
                    page->getComboBox()->Append( ename );
            }
            else
            {
                wxMessageBox(_T("Expression has not been saved"), _T("Error"), wxOK | wxICON_HAND );
            }
        }
    }

    event.Skip();
}

void ExpressionPanel::OnDelete( wxCommandEvent& event )
{
    PageBase* page = (PageBase*)m_notebook->GetCurrentPage();

    wxString ename = page->getComboBox()->GetValue();

    if ( ename != wxEmptyString )
    {
        ExpressionSQL::deleteExpression( ename );
        int exprPos  = page->getComboBox()->FindString(ename);
        if ( exprPos != wxNOT_FOUND )
            page->getComboBox()->Delete( exprPos );
    }
    else
    {
         wxMessageBox( _T("Select a name first"), _T("Error"), wxOK | wxICON_EXCLAMATION );
    }
    event.Skip();
}

