#ifndef LOGHISTORYMGR_H
#define LOGHISTORYMGR_H

#include "LogHistoryBase.h"
#include "../vemsprotocol/VEMSLogBuffer.h"
#include "../vemsprotocol/SIPRDescriptor.h"
#include "../Symbol.h"
#include "../SymbolMgr.h"
#include "../common/types/DoubleHashArray.h"

#include "../stdatamanager/STDataModelSerial.h"
#include "../common/types/Int2DoubleHashHash.h"

class RT2ViewerConverter;
class SIPRDescriptor;

class LogHistoryMgr: public LogHistoryBase
{
    public:
        LogHistoryMgr( RTDataMgrBase* rtMgr, STDataMgrBase* stMgr, SymbolMgr* symbolMgr );

        /// only for analyzeHistory()
        LogHistoryMgr( const LogHistoryMgr& mgr );

        virtual ~LogHistoryMgr();

        // LogHistoryBase virtual methods

        void                clear();

        void                getHistory( const IntArray& idxA, DoubleHashArray& viewerData );
        void                getHistory( size_t startIdx, size_t endIdx, DoubleHashArray& viewerData );
        void                emptyHistory();

        /// analyzeHistory, getGPS - needs LHM copy ctor, RTDataViewer to process
        void                analyzeHistory();

        bool                hasGPSdata();
        void                getGPSData( DoubleHashArray& gpsdata );

        void                getMarkers( IntArray& idxA, uint8_t markerType );
        size_t              getHistoryCount( size_t pType = IDX_RTDATA_NORMAL );

        size_t              getMdxFrameMarkersCount();
        void                getMdxSiprPackets( size_t idx, SIPRPacket*** packets, unsigned int& numberOfPacketsInFrame );
        void                getMdxSiprIndices( size_t idx, IntArray& indices );

        void                refreshLastPosition();
        void                refresh( size_t idx, bool notifySlider = true );
        void                process(const SIPRPacket& p);

        void                processFilter(size_t startIdx, size_t endIdx);
        void                setPacketDelay( long delay );
        wxString            getLogFileName();

        // ObserverBase virtual methods

        void                notify( int i );

        // other methods

        void                runGPSData();
        bool                loadHistoryFromLog(const wxString& filename);

        size_t              getNextPacket(SIPRPacket** packet);
        size_t              getPacket( size_t idx, SIPRPacket** packet );
        void                setPosition(unsigned int position) { m_vemslogbuffer->setPosition(position); }

        StringHash          getHeaderHash() { return m_headerHash; }
        long int            getLogVersion() { return m_logversion; }

        //VEMSLogBuffer*      getVemsLogBuffer() { return m_vemslogbuffer; }

        void                initST(const SIPRDescriptor& siprDesc);
        void                enableSlider( bool enable = true );

        long int            getIdxByRTIdx( size_t idx );

    protected:

    private:
        bool                m_fromlog;
        wxString            m_logFileName;
        RT2ViewerConverter* m_converter;
        VEMSLogBuffer*      m_vemslogbuffer;
        DoubleHash          m_viewerModel;
        StringHash          m_headerHash;
        long int            m_logversion;
        STDataModelBase*    m_stdatamodel;
        SymbolTable         m_symboltable;

        Int2DoubleHashHash  m_cache;

        size_t              m_oldidx;

        long                m_packetdelay;

        long                m_st_belonging2rtidx;

        static unsigned     m_counter;
        SIPRPacket*         m_prevGPSPtr;

        DoubleArray         m_gpsDistance;
        bool                m_gpscalculated;
        bool                m_canSliderEnabled;

        int                 m_currentIdx;

        void            purgeCache();
        void            updateConfigModel(size_t idx);
        void            updateViewerModelWithConfig();
        DoubleHash*     getDHData( int idx, const DoubleHash& requiredH );
        void            calculateModel(size_t idx,  DoubleHash** data);
        void            calcgpsdist();
        // delay in ms-> idx
        size_t          findPacketByDelay( size_t startIdx, long delay );
};

#endif // LOGHISTORYMGR_H
