#include "mtconfigdata.h"

#include "vemsdialog/UserMenu.h"
#include "vemsdialog/UserDialog.h"
#include "logger.h"

DataLogEntry::DataLogEntry(wxString name, wxString label, wxString type, wxString format)
    :m_name (name)
    ,m_label (label)
    ,m_type (type)
    ,m_format (format)
{
    //ctor
}

MTConfigData::MTConfigData()
{
    m_isValid = false;

    m_UserMenuList = new UserMenuList();
    m_UserDialogList = new UserDialogList();
}

MTConfigData::~MTConfigData()
{
    clear();
    if(m_UserMenuList) delete m_UserMenuList;
    if(m_UserDialogList) delete m_UserDialogList;
}

void MTConfigData::addSymbol(Symbol* symbol)
{
    m_symbolMgr.addSymbol(symbol);
}

Symbol* MTConfigData::getSymbol(const wxString& symbolName)
{
    return m_symbolMgr.getSymbol(symbolName);
}

int MTConfigData::getSymbolPage(const wxString& symbolName)
{
    return m_symbolMgr.getSymbolPage(symbolName);
}

wxString MTConfigData::getSymbolType(const wxString& symbolName)
{
    return m_symbolMgr.getSymbolType(symbolName);
}


void MTConfigData::addDataLogEntry(wxString name, wxString label, wxString type, wxString format)
{
    m_DataLogEntryArray.Add(new DataLogEntry(name,label,type,format));
}


void MTConfigData::addGroup( const wxString& name )
{
    m_symbolMgr.addGroup(name);
}


void MTConfigData::setGroupOffset( const wxString& name, int offset )
{
    m_symbolMgr.setGroupOffset(name, offset);
}


SymbolGroup* MTConfigData::getGroup( const wxString& groupName )
{
    return m_symbolMgr.getGroup(groupName);
}


void MTConfigData::clear()
{
    //m_userDataDescA.Clear();

    WX_CLEAR_HASH_MAP( InterpolationWidgetDescH, m_interpolationWidgets );
    WX_CLEAR_ARRAY( m_userDataDescA );

    WX_CLEAR_HASH_MAP( VemsCurveDialogDescH, m_curveDialogs );
    WX_CLEAR_HASH_MAP( SuperCurveDescriptorH, m_superCurveDialogs );
    m_symbolMgr.clear();
    m_symbolMgr.m_expTable.clear();
    WX_CLEAR_HASH_MAP( VemsTableDescH, m_tables );
    WX_CLEAR_HASH_MAP( TblMapDescHash, m_tableMappers );

    //m_DataLogEntryArray.Clear();
    WX_CLEAR_ARRAY( m_DataLogEntryArray );

    m_symbolMgr.m_dag.clear();

    m_UserDialogList->clear();
    m_UserMenuList->clear();

    m_signature.Clear();
    m_SIPRDescriptor.clear();
    m_params.clear();

    m_menuAndDialogNamesH.clear();

    //WX_CLEAR_HASH_MAP( Int2Int2StringSetHash, m_dialogSetByPageAndOffset );

/// TODO: cut this:
/*
	Int2Int2StringSetHash::const_iterator itPage = m_dialogSetByPageAndOffset.begin();

    for ( ; itPage != m_dialogSetByPageAndOffset.end(); ++itPage )
    {
        Int2StringSetHash* offsetHash = itPage->second;

        Int2StringSetHash::const_iterator itOffset = offsetHash->begin();

        for ( ; itOffset != offsetHash->end(); ++itOffset )
        {
            StringSet* dialogNameSet = itOffset->second;

			delete dialogNameSet;
        }
		offsetHash->clear();

        delete offsetHash;
    }

	m_dialogSetByPageAndOffset.clear();
*/
// <----

    Int2Int2Int2StringSetHash::const_iterator it = m_dialogSetByPageAndOffsetAndMask.begin();

    for ( ; it != m_dialogSetByPageAndOffsetAndMask.end(); ++it )
    {
        Int2Int2StringSetHash ish = it->second;

        Int2Int2StringSetHash::const_iterator itPage = ish.begin();

        for ( ; itPage != ish.end(); ++itPage )
        {
            Int2StringSetHash* offsetHash = itPage->second;

            Int2StringSetHash::const_iterator itOffset = offsetHash->begin();

            for ( ; itOffset != offsetHash->end(); ++itOffset )
            {
                StringSet* dialogNameSet = itOffset->second;

                delete dialogNameSet;
            }
            offsetHash->clear();

            delete offsetHash;
        }

        ish.clear();
    }

    m_dialogSetByPageAndOffsetAndMask.clear();

/*
    Int2StringSetHash* offsetHash = NULL;
    Int2Int2StringSetHash::const_iterator in2intIT = m_dialogSetByPageAndOffset.begin();
    for(; in2intIT != m_dialogSetByPageAndOffset.end() ; ++in2intIT){
        offsetHash = in2intIT->second;
        delete offsetHash;

    }
    //Int2Int2StringSetHash    m_dialogSetByPageAndOffset;
*/

    WX_CLEAR_ARRAY( m_expressionGridDescriptors );
}


const StringSet* MTConfigData::getDialogNamesBySymbol(const Symbol& symbol) const
{
/// TODO: cut this
/*
    int pageNo = symbol.getPage();
    int offset = symbol.getOffset();
    //Logger::logf(LOG_MSG,_T("MTConfigData::getDialogNamesBySymbol() pageNo:%d, offset:%d"), pageNo, offset);
    Int2Int2StringSetHash::const_iterator in2intIT = m_dialogSetByPageAndOffset.find(pageNo);
    if(in2intIT == m_dialogSetByPageAndOffset.end()) return NULL;

    Int2StringSetHash* offsetHash = in2intIT->second;
    //Logger::logf(LOG_MSG,_T("MTConfigData::getDialogNamesBySymbol() 10"));
    Int2StringSetHash::const_iterator intIT = offsetHash->find(offset);
    if(intIT == offsetHash->end() ) return NULL;

    //Logger::logf(LOG_MSG,_T("MTConfigData::getDialogNamesBySymbol() 20"));
    StringSet* dialogNameSet =  intIT->second;
    return dialogNameSet;
*/
//<---------------

    int page = symbol.getPage();
    int offset = symbol.getOffset();
    int bitmask = symbol.getBitMask();

    Int2Int2Int2StringSetHash::const_iterator page2offset2mask2name_it = m_dialogSetByPageAndOffsetAndMask.find( page );

    if ( page2offset2mask2name_it == m_dialogSetByPageAndOffsetAndMask.end() )
        return NULL;

    Int2Int2StringSetHash offset2mask2nameH = page2offset2mask2name_it->second;

    Int2Int2StringSetHash::const_iterator off2mask2nameH_it = offset2mask2nameH.find( offset );

    if ( off2mask2nameH_it == offset2mask2nameH.end() )
        return NULL;

    Int2StringSetHash* mask2nameHash = off2mask2nameH_it->second;
    Int2StringSetHash::const_iterator mask2nameHash_it = mask2nameHash->find( bitmask );

    if ( mask2nameHash_it == mask2nameHash->end() )
        return NULL;

    StringSet* dialogNameSet = mask2nameHash_it->second;

    return dialogNameSet;
}

void MTConfigData::fillDialogNamesBySymbol(const wxString& currentDialog, const wxString& symbolName)
{
/// TODO: cut this
/*    SymbolTable& symbolTable = getSymbolTable();
    //Logger::logf(LOG_MSG,_T("MTConfigData::fillDialogNamesBySymbol() START symbolsize:%d,")+currentDialog+ _T(",")+ symbolName, symbolTable.size() );
    SymbolTable::const_iterator IT = symbolTable.find(symbolName);
    if(IT == symbolTable.end()) return;
    const Symbol* symbol = IT->second;

    int pageNo = symbol->getPage();
    int offset = symbol->getOffset();

    //Logger::logf(LOG_MSG,_T("MTConfigData::fillDialogNamesBySymbol() symbol: ")+symbolName +_T(" pageNo:%d,offset:%d"),pageNo, offset);
    Int2Int2StringSetHash::const_iterator in2intIT = m_dialogSetByPageAndOffset.find(pageNo);
    Int2StringSetHash* offsetHash;
    if(in2intIT == m_dialogSetByPageAndOffset.end()){
        offsetHash = new Int2StringSetHash();
        m_dialogSetByPageAndOffset[pageNo] = offsetHash;
    }else{
        offsetHash = in2intIT->second;
    }

    Int2StringSetHash::const_iterator intIT = offsetHash->find(offset);
    StringSet* dialogNameSet;
    if(intIT == offsetHash->end() ){
        dialogNameSet = new StringSet();
        (*offsetHash)[offset] = dialogNameSet;
    }else{
        dialogNameSet = intIT->second;
    }

    dialogNameSet->insert( currentDialog );
*/
// <---

    SymbolTable& symbolTable = getSymbolTable();

    SymbolTable::const_iterator IT = symbolTable.find(symbolName);

    if( IT == symbolTable.end() )
        return;

    const Symbol* symbol = IT->second;

    int pageNo = symbol->getPage();
    int offset = symbol->getOffset();
    int bitmask = symbol->getBitMask();

    Int2Int2Int2StringSetHash::iterator p2o2m2n_it = m_dialogSetByPageAndOffsetAndMask.find( pageNo );
    Int2StringSetHash* maskHash;
    if ( p2o2m2n_it == m_dialogSetByPageAndOffsetAndMask.end() )
    {
        // page not found
        maskHash = new Int2StringSetHash();
        Int2Int2StringSetHash off2mask2name;

        off2mask2name[offset] = maskHash;

        m_dialogSetByPageAndOffsetAndMask[pageNo] = off2mask2name;
}
    else {
        // we got page, look for offset
        Int2Int2StringSetHash& off2mask2name = p2o2m2n_it->second;
        Int2Int2StringSetHash::const_iterator o2m2n_it = off2mask2name.find( offset );
        if ( o2m2n_it == off2mask2name.end() )
        {
            // offset not found
            maskHash = new Int2StringSetHash();
            off2mask2name[offset] = maskHash;
        }
        else
        {
            // we got the offset
            maskHash = o2m2n_it->second;
        }
    }
    //
    Int2StringSetHash::const_iterator intIT = maskHash->find(bitmask);

    StringSet* dialogNameSet;

    if ( intIT == maskHash->end() )
    {
        dialogNameSet = new StringSet();
        (*maskHash)[bitmask] = dialogNameSet;
    }
    else
    {
        dialogNameSet = intIT->second;
    }

    dialogNameSet->insert( currentDialog );
}

Symbol* MTConfigData::getCfgSymbol(const wxString& symbolName)
{
    SymbolTable& cfgSymbolTable = m_symbolMgr.getCfgSymbolTable();

    SymbolTable::const_iterator itSymTbl = cfgSymbolTable.find( symbolName );

    if( itSymTbl != cfgSymbolTable.end() ){
        return itSymTbl->second;
    }else{
        return NULL;
    }
}

wxString MTConfigData::getMenuAndDialogNameByDialogId(const wxString& id) const
{
    StringHash::const_iterator IT = m_menuAndDialogNamesH.find( id );

    if( IT != m_menuAndDialogNamesH.end() ){
        return IT->second;
    }else{
        return wxEmptyString;
    }

}

void MTConfigData::fillMenuAndDialogNamesByDialogId(const wxString& dialogId, const wxString& name )
{
    m_menuAndDialogNamesH[dialogId] = name;
}


bool MTConfigData::isRawLambdaUsed()
{
    BoolHash::const_iterator it = m_params.find( _T("USE_RAW_LAMBDA_TABLE") );

    if ( it == m_params.end() ) { return false; }

    return it->second;
}

ExpressionGridDesc* MTConfigData::getExpressionGridDescriptor( const wxString& name )
{
    for ( size_t i = 0; i < m_expressionGridDescriptors.GetCount(); ++i )
    {
        if ( m_expressionGridDescriptors[i]->name.IsSameAs( name ) ) return m_expressionGridDescriptors[i];
    }

    return NULL;
}
