#ifndef PROJECTMANAGER_H
#define PROJECTMANAGER_H

#include <wx/wx.h>
#include <wx/xml/xml.h>
#include "ProjectConfigSaver.h"
#include "../common/types/IntArray.h"
#include "../FirmwareInfo.h"

class Project;
class FirmwareConfigItem;
class FirmwareConfigItemArray;
class MenuRefresher;

WX_DECLARE_STRING_HASH_MAP( wxXmlNode*, wxXmlNodeHash );

WX_DEFINE_ARRAY(Project*, ProjectArray);

WX_DEFINE_ARRAY(MenuRefresher*, MenuRefresherArray);

class ProjectManager
{
    public:
        ProjectManager();
        virtual ~ProjectManager();


        Project* getProject(const wxString& filename);
        Project* AddProject(const wxString& filename);
        bool AddProject(Project * prj);

        bool InsertProject(const wxString& filename,int idx);
        bool InsertProject(Project * prj,int idx);

        bool MoveProject(Project * prj, int idx);

        bool SetCurrPrj(const wxString& name) {
            m_currPrj = GetProjectByName(name);           // Logger::logf(LOG_MSG,_T("m_currPrj=%p"),m_currPrj);
            if(m_currPrj)
                return true;
            else
                return false;
        };

        void SetCurrPrj(Project * prj);

        Project* GetCurrPrj() { return m_currPrj; };

        Project * GetProjectByName(const wxString& name);
        Project * GetProjectByPath(const wxString& path);
        Project * GetProjectByMenuID(int id);
        Project * GetProjectByIdx(int idx) { return m_projects[idx]; };

        ProjectArray* GetProjects() { return &m_projects; };

        void SetMainStatusBar(wxStatusBar * sbar) { m_mainStatusBar = sbar; };

        void UpdateStatusBar();

        void SaveRecents();

        void AddMenuRefresher(MenuRefresher * mr) { m_menuRefreshers.Add(mr); };

        void RefreshMenus();

    protected:

        void LoadRecents();
        wxString GetProjectName(const wxString& path);

    private:



        Project*        m_currPrj;
        ProjectArray    m_projects;

        wxStatusBar *   m_mainStatusBar;

        wxString        m_recentsFile;

        MenuRefresherArray  m_menuRefreshers;

};


class MenuRefresher
{
    public:
        MenuRefresher() {};
        ~MenuRefresher() {};

    wxMenu * Menu;
    IntArray IDs;



};

/*********************************************************
 *                 Project declarations
 *********************************************************/

class Project: public ProjectConfigSaver
{

    public:
        Project() { MenuId = -1; };
        virtual ~Project() {};

        wxXmlDocument   Doc;
        wxString        Name;
        wxString        Path;
        long            Type;
        int             MenuId;

        FirmwareConfigItemArray CreateFirmwareConfigItems(wxArrayString &types);
        wxArrayString GetConfigFiles();
        wxString GetConfigIniFile();
        void SetFwVers(const wxString& str);

        virtual bool SaveConfig(const FirmwareConfigItemArray  &arr);

        void Set();
        void SaveSettings();
        const FirmwareInfo& getFirmwareInfo(){return m_fwInfo; }

    protected:


        FirmwareConfigItem * FillFwCfgItem(wxXmlNode *itemNode, wxArrayString &types, FirmwareConfigItem *defitem);

        bool SaveFirmwareConfigItem(FirmwareConfigItem *item);

        bool SetItemNodeProperties(wxXmlNode* itemNode, const wxString& value, bool isActive );


    private:
        FirmwareInfo    m_fwInfo;

        bool AddFirmwareConfigItem(FirmwareConfigItem *beforeitem, FirmwareConfigItem *item);
        bool RemoveFirmwareConfigItem(FirmwareConfigItem *item);
        bool ModFirmwareConfigItem(FirmwareConfigItem *item);

};


#endif // PROJECTMANAGER_H
