#include "ProjectSettingsFrame.h"

#include "ProjectManager.h"
#include "ProjectChooserFrame.h"
#include "../logger.h"
#include "../wxXmlHelper.h"

#include <wx/gbsizer.h>
#include "../common/typedef.h"

int idButton1 = wxNewId();

BEGIN_EVENT_TABLE(ProjectSettingsFrame, wxDialog)

    EVT_BUTTON(idButton1, ProjectSettingsFrame::OnButton1)
    EVT_BUTTON(idSaveButton, ProjectSettingsFrame::OnButton)
    EVT_BUTTON(idCancelButton, ProjectSettingsFrame::OnButton)
    EVT_LISTBOX(idPrjList, ProjectSettingsFrame::OnListBoxSel)

END_EVENT_TABLE()

ProjectSettingsFrame::ProjectSettingsFrame(wxWindow *parent, Project *project, ProjectManager *prjmgr, ProjectNodeTypes type)
{
    //ctor
    m_prjMgr = prjmgr;

    long style = 0 |
        wxMINIMIZE_BOX |
        wxRESIZE_BORDER |
        wxSYSTEM_MENU |
        wxCAPTION |
        wxCLOSE_BOX |
        wxCLIP_CHILDREN;

/*
    if(!m_prjMgr) return;

    if(m_prjMgr->GetProjects()->GetCount() == 0) return;
*/
    if(!project) project = m_prjMgr->GetProjects()->Item(0);



    Create(parent, wxID_ANY, project->Name + _T(""), wxDefaultPosition, wxDefaultSize, style);


    m_type = type;

    m_projectList = new wxListBox(this, idPrjList);
    m_prjBook = new wxBookCtrl(this, idPrjBook);

    m_prjPath = new wxStaticText(this,wxID_ANY, _T(""));

//    wxButton * okbutt   = new wxButton(this, idOkButton, _T("Set to default"));
    wxButton * savebutt = new wxButton(this, idSaveButton, _T("Save"));

//    wxButton * addbutt  = new wxButton(this, idAddButton, _T("Create new..."));
    wxButton * cancbutt = new wxButton(this, idCancelButton, _T("Close"));


    wxBoxSizer * mainsizer = new wxBoxSizer(wxHORIZONTAL);
   // mainsizer->Add(m_prjBook, 1, wxEXPAND | wxALL, 5);


    mainsizer->Add(m_projectList, 0, wxALL|wxEXPAND, 5);
    mainsizer->SetItemMinSize(m_projectList,80,-1);

    wxBoxSizer * prjmainsizer = new wxBoxSizer(wxVERTICAL);

        prjmainsizer->Add(m_prjPath, 0, wxEXPAND | wxALL, 5);
        prjmainsizer->Add(m_prjBook, 1, wxEXPAND | wxALL, 5);
    //    prjmainsizer->Add(okbutt, 1, wxEXPAND | wxALL, 5);

        wxBoxSizer * buttsizer = new wxBoxSizer(wxHORIZONTAL);

        buttsizer->Add(savebutt, 0, wxALL , 5);
        buttsizer->Add(cancbutt, 0, wxALL , 5);

        prjmainsizer->Add(buttsizer, 0, wxALIGN_RIGHT);

    mainsizer->Add(prjmainsizer, 1, wxEXPAND|wxALL,0);

    //new wxButton(this, idButton1, _T("load"), wxPoint(200,10));


    Init( project );


    SetSizer(mainsizer);
	mainsizer->Fit(this);
	mainsizer->SetSizeHints(this);
}


ProjectSettingsFrame::~ProjectSettingsFrame()
{
    //dtor
}


void ProjectSettingsFrame::OnListBoxSel(wxCommandEvent &evt)
{

  //  Logger::logf(LOG_MSG,_T("%i"),evt.GetSelection());


    //LoadPrj( m_prjMgr->GetProjectByName( m_projectList->GetString(evt.GetSelection()) ) );
    LoadPrj( m_prjMgr->GetProjectByIdx(m_projectList->GetSelection()) );

}

void ProjectSettingsFrame::SetProjectToTitle(Project *prj)
{

    SetTitle(prj->Name + _T(" settings"));

}

void ProjectSettingsFrame::OnButton1(wxCommandEvent &evt)
{/*
    ProjectChooserFrame dlg;
    if(dlg.ShowModal() == wxID_OK)
        LoadPrj(dlg.GetProject());
*/
    //EnableTextControls(false);
    m_prjBook->GetPage(0)->Show(0);
    m_prjBook->RemovePage(0);

}


void ProjectSettingsFrame::OnButton(wxCommandEvent &evt)
{
    int id = evt.GetId();
    Project * prj;
    wxString str;
    wxXmlNode * node;

    switch(id){

        case idSaveButton:

            prj = m_prjMgr->GetProjectByPath(m_prjPath->GetLabel());

            node = wxXmlHelper::GetNodeByName( _T("set") , wxXmlHelper::GetNodeByName( _T("ProjectSettings") , prj->Doc.GetRoot()));

            str = m_prjContent[_T("ProjectSettings")]->Item(0)->find(_T("name"))->second->Value->GetValue();
            wxXmlHelper::SetPropVal(node, _T("name"), str);
            prj->Name = str;

            str = m_prjContent[_T("ProjectSettings")]->Item(0)->find(_T("firmwareversion"))->second->Value->GetValue();
            wxXmlHelper::SetPropVal(node, _T("firmwareversion"), str);
            prj->SetFwVers(str);

            prj->SaveSettings();

            m_prjMgr->UpdateStatusBar();
            //Logger::logf(LOG_MSG,_T("")+str);

            m_projectList->SetString(m_projectList->GetSelection(),prj->Name);

            SetProjectToTitle(prj);

            break;

        case idCancelButton:

            EndModal(wxID_OK); //yes, i know....

            break;

        default:
            break;

    }

}


void ProjectSettingsFrame::Init(Project *project)
{
    if(!m_prjMgr) return;

    ProjectArray * arr = m_prjMgr->GetProjects();

    if(arr->GetCount() == 0) return;

    for(uint16_t a=0; a<arr->GetCount();a++)
        m_projectList->Append(arr->Item(a)->Name);


    m_projectList->Select(0);

    LoadPrj(project,true);


}


void ProjectSettingsFrame::FillDlg(Project *project, bool firstload)
{


    SetProjectToTitle(project);

    int a=0;
    uint8_t itemCount = 0, propcnt = 0;

    PropertyGuiItemHash * pgih = NULL;
    wxPanel* panel = NULL;
    wxBoxSizer* panelsizer = NULL;
    wxGridBagSizer* itemsizer = NULL;

    /*int sel =*/ m_prjBook->GetSelection();

    if(!firstload) {
       // RemovePages(m_prjBook);

        EnableTextControls(false);
     //   EnablePages(false);

    }

    for(wxXmlNode* detailNode = project->Doc.GetRoot()->GetChildren(); detailNode ; detailNode = detailNode->GetNext())
    {
        if(detailNode->GetName() == _T("text") || detailNode->GetName() == _T("comment")) continue;

        if(m_type & PNT_MINIMAL) {
            if( detailNode->GetName() != _T("ProjectSettings")) continue;
        }

        if( m_prjContent.find(detailNode->GetName()) == m_prjContent.end() ) firstload = true;

    //    Logger::logf(LOG_MSG,_T("ProjectSettingsFrame::LoadPrj 1 = ")+detailNode->GetName());

        if(firstload) {
            m_prjContent[detailNode->GetName()] = new PropertyGuiItemHashArray();
            panel = new wxPanel(m_prjBook);
            m_pageHash[detailNode->GetName()] = panel;
            panelsizer = new wxBoxSizer(wxHORIZONTAL);
            m_prjBook->AddPage(m_pageHash[detailNode->GetName()], detailNode->GetName());
            a=0;

        } else {


         //   wxWindow * win = GetPageByName( detailNode->GetName(), m_prjBook);
           // if(win) win->Show(true);


        }


        //m_pageHash[detailNode->GetName()]->Show(true);

        itemCount = 0;

        for(wxXmlNode * itemNode = detailNode->GetChildren(); itemNode;itemNode = itemNode->GetNext() )
        {
            if(itemNode->GetName() == _T("text") || itemNode->GetName() == _T("comment")) continue;

       //     Logger::logf(LOG_MSG,_T("ProjectSettingsFrame::LoadPrj itemCount=%i, 2 = ")+itemNode->GetName(),itemCount);

            wxXmlProperty* properties = itemNode->GetProperties();

            if(firstload) {
                pgih = new PropertyGuiItemHash();
                m_prjContent[detailNode->GetName()]->Add(pgih);

                itemsizer = new wxGridBagSizer();
                propcnt = 0;


            } else {
                pgih = m_prjContent[detailNode->GetName()]->Item(itemCount);
            }

            while( properties )
            {
                if( pgih->find(properties->GetName()) == pgih->end() ) firstload = true;

           //     Logger::logf(LOG_MSG,_T("ProjectSettingsFrame::LoadPrj 3 = ")+properties->GetName());

                if(firstload) {
                    wxTextCtrl *tc = new wxTextCtrl(panel, wxID_ANY,properties->GetValue(),wxDefaultPosition, wxSize(150,25));
                    wxStaticText *st = new wxStaticText(panel, wxID_ANY, properties->GetName(), wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT  );
                    PropertyGuiItem * pgi = new PropertyGuiItem( st, tc);

                    (*pgih)[properties->GetName()] = pgi;

                    tc->SetInitialSize();

         //           wxBoxSizer * propsizer = new wxBoxSizer(wxHORIZONTAL);

                    itemsizer->Add(st, wxGBPosition(propcnt,0),wxDefaultSpan, wxALL| wxALIGN_CENTER_VERTICAL|wxALIGN_RIGHT, 2);
                    itemsizer->Add(tc, wxGBPosition(propcnt,1),wxDefaultSpan, wxALL|wxEXPAND|wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL, 2);
                    propcnt++;

                } else {
                    PropertyGuiItem * pgi = (*pgih)[properties->GetName()];
                    if(pgi) {
                        pgi->Value->Enable(true);
                        pgi->Value->SetValue(properties->GetValue());
                    }
                    else Logger::logf(LOG_MSG,_T("ProjectSettingsFrame::LoadPrj not found PropertyGuiItem!!!!!!!!!!"));

                }

          //      Logger::logf(LOG_MSG,_T("ProjectSettingsFrame::LoadPrj 3 = ")+properties->GetName()+_T(" END"));

                properties = properties->GetNext();
            }

            if(firstload) {

                panelsizer->Add(itemsizer, 0, wxALL, 3);

            }

   //         Logger::logf(LOG_MSG,_T("ProjectSettingsFrame::LoadPrj itemCount=%i, 2 = ")+itemNode->GetName()+_T("END"),itemCount);

            itemCount++;
            //if( m_prjContent[detailNode->GetName()]->GetCount() <= itemCount ) firstload = true;



        }

        if(firstload){
            panel->SetSizer(panelsizer);
            panelsizer->Fit(panel);
            panelsizer->SetSizeHints(panel);
        }


   //     Logger::logf(LOG_MSG,_T("ProjectSettingsFrame::LoadPrj 1 = ")+detailNode->GetName()+_T("END"));

    }

    //ShowNotEmptyPage();


    HideNotValidPages(project);



//    wxWindow *win = 0;
  //  if(sel>=0) win = m_prjBook->GetPage(sel);
    //if(win) win->Show(true);

}


void ProjectSettingsFrame::LoadPrj(Project *project, bool firstload)
{
    if(!project) return;

    if(project->Doc.GetRoot()->GetName() != _T("VemsTuneProjectFile")) return;

    m_prjPath->SetLabel(project->Path);

    SetInitialSize();

    Logger::logf(LOG_MSG,_T("ProjectSettingsFrame::LoadPrj prj=%p"),project);


    FillDlg(project,firstload);


}

void ProjectSettingsFrame::ShowValidPages(Project *prj)
{


    wxXmlNode *root = prj->Doc.GetRoot();

    for(wxWindowHash::iterator it = m_pageHash.begin(); it != m_pageHash.end() ; it++){

        if( !wxXmlHelper::FindNameInChildNodes(it->first, root) ) {




        }

    }

}


void ProjectSettingsFrame::HideNotValidPages(Project *prj)
{

    wxXmlNode *root = prj->Doc.GetRoot();

    for(wxWindowHash::iterator it = m_pageHash.begin(); it != m_pageHash.end() ; it++){

        if( !wxXmlHelper::FindNameInChildNodes(it->first, root) ) {

            it->second->Show(false);
            RemovePageByName(it->first, m_prjBook);

        }
    }
}

void ProjectSettingsFrame::EnableTextControls(bool enable)
{
    PropertyGuiItemHash * pgih;

    for(PropertyGuiItemHashArrayHash::iterator it1 = m_prjContent.begin(); it1 != m_prjContent.end() ; it1++){

        for(uint16_t a=0; a < it1->second->GetCount(); a++) {
            pgih = it1->second->Item(a);
            for(PropertyGuiItemHash::iterator itc = pgih->begin(); itc != pgih->end(); itc++)
                itc->second->Value->Enable(enable);

        }

    }


}

void ProjectSettingsFrame::EnablePages(bool enable)
{

    size_t cnt = m_prjBook->GetPageCount();

    for(size_t a=0; a<cnt;a++) {
        wxWindow *win = m_prjBook->GetPage(a);
        if(win) win->Show(enable);
    }

}


/*******************************************************************************
                        static members

*/


void ProjectSettingsFrame::RemovePageByName(const wxString& name, wxBookCtrl *bctrl)
{
    size_t cnt = bctrl->GetPageCount();

    for(size_t a=0; a<cnt;a++) {
        if( bctrl->GetPageText(a) == name) {
            bctrl->RemovePage(a);
            return;
        }

    }

}

wxWindow* ProjectSettingsFrame::GetPageByName(const wxString& name, wxBookCtrl *bctrl)
{
    size_t cnt = bctrl->GetPageCount();

    for(size_t a=0; a<cnt;a++) {
        if( bctrl->GetPageText(a) == name)
            return bctrl->GetPage(a);
    }


    return NULL;


}


void ProjectSettingsFrame::RemovePages(wxBookCtrl *bctrl)
{
    size_t cnt = bctrl->GetPageCount();

    for(size_t a=0; a<cnt;a++) {
        bctrl->GetPage(0)->Show(false);
        bctrl->RemovePage(0);
    }


}
