#include "AudioPlayRecordThread.h"

//#include <portaudiocpp/BlockingStream.hxx>
#include "portaudiocpp/PortAudioCpp.hxx"

#include "RingBuffer.h"
#include "SyncObj.h"

#include "../logger.h"


AudioPlayRecordThread::AudioPlayRecordThread(RingBuffer* outputData, SyncObj* syncObjPlay,
    RingBuffer* inputData, SyncObj* syncObjRecord)
:RBConsumerBase(outputData, syncObjPlay)
{
    Logger::logf(LOG_ERROR, _T("AudioPlayRecordThread: CTOR START"));
    m_recordBuffer = inputData;

    //m_framesPerBuffer = streamParameters.framesPerBuffer();
    m_blockingStream = NULL;
    m_streamParameters = NULL;
    m_framesPerBuffer = 4096;
    m_needsToReopen = false;
    m_syncObjRecord = syncObjRecord;

}

void AudioPlayRecordThread::setStreamParameters(const StreamParameters* streamParameters)
{
    m_streamParameters = streamParameters;
}
//void AudioPlayRecordThread::init(const StreamParameters* streamParameters)
void AudioPlayRecordThread::init()
{
    AutoSystem autoSys;
    //System& paSystem = System::instance();
    System::instance();
    m_framesPerBuffer = m_streamParameters->framesPerBuffer();
    Logger::logf(LOG_ERROR, _T("AudioPlayRecordThread: init 3"));
    m_blockingStream = new BlockingStream();
    Logger::logf(LOG_ERROR, _T("AudioPlayRecordThread: init 5 p=%p"), m_blockingStream);

    m_blockingStream->open(*m_streamParameters); // This may throw exception!!!
    Logger::logf(LOG_ERROR, _T("AudioPlayRecordThread: init 7"));

    bool isActive = m_blockingStream->isActive();

    Logger::logf(LOG_ERROR, _T("AudioPlayRecordThread: init 8 isActive=%i"), isActive );
}

AudioPlayRecordThread::~AudioPlayRecordThread()
{
    //dtor
     Logger::logf(LOG_ERROR, _T("DTOR AudioPlayRecordThread: DTOR START"));

    stopStream();

     if(m_blockingStream == NULL){
         Logger::logf(LOG_ERROR, _T("AudioPlayRecordThread DTOR stream NULL") );
        return;
     }
    delete m_blockingStream;
    //m_blockingStream = NULL;
     Logger::logf(LOG_ERROR, _T("DTOR AudioPlayRecordThread: DOR END "));
}

void * AudioPlayRecordThread::Entry()
{
/*
    Logger::logf(LOG_ERROR, _T("AudioPlayRecordThread::Entry() start"));
    if(m_blockingStream == NULL){
        Logger::logf(LOG_ERROR, _T("AudioPlayRecordThread::Entry()Null"));
        return 0;
    }
  */
 Logger::logf(LOG_ERROR, _T("AudioPlayRecordThread::Entry() 0"));
 //waitEnable();

 try{
    //init();
    /// INIT
    AutoSystem autoSys;
    System& paSystem = System::instance();

    m_framesPerBuffer = m_streamParameters->framesPerBuffer();
    Logger::logf(LOG_ERROR, _T("AudioPlayRecordThread: init 3"));
    m_blockingStream = new BlockingStream();
    Logger::logf(LOG_ERROR, _T("AudioPlayRecordThread: init 5 p=%p"), m_blockingStream);

    m_blockingStream->open(*m_streamParameters); // This may throw exception!!!
    Logger::logf(LOG_ERROR, _T("AudioPlayRecordThread: init 7"));

    bool isActive = m_blockingStream->isActive();

    Logger::logf(LOG_ERROR, _T("AudioPlayRecordThread: init 8 isActive=%i"), isActive );

    /// END INIT

//    portaudio::AutoSystem autoSys;
    Logger::logf(LOG_ERROR, _T("AudioPlayRecordThread::Entry() 10"));
//    System& paSystem = System::instance();
    int version = paSystem.version();
     Logger::logf(LOG_ERROR, _T("AudioPlayRecordThread::Entry() 15 version:%i"),version);
     Logger::logf(LOG_ERROR, _T("AudioPlayRecordThread::Entry() 16 p=%p"), m_blockingStream);

    isActive = m_blockingStream->isActive();

    Logger::logf(LOG_ERROR, _T("AudioPlayRecordThread::Entry() 20 isActive=%i"), isActive );

    m_blockingStream->start();
    Logger::logf(LOG_ERROR, _T("AudioPlayRecordThread::Entry() 30"));
    m_needsToReopen = false;
    Logger::logf(LOG_ERROR, _T("AudioPlayRecordThread::Entry() 40"));
    while(!m_isStoped){

        if(m_disabled){
            m_blockingStream->stop();
            Logger::logf(LOG_MSG, _T("AudioPlayRecordThread is disabled"));
            m_needsToReopen = true;
        }
        waitEnable();
        if(m_needsToReopen){
             Logger::logf(LOG_MSG, _T("AudioPlayRecordThread is started"));
            m_blockingStream->start();
            m_needsToReopen = false;
        }


        if( m_ringBuffer->isEmpty() ){
            Logger::logf(LOG_MSG, _T("AudioPlayRecordThread::Entry() RingBuffer is empty. This is serious. Is singal producer dead ?"));
        }
        waitRingBufferNotEmpty();
        //////////////////// EXIT
        if(m_isStoped){
            Logger::logf(LOG_ERROR, _T("AudioPlayRecordThread Entry(): isStopped"));
            try{
                m_blockingStream->stop();
                m_blockingStream->close();
            }catch(Exception& e){
                    const char * errorMsg = e.what();
                    //printf( "Exception! %s", m_errorMsg);
                    wxString msg = wxString::FromAscii(errorMsg);
                     Logger::logf(LOG_ERROR, _T("AudioPlayRecordThread Entry(): Entry Exception: ")+ msg);
            }
             Logger::logf(LOG_MSG, _T("AudioPlayRecordThread() Entry() Finished (stopped) !!!!!!!!"));
             return 0;
        }
        //////////////////////// END EXIT
        //m_buff16 = (short *) sineTest->generate();
        writeOutput();
        //readInput();system:/
    }

     }catch(Exception& e){
        const char * errorMsg = e.what();
        //printf( "Exception! %s", m_errorMsg);
        wxString msg = wxString::FromAscii(errorMsg);
         Logger::logf(LOG_ERROR, _T("AudioPlayRecordThread: Exception1: ")+ msg);
    }

    try{
        portaudio::AutoSystem autoSys;
        //System& paSystem = System::instance();
        System::instance();
        m_blockingStream->stop();
        m_blockingStream->close();
    }catch(Exception& e){
        const char * errorMsg = e.what();
        //printf( "Exception! %s", m_errorMsg);
        wxString msg = wxString::FromAscii(errorMsg);
         Logger::logf(LOG_ERROR, _T("AudioPlayRecordThread: Exception2: ")+ msg);
    }

    Logger::logf(LOG_MSG, _T("AudioPlayRecordThread() Finished!!"));

    return 0;
}

void AudioPlayRecordThread::writeOutput()
{
        short * buf = (short*) m_ringBuffer->getTailPointer();

        //generateBlock(m_buff16, m_framesPerBuffer);

        m_blockingStream->write( buf, m_framesPerBuffer );
        m_ringBuffer->nextTail();
        m_isNotFull->Broadcast();
}

void AudioPlayRecordThread::readInput()
{
    if( m_recordBuffer->isFull() ){
        Logger::logf(LOG_MSG, _T("RingBuffer is full. This is serious. Is SignalProcessThread dead ?"));
    } else {
        short * buf = (short*) m_recordBuffer->getHeadPointer();
        m_blockingStream->read(buf, m_framesPerBuffer );
        m_recordBuffer->nextHead(); // not empty any longer
        m_syncObjRecord->m_isNotEmpty->Broadcast();
    }

}

void AudioPlayRecordThread::stopStream()
{
    if(m_blockingStream == NULL) return;
    try{
        portaudio::AutoSystem autoSys;
        //System& paSystem = System::instance();
        System::instance();

        m_blockingStream->stop();
        m_blockingStream->close();
    }catch(Exception& e){
        const char * errorMsg = e.what();
        //printf( "Exception! %s", m_errorMsg);
        wxString msg = wxString::FromAscii(errorMsg);
         Logger::logf(LOG_ERROR, _T("AudioPlayRecordThread::stopStream() Exception: ")+ msg);
    }

}

void AudioPlayRecordThread::Exit()
{
        //Exit();
}
