#include "RingBufferWriterThread.h"

#include "../DirHelper.h"
#include "../logger.h"
#include "../RTDataInputBase.h"
#include "../RealTimeDesc.h"    //should remove
#include "../RawLogHelper.h"

#include "../RTDataMgrFacade.h"
#include "../vemsprotocol.h"
#include "../RTDataRawLogger.h"

#include "../stdatamanager/STDataMgrBase.h"

#include "RingBuffer.h"
#include "ThreadConditionBase.h"
#include "..\vemsprotocol\ECUInfo.h"
#include "../SerialException.h"

RingBufferWriterThread::RingBufferWriterThread()
: m_fwInfo (NULL)
, m_inputMgr (NULL)
, m_stdataMgr (NULL)
, m_ringBuffer (NULL)
, m_tc (NULL)
, m_locked (false)
, m_inputType (0)
, m_connected (false)
, m_RTBuff (NULL)
,m_RTLen (61)
, m_rtDataRawLogger (NULL)
, m_vp (NULL)
, m_isRawLog(true)
, m_isbootMode (false)
{
    //ctor

    m_rtDataRawLogger = new RTDataRawLogger();
}

RingBufferWriterThread::~RingBufferWriterThread()
{
    //dtor
}

void* RingBufferWriterThread::Entry()
{
    //Logger::logf(LOG_MSG, _T("RingBufferWriterThread::Entry() start"));

    if(m_inputMgr == NULL || m_ringBuffer == NULL || m_tc == NULL || m_vp == NULL){
        Logger::logf(LOG_ERROR, _T("RingBufferWriterThread::Entry() NOT INITIALIZED CORECTLY"));
        return false;
    }



    while ( true )
    {
        waitOnline();
        m_inputType = m_inputMgr->getInputType();

        if( m_inputType == INPUT_TYPE_RINGBUFFER){
            m_tc->m_isRingBuffer->Broadcast(); // waitInputSource();
        }

       // if(m_stDataMgr != NULL ) m_stDataMgr->updateViewers();

        writeRingBuffer(); // set the connected flag, but DO NOT increment the head pointer

        if(m_stdataMgr != NULL) m_stdataMgr->doSync();

        //TODO: check STDataWriteQueue and write the data to ECU;
        //writeSTDataToECU(); //Just fetch the data and write the flag(send event);
        //TODO: check STDataRequestQueue and load data from ECU;
        //readSTDataFromECU();

         if(m_connected && (m_inputType == INPUT_TYPE_RINGBUFFER) &&
        ( m_inputMgr->getRegisteredCount() > 0 )){
            if( !m_ringBuffer->isFull() ) m_ringBuffer->nextHead(); 	//increment the rb head index
            m_tc->m_isNotEmpty->Broadcast(); // waitRingBufferNotEmpty();
        }else if(!m_connected){ //if not connected test the serial every 1sec
                wxThread::Sleep(1000);
        }
        if(m_inputMgr->getRegisteredCount() > 0) m_tc->m_Viewers->Broadcast(); //  waitViewers();
    }
}

// TODO: should set the m_connected flag;
bool RingBufferWriterThread::writeRingBuffer()
{
    //m_buff;
     //Logger::logf(LOG_MSG, _T("RingBufferWriterThread::Entry() writeRingBuffer() %i"), b);
    m_RTBuff = (uint8_t*) m_ringBuffer->getHeadPointer(); // not increment the ringbuffer head index
     //Logger::logf(LOG_MSG, _T("RingBufferWriterThread::Entry() writeRingBuffer() %p"), m_RTBuff);

//     char * tmp = "111111111111111111111111111111111111111111112222222222222222222222222222222222222222222222";
//     memcpy ( m_RTBuff, tmp,  strlen(tmp) );


 //Logger::logf(LOG_MSG, _T("RingBufferWriterThread::Entry() writeRingBuffer() 1"));

    int res =-1;
    try{
        //Logger::logMilli(LOG_MSG,_T("RingBufferWriterThread::writeRingBuffer() serialRead START"));
        res=  m_vp->getRuntimeData(m_RTBuff, !m_connected);
        //Logger::logMilli(LOG_MSG,_T("RingBufferWriterThread::writeRingBuffer() serialRead END"));
    }catch (SerialException& se){
       // Logger::logf(LOG_MSG, _T("RingBufferWriterThread::Entry() writeRingBuffer() SerialException"));
        m_connected = false;
        //Thread::Sleep(1000);
        return true;
    }catch (...){
       // Logger::logf(LOG_MSG, _T("RingBufferWriterThread::Entry() writeRingBuffer() Unknown Exception"));
    }

    //Logger::logf(LOG_MSG, _T("RingBufferWriterThread::Entry() writeRingBuffer() 2 res=%i"), res);
    if(res < 0){
        m_connected = false;
        return false;
    }else if(res == ECUInfo::BOOT){//stay in boot mode
        setLocked(true);
        m_isbootMode = true;
        return false;
    }

    m_isbootMode = false;

    if(!m_connected ) readFirmwareInfo(5, true);

    if(m_isRawLog){
        if(!m_connected ) { writeHeader(); }
        m_rtDataRawLogger->Write((const char*) m_RTBuff);
    }

    m_connected = true;


    return true;
}

void RingBufferWriterThread::setVemsProtocol(  VemsProtocol *vp)
{
    m_vp = vp;
}

void RingBufferWriterThread::setInputMgr(RTDataMgrFacade * inputMgr)
{
    m_inputMgr = inputMgr;
}

void RingBufferWriterThread::setRingBuffer(RingBuffer *rb)
{
    m_ringBuffer = rb;
}

void RingBufferWriterThread::setThreadConditionBase(ThreadConditionBase *cb)
{
    m_tc = cb;
}

void RingBufferWriterThread::setLocked(bool b)
{
    m_locked = b;
    if( !m_locked)  m_tc->m_isNotOnline->Broadcast();
}

void RingBufferWriterThread::setRawLog(bool b)
{
    m_isRawLog = b;

    if(m_connected && b )
        writeHeader();

}

void RingBufferWriterThread::waitOnline()
{
     while( m_locked  )
        {
            m_connected = false;
            Logger::logf(LOG_MSG, _T("RingBufferWriterThread::waitOnline() "));
            m_tc->m_isNotOnline->Wait();
            Logger::logf(LOG_MSG, _T("RingBufferWriterThread::waitOnline() released"));
        }
}

void RingBufferWriterThread::waitRingBufferNotFull()
{
     while( m_ringBuffer->isFull()  )
        {
    //        Logger::logf(LOG_MSG, _T("RingBufferWriterThread::Entry() isFull() "));
            m_tc->m_isNotFull->Wait(); // signaled by reader;

      //      Logger::logf(LOG_MSG, _T("RingBufferWriterThread::Entry() isFull() released"));
        }
}

bool RingBufferWriterThread::isConnected()
{
    return m_connected;
}

bool RingBufferWriterThread::readFirmwareInfo(uint8_t trycount, bool force)
{

    bool result = false;

    if(!force && m_fwInfo != NULL) return true;

    m_fwInfo = NULL;

    for(char a=0; a < trycount ; a++){

        try{
             m_fwInfo = m_vp->getFirmwareInfo();
             break;
        }catch(SerialException se){
            continue;
        }

    }

    if(m_fwInfo != NULL) result = true;

    return result;

}

void RingBufferWriterThread::writeHeader()
{
    Logger::logf(LOG_MSG,_T("RingBufferWriterThread::writeRingBuffer switched START"));

        m_rtDataRawLogger->SetLog(false);

        m_rtDataRawLogger->Close();

        m_RTLen = RealTimeDesc::getDataSize();


        readFirmwareInfo(5, true);


//            if(fwInfo == NULL) Logger::logf(LOG_MSG,_T("FirmwareInfo *fwInfo = %p"),fwInfo);

        m_rtDataRawLogger->SetBlockSize(512);
        m_rtDataRawLogger->SetSize(m_RTLen);
        //m_rtDataRawLogger->SetFile(DirHelper::GetPath(dLOG) + _T("rawECUlog.txt"));
        m_rtDataRawLogger->SetFile( DirHelper::CreateNewFileName(_T("rawECUlog.txt"),DirHelper::GetPath(dLOG),true,true) );
        m_rtDataRawLogger->SetLog(true);

        m_rtDataRawLogger->AddHeaderItem(RawLogHelper::dataLen,wxString::Format(_T("%i"),m_RTLen));
        m_rtDataRawLogger->AddHeaderItem(RawLogHelper::blockSize,wxString::Format(_T("%i"),512));

        if(m_fwInfo != NULL){
            m_rtDataRawLogger->AddHeaderItem(RawLogHelper::fwVersion,m_fwInfo->getFirmwareVersion());
            m_rtDataRawLogger->AddHeaderItem(RawLogHelper::fwcompTime,m_fwInfo->getCompilationTime());
            m_rtDataRawLogger->AddHeaderItem(RawLogHelper::fwSN,m_fwInfo->getSerialNumber());
        }

        m_rtDataRawLogger->WriteHeader();

        Logger::logf(LOG_MSG,_T("RingBufferWriterThread::writeRingBuffer switched END"));

}
