#include "RTDataAnalyzer.h"
#include "../logger.h"

RTDataAnalyzer::RTDataAnalyzer() : RTDataAnalyzerBase()
{
    m_values = new DoubleHash;
    m_wasTriggerError = false;
    resetValues();

    wxArrayString reqDH;
    reqDH.Add( _T("rpm") );
    reqDH.Add( _T("map") );
    reqDH.Add( _T("isTriggerError") );

    setRequiredSymbols( reqDH );
}

RTDataAnalyzer::~RTDataAnalyzer()
{
    delete m_values;
}

void RTDataAnalyzer::processData()
{
    DoubleHash::const_iterator it = m_data->find( _T("rpm") );

    if ( it != m_data->end() )
    {
        int rpm = (int) it->second;
        if ( m_rpmMax < rpm )
            m_rpmMax = rpm;
    }

    //
    it = m_data->find( _T("map") );

    if ( it != m_data->end() )
    {
        int map = (int) it->second;
        if ( m_mapMax < map )
            m_mapMax = map;
    }

    //
    it = m_data->find( _T("isTriggerError") );

    if ( it != m_data->end() )
    {
        bool isTriggerError = (bool) it->second;

        if ( isTriggerError && ( m_wasTriggerError == false) )
        {
            m_wasTriggerError = true;
            m_triggerError++;
        }
        else if ( m_wasTriggerError && ( isTriggerError == false ) )
        {
            m_wasTriggerError = false;
        }
    }
}

void RTDataAnalyzer::resetValues()
{
    (*m_values)[_T("rpmMax")] = m_rpmMax = -1;
    (*m_values)[_T("mapMax")] = m_mapMax = -1;
    (*m_values)[_T("triggerError")] = m_triggerError = 0;
}

DoubleHash* RTDataAnalyzer::getValues()
{
    (*m_values)[_T("rpmMax")] = (double) m_rpmMax;
    (*m_values)[_T("mapMax")] = (double) m_mapMax;
    (*m_values)[_T("triggerError")] = (double) m_triggerError;

    return m_values;
}
