#include "RTDataInputRingBufferSyncGPS.h"
#include "../CommunicationBase.h"
#include "../SerialException.h"
#include "../logger.h"
#include "../DirHelper.h"
#include "../GPSData.h"
#include "../Math.h"
#include "RTDataRawLogger.h"

RTDataInputRingBufferSyncGPS::RTDataInputRingBufferSyncGPS( RingBuffer* rb, SymbolMgr* mt, SyncObj* so, CommunicationBase* sph, GPSData* gps, RTDataRawLogger* rawLogger )
    : RTDataInputRingBufferSync( rb, mt, so )
{
    m_commBase = sph;
    m_fileOutputStream  = NULL;
    m_gpsDataBuffer = new char[ 2*256 ];
    m_gpsDataIdx    = 0;
    m_recording     = false;
    m_state         = GPSInfo::NOT_CONNECTED;
    m_isLog         = false;
    m_gpsData       = gps;
    m_rawLogger     = rawLogger;
}

RTDataInputRingBufferSyncGPS::~RTDataInputRingBufferSyncGPS()
{
    Logger::logf( LOG_MSG, _T("RTDataInputRingBufferSyncGPS::RTDataInputRingBufferSyncGPS() DTOR") );
    delete[] m_gpsDataBuffer;
    delete m_fileOutputStream;
}

RTDataInputType RTDataInputRingBufferSyncGPS::getType()
{
    return INPUT_TYPE_GPS;
}

int RTDataInputRingBufferSyncGPS::updateViewerModel()
{
    return 0; //OK
}

int RTDataInputRingBufferSyncGPS::processGPSData()
{
  //  char* rbTail = m_ringBuffer->getTailPointer();
  //  int status = m_parser.feedGPScharacter( rbTail[0] );
     int status = m_parser.feedGPScharacter( m_buff );
    switch( status )
    {
        case NMEAParse::GPS_FORMAT_GPRMC:
        case NMEAParse::GPS_FORMAT_GPGGA:
              m_rawLogger->captureEcuBlock();
              break;
        case END :
            //Logger::logf( LOG_MSG, _T("RTDataInputRingBufferSyncGPS::processGPSData()") );
            packData();
            m_gpsData->set( m_packedGpsData );

            m_state = GPSInfo::CONNECTED;
            break;
    }

    return status;
}

int RTDataInputRingBufferSyncGPS::updateInputModel()
{
    m_state = getGPSData();
    if(m_state == GPSInfo::DATA_RECEIVED){
        processGPSData(); /// Set m_state to connected when valid record found
    }
    return m_state;
}

GPSInfo::State RTDataInputRingBufferSyncGPS::getGPSData()
{
  //  Logger::logf( LOG_MSG, _T("RTDataInputRingBufferSyncGPS::getGPSData() - START") );
  //  waitRingBufferNotFull();

  //  char* m_buff = m_ringBuffer->getHeadPointer(); //not increment the ringbuffer head index

    int result = -1;

    try{
        result = m_commBase->ReadString( &m_buff, 1 );
    }catch( SerialException& se ){
        //Logger::logf( LOG_MSG, _T("RTDataInputRingBufferSyncGPS::getGPSData() - SerialException : %s"), se.toString().c_str() );
        return GPSInfo::NOT_CONNECTED;
    }catch (...){
        //Logger::logf( LOG_MSG, _T("RTDataInputRingBufferSyncGPS::getGPSData() - Unknown SerialException") );
    }
  //  Logger::logf( LOG_MSG, _T("RTDataInputRingBufferSyncGPS::getGPSData() - 1") );
    if( result <= 0 )
    {
        return GPSInfo::NO_GPS_DATA;
    }

/// Write to file!!
    //if( m_isLog)
    //{

        if( !m_fileOutputStream )
        {
            wxString fileName = DirHelper::CreateNewFileName( _T("rawGPSlog.txt"), DirHelper::GetPath(dLOG), true, true );
            m_fileOutputStream = new wxFileOutputStream( fileName );
        }

        m_gpsDataBuffer[ m_gpsDataIdx ] = m_buff;
        ++m_gpsDataIdx;

        if( m_gpsDataIdx == 256 ){
            m_fileOutputStream->Write( m_gpsDataBuffer, m_gpsDataIdx );
            m_gpsDataIdx=0;
        }

    //}


    return GPSInfo::DATA_RECEIVED;
}


void RTDataInputRingBufferSyncGPS::enableLog( bool enable )
{
    Logger::logf( LOG_MSG, _T("RTDataInputRingBufferSyncGPS::enableLog()") );
    if( m_fileOutputStream )
    {
    Logger::logf( LOG_MSG, _T("RTDataInputRingBufferSyncGPS::enableLog() delete") );
        //delete m_fileOutputStream;
        //m_fileOutputStream = NULL;
    }

    m_isLog = enable;
}


void RTDataInputRingBufferSyncGPS::packData()
{
    GPSDataStruct gpsData = m_parser.gpsdata;

    if( gpsData.errorbyte )
    {
        return;
    }

    int  lat = gpsData.latitude;

    //N = 0 , S = -1
    if( gpsData.latitudeindicator == 'S' )
    {
        lat *= -1;
    }

    //Logger::logf( LOG_MSG, _T("RTDataInputRingBufferSyncGPS::packData() latitude: %u"), lat );

    Math::getBytes( lat, &m_packedGpsData[0] );

    int lon = gpsData.longnitude;

    //E = 0, W = -1
    if( gpsData.longnitudeindicator == 'W' )
    {
        lon *= -1;
    }

    Math::getBytes( lon, &m_packedGpsData[4] );

    //Logger::logf( LOG_MSG, _T("RTDataInputRingBufferSyncGPS::packData() dataFormat: %i"), gpsData.dataFormat );
    //if( gpsData.dataFormat == NMEAParse::GPS_FORMAT_GPGGA )
    //{

        int height = gpsData.antennaheight;
        //Logger::logf( LOG_MSG, _T("RTDataInputRingBufferSyncGPS::packData() height: %i"), height );
        height += 10000;
         Math::getBytes( height, &m_packedGpsData[8], 3);
    //}

    if( gpsData.dataFormat == NMEAParse::GPS_FORMAT_GPRMC )
    {
        //Logger::logf( LOG_MSG, _T("RTDataInputRingBufferSyncGPS::packData() format : GPRMC") );
        Math::getBytes( gpsData.speed , &m_packedGpsData[11], 2);
        //0 = Valid
        //1 = Invalid
        if( gpsData.status == 'A' )
        {
            m_packedGpsData[8] &= 127;  /// 7.bit 0 Valid GPS Data
        }
        else
        {
            m_packedGpsData[8] |= 128; ///7.bit 1 Invalid GPS Data
        }
    }
}
