#include <wx/filename.h>

#include "RTDataLogWriter.h"
#include "../mtconfigdata.h"
#include "../RealTimeDesc.h"

#include "../logger.h"

/** @brief RTDataLogWriter
  *
  * @todo: document this function
  */
RTDataLogWriter::RTDataLogWriter(const DataLogEntryArray* entries)
{
    m_entries = entries;
    m_interpolate = new InterpolateViewerData(m_entries);
    m_file = new wxFile();
    //m_RecordPerBlock = 512/(RealTimeDesc::getDataSize()+14);

    //m_prevBlockCount = 0;


    m_interpolate->SetRefKey(_T("gpsCaptureLine") );
    wxArrayString* valueList = new wxArrayString();
    valueList->Add(_T("distanceSum"));
    m_interpolate->SetValues(valueList);
    m_isSymbolsRegistered = false;

    init();

}

/** @brief ~RTDataLogWriter
  *
  * @todo: document this function
  */
RTDataLogWriter::~RTDataLogWriter()
{
    UnRegisterViewer();
    m_file->Close();
    delete m_file;
    delete m_interpolate;
}

void RTDataLogWriter::init()
{
    m_timeInterval = 0.100;
    m_samplesBeforeFirstChange = 0;
    m_rowCount = 0;
    m_count = 0;
    m_gpsCaptureLine = 0;
    m_prevCapturedIdx = 0;
    m_sameIdxCount = 0;
    m_RTDataPerBlock = 1;
}

/** @brief refresh
  *
  * @todo: document this function
  */
void RTDataLogWriter::refresh(DoubleHash* table)
{
    //Logger::logf(LOG_MSG,_T("RTDataLogWriter::refresh() "));

    calcualteGPSCaptureLine(table);
    calculateTimeMilli(table);
    //Logger::logf(LOG_MSG,_T("RTDataLogWriter::refresh() 111"));
 /*
    DoubleHash::const_iterator it;
    for(it = table->begin(); it != table->end(); it++ ){
        wxString key = it->first;
         Logger::logf(LOG_MSG, _T("RTDataLogWriter::refresh() table key: ") + key);
    }
   */


    if( m_interpolate->AddRecord(table) ){
        //Write
        wxString lines = m_interpolate->getLines();
        m_file->Write(lines);
        #if defined (__WINCE_STANDARDSDK__)
            m_file->Flush();
        #endif

    }

    //Logger::logf(LOG_MSG,_T("RTDataLogWriter::refresh() 222"));
}

/** @brief require
  *
  * @todo: document this function
  */
/*
void RTDataLogWriter::require(DoubleHash* table)
{
    for(size_t i=0; i<m_entries->GetCount(); i++){
        (*table)[m_entries->Item(i)->m_name];
    }
}
*/

void RTDataLogWriter::require()
{
    if( !m_isSymbolsRegistered )
    {
        for(size_t i=0; i<m_entries->GetCount(); i++)
        {
            m_rtDataMgr->registerSymbol( m_entries->Item(i)->m_name );
        }

        m_isSymbolsRegistered = true;
    }
}

void RTDataLogWriter::unregisterSymbols()
{
    if( m_isSymbolsRegistered )
    {
        for( size_t i = 0; i<m_entries->GetCount(); i++ )
        {
            m_rtDataMgr->unregisterSymbol( m_entries->Item(i)->m_name );
        }
    }
}

/** @brief set
  *
  * @todo: document this function
  */
void RTDataLogWriter::setFileName(const wxString& filename)
{
    m_fileName = filename;
}

/** @brief writeHeaders
  *
  * @todo: document this function
  */
bool RTDataLogWriter::writeHeaders()
{
    if(!m_file->Open(m_fileName, wxFile::write )) return false;

    //Logger::logf(LOG_MSG,_T("RTDataLogWriter::writeHeaders() columns:%i"),  m_entries->GetCount() );
    wxString header = _T("#VemsTune\n");

    for(size_t i=0; i<m_entries->GetCount(); i++){
        header += m_entries->Item(i)->m_label + _T("\t");
    }

    header += _T("\n\0");

    //m_file->AddLine(header);
    m_file->Write(header);
    init();
    return true;
}

wxString RTDataLogWriter::getFileName()
{

    wxFileName *tmp = new wxFileName(m_fileName);
	wxString *tmp1 = new wxString(), *tmp2 = new wxString(), *tmp3 = new wxString();
	tmp->SplitPath(m_fileName, tmp1, tmp2, tmp3);

    wxString retdir = wxString(*tmp2);
    if(!(tmp3->IsEmpty()))
        retdir += _T(".") + wxString(*tmp3);

    delete tmp;
    delete tmp1;
    delete tmp2;
    delete tmp3;

    return retdir;

}

wxString RTDataLogWriter::getFilePath()
{
    return m_fileName;
}

void RTDataLogWriter::calculateTimeMilli(DoubleHash *table)
{
    //m_currSec = (*table)[_T("secl")];
    if(m_rowCount == 0){
        (*table)[_T("time")] = 0;
         m_prevSec = (*table)[_T("secl")];

         m_rowCount++;
         m_prevMilliSec = 0.0;
         m_prevRowCount = 0;
         m_sampleCountPerSec=1;
         return;
    }
    m_currSec = (*table)[_T("secl")];
    if(m_prevSec != m_currSec){
        if(m_count > 1){    /// m_count: elapsed second;
            m_timeInterval = (m_timeInterval  + (1.0/(double)m_sampleCountPerSec) )/2.0;
            m_prevMilliSec = m_count + m_sampleCountPerSec *m_timeInterval;
            //double samplePerSec = (double) (m_rowCount - m_prevRowCount);
            //if(samplePerSec > 0.03){
               // Logger::logf(LOG_MSG, _T("RTDataLogWriter::calculateTimeMilli() m_currMilliSec: %f, samplePerSec:%f"), m_currMilliSec, samplePerSec);
            //Logger::logf(LOG_MSG, _T("RTDataLogWriter::calculateTimeMilli() ecuElapsedTime: %f, interval: %f"), ecuElapsedTime, m_timeInterval);
            //}
        }
        m_sampleCountPerSec =0;
        m_prevSec = m_currSec;
        //m_prevRowCount = m_rowCount;
        m_count ++;
    }
    m_sampleCountPerSec++;

    m_prevMilliSec = m_prevMilliSec + m_timeInterval;
    (*table)[_T("time")] = m_prevMilliSec;
    /* test
    (*table)[_T("m_timeInterval")] = m_timeInterval;
    (*table)[_T("m_sampleCountPerSec")] = m_sampleCountPerSec;
    (*table)[_T("m_count")] = m_count;
    */
    ///
    m_rowCount++;
    //m_prevMilliSec = m_currMilliSec;
}

void RTDataLogWriter::calcualteGPSCaptureLine(DoubleHash *table)
{
    //uint32_t blockCount = m_RecordCount/m_RecordPerBlock;
    //Logger::logf(LOG_MSG, _T("RTDataLogWriter::calcualteGPSCaptureLine()line count: %d, blockCount: %d, recordPerBlock: %d"),
    //m_rowCount, blockCount, m_RecordPerBlock);

    if( *m_isGPSCapturedRef){
        m_gpsCaptureLine = 0;
    }else{
        m_gpsCaptureLine --;
    }

/*
    int capturedIdx = (int) (*table)[_T("capturedIdx")];

    if(capturedIdx == 255){
        m_gpsCaptureLine--;
    }else if(capturedIdx == m_prevCapturedIdx){
        if(m_sameIdxCount < m_RTDataPerBlock-1){
             m_gpsCaptureLine--;
             m_sameIdxCount++;
        }else{
            m_gpsCaptureLine = capturedIdx;
            m_sameIdxCount = 0;
        }
    }else{
         m_gpsCaptureLine = capturedIdx;
         m_sameIdxCount = 0;
    }

*/
    (*table)[_T("gpsCaptureLine")] = m_gpsCaptureLine;
    //m_prevCapturedIdx = capturedIdx;
}

void RTDataLogWriter::setRTDataPerBlock(long rtDataPerBlock)
{
    m_RTDataPerBlock = rtDataPerBlock;
}
void RTDataLogWriter::flushBuffers()
{
    writeLogWithoutInterpolate();
    m_interpolate->clearDoubleHashArray();
}
void RTDataLogWriter::writeLogWithoutInterpolate()
{
    m_interpolate->interlaceStringLines();
    wxString lines = m_interpolate->getLines();
    m_file->Write(lines);
    #if defined (__WINCE_STANDARDSDK__)
        m_file->Flush();
    #endif
}
