#ifndef RTDATAMGRBASE_H
#define RTDATAMGRBASE_H

#include <wx/string.h>
#include <wx/settings.h>

#include "../common/types/DoubleHash.h"
#include "../common/types/String2IntHash.h"


/// Forward Declarations
/// see: http://www.eventhelix.com/RealtimeMantra/HeaderFileIncludePatterns.htm
class RTDataViewerBase;
class SymbolMgrBase;

WX_DEFINE_ARRAY(RTDataViewerBase *, ArrayOfRTDataViewerBase);

/**

*/

class RTDataMgrBase
{
    public:
        RTDataMgrBase(SymbolMgrBase* symbolmgrbase = NULL);
        virtual ~RTDataMgrBase();
        bool isRequireChanged(){
            return m_isRequireChanged;
        }
        virtual int updateInputModel() =0;
        //virtual void sendInputEvent(int type, int inputEvent) =0;
        int updateViewers();
        void requireChanged();
        void RegisterViewer(RTDataViewerBase* viewer);
        void UnRegisterViewer(RTDataViewerBase* viewer);
        size_t  getRegisteredCount();
        virtual bool   isExist(const wxString& symbolname) =0;
        void UnregisterAll();

        void notifyViewers(); // Signalgen uses it!!!
        void filteredNotifyViewers();
        void registerSymbol( const wxString& symbolName );      //register a symbol in the symbol registry
        void unregisterSymbol( const wxString& symbolName );
        const DoubleHash& getRequiredDH(){
            return m_RegisteredSymbolTable;
        }

        DoubleHash& getRequiredDH_RW() {
            return m_RegisteredSymbolTable;
        }

        void getRequiredSymbols();

        virtual void reset();

        void setMinRefreshInterval(int milliseconds = 0) { m_minRefreshInterval = milliseconds; };
        int  getMinRefreshInterval() { return m_minRefreshInterval; };

    protected:

        /**
            Refresh the doubleHash;
        */
        virtual int updateViewerModel() =0;
        virtual double getValue(const wxString& symbolname) =0;
        void setRequestedSymbols();     //set requested symbol in the viewerModel

        ArrayOfRTDataViewerBase* m_RTDataViewerBaseA;   //array of registered viewers
        /**
            Model of Viewers
            key = the currently registered symbols (displayed item)
            value = value of the item
        */
        DoubleHash      m_RegisteredSymbolTable;
        String2IntHash  m_requestedSymbolRegistry;
        bool    m_isRequireChanged;     //true if any viewer change it's symbol

        wxStopWatch     m_packetFilterTimer;

        int             m_minRefreshInterval;

        SymbolMgrBase*  m_symbolMgrBase;

    private:
};


#endif // RTDATAMGRBASE_H
