#include "RTDataMgrFacade.h"

#include "../logger.h"
#include "../vemsprotocol/Sipr.h"
#include "RTDataInputSIPR.h"

RTDataMgrFacade::RTDataMgrFacade(SymbolMgrBase* smgrb)
    :RTDataMgrBase(smgrb)
{
    //ctor
    m_RTDataViewerBaseA = new ArrayOfRTDataViewerBase();
    m_currInput = NULL;

    //Connect(wxEVT_RT_DATA_INPUT, RTDataInputSourceEventHandler(RTDataMgrFacade::OnInputSourceEvent) );
}

RTDataMgrFacade::~RTDataMgrFacade()
{
    //dtor
    delete m_RTDataViewerBaseA;
}

/** @brief getValue
  *
  * @todo: document this function
  */
double RTDataMgrFacade::getValue(const wxString& symbolname)
{
    if(m_currInput) return m_currInput->getValue(symbolname);

    return 0; //temporary
}

/** @brief
  *
  * @todo: document this function
  */
int RTDataMgrFacade::updateViewerModel()
{
    //Logger::logf(LOG_MSG,_T("RTDataMgrFacade::updateViewerModel()"));
    if(m_currInput)
    {
        //Logger::logf(LOG_MSG,_T("RTDataMgrFacade::updateViewerModel() input type=%d"),m_currInput->getType() );
        return m_currInput->updateViewerModel();
    }
    Logger::logf(LOG_ERROR,_T("RTDataMgrFacade::updateViewerModel() ERROR: m_currInput == NULL"));
    return 0;
}

int RTDataMgrFacade::updateInputModel()
{
    // Logger::logf(LOG_MSG,_T("RTDataMgrFacade::refresh()"));
    if(m_currInput) return m_currInput->updateInputModel();
    // Logger::logf(LOG_MSG,_T("RTDataMgrFacade::refresh() END"));
    return 0;
}

int RTDataMgrFacade::updateInputModel(RTDataInputType type)
{
    //Logger::logf(LOG_MSG, _T("RTDataMgrFacade::updateInputModel() start"));
    // Logger::logf(LOG_MSG,_T("RTDataMgrFacade::refresh()"));
    RTDataInputBase* input =  m_inputSourceH[type];
    if(input) return input->updateInputModel();
    // Logger::logf(LOG_MSG,_T("RTDataMgrFacade::refresh() END"));
    return 0;
}

 int RTDataMgrFacade::getInputType()
 {
     if(m_currInput) return m_currInput->getType();
     return INPUT_TYPE_NONE;
 }

bool RTDataMgrFacade::isValid()
{
     if(m_currInput) return m_currInput->isValid();
     return false;
}

/** @brief isExist
  *
  * @todo: document this function
  */
bool RTDataMgrFacade::isExist(const wxString& symbolname)
{
    if(m_currInput) return m_currInput->isExist(symbolname);

    return false;
}

/** @brief setInput
  *
  * @todo: document this function
  */
void RTDataMgrFacade::SetInput(RTDataInputBase* input)
{
    registerInput(input);
    if(input != NULL){
        selectInput(input->getType());
    }else{
        selectInput(INPUT_TYPE_NONE );
    }
}

void RTDataMgrFacade::registerInput(RTDataInputBase* input)
{

    if(input){
        int type = input->getType();
        if(m_inputSourceH.find(type) != m_inputSourceH.end() ){
            RTDataInputBase* old = m_inputSourceH[type];
            delete old;
        }
        m_inputSourceH[type] = input;
        input->setMgr(this);
        //observerNotify(input->getType(),0);
    }else{
        //observerNotify( INPUT_TYPE_NONE,0);
    }
                                                //Logger::logf(LOG_MSG,_T("RTDataMgrFacade::SetInput START"));
    m_currInput = input;                        //Logger::logf(LOG_MSG,_T("RTDataMgrFacade::SetInput 1"));
    m_RegisteredSymbolTable.clear();                       //Logger::logf(LOG_MSG,_T("RTDataMgrFacade::SetInput 2"));
    if(input) input->setViewerModel(&m_RegisteredSymbolTable);   //Logger::logf(LOG_MSG,_T("RTDataMgrFacade::SetInput 3"));
    requireChanged();                           //Logger::logf(LOG_MSG,_T("RTDataMgrFacade::SetInput 4"));
    //updateViewers(); // refresh the gauge AT ONCE with --- or the actual value
                                                //Logger::logf(LOG_MSG,_T("RTDataMgrFacade::SetInput 5 END"));
}

void RTDataMgrFacade::selectInput(RTDataInputType type)
{
    if(m_currInput != NULL) m_currInput->setActive(false);
    if(type == INPUT_TYPE_NONE){
        m_currInput = NULL;
        return;
    }
    m_currInput =  m_inputSourceH[type];
    //input changed so we need to refresh the model
    m_currInput->setActive(true);

    //throw IS Activated event;

}
void RTDataMgrFacade::selectInput(const wxString& fileName)
{

}
/*
void RTDataMgrFacade::sendInputEvent(int type, int inputEvent)
{
    RTDataInputSourceEvent event;
    event.type = type;
    event.eventID = inputEvent;

    event.SetEventObject( this );
    AddPendingEvent( event );
}
*/
/*
void RTDataMgrFacade::OnInputSourceEvent(RTDataInputSourceEvent& evt)
{
    int type = evt.type;
    int eventID = evt.eventID;

    Logger::logf(LOG_MSG, _T("RTDataMgrFacade::OnInputSourceEvent(): event type:%d,ID:%d"),type, eventID);

    /// in PDA crash
    //RTDataInputBase* input = m_inputSourceH[type];
    //input->observerNotify(type, eventID);
}
*/

bool RTDataMgrFacade::filteredProcess(const SIPRPacket& p )
{
     if(m_minRefreshInterval > 0) {
        if(m_packetFilterTimer.Time() < m_minRefreshInterval) return false;
        m_packetFilterTimer.Start(0);
    }

    return process(p);
}


bool RTDataMgrFacade::process(const SIPRPacket& p )
{
    //Logger::logf( LOG_MSG, _T("RTDataMgrFacade::process() - m_currInput : %p"), m_currInput );
    //Logger::logf( LOG_MSG, _T("RTDataMgrFacade::process() - realtime type: %d"), p.getSubType() );

    if ( p.type() != SIPR_READ_SENSORS &&
         p.type() != SIPR_READ_MEMORY_32 &&
         p.type() != SIPR_AIM ) { return false; }

    //bool isRequire = isRequireChanged();
    getRequiredSymbols();
    //if(isRequire){
        m_currInput->refreshDependency();
        //Logger::logf( LOG_MSG, _T("RTDataMgrFacade::process() dependency updated"));
    //}

    ((RTDataInputSIPR*)m_currInput)->convert(p);
    return true;

    /// DELETED -->
//    if(isNeedsToNotify) notifyViewers();
    /// <-- DELETED

/*
    //selectInput( INPUT_TYPE_SIPR );
    int subType = p.getSubType();

    switch(subType){
        case SiprSensorType::SIPR_RT_MT_NORMAL:
        case  SiprSensorType::SIPR_RT_MT_EXT:
            ((RTDataInputSIPR*)m_currInput)->setRTDataBuffer( &p.data()[3], p.size() - 3 );
            break;
        default:
            ((RTDataInputSIPR*)m_currInput)->setRTDataBuffer( p.data(), p.size() );
    }
    //Logger::logf( LOG_MSG, _T("RTDataMgrFacade::process( SIPRPacket p ) 111") );
    getRequiredSymbols();
    //Logger::logf( LOG_MSG, _T("RTDataMgrFacade::process( SIPRPacket p ) 222") );
    ((RTDataInputSIPR*)m_currInput)->updateViewerModelSIPR( subType );
    //Logger::logf( LOG_MSG, _T("RTDataMgrFacade::process( SIPRPacket p ) 333") );
    if(isNeedsToNotify) notifyViewers();
    //Logger::logf( LOG_MSG, _T("RTDataMgrFacade::process( SIPRPacket p ) 444") );
    */
}

void RTDataMgrFacade::reset()
{
    m_currInput->reset();
}
