#include "RTDataRawInputBase.h"

#include "../logger.h"
#include "../Math.h"
#include "../SymbolMgr.h"

RTDataRawInputBase::RTDataRawInputBase(SymbolMgr* mt, DoubleHash* viewerModel )
{


    m_symbolMgr = mt;

    if( viewerModel )
    {
        m_viewerModel = viewerModel;
        m_hasInternalModel = false;
    }
    else
    {
        m_viewerModel = new DoubleHash();
        m_hasInternalModel = true;
    }

    m_variableData = VariableDataDH( m_viewerModel );
    m_rpn.setVariableData( &m_variableData );

}

RTDataRawInputBase::~RTDataRawInputBase()
{
    if( m_hasInternalModel ) { delete m_viewerModel; }
}

void RTDataRawInputBase::refreshDependency()
{
}

void RTDataRawInputBase::setRTDataBuffer(uint8_t* buff, int size )
{
    m_rtDataBufferSize  = size;
    m_rtDataBuffer      = buff;
}


void RTDataRawInputBase::fillGraphNodeSet(GraphNodeSet& nodes)
{
    const SymbolTable& symbolTable = m_symbolMgr->getExpTable();
    DoubleHash::iterator it;
    for( it = m_RegisteredSymbolTable->begin(); it != m_RegisteredSymbolTable->end(); ++it )
    {
        wxString symbolName = it->first;
        //Logger::logf( LOG_MSG, _T("RTDataRawInputBase::fillGraphNodeSet(): m_RegisteredSymbolTable: ")+ symbolName);

        SymbolTable::const_iterator it = symbolTable.find(symbolName);
        if(it == symbolTable.end() ) continue;
        Symbol* aSymbol = it->second;
        if( aSymbol->hasDepending() ){
            nodes.insert( aSymbol );
            //Logger::logf( LOG_MSG, _T("RTDataRawInputBase::fillGraphNodeSet(): depending: ")+ symbolName);
        }
    }
}

void RTDataRawInputBase::fillCalculatedValues(GraphNodeSetArray* levelA)
{
    uint32_t i;
    //Logger::logf( LOG_MSG, _T("RTDataRawInputBase::fillCalculatedValues() START Dag level: %i"),levelA->GetCount() );
    for( i = 0; i < levelA->GetCount(); i++ )
    {
        GraphNodeSet* levelSet = levelA->Item( i );

        //Logger::logf( LOG_MSG, _T("RTDataRawInputBase::fillCalculatedValues() - level : %i"), i );

        //Logger::logf( LOG_MSG, _T("RTDataRawInputBase::fillCalculatedValues() - level size: %i"), levelSet->size() );

        GraphNodeSet::const_iterator itSet = levelSet->begin();
        for( ; itSet != levelSet->end(); ++itSet )
        {
            //Logger::logf( LOG_MSG, _T("RTDataRawInputBase::fillSymbolValues() - node addres : %p"), *itSet );
            Symbol* eSymbol = ( Symbol* )(*itSet);
            //m_rtRawValues[ eSymbol->getName() ] = getValue( eSymbol->getName() );
            wxString symbolName = eSymbol->getName();
            double value = getValue( symbolName );
            (*m_viewerModel)[ symbolName ] = value;
            //Logger::logf( LOG_MSG, _T("RTDataRawInputBase::fillCalculatedValues() - expression : ")
            //           + symbolName + _T(" = %f"), value );
        }
    }

    //delete containers( array, set )
    for( i = 0; i < levelA->GetCount(); i++ )
    {
        GraphNodeSet* levelSet = levelA->Item( i );
        delete levelSet;
    }

    delete levelA;
}

void RTDataRawInputBase::fillRegisteredSymbolTable()
{
    //Logger::logf( LOG_MSG, _T("RTDataRawInputBase::fillRegistered()size:%i"),  (*m_RegisteredSymbolTable).size());

    DoubleHash::iterator it;
    for( it = m_RegisteredSymbolTable->begin(); it != m_RegisteredSymbolTable->end(); ++it )
    {
        wxString symbolName = it->first;

        //if values already evaluated no need evalute ( use evaluated data )
        /*
        if( m_viewerModel->find( symbolName ) != m_viewerModel->end() )
        {
            (*m_RegisteredSymbolTable)[symbolName] = (*m_viewerModel)[ symbolName ];
            Logger::logf( LOG_MSG, _T("RTDataRawInputBase::fillRegisteredSymbolTable() 1 - %f symbol: ") + symbolName,  (*m_RegisteredSymbolTable)[symbolName]);
        }
        else
        {*/
            double value = getValue(symbolName);
            (*m_RegisteredSymbolTable)[symbolName] = value;
            //(*m_viewerModel)[ symbolName ] = value;
            // Logger::logf( LOG_MSG, _T("RTDataRawInputBase::fillRegistered() %f symbol: ") + symbolName,  (*m_RegisteredSymbolTable)[symbolName]);
        //}

       // Logger::logf( LOG_MSG, _T("RTDataRawInputBase::fillSymbolValues() - symbol : ") + symbolName );
    }

}

void RTDataRawInputBase::fillSymbolValues(int inputType)
{
    //TODO - it need optimization
    if(inputType == -1){
        fillScalarValues();
    }else{
        //Logger::logf(LOG_MSG, _T("RTDataRawInputBase::fillSymbolValues() inputType:%i"),inputType );
        fillScalarValues(inputType);
        //fillBitValues(inputType);
    }
    //Logger::logMilli(LOG_MSG,wxString::Format(_T("RTDataInputRingBuffer::refresh() START m_RegisteredSymbolTable->size = %i"),m_RegisteredSymbolTable->size()));

    //collect expression symbols with dependency
    GraphNodeSet nodes;
    fillGraphNodeSet(nodes);

    //evaluate expression symbols in order of dependency level
    const Graph& dag = m_symbolMgr->getDAG();
    GraphNodeSetArray* levelA = dag.getNodeDedepending( nodes );
//    dag.printDepending(*levelA);

    //fill calculated viewerData
    fillCalculatedValues(levelA);

    //evaluate all symbol requested by viewers
    fillRegisteredSymbolTable();

    //Logger::logMilli(LOG_MSG,wxString::Format(_T("RTDataInputRingBuffer::refresh() END")));
}


void RTDataRawInputBase::fillScalarValues(int inputType)
{
    const SymbolTable* st = m_symbolMgr->getScalarTable(inputType);

    //Logger::logf(LOG_MSG, _T("RTDataRawInputBase::fillScalarValues() - %p"), st );

    if(!st) return;

    //Logger::logf(LOG_MSG, _T("RTDataRawInputBase::fillScalarValues()") - 10 );

    for(SymbolTable::const_iterator it = st->begin(); it != st->end(); it++){
        Symbol* s = it->second;
        if(!s) { continue ; } //is it possible?
        double value = getValue(s->getSize(), s->getOffset(), s->getScale(), s->getTranslation());
        (*m_viewerModel)[s->getName()] = value;
        //Logger::logf(LOG_MSG, _T("RTDataRawInputBase::fillScalarValues() o:%i,v:%f name:") + s->getName(), s->getOffset(), value);
    }
}



void RTDataRawInputBase::fillScalarValues()
{
    SymbolTable& st = m_symbolMgr->getSymbolTable();
    for(SymbolTable::iterator it = st.begin(); it != st.end(); it++){
        Symbol* s = it->second;
        if(!s) { continue ; } //is it possible?
        if(!s->isConst() && s->isScalar()){
            double value = getValue(s->getSize(), s->getOffset()
                                            , s->getScale(), s->getTranslation());
            (*m_viewerModel)[s->getName()] = value;
             //Logger::log(LOG_MSG, wxString::Format(_T("RTDataMgrSerial::fillScalarValues() Symbol: ") + s->getName()+ _T(", Value: %f"), value));
            //m_rtRawValues[s->getName()] = m_RealTimeDesc.getValue(s->getSize(), s->getOffset()
            //,1, 0);
        }
    }
}


/** @brief getValue
*
*   @todo: document this function
* copied from RealTimeDesc
*/
double RTDataRawInputBase::getValue(int size, int offset, double scale, double translate)
{
   int value = getValue(size, offset);

    //Logger::log(LOG_MSG, wxString::Format(_T("Offset: %d, Size: %d, Scale: %f Tr: %f"), offset, size, scale, translate));
   return scale * ((double)value + translate);
}

int RTDataRawInputBase::getValue(int size, int offset)
{
    if ( ( offset + size ) > m_rtDataBufferSize ) { return 0; }
    if(size < 1 || size > 4) return 0;
    return Math::getNumber((uint8_t*)&(m_rtDataBuffer[offset]),size);
}

/** @brief getValue
  *
  * @todo: document this function
  */
double RTDataRawInputBase::getValue(const wxString& symbolname)
{
    //return 10 +rand()/1000;
    //Logger::log(LOG_MSG,_T("----DataMgrSerial::getValue()---- Symbol: ") + symbolname  );

    if(!m_symbolMgr) { return -1; } //exception needed ?!

    Symbol* s = m_symbolMgr->getSymbol(symbolname);

	if(!s){ return -1; } // exception needed ?!

    if( /*m_rtRawValues.find(symbolname) != m_rtRawValues.end() */ s->isScalar() ){
//	    Logger::log(LOG_MSG, wxString::Format(_T("Symbol (scalar): ") + symbolname + _T(", Value: %f !!!!!!!!!!!!!"), m_rtRawValues[symbolname] ));
		//Logger::logf( LOG_MSG, _T("RTDataRawInputBase::getValue() - ") + s->getName() + _T(" = %f"), m_rtRawValues[symbolname] );
		return (*m_viewerModel)[symbolname];

	}else if(s->isExpr()){

	    wxString expr = s->getExpr();
	    //double value = m_eval.calculate(expr);
        //Logger::logf( LOG_MSG, _T("RTDataRawInputBase::getValue() - symbol : ") + s->getName() );

        double value = m_rpn.calculate( expr );
        //Logger::logf( LOG_MSG, _T("RTDataRawInputBase::getValue() - ") + s->getName() + _T(" = %f"), value );
        //Logger::logf(LOG_MSG, _T("Symbol: ") + symbolname + _T(" Expr: ") + expr + _T(", Value: %f"), value );
        /*if( symbolname == _T("longitude") )
        {
            Logger::logf( LOG_MSG, _T("RTDataRawInputBase::getValue() - longitude = %f"), value );
        }*/

	  	return value;
    }else{
        return -1; // exception needed ?!
    }

}

void RTDataRawInputBase::printViewerModel()
{
    size_t s = m_viewerModel->size();
    Logger::logf(LOG_MSG, _T("RTDataRawInputBase::printViewerModell() size: %d"), s);
    DoubleHash::const_iterator IT =  m_viewerModel->begin();
    for(; IT != m_viewerModel->end();++IT ){
        wxString key = IT->first;
        double value = IT->second;
        Logger::logf(LOG_MSG, _T("RTDataRawInputBase::printViewerModell(): ")+key+_T("=%f"), value);
    }
}
