#ifndef RTDATARAWLOGGER_H
#define RTDATARAWLOGGER_H

#include <wx/wfstream.h>
#include "../common/typedef.h"
#include "../common/types/StringHash.h"
class GPSData;
class FirmwareInfo;

class RTDataRawLogger
{
    public:
        RTDataRawLogger(FirmwareInfo* fwInfo, GPSData* gps = NULL );
        virtual ~RTDataRawLogger();
        void SetFile(const wxString& ofileName);
        void SetSize(int size);
        void SetBlockSize(int bSize);
        void SetLog(bool b);
        bool isRawLog();
        void Write(const char * buff);
        void Close();
        void WriteHeader();
        void AddHeaderItem(const wxString &key, const wxString &value);

        void Flush();
        void Flush(uint8_t* data);

        bool isHeaderWritten(){ return m_isHeaderWritten;}
        void setHeaderWritten(bool b){ m_isHeaderWritten =b;}

        void setGPSLog( bool gpsLog ) { m_gpsLog = !m_isLog && gpsLog; }
        bool isGPSLog() { return m_gpsLog; }
        //void WriteGPSData( const char* buffer );
        //void SetGPSDataSize( int size ) { m_gpsDataSize = size; }
        void captureEcuBlock();
        int getRecordCountInBlock() const;
        int getRecordSize() const;

        FirmwareInfo* GetFWInfo() { return m_fwInfo; };

    protected:

    private:
        bool m_isLog;
        bool m_gpsLog;
        int m_bufferCount;
        int m_CapturedBufferCount;
        char * m_buffer;
        wxFileOutputStream * m_fos;

        int m_bufferSize;
        int m_dataSize;
        //int m_gpsDataSize;
        bool m_isHeaderWritten;

        wxString m_fileName;

        StringHash  m_headerHash;
        GPSData*    m_gpsData; //an external packed gps data container
        FirmwareInfo* m_fwInfo;

        int m_maxRTDataCount;
        int m_RTDataIdx;

        void fillBuffer();
        void WriteGPSData();
        void Write(const char * buff, int size);
        void calcMaxRTDataCount();
        bool getNextRTDataIdx();
        void WriteHeader(const char * buff, size_t size);
        void fillHeader();
        //wxBufferedOutputStream * m_bfos;

};



#endif // RTDATARAWLOGGER_H
