#ifndef RTDATAVIEWERBASE_H
#define RTDATAVIEWERBASE_H

#include "RTDataMgrBase.h"
#include "../logger.h"
/// Forward Declarations
//class RTDataMgrBase;
     /**
 *  RTDataViewerBase class. <b>The Base class for every Realtime data viewer.</b> A viewer can be a GUI widget, file writer, etc.
 *  Currently all GUI viewer widgets and the high level(csv) log writer inherited from this class.
 *  This class is do not care from where the data is cme from (it could came from everywere).
 *  The viewers must be registered  into RTDataMgrBase. To
 *  RTDataMgrBase Put together (The viewers and the input source)
 */

class RTDataViewerBase
{
    public:
        RTDataViewerBase();
        virtual ~RTDataViewerBase();
        virtual void SetValue(double value) = 0;
        //virtual double GetValue() = 0;
        virtual void refresh(DoubleHash* table) = 0;
        //virtual void require(DoubleHash* table) = 0;
        virtual void require() = 0;
        virtual void unregisterSymbols() = 0;

        virtual void SetValueIdx(int idx) = 0;

        void requireChanged()
        {
            if( m_rtDataMgr != NULL ) { m_rtDataMgr->requireChanged(); }
        }

		void SetDataMgr (RTDataMgrBase * mgr)
		{
            m_rtDataMgr = mgr;
            requireChanged();
        }

        void RegisterViewer()
        {
            if(m_rtDataMgr != NULL) { m_rtDataMgr->RegisterViewer(this); }
        }

        void printThisptr(){
          Logger::logf(LOG_MSG,_T("RTDataViewerBase ptr=%p"),this);
        }


        void UnRegisterViewer()
        {
            m_rtDataMgr->UnRegisterViewer(this);
        }


        bool isExist(const wxString& symbolname){
            if(m_rtDataMgr == NULL) return false;

            return m_rtDataMgr->isExist(symbolname);
        }
/*
        void require(DoubleHash* table)
        {
            (*table)[GetSelectedSymbol()];
        }
*/
        //update?

        bool IsRegistered() {
//            Logger::logf(LOG_MSG, _T("RTDataViewerBase::IsRegistered ptr=%p"),this);
            return m_isRegistered;
        }

        void setRegistered(bool reg){
            m_isRegistered = reg;
        }

    protected:
        double m_value;
        RTDataMgrBase * m_rtDataMgr;   // do not delete it in desctructor
    private:
        bool m_isRegistered;
};

WX_DEFINE_ARRAY( RTDataViewerBase*, ViewerBaseArray );

#endif // RTDATAVIEWERBASE_H
