#include "MMCLogger.h"
#include "../common/typedef.h"


#include "../Math.h"
#include "../DirHelper.h"
#include "../RawLogHelper.h"
#include "../logger.h"
#include "../FirmwareInfo.h"
#include "../SymbolMgr.h"
#include "../mtconfigdata.h"

#include "MMCLoggerViewerBase.h"

#include "../vemsprotocol/STDataModelWriter.h"
#include "../stdatamanager/STDataModelBase.h"
#include "../vemsprotocol/SIPRFileLogger.h"
#include "../vemsprotocol/SIPRTypes.h"
#include "../common/VemsContext.h"
#include "../rtdatamanager/RT2ViewerConverter.h"
#include "../gui/FileOpenSQL.h"


MMCLogger::MMCLogger()
{
    m_stopIdx = 0;
    m_size =0;
    m_needUpdate = true;
    m_isAborted = false;
    ///m_fileName;
    m_lastSecCountInBlock = -255;
    m_startNewLogBlockIdx = -1;
    m_blockCount =1;

}

MMCLogger::~MMCLogger()
{
    //dtor
}


void MMCLogger::notify( int i )
{
    if( i == 0 ) { m_isAborted = true; }
}

void MMCLogger::Update(int logtype)
{
    Logger::logf(LOG_MSG,_T("MMCLogger::Update "));
    m_stopIdx=getStopIdx(logtype);

    Logger::logf(LOG_MSG,_T("MMCLogger::Update StopIdx=%i"), m_stopIdx);

    int errorCount = 0;

    do{
        m_size = getDataSize(m_stopIdx);
        Logger::logf(LOG_MSG,_T("MMCLogger::Update Size=%i"), m_size);

        if (m_size < 0) {
            errorCount++;
        }
    }while( (m_size < 0) &&  errorCount < 10);
    if (errorCount==10) return;

    m_needUpdate=false;

}
bool MMCLogger::CheckSectorIsEmpty(uint8_t * data, size_t len)
{
    size_t nulls = 0;
    size_t ffs = 0;

    for(size_t f=0; f<len;f++){
        if( data[f] == 0 ) nulls++;
        if( data[f] == 0xFF ) ffs++;
    }

    if( nulls == len || ffs == len )
        return true;
    else
        return false;
}

uint32_t MMCLogger::CalcLogTimeLenght( uint32_t addresslen, int datasize )
{
    return addresslen * recordPerBlock( 512, datasize ) / 10; // 100ms, return in sec
    //return (uint32_t)(addresslen * 51.2 / datasize );
}

uint32_t MMCLogger::getIntervalIdx(uint16_t interval_time)//min
{
    Logger::logf(LOG_MSG,_T("MMCLogger::getIntervalIdx"));

    uint32_t int_idx=(uint32_t) (interval_time*60*m_size/51.2);
    return int_idx;
}
uint32_t MMCLogger::getStartIdx(uint16_t back_time)//min
{
    Logger::logf(LOG_MSG,_T("MMCLogger getStartIdx time=%i"), back_time);

    uint32_t blockIdx;
    uint32_t back_block=(uint16_t)(back_time*60*m_size/51.2);
    back_block < m_stopIdx ? blockIdx = m_stopIdx-back_block : blockIdx = 0;
    Logger::logf(LOG_MSG,_T("MMCLogger getstartidx final blockidx=%i ,back_block:%d"),blockIdx, back_block);

    return blockIdx;
}
uint32_t MMCLogger::getStopIdx(int logtype)
{
    flushBuffer();

/*  old binary search
    int8_t count=23;
    uint32_t blockIdx=0x200000;
    uint32_t mask=0x800000;
    for(count=23;count>=0;count--){
        Logger::logf(LOG_MSG,_T("MMCLogger getstartidx%d() blockidx=%i maszk=%d"), count,blockIdx,mask);
         wxString message = wxString::Format(_T("Searching: %d"), blockIdx );
        observerNotify(message);
        if ( getBlockState(blockIdx) == -2 ) blockIdx &= ~mask;
        mask >>= 1;
        blockIdx |= mask;

    }
*/
    uint32_t blockIdx;
    uint32_t minIdx;

    if (logtype==0){//old inode-less behaviour
        blockIdx=0x00;
        minIdx=0x00;
    }else if (logtype==1){// first inode mode
        blockIdx=0x100;
        minIdx=0x100;
    }else {
        blockIdx=0x100;
        minIdx=0x100;
    }

    int8_t count=0;

    uint32_t mask=0x01;
    do{
        Logger::logf(LOG_MSG,_T("MMCLogger getstopidx%d()up blockidx=%i maszk=%d"), count,blockIdx,mask);
        wxString message = wxString::Format(_T("Searching: %d"), blockIdx );
        observerNotify(message);

        blockIdx |=mask;
        mask <<=1;
        count++;
    }while( !(CheckSectorIsEmpty(blockIdx)));
    blockIdx++;
    mask <<=1;
    count++;
    do{
        Logger::logf(LOG_MSG,_T("MMCLogger getstopidx%d()down blockidx=%i maszk=%d"), count,blockIdx,mask);
        wxString message = wxString::Format(_T("Searching: %d"), blockIdx );
        observerNotify(message);

        if ( CheckSectorIsEmpty(blockIdx) ) blockIdx &= ~mask;
        mask >>= 1;
        blockIdx |= mask;
        if ( blockIdx<minIdx ) blockIdx=minIdx;
        count--;
    }while(count>=0);

    return blockIdx;

}

bool MMCLogger::CheckSectorIsEmpty(uint32_t blockIdx)
{
    Logger::logf(LOG_MSG,_T("MMCLogger::CheckSectorIsEmpty()"));

    uint8_t data[512+2+2];

    if( getMMCBlock(blockIdx, data, 514) ){
        int data_or=0;
        int data_and=0xff;
        int i;
        for(i = 0; i < 512; i++){
            data_or|=data[i];
            data_and&=data[i];
        }
        Logger::logf(LOG_MSG,_T("MMCLogger::CheckSectorIsEmpty data_or=%x data_and=%x"), data_or,data_and);

        if ((data_or==0) || (data_and==0xff)) return true;
        else return false;
    }
    Logger::logf(LOG_MSG,_T("MMCLogger::CheckSectorIsEmpty timed out"));

    return true; //COMM_timed_out

}
/*
int MMCLogger::getBlockState(uint32_t blockIdx)
{
   uint8_t data[512+2+2];

    if( getMMCBlock(blockIdx, data, 514) ){
        bool iscrcOK = isCRCValid(data, 514);
        if(!iscrcOK) return -1; //COMM_ERROR

        int dataSize = data[510];
        if(dataSize == 0 || dataSize == 255) return -2; //CARD EMPTY
        return dataSize;
    }
    return -2; //COMM_timed_out

}
*/

bool MMCLogger::isNextLog(uint8_t * data)
{
//    Logger::logf(LOG_MSG,_T("MMCLogger isNextLog() START"));
    int secOffset = 0;
    int firstSecCountInBlock = data[secOffset];


    if(firstSecCountInBlock == m_lastSecCountInBlock){
        m_lastSecCountInBlock = firstSecCountInBlock;
        return false;
    }
    if( (firstSecCountInBlock - m_lastSecCountInBlock) == 1 ){
        m_lastSecCountInBlock = firstSecCountInBlock;
        return false;
    }
    if( (firstSecCountInBlock == 0) && (m_lastSecCountInBlock == 255) ){
        m_lastSecCountInBlock = firstSecCountInBlock;
        return false;
    }
    m_lastSecCountInBlock = firstSecCountInBlock;

    Logger::logf(LOG_MSG,_T("MMCLogger isNextLog() blockIdx = TRUE"));
/*
    if( (blockIdx == 0) ){
         Logger::logf(LOG_MSG,_T("MMCLogger isNextLog() blockIdx = TRUE"));
        return true;
    }
    // && ( (blockIdx - m_startNewLogBlockIdx) < 10) )
    //

    m_startNewLogBlockIdx = blockIdx;
    //Logger::logf(LOG_MSG,_T("MMCLogger isNextLog() TRUE"));
*/
    return true;
}

int MMCLogger::getDataSize(uint32_t address)
{
     uint8_t data[512+2+2];
    if( getMMCBlock(address, data, 514) ){
        bool iscrcOK = isCRCValid(data, 514);
        if(!iscrcOK) return -1; //COMM_ERROR

        int dataSize = data[510];
        if(dataSize <= 0 || dataSize == 255) return -2; //CARD EMPTY
        return dataSize;
    }
    return -1; //COMM_ERROR

}

int MMCLogger::save(const FirmwareInfo& fwInfo, wxArrayString& files, const MMCFilesDesc& mmcFileDesc, MTConfigData* mtConfigData)
{
    /// online analyze SD Card log
    // init
    DoubleHash rtViewerModel;

    if ( mtConfigData == NULL )
    {
        Logger::logf( LOG_MSG, _T("MMCLogger::save mtConfigData is NULL!") );
    }
    SymbolMgr& symbolmgr = mtConfigData->getSymbolMgr();

    RT2ViewerConverter* converter = new RT2ViewerConverter( &symbolmgr, &rtViewerModel );
    DoubleHash requiredD;
    requiredD[_T("rpm")] = 0.0f; requiredD[_T("map")] = 0.0f; requiredD[_T("isTriggerError")] = 0.0f;

    double rpmMax = -1;
    double mapMax = -1;
    int triggerError = 0;
    bool wasTriggerErr = false;


    ///

    m_lastSecCountInBlock = -255;

    uint32_t blockIdx = mmcFileDesc.StartAddress;
    uint32_t endIdx = mmcFileDesc.EndAddress;
    Logger::logf(LOG_MSG,_T("MMCLogger::save startIdx = %d, endIdx = %d"), blockIdx, endIdx);
    if (endIdx==0)endIdx--;
    if (endIdx>m_stopIdx) endIdx=m_stopIdx;

    flushBuffer();

    SIPRFileLogger* sipr_logger = NULL;


    int blockSize = 514; /// 512 byte data + 2 byte checksum
    bool isValid = true;
    bool iscrcOK = false;
    int crcErrorCount = 0;

    //!! important: m_blockCount must be 1 for image / 1 or 60 for V3
    int blockCount = m_blockCount; /// firmware support 2 option: it can be 1 or 60

    // uint8_t data[ (60+4) * (514+2)]; /// we allocate a litte bit more space
    uint8_t * data = new uint8_t[ (blockCount+4) * (514+2)];

    uint8_t* siprdata = new uint8_t[m_size+3];
    uint8_t counter = 0;

    siprdata[0] = 0; siprdata[1] = 0;
    siprdata[2] = SiprSensorType::SIPR_RT_MT_NORMAL;

    int i;

    wxString  fileName;

    long timecounter = 0;

    do{
        observerNotify(blockIdx);

        int percent = (int)((blockIdx + 1 - mmcFileDesc.StartAddress) * 100.0 / (mmcFileDesc.EndAddress - mmcFileDesc.StartAddress));

        observerNotify( -1, percent );

        if( !getMMCBlock(blockIdx, data, blockCount * blockSize) ){
            Logger::logf(LOG_MSG,_T("MMCLogger save() data(0,1,510,511): %02x, %02x, %02x, %02x"),
            data[0], data[1], data[510], data[511]);
            break;
        }else{
            //Logger::logf(LOG_MSG,_T("MMCLogger save() blockIdx=0x%08x, %08d ok"),blockIdx,blockIdx);

        }

        /// CRC for all block
        for(i = 0; i< blockCount; i++){
            iscrcOK = isCRCValid(&data[i*blockSize], blockSize);
            if( !iscrcOK){
             Logger::logf(LOG_MSG,_T("MMCLogger save() CRC16 failed! idx: %i"), i);
             crcErrorCount++;
             continue; // Even for a full 0xFF block we require valid CRC.
            }else{
                crcErrorCount = 0;
            }
        }

        if ( endIdx < (blockIdx+blockCount) ){
            blockCount=endIdx-blockIdx+1;
            }

        for(i = 0; i< blockCount; i++){
            uint8_t* dataPtr = &data[i*blockSize];
            isValid = isDataValid(dataPtr, blockSize-18);
            if(isValid ){
                //Logger::logf(LOG_MSG,_T("MMCLogger save() block valid : %i"), i);

                if( isNextLog(dataPtr ) ){
                    counter = 0;
                    if (sipr_logger != NULL) delete sipr_logger;
                    sipr_logger = new SIPRFileLogger(512);
                    sipr_logger->enableLog(true);
                    timecounter = 0;

                    wxString header = RawLogHelper::getVemslogHeader(fwInfo, 2);

                    wxString sn = fwInfo.getSerialNumber();
                    wxString prefix = wxString::Format(_T("-%03d"), mmcFileDesc.RowId + 1);
                    fileName = DirHelper::CreateNewFileName(sn+prefix+_T("-SDCard.vemslog"),
                            DirHelper::GetPath(dLOG),true,true);

                    sipr_logger->setFile(fileName);

                    sipr_logger->setHeader(header);
                    wxObject* modelbase = VemsContext::getObject(_T("STDataModelBase"));
                    if (modelbase != NULL)
                        STDataModelWriter::logPages((STDataModelBase*)modelbase, sipr_logger, mtConfigData->getSIPRDescriptor() );
                    else
                        Logger::logf(LOG_MSG, _T("STDataModelSerial modelbase is NULL!!"));

                    files.Add(fileName);
                } // isNextLog

                if (sipr_logger == NULL)
                    continue;

//                if (sipr_logger == NULL)
//                {
//                    delete siprdata;
//                    delete data;
//
//                    if (sipr_logger != NULL) delete sipr_logger;
//                    return 0;
//                }

                uint8_t _size = dataPtr[511];

                size_t block_count = (int)(floorf((blockSize-3)/_size));
                unsigned int bfrom = 0;
                //Logger::logf(LOG_MSG, _T("MMCLogger::save() Block count: %d, size: %d"), block_count, _size);
                for ( unsigned j = 0; j < block_count; ++j)
                {
                    bfrom = j * _size;
                    //Logger::logf(LOG_MSG, _T("MMCLogger::save() before memcpy") );
                    memcpy( &siprdata[3], &dataPtr[bfrom], _size );
                    //Logger::logf(LOG_MSG, _T("MMCLogger::save() after memcpy") );

                    uint8_t timebuffer[4] = { 0, 0, 0, 0 };
                    Math::getBytes( timecounter, timebuffer, 4 );

                    SIPRPacket timePacket( 0, SIPR_RT_TIME, 4, timebuffer );
                    sipr_logger->process( timePacket );

                    SIPRPacket p( counter , SIPR_READ_SENSORS, _size+3, siprdata );
                    //Logger::logf(LOG_MSG, _T("MMCLogger::save() before process") );
                    sipr_logger->process(p);

                    timecounter += 100; // add 100 ms

                    /// analyze
                    DoubleHash reqdata(requiredD);
                    converter->convert( p, reqdata );
                    rpmMax = ( rpmMax < reqdata[_T("rpm")] ) ? reqdata[_T("rpm")] : rpmMax;
                    mapMax = ( mapMax < reqdata[_T("map")] ) ? reqdata[_T("map")] : mapMax;
                    bool isTE = reqdata[_T("isTriggerError")];
                    if ( isTE && ( wasTriggerErr == false ) )
                    {
                        wasTriggerErr = true;
                        triggerError++;
                    }
                    else if ( wasTriggerErr && ( isTE == false ) )
                    {
                        wasTriggerErr = false;
                    }

                    /// end of analyze

                    //Logger::logf(LOG_MSG, _T("MMCLogger::save() afrer process") );
                    if (counter == 255) counter = 0;
                    else counter++;
                }
            }else{
                //Logger::logf(LOG_MSG, _T("MMCLogger::save() block ondataPtr: %d is not valid"), dataPtr);

            }
        }
        blockIdx += blockCount;
    } while( !m_isAborted && crcErrorCount < 4 && blockIdx<endIdx);

    Logger::logf(LOG_MSG,_T("MMCLogger save() isValid:%i, isAborted:%i"), isValid, m_isAborted );

    FileInfo info;
    info.fw_major = fwInfo.getFirmwareVersion().major;
    info.fw_minor = fwInfo.getFirmwareVersion().minor;
    info.fw_build = fwInfo.getFirmwareVersion().build;
    info.rpmMax = (int)rpmMax;
    info.mapMax = (int)mapMax;
    info.triggerErr = triggerError;

    FileAnalyzeSQL::addData( fileName, info );

    delete[] siprdata;
    delete[] data;

    if (sipr_logger != NULL) delete sipr_logger;

    return 0;
}



int MMCLogger::ExtOverview( MMCLoggerViewerBase * oframe )
{
    flushBuffer();

    Uint32_tArray arr;

    Logger::logf(LOG_MSG,_T("MMCLogger::ExtOverview start"));

    for( uint32_t blockIdx= 0; 1 ; blockIdx++){
        uint8_t data[514];

        getMMCBlock(blockIdx, data, 514 );
//        m_commBase->SendString( wxString::Format(_T("msg%06X"),a) );
//        m_commBase->ReadString(data, 514, 16);

        Logger::logf(LOG_MSG,_T("MMCLogger::ExtOverview blockIdx=%i 3"),blockIdx);

        if(CheckSectorIsEmpty(data, 512)) break;

        wxString vh = wxString::From8BitData((const char*)data, 4);

        Logger::logf(LOG_MSG,_T("MMCLogger::ExtOverview 4 %i,%i, s=")+vh,data[0],data[1]);

        if( vh != _T("VEMS") ) return -2;

        Logger::logf(LOG_MSG,_T("MMCLogger::ExtOverview 5"));

        uint16_t rver = data[4] << 8;
        rver += data[5];

        FirmwareVersion fv;

        fv.build = rver % 100;
        rver /= 100;
        fv.minor = rver % 100;
        rver /= 100;
        fv.major = rver;

        data[512] = 0;
        data[513] = 0;

        switch( data[7] ) {
            case 1: {
                for(uint16_t i=8; i<512; i+=3){
                    uint32_t ad = data[i] << 15;
                    ad += data[i+1] << 7;
                    ad += data[i+2] >> 1;
                    if(ad == 0) break;
                    arr.Add( ad );
                    Logger::logf(LOG_MSG,_T("MMCLogger::ExtOverview 10 s=%X"),ad);
                }
               }break;

            default:
                return -1;
        }
    }

    if(!arr.GetCount()) return -3;

    for(size_t a=0; a< arr.GetCount()-1;a++){

        MMCFilesDesc desc;

        desc.StartAddress = arr[a];
        desc.EndAddress = arr[a+1] - 1;
        desc.DataSize = m_size;

        desc.Time = CalcLogTimeLenght( desc.EndAddress - desc.StartAddress, m_size );

        Logger::logf(LOG_MSG,_T("MMCLogger::ExtOverview 15 s=%i, e=%i, ds=%i"),desc.StartAddress,desc.EndAddress,m_size);

        if (desc.StartAddress<desc.EndAddress) oframe->AddFileListLine( desc );
    }

    MMCFilesDesc desc;

    desc.StartAddress = arr[arr.GetCount()-1];
    desc.EndAddress = m_stopIdx;
    desc.DataSize = m_size;

    desc.Time = CalcLogTimeLenght( desc.EndAddress - desc.StartAddress, m_size );

    Logger::logf(LOG_MSG,_T("MMCLogger::ExtOverview last block s=%i, e=%i, ds=%i"),desc.StartAddress,desc.EndAddress,m_size);

    if (desc.StartAddress<desc.EndAddress) oframe->AddFileListLine( desc );

    return 0;
}

int MMCLogger::Summary( MMCFilesDesc * desc )
{
    if( desc == NULL ) return -1;

    Logger::logf(LOG_MSG,_T("MMCLogger Summary() start"));
    Logger::logf(LOG_MSG,_T("MMCLogger Summary() 1 s=%i"),desc->SampleCnt);

    flushBuffer();

    uint8_t errorcnt=0;
    bool result;
    bool iscrcOK;
    bool isValid;

    uint8_t mmcdata[512+2+2];

    uint32_t diff = desc->EndAddress - desc->StartAddress;

    uint32_t inc = (desc->EndAddress - desc->StartAddress) / desc->SampleCnt;
    if (inc==0) inc=1;
    desc->Summaries[_T("Max RPM")] = 0L;
    desc->Summaries[_T("AVG RPM")] = 0L;
    desc->Summaries[_T("Max MAP")] = 0L;
    desc->Summaries[_T("AVG MAP")] = 0L;
    desc->Summaries[_T("ALS On")] = 0L;
    desc->Summaries[_T("Launch On")] = 0L;

    Logger::logf(LOG_MSG,_T("MMCLogger Summary() 5"));

    int realSampleCnt = 0;

    for( uint32_t idx = desc->StartAddress; idx < desc->EndAddress; idx += inc ){
            Logger::logf(LOG_MSG,_T("MMCLogger Summary() 10 idx=%d inc=%d"),idx,inc);
            realSampleCnt++;

        do{
            result = getMMCBlock(idx, mmcdata, 514);
            observerNotify( idx - (desc->StartAddress), diff );
            //observerNotify( 50, 100 );
            iscrcOK = isCRCValid(mmcdata, 514);
            isValid = isDataValid(mmcdata, 514);

            if( iscrcOK && isValid){
                errorcnt=0;

                long rpm = (mmcdata[14]*0xff)+mmcdata[15];
                long map = ((mmcdata[4]*0xff)+mmcdata[5])/4;

                if( desc->Summaries[_T("Max RPM")].GetLong() < rpm ) desc->Summaries[_T("Max RPM")] = rpm;
                if( desc->Summaries[_T("Max MAP")].GetLong() < map ) desc->Summaries[_T("Max MAP")] = map;

                desc->Summaries[_T("AVG RPM")] = desc->Summaries[_T("AVG RPM")].GetLong() + rpm;
                desc->Summaries[_T("AVG MAP")] = desc->Summaries[_T("AVG MAP")].GetLong() + map;

                if ( (mmcdata[47]&128) == 128 ){
                    desc->Summaries[_T("ALS On")] = desc->Summaries[_T("ALS On")].GetLong() + 1;
                }
                if ( (mmcdata[47]&64) == 64 ){
                    desc->Summaries[_T("Launch On")] = desc->Summaries[_T("Launch On")].GetLong() + 1;
                }


            } else {
                errorcnt++;
            }

        }while((errorcnt<4) && !result );

    }

    desc->SampleCnt = realSampleCnt;

    desc->Summaries[_T("AVG RPM")] = desc->Summaries[_T("AVG RPM")].GetDouble() / (double)(desc->SampleCnt);
    desc->Summaries[_T("AVG MAP")] = desc->Summaries[_T("AVG MAP")].GetDouble() / (double)(desc->SampleCnt);
    desc->Summaries[_T("ALS On")] = desc->Summaries[_T("ALS On")].GetDouble() *100.0 / (double)(desc->SampleCnt);
    desc->Summaries[_T("Launch On")] = desc->Summaries[_T("Launch On")].GetDouble() *100.0 / (double)(desc->SampleCnt);


    return 1;
}

int MMCLogger::overview(MMCLoggerViewerBase* OvFrame,uint32_t startIdx)
{
    flushBuffer();
    //if ( id == idMenuMMCLogSaveT ) startIdx=mmcLog.getStartIdx(m_mmcBackTime); // request( new SDLogSaveCommand( backTime ) )

    uint32_t blockIdx=startIdx;
    uint8_t mmcdata[512+2+2];
    bool result;
    bool iscrcOK;
    bool isValid;
    uint8_t errorcnt=0;

    double timescaler=60*m_size/51.2;
    do{
        result = getMMCBlock(blockIdx, mmcdata, 514);
        observerNotify(blockIdx);

        /// CRC
        iscrcOK = isCRCValid(mmcdata, 514);
        isValid = isDataValid(mmcdata, 514);
        if( iscrcOK && isValid){
            errorcnt=0;
            uint8_t tps=mmcdata[8];
            float tps_f=tps*100/256;

            DoubleHash Ovhash;
            Ovhash[_T("block")]=blockIdx;
            Ovhash[_T("time")]=blockIdx/timescaler;
            Ovhash[_T("tps")]=tps_f;
            Ovhash[_T("rpm")]=(mmcdata[14]*0xff)+mmcdata[15];
            Ovhash[_T("map")]=((mmcdata[4]*0xff)+mmcdata[5])/4;
            Ovhash[_T("als")]=(mmcdata[47]&128)==128;
            Ovhash[_T("mat")]=mmcdata[6]-100;
            Ovhash[_T("coolant")]=mmcdata[40];//mmcdata[7]-100;



            OvFrame->AddGridLine(Ovhash);
            blockIdx+=1;//120;


        }else{
            errorcnt++;
        }

    }while(result && (errorcnt<4) && isValid);
    if (result) return 1;//succes
    else return -1;//comm error

}

bool MMCLogger::isDataValid( uint8_t* data, int len)
{/*
    int i;
    for(i = 0; i < len; i++){
        if( data[i] != 0xFF) return true;
    }
    return false;
   */

    int data_or=0;
    int data_and=0xff;
    int i;
    for(i = 0; i < len; i++){
    data_or|=data[i];
    data_and&=data[i];
    }
    if ((data_or==0) || (data_and==0xff)) return false;
    if (data[511]!=m_size) return false;
    else return true;

}
