#include "MMCLoggerManager.h"

#include "../logger.h"

#include "../SymbolMgr.h"
#include "../mtconfigdata.h"

#include "../VemsTuneController.h"
#include "../ProgressBarObserver.h"

#include "MMCLoggerV3.h"
#include "MMCLoggerImg.h"

#include "MMCLoggerExtOverviewFrame.h"
#include "MmcOverviewFrame.h"

#include "../vemsprotocol/ECUStateMgrBase.h"
#include "../vemsprotocol/ECUInfo.h"

#include "../UserLog.h"
#include <wx/progdlg.h>

MMCLoggerManager::MMCLoggerManager( VemsTuneController * vtc, CommunicationBase* commBase, MTConfigData* mtConfigData, SdImage* source)
{

    m_vemsTuneController = vtc;
    m_source = source;
    m_mtConfigData = mtConfigData;

    // bool wasECUOnline = ( m_vemsTuneController->getDevice() == ECUInfo::DEVICE_V3 );

    if ( m_source == NULL ) m_mmcLog = new MMCLoggerV3( commBase );
    else m_mmcLog = new MMCLoggerImg( source );

    m_observer = NULL;
}


MMCLoggerManager::~MMCLoggerManager()
{
    delete m_mmcLog;
}


bool MMCLoggerManager::StartSession()
{
    if( m_vemsTuneController->getDevice() != ECUInfo::DEVICE_V3){ return false; }


    m_vemsTuneController->setECUOnline(false, true);

    m_vemsTuneController->getEcuStateMgr()->setState( ECUInfo::MAN );
//    m_vemsTuneController->getECUController()->stop();

    return true;
}

bool MMCLoggerManager::EndSession()
{
//    if ( !wasECUOnline )
//        return false;

    m_vemsTuneController->setECUOnline(true, false);
    return true;
}

void MMCLoggerManager::Update(int logtype)
{

    if( m_source==NULL && !(m_mmcLog->needsUpdate()) ) return;

    wxString message    = _T("Searching last block: ");
    wxString title      = _T("SD Log Opening");

    ProgressBarObserver pbo;
    pbo.InitPulse( title, message );
    m_mmcLog->registerObserver( &pbo );
    pbo.registerObserver( m_mmcLog );

    m_mmcLog->Update(logtype);

    m_mmcLog->unRegisterObserver( &pbo );
    pbo.unRegisterObserver( m_mmcLog );
    pbo.Destroy();

}

void MMCLoggerManager::StartLog()
{
    if( !StartSession() ) return;
    m_mmcLog->start();
    EndSession();
}

void MMCLoggerManager::StopLog()
{

    if( !StartSession() ) return;
    m_mmcLog->stop();
    EndSession();
}

int MMCLoggerManager::EraseLog(wxWindow * parent)
{
    if( !StartSession() ) return -2;

    wxProgressDialog* pbd= new wxProgressDialog(_T("SD card erase progress"),_T("SD card erase progress\n This can be take 2 minutes.") , 5, parent);
    pbd->Update(0);
    wxSafeYield( pbd, true );
    pbd->Show();
    uint8_t trycount = 0;
    while ( (m_mmcLog->CheckSectorIsEmpty(uint32_t (0x00000000))==false) && (trycount < 5)){
        trycount++;
        pbd->Update(trycount);
        Logger::logf(LOG_MSG,_T("MMCLoggerManager::EraseLog trycount=%d"),trycount);

        m_mmcLog->erase();
        wxThread::Sleep(3000);
    }
    EndSession();
    pbd->Destroy();
    if (trycount == 5) return -1;//erase failed
    return 0;
}

void MMCLoggerManager::OverView(wxWindow * parent)
{
    // if V3:
    if ( m_source == NULL )
    {
        if( !StartSession() ) return;
    }

    Update();

    m_mmcLog->SetBaudrate();


    MMCLoggerExtOverviewFrame* obsFrame = new MMCLoggerExtOverviewFrame(parent);
    m_observer = obsFrame;
    obsFrame->SetMMCLoggerManager(this);
    int result = m_mmcLog->ExtOverview( obsFrame );

    wxString title = _T("SD overview error");

    switch ( result )
    {
        case -1:
            wxMessageBox( _T("Unknown data type\n try to update your VemsTune"), title, wxICON_ERROR | wxOK );
            break;
        case -2:
            wxMessageBox( _T("The Card not contains Vems data logs or it is uses the older mode\n try to view with OverviewOld"), title );
            break;
        case -3:
            wxMessageBox( _T("The Card is empty"), title );
            break;
        default:
            break;
    }

    obsFrame->ShowModal();

}

void MMCLoggerManager::Summary( MMCFilesDesc *desc, ObserverBase* observerBase, Observable* observable)
{
    //if( !StartSession() ) return;

    //Update();

    //SetBaudrate();

    m_mmcLog->registerObserver( observerBase );
    observable->registerObserver( m_mmcLog );
    m_mmcLog->Summary( desc );

    //ResetBaudrate();

    //EndSession();

}

void MMCLoggerManager::OverViewOld(wxWindow * parent)
{
    if( !StartSession() ) return;

    Update(0);

    m_mmcLog->SetBaudrate();

    MmcOverviewFrame *Ovframe = new MmcOverviewFrame(parent, this);
    Ovframe->Show(true);
    Logger::logf(LOG_MSG,_T("new MmcOverviewFrame"));

    wxString message    = _T("scanning block: ");
    wxString title      = _T("Scanning SD Card");

    ProgressBarObserver pbo;
    pbo.InitPulse( title, message );
    m_mmcLog->registerObserver( &pbo );
    pbo.registerObserver( m_mmcLog );
    uint32_t startIdx=0;
    int result=m_mmcLog->overview(Ovframe,startIdx);


    m_mmcLog->unRegisterObserver( &pbo );
    pbo.unRegisterObserver( m_mmcLog );
    pbo.Destroy();


    if(result <1){
        m_mmcLog->stop();
        // m_vp->SendString( (uint8_t*)("msd") );

    }else {
        Ovframe->Show(true);
    }

    if(result == -1){
        message =_T("Communication error! (Most probably the firmware doesnt support this feature)");
        wxMessageBox(message,_T("SD Card overview confirm"));

    }else if(result == 0){
        message =_T("SDCard is empty");
        wxMessageBox(message,_T("SD Card overview confirm"));

    }

    m_mmcLog->ResetBaudrate();

    EndSession();


}


bool MMCLoggerManager::Save(const MMCFilesDesc& mmcFileDesc,  bool needLock )
{
    //m_mmcLog->setSymbolMgr( m_symbolmgr );

    if ( needLock ){

        if( !StartSession() ) return false;

        Update();

        m_mmcLog->SetBaudrate();
    }

    wxString message    = _T("saving block: ");
    wxString title      = _T("Save SD Card log");

    m_mmcLog->registerObserver( m_observer );

    wxArrayString files;

    wxDateTime startTime = wxDateTime::Now();

    int result = m_mmcLog->save(*(m_vemsTuneController->getFirmwareInfo()), files, mmcFileDesc, m_mtConfigData);

    wxTimeSpan elapsedTime = wxDateTime::Now() - startTime;
    wxString elapsedTimeStr = elapsedTime.Format(_T("%H:%M:%S"));

    m_mmcLog->unRegisterObserver( m_observer );

    if(result == -1){
        message =_T("Communication error! (Most probably the firmware doesnt support this feature)");
    }else if(files.IsEmpty() || result == -2){
        message =_T("SDCard was empty no files have been saved!");
    }

    if(result < 0){
        // m_vp->SendString( (uint8_t*)("msd") );
        m_mmcLog->stop();
        wxMessageBox(message,_T("SD Card save confirm"));
    }else{
        message = _T("SDCard log has been saved to: ");
        size_t i;
        for(i =0; i< files.Count(); i++){
            message += files.Item(i) + _T("\n");
        }
        message += _T("Elapsed time: "+ elapsedTimeStr+_T("\n"));
        UserLog::uLog( ULOG_OK, message );
    }

    if ( needLock ){

        m_mmcLog->ResetBaudrate();

        EndSession();
    }
    return true;

}



void MMCLoggerManager::OverViewEnd()
{
    m_mmcLog->ResetBaudrate();
    EndSession();
}

/// process

void MMCLoggerManager::start(const MMCFilesDesc& mmcFileDesc, bool needLock)
{
    m_mmcFileDesc = mmcFileDesc;
    m_needLock = needLock;
    m_run = false;
    setDisabled(false);
    this->ProcessBase::start();
}

bool MMCLoggerManager::process()
{
    if ( m_isDisabled )
        return false;
    if ( m_run )
        return false;

    Save( m_mmcFileDesc, m_needLock );

    m_run = true;
    return false;
}

void MMCLoggerManager::reset()
{
}

void MMCLoggerManager::stop()
{
    setDisabled(true);
    this->ProcessBase::stop();
    m_run = false;
}

void MMCLoggerManager::finished()
{
    int id = m_mmcFileDesc.RowId;
    observerNotify(wxString::Format(_T("%d"), id));
}
