#include "MMCLoggerV3.h"

#include "../CommunicationBase.h"

#include "../logger.h"
#include "../Math.h"
#include "../configmanager/ConfigHelper2.h"
#include "../SymbolMgr.h"

MMCLoggerV3::MMCLoggerV3(CommunicationBase* commBase)
{
    //ctor
    m_commBase =commBase;


    m_isAborted = false;
    m_lastSecCountInBlock = -255;
    m_startNewLogBlockIdx = -1;
    m_needUpdate=true;
    m_blockCount = 60;
}

MMCLoggerV3::~MMCLoggerV3()
{
    //dtor
}

void MMCLoggerV3::start()
{
    m_commBase->SendString((uint8_t*)"mse");
    m_needUpdate=true;
}

void MMCLoggerV3::stop()
{
    m_commBase->SendString((uint8_t*)"msd");
    m_needUpdate=true;
}

void MMCLoggerV3::erase()
{
    m_commBase->SendString((uint8_t*)"msc");
    m_needUpdate=true;
}

bool MMCLoggerV3::getStatus()
{
    int len = 1;
    uint8_t data[8];
    m_commBase->SendString((uint8_t*)"mss");
    /// time is independent from baud rate
	wxMilliSleep(5000);

    /*int result =*/ m_commBase->ReadString(data, len+2);
    if(data[0] == '1') return true;
    return false;

}

bool MMCLoggerV3::needsUpdate()
{

    if (m_needUpdate) return true;
    if (getStatus()) return true;

    ///todo paste new rules to update
    return false;
}

void MMCLoggerV3::flushBuffer()
{
    uint8_t tmp[32];
    m_commBase->ReadString(tmp, 16);
}

bool MMCLoggerV3::getMMCBlock(uint32_t blockIdx, uint8_t* data, int len )
{
    bool res = false;
    int tryCount = 0;
    do{
        res = getMMCBlock_priv(blockIdx, data, len );
        if(!res){tryCount++; }
    }while( !res && (tryCount < 5)) ;
    return res;
}

bool MMCLoggerV3::getMMCBlock_priv(uint32_t blockIdx, uint8_t* data, int len )
{
    m_commBase->ReadString(data, 16);
    if(len == 514){
        m_commBase->SendString((uint8_t*)"msdmsg");
    }else{
        m_commBase->SendString((uint8_t*)"msdmsG");
    }
    blockIdx &= 0xFFFFFF; // 3 bytes block address space
    wxString hexIdx = wxString::Format(_T("%06X"),blockIdx);
    m_commBase->SendString(hexIdx);
    Logger::logf(LOG_MSG,_T("MMCLogger getMMCBlock_priv() blockIdx=")+hexIdx);
//#ifdef SFILE_LIB
//	wxMilliSleep(((len+3))*20000/(m_commBase->getBaudRate()));
//#endif

    int result = 0;
    int tryCount = 0;
    do{
        int tmp = m_commBase->ReadString(&(data[result]), len, 200); /// 10000ms max
        if(tmp == 0){ tryCount++; }
        result += tmp;
    }while( (result < len) && (tryCount < 5)) ;

     Logger::logf(LOG_MSG,_T("VemsProtocol getMMCBlock_priv() len=%i result=%i"),len,result);
    if(result == len) return true;

    return false;

}
/*
bool MMCLoggerV3::isDataValid( uint8_t* data, int len)
{

    int data_or=0;
    int data_and=0xff;
    int i;
    for(i = 0; i < len; i++){
    data_or|=data[i];
    data_and&=data[i];
    }
    if ((data_or==0) || (data_and==0xff)) return false;
    if (data[511]!=m_size) return false;
    else return true;

}
*/
/**
* 16bit CCITT CRC found after 512 bytes of data
* returns true if CRC is valid.
*/
bool MMCLoggerV3::isCRCValid( uint8_t* data, int len)
{
    /*
    if( len < 514){
        return false;
    }
    */

    uint16_t crccalc=getCRC( data, len-2 );
    uint16_t crcrecv=data[len-2] << 8; // BIG endian
    crcrecv += data[len-1];

    if( crccalc == crcrecv){
        return true;
    }

    Logger::logf(LOG_MSG,_T("VemsProtocol isCRCValid() invalid, calc=%04x, recv=%04x"), crccalc, crcrecv);
    return false;
}

uint16_t MMCLoggerV3::getCRC(uint8_t* data, int len)
{
    uint16_t crc = 0;
    int i;
    for(i = 0; i < len; i++){
        crc = Math::crc16(crc, data[i]);
    }
    return crc;
}

void MMCLoggerV3::ResetBaudrate()
{
    uint8_t buffer[16];
    /*
    m_commBase->SendString( (uint8_t*)("mcB67") );
    m_commBase->ReadString(buffer, 4);
    m_commBase->setBaudRate(9600);
    */

    m_commBase->SendString( (uint8_t*)("mcB33") );
#ifdef SFILE_LIB
	wxMilliSleep(((4+5)*20)*10*1000/(m_commBase->getBaudRate()));
#endif
    m_commBase->ReadString(buffer, 4);
    m_commBase->setBaudRate(19200);
}

void MMCLoggerV3::SetBaudrate()
{
    uint8_t buffer[16];

    ConfigBase* config = ConfigHelper2::getConfig( ConfigHelper2::CONFIG_MAIN );
    config->setPath(_T("/sdcard"));

    long baudRate;
    if( !config->read(_T("baudrate"), &baudRate) ){ baudRate = 57600; }
    int command;
    switch(baudRate){
        case 115200:
            command = 8;
            break;
        default:
            command = 1000000/baudRate-1;
    }
    wxString commandStr = wxString::Format(_T("mcB%02X"),command);

    m_commBase->SendString(commandStr);
#ifdef SFILE_LIB
	wxMilliSleep(((4+5)*20)*10*1000/(m_commBase->getBaudRate()));
#endif
    m_commBase->ReadString(buffer, 4);
    m_commBase->setBaudRate(baudRate);
}
