#include "MmcOverviewFrame.h"
#include "../logger.h"

#include "MMCLoggerManager.h"

int idOverViewGrid = wxNewId();
int idSelectStartTimeButton = wxNewId();
int idSelectStopTimeButton = wxNewId();
int idSaveButton = wxNewId();

BEGIN_EVENT_TABLE(MmcOverviewFrame, wxFrame)

EVT_BUTTON  (idSaveButton,MmcOverviewFrame::OnSaveButton )
EVT_BUTTON  (idSelectStartTimeButton,MmcOverviewFrame::OnStartButton )
EVT_BUTTON  (idSelectStopTimeButton,MmcOverviewFrame::OnStopButton )
EVT_GRID_RANGE_SELECT (MmcOverviewFrame::OnGridRangeSelect )
END_EVENT_TABLE()


MmcOverviewFrame::MmcOverviewFrame(wxWindow* parent, MMCLoggerManager * mgr)
:wxFrame(parent, wxID_ANY, _T("SD Card overview"), wxPoint(50,50), wxSize(1000, 650))
{
    SetIcon(wxIcon(_T("vemsLogoIcon2")));
    //ctor
    m_startIdx=0;
    m_stopIdx=0;
    m_selectedTop=0;
    m_selectedBottom=0;

    m_mmcLoggerMgr = mgr;

    wxBoxSizer * mainSizer = new wxBoxSizer(wxHORIZONTAL);

    SetInitialSize(wxSize(640, 450));

    m_overviewGrid = new wxGrid(this,idOverViewGrid, wxPoint(100,100), wxSize(700,450));
    m_overviewGrid->CreateGrid(0,7);
    m_overviewGrid->EnableEditing(false);
    m_overviewGrid->SetSelectionMode(wxGrid::wxGridSelectRows);
    m_overviewGrid->DisableDragColSize();
    m_overviewGrid->DisableDragRowSize();

    m_overviewGrid->SetColSize(-1,20);

    m_selectStartTimeButton = new wxButton(this,idSelectStartTimeButton,_T("Set start timestamp"),wxPoint(700,100),wxSize(150,50));
    m_selectStopTimeButton = new wxButton(this,idSelectStopTimeButton,_T("Set stop timestamp"),wxPoint(700,150),wxSize(150,50));
    m_SaveButton = new wxButton(this,idSaveButton,_T("Save selected log"),wxPoint(700,200),wxSize(150,50));

    m_startText = new wxStaticText(this, wxID_ANY, _T("0000 min"));
    m_stopText = new wxStaticText(this, wxID_ANY, _T("0000 min"));
    m_intervalText = new wxStaticText(this, wxID_ANY, _T("Selected interval: 0000 min"));
    m_messageText = new wxStaticText(this, wxID_ANY, _T("1.: select a row in the table \n2.:click on \"Set start timestamp\"\n3.:select the end row\n4.:click on \"Set stop timestamp\"\n5.:click on \"Save selected log\""));
    m_errorText = new  wxStaticText(this, wxID_ANY, _T(""),wxPoint(750,300));
    for(uint8_t a=0;a<40;a++) {
        wxString key = wxString::Format(_T("%i"),a);

        //m_grid->SetColLabelValue(a,key);
        m_overviewGrid->SetRowLabelValue(a,key);
    }

    m_overviewGrid->SetColLabelValue(0,_T("time[min]"));
    m_overviewGrid->SetColLabelValue(1,_T("tps"));
    m_overviewGrid->SetColLabelValue(2,_T("RPM"));
    m_overviewGrid->SetColLabelValue(3,_T("MAP"));
    m_overviewGrid->SetColLabelValue(4,_T("ALS"));
    m_overviewGrid->SetColLabelValue(5,_T("MAT"));
    m_overviewGrid->SetColLabelValue(6,_T("COOLANT"));
    //m_overviewGrid->AutoSize();
    mainSizer->Add(m_overviewGrid,0,wxALL,5);

    wxBoxSizer * buttonsizer = new wxBoxSizer(wxVERTICAL);

    buttonsizer->Add( m_selectStartTimeButton ,0 , wxALL, 5);
    buttonsizer->Add( m_startText ,0 , wxALL | wxALIGN_CENTER_VERTICAL, 2);
    buttonsizer->Add( m_selectStopTimeButton ,0 , wxALL, 5);
    buttonsizer->Add( m_stopText ,0 , wxALL | wxALIGN_CENTER_VERTICAL, 2);
    buttonsizer->Add( m_SaveButton ,0 , wxALL, 5);
    buttonsizer->Add( m_intervalText ,0 , wxALL, 5);
    buttonsizer->Add( m_messageText ,0 , wxALL, 5);
    buttonsizer->Add( m_errorText ,0 , wxALL, 5);


    mainSizer->Add( buttonsizer );

    SetSizer(mainSizer);
	mainSizer->Fit(this);
	mainSizer->SetSizeHints(this);

	Show(1);
}

MmcOverviewFrame::~MmcOverviewFrame()
{
    //dtor
}

void MmcOverviewFrame::AddGridLine(DoubleHash& Ovhash)
{
    m_overviewGrid->AppendRows();
    int lastRow = m_overviewGrid->GetNumberRows() -1;

    wxString time = wxString::Format(wxT("%.2f"),Ovhash[_T("time")]);
    wxString tps = wxString::Format(wxT("%03.2f"),Ovhash[_T("tps")]);
    wxString rpm = wxString::Format(wxT("%05d"),(int)Ovhash[_T("rpm")]);
    wxString map = wxString::Format(wxT("%05d"),(int)Ovhash[_T("map")]);
    wxString als;
        if((int)Ovhash[_T("als")]){
            als = _T("ON") ;
        } else {
            als = _T("OFF");
        }
    wxString mat = wxString::Format(wxT("%03d"),(int)Ovhash[_T("mat")]);
    wxString coolant = wxString::Format(wxT("%03d"),(int)Ovhash[_T("coolant")]);
    wxString blocknumber =wxString::Format(wxT("%d"),(int)Ovhash[_T("block")]);

    m_overviewGrid->SetCellValue(lastRow,0,time);
    m_overviewGrid->SetCellValue(lastRow,1,tps);
    m_overviewGrid->SetCellValue(lastRow,2,rpm);
    m_overviewGrid->SetCellValue(lastRow,3,map);
    m_overviewGrid->SetCellValue(lastRow,4,als);
    m_overviewGrid->SetCellValue(lastRow,5,mat);
    m_overviewGrid->SetCellValue(lastRow,6,coolant);
    //m_overviewGrid->SetRowLabelValue(lastRow,blocknumber);

    DoubleHash* timehash=new(DoubleHash);
    (*timehash)[_T("time")]=Ovhash[_T("time")];
    (*timehash)[_T("block")]=Ovhash[_T("block")];

    m_timeArray.Add(timehash);
}

void MmcOverviewFrame::OnSaveButton(wxCommandEvent &evt)
{
    if (m_startIdx==0 && m_stopIdx==0){
        if (m_selectedBottom!=0){
            Logger::logf(LOG_MSG,_T("MmcOverviewFrame::OnSaveButton()new interval %d  %d"),m_startIdx,m_stopIdx);
            DoubleHash *tmp;
            tmp=m_timeArray.Item(m_selectedTop);
            m_startIdx=(uint32_t)(*tmp)[_T("block")];
            m_startTime=(*tmp)[_T("time")];
            wxString value=m_overviewGrid->GetCellValue(m_selectedTop,0);
            m_startText->SetLabel(value+_T(" min"));

            tmp=m_timeArray.Item(m_selectedBottom);
            m_stopIdx=(uint32_t)(*tmp)[_T("block")];
            m_stopTime=(*tmp)[_T("time")];
            value=m_overviewGrid->GetCellValue(m_selectedBottom,0);
            m_stopText->SetLabel(value+_T(" min"));

            value=wxString::Format(wxT("Selected interval: %.2f min"),(m_stopTime-m_startTime));
            m_intervalText->SetLabel(value);

        }
    }
    int interval=m_stopIdx-m_startIdx;
    if (interval<0) {
        m_errorText->SetLabel(_T("BAD INTERVAL"));
    Logger::logf(LOG_MSG,_T("MmcOverviewFrame::OnSaveButton()bad %d  %d"),m_startIdx,interval);
    }else if(interval==0){
        m_errorText->SetLabel(_T("NO INTERVAL SELECTED"));
        Logger::logf(LOG_MSG,_T("MmcOverviewFrame::OnSaveButton()no interval"));
    }else{
    Logger::logf(LOG_MSG,_T("MmcOverviewFrame::OnSaveButton()save %d  %d"),m_startIdx,interval);
    m_errorText->SetLabel(_T(""));
    MMCFilesDesc desc;
    desc.StartAddress = m_startIdx;
    desc.EndAddress = m_stopIdx;
    m_mmcLoggerMgr->Save(desc);
    }
}
void MmcOverviewFrame::OnStartButton( wxCommandEvent &evt )
{
    Logger::logf(LOG_MSG,_T("MmcOverviewFrame::OnStartButton()_clicked c=%i"),m_selectedTop);
    wxString value=m_overviewGrid->GetCellValue(m_selectedTop,0);
    //Logger::logf(LOG_MSG,_T("MmcOverviewFrame::OnStartButton()_clicked ")+value);
    m_startText->SetLabel(value+_T(" min"));
    DoubleHash *tmp;
    tmp=m_timeArray.Item(m_selectedTop);
    m_startTime=(*tmp)[_T("time")];
    m_startIdx=(uint32_t)(*tmp)[_T("block")];
    Logger::logf(LOG_MSG,_T("MmcOverviewFrame::OnStartButton()_time= %f"),m_startTime);
    value=wxString::Format(wxT("Selected interval: %.2f min"),(m_stopTime-m_startTime));
    m_intervalText->SetLabel(value);

}
void MmcOverviewFrame::OnStopButton( wxCommandEvent &evt )
{
    Logger::logf(LOG_MSG,_T("MmcOverviewFrame::OnStopButton()_clicked c=%i"),m_selectedBottom);
   wxString value=m_overviewGrid->GetCellValue(m_selectedBottom,0);
    //Logger::logf(LOG_MSG,_T("MmcOverviewFrame::OnStopButton()_clicked ")+value);
    m_stopText->SetLabel(value+_T(" min"));

    DoubleHash *tmp;
    tmp=m_timeArray.Item(m_selectedBottom);
    m_stopTime=(*tmp)[_T("time")];
    m_stopIdx=(uint32_t)(*tmp)[_T("block")];
    Logger::logf(LOG_MSG,_T("MmcOverviewFrame::OnStopButton()_time= %f"),m_stopTime);
    value=wxString::Format(wxT("Selected interval: %.2f min"),(m_stopTime-m_startTime));
    m_intervalText->SetLabel(value);

}

void MmcOverviewFrame::OnGridRangeSelect(wxGridRangeSelectEvent &evt)
{
    Logger::logf(LOG_MSG,_T("MmcOverviewFrame::OnGridRangeSelect()_event"));

    if (!evt.Selecting()) return;

    wxArrayInt selected=m_overviewGrid->GetSelectedRows();

    if (selected.GetCount()!=0){
        if(selected.Item(0)<selected.Last()){
            m_selectedTop=selected.Item(0);
            m_selectedBottom=selected.Last();
        }else{
            m_selectedTop=selected.Last();
            m_selectedBottom=selected.Item(0);
        }
    }else{
        m_selectedTop=evt.GetTopRow();
        m_selectedBottom=evt.GetBottomRow();
    }

    Logger::logf(LOG_MSG,_T("MmcOverviewFrame::OnGridRangeSelect()_top:,%d"),m_selectedTop);
    Logger::logf(LOG_MSG,_T("MmcOverviewFrame::OnGridRangeSelect()_bottom:,%d"),m_selectedBottom);



}
