#ifndef SDCARDSAVEDIALOG_H
#define SDCARDSAVEDIALOG_H

#include <wx/wx.h>
#include "SDProcess.h"

class SDCardSaveDialog : public wxDialog
{
    public:
        SDCardSaveDialog( wxWindow* parent );
        virtual ~SDCardSaveDialog();

        wxString    GetImageName() { return m_ofname; }

    protected:
        friend class SDProcess;

        void        Notify( const wxString& msg );
        void        OnProcessTerminated( SDProcess *process );

    private:

        // location of dd
        wxString            m_ddprog;

        wxString            m_ofname;

        ProcessesArray      m_running;
        wxTimer             m_timerIdleWakeUp;
        bool                m_killed;

        wxArrayString       m_outputA;
        wxArrayString       m_errorA;

        wxArrayString       m_devicesName;

        wxComboBox*         m_devicesCombo;

        wxTextCtrl*         m_messages;
        wxStaticText*       m_nrread;

        wxButton*           m_detectButton;
        wxButton*           m_saveButton;
        wxButton*           m_refreshButton;
        wxButton*           m_closeButton;
        wxButton*           m_cancelButton;

        wxGauge*            m_progressbar;

        void        buildGUI( wxWindow* parent );
        void        enableButtons( bool enable = true );

        void        runSync( const wxString& cmd );
        void        runAsync( const wxString& cmd, bool closeParent = false );

        void        AddAsyncProcess( SDProcess *process );
        void        RemoveAsyncProcess( SDProcess *process );

        bool        DetectVemsSD( const wxString device );

        void        refreshDevices();

        /// event handlers
        void        OnSaveImg( wxCommandEvent& event );
        void        OnDetectSD( wxCommandEvent& event );
        void        OnRefreshDevices( wxCommandEvent& event );
        void        OnCancelProgress( wxCommandEvent& event );
        void        OnCloseDlg( wxCommandEvent& event );

        void        OnClose( wxCloseEvent& event );
        void        OnIdle( wxIdleEvent& event );
        void        OnTimer( wxTimerEvent& event );

        DECLARE_EVENT_TABLE()

};

#endif // SDCARDSAVEDIALOG_H
