#include "serialporthandler.h"

#include <wx/string.h>

#include "serialconfig.h"
#include "UnicodeHelper.h"
#include "SerialException.h"
#include "logger.h"
#include "Math.h"

SerialPortHandler::SerialPortHandler(SerialConfig* serialConfig)
{
   //  Logger::log(LOG_MSG, _T("SerialPortHandler CTOR 0") );
#ifdef SFILE_LIB
    dev = new SFile();
#else
    #ifdef CTB15
        dev = new ctb::SerialPort();
    #else
        dev = new wxSerialPort();
    #endif
#endif

    //m_baudrate = wxBAUD_19200;
    //m_serialConfig = NULL;

    m_serialConfig = serialConfig;
    setBaudRate(m_serialConfig->getBaudRate() );
}


SerialPortHandler::~SerialPortHandler()
{
     Logger::log(LOG_MSG, _T("SerialPortHandler DTOR") );
    //dtor
    dev->Close();
    delete dev;
}

/*
void SerialPortHandler::setSerialConfig(SerialConfig * serialConfig){
    m_serialConfig = serialConfig;
    setBaudRate(m_serialConfig->getBaudRate() ); // INItial baudRate
}
*/

bool SerialPortHandler::IsOpen(){
    return dev->IsOpen();
}

int SerialPortHandler::open()
{
//fprintf(stdout, "SerialPortHandler this: %p\n",this);

    //fprintf(stdout,  "SerialPortHandler:open() START: dev: %p\n", dev);
    //Logger::logf(LOG_WARNING, _T("SerialPortHandler::open() init!!") );
    if(dev == NULL){
#ifdef SFILE_LIB
         dev = new SFile(); Logger::logf(LOG_MSG, _T("SerialPortHandler::open() new instance!!") );
#else

    #ifdef CTB15
        dev = new ctb::SerialPort();
    #else
        dev = new wxSerialPort();
    #endif
     Logger::logf(LOG_MSG, _T("SerialPortHandler::open() new instance!!") );
#endif
    }
       // Logger::log(LOG_MSG, _T("SerialPortHandler:open()dev NOT NULL!!") );
    if( dev->IsOpen() ){ return 0; }
    //Logger::logf(LOG_MSG, _T("SerialPortHandler::open() init!!") );

    //initSerialPort();

	// try to open the given port appr 500 times (you guess right, windows sucks!)
    int trycnt=4;

#ifdef SFILE_LIB
	long port=0;
	wxString ports = m_serialConfig->getPortName().AfterLast('M');
	ports.ToLong(&port);
    while( trycnt > 0 && dev->Open( port,19200 ) < 0 ) {
#else
    while( trycnt > 0 && dev->Open( m_serialConfig->getPortNameC() ) < 0 ) {
#endif
        //Logger::logf(LOG_MSG, _T("SerialPortHandler::open() Cannot open: ") + m_serialConfig->getPortName() );
        //Logger::logf(LOG_MSG, _T("SerialPortHandler::open() Cannot openC: %s"), m_serialConfig->getPortNameC() );
        //wxLogError(_T("Cannot open: %s\n"),devname );
        //printf(" SerialPortHandler::open() %s", m_serialConfig->getPortNameC() );
        trycnt--;
        // sleep(1); or wxsleep() ?
    }
    if( trycnt == 0 ){
//        Logger::logf(LOG_MSG,  _T("Giving up opening ")+ m_serialConfig->getPortName() );
        // Wrong: Logger::log(_T("Giving up opening 25 %s\n"),UnicodeHelper::U2C( portName.c_str() ) );
        //delete dev;
        //dev = NULL;
        //if(dev == NULL){
        //    fprintf(stdout, "dev is NULL\n");
        //}
        //serialConfig.setStatus("Unable to open!");
        throw SerialException(SP_CANNOT_OPEN);
        //return SP_CANNOT_OPEN;

    }

#ifdef SFILE_LIB
    dev->SetBaudRate(m_baudrate);
#else
    dev->SetBaudrate(m_baudrate);
#endif

    // Success! port opened
    // set the baudrate
    //((wxSerialPort*)dev)->SetBaudRate(baudrate);
    //return 0;
    // Logger::logf(LOG_MSG, _T("SerialPortHandler::SerialPortHandler() open END") );
   //  isInit = 1;
   //serialConfig.setStatus("Opened!");
   return 0;
}

void SerialPortHandler::close()
{
    //Logger::logf(LOG_MSG, _T("SerialPortHandler::close() ") );
    if( dev == NULL) { return; }
    //Logger::logf(LOG_MSG, _T("SerialPortHandler::close() dev not null") );
    if( !dev->IsOpen() ){ return; }
    //Logger::logf(LOG_MSG, _T("SerialPortHandler::close() dev->Close()") );
    dev->Close();
}

int SerialPortHandler::SendString(const uint8_t * str)
{
     //fprintf(stdout, "LINE: %s\n", str );
     return SendString(str,  strlen((char*)str));
}

int SerialPortHandler::SendString(const uint8_t* str, size_t len)
{
    open();
#ifdef SFILE_LIB
    DWORD result;
	dev->Write( (char *) str, len,result);
#else
    int result = dev->Write( (char *) str, len);
#endif
    if((int)len != result){
           Logger::log(LOG_ERROR, _T("SerialPortHandler::SendString() WRITE FAILED!") );
            throw SerialException(SP_WRITE_ERROR);
            //return SP_WRITE_ERROR;
     }
     return result;
}

int SerialPortHandler::SendString(const wxString& str)
{
    const char * c = UnicodeHelper::U2C(str);
    int res = SendString( (uint8_t*)c );
    delete (char *)c;
    return res;
     /*
     int timeout = 10;
     int to = 0;
     size_t len = str->Length();

     timer t(timeout, &to, NULL);
     t.start();

     return dev->Writev((char *) str->c_str(), len, &to);
    */
}
/*
int SerialPortHandler::SendString(wxChar * chr)
{
   wxString * str = new wxString(chr);
   return SendString(str);
}
*/


int SerialPortHandler::Read(uint8_t* str, size_t len)
{
        open();
#ifdef SFILE_LIB
        DWORD rd=0;
		DWORD bytes = dev->BytesInComm();
		if(bytes)
		{
			if(bytes<len)
				dev->Read((char*)str, bytes, rd);
			else
				dev->Read((char*)str, len, rd);
		}
		else
			rd=0; //just for easy debug
#else
        int rd = dev->Read((char*)str, len);
#endif
        return rd;
}

/**
return <0: FAILED: error code
       >0: OK:  number of read byte
*/
int SerialPortHandler::ReadString(uint8_t* str, size_t len)
{
  //      Logger::logf(LOG_MSG, _T("SerialPortHandler::ReadString()"));
        open();
  //      Logger::logf(LOG_MSG, _T("SerialPortHandler::ReadString() sp open"));
       // if( open() == SP_CANNOT_OPEN){ return SP_CANNOT_OPEN;}
        //if(dev == NULL){return SP_UNKNOWN;}
        //int timeout = 3 * len; //?????
        int timeout = getTimeOut(len);
        int to = 0;
        //size_t len = strlen(str);
         wxString msg;

    #ifdef CTB15
        ctb::Timer t(timeout, &to, NULL);
    #else
        timer t(timeout, &to, NULL);
    #endif

        t.start();
#ifdef SFILE_LIB
		int rd=0;
		DWORD bytes = dev->BytesInComm();
		if(bytes)
		{
			if(bytes<len)
				rd = dev->Readv ((char*)str, bytes, &to, true);
			else
				rd = dev->Readv ((char*)str, len, &to, true);
		}
		else
			rd=0; //just for easy debug
#else
        int rd = dev->Readv ((char*)str, len, &to, true);
#endif
        /*
        if(rd > 0) {

            //msg.Printf(_T("SerialPortHandler::ReadString(): (%d) Received size: (%d) string: (%s)"),len,rd,str);
            // Logger::log(LOG_MSG, msg);
        }else if(rd == 0){
            //msg.Printf(_T("SerialPortHandler::ReadString() Zero byte received expected: (%d)"), len);
            //Logger::log(LOG_MSG, msg);
        }
*/
   //     close();
        return rd;
/*
        char* receivedBytes = NULL;
        int rd = dev->ReadUntilEOS(&receivedBytes,"\n",50);
        if(rd > 0) {
            receivedBytes[rd] = 0;
            printf("answer (%i): %s\n",rd,receivedBytes);

      //      const wxChar * receivedChar = new wxChar(*receivedBytes);
            TextCtrlReceived->AppendText( receivedBytes );
        }else if(rd == 0){
            printf("Zero byte received\n");

        }
*/

}

int SerialPortHandler::ReadString(uint8_t* data, size_t len, int maxTryCount)
{
    int tryCount =0;
    size_t result = 0;
    do{
        size_t expLen = len - result;
        int tmp = ReadString(&data[result], expLen);
        if(tmp == 0 ){
            tryCount++;
            wxThread::Sleep(1);
        }
        result += tmp;
    }while( (result < len) && (tryCount < maxTryCount));
    return (int)result;
}

/*
int SerialPortHandler::ReadUntilEOS(char ** readbuf, char * eosString , long timeout_in_ms)
{
    open();
//	size_t bytes;
    //if( open() == SP_CANNOT_OPEN){ return SP_CANNOT_OPEN;}
    //if(dev == NULL){return SP_UNKNOWN;}
   // char* receivedBytes = NULL;
//0.13    int rd = dev->ReadUntilEOS(*readbuf,&bytes, eosString, timeout_in_ms);
    int rd = dev->ReadUntilEOS(readbuf, eosString, timeout_in_ms);
   // if(rd > 0) {
//       readbuf[rd] = 0;
//    }
    return rd;
}
*/

int SerialPortHandler::getBaudRate()
{
    return m_baudrate;
}

void SerialPortHandler::setBaudRate(int bRate)
{
    Logger::logf( LOG_MSG, _T("SerialPortHandler::setBaudRate: %i"), bRate );
    //open();
    //if( open() == SP_CANNOT_OPEN) return ;
    wxString msg;
#ifdef SFILE_LIB
	m_baudrate= bRate;
#else
    #ifdef CTB15
        m_baudrate = bRate;
    #else
    switch(bRate)
    {
        case (4800):
                m_baudrate = wxBAUD_4800;
                break;
        case (9600):
                m_baudrate = wxBAUD_9600;
                break;
        case (19200):
                m_baudrate = wxBAUD_19200;
                break;
         case (38400):
                m_baudrate = wxBAUD_38400;
                break;
         case (57600):
                m_baudrate = wxBAUD_57600;
                break;
         case (115200):
                m_baudrate = wxBAUD_115200;
                break;
        default:
                m_baudrate = wxBAUD_19200;

    }
    #endif
#endif

    m_magicTimeoutConstA = 12000.0/m_baudrate;
    m_magicTimeoutConstB = 12.0 * 10000.0/m_baudrate;
    //open();
#ifdef SFILE_LIB
    if( IsOpen() ) dev->SetBaudRate(m_baudrate);
#else
    if( IsOpen() ) dev->SetBaudrate(m_baudrate);
#endif


}


int SerialPortHandler::clearSerialBuffer(int len)
{
        uint8_t * tmp = new uint8_t[len+32];
        int res = ReadString(tmp, len);
        delete[] tmp;
        return res;
}

int SerialPortHandler::clearSerialBufferCircle(int len, int count)
{
    int res;
    int c = 0;
    while(count > 0){
        res = clearSerialBuffer(len);
        if(res < len)return (c * len + res);
        count--;
        c++;
    }
   return -1;
   // throw SerialException(SP_CANNOT_CLEAR_BUFF);
   // return SP_CANNOT_CLEAR_BUFF;
}

/*virtual*/
int SerialPortHandler::clear()
{
#ifdef SFILE_LIB
	return dev->PurgeComm();
#else
    return clearSerialBufferCircle(8, 64);
#endif
}

int SerialPortHandler::sendStringBlock(uint8_t* value, size_t len, size_t blockLen)
{
    //char * p = (char *) value;
    int result = 0;
    size_t i = 0;
    size_t currBlock;
    for(i=0; i< len; i += blockLen){
        currBlock = len - i;
        if( currBlock > blockLen ){
            currBlock = blockLen;
        }
        result += SendString( (&(value[i])), currBlock);
    }
    return result;
}

int SerialPortHandler::getTimeOut(size_t len)
{
    return (int)(len * m_magicTimeoutConstA + m_magicTimeoutConstB);

    //return ((len*10)>>3) + 12; //9600 10000/baudRate

    //return ((len*10000)/m_baudrate) + 12;
}

int SerialPortHandler::sendNumber(uint32_t number, int length)
{
    uint8_t str[4];
    Math::getBytes(number, str, length);
    return SendString(str, length);
}

int SerialPortHandler::sendNumber(uint16_t number)
{
    uint8_t str[2];

    str[0] = (number >> 8) &0xFF;
    str[1] = number & 0xFF;

    return SendString( str, 2);
}

