#include "FullDuplexTest.h"

#include <portaudiocpp/DirectionSpecificStreamParameters.hxx>
#include <portaudiocpp/StreamParameters.hxx>
#include <portaudiocpp/BlockingStream.hxx>
#include <portaudiocpp/Device.hxx>
#include <portaudiocpp/Exception.hxx>

#include "portaudiocpp/PortAudioCpp.hxx"

#include <wx/msgdlg.h>
#include "../logger.h"
#include "../ringbuffer/RingBuffer.h"
#include "../ringbuffer/SyncObj.h"

// treshold detect: /svn/firmware/firmware/branches/stable1_1/gyak3/munkahely/hiszterezis
// hisztogram : /svn/firmware/firmware/branches/stable1_1/gyak3/munkahely/hisztogram

using namespace portaudio;

// pass ringbuffer pointer for the storage of sample-blocks
FullDuplexTest::FullDuplexTest(RingBuffer *rbp, int audioDevice, SyncObj* syncObj)
{
    //ctor
    //m_frame = frame;
    m_ringBuffer = rbp;
    m_audioDevice = audioDevice;
    m_syncObj = syncObj;
}

FullDuplexTest::~FullDuplexTest()
{
    //dtor

}

void FullDuplexTest::initBuffer()
{ /*
    // *2 missing (2 bytes per channel) ?
    int numBytes = FRAMES_PER_BUFFER * NUM_CHANNELS;// sizeof(m_sampleBlock);
    m_sampleBlock = new  short[numBytes];
    int i;

    for( i=0; i<FRAMES_PER_BUFFER*NUM_CHANNELS; i++ ){
        m_sampleBlock[i] = SAMPLE_SILENCE;
    }
    */
}

void FullDuplexTest::init()
{
     System& paSystem = System::instance();
    try{
        paSystem.initialize();
     }catch(Exception& e){
//        printf( "Exception!");
        const char* str = e.what();
        serialErrorMsgDialog((const char *) str);
        return;
    }
}

void FullDuplexTest::ShowDevices()
{

   int i;
   try{
        portaudio::AutoSystem autoSys;
         System& paSystem = System::instance();
        int deviceCount = paSystem.deviceCount();
        for(i=0; i< deviceCount; i++){
        Device& ad = paSystem.deviceByIndex(i);
        const char * name = ad.name();
        wxString nameStr = wxString::FromAscii(name);

         wxString msg = wxString::Format(_T("Audio device: %i, name: ") + nameStr, i);
         Logger::log(LOG_MSG, msg);
    }

    }catch(Exception& e){
//        printf( "Exception!");
        const char* str = e.what();
        serialErrorMsgDialog((const char *) str);
        return;
    }
}

/**
 * Blocks for RECORD_SECONDS seconds, so call from separate thread (AudioThread) not from main.
 */
void FullDuplexTest::fullDuplexTest()
{

    Logger::logf(LOG_MSG, _T("FullDuplexTest::fullDuplexTest()") );
    portaudio::AutoSystem autoSys;
    System& paSystem = System::instance();

    int i;
    //serialErrorMsgDialog("!!!!!!!!!!!!!!");
    DirectionSpecificStreamParameters inputParameters = DirectionSpecificStreamParameters ();
#if defined(__WXMSW__)
    inputParameters.setDevice(paSystem.defaultInputDevice() );
#else
    inputParameters.setDevice(paSystem.deviceByIndex(m_audioDevice) );
#endif
    /*
    printf( "Input device # %d.\n", inputParameters.device );
    printf( "Input LL: %g s\n", Pa_GetDeviceInfo( inputParameters.device )->defaultLowInputLatency );
    printf( "Input HL: %g s\n", Pa_GetDeviceInfo( inputParameters.device )->defaultHighInputLatency );
    */
    inputParameters.setNumChannels(NUM_CHANNELS);
    inputParameters.setSampleFormat(PA_SAMPLE_TYPE);
    inputParameters.setSuggestedLatency(inputParameters.device().defaultHighInputLatency () );
    //Pa_GetDeviceInfo( inputParameters.device )->defaultHighInputLatency ;
    inputParameters.setHostApiSpecificStreamInfo(NULL);


    DirectionSpecificStreamParameters outputParameters = DirectionSpecificStreamParameters ();
#if defined(__WXMSW__)
    outputParameters.setDevice(paSystem.defaultOutputDevice() );
#else
    outputParameters.setDevice(paSystem.deviceByIndex(m_audioDevice) );
#endif
/*
    printf( "Output device # %d.\n", outputParameters.device );
    printf( "Output LL: %g s\n", Pa_GetDeviceInfo( outputParameters.device )->defaultLowOutputLatency );
    printf( "Output HL: %g s\n", Pa_GetDeviceInfo( outputParameters.device )->defaultHighOutputLatency );
 */
    outputParameters.setNumChannels(NUM_CHANNELS);
    outputParameters.setSampleFormat(PA_SAMPLE_TYPE);
    outputParameters.setSuggestedLatency(outputParameters.device().defaultHighInputLatency ());
    // = Pa_GetDeviceInfo( outputParameters.device )->defaultHighOutputLatency;
    outputParameters.setHostApiSpecificStreamInfo(NULL);

    StreamParameters paStreamParam = StreamParameters();
    paStreamParam.setInputParameters(inputParameters);
    paStreamParam.setOutputParameters(outputParameters);
    paStreamParam.setSampleRate(SAMPLE_RATE);
    paStreamParam.setFramesPerBuffer(FRAMES_PER_BUFFER);

    paStreamParam.clearFlags ();

    m_sineTest = new SineTest(FRAMES_PER_BUFFER, SAMPLE_RATE);
    m_sineTest->setFrequency(1000);


    BlockingStream * bl = new BlockingStream ();
    try{
        bl->open(paStreamParam); /* exception? */
     }catch(Exception& e){
//        printf( "Exception!");
        const char* str = e.what();
        serialErrorMsgDialog((const char *) str);
        return;
    }
    bl->start();
    Logger::logf(LOG_MSG, _T("FullDuplexTest::fullDuplexTest() stream STARTED!!") );
//     int i;
    // prefill buffer:
    //short *bufO = (short *) m_sineTest->generate();
    //bl->write(bufO, FRAMES_PER_BUFFER );
    for( i=0; i<(RECORD_SECONDS*SAMPLE_RATE)/FRAMES_PER_BUFFER; ++i ){
    // write to sound card
        //short * bufOut = (short*) m_outRingBuffer->getHeadPointer();

        //short *bufOut = (short *) m_sineTest->generate();

        ////////// EMULATE SOUND IO () /////////
        /*
         if( m_ringBuffer->isFull() ){
             wxThread::Sleep(1);
         }else{
            short * bufIn = (short*) m_ringBuffer->getHeadPointer();
            memcpy(bufIn, bufOut, 4 *FRAMES_PER_BUFFER);
            m_ringBuffer->nextHead();
            uint32_t milli = FRAMES_PER_BUFFER*1000/SAMPLE_RATE;
            wxThread::Sleep(milli);
        }
        */
        /////////////////////////////////////


        //bl->write(bufOut, FRAMES_PER_BUFFER );
        //if( err ) goto xrun;
    /////////////////

        if( m_ringBuffer->isFull() ){
            Logger::logf(LOG_MSG, _T("FullDuplexTest::fullDuplexTest() RingBuffer is full. This is serious. Is SignalProcess consumer dead ?"));
            wxThread::Sleep(1000);
        } else {
            short * buf = (short*) m_ringBuffer->getHeadPointer();
            bl->read(buf, FRAMES_PER_BUFFER );
            m_ringBuffer->nextHead(); // not empty any longer
            if(m_syncObj != NULL){
                m_syncObj->m_isNotEmpty->Broadcast();
                //Logger::logf(LOG_MSG, _T("FullDuplexTest::fullDuplexTest() after broadcast!"));
            }else{
                Logger::logf(LOG_MSG, _T("FullDuplexTest::fullDuplexTest() syncObj is NULL!!"));
            }
        }
       //if( err ) goto xrun;


/*
       if( i % (SAMPLE_RATE/FRAMES_PER_BUFFER) == 0 ){
          getAvarage(m_sampleBlock, FRAMES_PER_BUFFER);
           Logger::logf(LOG_MSG, _T("Average: Left: %d, Right: %d"),m_avgLeft, m_avgRight);
       }
*/

    }
    bl->stop();
    bl->close();

    paSystem.terminate();

}

void FullDuplexTest::serialErrorMsgDialog(const char * msg)
{
     wxString msgStr = wxString::FromAscii(msg);

     wxMessageDialog *dialog = new wxMessageDialog(NULL, msgStr, _T("Error"), wxOK);

    dialog->ShowModal();
    //delete dialog;
}

void FullDuplexTest::getAvarage( short * m_sampleBlock,int length)
{
    m_avgLeft = 0;
    m_avgRight = 0;
    for(int i=0; i < length; i++){
        m_avgLeft += m_sampleBlock[2*i];
        m_avgRight += m_sampleBlock[2*i+1];
    }
    m_avgLeft /= length;
    m_avgRight /= length;
}

