#ifndef TRIGGERDETECT_H
#define TRIGGERDETECT_H

/** these TOWARD_... values are just defaults, can be redefined
*/
// divide by 256 after multiplying with these:
// higher is faster convergence
#define TOWARD_PEAK 30

// amplitude threshold, name comes from 1/3 (256/3 = 85, that is 33% of signal)
// Theoretical maximum (meaningful) value is 254
// LM1815 is appr 80% of peak, that is 80% * 256 = 202
#define THIRD 85

// this is a slow convergence, divide by 65536 after multiplying with this:
#define TOWARD_MID 50//100

// also slow convergence, for average calculation:
// very low value like 3 screws the whole averaging
#define TOWARD_AVERAGE 40
#define MIDPOINT 32767

#define THRESHOLD_STRATEGY_BETWEEN_PEAKS (0)
// around average (mean) value
#define THRESHOLD_STRATEGY_AROUND_MEAN (1)

// /svn/firmware/firmware/branches/stable1_1/gyak3/munkahely/hiszterezis
#include <wx/string.h>
#include "../common/typedef.h"

#define arraysize  30
//#define perarraysize = 1/( (arraysize-2) / 1.0)
class TriggerDetect
{
    public:
        TriggerDetect();
        virtual ~TriggerDetect();
        void setName(const wxString &name); // set descriptive name for object
        void setThresholdMinSamples( uint8_t sampleCount );
        uint16_t average(uint16_t sample);
        void peakDetect(uint16_t sample);
        void setThresholds( void );
        void setThresholdsAround( uint16_t val );
        uint8_t thresholdDetect(uint16_t sample);
        // most useful interface:
        uint8_t feedSample(uint16_t sample);
        void printData(); // print some data for debugging
        uint8_t getState(){
                return m_state;
        }
        void setThresh256( uint8_t thresh ) { m_thresh256 = thresh; }
        void setThresholdStrategy( uint8_t strategy ){ m_threshold_strategy = strategy; }

        uint16_t getAverage(){ return m_average; }
        uint16_t getPeakP() { return peak_p; }
        uint16_t getPeakN(){ return peak_n; }
        uint16_t getLastSample(){ return m_sample; }
        void setTowardAverage(uint16_t ave){ m_toward_average = ave; };
        //void setTowardMid(uint16_t mid){ m_toward_mid = mid; };
        void setTowardPeak(uint16_t peak){ m_toward_peak = peak; };
        void saveLastSamples(uint16_t currsample);
        void resetLocalPeak(uint16_t sample);
        uint16_t diffPeak();
        uint32_t outlierElimination(void);
        void loadmprevsample(void);
        uint16_t m_prevsample[arraysize]; // sample's before current sample
        uint16_t getUpTreshold(){ return up_threshold;}
        uint16_t getLoTreshold(){ return lo_threshold;}
        uint16_t getHighCounter(){ return m_highCounter;}
        uint16_t getLowCounter(){ return m_lowCounter;}
        void setTowardMid(uint16_t rpm);

    protected:
    private:
    wxString m_name;    // descriptive name for printData()
    uint16_t up_threshold ;    // upper trheshold
    uint16_t lo_threshold ;    // lower_threshold
    uint16_t peak_p ;          // positive peak value
    uint16_t peak_n ;          // negative peak value
    //uint16_t local_peak_p  ;          // positive local peak value
    // uint16_t local_peak_n ;          // negative local peak value
    uint8_t m_state;    // bit0: state
    uint32_t m_highCounter;  // upcounter ( when == m_overthresholdsamplecnt => transition new state)
    uint32_t m_lowCounter;
    uint8_t m_overthresholdsamplecnt; // 0..127 minimum number of samples the signal must be over threshold (to be detected)
    uint8_t m_sample_received; // initially 0, 1 after first sample received
    uint32_t m_transitionCount; // number of signal transitions (low=>high + high=>low)
    uint16_t m_minThreshold;    // TODO: add a setter function
    uint16_t m_average;     // lowpass filtered signal
    uint16_t m_sample;  // example sample
    uint8_t m_threshold_strategy;  // if 0 calculates thresholds from peaks. Otherwise around average
    uint16_t m_toward_average;
    uint16_t m_toward_mid;
    uint16_t m_toward_peak;
    uint8_t m_thresh256;    // threshold (relative to measured amplitude) 0..127
    double m_onePerArraySize;


};


#endif // TRIGGERDETECT_H
