#include "main.h"

#include "timingtester.h"
#include "FullDuplexTest.h"
#include "soundFrame.h"
#include "SignalProcessThread.h"
#include "AudioThread.h"


#include "../vemsgauge/vemsgaugedesc.h"
#include "../vemsgauge/VemsGridWidget.h"

#include "../vemsgauge/VemsGaugeAgg.h"

#include "../rtdatamanager/RTDataViewerBase.h"
#include "../rtdatamanager/RTDataInputSignalgen.h"
#include "../rtdatamanager/RTDataMgrFacade.h"

#include "../ringbuffer/RingBuffer.h"
#include "../ringbuffer/GUIConsumerThread.h"
#include "../ringbuffer/SyncObj.h"

#include "signalgauges/WriteInFileByTime.h"
#include "signalgauges/SparkTesterWidgetFactory.h"

#include "../logger.h"
//#include "../ThemesHelper.h"

//helper functions

enum wxbuildinfoformatsparktester {
    short_f, long_f };

wxString wxbuildinfo(wxbuildinfoformatsparktester format)
{
    wxString wxbuild(wxVERSION_STRING);

    if (format == long_f )
    {
#if defined(__WXMSW__)
        wxbuild << _T("-Windows");
#elif defined(__UNIX__)
        wxbuild << _T("-Linux");
#endif

#if wxUSE_UNICODE
        wxbuild << _T("-unicode build");
#else
        wxbuild << _T("-ANSI build");
#endif // wxUSE_UNICODE
    }

    return wxbuild;
}

int ID_BOOK = wxNewId();

int idFullDuplex0 = wxNewId();
int idFullDuplex1 = wxNewId();
int idAudioDevicesID = wxNewId();
int idMenuSparkQuit = wxNewId();
int idMenuSparkAbout = wxNewId();
int SPARKTIMER_ID = wxNewId();
int idMenuAddGauge = wxNewId();
const int idAddGrid = wxNewId();
const int idAdd2DHist = wxNewId();
const int idAddShowspark = wxNewId();
const int idAddGridByTime = wxNewId();
const int idAddVemsGauge= wxNewId();
const int idWriteInFile= wxNewId();


BEGIN_EVENT_TABLE(SparkMyFrame, wxFrame)
    EVT_MENU(idFullDuplex0, SparkMyFrame::OnFullDuplex)
    EVT_MENU(idFullDuplex1, SparkMyFrame::OnFullDuplex1)
    EVT_MENU(idAudioDevicesID, SparkMyFrame::OnShowDevices)
    EVT_MENU(idMenuSparkQuit, SparkMyFrame::OnQuit)
    EVT_MENU(idMenuSparkAbout, SparkMyFrame::OnAbout)
    EVT_MENU(idMenuAddGauge, SparkMyFrame::OnAddGauge)
    EVT_MENU(idAddGrid, SparkMyFrame::OnGaugeTest)
    EVT_MENU(idAdd2DHist, SparkMyFrame::OnGaugeTest)
    EVT_MENU(idAddShowspark, SparkMyFrame::OnGaugeTest)
    EVT_MENU(idAddGridByTime, SparkMyFrame::OnGaugeTest)
    EVT_MENU(idAddVemsGauge, SparkMyFrame::OnGaugeTest)
    EVT_MENU(idWriteInFile, SparkMyFrame::onWriteInFile)

    EVT_TIMER(SPARKTIMER_ID, SparkMyFrame::OnTimer)
END_EVENT_TABLE()

SparkMyFrame::SparkMyFrame(wxFrame *frame, const wxString& title)
    : wxFrame(frame, -1, title)
    , m_timer(this, SPARKTIMER_ID)
{

    Logger::initLog(wxEmptyString);
    Logger::log(LOG_MSG, _T("SPARK tester START!"));
    fprintf(stderr, "-----------------------------");

#if wxUSE_MENUS
    // create a menu bar
    wxMenuBar* mbar = new wxMenuBar();
    wxMenu* fileMenu = new wxMenu(_T(""));
    fileMenu->Append(idAudioDevicesID, _("&Show audio devices"), _("Show audio devices"));
    fileMenu->Append(idFullDuplex0, _("&Test Fullduplex 0"), _("Test Fullduplex 0"));
    fileMenu->Append(idFullDuplex1, _("&Test Fullduplex 1"), _("Test Fullduplex 1"));
    fileMenu->Append(idMenuAddGauge, _("&Test Gauge"), _("Test Gauge"));

    fileMenu->Append(idWriteInFile, _("Write In File"), _("Write In File"));
    fileMenu->Append(idMenuSparkQuit, _("&Quit\tAlt-F4"), _("Quit the application"));
    mbar->Append(fileMenu, _("&File"));

    wxMenu* helpMenu = new wxMenu(_T(""));
    helpMenu->Append(idMenuSparkAbout, _("&About\tF1"), _("Show info about this application"));
    mbar->Append(helpMenu, _("&Help"));

    wxMenu* gaugeMenu = new wxMenu(_T(""));
    gaugeMenu->Append(idAddGrid, _T("Grid Widget"), _T("Add Grid Widget"));
    gaugeMenu->Append(idAdd2DHist, _T("2D Hist Widget"), _T("Add 2D Hist Widget"));
    gaugeMenu->Append(idAddShowspark, _T("Spark RPM Ploter"), _T("Add Spark Widget"));
    gaugeMenu->Append(idAddGridByTime, _T("Grid By Time"), _T("Add Grid By Time"));
    gaugeMenu->Append(idAddVemsGauge, _T("VemsGauge"), _T("Add VemsGauge"));

    mbar->Append(gaugeMenu, _T("&Test Gauges"));

    SetMenuBar(mbar);
#endif // wxUSE_MENUS

    m_book      = new wxBookCtrl( this, ID_BOOK );
    m_logPanel  = new wxPanel( m_book );
    m_settingsPanel = new wxPanel( m_book );

    m_book->AddPage( m_logPanel, _T("Log"), true );
    m_book->AddPage( m_settingsPanel, _T("Settings") );

    wxTextCtrl* m_debug_text = new wxTextCtrl( m_logPanel, wxID_ANY, _T(""), wxPoint(0, 0), wxSize(100, 150), wxTE_MULTILINE);

    wxBoxSizer* logSizer = new wxBoxSizer( wxHORIZONTAL );
    logSizer->Add( m_debug_text, 1, wxEXPAND | wxALL, 5 );
    m_logPanel->SetSizer( logSizer );
/*
#if defined(__WXMSW__)
    wxLog::SetActiveTarget(new wxLogTextCtrl(m_debug_text));
#endif
*/
#if wxUSE_STATUSBAR
    // create a status bar with some information about the used wxWidgets version
    CreateStatusBar(2);
    SetStatusText(_("Hello Code::Blocks user !"),0);
    SetStatusText(wxbuildinfo(short_f),1);
#endif // wxUSE_STATUSBAR


    //TimingTester * tt = new TimingTester();
    //tt->init();

/*
    fprintf(stderr, "Buff size: %d\n", size);
    int i;
    for( i=200; i< 600; i++){
        uint16_t leftSample = getSample(buf, i, LEFT);
        uint16_t rightSample = getSample(buf, i, RIGHT);
        fprintf(stderr, "%d\t%d\n", leftSample, rightSample);
    }
*/
    init();

// TODO: kill timer     m_timer.Start(2000);
}

void SparkMyFrame::init()
{

// TODO: calculate necessary size.
// for now total ringbuf is appr 120 * 1024 samples (more than 1 second even with 96000 sps)
m_ringBufferRecord = new RingBuffer(8, 4 * FRAMES_PER_BUFFER);

//    m_rtDataMgr = new RTDataMgrFacade();
//    m_rtDataMgr = m_vemsTuneController->m_rtDataMgr;
//    ((RTDataMgrFacade*)m_rtDataMgr)->SetInput(new RTDataInputSignalgen() );

    // create and run signal processing thread (consumer of m_ringBufferRecord, create consumer first):
    // if you see "RingBuffer is full. This is serious. Is SignalProcessThread dead ?"
    // than this did not work out, or AudioThread is producing into another RingBuffer ...

    /*
    m_signalProcessMgr = new SignalProcessThread( m_ringBufferRecord );
     if ( m_signalProcessMgr->Create() != wxTHREAD_NO_ERROR ){
        Logger::logf(LOG_MSG, _T("Can't create SignalProcessThread thread!"));
    }


    //m_signalProcessMgr->SetPriority(WXTHREAD_MAX_PRIORITY);
    m_signalProcessMgr->Run();
    */
    // ================= create and run audio processing thread (producer of m_ringBufferRecord):
    SyncObj* syncObj = new SyncObj();
    // ---------------------
    m_signalProcessMgr = new RTDataMgrFacade();
    m_sparkTesterWidgetFactory = new SparkTesterWidgetFactory(this, m_signalProcessMgr);

    RTDataInputSignalgen* dataInput = new RTDataInputSignalgen(m_ringBufferRecord, NULL, syncObj);
    ((RTDataMgrFacade*)m_signalProcessMgr)->SetInput( dataInput );
    dataInput->setActive(true);
    dataInput->initSignalProcess(); //MUST call after  SetInput()


    m_guiThread = new GUIConsumerThread(m_signalProcessMgr);
    if ( m_guiThread->Create() != wxTHREAD_NO_ERROR ){
        Logger::logf(LOG_MSG, _T("Can't create GUI Thread!"));
    }
    m_guiThread->Run();

    m_audioThread = new AudioThread( m_ringBufferRecord, syncObj );
    if ( m_audioThread->Create() != wxTHREAD_NO_ERROR ){
        Logger::logf(LOG_MSG, _T("Can't create audioThread thread!"));
    }
    // m_audioThread->SetPriority(WXTHREAD_MAX_PRIORITY);
    m_audioThread->Run();

    initGaugeDesc();
}

SparkMyFrame::~SparkMyFrame()
{
}


/**
 * we do NOT call fullDuplexTest() from here,
 * just signal AudioThread() so recording runs in separate thread.
*/
void SparkMyFrame::OnFullDuplex(wxCommandEvent& event)
{
    m_audioThread->setAudioDevice(0);
    m_audioThread->setRecordFlag( 1 );
    // signalling would enable immediate action, but waiting 500msec won't hurt in this case
}

void SparkMyFrame::OnFullDuplex1(wxCommandEvent& event)
{
     m_audioThread->setAudioDevice(1);
    m_audioThread->setRecordFlag( 1 );
    // signalling would enable immediate action, but waiting 500msec won't hurt in this case
}

/**
 * no separate thread for this, showing devices is quick
 */
void SparkMyFrame::OnShowDevices(wxCommandEvent& event)
{
    FullDuplexTest * fullDuplex = new FullDuplexTest( m_ringBufferRecord, 0, NULL);
    fullDuplex->initBuffer();
//    fullDuplex->init();
    fullDuplex->ShowDevices();
    delete fullDuplex;
}

void SparkMyFrame::SetSoundDialog(){

    int x = 50,
        y = 50;
    // Create the main frame window
    SoundFrame *frame = new SoundFrame(this,_T("Signalgen GUI"), x, y);
    frame->Show(true);
}

void SparkMyFrame::OnQuit(wxCommandEvent& event)
{
    Close();
}

void SparkMyFrame::OnAbout(wxCommandEvent& event)
{
    wxString msg = wxbuildinfo(long_f);
    wxMessageBox(msg, _("Welcome to..."));
}

void SparkMyFrame::OnTimer(wxTimerEvent& event)
{
     Logger::logf(LOG_MSG, _T("SparkMyFrame::OnTimer() "));
}

void SparkMyFrame::OnAddGauge(wxCommandEvent& event){

    fprintf(stderr, "MainFrame::SetGaugeDialog\n");

    VemsGridWidget *tmp = new VemsGridWidget(this);
    tmp->Show(1);


    m_signalProcessMgr->RegisterViewer(tmp);
    tmp->SetDataMgr(m_signalProcessMgr);


    /*
    GaugeFrame *frame = new GaugeFrame(this, wxNewId(), _("VemsGauge Test"), wxPoint(50,50),
        wxSize(240,240), _T("rpm"), m_gaugeDescA);
    registerGauge( frame );

    GaugeFrame *frame1 = new GaugeFrame(this, wxNewId(), _("VemsGauge Test"), wxPoint(290,50),
        wxSize(240,240), _T("sparkPeak"), m_gaugeDescA);
    registerGauge( frame1 );

    */
}

void SparkMyFrame::onWriteInFile(wxCommandEvent& event)
{
    WriteInFileByTime* widget = new WriteInFileByTime();
    widget->SetDataMgr( m_signalProcessMgr );
    //widget->SetDataMgr(m_signalProcessMgr);
    widget->RegisterViewer();

}
void SparkMyFrame::OnGaugeTest(wxCommandEvent& event)
{
    int id = event.GetId();

    if( id == idAdd2DHist) {
        m_sparkTesterWidgetFactory->ShowWidget(SPARKW_HISTOGRAM_2D);
    }else if(id == idAddGrid){
         m_sparkTesterWidgetFactory->ShowWidget(SPARKW_GRID);
    }else if( id == idAddShowspark) {
         m_sparkTesterWidgetFactory->ShowWidget(SPARKW_PHASE);
    }else if(id == idAddGridByTime){
         m_sparkTesterWidgetFactory->ShowWidget(SPARKW_GRID_BY_TIME);
    }else if(id == idAddVemsGauge){
         m_sparkTesterWidgetFactory->ShowWidget(SPARKW_GAUGE);
    }
}
/*
void SparkMyFrame::registerGauge(GaugeFrame *frame)
{

    if(m_signalProcessMgr != NULL){
        m_signalProcessMgr->RegisterViewer( (RTDataViewerBase*) frame->GetGauge() );
        frame->GetGauge()->SetDataMgr(m_signalProcessMgr);
        frame->GetGauge()->setMenuItemsState();
    }
    frame->Show( true );
}
*/

void SparkMyFrame::initGaugeDesc()
{

    m_gaugeDescA = new VEMSGaugeDescArray();

    VEMSGaugeDesc * vgd = new VEMSGaugeDesc();
	 vgd->SetSymbolName(_T("rpm"));
     vgd->SetUnit(_T("RPM"));
     vgd->SetMinValue(0);
     vgd->SetMaxValue(8000);
     vgd->SetMajorDivisions(8);
     vgd->SetMinorDivisions(10);
     vgd->SetUnitsScaleFactor(1000);
     vgd->SetGaugeNumberFormat(_T("%.0f"));
     vgd->coloredA.Add( new ColoredArea(wxColor(43,1,210),0,800 ));     /// wtf?
     vgd->coloredA.Add( new ColoredArea(wxColor(210,210,43),6000,7000 ));
     vgd->coloredA.Add( new ColoredArea(wxColor(210,1,43),7000,8000 ));

     m_gaugeDescA->Add(vgd);

     vgd = new VEMSGaugeDesc();
	 vgd->SetSymbolName(_T("sparkPeak"));
     vgd->SetUnit(_T(""));
     vgd->SetMinValue(0);
     vgd->SetMaxValue(50);
     vgd->SetMajorDivisions(5);
     vgd->SetMinorDivisions(5);
     vgd->SetUnitsScaleFactor(1);
     vgd->SetGaugeNumberFormat(_T("%.0f"));

    m_gaugeDescA->Add(vgd);

}
