#include "GridByTime.h"
#include "../../logger.h"
#include <wx/datetime.h>

#include "../../events/UpdateViewerEvent.h"
#include "../../wxVariantDataDoubleHash.h"

int idTimeGrid = wxNewId();
const long GridByTime::idScrollBar = wxNewId();

BEGIN_EVENT_TABLE(GridByTime, wxFrame)
    EVT_SCROLL(GridByTime::scrollGrid)

    EVT_UPDATE_VIEWER_EVENT( wxID_ANY, GridByTime::OnUpdateValue )

END_EVENT_TABLE()

GridByTime::GridByTime(wxWindow* parent, wxWindowID id, const wxString &title,  const wxPoint& pos, const wxSize& size, long style, const wxString& name)
        :wxFrame(parent, id, title ,pos, size, style)
{
    //Create(parent,1,_T(""));
  //  rowCnt = 0;

    wxBoxSizer *mainSizer = new wxBoxSizer(wxHORIZONTAL);
    SetInitialSize(size);
    m_grid = new wxGrid(this ,idTimeGrid);

    m_grid->CreateGrid(1,3);
    //m_grid->CreateGrid(55,3);

    m_grid->SetColLabelValue(0,_T("SparkPhase"));
    m_grid->SetColLabelValue(1,_T("RPM"));
    m_grid->SetColLabelValue(2,_T("Dwell"));
    //scrollBar = new wxScrollBar(this, idScrollBar,wxDefaultPosition,wxDefaultSize,wxSB_VERTICAL, wxDefaultValidator, _T("scrollBar"));
    mainSizer->Add(m_grid,0,wxEXPAND);
    //mainSizer->Add(scrollBar,0,wxEXPAND);
	SetSizer(mainSizer);
	mainSizer->Fit(this);
	mainSizer->SetSizeHints(this);
//	connect(idScrollBar,wxEVT_COMMAND_SCROLLBAR_UPDATED,wxScrollBarEvent(GridByTime::scrollGrid));
	//Connect(id,,wxCommandEventHandler(VemsNumPad::OnButton));
    //Refresh();
	Show(1);
    //ctor
}

GridByTime::~GridByTime()
{
    //dtor
    for(uint16_t cnt = 0;cnt < dataDoubleHashArray.GetCount();cnt++){
        delete dataDoubleHashArray[cnt];
    }
    UnRegisterViewer();
}
void GridByTime::storeData(DoubleHash *table)
{
    DoubleHash *temp = new DoubleHash();
    *temp = *table;

    wxDateTime now = wxDateTime::Now();
    wxString time = now.FormatTime();
    time = time.SubString(0,1)+time.SubString(3,4)+time.SubString(6,7);
    Logger::logf(LOG_MSG,_T("void GridByTime::storeData(DoubleHash *table)time=")+time);
    //m_grid->SetRowLabelValue(cnt,now.FormatTime());
    double timeIndouble;
    time.ToDouble(&timeIndouble);
    (*temp)[_T("time")] = timeIndouble;
    dataDoubleHashArray.Add(temp);

}

void GridByTime::refreshGrid()
{
    int start = dataDoubleHashArray.GetCount() - m_grid->GetNumberRows();
    if(start < 0){
        start = 0;
    }
    Logger::logf(LOG_MSG,_T("GridByTime::refreshGrid()start = %d"),start);
    int end = dataDoubleHashArray.GetCount();
    Logger::logf(LOG_MSG,_T("GridByTime::refreshGrid()end = %d"),end);

    for(uint16_t cnt = start;cnt < end ;cnt++){
        Logger::logf(LOG_MSG,_T("GridByTime::refreshGrid()0 cnt =%d"),cnt);
        if(cnt == dataDoubleHashArray.GetCount()){
            return;
        }
        DoubleHash *tmp = new DoubleHash();
        DoubleHash *tmpOld = new DoubleHash();
        tmp = dataDoubleHashArray[cnt];
        tmpOld = dataDoubleHashArray[cnt-1];
        int currentRow = cnt-start;
        int missing = (int)(*tmp)[_T("missing")];
        int sparkP = (int)(*tmp)[_T("phase")];
        int rpm    = (int)(*tmp)[_T("rpm")];
        int dwell  = (int)(*tmp)[_T("dwell")];
        int timeInInt = (int)(*tmp)[_T("time")];
        int sparkOld =sparkP;
        if(cnt >0){
            sparkOld = (int)(*dataDoubleHashArray[cnt-1])[_T("phase")];
        }
        Logger::logf(LOG_MSG,_T("GridByTime::refreshGrid()"));
        wxString timeInString = wxString::Format(wxT("%d"),timeInInt);
        timeInString=timeInString.SubString(0,1)+_T(":")+timeInString.SubString(2,3)+_T(":")+timeInString.SubString(4,5);
        //m_grid->AppendRows();
        //int lastRow = m_grid->GetNumberRows() -1;
        if(missing){
            m_grid->SetCellValue(currentRow,0,_T("missing"));
        }else{
            wxString sparkPString = wxString::Format(wxT("%d"),sparkP);
            m_grid->SetCellValue(currentRow,0,sparkPString);
        }


        wxString rpmString = wxString::Format(wxT("%d"),rpm);
        m_grid->SetCellValue(currentRow,1,rpmString);


        wxString dwellString = wxString::Format(wxT("%d"),dwell);
        Logger::logf(LOG_MSG,_T("GridByTime::refreshGrid()cnt1 =%d"),cnt);
        Logger::logf(LOG_MSG,_T("GridByTime::refreshGrid()dwellString:")+dwellString);
        m_grid->SetCellValue(currentRow,2,dwellString);
        Logger::logf(LOG_MSG,_T("GridByTime::refreshGrid()cnt2 =%d"),cnt);

        wxDateTime now = wxDateTime::Now();
//    printf("Current time in Paris:\t%s\n", now.Format("%c", wxDateTime::CET).c_str());
        //m_grid->SetRowLabelValue(cnt-start,timeInString);
    //m_grid->Refresh();
        if( !((-3< (sparkOld-sparkP) && (sparkOld-sparkP)<3) )   ) {//||(sparkPhaseOld-sparkP<=-3)){
            for(int i=0;i<m_grid->GetNumberCols();i++){
                m_grid->SetCellBackgroundColour(currentRow,i,wxColor(_T("RED")));
            }
            //Logger::logf(LOG_MSG,_T("GridByTime::refreshGrid()cnt1 =%d"),cnt);
        }else{
            for(int i=0;i<m_grid->GetNumberCols();i++){
                m_grid->SetCellBackgroundColour(currentRow,i,wxColor(_T("WHITE")));
            }
        }

        //sparkPhaseOld=sparkP;

    Logger::logf(LOG_MSG,_T("GridByTime::refreshGrid()cnt2 =%d"),cnt);
    }
    Logger::logf(LOG_MSG,_T("GridByTime::refreshGrid()The end "));
}

void GridByTime::scrollGrid(wxScrollEvent& event)
{
    Logger::logf(LOG_MSG,_T("-------------------------------GridByTime::scrollGrid(wxScrollEvent& event)"));
}

void GridByTime::refresh(DoubleHash* table)
{
    UpdateViewerEvent event;

    //...put some data to hash
    event.m_userData[_T("missing")] = (*table)[_T("missing")];
    event.m_userData[_T("phase")]   = (*table)[_T("phase")];
    event.m_userData[_T("rpm")]     = (*table)[_T("rpm")];
    event.m_userData[_T("dwell")]   = (*table)[_T("dwell")];

    this->AddPendingEvent( event );
}
int GridByTime::updateInputModel()
{
    return 0;
}

void GridByTime::sendInputEvent(int type, int inputEvent)
{
}

bool GridByTime::isExist(const wxString& symbolname)
{
    return true;
}

int GridByTime::updateViewerModel()
{
    return 0;
}

double GridByTime::getValue(const wxString& symbolname)
{
    return 0;
}

void GridByTime::OnUpdateValue( UpdateViewerEvent& event )
{

    int missing = event.m_userData[_T("missing")].GetLong();
    int sparkP  = event.m_userData[_T("phase")].GetLong();
    int rpm     = event.m_userData[_T("rpm")].GetLong();
    int dwell   = event.m_userData[_T("dwell")].GetLong();

 //   Logger::logf(LOG_MSG,_T("GridByTime::Update(DoubleHash* table)"));
 //   storeData(table);
    //if(dataDoubleHashArray.GetCount() <= m_grid->GetNumberRows()){
 //   refreshGrid();
    //}

//    Logger::logf(LOG_MSG,_T("GridByTime::Update(DoubleHash* table) spark = %d rpm = %d"),sparkP,rpm);
//    Logger::logf(LOG_MSG,_T("GridByTime::Update(DoubleHash* table) before m_grid->AppendRows();"));
    m_grid->AppendRows();
    Logger::logf(LOG_MSG,_T("GridByTime::refresh(DoubleHash* table) before m_grid->GetNumberRows();"));
    int lastRow = m_grid->GetNumberRows() -1;
//    Logger::logf(LOG_MSG,_T("GridByTime::Update(DoubleHash* table) before m_grid->SetCellValue1()1;"));
    if(missing){
        m_grid->SetCellValue(lastRow,0,_T("missing"));
    }else{
        wxString sparkPString = wxString::Format(wxT("%d"),sparkP);
        m_grid->SetCellValue(lastRow,0,sparkPString);
    }
    wxString rpmString = wxString::Format(wxT("%d"),rpm);
//    Logger::logf(LOG_MSG,_T("GridByTime::Update(DoubleHash* table) before m_grid->SetCellValue1()2;"));
    m_grid->SetCellValue(lastRow,1,rpmString);

    wxString dwellString = wxString::Format(wxT("%d"),dwell);
    m_grid->SetCellValue(lastRow,2,dwellString);

//    Logger::logf(LOG_MSG,_T("GridByTime::Update(DoubleHash* table) END"));
    wxDateTime now = wxDateTime::Now();
//    printf("Current time in Paris:\t%s\n", now.Format("%c", wxDateTime::CET).c_str());
    m_grid->SetRowLabelValue(lastRow,now.FormatTime());
    //m_grid->Refresh();
    if( !((-3< (sparkPhaseOld-sparkP) && (sparkPhaseOld-sparkP)<3) )   ) {//||(sparkPhaseOld-sparkP<=-3)){
            for(int i=0;i<m_grid->GetNumberCols();i++){
            m_grid->SetCellBackgroundColour(lastRow,i,wxColor(_T("RED")));
        }
    }
    sparkPhaseOld=sparkP;

    Logger::logf(LOG_MSG,_T("GridByTime::refresh(DoubleHash* table) end"));

}
