#include "ShowSparkPWidget.h"
#include <iostream>
#include "../Histogram.h"
#include "../../logger.h"

#include <wx/dcbuffer.h>
#include <wx/dcmemory.h>
#include <stdlib.h>
#include <wx/string.h>
#include <wx/font.h>

//#include <wx/ctrl.h>

#include "../../events/UpdateViewerEvent.h"

BEGIN_EVENT_TABLE(ShowSparkPhaseWidget, wxControl)

    EVT_PAINT(ShowSparkPhaseWidget::OnPaint)
#ifdef __WXGTK__
    EVT_WINDOW_CREATE(ShowSparkPhaseWidget::OnWindowCreate)
#endif

    EVT_UPDATE_VIEWER_EVENT( wxID_ANY, ShowSparkPhaseWidget::OnUpdateValue )

END_EVENT_TABLE()


ShowSparkPhaseWidget::ShowSparkPhaseWidget(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style, const wxString& name)
        :wxControl(parent, id, pos, size, style)
        ,m_idx(0)
        ,m_dataidx(0)
        ,m_changerefresh(0)
        ,bgsem(true)
{
    for (int i=0;i<MISSINGDATACNT;i++){
        m_missingdata[i]=0;
        m_missingdata2[i]=0;
    }
    m_missingPointer=0;
    m_missingRatio=100;
    m_missingRatio2=100;
    m_time= wxDateTime::UNow();

    statisticsdata_t tmp;
    tmp.min = 0;
    tmp.max = 0;
    tmp.cnt = 0;

    CopyStData(tmp,m_currPw);
    CopyStData(tmp,m_currPw2);
    CopyStData(tmp,m_currRpm);
    CopyStData(tmp,m_currSpark);
    CopyStData(tmp,m_currSpark2);
    CopyStData(tmp,m_currDphase);
    CopyStData(tmp,m_currDphase2);
    CopyStData(tmp,m_currDwell);
    CopyStData(tmp,m_currDwell2);
    CopyStData(tmp,m_prevDwell);
    CopyStData(tmp,m_prevDwell2);
    CopyStData(tmp,m_prevPw);
    CopyStData(tmp,m_prevPw2);
    CopyStData(tmp,m_prevRpm);
    CopyStData(tmp,m_prevSpark);
    CopyStData(tmp,m_prevSpark2);

    m_missing_prev = 0;
    m_missing_curr = 0;
    m_missing_prev2 = 0;
    m_missing_curr2 = 0;

    SetInitialSize(size);
    SetSize(size);
    m_backGround = new wxBitmap( 800 , 800 );
    m_checkbox_rpm = new wxCheckBox(this,wxID_ANY,_T("rpm") ,wxPoint(780,10));
    m_checkbox_rpm->SetValue(true);
    m_checkbox_rpm->SetForegroundColour(wxColor(0xd5,0x20,0x52));
    m_checkbox_dwell1 = new wxCheckBox(this,wxID_ANY,_T("dwell_ch1"),wxPoint(780,50));
    m_checkbox_dwell1->SetValue(true);
    m_checkbox_dwell1->SetForegroundColour(wxColor( 0x28,0x83,0x1e ));
    m_checkbox_spark1 = new wxCheckBox(this,wxID_ANY,_T("spark_ch1"),wxPoint(780,100));
    m_checkbox_spark1->SetValue(true);
    m_checkbox_spark1->SetForegroundColour(wxColor( 0x00,0x00,0xc2 ));
    m_checkbox_missing1 = new wxCheckBox(this,wxID_ANY,_T("missing1_ch1"),wxPoint(780,150));
    m_checkbox_missing1->SetValue(true);
    m_checkbox_missing1->SetForegroundColour(wxColor( 0xa6,0x93,0x2d ));
    m_checkbox_pw1 = new wxCheckBox(this,wxID_ANY,_T("pulse_width_ch1"),wxPoint(780,200));
    m_checkbox_pw1->SetValue(false);
    m_checkbox_pw1->SetForegroundColour(wxColor( 0x00,0x00,0xc2 ));
    m_checkbox_dwell2 = new wxCheckBox(this,wxID_ANY,_T("dwell_ch2"),wxPoint(780,250));
    m_checkbox_dwell2->SetValue(true);
    m_checkbox_dwell2->SetForegroundColour(wxColor( 0x5b,0xff,0x1e ));
    m_checkbox_spark2 = new wxCheckBox(this,wxID_ANY,_T("spark_ch2"),wxPoint(780,300));
    m_checkbox_spark2->SetValue(true);
    m_checkbox_spark2->SetForegroundColour(wxColor( 0x5c,0xcf,0xc2 ));
    m_checkbox_missing2 = new wxCheckBox(this,wxID_ANY,_T("missing1_ch2"),wxPoint(780,350));
    m_checkbox_missing2->SetValue(true);
    m_checkbox_missing2->SetForegroundColour(wxColor( 0xff,0xc9,0x38 ));
    m_checkbox_pw2 = new wxCheckBox(this,wxID_ANY,_T("pulse_width_ch2"),wxPoint(780,400));
    m_checkbox_pw2->SetValue(false);
    m_checkbox_pw2->SetForegroundColour(wxColor( 0x00,0x00,0xc2 ));
    m_checkbox_dwellphase1 = new wxCheckBox(this,wxID_ANY,_T("dwell phase ch1"),wxPoint(780,450));
    m_checkbox_dwellphase1->SetValue(false);
    m_checkbox_dwellphase1->SetForegroundColour(wxColor( 0x06,0xff,0xc2 ));
    m_checkbox_dwellphase2 = new wxCheckBox(this,wxID_ANY,_T("dwell phase ch2"),wxPoint(780,500));
    m_checkbox_dwellphase2 = new wxCheckBox(this,wxID_ANY,_T("dwell phase ch2"),wxPoint(780,500));
    m_checkbox_dwellphase2->SetValue(false);
    m_checkbox_dwellphase2->SetForegroundColour(wxColor( 0x06,0xff,0xc2 ));



    Refresh(false);
    //ctor

    m_refreshInterval = wxTimeSpan::Milliseconds(REFRESHTIME);
}

ShowSparkPhaseWidget::~ShowSparkPhaseWidget()
{
    UnRegisterViewer();
    delete m_backGround;
    //dtor
}

void ShowSparkPhaseWidget::OnPaint(wxPaintEvent &event)
{
    wxBufferedPaintDC dc(this);

    dc.SetBackground( *wxWHITE_BRUSH );
    dc.Clear();

    if(bgsem){
        PaintBackGround();
        bgsem = false;
    }



    if(m_newDatas){
        m_newDatas = false;

        //Logger::logf(LOG_MSG,_T("m_changerefresh=%i"),m_changerefresh);

        wxMemoryDC memdc;
        short margin = 35;

        uint16_t counter=0;

        if(m_changerefresh == 0) {
            memdc.SelectObjectAsSource( *m_backGround );
            counter = m_dataidx;
        } else {
            wxBitmap tmp = m_backGround->GetSubBitmap( wxRect(margin, 0, m_backGround->GetWidth()-margin, m_backGround->GetHeight()) );
            memdc.SelectObject(*m_backGround);
            memdc.DrawBitmap( tmp, margin-5, 0, 1);
            counter = IDX;

        }

        if (m_checkbox_dwell1->GetValue()){
            memdc.SetPen(wxColor( 0x1f,0xbf,0x1e ));
            memdc.DrawLine((counter-1)*5+(margin-5),740-m_prevDwell.min,(counter)*5+(margin-5),740-m_currDwell.min);

            memdc.SetPen(wxColor( 0x28,0x83,0x1e ));
            memdc.DrawLine((counter-1)*5+(margin-5),740-m_prevDwell.max,(counter)*5+(margin-5),740-m_currDwell.max);
        }

        if (m_checkbox_spark1->GetValue()){

            memdc.SetPen(wxColor( 0x11,0x91,0xc2 ));
            memdc.DrawLine((counter-1)*5+(margin-5),740-m_prevSpark.min,(counter)*5+(margin-5),740-m_currSpark.min);

            memdc.SetPen(wxColor( 0x00,0x00,0xc2 ));
            memdc.DrawLine((counter-1)*5+(margin-5),740-m_prevSpark.max,(counter)*5+(margin-5),740-m_currSpark.max);

        }
        if (m_checkbox_dwellphase1->GetValue()){

            memdc.SetPen(wxColor( 0x06,0xff,0xc2 ));
            memdc.DrawLine((counter-1)*5+(margin-5),740-m_prevDphase.min,(counter)*5+(margin-5),740-m_currDphase.min);

            memdc.SetPen(wxColor( 0x0c,0xcf,0xc2 ));
            memdc.DrawLine((counter-1)*5+(margin-5),740-m_prevDphase.max,(counter)*5+(margin-5),740-m_currDphase.max);

        }

        if (m_checkbox_rpm->GetValue()){

            memdc.SetPen(wxColor( 0xee,0x6d,0x94 ));
            memdc.DrawLine((counter-1)*5+(margin-5),740-m_prevRpm.min,(counter)*5+(margin-5),740-m_currRpm.min);

            memdc.SetPen(wxColor( 0xd5,0x20,0x52 ));
            memdc.DrawLine((counter-1)*5+(margin-5),740-m_prevRpm.max,(counter)*5+(margin-5),740-m_currRpm.max);
        }

        if (m_checkbox_missing1->GetValue()){
            memdc.SetPen(wxColor( 0xa6,0x93,0x2d ));
            memdc.DrawLine((counter-1)*5+(margin-5),740-(m_missing_prev/16),(counter)*5+(margin-5),740-(m_missing_curr)/16);

            memdc.SetPen(wxColor( 0x00,0x00,0x00 ));
            memdc.DrawLine((counter-1)*5+(margin-5),740-(m_missingRatio_prev),(counter)*5+(margin-5),740-(m_missingRatio));
        }

       if (m_checkbox_pw1->GetValue()){
            memdc.SetPen(wxColor( 0xff,0xc9,0x38 ));
            memdc.DrawLine((counter-1)*5+(margin-5),740-(m_prevPw.min),(counter)*5+(margin-5),740-(m_currPw.min));

            memdc.SetPen(wxColor( 0x77,0x77,0x77 ));
            memdc.DrawLine((counter-1)*5+(margin-5),740-(m_prevPw.max),(counter)*5+(margin-5),740-(m_currPw.max));
        }

        if (m_checkbox_dwell2->GetValue()){
            memdc.SetPen(wxColor( 0xa2,0xff,0x1e ));
            memdc.DrawLine((counter-1)*5+(margin-5),740-m_prevDwell2.min,(counter)*5+(margin-5),740-m_currDwell2.min);

            memdc.SetPen(wxColor( 0x5b,0xff,0x1e ));
            memdc.DrawLine((counter-1)*5+(margin-5),740-m_prevDwell2.max,(counter)*5+(margin-5),740-m_currDwell2.max);
        }

        if (m_checkbox_spark2->GetValue()){

            memdc.SetPen(wxColor( 0xb6,0xff,0xc2 ));
            memdc.DrawLine((counter-1)*5+(margin-5),740-m_prevSpark2.min,(counter)*5+(margin-5),740-m_currSpark2.min);

            memdc.SetPen(wxColor( 0x5c,0xcf,0xc2 ));
            memdc.DrawLine((counter-1)*5+(margin-5),740-m_prevSpark2.max,(counter)*5+(margin-5),740-m_currSpark2.max);

        }

        if (m_checkbox_dwellphase2->GetValue()){

            memdc.SetPen(wxColor( 0x06,0xff,0xc2 ));
            memdc.DrawLine((counter-1)*5+(margin-5),740-m_prevDphase2.min,(counter)*5+(margin-5),740-m_currDphase2.min);

            memdc.SetPen(wxColor( 0x0c,0xcf,0xc2 ));
            memdc.DrawLine((counter-1)*5+(margin-5),740-m_prevDphase2.max,(counter)*5+(margin-5),740-m_currDphase2.max);

        }

        if (m_checkbox_missing2->GetValue()){
            memdc.SetPen(wxColor( 0xff,0xc9,0x38 ));
            memdc.DrawLine((counter-1)*5+(margin-5),740-(m_missing_prev2/16),(counter)*5+(margin-5),740-(m_missing_curr2)/16);

            memdc.SetPen(wxColor( 0x77,0x77,0x77 ));
            memdc.DrawLine((counter-1)*5+(margin-5),740-(m_missingRatio_prev2),(counter)*5+(margin-5),740-(m_missingRatio2));
        }

        if (m_checkbox_pw2->GetValue()){
            memdc.SetPen(wxColor( 0xff,0xc9,0x38 ));
            memdc.DrawLine((counter-1)*5+(margin-5),740-(m_prevPw2.min),(counter)*5+(margin-5),740-(m_currPw2.min));

            memdc.SetPen(wxColor( 0x77,0x77,0x77 ));
            memdc.DrawLine((counter-1)*5+(margin-5),740-(m_prevPw2.max),(counter)*5+(margin-5),740-(m_currPw2.max));
        }

        dc.Blit(0,0,800,800,&memdc, 0, 0);

        memdc.SelectObject( wxNullBitmap );
        CopyStData(m_currPw,m_prevPw);
        CopyStData(m_currPw2,m_prevPw2);
        CopyStData(m_currRpm,m_prevRpm);
        CopyStData(m_currSpark,m_prevSpark);
        CopyStData(m_currSpark2,m_prevSpark2);
        CopyStData(m_currDphase,m_prevDphase);
        CopyStData(m_currDphase2,m_prevDphase2);
        CopyStData(m_currDwell,m_prevDwell);
        CopyStData(m_currDwell2,m_prevDwell2);
        m_missing_prev = m_missing_curr;
        m_missing_prev2 = m_missing_curr2;
        m_missingRatio_prev = m_missingRatio;
        m_missingRatio_prev2 = m_missingRatio2;



    } else {

        dc.DrawBitmap(*m_backGround,0,0,1);

    }

    dc.DrawBitmap( wxBitmap(m_coordsImg), 0,0,1);

}

void ShowSparkPhaseWidget::PaintBackGround()
{
    wxMemoryDC memdc;
    memdc.SelectObject( *m_backGround );
    memdc.SetBackground( *wxWHITE_BRUSH );
    memdc.Clear();

    //memdc.DrawLine(30,10,30,730);
    memdc.DrawLine(30,0,30,770);
    memdc.DrawLine(0,740,800,740);
    memdc.DrawLine(30,0,35,10);
    memdc.DrawLine(30,0,25,10);
    memdc.DrawLine(800,740,790,745);
    memdc.DrawLine(800,740,790,735);
    memdc.SetFont(*wxSMALL_FONT);
    int n;
    for( n = 0; n < 6; n++ ){
       wxString stringnumber = wxString::Format(wxT("%d"), (int)(6-n)*100);
       memdc.DrawText( stringnumber, 0, 140+n*100);
       memdc.DrawLine(20,140+n*100,40,140+n*100);
    }
    memdc.DrawText( _T("720"),0,20);
    memdc.DrawLine(20,20,40,20);
    memdc.DrawText( _T("10000"),35,640);

    wxString stringnumber =_T("SparkPhase( degree )");

    //memdc.DrawText(stringnumber,0,40);
    //stringnumber =_T("phase");
    //memdc.DrawText(stringnumber,0,50);
    memdc.DrawRotatedText(stringnumber,20,120,90);
    stringnumber =_T("RPM");
    memdc.DrawRotatedText(stringnumber,30,120,90);
    stringnumber =_T("time");
    memdc.DrawText(stringnumber,750,740);
    wxColor color = wxColor(_T("blue"));
    memdc.SetTextForeground(color);
    memdc.DrawText(_T("max"),0,740);
    memdc.SetTextForeground(wxColor(_T("Black")));
    memdc.DrawText(_T("max"),35,740);
    memdc.SetTextForeground(wxColor(128,128,128));
    memdc.DrawText(_T("missing"),70,740);
    color = wxColor(_T("red"));
    memdc.SetTextForeground(color);
    memdc.SetPen(color);
    memdc.DrawText(_T("min"),0,750);
    memdc.SetTextForeground(wxColor(_T("green")));
    memdc.DrawText(_T("min"),35,750);



    m_coordsImg = m_backGround->ConvertToImage();

    uint32_t width = m_coordsImg.GetWidth();
    uint32_t height = m_coordsImg.GetHeight();


    m_coordsImg.SetAlpha(0);

    for(uint32_t y = 0; y < height; y++) {
        for(uint32_t x = 0; x < width; x++) {

            uint8_t r = m_coordsImg.GetRed(x,y);
            uint8_t g = m_coordsImg.GetGreen(x,y);
            uint8_t b = m_coordsImg.GetBlue(x,y);


            if( !(r == 255 && g == 255 && b == 255) )
                m_coordsImg.SetAlpha(x,y,255);

        }
    }

   // m_coordsImg.SaveFile(_T("/tmp/1.png"),wxBITMAP_TYPE_PNG);




    memdc.Clear();
    memdc.SelectObject(wxNullBitmap);


}
void ShowSparkPhaseWidget::OnWindowCreate(wxWindowCreateEvent& event)
{
    Logger::logf(LOG_MSG,_T("Vems2HistogramWidget::OnWindowCreate"));
    Refresh(false);
}


void ShowSparkPhaseWidget::storeminmax(uint16_t value,struct statisticsdata_t &data)
{

        if( data.cnt != 0 ){
            if( value < data.min ){
                data.min = value;
            }

            if( value > data.max ){
                data.max = value;
            }
        }
        if( data.cnt == 0 ){
           // data[m_dataidx].cnt = 0;
            data.min = value;
            data.max = value;
        }

        data.cnt++;

}

void ShowSparkPhaseWidget::CopyStData(const statisticsdata_t &from, statisticsdata_t &to)
{

    to.min = from.min;
    to.max = from.max;
    to.cnt = from.cnt;

}

void ShowSparkPhaseWidget::refresh(DoubleHash* table)
{
   // Logger::logf(LOG_MSG,_T("------------------------------------------------------------------update"));

   // Logger::logf(LOG_MSG,_T("----------ShowSparkPhaseWidget::Update m_dataidx=%d spark.cnt=%d"),m_dataidx,spark[m_dataidx].cnt);
    uint16_t dwell2  = (uint16_t)(*table)[_T("dwell2")];
    uint16_t phase2 = (uint16_t) (*table)[_T("phase2")];
    uint16_t Dphase2 = (uint16_t) (*table)[_T("Dphase2")];
    uint16_t dwell  = (uint16_t)(*table)[_T("dwell")];
    uint16_t phase = (uint16_t) (*table)[_T("phase")];
    uint16_t Dphase = (uint16_t) (*table)[_T("Dphase")];
    uint16_t rpmvalue = (uint16_t) (*table)[_T("rpm")];
    uint16_t pulse_width = (uint16_t) (*table)[_T("pulse_width")];
    uint16_t pulse_width2 = (uint16_t) (*table)[_T("pulse_width2")];

    if ( m_checkbox_pw1->GetValue() ) Logger::logf(LOG_MSG,_T("ShowSparkPhaseWidget::refresh pulse width1=%d"),pulse_width);
    if ( m_checkbox_pw2->GetValue() ) Logger::logf(LOG_MSG,_T("ShowSparkPhaseWidget::refresh pulse width2=%d"),pulse_width2);
    if ( m_checkbox_spark1->GetValue() ) Logger::logf(LOG_MSG,_T("ShowSparkPhaseWidget ch1 spark pos=%2X"),(uint16_t) (*table)[_T("sparkPos")]);
    if ( m_checkbox_spark2->GetValue() ) Logger::logf(LOG_MSG,_T("ShowSparkPhaseWidget ch2 spark pos=%2X"),(uint16_t) (*table)[_T("sparkPos2")]);
    if ( m_checkbox_dwellphase1->GetValue() ) Logger::logf(LOG_MSG,_T("ShowSparkPhaseWidget ch1 dwell pos=%2X"),(uint16_t) (*table)[_T("dwellPos")]);
    if ( m_checkbox_dwellphase2->GetValue() ) Logger::logf(LOG_MSG,_T("ShowSparkPhaseWidget ch2 dwell pos=%2X"),(uint16_t) (*table)[_T("dwellPos2")]);



    if ((uint16_t) (*table)[_T("missing")]) m_missing_curr ++;
    if (m_missing_curr > 720*16) m_missing_curr-=720*16;

    if ((uint16_t) (*table)[_T("missing2")]) m_missing_curr2 ++;
    if (m_missing_curr2 > 720*16) m_missing_curr2-=720*16;

    m_missingdata[m_missingPointer]=(uint8_t) (*table)[_T("missing")];
    m_missingdata2[m_missingPointer]=(uint8_t) (*table)[_T("missing2")];

    m_missingPointer++;
    if (m_missingPointer==MISSINGDATACNT) m_missingPointer=0;

    uint16_t sum=0;
    uint16_t sum2=0;
    for (int i=0;i<MISSINGDATACNT;i++){
        sum=sum+m_missingdata[i];
        sum2=sum2+m_missingdata2[i];
    }
    sum=sum*100;
    m_missingRatio=sum/MISSINGDATACNT+2;
    if ( m_checkbox_missing1->GetValue() ) Logger::logf(LOG_MSG,_T("ShowSparkPhaseWidget::refresh m_missingRatio=%d"),m_missingRatio);
    //Logger::logf(LOG_MSG,_T("ShowSparkPhaseWidget::refresh pulse_width=%d"),pulse_width);

    sum2=sum2*100;
    m_missingRatio2=sum2/MISSINGDATACNT;
    if ( m_checkbox_missing2->GetValue() ) Logger::logf(LOG_MSG,_T("ShowSparkPhaseWidget::refresh m_missingRatio2=%d"),m_missingRatio2);


    rpmvalue = (uint16_t)(rpmvalue*0.05);

    dwell = dwell/20;
    dwell2 = dwell2/20;
    //phase = phase /10;
    storeminmax(pulse_width,m_currPw);
    storeminmax(pulse_width2,m_currPw2);
    storeminmax(dwell,m_currDwell);
    storeminmax(dwell2,m_currDwell2);
    storeminmax(rpmvalue,m_currRpm);
    storeminmax(phase,m_currSpark);
    storeminmax(phase2,m_currSpark2);
    storeminmax(Dphase,m_currDphase);
    storeminmax(Dphase2,m_currDphase2);

    //Logger::logf(LOG_MSG,_T("ShowSparkPhaseWidget::refresh pw_min:%d"),m_currPw.min );

    wxDateTime currtime = wxDateTime::UNow();



    if( currtime-m_refreshInterval > m_lastTime  ){

        UpdateViewerEvent event;
        this->AddPendingEvent( event );

        m_lastTime = currtime;
    }

}
int ShowSparkPhaseWidget::updateInputModel()
{
    return 0;
}
void ShowSparkPhaseWidget::sendInputEvent(int type, int inputEvent)
{

}
bool ShowSparkPhaseWidget::isExist(const wxString& symbolname)
{
    return false;
}
int ShowSparkPhaseWidget::updateViewerModel()
{
    return 0;
}
double ShowSparkPhaseWidget::getValue(const wxString& symbolname)
{
    return 0;
}

void ShowSparkPhaseWidget::OnUpdateValue( UpdateViewerEvent& event )
{
    wxDateTime now= wxDateTime::UNow();

    if (now<m_time) return;

    wxTimeSpan period_time(0,0,0,5);


    m_time=now.Add(period_time);

    m_newDatas = true;
    Refresh(false);

    m_dataidx++;
    //Logger::logf(LOG_MSG,_T("ShowSparkPhaseWidget::refresh m_dataidx =%d"),m_dataidx);

    if (m_dataidx > IDX){
        m_dataidx = 0;
        m_changerefresh++;
    }

    m_currDwell.cnt = 0;
    m_currSpark.cnt = 0;
    m_currDphase.cnt = 0;
    m_currDwell2.cnt = 0;
    m_currSpark2.cnt = 0;
    m_currDphase2.cnt = 0;
    m_currRpm.cnt = 0;
    m_currPw.cnt = 0;
    m_currPw2.cnt = 0;




}


