#include "Vems2DHistogramWidget.h"

#include "../Histogram.h"
#include "../../logger.h"
#include "../../events/UpdateViewerEvent.h"
#include "../../wxVariantDataDoubleHash.h"

#include <wx/dcbuffer.h>
#include <wx/dcmemory.h>
#include <stdlib.h>
#include <wx/string.h>
#include <wx/font.h>
#include <wx/image.h>
#include <wx/frame.h>

BEGIN_EVENT_TABLE(Vems2DHistogramWidget, wxControl)

    EVT_PAINT(Vems2DHistogramWidget::OnPaint)
    EVT_MIDDLE_DOWN(Vems2DHistogramWidget::onoffZoomFunction)
    EVT_RIGHT_DOWN(Vems2DHistogramWidget::zoomWindow)
    EVT_MOUSEWHEEL(Vems2DHistogramWidget::setzoom)

    EVT_MOTION(Vems2DHistogramWidget::getMouseCoordinates)
#ifdef __WXGTK__
    EVT_WINDOW_CREATE(Vems2DHistogramWidget::OnWindowCreate)
#endif

    EVT_UPDATE_VIEWER_EVENT( wxID_ANY, Vems2DHistogramWidget::OnUpdateValue )

END_EVENT_TABLE()

Vems2DHistogramWidget::Vems2DHistogramWidget(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style, const wxString& name)
        :wxControl(parent, id, pos, size, style)
        ,m_redrawFace (true)
        ,bgsem(true)
        ,zoomheigh(25)
        ,zoomwidth(25)
        ,zoomsize(4)
        ,state(0)
{
    //ctor

    SetInitialSize(size);

    SetSize(size);
/*
    for(uint8_t a = 0; a < 25; a++) {
        for(uint8_t b = 0; b < 72; b++) {
            m_tmp[rand()%255 + 1][rand()%719 + 1] = rand()%1000 + 1;
        }
    }
*/

    //m_backGround = new wxBitmap( 750 , 280 );
    m_backGround = new wxBitmap( 800 , 400 );

    //m_backGround->
    Refresh(false);

}

Vems2DHistogramWidget::~Vems2DHistogramWidget()
{
    //dtor
    UnRegisterViewer();
    delete m_backGround;
}

/**
 * 0 = white
 * 1..31 red + green + 224 blue
 * 32..255 red + green + partialblue
 * 256..511 red + partialgreen
 * 512..767 partial red
 */
wxColor colorGradient(uint16_t hv, uint16_t maxrate)
{
    uint8_t r, g, b;
    if(hv == 0){
        r=g=b = 0xFF; // all white
    } else {
        uint16_t rate = (hv * 767) / maxrate;   // rate = 0..767
        if( rate <= 55 ){ // r==0 .. 55
            r = g = 0xFF;
            b = 200; // not higher, we do NOT want almost white
        } else if (rate <= 255){ // 56 .. 255
            r = g = 0xFF;
            b = 0xFF - rate;
        } else if (rate <= 511){ // 256 .. 511
            r = 0xFF;
            g = 511 - rate;
            b = 0;
        } else { // 512 .. 767
            r = 767 - rate;
            g = b = 0;
        }
    }
    wxColor color = wxColor( r, g, b );
    return color;
}

void Vems2DHistogramWidget::OnPaint(wxPaintEvent &event)
{
    //Logger::logf(LOG_MSG,_T("Vems2DHistogramWidget::OnPaint("));
    wxBufferedPaintDC dc(this);
    wxBitmap zoomplace ;
    wxMemoryDC memdc;
    zoomplace = wxBitmap(100, 32, -1);

    dc.SetBackground( *wxWHITE_BRUSH );

    dc.Clear();
    if(bgsem)
    {
        Logger::logf(LOG_MSG,_T("Vems2DHistogramWidget::OnPaint bsgem"));
        PaintBackGround();
        bgsem = false;
    }

    dc.DrawBitmap( *m_backGround, 0,0,1);

    if( m_redrawFace ) {
        uint16_t maxrate = m_hist->GetMaxRate();
        for(uint16_t a = 0; a < AMPLIDUDE_BINS; a++) {
            for(uint16_t b = 0; b < SPARKPHASE_BINS ; b++) {
                uint16_t hv = m_hist->m_hisztogram_array[a][b]; // histogram value
                if( hv > 0 ) { // nonempty bin
                    dc.SetPen( colorGradient(hv, maxrate) );
                    dc.DrawPoint(b,a);
                }
            }
        }
        m_redrawFace = false;
    } else {
        dc.SetPen( m_pixelColor );
        dc.DrawPoint( m_x, m_y);
    }
    //dc.DrawPoint( 150, 150);
    //dc.DrawPoint( 152, 150);
    memdc.SelectObject( *m_backGround );
    memdc.SetBackground( *wxWHITE_BRUSH );
    memdc.Clear();

    //Logger::logf(LOG_MSG,_T("OnPaint x=%i, y=%i"),dc.GetSize().x,dc.GetSize().y);
    memdc.Blit(0,0,dc.GetSize().x,dc.GetSize().y,&dc,0,0);

    if( state == 1 ){//make or refresh zoom rack
     Logger::logf(LOG_MSG,_T("Vems2DHistogramWidget::OnPaint state = 1" ));

        wxRect rect =wxRect(m_mousex-zoomwidth,m_mousey-zoomheigh,zoomwidth*2,zoomheigh*2);
        zoomplace = m_backGround->GetSubBitmap(rect);

        wxImage z;
        z = zoomplace.ConvertToImage();
        z.Rescale(zoomwidth*zoomsize,zoomheigh*zoomsize);
        zoomplace = wxBitmap(z,-1 );

        dc.DrawRectangle(m_mousex-(zoomwidth*zoomsize)/2-2, m_mousey-zoomheigh*zoomsize/2-2, zoomwidth*zoomsize+4, zoomheigh*zoomsize+4);
        dc.DrawBitmap( zoomplace,m_mousex-(zoomwidth*zoomsize)/2,m_mousey-zoomheigh*zoomsize/2,0);

        wxString stringnumber =_T("Sparkphase =");
        wxString stringzoomtext =_T("8XZOOM");
        stringnumber += wxString::Format(wxT("%d"), (int)m_mousex-35);
        int offsettotext =20;

        if(zoomsize == 4){
            dc.SetFont(*wxSMALL_FONT);
            stringzoomtext =_T("4XZOOM");
            offsettotext = 15;
        }

        dc.DrawText(stringnumber,m_mousex-(zoomwidth*zoomsize)/2-2,m_mousey-zoomheigh*zoomsize/2-2);
        dc.DrawText(stringzoomtext,m_mousex-(zoomwidth*zoomsize)/2-offsettotext,m_mousey-zoomheigh*zoomsize/2-offsettotext);
    }
}

void Vems2DHistogramWidget::PaintBackGround()
{
    Logger::logf(LOG_MSG,_T(">>>> Vems2DHistogramWidget::PaintBackGround()1"));

    wxMemoryDC memdc;
    memdc.SelectObject( *m_backGround );
    memdc.SetBackground( *wxWHITE_BRUSH );
    memdc.Clear();
    int n =0;

    Logger::logf(LOG_MSG,_T(">>>> Vems2DHistogramWidget::PaintBackGround()2"));

    wxString stringnumber = wxString::Format(wxT("%d"), (int)(SPARKPHASE_BINS));
    memdc.SetFont(*wxSMALL_FONT);
    memdc.DrawText( stringnumber, 745, 270);

    stringnumber = wxString::Format(wxT("%d"), (int)(AMPLIDUDE_BINS*AMPLIDUDE_BINS));
    memdc.DrawText( stringnumber, 0, 0);

    stringnumber = wxString::Format(wxT("%d"), (int)(AMPLIDUDE_BINS*128));
    memdc.DrawText( stringnumber, 0, 10);

    stringnumber = wxString::Format(wxT("%d"), (int)(0));
    memdc.DrawText( stringnumber, 25, 270);
    Logger::logf(LOG_MSG,_T(">>>> Vems2DHistogramWidget::PaintBackGround()3"));
    paintGrid(memdc,10, 10, 10);
    for( n = 0; n < 7; n++ ){
        stringnumber = wxString::Format(wxT("%d"), (int)n*100);
        memdc.DrawText( stringnumber, 25+n*100, 270);
    }
    Logger::logf(LOG_MSG,_T(">>>> Vems2DHistogramWidget::PaintBackGround()4"));
    wxColor color = wxColor( 0, 0, 0 );
    memdc.SetPen(color);

    memdc.DrawLine(35,0,35,295);
    memdc.DrawLine(0,266,764,266);
    memdc.DrawLine(30,9,40,9);
    memdc.DrawLine(755,260,755,270);
    Logger::logf(LOG_MSG,_T(">>>> Vems2DHistogramWidget::PaintBackGround()5"));
   for( n = 1; n < 73; n++ ){//make rating
        memdc.DrawLine(35+n*10,266,35+n*10,270);
    }
//    m_backGround->SaveFile(_T("teve.bmp"),wxBITMAP_TYPE_BMP);
    Logger::logf(LOG_MSG,_T(">>>> Vems2DHistogramWidget::PaintBackGround() END"));
}

void Vems2DHistogramWidget::paintGrid(wxMemoryDC& memdc,int startx,int starty ,int  distance)
{
    int n = 0;
    wxColor color = wxColor( 240, 240, 240 );
    memdc.SetPen(color);
    for( n = 1; n < 73; n++ ){
        memdc.DrawLine(35+n*distance,9,35+n*distance,270);
    }
    for( n = 1; n < 26; n++ ){
        memdc.DrawLine(35,265-n*distance,755,265-n*distance);
    }
}
void Vems2DHistogramWidget::paintAxle(wxMemoryDC& memdc,int startx,int starty ,int  distance)
{
    wxColor color = wxColor( 0, 0, 0 );
    memdc.SetPen(color);
    memdc.DrawLine(35,0,35,295);
    memdc.DrawLine(0,266,764,266);
    memdc.DrawLine(30,9,40,9);
    memdc.DrawLine(755,260,755,270);

}

void Vems2DHistogramWidget::refresh(DoubleHash* table)
{
    //Logger::logf(LOG_MSG,_T(">>>>Vems2DHistogramWidget:Update"));
    UpdateViewerEvent event;

    event.m_userData[_T("phase")] = (*table)[_T("phase")];
    event.m_userData[_T("ampl")]  = (*table)[_T("ampl")];

    this->AddPendingEvent( event );
}

void Vems2DHistogramWidget::getMouseCoordinates(wxMouseEvent& event)
{
    if(state == 1){
        Logger::logf(LOG_MSG,_T("Vems2DHistogramWidget::makeZoomFrame"));
            m_mousex = event.GetX();
            m_mousey = event.GetY();
            Refresh(false);
    }
}
void Vems2DHistogramWidget::setzoom(wxMouseEvent& event)
{
    if(state == 1){
        if(zoomsize == 4){
            zoomsize *= 2;
        }else{//zoomsize 8X
            zoomsize /= 2;
        }
        Refresh(false);
    }
}
void Vems2DHistogramWidget::onoffZoomFunction(wxMouseEvent& event)
{
       // Logger::logf(LOG_MSG,_T("Vems2DHistogramWidget::onoff"));
    if( state == 0 ){
        state = 1;
        //Logger::logf(LOG_MSG,_T("state =%i"),state);
    }else if( state == 1 ){
        state = 0;
        //Logger::logf(LOG_MSG,_T("state=%i"),state);
    }
    m_mousex = event.GetX();
    m_mousey = event.GetY();
    Refresh(false);

}

void Vems2DHistogramWidget::OnWindowCreate(wxWindowCreateEvent& event)
{
 Logger::logf(LOG_MSG,_T("Vems2DHistogramWidget::OnWindowCreate"));
Refresh(false);
}

void Vems2DHistogramWidget::zoomWindow(wxMouseEvent& event)
{
        wxFrame *tmpframe = new wxFrame(this, wxID_ANY, _T("new widget test"), wxPoint(400,400), wxSize(320,120));
        tmpframe->Show(1);

//        zoom2dhistogram *widget = new zoom2dhistogram(tmpframe, wxID_ANY);


}
int Vems2DHistogramWidget::updateInputModel()
{
    return 0;
}

void Vems2DHistogramWidget::sendInputEvent(int type, int inputEvent)
{

}

bool Vems2DHistogramWidget::isExist(const wxString& symbolname)
{
    return true;
}
int Vems2DHistogramWidget::updateViewerModel()
{
    return 0;
}

double Vems2DHistogramWidget::getValue(const wxString& symbolname)
{
    return 0;
}

void Vems2DHistogramWidget::OnUpdateValue( UpdateViewerEvent& event )
{


    uint16_t phase  = event.m_userData[_T("phase")].GetLong();
    uint16_t ampl   = event.m_userData[_T("ampl")].GetLong();



    if( phase >= SPARKPHASE_BINS ){
        phase = SPARKPHASE_BINS - 1 ;
    }
    if( ampl > 255) return; // should NEVER happen
    if(m_hist == NULL) return;

    int rate = m_hist->m_hisztogram_array[ampl][phase];

   // Logger::logf(LOG_MSG,_T(">>>>Vems2DHistogramWidget::Update ampl=%i,phase=%i, value=%i"),ampl,phase,rate);

    m_y = 9 + AMPLIDUDE_BINS - ampl;
    m_x = 35 + phase; //+10

//    Logger::logf(LOG_MSG,_T("Update rate=%i, locmax=%i, globacc=%i"),rate,m_maxLocalRate,m_hist->GetMaxRate() );
    int maxrate = m_hist->GetMaxRate() ;
    m_pixelColor = colorGradient(rate, maxrate);
    Refresh(false);

}
