#ifndef SIGNALGEN_H
#define SIGNALGEN_H

#include <stdio.h>
#include "../common/typedef.h"

enum sg_codes{
    INVALID_FILENAME,
    CANNOT_OPEN_FILE
};

// 720 degree => 65520 (almost 65536)
#define conv_degree_angle(d) ((d)*91)
#define ARR_MAX_LEN 999

// wav samples per second
// #define WAV_SPS 96000
//#define WAV_SPS 96000

/* maximum: 4000 RPM / second .. TODO: 12000 RPM /sec and GUI-bol : */
//#define DELTARPM_MAXCLIMB (12000.0 / 96000.0 )

#define PI_4 (4.0 * M_PI)

struct tokens_t
{
	uint16_t len;
	uint16_t arr[ARR_MAX_LEN+1];
};


// these were needed for win32 mingw compile, hopefully they don't break linux:

struct wavheader_t {
        char    RiffTag[4];     // 'RIFF'
        int32_t FileLength;
        char    FormatTag[8];   // 'WAVEfmt '
        int32_t FormatLength;   // 16
        int16_t DataFormat;
        int16_t NumChannels;
        int32_t SampleRate;
        int32_t BytesPerSecond;
        int16_t BlockAlignment;
        int16_t SampleDepth;
        char    DataTag[4];     // 'data' DataTag[3] was an error
        int32_t DataLength;
};

// elise
// toothcnt = 32
// missing = 4
struct wheel_param_t {
	uint8_t toothcnt;       // 58 for 60-2 for "m582"; 27 for "c270" (auditrigger), where missing "concatenated" later
	uint8_t missing;        // 1 or 2
};

#define lo8(x)   (x & 0xFF)
#define hi8(x)   ((x >> 8) & 0xFF)
#define PI 3.14159265

class SignalGen
{
    public:
/*
        enum TriggerType{
            SINE,
            C004,
            C008,
            C270,
            M582,
            M351,
            F000,
            E324,
            S060,
            M031,
            M631,
            C001,
            M111,
            C010,
            L008,
            C024,
            M171
        };
*/
        SignalGen();
        virtual ~SignalGen();

        int process_param( int argc, char **argv );
        void usage(char *progname);
        int generate_wav_output( void );
        long generate_trigger_cycle( unsigned char *buf, uint32_t samples, uint16_t rpm, const char *type, const char *secondtrg, char* amplitudex );
        void generateFileName();
        void generateFileName(const char * dir);
        bool openWavFile();
        /// These fvs used by vemTune
        void initBlock( uint16_t currentRpm, double currentAmplitude, int sampleRate );

        void setRpm(int16_t endRpm);
        void setAmplitude(double endAmplitude);
        void setTriggerType(const char * primstr);
//        void setTriggerType(TriggerType type);
        void setdeltaMaxClimb(double deltaMaxClimb);
        void setSecTrigAngle(double d);
        void setFluctuatingScaler(uint8_t f){m_fluctuatingScaler = f;}

        long generateBlock( int16_t *buf, uint32_t samples );

        void mute_prim_signal(bool mute ){
            m_prim_mute = mute;
        }
        void mute_sec_signal(bool mute ){
            m_sec_mute = mute;
        }

    protected:
        uint8_t parse_tokens( struct tokens_t *token_ptr, char *rpm_str, char *delim);
        uint8_t parse_tokens_min2entries( struct tokens_t *token_ptr, char *str, char *delim );
        int16_t onetooth(uint16_t phase);
        int16_t trigger_lotus_elise(struct wheel_param_t *wheel, uint16_t angle, double normaltooth_amplitude);
        int16_t trigger_missing_tooth( struct wheel_param_t *wheel, uint16_t angle, double normaltooth_amplitude );
        int16_t trigger_coil( struct wheel_param_t *wheel, uint16_t angle );
        int16_t trigger_fiat_stilo( uint16_t angle );
        int16_t trigger_simple_cam( uint16_t angle, uint16_t cam_start_pos );
        int16_t trigger_lancia_cam( uint16_t angle, uint16_t cam_start_pos );
        int16_t subaru_vr_signal( uint16_t base, uint16_t angle, uint16_t width );
        // int16_t trigger_subaru_crank( struct wheel_param_t *wheel, uint16_t angle, double amplitude );
        int16_t trigger_subaru_cam ( uint16_t angle, uint16_t cam_start_pos );
        int16_t trigger_subaru36222_cam ( uint16_t angle, uint16_t cam_start_pos );
        int16_t trigger_subaru36222_crank( uint16_t angle );
        int16_t cam_trigger_sample( uint16_t angle, uint16_t cam_start_pos, double amplitude );
        void wavheader_setup( struct wavheader_t *hdr, int length );
        int16_t primary_trigger_sample( uint16_t angle, double amplitude, double normaltooth_amplitude );
        int16_t trigger_oneshot( uint16_t angle, uint16_t cam_start_pos );


        double lookup_and_weight( struct tokens_t *tokens_ptr, double position );


        char type_str[32];
        char *rpm_str, *seconds_str, *campos_str, *amplitude_str, *fading_str;
        #define FILENAMEMAXLEN 4090
        char * filename; //= new char[FILENAMEMAXLEN];

    private:
        uint8_t prim_trig_type;
        uint8_t prim_trig_polarity;	// nonzero means reverse
        struct wheel_param_t wheel;
        //int sectrig_start,sectrig_end;

        struct tokens_t tokens;
        struct tokens_t cam_pos_tokens ;

        struct tokens_t amplitude_tokens ;
        struct tokens_t fading_tokens ;

        int wav_seconds;

        FILE *wavout;

        double    m_currentAngle;
        double    m_currentRpm;
        double    m_currentAmplitude;
        double      m_sampleRate;

        double   m_deltaMaxClimb;
        uint16_t m_expRpm;
        double   m_expAmp;
        double   m_secTriggerAngle;
        int16_t  m_oldcamsample;
        //TriggerType    m_triggerType;
        //char**  m_triggerTypesChar;

        int16_t generateLeftChannel();
        int16_t generateRightChannel();
        uint8_t m_fluctuatingScaler;
        //const char * getTriggerTypeAsChar();

        // mute primary and secondary channel separately
        bool m_prim_mute;
        bool m_sec_mute;


};



#endif // SIGNALGEN_H
