#include "STDataIOMSQ.h"

#include "STDataModelBase.h"
#include "../logger.h"
#include "../SymbolMgr.h"
#include "../vemsprotocol/SIPRDescriptor.h"

STDataIOMSQ::STDataIOMSQ(SymbolMgr* symbolMgr, const SIPRDescriptor*  siprDesc, const wxArrayString& files_in)
{
    m_symbolMgr = symbolMgr;
    m_siprDesc = siprDesc;
    m_filesArray = files_in;

    m_neededSymbols.insert(_T("veTable"));
    m_neededSymbols.insert(_T("rpmBinsVE"));
    m_neededSymbols.insert(_T("mapBinsVE"));

    m_neededSymbols.insert(_T("lambdaTable"));
    m_neededSymbols.insert(_T("rpmBinsLambda"));
    m_neededSymbols.insert(_T("mapBinsLambda"));

    m_neededSymbols.insert(_T("sparkTable"));
    m_neededSymbols.insert(_T("rpmBinsSpark"));
    m_neededSymbols.insert(_T("mapBinsSpark"));

}

STDataIOMSQ::~STDataIOMSQ()
{
}

void STDataIOMSQ::writeAllData()
{
    Logger::logf( LOG_MSG, _T("STDataIOMSQ::writeAllData START") );
    //SymbolMgr& symbolMgr = m_mtconfigData->getSymbolMgr();
    const Int2SymbolArrayHash& page2Symbol = m_symbolMgr->getTheSymbolOrder();
    m_page2SymbolArrayHash = &(page2Symbol);

    makeXmlStructure( m_filesArray[0] );

}
wxString STDataIOMSQ::getSymbolValue(wxString type,int rowSize,int colSize,const wxString& symbolName)
{
    wxString value = _T("");
    double scalar;
    Symbol* symbol = m_symbolMgr->getSymbol(symbolName);
    wxString format = _T("%.") + wxString::Format( _T("%i"), symbol->getDigits() ) + _T("f");
    if(type == _T("SCALAR")){
        m_model->getValue(symbolName,&scalar);
        value = wxString::Format(format, scalar);

    }else if(type == _T("ARRAY")){
        value =_T("\n     ");
        int currRowSize = rowSize;
        if(colSize == 1){
            currRowSize = 1;
        }

        for(int rowCnt = 0; rowCnt < rowSize; rowCnt++){
            for(int colCnt = 0; colCnt <colSize ;colCnt++){
                scalar =   m_model->getViewerData(symbolName, rowCnt, colCnt);
                //int2symbolValueHash[(rowCnt*currRowSize)+colCnt];


                value +=  wxString::Format(format, scalar);
                if(rowSize == 1){
                    if(colCnt < (colSize-1) ) value +=_T("\n     ");
                }else{
                    if(colCnt < (colSize-1) ) value +=_T(" ");
                }
            }
            if( rowCnt == rowSize - 1){
                value +=_T("\n     ");
            }else{
                value +=_T("\n     ");
            }
        }
    }
    return value;
}


void  STDataIOMSQ::makeXmlStructure(const wxString& filname)
{
    wxXmlDocument doc;
    doc.SetFileEncoding(_T("ISO-8859-1"));
    //doc.SetEncoding(_T("ANSI"));
    doc.SetVersion(_T("1.0"));
    wxXmlNode* rootNode;
    wxXmlNode* pageNode;

    rootNode = new wxXmlNode();

    rootNode->SetType(wxXML_ELEMENT_NODE);
    rootNode->SetName(_T("msq"));
    rootNode->AddProperty(_T("xmlns"),_T("http://www.msefi.com/:msq"));

    SymbolArray* symbolArray = NULL;

    int pages = m_siprDesc->getPageCount();
    Logger::logf( LOG_MSG, _T("STDataIOMSQ::makeXmlStructure pages=%i"), pages );
    const IntArray& pagesOrder = m_siprDesc->getPagesOrder();

    for( unsigned int i =0; i < pagesOrder.Count(); i++ )
    {
        /**get the information of page */
        int pageNo = pagesOrder.Item(i);
        wxString pageNumber = wxString::Format(wxT("%d"),(int)pageNo);
        //int size = m_mtconfigData->getPageSize(pageNo);
        //wxString pageSize = wxString::Format(wxT("%d"),(int)size);
        /**creat new node of page and set the information */
        pageNode = new wxXmlNode();
        rootNode->AddChild(pageNode);
        pageNode->SetType(wxXML_ELEMENT_NODE);
        pageNode->SetName(_T("page"));
        pageNode->AddProperty(_T("number"),pageNumber);
        //pageNode->AddProperty(_T("size"),pageSize);

        if(m_page2SymbolArrayHash->find(pageNo) == m_page2SymbolArrayHash->end()) continue;

        Int2SymbolArrayHash::const_iterator IT = m_page2SymbolArrayHash->find(pageNo);
        if(IT != m_page2SymbolArrayHash->end() ){
             symbolArray = IT->second;
            makeSybolNodes(symbolArray,pageNode);
        }

    }

    Logger::logf( LOG_MSG, _T("STDataIOMSQ::makeXmlStructure before save ") );
    doc.SetRoot(rootNode);
    doc.Save(filname);
    Logger::logf( LOG_MSG, _T("STDataIOMSQ::makeXmlStructure after save ") );
}

void STDataIOMSQ::makeSybolNodes(SymbolArray* symbolArray,wxXmlNode* pageNode)
{
     Logger::logf( LOG_MSG, _T("STDataIOMSQ::makeSybolNodes()") );
    wxXmlNode* symbolNode;
    wxXmlNode* symbolValueNode;
    for(uint32_t cnt = 0; cnt < symbolArray->GetCount(); cnt++){
        wxString symbolName = (*symbolArray)[cnt]->getName();
        if( m_neededSymbols.find(symbolName) == m_neededSymbols.end() ){
            //Logger::logf(LOG_MSG,_T("STDataIOMSQ::makeSybolNodes dropped symbol: ")+ symbolName);
            continue;
        }

        symbolNode = new wxXmlNode();
        pageNode->AddChild(symbolNode);
        /** set the symbolnode information*/
        symbolNode->SetType(wxXML_ELEMENT_NODE);
        symbolNode->SetName(_T("constant"));

        symbolNode->AddProperty(_T("name"),symbolName);

        int rowSize =  (*symbolArray)[cnt]-> getShape1();
        if(rowSize > 1){
            wxString rowSizeString =  wxString::Format(wxT("%d"),(int)rowSize);
            symbolNode->AddProperty(_T("rows"),rowSizeString);
        }
        int colSize =   (*symbolArray)[cnt]->  getShape0();
        if(colSize > 1){
            wxString colSizeString =  wxString::Format(wxT("%d"),(int)colSize);
            if(rowSize > 1){
                symbolNode->AddProperty(_T("cols"),colSizeString);
            }else{
                symbolNode->AddProperty(_T("rows"),colSizeString); /// 1 dim is allways row
            }
        }
        wxString unit = (*symbolArray)[cnt]->getUnit();
        if( !(unit.IsEmpty()) ){
            symbolNode->AddProperty(_T("units"),unit);
        }

        int offset =  (*symbolArray)[cnt]->getOffset();
        wxString offsetString = wxString::Format(wxT("%d"),(int)offset);
        symbolNode->AddProperty(_T("offset"),offsetString);

        wxString type;
        type = (*symbolArray)[cnt]->getType();
        /** make valueNode and set*/
        symbolValueNode = new wxXmlNode();
        symbolNode->AddChild(symbolValueNode);
        symbolValueNode->SetType(wxXML_TEXT_NODE);
        wxString value = getSymbolValue(type,rowSize,colSize,symbolName);
        symbolValueNode->SetContent(value);
    }
}
