#include "UserDataLoader.h"
#include "UserDataDesc.h"
#include "STDataModelBase.h"
#include "../SymbolMgr.h"
#include "../Math.h"
#include "../logger.h"


#include <wx/tokenzr.h>


void UserDataLoader::loadToModel( const UserDataDescArray& userDataDescA, SymbolMgr& symbolManager, STDataModelBase* model )
{
	for ( size_t i = 0; i < userDataDescA.GetCount(); ++i )
	{
		UserDataDesc* desc = userDataDescA[i];

		wxString symbolName = desc->symbolName;

		Symbol* symbol = symbolManager.getSymbol( symbolName );

		if ( !symbol ) { continue; }

		int page = symbol->getPage();
		int offset = symbol->getOffset();

		if ( symbol->isArray() )
		{
			int size = symbol->getArraySize();

			wxArrayString userData = wxStringTokenize( desc->symbolData, _T(" ") );

			//Logger::logf( LOG_MSG, _T("UserDataLoader::loadToModel() - ") + symbolName + _T(" - found %d tokens"), userData.GetCount()  );

            for ( size_t j = 0; j < userData.GetCount(); ++j )
            {
                //Logger::logf( LOG_MSG, _T("---UserDataLoader::loadToModel() - idx: %d, token: ") + userData[j], j );

                if ( (int)j >= size )
                {
                    Logger::logf( LOG_WARNING, _T("-----UserDataLoader::loadToModel() - ") + symbolName +
                        _T(" user data size mismatch ( symbol: %d, user: %d )"), size, userData.GetCount() );
                    break;
                }

                uint8_t rawData = Math::hex2int( userData[j] );

                //Logger::logf( LOG_MSG, _T("---UserDataLoader::loadToModel() - page: %d, offset: %d, data: %u"), page, offset + j, rawData );
                model->setRawData( rawData, page, offset + j );
            }
		}
	}
}
