#include "Raw2HighConverter.h"

#include <wx/filename.h>

#include "../../DirHelper.h"
#include "../../RawToHighLogConverter.h"
#include "../../mtconfigdata.h"
#include "../../MTConfigHelper.h"
#include "../../logger.h"
#include "../../RawLogHelper.h"
#include "../../FirmwareInfo.h"

#include "../../rtdatamanager/RTDataCSVWriter.h"
#include "../../rtdatamanager/RTDataMgrFacade.h"
#include "../../rtdatamanager/RTDataInputSIPR.h"

#include "../../stdatamanager/STDataMgrFacade.h"
#include "../../stdatamanager/STDataModelSerial.h"
#include "../../stdatamanager/STDataViewerHash.h"

#include "../../vemsprotocol/VemsLogReader.h"

#include "../../expression/RPNHelper.h"
#include "../../UserLog.h"

#include "../../loghistory/LogHistoryMgr.h"
#include "../../loghistory/LogFilter.h"
#include "../../rtdatastat/RTDataStatHelper.h"


Raw2HighConverter::Raw2HighConverter()
    //: m_iniFileName (wxEmptyString)
    //, m_cond1 (0)
	: m_converter (NULL)
    , m_mtConfigData (NULL)
	, m_isFinished (false)
{
    init();
}

Raw2HighConverter::~Raw2HighConverter()
{
    m_historyConverter->stop();
	delete m_historyConverter;

    m_logHisoryMgr->unregisterLogFilter(m_dataLogWriter);
    delete m_logHisoryMgr;

	delete m_dataLogWriter;

	m_rtDataMgr->UnregisterAll();
	delete m_rtDataMgr;

	delete m_rtInput;

	clearStaticViewers();

	delete m_stController;
	delete m_staticModel;

	delete m_mtConfigData;
	delete m_vemsLogReader;
}

void Raw2HighConverter::init()
{
//    m_logInitDir = DirHelper::GetPath(dLOG, true);

    m_mtConfigData = new MTConfigData();

    initSTModule();
	initRTModule();

}

void Raw2HighConverter::initSTModule()
{
    m_stController = new STDataMgrFacade();
    SymbolTable& symbolTbl = m_mtConfigData->getSymbolTable();

    m_staticModel = new STDataModelSerial( symbolTbl, m_mtConfigData->getSIPRDescriptor() );
    m_stController->setModel( m_staticModel );
    m_stController->setDeviceModel( m_staticModel );
}

 void Raw2HighConverter::initRTModule()
 {
    m_vemsLogReader = new VemsLogReader();
    SymbolMgr* symbolMgr = &(m_mtConfigData->getSymbolMgr());
    m_rtDataMgr = new RTDataMgrFacade();
    m_rtInput = new RTDataInputSIPR(symbolMgr, &m_viewerModel);
    m_rtDataMgr->registerInput(m_rtInput );
    m_rtDataMgr->selectInput( INPUT_TYPE_SIPR );

    m_dataLogWriter = new RTDataCSVWriter(m_mtConfigData->getDataLogEntries() );

    m_rtDataMgr->RegisterViewer(m_dataLogWriter);
    m_dataLogWriter->SetDataMgr(m_rtDataMgr);

    m_logHisoryMgr = new  LogHistoryMgr(m_rtDataMgr, m_stController, symbolMgr);

    /// register progressbar
    //m_logHisoryMgr->registerObserver(m_progBarObWidg);

    m_logHisoryMgr->registerLogFilter(m_dataLogWriter);

    m_historyConverter = new RTDataStatHelper(m_logHisoryMgr);

    /// notify this object if convert finished to start next file
    m_historyConverter->registerObserver(this);
   
	//ProcessBase m_histConverter = historyConverter;
    //Observable  m_histObs = historyConverter;

 }

void Raw2HighConverter::registerStaticViewers()
{
    SymbolMgr symbolMgr = m_mtConfigData->getSymbolMgr();
    RPNHelper::setUsedStatic(symbolMgr);
    SymbolTable& symbolTable = symbolMgr.getUsedStaticAndBit();

    //register scalar viewers to connect RT and ST model

    for( SymbolTable::const_iterator it = symbolTable.begin(); it != symbolTable.end(); ++it )
    {
        Symbol* symbol= it->second;
        STDataViewerHash* viewer = new STDataViewerHash( &m_viewerModel );
        viewer->setSymbol( *symbol );
        //m_stScalarViewerA.Add( viewer );
        m_stController->RegisterViewer( viewer );
    }
}

void Raw2HighConverter::convert( const wxString& inputFile )
{
    wxFileName outputFileName = wxFileName( inputFile );
	outputFileName.SetExt( _T("csv") );
	wxString outputFile = outputFileName.GetFullPath();

    DoConvert( inputFile, outputFile );
}

void Raw2HighConverter::clearStaticViewers()
{
    m_stController->UnregisterAll( true );
}

//void Raw2HighConverter::setInputFile( const wxString& inputFile )
//{
//    m_tListInput->Append(inputFile);
//    m_inputFiles.Add(inputFile);
//
//    wxFileName fileName(inputFile);
//    m_tEditOutputPath->SetValue(fileName.GetPath());
//}
//
//
//void Raw2HighConverter::OnConvert(wxCommandEvent& event)
//{
//
//    bool res = ConvertFirstItem();
//
//}
//
//
//bool Raw2HighConverter::ConvertFirstItem()
//{
//    if(m_inputFiles.GetCount() < 1) return false;
//
//    wxFileName tmp = wxFileName(m_inputFiles.Item(0));
//
//    wxString oname = tmp.GetName();
//
//    wxString fnameend = _T("");
//
//    if(m_tEditOutputExt->GetValue() != wxEmptyString)
//        fnameend = _T(".") + m_tEditOutputExt->GetValue();
//
//
//    return DoConvert(m_inputFiles.Item(0),m_tEditOutputPath->GetValue() + DirHelper::Separ() + oname + fnameend);
//
//
//}
//
//
//void Raw2HighConverter::OnBrwIni(wxCommandEvent& event)
//{
//    m_fileDlg->SetWildcard(_T("Ini files (*.ini)|*.ini|Any files (*.*)|*.*"));
//    m_fileDlg->SetMessage(_T("Choose an Ini file"));
//    m_fileDlg->SetWindowStyle(wxFD_OPEN|wxFD_FILE_MUST_EXIST);
//
//    if(m_fileDlg->ShowModal() != wxID_OK) return;
//
//   // m_iniFileName = m_fileDlg->GetDirectory() + DirHelper::Separ() + m_fileDlg->GetFilename();
//
//    wxString iniFile =  m_fileDlg->GetDirectory() + DirHelper::Separ() + m_fileDlg->GetFilename();
//
//    SetIniCtrlStyle(true);
//
//    m_tEditIni->SetValue(iniFile);
//}
//
//
//void Raw2HighConverter::OnBrwOutput(wxCommandEvent& event)
//{
//
//    m_dirDialog->SetPath(m_tEditOutputPath->GetValue());
//
//    if(m_dirDialog->ShowModal() != wxID_OK) return;
//
//    m_tEditOutputPath->SetValue(m_dirDialog->GetPath());
//
//
//}
//
//void Raw2HighConverter::OnBrwInput(wxCommandEvent& event)
//{
//    m_fileDlg->SetPath(m_logInitDir + DirHelper::Separ());
//
//    m_fileDlg->SetWildcard(_T("Log files (*.vemslog)|*.vemslog;|Any files (*.*)|*.*"));
//    m_fileDlg->SetMessage(_T("Choose a Vems RawLog input file(s)"));
//    m_fileDlg->SetWindowStyle(wxFD_OPEN|wxFD_FILE_MUST_EXIST
//    |wxFD_MULTIPLE
//    );
//
//    if(m_fileDlg->ShowModal() != wxID_OK) return;
//
//    m_logInitDir = m_fileDlg->GetDirectory();
//
//    wxArrayString files;
//
//    m_fileDlg->GetFilenames(files);
//
//    for(uint32_t a=0;a<files.GetCount();a++)
//        m_inputFiles.Add(m_logInitDir + DirHelper::Separ() + files.Item(a));
//
//    m_tListInput->Append(m_inputFiles);
//
//    m_tEditOutputPath->SetValue(m_logInitDir);
//
//
///*
//    wxConfig * v3conf = ConfigHelper::getConfig(1);
//    v3conf->SetPath(_T("/paths"));
//    v3conf->Write(_T("r2hconvinput"),m_logInitDir + DirHelper::Separ());
//    v3conf->Flush();
//    ConfigHelper::save(1);
//*/
//
//
//}
//
//void Raw2HighConverter::OnBrwInputDel(wxCommandEvent& event)
//{
//    m_tListInput->Clear();
//    m_inputFiles.Clear();
//}
//
bool Raw2HighConverter::DoConvert(const wxString &input, const wxString &output)
{
    FirmwareInfo fwInfo;

    if( !ParseHeader( input, fwInfo ) ) return true;

 //   wxString inifile = fwInfo.getIniFile();
 //   if( inifile == wxEmptyString ) {
 //       //UserLog::uLog(ULOG_ERROR, _T("Vemslog 2 csv converter: ini file is not found  for: ")+ input);
 //       //wxMessageBox(_T("Ini file error"),_T("Cannot find the current ini file"));
 //       //TSNotify(TS_EXIT);
 //       return true;
 //   }
 //   //UserLog::uLog(ULOG_INFO, _T("Convert: <b>")+ input+_T("</b> to <b>")+ output+ _T("</b>"));

 //   /// this may take (1-3 sec)
	m_logHisoryMgr->emptyHistory();
    m_logHisoryMgr->loadHistoryFromLog( input );

    m_mtConfigData->clear();
	MTConfigHelper::parseMTConfigData( fwInfo, m_mtConfigData);
 //   
	//
	m_logHisoryMgr->initST(*(m_mtConfigData->getSIPRDescriptor()) );
    //size_t count = m_mtConfigData->getDataLogEntries()->GetCount();
 //   Logger::logf(LOG_MSG,_T("Raw2HighConverter::DoConvert() dataLogEntries size: %i"), count );

 //   //SetIniCtrlStyle(true);
 //   //m_tEditIni->SetValue( inifile );

    m_dataLogWriter->setFileName(output);
    if( !m_dataLogWriter->writeHeaders() ) return false;
 //       return false;
 //   };

 //   Logger::logf(LOG_MSG,_T("Raw2HighConverter::DoConvert() run") );
	registerStaticViewers();
 //   //printDoubleHash();
 //   //m_rtInput->printViewerModel();
 //   Logger::logf(LOG_MSG,_T("Raw2HighConverter::DoConvert() before start") );
 //   //m_historyConverter ->start();

	m_logHisoryMgr->processFilter( 0, m_logHisoryMgr->getHistoryCount() );


 //   Logger::logf(LOG_MSG,_T("Raw2HighConverter::DoConvert() finish") );
 //   //printDoubleHash();
 //   //m_rtInput->printViewerModel();
 //   //clearStaticViewers();
    return true;

}
//
//
//void Raw2HighConverter::startNext()
//{
//    clearStaticViewers();
//
////    m_dataLogWriter->flushBuffers();
//    //Logger::logf(LOG_MSG, _T("m_converter = NULL;\n"));
//    m_converter = NULL;
//    //Logger::logf(LOG_MSG, _T("m_inputFiles.RemoveAt(0);\n"));
//    m_inputFiles.RemoveAt(0);
//    //Logger::logf(LOG_MSG, _T("m_tListInput->Delete(0);\n"));
//    m_tListInput->Delete(0);
//    if(m_inputFiles.GetCount()){
//        ConvertFirstItem();
//        Logger::logf(LOG_MSG, _T("TSNotify ConvertFirstItem();\n"));
//    }
//    else {
//        //Logger::logf(LOG_MSG, _T("m_btnStop->Enable(false);\n"));
//        m_btnStop->Enable(false);
//        //Logger::logf(LOG_MSG, _T("m_btnStop->Enable(false);\n"));
//        m_btnPause->Enable(false);
//        //Logger::logf(LOG_MSG, _T(" m_btnPause->Enable(false);\n"));
//        m_btnConvert->Enable(true);
//        //Logger::logf(LOG_MSG, _T("  if(m_autoCloseCB->GetValue()) Destroy();\n"));
//        if(m_autoCloseCB->GetValue()) Destroy();
//    }
//
//}
//
/// RTDataStatHelper will invoke this when finised: observerNotify(_T("rtdatastatHelperfinished"));
void Raw2HighConverter::notify(const wxString& str)
{
	m_stController->UnregisterAll();
	m_isFinished = true;
}

//void Raw2HighConverter::TSNotify(ThreadState state)
//{
//    //printDoubleHash();
//    //m_rtInput->printViewerModel();
//    clearStaticViewers();
//
//    if(state == TS_EXIT) {
//        startNext();
//    }
//}
//
//void Raw2HighConverter::printDoubleHash()
//{
//    size_t s = m_viewerModel.size();
//    Logger::logf(LOG_MSG, _T("Raw2HighConverter::printDoubleHash() size: %d"), s);
//    DoubleHash::const_iterator IT =  m_viewerModel.begin();
//    for(; IT != m_viewerModel.end();++IT ){
//        wxString key = IT->first;
//        double value = IT->second;
//        Logger::logf(LOG_MSG, _T("Raw2HighConverter::printDoubleHash(): ")+key+_T("=%f"), value);
//    }
//}
//
bool Raw2HighConverter::ParseHeader(const wxString& filename, FirmwareInfo &fwInfo )
{
    //m_vemsLogReader = new RTDataInputRawLog( &(m_mtConfigData->getSymbolMgr()), m_viewerModel );

    if( m_vemsLogReader->SetInputFile( filename ) < 0 ) {
        //delete m_vemsLogReader;
        return false;
    }

    if( m_vemsLogReader->ParseHeader() < 0 ) {
        //delete m_vemsLogReader;
        return false;
    }

    //((RTDataMgrFacade*)m_rtDataMgr)->SetInput(m_vemsLogReader);

    const StringHash& headerHash = m_vemsLogReader->getHeaderHash();
    RawLogHelper::fillFirmwareInfo(fwInfo, headerHash);

    return true;
}

