#include "wx/defs.h"
#include <wx/filename.h>

#include "../logger.h"
#include "../DirHelper.h"

#include "../Symbol.h"
#include "../mtconfigdata.h"
#include "../fileparser/V3IniParser.h"

#include "../firmwareconfig/FirmwareConfigData.h"
#include "../fileparser/FirmwareConfigParser.h"

#include "../fileparser/ParseHeader.h"

#include "../MTConfigHelper.h"
#include "../stdatamanager/STDataModelSerial.h"
#include "../firmwareconfig/FirmwareConfigLoader.h"
#include "../confighelper.h"

#include "../FirmwareInfo.h"
#include "../vemsprotocol/VemsLogReader.h"
#include "../RawLogHelper.h"


void testMTConfigData()
{
	const wxString filename = _T("vemsTune-v3-1.1.56.ini");
	const wxString iniFilePath = DirHelper::GetPath( dCONFIG ) + filename ;

	MTConfigData config;
	V3iniParser parser( &config, NULL );
	
	parser.parse( iniFilePath );
}


void testMTConfigHelper()
{
	//ConfigHelper::initConfig(CONFIG_TYPE_GUI);

	const wxString logFileName	= _T("v3.2_n000315-2009.09.05-15.32.52.vemslog");
	const wxString logFileDir	= DirHelper::GetPath( dDEBUGLOG ) + _T("CSVConverterTest") + DirHelper::Separ();
	wxString logFileFullPath = logFileDir + logFileName;
	
	VemsLogReader reader;
	reader.SetInputFile( logFileFullPath );
	reader.ParseHeader();

	FirmwareInfo info;
    const StringHash& headerHash = reader.getHeaderHash();
    RawLogHelper::fillFirmwareInfo( info, headerHash );

	MTConfigData config;
	MTConfigHelper::parseMTConfigData( info, &config );
}

void testVemsLogReader( )
{
	const wxString filename = _T("v3.2_n000315-2009.09.05-15.32.52.vemslog");
	const wxString dir = DirHelper::GetPath( dDEBUGLOG ) + _T("CSVConverterTest") + DirHelper::Separ();
	wxString fullPath = dir + filename;

	VemsLogReader reader;
	reader.SetInputFile( fullPath );
	reader.ParseHeader();
}


void testFirmwareConfigParser()
{
	const wxString filename = _T("v3.2_n000315-2009.10.21-20.41.34.vemscfg");
	const wxString iniFilePath = DirHelper::GetPath( dDEBUGLOG ) + filename ;

	FirmwareConfigData config;
	FirmwareConfigParser parser( &config );
	parser.parse( iniFilePath );

	wxString tableData = config.tableDataToString();
	Logger::logf( LOG_MSG, tableData );
	wxString scalarData = config.scalarDataToString();
	Logger::logf( LOG_MSG, scalarData );

}

void testHeaderParser()
{
	const wxString filename		= _T("global.h");
	const wxString firmwareDir	= _T("firmware_1.1.63");
	const wxString filePath = DirHelper::GetPath( dPROJECTS ) + firmwareDir + DirHelper::Separ() + filename;

	SymbolTable symbols;
	ParseHeader parser( symbols );
	parser.parse( filePath );
	
	Logger::logf( LOG_MSG, _T("testHeaderParser() - parse: ") + filePath );
	Logger::logf( LOG_MSG, _T("testHeaderParser() - parsed symbol count: %d"), symbols.size() );
	
	WX_CLEAR_HASH_MAP( SymbolTable, symbols );
}

void testFirmwareConfigLoader()
{
	//MTConfigHelper::createMTConfigData();
	//ConfigHelper::initConfig(CONFIG_TYPE_GUI);

	const wxString filename = _T("v3.2_n000315-2009.10.05-19.35.25.vemscfg");
	const wxString iniFilePath = DirHelper::GetPath( dDEBUGLOG ) + _T("ConfigLoaderTest") + DirHelper::Separ() + filename;

	wxArrayString filenames;
	filenames.Add( iniFilePath );

	SymbolTable symbols;
	STDataModelSerial  serialModel( symbols );

	MTConfigData configData;
	FirmwareInfo aFirmwareInfo;
	aFirmwareInfo.setFirmwareVersion( _T("1.1.61") );
	FirmwareConfigLoader configLoader;
	
	configLoader.setDestinationFwInfo( &aFirmwareInfo );
	configLoader.setMTConfigData( &configData );

	configLoader.parse( filenames );
	configLoader.load( &serialModel );

	WX_CLEAR_HASH_MAP( SymbolTable, symbols );

	ConfigHelper::cleanup();
}

int main( int argc, char **argv )
{
    wxFileName tmp = wxFileName( wxString::FromAscii( argv[0] ) );
    DirHelper::SetProgramRootDir( tmp.GetPath() );
	
	wxString targetFileFullPath = DirHelper::GetPath(dDEBUGLOG) + _T("filelog.txt");
	Logger::initLog( targetFileFullPath );
    
	ConfigHelper::initConfig(CONFIG_TYPE_GUI);

	//testFirmwareConfigParser();
	//testHeaderParser();
	//testMTConfigData();
	//testFirmwareConfigLoader();
	//testVemsLogReader();
	testMTConfigHelper();

	Logger::logf( LOG_MSG, _T("Test finished! Press ENTER to exit!") );

	getchar();

	wxString logFile = DirHelper::GetPath( dDEBUGLOG ) + _T("filelog.txt");

	::wxExecute( _T("notepad++.exe ") + logFile );

	ConfigHelper::cleanup();
	Logger::cleanup();

	return 0;
}

