#include "wx/defs.h"
#include <wx/filename.h>

#include "../../logger.h"
#include "../../DirHelper.h"

#include "../../vemsprotocol/ECUInfo.h"
#include "../../vemsprotocol/ECUState.h"
#include "../../vemsprotocol/ECUStateMgr.h"
#include "../../vemsprotocol/ECUStateGraph.h"
#include "../../serialconfig.h"
#include "../../serialporthandler.h"

struct TestRoute{
    int from;
    int to;

    TestRoute( int from, int to )
    {
        this->from = from;
        this->to = to;
    }
};

WX_DEFINE_ARRAY( TestRoute*, TestRouteArray );

void fillTestRoutes( TestRouteArray& testRoutes )
{
    testRoutes.Add( new TestRoute( ECUInfo::BOOT , ECUInfo::MEGATUNE ) );
    testRoutes.Add( new TestRoute( ECUInfo::BOOT , ECUInfo::MAN ) );
    testRoutes.Add( new TestRoute( ECUInfo::BOOT , ECUInfo::AIM ) );

    testRoutes.Add( new TestRoute( ECUInfo::MEGATUNE , ECUInfo::BOOT ) );
    testRoutes.Add( new TestRoute( ECUInfo::MEGATUNE , ECUInfo::MAN ) );
    testRoutes.Add( new TestRoute( ECUInfo::MEGATUNE , ECUInfo::AIM ) );

    testRoutes.Add( new TestRoute( ECUInfo::MAN , ECUInfo::BOOT ) );
    testRoutes.Add( new TestRoute( ECUInfo::MAN , ECUInfo::MEGATUNE ) );
    testRoutes.Add( new TestRoute( ECUInfo::MAN , ECUInfo::AIM ) );

    testRoutes.Add( new TestRoute( ECUInfo::AIM , ECUInfo::BOOT ) );
    testRoutes.Add( new TestRoute( ECUInfo::AIM , ECUInfo::MEGATUNE ) );
    testRoutes.Add( new TestRoute( ECUInfo::AIM , ECUInfo::MAN ) );
}


void runSearchRouteTest( const TestRouteArray& testRoutes )
{
    ECUStateMgr stateMgr( NULL );

    for( size_t i = 0; i < testRoutes.Count(); ++i )
    {
        int from = testRoutes.Item( i )->from;
        int to   = testRoutes.Item( i )->to;

        Logger::logf( LOG_MSG, _T("testRoutes() - %i - find route from %s to %s")
                    ,i
                    ,ECUInfo::getStateAsString( from ).c_str()
                    ,ECUInfo::getStateAsString( to ).c_str() );

        stateMgr.setState( from, to );
    }
}

void runChangeStatesTest( const TestRouteArray& testRoutes )
{
    SerialConfig ecuConfig;
    ecuConfig.setPortName( _T("COM1") );
    ecuConfig.setBaudRate( 9600 );

    SerialPortHandler ecuPortHandler;
    ecuPortHandler.setSerialConfig( &ecuConfig );

    ECUStateMgr stateMgr( &ecuPortHandler );

    for( size_t i = 0; i < testRoutes.Count(); ++i )
    {
        int from = testRoutes.Item( i )->from;
        int to   = testRoutes.Item( i )->to;
        wxString fromStr = ECUInfo::getStateAsString( from );
        wxString toStr   = ECUInfo::getStateAsString( to );

        Logger::logf( LOG_MSG, _T("runChangeStatesTest - %i"), i );
        Logger::logf( LOG_MSG, _T(" \t change state from %s to %s"), fromStr.c_str(), toStr.c_str() );

        //int state = stateMgr.getState();
        //Logger::logf( LOG_MSG, _T(" \t current state: ")+ ECUInfo::getStateAsString( state ) );
        stateMgr.setState( from );

        int dest = stateMgr.setState( to );
        Logger::logf( LOG_MSG, _T(" \t dest state: ")+ ECUInfo::getStateAsString( dest ) );
        if(  dest != to )
        {
            Logger::logf( LOG_MSG, _T("\t FAILURE") );
            Logger::logf( LOG_MSG, _T("\t reach state %s"),  ECUInfo::getStateAsString( dest ).c_str() );

            continue;
        }

        Logger::logf( LOG_MSG, _T("\t SUCCES") );

    }
}

void runDeviceTest()
{
    SerialConfig ecuConfig;
    ecuConfig.setPortName( _T("COM1") );
    ecuConfig.setBaudRate( 9600 );

    SerialPortHandler ecuPortHandler;
    ecuPortHandler.setSerialConfig( &ecuConfig );

    ECUStateMgr stateMgr( &ecuPortHandler );

    int result = stateMgr.getState();
    int device = stateMgr.getDevice();

    Logger::logf( LOG_MSG, _T("Detected device : ") + ECUInfo::getDeviceAsString( device ) );
    Logger::logf( LOG_MSG, _T("Detected state : ") + ECUInfo::getStateAsString( result ) );
}

int main( int argc, char **argv )
{
    wxFileName tmp = wxFileName( wxString::FromAscii( argv[0] ) );
    DirHelper::SetProgramRootDir( tmp.GetPath() );

    TestRouteArray routes;

    fillTestRoutes( routes );

    try{
        //runSearchRouteTest( routes );
        //runChangeStatesTest( routes );
        runDeviceTest();
    }catch( SerialException& e ){
        Logger::logf( LOG_MSG, e.toString() );
    }

    return 0;
}

