
/** \class TbTbECUState
  *
  * \~english
  * \brief Show the ECU state.
  * \details
  *
  *
  * \~hungarian
  * \brief Az ECU állapotát mutaja.
  * \details
  *
  *
  */
#include "TbTbECUState.h"
#include "../logger.h"
#include "../DirHelper.h"
#include "../events/VemsDeviceControllerEvent.h"

enum ToolBitmapIds
{
    TBI_ECU_CONN,
    TBI_ECU_NC,
    TBI_ECU_OFFLINE,
    TBI_ECU_AIM,
    TBI_ECU_BOOT,
    TBI_ECU_RT,
    TBI_ECU_LOCK
};

IMPLEMENT_DYNAMIC_CLASS(TbTbECUState,VemsToolBarToolBase)


BEGIN_EVENT_TABLE(TbTbECUState,VemsToolBarToolBase)
    EVT_VEMS_DEVICE_CONTROLLER( VDC_EVENT_CANNOT_OPEN_PORT, TbTbECUState::OnCannotOpenPort )
    EVT_VEMS_DEVICE_CONTROLLER( VDC_EVENT_DEVICE_NOT_FOUND, TbTbECUState::OnDeviceNotFound )
    EVT_VEMS_DEVICE_CONTROLLER( VDC_EVENT_DEVICE_FOUND, TbTbECUState::OnDeviceConnected )
    EVT_VEMS_DEVICE_CONTROLLER( VDC_EVENT_DEVICE_CONNECTED, TbTbECUState::OnDeviceConnected )
    EVT_VEMS_DEVICE_CONTROLLER( VDC_EVENT_DEVICE_DISCONNECTED, TbTbECUState::OnDeviceDisconnected )
    EVT_VEMS_DEVICE_CONTROLLER( VDC_EVENT_DEVICE_RECONNECTED, TbTbECUState::OnDeviceConnected )
    EVT_VEMS_DEVICE_CONTROLLER( VDC_EVENT_STOPPED, TbTbECUState::OnDeviceControllerStopped )
    EVT_VEMS_DEVICE_CONTROLLER( VDC_EVENT_RESUMED, TbTbECUState::OnDeviceResumed )
    EVT_VEMS_DEVICE_CONTROLLER( VDC_EVENT_DEVICE_PROTECTED, TbTbECUState::OnDeviceProtected )
END_EVENT_TABLE()


TbTbECUState::TbTbECUState(int id) : VemsToolBarToolBase(id)
{
    m_bmpKeyListArray[TBI_ECU_NC].Add( DirHelper::GetPath(dICON) + _T("tbEcuState48-false.png") );
    m_bmpKeyListArray[TBI_ECU_NC].Add( DirHelper::GetPath(dICON) + _T("tbEcuState-false.png") );

    m_bmpKeyListArray[TBI_ECU_CONN].Add( DirHelper::GetPath(dICON) + _T("tbEcuState48-true.png") );
    m_bmpKeyListArray[TBI_ECU_CONN].Add( DirHelper::GetPath(dICON) + _T("tbEcuState-true.png") );

    m_bmpKeyListArray[TBI_ECU_OFFLINE].Add( DirHelper::GetPath(dICON) + _T("tbEcuState48-offline.png") );
    m_bmpKeyListArray[TBI_ECU_OFFLINE].Add( DirHelper::GetPath(dICON) + _T("tbEcuState-offline.png") );

    m_bmpKeyListArray[TBI_ECU_AIM].Add( DirHelper::GetPath(dICON) + _T("tbEcuState48-aim.png") );

    m_bmpKeyListArray[TBI_ECU_RT].Add( DirHelper::GetPath(dICON) + _T("tbEcuState48-rt.png") );

    m_bmpKeyListArray[TBI_ECU_BOOT].Add( DirHelper::GetPath(dICON) + _T("tbEcuState48-boot.png") );

    m_bmpKeyListArray[TBI_ECU_LOCK].Add( DirHelper::GetPath(dICON) + _T("tbEcuState48-locked.png") );
}

TbTbECUState::~TbTbECUState()
{
}

void TbTbECUState::OnTool(wxCommandEvent &event)
{
}

void TbTbECUState::Init()
{
    InitBmpRes();

    m_shortHelpString = _T("EcuState");
    m_longHelpString  = _T("EcuState");

    m_defaultBitmap = m_bmpHash[TBI_ECU_OFFLINE];
}

bool TbTbECUState::SendObject(int id, wxObject *object, const wxString &message)
{
    return true;
}

void TbTbECUState::OnCannotOpenPort( VemsDeviceControllerEvent& event )
{
    SetToolBmp(TBI_ECU_OFFLINE);
//    m_shortHelpString = _T("ECU Disabled!");
//    InitHelpStrings();
}

void TbTbECUState::OnDeviceNotFound( VemsDeviceControllerEvent& event )
{
    SetToolBmp(TBI_ECU_NC);
//    m_shortHelpString = _T("Device: not found");
//    InitHelpStrings();
}

void TbTbECUState::OnDeviceConnected( VemsDeviceControllerEvent& event )
{
    switch( event.getDeviceMode() )
    {
        case ECUInfo::BOOT:
            SetToolBmp(m_bmpHash[TBI_ECU_BOOT]);
//            m_shortHelpString = _T("Device: V3");
            break;

/// TODO         TBI_ECU_AIM

        default:
            SetToolBmp(m_bmpHash[TBI_ECU_RT]);
//            m_shortHelpString = _T("Device: Round");
            break;
    }

//    InitHelpStrings();
}

void TbTbECUState::OnDeviceDisconnected( VemsDeviceControllerEvent& event )
{
    SetToolBmp(TBI_ECU_NC);
//    m_shortHelpString = _T("Device: not found");
//    InitHelpStrings();
}

void TbTbECUState::OnDeviceControllerStopped( VemsDeviceControllerEvent& event )
{
    SetToolBmp(TBI_ECU_OFFLINE);
//    m_shortHelpString = _T("ECU Disabled!");
//    InitHelpStrings();
}

void TbTbECUState::OnDeviceResumed( VemsDeviceControllerEvent& event )
{
    switch( event.getDeviceMode() )
    {
        case ECUInfo::BOOT:
            SetToolBmp(m_bmpHash[TBI_ECU_BOOT]);
//            m_shortHelpString = _T("Device: V3");
            break;

/// TODO         TBI_ECU_AIM

        default:
            SetToolBmp(m_bmpHash[TBI_ECU_RT]);
//            m_shortHelpString = _T("Device: Round");
            break;
    }

//    InitHelpStrings();
}

void TbTbECUState::OnDeviceProtected( VemsDeviceControllerEvent& event )
{
    SetToolBmp(m_bmpHash[TBI_ECU_LOCK]);
}
