#include "TbTbEditMode.h"

#include "../logger.h"
#include "../DirHelper.h"
#include "../vemsgauge/GaugeGroupManager.h"

IMPLEMENT_DYNAMIC_CLASS(TbTbEditMode,VemsToolBarToolBase)

enum ToolBitmapIds
{

    TBI_EDITMODE_ON,
    TBI_EDITMODE_ONB,
    TBI_EDITMODE_OFF
};


TbTbEditMode::TbTbEditMode(int id)
    :VemsToolBarToolBase(id)
    ,m_gaugeGroupManager(NULL)
    ,m_popupMenu(NULL)
    ,m_timer(this)
{
    //ctor
    m_bmpKeyListArray[TBI_EDITMODE_ON].Add( DirHelper::GetPath(dICON) + _T("tbEditMode48-on.png") );
    m_bmpKeyListArray[TBI_EDITMODE_ONB].Add( DirHelper::GetPath(dICON) + _T("tbEditMode48-on-w.png") );
    m_bmpKeyListArray[TBI_EDITMODE_OFF].Add( DirHelper::GetPath(dICON) + _T("tbEditMode48-off.png") );



    Connect(wxEVT_TIMER, wxTimerEventHandler(TbTbEditMode::OnTimer));
}

TbTbEditMode::~TbTbEditMode()
{
    //dtor
}
/** @brief OnTool
  *
  * @todo: document this function
  */
void TbTbEditMode::OnTool(wxCommandEvent &event)
{
    if(!m_gaugeGroupManager) return;

    bool nmode = !(m_gaugeGroupManager->IsEditMode());

    if( nmode ) {
        m_timer.Start(1000);
        SetToolBmp( TBI_EDITMODE_ON );
    } else {
        m_timer.Stop();
        SetToolBmp( TBI_EDITMODE_OFF );
    }


    m_gaugeGroupManager->SetEditMode( nmode );
}

void TbTbEditMode::OnTimer(wxTimerEvent& evt)
{
    if(m_ToolBmpId == TBI_EDITMODE_ON)
        SetToolBmp(TBI_EDITMODE_ONB);
    else
        SetToolBmp(TBI_EDITMODE_ON);
}

/** @brief OnToolR
  *
  * @todo: document this function
  */
void TbTbEditMode::OnToolR(wxCommandEvent &event)
{
    if(m_gaugeGroupManager && m_popupMenu && m_gaugeGroupManager->IsEditMode())
        m_toolBar->PopupMenu(m_popupMenu);
}

/** @brief OnToolEnter
  *
  * @todo: document this function
  */
void TbTbEditMode::OnToolEnter(wxCommandEvent &event)
{

}

/** @brief Init
  *
  * @todo: document this function
  */
void TbTbEditMode::Init()
{
    InitBmpRes();

    m_defaultBitmap = m_bmpHash[TBI_EDITMODE_OFF];

    AddRequest(TN_GaugeGroupManager);

    m_shortHelpString = _T("Edit mode");
    m_longHelpString  = _T("On/Off Edit mode");


    m_popupMenu = new wxMenu();
    idMenuSave = wxNewId();
    idMenuSaveAndSwitch = wxNewId();
    idMenuUndo = wxNewId();

    m_popupMenu->Append( idMenuSave, _T("Save gauges layout") );
    m_popupMenu->Append( idMenuSaveAndSwitch, _T("Save gauges layout and switch to View mode") );
    //m_popupMenu->Append( idMenuUndo, _T("undo") );

    m_toolBar->Connect( idMenuSave,  wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( TbTbEditMode::OnMenu ), NULL, this  );
    m_toolBar->Connect( idMenuSaveAndSwitch,  wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( TbTbEditMode::OnMenu ), NULL, this  );

}

void TbTbEditMode::OnMenu(wxCommandEvent &evt)
{
    int id = evt.GetId();

    if( id == idMenuSave ) {

        if( !m_gaugeGroupManager ) return;

        m_gaugeGroupManager->saveAllGroup();

    } else if( id == idMenuSaveAndSwitch ) {

        m_timer.Stop();

        if( !m_gaugeGroupManager ) return;

        m_gaugeGroupManager->saveAllGroup();

        SetToolBmp( TBI_EDITMODE_OFF );
        m_gaugeGroupManager->SetEditMode( false );

    }

}

/** @brief SendObject
  *
  * @todo: document this function
  */
bool TbTbEditMode::SendObject(int id, wxObject *object, const wxString &message)
{
    switch(id) {

        case TN_GaugeGroupManager:
            m_gaugeGroupManager = (GaugeGroupManager*)object;

            break;

        default:
            break;
    }
    return true;
}

