/** \class TbTbIsLogging
  *
  * \~english
  * \brief Set the log and show state
  * \details
  *
  *
  * \~hungarian
  * \brief Beállítható a loggolás és mutatja az állapotát
  * \details
  *
  *
  */
#include "TbTbIsLogging.h"
#include "../logger.h"
#include "../DirHelper.h"
#include "../VemsTuneController.h"
#include "../LogDirManager.h"

enum ToolBitmapIds
{
    TBI_LOG_0,
    TBI_LOG_1
};

IMPLEMENT_DYNAMIC_CLASS(TbTbIsLogging,VemsToolBarToolBase)


TbTbIsLogging::TbTbIsLogging(int id)
             :VemsToolBarToolBase(id)
             ,m_VemsTuneController (NULL)
{
    m_bmpKeyListArray[TBI_LOG_0].Add( DirHelper::GetPath(dICON) + _T("tbIsLogging48-false.png") );
    m_bmpKeyListArray[TBI_LOG_0].Add( DirHelper::GetPath(dICON) + _T("tbIsLogging-false.png") );

    m_bmpKeyListArray[TBI_LOG_1].Add( DirHelper::GetPath(dICON) + _T("tbIsLogging48-true.png") );
    m_bmpKeyListArray[TBI_LOG_1].Add( DirHelper::GetPath(dICON) + _T("tbIsLogging-true.png") );

}

TbTbIsLogging::~TbTbIsLogging()
{
    if ( m_VemsTuneController ) {
        m_VemsTuneController->Disconnect( wxEVT_ECU_LOG, ECULogEventHandler( TbTbIsLogging::OnECULogEvent ),NULL, this);
    }
}

void TbTbIsLogging::OnTool(wxCommandEvent &event)
{
    if(m_VemsTuneController == NULL){
        Logger::logf(LOG_MSG,_T("TbTbIsLogging::OnTool m_VemsTuneController is NULL"));
        return;
    }

    m_VemsTuneController->setECULog(!m_VemsTuneController->isECULog());
}

void TbTbIsLogging::OnToolR(wxCommandEvent &event)
{
    wxDialog dlg(NULL, wxID_ANY, _T("Information about logging"), wxDefaultPosition, wxDefaultSize);
    wxString txt = _T("Current working directory is ") + LogDirManager::GetDefaultWorkDir() +
        _T("\n\nIn File menu you can change it. VEMS logfiles are sorted into different directories based on the used ECUs");

    wxBoxSizer sizer(wxVERTICAL);
    wxTextCtrl text(&dlg, wxID_ANY, txt, wxDefaultPosition, wxSize(300,120), wxTE_MULTILINE | wxTE_READONLY );
    sizer.Add(&text, 1, wxEXPAND | wxALL, 10);
    dlg.SetSizer(&sizer);
	sizer.Fit( &dlg );
	sizer.SetSizeHints(&dlg );
    dlg.ShowModal();
}

void TbTbIsLogging::OnToolEnter(wxCommandEvent &event) {}

void TbTbIsLogging::OnECULogEvent(ECULogEvent& event)
{
    if ( event.state == ECULogEvent::ON ) {
        m_shortHelpString = _T("Logging is On");
        SetToolBmp(m_bmpHash[TBI_LOG_1]);
    } else {
        m_shortHelpString = _T("Logging is Off");
        SetToolBmp(m_bmpHash[TBI_LOG_0]);
    }

    InitHelpStrings();
}

void TbTbIsLogging::Init()
{

    InitBmpRes();

    m_defaultBitmap = m_bmpHash[TBI_LOG_0];
    AddRequest(TN_VemsController);

    m_shortHelpString = _T("Logging is Off");
    m_longHelpString  = _T("On/Off the raw log");
}


bool TbTbIsLogging::SendObject(int id, wxObject *object, const wxString &message)
{

    switch(id) {
        case TN_VemsController:
                m_VemsTuneController = dynamic_cast<VemsTuneController*> (object);
                if(!m_VemsTuneController){
                    Logger::logf(LOG_MSG,_T("TbTbIsLogging::SendObject() dynamic_cast failed!"));
                    return false;
                }
                m_VemsTuneController->Connect( wxEVT_ECU_LOG, ECULogEventHandler( TbTbIsLogging::OnECULogEvent ),NULL, this);
            break;

        default:
            break;
    }
    return true;
}
