#include "TbTbLogPlaySpeed.h"

#include "../loghistory/FilePlayCommander.h"
#include "../DirHelper.h"
#include "../logger.h"

enum ToolBitmapIds
{
    TBI_LOGPLAY_INACTIVE,
    TBI_LOGPLAY_ACTIVE_1,
    TBI_LOGPLAY_ACTIVE_2,
    TBI_LOGPLAY_ACTIVE_4,
};

IMPLEMENT_DYNAMIC_CLASS( TbTbLogPlaySpeed, VemsToolBarToolBase );

TbTbLogPlaySpeed::TbTbLogPlaySpeed( int id ) :
        VemsToolBarToolBase( id ),
        m_fpc ( NULL )
{
    m_bmpKeyListArray[TBI_LOGPLAY_INACTIVE].Add( DirHelper::GetPath(dICON) + _T("tbPlaySpeedInactive48.png"));
    m_bmpKeyListArray[TBI_LOGPLAY_ACTIVE_1].Add( DirHelper::GetPath(dICON) + _T("tbPlaySpeed148.png"));
    m_bmpKeyListArray[TBI_LOGPLAY_ACTIVE_2].Add( DirHelper::GetPath(dICON) + _T("tbPlaySpeed248.png"));
    m_bmpKeyListArray[TBI_LOGPLAY_ACTIVE_4].Add( DirHelper::GetPath(dICON) + _T("tbPlaySpeed448.png"));

    m_currentSpeed = 1;
}

TbTbLogPlaySpeed::~TbTbLogPlaySpeed()
{
    if ( m_fpc )
    {
        m_fpc->unregisterTool( this );
    }
}

void TbTbLogPlaySpeed::OnTool( wxCommandEvent &event )
{
    if ( m_fpc )
    {
        switch ( m_currentSpeed )
        {
            case 1: SetToolBmp( m_bmpHash[TBI_LOGPLAY_ACTIVE_2] );
                    m_shortHelpString = _T("Playing with speed x2");
                    m_currentSpeed = 2;
                    break;
            case 2: SetToolBmp( m_bmpHash[TBI_LOGPLAY_ACTIVE_4] );
                    m_shortHelpString = _T("Playing with speed x4");
                    m_currentSpeed = 4;
                    break;
            case 4: SetToolBmp( m_bmpHash[TBI_LOGPLAY_ACTIVE_1] );
                    m_shortHelpString = _T("Playing with speed x1");
                    m_currentSpeed = 1;
                    break;
            default:
                    break;

        }

        m_fpc->setReplaySpeed( m_currentSpeed );
    }

    InitHelpStrings();
}


void TbTbLogPlaySpeed::Init()
{
    InitBmpRes();

    m_defaultBitmap = m_bmpHash[TBI_LOGPLAY_ACTIVE_1];
    m_inactiveBitmap = m_bmpHash[TBI_LOGPLAY_INACTIVE];

    m_shortHelpString = _T("Replay log speed");
    m_longHelpString  = _T("Replay log speed");

    AddRequest( TN_FilePlayCommander );
}

void TbTbLogPlaySpeed::OnToolR( wxCommandEvent &event )
{
}

bool TbTbLogPlaySpeed::SendObject( int id, wxObject *object, const wxString &message )
{
    switch( id )
    {
        case TN_FilePlayCommander:
            m_fpc = dynamic_cast<FilePlayCommander*>(object);

            if ( !m_fpc )
            {
                LOGP(_T("Cannot cast to FilePlayCommander"))
                return false;
            }

            m_fpc->registerTool( _T("PlaySpeed"), this );

            m_toolBar->EnableTool( m_toolID, false );

            SetToolBmp( m_bmpHash[ TBI_LOGPLAY_ACTIVE_1 ] );

            /*switch ( m_fpc->getReplaySpeed() )
            {
                case 1: SetToolBmp( m_bmpHash[ TBI_LOGPLAY_ACTIVE_1 ] );
                        break;

                case 2: SetToolBmp( m_bmpHash[ TBI_LOGPLAY_ACTIVE_2 ] );
                        break;

                case 4: SetToolBmp( m_bmpHash[ TBI_LOGPLAY_ACTIVE_4 ] );
                        break;

                default: SetToolBmp( m_bmpHash[ TBI_LOGPLAY_ACTIVE_1 ] );
                        break;
            }*/

            break;

        default:
            break;
    }

    return true;
}
