#include "TbTbPlay.h"

#include "../DirHelper.h"

#include "../loghistory/FilePlayCommander.h"

#include "../logger.h"


/** \class TbTbPlay
  *
  * \~english
  * \brief Play or pause the log replay.
  * \details
  *
  *
  *
  *
  * \~hungarian
  * \brief Elindítja vagy pillanatáljj-olja a log visszajátszást.
  * \details
  *
  *
  *
  *
  */



IMPLEMENT_DYNAMIC_CLASS(TbTbPlay,VemsToolBarToolBase)

TbTbPlay::TbTbPlay(int id)
        : VemsToolBarToolBase(id)
        , m_fpc ( NULL )
{
    m_bmpKeyListArray[TBI_PLAY_ACTIVE].Add( DirHelper::GetPath(dICON) + _T("tbPlayActive48.png"));
    m_bmpKeyListArray[TBI_PLAY_INACTIVE].Add( DirHelper::GetPath(dICON) + _T("tbPlayInactive48.png"));

    m_bmpKeyListArray[TBI_PAUSE_ACTIVE].Add( DirHelper::GetPath(dICON) + _T("tbPauseActive48.png"));
    m_bmpKeyListArray[TBI_PAUSE_INACTIVE].Add( DirHelper::GetPath(dICON) + _T("tbPauseInactive48.png"));
}

TbTbPlay::~TbTbPlay()
{
    if ( m_fpc )
    {
        m_fpc->unregisterTool( this );
    }
}

void TbTbPlay::OnTool(wxCommandEvent &event)
{
    if ( m_fpc )
    {
        if ( m_fpc->IsPlaying() )
        {
            SetToolBmp(m_bmpHash[ TBI_PLAY_ACTIVE ]);
            m_shortHelpString = _T("Play");

            m_fpc->Pause();
        }
        else
        {
            SetToolBmp(m_bmpHash[ TBI_PAUSE_ACTIVE ]);
            m_shortHelpString = _T("Pause");

            m_fpc->Play();
        }
    }

    InitHelpStrings();
}

void TbTbPlay::Init()
{
    InitBmpRes();

    m_defaultBitmap = m_bmpHash[TBI_PLAY_ACTIVE];
    m_inactiveBitmap = m_bmpHash[TBI_PLAY_INACTIVE];

    m_shortHelpString = _T("Pause");
    m_longHelpString  = _T("On/Off play log");

    AddRequest(TN_FilePlayCommander);

}

bool TbTbPlay::SendObject(int id, wxObject *object, const wxString &message)
{
    switch( id )
    {
        case TN_FilePlayCommander:
            m_fpc = dynamic_cast<FilePlayCommander*>(object);

            if ( !m_fpc )
            {
                LOGP(_T("Cannot cast to FilePlayCommander"))
                return false;
            }

            m_fpc->registerTool(_T("Play"), this);

            m_toolBar->EnableTool(m_toolID, false);

            SetToolBmp(m_bmpHash[ TBI_PLAY_ACTIVE  ]);

            break;

        default:
            break;
    }

    return true;
}


