#ifndef TRIGGERLOGGRAPH_H
#define TRIGGERLOGGRAPH_H

#include <wx/wx.h>
#include <wx/spinctrl.h>

#include "../common/types/DoubleArray.h"
#include "../common/types/Int2DoubleArrayHash.h"
#include "../common/types/wxColourArray.h"
#include "../events/TriggerLogEvent.h"
#include "../events/CommunicationEvent.h"
#include "../ObserverBase.h"
#include "../bcWidgets/BitmapCanvasParentBase.h"

class RTPlotter;
class bcButton;
class bcLabel;
class bcScrollBar;
class bcProgressDialog;

class TriggerFrame;

class TriggerLog;
class TriggerLogManager;
class VemsTuneEventDispatcher;

class TriggerLogGraph : public wxFrame, public BitmapCanvasParentBase, public ObserverBase
{
    public:
        TriggerLogGraph( wxWindow* parent, VemsTuneEventDispatcher* evtDispatcher = NULL, const wxString& logdir = wxEmptyString );
        virtual ~TriggerLogGraph();


        void    setTriggerLogMgr( TriggerLogManager* mgr );

        void    clearData();

        void    setTriggerLog( TriggerLog* triggerlog );

        void    setContinuous( bool iscontinuous ) { m_isContinuous = iscontinuous; }
        void    add( TriggerFrame& trFrame );

        void    fillTimeLines( Int2DoubleArrayHash& timelines ) { timelines = m_timelines; }
        void    fillTimeLines( DoubleArray& tl, int n ) { tl = m_timelines[n]; }

        void    notify( int i );

    private:
        bool                m_tlogsupported; // fw should be greater than or equal to 1.1.78
        wxString            m_foundTlogVer; // fw version of the log

        bool                m_noserial; // flag (communicationevent seen oncomm..)

        bcProgressDialog*   m_progdlg;

        TriggerLogManager*  m_tlogmanager;

        unsigned int        m_timerTime;

        TriggerLog*         m_triggerlog;

        wxEvtHandler*       m_evtHandler; // start&stop V3

        bool                m_created;
        //wxScrollBar*        m_scrollBar;
        bcScrollBar*        m_scrollBar;

        Int2DoubleArrayHash m_timelines;

        RTPlotter*          m_plotter;
        wxTimer             m_timer;
        wxTimer             m_commtimer;

        //double              m_xstart;
        double              m_max;
        double              m_min;
        double              m_maxvalue;
        unsigned long       m_cursor;

        wxColor             m_bgColor;
        wxColor             m_scopeColor;
        wxColor             m_textcolor;

        wxString            m_logdir;

        // on-line drawing
        bool                m_isContinuous;
        bool                m_showLast;
        double              m_timewindow;
        //wxStaticText*       m_totalTime;
        bool                m_paused;
        wxButton*           m_pauseButton;
        int                 m_prevTime, m_clockBase;

        // selecting
        bool                m_isSelecting;

        bcButton*           m_startButton;
        bcButton*           m_scrollButton;
        bcButton*           m_saveSButton;
        bcButton*           m_saveWButton;
        bcButton*           m_colorSettingsButton;
        bcLabel*            m_totalTime;

        bcButton*           m_closeButton;
        bcButton*           m_helpButton;
        bcButton*           m_maximizeButton;

        bcLabel*            m_helpPanel;
        bcLabel*            m_helpTxt;
        bcLabel*            m_commerrorTxt;
        bool                m_helpPanelShown;

        // online - v3 case
        bcButton*           m_v3start;
        bcButton*           m_v3stop;
        bcButton*           m_v3showResultButton;

        bcLabel*            m_statistics;

        bool                m_needstoRedraw;
        wxBitmap*           m_canvasBmp;
        wxMemoryDC*         m_memDC;

        int                 m_prevReceivedData;

        wxStopWatch         m_stopper;

        VemsTuneEventDispatcher* m_evtDispatcher;

        wxSize              m_oldSize;
        bool                m_isMaximized;

    private:
        void                buildGUI( wxWindow* parent );

        void                OnPaint( wxPaintEvent& event );
        void                OnScroll( wxScrollEvent& event );
        void                OnSizeEvent( wxSizeEvent& event );
        void                OnKeyEvent( wxKeyEvent& event );
        void                OnMouse( wxMouseEvent& event );
        void                OnDragging( wxMouseEvent& event );
        void                OnMouseWheel( wxMouseEvent& event );
        void                OnButton( wxCommandEvent& event );
        void                OnActivate( wxActivateEvent& event );
        void                OnDblClick( wxMouseEvent& event );

        void                OnV3Button( wxCommandEvent& event );

        void                OnTriggerLogEvent( TriggerLogEvent& event );
        void                OnCommunicationEvent( CommunicationEvent& event );

        void                UpdateScrollBarPosition();
        void                OnTimer( wxTimerEvent& event );
        void                OnCommTimer( wxTimerEvent& event );

        void                GetSettings();
        void                SaveSettings();

        void                DrawStat();

        void                ResetValues();

        void                ToggleHelpPanel();

        void                Zoom( bool zoomin );

        DECLARE_EVENT_TABLE()

    private:
        /// DEPRECATED - only prim&sectrig
//        void         fillTimeLines( DoubleArray& prim, DoubleArray& sec ) { prim = m_primary; sec = m_secondary; }
//        DoubleArray     m_primary;
//        DoubleArray     m_secondary;
};

#endif // TRIGGERLOGGRAPH_H
