#ifndef TRIGGERLOGMANAGER_H
#define TRIGGERLOGMANAGER_H

#include <wx/file.h>

#include "../common/ProcessBase.h"
#include "../Observable.h"
#include "TriggerLogResults.h"
#include "../events/RawTriggerLogEvent.h"
#include "Unescaper.h"

class VemsTuneController;
class CommunicationBase;
class TriggerLog;
class TriggerLogGraph;
class TriggerLogController;
class VemsDeviceControllerBase;

class TriggerLogManager : public wxEvtHandler, public Observable
{
    public:
        // filename: is it from log or online (wxEmptyString)
        // parent should not be null if you want to close resultframe & graph automatically when closing VT
        // tltype: see TriggerLogSelector::TLOG_SOURCE ( tltype not needed in normal case )
        TriggerLogManager( VemsDeviceControllerBase* vdc, const wxString& fileName, wxEvtHandler* evtHandler = NULL, int tltype = 0 );
        virtual ~TriggerLogManager();

    public:
        TriggerLog*         getTriggerLog() { return m_triggerLog; }
        unsigned char       getType() const { return m_type; }

        bool                isContinuous() { return m_iscontinuous; }
        void                handleButtons( int type );
        void                setWindowParent( wxWindow* parent ) { m_winparent = parent; }
        void                clearData() { m_unescapedFrame.Clear(); m_triggerLog->clearData(); }

    protected:
    private:
        wxEvtHandler*           m_evtHandler;
        wxWindow*               m_winparent;
        unsigned char           m_type;         // v3 (0) or logfile (1)
        bool                    m_wasECUOnline;
        bool                    m_iscontinuous;

        wxFile                  m_triggerlogfile;

        VemsDeviceControllerBase*   m_vdc;

        TriggerLog*             m_triggerLog;

        TriggerLogResults*      m_resultFrame;

        wxString                m_fileName;

        TriggerLogController*   m_tlcontroller;
        Unescaper               m_unescaper;
        IntArray                m_unescapedFrame;

        // TriggerLogGraph - start V3 / stop V3, closed graph...
        void        OnStartStop( wxCommandEvent& event );

        void        OnRawTriggerLogEvent( RawTriggerLogEvent& event );

        // start/stop ECU
        /*bool        StartSession();
        bool        EndSession();*/

        // show result frame
        void    showResults();
        // log to file
        void    writeToLogFile( wxMemoryBuffer* buffer, bool start = false );

        DECLARE_EVENT_TABLE()

};

#endif // TRIGGERLOGMANAGER_H
