#include "TriggerLogSettingsHelper.h"
#include "TriggerFrame.h"

#include <wx/xml/xml.h>

/// helper
Int2wxColorHash TriggerLogSettingsHelper::tLogColors;
Int2StringHash TriggerLogSettingsHelper::tLogNames;
IntArray TriggerLogSettingsHelper::tLogEventsOrder;
Int2DoubleHash TriggerLogSettingsHelper::tLogVerticalOffsets;
wxColor TriggerLogSettingsHelper::tLog_bg;
wxColor TriggerLogSettingsHelper::tLog_scope;
wxColor TriggerLogSettingsHelper::tLog_text;

TriggerLogSettingsHelper::TriggerLogSettingsHelper()
{
    setDefaults();
}

void TriggerLogSettingsHelper::setDefaults()
{
    tLog_bg = wxColor( 231, 218, 190 );
    tLog_scope = wxColor( 132, 194, 251, 180 );
    tLog_text = wxColor( 0, 0, 0 );

    /// set up name of events
    tLogNames.clear();

    tLogNames[TriggerFrame::CRC_ONLY]       = _T("Primary trigger");
    tLogNames[TriggerFrame::SEC_TRIG]       = _T("Secondary trigger");
    tLogNames[TriggerFrame::CAMSHAFT_EX]    = _T("Camshaft exhaust");
    tLogNames[TriggerFrame::EVENT_4]        = _T("trigger Nr.4");
    tLogNames[TriggerFrame::EVENT_5]        = _T("trigger Nr.5");
    tLogNames[TriggerFrame::ROAD_SPEED1]    = _T("road speed");
    tLogNames[TriggerFrame::ROAD_SPEED2]    = _T("road speed2");
    tLogNames[TriggerFrame::EVENT_8]        = _T("trigger Nr.8");
    tLogNames[TriggerFrame::EVENT_9]        = _T("trigger Nr.9");
    tLogNames[TriggerFrame::EVENT_10]       = _T("trigger Nr.10");
    tLogNames[TriggerFrame::EVENT_11]       = _T("trigger Nr.11");
    tLogNames[TriggerFrame::EVENT_12]       = _T("trigger Nr.12");
    tLogNames[TriggerFrame::EVENT_13]       = _T("trigger Nr.13");
    tLogNames[TriggerFrame::EVENT_14]       = _T("trigger Nr.14");
    tLogNames[TriggerFrame::EVENT_15]       = _T("trigger Nr.15");
    tLogNames[TriggerFrame::EVENT_EEPROM_READ]  = _T("eeprom read");
    tLogNames[TriggerFrame::EVENT_REALTIME] = _T("realtime");
    tLogNames[TriggerFrame::EVENT_DATA_TIMESTAMP_1] = _T("data+timestamp");
    // add new events if necessary

    /// set up colors
    tLogColors.clear();

    tLogColors[TriggerFrame::CRC_ONLY]       = wxColor( 19, 58, 193, 180 );
    tLogColors[TriggerFrame::SEC_TRIG]       = wxColor( 255, 255, 255, 180 );
    tLogColors[TriggerFrame::CAMSHAFT_EX]    = wxColor( 255, 0, 0, 180 );
    tLogColors[TriggerFrame::EVENT_4]        = wxColor( 0, 255, 0, 180 );
    tLogColors[TriggerFrame::EVENT_5]        = wxColor( 0, 0, 255, 180 );
    tLogColors[TriggerFrame::ROAD_SPEED1]    = wxColor( 255, 255, 0, 180 );
    tLogColors[TriggerFrame::ROAD_SPEED2]    = wxColor( 128, 0, 255, 180 );
    tLogColors[TriggerFrame::EVENT_8]        = wxColor( 255, 0, 255, 180 );
    tLogColors[TriggerFrame::EVENT_9]        = wxColor( 64, 64, 64, 180 );
    tLogColors[TriggerFrame::EVENT_10]       = wxColor( 255, 128, 64, 180 );
    tLogColors[TriggerFrame::EVENT_11]       = wxColor( 128, 255, 128, 180 );
    tLogColors[TriggerFrame::EVENT_12]       = wxColor( 100, 100, 200, 180 );
    tLogColors[TriggerFrame::EVENT_13]       = wxColor( 205, 66, 105, 180 );
    tLogColors[TriggerFrame::EVENT_14]       = wxColor( 54, 0, 34, 180 );
    tLogColors[TriggerFrame::EVENT_15]       = wxColor( 75, 34, 88, 180 );
    tLogColors[TriggerFrame::EVENT_EEPROM_READ]  = wxColor( 0, 0, 0, 0 );
    tLogColors[TriggerFrame::EVENT_REALTIME] = wxColor( 0, 0, 0, 0 );
    tLogColors[TriggerFrame::EVENT_DATA_TIMESTAMP_1] = wxColor( 0, 0, 0, 0 );
    // add new events if necessary - set color to 0,0,0,0 if trigger should not be shown on the graph

    /// set up vertical offsets
    tLogVerticalOffsets.clear();

    tLogVerticalOffsets[TriggerFrame::CRC_ONLY]       = 0.0;
    tLogVerticalOffsets[TriggerFrame::SEC_TRIG]       = 0.0;
    tLogVerticalOffsets[TriggerFrame::CAMSHAFT_EX]    = 0.0;
    tLogVerticalOffsets[TriggerFrame::EVENT_4]        = 0.0;
    tLogVerticalOffsets[TriggerFrame::EVENT_5]        = 0.0;
    tLogVerticalOffsets[TriggerFrame::ROAD_SPEED1]    = 0.0;
    tLogVerticalOffsets[TriggerFrame::ROAD_SPEED2]    = 0.0;
    tLogVerticalOffsets[TriggerFrame::EVENT_8]        = 0.0;
    tLogVerticalOffsets[TriggerFrame::EVENT_9]        = 0.0;
    tLogVerticalOffsets[TriggerFrame::EVENT_10]       = 0.0;
    tLogVerticalOffsets[TriggerFrame::EVENT_11]       = 0.0;
    tLogVerticalOffsets[TriggerFrame::EVENT_12]       = 0.0;
    tLogVerticalOffsets[TriggerFrame::EVENT_13]       = 0.0;
    tLogVerticalOffsets[TriggerFrame::EVENT_14]       = 0.0;
    tLogVerticalOffsets[TriggerFrame::EVENT_15]       = 0.0;
    tLogVerticalOffsets[TriggerFrame::EVENT_EEPROM_READ]  = 0.0;
    tLogVerticalOffsets[TriggerFrame::EVENT_REALTIME] = 0.0;
    tLogVerticalOffsets[TriggerFrame::EVENT_DATA_TIMESTAMP_1] = 0.0;
    // add new events if necessary

    /// set up order of triggerevents, add event id that should be shown on graph
    tLogEventsOrder.Clear();

    tLogEventsOrder.Add( TriggerFrame::CRC_ONLY );
    tLogEventsOrder.Add( TriggerFrame::SEC_TRIG );
    tLogEventsOrder.Add( TriggerFrame::CAMSHAFT_EX );
    tLogEventsOrder.Add( TriggerFrame::EVENT_4 );
    tLogEventsOrder.Add( TriggerFrame::EVENT_5 );
    tLogEventsOrder.Add( TriggerFrame::ROAD_SPEED1 );
    tLogEventsOrder.Add( TriggerFrame::ROAD_SPEED2 );
    tLogEventsOrder.Add( TriggerFrame::EVENT_8 );
    tLogEventsOrder.Add( TriggerFrame::EVENT_9 );
    tLogEventsOrder.Add( TriggerFrame::EVENT_10 );
    tLogEventsOrder.Add( TriggerFrame::EVENT_11 );
    tLogEventsOrder.Add( TriggerFrame::EVENT_12 );
    tLogEventsOrder.Add( TriggerFrame::EVENT_13 );
    tLogEventsOrder.Add( TriggerFrame::EVENT_14 );
    tLogEventsOrder.Add( TriggerFrame::EVENT_15 );

    // uncomment these lines if you want to see that kind of events
    // tLogEventsOrder.Add( TriggerFrame::EVENT_REALTIME );
    // tLogEventsOrder.Add( TriggerFrame::EVENT_DATA_TIMESTAMP_1 );
}

/* static */
bool TriggerLogSettingsHelper::saveSettings( const wxString& filename )
{
    wxXmlNode* root = new wxXmlNode( NULL, wxXML_ELEMENT_NODE, _T("colors") );

    wxXmlNode* childnode = NULL, *contentnode;

    bool first = true;

    for ( short i = tLogEventsOrder.GetCount() - 1; i >= 0; --i )
    {
        int event_id = tLogEventsOrder[i];

        if ( tLogNames.find( event_id ) == tLogNames.end() )
        {
            continue;
        }

        wxString event_name = tLogNames[event_id];

        wxColor event_color = tLogColors[event_id];

        double event_offset = tLogVerticalOffsets[event_id];

        wxString content = wxString::Format( _T("#%02x%02x%02x"), event_color.Red(), event_color.Green(), event_color.Blue() );

        childnode = new wxXmlNode( root, wxXML_ELEMENT_NODE, _T("event") );
        childnode->AddProperty( _T("id"), wxString::Format( _T("%d"), event_id ) );
        childnode->AddProperty( _T("name"), event_name );
        childnode->AddProperty( _T("voffset"), wxString::Format( _T("%.0f"), event_offset ) );

        contentnode = new wxXmlNode( childnode, wxXML_TEXT_NODE, _T(""), content );

        if ( first )
        {
            root->SetNext( childnode );
            first = false;
        }
        else
        {
            root->SetChildren( childnode );
        }
    }

    wxColor carr[3] = { tLog_text, tLog_scope, tLog_bg };

    wxArrayString sarr;
    sarr.Add( _T("textcolor") );
    sarr.Add( _T("scope") );
    sarr.Add( _T("background") );

    for ( size_t i = 0; i < sarr.GetCount(); ++i )
    {
        wxColor& c = carr[i];

        wxString content = wxString::Format( _T("#%02x%02x%02x"), c.Red(), c.Green(), c.Blue() );

        childnode = new wxXmlNode( root, wxXML_ELEMENT_NODE, sarr[i] );
        contentnode = new wxXmlNode( childnode, wxXML_TEXT_NODE, _T(""), content );

        root->SetChildren( childnode );
    }

    wxXmlNode* cnode = new wxXmlNode( root, wxXML_COMMENT_NODE, _T(""), _T("VemsTune TriggerLogGraph color settings") );

    root->SetNext( cnode );

    wxXmlDocument xmldoc;

    xmldoc.SetRoot( root );

    return xmldoc.Save( filename );
}


/*static */
void TriggerLogSettingsHelper::loadSettings( const wxString& filename )
{
    wxXmlDocument doc;

    // use default values, if settings file does not exist
    if ( doc.Load( filename ) == false || doc.GetRoot()->GetName() != wxT("colors") )
    {
        return;
    }

    wxXmlNode *child = doc.GetRoot()->GetChildren();
    while ( child )
    {
        wxString color = child->GetNodeContent();

        if ( child->GetName() == _T("background") )
            tLog_bg = wxColor( color );

        else if ( child->GetName() == _T("scope") )
            tLog_scope = wxColor( color );

        else if ( child->GetName() == _T("textcolor") )
            tLog_text = wxColor( color );

        else if ( child->GetName() == _T("event") )
        {
            wxString numberS = child->GetPropVal( _T("id"), _T("-1") );

            long id;

            if ( numberS.ToLong( &id ) == false )
                continue;

            if ( id != -1 )
            {
                tLogColors[id] = wxColor( color );

                // get vertical offset
                wxString offset;
                if ( child->GetPropVal( _T("voffset"), &offset ) )
                {
                    double voff = 0;
                    if ( offset.ToDouble( &voff ) )
                    {
                        tLogVerticalOffsets[id] = voff;
                    }
                    else
                    {
                        tLogVerticalOffsets[id] = 0;
                    }
                }
                // get name
                wxString tname;
                if ( child->GetPropVal( _T("name"), &tname ) )
                {
                    tLogNames[id] = tname;
                }
                else
                {
                    tLogNames[id] = wxString::Format( _T("trigger Nr.%d"), id );
                }
            }
        }

        // get next child
        child = child->GetNext();
    }
}
