#include "CarConfigHelper.h"
#include "CarParameters.h"
#include "../eval.h"
#include "../logger.h"
#include "../configmanager/ConfigHelper2.h"


CarParameters* CarConfigHelper::getCarParams()
{
    CarParameters *carParams = new CarParameters();

    double wheelRadius =  getSessionTorqueParam(_T("wheelRadius"), 0.3);
    double aNull = getSessionTorqueParam(_T("aNull"),0.03 * 9.81);
    double fNull = getSessionTorqueParam(_T("fNull"), 500.0);
    double mass = getSessionTorqueParam(_T("mass"), 1600.0);

    carParams->setWheelRadius(wheelRadius);
    carParams->setaNull(aNull);
    carParams->setfNull(fNull);
    carParams->setMass(mass);

    DoubleHash * drivetrainH = getSessionTorqueDriveTrain();
    carParams->setdrivetrainH(drivetrainH);
    carParams->print();

    return carParams;
}

double CarConfigHelper::getSessionTorqueParam(const wxString& key, double defaultValue )
{

    ConfigBase* config = ConfigHelper2::getConfig();
    config->setPath( _T("/defaultTorqueConfig/") );
    wxString valueStr;
    bool res = config->read(key, &valueStr);

    if(! res){
         //config->SetPath(oldPath);
         return defaultValue;
    }
    double value = evaluate(valueStr, defaultValue);
    //config->SetPath(oldPath);
    return value;

}

DoubleHash * CarConfigHelper::getSessionTorqueDriveTrain()
{
    DoubleHash *dHash = new DoubleHash();

    StringHash vHash;

    wxString path = _T("/defaultTorqueConfig/");
    ConfigHelper2::getEntryHash(path, vHash);

    StringHash::iterator it;
    for(it = vHash.begin(); it != vHash.end(); it++){
        wxString key = it->first;
        wxString valueStr = it->second;
        double value = evaluate(valueStr, 1.0);
        (*dHash)[key] = value;
    }

    return dHash;
}

double CarConfigHelper::evaluate(const wxString& valueStr, double defaultValue)
{
    Eval eval;
    double param;
    try{
        param = eval.evaluate(valueStr);

    }catch(ZeroDivideException e){
         Logger::log(LOG_MSG,_T("Helper::getSessionTorqueParam() Eval ZeroDivideException")  );
        return  defaultValue;

    }catch(SyntaxError e){

        Logger::log(LOG_MSG,_T("Helper::getSessionTorqueParam() Eval Syntax error: " + e.get() ));
        return  defaultValue;

    }
    //Logger::log(LOG_MSG,_T("Helper::getSessionTorqueParam(): key=") + key + wxString::Format(_T("%d"), res) );
    return param;
}
