#include "VEMSDataLogMixtureView.h"

#include "../common/typedef.h"

VEMSDataLogMixtureView::VEMSDataLogMixtureView(wxWindow *parent,
VEMSDataLogDesc * dataLog)
: wxGrid(parent, wxID_ANY)
{
    m_dataLog = dataLog;

    const int max_rpm = 8000;
	const int rpm_step = 500;

	CreateGrid(25 + 1, max_rpm / rpm_step + 1);

	BeginBatch();

	SetDefaultCellAlignment(wxALIGN_CENTRE, wxALIGN_CENTRE);
	SetEditable(false);


	double afr_table[100][250];
	double afr_target_table[100][250];
	int   afr_count[100][250];

	memset(afr_table, 0, sizeof(afr_table));
	memset(afr_count, 0, sizeof(afr_count));
	memset(afr_target_table, 0, sizeof(afr_target_table));

	uint16_t i;
	for (i = 0; i < max_rpm / rpm_step + 1; ++i)
	{
		wxString colText;
		colText.sprintf(_T("%u"), i * rpm_step);
		SetColLabelValue(i, colText);
	}

	for (i = 0; i < 25 + 1; ++i)
	{
		wxString rowText;
		rowText.sprintf(_T("%u"), i * 10);
		SetRowLabelValue(i, rowText);
	}

    ECULogItemArray * logA =  m_dataLog->getVEMSDataLog();
    wxArrayString * headerA = m_dataLog->getVEMSDataLogHeader();

   // int timeIdx = headerA->Index(_T("Time"));
    int rpmIdx = headerA->Index(_T("RPM"));
   // int mapIdx = headerA->Index(_T("MAP"));
    int lambdaIdx = headerA->Index(_T("lambda"));
    int lambdaTargetIdx = headerA->Index(_T("lambdaTarget"));
    int loadIdx = headerA->Index(_T("MAP"));


	for (i = 0; i < logA->Count(); i++)
	{
        DoubleArray * rowArray = logA->Item(i);

        double lambdaV = rowArray->Item(lambdaIdx);
        double lambdaTargetV = rowArray->Item(lambdaTargetIdx);
        double loadV = rowArray->Item(loadIdx);
        double rpmV = rowArray->Item(rpmIdx);

		int col = (int)(rpmV / rpm_step);
		int row = (int)(loadV / 10);

		if (col < 100 && row < 250)
		{
			afr_count[col][row]++;			// +1 value collected
			afr_table[col][row] += lambdaV;
			afr_target_table[col][row] += lambdaTargetV;
		}
	}

	for (i = 0; i < max_rpm / rpm_step + 1; ++i)
	{
		for (int j = 0; j < 25 + 1; ++j)
		{
			if (afr_count[i][j] > 0)
			{
				const float afr = afr_table[i][j] / afr_count[i][j];
				const float afr_target = afr_target_table[i][j] / afr_count[i][j];

				wxString valText;
				valText.sprintf(_T("lambda=%3.2f\nCollected=%u\nlambda_Target=%3.2f"), afr, afr_count[i][j], afr_target);
				SetCellValue(valText, j, i);

				if (vemsABS(afr - afr_target) < 0.05f)
				{
					SetCellBackgroundColour(*wxGREEN, j, i);
				}
				else if (vemsABS(afr - afr_target) < 0.1f)
				{
					SetCellBackgroundColour(*wxBLUE, j, i);
				}
				else
				{
					SetCellBackgroundColour(*wxRED, j, i);
				}
			}
		}
	}

	AutoSize();

	EndBatch();
}


