#include "ArrayEditorDialog.h"

#include <wx/statline.h>

#include "../logger.h"

enum
{
    ID_INSERT_COMBO   = 1000
};

BEGIN_EVENT_TABLE( ArrayEditorDialog, wxDialog )
    EVT_COMBOBOX( ID_INSERT_COMBO, ArrayEditorDialog::OnComboBoxSelected )
END_EVENT_TABLE()

ArrayEditorDialog::ArrayEditorDialog(IntArray* intArray, wxWindow* parent, wxWindowID id)
{

    m_boxCount = 0;

	Create(parent, id, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE, _T("id"));
	m_topSizer = new wxStaticBoxSizer(wxVERTICAL, this, _(""));

	SetSizer(m_topSizer);
	m_topSizer->Fit(this);
	m_topSizer->SetSizeHints(this);

	m_intArray = intArray;

    m_arrayBoxName = _T("Current RPM/MAT headers");
    addBox(m_arrayBoxName);
    showArray();
    addBox(_T("Insert row"));
    insertRow();
    addDialogButtons();
}

ArrayEditorDialog::~ArrayEditorDialog()
{
    //dtor
}

void ArrayEditorDialog::showArray()
{
    for(size_t i = 0; i < m_intArray->Count(); i++ ){
        wxString idxValue = wxString::Format(_T("[%i]"), i);
        wxString value = wxString::Format(_T("%i"), m_intArray->Item(i) );

        wxStaticText *staticText = new wxStaticText(this,wxID_ANY,idxValue,wxDefaultPosition,wxDefaultSize,0,_T(""));
        m_currGridSizer->Add(staticText,0,wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL,0); // 1.

        wxTextCtrl* textCtrl1= new wxTextCtrl(this,wxID_ANY,value,wxDefaultPosition,wxDefaultSize,2,wxDefaultValidator,_T(""));
        m_currGridSizer->Add(textCtrl1,1,wxALL|wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL,1);
        textCtrl1->Disable();
    }
}

void ArrayEditorDialog::addBox(const wxString& boxName)
{
    wxStaticBoxSizer* staticBoxSizer = new wxStaticBoxSizer(wxHORIZONTAL,this,boxName);
	wxFlexGridSizer* gridSizer = new wxFlexGridSizer(0,2,0,0);
    gridSizer->AddGrowableCol(1,1);
    staticBoxSizer->Add(gridSizer,1,wxALL|wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL,1);

    m_topSizer->Insert(m_boxCount, staticBoxSizer,0,wxALL|wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL,1);

    m_currGridSizer = gridSizer;
    m_sizerHash[boxName] = m_currGridSizer;
    m_boxCount++;
    //boxRowCount = 0;
    //boxRowCountHash[boxName] = 0;
    //m_currBoxName = boxName;

    //addCaption();
}

void ArrayEditorDialog::addDialogButtons()
{
    const long ID_STATICLINE1 = wxNewId();
    wxStaticLine* StaticLine1 = new wxStaticLine(this,ID_STATICLINE1,wxDefaultPosition,wxSize(10,-1),wxLI_HORIZONTAL,_T("ID_STATICLINE1"));
    m_topSizer->Add(StaticLine1,0,wxALL|wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL,4);

    wxBoxSizer *buttsizer = new wxBoxSizer(wxHORIZONTAL);
    m_buttonOK = new wxButton(this,wxID_OK, _T("OK"));

    buttsizer->Add(m_buttonOK, 0, wxALL | wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL, 3);
    buttsizer->Add(new wxButton(this,wxID_CANCEL, _T("Cancel")),0, wxALL | wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL, 3);

    m_topSizer->Add(buttsizer,0,wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL,4);


	SetSizer(m_topSizer);
	m_topSizer->Fit(this);
	m_topSizer->SetSizeHints(this);
    m_topSizer->Layout();
}

void ArrayEditorDialog::insertRow()
{
    wxStaticText *staticText
    = new wxStaticText(this,wxID_ANY,_T("Insert New Row: "),wxDefaultPosition,wxDefaultSize,0,_T(""));
    m_currGridSizer->Add(staticText,0,wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL,0); // 1.

    wxArrayString choices;// = wxArrayString;
    for(size_t i = 0; i < m_intArray->Count()+1; i++ ){
        wxString idxValue = wxString::Format(_T("%i"), i);
        choices.Add(idxValue);
    }
    wxString defValue = _T("Select row index");

    m_insertComboBox
    = new wxComboBox(this,ID_INSERT_COMBO,defValue,wxDefaultPosition,wxDefaultSize,choices,0,wxDefaultValidator,_T("ADD_LINE_COMBO"));
	m_currGridSizer->Add(m_insertComboBox,0,wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL,0); // 2.


}

void ArrayEditorDialog::OnComboBoxSelected( wxCommandEvent &event )
{
    Logger::log(LOG_MSG, wxString::Format(_T("ArrayEditorDialog::OnComboBoxSelected()") ) );
    wxString valueStr = m_insertComboBox->GetValue();

    size_t idxValue = 0;
    int value;
    valueStr.ToLong((long*)&idxValue);
    m_currGridSizer = m_sizerHash[m_arrayBoxName];
    m_currGridSizer->Clear(true);
    if(idxValue == 0){
        value = m_intArray->Item(0);
    }else if(idxValue == m_intArray->Count() ){
        value = m_intArray->Item(m_intArray->Count()-1 );
    }else{
        value = (m_intArray->Item(idxValue-1) +  m_intArray->Item(idxValue)) /2 ;
    }
    m_intArray->Insert(value, idxValue);
    Logger::log(LOG_MSG, wxString::Format(_T("ArrayEditorDialog::OnComboBoxSelected() idx:%i, value:%i"), idxValue, value ) );

    m_insertComboBox->Append(wxString::Format(_T("%i"), m_intArray->Count()) );

    showArray();
    m_topSizer->Layout();
    m_topSizer->SetSizeHints(this);
}
