#include "CheckBoxMatrixViewer.h"

#include <wx/wx.h>

#include "../Symbol.h"
#include "../logger.h"
#include "../stdatamanager/STDataMgrBase.h"
#include "../stdatamanager/STDataModelBase.h"
#include "CheckBoxMatrix.h"
#include "EconosealPanel.h"

#include "../SymbolMgr.h"
#include "../vemsprotocol/VemsCommand.h"
#include "../vemsprotocol/CommandRequestBase.h"
#include "../gaugedesc/OutputChannelSQL.h"
#include "../Math.h"

/// Desc
IMPLEMENT_CLASS(CheckBoxMatrixDesc, wxObject)
CheckBoxMatrixDesc::CheckBoxMatrixDesc(){};
CheckBoxMatrixDesc::~CheckBoxMatrixDesc(){};

///
STViewerCBMatrix::STViewerCBMatrix(const Symbol& s, CheckBoxMatrixViewer* cbViewer, SymbolTable* symTable )
{

    m_symbol    = s;
    m_cbViewer  = cbViewer;
    m_symbols   = symTable;
    Logger::logf( LOG_MSG, _T("STViewerCBMatrix::ctor symbol: ") + m_symbol.getName());
    m_dbnames = OutputChannelSQL::getSymbols();

}

void STViewerCBMatrix::update()
{
    Logger::logf( LOG_MSG, _T("STViewerCBMatrix::updateGUI()") );
    size_t rowCount = m_symbol.getArraySize();
    for ( size_t i = 0; i < rowCount; ++i ){
        double viewerData = m_stDataMgr->getDataFromViewerModel( m_symbol, 0, i );
        m_cbViewer->setValue( i, (int)viewerData );
    }

    SymbolTable::iterator it = m_symbols->find(_T("alternateCyl"));

    if ( it == m_symbols->end() )
        return;

    Symbol* sym = it->second;

    int injOutputs = (int)m_stDataMgr->getDataFromViewerModel( *sym, 0, 0 );
    Logger::logf(LOG_MSG, _T("injoutputs=%d"),injOutputs);

    m_cbViewer->getCBMatrix()->setInjectorNum( injOutputs );

    // staging - if tps < 100% -> use staging
    it = m_symbols->find(_T("inj_stage2_start_tps"));

    bool staging = false;

    if ( it != m_symbols->end() )
    {
        sym = it->second;
        double stg = m_stDataMgr->getDataFromViewerModel( *sym, 0, 0 );
        if ( stg < 99.5 ) // because of resolution (99.6 is 100%)
            staging = true;
}

    /// dual solenoid
    // is dual solenoid mode set?
    int dual_solenoid_channel_out = -1;
    int dual_solenoid_channels[2] = { -1, -1 };
    SymbolTable::iterator s_it = m_symbols->find( _T("iac_dualsol") );
    if ( s_it != m_symbols->end() )
    {
        int val = (uint8_t)m_stDataMgr->getDataFromViewerModel( *(s_it->second), 0, 0 );
        if ( val == 1 )
        {
            s_it = m_symbols->find( _T("iac_sol_channel_names") );
            dual_solenoid_channel_out = (uint8_t)m_stDataMgr->getDataFromViewerModel( *(s_it->second), 0, 0 );

            if ( dual_solenoid_channel_out != 0x0f ) // 0x0f is disabled
            {
                s_it = m_symbols->find( _T("iac_sol_channel_bits_dual") );
                int dualval = (uint8_t)m_stDataMgr->getDataFromViewerModel( *(s_it->second), 0, 0 );

                if ( dualval < 7 )
                {
                    //0,i1 ; 1,i2..., 6,i7
                    dual_solenoid_channels[0] = 16 * dualval + dual_solenoid_channel_out;
                    dual_solenoid_channels[1] = 16 * ( dualval + 1 ) + dual_solenoid_channel_out + 128; // inverted
                }
                else
                {
                    // i0,1; i1,2... i6,7
                    dualval -= 7;
                    dual_solenoid_channels[0] = 16 * dualval + dual_solenoid_channel_out + 128; // inverted
                    dual_solenoid_channels[1] = 16 * ( dualval + 1) + dual_solenoid_channel_out;
                }
            }
        }
    }
    if ( dual_solenoid_channels[0] > 127 ) dual_solenoid_channels[0] -= 128;
    if ( dual_solenoid_channels[1] > 127 ) dual_solenoid_channels[1] -= 128;
    LOGP(wxString::Format( _T("dualsol1=%d, dualsol2=%d"), dual_solenoid_channels[0], dual_solenoid_channels[1]))
    // end of dual solenoid mode.
    ///--------

    int i = 0;

    while ( i < 8 )
    {
        wxString hexval = wxString::Format( _T("%d0"), i );
        int intval = Math::hex2int( hexval );
        if ( intval > 127 )
            intval -= 128;

        int found = 0;
        dbsymbol* dbs = NULL;
        wxString channels = wxEmptyString;

        bool idlesolenoidfound = false;

        if ( dual_solenoid_channels[0] >= 0 || dual_solenoid_channels[1] >= 0 )
        {
            if ( intval == dual_solenoid_channels[0] )
            {
                found++;
                channels += _T("Idle solenoid, ");
                idlesolenoidfound = true;
            }
            else if ( intval == dual_solenoid_channels[1] )
            {
                found++;
                channels += _T("Idle solenoid (dual), ");
            }
        }
        for ( size_t j = 0; j < m_dbnames.GetCount(); ++j)
        {
            dbs = (dbsymbol*)(m_dbnames[j]);
            Symbol* sym = NULL;

            SymbolTable::iterator it = m_symbols->find( dbs->name );
            if ( it != m_symbols->end() ) sym = it->second;

            if ( sym )
            {
                int symval = (int)( m_stDataMgr->getDataFromDeviceModel( *sym, 0, 0 ) );

                if ( symval > 127 )
                    symval -= 128;

                if ( symval == intval )
                {
                    if ( sym->getName() == _T("iac_sol_channel") && idlesolenoidfound == true );
                    else {
                    found++;
                    channels += dbs->fullname + _T(", ");

                        if ( symval == dual_solenoid_channels[0] || symval == dual_solenoid_channels[1] )
                        {
                            if ( ( symval & 15 ) || ( dual_solenoid_channel_out & 15 ) ); // disabled
                            else
                            {
                                found++;
                }
            }
        }
                }
            }
        }

        channels.RemoveLast( 2 );

        if ( found > 0 )
        {
            if ( i <= injOutputs )
                m_cbViewer->setOutputName( i, wxString::Format( _T("Injector %d"), i ), true, channels );
            else
            {
                if ( staging &&  i<(injOutputs+1)*2 )
                {
                    if ( i == 7 && injOutputs == 4 )
                    {
                        if ( found == 1 )
                            m_cbViewer->setOutputName( i, channels, false );
                        else
                            m_cbViewer->setOutputName( i, channels, true, channels );
                    }
                    else
                    {
                        m_cbViewer->setOutputName( i, wxString::Format( _T("Staging %d"), i-(injOutputs+1) ), true, channels );
                    }

                }
                else
                {
                    if ( found == 1 )
                        m_cbViewer->setOutputName( i, channels, false );
                    else
                        m_cbViewer->setOutputName( i, channels, true, channels );
                }
            }
        }
        else
        {
            if ( i <= injOutputs )
                m_cbViewer->setOutputName( i, wxString::Format( _T("Injector %d"), i ), false );
            else
            {
                if ( staging &&  i<(injOutputs+1)*2 )
                {
                    if ( i == 7 && injOutputs == 4 )
                        m_cbViewer->setOutputName( i, _T("(none)"), false );
                    else
                        m_cbViewer->setOutputName( i, wxString::Format( _T("Staging %d"), i-(injOutputs+1) ), false );
                }

                else
                    m_cbViewer->setOutputName( i, _T("(none)"), false );
            }
        }

        ++i;
    }
}

void STViewerCBMatrix::updateModel()
{
    size_t rowCount = m_symbol.getArraySize();

    for ( size_t i = 0; i < rowCount; ++i )
    {
        int value = m_cbViewer->getValue(i);
        m_stDataMgr->setModelData( m_symbol, 0, i, value );
    }
}


void STViewerCBMatrix::requestWrite()
{
    size_t rowCount = m_symbol.getArraySize();

    for ( size_t i = 0; i < rowCount; ++i )
    {
        m_stDataMgr->requestWrite( m_symbol, 0, i );
    }
}
wxString STViewerCBMatrix::toString()
{
    return _T("STViewerCBMatrix - ") + m_symbol.getName();
}

///

int idCheckBoxMatrixTest = wxNewId();

BEGIN_EVENT_TABLE( CheckBoxMatrixViewer, wxPanel )
    EVT_COMBOBOX( idCheckBoxMatrixTest, CheckBoxMatrixViewer::OnComboBox )
END_EVENT_TABLE()

CheckBoxMatrixViewer::CheckBoxMatrixViewer(CheckBoxMatrixDesc* desc,
    STDataMgrBase* stController, CommandRequestBase* requestMgr, SymbolMgr* symbolmgr, wxWindow* parent)
    : wxPanel( parent )
    , StaticViewersController( parent, desc->m_help )
{
    m_pDesc         = desc;
    ///TEST
    //testFillDesc();
    m_controller    = stController;
    m_requestMgr = requestMgr;
    m_symbolMgr = symbolmgr;
//    init();
    buildGUI();
    //ctor
}

CheckBoxMatrixViewer::~CheckBoxMatrixViewer()
{
    //dtor
    LOG
    if ( m_testMode->GetValue() == _T("ON") )
    {
        m_checkBoxMatrixpanel->SwitchOffAllsignals();
    }
}

void CheckBoxMatrixViewer::buildGUI()
{


    //main sizer
    SetSizer( new wxBoxSizer( wxVERTICAL ) );

    wxStaticText * text = new wxStaticText( this, -1, _T("Must not be changed with running engine") );
    wxFont font = text->GetFont();
    font.SetWeight( wxFONTWEIGHT_BOLD );
    text->SetFont( font );
    text->SetForegroundColour( wxColor(255,0,0 ) );

    wxStaticBoxSizer* testSizer = new wxStaticBoxSizer( wxHORIZONTAL, this, _T("Testing mode") );
    wxStaticText* txt = new wxStaticText( this, -1, _T("Switch testing mode") );

    wxArrayString choices;
    choices.Add( _T("OFF") ); choices.Add( _T("ON") );

    m_testMode = new wxComboBox( this, idCheckBoxMatrixTest, _T("OFF"), wxDefaultPosition, wxDefaultSize,
        choices, wxCB_DROPDOWN | wxCB_READONLY );

    testSizer->Add( txt, 0, wxALL, 5 );
    testSizer->Add( m_testMode, 0, wxALL, 5 );

    //checkboxPanel
    m_checkBoxMatrixpanel =
    new CheckBoxMatrix(m_pDesc, this,wxID_ANY);
    m_checkBoxMatrixpanel->registerObserver(this);
    //matrixSizerBox->Add( checkBoxMatrixpanel, 1, wxEXPAND | wxALL, 5 );
    //matrixSiterBox
    wxBoxSizer* matrixSizerBox = new wxStaticBoxSizer( wxVERTICAL, this, m_pDesc->m_title );
    matrixSizerBox->Add( m_checkBoxMatrixpanel,0,wxALL , 5 );

//    m_ViewerBaseA.Add(  );
//    m_controller->RegisterViewer( pRowHeaderViewer );

    m_econosealPanel = new EconosealPanel(this,36);
//    econosealBox->Add( econosealBox, 1, wxEXPAND | wxALL, 5 );

    wxBoxSizer* econosealBox = new wxStaticBoxSizer(wxVERTICAL, this, _T("Econoseal 36 pinout") );
    econosealBox->Add( m_econosealPanel,0,wxALIGN_CENTER_HORIZONTAL| wxALIGN_CENTER_VERTICAL, 5 );

    GetSizer()->Add( text, 0, wxALL, 5 );
    GetSizer()->Add( testSizer, 0, wxALL, 5 );
    GetSizer()->Add( matrixSizerBox, 0, wxEXPAND | wxALL, 5 );
    GetSizer()->Add( econosealBox, 0, wxALL, 5 );
    GetSizer()->Fit(this);
    SetInitialSize();

    Symbol* symbol = m_symbolMgr->getSymbol(m_pDesc->m_SymbolName);
    if(!symbol) return;

    int size = symbol->getArraySize();
    Logger::logf( LOG_MSG, _T("CheckBoxMatrixViewer::buildGUI() size:%i ") + symbol->getName(), size );


    m_stViewer = new STViewerCBMatrix(*symbol, this, &(m_symbolMgr->getSymbolTable()) );
    m_ViewerBaseA.Add( m_stViewer );
    m_controller->RegisterViewer( m_stViewer );

    updateData();
}

/// Notify the checkbox state on/off
void CheckBoxMatrixViewer::notify( int i )
{
    if(i) m_isChecked = true;
    else m_isChecked = false;
};

/// Notify the coordinates;
void CheckBoxMatrixViewer::notify(int row, int col)
{
    int pinNum = m_pDesc->m_maskPinNumbers.Item(col);
    wxString color = m_pDesc->m_groupColors.Item(row);
    if(m_isChecked){
        /// Uncheck all in matrix panel:
        for(size_t i = 0; i < m_pDesc->m_groupNames.Count(); i++ ){
            //m_econosealPanel->SetPinColor(i, wxColor(_T("#808080")) );
            if(row != (int)i) m_checkBoxMatrixpanel->setValue(i, col, false);
//             m_values[row] &= !(m_pDesc->m_maskValues.Item(col));
        }

        ///
        ///for(size_t i = 0; i < m_pDesc->m_maskPinNumbers.Count(); i++ ){
        ///    m_econosealPanel->SetPinColor(i, wxColor(_T("#808080")) );
        ///}
        m_econosealPanel->SetPinColor(pinNum, wxColor(color) );

//        m_values[row] |= m_pDesc->m_maskValues.Item(col);

    }else{
        m_econosealPanel->SetPinColor(pinNum, wxColor(_T("#FFFFFF")) );

//        m_values[row] &= !(m_pDesc->m_maskValues.Item(col));

    }
    //m_stViewer->updateModell();
    m_stViewer->updateModel();
    m_stViewer->requestWrite();
    m_controller->updateRegisteredViewers();
}

void CheckBoxMatrixViewer::notify(const wxString& str)
{
    VemsCommand* command = new StringCommand(str);
    m_requestMgr->request(command);
}

int CheckBoxMatrixViewer::getValue(int offset)
{
    return m_checkBoxMatrixpanel->getValue(offset);
}

void CheckBoxMatrixViewer::setValue(size_t offset, int value)
{
    IntArray& maskValueA  = m_pDesc->m_maskValues;
    wxArrayString& colorA = m_pDesc->m_groupColors;
    if(offset >= colorA.GetCount() ) return;

    wxString color = colorA.Item(offset);

    for(size_t i = 0; i < maskValueA.GetCount(); i++){
        int maskValue = maskValueA.Item(i);
        int res = value & maskValue;
        if(res > 0){
            m_checkBoxMatrixpanel->setValue(offset, i, true);
            int pinNum = m_pDesc->m_maskPinNumbers.Item(i);
            m_econosealPanel->SetPinColor(pinNum, wxColor(color) );
        }else{
            m_checkBoxMatrixpanel->setValue(offset, i, false);
        }
    }
}


CheckBoxMatrixDesc* CheckBoxMatrixViewer::getTestDesc()
{
    CheckBoxMatrixDesc* tmp =  new CheckBoxMatrixDesc();
    CheckBoxMatrixDesc& m_ptestDesc = *tmp;
    //m_ptestDesc.m_maskHeaderName = _T("Econoseal 36 Pinout");
    m_ptestDesc.m_groupHeaderName = _T("");
    m_ptestDesc.m_SymbolName = _T("h0");

    m_ptestDesc.m_groupNames.Add( _T("InjGroup 0") );
    m_ptestDesc.m_groupNames.Add( _T("InjGroup 1") );
    m_ptestDesc.m_groupNames.Add( _T("InjGroup 2") );
    m_ptestDesc.m_groupNames.Add( _T("InjGroup 3") );
    m_ptestDesc.m_groupNames.Add( _T("InjGroup 4") );
    m_ptestDesc.m_groupNames.Add( _T("InjGroup 5") );
    m_ptestDesc.m_groupNames.Add( _T("InjGroup 6") );
    m_ptestDesc.m_groupNames.Add( _T("InjGroup 7") );
/*
    m_ptestDesc.m_maskNames.Add(_T("7"));
    m_ptestDesc.m_maskNames.Add(_T("6"));
    m_ptestDesc.m_maskNames.Add(_T("5"));
    m_ptestDesc.m_maskNames.Add(_T("4"));
    m_ptestDesc.m_maskNames.Add(_T("3"));
    m_ptestDesc.m_maskNames.Add(_T("2"));
    m_ptestDesc.m_maskNames.Add(_T("1"));
    m_ptestDesc.m_maskNames.Add(_T("0"));

    m_ptestDesc.m_maskValues.Add(128);
    m_ptestDesc.m_maskValues.Add(64);
    m_ptestDesc.m_maskValues.Add(32);
    m_ptestDesc.m_maskValues.Add(16);
    m_ptestDesc.m_maskValues.Add(8);
    m_ptestDesc.m_maskValues.Add(4);
    m_ptestDesc.m_maskValues.Add(2);
    m_ptestDesc.m_maskValues.Add(1);

    m_ptestDesc.m_maskPinNumbers.Add(17);
    m_ptestDesc.m_maskPinNumbers.Add(6);
    m_ptestDesc.m_maskPinNumbers.Add(18);
    m_ptestDesc.m_maskPinNumbers.Add(9);
    m_ptestDesc.m_maskPinNumbers.Add(20);
    m_ptestDesc.m_maskPinNumbers.Add(8);
    m_ptestDesc.m_maskPinNumbers.Add(19);
    m_ptestDesc.m_maskPinNumbers.Add(7);
*/
    m_ptestDesc.m_maskNames.Add(_T("0"));
    m_ptestDesc.m_maskNames.Add(_T("1"));
    m_ptestDesc.m_maskNames.Add(_T("2"));
    m_ptestDesc.m_maskNames.Add(_T("3"));
    m_ptestDesc.m_maskNames.Add(_T("4"));
    m_ptestDesc.m_maskNames.Add(_T("5"));
    m_ptestDesc.m_maskNames.Add(_T("6"));
    m_ptestDesc.m_maskNames.Add(_T("7"));

    m_ptestDesc.m_maskValues.Add(1);
    m_ptestDesc.m_maskValues.Add(2);
    m_ptestDesc.m_maskValues.Add(4);
    m_ptestDesc.m_maskValues.Add(8);
    m_ptestDesc.m_maskValues.Add(16);
    m_ptestDesc.m_maskValues.Add(32);
    m_ptestDesc.m_maskValues.Add(64);
    m_ptestDesc.m_maskValues.Add(128);


    m_ptestDesc.m_maskPinNumbers.Add(7);
    m_ptestDesc.m_maskPinNumbers.Add(19);
    m_ptestDesc.m_maskPinNumbers.Add(8);
    m_ptestDesc.m_maskPinNumbers.Add(20);
    m_ptestDesc.m_maskPinNumbers.Add(9);
    m_ptestDesc.m_maskPinNumbers.Add(18);
    m_ptestDesc.m_maskPinNumbers.Add(6);
    m_ptestDesc.m_maskPinNumbers.Add(17);

    m_ptestDesc.m_groupColors.Add(_T("#FF0000"));
    m_ptestDesc.m_groupColors.Add(_T("#00FF00"));
    m_ptestDesc.m_groupColors.Add(_T("#008080"));
    m_ptestDesc.m_groupColors.Add(_T("#800000"));
    m_ptestDesc.m_groupColors.Add(_T("#008000"));
    m_ptestDesc.m_groupColors.Add(_T("#000080"));
    m_ptestDesc.m_groupColors.Add(_T("#808080"));
    m_ptestDesc.m_groupColors.Add(_T("#808000"));


    m_ptestDesc.m_title = _T("Injector group table (red arrow shows order of firing)");
    m_ptestDesc.m_help = _T("Injector outputs");
    m_ptestDesc.m_name = _T("injOutVisual");

    return tmp;
}

void CheckBoxMatrixViewer::OnComboBox( wxCommandEvent& event )
{
    bool show = ( m_testMode->GetValue() == _T("ON") );
    m_checkBoxMatrixpanel->showTestButtons( show );
}

void CheckBoxMatrixViewer::setOutputName( int row, const wxString& value, bool isConflicted, wxString conflicts )
{
    m_checkBoxMatrixpanel->setOutput( row, value, isConflicted, conflicts );
}
