#ifndef CURVEEDITOR_H
#define CURVEEDITOR_H

#include "StaticViewersController.h"
#include "../Symbol.h"
#include "../rtdatamanager/RTDataViewerBase.h"
#include "../stdatamanager/STDataViewerBase.h"

class SymbolMgr;
class VemsCurveDialogDesc;
class AggGraphControl;
class CurveEditor;
class UpdateViewerEvent;
class FirmwareConfigLoader;
class STDataModelBase;
class MTConfigData;
class wxHotKeyManager;

/**********************************************************
*   STArrayViewerTextCtrl
**********************************************************/
class STArrayViewerTextCtrl : public wxTextCtrl, public STDataViewerBase
{
    public:
        STArrayViewerTextCtrl( wxWindow* parent, wxWindowID id, const wxString& value = _T("")
                               , const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize
                               , long style = 0, const wxValidator& validator = wxDefaultValidator
                               , const wxString& name = wxTextCtrlNameStr );

        void setIndex( uint32_t rowIdx, uint32_t colIdx ); //set viewed element index default = 0;
        void setCurveEditor( CurveEditor* curve ) { m_curveEditor = curve; }

        void update();
        void requestRead();
        void updateModel();
        void requestWrite();

        void setFormat( int digits_in );
        uint32_t getValueIndex();   //get watched array index, depend by dimension

        void getItemPosition( size_t& row, size_t& col );

        void setSymbol( const Symbol& symbol_in ) { m_symbol = symbol_in; }
        wxString toString();

        void setSmallStep( double step ) { m_smallStep = step; }
        void setLargeStep( double step ) { m_largeStep = step; }
        void setHotKeyManager( wxHotKeyManager* hotkeyManager ) { m_hotkeyMgr = hotkeyManager; }

    private:
        CurveEditor*    m_curveEditor;
        wxString        m_format;
        uint32_t        m_rowIdx;
        uint32_t        m_colIdx;
        Symbol          m_symbol;
        wxString        m_expr;
        bool            m_isFirstKillFocus;
        double          m_smallStep;
        double          m_largeStep;

        wxHotKeyManager* m_hotkeyMgr;

        void changeValueWith( double change );

        void OnKillFocus( wxFocusEvent& event );
        void OnSetFocus( wxFocusEvent& event );
        void OnKeyDown( wxKeyEvent& event );

    DECLARE_EVENT_TABLE()
};

WX_DEFINE_ARRAY( STArrayViewerTextCtrl*, STArrayViewerTextCtrlA );

/**********************************************************
*   CurveEditor
**********************************************************/
class CurveEditor : public wxPanel, public RTDataViewerBase, public StaticViewersController
{
    DECLARE_DYNAMIC_CLASS(CurveEditor);

    public:
        CurveEditor() {}
        CurveEditor( wxWindow* parent, VemsCurveDialogDesc* desc, SymbolMgr* symbolMgr);
        ~CurveEditor();

        bool init( STDataMgrBase* stController, RTDataMgrBase* rtController );
        const STArrayViewerTextCtrlA& getXAxisViewers() { return m_xAxisViewers; }
        const STArrayViewerTextCtrlA& getYAxisViewers() { return m_yAxisViewers; }
        void update();

        //RTDataViewerBase virtual functions
        void refresh( DoubleHash* table );
        void require();
        void unregisterSymbols();
        void SetValue( double value ) {}
        void SetValueIdx(int idx) {}

        void OnUpdateValue( UpdateViewerEvent& event );
        bool onDropFiles( const wxArrayString& filenames );
        void importFromFile( const wxString& file );
        wxString getDialogName() const;
    private:
        SymbolMgr*              m_symbolMgr;
        AggGraphControl*        m_curve;
        VemsCurveDialogDesc*    m_curveDesc;    // curve editor descriptor
        STArrayViewerTextCtrlA  m_xAxisViewers; // x axis static viewers
        STArrayViewerTextCtrlA  m_yAxisViewers; // y axis static viewers

        Symbol*                 m_xBinSymbol;
        wxString                m_yBinName;
        wxString                m_xRealTimeName;
        wxString                m_yRealTimeName;
        wxString                m_seqXAxisName;

        //Symbol*                 m_xBins;        // x axis array symbol
        //Symbol*                 m_yBins;        // y axis array symbol
        //Symbol*                 m_xRealTime;    // x real time symbol
        bool                    m_isSymbolRegistered;

        MTConfigData*           m_mtConfiData;

        wxHotKeyManager*        m_hotkeyMgr;

        void buildGUI( STDataMgrBase* stController, RTDataMgrBase* rtController );
        void addConfigletPanel();

        void loadStaticViewers( FirmwareConfigLoader& configLoader );
        void loadModelFromDialog( STDataModelBase& model );

        void OnMouseWheel( wxMouseEvent& event );
        DECLARE_EVENT_TABLE();
};
#endif // CURVEEDITOR_H
