#include "DiffGaugeDescEditor.h"

#include "../logger.h"

#include <wx/statline.h>

#include "../customwidgets/wxTextCtrlNullEx.h"
#include "../gaugedesc/DiffGaugeDescManager.h"
#include "../gaugedesc/GaugeDescManager.h"

#include "../customwidgets/ColorSetupPanel.h"
#include "../customwidgets/SymbolChooserPanel.h"
#include "../wxVariantHelper.h"
#include "../events/EditorValueChangedEvent.h"
#include "../DrawHelper.h"
#include "../wxSizerHelper.h"
#include "../vemsgauge/DifferentialGauge.h"

BEGIN_EVENT_TABLE(DiffGaugeDescEditor,wxFrame)

    EVT_LISTBOX(idProfListBox,DiffGaugeDescEditor::OnProfListBox )

    EVT_BUTTON(idBtnDelProf, DiffGaugeDescEditor::OnButton)
    EVT_BUTTON(idBtnEditProf, DiffGaugeDescEditor::OnButton)
    EVT_BUTTON(idBtnAppendProf, DiffGaugeDescEditor::OnButton)
    EVT_BUTTON(idBtnCloseEditor, DiffGaugeDescEditor::OnButton)
    EVT_BUTTON(idBtnAddSymbol, DiffGaugeDescEditor::OnButton)
    EVT_BUTTON(idBtnSaveProfileProps, DiffGaugeDescEditor::OnButton)
    EVT_BUTTON(idBtnOK, DiffGaugeDescEditor::OnButton)

    EVT_CLOSE(DiffGaugeDescEditor::OnClose)

END_EVENT_TABLE()

DiffGaugeDescEditor::DiffGaugeDescEditor(wxWindow* parent, GaugeDescManager* gdMgr, bool isShowProfiles, DifferentialGauge* diffGauge)
{
    //ctor
    Create(parent, wxID_ANY, _T("Differential gauge descriptor editor"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_FRAME_STYLE|wxTAB_TRAVERSAL);
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
    ClearBackground();
    m_isShowProfiles = isShowProfiles;
    SetIcon(wxIcon(_T("vemsLogoIcon2")));

    m_diffGauge = diffGauge;
    m_currProfSizer = NULL;

    m_profilesList = NULL;
    m_newProfileCtrl = NULL;
    m_currentProfileCtrl = NULL;
    m_setProfNameBtn = NULL;
    m_savePropBtn = NULL;
    m_profilePropsPanel = NULL;
    m_currentProfileMinVal = NULL;
    m_currentProfileMaxVal = NULL;

    m_gdMgr = gdMgr;

    m_diffMgr = NULL;//m_gdMgr->GetDiffGaugeDescMgr();

    m_db.Open( m_diffMgr->GetLastDbFile() );

    CreateStateStructDesc();

    m_profPropLinePadding = 3;
    m_lastProfPropLineRowIdx = 1;

    Show( BuildGUI( diffGauge == NULL ? -1 : diffGauge->GetProfileID() ) );
}

DiffGaugeDescEditor::~DiffGaugeDescEditor()
{
    //dtor
}

void DiffGaugeDescEditor::OnClose(wxCloseEvent& evt)
{
    if(m_savePropBtn && m_savePropBtn->IsEnabled()) {
        if(wxMessageBox(_T("Do you want to save the changes?"),_T("Warning"), wxYES_NO | wxICON_QUESTION ) != wxNO) {
            SaveProfileProps();
        }
    }

    evt.Skip();

    if(m_diffGauge) m_diffGauge->SetFocus();
}


bool DiffGaugeDescEditor::BuildGUI(int profileId)
{
    Int2DiffGaugeDescHash& profiles = m_diffMgr->GetProfiles();

    if(profiles.empty()) {
        wxString msg = _T("Not found any profile!\nYou should open a log or vemscfg, or connect an ECU.");
        wxMessageBox(msg,_T("Differential gauge descriptor editor warning"), wxOK | wxICON_INFORMATION);
    }


    m_profilesList = new wxListBox(this, idProfListBox, wxDefaultPosition, wxDefaultSize, 0, NULL, wxLB_EXTENDED | wxLB_NEEDED_SB | wxLB_SORT);

    int selectedProfile = 0;

    for(Int2DiffGaugeDescHash::iterator it = profiles.begin(); it != profiles.end(); it++ ) {

        DiffGaugeDesc& desc = it->second;

        m_profilesList->Append( desc.GetName() );

        if( profileId != -1 && desc.GetId() == profileId ) selectedProfile = m_profilesList->GetCount()-1;
    }

    wxBoxSizer * mainsizer = new wxBoxSizer(wxVERTICAL);

    wxBoxSizer * editsizer = new wxBoxSizer(wxHORIZONTAL);

    if(m_isShowProfiles) {

            wxStaticBoxSizer * sb1 = new wxStaticBoxSizer(wxVERTICAL, this, _T("Profiles"));

            sb1->Add( m_profilesList, 1, wxEXPAND | wxALL, 5 );

                wxBoxSizer * profeditsizer = new wxBoxSizer(wxVERTICAL);

                    wxStaticBoxSizer * sbadd = new wxStaticBoxSizer(wxVERTICAL, this, _T("New profile"));
                    m_newProfileCtrl = new wxTextCtrl(this, wxID_ANY, wxEmptyString);
                    sbadd->Add( m_newProfileCtrl, 1, wxALL, 3 );
                    sbadd->Add( new wxButton(this, idBtnAppendProf, _T("Append")), 0, wxALL | wxALIGN_CENTER_HORIZONTAL , 3 );

                profeditsizer->Add(sbadd, 0, wxALL, 2);

                profeditsizer->Add( new wxButton(this, idBtnDelProf, _T("Remove")), 0, wxALL | wxALIGN_CENTER_HORIZONTAL , 3  );

            sb1->Add( profeditsizer, 0, wxALL, 2 );

        editsizer->Add( sb1, 0, wxALL | wxEXPAND, 5 );

    } else {
        m_profilesList->Hide();
    }

        wxStaticBoxSizer * sb2 = new wxStaticBoxSizer(wxVERTICAL, this, _T("Profile properties"));

        wxFlexGridSizer* profpropsizer = new wxFlexGridSizer(3);

        profpropsizer->Add(new wxStaticText(this, wxID_ANY, _T("Name")), 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 3);
        profpropsizer->Add(new wxStaticText(this, wxID_ANY, _T("Minimum value")), 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 3);
        profpropsizer->Add(new wxStaticText(this, wxID_ANY, _T("Maximum value")), 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 3);

        wxTextValidator tv(wxFILTER_NUMERIC);


        m_currentProfileCtrl = new wxTextCtrlNullEx(this, wxID_ANY, wxEmptyString);
        m_currentProfileMinVal = new wxTextCtrlNullEx(this, wxID_ANY, _T(""));
        m_currentProfileMaxVal = new wxTextCtrlNullEx(this, wxID_ANY, _T(""));
        m_currentProfileMinVal->SetValidator(tv);
        m_currentProfileMaxVal->SetValidator(tv);



        profpropsizer->Add(m_currentProfileCtrl, 1, wxALL, 3);
        profpropsizer->Add(m_currentProfileMinVal, 0, wxALL, 3);
        profpropsizer->Add(m_currentProfileMaxVal, 0, wxALL, 3);

        sb2->Add(profpropsizer, 0, wxEXPAND | wxALL | wxALIGN_RIGHT, 5);

        m_profilePropsPanel = new wxPanel(this, wxID_ANY);

        wxBoxSizer * fssizer = new wxBoxSizer(wxVERTICAL);

        m_profilePropsPanel->SetSizer(fssizer);

        sb2->Add(m_profilePropsPanel, 1, wxEXPAND | wxALL, 5);


    editsizer->Add( sb2, 1, wxALL | wxEXPAND, 5 );

    mainsizer->Add(editsizer, 1, wxEXPAND);

    wxBoxSizer* buttonsizer = new wxBoxSizer(wxHORIZONTAL);

    wxButton* okbutton = new wxButton(this,idBtnOK,_T("OK"));
    wxButton* closebutton = new wxButton(this,idBtnCloseEditor,_T("Close"));
    okbutton->SetToolTip(_T("Save the changes and close"));

    buttonsizer->Add(okbutton,0,wxALL,5);
    buttonsizer->Add(closebutton,0,wxALL,5);

    mainsizer->Add(buttonsizer, 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 5 );

    SetSizer(mainsizer);
    mainsizer->Fit(this);
    mainsizer->SetSizeHints(this);


    m_profilesList->Select(selectedProfile);

    LoadProfileProps( GetSelectedProfile() );

    Connect( EditorValueChangedEvent, wxCommandEventHandler(DiffGaugeDescEditor::OnChangeCurrentProfCtrl) );

    return true;
}


void DiffGaugeDescEditor::OnButton(wxCommandEvent& evt)
{
    switch( evt.GetId() ) {

        case idBtnDelProf:
            DelSelectedProfiles();
            break;

        case idBtnEditProf:
            SaveProfileName();
            break;

        case idBtnAppendProf:
            AddNewProfile();
            break;

        case idBtnCloseEditor:
            Close();
            break;

        case idBtnAddSymbol:
            AddNewSymbolLine();
            break;

        case idBtnSaveProfileProps:
            SaveProfileProps();
            break;

        case idBtnOK:
            SaveProfileProps();
            Close();
            break;
    }

}

void DiffGaugeDescEditor::DelSelectedProfiles()
{
    wxArrayInt arr;
    m_profilesList->GetSelections(arr);

    if(!arr.GetCount()) return;

    wxString msg = wxString::Format(_T("Do you really want to delete the %i selected profile(s)?"),arr.GetCount());

    if( wxMessageBox(msg,_T("Confirm"), wxYES_NO | wxICON_QUESTION ) == wxNO ) return;

    Int2DiffGaugeDescHash& profiles = m_diffMgr->GetProfiles();

    m_db.Begin();

    for(int a = arr.GetCount()-1; a >= 0; a-- ) {
        int idx = arr[a];
        wxString selectedProfName = m_profilesList->GetString(idx);

        if( !m_diffMgr->IsProfileExists(selectedProfName) ) continue;

        DiffGaugeDesc& profile = m_diffMgr->GetProfile(selectedProfName);

        int prof_id = profile.GetId();

        profiles.erase(prof_id);

        m_db.ExecuteUpdate(wxString::Format(_T("delete from profiles where id=%i;"),prof_id));
        m_db.ExecuteUpdate(wxString::Format(_T("delete from symbols where profile_id=%i;"),prof_id));

        m_profilesList->Delete(idx);
    }

    m_db.Commit();

    m_profilesList->Select(0);

    m_profilePropsPanel->DestroyChildren();
    wxSizer* fmainsizer = m_profilePropsPanel->GetSizer();
    fmainsizer->Clear();

    LoadProfileProps( GetSelectedProfile() );
}

void DiffGaugeDescEditor::AddNewProfile()
{
    wxString newprofname = m_newProfileCtrl->GetValue();

    if( newprofname == wxEmptyString ) return;

    Int2DiffGaugeDescHash& profiles = m_diffMgr->GetProfiles();

    if( m_diffMgr->IsProfileExists(newprofname) ) {
        wxMessageBox(_T("This name is engaged.\nPlease choose an other one."),_T("Warning"));
        return;
    }

    m_db.ExecuteUpdate( wxString::Format(_T("insert into profiles (name) values ('%s');"),newprofname.c_str()) );

    long prof_id = m_db.GetLastRowId().ToLong();

    DiffGaugeDesc pdesc(prof_id, newprofname, 0.0, 1.0);

    profiles[prof_id] = pdesc;

    m_profilesList->AppendString( newprofname );
}

wxString DiffGaugeDescEditor::GetSelectedProfile()
{
    wxArrayInt arr;
    m_profilesList->GetSelections(arr);
    if(arr.GetCount() < 1) return wxEmptyString;

    return m_profilesList->GetString( arr[0] );
}


void DiffGaugeDescEditor::AddNewSymbolLine()
{
    wxString currname = GetSelectedProfile();

    if( currname == wxEmptyString ) return;

    DiffGaugeDesc& desc = m_diffMgr->GetProfile(currname);

    long prof_id = desc.GetId();

    m_db.ExecuteUpdate(wxString::Format(_T("insert into symbols (profile_id,symbol_name) values(%i,'rpm');"),prof_id));

    long symbolLine_id = m_db.GetLastRowId().ToLong();

    wxVariantHash symbolprop;

    symbolprop[_T("id")] = symbolLine_id;
    symbolprop[_T("profile_id")] = prof_id;
    symbolprop[_T("symbol_name")] = _T("rpm");

    desc.AddSymbolProps(symbolprop);

    AddPropLine(symbolprop, m_lastProfPropLineRowIdx+1, m_profPropLinePadding);

    RefreshProfilePropPanel();

    RefreshFrameLayout();
}

void DiffGaugeDescEditor::RefreshProfilePropPanel()
{

    m_currProfSizer->RecalcSizes();
    m_currProfSizer->Layout();


    wxSizer* fmainsizer = m_profilePropsPanel->GetSizer();
    fmainsizer->Layout();
    //fmainsizer->Fit(m_profilePropsPanel);
    //fmainsizer->SetSizeHints(m_profilePropsPanel);

    m_profilePropsPanel->Refresh(false);

    GetSizer()->RecalcSizes();
    GetSizer()->Layout();
    //GetSizer()->Fit(this);
    //GetSizer()->SetSizeHints(this);

}

void DiffGaugeDescEditor::SaveProfileProps()
{
    m_db.Begin();

    DiffGaugeDesc& profile = m_diffMgr->GetProfile( GetSelectedProfile() );

    for(GUIDescGUIItemArrayIntHash::iterator it = m_profPropGUIHash.begin(); it != m_profPropGUIHash.end(); it++) {
        long symbol_id = it->first;
        GUIDescGUIItemArray& descarr = it->second;

        bool isChangedState = false;

        wxString qs = _T("update symbols set ");

        wxVariantHash& symbolProp = profile.GetSymbol(symbol_id);

        for(size_t a=0; a<descarr.GetCount(); a++) {
            GUIDescGUIItem& guidesc = descarr[a];

            if( !guidesc.IsChanged() ) continue;

            wxVariant val = guidesc.GetValue();

            qs += guidesc.Name + _T(" = '");

            qs += wxVariantHelper::MakeString(val) + _T("',");

            symbolProp[guidesc.Name] = val;

            guidesc.ResetChange();

            isChangedState = true;
        }

        qs.RemoveLast();

        qs += wxString::Format(_T(" where id=%i;"),symbol_id);

        Logger::logf(LOG_MSG,_T("isChangedState = %i, qs=")+qs,isChangedState);

        if(isChangedState)
            m_db.ExecuteUpdate(qs);
    }

    double minv,maxv;

    m_currentProfileMinVal->GetValue().ToDouble(&minv);
    m_currentProfileMaxVal->GetValue().ToDouble(&maxv);

    profile.SetProp(_T("min_value"),minv);
    profile.SetProp(_T("max_value"),maxv);

    m_db.ExecuteUpdate(wxString::Format(_T("update profiles set min_value=%f, max_value=%f, name='%s' where id = %i;"),
                                                minv,maxv,m_currentProfileCtrl->GetValue().c_str(),profile.GetId()));


    if(m_currentProfileCtrl->IsChanged()) {
        wxArrayInt arr;
        m_profilesList->GetSelections(arr);
        if(arr.GetCount()){
            m_profilesList->SetString(arr[0], m_currentProfileCtrl->GetValue());
        }
        profile.SetProp(_T("name"),m_currentProfileCtrl->GetValue());
    }

    m_currentProfileMinVal->ResetChange();
    m_currentProfileMaxVal->ResetChange();
    m_currentProfileCtrl->ResetChange();

    m_db.Commit();

    m_savePropBtn->Enable(false);

    if(m_diffGauge) m_diffGauge->RedrawWidget();
}

void DiffGaugeDescEditor::OnProfListBox(wxCommandEvent& evt)
{
    LoadProfileProps( evt.GetString() );
}

void DiffGaugeDescEditor::SaveProfileName()
{

}

void DiffGaugeDescEditor::OnChangeCurrentProfCtrl(wxCommandEvent& evt)
{
    m_savePropBtn->Enable(true);
}

void DiffGaugeDescEditor::CreateStateStructDesc()
{
    m_profPropGuiDesc.Add( GUIDescStructItem(_T("symbol_name"), GDSI_SYMBOLNAME, true, false, _T("Symbol name"),_T("")) );
    m_profPropGuiDesc.Add( GUIDescStructItem(_T("color"), GDSI_COLOR, false, false, _T("Color"),_T("")) );
}


bool DiffGaugeDescEditor::LoadProfileProps(const wxString& name, bool checkBlink)
{
    if( !m_diffMgr->IsProfileExists(name) ) {
        m_currentProfileMinVal->Hide();
        m_currentProfileMaxVal->Hide();
        m_currentProfileCtrl->Hide();
        return false;
    }

    DiffGaugeDesc& desc = m_diffMgr->GetProfile(name);

    if( checkBlink && m_currentProfileCtrl->GetValue() == name ) return false;

    m_profilePropsPanel->Hide();

    m_profilePropsPanel->DestroyChildren();
    m_currentProfileCtrl->ChangeValueEx(name);
    m_currentProfileCtrl->ResetChange();
    //m_setProfNameBtn->Enable(false);

    wxSizer* fmainsizer = m_profilePropsPanel->GetSizer();
    fmainsizer->Clear();

    wxGridBagSizer * gbsizer = new wxGridBagSizer();

    m_currProfSizer = gbsizer;

    m_currentProfileMinVal->Show(1);
    m_currentProfileMaxVal->Show(1);

    m_currentProfileMinVal->ChangeValueEx( wxString::Format(_T("%.3f"),desc.GetMinValue() ), false );
    m_currentProfileMaxVal->ChangeValueEx( wxString::Format(_T("%.3f"),desc.GetMaxValue() ), false );


    m_currProfSizer->AddGrowableCol(0);

    int padding = m_profPropLinePadding;

    size_t colc = m_profPropGuiDesc.GetCount();

    int row = 0;

    for(size_t a = 0; a < colc; a++ ) {
        GUIDescStructItem& gdi = m_profPropGuiDesc[a];

        if( gdi.ShowOnEditor == false ) continue;

        wxStaticText * text = new wxStaticText(m_profilePropsPanel, wxID_ANY, gdi.Title);
        wxFont font = text->GetFont();
        font.SetWeight( wxFONTWEIGHT_BOLD );
        text->SetFont(font);

        gbsizer->Add( text, wxGBPosition(row,a), wxDefaultSpan, wxALL | wxALIGN_CENTER_HORIZONTAL, padding );
    }

    row++;

    gbsizer->Add( new wxStaticLine(m_profilePropsPanel, wxID_ANY) , wxGBPosition(row,0), wxGBSpan( 1, colc+1 ), wxALL | wxEXPAND, padding );

    row++;

    m_profPropGUIHash.clear();
    m_delBtnId2ProfPropId.clear();

    wxVariantHashArray& sarr = desc.GetSymbols();

    for(size_t a=0; a<sarr.GetCount(); a++ ) {

        AddPropLine(sarr[a], row, padding);

        row++;
    }

    fmainsizer->Add(gbsizer,0,wxEXPAND);


    wxButton * ansbtn = new wxButton(m_profilePropsPanel, idBtnAddSymbol, _T("Add new symbol line"));
    fmainsizer->Add( ansbtn, 0, wxALL | wxALIGN_CENTER_HORIZONTAL, padding );
    fmainsizer->AddStretchSpacer();

    fmainsizer->Add( new wxStaticLine(m_profilePropsPanel, wxID_ANY), 0, wxALL | wxEXPAND, padding );
    m_savePropBtn = new wxButton(m_profilePropsPanel, idBtnSaveProfileProps, _T("Save profile properties"));
    fmainsizer->Add( m_savePropBtn, 0, wxALL | wxALIGN_CENTER_HORIZONTAL, padding );
    m_savePropBtn->Enable(false);

    fmainsizer->Layout();
    fmainsizer->Fit(m_profilePropsPanel);

    m_profilePropsPanel->Show();

    RefreshFrameLayout();


    return true;
}

void DiffGaugeDescEditor::RefreshFrameLayout()
{
    wxSizer* mainsizer = GetSizer();

    if(!mainsizer) return;

    mainsizer->RecalcSizes();
    mainsizer->Layout();

    wxSize minsize = mainsizer->ComputeFittingWindowSize(this);

    SetSizeHints(minsize);

    wxSize size = GetSize();

    if(size.x < minsize.x || size.y < minsize.y) {
        wxSize newsize;
        newsize.x = wxMax(minsize.x,size.x);
        newsize.y = wxMax(minsize.y,size.y);
        SetSize(newsize);
    }
}

void DiffGaugeDescEditor::AddPropLine(wxVariantHash& props, int row, int padding)
{

    int symbol_id = props[_T("id")].GetLong();

    wxWindow * win = NULL;

    size_t colc = m_profPropGuiDesc.GetCount();

    for(size_t a = 0; a < colc; a++ ) {
        GUIDescStructItem& gdi = m_profPropGuiDesc[a];

        if( gdi.ShowOnEditor == false ) continue;

        switch( gdi.Type ) {

            case GDSI_COLOR:{
                wxColor c = wxNullColour;
                if(props.find(gdi.Name) != props.end() ) c = DrawHelper::ParseExtColorStr(props[gdi.Name].GetString());
                win = new ColorSetupPanel(m_profilePropsPanel, c, wxC2S_HTML_SYNTAX );
                }break;

            case GDSI_NUMBER:{
                wxTextCtrlNullEx * ctrl = new wxTextCtrlNullEx(m_profilePropsPanel, wxID_ANY, props[gdi.Name].GetString() );
                wxTextValidator validator(wxFILTER_NUMERIC);
                ctrl->SetValidator(validator);
                win = ctrl;
                }break;

            case GDSI_TEXT:
                win = new wxTextCtrlNullEx(m_profilePropsPanel, wxID_ANY, props[gdi.Name].GetString() );
                break;

            case GDSI_SYMBOLNAME:{
                SymbolChooserPanel * panel = new SymbolChooserPanel(m_profilePropsPanel,m_gdMgr);
                panel->SetSymbolName( props[gdi.Name].GetString(), false );
                win = panel;
                }break;
        }

        m_currProfSizer->Add( win, wxGBPosition(row,a), wxDefaultSpan, wxALL | wxALIGN_CENTER_VERTICAL | wxEXPAND, padding );

        m_profPropGUIHash[symbol_id].Add( GUIDescGUIItem(gdi.Name, gdi.Type, win) );

    }

    int did = wxNewId();
    wxBitmapButton * btn = new wxBitmapButton(m_profilePropsPanel, did, DrawHelper::GetBitmap(_T("remove-16.png")) );
    m_currProfSizer->Add( btn, wxGBPosition(row,colc), wxDefaultSpan, wxALL, padding );
    btn->Connect(wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( DiffGaugeDescEditor::OnProfPropLineDelButton ), NULL, this );
    btn->SetToolTip(_T("Delete symbol"));

    m_delBtnId2ProfPropId[did] = symbol_id;

    m_lastProfPropLineRowIdx = row;
}

void DiffGaugeDescEditor::OnProfPropLineDelButton(wxCommandEvent& evt)
{
    int id = evt.GetId();

    if( m_delBtnId2ProfPropId.find(id) == m_delBtnId2ProfPropId.end() ) return;

    int symbol_id = m_delBtnId2ProfPropId[id];

    if( m_profPropGUIHash.find(symbol_id) == m_profPropGUIHash.end() ) return;

    wxString currname = GetSelectedProfile();

    if( currname == wxEmptyString ) return;

    DiffGaugeDesc& profile = m_diffMgr->GetProfile(currname);
    wxVariantHashArray& symbols = profile.GetSymbols();

    size_t size = symbols.GetCount();

    for(size_t a=0; a< symbols.GetCount(); a++ ){
        if( symbols[a][_T("id")].GetLong() != symbol_id ) continue;

        symbols.RemoveAt(a);
        break;
    }

    if( size == symbols.GetCount() ) return;

    m_db.ExecuteUpdate( wxString::Format(_T("delete from symbols where id=%i;"),symbol_id) );

    GUIDescGUIItemArray& garr = m_profPropGUIHash[symbol_id];

    if(garr.GetCount() < 1 || garr[0].Ctrl == NULL ) return;

    wxWindow * win = garr[0].Ctrl;

    if( m_currProfSizer == NULL ) return;

    wxSizerHelper::DeleteRow(m_currProfSizer, win);

    m_profPropGUIHash.erase(symbol_id);

    m_lastProfPropLineRowIdx--;

    RefreshProfilePropPanel();

    RefreshFrameLayout();
}

