#include "DifferentConfigDialog.h"

#include "../DrawHelper.h"

#include "../bcWidgets/bcLabel.h"
#include "../bcWidgets/bcStaticBitmap.h"
#include "../bcWidgets/bcWrapper.h"


BEGIN_EVENT_TABLE(DifferentConfigDialog, wxDialog)

    EVT_RADIOBUTTON(idRbWrite, DifferentConfigDialog::OnRadio)
    EVT_RADIOBUTTON(idRbRefresh, DifferentConfigDialog::OnRadio)

END_EVENT_TABLE()


DifferentConfigDialog::DifferentConfigDialog(wxWindow* parent, int id, const wxString& title)
{
    Create(parent,id,title);

    BuildGui();
}

DifferentConfigDialog::~DifferentConfigDialog()
{
    //dtor
}

int DifferentConfigDialog::GetChoosed()
{
    if(m_rbWrite->GetValue()) return DCR_WRITE;

    if(m_rbRefresh->GetValue()) return DCR_REFRESH;

    return DCR_WRITE;
}

void DifferentConfigDialog::OnRadio(wxCommandEvent& evt)
{
    int target = DCR_WRITE;
    switch(evt.GetId()){
        case idRbWrite: target = DCR_WRITE; break;
        case idRbRefresh: target = DCR_REFRESH; break;
    }

    m_headPanel->SetTarget(target);
}

void DifferentConfigDialog::BuildGui()
{
    wxBoxSizer* mainsizer = new wxBoxSizer(wxVERTICAL);

    m_headPanel = new DcHeadPanel(this, wxID_ANY);

    mainsizer->Add(m_headPanel, 0, wxEXPAND);

    wxBoxSizer* radiosizer = new wxBoxSizer(wxVERTICAL);

    m_rbWrite = new wxRadioButton(this, idRbWrite, _T("Write changes to ECU(recommended) "));
    radiosizer->Add(m_rbWrite, 0 , wxALL , 5 );
    radiosizer->AddSpacer(20);

    m_rbRefresh = new wxRadioButton(this, idRbRefresh, _T("Drop changes and refresh config from ECU"));
    radiosizer->Add(m_rbRefresh, 0 , wxALL , 5 );
    radiosizer->AddSpacer(20);

    mainsizer->Add(radiosizer, 0, wxALL, 15);

    mainsizer->Add( CreateStdDialogButtonSizer(wxOK | wxCANCEL), 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 10 );

    SetSizer(mainsizer);
    mainsizer->SetSizeHints(this);

    m_rbWrite->SetValue(true);

}

wxString DcHeadPanel::GetTargetIconFile(int target)
{
    switch(target){
        case DCR_WRITE: return _T("laptop2ecu-a.png");
        case DCR_REFRESH: return _T("laptop2ecu-b.png");
    }
    return wxEmptyString;
}

void DcHeadPanel::SetTarget(int target)
{
    m_icon->SetLabel( DrawHelper::GetImage(GetTargetIconFile(target)) );
}

DcHeadPanel::DcHeadPanel(wxWindow* parent, int id, int target)
    :bcParentPanel(parent, id)
{
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));

    wxBoxSizer* mainsizer = new wxBoxSizer(wxHORIZONTAL);

    mainsizer->AddSpacer(30);

    bcLabel * label = new bcLabel(this, wxID_ANY, _T("Config has been modified."));
    label->SetProperty( BCBS_BORDER_WIDTH, 0.0, true );

    mainsizer->Add( new bcWrapper(label), 0, wxALIGN_CENTER_VERTICAL );

    m_icon = new bcStaticBitmap(this, wxID_ANY, DrawHelper::GetImage( GetTargetIconFile(target) ) );

    mainsizer->AddStretchSpacer();

    mainsizer->Add( new bcWrapper(m_icon), 0, wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT | wxRIGHT, 10 );

    SetSizer(mainsizer);
}
