#include "MultiGaugeDescEditor.h"

#include "../logger.h"

#include <wx/statline.h>

#include "../customwidgets/wxTextCtrlNullEx.h"
#include "../gaugedesc/MultiSymbolGaugeDescMgr.h"
#include "../gaugedesc/GaugeDescManager.h"

#include "../customwidgets/ColorSetupPanel.h"
#include "../customwidgets/SymbolChooserPanel.h"
#include "../wxVariantHelper.h"
#include "../events/EditorValueChangedEvent.h"
#include "../DrawHelper.h"
#include "../wxSizerHelper.h"

#include "../vemsgauge/VemsWidgetBase.h"

#include "../common/types/wxVariantHashClientData.h"

BEGIN_EVENT_TABLE(MultiGaugeDescEditor,wxFrame)

    EVT_LISTBOX(idProfListBox,MultiGaugeDescEditor::OnProfListBox )

    EVT_BUTTON(idBtnDelProf, MultiGaugeDescEditor::OnButton)
    EVT_BUTTON(idBtnEditProf, MultiGaugeDescEditor::OnButton)
    EVT_BUTTON(idBtnAppendProf, MultiGaugeDescEditor::OnButton)
    EVT_BUTTON(idBtnCloseEditor, MultiGaugeDescEditor::OnButton)
    EVT_BUTTON(idBtnAddSymbol, MultiGaugeDescEditor::OnButton)
    EVT_BUTTON(idBtnSaveProfileProps, MultiGaugeDescEditor::OnButton)
    EVT_BUTTON(idBtnOK, MultiGaugeDescEditor::OnButton)

    EVT_CLOSE(MultiGaugeDescEditor::OnClose)

END_EVENT_TABLE()

MultiGaugeDescEditor::MultiGaugeDescEditor(wxWindow* parent, const wxString& title, GaugeDescManager* gdMgr, MultiSymbolGaugeDescMgr* mgr, bool isShowProfiles, VemsWidgetBase* gauge, bool showMinMax, int profid)
{
    //ctor
    Create(parent, wxID_ANY, title, wxDefaultPosition, wxDefaultSize, wxDEFAULT_FRAME_STYLE|wxTAB_TRAVERSAL);
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
    ClearBackground();
    m_isShowProfiles = isShowProfiles;
    SetIcon(wxIcon(_T("vemsLogoIcon2")));
    m_isShowMinMax = showMinMax;

    //m_profPropGuiDesc = profileSettingsGUIStruct;
    m_profiledId = -1;
    m_gauge = gauge;
    m_currProfSizer = NULL;

    m_profilesList = NULL;
    m_newProfileCtrl = NULL;
    m_currentProfileCtrl = NULL;
    m_setProfNameBtn = NULL;
    m_savePropBtn = NULL;
    m_profilePropsPanel = NULL;
    m_currentProfileMinVal = NULL;
    m_currentProfileMaxVal = NULL;

    m_gdMgr = gdMgr;
    m_multiSymbolMgr = mgr;

    m_db.Open( m_multiSymbolMgr->GetLastDbFile() );

    CreateStateStructDesc();

    m_profPropLinePadding = 3;
    m_lastProfPropLineRowIdx = 1;

    Show( BuildGUI( profid ) );
}

MultiGaugeDescEditor::~MultiGaugeDescEditor()
{
    //dtor
}

void MultiGaugeDescEditor::OnClose(wxCloseEvent& evt)
{
    if(m_savePropBtn && m_savePropBtn->IsEnabled()) {
        if(wxMessageBox(_T("Do you want to save the changes?"),_T("Warning"), wxYES_NO | wxICON_QUESTION ) != wxNO) {
            SaveProfileProps();
        }
    }

    evt.Skip();

    if(m_gauge) m_gauge->SetFocus();
}


bool MultiGaugeDescEditor::BuildGUI(int profileId)
{
    m_profiledId = profileId;

    Int2MultiSymbolGaugeDescHash& profiles = m_multiSymbolMgr->GetProfiles();

    if(profiles.empty()) {
        wxString msg = _T("Not found any profile!\nYou should open a log or vemscfg, or connect an ECU.");
        wxMessageBox(msg,_T("Differential gauge descriptor editor warning"), wxOK | wxICON_INFORMATION);
    }


    m_profilesList = new wxListBox(this, idProfListBox, wxDefaultPosition, wxDefaultSize, 0, NULL, wxLB_EXTENDED | wxLB_NEEDED_SB | wxLB_SORT);

    int selectedProfile = 0;

    for(Int2MultiSymbolGaugeDescHash::iterator it = profiles.begin(); it != profiles.end(); it++ ) {

        MultiSymbolGaugeDesc& desc = it->second;

        wxVariantHash deschash;

        deschash[_T("name")] = desc.GetName();
        deschash[_T("id")] = (long)desc.GetId();

        wxVariantHashClientData* data = new wxVariantHashClientData(deschash);

        m_profilesList->Append( desc.GetName(), data );

        if( profileId != -1 && desc.GetId() == profileId ) {
            selectedProfile = m_profilesList->GetCount()-1;
    }
    }


    wxBoxSizer * mainsizer = new wxBoxSizer(wxVERTICAL);

    wxBoxSizer * editsizer = new wxBoxSizer(wxHORIZONTAL);

    if(m_isShowProfiles) {

            wxStaticBoxSizer * sb1 = new wxStaticBoxSizer(wxVERTICAL, this, _T("Profiles"));

            sb1->Add( m_profilesList, 1, wxEXPAND | wxALL, 5 );

                wxBoxSizer * profeditsizer = new wxBoxSizer(wxVERTICAL);

                    wxStaticBoxSizer * sbadd = new wxStaticBoxSizer(wxVERTICAL, this, _T("New profile"));
                    m_newProfileCtrl = new wxTextCtrl(this, wxID_ANY, wxEmptyString);
                    sbadd->Add( m_newProfileCtrl, 1, wxALL, 3 );
                    sbadd->Add( new wxButton(this, idBtnAppendProf, _T("Save new")), 0, wxALL | wxALIGN_CENTER_HORIZONTAL , 3 );

                profeditsizer->Add(sbadd, 0, wxALL, 2);

                profeditsizer->Add( new wxButton(this, idBtnDelProf, _T("Remove selected")), 0, wxALL | wxALIGN_CENTER_HORIZONTAL , 3  );

            sb1->Add( profeditsizer, 0, wxALL, 2 );

        editsizer->Add( sb1, 0, wxALL | wxEXPAND, 5 );

    } else {
        m_profilesList->Hide();
    }

        wxStaticBoxSizer * sb2 = new wxStaticBoxSizer(wxVERTICAL, this, _T("Profile properties"));

        wxFlexGridSizer* profpropsizer = new wxFlexGridSizer(3);

        profpropsizer->Add(new wxStaticText(this, wxID_ANY, _T("Name")), 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 3);

    wxStaticText* minvallabel = new wxStaticText(this, wxID_ANY, _T("Minimum value"));
    wxStaticText* maxvallabel = new wxStaticText(this, wxID_ANY, _T("Maximum value"));

    minvallabel->Show(m_isShowMinMax);
    maxvallabel->Show(m_isShowMinMax);

    profpropsizer->Add(minvallabel, 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 3);
    profpropsizer->Add(maxvallabel, 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 3);

        wxTextValidator tv(wxFILTER_NUMERIC);


        m_currentProfileCtrl = new wxTextCtrlNullEx(this, wxID_ANY, wxEmptyString);


        m_currentProfileMinVal = new wxTextCtrlNullEx(this, wxID_ANY, _T(""));
        m_currentProfileMaxVal = new wxTextCtrlNullEx(this, wxID_ANY, _T(""));
        m_currentProfileMinVal->SetValidator(tv);
        m_currentProfileMaxVal->SetValidator(tv);

    m_currentProfileMinVal->Show(m_isShowMinMax);
    m_currentProfileMaxVal->Show(m_isShowMinMax);


        profpropsizer->Add(m_currentProfileCtrl, 1, wxALL, 3);
        profpropsizer->Add(m_currentProfileMinVal, 0, wxALL, 3);
        profpropsizer->Add(m_currentProfileMaxVal, 0, wxALL, 3);

        sb2->Add(profpropsizer, 0, wxEXPAND | wxALL | wxALIGN_RIGHT, 5);

        m_profilePropsPanel = new wxPanel(this, wxID_ANY);

        wxBoxSizer * fssizer = new wxBoxSizer(wxVERTICAL);

        m_profilePropsPanel->SetSizer(fssizer);

        sb2->Add(m_profilePropsPanel, 1, wxEXPAND | wxALL, 5);


    editsizer->Add( sb2, 1, wxALL | wxEXPAND, 5 );

    mainsizer->Add(editsizer, 1, wxEXPAND);

    wxBoxSizer* buttonsizer = new wxBoxSizer(wxHORIZONTAL);

    wxButton* okbutton = new wxButton(this,idBtnOK,_T("OK"));
    wxButton* closebutton = new wxButton(this,idBtnCloseEditor,_T("Close"));
    okbutton->SetToolTip(_T("Save the changes and close"));

    buttonsizer->Add(okbutton,0,wxALL,5);
    buttonsizer->Add(closebutton,0,wxALL,5);

    mainsizer->Add(buttonsizer, 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 5 );

    SetSizer(mainsizer);
    mainsizer->Fit(this);
    mainsizer->SetSizeHints(this);

    m_profilesList->Select(selectedProfile);

    if(m_profiledId == -1)
    LoadProfileProps( GetSelectedProfile() );
    else
        LoadProfileProps( m_profiledId );


    Connect( EditorValueChangedEvent, wxCommandEventHandler(MultiGaugeDescEditor::OnChangeCurrentProfCtrl) );

    return true;
}


void MultiGaugeDescEditor::OnButton(wxCommandEvent& evt)
{
    switch( evt.GetId() ) {

        case idBtnDelProf:
            DelSelectedProfiles();
            break;

        case idBtnEditProf:
            SaveProfileName();
            break;

        case idBtnAppendProf:
            AddNewProfile();
            break;

        case idBtnCloseEditor:
            Close();
            break;

        case idBtnAddSymbol:
            AddNewSymbolLine();
            break;

        case idBtnSaveProfileProps:
            SaveProfileProps();
            break;

        case idBtnOK:
            SaveProfileProps();
            Close();
            break;
    }

}

void MultiGaugeDescEditor::DelSelectedProfiles()
{
    wxArrayInt arr;
    m_profilesList->GetSelections(arr);

    if(!arr.GetCount()) return;

    wxString msg = wxString::Format(_T("Do you really want to delete the %i selected profile(s)?"),arr.GetCount());

    if( wxMessageBox(msg,_T("Confirm"), wxYES_NO | wxICON_QUESTION ) == wxNO ) return;

    Int2MultiSymbolGaugeDescHash& profiles = m_multiSymbolMgr->GetProfiles();

    m_db.Begin();

    for(int a = arr.GetCount()-1; a >= 0; a-- ) {
        int idx = arr[a];
        wxString selectedProfName = m_profilesList->GetString(idx);

        if( !m_multiSymbolMgr->IsProfileExists(selectedProfName) ) continue;

        MultiSymbolGaugeDesc& profile = m_multiSymbolMgr->GetProfile(selectedProfName);

        int prof_id = profile.GetId();

        profiles.erase(prof_id);

        m_db.ExecuteUpdate(wxString::Format(_T("delete from profiles where id=%i;"),prof_id));
        m_db.ExecuteUpdate(wxString::Format(_T("delete from symbols where profile_id=%i;"),prof_id));

        m_profilesList->Delete(idx);
    }

    m_db.Commit();

    m_profilesList->Select(0);

    m_profilePropsPanel->DestroyChildren();
    wxSizer* fmainsizer = m_profilePropsPanel->GetSizer();
    fmainsizer->Clear();

    LoadProfileProps( GetSelectedProfile() );
}

void MultiGaugeDescEditor::AddNewProfile()
{
    wxString newprofname = m_newProfileCtrl->GetValue();

    if( newprofname == wxEmptyString ) return;

    Int2MultiSymbolGaugeDescHash& profiles = m_multiSymbolMgr->GetProfiles();

    if( m_multiSymbolMgr->IsProfileExists(newprofname) ) {
        wxMessageBox(_T("This name is engaged.\nPlease choose an other one."),_T("Warning"));
        return;
    }

    m_db.ExecuteUpdate( wxString::Format(_T("insert into profiles (name) values ('%s');"),newprofname.c_str()) );

    long prof_id = m_db.GetLastRowId().ToLong();

    ///MultiSymbolGaugeDesc pdesc(prof_id, newprofname, 0.0, 1.0);
    MultiSymbolGaugeDesc pdesc(prof_id, newprofname);

    profiles[prof_id] = pdesc;

    m_profilesList->AppendString( newprofname );
}

wxVariantHash MultiGaugeDescEditor::GetSelectedProfileHash()
{
    wxArrayInt arr;
    m_profilesList->GetSelections(arr);

    if(!arr.GetCount()) return wxVariantHash();

    wxClientData* cdata = m_profilesList->GetClientObject( arr[0] );

    if(!cdata) return wxVariantHash();

    wxVariantHashClientData* data = dynamic_cast<wxVariantHashClientData*>(cdata);

    if(!data) return wxVariantHash();

    return data->GetData();
}

wxString MultiGaugeDescEditor::GetSelectedProfile()
{
    wxArrayInt arr;
    m_profilesList->GetSelections(arr);
    if(arr.GetCount() < 1) return wxEmptyString;

    return m_profilesList->GetString( arr[0] );
}


void MultiGaugeDescEditor::AddNewSymbolLine()
{
    long currid = m_profiledId;

    if(currid == -1) {
        wxVariantHash hash = GetSelectedProfileHash();

        if(hash.find(_T("id")) == hash.end()) return;

        currid = hash[_T("id")].GetLong();
    }

    if(currid == -1) return;

    MultiSymbolGaugeDesc& desc = m_multiSymbolMgr->GetProfile(currid);

    m_db.ExecuteUpdate(wxString::Format(_T("insert into symbols (profile_id,symbol_name) values(%i,'rpm');"),currid));

    long symbolLine_id = m_db.GetLastRowId().ToLong();

    wxVariantHash symbolprop;

    symbolprop[_T("id")] = symbolLine_id;
    symbolprop[_T("profile_id")] = currid;
    symbolprop[_T("symbol_name")] = _T("rpm");

    desc.AddSymbolProps(symbolprop);

    AddPropLine(symbolprop, m_lastProfPropLineRowIdx+1, m_profPropLinePadding);

    RefreshProfilePropPanel();

    RefreshFrameLayout();

    if(m_gauge) m_gauge->RedrawWidget();
}

void MultiGaugeDescEditor::RefreshProfilePropPanel()
{

    m_currProfSizer->RecalcSizes();
    m_currProfSizer->Layout();


    wxSizer* fmainsizer = m_profilePropsPanel->GetSizer();
    fmainsizer->Layout();
    //fmainsizer->Fit(m_profilePropsPanel);
    //fmainsizer->SetSizeHints(m_profilePropsPanel);

    m_profilePropsPanel->Refresh(false);

    GetSizer()->RecalcSizes();
    GetSizer()->Layout();
    //GetSizer()->Fit(this);
    //GetSizer()->SetSizeHints(this);

}

void MultiGaugeDescEditor::SaveProfileProps()
{
    m_db.Begin();

    MultiSymbolGaugeDesc* profile;

    if(m_profiledId == -1)
        profile = &m_multiSymbolMgr->GetProfile( GetSelectedProfile() );
    else
        profile = &m_multiSymbolMgr->GetProfile( m_profiledId );

    for(GUIDescGUIItemArrayIntHash::iterator it = m_profPropGUIHash.begin(); it != m_profPropGUIHash.end(); it++) {
        long symbol_id = it->first;
        GUIDescGUIItemArray& descarr = it->second;

        bool isChangedState = false;

        wxString qs = _T("update symbols set ");

        wxVariantHash& symbolProp = profile->GetSymbol(symbol_id);

        for(size_t a=0; a<descarr.GetCount(); a++) {
            GUIDescGUIItem& guidesc = descarr[a];

            if( !guidesc.IsChanged() ) continue;

            wxVariant val = guidesc.GetValue();

            qs += guidesc.Name + _T(" = '");

            qs += wxVariantHelper::MakeString(val) + _T("',");

            symbolProp[guidesc.Name] = val;

            guidesc.ResetChange();

            isChangedState = true;
        }

        qs.RemoveLast();

        qs += wxString::Format(_T(" where id=%i;"),symbol_id);

        Logger::logf(LOG_MSG,_T("isChangedState = %i, qs=")+qs,isChangedState);

        if(isChangedState)
            m_db.ExecuteUpdate(qs);
    }

    wxString minvs, maxvs;

    if(m_currentProfileMinVal->IsNull()) {
        profile->SetProp(_T("min_value"),wxNullVariant);
        minvs = _T("NULL");
    } else {
        double minv;
    m_currentProfileMinVal->GetValue().ToDouble(&minv);
        profile->SetProp(_T("min_value"),minv);
        minvs = wxString::Format(_T("%f"),minv);
    }

    if(m_currentProfileMaxVal->IsNull()) {
        profile->SetProp(_T("max_value"),wxNullVariant);
        maxvs = _T("NULL");
    } else {
        double maxv;
    m_currentProfileMaxVal->GetValue().ToDouble(&maxv);
        profile->SetProp(_T("max_value"),maxv);
        maxvs = wxString::Format(_T("%f"),maxv);
    }

    wxString pqs = wxString::Format(_T("update profiles set min_value=%s, max_value=%s, name='%s' where id = %i;"),
                                                minvs.c_str(),maxvs.c_str(),m_currentProfileCtrl->GetValue().c_str(),profile->GetId());

    m_db.ExecuteUpdate(pqs);

    Logger::logf(LOG_MSG,_T("qs=")+pqs);

    if(m_currentProfileCtrl->IsChanged()) {
        wxArrayInt arr;
        m_profilesList->GetSelections(arr);
        if(arr.GetCount()){
            m_profilesList->SetString(arr[0], m_currentProfileCtrl->GetValue());
        }
        profile->SetProp(_T("name"),m_currentProfileCtrl->GetValue());
    }

    m_currentProfileMinVal->ResetChange();
    m_currentProfileMaxVal->ResetChange();
    m_currentProfileCtrl->ResetChange();

    m_db.Commit();

    if(m_savePropBtn) m_savePropBtn->Enable(false);

    if(m_gauge) m_gauge->RedrawWidget();
}

void MultiGaugeDescEditor::OnProfListBox(wxCommandEvent& evt)
{
    //LoadProfileProps( evt.GetString() );

    wxVariantHash hash = GetSelectedProfileHash();

    if(hash.find(_T("id")) != hash.end())
        LoadProfileProps( hash[_T("id")].GetLong() );

}

void MultiGaugeDescEditor::SaveProfileName()
{

}

void MultiGaugeDescEditor::OnChangeCurrentProfCtrl(wxCommandEvent& evt)
{
    if(m_savePropBtn) m_savePropBtn->Enable(true);
}

void MultiGaugeDescEditor::CreateStateStructDesc()
{
    m_profSymbolPropGuiDesc.Add( GUIDescStructItem(_T("symbol_name"), GDSI_SYMBOLNAME, true, false, _T("Symbol name"),_T("")) );
    m_profSymbolPropGuiDesc.Add( GUIDescStructItem(_T("color"), GDSI_COLOR, false, false, _T("Color"),_T("")) );
    m_profSymbolPropGuiDesc.Add( GUIDescStructItem(_T("custom_symbol_title"), GDSI_TEXT, true, false, _T("Custom symbol title"),_T("")) );


}
bool MultiGaugeDescEditor::LoadProfileProps(const wxString& name, bool checkBlink)
{
    if( !m_multiSymbolMgr->IsProfileExists(name) ) {
        m_currentProfileMinVal->Hide();
        m_currentProfileMaxVal->Hide();
        m_currentProfileCtrl->Hide();
        return false;
    }

    MultiSymbolGaugeDesc& desc = m_multiSymbolMgr->GetProfile(name);

    return LoadProfileProps(desc, checkBlink);
}

bool MultiGaugeDescEditor::LoadProfileProps(int id)
{
    if( !m_multiSymbolMgr->IsProfileExists(id) ) {
        m_currentProfileMinVal->Hide();
        m_currentProfileMaxVal->Hide();
        m_currentProfileCtrl->Hide();
        return false;
    }

    MultiSymbolGaugeDesc& desc = m_multiSymbolMgr->GetProfile(id);

    return LoadProfileProps(desc, false);
}

bool MultiGaugeDescEditor::LoadProfileProps(MultiSymbolGaugeDesc& desc, bool checkBlink)
{
    wxString name = desc.GetName();

    Logger::logf(LOG_MSG,_T("LoadProfileProps name=")+name);

    //Logger::logf(LOG_MSG,_T("%p"),&desc);

    if( checkBlink && m_currentProfileCtrl->GetValue() == name ) return false;

    m_profilePropsPanel->Hide();

    m_savePropBtn = NULL;
    m_profilePropsPanel->DestroyChildren();
    m_currentProfileCtrl->ChangeValueEx(name);
    m_currentProfileCtrl->ResetChange();
    //m_setProfNameBtn->Enable(false);

    wxSizer* fmainsizer = m_profilePropsPanel->GetSizer();
    fmainsizer->Clear();

    wxGridBagSizer * gbsizer = new wxGridBagSizer();

    m_currProfSizer = gbsizer;

    if(m_isShowMinMax) {
    m_currentProfileMinVal->Show(1);
    m_currentProfileMaxVal->Show(1);
    }

    wxVariant minv = desc.GetProp(_T("min_value"));
    wxVariant maxv = desc.GetProp(_T("max_value"));

    if(minv == wxNullVariant)
        m_currentProfileMinVal->ChangeToNull();
    else
        m_currentProfileMinVal->ChangeValueEx( wxString::Format(_T("%.3f"),minv.GetDouble() ), false );

    if(maxv == wxNullVariant)
        m_currentProfileMaxVal->ChangeToNull();
    else
        m_currentProfileMaxVal->ChangeValueEx( wxString::Format(_T("%.3f"),maxv.GetDouble() ), false );

    m_currProfSizer->AddGrowableCol(0);

    int padding = m_profPropLinePadding;

    size_t colc = m_profSymbolPropGuiDesc.GetCount();

    int row = 0;

    for(size_t a = 0; a < colc; a++ ) {
        GUIDescStructItem& gdi = m_profSymbolPropGuiDesc[a];

        if( gdi.ShowOnEditor == false ) continue;

        wxStaticText * text = new wxStaticText(m_profilePropsPanel, wxID_ANY, gdi.Title);
        wxFont font = text->GetFont();
        font.SetWeight( wxFONTWEIGHT_BOLD );
        text->SetFont(font);

        gbsizer->Add( text, wxGBPosition(row,a), wxDefaultSpan, wxALL | wxALIGN_CENTER_HORIZONTAL, padding );
    }

    row++;

    gbsizer->Add( new wxStaticLine(m_profilePropsPanel, wxID_ANY) , wxGBPosition(row,0), wxGBSpan( 1, colc+1 ), wxALL | wxEXPAND, padding );

    row++;

    m_profPropGUIHash.clear();
    m_delBtnId2ProfPropId.clear();

    wxVariantHashArray& sarr = desc.GetSymbols();

    for(size_t a=0; a<sarr.GetCount(); a++ ) {

        AddPropLine(sarr[a], row, padding);

        row++;
    }

    fmainsizer->Add(gbsizer,0,wxEXPAND);


    wxButton * ansbtn = new wxButton(m_profilePropsPanel, idBtnAddSymbol, _T("Add new symbol line"));
    fmainsizer->Add( ansbtn, 0, wxALL | wxALIGN_CENTER_HORIZONTAL, padding );
    fmainsizer->AddStretchSpacer();

    fmainsizer->Add( new wxStaticLine(m_profilePropsPanel, wxID_ANY), 0, wxALL | wxEXPAND, padding );
    m_savePropBtn = new wxButton(m_profilePropsPanel, idBtnSaveProfileProps, _T("Save profile properties"));
    fmainsizer->Add( m_savePropBtn, 0, wxALL | wxALIGN_CENTER_HORIZONTAL, padding );
    m_savePropBtn->Enable(false);

    fmainsizer->Layout();
    fmainsizer->Fit(m_profilePropsPanel);

    m_profilePropsPanel->Show();

    RefreshFrameLayout();


    return true;
}

void MultiGaugeDescEditor::RefreshFrameLayout()
{
    wxSizer* mainsizer = GetSizer();

    if(!mainsizer) return;

    mainsizer->RecalcSizes();
    mainsizer->Layout();

    wxSize minsize = mainsizer->ComputeFittingWindowSize(this);

    SetSizeHints(minsize);

    wxSize size = GetSize();

    if(size.x < minsize.x || size.y < minsize.y) {
        wxSize newsize;
        newsize.x = wxMax(minsize.x,size.x);
        newsize.y = wxMax(minsize.y,size.y);
        SetSize(newsize);
    }
}

void MultiGaugeDescEditor::AddPropLine(wxVariantHash& props, int row, int padding)
{

    int symbol_id = props[_T("id")].GetLong();

    wxWindow * win = NULL;

    size_t colc = m_profSymbolPropGuiDesc.GetCount();

    for(size_t a = 0; a < colc; a++ ) {
        GUIDescStructItem& gdi = m_profSymbolPropGuiDesc[a];

        if( gdi.ShowOnEditor == false ) continue;

        bool isExpand = true;

        switch( gdi.Type ) {

            case GDSI_COLOR:{
                wxColor c = wxNullColour;
                if(props.find(gdi.Name) != props.end() ) c = DrawHelper::ParseExtColorStr(props[gdi.Name].GetString());
                win = new ColorSetupPanel(m_profilePropsPanel, c, wxC2S_HTML_SYNTAX );
                }break;

            case GDSI_NUMBER:{
                wxTextCtrlNullEx * ctrl = new wxTextCtrlNullEx(m_profilePropsPanel, wxID_ANY, props[gdi.Name].GetString() );
                wxTextValidator validator(wxFILTER_NUMERIC);
                ctrl->SetValidator(validator);
                win = ctrl;
                isExpand = false;
                }break;

            case GDSI_TEXT:
                win = new wxTextCtrlNullEx(m_profilePropsPanel, wxID_ANY, props[gdi.Name].GetString() );
                isExpand = false;
                break;

            case GDSI_SYMBOLNAME:{
                SymbolChooserPanel * panel = new SymbolChooserPanel(m_profilePropsPanel,m_gdMgr);
                panel->SetSymbolName( props[gdi.Name].GetString(), false );
                win = panel;
                }break;
        }

        m_currProfSizer->Add( win, wxGBPosition(row,a), wxDefaultSpan, wxALL | wxALIGN_CENTER_VERTICAL | (isExpand ? wxEXPAND : 0), padding );

        m_profPropGUIHash[symbol_id].Add( GUIDescGUIItem(gdi.Name, gdi.Type, win) );

    }

    int did = wxNewId();
    wxBitmapButton * btn = new wxBitmapButton(m_profilePropsPanel, did, DrawHelper::GetBitmap(_T("remove-16.png")) );
    m_currProfSizer->Add( btn, wxGBPosition(row,colc), wxDefaultSpan, wxALL, padding );
    btn->Connect(wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( MultiGaugeDescEditor::OnProfPropLineDelButton ), NULL, this );
    btn->SetToolTip(_T("Delete symbol"));

    m_delBtnId2ProfPropId[did] = symbol_id;

    m_lastProfPropLineRowIdx = row;
}

void MultiGaugeDescEditor::OnProfPropLineDelButton(wxCommandEvent& evt)
{
    int id = evt.GetId();

    if( m_delBtnId2ProfPropId.find(id) == m_delBtnId2ProfPropId.end() ) return;

    int symbol_id = m_delBtnId2ProfPropId[id];

    if( m_profPropGUIHash.find(symbol_id) == m_profPropGUIHash.end() ) return;



    int currid = -1;

    if(m_profiledId == -1) {
        wxVariantHash hash = GetSelectedProfileHash();

        if(hash.find(_T("id")) == hash.end()) return;

        currid = hash[_T("id")].GetLong();

    } else {

        currid = m_profiledId;

    }

    if(currid == -1) return;

    MultiSymbolGaugeDesc& profile = m_multiSymbolMgr->GetProfile(currid);

    wxVariantHashArray& symbols = profile.GetSymbols();

    size_t size = symbols.GetCount();

    for(size_t a=0; a< symbols.GetCount(); a++ ){
        if( symbols[a][_T("id")].GetLong() != symbol_id ) continue;

        symbols.RemoveAt(a);
        break;
    }

    if( size == symbols.GetCount() ) return;

    m_db.ExecuteUpdate( wxString::Format(_T("delete from symbols where id=%i;"),symbol_id) );

    GUIDescGUIItemArray& garr = m_profPropGUIHash[symbol_id];

    if(garr.GetCount() < 1 || garr[0].Ctrl == NULL ) return;

    wxWindow * win = garr[0].Ctrl;

    if( m_currProfSizer == NULL ) return;

    wxSizerHelper::DeleteRow(m_currProfSizer, win);

    m_profPropGUIHash.erase(symbol_id);

    m_lastProfPropLineRowIdx--;

    RefreshProfilePropPanel();

    RefreshFrameLayout();

    if(m_gauge) m_gauge->RedrawWidget();
}

