#ifndef MULTIGAUGEDESCEDITOR_H
#define MULTIGAUGEDESCEDITOR_H

#include <wx/gbsizer.h>
#include <wx/wx.h>

#include <wx/wxsqlite3.h>

#include "../gaugedesc/GUIDescStructItem.h"
#include "../gaugedesc/GUIDescGUIItem.h"
#include "../common/types/Int2IntHash.h"

class GaugeDescManager;
class wxTextCtrlNullEx;
class VemsWidgetBase;
class MultiSymbolGaugeDescMgr;
class MultiSymbolGaugeDesc;

class MultiGaugeDescEditor: public wxFrame
{
    public:
        MultiGaugeDescEditor(wxWindow* parent, const wxString& title, GaugeDescManager* gdMgr, MultiSymbolGaugeDescMgr* mgr, bool isShowProfiles = true, VemsWidgetBase* gauge = NULL, bool showMinMax = true, int profid = -1);
        virtual ~MultiGaugeDescEditor();

        bool BuildGUI(int profileId = -1);


    protected:

        bool LoadProfileProps(const wxString& name, bool checkBlink = true);
        bool LoadProfileProps(int id);
        bool LoadProfileProps(MultiSymbolGaugeDesc& desc, bool checkBlink = true);
        void CreateStateStructDesc();
        void AddPropLine(wxVariantHash& props, int row, int padding);
        void OnProfPropLineDelButton(wxCommandEvent& evt);

        void OnButton(wxCommandEvent& evt);
        void OnProfListBox(wxCommandEvent& evt);
        void OnClose(wxCloseEvent& evt);

        void OnChangeCurrentProfCtrl(wxCommandEvent& evt);

        void DelSelectedProfiles();
        void SaveProfileName();
        void AddNewProfile();
        void AddNewSymbolLine();
        void SaveProfileProps();

        wxString GetSelectedProfile();
        void RefreshProfilePropPanel();
        void RefreshFrameLayout();

        wxVariantHash GetSelectedProfileHash();

    private:

        enum {
            idProfListBox,
            idBtnAppendProf,
            idBtnEditProf,
            idBtnDelProf,
            idBtnSaveProfileProps,
            idBtnAddSymbol,
            idBtnCloseEditor,
            idBtnOK
        };

        wxListBox*          m_profilesList;
        wxTextCtrl*         m_newProfileCtrl;
        wxTextCtrlNullEx*   m_currentProfileCtrl;
        wxButton*           m_setProfNameBtn;
        wxButton*           m_savePropBtn;
        wxPanel*            m_profilePropsPanel;
        wxTextCtrlNullEx*   m_currentProfileMinVal;
        wxTextCtrlNullEx*   m_currentProfileMaxVal;

        GUIDescStructItemArray  m_profSymbolPropGuiDesc;
        //GUIDescStructItemArray  m_profPropGuiDesc;
        wxGridBagSizer *        m_currProfSizer;

        GUIDescGUIItemArrayIntHash m_profPropGUIHash;
        Int2IntHash             m_delBtnId2ProfPropId;

        int                     m_profPropLinePadding;
        int                     m_lastProfPropLineRowIdx;
        bool                    m_isShowProfiles;
        bool                    m_isShowMinMax;

        wxSQLite3Database       m_db;
        MultiSymbolGaugeDescMgr*   m_multiSymbolMgr;
        GaugeDescManager*       m_gdMgr;

        VemsWidgetBase*         m_gauge;

        int                     m_profiledId;

        DECLARE_EVENT_TABLE();
};

#endif // MULTIGAUGEDESCEDITOR_H
