#include "StaticControlPanel.h"


#include "StaticViewersController.h"
#include "../DirHelper.h"
#include "../logger.h"

#include "../stdatamanager/STDataMgrBase.h"


int ID_NOT_SENT_TEXT = wxNewId();


BEGIN_EVENT_TABLE( StaticControlPanel, wxPanel )
    EVT_COMMAND_RANGE( REFRESH_BUTTON, NEXT_DIALOG, wxEVT_COMMAND_BUTTON_CLICKED, StaticControlPanel::OnButton )
END_EVENT_TABLE()


StaticControlPanel::StaticControlPanel()
{

}


StaticControlPanel::StaticControlPanel( wxWindow* parent,
                                        StaticViewersController* controller,
                                        int buttons,
                                        wxWindowID id,
                                        const wxPoint& pos,
                                        const wxSize& size,
                                        long style )
{
    Create( parent, controller, buttons, id, pos, size, style );

    STDataMgrBase* stMgr = m_controller->getController();
    stMgr->RegisterViewer( this );
}


StaticControlPanel::~StaticControlPanel()
{
    unregisterViewer();
}


bool StaticControlPanel::Create( wxWindow* parent,
                                 StaticViewersController* controller,
                                 int buttons,
                                 wxWindowID id,
                                 const wxPoint& pos,
                                 const wxSize& size,
                                 long style )
{
    m_controller = controller;
    wxPanel::Create( parent, id, pos, size, style );
    SetSizer( new wxBoxSizer( wxHORIZONTAL ) );
    CreateControls( buttons );
    SetInitialSize();

    return true;
}


bool StaticControlPanel::CreateControls( int buttons )
{
    bool expand = false;

#if defined (__WINCE_STANDARDSDK__)
    expand = true;
#endif

    wxSizer* lineSizer = GetSizer();//new wxBoxSizer( wxHORIZONTAL );

    if ( buttons & REFRESH_BUTTON )
    {
        wxButton* refreshButton = new wxButton( this, REFRESH_BUTTON, _T("Refresh") );
        refreshButton->SetToolTip( _T("Read data from ECU\nHotkey: Alt-R") );
        refreshButton->SetWindowStyle( wxBU_EXACTFIT );
        lineSizer->Add( refreshButton,expand );
    }

    if ( buttons & WRITE_BUTTON )
    {
        wxButton* writeButton   = new wxButton( this, WRITE_BUTTON, _T("Send") );
        writeButton->SetToolTip( _T("Send data to ECU\nHotkey: Alt-W") );
        writeButton->SetWindowStyle( wxBU_EXACTFIT );
        lineSizer->Add( writeButton,expand );
    }

    if ( buttons & BURN_BUTTON )
    {
        wxButton* burnButton   = new wxButton( this, BURN_BUTTON, _T("Burn") );
        burnButton->SetToolTip( _T("Burn data to ECU\nHotkey: Alt-B") );
        burnButton->SetWindowStyle( wxBU_EXACTFIT );
        lineSizer->Add( burnButton,expand );
    }

    if ( buttons & REMAP_BUTTON )
    {
        wxButton* remapButton = new wxButton( this, REMAP_BUTTON, _T("Remap") );
        lineSizer->Add( remapButton,expand );
    }

    if ( buttons & SHOW_REMAP_BUTTON )
    {
        wxButton* showRemapButton = new wxButton( this, SHOW_REMAP_BUTTON, _T("Remap") );
        lineSizer->Add( showRemapButton,expand );
    }

    if ( buttons & IMPORT_BUTTON )
    {
        wxButton* importButton   = new wxButton( this, IMPORT_BUTTON, _T("Import") );
        importButton->SetWindowStyle( wxBU_EXACTFIT );
        lineSizer->Add( importButton,expand );
    }

    if ( buttons & EXPORT_BUTTON )
    {
        wxButton* exportButton   = new wxButton( this, EXPORT_BUTTON, _T("Export") );
        lineSizer->Add( exportButton,expand );
    }

    if ( buttons & START_CALIBRATE )
    {
        wxButton* button   = new wxButton( this, START_CALIBRATE, _T("Start Cal.") );
        lineSizer->Add( button,expand );
    }

    if ( buttons & STOP_CALIBRATE )
    {
        wxButton* button   = new wxButton( this, STOP_CALIBRATE, _T("Stop Cal.") );
        lineSizer->Add( button,expand );
    }

    if ( buttons & PREVIOUS_DIALOG )
    {
        lineSizer->AddStretchSpacer();
        wxButton* button   = new wxButton( this, PREVIOUS_DIALOG, _T("<") );
        button->SetWindowStyle( wxBU_EXACTFIT );
        lineSizer->Add( button,expand );
    }

    if ( buttons & NEXT_DIALOG )
    {
        wxButton* button   = new wxButton( this, NEXT_DIALOG, _T(">") );
        button->SetWindowStyle( wxBU_EXACTFIT );
        lineSizer->Add( button,expand );
    }

    if ( buttons & CLOSE_BUTTON )
    {
        wxButton* button   = new wxButton( this, CLOSE_BUTTON, _T("Close") );
        button->SetWindowStyle( wxBU_EXACTFIT );
        lineSizer->AddStretchSpacer();
        lineSizer->Add( button,expand );
    }

    lineSizer->AddSpacer(10);

    //wxSize s = lineSizer->GetSize();

    //Logger::logf(LOG_MSG,_T("!!!!!!!!!!!!!!!!!!! s.x=%i,y=%i"),s.x,s.y);

    //lineSizer->SetMinSize( wxSize(300,30) );
    //GetSizer()->Add( lineSizer, 1, wxALL, 0 );
    //GetSizer()->Fit( this );
    GetSizer()->SetSizeHints( this );

    return true;
}


void StaticControlPanel::OnButton( wxCommandEvent& event )
{
    int id = event.GetId();

    switch( id )
    {
        case REFRESH_BUTTON :
            enableButton( REFRESH_BUTTON, false );
            m_controller->refreshData();
            break;
        case WRITE_BUTTON :
            m_controller->writeData();
            break;
        case BURN_BUTTON :
            m_controller->burnData();
            break;
        case REMAP_BUTTON :
            m_controller->remap();
            break;
        case CLOSE_BUTTON :
            LOG
            GetParent()->Close();
            break;
        case SHOW_REMAP_BUTTON :
            m_controller->showRemapDialog();
            break;
        case EXPORT_BUTTON :
            //m_controller->exportToFile();
            break;
        case IMPORT_BUTTON :
        {
            wxString msg = _T("Choose a file");
            wxString dir = DirHelper::GetPath( dLOG );

            wxString wildcard = _T("VemsCfg files (*.vemscfg)|*.vemscfg|Any files (*.*)|*.*");
            wxFileDialog dialog( this, msg, dir, _T(""), wildcard, wxFD_OPEN | wxFD_FILE_MUST_EXIST );

            if ( dialog.ShowModal() == wxID_OK )
            {
                m_controller->importFromFile( dialog.GetPath() );
            }
        }
            break;
        case START_CALIBRATE:
            Logger::logf(LOG_MSG, _T("StaticControlPanel::OnButton() Start Clalibration()"));
            m_controller->startCalibration();
            break;
        case STOP_CALIBRATE:
            m_controller->stopCalibration();
            break;
        case PREVIOUS_DIALOG:
            m_controller->changeToPreviousDialog();
            break;
        case NEXT_DIALOG:
            m_controller->changeToNextDialog();
            break;
    }
}


void StaticControlPanel::setButtonHighlighted( int id, bool highlighted )
{
    if ( IsEnabled() == false )
        enableButtons( false );

    wxWindow* button = wxWindow::FindWindowById( id, this );

    if ( button )
    {
        bool b = button->GetBackgroundColour() != wxSystemSettings::GetColour( wxSYS_COLOUR_BTNFACE );

        if ( b == highlighted ) { return; }

        if ( !highlighted )
        {
            button->SetBackgroundColour( wxSystemSettings::GetColour( wxSYS_COLOUR_BTNFACE ) );
            wxFont font = button->GetFont();
            font.SetWeight( wxFONTWEIGHT_NORMAL );
            button->SetFont( font );
        }
        else
        {
            button->SetBackgroundColour( *wxRED );
            wxFont font = button->GetFont();
            font.SetWeight( wxFONTWEIGHT_BOLD );
            button->SetFont( font );
        }

        Refresh();
    }
}


void StaticControlPanel::update()
{
    //Logger::logf( LOG_MSG, _T("StaticControlPanel::update() - online: %d"), m_stDataMgr->isOnline() );

    enableButton( REFRESH_BUTTON, true );

    if ( m_stDataMgr->isOnline() && m_stDataMgr->isConnected() )
    {
        enableButtons();
        setButtonHighlighted( WRITE_BUTTON, m_stDataMgr->isWaitingForWrite() );
        setButtonHighlighted( BURN_BUTTON, m_stDataMgr->isWaitingForBurn() || m_stDataMgr->isWaitingForWrite() );
    }
    else
    {
        enableButtons( false );
        setButtonHighlighted( WRITE_BUTTON, false );
        setButtonHighlighted( BURN_BUTTON, false );
    }
}


void StaticControlPanel::enableButtons( bool enable )
{
    //enableButton( REFRESH_BUTTON, enable );
    enableButton( WRITE_BUTTON, enable );
    enableButton( BURN_BUTTON, enable );
}


void StaticControlPanel::enableButton( int id, bool enable )
{
    //Logger::logf( LOG_MSG, _T("StaticControlPanel::enableButton()") );

    wxWindow* button = wxWindow::FindWindowById( id, this );

    if ( button )
    {
        //Logger::logf( LOG_MSG, _T("StaticControlPanel::enableButton()") );
        button->Enable( enable );
    }
}

wxString StaticControlPanel::toString()
{
    return _T("StaticControlPanel");
}

void StaticControlPanel::setStaticViewersController( StaticViewersController* controller )
{
    m_controller = controller;
}
