#ifndef AGGGRAPHCONTROL_H
#define AGGGRAPHCONTROL_H

#include <wx/gdicmn.h>
#include <wx/wx.h>

class wxAgg;

WX_DEFINE_ARRAY( wxRealPoint*, wxRealPointArray );

class AggGraphControl : public wxControl
{
    public:
        AggGraphControl( wxWindow* parent );
        /*wxWindow(wxWindow* parent, wxWindowID id, const wxPoint& pos = wxDefaultPosition,
        const wxSize& size = wxDefaultSize, long style = 0, const wxString& name = wxPanelNameStr) */
        virtual ~AggGraphControl();

        void showCrosshair( bool enable ) { m_showCrosshair = enable; }
        void showLine( bool enable ) { m_showLine = enable; }

        void setMinX( double minX ) { m_minX = minX; }
        void setMaxX( double maxX ) { m_maxX = maxX; }
        void setDivisionsX( int divisionsX ) { m_divisionsX = divisionsX; }
        void setMinY( double minY ) { m_minY = minY; }
        void setMaxY( double maxY ) { m_maxY = maxY; }
        void setDivisionsY( int divisionsY ) { m_divisionsY = divisionsY; }
        void setCrosshairs( double x, double y )
        {
            m_crosshairsX = x;
            m_crosshairsY = y;
        }
        void setHorizontalLine( double y ) { m_lineY = y; }
        void showHorizontalHelpLine( bool enabled ) { m_showHorizontalHelpLine = enabled; }
        void showVerticalHelpLine( bool enabled ) { m_showVerticalHelpLine = enabled; };

        void addPoint( double x, double y );
        void clearPoints() { m_points.Clear(); }
        double interpolate( double x );

        void setRowLabelFormat( const wxString& format ) { m_rowLabelFormat = format; }
        void setColLabelFormat( const wxString& format ) { m_colLabelFormat = format; }
    protected:
    private:
        wxRealPointArray m_points;
        double      m_minX;
        double      m_maxX;
        double      m_divisionsX;
        double      m_minY;
        double      m_maxY;
        double      m_divisionsY;
        double      m_crosshairsX;
        double      m_crosshairsY;
        wxString    m_colLabelFormat;
        wxString    m_rowLabelFormat;
        bool        m_isResized;
        wxBitmap*   m_bmp;              //grid, labels, lines, points
        wxBitmap*   m_bmpCrosshairs;

        double      m_textWidth;        // 8888.8
        double      m_textHeight;
        int         m_width;            //control width
        int         m_height;           //control height
        double      m_pointSize;
        double      m_borderSize;
        double      m_crosshairsSize;
        double      m_graphWidth;
        double      m_graphHeight;
        double      m_rangeX;
        double      m_rangeY;
        double      m_scaleX;
        double      m_scaleY;
        double      m_offsetX;
        double      m_offsetY;
        double      m_stepX;
        double      m_stepY;
        double      m_lineY;

        bool        m_showCrosshair;
        bool        m_showLine;
        bool        m_showHorizontalHelpLine;
        bool        m_showVerticalHelpLine;
        int         m_fontPointSize;

        void createCrosshairs();
        void updateSize();
        void drawGrids();
        void drawLabels();
        void drawLines();
        void drawPoints();
        void drawCrosshairs( wxDC& dc );
        void showLine( wxDC& dc );

        wxRealPoint translate( double x, double y, bool round = false );  //translate values to graph coordinates

        void OnSize( wxSizeEvent& event );
        void OnPaint( wxPaintEvent& event );
        DECLARE_EVENT_TABLE()
};

#endif // AGGGRAPHCONTROL_H
