#ifndef COLUMNSGAUGE_H
#define COLUMNSGAUGE_H

#include "VemsWidgetBase.h"
#include "../common/types/wxVariantIntHashHash.h"
#include "../gaugedesc/MultiSymbolGaugeDescMgr.h"
#include "../common/types/DoubleArray.h"
#include "../wxRealRect.h"

class wxAgg2;


WX_DECLARE_STRING_HASH_MAP(ColoredAreaRefArray, ColoredAreaRefArrayHash);
WX_DECLARE_STRING_HASH_MAP(DoubleArray, DoubleArrayHash);

class ColumnsGauge : public VemsWidgetBase
{
    DECLARE_DYNAMIC_CLASS(ColumnsGauge);

    public:
        ColumnsGauge();
        ColumnsGauge(wxWindow* parent, wxWindowID id, GaugeDescManager * gdmgr, GaugeProperties &prop);
        virtual ~ColumnsGauge();

        virtual wxRegion GetShape();
        virtual wxRegion GetBorderRegion();
        virtual void UpdateSize();
        virtual void setSize(int x , int y) {}

        virtual void SetValue( double value );
        virtual void SetNewTheme(WidgetThemeTable *theme);

        virtual void require();
        virtual void refresh(DoubleHash* table);

        virtual void GaugeRefresh();
        virtual void RedrawWidget();
        virtual void InitGauge();

        int GetProfileID() { return m_profileID; };

    protected:

        MultiSymbolGaugeDescMgr* GetMgr() { return m_gaugeDescMgr->GetColsGaugeDescMgr(); };
        MultiSymbolGaugeDesc& GetDesc() { return GetMgr()->GetProfile(m_profileID); };

        void OnPaint(wxPaintEvent& evt);
        void OnProfileMenu(wxCommandEvent& evt);
        void OnRightDown(wxMouseEvent& evt);
        void OnMenu(wxCommandEvent& evt);
        void OnSize(wxSizeEvent& evt);

        void CreateProfileChooserMenu(wxMenu* menu);
        void createWidgetMenu( wxMenu& popupMenu );

        void RegisterSymbols();
        void UnRegisterSymbols();
        void CacheCustomProps(wxVariantHash& symbolHash);
        void CacheCustomProps();
        void CacheAVGValue(const wxString& name, double value);

        void PaintWidget(wxDC& dc);
        void PaintWidgetAgg(wxDC& dc);
        void PaintStick(const wxString& name, wxDC& dc);
        void PaintNumberValueAgg(const wxString& name, wxDC& dc);
        void PaintNumberValueDC(const wxString& name, wxDC& dc, double col_width);

        int    CalcLimitsSize(int type, int col_width, wxDC& dc);
        double CalcLimitsSize(int type, double col_width, wxAgg2& aggCanvas);
        void   CalcAVG(const wxString& name);
        double CalcColWidth(size_t symbols_count);
        wxRect CalcColumn(const wxString& name);

        void DrawSymbolColumn(const wxString& name, wxAgg2& aggCanvas, double col_width, int a);
        void DrawSymbolColumns();
        void DrawWidgetMainFace();
        void DrawSymbolStick();
        void DrawNumberValue(const wxString& name, wxAgg2& aggCanvas, double col_width);
        void DrawNumberValuesAgg();

        bool IsChangedValue(const wxString& name);
        wxBitmap* GetBitmap(const wxVariantIntHash& hash, int type);

    private:

        void OnTestTimer(wxTimerEvent& evt);

        void Benchmark();
        void clearAll();

        enum {
            idMenuOpenDescEditor,
            idMenuOpenDescEditorProfile,
            idTestMenu,
            idMenuEND,
            idTestTimer
        };

        bool                    m_isRegistered;
        bool                    m_needRefresh;
        DoubleHash              m_PrevValueHash;
        DoubleHash              m_hash;
        int m_profileID;
        Int2IntHash             m_menuId2ProfileId;
        wxVariantIntHashHash    m_customProps;

        bool                    m_isUseAgg;
        bool                    m_isShowAvg;
        bool                    m_isColShowAvg;

        ColoredAreaRefArrayHash m_coloredAreas;

        DoubleArrayHash         m_avgCache;
        size_t                  m_acSize; // avg cache size

        double                  m_border;
        double                  m_colPadding;
        double                  m_gradient_lum_diff;
        wxBitmap*               m_faceBitmap;
        wxBitmap*               m_stickBmp;

        wxRealRect              m_widgetBorderBox;

        wxTimer                 m_testTimer;

        DECLARE_EVENT_TABLE();

};

#endif // COLUMNSGAUGE_H
