#include "GPSCompassGauge.h"

#include <wx/dcbuffer.h>

#include "../WidgetThemeTable.h"
#include "../common/types/DoubleHash.h"
#include "../agg/wxAgg.h"
#include "../logger.h"


BEGIN_EVENT_TABLE( GPSCompassGauge, VemsWidgetBase )
    EVT_PAINT( GPSCompassGauge::OnPaint )
    //EVT_UPDATE_VIEWER_EVENT( wxID_ANY, GPSCompassGauge::OnUpdateValue )
END_EVENT_TABLE()

IMPLEMENT_DYNAMIC_CLASS( GPSCompassGauge, VemsWidgetBase )

GPSCompassGauge::GPSCompassGauge()
{
}

GPSCompassGauge::~GPSCompassGauge()
{
    if ( m_compassBg )
    {
        delete m_compassBg;
        m_compassBg = NULL;
    }
}

GPSCompassGauge::GPSCompassGauge( wxWindow* parent, wxWindowID id, GaugeDescManager* gdmgr, GaugeProperties& prop )
{
    m_isSymbolMenu = false;

    m_sp.Start();
    m_compassBg = NULL;

    m_needResize = false;
    CreateWidget( parent, id, gdmgr, prop );

    m_sideRatioRestriction = 0.83;
    m_isSymbolMenu = false;
    m_isGaugeDescMenu = false;
}

void GPSCompassGauge::InitGauge()
{
    Connect( wxEVT_SIZE, wxSizeEventHandler( GPSCompassGauge::OnSize ) );
}

wxRegion GPSCompassGauge::GetShape()
{
    return wxRegion( 0, 0, GetSize().x, GetSize().y );
}

wxRegion GPSCompassGauge::GetBorderRegion()
{
    return wxRegion( 0, 0, 1, 1 );
}

void GPSCompassGauge::UpdateSize()
{
    //GaugeRefresh();
}

void GPSCompassGauge::SetNewTheme( WidgetThemeTable* theme )
{
    m_theme = theme;
    Refresh( false );
}

void GPSCompassGauge::OnSize( wxSizeEvent& event )
{
    int width, height;
    m_needResize = true;

	wxWindow* parent = 0;

	if( parent )
	{
	    parent->GetSize( &width, &height );
    }
    else
    {
        GetSize( &width, &height );
    }

    if( width > m_sideRatioRestriction * height )
        height = (int)(width / m_sideRatioRestriction);
    if( height > width / m_sideRatioRestriction )
        width = (int)(height * m_sideRatioRestriction);

    SetSize( width, height );

    Refresh( false );
    event.Skip();

}

void GPSCompassGauge::GaugeRefresh()
{
    if ( m_sp.Time() < 50 )
    {
        return;
    }

    m_sp.Start();

    Refresh( false );
}

void GPSCompassGauge::OnPaint( wxPaintEvent& event )
{
    wxBufferedPaintDC dc( this );
    wxMemoryDC arrowdc;
    dc.SetBackground( m_theme->BackgroundColor );

    dc.Clear();

    int width = GetSize().x;
    int height = GetSize().y;

    if ( ! m_compassBg || m_needResize )
        getCompassBg( width, height );

    dc.DrawBitmap( *m_compassBg, 0, 0, true );

    wxBitmap* arrow = getArrow( width, height );

    dc.DrawBitmap( *arrow, 0, 0, true );
    delete arrow;


    DrawExtraLayer(dc);
}

void GPSCompassGauge::getCompassBg( int w, int h )
{
    wxAgg canvas( w, h );

    if ( m_compassBg )
    {
        delete m_compassBg;
        m_compassBg = NULL;
    }


    double rBase = (double) w / 2.0;

    double r1 = 0.86 * rBase;
    //double r2 = 0.61 * rBase;
    double r3 = 0.59 * rBase;
    double r4 = 0.55 * rBase;

    int cx = (int)(w/2), cy = (int)(h/( 2.0 / m_sideRatioRestriction));

    canvas.drawGradientFilledCircle( wxColor( 20,25,14 ), wxColor( 11,11,10 ), 90, cx, cy, r1 );
    canvas.setPen( *wxBLACK );
    // canvas.drawCircle( cx, cy, r2, 0.01*rBase );
    canvas.drawGradientFilledCircle( wxColor( 74,69,109 ), wxColor( 255,255,255 ), 90, cx, cy, r3 );

    canvas.setBrush( *wxWHITE );
    canvas.drawFilledCircle( cx, cy, r4 );


    // N,E,S,W
    double sr = 0.1 * rBase;
    double th = 2 * sr * 0.98 * 0.6;
    canvas.setBrush( *wxWHITE ); canvas.setPen( *wxWHITE );
    canvas.drawFilledCircle( cx, cy - r1*.98, sr );
    canvas.setPen( *wxBLACK ); canvas.setBrush( *wxBLACK );

    double tw = wxAgg::getTextWidth( _T("N"), th );
    canvas.drawText( _T("N"), cx - tw/2, cy - r1*.98 + th/2, -1, 2 * sr * 0.98 * 0.6 , 4.0 );

    canvas.drawFilledCircle( cx, cy + r1*.98, sr );
    canvas.drawFilledCircle( cx - r1*.98, cy, sr ); canvas.drawFilledCircle( cx + r1*.98, cy, sr );

    tw = wxAgg::getTextWidth( _T("S"), th );
    canvas.setBrush( *wxWHITE ); canvas.setPen( *wxWHITE );
    canvas.drawText( _T("S"), cx - tw/2, cy + r1*.98 + th/2, -1, 2 * sr * 0.98 * 0.6 , 4.0 );

    tw = wxAgg::getTextWidth( _T("W"), th );
    canvas.drawText( _T("W"), cx - r1*0.98 - tw/2, cy + th/2, -1, 2 * sr * 0.98 * 0.6 , 4.0 );

    tw = wxAgg::getTextWidth( _T("E"), th );
    canvas.drawText( _T("E"), cx + r1*0.98 - tw/2, cy + th/2, -1, 2 * sr * 0.98 * 0.6 , 4.0 );

    // lines

    canvas.setPen( *wxWHITE );
    for ( unsigned int i = 0; i < 360; i += 10 )
    {
        if ( i % 90 == 0 )
            continue;
        if ( i % 30 == 0 )
        {
            int deg = 0;
            if ( i < 360 ) deg = 450 - i;
            if ( i < 90 ) deg = 90 - i;

            wxString degS;
            degS << deg;

            tw = wxAgg::getTextWidth( degS, th * 0.4 );

            double x, y;
            double fs = th * 0.4;

            x = cx + r1 * cos( i * 3.1415926 / 180 ) * 0.98;
            if ( i < 90 || i > 270 )
                x -= tw;

            y = cy - r1 * sin( i * 3.1415926 / 180 ) * 0.98;
            if ( i < 180 )
                 y += th * 0.4;

            canvas.setPen( m_theme->ValueColor );
            canvas.drawText( degS, x, y, -1, fs, 1.5 );

            continue;
        }
        else
        {
            double x1, x2, y1, y2;
            x1 = cx + r1 * cos( i * 3.1415926 / 180 );
            y1 = cy + r1 * sin( i * 3.1415926 / 180 );

            x2 = cx + ( r1 + r3 ) / 2.0 * cos( i * 3.1415926 / 180 );
            y2 = cy + ( r1 + r3 ) / 2.0 * sin( i * 3.1415926 / 180 );

            canvas.setPen( m_theme->DivColor );
            canvas.drawLine( x1, y1, x2, y2 );
        }
    }

    //
    m_compassBg = canvas.renderToBitmap();
    m_needResize = false;
}

wxBitmap* GPSCompassGauge::getArrow( int w, int h )
{
    wxAgg canvas( w, h );
    canvas.setBrush( wxColor(88,160,32,255) );
    canvas.setPen( wxColor(88,160,32,255) );

    double cx = w / 2.0, cy = h/( 2.0 / m_sideRatioRestriction);
    double rBase = 0.59 * (double)w / 2.0;
    //const double M_PI = 3.1415926;

    double Ax, Ay, Bx, By, Cx, Cy, Dx, Dy, Ex, Ey, Fx, Fy, Gx, Gy, Hx, Hy, Qx, Qy;

    // arrow shape const
    double m_beta               = 0.5255;
    double m_headWidthRef       = 0.18;
    double m_tailHeightRefOut   = 0.85;
    double m_tailHeightRef      = 0.85;
    double m_tailWidthRef       = 0.11;

    int deg = 0;

    deg =  (int)((90 - m_course)*-1 + 180);

    Dx = cx - rBase * 1.1 * cos( M_PI * (double)deg / 180.0);
    Dy = cy - rBase * 1.1 * sin( M_PI * (double)deg / 180.0);

    deg -= 180;

    Hx = cx - rBase * 0.8 * cos( M_PI * (double)deg / 180.0);
    Hy = cy - rBase * 0.8 * sin( M_PI * (double)deg / 180.0);

    double a_length = sqrt( pow( Dx - Hx, 2 ) + pow( Dy - Hy, 2 ) );
    double W = a_length * m_tailWidthRef;
    double alphaR = atan2( Hy - Dy, Dx - Hx);

    Ax = Hx + W / 2 * cos( M_PI / 2 - alphaR );
    Ay = Hy + W / 2 * sin( M_PI / 2 - alphaR );
    Gx = Hx - W / 2 * cos( M_PI / 2 - alphaR );
    Gy = Hy - W / 2 * sin( M_PI / 2 - alphaR );
    Fx = Gx + a_length * m_tailHeightRef * cos( alphaR );
    Fy = Gy - a_length * m_tailHeightRef * sin( alphaR );
    Bx = Ax + a_length * m_tailHeightRef * cos( alphaR );
    By = Ay - a_length * m_tailHeightRef * sin( alphaR );
    Qx = Hx + a_length * m_tailHeightRefOut * cos( alphaR );
    Qy = Hy - a_length * m_tailHeightRefOut * sin( alphaR );
    Ex = Qx + a_length * m_headWidthRef * tan( m_beta ) * cos( M_PI / 2 + alphaR );
    Ey = Qy - a_length * m_headWidthRef * tan( m_beta ) * sin( M_PI / 2 - alphaR );
    Cx = Qx + a_length * m_headWidthRef * tan( m_beta ) * cos( alphaR - M_PI /2 );
    Cy = Qy - a_length * m_headWidthRef * tan( m_beta ) * sin( alphaR - M_PI /2 );

    RealPointArray arr;
    arr.Add( wxRealPoint( Ax, Ay ) ); arr.Add( wxRealPoint( Bx, By ) );
    arr.Add( wxRealPoint( Cx, Cy ) ); arr.Add( wxRealPoint( Dx, Dy ) );
    arr.Add( wxRealPoint( Ex, Ey ) ); arr.Add( wxRealPoint( Fx, Fy ) );
    arr.Add( wxRealPoint( Gx, Gy ) );

    canvas.drawSolidPolygon( arr, 1 );

    canvas.setBrush( wxColor(0,0,0,255) );
    canvas.setPen( wxColor(0,0,0,255) );

    canvas.drawFilledCircle( cx, cy, ( W * 0.5 * 0.8 ) );

    // heading text:
    double htH = 0.1; double htW;
    wxString hStr = wxString::Format( _T("Heading: %.1f"), m_course );

    while ( ( htW = wxAgg::getTextWidth( _T("Heading: 999.9"), htH) ) < w )
    {
        htH += 0.3;
    }
    htH -= 0.3;

    canvas.setPen( m_theme->ValueColor );
    canvas.drawText( hStr, cx - htW/2, 2*cy*1.1, -1, htH, 4.0 );

    wxBitmap* arrow = canvas.renderToBitmap();

    return arrow;
}

void  GPSCompassGauge::SetValue( double value )
{
    m_course = value;
    Refresh(false);
}
