#ifndef GPSROUTEGAUGE_H
#define GPSROUTEGAUGE_H

#include "../common/types/RealPointArray.h"
#include "../common/types/DoubleHashArray.h"

#include "VemsWidgetBase.h"
#include "../wxRealRect.h"
#include "../ObserverBase.h"
#include "../web/OSMTileManager.h"
#include "../bcWidgets/BitmapCanvasParentBase.h"
#include "../wxRealPointEx.h"

class LogHistoryBase;
class AggToolTip;
class RouteElement;
class wxHotKeyManager;
class bcSlider;
class bcLabel;

WX_DEFINE_ARRAY( RouteElement* , RouteElementArray);

WX_DECLARE_HASH_MAP( int, wxBitmap, wxIntegerHash, wxIntegerEqual, Int2wxBitmapHash );
WX_DECLARE_HASH_MAP( int, Int2wxBitmapHash, wxIntegerHash, wxIntegerEqual, Int2Int2wxBitmapHash );
WX_DECLARE_HASH_MAP( int, Int2Int2wxBitmapHash, wxIntegerHash, wxIntegerEqual, Int2Int2Int2wxBitmapHash );

WX_DECLARE_HASH_MAP( int, wxRegion, wxIntegerHash, wxIntegerEqual, Int2wxRegionHash );

class GPSRouteGauge: public VemsWidgetBase, public ObserverBase, public BitmapCanvasParentBase
{
    DECLARE_DYNAMIC_CLASS(GPSRouteGauge);

    public:
        GPSRouteGauge() {};
        GPSRouteGauge(wxWindow* parent, wxWindowID id, GaugeDescManager * gdmgr, GaugeProperties &prop, LogHistoryBase * lhb = NULL);
        virtual ~GPSRouteGauge();

        virtual wxRegion GetShape();
        virtual wxRegion GetBorderRegion();
        virtual void UpdateSize();
        virtual void setSize(int x , int y);

        virtual void SetValue( double value );
        virtual void SetNewTheme(WidgetThemeTable *theme);


        virtual void require();
        virtual void refresh(DoubleHash* table);
        virtual void OnUpdateValue( UpdateViewerEvent& event );

        void SetLogHistoryMgr( LogHistoryBase * lhb ) { m_dataMgr = lhb; } ;

        virtual void InitGauge();


        virtual void notify( wxVariantHash& hash );

        static int long2tileX(double lon, int z);
        static int lat2tileY(double lat, int z);
        static double long2tileXref(double lon, int z);
        static double lat2tileYref(double lat, int z);
        static wxRealPoint deg2tileRef(const wxRealPoint& deg, int z);

        static double tileX2long(int x, int z);
        static double tileY2lat(int y, int z);
        static double tileXref2long(double x, int z);
        static double tileYref2lat(double y, int z);
        static wxRealPoint tileRef2deg(const wxRealPoint& tile, int z);

        static wxRealRect GetTileRealRect(const wxRealRect& degrect, int zoom);
        static wxRealRect GetDegFromTileRef(const wxRealRect& tilerect, int zoom);

        static wxRealRect tile2DegRect(TileDesc& desc);

        void Zoom(int direction, const wxPoint& zoomcenter = wxDefaultPosition);
        void ZoomTo(int newzoom, const wxPoint& zoomcenter = wxDefaultPosition);


    protected:

        void OnPaint(wxPaintEvent& evt);
        void OnSize(wxSizeEvent& evt);
        void OnMouseMove(wxMouseEvent& evt);
        void OnLeftDown(wxMouseEvent& evt);
        void OnLeftUp(wxMouseEvent& evt);
        void OnWheel(wxMouseEvent& evt);
        void OnKeyDown(wxKeyEvent &evt);
        void OnSliderZoom(wxScrollEvent& evt);
        void OnMouseRightDown(wxMouseEvent& evt);



        void PaintRoute(wxDC& dc);
        void PaintDebugPos(wxDC& dc);
        void PaintLocPos(wxDC& dc);

        //void OnMouse(wxMouseEvent& evt);


        void InitRoute();
        void DrawRoute(const wxColor& linecolor = wxNullColour, bool calcregion = false);
        bool DrawRouteTile(TileDesc& desc, const wxColor& linecolor = wxNullColour, bool calcregion = false);
        void DrawNullRoute();
        void DrawBorders();
        void DrawLocationPoint();
        void DrawToolTip(RouteElement * re, wxBitmap ** bmp);

        wxRegion GetPathRegion(const RealPointArray& path);

        bool SearchCache(const wxRealRect& rect, int scale);
        bool SaveToCache();

        wxRealRect SearchBorder();
        void CheckMapTiles();

        wxColor GetThemeColor(const wxString& name);
        double GetThemeDouble(const wxString& name);
        long GetThemeLong(const wxString& name);

        void BuildGPS(const DoubleHashArray& gpsdata);

        RouteElement* SearchByPoint(const wxPoint& pos);
        RouteElement* SearchByData(double longitude, double latitude);

        wxString    FormatTimeSpan(int time_millisec, bool isminsize = false, bool ismillisec = false);

        wxPoint GetPaddingPT() { return wxPoint(m_padding,m_padding); }

        void LoadTileFromFile(TileDesc& desc);
        void SaveTile(wxBitmap& bmp, TileDesc& desc);
        void SaveTile(wxBitmap& bmp, int level, int x, int y);

        void PaintTiles(wxDC& dc);

        wxRealSize GetTileDegSize(int zoom, const wxRect& tilerect);
        //wxRealSize GetTileDegSize(int zoom, const wxRealRect& tilerefrect);
        wxRealSize GetTileDegSize(int zoom, const wxPoint& tile);

        wxPoint GetTile(int zoom, const wxRealPoint& deg);

        void MoveMap(int dir);

        void OnEnter(wxMouseEvent& evt);
        void OnLeave(wxMouseEvent& evt);


        bool TileExistInModel(TileDesc& desc, Int2Int2Int2wxBitmapHash& model);

        void ToggleHelpPanel();
        void ToggleInfoPanel();

        void OnRefreshTimer(wxTimerEvent& evt);

        void GaugeRefresh();

        void InitHelpPanel();

        size_t GetModelSize(Int2Int2Int2wxBitmapHash& model);

        void CreatePopupMenu(wxMenu& menu);

        wxString GetCacheDir();

    private:

        enum {
            idZoomSlider,
            idRefreshTimer
        };

        enum MouseStates {
            RMS_NONE,
            RMS_OVERPATH,
            RMS_ONMAPDRAG
        };

        enum MoveDirections {
            MD_LEFT,
            MD_RIGHT,
            MD_UP,
            MD_DOWN
        };

        LogHistoryBase* m_dataMgr;


        AggToolTip  *   m_toolTipMgr;

        bool            m_isRegistered;
        bool            m_isInited;

        wxBitmap    *   m_bgBmp;
        wxBitmap    *   m_routeBmp;
        wxBitmap    *   m_bgBorder;
        wxBitmap    *   m_toolTipBmp;
        wxBitmap    *   m_toolTipRTPosBmp;
        wxBitmap    *   m_locationPointBmp;

        wxRealRect      m_oriGPSBorder;
        wxRealRect      m_actualMapRect;
        int             m_actualZoom;
        wxRect          m_actualTileRect;
        wxRealRect      m_expandedBorder;
        wxRect          m_mapRect;
        int             m_currScale;
        wxRealPoint     m_mouseDegCoord;

        int             m_padding;

        wxRegion        m_pathRegion;

        Int2wxRegionHash m_pathRegions;

        wxString        m_cacheFileFormat;
        //wxString        m_mapServerFormat;
        wxString        m_mapTileServer;

        int             m_mouseState;

        RouteElementArray m_routeData;

        wxRealPointEx   m_location;


        double          m_deltaOver;

        wxVariantHash   m_themeDesc;

        enum ATTExtraTypes {
            ATTET_ROUTEITEMPOS = 4200
        };

        wxPoint         m_ttPos;

        Int2Int2Int2wxBitmapHash    m_tileBitmaps;
        Int2Int2Int2wxBitmapHash    m_routeBitmaps;

        wxPoint         m_lastMapDragPoint;

        int             m_tileSizePx;
        wxRealRect      m_startDragRect;


        wxCursor        m_dragCursor;
        wxCursor        m_dragDownCursor;

        wxPoint         m_lastMousePos;

        wxHotKeyManager* m_hotKeyMgr;

        OSMTileManager* m_tileMgr;

        wxString        m_textFontPath;

        wxRealPoint     m_defaultGPSCenter;
        long            m_defaultLevel;

        bcSlider*       m_zoomSlider;
        bcLabel*        m_helpPanel;
        bcLabel*        m_infoPanel;


        long            m_debugFlags;

        enum GPSRouteGaugeDebugFlags{
            GDF_NO_DEBUG_DRAW = 0,
            GDF_PAINT_TILE_INFO = 1,
            GDF_USE_DRAW_CURRENT_GPS_POS = 2,
            GDF_ALL = 255
        };

        wxTimer     m_refreshTimer;
        bool        m_isRefresh;
        int         m_refreshTimeBase;

        DECLARE_EVENT_TABLE();
};

class RouteElement
{
    public:
        RouteElement() {};
        RouteElement(DoubleHash* data, const wxRealPoint& point = wxRealPoint(-1,-1)) { Point = point, Data = data; }
        ~RouteElement() { delete Data; }

        double GetVal(const wxString& key, double defval = 0.0) {
            if( Data == NULL ) return defval;
            if( Data->find(key) == Data->end() ) return defval;
            return (*(Data))[key];
        }

        wxRealPoint Point;
        DoubleHash* Data;

};


#endif // GPSROUTEGAUGE_H
