#include "GaugeGroupManager.h"

// need interface the GaugeCarrierFrame.h seems lightweight but cpp has unnecessary dependencies
#include "GaugeCarrierFrame.h"
#include "GaugeType.h"
#include "../common/FunctionMeter.h"

#include "../ConfigBase.h"
#include "../configmanager/ConfigHelper2.h"
#include "../common/MainFrameGUICommandTypes.h"
#include "../events/VemsGUICommandEvent.h"


#include "../logger.h"

#ifndef __WINCE_STANDARDSDK__
	#include "../SendObjectBase.h"
#endif

#include "../mtconfigdata.h"

#include "../loghistory/LogHistoryBase.h"       // not used just fot to pass it!
#include "../UserLog.h"
#include "../common/VemsMessageHashHelper.h"
#include "GroupMemberObject.h"

#include "../gaugedesc/GaugeProperties.h"
#include "../toolbartools/ToolBarToolsNeedId.h"


/** @brief GaugeGroupManager
  *
  * @todo: document this function
  */
GaugeGroupManager::GaugeGroupManager( wxWindow* window, wxMenu* parent, MTConfigData* mtConfigData, LogHistoryBase * loghist )
    :m_timer(this,m_timerID)
{
    m_logHistory = loghist;

#ifndef __WINCE_STANDARDSDK__
    m_MDIparent     = NULL;
    m_AuiMDIparent  = NULL;
#endif
    m_themeMenu     = new wxMenu();
    //m_vemsTuneCtrl  = vemsController;
    m_window        = window;
    m_menu          = new wxMenu(_T(""));
    m_firstItemFocus= -1;

    parent->AppendSubMenu( m_menu, _T("Group") );
    m_window->PushEventHandler( this );

    //m_framelh = NULL;
    m_sendObj = NULL;
    m_lastShowedGroup = wxEmptyString;

    m_mtConfigData = mtConfigData;

    //m_rtDataMgrBase = m_logHistory->getRTDataMgr();
    //m_stDataMgrBase = m_logHistory->getSTDataMgr();

    m_lastFocusedCarrier = NULL;
    m_lastCreatedCarrier = 0;

    m_isEditMode = false;


    m_shwoGrpEvtID =  wxNewId();

    Connect(m_shwoGrpEvtID, wxEVT_COMMAND_ENTER , wxCommandEventHandler(GaugeGroupManager::OnShowGroup) );

    m_timerID = wxNewId();



    Connect(m_timerID, wxEVT_TIMER, wxTimerEventHandler(GaugeGroupManager::OnTimer));


    m_displayRefRatio = 1.0;

}

/** @brief ~GaugeGroupManager
  *
  * @todo: document this function
  */
GaugeGroupManager::~GaugeGroupManager()
{
    //dtor
    hideAndSaveGroups();
}

void GaugeGroupManager::SetFactory(WidgetFactoryBase* factory)
{
    wxString className =  factory->GetClassInfo()->GetClassName();
    m_widgetFactories[className] = factory;
}

void GaugeGroupManager::SetFactory(const wxString& type, WidgetFactoryBase* factory)
{
    m_widgetFactories[type] = factory;
}

void GaugeGroupManager::SetFactory(WidgetFactoryBaseHash& factories)
{
    for(WidgetFactoryBaseHash::iterator it = factories.begin(); it != factories.end(); it++ ) {
        m_widgetFactories[it->first] = it->second;
    }
}

void GaugeGroupManager::SetEditMode(bool edit)
{
    m_isEditMode = edit;


    for( GaugeCarrierBaseArrayHash::iterator it = m_carrierGroups.begin(); it != m_carrierGroups.end(); it++ ){

        if( it->second->GetCount() < 1 ) continue;

        GaugeCarrierBase * carrier = it->second->Item(0);

        if( carrier->IsFloating() ) continue;

        carrier->SetEditMode( m_isEditMode );

    }

}

void GaugeGroupManager::OnTimer(wxTimerEvent &evt)
{
    //Logger::logf(LOG_MSG,_T("GaugeGroupManager::OnTimer m_firstItemFocus=%i, m_lastCreatedCarrier=%p"),m_firstItemFocus,m_lastCreatedCarrier);

    if(m_lastCreatedCarrier) {
        m_lastCreatedCarrier->SetFocusToItem( m_firstItemFocus );
    }

}

void GaugeGroupManager::OnShowGroup(wxCommandEvent &evt)
{
    if( evt.GetId() != m_shwoGrpEvtID ) return;

    Logger::logf(LOG_MSG,_T("GaugeGroupManager::OnShowGroup"));

    m_isGrpChanged = true;
    m_timer.Start(300,true);


    VemsGUICommandEvent guievt(MFCMD_OPENED_GROUP);
    m_window->AddPendingEvent(guievt);
}

bool GaugeGroupManager::SlideFocusToNextTable(wxWindow * mainwin)
{

    bool isfoundedlast = false;

    for( GaugeCarrierBaseArrayHash::iterator it = m_carrierGroups.begin(); it != m_carrierGroups.end(); it++ ){
        GaugeCarrierBaseArray * arr = it->second;

        for( size_t a=0; a < arr->GetCount(); a++ ){

            GaugeCarrierBase * gcb = arr->Item(a);

            VemsControlBase* vcb = gcb->GetWidgetControl();

            if(!vcb) continue;
/*
            TableEditor* t = dynamic_cast<TableEditor*>(vcb);

            if(!t) continue;
*/
            if(!m_lastFocusedCarrier) {
                gcb->SetFocusToItem();
                m_lastFocusedCarrier = gcb;
                return true;
            }

            if(isfoundedlast)  {
                gcb->SetFocusToItem();
                m_lastFocusedCarrier = gcb;
                return true;
            }

            if( gcb == m_lastFocusedCarrier )
                isfoundedlast = true;

        }
    }


    if(!m_lastFocusedCarrier || !mainwin) return false;

    mainwin->SetFocus();
    m_lastFocusedCarrier = NULL;
    return true;
}


/** @brief OnGroup
  *
  * @todo: document this function
  */
void GaugeGroupManager::OnGroup(wxCommandEvent& event)
{
    //wxLogMessage(wxString::Format(_T("OnGroup ID = %d"),event.GetId()));
    int id = event.GetId();
    wxMenuItem* mi = m_menu->FindItem(id);
    wxString groupWithHotKey = mi->GetText();
    wxString groupName = groupWithHotKey.BeforeFirst(_T('\t'));

    //Logger::logf(LOG_MSG, _T("GaugeGroupManager::OnGroup() name=")+groupName);

    if(mi->IsChecked()){
        showGroup(groupName);
    }else{
        hideGroup(groupName);
    }

}


/** @brief OnSave
  *
  * @todo: document this function
  */
void GaugeGroupManager::OnSave( wxCommandEvent& event )
{
    saveAllGroup();
}


/** @brief OnFrameClose
  *
  * @todo: document this function
  */
void GaugeGroupManager::OnFrameClose( wxCloseEvent& event )
{
    Logger::logf( LOG_MSG, _T("GaugeGroupManager::OnFrameClose()") );
    wxString    groupName   = ( (GroupMemberObject*)event.m_callbackUserData )->m_groupName;

    wxWindow * win = ( (GroupMemberObject*)event.m_callbackUserData )->m_parent;

    GaugeCarrierBase* parent = NULL;

    parent = dynamic_cast<GaugeCarrierBase*> (win);

    if( !parent ) {
        Logger::logf(LOG_MSG,_T("GaugeGroupManager::OnFrameClose GaugeCarrierBase dynamic_cast failed"));
        return;
    }

    //frame and group must be exist, no need to check


#ifndef __WINCE_STANDARDSDK__

    if(!parent->IsFloating() && m_sendObj) {
        //m_framelh->SendObject( TN_GGEventHide, NULL, groupName );
        hideGroup( groupName );
        return;
    }

#endif


    m_carrierGroups[ groupName ]->Remove( parent );
    parent->unregisterViewer();
    parent->CloseCarrier();


}

void GaugeGroupManager::OnSetItemFocusId( wxCommandEvent& event )
{
    GroupMemberObject * gmo = ( (GroupMemberObject*)event.m_callbackUserData );

    wxString groupName = gmo->m_groupName;
    int itemID = gmo->m_itemID;

    SetGroupFirstFocusedItem(groupName,itemID,gmo->m_winid);
}


/** @brief Add a gauge to the group
  *
  *
  */
void GaugeGroupManager::OnAddGauge( wxCommandEvent& event )
{

    GroupMemberObject * gmo = ( (GroupMemberObject*)event.m_callbackUserData );

    wxString groupName = gmo->m_groupName;
    wxString gaugeType = gmo->m_type;

    wxPoint pos(0,0);

    if( gmo->m_startPos != 0 ) {
        pos.x = gmo->m_startPos->x;
        pos.y = gmo->m_startPos->y;
    }

    DoAddGauge(groupName,gaugeType,pos);

}

/** @brief OnRemoveGauge
  *
  * @todo: document this function
  */
void GaugeGroupManager::OnRemoveGauge( wxCommandEvent& event )
{
    //GaugeCarrierFrame* gaugeFrame = ( GaugeCarrierFrame* )( ( (GroupMemberObject*)event.m_callbackUserData )->m_parent );


    DoRemoveGauge( ((GroupMemberObject*)event.m_callbackUserData) );
}

GaugeCarrierBase* GaugeGroupManager::GetGroupCarrier(const wxString& groupName)
{
    GaugeCarrierBaseArray * arr = m_carrierGroups[ groupName ];

    for( size_t a=0; a<arr->GetCount(); a++  ) {
        if( arr->Item(a)->IsOnlyOneGauge() ) continue;

        return arr->Item(a);
    }

    return NULL;
}

wxString GaugeGroupManager::GetGroupName(GaugeCarrierBase* carrier)
{
    for(GaugeCarrierBaseArrayHash::iterator it = m_carrierGroups.begin(); it != m_carrierGroups.end(); it++) {

        wxString grpName = it->first;

        GaugeCarrierBaseArray * arr = it->second;

        for( size_t a = 0; a < arr->GetCount(); a++  ) {
            if( arr->Item(a) == carrier ) return grpName;
        }
    }

    return wxEmptyString;
}


/**
  * @brief add gauge to ... group
  */
GaugeCarrierBase* GaugeGroupManager::addGauge(    int id,
                                                const wxString& group,
                                                const wxString& type,
                                                const wxString& name,
                                                const wxPoint& position,
                                                const wxSize& size,
                                                const wxString& theme,
                                                const wxString& desc,
                                                int itemid)
{

    int frameStyle =  wxFRAME_SHAPED | wxFRAME_NO_TASKBAR;

    if ( type.IsSameAs( _T("table") ) || type.IsSameAs( _T("table3d") ) )
    {
        frameStyle = wxMINIMIZE_BOX | wxSYSTEM_MENU | wxCAPTION | wxCLOSE_BOX | wxCLIP_CHILDREN | wxRESIZE_BORDER;
    }


//    m_frameGroups[ group ]->

    GaugeProperties prop;
    const wxPoint POSITION  = wxPoint( 0, 0 ); //gauge position inside the frame

    GaugeCarrierBase * carrier = NULL;



    if( IsGroupFloating(group) ) {

        GaugeCarrierFrame *gFrame = new GaugeCarrierFrame( m_window, id, type, position, size, frameStyle );

        gFrame->SetFactory(m_widgetFactories);
//
//        gFrame->SetDataMgrObj(m_rtDataMgrBase);
//        gFrame->SetDataMgrObj(m_stDataMgrBase);

        gFrame->SetName( name );
        gFrame->OnlyOneGauge( true );


        m_carrierGroups[ group ]->Add( gFrame );

        carrier = gFrame;

        prop.setPos( POSITION );
        prop.extraParams[_T("showBorder")] = true;


    } else {

        carrier = GetGroupCarrier(group);

        prop.setPos( position );
        prop.extraParams[_T("showBorder")] = false;

    }


    if(!carrier) return 0;

    // const int GAUGE_WIDTH   = size.GetWidth();
//    const int GAUGE_HEIGHT  = size.GetHeight();

    prop.gaugeType = GaugeType::getType( type );
    prop.symbolName = name;
    prop.setSize( size );
    prop.themeName = theme;
    prop.descName = desc;
    prop.itemID = itemid;
    prop.groupName = group;

    prop.extraParams[_T("hasControlPanel")] = carrier->IsFloating();

    if ( type.IsSameAs( _T("table") ) )
    {
        prop.symbolName = _T("veTableTbl");

        if(carrier->IsFloating())
            carrier->SetBGColor( wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE ) );

        carrier->AddVemsTable( prop );

    }
    else if ( type.IsSameAs( _T("table3d") ) )
    {

        prop.symbolName = _T("veTableMap");

        if(carrier->IsFloating())
            carrier->SetBGColor( wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE ) );

        carrier->AddVemsTable( prop );
    }
    else
    {
        carrier->AddGauge( prop );
    }


    Logger::logf(LOG_MSG,_T("GaugeGroupManager::addGauge"));

    return carrier;
}

GaugeGroupBehav GaugeGroupManager::getGaugeGroupsBehav()
{
    ConfigBase* cfg = ConfigHelper2::getConfig();
    //if(!cfg) return GGB_MULTI;
    cfg->setPath(_T("/environment"));
    wxString res;
    cfg->read(_T("gauge_groups_behav"), &res, _T("multi"));

    if( res == _T("single") )
        return GGB_SINGLE;
    else if( res == _T("multi") )
        return GGB_MULTI;

    return GGB_SINGLE;


}

int GaugeGroupManager::getGaugeGroupsFloatBehav()
{
    ConfigBase* cfg = ConfigHelper2::getConfig();
    cfg->setPath(_T("/environment"));
    long res = 0;
    cfg->read(_T("gauge_groups_float_behav"), &res, 0L);
    return res;

}


bool GaugeGroupManager::saveAllGroup(const wxString& filename)
{
    FunctionMeter fm(_T("GaugeGroupManager::save"));

    for(GaugeCarrierBaseArrayHash::const_iterator it = m_carrierGroups.begin(); it != m_carrierGroups.end(); it++ )
    {
        bool res = saveGroup(it->first);
        if(!res) return false;
    }

    return true;
}


/** @brief Hide group by name
  */
short GaugeGroupManager::hideGroup( const wxString& groupName, bool removeFromSaved )
{
    Logger::logf( LOG_MSG, _T("GaugeGroupManager::hideGroup() - ") + groupName );

    if(groupName == wxEmptyString) return -2;

    saveGroup(groupName);

    if ( m_carrierGroups.find( groupName ) == m_carrierGroups.end() ) { return -1; }

    FunctionMeter fm(_T("GaugeGroupManager::hideGroup"), true);

    GaugeCarrierBaseArray* fa = m_carrierGroups[ groupName ];

    if( !fa ) { return -1; }

    for(size_t i=0; i<fa->GetCount(); i++)
    {

        GaugeCarrierBase* carrier = fa->Item(i);
        carrier->unregisterViewer();
        carrier->CloseCarrier();

        if(m_lastCreatedCarrier == carrier) m_lastCreatedCarrier = NULL;
    }
    fa->Clear();

    m_carrierGroups.erase( groupName );

#ifndef __WINCE_STANDARDSDK__
    if(m_sendObj) m_sendObj->SendObject( TN_GGEventHide, NULL, groupName );
#endif

    delete fa;
    m_menu->Check(m_menu->FindItem( groupName ), false);

    if ( removeFromSaved )
    {

        int groupIndex = m_activeGroupNames.Index( groupName );
        if ( groupIndex != wxNOT_FOUND ) m_activeGroupNames.RemoveAt( groupIndex );

        Logger::logf( LOG_MSG, _T("GaugeGroupManager::hideGroup() - ") + groupName );
    }

    return 1;
}

void GaugeGroupManager::hideAndSaveGroups()
{
//    Logger::logf(LOG_MSG,_T("GaugeGroupManager::hideAll() - BEGIN"));
//
//    GaugeCarrierBaseArrayHash::const_iterator it;
//    //wxArrayString groupNames;
//    m_activeGroupNames.clear();
//
////    for( it = m_carrierGroups.begin(); it != m_carrierGroups.end(); ++it )
////    {
////
////        m_activeGroupNames.Add( it->first );
////        Logger::logf( LOG_MSG, _T("GaugeGroupManager::hideAll() - group name: ") + it->first );
////
////    }

    for ( size_t i = 0; i < m_activeGroupNames.GetCount(); ++i )
    {
        //wxThread::Sleep( 500 );
        Logger::logf( LOG_MSG, _T("GaugeGroupManager::hideAll() - group name: ") + m_activeGroupNames[i] );
        hideGroup( m_activeGroupNames[i], false );

    }


    //Logger::logf(LOG_MSG,_T("GaugeGroupManager::hideAll() - END"));
}

void GaugeGroupManager::restoreGroups()
{
    for ( size_t i = 0; i < m_activeGroupNames.GetCount(); ++i )
    {
        //wxThread::Sleep( 500 );
        Logger::logf( LOG_MSG, _T("GaugeGroupManager::restoreGroups() - ") +
                        m_activeGroupNames[i]);
        showGroup( m_activeGroupNames[i], false );
    }
}


void GaugeGroupManager::OnTheme( wxCommandEvent& event )
{
    int id = event.GetId();

    SetTheme( id );
}

wxString GaugeGroupManager::getGroupTheme( const wxString& grpname )
{
    wxMenuItem *grpmenu = m_themeMenu->FindItem(m_themeMenu->FindItem( grpname ));

    wxMenu *grpthememenu = grpmenu->GetSubMenu();

    for(uint8_t a=0; a < grpthememenu->GetMenuItemCount(); a++){
        wxMenuItem * thm = grpthememenu->FindItemByPosition(a);
        if( thm->IsChecked() ) {
            return thm->GetItemLabel();
        }
    }

    return wxEmptyString;
}

void GaugeGroupManager::ProcessWidgetCommand(wxVariantHash& cmdHash)
{

    if(cmdHash.find(_T("groupname")) != cmdHash.end() ) {
        ProcessWidgetCommandGrp(cmdHash[_T("groupname")].GetString(), cmdHash);
    } else {
        for( GaugeCarrierBaseArrayHash::iterator it = m_carrierGroups.begin(); it != m_carrierGroups.end(); it++ ){
            ProcessWidgetCommandGrp( it->first, cmdHash );
        }
    }

}

void GaugeGroupManager::ProcessWidgetCommandGrp(const wxString& grpname, wxVariantHash& cmdHash)
{
    if(m_carrierGroups.find(grpname) == m_carrierGroups.end()) return;

    GaugeCarrierBaseArray* fa = m_carrierGroups[ grpname ];

    if( !fa ) { return; }

    for(size_t i=0; i<fa->GetCount(); i++)
    {
        fa->Item(i)->ProcessWidgetCommand(cmdHash);
    }
}

void GaugeGroupManager::ResetAllGaugesMinMax()
{
    for( GaugeCarrierBaseArrayHash::iterator it = m_carrierGroups.begin(); it != m_carrierGroups.end(); it++ ){
        ResetGaugesMinMax( it->first );
    }
}

void GaugeGroupManager::ResetGaugesMinMax(const wxString &groupname)
{
    if(m_carrierGroups.find(groupname) == m_carrierGroups.end()) return;

    GaugeCarrierBaseArray* fa = m_carrierGroups[ groupname ];

    if( !fa ) { return; }

    for(size_t i=0; i<fa->GetCount(); i++)
    {
        fa->Item(i)->ResetMinMax();
    }
}

bool GaugeGroupManager::showGroupByHotkey(const wxString &hkey)
{
    if(m_hotkeyHash.find(hkey) == m_hotkeyHash.end()) return false;

    showGroup( m_hotkeyHash[hkey] );

    return true;
}

void GaugeGroupManager::SetFocus(){

    if( !m_isGrpChanged ) return;
    if( !m_lastCreatedCarrier ) return;

    m_lastCreatedCarrier->SetFocusToItem();

    m_isGrpChanged = false;
}

int GaugeGroupManager::AddGaugeToGroup(GaugeProperties& prop, GaugeCarrierBase* maincarrier, bool isFloating, GaugeCarrierBase** vgcarrier, GaugeCarrierBase** tablecarrier)
{

    int gaugewinid = -1;

    int id = wxNewId();

    if( prop.gaugeType == GaugeType::TABLE ) {

        VemsTableDescH* tableDescH = m_mtConfigData->getTables();
        if( tableDescH->find( prop.symbolName ) == tableDescH->end() )
        {

            UserLog::uLog(ULOG_WARNING, _T("Table has not been found: ")+ prop.symbolName);
            return -1;
        }

        if(isFloating) {

            *tablecarrier = new GaugeCarrierFrame( m_window, id, prop.symbolName, prop.getPos(), wxDefaultSize, wxMINIMIZE_BOX | wxSYSTEM_MENU | wxCAPTION | wxCLOSE_BOX | wxCLIP_CHILDREN | wxRESIZE_BORDER );
            //(*tablecarrier)->SetGroupMgr( this );

            (*tablecarrier)->OnlyOneGauge( true );

            //(*tablecarrier)->SetDataMgrObj(m_rtDataMgrBase);
//            (*tablecarrier)->SetDataMgrObj(m_stDataMgrBase);

            (*tablecarrier)->SetBGColor( wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE ) );

            (*tablecarrier)->SetFactory(m_widgetFactories);

            prop.setPos(-1,-1);

            prop.extraParams[_T("hasControlPanel")] = true;

            wxWindow* gm = (*tablecarrier)->AddVemsTable( prop );
            if(gm) gaugewinid =  gm->GetId(); //gm->GetWinID();

            m_carrierToItemId[ *tablecarrier ] = prop.itemID;
            m_carrierGroups[ prop.groupName ]->Add( *tablecarrier );


        } else {

            prop.extraParams[_T("hasControlPanel")] = false;

            wxWindow* gm = maincarrier->AddVemsTable( prop );
            if(gm)gaugewinid = gm->GetId(); //gm->GetWinID();
        }

    } else {

        StringHash hash;
        hash[_T("symbol")] = prop.symbolName;

        if ( prop.gaugeType == GaugeType::EXPRESSION_GRID )
        {
            if ( m_mtConfigData->getExpressionGridDescriptor( prop.symbolName ) == 0 )
            {
                UserLog::vLog( VemsMessageHashHelper::GetVemsMsg(_T("symbol_not_found_2"),&hash) );
                return -1;
            }
        }
        else
        {
            if(m_mtConfigData->getSymbol( prop.symbolName ) == 0 ) {
                UserLog::vLog( VemsMessageHashHelper::GetVemsMsg(_T("symbol_not_found_2"),&hash) );
                return -1;
            }
        }

        //Logger::logf(LOG_MSG,_T("load pos.x=%i,pos.y=%i,w=%i,h=%i"),prop.x,prop.y,prop.width,prop.height);

        prop.extraParams[_T("showBorder")] = false;

        if(isFloating) {
            *vgcarrier = new GaugeCarrierFrame( m_window, id, GaugeType::toString(prop.gaugeType), prop.getPos(), prop.getSize(), 0 | wxFRAME_SHAPED | wxFRAME_NO_TASKBAR );
//            (*vgcarrier)->SetGroupMgr( this );
            (*vgcarrier)->OnlyOneGauge( true );
            (*vgcarrier)->SetFactory(m_widgetFactories);
            prop.x = 0;
            prop.y = 0;
            m_carrierToItemId[ *vgcarrier ] = prop.itemID;
            m_carrierGroups[ prop.groupName ]->Add( *vgcarrier );
//            (*vgcarrier)->SetDataMgrObj(m_rtDataMgrBase);
//            (*vgcarrier)->SetDataMgrObj(m_stDataMgrBase);

            wxWindow* vwb = (*vgcarrier)->AddGauge( prop );

            if(vwb) gaugewinid = vwb->GetId();

        } else {

            wxWindow* vwb = maincarrier->AddGauge( prop );

            if(vwb) gaugewinid = vwb->GetId();

        }


    }

    return gaugewinid;
}


