#include "GaugeGroupManagerSQL.h"

#include "GaugeCarrierFrame.h"
#include "GaugeType.h"

#include "../configmanager/DeviceConfigHelper.h"
#include "../DisplayHelper.h"
#include "../common/FunctionMeter.h"
#include "../SendObjectBase.h"
#include "../toolbartools/VemsToolBarToolBase.h"
#include "VemsControlBase.h"

#include "../mtconfigdata.h"
#include "../wxSqliteHelper.h"
#include "../logger.h"
#include "../UserLog.h"
#include "../gaugedesc/GaugeDescHelper.h"
#include "GaugeCarrierPanel.h"
#include "GaugeCarrierMDIChildFrame.h"
#include "GaugeCarrierAuiMDIChildFrame.h"

#include "TableEditorPanel.h"

#include "GroupMember.h"
#include "GroupMemberObject.h"

#include "../configmanager/ConfigHelper2.h"
#include "../wxSizeEx.h"

#include "../loghistory/LogHistoryBase.h"

#define chkstr(var,defval) \
if( var == wxEmptyString ) var = defval;

#define chkdb(retval) if( ! (m_db.IsOpen()) ) return retval;


GaugeGroupManagerSQL::GaugeGroupManagerSQL(wxWindow* window, wxMenu* parent, MTConfigData* mtConfigData, LogHistoryBase * loghist)
    :GaugeGroupManager( window, parent, mtConfigData, loghist )
{
    //ctor
    m_groupShootTime = wxDateTime::UNow();
}

GaugeGroupManagerSQL::~GaugeGroupManagerSQL()
{
    //dtor
    m_db.Close();
}

void GaugeGroupManagerSQL::StartTransaction()
{
    chkdb();

    m_db.Begin();
}

void GaugeGroupManagerSQL::EndTransaction()
{
    chkdb();

    m_db.Commit();
}

wxString GaugeGroupManagerSQL::GetHotKeyStringByGroupname(const wxString &grpname)
{
    chkdb(wxEmptyString);

    wxSQLite3ResultSet res = m_db.ExecuteQuery( wxString::Format(_T("select hotkey from groups where name = '%s';"),grpname.c_str()) );
    if(res.Eof()) {
        return wxEmptyString;
    }

    return res.GetString(0);
}

void GaugeGroupManagerSQL::reloadHotkeys()
{
    chkdb();

    m_hotkeyHash.clear();

    wxSQLite3ResultSet res = m_db.ExecuteQuery( _T("select name,hotkey from groups;") );

    while( res.NextRow() ) {
        m_hotkeyHash[res.GetString(_T("hotkey"))] = res.GetString(_T("name"));
    }
}


void GaugeGroupManagerSQL::CheckGroupDB()
{
    ColumnDescArrayHash desc;

    ColumnDescArray arr1;

    arr1.Add( ColumnDesc(_T("id"), _T("INTEGER"), wxNullVariant, true, true ) );
    arr1.Add( ColumnDesc(_T("name"), _T("VARCHAR(64)")) );
    arr1.Add( ColumnDesc(_T("short_name"), _T("VARCHAR(32)")) );
    arr1.Add( ColumnDesc(_T("hotkey"), _T("VARCHAR(16)")) );
    arr1.Add( ColumnDesc(_T("sticked"), _T("INTEGER(1)"), 0L ) );
    arr1.Add( ColumnDesc(_T("theme"), _T("VARCHAR(32)")) );
    arr1.Add( ColumnDesc(_T("show_on_startup"), _T("INTEGER(1)"), 0L ) );
    arr1.Add( ColumnDesc(_T("show_on_startup_offline_cfg"), _T("INTEGER(1)"), 0L ) );
    arr1.Add( ColumnDesc(_T("show_on_startup_online"), _T("INTEGER(1)"), 0L ) );
    arr1.Add( ColumnDesc(_T("floated"), _T("INTEGER(1)"), 1L ) );
    arr1.Add( ColumnDesc(_T("settofocus_gaugeid"), _T("INTEGER"), -1 ) );

    desc[_T("groups")] = arr1;

    ColumnDescArray arr2;

    arr2.Add( ColumnDesc(_T("id"), _T("INTEGER"), wxNullVariant, true, true ) );
    arr2.Add( ColumnDesc(_T("gauge_id"), _T("INTEGER")) );
    arr2.Add( ColumnDesc(_T("type"), _T("VARCHAR(32)")) );
    arr2.Add( ColumnDesc(_T("name"), _T("VARCHAR(32)")) );

    desc[_T("multigauge")] = arr2;

    wxSqliteHelper::DatabaseCheck(m_dbGroups,desc);
}

bool GaugeGroupManagerSQL::load(const wxString& gaugesfile, const wxString& groupsfile)
{
    clearThemeMenu();
    clearGroupMenu();

    if( !wxSqliteHelper::Exist(gaugesfile) ) return false;
    if( !wxSqliteHelper::Exist(groupsfile) ) return false;

    m_dbGauges = gaugesfile;
    m_dbGroups = groupsfile;

    CheckGroupDB();

    m_db.Open( gaugesfile );

    m_db.ExecuteQuery(wxString::Format(_T("attach database \"%s\" as groupsdb;"),groupsfile.c_str()));

    m_mainThemeName = ThemesHelper::m_defaultThemeName;

    wxSQLite3ResultSet res = m_db.ExecuteQuery( _T("select * from groups order by name asc;") );

    m_hotkeyHash.clear();

    while( res.NextRow() ) {

        wxString groupName = res.GetString( _T("name") );
        wxString hotKey = res.GetString( _T("hotkey") );

        m_hotkeyHash[hotKey] = groupName;

        int id = wxNewId();

        hotKey.Replace( _T("+"), _T(" + "), false );

        m_menu->AppendCheckItem( id, groupName + _T("\t") + hotKey );

        Connect( id,  wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( GaugeGroupManager::OnGroup ) );

        wxString theme = res.GetString( _T("theme") );

        wxMenu *tmenu = ThemesHelper::CreateThemeMenu(ThemesHelper::GetWidgetThemes(), this, wxCommandEventHandler( GaugeGroupManager::OnTheme ),theme,groupName,&m_themeGrpHash);
        m_themeMenu->AppendSubMenu(tmenu,groupName);

    }

    m_menu->AppendSubMenu(m_themeMenu,_T("Group Theme"));

    wxSize rs = DeviceConfigHelper::GetRefDisplaySize();

    if( rs != wxDefaultSize ) {

        wxSize ds = DisplayHelper::GetDisplaySize();
        m_displayRefRatio = (double)(ds.x) / (double)(rs.x);

    }

    return true;
}

void GaugeGroupManagerSQL::showOnStartup(ShowOnStartupFlags flags)
{

    chkdb();

    wxString qs = _T("select name from groups where ");
    switch(flags) {
        case SOSF_ONLINE:
            qs += _T("show_on_startup_online");
            break;
        case SOSF_OFFLINE_CONFIG:
            qs += _T("show_on_startup_offline_cfg");
            break;
        case SOSF_OFFLINE_LOG:
            qs += _T("show_on_startup");
            break;
        default:
            return;
    }

    qs += _T(" = 1;");

    //Logger::logf(LOG_MSG,_T("qs=")+qs);

    wxSQLite3ResultSet res = m_db.ExecuteQuery( qs );

    if(res.IsNull(0))
        return;

    wxString groupName = res.GetString(_T("name"));

    int r = showGroup( groupName );

    //Logger::logf(LOG_MSG,_T("groupName=")+groupName);

	if( r == 1 ){
        m_menu->Check( m_menu->FindItem( groupName ), true );
	}
}

bool GaugeGroupManagerSQL::SaveSymbol(int itemid, const wxString& name)
{
    SaveSymbolname(itemid, name);
    return true;
}

void GaugeGroupManagerSQL::SetGroupFirstFocusedItem(const wxString &grpname, int itemid, int winid)
{
    chkdb();

    int res = m_db.ExecuteUpdate( wxString::Format(_T("update groups set settofocus_gaugeid='%i' where name='%s';"),itemid,grpname.c_str() ) );

    if(res == 1)
        m_firstItemFocus = winid;
}

short GaugeGroupManagerSQL::showGroup(const wxString& groupName, bool wait )
{
    chkdb(-5)

    Logger::logf(LOG_MSG,_T(" - new id = %i"),wxNewId());

    Logger::logf(LOG_MSG,_T("GaugeGroupManagerSQL::showGroup groupName=")+groupName);

    FunctionMeter fm(_T("GaugeGroupManagerSQL::showGroup"));

    wxDateTime tmp = m_groupShootTime;

    if( wait && tmp.Add(wxTimeSpan(0,0,0,500)).IsLaterThan( wxDateTime::UNow() )  ) {
        Logger::logf(LOG_MSG,_T("take it easy!!!"));
        return -4;
    }

    if( !m_mtConfigData->isValid() ){
        wxString msg = _T("<b>Unable to show gauge group: ") + groupName + _T("! It has 2 reasons:</b><br></font><font>");
        msg +=_T("<ul><li>ECU not connected and project is not selected!</li>");
        msg +=_T("<li>ECU connected but the firmware is older than 1.0.73!</li></ul>");
        UserLog::uLog( ULOG_WARNING, msg );
        return -2;
    }

    if(m_lastShowedGroup != groupName && m_lastShowedGroup != wxEmptyString)
        m_isGrpChanged = true;
    else
        m_isGrpChanged = false;



    short hgr = 0;
    if( getGaugeGroupsBehav() == GGB_SINGLE && !isStickedGroup(m_lastShowedGroup) && m_lastShowedGroup != wxEmptyString ) {
        hgr = hideGroup( m_lastShowedGroup );

        if( m_lastShowedGroup == groupName && hgr == 1 ){
            m_lastShowedGroup = wxEmptyString;
            return -1;
        }
    }


    if( m_carrierGroups.find( groupName ) == m_carrierGroups.end() ) {

        ConfigBase* cfg = ConfigHelper2::getConfig();
        cfg->setPath(_T("/"));
        size_t max = cfg->read(_T("max_open_group"),0L);

        if(max && m_carrierGroups.size() >= max && m_activeGroupNames.GetCount() > 0) {
            hideGroup(m_activeGroupNames[0]);
        }

        m_carrierGroups[ groupName ]= new GaugeCarrierBaseArray();
        if ( m_activeGroupNames.Index( groupName ) == wxNOT_FOUND ) {
            m_activeGroupNames.Add( groupName );
        }

    } else {

        if(m_AuiMDIparent) {

            wxAuiNotebook* nbook = m_AuiMDIparent->GetNotebook();

            int sel = nbook->GetSelection();

            if(nbook->GetPageText(sel) == groupName) {
                hideGroup( groupName );
            } else {
                for(size_t a=0; a<nbook->GetPageCount(); a++){
                    if(nbook->GetPageText(a) == groupName) {
                        nbook->SetSelection(a);
                        break;
                    }
                }
            }

        } else {
            hideGroup( groupName );
        }

        return -1;
    }


    wxArrayString arr;
    arr.Add(_T("gauges"));


    if( !wxSqliteHelper::TablesCheck( arr, m_dbGauges ) )
        return -3;


    arr.Clear();
    arr.Add(_T("groups"));
    arr.Add(_T("multigauge"));

    if( !wxSqliteHelper::TablesCheck( arr, m_dbGroups ) )
        return -3;


    wxSQLite3ResultSet groupRes = m_db.ExecuteQuery( wxString::Format(_T("select * from groups where name = '%s';"),groupName.c_str()) );

    if( groupRes.IsNull(0) )
        return -2;





    int group_id = groupRes.GetInt(_T("id"));

    wxString groupTheme = groupRes.GetString(_T("theme"));
    chkstr(groupTheme,m_mainThemeName);

    wxString hotkey = groupRes.GetString(_T("hotkey"));

    bool isFloating = groupRes.GetBool(_T("floated"));

    int firstfocusitem = groupRes.GetInt(_T("settofocus_gaugeid"));

    int fb = getGaugeGroupsFloatBehav();

    switch(fb){
        case GGFB_ATTACH: isFloating = false; break;
        case GGFB_FLOAT: isFloating = true; break;
        default: break;
    }

    wxString qs = wxString::Format( _T("select * from gauges where group_id = %i;"), group_id);

    wxSQLite3ResultSet res = m_db.ExecuteQuery( qs );

    GaugeCarrierAuiMDIChildFrame * maingaugepanel = NULL;


    wxBookCtrl * book = NULL;

    int leftcorner = 0, topcorner = 0, bottomcorner = 0, rightcorner = 0;

    Logger::logf(LOG_MSG,_T("isFloating=%i"),isFloating);

    if(!isFloating){

        maingaugepanel = new GaugeCarrierAuiMDIChildFrame( m_AuiMDIparent, wxNewId(), groupName, this);

//        maingaugepanel->SetGroupMgr( this );
        maingaugepanel->SetGaugeShowInit( true );
        maingaugepanel->SetFloating( false );
        maingaugepanel->OnlyOneGauge( false );

//        maingaugepanel->SetDataMgrObj(m_rtDataMgrBase);
//        maingaugepanel->SetDataMgrObj(m_stDataMgrBase);

//        Logger::logf(LOG_MSG,_T("!!!!!!m_widgetFactories size=%i"),m_widgetFactories.size());

        maingaugepanel->SetFactory(m_widgetFactories);


        wxSQLite3ResultSet minres = m_db.ExecuteQuery( wxString::Format(_T("select min(x),min(y),max(x+width),max(y+height) from gauges where group_id = %i;"),group_id) );
        leftcorner = minres.GetInt(0);
        topcorner  = minres.GetInt(1);
        rightcorner = minres.GetInt(2);
        bottomcorner  = minres.GetInt(3);

        maingaugepanel->SetVirtualPos( wxPoint(leftcorner,topcorner) );

        Logger::logf(LOG_MSG,_T("virtpos = %i,%i"),leftcorner, topcorner);

    }

    GaugeCarrierBase * vgcarrier = NULL;
    GaugeCarrierBase * tablecarrier = NULL;

    wxSize ds = DisplayHelper::GetDisplaySize();

    while( res.NextRow() ) {

        GaugeProperties prop;

        prop.groupName      = groupName;
        prop.groupHotkey    = hotkey;
        prop.itemID         = res.GetInt(_T("id"));
        prop.x              = res.GetInt(_T("x")) - leftcorner;
        prop.y              = res.GetInt(_T("y")) - topcorner;
        prop.width          = res.GetInt(_T("width"));
        prop.height         = res.GetInt(_T("height"));
        prop.symbolName     = res.GetString(_T("name"));
        prop.gaugeType      = GaugeType::getType( res.GetString(_T("type")) );
        prop.themeName      = res.GetString(_T("theme"));
        prop.descName       = res.GetString(_T("descriptor"));

        prop.x = (long int)(prop.x * m_displayRefRatio);
        prop.y = (long int )(prop.y * m_displayRefRatio);
        prop.width = (long int)(prop.width * m_displayRefRatio);
        prop.height = (long int)(prop.height * m_displayRefRatio);

        if( prop.x > ds.x )
            prop.x = ds.x - prop.width;

        if( prop.y > ds.y )
            prop.y = ds.y - prop.height;

        if( prop.x < 0 - prop.width )
            prop.x = 0;

        if( prop.y < 0 - prop.height )
            prop.y = 0;

        //Logger::logf(LOG_MSG,_T("prop.x=%i,prop.y=%i, leftcorner=%i, topcorne=%i"),prop.x,prop.y, leftcorner, topcorner);

///************************ from

        if(prop.symbolName == _T("MULTIGAUGE")) /// need to refactor
            continue;


        chkstr(prop.descName,_T("defaultdesc"));
        chkstr(prop.themeName,groupTheme);

        if( prop.height == 0 )
            prop.height = prop.width;



        int gaugewinid = AddGaugeToGroup(prop,maingaugepanel, isFloating, &vgcarrier, &tablecarrier);


        ///gFrame->SetName( prop.symbolName );


        if(vgcarrier) vgcarrier->ShowCarrier(true);
        if(tablecarrier) tablecarrier->ShowCarrier(true);



        if( firstfocusitem == -1 ) {
            m_firstItemFocus = gaugewinid;
            firstfocusitem = -2;
        }

        if(prop.itemID == firstfocusitem)
            m_firstItemFocus = gaugewinid;



    }

    if(!isFloating && maingaugepanel){

        if(book)
            book->AddPage(maingaugepanel,groupName,true);

        maingaugepanel->ShowCarrier(true);

        m_carrierGroups[ groupName ]->Add( maingaugepanel );
    }

    m_lastShowedGroup = groupName;

    if(m_sendObj) m_sendObj->SendObject( TN_GGEventShow, NULL, groupName );


    m_menu->Check(m_menu->FindItem( groupName ), true);


    if(maingaugepanel) {

        m_lastCreatedCarrier = maingaugepanel;

    } else if(tablecarrier) {
       // tablecarrier->GetTable()->GetTable()->SetFocus();
        m_lastCreatedCarrier = tablecarrier;
    } else {
        m_lastCreatedCarrier = vgcarrier;
    }



    m_groupShootTime = wxDateTime::UNow();

    wxCommandEvent evt(wxEVT_COMMAND_ENTER, m_shwoGrpEvtID);
    evt.SetEventObject( this );
    AddPendingEvent( evt );

    ResetAllGaugesMinMax();

    m_logHistory->refreshLastPosition();

    return 1;
}


/// set theme for all the gauges on the group
void GaugeGroupManagerSQL::SetTheme(int id) /// menu id
{
    chkdb();

    if( m_themeGrpHash.find(id) == m_themeGrpHash.end() ) return;

    m_db.ExecuteUpdate( wxString::Format(_T("update groups set theme = '%s' where name = '%s';"), m_themeGrpHash[id]->Theme.c_str(), m_themeGrpHash[id]->Group.c_str()) );

    if( m_carrierGroups.find(m_themeGrpHash[id]->Group) == m_carrierGroups.end() ) return;

    size_t c = m_carrierGroups[m_themeGrpHash[id]->Group]->GetCount();

    for(size_t a=0; a < c; a++){
        //ThemeAble *w = m_carrierGroups[m_themeGrpHash[id]->Group]->Item(a)->GetThemedObj();

        //VemsWidgetBase* vwb  = m_carrierGroups[m_themeGrpHash[id]->Group]->Item(a)->GetVemsWidget();
        VemsControlBase* c = m_carrierGroups[m_themeGrpHash[id]->Group]->Item(a)->GetWidgetControl();

        if(!c) continue;

//        VemsWidgetBase* vwb = dynamic_cast<VemsWidgetBase*>(c);
//
//        if(!vwb) continue;

        ThemeAbleBase* w = dynamic_cast<ThemeAbleBase*>(c);

        if(!w) continue;

            w->SetNewTheme(m_themeGrpHash[id]->Theme);
    }


    //Logger::logf(LOG_MSG,_T("%p"),m_themeGrpHash[id]);
}

/// save theme with one gauge
void GaugeGroupManagerSQL::SaveTheme(int gaugeID, const wxString &name)
{
    chkdb();

    wxSQLite3ResultSet res = m_db.ExecuteQuery( wxString::Format(_T("select theme from groups where id = (select group_id from gauges where id=%i);"),gaugeID) );

    wxString theme = name;

    if( res.GetString(0) == theme )
        theme = wxEmptyString;

    m_db.ExecuteUpdate( wxString::Format(_T("update gauges set theme='%s' where id=%i;"), theme.c_str(),gaugeID) );

}

void GaugeGroupManagerSQL::SaveSymbolname(int gaugeID, const wxString &name)
{
    chkdb();

    m_db.ExecuteUpdate( wxString::Format(_T("update gauges set name='%s' where id='%i';"),name.c_str(), gaugeID) );
}

void GaugeGroupManagerSQL::SaveDesc(int gaugeID, const wxString &name)
{
    chkdb();

    wxString qs = wxString::Format(_T("update gauges set descriptor='%s' where id='%i';"),name.c_str(), gaugeID);

    m_db.ExecuteUpdate( qs );
}

void GaugeGroupManagerSQL::setStickGroup(const wxString &groupname, bool isStick)
{
    chkdb();

    m_db.ExecuteUpdate( wxString::Format(_T("update groups set sticked = %i where name = '%s';"), isStick, groupname.c_str()) );
}

bool GaugeGroupManagerSQL::isStickedGroup(const wxString &groupname)
{
    chkdb(false);

    wxSQLite3ResultSet res = m_db.ExecuteQuery( wxString::Format(_T("select sticked from groups where name = '%s';"), groupname.c_str()) );

    return res.GetInt(0)!=0?true:false;
}

void GaugeGroupManagerSQL::SaveMultiSymbolname(int gaugeID, const wxString &name)
{
    chkdb();

    m_db.ExecuteUpdate( wxString::Format(_T("update multigauge set name='%s' where id='%i';"),name.c_str(), gaugeID) );
}

void GaugeGroupManagerSQL::DoRemoveGauge(GaugeCarrierFrame* gaugeFrame)
{
    chkdb();

    m_db.ExecuteUpdate( wxString::Format(_T("delete from gauges where id=%i;"), m_carrierToItemId[ gaugeFrame ]) );
    Logger::logf(LOG_MSG,_T("GaugeGroupManagerSql::DoRemoveGauge GaugeCarrierFrame"));

    //m_carrierGroups[v->GetGroupName()]->Remove( gaugeFrame );
    m_carrierToItemId.erase( gaugeFrame );

    gaugeFrame->Close();
}

void GaugeGroupManagerSQL::DoRemoveGauge(GaugeProperties& props)
{
    chkdb();

    m_db.ExecuteUpdate( wxString::Format(_T("delete from gauges where id=%i;"), props.itemID ) );

    Logger::logf(LOG_MSG,_T("GaugeGroupManagerSql::DoRemoveGauge GaugeProperties"));

    GaugeCarrierBase * gaugecarrier = NULL;

    gaugecarrier = GetGroupCarrier( props.groupName );

    if(gaugecarrier == NULL) return;

    if( gaugecarrier->IsFloating() ) {

        m_carrierGroups[ props.groupName ]->Remove( gaugecarrier );
        m_carrierToItemId.erase( gaugecarrier );

        gaugecarrier->unregisterViewer();

        gaugecarrier->CloseCarrier();

    } else {

        gaugecarrier->CloseGauge( props.winID );
    }
}

void GaugeGroupManagerSQL::DoRemoveGauge(GroupMemberObject * gmo)
{
    GaugeProperties prop;

    prop.groupName = gmo->m_groupName;
    prop.itemID = gmo->m_itemID;
//    prop.gaugeType = gmo->m_type;
    prop.winID = gmo->m_winid;

    DoRemoveGauge(prop);
}

bool GaugeGroupManagerSQL::IsGroupFloating(const wxString &grpname)
{
    chkdb(false);

    int fb = getGaugeGroupsFloatBehav();

    switch(fb){
        case GGFB_ATTACH: return false;
        case GGFB_FLOAT: return true;
        default: break;
    }

    wxSQLite3ResultSet res = m_db.ExecuteQuery( wxString::Format(_T("select floated from groups where name='%s';"),grpname.c_str()) );
    return res.GetBool(0);
}

bool GaugeGroupManagerSQL::SaveGauge(GaugeProperties& prop)
{
    chkdb(false);

    wxString qs  = _T("insert into gauges (name,type,x,y,width,height,descriptor,theme,group_id) ");

    qs += wxString::Format( _T(" values ('%s','%s',%i,%i,%i,%i,'%s','%s', (select id from groups where name = '%s') );"),
                            prop.symbolName.c_str(),
                            GaugeType::toString(prop.gaugeType).c_str(),
                            prop.x,
                            prop.y,
                            prop.width,
                            prop.height,
                            prop.descName.c_str(),
                            prop.themeName.c_str(),
                            prop.groupName.c_str() );


    m_db.ExecuteUpdate( qs );

    prop.itemID = m_db.GetLastRowId().ToLong();

    return true;
}

void GaugeGroupManagerSQL::DoAddGauge( GaugeProperties prop )
{
    chkdb();

    wxString groupName = prop.groupName;

    if( groupName == wxEmptyString ) { return; }

    wxString gaugeType = GaugeType::toString(prop.gaugeType);

    //set new gauge parameters
    wxString  name =  prop.symbolName == _T("rpm") ? GaugeType::GetDefault(_T("symbol"), gaugeType).GetString() : prop.symbolName;

    if ( gaugeType.IsSameAs( _T("table") ) ) { name = _T("veTableTbl"); }
    else if ( gaugeType.IsSameAs( _T("table3d") ) ) { name = _T("veTableMap"); }
    else if ( prop.gaugeType == GaugeType::EXPRESSION_GRID )
    {
        name = GaugeType::GetDefault(_T("symbol"), gaugeType ).GetString();
    }

    double defsideratio = GaugeType::GetDefault(_T("sideratio"), gaugeType).GetDouble();

    wxSizeEx defsize(200,200);
    defsize << GaugeType::GetDefault(_T("size"), gaugeType );

    wxPoint   position    = prop.getPos();
    wxSize    size        = prop.getSize() == wxDefaultSize ? wxSize( defsize.x, (int)(defsize.y*defsideratio) ) : prop.getSize();
    int       id          = wxNewId();
    wxString  theme       = prop.themeName == wxEmptyString ? ThemesHelper::m_defaultThemeName : prop.themeName;
    wxString  desc        = prop.descName == wxEmptyString ? GaugeType::GetDefault(_T("descriptor"), gaugeType ).GetString() : prop.descName;

    wxString targetGType = gaugeType;

    if ( targetGType.IsSameAs( _T("table3d") ) ) { targetGType= _T("table"); }


    Logger::logf(LOG_MSG,_T("m_displayRefRatio=%0.2f"),m_displayRefRatio);

    wxPoint vpos(0,0);

    GaugeCarrierBase* tc = GetGroupCarrier(groupName);

    if( tc )
        vpos = tc->GetVirtualPos();

    Logger::logf(LOG_MSG,_T("addgauge ori y=%i"),position.y+vpos.y);

    int x = (int)((position.x + vpos.x) / m_displayRefRatio);
    int y = (int)((position.y + vpos.y) / m_displayRefRatio);
    int w = (int)(size.x / m_displayRefRatio);
    int h = (int)(size.y / m_displayRefRatio);



    wxString qs  = _T("insert into gauges (name,type,x,y,width,height,descriptor,theme,group_id) ");
             qs += wxString::Format( _T(" values ('%s','%s',%i,%i,%i,%i,'%s','%s', (select id from groups where name = '%s') );"),
                                     name.c_str(), targetGType.c_str(), x, y, w, h, desc.c_str(), theme.c_str(), groupName.c_str() );


    m_db.ExecuteUpdate( qs );

    long itemid = m_db.GetLastRowId().ToLong();

    Logger::logf(LOG_MSG,_T("addgauge %i modded y=%i"),itemid,y);

    //Logger::logf(LOG_MSG,_T("GaugeGroupManagerSQL::DoAddGauge itemid=%i"),itemid);


    //create a new gauge
    GaugeCarrierBase* carrier = addGauge( id, groupName, gaugeType , name, position, size, theme , desc , itemid );

    if(carrier->IsOnlyOneGauge()) m_carrierToItemId[ carrier ] = itemid;

    carrier->ShowCarrier(true);
}

void GaugeGroupManagerSQL::DoAddGauge( const wxString &groupName, wxString gaugeType, const wxPoint& pos)
{
    GaugeProperties prop;
    prop.groupName = groupName;
    prop.gaugeType = GaugeType::getType(gaugeType);
    prop.setPos( pos );

    DoAddGauge( prop );
}

//save visible gauges properties
bool GaugeGroupManagerSQL::saveGroup(const wxString& groupName)
{
    if ( m_carrierGroups.find( groupName ) == m_carrierGroups.end() ) { return false; }

    chkdb(false);

    wxString qs;

    GaugeCarrierBaseArray* fa = m_carrierGroups[ groupName ];

    if( !fa ) { return false; }

    Logger::logf(LOG_MSG,_T("SAVE groupName=")+groupName);

    m_db.Begin();

        uint32_t i;
        for( i = 0; i < fa->GetCount(); ++i )
        {
            GaugeCarrierBase* carrier  = fa->Item( i );

            if( carrier->IsOnlyOneGauge() || carrier->IsFloating() ) {

                int gauge_id  = m_carrierToItemId[ carrier ];
                GaugeCarrierFrame* frame = (GaugeCarrierFrame*)carrier;

                int w = frame->GetSize().x;
                int h = frame->GetSize().y;

                if( frame->getType() == GaugeType::ROUND  ) { w += 4; }

                if( frame->getType() == GaugeType::ROUND ) { h = w; }

                int x = (int)(frame->GetPosition().x / m_displayRefRatio);
                int y = (int)(frame->GetPosition().y / m_displayRefRatio);

                w = (int)(w / m_displayRefRatio);
                h = (int)(h / m_displayRefRatio);

                qs =  wxString::Format( _T("update gauges set x=%i, y=%i, width=%i, height=%i where id = %i;"),x,y,w,h,gauge_id );

                m_db.ExecuteUpdate( qs );

                //Logger::logf(LOG_MSG,_T("qs=")+qs);

            } else {


                wxWindowHashInt &hash = carrier->GetIDControlHash();

                for( wxWindowHashInt::iterator it = hash.begin(); it != hash.end(); it++ ){
                    wxWindow * ctrl = it->second;

                    if(!ctrl) continue;
                    GroupMember * gm = NULL;
                    TableEditorPanel * tep = NULL;
                    int itemid = -1;

                    wxSize size;


                    if( gm = dynamic_cast<GroupMember*> (ctrl) ) {
                        itemid = gm->GetItemID();
                        size = ctrl->GetSize();

                    } else if( tep = dynamic_cast<TableEditorPanel*> (ctrl) ) {

                    GroupMember* gm = tep->GetGroupMember();
//                    Logger::logf(LOG_MSG,_T("gm=%p"),gm);
                        if(!gm) continue;
                        itemid = gm->GetItemID();

//                        ctrl = (wxWindow*)(tep->GetTableEditor());
                        size = tep->GetMainSizer()->GetSize();
                        //Logger::logf(LOG_MSG,_T("sizerGetSize() w=%i,h=%i"),size.x, size.y);

                    } else {
                        Logger::logf(LOG_MSG,_T("unknown class!"));
                        continue;
                    }

                    wxPoint pos = ctrl->GetPosition() + carrier->GetVirtualPos();

//                    Logger::logf(LOG_MSG,_T("save %i pos.y=%i"),itemid,pos.y);

                    pos.x = (int)(pos.x/m_displayRefRatio);
                    pos.y = (int)(pos.y/m_displayRefRatio);
                    size.x = (int)(size.x/m_displayRefRatio);
                    size.y = (int)(size.y/m_displayRefRatio);

                    qs =  wxString::Format( _T("update gauges set x=%i, y=%i, width=%i, height=%i where id = %i;"),
                                            pos.x, pos.y, size.x, size.y, itemid );

//                Logger::logf(LOG_MSG,_T("qs=")+qs);

                    m_db.ExecuteUpdate( qs );

                }
            }
        }

    m_db.Commit();

    return true;
}

bool GaugeGroupManagerSQL::GetGroupDataByField(wxVariantHashArrayHash& data, const wxString& field, const wxVariant& value)
{
    chkdb(false);

    wxString qs = wxString::Format(_T("select *,groups.name as groups_name, groups.id as groups_id from gauges \
                    inner join groups on gauges.group_id = groups.id where %s = '%s';"),field.c_str(), value.MakeString().c_str() );

    wxSQLite3ResultSet res;

    try{
        res = m_db.ExecuteQuery(qs);
    } catch (wxSQLite3Exception& se) {
        Logger::logf(LOG_MSG,_T("wrong query:")+se.GetMessage());
        return false;
    }


    while( res.NextRow() ) {

        wxString grpname = res.GetString(_T("groups_name"));

        wxVariantHashArray& array = data[grpname];

        wxVariantHash rowhash;
        for(int a=0; a < res.GetColumnCount(); a++ ) {
            wxString cname = res.GetColumnName( a );

            switch( res.GetColumnType( a ) ){
                case WXSQLITE_INTEGER:
                    rowhash[cname] = (long)res.GetInt(a);
                    break;

                case WXSQLITE_FLOAT:
                    rowhash[cname] = res.GetDouble(a);
                    break;

                case WXSQLITE_TEXT:
                    rowhash[cname] = res.GetString(a);
                    break;
            }
        }
        array.Add(rowhash);
    }

    return true;
}

void GaugeGroupManagerSQL::clearThemeMenu()
{
    IntArray menuItems;

    wxMenuItemList::iterator iter = m_themeMenu->GetMenuItems().begin();

    for ( ; iter != m_themeMenu->GetMenuItems().end(); ++iter ) {
        wxMenuItem* menuItem = *iter;
        menuItems.Add( menuItem->GetId() );
    }

    for ( size_t i = 0; i < menuItems.GetCount(); ++i ) {
        m_themeMenu->Destroy( menuItems[i] );
    }
}

void GaugeGroupManagerSQL::clearGroupMenu()
{
    IntArray menuItems;

    wxMenuItemList::iterator iter = m_menu->GetMenuItems().begin();

    for ( ; iter != m_menu->GetMenuItems().end(); ++iter ) {
        wxMenuItem* menuItem = *iter;
        if ( !menuItem->GetLabel().IsSameAs( _T("Group Theme") ) ) {
            menuItems.Add( menuItem->GetId() );
        }
    }

    for ( size_t i = 0; i < menuItems.GetCount(); ++i ) {
        int menuItemId = menuItems[i];
        m_menu->Destroy( menuItemId );
        Disconnect( menuItemId,  wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( GaugeGroupManager::OnGroup ) );
    }

    int themeMenuId = m_menu->FindItem( _T("Group Theme") );

    if ( themeMenuId != wxNOT_FOUND ) {
        m_menu->Remove( themeMenuId );
    }
}

GaugeGroupDesc GaugeGroupManagerSQL::GetGroupInfo(int id)
{
    return GetGroupInfo(_T("id"),(long)id);
}

GaugeGroupDesc GaugeGroupManagerSQL::GetGroupInfo(const wxString& field, const wxVariant& value)
{
    //FUNCTIME;

    GaugeGroupDesc desc;

    chkdb(desc);

    wxString qs = wxString::Format(_T("select * from groups where %s = '%s';"),field.c_str(), value.MakeString().c_str() );

    wxSQLite3ResultSet res;

    res = m_db.ExecuteQuery(qs);

    wxSqliteHelper::SQLLineToVariantHash(res,desc.Properties);

    return desc;
}

