#ifndef GAUGEGROUPMANAGERSQL_H
#define GAUGEGROUPMANAGERSQL_H

#include <wx/wxsqlite3.h>

#include "GaugeGroupManager.h"

class GaugeCarrierFrame;



WX_DECLARE_HASH_MAP( GaugeCarrierFrame*, int, wxPointerHash, wxPointerEqual, FramePtrInt );


class GaugeGroupManagerSQL: public GaugeGroupManager
{
    public:
        GaugeGroupManagerSQL(wxWindow* window, wxMenu* parent,MTConfigData* mtConfigData, LogHistoryBase * loghist);
        virtual ~GaugeGroupManagerSQL();

        virtual bool load(const wxString& gaugesfile, const wxString& groupsfile);
        virtual bool saveGroup(const wxString& groupName);

        virtual void showOnStartup(ShowOnStartupFlags flags = SOSF_OFFLINE_LOG);
        virtual short showGroup(const wxString& groupName, bool wait = true );

        virtual void setStickGroup(const wxString &groupname, bool isStick);
        virtual bool isStickedGroup(const wxString &groupname);

        virtual void SaveTheme(int gaugeID, const wxString &name);
        virtual void SaveDesc(int gaugeID, const wxString &name);
        virtual void SaveSymbolname(int gaugeID, const wxString &name);
        virtual void SaveMultiSymbolname(int gaugeID, const wxString &name);


        virtual bool IsGroupFloating(const wxString &grpname);
        virtual wxString GetHotKeyStringByGroupname(const wxString &grpname);

        virtual void SetGroupFirstFocusedItem(const wxString &grpname, int itemid, int winid);

        virtual wxString& GetGaugesDBFileName() { return m_dbGauges; };
        virtual wxString& GetGroupsDBFileName() { return m_dbGroups; };

        virtual void DoAddGauge( GaugeProperties prop );
        virtual void DoAddGauge( const wxString &groupName, wxString gaugeType, const wxPoint& pos = wxPoint(0,0));

        virtual bool SaveSymbol(int itemid, const wxString& name);

        virtual void StartTransaction();
        virtual void EndTransaction();

        virtual bool GetGroupDataByField(wxVariantHashArrayHash& data, const wxString& field, const wxVariant& value);

        virtual bool SaveGauge(GaugeProperties& prop);

        virtual void reloadHotkeys();

        virtual GaugeGroupDesc GetGroupInfo(int id);
        virtual GaugeGroupDesc GetGroupInfo(const wxString& field, const wxVariant& value);


    protected:

        virtual void DoRemoveGauge(GaugeCarrierFrame* gaugeFrame);
        virtual void DoRemoveGauge(GroupMemberObject * gmo);
        virtual void DoRemoveGauge(GaugeProperties& props);
        virtual void SetTheme(int id);

        void CheckGroupDB();

    private:


        wxString    m_dbGauges;
        wxString    m_dbGroups;

        FramePtrInt  m_frameToItemId;



        wxDateTime    m_groupShootTime;



        wxSQLite3Database m_db;

        void clearThemeMenu();
        void clearGroupMenu();

};

#endif // GAUGEGROUPMANAGERSQL_H

