#include "MultiGaugeGroupManager.h"

#include "../VemsTuneController.h"

#include "../DisplayHelper.h"
#include "../ThemesHelper.h"
#include "../DirHelper.h"
#include "../vemstoolicon/ExitIcon.h"
#include "../vemstoolicon/ScreenIcon.h"
#include "../vemstoolicon/SelectInputIcon.h"
#include "../vemstoolicon/HideIcon.h"
#include "../vemstoolicon/FileIcon.h"
#include "../vemstoolicon/GPSIcon.h"
#include "../vemstoolicon/StateIcon.h"
#include "../vemstoolicon/DevicesIcon.h"
#include "VemsFlagWidget.h"

#include "../gaugedesc/GaugeDescHelper.h"
#include "GaugeType.h"


/**********************************************************
*   Item methods
**********************************************************/

/** @brief Item
  *
  * @todo: document this function
  */
Item::Item(const wxString& type, const wxString& name,int x, int y, int size )
{
    m_x     = x;
    m_y     = y;
    m_type  = type;
    m_name  = name;
    m_size  = size;
    m_xmlNode   = NULL;
    m_frame     = NULL;
}

/** @brief ~Item
  *
  * @todo: document this function
  */
Item::~Item()
{
    //dtor

}

void Item::refresh()
{
    if( !m_frame )
    {
        return;
    }

    m_size  = m_frame->GetSize().GetWidth();

    if( m_type == _T("VemsGauge") )
    {
        Logger::logf( LOG_MSG, _T("Item::refresh() - type : ") + m_type );
        m_size += 8; //TODO
    }

    m_x     = m_frame->GetPosition().x;
    m_y     = m_frame->GetPosition().y;
    m_name  = m_frame->GetName();
    Logger::logf( LOG_MSG, _T("Item::refresh() - name : %s, x : %d, y : %d, width : %d, height : %d"), m_name.c_str(), m_x, m_y, m_size, m_frame->GetSize().GetHeight() );
    //Logger::logf( LOG_MSG, _T("Item::refresh() - name : %s, x : %d, y : %d, virtual width : %d, virtual height : %d"), m_name.c_str(), m_x, m_y, m_frame->GetVirtualSize().GetWidth(), m_frame->GetVirtualSize().GetHeight() );

}


void Item::save()
{
    if( !m_xmlNode || !m_frame )
    {
        return;
    }

    refresh();

    wxXmlProperty* properties = m_xmlNode->GetProperties();

    bool typeIsSet = false;

    while( properties )
    {
        wxString name = properties->GetName();

        //Logger::logf( LOG_MSG, _T("Item::save() - property name = >%s<"), name.c_str() );

  /*      if( name = _T("name") )
        {
            //properties->SetValue( m_size );
        }*/
        if( name == _T("x") )
        {
            properties->SetValue( wxString::Format( _T("%i"), m_x ) );
        }
        else if( name == _T("y") )
        {
            properties->SetValue( wxString::Format( _T("%i"), m_y ) );
        }
        else if( name == _T("size") )
        {
            properties->SetValue( wxString::Format( _T("%i"), m_size ) );
        }
        else if( name == _T("name") )
        {
            properties->SetValue( m_name );
        }
        else if( name == _T("type") )
        {
            properties->SetValue( m_type );
            typeIsSet = true;
        }

        //Logger::logf( LOG_MSG, _T("Item::save() - property : >%s< = %s"), name.c_str(), properties->GetValue().c_str() );

        properties = properties->GetNext();
    }

    if( !typeIsSet )
    {
        m_xmlNode->AddProperty( _T("type"), m_type );
    }
}

/**********************************************************
*   ItemGroup methods
**********************************************************/

/** @brief ItemGroup
  *
  * @todo: document this function
  */
ItemGroup::ItemGroup(const wxString& name, const wxString& hotKey, uint8_t orient)
{
    m_name = name;
    m_hotKey = hotKey;
    m_orientation = orient;
}

/** @brief ~ItemGroup
  *
  * @todo: document this function
  */
ItemGroup::~ItemGroup()
{
    //dtor
}

/** @brief add
  *
  * @todo: document this function
  */
Item* ItemGroup::add(const wxString& type, const wxString& name, int x, int y, int size )
{
    Item* i = new Item( type, name, x, y, size );
    m_items.Add(i);
    return i;
}

/** @brief remove
  *
  * @todo: document this function
  */
void ItemGroup::remove(size_t idx)
{
    if(idx<size() && !m_items.IsEmpty()){
        Item* i = m_items[idx];
        delete i;
        m_items.Remove(i);
    }
}


/** @brief remove
  *
  * @todo: document this function
  */
bool ItemGroup::remove( Item* item_in )
{
    int i = m_items.Index( item_in );

    if( i == wxNOT_FOUND ) { return false; }

    m_items.RemoveAt( i );

    return true;
}


/** @brief size
  *
  * @todo: document this function
  */
size_t ItemGroup::size()
{
    return m_items.GetCount();
}

/** @brief get
  *
  * @todo: document this function
  */
Item* ItemGroup::get(size_t idx)
{
    if( m_items.IsEmpty() || idx >= size()){ return NULL; }

    return m_items[idx];
}

/** @brief clear
  *
  * @todo: document this function
  */
void ItemGroup::clear()
{
    for(size_t i=0; i<size(); i++){
        delete m_items[i];
    }

    m_items.Clear();
}

/**********************************************************
*   ItemGroupManager methods
**********************************************************/

/** @brief ItemGroupManager
  *
  * @todo: document this function
  */
ItemGroupManager::ItemGroupManager()
{
    //ctor
}

/** @brief ~ItemGroupManager
  *
  * @todo: document this function
  */
ItemGroupManager::~ItemGroupManager()
{
    //dtor
}

/** @brief addGroup
  *
  * @todo: document this function
  */
ItemGroup* ItemGroupManager::addGroup(const wxString& name, const wxString& hotKey, uint8_t ori)
{
    if(find(name) != wxNOT_FOUND || name == wxEmptyString ){ return NULL; }

    ItemGroup* g = new ItemGroup(name, hotKey, ori);

    m_groups.Add(g);

    return g;
}

/** @brief removeGroup
  *
  * @todo: document this function
  */
void ItemGroupManager::removeGroup(const wxString& name)
{
    int idx = find(name);

    if(idx != wxNOT_FOUND){
        ItemGroup* g = m_groups[idx];
        delete g;
        m_groups.Remove(g);
    }
}

/** @brief addItem
  *
  * @todo: document this function
  */
Item* ItemGroupManager::addItem(const wxString& group, const wxString& type, const wxString& item, int x, int y, int size )
{
    int idx = find(group);

    if(idx == wxNOT_FOUND){ return NULL; }

    Item* i = m_groups[idx]->add( type, item, x, y, size );

    return i;
}


/**
  * @brief Add an item to group ( add item to XMLDocument too )
  */
Item* ItemGroupManager::addNewItem( const wxString& group, const wxString& type, const wxString& name, int x, int y, int size )
{
    wxXmlNode* gaugeGroupNode = m_groupsXML.GetRoot()->GetChildren();

    while( gaugeGroupNode )
    {
        if( gaugeGroupNode->GetName() != _T("GAUGE_GROUP") )
        {
            gaugeGroupNode = gaugeGroupNode->GetNext();
            continue;
        }

        wxString groupName = gaugeGroupNode->GetPropVal( _T("name"), wxEmptyString );

        if( groupName == group )
        {
            //create node from item
            wxXmlNode* itemNode = new wxXmlNode( wxXML_ELEMENT_NODE, _T("GAUGE") );
            itemNode->AddProperty( _T("name"), name );
            itemNode->AddProperty( _T("x"), wxString::Format( _T("%d"), x ) );
            itemNode->AddProperty( _T("y"), wxString::Format( _T("%d"), y ) );
            itemNode->AddProperty( _T("width"), wxString::Format( _T("%d"), size ) );

            //add item to group
            Item* anItem = addItem( group, type, name, x, y, size );

            if( !anItem ) { break; }

            //add item node to group node
            gaugeGroupNode->AddChild( itemNode );

            //register node to item
            anItem->setNode( itemNode );

            return anItem;
        }

        gaugeGroupNode = gaugeGroupNode->GetNext();
    }

    return NULL;
}


/** @brief removeItem
  *
  * @todo: document this function
  */
void ItemGroupManager::removeItem(const wxString& group, const wxString& item)
{

}


void ItemGroupManager::removeItem( Item* item_in )
{
    wxXmlNode* gaugeGroupNode = m_groupsXML.GetRoot()->GetChildren();

    bool isRemoved = false;

    while( gaugeGroupNode && !isRemoved )
    {
        isRemoved = gaugeGroupNode->RemoveChild( item_in->getNode() );

        if( isRemoved )
        {
            delete item_in->getNode();

            for( uint32_t i = 0; i < m_groups.GetCount(); ++i )
            {
                ItemGroup* group = m_groups.Item( i );
                if( group->remove( item_in ) )
                {
                    Logger::logf( LOG_MSG, _T("ItemGroupManager::removeItem() - ") + item_in->getName() );
                    break;
                }
            }

            break;
        }

        gaugeGroupNode = gaugeGroupNode->GetNext();
    }

    Logger::logf( LOG_MSG, _T("ItemGroupManager::removeItem() - removed %p, %d"), item_in, isRemoved );
}


/** @brief load
  *
  * @todo: document this function
  */
bool ItemGroupManager::load(const wxString& filename)
{
    wxXmlDocument& doc = m_groupsXML;
    wxString firstgroup = wxEmptyString;

    if(!doc.Load(filename, _T("UTF-8"), wxXMLDOC_KEEP_WHITESPACE_NODES)){
        return false;
    }

    // start processing the XML file
    if(doc.GetRoot()->GetName() != _T("GAUGE_GROUPS")){
        return false;
    }

    wxXmlNode* gaugeGroupNodeRoot = doc.GetRoot();

    m_defaultGroup = gaugeGroupNodeRoot->GetPropVal(_T("default"),wxEmptyString);

    wxXmlNode* gaugeGroupNode = doc.GetRoot()->GetChildren();

    while(gaugeGroupNode){
        if(gaugeGroupNode->GetName() != _T("GAUGE_GROUP")){
            gaugeGroupNode = gaugeGroupNode->GetNext();
            continue;
        }

        wxString groupName = gaugeGroupNode->GetPropVal(_T("name"),wxEmptyString);
        wxString hotKey = gaugeGroupNode->GetPropVal(_T("hotKey"),wxEmptyString);

        uint8_t orint = 0;

        wxString ori = gaugeGroupNode->GetPropVal(_T("orientation"),wxEmptyString);

#if defined (__WINCE_STANDARDSDK__)

        if(ori  == _T("multi"))
            orint = DMDO_DEFAULT;
        else if(ori == _T("portrait-top"))
            orint = DMDO_0;
        else if(ori == _T("landscape-left"))
            orint = DMDO_270;
        else if(ori == _T("portrait-bottom"))
            orint = DMDO_180;
        else if(ori == _T("landscape-right"))
            orint = DMDO_90;
        else
            orint = DMDO_DEFAULT;

#endif

        addGroup(groupName, hotKey, orint);

        wxXmlNode* gaugeNode = gaugeGroupNode->GetChildren();

        while(gaugeNode){
            if(gaugeNode->GetName() != _T("GAUGE")){
                gaugeNode = gaugeNode->GetNext();
                continue;
            }

            long x = -1;
            long y = -1;
            long size = 0;

            wxString type = gaugeNode->GetPropVal(_T("type"),_T("VemsGauge"));
            wxString name = gaugeNode->GetPropVal(_T("name"),wxEmptyString);
            gaugeNode->GetPropVal(_T("x"),wxEmptyString).ToLong(&x);
            gaugeNode->GetPropVal(_T("y"),wxEmptyString).ToLong(&y);
            gaugeNode->GetPropVal(_T("width"),wxEmptyString).ToLong(&size);

            if(name != wxEmptyString && x > -1 && y > -1 && size > 0){
                Item* item = addItem(groupName,type,name,x,y,size );
                item->setNode( gaugeNode );
                //Logger::logf(LOG_MSG,_T("ItemGroupManager::load() ITEM= ") + groupName + _T(", ") + type + _T(", ") + name+_T(" p=%p"),item);
            }

            gaugeNode = gaugeNode->GetNext();
        }//gaugeNode


        gaugeGroupNode = gaugeGroupNode->GetNext();
    }//gaugeGroupNode*/
    return true;
}//load



/** @brief save
  *
  * @todo: document this function
  */
bool ItemGroupManager::save(const wxString& filename)
{
    wxXmlDocument& doc = m_groupsXML;

    //wxXmlNode* root = m_groupsXML.GetRoot();
    //m_groupsXML.DetachRoot();
    //m_groupsXML.SetRoot( root );

    for( size_t i = 0; i < m_groups.GetCount(); ++i )
    {
        for( size_t j = 0; j < m_groups[i]->size(); ++j )
        {
            Item* it = m_groups[i]->get(j);
            if( !it ) { continue; }
            it->save();
        }
    }

    //wxString groupFile = ConfigHelper::getDeviceFile(_T("groupFile"), _T("gauges.xml") );
    wxString groupFile = _T("gauges.xml");

    if( !doc.Save( DirHelper::GetPath( dCONFIG ) + groupFile, wxXML_NO_INDENTATION ) )
    {
        wxLogError( _T("ItemGroupManager::save() - error") );
        return false;
    }

    return true;
}

/** @brief find
  *
  * @todo: document this function
  */
int ItemGroupManager::find(const wxString& name)
{
    for(size_t i=0; i<m_groups.GetCount(); i++){
        //Logger::logf(LOG_MSG, _T("ItemGroupManager::find() key1:")+m_groups[i]->getName()+_T("||"));
        //Logger::logf(LOG_MSG, _T("ItemGroupManager::find() name:")+name+_T("||"));
        if(m_groups[i]->getName().IsSameAs(name) ){
            return i;
        }
    }

    return wxNOT_FOUND;
}

/** @brief get
  *
  * @todo: document this function
  */
ItemGroup* ItemGroupManager::get(size_t idx)
{
    if(m_groups.IsEmpty() || idx >= size()){ return NULL; }

    return m_groups[idx];
}

/** @brief get
  *
  * @todo: document this function
  */
ItemGroup* ItemGroupManager::get(const wxString& name)
{
    int idx = find(name);
    if(idx == wxNOT_FOUND) return NULL;
    return m_groups[idx];
}

/** @brief size
  *
  * @todo: document this function
  */
size_t ItemGroupManager::size()
{
    return m_groups.GetCount();
}

/** @brief clear
  *
  * @todo: document this function
  */
void ItemGroupManager::clear()
{
    for(size_t i=0; i<size(); i++){
        m_groups[i]->clear();
    }

    m_groups.Clear();
}

/**********************************************************
*   GaugeGroupManager methods
**********************************************************/


/**********************************************************
*   MultiGaugeGroupManager methods
**********************************************************/

MultiGaugeGroupManager::MultiGaugeGroupManager(wxWindow* window, wxMenu* parent, VemsTuneController* ctrl, bool CreateNewFrame,  bool isCacheScreens)
    :lastframe (NULL)
    ,m_GBframe (NULL)
    ,m_autoLog (false)
{

//    Logger::log(LOG_MSG,_T("MultiGaugeGroupManager::MultiGaugeGroupManager START"));
    m_vemsTuneController = ctrl;

    m_window = window;
    m_rtDataMgr = m_vemsTuneController->getRTDataMgr();



#if defined (VEMSDEBUG)
    m_menu = new wxMenu(_T(""));
    parent->AppendSubMenu(m_menu, _T("Screen"));
#endif

    m_window->PushEventHandler(this);

    m_CreateNewFrame = CreateNewFrame;
    m_isCacheScreens = isCacheScreens;

    //if(window == NULL) m_CreateNewFrame = false;


    Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::MultiGaugeGroupManager END m_screenHash size:%i"), m_screenHash.size());

}

MultiGaugeGroupManager::~MultiGaugeGroupManager()
{
    //dtor
}

void MultiGaugeGroupManager::SetGaugeFrameBase(GaugeFrameBase *frame)
{

    m_GBframe = frame;
    m_CreateNewFrame = false;

}

bool MultiGaugeGroupManager::load(const wxString& filename)
{
    if(!m_groupManager.load(filename)){ return false; }
//    int idn =  wxNewId();
/*
#if defined (__WINCE_STANDARDSDK__)
    m_menu->AppendRadioItem(idn,_T("None"));
    Connect( idn,  wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MultiGaugeGroupManager::OnGroup) );
#endif
*/
    for(size_t i=0; i < m_groupManager.size(); i++){
        wxString groupName = m_groupManager.get(i)->getName();
        if(m_menu->FindItem(groupName) != wxNOT_FOUND ){ continue; }
        int id =  wxNewId();

/*
    #if defined (__WINCE_STANDARDSDK__)
        m_menu->AppendRadioItem(id,groupName);
    #else
        m_menu->AppendCheckItem(id,groupName);
    #endif
*/

        if(!m_CreateNewFrame)
            m_menu->AppendRadioItem(id,groupName);
        else
            m_menu->AppendCheckItem(id,groupName);


        Connect( id,  wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MultiGaugeGroupManager::OnGroup) );
    }
    Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::load() END m_screenHash size:%i"), m_screenHash.size());

    return true;
}


bool MultiGaugeGroupManager::save(const wxString& filename)
{


	return 0;
}

void MultiGaugeGroupManager::showDefaultGroup(Orientations ori)
{

    createGroup(m_groupManager.m_defaultGroup);
    m_menu->Check(m_menu->FindItem(m_groupManager.m_defaultGroup), true);

}

void MultiGaugeGroupManager::showGroup(const wxString& groupName)
{
    Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::showGroup START name = ") + groupName);

    if( m_screenHash.find(groupName) == m_screenHash.end() ) return;

#if defined (__WINCE_STANDARDSDK__)

    uint8_t Angle = m_groupManager.get(groupName)->getOrientation();
	Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::showGroup name=")+groupName+_T(" Angle=%i"),Angle);

    DisplayHelper::SetOrientation(Angle);

#endif

    m_lastGroup = groupName;

    wxControlHash *tmphash = m_screenHash[groupName];

	Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::showGroup m_screenHash size=%i"),m_screenHash.size());
	Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::showGroup tmphash size=%i"),tmphash->size());

    for( wxControlHash::iterator it = tmphash->begin(); it != tmphash->end() ;it++) {
		int idx = it->first;
		Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::showGroup idx=%i"),  idx);
        if(it->second->GetName() == _T("VemsWidgetBase")){
            Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::VemsWidgetBase 200"));
			VemsWidgetBase* viewerBase = ((VemsWidgetBase*)(it->second));
           if( !viewerBase->IsRegistered() ){
			    Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::VemsWidgetBase 300"));
				viewerBase->RegisterViewer();
				Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::VemsWidgetBase 400"));
           }
        }else if(it->second->GetName() == _T("VemsFlagWidget")){
            Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::VemsFlagWidget 200"));
			VemsFlagWidget* viewerBase = ((VemsFlagWidget*)(it->second));
           if( !viewerBase->IsRegistered() ){
			    Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::VemsFlagWidget 300"));
				viewerBase->RegisterViewer();
				Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::VemsFlagWidget 400"));
           }
        }

		Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::showGroup 500"));
        it->second->Show(true);
		Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::showGroup 600"));
    }

    Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::showGroup END"));

}

void MultiGaugeGroupManager::createAllGroup()
{
    GroupArray tmparr = m_groupManager.get();

    for(uint16_t a=0;a<tmparr.GetCount();a++)
        createGroup(tmparr.Item(a)->getName(), false);

}

void MultiGaugeGroupManager::createGroup(const wxString& groupName, bool showAfterLoad)
{
    Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::createGroup()- START 1234"));
    Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::createGroup() m_screenHash ptr:%p"), &m_screenHash );
    Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::createGroup() m_screenHash size:%i"), m_screenHash.size());

    if( m_screenHash.find(groupName) != m_screenHash.end() && m_isCacheScreens ) {
        Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::createGroup()- invoke showGroup"));
        showGroup(groupName);
        return;
    }

//    Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::createGroup() m_CreateNewFrame:%i"), m_CreateNewFrame);
    Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::createGroup:")+groupName);

    wxPoint maxpoint = wxPoint(0,0);
    wxSize fsize;
	size_t i;

	//if(groupName == _T("None")) return;

    ItemGroup* g = m_groupManager.get(groupName);

    //m_orientationHash[groupName] = g->getOrientation();

    //Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::showGroup: 1"));

    for(i=0; i<g->size(); i++) {
        Item* it = g->get(i);
        if( it->getX()+ it->getSize() >= maxpoint.x ) { maxpoint.x = it->getX()+it->getSize(); fsize.x = it->getX() + it->getSize(); }
        if( it->getY()+ it->getSize() >= maxpoint.y ) { maxpoint.y = it->getY()+it->getSize(); fsize.y = it->getY() + it->getSize(); }
    }

    //Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::showGroup: 2"));

    fsize.IncBy(fsize.x/20,fsize.y/20);

    int idFrame = wxNewId();

    GaugeFrameBase *frame;
    //Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::showGroup: 3"));
    if(m_CreateNewFrame) {
        frame = new GaugeFrameBase(m_window, idFrame/*, m_vemsTuneController*/, groupName,wxPoint(0,28),fsize,wxSTAY_ON_TOP | wxNO_BORDER );

        //frame->SetWindowReg(wreg);
    }

    else
        frame = m_GBframe;

    frame->SetDataMgrObj( m_vemsTuneController->getRTDataMgr() );
    frame->SetDataMgrObj( m_vemsTuneController->getSTDataMgr() );
    //Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::showGroup: 4"));

    wxControlHash *screen = new wxControlHash();

    frame->SetCustomControlHash(screen);


    lastframe = frame;

    frame->Connect( idFrame,  wxEVT_CLOSE_WINDOW , wxCloseEventHandler(MultiGaugeGroupManager::OnFrameClose),NULL, this);


    for(i=0; i<g->size(); i++) {

        Item* it = g->get(i);


        GaugeProperties prop;

        prop.gaugeType = GaugeType::getType(it->getType());
        prop.symbolName = it->getName();


        prop.x = it->getX();
        prop.y = it->getY();
        prop.width = it->getSize();
        prop.height = it->getSize();
        prop.extraParams[_T("showBorder")] = false;

        if( prop.height == -1 )
            prop.height = prop.width;

        Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::createGroup: 5 type = ") + it->getType() + _T(", name = ") + it->getName() + _T(", index = %i"),i);

        if(it->getType() == _T("ICON"))
            AddIcon(it->getName(),frame,wxPoint(it->getX(),it->getY()), m_menu);

        else if(it->getType() == _T("Flag"))
            frame->AddFlagWidget(it->getName(),wxPoint(it->getX(),it->getY()),it->getSize());

        else if(it->getType() == _T("VemsState"))
            frame->AddStateWidget(it->getName(),wxPoint(it->getX(),it->getY()),it->getSize());

        else if(it->getType() == _T("VemsTableEditor"))
            frame->AddVemsTable( GaugeDescHelper::GetGDMgr(), prop );
            //frame->AddVemsTable(it->getName(),wxPoint(it->getX(),it->getY()),it->getSize());
        else
            frame->AddGauge( GaugeDescHelper::GetGDMgr(), prop );
            //frame->AddGauge(-1,MTConfigHelper::getMTConfigData()->getVEMSGaugeDescArray(),ThemesHelper::GetWidgetThemes(), it->getType(),it->getName(),wxPoint(it->getX(),it->getY()),it->getSize(),it->getSize(),false);



        //Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::createGroup add new widget END"));

    }

    //Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::createGroup: 6"));

    m_frameHash[groupName] = frame;
    m_IDtoFrame[idFrame] = frame;


    //Logger::logf(LOG_MSG,_T("screen size %i"),screen->size());

    //Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::createGroup ptr1=%p, ptr2=%p"), (*screen) );

    m_screenHash[groupName] = screen;

//    m_screenHash[groupName] = frame->GetControlHash();

    if(showAfterLoad)
        showGroup(groupName);

    frame->Show(true);

    Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::createGroup END"));

}

int MultiGaugeGroupManager::AddIcon(const wxString name, GaugeFrameBase *frame,  wxPoint pos, wxMenu *menu)
{
    int id = wxNewId();

    //Logger::log(LOG_MSG,_T("GaugeFrameBase::AddIcon !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"));

    VemsToolIconBase *icon = NULL;

    wxString icondir = DirHelper::GetPath( dICON );


    if(name == _T("Exit")){
        icon = new ExitIcon(frame,id,icondir,pos);

    }else if(name == _T("Screen")){
        icon = new ScreenIcon(frame,id,icondir,pos);
        ((ScreenIcon*)icon)->SetMenu(menu);

    }else if(name == _T("Input")){
        icon = new SelectInputIcon(frame,id,icondir,pos,m_rtDataMgr);
        ((SelectInputIcon*)icon)->SetVemsTuneController(m_vemsTuneController); /// setECUonline, selectInput

    }else if(name == _T("Gps")){
        icon = new GPSIcon(frame,id,icondir,pos, m_vemsTuneController, m_rtDataMgr);
        //((GPSIcon*)icon)->SetVemsTuneController(m_vemsTuneController); /// only EventHandler used

    }else if(name == _T("Hide")){
        icon = new HideIcon(frame,id,icondir,pos);
        //((HideIcon*)icon)->SetWindowReg(m_wreg);

    }else if(name == _T("File")){
        icon = new FileIcon(frame,id,icondir,pos);
        //((FileIcon*)icon)->SetWindowReg(m_wreg);
        ((FileIcon*)icon)->SetVemsCtrl(m_vemsTuneController); /// getFwInfo()

    }else if(name == _T("State.ECU") || name == _T("State.GPS") || name == _T("State.AIM") ){
        icon = new StateIcon(frame,id,icondir,name,pos);
        ((StateIcon*)icon)->SetVemsTuneController(m_vemsTuneController); /// isECUOnline(); isGPSOnline;

    }else if(name == _T("Devices")){

//        icon = new DevicesIcon(frame,id,icondir,pos, m_vemsTuneController, m_vemsTuneController->getECUCtrl() );
    }


    frame->RegIcon(id,icon);


    return 0;

}

void MultiGaugeGroupManager::hideGroup(const wxString& groupName)
{

    Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::hideGroup, groupName = ")+groupName);

    if( m_screenHash.find(groupName) == m_screenHash.end() ) return;

    //Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::hideGroup, groupName MATCH!!! "));

    wxControlHash *tmphash = m_screenHash[groupName];

    for( wxControlHash::iterator it = tmphash->begin(); it != tmphash->end() ;it++){
        if(it->second->GetName() == _T("VemsWidgetBase")){
            Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::hideGroup 222"));
            if( ((VemsWidgetBase*)it->second)->IsRegistered() ) {
                Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::hideGroup 333"));
                ((VemsWidgetBase*)it->second)->UnRegisterViewer();
                Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::hideGroup 444"));

            }
        }else if(it->second->GetName() == _T("VemsFlagWidget")){
            Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::hideGroup VemsFlagWidget 222"));
            if( ((VemsFlagWidget*)it->second)->IsRegistered() ) {
                Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::hideGroup VemsFlagWidget 333"));
                ((VemsFlagWidget*)it->second)->UnRegisterViewer();
                Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::hideGroup VemsFlagWidget 444"));

            }
        }

        it->second->Hide();
    }

    //Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::hideGroup END "));

}

void MultiGaugeGroupManager::deleteGroup(const wxString& groupName)
{
    if(!m_frameHash.count(groupName)) return;

    Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::deleteGroup"));

    if(m_CreateNewFrame) {
        GaugeFrameBase* frame = m_frameHash[groupName];
        m_IDtoFrame.erase(frame->GetId());
        m_frameHash.erase(groupName);
        frame->unregisterViewer();
        frame->Destroy();
    }
    else {
        m_GBframe->ClearFrame();
    }

}

void MultiGaugeGroupManager::OnFrameClose(wxCloseEvent& event)
{

    Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::OnFrameClose()-START"));


    int frameid = event.GetId();
    if(m_IDtoFrame.find(frameid) == m_IDtoFrame.end()){
        Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::OnFrameClose() frameid not found:%i"), frameid);
        return;

    }
    Logger::log(LOG_MSG,_T("MultiGaugeGroupManager::OnFrameClose() 123"));
    GaugeFrameBase *frame = m_IDtoFrame[frameid];
    wxString groupName;

    for(FrameBaseHash::iterator it = m_frameHash.begin(); it != m_frameHash.end(); it++){
        Logger::log(LOG_MSG,_T("MultiGaugeGroupManager::OnFrameClose() 125-for"));
        if (it->second == frame){
            groupName = it->first;
        }
    }
    Logger::log(LOG_MSG,_T("MultiGaugeGroupManager::OnFrameClose() 456"));
    m_frameHash.erase(groupName);
    m_IDtoFrame.erase(frameid);
    frame->unregisterViewer();
    frame->Destroy();

    m_menu->Check(m_menu->FindItem(groupName),false);

}

ItemGroup* MultiGaugeGroupManager::addGroup(const wxString& groupName)
{

	return 0;
}

//GaugeFrame* MultiGaugeGroupManager::findFrame(int id)
void MultiGaugeGroupManager::remove(int id)
{


}

void MultiGaugeGroupManager::AddOrCreateLoggerScreen(bool startlogging)
{

    wxString groupName = _T("LOGGER");

    if(m_groupManager.find(groupName) == wxNOT_FOUND){
        m_groupManager.addGroup(groupName,wxEmptyString, 0);

        m_groupManager.addItem(groupName,_T("ICON"),_T("Screen"),200,280,40);
        m_groupManager.addItem(groupName,_T("ICON"),_T("Exit"),212,10,40);
        m_groupManager.addItem(groupName,_T("LOGGER"),_T("High"),5,50,230);
        m_groupManager.addItem(groupName,_T("VemsState"),_T("TextOnly"),5,220,150);
    }

    m_autoLog = startlogging;
    createGroup(groupName);
    m_autoLog = false;

}

void MultiGaugeGroupManager::OnGroup(wxCommandEvent& event)
{
    //wxLogMessage(wxString::Format(_T("OnGroup ID = %d"),event.GetId()));

    Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::OnGroup"));

    int id = event.GetId();
    wxMenuItem* mi = m_menu->FindItem(id);

    Logger::logf(LOG_MSG,_T("MultiGaugeGroupManager::OnGroup m_CreateNewFrame = %i"), m_CreateNewFrame);

    //wxString groupName = mi->GetText();
    wxString groupWithHotKey = mi->GetText();
    wxString groupName = groupWithHotKey.BeforeFirst(_T('\t'));

    //wxString groupName = groupNameWithHotKey.BeforeFirst(_T("\t"));

    if(m_CreateNewFrame){

        if(mi->IsChecked())
            createGroup(groupName);
        else
            deleteGroup(groupName);

    } else {
        //m_GBframe->ClearFrame();
        hideGroup(m_lastGroup);
        //m_GBframe->ClearFrameShape();

        createGroup(groupName);

    }

}

