#ifndef MULTIGAUGEGROUPMANAGER_H
#define MULTIGAUGEGROUPMANAGER_H

#include <wx/xml/xml.h>

#include "GaugeFrameBase.h"

#include "../common/types/wxControlHash.h"

class VemsTuneController;
class RTDataMgrBase;


/**********************************************************
*   Item
**********************************************************/

class Item
{
    public:
        Item(const wxString& type, const wxString& name,int x, int y, int size );
        virtual ~Item();

        //GaugeFrame* getFrame() { return m_frame; }
        wxXmlNode*  getNode() { return m_xmlNode; }
        void refresh();
        void save();
        wxString    getType(){ return m_type; }
        wxString    getName(){ return m_name; }
        int getX(){ return m_x; }
        int getY(){ return m_y; }
        int getSize(){ return m_size; }


        void setType(const wxString& type){ m_type = type; }
        void setName(const wxString& name){ m_name = name; }
        void setX(int x){ m_x = x; }
        void setY(int y){ m_y = y; }
        void setSize(int size){ m_size = size; }
        void setFrame( wxFrame* frame ) { m_frame = frame; }
        void setNode( wxXmlNode* node ) { m_xmlNode = node; }
    private:
        wxFrame*    m_frame;
        wxXmlNode*  m_xmlNode;
        wxString    m_type;
        wxString    m_name;
        int         m_x;
        int         m_y;
        int         m_size;

};

/**********************************************************
*   ItemGroup
**********************************************************/

WX_DEFINE_ARRAY(Item*,  ItemArray);

class ItemGroup
{
    public:
        ItemGroup( const wxString& name,const wxString& hotKey, uint8_t orient );
        virtual ~ItemGroup();

        Item*   add(const wxString& type, const wxString& name, int x, int y, int size );
        void    remove(size_t idx);
        bool    remove( Item* item_in ); //return true if it found
        void    clear();

        wxString    getName(){ return m_name; }
        wxString    getHotKey(){ return m_hotKey; }
        uint8_t     getOrientation() { return m_orientation; }
        Item*       get(size_t idx);
        size_t      size();
    private:
        wxString    m_name;
        wxString    m_hotKey;
        uint8_t     m_orientation;
        ItemArray   m_items;
};

/**********************************************************
*   ItemGroupManager
**********************************************************/

WX_DEFINE_ARRAY(ItemGroup*,  GroupArray);

class ItemGroupManager
{
    public:
        ItemGroupManager();
        virtual ~ItemGroupManager();

        ItemGroup* addGroup(const wxString& name, const wxString& hotKey, uint8_t ori);
        void removeGroup(const wxString& name);
        Item* addItem(const wxString& group, const wxString& type, const wxString& item, int x, int y, int size );
        Item* addNewItem(const wxString& group, const wxString& type, const wxString& item, int x, int y, int size );
        void removeItem( const wxString& group, const wxString& item );
        void removeItem( Item* item_in );
        void clear();
        const GroupArray& get() {return m_groups; };
        ItemGroup* get(size_t idx);
        ItemGroup* get(const wxString& name);
        size_t    size();
        bool load(const wxString& filename);
        bool save(const wxString& filename);

        int find(const wxString& name);

        wxString m_defaultGroup;

    private:
        GroupArray      m_groups;
        wxXmlDocument   m_groupsXML;


};

/**********************************************************
*   MultiGaugeGroupManager
**********************************************************/

WX_DECLARE_STRING_HASH_MAP(GaugeFrameBase*, FrameBaseHash);
WX_DECLARE_HASH_MAP(int, GaugeFrameBase*, wxIntegerHash, wxIntegerEqual, FrameBaseIDHash);
WX_DECLARE_STRING_HASH_MAP(wxControlHash*, ScreenHash);



class MultiGaugeGroupManager : public wxEvtHandler
{
    public:
        MultiGaugeGroupManager(wxWindow* window, wxMenu* parent, VemsTuneController* ctrl, bool CreateNewFrame = true, bool isCacheScreens = false);
        virtual ~MultiGaugeGroupManager();

        enum Orientations
        {
            PORTRAIT,
            LANDSCAPE

        };

        bool load(const wxString& filename);
        bool save(const wxString& filename);

        void createAllGroup();

        void showGroup(const wxString& groupName);
        void createGroup(const wxString& groupName, bool showAfterLoad = true);

        void hideGroup(const wxString& groupName);

        /// WINCE
        void showDefaultGroup(Orientations ori = PORTRAIT);
        void SetGaugeFrameBase(GaugeFrameBase *frame);
        void deleteGroup(const wxString& groupName);

    private:

        GaugeFrameBase *lastframe;
        GaugeFrameBase *m_GBframe;

        FrameBaseIDHash     m_IDtoFrame;
        ItemGroupManager    m_groupManager;
        RTDataMgrBase*      m_rtDataMgr;
        wxMenu*             m_menu;
        wxWindow*           m_window;

        FrameBaseHash       m_frameHash;




        ScreenHash      m_screenHash;

        wxString        m_lastGroup;

        bool m_CreateNewFrame;
        bool m_isCacheScreens;
        bool m_autoLog;

        String2IntHash  m_orientationHash;

        VemsTuneController* m_vemsTuneController;

        //wxFrame *m_extframe;

        void OnGroup(wxCommandEvent& event);
        void OnFrameClose(wxCloseEvent& event);

        void AddOrCreateLoggerScreen(bool startlogging);
        int AddIcon(const wxString name, GaugeFrameBase *frame,  wxPoint pos, wxMenu *menu);


        ItemGroup* addGroup(const wxString& groupName);
        //GaugeFrame* findFrame(int id);
        void remove(int id);


};


class ScreenDesc
{
    public:
        ScreenDesc();
        virtual ~ScreenDesc();


    private:

        wxControlHash   m_screen;
        wxRegion        m_shapeRegion;


};


#endif // MULTIGAUGEGROUPMANAGER_H
