#include "MVG_DescManager.h"


#include "../../mtconfigdata.h"
#include "../../Symbol.h"
#include "../../vemsprotocol/SIPRTypes.h"

#include "../../customwidgets/wxTextCtrlNullEx.h"
#include "../../customwidgets/wxComboBoxNullEx.h"
#include "../../DrawHelper.h"
#include "MVG_SymbolChooserPanel.h"
#include "MultiValueGraph.h"

#include "../../logger.h"






MVG_DescManager::MVG_DescManager(GaugeDescManager * gdmgr, int profileID)
{
    //ctor
    m_profileID = profileID;
    m_gdMgr = gdmgr;
    m_mgMgr = gdmgr->GetMultiGraphDescMgr();

    ParseSymbolList();

    Settings[SETTINGS_TOGGLE] = true;

    m_mvg = NULL;
}

MVG_DescManager::~MVG_DescManager()
{
    //dtor
}

void MVG_DescManager::RefreshTargetGraph()
{
    if( !m_mvg ) return;

    m_mvg->RefreshRTSymbols();

    m_mvg->RedrawWidget();
}

void MVG_DescManager::ParseSymbolList()
{
    if(!m_gdMgr) return;

    if(!(m_gdMgr->getMTConfigData())) return;


    SymbolGaugeDescArray symboldescarr;

    m_gdMgr->GetSymbolDescs( symboldescarr );

    for ( size_t i = 0; i < symboldescarr.GetCount(); i++ ){

        m_symbolList.Add(symboldescarr[i]->Name);

    }

}


bool MVG_DescManager::SwapSymbolArrayItems(wxVariantHashArray& array, size_t idx1, size_t idx2)
{
    if( idx1 == idx2 ) return false;
    if( idx1 >= array.GetCount() || idx2 >= array.GetCount() ) return false;

    size_t idx_lower = idx1 < idx2 ? idx1 : idx2;
    size_t idx_higher = idx1 > idx2 ? idx1 : idx2;

    wxVariantHash * item_higher = array.Detach(idx_higher);
    wxVariantHash * item_lower  = array.Detach(idx_lower);

    array.Insert( item_higher, idx_lower );
    array.Insert( item_lower, idx_higher );

    return true;
}

/** \~english
  *
  * \~hungarian
  * Az ordernum mező rendre 1 2 3 értéket vesz fel, és rendre ilyen sorrendben vannak a wxVariantHash symbol leíró objektumok a tömbben.
  */
bool MVG_DescManager::SlideSymbolTo(wxVariantHash * currentSymbolDesc, int target)
{
    bool isLeft;
    switch(target){
        case MVGD_SLIDE_LEFT: isLeft = true; break;
        case MVGD_SLIDE_RIGHT: isLeft = false; break;
        default: return false;
    }

    MultiGraphDesc * desc = GetDesc();

    if(!desc) return false;

    size_t cc = desc->Cells.GetCount();

    int node_id = (*currentSymbolDesc)[_T("node_id")].GetLong();
    int fromordernum = (*currentSymbolDesc)[_T("ordernum")].GetLong();

    wxVariantHashArray * symbolarray = NULL;

    int slidediff = isLeft ? 1 : -1;


    for( size_t a = 0; a < cc; a++ ) {

        size_t cac = desc->Cells[a].Symbols.GetCount();

        int borderidx = isLeft ? cac-1 : 0;
        int borderlim = isLeft ? MVG_FIRST_ORDER_NUMBER : cac;

        if(desc->Cells[a].Properties[_T("id")].GetLong() != node_id) continue;

        symbolarray = &(desc->Cells[a].Symbols);

        if(fromordernum == borderlim) {
            wxVariantHash * item = symbolarray->Detach( borderlim - 1 );
            symbolarray->Insert( item, borderidx );
            break;
        }

        for( size_t b = (isLeft?0:1); b < cac-(isLeft?1:0); b++ ) {
            wxVariantHash &hash = desc->Cells[a].Symbols[b];
            if(hash[_T("ordernum")].GetLong() == fromordernum - slidediff){
                SwapSymbolArrayItems( *symbolarray , b, b + slidediff);
                break;
            }
        }

        break;
    }


    if(symbolarray == NULL) return false;

    NormalizeSymbolOrder(*symbolarray);

    return true;
}

bool MVG_DescManager::RemoveSymbolDesc(wxVariantHash * currentSymbolDesc)
{
    int id = (*currentSymbolDesc)[_T("id")].GetLong();
    int node_id = (*currentSymbolDesc)[_T("node_id")].GetLong();

    MultiGraphDesc * desc = GetDesc();

    if(!desc) return false;

    size_t cc = desc->Cells.GetCount();

    for( size_t a = 0; a < cc; a++ ) {

        size_t cac = desc->Cells[a].Symbols.GetCount();

        if(desc->Cells[a].Properties[_T("id")].GetLong() != node_id) continue;

        for( size_t b = 0; b < cac; b++ ) {
            wxVariantHash &hash = desc->Cells[a].Symbols[b];
            if( hash[_T("id")].GetLong() == id ){
                desc->Cells[a].Symbols.RemoveAt(b);
                desc->AddRemovedSymbolID(id);
                NormalizeSymbolOrder(desc->Cells[a].Symbols);
                return true;
            }
        }

    }

    return false;
}

void MVG_DescManager::NormalizeSymbolOrder(wxVariantHashArray& symbolarray)
{
    for(size_t a = 0; a < symbolarray.GetCount(); a++){
        wxVariantHash &hash = symbolarray[a];
        hash[_T("ordernum")] = (long)(a+1);
    }
}


bool MVG_DescManager::SlideNodeTo(MultiGraphDescNode * nodeDesc, int target)
{
    bool isUp;
    switch(target){
        case MVGD_SLIDE_UP: isUp = true; break;
        case MVGD_SLIDE_DOWN: isUp = false; break;
        default: return false;
    }

    MultiGraphDesc * desc = GetDesc();

    if(!desc) return false;

    size_t cc = desc->Cells.GetCount();

    int slidediff = isUp ? 1 : -1;
    int borderidx = isUp ? cc-1 : 0;
    int borderlim = isUp ? MVG_FIRST_ORDER_NUMBER : cc;

    int fromOrderNum = nodeDesc->Properties[_T("ordernum")].GetLong();

    if(fromOrderNum == borderlim) {
        desc->Cells.Detach( borderlim-1 );
        desc->Cells.Insert( nodeDesc, borderidx );
        NormalizeNodeOrder(desc->Cells);
        return true;
    }

    for( size_t a = (isUp?0:1); a < cc - (isUp?1:0); a++ ) {

        MultiGraphDescNode& currentNode = desc->Cells[a];

        if(currentNode.Properties[_T("ordernum")].GetLong() == fromOrderNum - slidediff){
            SwapNodeArrayItems(desc->Cells, a , a + slidediff);
            NormalizeNodeOrder(desc->Cells);
            return true;
        }
    }

    return false;
}

bool MVG_DescManager::RemoveNode(MultiGraphDescNode * nodeDesc)
{
    int id = nodeDesc->Properties[_T("id")].GetLong();

    MultiGraphDesc * desc = GetDesc();

    if(!desc) return false;

    size_t cc = desc->Cells.GetCount();

    for( size_t a = 0; a < cc; a++ ) {
        if(desc->Cells[a].Properties[_T("id")].GetLong() == id) {
            desc->Cells.RemoveAt(a);
            desc->AddRemovedNodeID(id);
            NormalizeNodeOrder(desc->Cells);
            return true;
        }
    }

    return false;
}

bool MVG_DescManager::SwapNodeArrayItems(MultiGraphDescNodeArray& array, size_t idx1, size_t idx2)
{
    if( idx1 == idx2 ) return false;
    if( idx1 >= array.GetCount() || idx2 >= array.GetCount() ) return false;

    size_t idx_lower = idx1 < idx2 ? idx1 : idx2;
    size_t idx_higher = idx1 > idx2 ? idx1 : idx2;

    MultiGraphDescNode * item_higher = array.Detach(idx_higher);
    MultiGraphDescNode * item_lower  = array.Detach(idx_lower);

    array.Insert( item_higher, idx_lower );
    array.Insert( item_lower, idx_higher );

    return true;
}

void MVG_DescManager::NormalizeNodeOrder(MultiGraphDescNodeArray& nodeArray)
{
    for(size_t a = 0; a < nodeArray.GetCount(); a++){
        MultiGraphDescNode& node = nodeArray[a];
        node.Properties[_T("ordernum")] = (long)(a+1);
    }
}

MultiGraphDescNode* MVG_DescManager::InsertNode(MultiGraphDescNode * nodeDesc, int target)
{
    bool isBefore;
    switch(target){
        case MVGD_INSERT_BEFORE: isBefore = true; break;
        case MVGD_INSERT_AFTER: isBefore = false; break;
        default: return NULL;
    }

    MultiGraphDesc * desc = GetDesc();

    if(!desc) return NULL;

    int ordern = nodeDesc->Properties[_T("ordernum")].GetLong();

    int newidx = isBefore ? ordern - 1 : ordern;

    MultiGraphDescNode newNode;
    newNode.Properties[_T("id")] = desc->GetNewItemID();
    newNode.Properties[_T("profile_id")] = nodeDesc->Properties[_T("profile_id")];
    newNode.Properties[_T("flagdisplayratio")] = nodeDesc->Properties[_T("flagdisplayratio")];

    desc->Cells.Insert(newNode,newidx);
    NormalizeNodeOrder(desc->Cells);

    return &(desc->Cells.Item(newidx));
}

MultiGraphDescNode* MVG_DescManager::AppendNode()
{
    MultiGraphDesc * desc = GetDesc();

    if(!desc) return NULL;

    MultiGraphDescNode newNode;
    newNode.Properties[_T("id")] = desc->GetNewItemID();
    newNode.Properties[_T("profile_id")] = (long)desc->ProfileID;
    newNode.Properties[_T("flagdisplayratio")] = 0.0;

    desc->Cells.Add(newNode);
    NormalizeNodeOrder(desc->Cells);

    return &(desc->Cells.Last());
}

wxVariantHash* MVG_DescManager::AddSymbol(long nodeID, wxVariantHash sDesc)
{
    MultiGraphDesc * desc = GetDesc();

    if(!desc) return NULL;

    size_t cc = desc->Cells.GetCount();

    LOGP(wxString::Format(_T("nodeID=%i"),nodeID));


    for( size_t a = 0; a < cc; a++ ) {
        if(desc->Cells[a].Properties[_T("id")].GetLong() == nodeID) {
            sDesc[_T("id")] = desc->GetNewItemID();
            sDesc[_T("node_id")] = nodeID;
            desc->Cells[a].Symbols.Add(sDesc);
            NormalizeSymbolOrder(desc->Cells[a].Symbols);
            return &(desc->Cells[a].Symbols.Last());
        }
    }
    return NULL;
}

wxString MVG_DescManager::GetUnusedSymbolname()
{
    SymbolGaugeDescArray arr;
    m_gdMgr->GetSymbolDescs(arr);

    MultiGraphDesc * desc = GetDesc();

    for(size_t a=0; a < arr.GetCount(); a++){
        SymbolGaugeDesc* sgd = arr[a];
        if( desc->FindBy( sgd->Name, _T("name") ) == NULL ) return sgd->Name;
    }

    return wxEmptyString;
}

VemsWidgetBase* MVG_DescManager::GetVemsWidgetBase()
{
    return m_mvg;
}
