#ifndef MVG_DESCMANAGER_H
#define MVG_DESCMANAGER_H


#include <wx/wx.h>
#include "../../common/types/wxVariantHash.h"
#include "../../gaugedesc/MultiGraphDesc.h"
#include "../../gaugedesc/GaugeDescManager.h"
#include "../../events/EditorValueChangedEvent.h"
#include "../../gaugedesc/MultiGraphDescManager.h"
#include "../../gaugedesc/GUIDescGUIItem.h"
#define MVG_FIRST_ORDER_NUMBER 1

enum {
    MVGD_SLIDE_LEFT,
    MVGD_SLIDE_RIGHT,
    MVGD_SLIDE_UP,
    MVGD_SLIDE_DOWN,
    MVGD_INSERT_AFTER,
    MVGD_INSERT_BEFORE
};

class MultiGraphDescManager;
class MultiValueGraph;
class VemsWidgetBase;
class SymbolPanel;
class NodePanel;

WX_DECLARE_HASH_MAP( int, SymbolPanel*, wxIntegerHash, wxIntegerEqual, SymbolPanelHash);
WX_DECLARE_HASH_MAP( int, NodePanel*, wxIntegerHash, wxIntegerEqual, NodePanelHash);


#define SETTINGS_TOGGLE _T("isToggleSymbols")

class MVG_DescManager
{
    public:
        MVG_DescManager(GaugeDescManager * gdmgr, int profileID);
        virtual ~MVG_DescManager();

        wxArrayString& GetSymbolList() { return m_symbolList; };
        GUIDescStructItemArray& GetDescStruct() { return m_mgMgr->GetMGDescStruct(); };


        void ParseSymbolList();


        GaugeDescManager* GetGaugeDescMgr() { return m_gdMgr; };

        wxVariantHash   Settings;

        wxVariantHash& GetSymbolDescClipboard() { return m_symbolDescClipboard; }
        void SetSymbolDescClipboard(wxVariantHash desc) { m_symbolDescClipboard = desc; }

        bool SlideSymbolTo(wxVariantHash * currentSymbolDesc, int target);
        bool RemoveSymbolDesc(wxVariantHash * currentSymbolDesc);
        bool SwapSymbolArrayItems(wxVariantHashArray& array, size_t idx1, size_t idx2);
        void NormalizeSymbolOrder(wxVariantHashArray& symbolarray);

        bool SlideNodeTo(MultiGraphDescNode * nodeDesc, int target);
        bool RemoveNode(MultiGraphDescNode * nodeDesc);
        bool SwapNodeArrayItems(MultiGraphDescNodeArray& array, size_t idx1, size_t idx2);
        void NormalizeNodeOrder(MultiGraphDescNodeArray& nodeArray);

        void SetMultiValueGraph(MultiValueGraph * mvg) { m_mvg = mvg; }
        void RefreshTargetGraph();

        MultiGraphDescNode* InsertNode(MultiGraphDescNode * nodeDesc, int target);
        MultiGraphDescNode* AppendNode();
        wxVariantHash* AddSymbol(long nodeID, wxVariantHash sDesc);

        wxString GetUnusedSymbolname();

        MultiGraphDesc* GetDesc() { return m_mgMgr->GetMultiGraphDesc(m_profileID); };

        MultiValueGraph* GetMultiValueGraph() { return m_mvg; };

        VemsWidgetBase* GetVemsWidgetBase();

    protected:

    private:

        int                 m_profileID;

        wxArrayString       m_symbolList;

        GaugeDescManager*   m_gdMgr;
        MultiGraphDescManager* m_mgMgr;

        wxVariantHash       m_symbolDescClipboard;

        MultiValueGraph *   m_mvg;


};


#endif // MVG_DESCMANAGER_H
