#include "MVG_SymbolDescChooserPanel.h"

#include "../../logger.h"
#include "../../DrawHelper.h"
#include "../VemsWidgetBase.h"

BEGIN_EVENT_TABLE(MVG_SymbolDescChooserPanel, wxPanelWithCtrlExBase)

    EVT_BUTTON(idSelectButton,MVG_SymbolDescChooserPanel::OnButton)
    EVT_BUTTON(idEditButton,MVG_SymbolDescChooserPanel::OnButton)


END_EVENT_TABLE()

IMPLEMENT_DYNAMIC_CLASS(MVG_SymbolDescChooserPanel,wxPanelWithCtrlExBase);

MVG_SymbolDescChooserPanel::MVG_SymbolDescChooserPanel(wxWindow* parent, MVG_DescManager* mgr)
{
    //ctor
    m_mgr = mgr;
    m_selectMenu = NULL;
    m_currentSymbolDescID = -1;

    Create(parent, wxID_ANY);

    m_label = new wxStaticText(this, wxID_ANY, _T(""),wxDefaultPosition, wxDefaultSize, wxALIGN_CENTRE | wxST_NO_AUTORESIZE | wxBORDER_SIMPLE);

    m_label->SetBackgroundColour( wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW) );

    CreateValidator(m_label,_T(""),_T("DEFAULT"));

    wxBitmapButton* sbutt = new wxBitmapButton(this,idSelectButton, DrawHelper::GetBitmap(_T("settings-16.png")) );
    wxBitmapButton* ebutt = new wxBitmapButton(this,idEditButton, DrawHelper::GetBitmap(_T("edit-14.png")) );

    wxBoxSizer* mainsizer = new wxBoxSizer(wxHORIZONTAL);

    mainsizer->Add( m_label, 1, wxRIGHT | wxTOP | wxBOTTOM | wxALIGN_CENTER_VERTICAL , 2 );
    mainsizer->Add( sbutt, 0, wxLEFT | wxTOP | wxBOTTOM , 2 );
    mainsizer->Add( ebutt, 0, wxLEFT | wxTOP | wxBOTTOM , 2 );

    SetSizer(mainsizer);
    mainsizer->SetSizeHints(this);

    m_gaugeDescA = m_mgr->GetGaugeDescMgr()->GetVEMSGaugeDescArray();
}

MVG_SymbolDescChooserPanel::~MVG_SymbolDescChooserPanel()
{
    //dtor
}

void MVG_SymbolDescChooserPanel::SetValue(const wxVariant& value)
{
    SetDescId( value.GetLong() );
}

wxVariant MVG_SymbolDescChooserPanel::GetValue()
{
    if(m_currentSymbolDescID == -1) return wxNullVariant;

    return (long)m_currentSymbolDescID;
}

void MVG_SymbolDescChooserPanel::SetSymbol(wxVariantHash* desc)
{
    m_symbolDesc = desc;

    wxString title = m_nullStr;

    if(desc->find(_T("gaugedesc_id")) != desc->end()) {
        m_currentSymbolDescID = (*(desc))[_T("gaugedesc_id")].GetLong();
        title = GetDescTitle( m_currentSymbolDescID );
    }


    m_label->SetLabel( title );

    SetStyleEx();
}

void MVG_SymbolDescChooserPanel::SetDescId(int id)
{
    m_currentSymbolDescID = id;

    m_label->SetLabel( GetDescTitle( m_currentSymbolDescID ) );

    DoOnUpdated();
}

wxString MVG_SymbolDescChooserPanel::GetDescTitle(int desc_id)
{
    for(size_t i = 0; i< m_gaugeDescA->Count(); i++){
        VEMSGaugeDesc *gd = m_gaugeDescA->Item(i);
        if( desc_id != gd->Properties[_T("id")].GetLong() ) continue;

        return gd->GetName();
    }

    return m_nullStr;
}

void MVG_SymbolDescChooserPanel::OnPopupMenuSelectSymbol(wxCommandEvent& evt)
{
    int id = evt.GetId();

    if( m_menuId2symbolDescId.find(id) == m_menuId2symbolDescId.end() ) return;

    SetDescId(m_menuId2symbolDescId[id]);
}

void MVG_SymbolDescChooserPanel::OpenDescEditor()
{
    VemsWidgetBase * vwb = m_mgr->GetVemsWidgetBase();
    if(!vwb) return;

    VEMSGaugeDesc* curr_gd = NULL;

    int gd_id = m_currentSymbolDescID;

    if(IsNull()) {
        wxString symbolname = (*(m_symbolDesc))[_T("name")].GetString();
        gd_id = m_mgr->GetGaugeDescMgr()->GetGaugeDescFieldBySymbolName( symbolname, _T("descriptors.id") ).GetLong();
    }

    for(size_t i = 0; i< m_gaugeDescA->Count(); i++){
        VEMSGaugeDesc *gd = m_gaugeDescA->Item(i);
        int currgdid = gd->Properties[_T("id")].GetLong();

        if( gd_id != currgdid ) continue;
        curr_gd = gd;
        break;
    }

    Logger::logf(LOG_MSG,_T("curr_gd=%p"),curr_gd);
    vwb->ShowDescEditor(curr_gd);
    LOG
}

void MVG_SymbolDescChooserPanel::OnButton(wxCommandEvent& evt)
{
    switch(evt.GetId()) {

        case idEditButton:
            OpenDescEditor();
        break;

        case idSelectButton: {

            wxMenu popupMenu;

            wxString symbolname = (*(m_symbolDesc))[_T("name")].GetString();

            wxString stype = m_mgr->GetGaugeDescMgr()->GetGaugeSymbolType(symbolname);

            for(size_t i = 0; i< m_gaugeDescA->Count(); i++){

                VEMSGaugeDesc *gd = m_gaugeDescA->Item(i);

                if(gd->GetType() != stype) continue;

                int id = wxNewId();

                popupMenu.AppendRadioItem(id, gd->GetName());

                int did = gd->Properties[_T("id")].GetLong();

                m_menuId2symbolDescId[id] = did;

                if( m_currentSymbolDescID == did ) popupMenu.Check(id,true);

                Connect( id, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MVG_SymbolDescChooserPanel::OnPopupMenuSelectSymbol) );

            }

            PopupMenu(&popupMenu);
        }break;

    }
}
