#include "MultiValueGraph.h"

#include <wx/keybinder.h>
#include <wx/dcbuffer.h>
#include <wx/cursor.h>
#include <wx/bitmap.h>

#include "../../agg/wxAgg.h"
#include "../../agg/wxAgg2.h"
#include "../../Math.h"

#include "../../events/UpdateViewerEvent.h"
#include "../../common/FunctionMeter.h"
#include "../../hotkeys/wxHotKeyManager.h"
#include "../../common/wxArrayHelper.h"
#include "../../loghistory/LogHistoryBase.h"
#include "../GaugeGroupManager.h"
#include "../../customwidgets/BubbleStatusBar.h"
#include "MVG_ToolbarPanel.h"
#include "../../customwidgets/wxBitmapToggleButton.h"
#include "../../UserLog.h"
#include "../../gaugedesc/MultiGraphDesc.h"
#include "../../DrawHelper.h"
#include "MVG_DescEditor.h"
#include "MVG_ThemeEditor.h"
#include "../../customwidgets/AggToolTip.h"
#include "../../wxVariantHelper.h"
#include "../../loghistory/SearchDialog.h"
#include "../../configmanager/LogSearchHistoryHelper.h"
#include "../../configmanager/ConfigHelper2.h"
#include "../../gaugedesc/VemsFlagDescManager.h"

#include "../../bcWidgets/bcLabel.h"
#include "../../bcWidgets/bcButton.h"
#include "../../bcWidgets/bcCheckBox.h"
#include "../../bcWidgets/bcWrapper.h"

#include "../../DialogHelper.h"
#include "../../DirHelper.h"

#include "../../bcWidgets/bcTextCtrl.h"
#include "../../bcWidgets/bcRadioButton.h"

#include "../../loghistory/SearchResultDialog.h"


WX_DECLARE_LIST( wxVariant, wxVariantList );
#include <wx/listimpl.cpp>
WX_DEFINE_LIST( wxVariantList );

/** TODO:

    MagnifySymbol, OffsetSymbol - redraw only actually symbol

*/



BEGIN_EVENT_TABLE(MultiValueGraph,VemsWidgetBase)

    EVT_PAINT(MultiValueGraph::OnPaint)
    EVT_SIZE ( MultiValueGraph::OnSize )

    EVT_UPDATE_VIEWER_EVENT( wxID_ANY, MultiValueGraph::OnUpdateValue )

    EVT_KEY_DOWN(MultiValueGraph::OnKeyDown)
    EVT_KEY_UP(MultiValueGraph::OnKeyUp)

    EVT_LEFT_DOWN(MultiValueGraph::OnLeftDown)
    EVT_LEFT_UP(MultiValueGraph::OnLeftUp)
    EVT_MOTION(MultiValueGraph::OnMouseMove)
    EVT_RIGHT_DOWN(MultiValueGraph::OnMouseRightDown)

    EVT_ENTER_WINDOW(MultiValueGraph::OnEnter)
    EVT_LEAVE_WINDOW(MultiValueGraph::OnLeave)

//    EVT_COMMAND_SCROLL(idScrollBar, MultiValueGraph::OnScrollBar)

    EVT_COMMAND_SCROLL_TOP(idScrollBar, MultiValueGraph::OnScrollBar)
    EVT_COMMAND_SCROLL_BOTTOM(idScrollBar, MultiValueGraph::OnScrollBar)
    EVT_COMMAND_SCROLL_LINEUP(idScrollBar, MultiValueGraph::OnScrollBar)
    EVT_COMMAND_SCROLL_LINEDOWN(idScrollBar, MultiValueGraph::OnScrollBar)
    EVT_COMMAND_SCROLL_PAGEUP(idScrollBar, MultiValueGraph::OnScrollBar)
    EVT_COMMAND_SCROLL_PAGEDOWN(idScrollBar, MultiValueGraph::OnScrollBar)
    EVT_COMMAND_SCROLL_THUMBRELEASE(idScrollBar, MultiValueGraph::OnScrollBar)

    EVT_BUTTON(idTbButton, MultiValueGraph::OnButton )

    EVT_MENU_RANGE(idPopupmenuOpenDescEditor, idPopupMenuEND, MultiValueGraph::OnPopupMenu)


    EVT_TIMER(idRefreshTimer,MultiValueGraph::OnRefreshTimer)
    //EVT_TIMER(idTestTimer,MultiValueGraph::OnRefreshTimer)

    EVT_SET_FOCUS(MultiValueGraph::OnSetFocus)

END_EVENT_TABLE()

IMPLEMENT_DYNAMIC_CLASS(MultiValueGraph, VemsWidgetBase);


MultiValueGraph::MultiValueGraph( wxWindow* parent, wxWindowID id, GaugeDescManager * gdmgr, GaugeProperties &prop, LogHistoryBase * lhb)
    :VemsWidgetBase(parent,id, gdmgr, prop)
    ,m_refreshTimer(this, idRefreshTimer)
    ,m_testTimer(this, idTestTimer)
{
    CreateBcParent(this);
    m_hasOwnDescriptorMgr = true;

    m_sizeCursorWidth = 22;///img.GetOptionInt(wxIMAGE_OPTION_CUR_HOTSPOT_X);

    LOGP(wxString::Format(_T("m_sizeCursorWidth=%i"),m_sizeCursorWidth));

    m_profileId = 1;
    if(prop.descName.ToLong(&m_profileId) == false) m_profileId = 1;

    m_themeId = 1;
    if(prop.themeName.ToLong(&m_themeId) == false) m_themeId = 1;

    m_currWinCursor = -1;

    m_dataMgr = NULL;
    m_prevValue = NULL;
    m_scrollBar = NULL;
    m_hotkeyMgr = NULL;

    for(int a = MGBMP_DUMMY_FIRST; a < MGBMP_DUMMY_LAST; a++) {
        m_layerBitmaps[a] = NULL;
    }



    m_dataMgr = lhb;
    m_isSymbolMenu = false;
    m_isThemeMenu = false;
    m_isGaugeDescMenu = false;
    m_isNeedCalcBmpPositions = true;
    m_inited = false;
    m_minDataLimit = 4;
    m_highlightedMarkerIdx = -1;
    m_lockedHistoryLen = -1;

    m_searchResultSections.Clear();

    m_hotkeyMgr = new wxHotKeyManager(this);
    m_statusBarMgr = new BubbleStatusBar();
    m_toolTipMgr = new AggToolTip();

    m_isRegistered = false;
    m_isLocked = true;

    m_isInSelect = false;
    m_isCursorMoved = false;

    m_cursorState = MGCS_DEFAULT;

    m_startSelectedDataIdx = -1;
    m_currentSelectedDataIdx = -1;

    m_toolbar =  new MVG_ToolbarPanel(this);

    m_scrollBar = new wxScrollBar(this,idScrollBar);
    m_scrollBar->Hide();

    //m_tbButton = new wxButton(this, idTbButton, _T("Open toolbar") );
    m_tbButton = new bcButton(this, idTbButton, _T("Open toolbar"), wxPoint(10,50));
    m_tbButton->SetProperty( BCBS_BORDER_COLOR, wxVariantHelper::ToVariant(wxColor(0,0,0)) );
    //m_tbButton->SetStyleFlag( bcBUTTON_TOGGLE );

    m_refreshTimeBase = 20;

    m_isRefresh = false;



    SetVariables();

    m_helpPanel = new bcLabel(this, wxID_ANY, wxEmptyString,wxPoint(20, prop.height + 20), wxSize(10,10), BCLS_AUTO_SIZE );
    m_helpPanel->Hide();

    m_helpPanel->SetProperty( BCBS_BACKGROUND_COLOR, wxVariantHelper::ToVariant(wxColor(255,255,255,240)) );

    DrawHelpPanel();

}

MultiValueGraph::MultiValueGraph()
{
    m_captionSize = 0;
    m_cellHeight = 0;
    m_cellsepar = 0;
    m_currentSelectedDataIdx = 0;
    m_currWinCursor = 0;
    m_cursorAccel = 0;
    m_cursorDelta = 0;
    m_cursorState = 0;
    m_cursorWidth = 0;
    m_dataMgr = 0;
    m_dataWidthLimitVert = 0;
    m_helpPanel = 0;
    m_helptextlen = 0;
    m_highlightedMarkerIdx = 0;
    m_historyLockLen = 0;
    m_hotkeyMgr = 0;
    m_inited = 0;
    m_isCursorMoved = 0;
    m_isInSelect = 0;
    m_isLocked = 0;
    m_isNeedCalcBmpPositions = 0;
    m_isRefresh = 0;
    m_isRegistered = 0;
    m_limitValHeight = 0;
    m_limitValStroke = 0;
    m_lockedHistoryLen = 0;
    m_magnifyRatio = 0;
    m_magStep = 0;
    m_maxMagnify = 0;
    m_minDataLimit = 0;
    m_minDataSize = 0;
    m_numValueHeight = 0;
    m_numValueStroke = 0;
    m_prevCursorState = 0;
    m_prevValue = 0;
    m_profileId = 0;
    m_refreshTimeBase = 0;
    m_resolution = 0;
    m_resStep = 0;
    m_scrollBar = 0;
    m_scrollBarSize = 0;
    m_sizeCursorWidth = 0;
    m_startSelectedDataIdx = 0;
    m_statusBarMgr = 0;
    m_statusBarSize = 0;
    m_summaryBarSize = 0;
    m_tbButton = 0;
    m_themeId = 0;
    m_timeRulerSize = 0;
    m_titleHeight = 0;
    m_titleStroke = 0;
    m_toolbar = 0;
    m_toolBarSize = 0;
    m_toolTipMgr = 0;
}

MultiValueGraph::~MultiValueGraph()
{
    m_refreshTimer.Stop();
    wxVariantHash datas;

    datas[_T("magnifyratio")] = (long)m_magnifyRatio;
    datas[_T("resolution")] = (long)m_resolution;
    datas[_T("cursoridx")] = (long)m_currentSelectedDataIdx;
    datas[_T("refreshgauges")] = m_toolbar->GetValue(_T("refreshgauges"));
    datas[_T("showtoolbar")] = m_toolbar->IsShown();

    datas[_T("showusermarker")] = m_markerTypes[LH_MARKER_USER][_T("show")];
    datas[_T("showstaticmarker")] = m_markerTypes[LH_MARKER_STATIC][_T("show")];

    SaveSettingsBase(datas);


    //dtor
    ClearGraphDataArray();
    ClearDescExtraFields();

    ptrdel(m_statusBarMgr);
    ptrdel(m_toolTipMgr);
    ptrdel(m_hotkeyMgr);

    for(int a = MGBMP_DUMMY_FIRST; a < MGBMP_DUMMY_LAST; a++) {
        ptrdel(m_layerBitmaps[a]);
    }

    WX_CLEAR_HASH_MAP(NumValueBoxesHash, m_numBoxes);

    //LOG
}

void MultiValueGraph::OnSetFocus(wxFocusEvent& evt)
{
    LOG
}

void MultiValueGraph::OnRefreshTimer(wxTimerEvent& evt)
{
    switch(evt.GetId()) {

        case idRefreshTimer:
    if(m_isRefresh)
        Refresh(false);
            m_isRefresh = false;
            break;

}
}

void MultiValueGraph::OnSearchResultBox(GeneralEvent& evt)
{
    wxVariantHash hash = evt.GetVariantHash();
    if(hash.find(_T("start_idx")) == hash.end() || hash.find(_T("end_idx")) == hash.end()) return;

    long sidx = hash[_T("start_idx")].GetLong();
    //long eidx = hash[_T("end_idx")].GetLong();

    m_startSelectedDataIdx = -1;//eidx;

    MoveCursorTo(sidx);

    GaugeRefresh();
}

void MultiValueGraph::OnEnter(wxMouseEvent& evt)
{
    m_cursorState = MGCS_DEFAULT;
}

void MultiValueGraph::OnLeave(wxMouseEvent& evt)
{
    m_cursorState = MGCS_DEFAULT;
    SetGraphCursor(-5);
}

void MultiValueGraph::OnButton(wxCommandEvent& evt)
{
    KlappeToolbar();
    GaugeRefresh();
    SetFocus();
}

void MultiValueGraph::OpenDescEditor(long flags)
{
    wxString dname = _T("MultiValueGraphDescEditor_Inner");

    wxWindow* win = DialogHelper::GetDialog( dname );

    if(win != NULL) {
        win->SetFocus();
        return;
    }

    MVG_DescEditor * editor = new MVG_DescEditor(this, m_gaugeDescMgr, m_profileId );

    wxVariantHash* hash = SearchSymbolByName(m_hoverSymbol);

    int nodeid = 0;
    int symbolid = 0;

    if( hash && ((flags & ODE_NODE) || (flags & ODE_SYMBOL)) )
        nodeid = (*hash)[_T("node_id")].GetLong();

    if( hash && (flags & ODE_SYMBOL) )
        symbolid = (*hash)[_T("id")].GetLong();

    editor->InitEditor( nodeid, symbolid );

    editor->SetMultiGraph(this);
    editor->Show(1);


    DialogHelper::SetDialog(dname,editor);
}

void MultiValueGraph::OpenThemeEditor()
{
    wxString dname = _T("MultiValueGraphThemeEditor_Inner");

    wxWindow* win = DialogHelper::GetDialog( dname );

    if(win != NULL) {
        win->SetFocus();
        return;
    }

    MVG_ThemeEditor* te = new MVG_ThemeEditor(this, m_gaugeDescMgr, m_themeId);
    te->SetMultiGraph(this);

    DialogHelper::SetDialog(dname,te);
}

void MultiValueGraph::OnPopupMenu(wxCommandEvent& evt)
{
    switch(evt.GetId()){

        case idPopupmenuOpenDescEditor:
            OpenDescEditor();
            break;

        case idPopupmenuOpenNodeDescEditor:
            OpenDescEditor( ODE_NODE );
            break;

        case idPopupmenuOpenSymbolDescEditor:
            OpenDescEditor( ODE_SYMBOL );
            break;

        case idPopupmenuOpenThemeEditor:
            OpenThemeEditor();
            break;

        case idPopupmenuSearchFor:
            SearchDlg(m_tmphoverSymbol);
            break;
    }
}

void MultiValueGraph::OnPopupMenuSelectSymbol(wxCommandEvent& evt)
{
    int id = evt.GetId();

    if( m_menuID2symbolname.find(id) == m_menuID2symbolname.end() ) return;

    wxString selSymbol = m_menuID2symbolname[id];

    if(m_tmphoverSymbol == selSymbol) return;

    wxVariantHash* symbolhash = SearchSymbolByName(m_tmphoverSymbol);

    if(!symbolhash) return;

    (*symbolhash)[_T("name")] = selSymbol;
    symbolhash->erase(_T("gaugedesc_id"));

    GetDescMgr()->UpdateSymbol(*symbolhash);

    RedrawWidget();
}

void MultiValueGraph::OnPopupMenuSelectDesc(wxCommandEvent& evt)
{
    int wid = evt.GetId();

    if( m_menuID2profID.find(wid) == m_menuID2profID.end() ) return;

    if(m_profileId == m_menuID2profID[wid]) return;

    UnRegisterSymbols();

    m_profileId = m_menuID2profID[wid];

    RegisterSymbols();

    wxString descstr = wxString::Format(_T("%i"),m_profileId);

    m_props.descName = descstr;

    m_groupManager->SaveDesc( GetItemID(), descstr );

    RedrawWidget();
}

void MultiValueGraph::OnPopupMenuSelectTheme(wxCommandEvent& evt)
{
    int wid = evt.GetId();

    if( m_menuID2themeID.find(wid) == m_menuID2themeID.end() ) return;

    if(m_themeId == m_menuID2themeID[wid]) return;

    m_themeId = m_menuID2themeID[wid];

    wxString themestr = wxString::Format(_T("%i"),m_themeId);

    m_props.themeName = themestr;

    m_groupManager->SaveTheme( GetItemID(), themestr );

    RedrawWidget();
}

void MultiValueGraph::OnPopupMenuSelectMarkers(wxCommandEvent& evt)
{
    int wid = evt.GetId();

    for(Int2wxVariantHashHash::iterator it = m_markerTypes.begin(); it != m_markerTypes.end(); it++) {
        wxVariantHash& hash = it->second;

        if( hash[_T("popupMenuID")].GetLong() != wid ) continue;

        hash[_T("show")] = evt.IsChecked();

        GaugeRefresh();

        return;
    }


}

void MultiValueGraph::MouseCursor(int mouse_state, wxMouseEvent &evt)
{/*
    wxPoint pos = evt.GetPosition();

    switch(mouse_state){
        case MSC_MOVE: {


            }break;

    };

    switch(m_cursorState){

        case MGCS_OVER_SYMBOL_TITLE:{


            }break;

    };
*/
}

wxString MultiValueGraph::CursorStateToStr()
{
    wxString str;


    if( m_cursorState & MGCS_OVER_SYMBOL_TITLE ) str += _T("MGCS_OVER_SYMBOL_TITLE | ");

    if( m_cursorState & MGCS_OVER_GRAPH ) str += _T("MGCS_OVER_GRAPH | ");

    if( m_cursorState & MGCS_OVER_HELPPANEL ) str += _T("MGCS_OVER_HELPPANEL | ");

    if( m_cursorState & MGCS_OVER_SELECTION ) str += _T("MGCS_OVER_SELECTION | ");

    if( m_cursorState & MGCS_OVER_SELECT_RESIZE_START ) str += _T("MGCS_OVER_SELECT_RESIZE_START | ");

    if( m_cursorState & MGCS_OVER_SELECT_RESIZE_CURRENT ) str += _T("MGCS_OVER_SELECT_RESIZE_CURRENT | ");

    if( m_cursorState & MGCS_IN_SELECT ) str += _T("MGCS_IN_SELECT | ");

    if( m_cursorState & MGCS_IN_SELECT_RESIZE_START ) str += _T("MGCS_IN_SELECT_RESIZE_START | ");

    if( m_cursorState & MGCS_IN_SELECT_RESIZE_CURRENT ) str += _T("MGCS_IN_SELECT_RESIZE_CURRENT | ");

    if( m_cursorState & MGCS_IN_SELECT_MOVE ) str += _T("MGCS_IN_SELECT_MOVE | ");

    str.RemoveLast(3);

    return str;
}

void MultiValueGraph::OnLeftDown(wxMouseEvent &evt)
{
    if( IsSlimData() ) {
        evt.Skip();
        return;
    }

    if( m_groupManager->IsEditMode() ) {
        evt.Skip();
        return;
    }

    wxPoint pos = evt.GetPosition();

    m_lastMousePos = pos;

    if( m_cursorState & MGCS_OVER_HELPPANEL ) {
        evt.Skip();
        return;
    }

    //Logger::logf(LOG_MSG,_T("mouse left down state=")+CursorStateToStr());



    if( m_cursorState & MGCS_OVER_SYMBOL_TITLE ) {
        Logger::logf(LOG_MSG,_T("m_hoverSymbol=")+m_hoverSymbol);
        DrawBackgroundFaceAgg(m_hoverSymbol);
        GaugeRefresh();
        evt.Skip();
        return;
    }

    if( m_cursorState & MGCS_OVER_GRAPH ) {

        if( m_cursorState & MGCS_OVER_SELECTION ) m_cursorState |= MGCS_IN_SELECT_MOVE;

        else if( m_cursorState & MGCS_OVER_SELECT_RESIZE_START ) m_cursorState |= MGCS_IN_SELECT_RESIZE_START;

        else if( m_cursorState & MGCS_OVER_SELECT_RESIZE_CURRENT ) m_cursorState |= MGCS_IN_SELECT_RESIZE_CURRENT;

        else {

            if(evt.ShiftDown()) {
                RefreshSelectionByMousePos(pos.x);
            } else {

                if( (m_cursorState & MGCS_IN_SELECT) == 0 ) {

                    m_startSelectedDataIdx = -1;

                    RefreshSelectionByMousePos(pos.x,true);

                    m_cursorState |= MGCS_IN_SELECT;
                }

            }

        }

    }


    if(m_cursorState & MGCS_OVER_SUMMARY_BAR) {
        int jump_to = -1;
        if(m_cursorState & MGCS_HIGHLIGHT_MARKER) {
            jump_to = m_highlightedMarkerIdx;
        } else {
            m_startSelectedDataIdx = -1;
            m_currentSelectedDataIdx = (int)((double)pos.x / (double)(GetClientSize().x) * (double)(getHistoryLen()));
            jump_to = m_currentSelectedDataIdx;
            if(m_toolbar->GetValue(_T("refreshgauges")).GetBool()) RefreshHistory();
            DrawSummaryBar();
        }

        MoveScreenTo( jump_to - (GetRightIdx() - GetLeftIdx())/2 );
        GaugeRefresh();
    }


    evt.Skip();
}

void MultiValueGraph::SetCursorState(const wxPoint& pos)
{


    if( IsInRect(m_graphRect,pos) ) {
        m_cursorState |= MGCS_OVER_GRAPH;
        SetGraphCursor(wxCURSOR_IBEAM);
    } else {
        m_cursorState = MGCS_DEFAULT;
        SetGraphCursor(-2);
    }

    int spos = -1;
    int cpos = -1;
    int sens_delta = m_sizeCursorWidth/2;

    //LOGP(wxString::Format(_T("m_sizeCursorWidth=%i"),m_sizeCursorWidth));

    if( m_cursorState & MGCS_OVER_GRAPH && m_startSelectedDataIdx != -1 && m_currentSelectedDataIdx != m_startSelectedDataIdx && (m_cursorState & MGCS_IN_SELECT) == 0 ) {

        GraphDataDescriptor * gdata = SearchBGraphDataByIdx(m_startSelectedDataIdx);
        if(gdata)
            spos = gdata->GetXPos();

        gdata = SearchBGraphDataByIdx(m_currentSelectedDataIdx);

        if(gdata)
            cpos = gdata->GetXPos();
/*
        int minpos = spos < cpos ? spos : cpos;
        int maxpos = spos > cpos ? spos : cpos;

        if(pos.x+sens_delta >= minpos && pos.x-sens_delta <= maxpos) {
            m_cursorState |= MGCS_OVER_SELECTION;
            SetGraphCursor(wxCURSOR_SIZING);
        } else {
            m_cursorState &= ~MGCS_OVER_SELECTION;
        }
*/

        if( (m_cursorState & MGCS_IN_SELECT_RESIZE_START) == 0 ) {
            if( pos.x <= spos+sens_delta && pos.x >= spos ) m_cursorState |= MGCS_OVER_SELECT_RESIZE_START;
            else m_cursorState &= ~MGCS_OVER_SELECT_RESIZE_START;
        }

        if( (m_cursorState & MGCS_IN_SELECT_RESIZE_CURRENT) == 0 ) {
            if(  pos.x <= cpos+sens_delta && pos.x >= cpos ) m_cursorState |= MGCS_OVER_SELECT_RESIZE_CURRENT;
            else m_cursorState &= ~MGCS_OVER_SELECT_RESIZE_CURRENT;
        }

        if( m_cursorState & (MGCS_OVER_SELECT_RESIZE_START | MGCS_OVER_SELECT_RESIZE_CURRENT) ) SetGraphCursor(wxCURSOR_SIZEWE);

    }

    if( (m_cursorState & MGCS_OVER_HELPPANEL) == 0 ) {

        wxString hsymbol = GetHoverSymbolByPos(pos);

        if(hsymbol != wxEmptyString) {
            m_hoverSymbol = hsymbol;
                SetGraphCursor(wxCURSOR_HAND);
                m_cursorState |= MGCS_OVER_SYMBOL_TITLE;
        } else {
            m_cursorState &= ~MGCS_OVER_SYMBOL_TITLE;
            }

    }

    if(pos.y > m_graphRect.GetBottom() + m_cellsepar && pos.y <= (m_graphRect.GetBottom() + m_summaryBarSize + m_cellsepar )) {
        m_cursorState |= MGCS_OVER_SUMMARY_BAR;
        SetGraphCursor(wxCURSOR_CROSS);
    } else {
        m_cursorState &= ~MGCS_OVER_SUMMARY_BAR;
        //SetGraphCursor(-3);
    }

    if( m_cursorState & MGCS_OVER_SUMMARY_BAR ) {

        bool match = false;

        for(wxRectHash::iterator it = m_markerSummaryRects.begin(); it != m_markerSummaryRects.end(); it++) {
            if(IsInRect(it->second,pos)) {
                SetGraphCursor(wxCURSOR_HAND);
                m_cursorState |= MGCS_HIGHLIGHT_MARKER;
                match = true;
                m_highlightedMarkerIdx = it->first;
                break;
            }
        }

        if(!match) m_cursorState &= ~MGCS_HIGHLIGHT_MARKER;

    }

    //Logger::logf(LOG_MSG,_T("mouse move, state=")+CursorStateToStr());

}

void MultiValueGraph::OnMouseMove(wxMouseEvent& evt)
{

    if( IsSlimData() ) {
        evt.Skip();
        return;
    }

    if( m_groupManager->IsEditMode() ) {
        SetGraphCursor(-4);
        evt.Skip();
        return;
    }

    wxPoint pos = evt.GetPosition();

    SetCursorState(pos);

    if( m_cursorState & MGCS_IN_SELECT ) {

        RefreshSelectionByMousePos(pos.x, false, false);
        evt.Skip();
        return;

    }

    if( m_cursorState & MGCS_IN_SELECT_MOVE ) {

    }


    if( m_cursorState & MGCS_IN_SELECT_RESIZE_START ) {


        GraphDataDescriptor * gdata = SearchBNearlyGraphDataByPosition( pos.x );
        if(gdata)
            m_startSelectedDataIdx = gdata->GetIdx();


        DrawSelectArea();
        RefreshSelectedTimeInfoBar();
        DrawSummaryBar();
        GaugeRefresh();

    }

    if( m_cursorState & MGCS_IN_SELECT_RESIZE_CURRENT ) {

        GraphDataDescriptor * gd = SearchBNearlyGraphDataByPosition( pos.x );

        if(gd)
            m_currentSelectedDataIdx = gd->GetIdx();


        DrawNumValues(gd->GetRTData());
        DrawSelectArea();
        RefreshSelectedTimeInfoBar();
        DrawSummaryBar();
        GaugeRefresh();

    }


    if( m_cursorState & MGCS_HIGHLIGHT_MARKER ) {

        DrawSummaryBar(m_highlightedMarkerIdx);
        DrawMarkerToolTip(m_highlightedMarkerIdx);
        GaugeRefresh();

    }

    if( (m_cursorState & MGCS_HIGHLIGHT_MARKER) == 0 && m_highlightedMarkerIdx != -1 ) {

        m_highlightedMarkerIdx = -1;
        DrawSummaryBar(m_highlightedMarkerIdx);
        DrawMarkerToolTip(m_highlightedMarkerIdx);
        GaugeRefresh();
    }



    evt.Skip();

}

void MultiValueGraph::OnLeftUp(wxMouseEvent &evt)
{
    if( IsSlimData() ) {
        evt.Skip();
        return;
    }

    if( m_groupManager->IsEditMode() ) {
        evt.Skip();
        return;
    }

    //Logger::logf(LOG_MSG,_T("mouse left up state=")+CursorStateToStr());

    wxPoint pos = evt.GetPosition();

    if( m_cursorState & MGCS_OVER_GRAPH ) {

        if( m_cursorState & MGCS_OVER_SELECTION ) m_cursorState &= ~MGCS_IN_SELECT_MOVE;

        else if( m_cursorState & MGCS_OVER_SELECT_RESIZE_START ) m_cursorState &= ~MGCS_IN_SELECT_RESIZE_START;

        else if( m_cursorState & MGCS_OVER_SELECT_RESIZE_CURRENT ) m_cursorState &= ~MGCS_IN_SELECT_RESIZE_CURRENT;

        else {

            if(evt.ShiftDown()) {
                RefreshSelectionByMousePos(pos.x);

            } else {

                if( m_cursorState & MGCS_IN_SELECT ) {

                    RefreshSelectionByMousePos(pos.x);

                    m_cursorState &= ~MGCS_IN_SELECT;
                }

            }

        }

    }

    m_lastMousePos = wxDefaultPosition;

}

void MultiValueGraph::OnMouseRightDown(wxMouseEvent& evt)
{
    //Logger::logf(LOG_MSG,_T("MultiValueGraph::OnMouseRightDown"));

    if( m_groupManager->IsEditMode() ) {
        evt.Skip();
        return;
    }

    wxMenu popupMenu;

    CreatePopupMenu(popupMenu);

    PopupMenu(&popupMenu);
}

bool MultiValueGraph::IsInRect(const wxRect& rect, const wxPoint& point )
{
    if(rect.GetTop() <= point.y && rect.GetBottom() >= point.y && rect.GetLeft() <= point.x && rect.GetRight() >= point.x)
        return true;
    else
        return false;
}

void MultiValueGraph::DrawSelectArea()
{
    if( IsSlimData() ) return;

    if( m_startSelectedDataIdx == -1 ) {
        ptrdel(m_layerBitmaps[MGBMP_SELECTED_FIELD]);
        return;
    }

    int li = GetLeftIdx();
    int ri = GetRightIdx();
    int step = getStep();

    int start_idx = m_startSelectedDataIdx;
    int end_idx = m_currentSelectedDataIdx;

    chkminmax( start_idx, li-m_resolution, ri );
    chkminmax( end_idx, li, ri );

    if( end_idx == start_idx ) {
        ptrdel(m_layerBitmaps[MGBMP_SELECTED_FIELD]);
        //Logger::logf(LOG_MSG,_T("DrawSelectArea() start_idx == end_idx ! "));
        return;
    }

    int w = step * abs(start_idx-end_idx) / m_resolution;
    int h = m_graphRect.GetHeight() - m_cellsepar;

//    Logger::logf(LOG_MSG,_T("w=%i, d=%i"),w, abs(start_idx-end_idx));

    chkminmax(w,0,m_graphRect.GetWidth());

    if( w < step ) {
        ptrdel(m_layerBitmaps[MGBMP_SELECTED_FIELD]);
        return;
    }

#if 0
/// 1,79 ms
    wxAgg aggCanvas(w,h);

    //aggCanvas.clear(GetDesc()->GetColor(_T("selectcolor")));
    aggCanvas.setBrush( GetTheme()->GetColor(_T("selectcolor")) );
    aggCanvas.drawSolidRectangle(0,0,w,h);

    ptrdel(m_layerBitmaps[MGBMP_SELECTED_FIELD]);

    m_layerBitmaps[MGBMP_SELECTED_FIELD] = aggCanvas.renderToBitmap();

#else
/// 1,07 ms (faster then the other method with 40%, and looks like better)
    wxAgg2 aggCanvas(w,h,GetTheme()->GetColor(_T("selectcolor")));

    ptrdel(m_layerBitmaps[MGBMP_SELECTED_FIELD]);

    m_layerBitmaps[MGBMP_SELECTED_FIELD] = aggCanvas.renderToBitmap();

#endif

}

void MultiValueGraph::DrawSummaryBar(int highlightMarkerIdx)
{
    //Logger::logf(LOG_MSG,_T("MultiValueGraph::DrawSummaryBar highlightMarkerIdx=%i"),highlightMarkerIdx);


    int w = GetClientSize().x;
    int h = m_summaryBarSize;
    int hc = getHistoryLen();

    m_markerSummaryRects.clear();

//    Logger::logf(LOG_MSG,_T("w=%i, h=%i, hc=%i"),w,h,hc);


    MultiGraphTheme * theme = GetTheme();

    wxAgg2 aggCanvas( w, h );
    //aggCanvas.clear( theme->GetColor(_T("summarybar_bgcolor")) );

    int y = m_graphRect.GetBottom() + m_cellsepar;

    wxAggBrush* brush = aggCanvas.getBrush();
    wxAggPen* pen = aggCanvas.getPen();



    const double brd = 1.0;
    double lw = 0;

// draw markers
    pen->SetStyle(wxAGGPEN_SOLID);
    pen->SetWidth(lw);

    for( Int2wxVariantHashHash::iterator it = m_markerTypes.begin(); it != m_markerTypes.end(); it++ ) {

        if( m_markerList.find(it->first) == m_markerList.end() ) continue;

        wxVariantHash& hash = it->second;
        wxColor c = theme->GetColor( hash[_T("color")].GetString() );

        pen->SetColor( c );
        lw = theme->GetDouble( hash[_T("width")].GetString() ,1.0);

        IntArray& arr = m_markerList[it->first];

        for( size_t i = 0; i < arr.GetCount(); i++ ) {
            int idx = arr[i];

            if(highlightMarkerIdx == idx) {
                pen->SetColor( DrawHelper::InvertColor(c) );
            }

            double x = (double)idx / (double)hc * (double)w;

            m_markerSummaryRects[idx] = wxRect( ((int)x)-1, y, (int)ceil(lw)+2, m_summaryBarSize );

            aggCanvas.drawLine(x-lw/2.0,brd/2.0,x-lw/2.0,h-brd);

            if(highlightMarkerIdx == idx) {
                pen->SetColor( c );
                m_toolTipMgr->SetProperty( ATTET_MARKERPOS, x );
                //Logger::logf(LOG_MSG,_T("x=%.2f"),x);
            }

        }
    }

// draw current cursor position
    double x = (double)m_currentSelectedDataIdx / (double)hc * (double)w;
    pen->SetColor( theme->GetColor(_T("cursorcolor")) );
    lw = 1.0;

    aggCanvas.drawLine(x-lw/2.0,brd/2.0,x-lw/2.0,h-brd);

    pen->SetColor( theme->GetColor(_T("fgcolor")) );
    pen->SetWidth(brd);
    aggCanvas.drawRectangle(brd/2.0,brd/2.0,w-brd,h-brd);

// draw search result
    pen->SetStyle(wxAGGPEN_TRANSPARENT);
    brush->SetColor(wxColor(180,0,0,200));
    brush->SetStyle(wxAGGBRUSH_SOLID);

    //Logger::logf(LOG_MSG,_T("m_searchResultSections.GetCount()=%i"),m_searchResultSections.GetCount());

    double minSearchWidth = 1.0;

    if(m_searchResultSections.GetCount()) {

        for(size_t a=0; a<m_searchResultSections.GetCount();a++){

            Section& se = m_searchResultSections[a];

            wxRealRect rect;

            rect.SetTopLeft(0,  (double)se.beginIdx / hc * w );
            rect.SetBottomRight(h, (double)se.endIdx / hc * w );

            //Logger::logf(LOG_MSG,_T("rect: ")+rect.GetAsString());
            if( rect.w < minSearchWidth ) rect.w = minSearchWidth;

            aggCanvas.drawRectangle( rect );
        }

    }

// draw selection
    if( m_startSelectedDataIdx != -1 ) {
        int sidx = m_startSelectedDataIdx < m_currentSelectedDataIdx ? m_startSelectedDataIdx : m_currentSelectedDataIdx;
        int eidx = m_startSelectedDataIdx > m_currentSelectedDataIdx ? m_startSelectedDataIdx : m_currentSelectedDataIdx;

        wxColor c = theme->GetColor(_T("selectcolor"));

        //Logger::logf(LOG_MSG,_T("c=")+ DrawHelper::GetColorAsString(c,wxC2S_CSS_SYNTAX) );
        brush->SetColor( c );

        double sx = (double)sidx / (double)hc * (double)w;
        double ex = (double)eidx / (double)hc * (double)w;

        aggCanvas.drawRectangle(sx,brd/2.0,ex-sx,h-brd);
    }

    ptrdel(m_layerBitmaps[MGBMP_SUMMARY]);

    m_layerBitmaps[MGBMP_SUMMARY] = aggCanvas.renderToBitmap();
}

void MultiValueGraph::DrawMarkerToolTip(int idx)
{
    if(idx == -1) {
        ptrdel(m_layerBitmaps[MGBMP_TOOLTIP]);
        return;
    }

    IntArray idxs;
    idxs.Add(idx);
    DoubleHashArray datas;

    m_dataMgr->getHistory(idxs, datas);

    if(datas.GetCount() != 1) return;

    DoubleHash* data = datas[0];

    if(!data) return;
    if(!CheckData(data,_T("time"))) return;

    wxString str = wxString::Format(_T("Idx: %i\nTime: "),idx) + FormatTimeSpan( getTimeMs(data) );

    m_toolTipMgr->SetProperty( ATT_TEXT_CONTENT, str );

    double x = m_toolTipMgr->GetProperty( ATTET_MARKERPOS, -2000.0 );

    double refpos = 0.5;

    if( x != -2000 ) {

        wxSize size = m_toolTipMgr->CalcSize();

        if( x < 0 ) {
            refpos = (fabs(x) / (double)(size.x));
        } else {
            int w = GetClientSize().x;
            if( x > w - size.x ) {
                int diff = (int)x+size.x - w;
                refpos = ((double)diff / (double)(size.x));
            }
        }
    }


    m_toolTipMgr->SetProperty( ATT_ARROW_REFPOSITION, refpos );

    ptrdel(m_layerBitmaps[MGBMP_TOOLTIP]);

    m_layerBitmaps[MGBMP_TOOLTIP] = m_toolTipMgr->DrawBox();

}

void MultiValueGraph::DrawMarkerLines()
{
    int h = GetClientSize().y - GetHeaderSize() - GetFooterSize();


    MultiGraphTheme* theme = GetTheme();

    for( Int2wxVariantHashHash::iterator it = m_markerTypes.begin(); it != m_markerTypes.end(); it++ ) {

        wxVariantHash& hash = it->second;

        int w = (int)ceil( theme->GetDouble( hash[_T("graphwidth")].GetString() ) );

        wxColor color = theme->GetColor( hash[_T("graphcolor")].GetString() );

        wxAgg aggCanvas(w,h);
        aggCanvas.setBrush(color);
        aggCanvas.drawSolidRectangle(0,0,w,h);
        wxBitmap * bmp = aggCanvas.renderToBitmap();

        wxBitmap newBitmap = bmp->GetSubBitmap(wxRect(0, 0, bmp->GetWidth(), bmp->GetHeight()));

        delete bmp;

        if( hash.find(_T("bmp")) != hash.end() ) {
            hash.erase(_T("bmp"));
        }

        hash[_T("bmp")] << newBitmap;
    }

}

void MultiValueGraph::SetThemeVars()
{
    MultiGraphTheme* theme = GetTheme();

    m_cellsepar = theme->GetInt(_T("cellseparator"));

    m_numValueStroke = theme->GetDouble(_T("numbervaluestroke"));
    m_numValueHeight = theme->GetDouble(_T("numbervalueheight"));

    m_titleStroke = theme->GetDouble(_T("titlestroke"));
    m_titleHeight = theme->GetDouble(_T("titleheight"));

    m_limitValStroke = theme->GetDouble(_T("limitvalsstroke"));
    m_limitValHeight = theme->GetDouble(_T("limitvalsheight"));

    m_cursorDelta = theme->GetInt(_T("cursorsmallstep"));
    m_cursorAccel = theme->GetInt(_T("cursoraccel"));

    m_dataWidthLimitVert = theme->GetInt(_T("datawidthlimit_for_verticallines"));

    m_captionSize = theme->GetInt(_T("captionsize"));
    m_timeRulerSize = theme->GetInt(_T("timerulerbarsize"));
    m_scrollBarSize = theme->GetInt(_T("scrollbarsize"));
    m_statusBarSize = theme->GetInt(_T("statusbarsize"));
    m_summaryBarSize = theme->GetInt(_T("summarybarsize"));

    m_cursorWidth = theme->GetDouble(_T("cursorwidth"));

    m_maxMagnify = theme->GetInt(_T("maxmagnify"));

    m_minDataSize = theme->GetInt(_T("mindatasize"));

    m_resStep = theme->GetInt(_T("resolutionsteps"));
    m_magStep = theme->GetInt(_T("magnifysteps"));

}

void MultiValueGraph::SetVariables()
{

    wxVariantHash datas;
    datas[_T("refreshgauges")] = true;
    datas[_T("showtoolbar")] = true;

    int res = LoadSettingsBase( datas );

    if( res == 0 ) {

        m_magnifyRatio = datas[_T("magnifyratio")].GetLong();
        m_resolution = datas[_T("resolution")].GetLong();

        m_currentSelectedDataIdx = datas[_T("cursoridx")].GetLong();

    } else {

        m_magnifyRatio = -1;
        m_resolution = -1;
        m_currentSelectedDataIdx = -1;

    }

    m_toolbar->Show( datas[_T("showtoolbar")].GetBool() );
    m_toolbar->SendCommand(_T("refreshgauges"),datas[_T("refreshgauges")]);

    UpdateToolbarButton();

    SetThemeVars();

    chkminmax(m_magnifyRatio,1,m_maxMagnify);

    m_toolBarSize = 28;

}

void MultiValueGraph::RefreshRTSymbols()
{
    UnRegisterSymbols();

    RegisterSymbols();
}

void MultiValueGraph::RegisterSymbols()
{
    m_isRegistered = false;

    requireChanged();

    m_rtDataMgr->getRequiredSymbols();

}

void MultiValueGraph::UnRegisterSymbols()
{
    wxArrayString arr;
    GetDesc()->GetSymbolNameList(arr);

    arr.Add(_T("secl"));

    for(size_t a=0;a<arr.GetCount();a++) {
        m_rtDataMgr->unregisterSymbol( arr[a] );
    }
}

wxMenu* MultiValueGraph::GetSymbolChooserMenu()
{
    m_tmphoverSymbol = GetHoverSymbolByPos(ScreenToClient(wxGetMousePosition()));

    wxMenu* popupMenu = new wxMenu();

    int cid=0;

    m_menuID2symbolname.clear();

    wxSortedArrayString grparr;

    m_gaugeDescMgr->GetSymbolGroupListM(grparr);

    for(size_t a=0; a < grparr.GetCount(); a++){

        wxString grpname = grparr[a];

        wxMenu * sgmenu = new wxMenu();

        SymbolGaugeDescArray arr;

        m_gaugeDescMgr->GetSymbolDescs(grpname, arr);

        for(size_t i = 0; i < arr.GetCount(); i++ ) {

            SymbolGaugeDesc * desc = arr[i];

            if(i%20 == 0) sgmenu->Break();

            int id = wxNewId();

            sgmenu->AppendCheckItem(id, desc->Title );

            m_menuID2symbolname[id] = desc->Name;

            if(m_tmphoverSymbol == desc->Name) cid = id;

            Connect( id, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MultiValueGraph::OnPopupMenuSelectSymbol) );

        }

        popupMenu->AppendSubMenu(sgmenu,grpname);
    }

    wxMenu* smenu = new wxMenu();

    wxVariantHashArray& flagGroups = m_gaugeDescMgr->GetVemsFlagDescMgr()->GetGroups();

    for(size_t a=0;a<flagGroups.GetCount();a++){
        wxVariantHash& group = flagGroups[a];
        smenu->AppendSubMenu(new wxMenu(), group[_T("title")].GetString());
    }

    VemsFlagDescHash& fhash = m_gaugeDescMgr->GetVemsFlagDescMgr()->GetAllDesc();

    wxArrayString flags;
    wxVariantHash flag2group;

    for(VemsFlagDescHash::iterator it = fhash.begin(); it != fhash.end(); it++ ) {
        flags.Add(it->first);
        flag2group[it->first] = it->second.Properties[_T("group_title")].GetString();
    }

    flags.Sort();

    for(size_t a=0; a<flags.GetCount(); a++) {
        int id = wxNewId();

        wxString name = flags[a];

        wxMenu* gmenu = smenu->FindItem(smenu->FindItem(flag2group[name].GetString()))->GetSubMenu();
        if(!gmenu) continue;

        if(gmenu->GetMenuItemCount() % 20 == 0) gmenu->Break();

        gmenu->AppendCheckItem(id, name );

        m_menuID2symbolname[id] = name;

        if(m_tmphoverSymbol == name) cid = id;

        Connect( id, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MultiValueGraph::OnPopupMenuSelectSymbol) );
    }

    popupMenu->AppendSubMenu(smenu,_T("Flags"));

    popupMenu->Check(cid, true);

    return popupMenu;
}

void MultiValueGraph::CreatePopupMenu(wxMenu& parent)
{

    if(m_cursorState & MGCS_OVER_SYMBOL_TITLE) {

        wxMenu* scm = GetSymbolChooserMenu();
        parent.AppendSubMenu(scm,_T("Select symbol"));

    }


    wxMenu * descMenu = new wxMenu();
    wxVariantHashArray arr;
    m_menuID2profID.clear();
    GetDescMgr()->GetMultiGraphDescProfiles(arr);

    for(size_t idx = 0; idx < arr.GetCount(); idx++) {
        wxVariantHash& hash = arr[idx];

        int wid = wxNewId();
        int pid = hash[_T("id")].GetLong();

        descMenu->AppendRadioItem(wid,hash[_T("title")].GetString());

        if( pid == m_profileId ) descMenu->Check(wid,true);

        Connect(wid, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MultiValueGraph::OnPopupMenuSelectDesc));

        m_menuID2profID[wid] = pid;
    }

    parent.AppendSubMenu(descMenu,_T("Select descriptor"));


    arr.Clear();

    m_menuID2themeID.clear();

    GetDescMgr()->GetMultiGraphThemeProfiles(arr);

    wxMenu * themeMenu = new wxMenu();

    for(size_t idx = 0; idx < arr.GetCount(); idx++) {
        wxVariantHash& hash = arr[idx];

        int wid = wxNewId();
        int pid = hash[_T("id")].GetLong();

        themeMenu->AppendRadioItem(wid,hash[_T("title")].GetString());

        if( pid == m_themeId ) themeMenu->Check(wid,true);

        Connect(wid, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MultiValueGraph::OnPopupMenuSelectTheme));

        m_menuID2themeID[wid] = pid;
    }

    parent.AppendSubMenu(themeMenu,_T("Select theme"));



    wxMenu * edit = new wxMenu();

    edit->Append( idPopupmenuOpenDescEditor, _T("Open descriptor editor with the current profile") );
    edit->Append( idPopupmenuOpenNodeDescEditor, _T("Open descriptor editor with the selected node") );
    edit->Append( idPopupmenuOpenSymbolDescEditor, _T("Open descriptor editor with the selected symbol") );

    edit->AppendSeparator();

    edit->Append( idPopupmenuOpenThemeEditor, _T("Open theme editor with the current profile") );

    parent.AppendSubMenu(edit,_T("Editors"));

    if(m_cursorState & MGCS_OVER_SYMBOL_TITLE) {
    parent.AppendSeparator();
        parent.Append(idPopupmenuSearchFor,_T("Search for ")+m_tmphoverSymbol);
    }

    parent.AppendSeparator();

    wxMenu* markerMenu = new wxMenu();


    for(Int2wxVariantHashHash::iterator it = m_markerTypes.begin(); it != m_markerTypes.end(); it++) {
        wxVariantHash& hash = it->second;

        long mid = wxNewId();

        wxString title = hash[_T("title")].GetString();
        bool isShowed = hash[_T("show")].GetBool();

        hash[_T("popupMenuID")] = mid;

        markerMenu->AppendCheckItem(mid,title);
        markerMenu->Check(mid,isShowed);

        Connect(mid, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MultiValueGraph::OnPopupMenuSelectMarkers));
    }

    parent.AppendSubMenu(markerMenu,_T("Markers"));

}

void MultiValueGraph::RefreshSelectionByMousePos(int xpos, bool isStartSelection, bool isRefreshGauges)
{
    GraphDataDescriptor * gdata = NULL;

    gdata = SearchNearlyGraphDataByPosition(xpos);

    if(gdata == NULL) return;

    m_currentSelectedDataIdx = gdata->GetIdx();

    DrawNumValues(gdata->GetRTData());

    DrawSelectArea();

    if(isStartSelection) m_startSelectedDataIdx = m_currentSelectedDataIdx;

    RefreshSelectedTimeInfoBar();

    GaugeRefresh();

    if(m_toolbar->GetValue(_T("refreshgauges")).GetBool() && isRefreshGauges) RefreshHistory();

    DrawSummaryBar();

}

void MultiValueGraph::SetZoomBox()
{
    wxString zs = wxString::Format(_T("%i%%"),m_magnifyRatio*100);

    if( m_resolution > 1 ) zs = wxString::Format(_T("1/%i"),m_resolution);

    m_toolbar->SendCommand(_T("setzoom"),zs);
}

void MultiValueGraph::RefreshSelectedTimeInfoBar()
{
    if( IsSlimData() ) return;
    //Logger::logf(LOG_MSG,_T("m_startSelectedDataIdx=%i"),m_startSelectedDataIdx );

    if( m_startSelectedDataIdx == -1 || m_startSelectedDataIdx == m_currentSelectedDataIdx ) {
        m_statusBarMgr->SetBubbleStateAndText(_T("time_selected"), _T("disabled"), _T("No selection") );
        return;
    }

    int li = GetLeftIdx();
    int ri = GetRightIdx();

    int sidx = (m_startSelectedDataIdx < m_currentSelectedDataIdx ? m_startSelectedDataIdx : m_currentSelectedDataIdx );
    int eidx = (m_startSelectedDataIdx > m_currentSelectedDataIdx ? m_startSelectedDataIdx : m_currentSelectedDataIdx );

    DoubleHash * sdata;
    DoubleHash * edata;

    if( sidx < li || sidx > ri )
        sdata = m_dataMgr->getData(sidx);
    else
        sdata = SearchBGraphDataByIdx(sidx)->GetRTData();

    if( eidx < li || eidx > ri )
        edata = m_dataMgr->getData(eidx);
    else
        edata = SearchBGraphDataByIdx(eidx)->GetRTData();


    if(sdata == NULL) return;
    if(edata == NULL) return;

    if( !CheckData(sdata, _T("time")) ) return;
    if( !CheckData(edata, _T("time")) ) return;


    wxString str = FormatTimeSpan( getTimeMs(sdata), false ) + _T(" -> ");
    str += FormatTimeSpan( getTimeMs(edata), false ) + _T(" = ");
    str += FormatTimeSpan( getTimeMs(edata) - getTimeMs(sdata), false );

    m_statusBarMgr->SetBubbleStateAndText(_T("time_selected"), _T("enabled"), str , false);
}

bool MultiValueGraph::CheckData(DoubleHash * data, const wxString& name)
{
    if( data->find(name) == data->end() ) return false;
    return true;
}

void MultiValueGraph::RefreshGUILayout()
{
    m_graphRect = wxRect(0,GetHeaderSize(), GetClientSize().x, GetClientSize().y - GetFooterSize()-GetHeaderSize() - m_cellsepar );

    m_scrollBar->Move( 0, GetClientSize().y - m_statusBarSize - m_scrollBarSize );
    m_scrollBar->SetSize( GetClientSize().x, m_scrollBarSize );

    m_toolbar->Move( 0, m_captionSize );
    m_toolbar->SetSize( GetClientSize().x, m_toolBarSize );

    m_tbButton->SetPosition(m_helptextlen+20, (m_captionSize - m_tbButton->GetSize().Ceil().y ) / 2 );
}

void MultiValueGraph::SetStatusBarLayout()
{
    m_statusBarMgr->SetBarRect( 0, GetClientSize().y - m_statusBarSize, GetClientSize().x, m_statusBarSize );
    m_statusBarMgr->SetPadding(2);
}

void MultiValueGraph::InitStatusBar()
{
    SetStatusBarLayout();

    BubbleStateDescHash hash;

    hash[_T("red")] = BubbleStateDesc(wxColor(255,255,255,255),wxColor(255,0,0,255),wxColor(255,0,0,255));
    hash[_T("black")] = BubbleStateDesc(wxColor(255,255,255,255),wxColor(0,0,0,255));

    //m_statusBarMgr->AddBubble(_T("lock"), wxALIGN_RIGHT, hash );
    m_statusBarMgr->AddBubble(_T("magnify"), wxALIGN_RIGHT );


    BubbleStateDescHash hash2;

    hash2[_T("disabled")] = BubbleStateDesc(wxColor(255,255,255,255),wxColor(128,128,128,255),wxColor(128,128,128,255));
    hash2 [_T("enabled")] = DefaultBubbleState;

    m_statusBarMgr->AddBubble(_T("time_selected"), wxALIGN_LEFT, hash2);

    if( IsSlimData() ) {
          m_statusBarMgr->SetBubbleStateAndText(_T("time_selected"),_T("disabled"),_T("ERROR"));
          m_statusBarMgr->SetBubbleStateAndText(_T("lock"),_T("red"),_T("ERROR"));
          m_statusBarMgr->SetBubbleText(_T("magnify"),_T("ERROR"));
    }

}

void MultiValueGraph::InitMarkerDesc()
{
    wxVariantHash datas;
    int res = LoadSettingsBase( datas );


    wxVariantHash& hash =  m_markerTypes[LH_MARKER_USER];
    hash[_T("color")] = _T("summarybar_usermarker_color");
    hash[_T("width")] = _T("summarybar_usermarker_width");
    hash[_T("graphwidth")] = _T("summarybar_usermarker_graphwidth");
    hash[_T("graphcolor")] = _T("summarybar_usermarker_graphcolor");
    hash[_T("show")] = res == 0 ? datas[_T("showusermarker")].GetBool() : true;
    hash[_T("title")] = _T("User marker");


    wxVariantHash& hashs =  m_markerTypes[LH_MARKER_STATIC];
    hashs[_T("color")] = _T("summarybar_staticmarker_color");
    hashs[_T("width")] = _T("summarybar_staticmarker_width");
    hashs[_T("graphwidth")] = _T("summarybar_staticmarker_graphwidth");
    hashs[_T("graphcolor")] = _T("summarybar_staticmarker_graphcolor");
    hashs[_T("show")] = res == 0 ? datas[_T("showstaticmarker")].GetBool() : true;;
    hashs[_T("title")] = _T("Static marker");
}

bool MultiValueGraph::IsSlimData()
{
    //if( !m_inited ) return false;

    size_t hc = getHistoryLen();

    return ( hc < m_minDataLimit ? true : false );
}

void MultiValueGraph::InitGauge()
{
LOG
    m_lockedHistoryLen = m_dataMgr->getHistoryCount();

    InitStatusBar();

    DrawBackgroundFaceAgg();
    RefreshGUILayout();

    DrawCursor();


    //m_statusBarMgr->SetBubbleStateAndText(_T("lock"),_T("red"),_T("Status: Locked"),false);

    m_scrollBar->Show(1);
    SetScrollBar();

    DrawTimeRuler();

    Connect( wxEVT_SIZE, wxSizeEventHandler(MultiValueGraph::OnSize) );

    RefreshSelectedTimeInfoBar();

    SetToolBarZoomSelect();

    SetZoomBox();

    m_dataMgr->getMarkers( m_markerList[LH_MARKER_USER] , LH_MARKER_USER );
    m_dataMgr->getMarkers( m_markerList[LH_MARKER_STATIC] , LH_MARKER_STATIC );

    if(IsECUOnline()) {
        m_toolbar->SendCommand(_T("enable_refresh_button"),false);
    }

    InitMarkerDesc();

    m_rtDataMgr->getRequiredSymbols();

    if( IsSlimData() ) return;

    Init();
//    DrawCurrentTime();


    DrawNumValues();

    InitToolTipper();

    DrawMarkerLines();

    DrawSummaryBar();

    m_refreshTimer.Start(m_refreshTimeBase);

    m_inited = true;

    Logger::logf(LOG_MSG,_T("initgauge end"));
}

void MultiValueGraph::InitToolTipper()
{
    MultiGraphTheme* theme = GetTheme();

    wxVariantIntHash props;

    props[ ATT_ARROW_ALIGMENT ] = (long)ATT_AA_BOTTOM;
    props[ ATT_TEXT_ALIGN ] = (long)ATT_TA_LEFT;
    props[ ATT_ROUND_TYPE ] = (long)ATT_RT_ARC;

    props[ ATT_ARROW_REFPOSITION ] = 0.5;
    props[ ATT_BORDER_WIDTH ] = 1.5;
    props[ ATT_TEXT_HEIGHT ] = 8.0;
    props[ ATT_ROUND_RADIUS ] = 3.5;
    props[ ATT_BACKGROUNDCOLOR ] << theme->GetColor(_T("bgcolor"));
    props[ ATT_BORDER_COLOR ] << theme->GetColor(_T("textcolor"));
    props[ ATT_TEXT_COLOR ] << theme->GetColor(_T("textcolor"));

    m_toolTipMgr->SetProperties( props );
}

void MultiValueGraph::SetToolBarZoomSelect()
{

    int maxr = getMaxResolution();

    wxArrayString zarr;

    zarr.Clear();

    //Logger::logf(LOG_MSG,_T("maxr=%i"),maxr);

    for( int a = m_maxMagnify; a > 0; a-- ) {
        zarr.Add( wxString::Format(_T("%i%%"),a*100) );
    }

    for( int a = 2; a <= maxr; a++ ) {
        zarr.Add( wxString::Format(_T("1/%i"),a) );
    }


    m_toolbar->SendCommand(_T("zoomboxinit"),zarr);

}

void MultiValueGraph::Init() /// ok
{

    if( !m_dataMgr ) return;

    if( IsSlimData() ) return;



    //Logger::logf(LOG_MSG,_T("m_currentSelectedDataIdx=%i,m_magnifyRatio=%i,m_resolution=%i"),m_currentSelectedDataIdx,m_magnifyRatio,m_resolution);

    size_t hc = getHistoryLen();

    int sc = getScreenDataCount();

    if( m_resolution != -1 && m_magnifyRatio != -1 ) {

        MagnifyGraphTo(m_magnifyRatio,m_resolution);
        return;
    }


    int m = hc / sc;

//    Logger::logf(LOG_MSG,_T("hc=%i"),hc);

    if( m < 1 ) {
        m = 1;

        m_magnifyRatio = (int)ceil( (double)sc / (double)hc * (double)m_minDataSize );

        chkminmax(m_magnifyRatio,1,m_maxMagnify);

    }

    IntArray arr;

    for(size_t a=0; a<hc; a+=m) {
        arr.Add( a );
    }

    DoubleHashArray dharr;

    m_dataMgr->getHistory( arr, dharr );

    if( dharr.GetCount() < 2 ) return;

    m_resolution = m;

    ClearGraphDataArray();

    DrawGraph(dharr);

    MakeGraphBitmap();

    m_currentSelectedDataIdx = hc/2;

    NormalizeCursor();

    RefreshMagnifyVal();
    SetScrollBar();
}

/** func exec time
    *1,b: 332,1 ms
    *1,a: 232,8 ms
*/

void MultiValueGraph::DrawBackgroundFaceAgg(const wxString& underlinedsymbol) /// ok
{
   // FunctionMeter fm(_T("MultiValueGraph::DrawBackgroundFaceAgg"));

    wxString big_errorstr = _T("");

    if( IsSlimData() ) {

        big_errorstr = _T("Error: not enough data!");
    }

    int w = GetClientSize().x;
    int h = GetClientSize().y;

    int graphHeight = h - GetHeaderSize() - GetFooterSize() - m_cellsepar;

    MultiGraphTheme* theme = GetTheme();

    wxAgg2 aggCanvas(w, h, theme->GetColor(_T("bgcolor")));
    ///*1,a
    //wxAgg2 aggCanvas(w, h);
    ///*1,b

    MultiGraphDesc * desc = GetDesc();


    //aggCanvas.clear( theme->GetColor(_T("bgcolor")) );
    ///*1,b

    aggCanvas.getBrush()->SetStyle(wxAGGBRUSH_SOLID);
    aggCanvas.setBrush( theme->GetColor(_T("fgcolor")) );

    size_t cc = desc->Cells.GetCount();

    double lastTitleWidth = 0.0;
    double lasttitlexpos = 0;

    aggCanvas.loadTTFont(DirHelper::GetPath(dFONTS) + _T("arial.ttf"), m_titleHeight, m_titleStroke );


    for( size_t a = 0; a < cc; a++ ) {

        int cellypos = GetHeaderSize() + a*((graphHeight)/cc);

        aggCanvas.getPen()->SetStyle(wxAGGPEN_TRANSPARENT);
        aggCanvas.drawRectangle( 0, cellypos , w, m_cellsepar);

        int celltextpos = cellypos + 20;

        size_t cac = desc->Cells[a].Symbols.GetCount();

        bool isflag = false;

        aggCanvas.getPen()->SetStyle(wxAGGPEN_SOLID);

        for( size_t b = 0; b < cac; b++ ) {

            wxVariantHash& hash = desc->Cells[a].Symbols[b];

            wxVariantHash& phash = GetPropHash(hash);

            wxColor color = GetSymbolColor(hash);

            aggCanvas.setPen( color );

            wxString str;

            wxString name = hash[_T("name")].GetString();

            if( hash.find(_T("type")) != hash.end() && hash[_T("type")].GetString() == _T("flag")) {
                if( !isflag ) lasttitlexpos += 100;
                str = _T("[") + name + _T("]");
                lasttitlexpos += lastTitleWidth + 10;
                isflag = true;
            } else {
                str = hash.find(_T("customtitle")) == hash.end() ? m_gaugeDescMgr->GetGaugeSymbolTitle( name ) : hash[_T("customtitle")].GetString() ;
                lasttitlexpos = 5 + (b*w/4);
            }

            wxRealRect titlebbox = aggCanvas.drawTTFText(lasttitlexpos,celltextpos,str);

            //aggCanvas.drawText( str, lasttitlexpos, celltextpos,-1, m_titleHeight , m_titleStroke );

            if(phash.find(_T("e_titlewidths")) == phash.end())
                phash[_T("e_titlewidths")] = titlebbox.w; //aggCanvas.getTextWidth(str, m_titleHeight);

            lastTitleWidth = phash[_T("e_titlewidths")].GetDouble();

            if( hash.find(_T("limit_max")) != hash.end() )
                aggCanvas.drawText( hash[_T("limit_max")].GetString() , w - (b+1)*(w/16) , cellypos+m_cellsepar+10,-1, m_limitValHeight, m_limitValStroke );

            if( hash.find(_T("limit_min")) != hash.end() )
                aggCanvas.drawText( hash[_T("limit_min")].GetString() , w - (b+1)*(w/16) , cellypos + (graphHeight/cc) - 15+10,-1, m_limitValHeight, m_limitValStroke );

            m_titleRegions[name] = wxRegion( (int)lasttitlexpos, (int)(celltextpos-(m_titleHeight*1.5)), (int)ceil(lastTitleWidth), (int)ceil(m_titleHeight*1.5) );

            if( name == underlinedsymbol )
                aggCanvas.drawLine( lasttitlexpos , celltextpos+5, lasttitlexpos+ceil(lastTitleWidth),celltextpos+5);

        }

    }


    aggCanvas.getPen()->SetStyle(wxAGGPEN_TRANSPARENT);
    aggCanvas.drawRectangle( 0, h - GetFooterSize() - m_cellsepar , w, m_cellsepar);



    wxString str = _T("Show/hide help: ");

    if(m_hotkeyMgr)
        str += m_hotkeyMgr->GetHotKeyStringBySymbol(_T("MG_HELP"));


    //m_helptextlen = (int)(aggCanvas.getTextWidth(str,hs));

    aggCanvas.getPen()->SetStyle(wxAGGPEN_SOLID);
    aggCanvas.setPen( wxColor(0,0,0,255) );
    //aggCanvas.drawText( str , 5, m_captionSize-((m_captionSize-hs)/2.0), -1, hs, 1.5);

//    Logger::logf(LOG_MSG,_T("m_captionSize=%i"),m_captionSize);

    double hs = ceil(0.45 * m_captionSize);

    aggCanvas.setFontHeight( hs );
    m_helptextlen = aggCanvas.drawTTFText(str, 5, m_captionSize-((m_captionSize-hs)/2.0) ).GetOuterBox().GetWidth();

    if( big_errorstr.Len() > 1 ) {

        double eh = 15;
        aggCanvas.setPen( wxColor(255,0,0,255) );
        double ew = aggCanvas.getTextWidth(big_errorstr,eh);

        aggCanvas.drawText(big_errorstr, (w-ew)/2.0, h/2, -1, eh, 2);
    }


    m_selectedMSymbol = underlinedsymbol;

    wxBitmap * tmpbmp = aggCanvas.renderToBitmap(false); ///*1,a (false)

    ptrdel(m_layerBitmaps[MGBMP_FACE]);
    m_layerBitmaps[MGBMP_FACE] = tmpbmp;

}

void MultiValueGraph::RefreshMagnifyVal()
{
    wxString text = wxString::Format(_T("Magnify: %i%%, Zoom: 1/%i"),m_magnifyRatio*100,m_resolution);

    m_statusBarMgr->SetBubbleText(_T("magnify"),text,false);

}

GraphDataDescriptor* MultiValueGraph::SearchNearlyGraphDataByIdx(int idx)
{
    if( IsNullGraph() ) return NULL;

    int mindelta = getHistoryLen()+1;
    int mindidx = 0;

    GraphDataDescriptor * gdata = m_graphData[0];

    for( size_t b = 0; b< m_graphData.GetCount(); b++ ) {

        GraphDataDescriptor * gdata2 = m_graphData[b];

        int cidx = gdata2->GetIdx();

        if( cidx == idx ) return gdata2;

        if( abs(idx - cidx) < abs(mindelta) ) {
            mindelta = idx - cidx;
            mindidx = cidx;
            gdata = gdata2;
        }

    }
    return gdata;

}

bool MultiValueGraph::IsECUOnline()
{
    return m_dataMgr->getLogFileName() == wxEmptyString ? true : false;
}

bool MultiValueGraph::IsNullGraph()
{
    return m_graphData.GetCount() == 0 ? true : false;
}

GraphDataDescriptor* MultiValueGraph::SearchNearlyGraphDataByPosition(int x_pos)
{
    if( IsNullGraph() ) return NULL;

    GraphDataDescriptor * gdata = m_graphData[0];

    int dist = GetClientSize().x * 2;

    for( size_t a = 0; a<m_graphData.GetCount(); a++){

        int pos_d = abs(m_graphData[a]->GetXPos() - x_pos);

        gdata = m_graphData[a];

        if( pos_d > dist ) {
            if(a>=2)
                return m_graphData[a-2];
            else
                return gdata;
        }

        dist = pos_d;

    }

    return gdata;
}

GraphDataDescriptor* MultiValueGraph::SearchGraphDataByIdx(int idx)
{
    //if( IsSlimData() ) return NULL;
    if( IsNullGraph() ) return NULL;

    int hc = getHistoryLen();
    chkminmax(idx,0,hc-1);

    GraphDataDescriptor * gdata = m_graphData[0];

    for( size_t a = 0; a<m_graphData.GetCount(); a++){

        gdata = m_graphData[a];

        if( gdata->GetIdx() == idx ) break;

    }

    return gdata;

}

GraphDataDescriptor* MultiValueGraph::SearchBNearlyGraphDataByIdx(int idx)
{
    if( IsNullGraph() ) return NULL;

    int N = m_graphData.GetCount();

    int p = 0;
    int L = 0;
    int R = N+1;

    int dist = getHistoryLen();

    int k = -1;
    int local_dist = 0;

    while(1){

        p = (R-L)/2;

        if( !p ) return m_graphData[k-1];

        p = L + p;

        local_dist = abs(m_graphData[p-1]->GetIdx() - idx);

        if( local_dist < dist ) k = p-1;

        dist = local_dist;

        if( m_graphData[p-1]->GetIdx() == idx ) {
            if(p>1)
                return m_graphData[p-2];
            else
                return m_graphData[0];
        }

        if( idx < m_graphData[p-1]->GetIdx() ) {
            R = p;
            continue;
        }

        if( m_graphData[p-1]->GetIdx() < idx ) {
            L = p;
            continue;
        }

    }
}

GraphDataDescriptor* MultiValueGraph::SearchBNearlyGraphDataByPosition(int x_pos)
{
    if( IsNullGraph() ) return NULL;

    int N = m_graphData.GetCount();

    int p = 0;
    int L = 0;
    int R = N+1;

    int dist = GetClientSize().x + 10;

    int k = -1;
    int local_dist = 0;

    while(1){

        p = (R-L)/2;

        if( !p ) {
            if(k > 0)
                return m_graphData[k-1];
            else
                return m_graphData[0];
        }


        p = L + p;

        local_dist = abs(m_graphData[p-1]->GetXPos() - x_pos);

        if( local_dist < dist ) k = p-1;

        dist = local_dist;

        if( m_graphData[p-1]->GetXPos() == x_pos ) {
            if(p > 1)
                return m_graphData[p-2];
            else
                return m_graphData[0];
        }

        if( x_pos < m_graphData[p-1]->GetXPos() ) {
            R = p;
            continue;
        }

        if( m_graphData[p-1]->GetXPos() < x_pos ) {
            L = p;
            continue;
        }

    }

}

GraphDataDescriptor* MultiValueGraph::SearchBGraphDataByIdx(int idx)
{
    if( IsNullGraph() ) return NULL;

    int hc = getHistoryLen();
    chkminmax(idx,0,hc-1);

    int N = m_graphData.GetCount();

    int p = 0;
    int L = 0;
    int R = N+1;

//    Logger::logf(LOG_MSG,_T("bc=%i"),N);

    while(1){

        p = (R-L)/2;

        if( !p )  return m_graphData[0];

        p = L + p;

//        Logger::logf(LOG_MSG,_T("p=%i"),p);

        if( m_graphData[p-1]->GetIdx() == idx ) {
            if( p > 0 )
                return m_graphData[p-1];
            else
                return m_graphData[0];
        }

        if( idx < m_graphData[p-1]->GetIdx() ) {
            R = p;
            continue;
        }

        if( m_graphData[p-1]->GetIdx() < idx ) {
            L = p;
            continue;
        }

    }

    return m_graphData[0];
}


/** sample: 6000

    - step: 2
    - avg height: 4 (from early measurement)
    - symbols: 15

    running time: 457 usec / 1 data

    without wxAgg::renderCopyToImage():     430 usec / 1 data
                - and wxAgg::drawLine():    194 usec / 1 data


*/

GraphDataDescriptor* MultiValueGraph::DrawData( wxAgg& aggCanvas, DoubleHash * currData, DoubleHash * prevData )
{
    if( IsSlimData() ) return NULL;

    MultiGraphDesc * desc = GetDesc();
    MultiGraphTheme * theme = GetTheme();

    int step = getStep();

    int ah = GetClientSize().y;

    int h2 = ah - GetHeaderSize() - GetFooterSize() - m_cellsepar;

    if(h2 < 10) return NULL;

    size_t cc = desc->Cells.GetCount();

    int maxHeight  = 0;

    GraphDataDescriptor * graphdata = new GraphDataDescriptor(currData, (int)((*currData)[strLogIdxPos]) , step );

    for( size_t b = 0; b < cc; b++  ) {

        double cellyposmin = b*(h2/cc) + 1;
        double cellyposmax = (b+1)*(h2/cc) - 1 - m_cellsepar;

        int flagc = 0;
        double flagdisplayratio = desc->Cells[b].Properties[_T("flagdisplayratio")].GetDouble();

        for( size_t c = 0; c < desc->Cells[b].Symbols.GetCount(); c++ ) {

            wxVariantHash &hash = desc->Cells[b].Symbols[c];

            wxVariantHash& phash = GetPropHash(hash);

            wxString name = hash[_T("name")].GetString();

            if( !CheckData(currData, name) ) continue;
            if( !CheckData(prevData, name) ) continue;


            wxString type = hash.find(_T("type")) == hash.end() ? _T("") : hash[_T("type")].GetString();

            double curr_value = (*currData)[name];
            double prev_value = (*prevData)[name];

            wxColor color;

            double curr_x = step;
            double prev_x = 0;

            short offset = 0;

            if( m_symbolOffsets.find(name) != m_symbolOffsets.end() ) offset = m_symbolOffsets[name];

            if( type == _T("flag") ) { /// draw flag value

                int cval = (int)curr_value;
                int pval = (int)prev_value;

                flagc++;

                if( !cval ) continue;

                if( pval == 0 ) prev_x = curr_x;

                color = GetSymbolColor(hash);


                double width = theme->GetDouble(_T("flaglinewidth"));

                //double fy = cellyposmax - ( (cellyposmax - cellyposmin) * hash[_T("flagdisplayratio")].GetDouble() ) - flagc*(width+2);

                double fy = cellyposmax - ( (cellyposmax - cellyposmin) * flagdisplayratio ) - flagc*(width+2);

                int ch = (int)ceil(width+2);


                wxRect clipRect = wxRect(0,0,step,ch);

                //aggCanvas.setClippingRect( clipRect );

                aggCanvas.clear(wxColor(0,0,0,0));

                aggCanvas.setPen( color );
                aggCanvas.drawLine( prev_x, ch/2.0, curr_x, ch/2.0, width, agg::round_cap );

                graphdata->AddImage(name, aggCanvas.renderCopyToImage( clipRect ), (int)fy + offset);

            } else {

                if(phash.find(_T("e_symbolmaxvalue")) == phash.end()) {
                    if( hash.find(_T("gaugedesc_id")) == hash.end() ) {
                        phash[_T("e_symbolmaxvalue")] = m_gaugeDescMgr->GetGaugeDescFieldBySymbolName( name, _T("Hi") ).GetDouble();
                        phash[_T("e_symbolminvalue")] = m_gaugeDescMgr->GetGaugeDescFieldBySymbolName( name, _T("Lo") ).GetDouble();
                    } else {
                        int did = hash[_T("gaugedesc_id")].GetLong();
                        phash[_T("e_symbolmaxvalue")] = m_gaugeDescMgr->GetGaugeDescFieldByAnyField(_T("id"),did,_T("Hi")).GetDouble();
                        phash[_T("e_symbolminvalue")] = m_gaugeDescMgr->GetGaugeDescFieldByAnyField(_T("id"),did,_T("Lo")).GetDouble();
                    }
                }


                double symbolm = 1.0;

                if( hash.find(_T("scale")) != hash.end() ) symbolm = hash[_T("scale")].GetDouble();

                double symbolMinValue = phash[_T("e_symbolminvalue")].GetDouble();
                double symbolMaxValue = phash[_T("e_symbolmaxvalue")].GetDouble();

//                if( name == _T("rpm") ) Logger::logf(LOG_MSG,_T("symbolMaxValue=%.0f"),symbolMaxValue);

//                if( m_symbolMagnifyRatios.find(name) != m_symbolMagnifyRatios.end() )
//                    symbolm = m_symbolMagnifyRatios[name];

                curr_value *= symbolm;
                double prev_value = (*prevData)[name] * symbolm;

                double curr_y = GetSymbolYPos(curr_value, symbolMinValue, symbolMaxValue, cellyposmin, cellyposmax);
                double prev_y = GetSymbolYPos(prev_value, symbolMinValue, symbolMaxValue, cellyposmin, cellyposmax);

                double width = theme->GetDouble(_T("linewidth"));

                wxRealPoint minP,maxP;
                double minV, maxV;
                if( prev_value < curr_value ) {
                    minP.x = prev_x; minP.y = prev_y; /// minP <- point to min value, but this is grater then maxP (position)
                    maxP.x = curr_x; maxP.y = curr_y;
                    minV = prev_value;
                    maxV = curr_value;
                } else {
                    minP.x = curr_x; minP.y = curr_y;
                    maxP.x = prev_x; maxP.y = prev_y;
                    minV = curr_value;
                    maxV = prev_value;
                }


                int sc_top = (int)floor( maxP.y - width ); /// symbolcell
                int sc_bottom = (int)ceil( minP.y + width );

                int img_height = sc_bottom - sc_top + 1;

                wxRect clipRect(0,0, step, img_height);

                //aggCanvas.setClippingRect( clipRect );

                aggCanvas.clear(wxColor(0,0,0,0));

              //  aggCanvas.clear(wxColor(0,0,0,0));


//                m_dataHeights.Add( ceil(fabs(prev_y - curr_y) + 1) );
//                m_cellHeight = fabs( cellyposmin - cellyposmax );

                wxRealPoint startP = minP, endP = maxP;
                double      startV = minV, endV = maxV;

                color = GetOverColor(hash);

                if(hash.find(_T("limit_min")) != hash.end()) {
                    double limit_min = hash[_T("limit_min")].GetDouble() * symbolm;
                    if( startV < limit_min ) {
                        aggCanvas.setPen( color );

                        wxRealPoint limit_minP;

                        bool isEq = false;
                        if(maxV < limit_min) {
                            limit_minP = maxP;
                            isEq = true;
                        } else {
                            limit_minP.y = GetSymbolYPos(limit_min, symbolMinValue, symbolMaxValue, cellyposmin, cellyposmax);
                            limit_minP.x = Math::GetPointOnLine(minP,maxP,limit_minP.y);
                        }

                        aggCanvas.drawLine( GetRelativePos(sc_top,startP), GetRelativePos(sc_top,limit_minP), width, agg::square_cap);

                        //if( isEq ) continue;

                        startP = limit_minP;
                        startV = limit_min;

                    }
                }

                if(hash.find(_T("limit_max")) != hash.end()) {
                    double limit_max = hash[_T("limit_max")].GetDouble() * symbolm;
                    if(maxV > limit_max) {
                        wxRealPoint limit_maxP;
                        aggCanvas.setPen( color );

                        bool isEq = false;
                        if( minV > limit_max ) {
                            limit_maxP = minP;
                            isEq = true;
                        } else {
                            limit_maxP.y = GetSymbolYPos(limit_max, symbolMinValue, symbolMaxValue, cellyposmin, cellyposmax);
                            limit_maxP.x = Math::GetPointOnLine(minP,maxP,limit_maxP.y);
                        }

                        aggCanvas.drawLine( GetRelativePos(sc_top,limit_maxP), GetRelativePos(sc_top,maxP), width, agg::square_cap);

                        //if( isEq ) continue;

                        endP = limit_maxP;
                        endV = limit_max;
                    }
                }

                color = GetSymbolColor(hash);
                aggCanvas.setPen( color );
                aggCanvas.drawLine( GetRelativePos(sc_top,startP) , GetRelativePos(sc_top,endP), width, agg::square_cap);

                wxImage *img = aggCanvas.renderCopyToImage( clipRect );


      //          Logger::logf(LOG_MSG,_T("imgh=%i"),img_height);

                maxHeight = wxMax(maxHeight,img_height);

                graphdata->AddImage(name,img,sc_top+m_cellsepar+offset);

            }

        }

    }

    return graphdata;

}

wxVariantHash& MultiValueGraph::GetPropHash(wxVariantHash& descHash)
{
    int sItemId = descHash[_T("id")].GetLong();
    return m_symbolTmpProps[sItemId];
}

wxRealPoint MultiValueGraph::GetRelativePos(int top, const wxRealPoint& pos)
{
    wxRealPoint p = pos;
    p.y = pos.y - (double)top;
    return p;
}

inline double MultiValueGraph::GetRelativePos(int top, double value)
{
    return value - (double)top;
}

inline double MultiValueGraph::GetSymbolYPos(double value, double minV, double maxV, double cellMin, double cellMax)
{
    double percent = (value - minV) / (maxV - minV);
    chkminmax(percent,0,1);
    double y = cellMax - (( cellMax - cellMin ) * percent );
    return y;
}

void MultiValueGraph::DrawCursor()
{
   // if( IsSlimData() ) return;

    int w = (int)m_cursorWidth;
    int h = GetClientSize().y - GetHeaderSize() - GetFooterSize();

    wxAgg aggCanvas(w, h);
    aggCanvas.clear(wxColor(0,0,0,0));

    aggCanvas.setPen( GetTheme()->GetColor(_T("cursorcolor")) );
    aggCanvas.drawLine( w/2, 0, w/2, h, w );

    if(m_layerBitmaps[MGBMP_CURSOR])
        ptrdel(m_layerBitmaps[MGBMP_CURSOR]);

    m_layerBitmaps[MGBMP_CURSOR] = aggCanvas.renderToBitmap();
}



void MultiValueGraph::DrawNumValues(DoubleHash* data, bool repos, bool forcepaint) /// ok
{
    if( IsSlimData() ) return;

    if(data == NULL) return;

    int w = GetClientSize().x;
    int h = GetClientSize().y;
    int h2 = h - GetHeaderSize() - GetFooterSize() - m_cellsepar;

    MultiGraphDesc * desc = GetDesc();

    size_t cc = desc->Cells.GetCount();

    wxAgg2 aggCanvas(w/2,(int)ceil(m_numValueHeight*2));

    aggCanvas.loadTTFont(DirHelper::GetPath(dFONTS) + _T("arial.ttf"), m_numValueHeight, m_numValueStroke );

    wxAggPen* pen = aggCanvas.getPen();

    for( size_t a = 0; a < cc; a++ ) {

        int celltextpos = a*(h2/cc) + 8 + GetHeaderSize();

        size_t cac = desc->Cells[a].Symbols.GetCount();

        for( size_t b = 0; b < cac; b++ ) {

            wxVariantHash &hash = desc->Cells[a].Symbols[b];

            wxVariantHash& phash = GetPropHash(hash);

            if( hash.find(_T("type")) != hash.end() && hash[_T("type")].GetString() == _T("flag"))
                continue;

            wxString name = hash[_T("name")].GetString();

            if( !CheckData(data, name) ) continue;


            if(m_numBoxes.find(name) == m_numBoxes.end())
                m_numBoxes[name] = new NumValueBoxes();


            wxRealPoint pos( 5 + (b*w/4) + phash[_T("e_titlewidths")].GetDouble() + 2, celltextpos );

            m_numBoxes[name]->Position = pos;

            if(repos && !forcepaint) continue;

            if(phash.find(_T("e_numformat")) == phash.end())
            {
                if(hash.find(_T("gaugedesc_id")) == hash.end()) {
                    phash[_T("e_numformat")] = m_gaugeDescMgr->GetGaugeDescFieldBySymbolName( name, _T("numformat") );
                } else {
                    int did = hash[_T("gaugedesc_id")].GetLong();
                    phash[_T("e_numformat")] = m_gaugeDescMgr->GetGaugeDescFieldByAnyField(_T("id"),did,_T("numformat"));
                }
            }

            int did = -1;

            if(phash.find(_T("e_symbolunit")) == hash.end()) {
                if(hash.find(_T("gaugedesc_id")) == hash.end()) {
                    phash[_T("e_symbolunit")] = m_gaugeDescMgr->GetGaugeDescFieldBySymbolName( name, _T("unit") );
                } else {
                    did = hash[_T("gaugedesc_id")].GetLong();
                    phash[_T("e_symbolunit")] = m_gaugeDescMgr->GetGaugeDescFieldByAnyField(_T("id"),did,_T("unit"));
                }
            }

            wxString unit = phash[_T("e_symbolunit")].GetString();

            //Logger::logf(LOG_MSG,_T("name=")+name+wxString::Format(_T(", did=%i"),did )+_T(", unit=")+unit );

            wxString value = _T("(") + wxString::Format( phash[_T("e_numformat")].GetString() ,(*(data))[name]);
            value +=  _T(" ") + unit + _T(")");

//            if( m_symbolMagnifyRatios.find(name) != m_symbolMagnifyRatios.end() && fabs(1.0-m_symbolMagnifyRatios[name]) > 0.01 )
//                value += _T("x") + wxString::Format(_T("%0.1f"),m_symbolMagnifyRatios[name]);

            if( hash.find(_T("scale")) != hash.end() && fabs(1.0-hash[_T("scale")].GetDouble()) > 0.01 )
                value += _T("x") + wxString::Format(_T("%0.1f"),hash[_T("scale")].GetDouble());

            if(m_symbolOffsets.find(name) != m_symbolOffsets.end() && m_symbolOffsets[name] != 0) {
                int offset = m_symbolOffsets[name];
                value += offset < 0 ? _T("+") : _T("-");
                offset = abs(offset);
                value += wxString::Format(_T("%ipx"),offset);
            }

            wxColor color = GetSymbolColor(hash);

            //if(hash.find(_T("e_valuewidth")) == hash.end()) hash[_T("e_valuewidth")] = wxAgg::getTextWidth( value, m_numValueHeight );

            if( m_numBoxes[name]->IsChangedValue( (*(data))[name] ) || forcepaint ) {

                pen->SetColor( color );

                aggCanvas.clear(wxColor(255,255,255,180));

                wxRealRect bbox = aggCanvas.drawTTFText(0,m_numValueHeight*1.8,value);

                bbox.Offset(0,-m_numValueHeight*0.8);

                wxBitmap* bmp = aggCanvas.renderToBitmap(true, bbox.GetOuterBox() );

                m_numBoxes[name]->SetBitmap( bmp );
            }

        }

    }


    DrawCurrentTime(data);
}

void MultiValueGraph::DrawNumValues(bool repos, bool forcepaint)
{
    if( IsSlimData() ) return;

    DoubleHash * dh = NULL;

    for( size_t a=0; a < m_graphData.GetCount(); a++ ){
        if( m_graphData[a]->GetIdx() == m_currentSelectedDataIdx ) {
            dh = m_graphData[a]->GetRTData();
            break;
        }
    }

    if(dh) {
        DrawNumValues(dh,repos,forcepaint);
    } else {

        DoubleHashArray datas;
        m_dataMgr->getHistory(m_currentSelectedDataIdx,m_currentSelectedDataIdx,datas);

        if( datas.GetCount() < 1 ) return;
        DrawNumValues( datas[0], repos, forcepaint);
    }

}

void MultiValueGraph::DrawCurrentTime(DoubleHash * data)
{
    if( IsSlimData() ) return;

    if( !CheckData(data,_T("time")) ) return;

    if( !data ) return;

    int ctime = getTimeMs(data);

    wxString str = _T("Time: ") + FormatTimeSpan(ctime, false);

#if defined ( VEMSDEBUG )
    str += wxString::Format(_T(", idx=%i"),m_currentSelectedDataIdx);
#endif


    double text_height = 15;

    wxAgg2 dummyCanvas(5,5);
    dummyCanvas.loadTTFont(DirHelper::GetPath(dFONTS) + _T("courbd.ttf"), text_height, 0 );

    wxRealRect bbox = dummyCanvas.getTTFTextBound(str);

    wxAgg2 aggCanvas( bbox.GetSize().Ceil() );

    aggCanvas.setPen( GetTheme()->GetColor(_T("textcolor")) );
    aggCanvas.loadTTFont(DirHelper::GetPath(dFONTS) + _T("courbd.ttf"), text_height, 0 );

    aggCanvas.drawTTFText(str,0,floor(bbox.GetBottom()*0.9),0);

    //aggCanvas.renderToFile(_T("teve.png"));

    ptrdel(m_layerBitmaps[MGBMP_CURRENT_TIME]);

    m_layerBitmaps[MGBMP_CURRENT_TIME] = aggCanvas.renderToBitmap();
}

wxString MultiValueGraph::FormatTimeSpan(int time_millisec, bool isminsize)
{
    int minutes = time_millisec / 1000 / 60;
    int seconds = time_millisec / 1000 - minutes * 60;
    int millisecs = time_millisec - minutes*60*1000 - seconds * 1000;

    wxString secformat = _T("%0");
    secformat += minutes > 0 ? _T("2") : _T("1");
    secformat += _T("i");

    wxString str = (minutes > 0 || !isminsize) ? wxString::Format(_T("%i:"),minutes) : _T("");
    str += wxString::Format( secformat, seconds);
    str += (millisecs > 0 || !isminsize) ? wxString::Format(_T(".%03i"),millisecs) : _T("");

    return str;
}

int MultiValueGraph::getTimeMs(DoubleHash * data)
{
    if(data==NULL) return 0;
    if( !CheckData(data,_T("time")) ) return 0;

    return (int)((*data)[_T("time")] * 1000.0);
}

wxColor MultiValueGraph::GetOverColor(wxVariantHash &hash)
{
    wxColor c;
    if( hash.find(_T("overlimitcolor")) == hash.end() ){
        c << hash[_T("color")];
        return DrawHelper::InvertColor(c);
    } else {
        c << hash[_T("overlimitcolor")];
        return c;
    }
}

wxColor MultiValueGraph::GetSymbolColor(wxVariantHash &hash)
{
    wxColor c(0,0,0);

    if(hash.find(_T("color")) != hash.end())
        c << hash[_T("color")];

    return c;
}

wxString MultiValueGraph::GetHoverSymbolByPos(const wxPoint& pos)
{
    for(wxRegionHash::iterator it = m_titleRegions.begin(); it != m_titleRegions.end(); it++) {
        if( it->second.Contains(pos) == wxInRegion ) {
            return it->first;
        }
    }
    return wxEmptyString;
}

void MultiValueGraph::DrawTimeRuler()
{
    //if( IsSlimData() ) return;

    int bc = m_graphData.GetCount();

    if( bc < 2 ) return;

    MultiGraphTheme* theme = GetTheme();

    int step = getStep();

    int canvas_width = GetClientSize().x;
    int canvas_height = m_timeRulerSize;
    int graph_height = m_graphRect.GetHeight();

//if needs the vertical lines, calc height for this and enlarge the canvas_height
    if( step >= m_dataWidthLimitVert ) {
        wxString vlhstr = theme->GetString(_T("verticallines_height"),_T("10"));

        wxString v = vlhstr.BeforeFirst('%');

        long val = 0;

        if(v.ToLong(&val) == false){

            Logger::logf(LOG_MSG,_T("wrong verticallines_height param"));

            UserLog::uLog(ULOG_WARNING, _T("MultiValueGraph config error</font><br>Wrong value for 'verticallines_height' parameter<font>") );

        } else {
            if(v == vlhstr) {

                canvas_height += val;

            } else {

                chkminmax(val,1,100);

                canvas_height += (int)((double)graph_height * (double)val / 100.0);

            }
        }

    }


    wxAgg2 aggCanvas(canvas_width,canvas_height);


    int majordiv = theme->GetInt(_T("timeruler_majordiv"),5);
//    int minordiv = desc->GetInt(_T("timeruler_minordiv"),1);


//the left and the right side of the graph which is shown
    GraphDataDescriptor * left_gdata = m_graphData[0];
    GraphDataDescriptor * right_gdata = m_graphData[bc-1];

    int left_time_ms = getTimeMs(left_gdata->GetRTData());
    int right_time_ms = getTimeMs(right_gdata->GetRTData());

    int delta_time_ms = right_time_ms - left_time_ms;

    int biggest = 0;

    int a;

    int delta_div = delta_time_ms / majordiv;

    if( delta_div == 0 ) return;

//calc the time step between two stick
    for( a = 1 ; 1 ; a *= 10 )  {
        int t = delta_div / a;
        if( t == 0 ) {
            biggest = (int)floor(((double)delta_div / (double)(a/10)) + 0.5);
            break;
        }
        biggest = t;
    }

    biggest *= (a/10);

    int start_time = 0;

    //Logger::logf(LOG_MSG,_T("biggest=%i"),biggest);

//calc the time of the first stick
    for( start_time = left_time_ms; start_time <= right_time_ms; start_time++ ) {
        if( start_time % biggest == 0 ) break;
    }

    aggCanvas.setPen(wxColor(0,0,0,255));

    double text_height = 11;

    aggCanvas.loadTTFont(DirHelper::GetPath(dFONTS) + _T("arial.ttf"), text_height, 0 );

    double x_offset = 0;

//if the left side of the graph is less then 0 calc an offset
    if(m_graphData.GetCount())
        x_offset = m_graphData[0]->GetXPos();

    //Logger::logf(LOG_MSG,_T("x_offset=%f"),x_offset);

    aggCanvas.getPen()->SetWidth(1);

    //Logger::logf(LOG_MSG,_T("left_time_ms=%i, start_time=%i, x_offset=%f"),left_time_ms,start_time,x_offset);

    if(start_time == biggest && x_offset > 0) start_time = 0;

    for( int ctime = start_time; ctime < right_time_ms; ctime += biggest ) {

        wxString str = FormatTimeSpan(ctime);

        wxRealRect rect = aggCanvas.getTTFTextBound(str);

        int time_dist = right_time_ms+1000;

        int idx;

        for( idx=0; idx<bc; idx++ ) {
            int atime = getTimeMs(m_graphData[idx]->GetRTData());

            int d = abs(ctime-atime);

            if( d < time_dist )
                time_dist = d;
            else
                break;

        }

        //double x = (double)(ctime - left_time_ms) / (double)delta_time_ms * (double)w;

        double x = (double)(canvas_width-x_offset) - (((double)(bc - idx) / (double)bc) * (double)(canvas_width-x_offset));

        x += x_offset;

        aggCanvas.drawLine( rastu(x) ,text_height*1.2, rastu(x),m_timeRulerSize);
        x -= rect.GetWidth()/2.0;
        x = (x + rect.GetWidth()) > canvas_width ? canvas_width - rect.GetWidth() : x;
        x = x < 0 ? 1 : x;

        aggCanvas.drawTTFText(str,x,text_height);
    }

//draw a short line to every data if needed ("vertical lines")
    if( step >= m_dataWidthLimitVert ) {

        //CalculateGraphDataPositions();

        wxColor defc = wxColor(128,128,128);

        aggCanvas.setPen( theme->GetColor(_T("verticallines_color"),defc) );

        for(size_t a=0; a < m_graphData.GetCount();a++ ) {

            int xpos = m_graphData[a]->GetXPos();

            if(xpos < 0 || xpos > canvas_width) continue;

            double dxpos = (double)xpos + 0.5;

            aggCanvas.drawLine(dxpos,m_timeRulerSize,dxpos,canvas_height);

        }

    }

    ptrdel(m_layerBitmaps[MGBMP_TIMERULER]);

    m_layerBitmaps[MGBMP_TIMERULER] = aggCanvas.renderToBitmap();

}

void MultiValueGraph::DrawHelpPanel()
{

    m_helpPanel->SetProperty(BCBS_BORDER_RADIUS, 3.0);
    m_helpPanel->SetProperty(BCBS_BORDER_WIDTH, 2.0);

    double text_height = 13;

    HotKeyDescArray arr;

    m_hotkeyMgr->GetHotKeyList(arr);

    wxString titleStr = wxEmptyString, hkeyStr = wxEmptyString;

    for(size_t a = 0; a < arr.GetCount(); a++) {
        titleStr += arr[a].Title + _T("\n");
        hkeyStr  += arr[a].HotKey + _T("\n");
    }

    hkeyStr.RemoveLast();
    titleStr.RemoveLast();

    m_helpPanel->ChangeLabel(TextDesc(titleStr,0,wxALIGN_LEFT,text_height));

    wxRect titlesBox = m_helpPanel->GetTextBox(0);

    m_helpPanel->ChangeLabel(TextDesc(hkeyStr,1,wxALIGN_LEFT,text_height, wxRealPoint(titlesBox.width+5,0) ));

    wxRect hkeysBox  = m_helpPanel->GetTextBox(1);

    titlesBox.width += 5 + hkeysBox.width;
    titlesBox.height -= 5;

    wxSize panelsize = titlesBox.GetSize();
    panelsize.IncBy(10);
    m_helpPanel->SetSize( panelsize );

    wxSize csize = GetClientSize();

//    wxBitmap * bmp = m_helpPanel->GetBitmap();

//    bmp->SaveFile(_T("teve.png"),wxBITMAP_TYPE_PNG);

    m_helpPanel->Centre();

}

int MultiValueGraph::GetLeftIdx()
{
    if( m_graphData.GetCount() < 1 ) return -1;

    return m_graphData[0]->GetIdx();
}

int MultiValueGraph::GetRightIdx()
{
    int bc = m_graphData.GetCount();

    if( bc < 1 ) return -1;

    return m_graphData[bc-1]->GetIdx();
}

int MultiValueGraph::GetDataLen()
{
    return (GetRightIdx()-GetLeftIdx())+m_resolution;
}

void MultiValueGraph::SetScrollBar()   /// ok
{
    int hc = getHistoryLen();

    int li = GetLeftIdx();
    int ri = GetRightIdx();

    int dw = ri - li;

    m_scrollBar->SetScrollbar( li, dw, hc, dw);

}

void MultiValueGraph::KlappeToolbar()
{

    m_toolbar->Show( !(m_toolbar->IsShown()) );

    DrawBackgroundFaceAgg();
    DrawCursor();
    DrawMarkerLines();
    m_graphRect = wxRect(0,GetHeaderSize(), GetClientSize().x, GetClientSize().y - GetFooterSize()-GetHeaderSize()-m_cellsepar );
    DrawSelectArea();
    RedrawGraph();
    DrawNumValues(true,true);
    MakeGraphBitmap();

    UpdateToolbarButton();
}

void MultiValueGraph::UpdateToolbarButton()
{
    m_tbButton->SetLabel( m_toolbar->IsShown() ? _T("Close toolbar") : _T("Open toolbar") );
}

wxString MultiValueGraph::SearchPrevSymbol()
{
    MultiGraphDesc * desc = GetDesc();

    int cell_count = desc->Cells.GetCount();

    if(cell_count == 0) return wxEmptyString;

    if(m_selectedMSymbol == wxEmptyString) return desc->GetLastSymbol();

    bool match = false;

    for( int b = cell_count-1; b >= 0; b-- ) {

        if(desc->Cells[b].Symbols.GetCount() < 1) continue;

        for( int c = desc->Cells[b].Symbols.GetCount()-1; c >= 0 ; c-- ) {

            wxVariantHash &hash = desc->Cells[b].Symbols[c];

            wxString name = hash[_T("name")].GetString();

            if( match ) return name;

            if(name == m_selectedMSymbol) match = true;
        }
    }

    return desc->GetLastSymbol();
}

wxString MultiValueGraph::SearchNextSymbol()
{
    MultiGraphDesc * desc = GetDesc();

    size_t cell_count = desc->Cells.GetCount();

    if(cell_count == 0) return wxEmptyString;

    if(m_selectedMSymbol == wxEmptyString) return desc->GetFirstSymbol();

    bool match = false;

    for( size_t b = 0; b < cell_count; b++  ) {

        for( size_t c = 0; c < desc->Cells[b].Symbols.GetCount(); c++ ) {

            wxVariantHash &hash = desc->Cells[b].Symbols[c];

            wxString name = hash[_T("name")].GetString();

            if( match ) return name;

            if(name == m_selectedMSymbol) match = true;
        }
    }

    return desc->GetFirstSymbol();
}

wxVariantHash* MultiValueGraph::SearchSymbolByName(const wxString& name)
{
    MultiGraphDesc * desc = GetDesc();

    for( size_t b = 0; b < desc->Cells.GetCount(); b++  ) {
        for( size_t c = 0; c < desc->Cells[b].Symbols.GetCount(); c++ ) {

            wxVariantHash &hash = desc->Cells[b].Symbols[c];

            if( hash[_T("name")].GetString() == name ) return &hash;
        }
    }

    return NULL;
}

void MultiValueGraph::TabToPrevSymbol()
{
    wxString symbol = SearchPrevSymbol();

    if(symbol == wxEmptyString) return;

    DrawBackgroundFaceAgg(symbol);
    GaugeRefresh();
}

void MultiValueGraph::TabToNextSymbol()
{
    wxString symbol = SearchNextSymbol();

    if(symbol == wxEmptyString) return;

    DrawBackgroundFaceAgg(symbol);
    GaugeRefresh();
}

void MultiValueGraph::JumpCursorToMarker(bool isNext)
{

    IntArray arr;

    for( Int2wxVariantHashHash::iterator it = m_markerTypes.begin(); it != m_markerTypes.end(); it++ ) {

        if( m_markerList.find(it->first) == m_markerList.end() ) continue;

        if( it->second[_T("show")].GetBool() == false ) continue;

        WX_APPEND_ARRAY(arr, m_markerList[it->first]);
    }

    size_t c = arr.GetCount();

    if( c < 1 ) return;

    arr.Sort( MultiValueGraph::CompareInts );

    size_t jumptoidx = isNext ? arr[0] : arr[c-1];

    if( isNext ) {

        for( size_t a = 0; a < c; a++ ) {
            int midx = arr[a];
            if(m_currentSelectedDataIdx < midx) {
                jumptoidx = midx;
                break;
            }
        }
    } else {

        for( int a = c-1; a >= 0; a-- ) {
            int midx = arr[a];
            if(m_currentSelectedDataIdx > midx) {
                jumptoidx = midx;
                break;
            }
        }
    }

    m_startSelectedDataIdx = -1;

    MoveCursorTo(jumptoidx);
}

void MultiValueGraph::MoveCursorTo(int idx) /// ok
{
    if( IsSlimData() ) return;

    int li = GetLeftIdx();
    int ri = GetRightIdx();

    int dw = getScreenDataMaxLen();

    int hc = getHistoryLen();

    int delta  = idx - m_currentSelectedDataIdx;

    int nli = li + delta;
    int nri = ri + delta;

    //Logger::logf(LOG_MSG,_T("target idx=%i"),idx);

    m_currentSelectedDataIdx = idx;

    if( nri > hc - 1 ) {
        nri = hc - 1;
        nli = nri - dw;
    }

    if( nli < m_resolution ) {
        nli = m_resolution;
        nri = nli + dw;
    }

    chkminmax(m_currentSelectedDataIdx,nli,nri);


    DoubleHashArray datas;

   // Logger::logf(LOG_MSG,_T("nli=%i,nri=%i, m_currentSelectedDataIdx=%i"),nli,nri,m_currentSelectedDataIdx);

    m_dataMgr->getHistory( GetNeededIdxs(nli,nri) , datas );

    ClearGraphDataArray();

    DrawGraph(datas);

    MakeGraphBitmap();

  //  Logger::logf(LOG_MSG,_T("datas c=%i"),datas.GetCount());


    GraphDataDescriptor * gdata = NormalizeCursor();

    if(gdata) {
        DoubleHash* curridxdata = gdata->GetRTData();
        DrawNumValues(curridxdata,false,true);
    }

    SetScrollBar();
    DrawTimeRuler();


    DrawSummaryBar();

    m_isCursorMoved = true;

//    Logger::logf(LOG_MSG,_T(" m_currentSelectedDataIdx=%i"),nli,nri,m_currentSelectedDataIdx);

}

bool MultiValueGraph::MoveScreenTo(int scrollpos)
{
    //FUNCTIME
    if( IsSlimData() ) return false;

    int hc = getHistoryLen();
    int ori = GetRightIdx();
    int oli = GetLeftIdx();

    int dw = ori - oli;

    int nli = scrollpos;

    if( oli == nli ) return false;

    if( abs(nli-oli) < m_resolution ) {
        nli = (nli-oli) < 0 ? oli - m_resolution : oli + m_resolution;
    }

    chkminmax(nli,m_resolution,hc-1-dw);

    int nri = nli + dw;

    DoubleHashArray datas;

    m_dataMgr->getHistory(  GetNeededIdxs(nli,nri), datas );

    ClearGraphDataArray();

    DrawGraph( datas );

    MakeGraphBitmap();

    DrawTimeRuler();

    SetScrollBar();

    //m_currentSelectedDataIdx += dw;

    return true;
}

void MultiValueGraph::MoveCursor(int delta, bool isSelect)
{
    if( IsSlimData() ) return;

   // FunctionMeter fm(_T("MoveCursor"));



    delta *= m_resolution;

    int oldidx = m_currentSelectedDataIdx;
    int bc = m_graphData.GetCount();

    int dc = getHistoryLen();
    int hc = dc;

    int li = GetLeftIdx();
    int ri = GetRightIdx();

    int dw = getScreenDataMaxLen();

    int limiter = (int)(dw * 0.05);
    int newdata = limiter*4;

    bool isOuterScreen = ( (m_currentSelectedDataIdx < li && m_currentSelectedDataIdx > m_resolution) ||
                           (m_currentSelectedDataIdx > ri && m_currentSelectedDataIdx < hc-1) ) ? true : false;

    m_currentSelectedDataIdx += delta;

    bool isFullLeft =  ri <= dw + m_resolution ? true : false;
    bool isFullRight =  ri >= dc - m_resolution ? true : false;

    bool isInLeftBorderR = m_currentSelectedDataIdx < li + limiter ? true : false;
    bool isInRightBorderR = m_currentSelectedDataIdx > ri - limiter ? true : false;

    bool isUnderSelected = m_currentSelectedDataIdx <= m_resolution ? true : false;
    bool isOverSelected = m_currentSelectedDataIdx > hc ? true : false;

    bool isChangedGraph = false;

    int nli = 0;
    int nri = 0;


//    Logger::logf(LOG_MSG,_T("isOuterScreen=%i"),isOuterScreen);
//    Logger::logf(LOG_MSG,_T("isInRightBorderR=%i"),isInRightBorderR);
//    Logger::logf(LOG_MSG,_T("isInLeftBorderR=%i"),isInLeftBorderR);
//
//    Logger::logf(LOG_MSG,_T("isOverSelected=%i"),isOverSelected);
//    Logger::logf(LOG_MSG,_T("isUnderSelected=%i"),isUnderSelected);
//
//    Logger::logf(LOG_MSG,_T("isFullRight=%i"),isFullRight);
//    Logger::logf(LOG_MSG,_T("isFullLeft=%i"),isFullLeft);
//

    //Logger::logf(LOG_MSG,_T("m_currentSelectedDataIdx=%i"),m_currentSelectedDataIdx);

    if( isOuterScreen ) {

    } else if( isUnderSelected ){

        m_currentSelectedDataIdx = m_resolution;

    } else if( isOverSelected  ){

        m_currentSelectedDataIdx = ri-m_resolution;
    }


    if( isOuterScreen ) {

        int hdw = dw/2;

        chkminmax(m_currentSelectedDataIdx,hdw,hc-1-hdw);

        NormalizeCursor();

        nli = m_currentSelectedDataIdx - hdw;
        nri = nli + dw;

        chkminmax(nli,m_resolution,hc-1);
        chkminmax(nri,nli,hc-1);

        DoubleHashArray datas;

        m_dataMgr->getHistory( GetNeededIdxs(nli,nri), datas );

        ClearGraphDataArray();

        DrawGraph( datas );

        isChangedGraph = true;

    } else if( isFullLeft && !isInRightBorderR ){

        // do nothing

    } else  if( isFullRight && !isInLeftBorderR ){

        // do nothing

    } else if( isInLeftBorderR ) {



        nri = ri - newdata;
        m_currentSelectedDataIdx = oldidx+delta;

        if( nri < dw )  {
            newdata = ri - dw - 1;
            nri = dw + 1;
        }

        nli = nri - dw;

        if( newdata < m_resolution ) return;

        chkminmax(nli,m_resolution,hc-1);
        chkminmax(nri,nli,hc-1);

        chkminmax(m_currentSelectedDataIdx,nli,nri);

        DoubleHashArray datas;

        m_dataMgr->getHistory( GetNeededIdxs(nli,li-m_resolution), datas );

        GraphDataDescriptorArray gdataarr;

        DrawGraph( datas, &gdataarr );

        for( int t = bc-1; 1 ; t-- ) {

            if( m_graphData[t]->GetIdx() > nri ) {
                ptrdel(m_graphData[t]);
                m_graphData.RemoveAt(t);
            } else
                break;

        }

        WX_PREPEND_ARRAY( m_graphData, gdataarr );

        isChangedGraph = true;

    } else if( isInRightBorderR ) {

        nri = ri + newdata;
        m_currentSelectedDataIdx = oldidx+delta;

        if( nri >= hc )  {
            newdata = hc - 1 - ri;
            nri = hc - 1;
        }

        nli = nri - dw;

        if( newdata < m_resolution ) return;

        chkminmax(nli,m_resolution,hc-1);
        chkminmax(nri,nli,hc-1);

        chkminmax(m_currentSelectedDataIdx,nli,nri);

        DoubleHashArray datas;

        m_dataMgr->getHistory( GetNeededIdxs(ri+m_resolution,nri), datas );

        GraphDataDescriptorArray gdataarr;

        DrawGraph( datas, &gdataarr );

        for( int t = bc-1; t >= 0 ; t-- ) {

            if( m_graphData[t]->GetIdx() < nli ) {
                ptrdel(m_graphData[t]);
                m_graphData.RemoveAt(t);
            }


        }

        WX_APPEND_ARRAY( m_graphData, gdataarr );

        isChangedGraph = true;

    }

    //int new_bc = m_graphData.GetCount();

    CalculateGraphDataPositions();

    RemoveUnneededGraphData();

    NormalizeCursor();

    if( !isSelect )
        m_startSelectedDataIdx = -1;


//    Logger::logf(LOG_MSG,_T("m_startSelectedDataIdx=%i, m_currentSelectedDataIdx=%i, isSelect=%i"),m_startSelectedDataIdx,m_currentSelectedDataIdx,isSelect);

    m_isCursorMoved = true;



    DrawSelectArea();
    RefreshSelectedTimeInfoBar();
    DrawNumValues();
    SetScrollBar();


    DrawSummaryBar();

    //int t = wxGetElapsedTime();


    if(isChangedGraph) {
        DrawTimeRuler();
        MakeGraphBitmap(false);
    }


   // Logger::logf(LOG_MSG,_T("MoveCursor=%i ms"),t);




}

IntArray MultiValueGraph::GetNeededIdxs(int from, int to)
{
    IntArray reqarr;

    if( IsSlimData() ) return reqarr;

    if( m_resolution < 1 ) m_resolution = 1;

    for(int a = from - m_resolution; a <= to ; a += m_resolution )
        reqarr.Add( a );


    return reqarr;
}

void MultiValueGraph::RefreshGraph()
{
    if( IsSlimData() ) return;

    wxSize currsize = GetClientSize();

    int li = GetLeftIdx();
    int ri = GetRightIdx();
    int hc = getHistoryLen();

    int dc = (int)ceil( ((double)getScreenDataCount() * (double)m_resolution) );

    int dl = GetDataLen();

    DoubleHashArray dh;

    GraphDataDescriptorArray gdataarr;

    if( dc > dl ) {

        int delta = dc - dl;

        int nli = ri - dc;

        if( nli < m_resolution ) {
            nli = m_resolution;
        }

        chkminmax(nli,m_resolution,hc-1);

        if( li - m_resolution - nli <= 0 ) {

            int fi = ri + m_resolution;
            int ti = fi + delta;

            chkminmax(fi,m_resolution,hc-1);
            chkminmax(ti,fi,hc-1);

            m_dataMgr->getHistory( GetNeededIdxs( fi,ti ), dh );
            DrawGraph(dh,&gdataarr);
            WX_APPEND_ARRAY( m_graphData, gdataarr );

        } else {
            m_dataMgr->getHistory( GetNeededIdxs(nli,li-m_resolution), dh );
            DrawGraph(dh,&gdataarr);
            WX_PREPEND_ARRAY( m_graphData, gdataarr );
        }

    } else if( dc < dl && m_graphData.GetCount() ) {

        int nli = ri - dc;

        for( int a = m_graphData.GetCount()-1; a >= 0 ; a-- ) {
            GraphDataDescriptor * gdata = m_graphData[a];

            if( gdata->GetIdx() >= nli ) continue;

            ptrdel(gdata);

            m_graphData.RemoveAt(a);
        }

    }


    //Logger::logf(LOG_MSG,_T("m_lastSize.y=%i, currsize.y=%i"),m_lastSize.y ,currsize.y);

    if( m_lastSize.y != currsize.y ) {
        RedrawGraph();
    }

    MakeGraphBitmap();

}

void MultiValueGraph::RefreshHistory()
{
    if(IsECUOnline()) return;

    m_dataMgr->refresh( m_currentSelectedDataIdx );
}

void MultiValueGraph::RedrawWidget()
{
    m_hoverSymbol = wxEmptyString;

    m_titleRegions.clear();

    SetThemeVars();

    SetStatusBarLayout();
    RefreshGUILayout();

    ClearNumValueBoxes();
    ClearDescExtraFields();

    RedrawGraph();

    MakeGraphBitmap();

    DrawBackgroundFaceAgg();

    DrawCursor();

    DrawMarkerLines();

    DrawNumValues(true,true);

    DrawSummaryBar();

    m_statusBarMgr->ReDrawBubbles();

    DrawTimeRuler();

    DrawSelectArea();

    GaugeRefresh();

    SetScrollBar();
}

void MultiValueGraph::ClearNumValueBoxes()
{
    WX_CLEAR_HASH_MAP(NumValueBoxesHash, m_numBoxes);
}

void MultiValueGraph::ClearTitleWidths()
{
    MultiGraphDesc * desc = GetDesc();

    for( size_t a = 0; a < desc->Cells.GetCount(); a++ ) {
        for( size_t b = 0; b < desc->Cells[a].Symbols.GetCount(); b++ ) {
            wxVariantHash& hash = desc->Cells[a].Symbols[b];
            if(hash.find(_T("e_titlewidths")) != hash.end())
                hash.erase(_T("e_titlewidths"));
        }
    }

}

void MultiValueGraph::ClearDescExtraFields()
{
    m_symbolTmpProps.clear();


//    MultiGraphDesc * desc = GetDesc();
//
//    for( size_t a = 0; a < desc->Cells.GetCount(); a++ ) {
//        for( size_t b = 0; b < desc->Cells[a].Symbols.GetCount(); b++ ) {
//            wxVariantHash& hash = desc->Cells[a].Symbols[b];
//
//            hash.erase(_T("e_titlewidths"));
//            hash.erase(_T("e_symbolmaxvalue"));
//            hash.erase(_T("e_symbolminvalue"));
//            hash.erase(_T("e_numformat"));
//            hash.erase(_T("e_symbolunit"));
//
//        }
//    }
}

void MultiValueGraph::RedrawGraph()
{


    int li = GetLeftIdx();
    int ri = GetRightIdx();

    DoubleHashArray dh;

    m_dataMgr->getHistory( GetNeededIdxs(li,ri), dh );

    ClearGraphDataArray();

    DrawGraph(dh);
}

GraphDataDescriptor* MultiValueGraph::NormalizeCursor()
{

    GraphDataDescriptor * d = SearchNearlyGraphDataByIdx( m_currentSelectedDataIdx );

    if(d == NULL) return NULL;

    m_currentSelectedDataIdx = d->GetIdx();

    return d;
}

/**
    running time: 16.2 ms
        - graphdata  = 400
        - symbols = 15
        - step = 2
            ( = imgs count: 400*15=6000; imgs width:2  avg height:4  )
        - isCalcPositions = false
        - cpu: amd 5600+ (2,6 GHz)

    running time: 81.2 ms
        - cpu: intel P4 3GHz

    running time: 20.1 ms (MergeToImage optimalization)
        - cpu: intel P4 3GHz

*/
void MultiValueGraph::MakeGraphBitmap(bool isCalcPositions)
{
    //FunctionMeter fm(_T("MakeGraphBitmap"));

    if( isCalcPositions ) CalculateGraphDataPositions();

    int w = GetClientSize().x;
    int h = GetClientSize().y;
    int bc = m_graphData.GetCount();

    wxImage img(w,h);
    img.InitAlpha();

    //Logger::logf(LOG_MSG,_T("w=%i,imgw=%i"),w,img.GetWidth());

    unsigned char* target_alpha = img.GetAlpha();

    int s = w*h;

    for( int j = 0; j < s; j++ ) target_alpha[j] = 0;


    for(int a = 0; a<bc;a++ ) {

        m_graphData[a]->MergeToImage(img);

    }

    ptrdel(m_layerBitmaps[MGBMP_GRAPH]);

    m_layerBitmaps[MGBMP_GRAPH] = new wxBitmap(img);

}

void MultiValueGraph::DrawGraph(DoubleHashArray& datas, GraphDataDescriptorArray* outerarr)
{
    //FUNCTIME

    if( IsSlimData() ) return;

    int oldc = m_currWinCursor;

    //Logger::logf(LOG_MSG,_T("oldc=%i"),oldc);

    SetGraphCursor(wxCURSOR_WAIT);

//    m_dataHeights.Clear();
    //Logger::logf(LOG_MSG,_T("DrawGraph c=%i"),datas.GetCount());

    //FunctionMeter fm(_T("DrawGraph"));


    MultiGraphDesc* desc = GetDesc();

    int canvas_height = m_graphRect.GetHeight() / desc->Cells.GetCount();

    wxAgg aggCanvas(getStep(), canvas_height);

    for( size_t b = 1; b < datas.GetCount(); b++ ) {
        GraphDataDescriptor * graphd = DrawData(aggCanvas, datas[b], datas[b-1]);

        if(graphd) {

            if( outerarr == NULL )
                m_graphData.Add( graphd );
            else
                outerarr->Add( graphd );
        }

    }


/*
    ///stats

    double hs=0;

    for( int a=0;a<m_dataHeights.GetCount();a++ ) {
        Logger::logf(LOG_MSG,_T("m_dataHeights[a]=%0.2f"),m_dataHeights[a]);
        hs += m_dataHeights[a];
    }

    hs /= (double)(m_dataHeights.GetCount());

    Logger::logf(LOG_MSG,_T("hs=%f"),hs/m_cellHeight*100.0);
*/


    SetGraphCursor( oldc );

}

void MultiValueGraph::FitGraphToScreen()
{
    if( IsSlimData() ) return;

    int sc = getScreenDataCount();
    int hc = getHistoryLen();

    m_resolution = hc / sc;

    if( m_resolution < 2 ) {

        m_resolution = 1;

        m_magnifyRatio = (int)ceil( (double)sc / (double)hc * (double)m_minDataSize );

        chkminmax(m_magnifyRatio,1,m_maxMagnify);

    }

}

bool MultiValueGraph::MagnifyGraphBetween(size_t start_idx, size_t end_idx) /// dont work!
{
    /*
    size_t hc = getHistoryLen();

    Logger::logf(LOG_MSG,_T("MultiValueGraph::MagnifyGraphBetween"));

    if( start_idx >= hc || end_idx >= hc || start_idx == end_idx ) return false;

    int sidx = start_idx < end_idx ? start_idx : end_idx;
    int eidx = start_idx > end_idx ? start_idx : end_idx;

    int w = GetClientSize().x;

    int mindatacnt = (int)ceil((double)w / (double)m_maxMagnify);

    int ndlen = eidx - sidx + 1;

    if( ndlen < mindatacnt ) {

        sidx = sidx + ndlen / 2 - mindatacnt/2;
        eidx = sidx + mindatacnt;

    }

    if( m_minDataSize * (eidx-sidx+1) > w ) {

        m_magnifyRatio = m_minDataSize;

        m_resolution = (eidx-sidx+1) / (w/m_minDataSize);

    } else {
        m_resolution =  1;
        m_magnifyRatio = (w/m_minDataSize) / (eidx-sidx+1);
    }

    chkminmax(sidx,m_resolution,hc-1);
    chkminmax(eidx,sidx,hc-1);

    DoubleHashArray datas;

    m_dataMgr->getHistory( GetNeededIdxs(sidx,eidx) , datas );

    ClearGraphDataArray();
    DrawGraph(datas);
    RemoveUnneededGraphData();

    NormalizeCursor();
    DrawSelectArea();

    RefreshSelectedTimeInfoBar();

    SetScrollBar();
    RefreshMagnifyVal();
    DrawTimeRuler();

    m_isCursorMoved = true;

    SetZoomBox();

    GaugeRefresh();

    return true;
    */
    return false;
}

bool MultiValueGraph::MagnifyGraph(int delta)
{

    int res = m_resolution;
    int mag = m_magnifyRatio;

    mag += delta;

    if( mag < 1 || res > 1 ) {
        res -= delta;
        mag = 1;
    }


    if( res < 1 ) {
        res = 1;
    }

    return MagnifyGraphTo(mag,res);

}

bool MultiValueGraph::MagnifyGraphTo(int mag_target, int res_target)
{
    if( IsSlimData() ) return false;

//    int oldres = m_resolution;
//    int oldmag = m_magnifyRatio;

    int hc = getHistoryLen();
    int maxres = getMaxResolution();

    //Logger::logf(LOG_MSG,_T("maxres=%i, res_target=%i,m_resolution=%i"),maxres,res_target,m_resolution);

    m_magnifyRatio = mag_target;
    m_resolution = res_target;

    chkminmax(m_resolution,1,maxres);

    chkminmax(m_magnifyRatio,1,m_maxMagnify);

//      if(m_resolution == oldres && oldmag == m_magnifyRatio ) return false;

    int odw = getScreenDataCount();

    int dc = (int)ceil( ((double)odw * (double)m_resolution) );

//    if( dc >= hc ) {
//
//        m_resolution = hc / odw;
//
//        if( m_resolution < 2 ) {
//            FitGraphToScreen();
//        }
//
//        if(m_resolution == oldres && oldmag == m_magnifyRatio ) return false;
//    }

//    dc = (int)ceil( ((double)odw * (double)m_resolution) );

    int li = GetLeftIdx();
    int ri = GetRightIdx();

    bool isOuterScreen = (m_currentSelectedDataIdx < li || m_currentSelectedDataIdx > ri) ? true : false;

    double riciar = isOuterScreen ? 0.5 : (double)(m_currentSelectedDataIdx-li) / (double)(GetDataLen());

    int nri = m_currentSelectedDataIdx + (int)((double)dc * (1.0-riciar));
    int nli = nri - dc;

    if( nli < m_resolution ) {
        nli = m_resolution;
        nri = nli + dc;
    }

    if( nri > hc-1 ) {
        nri = hc-1;
        nli = nri - dc;
    }

    chkminmax(nli,m_resolution,hc-1);
    chkminmax(nri,nli,hc-1);

    DoubleHashArray datas;

    m_dataMgr->getHistory( GetNeededIdxs(nli,nri) , datas );


    ClearGraphDataArray();
    DrawGraph(datas);
    MakeGraphBitmap();

    RemoveUnneededGraphData();

    NormalizeCursor();

    if( m_startSelectedDataIdx != -1 && m_startSelectedDataIdx >= nli && m_startSelectedDataIdx <= nri ) {

        GraphDataDescriptor * gdata = SearchNearlyGraphDataByIdx( m_startSelectedDataIdx );

        if(gdata != NULL)
            m_startSelectedDataIdx = gdata->GetIdx();

    }

    DrawSelectArea();

    RefreshSelectedTimeInfoBar();

    SetScrollBar();
    RefreshMagnifyVal();
    DrawTimeRuler();

    m_isCursorMoved = true;

    SetZoomBox();

    return true;
}

void MultiValueGraph::OnSize( wxSizeEvent &evt)
{
    UpdateSize();
    evt.Skip();
}

void MultiValueGraph::RemoveUnneededGraphData()
{
    int x = 0;
    int w = GetClientSize().x;

    int bc = m_graphData.GetCount();
    int step = getStep();

    for( int a = 0; a < bc ; a++ ) {

        x = w - (((bc)-a) * step);

        if( x <= 0 - step ) {
            ptrdel(m_graphData[a]);
            m_graphData.RemoveAt(a);

            continue;
        } else
            break;

   }

}
/** sample: 100000

    running time: 2.6 mikrosec
        - graphdata: 400
*/

void MultiValueGraph::CalculateGraphDataPositions()
{
    //FunctionMeter fm(_T("CalculateGraphDataPositions()"));
    int x = 0;
    int w = GetClientSize().x;

    int bc = m_graphData.GetCount();
    int step = getStep();


    for( int a = bc-1; a >= 0 ; a-- ) {

        x = w - (((bc)-a) * step);

        if( x <= 0 - step ) {
            ptrdel(m_graphData[a]);
            m_graphData.RemoveAt(a);
            continue;
        }

        m_graphData[a]->SetXPos( x );
   }

}

/** sample:1000

    running time: 6.8 ms
        - canvas width: 800
        - canvas height: 600

*/

void MultiValueGraph::PaintMarkers(wxDC& dc, int w, int h, int li, int ri, int step)
{

    for( Int2wxVariantHashHash::iterator it = m_markerTypes.begin(); it != m_markerTypes.end(); it++ ) {

        if( m_markerList.find(it->first) == m_markerList.end() ) continue;

        wxVariantHash& hash = it->second;

        bool show = hash[_T("show")].GetBool();

        if(!show) continue;

        IntArray& markers = m_markerList[it->first];

        for(size_t i = 0 ; i < markers.GetCount(); i++){
            int midx = markers[i];

            if(midx >= li && midx <= ri) {

                GraphDataDescriptor * gdd = SearchNearlyGraphDataByIdx(midx);

                if(gdd == NULL) return;

                int x = gdd->GetXPos() + step;

                wxBitmap bmp;
                bmp << hash[_T("bmp")];

                dc.DrawBitmap( bmp, x, GetHeaderSize(),1 );
            }
        }

    }

}

void MultiValueGraph::PaintCursor(wxDC& dc,int w, int li, int ri, int step)
{
    if( !m_layerBitmaps[MGBMP_CURSOR] ) return;

    if( m_currentSelectedDataIdx >= li && m_currentSelectedDataIdx <= ri ) {


        GraphDataDescriptor * gdata = NormalizeCursor();
        if(gdata) {
            int curs_x = gdata->GetXPos() + step;

            chkminmax(curs_x,0,w);

            dc.DrawBitmap(*m_layerBitmaps[MGBMP_CURSOR],curs_x,GetHeaderSize(),1);
        }

    }

}

void MultiValueGraph::PaintNumValues(wxDC& dc)
{
    for(NumValueBoxesHash::iterator it = m_numBoxes.begin(); it != m_numBoxes.end(); it++)
        dc.DrawBitmap( *(it->second->BmpBox), (int)it->second->Position.x, (int)it->second->Position.y, 1 );
}

void MultiValueGraph::PaintSelectedArea(wxDC& dc, int li, int ri, int step)
{
    if(m_startSelectedDataIdx != -1 && m_layerBitmaps[MGBMP_SELECTED_FIELD]) {

        int start_idx = m_startSelectedDataIdx < m_currentSelectedDataIdx ? m_startSelectedDataIdx : m_currentSelectedDataIdx;
        int end_idx = m_startSelectedDataIdx > m_currentSelectedDataIdx ? m_startSelectedDataIdx : m_currentSelectedDataIdx;

        start_idx += m_resolution;

        if( start_idx < ri ){
            if(start_idx > li) {
                GraphDataDescriptor * gdata = SearchBGraphDataByIdx( start_idx );
                if(gdata) {
                    int sx = gdata->GetXPos();
                    dc.DrawBitmap(*m_layerBitmaps[MGBMP_SELECTED_FIELD], sx, m_graphRect.GetTop()+m_cellsepar, 1 );
                }
            } else if( end_idx > li && end_idx < ri ) {
                GraphDataDescriptor * gdata = SearchBGraphDataByIdx( end_idx );
                if(gdata) {
                    int ex = gdata->GetXPos() + step;
                    int sx = ex - m_layerBitmaps[MGBMP_SELECTED_FIELD]->GetWidth();
                    wxBitmap bmp = m_layerBitmaps[MGBMP_SELECTED_FIELD]->GetSubBitmap(wxRect(abs(sx),0,ex,m_layerBitmaps[MGBMP_SELECTED_FIELD]->GetHeight()));
                    dc.DrawBitmap(bmp, 0, m_graphRect.GetTop()+m_cellsepar, 1 );
                }
            }
        }

    }
}

void MultiValueGraph::PaintToolTip(wxDC& dc, int h)
{
    if( !m_layerBitmaps[MGBMP_TOOLTIP] ) return;

    int x = m_toolTipMgr->GetProperty( ATTET_MARKERPOS, -1L ).GetLong();

    if( x == -1 ) return;

    wxPoint ap = m_toolTipMgr->GetArrowPoint();

    int y = h - GetFooterSize() - ap.y;

    x -= ap.x;

    dc.DrawBitmap(*m_layerBitmaps[MGBMP_TOOLTIP], x, y, true );
}

/**
    1,a: 7,32 ms
    1,b: 8,19 ms
*/
void MultiValueGraph::OnPaint(wxPaintEvent &evt)
{
    //Logger::logf(LOG_MSG,_T("OnPaint m_layerBitmaps[MGBMP_FACE].w=%i,h=%i, m_layerBitmaps[MGBMP_GRAPH].w=%i,h=%i"),m_layerBitmaps[MGBMP_FACE]->GetWidth(),m_layerBitmaps[MGBMP_FACE]->GetHeight(), m_layerBitmaps[MGBMP_GRAPH]->GetWidth(), m_layerBitmaps[MGBMP_GRAPH]->GetHeight() );
   // FunctionMeter fm(_("MultiValueGraph::OnPaint()"));

    //wxAutoBufferedPaintDC dc(this);
	wxBufferedPaintDC dc(this);

    dc.SetBackground( *wxWHITE_BRUSH );
    dc.Clear();

    int w = GetClientSize().x;
    int h = GetClientSize().y;


    if(m_layerBitmaps[MGBMP_FACE]) dc.DrawBitmap(*m_layerBitmaps[MGBMP_FACE],0,0);

    if( IsSlimData() ) return;

    int li = GetLeftIdx();
    int ri = GetRightIdx();

    int step = getStep();

    m_statusBarMgr->DrawStatusBar(dc);

    if(m_layerBitmaps[MGBMP_SUMMARY])
        dc.DrawBitmap(*m_layerBitmaps[MGBMP_SUMMARY],0, h-GetFooterSize(),1);

    PaintMarkers(dc,w,h,li,ri,step);

    if(m_layerBitmaps[MGBMP_GRAPH]) {
        /// sample:1000 | running time: 3.1 ms;  bmp width: 800 height: 600
        dc.DrawBitmap(*m_layerBitmaps[MGBMP_GRAPH],w-m_layerBitmaps[MGBMP_GRAPH]->GetWidth(),GetHeaderSize(),false);
    }



    PaintNumValues(dc);

    if(m_layerBitmaps[MGBMP_TIMERULER])
        dc.DrawBitmap(*m_layerBitmaps[MGBMP_TIMERULER],0,GetHeaderSize()-m_timeRulerSize,1);


    if(m_layerBitmaps[MGBMP_DYNAMIC_CAPTION])
        dc.DrawBitmap(*m_layerBitmaps[MGBMP_DYNAMIC_CAPTION],0,0,1);

    if(m_layerBitmaps[MGBMP_CURRENT_TIME])
        dc.DrawBitmap(*m_layerBitmaps[MGBMP_CURRENT_TIME],w-m_layerBitmaps[MGBMP_CURRENT_TIME]->GetWidth()-5, (m_captionSize-m_layerBitmaps[MGBMP_CURRENT_TIME]->GetHeight())/2 ,1);

    PaintSelectedArea(dc,li,ri,step);

    PaintToolTip(dc,h);

    PaintCursor(dc,w,li,ri,step);

    PaintWidgets(dc);



    DrawExtraLayer(dc);
}

int MultiValueGraph::getMaxResolution()
{
    int hc = getHistoryLen();

    int w = GetClientSize().x;

    int sc = (int)ceil((double)(w / m_minDataSize)) + 1;

    if(sc == 0) return 0;

    return (int)ceil((double)hc / (double)sc);
}

size_t MultiValueGraph::getHistoryLen()
{
    /*
    if( m_isLocked )
        return m_historyLockLen;
*/
    if(m_dataMgr) {
        if( m_lockedHistoryLen >= 0 )
            return m_lockedHistoryLen;
        else
            return m_dataMgr->getHistoryCount();
    } else
        return 0;
}

int MultiValueGraph::getScreenDataCount()
{
    int step = getStep();

    int w = GetClientSize().x;

    int r = (int)ceil((double)(w / step)) + 1;

    return r;
}

int MultiValueGraph::getScreenDataMaxLen()
{
    return getScreenDataCount() * m_resolution;
}

int MultiValueGraph::GetHeaderSize()
{
    return m_captionSize + m_timeRulerSize + (m_toolbar->IsShown() == true ? m_toolBarSize : 0);
}

int MultiValueGraph::GetFooterSize()
{
    return m_statusBarSize + m_scrollBarSize + m_summaryBarSize;
}

int MultiValueGraph::getStep()  /// return the size how many pixel store one data
{
    int step = m_magnifyRatio;

    if(step < m_minDataSize)
        step = m_minDataSize;

    return step;
}

bool MultiValueGraph::IsSelectModKey( wxKeyEvent &evt )
{
    wxString str = m_hotkeyMgr->GetHotKeyStringBySymbol(_T("MG_SELECT_GLOBAL_MODIFIER"));

    wxString mod = wxKeyBind::KeyModifierToString(wxKeyBind::GetKeyModifier(evt));

    return (mod == str ? true : false);
}

void MultiValueGraph::OnKeyUp(wxKeyEvent &evt)
{
    if( m_isCursorMoved )  {

        if(m_toolbar->GetValue(_T("refreshgauges")).GetBool()) RefreshHistory();

        m_isCursorMoved = false;
    }

}

/** other measurements:

        LogHistoryBase::getHistory : 274 usec / 1 data
            - sample: 100
            - data count : 401

*/

void MultiValueGraph::OnKeyDown(wxKeyEvent &evt)
{
    if( m_groupManager->IsEditMode() ) {
        evt.Skip();
        return;
    }

#if defined ( VEMSDEBUG )

    int kc = evt.GetKeyCode();

    if( kc == 'G' ) {


        Logger::logf(LOG_MSG,_T("start"));

        wxStopWatch s;

        s.Start();

        for(int a=0; a<1000;a++) {

            DrawSelectArea();

        }

        Logger::logf(LOG_MSG,_T("time: %i"),s.Time());

    }

#endif


    wxString sel_str = m_hotkeyMgr->GetHotKeyStringBySymbol(_T("MG_SELECT_GLOBAL_MODIFIER"));

    wxString sel_mod = wxKeyBind::KeyModifierToString(wxKeyBind::GetKeyModifier(evt));

    bool isSelect = (sel_mod.Find(sel_str) == wxNOT_FOUND ? false : true);//IsSelectModKey(evt);


    wxString symbol = m_hotkeyMgr->GetEvent( evt, sel_str );

    //Logger::logf(LOG_MSG,_T("OnKeyDown symbol=")+symbol);
    if( isSelect && m_startSelectedDataIdx == -1 ) m_startSelectedDataIdx = m_currentSelectedDataIdx;


    if( symbol == _T("MG_MOVE_CURSOR_SIMPLE_LEFT") ) {

        if( m_isLocked )
            MoveCursor(-1*m_cursorDelta, isSelect);
        else
            return;

    } else if( symbol == _T("MG_MOVE_CURSOR_SIMPLE_RIGHT") ) {

        if( m_isLocked )
            MoveCursor(m_cursorDelta, isSelect);
        else
            return;

    } else if( symbol == _T("MG_MOVE_CURSOR_FAST_LEFT") ) {

        if( m_isLocked )
            MoveCursor(-1*m_cursorDelta*m_cursorAccel, isSelect);
        else
            return;


    } else if( symbol == _T("MG_MOVE_CURSOR_FAST_RIGHT") ) {

        if( m_isLocked )
            MoveCursor(m_cursorDelta*m_cursorAccel, isSelect);
        else
            return;

    } else if( symbol == _T("MG_MOVE_CURSOR_END") ) {

        if( m_isLocked )
            MoveCursor( GetRightIdx() - m_currentSelectedDataIdx - m_resolution , isSelect);
        else
            return;

    } else if( symbol == _T("MG_MOVE_CURSOR_START") ) {

        if( m_isLocked )
            MoveCursor( GetLeftIdx() - m_currentSelectedDataIdx , isSelect);
        else
            return;

    } else if( symbol == _T("MG_MOVE_CURSOR_PAGE_LEFT") ) {

        if( m_isLocked )
            MoveCursorTo( m_currentSelectedDataIdx - (GetRightIdx()-GetLeftIdx()) );
        else
            return;

    } else if( symbol == _T("MG_MOVE_CURSOR_PAGE_RIGHT") ) {

        if( m_isLocked )
            MoveCursorTo( m_currentSelectedDataIdx + (GetRightIdx()-GetLeftIdx()) );
        else
            return;

    } else if( symbol == _T("MG_MOVE_CURSOR_GRAPH_START") ) {

        if( m_isLocked )
            MoveCursorTo(m_resolution);
        else
            return;

    } else if( symbol == _T("MG_MOVE_CURSOR_GRAPH_END") ) {

        if( m_isLocked )
            MoveCursorTo( getHistoryLen() - 1 );
        else
            return;

    } else if( symbol == _T("MG_MAGNIFY_PLUS") ) {

        if(!MagnifyGraph( m_magStep )) return;
        SetScrollBar();

    } else if( symbol == _T("MG_MAGNIFY_MINUS") ) {

        if(!MagnifyGraph(-1*m_magStep)) return;
        SetScrollBar();

    } else if( symbol == _T("MG_SYMBOL_MAGNIFY_PLUS") ) {

        if(m_selectedMSymbol == wxEmptyString) return;
        MagnifySymbol(1);

    } else if( symbol == _T("MG_SYMBOL_MAGNIFY_MINUS") ) {

        if(m_selectedMSymbol == wxEmptyString) return;
        MagnifySymbol(-1);

    } else if( symbol == _T("MG_SYMBOL_MAGNIFY_RESET") ) {

        if(m_selectedMSymbol == wxEmptyString) return;

        SetSymbolMagnify(m_selectedMSymbol,1.0);

        RedrawGraph();
        DrawNumValues(false,true);
        MakeGraphBitmap();

    } else if( symbol == _T("MG_SYMBOL_MAGNIFY_RESET_ALL") ) {

        SetSymbolMagnify();

        RedrawGraph();
        DrawNumValues(false,true);
        MakeGraphBitmap();

    } else if( symbol == _T("MG_SYMBOL_OFFSET_PLUS") ) {

        if(m_selectedMSymbol == wxEmptyString) return;
        OffsetSymbol(-1);

    } else if( symbol == _T("MG_SYMBOL_OFFSET_MINUS") ) {

        if(m_selectedMSymbol == wxEmptyString) return;
        OffsetSymbol(1);

    } else if( symbol == _T("MG_SYMBOL_OFFSET_RESET") ) {

        if(m_selectedMSymbol == wxEmptyString) return;
        m_symbolOffsets[m_selectedMSymbol] = 0;
        RedrawGraph();
        DrawNumValues(false,true);
        MakeGraphBitmap();

    } else if( symbol == _T("MG_SYMBOL_OFFSET_RESET_ALL") ) {

        for( Short2StringHash::iterator it = m_symbolOffsets.begin(); it != m_symbolOffsets.end(); it++ )
            it->second = 0;

        RedrawGraph();
        DrawNumValues(false,true);
        MakeGraphBitmap();

    } else if( symbol == _T("MG_HELP") ) {

        m_helpPanel->Centre();
        m_helpPanel->Show( !m_helpPanel->IsShown() );

    } else if( symbol == _T("MG_PLAY_PAUSE_LOG") ) {

    } else if( symbol == _T("MG_SHOW_HIDE_TOOLBAR") ) {

        KlappeToolbar();

    } else if( symbol == _T("MG_TAB_SELECT_SYMBOLS") ) {

        TabToNextSymbol();

    } else if( symbol == _T("MG_TAB_SELECT_SYMBOLS_PREV") ) {

        TabToPrevSymbol();

    } else if( symbol == _T("MG_JUMP_PREV_MARKER") ) {

        JumpCursorToMarker(false);

    } else if( symbol == _T("MG_JUMP_NEXT_MARKER") ) {

        JumpCursorToMarker(true);

    } else if( symbol == _T("MG_OPEN_DESC_EDITOR_ALL") ) {

        OpenDescEditor();

    } else if( symbol == _T("MG_OPEN_DESC_EDITOR_SELECTED_NODE") ) {

        OpenDescEditor( ODE_NODE );

    } else if( symbol == _T("MG_OPEN_DESC_EDITOR_SELECTED_SYMBOL") ) {

        OpenDescEditor( ODE_SYMBOL );

    } else if( symbol == _T("MG_MARKER_SETTINGS_DIALOG") ) {

        MarkerSettingsDialog dlg(this,m_markerTypes);

        if(dlg.ShowModal() == wxID_OK) {

            dlg.SetToHash( m_markerTypes );

        } else {
            return;
        }

    } else if( symbol == _T("MG_OPEN_SEARCH_DLG") ) {

        SearchDlg();

    } else if( symbol == _T("MG_CLEAR_SEARCH_RESULT") ) {

        m_searchResultSections.Clear();

        DrawSummaryBar();

        GaugeRefresh();

    } else if( symbol == _T("MG_LOCK_UNLOCK") ) {


        wxString str = _T("State: Locked");

        wxString state = _T("red");

        if( m_isLocked ) {
            str = _T("State: Unlocked");

            state = _T("black");
        } else {

            m_historyLockLen = getHistoryLen();
        }

        m_isLocked = !m_isLocked;

        //m_statusBarMgr->SetBubbleStateAndText(_T("lock"),state,str,false);
        GaugeRefresh();
        return;

    } else {

        evt.Skip();
        return;
    }

    GaugeRefresh();

}

void MultiValueGraph::OnScrollBar(wxScrollEvent& evt)
{
    if( MoveScreenTo(evt.GetPosition()) ) {
        GaugeRefresh();
        SetFocus();
    }
}

void MultiValueGraph::OffsetSymbol(short dir)
{
    if( IsSlimData() ) return;

    int delta = 10 * dir;

    if( m_symbolOffsets.find(m_selectedMSymbol) == m_symbolOffsets.end() )
        m_symbolOffsets[m_selectedMSymbol] = 0;

    m_symbolOffsets[m_selectedMSymbol] += delta;

    RedrawGraph();
    DrawNumValues(false,true);
    MakeGraphBitmap();
}

void MultiValueGraph::SetSymbolMagnify(const wxString& symbolname, double value)
{
    MultiGraphDesc * desc = GetDesc();

    size_t cc = desc->Cells.GetCount();

    for( size_t b = 0; b < cc; b++  ) {

        for( size_t c = 0; c < desc->Cells[b].Symbols.GetCount(); c++ ) {

            wxVariantHash &hash = desc->Cells[b].Symbols[c];

            if( hash[_T("name")].GetString() != symbolname && symbolname != wxEmptyString ) continue;

            if(fabs(value-1.0) < 0.001)
                hash.erase(_T("scale"));
            else
                hash[_T("scale")] = value;

            if(symbolname != wxEmptyString)
                return;
        }

    }
}

double MultiValueGraph::GetSymbolMagnify(const wxString& symbolname)
{
    MultiGraphDesc * desc = GetDesc();

    size_t cc = desc->Cells.GetCount();

    for( size_t b = 0; b < cc; b++  ) {

        for( size_t c = 0; c < desc->Cells[b].Symbols.GetCount(); c++ ) {

            wxVariantHash &hash = desc->Cells[b].Symbols[c];

            if( hash[_T("name")].GetString() != symbolname ) continue;

            if( hash.find(_T("scale")) == hash.end() ) return 1.0;

            return hash[_T("scale")].GetDouble();
        }

    }

    return 0.0;
}

void MultiValueGraph::MagnifySymbol(short direction)
{

    if( IsSlimData() ) return;

    double delta = 0.2 * direction;

//    if( m_symbolMagnifyRatios.find(m_selectedMSymbol) == m_symbolMagnifyRatios.end() )
//        m_symbolMagnifyRatios[m_selectedMSymbol] = 1.0;

    double currv = GetSymbolMagnify(m_selectedMSymbol);

    SetSymbolMagnify( m_selectedMSymbol, currv+delta );

    RedrawGraph();
    DrawNumValues(false,true);
    MakeGraphBitmap();
}

void MultiValueGraph::SetGraphCursor(int cursorid)
{
    m_currWinCursor = cursorid;

    //Logger::logf(LOG_MSG,_T("m_currWinCursor=%i"),m_currWinCursor);

    if(m_currWinCursor < 0)
        SetCursor(wxNullCursor);
    else
        SetCursor(wxCursor(m_currWinCursor));
}

void MultiValueGraph::GaugeRefresh()
{
    m_isRefresh = true;
}

wxRegion MultiValueGraph::GetShape()
{
    return wxRegion(0,0,GetClientSize().x,GetClientSize().y);
}

wxRegion MultiValueGraph::GetBorderRegion()
{
    return wxRegion(0,0,1,1);
}


void MultiValueGraph::UpdateSize()
{

    if( IsSlimData() ) return;

    if(!m_inited) return;

    ClearNumValueBoxes();

    ClearDescExtraFields();

    DrawBackgroundFaceAgg();

    DrawCursor();
    DrawMarkerLines();

    DrawNumValues(true,true);

    m_statusBarMgr->SetBarRect( 0, GetClientSize().y - m_statusBarSize, GetClientSize().x, m_statusBarSize );

    RefreshGraph();

    SetScrollBar();

    m_helpPanel->Centre();

    RefreshGUILayout();

    GaugeRefresh();

    SetToolBarZoomSelect();

    SetZoomBox();

    m_lastSize = GetClientSize();

    m_graphRect = wxRect(0,GetHeaderSize(), GetClientSize().x, GetClientSize().y - GetFooterSize()-GetHeaderSize()-m_cellsepar );
}

void MultiValueGraph::setSize(int x , int y)
{
}

void MultiValueGraph::SetValue( double value )
{
}

void MultiValueGraph::SetNewTheme(WidgetThemeTable *theme)
{
}

void MultiValueGraph::require()
{
    if( m_isRegistered ) return;

    wxArrayString arr;
    GetDesc()->GetSymbolNameList(arr);

    arr.Add(_T("secl"));

    for(size_t a=0;a<arr.GetCount();a++) {
        //Logger::logf(LOG_MSG,_T("arr[%i]=")+arr[a],a);
        m_rtDataMgr->registerSymbol( arr[a] );
    }

    m_isRegistered = true;

}

void MultiValueGraph::refresh(DoubleHash* table)
{
    /*
    wxArrayString arr;
    GetDesc()->GetSymbolNameList(arr);

    arr.Add(_T("secl"));

    UpdateViewerEvent event;

    wxVariantList list;

    event.m_variant = list;

    for(size_t a=0;a<arr.GetCount();a++) {
        wxVariant var((*table)[arr[a]], arr[a]);
        event.m_variant.Append(var);
    }

    AddPendingEvent( event );
    */
}

void MultiValueGraph::OnUpdateValue( UpdateViewerEvent& event )
{
    /*

    DoubleHash * hash = new DoubleHash();

    for( size_t a=0;a< event.m_variant.GetCount();a++) {

        (*hash)[event.m_variant[a].GetName()] = event.m_variant[a].GetDouble();

    }

    //double time = (*hash)[_T("time")]; /// do: refresh time bmp

    //int currentdata_idx = (int)((*hash)[strLogIdxPos]);

    if( m_isLocked ) {


    } else {



    }

    //GaugeRefresh();
    */
}

void MultiValueGraph::ClearGraphDataArray()
{
    WX_CLEAR_ARRAY(m_graphData);
}

bool MultiValueGraph::SendToolBarCommand(const wxString& cmd, wxVariant var)
{
 //   Logger::logf(LOG_MSG,_T("SendToolBarCommand cmd=")+cmd);

    if( cmd == _T("setzoom") ) {

        wxString str = var.GetString();

        wxString resstr = str.AfterFirst('/');

        if( resstr == wxEmptyString ) {
            long mag = -1;
            str.ToLong(&mag);
            if(mag == -1) return false;
            mag /= 100;
            MagnifyGraphTo(mag,1);
            GaugeRefresh();

        } else {

            long res;
            resstr.ToLong(&res);
            MagnifyGraphTo(1,res);
            GaugeRefresh();

        }


    } else if( cmd == _T("deltazoom") ) {

        MagnifyGraph( var.GetLong() );
        GaugeRefresh();

    } else if( cmd == _T("fittosel") ) {

        if( m_startSelectedDataIdx == -1 ) return false;

        MagnifyGraphBetween( m_startSelectedDataIdx, m_currentSelectedDataIdx );


    } else if( cmd == _T("openmarkersdlg") ) {

        MarkerSettingsDialog dlg(this,m_markerTypes);

        if(dlg.ShowModal() == wxID_OK) {
            dlg.SetToHash( m_markerTypes );
        }

    } else if( cmd == _T("opendesceditor") ) {

        OpenDescEditor();

    } else if( cmd == _T("openthemeeditor") ) {

        OpenThemeEditor();

    } else if( cmd == _T("refreshlockedlen") ) {

        m_lockedHistoryLen = m_dataMgr->getHistoryCount();

        RedrawWidget();

    } else if( cmd == _T("startsearch") ) {

        SearchDlg();

    } else
        return false;

    return true;
}

void MultiValueGraph::SearchDlg(const wxString& searchfor)
{

    wxArrayString searchhist;

    LogSearchHistoryHelper::Load(_T("/MultiGraph"),searchhist, ConfigHelper2::getConfig() );

    wxArrayString symbols;

    MultiGraphDesc * desc = GetDesc();

    size_t cc = desc->Cells.GetCount();

    for( size_t b = 0; b < cc; b++  ) {
        for( size_t c = 0; c < desc->Cells[b].Symbols.GetCount(); c++ ) {
            symbols.Add(desc->Cells[b].Symbols[c][_T("name")].GetString());
        }
    }

    SearchDialog dlg(this, m_dataMgr,_T("Search in log with expression..."), searchhist, symbols);

    if(searchfor != wxEmptyString) {
        dlg.SetSearchString(searchfor);
    }

    if(m_startSelectedDataIdx != m_currentSelectedDataIdx)
        dlg.SetSearchSection( Section( wxMin(m_startSelectedDataIdx,m_currentSelectedDataIdx), wxMax(m_startSelectedDataIdx,m_currentSelectedDataIdx) ) );

    if(dlg.ShowModal() != wxID_OK) {
        SetFocus();
        return;
    }

    m_searchResultSections = dlg.GetResult();

    LogSearchHistoryHelper::Save(_T("/MultiGraph"), dlg.GetSearchHistory(), ConfigHelper2::getConfig() );

    DrawSummaryBar();

    GaugeRefresh();

    if(m_searchResultSections.GetCount()){
        int id = wxNewId();
        SearchResultDialog* frame = new SearchResultDialog(this,id,m_searchResultSections);
        Connect(id, wxEVT_GENERAL, GeneralEventHandler(MultiValueGraph::OnSearchResultBox) );
        frame->SetHandler(this);
        frame->Show();
}

}

/*************************************************************
        GraphDataDescriptor class
*/

GraphDataDescriptor::GraphDataDescriptor(DoubleHash* data, int idx, int width, int xpos)
{
    m_rtData = data;
    m_logIdx = idx;
    m_xPos = xpos;
    m_width = width;
}

GraphDataDescriptor::~GraphDataDescriptor()
{
    delete m_rtData;
    for(wxImageStrHash::iterator it = m_imgs.begin(); it != m_imgs.end(); it++) {
        delete it->second;
    }
    m_imgs.clear();
}

void GraphDataDescriptor::AddImage(const wxString& name, wxImage * image, int y_position)
{
    if( m_imgs.find(name) != m_imgs.end() ) { delete m_imgs[name]; };

    m_imgs[name] = image;
    m_yPos[name] = y_position;
}

bool GraphDataDescriptor::MergeToImage(wxImage& targetImg)
{

    if(!targetImg.IsOk()) return false;

    if(!targetImg.HasAlpha()) return false;

    unsigned char* target_data = targetImg.GetData();
    unsigned char* target_alpha = targetImg.GetAlpha();

    wxImage * img;

    unsigned char* img_data;
    unsigned char* img_alpha;


    int target_x = m_xPos;
    uint16_t img_w, img_h, img_x, img_y,img_didx,img_aidx, target_y, target_w, target_h;
    uint32_t target_aidx,target_didx,target_dataSize;
    uint16_t start_img_x  = 0;

    double divider = 1.0/255.0;

    target_w = targetImg.GetWidth();
    target_h = targetImg.GetHeight();
    target_dataSize = target_w * target_h * 3;

    double img_a_rat,target_a_rat,new_a_rat;


    if(target_x < 0) {
        start_img_x = abs(target_x);
    }


    for(wxImageStrHash::iterator it = m_imgs.begin(); it != m_imgs.end(); it++) {
        img = it->second;

        //Logger::logf(LOG_MSG,_T("MTI img=%p, idx=%i, name=")+it->first,img,m_logIdx);

        img_data = img->GetData();
        img_alpha = img->GetAlpha();

        img_w = img->GetWidth();
        img_h = img->GetHeight();


        target_y = m_yPos[it->first];

        //wxString format = _T("x=%i\ty=%i\tidx=%i\ts=") + it->first;
        //Logger::logf(LOG_MSG,format,m_xPos,target_y,m_logIdx);

        for( img_y=0 ; img_y < img_h ; img_y++ ) {
            for( img_x=start_img_x ; img_x < img_w ; img_x++ ) {

                img_aidx = img_x + img_w * img_y;

                if( img_alpha[img_aidx] == 0 ) continue;

                img_didx = img_aidx * 3;

                target_aidx = (target_x + img_x) + target_w * ( target_y + img_y );
                target_didx = target_aidx * 3;

                if( target_didx >= target_dataSize ) return false;


/*
                target_data[target_didx] = img_data[img_didx];
                target_data[target_didx+1] = img_data[img_didx+1];
                target_data[target_didx+2] = img_data[img_didx+2];
                target_alpha[target_aidx] = img_alpha[img_aidx];
*/

                img_a_rat = img_alpha[img_aidx] * divider;

                target_a_rat = target_alpha[target_aidx] * divider;

                new_a_rat = img_a_rat + target_a_rat * ( 1.0 - img_a_rat );

                target_data[target_didx] = (uint8_t)((img_data[img_didx] * img_a_rat + target_data[target_didx] * target_a_rat * (1.0 - img_a_rat)) / new_a_rat);
                target_data[target_didx+1] = (uint8_t)((img_data[img_didx+1] * img_a_rat + target_data[target_didx+1] * target_a_rat * (1.0 - img_a_rat)) / new_a_rat);
                target_data[target_didx+2] = (uint8_t)((img_data[img_didx+2] * img_a_rat + target_data[target_didx+2] * target_a_rat * (1.0 - img_a_rat)) / new_a_rat);
                target_alpha[target_aidx] = (uint8_t)(new_a_rat * 255.0);

            }
        }
    }

    return true;
}

/*************************************************************
        NumValueBoxes class
*/

NumValueBoxes::NumValueBoxes()
{
    Position = wxRealPoint(-1.0,-1.0);
    BmpBox = NULL;

    m_lastValue = - 1000000.0;
}

NumValueBoxes::~NumValueBoxes()
{
    if(BmpBox)
        ptrdel(BmpBox);
}

void NumValueBoxes::SetBitmap(wxBitmap * bmp)
{

    if(BmpBox)
        ptrdel(BmpBox);

    BmpBox = bmp;
}

bool NumValueBoxes::IsChangedValue(double currentval)
{
    double diff = fabs(m_lastValue - currentval);

    m_lastValue = currentval;

    if( diff <= 0.001 )
        return false;
    else

        return true;

}

int MultiValueGraph::CompareInts(int* n1, int* n2)
{
    return *n1 - *n2;
}

/*************************************************************
            MarkerSettingsDialog class
*/

MarkerSettingsDialog::MarkerSettingsDialog(wxWindow* parent, Int2wxVariantHashHash& markerTypes)
    :bcParentDialog(parent, wxID_ANY, _T("Show or hide markers:"))
{

    wxBoxSizer * mainsizer = new wxBoxSizer(wxVERTICAL);

    for(Int2wxVariantHashHash::iterator it = markerTypes.begin(); it != markerTypes.end(); it++) {
        wxVariantHash& hash = it->second;

        bcCheckBox * cb = new bcCheckBox(this, wxID_ANY, hash[_T("title")].GetString() );

        cb->SetValue( hash[_T("show")].GetBool() );

        mainsizer->Add( new bcWrapper(cb),0,wxALL, 5);

        m_cbs[it->first] = cb;

    }


    mainsizer->Add( CreateBcStdDialogButtonSizer(wxOK|wxCANCEL), 0, wxALL, 5 );

	SetSizer(mainsizer);
	mainsizer->Fit(this);
	mainsizer->SetSizeHints(this);
}

void MarkerSettingsDialog::SetToHash(Int2wxVariantHashHash& markerTypes)
{
    for(Int2wxVariantHashHash::iterator it = markerTypes.begin(); it != markerTypes.end(); it++) {
        wxVariantHash& hash = it->second;

        if( m_cbs.find(it->first) == m_cbs.end() ) continue;

        hash[_T("show")] = m_cbs[it->first]->GetValue();
    }
}
