#ifndef MULTIVALUEGRAPH_H
#define MULTIVALUEGRAPH_H

#include "../VemsWidgetBase.h"
#include "../../common/types/DoubleArray.h"
#include "../../common/types/DoubleHashArray.h"
#include "../../common/types/wxBitmapArray.h"
#include "../../common/types/Short2StringHash.h"
#include "../../common/types/wxVaritantHashInt2Hash.h"
#include "../../gaugedesc/MultiGraphDescManager.h"
#include "../../bcWidgets/BitmapCanvasParentBase.h"
#include "../../loghistory/LogHistorySearch.h"
#include "../../bcWidgets/bcParentDialog.h"
#include "../../events/GeneralEvent.h"

class wxAgg;
class bcLabel;
class bcButton;
class bcCheckBox;
class AggToolTip;
class NumValueBoxes;
class LogHistoryBase;
class wxHotKeyManager;
class BubbleStatusBar;
class MVG_ToolbarPanel;
class GraphDataDescriptor;




WX_DECLARE_STRING_HASH_MAP( NumValueBoxes*, NumValueBoxesHash);
WX_DECLARE_STRING_HASH_MAP( wxRegion, wxRegionHash);
WX_DECLARE_HASH_MAP( int, wxRect, wxIntegerHash, wxIntegerEqual, wxRectHash);

WX_DEFINE_ARRAY( GraphDataDescriptor* , GraphDataDescriptorArray);
WX_DECLARE_STRING_HASH_MAP( wxImage* , wxImageStrHash );

WX_DECLARE_HASH_MAP( int, IntArray, wxIntegerHash, wxIntegerEqual, IntArrayHash);
//WX_DECLARE_HASH_MAP( int, wxVariantHash, wxIntegerHash, wxIntegerEqual, Int2wxVariantHashHash);

WX_DECLARE_HASH_MAP( int, wxCheckBox*, wxIntegerHash, wxIntegerEqual, Int2wxCheckBoxArray);
WX_DECLARE_HASH_MAP( int, bcCheckBox*, wxIntegerHash, wxIntegerEqual, Int2bcCheckBoxArray);

WX_DECLARE_HASH_MAP( int, wxBitmap* , wxIntegerHash, wxIntegerEqual, wxInt2BitmapPtrHash );


class MultiValueGraph: public VemsWidgetBase, public BitmapCanvasParentBase
{
    DECLARE_DYNAMIC_CLASS(MultiValueGraph);

    public:
        MultiValueGraph();
        MultiValueGraph( wxWindow* parent, wxWindowID id, GaugeDescManager * gdmgr, GaugeProperties &prop, LogHistoryBase * lhb = NULL );
        virtual ~MultiValueGraph();

        virtual wxRegion GetShape();
        virtual wxRegion GetBorderRegion();
        virtual void UpdateSize();
        virtual void setSize(int x , int y);

        virtual void SetValue( double value );
        virtual void SetNewTheme(WidgetThemeTable *theme);


        virtual void require();
        virtual void refresh(DoubleHash* table);
        virtual void OnUpdateValue( UpdateViewerEvent& event );

        void SetLogHistoryMgr( LogHistoryBase * lhb ) { m_dataMgr = lhb; } ;

        virtual void InitGauge();

        bool SendToolBarCommand(const wxString& cmd, wxVariant var = 0L);

        void RedrawWidget();

        void RefreshRTSymbols();

    protected:

        void OnPaint(wxPaintEvent &evt);
        void OnSize(wxSizeEvent &evt);
        void OnKeyDown(wxKeyEvent &evt);
        void OnKeyUp(wxKeyEvent &evt);
        void OnLeftDown(wxMouseEvent &evt);
        void OnLeftUp(wxMouseEvent &evt);
        void OnMouseMove(wxMouseEvent& evt);
        void OnMouseRightDown(wxMouseEvent& evt);
        void OnScrollBar(wxScrollEvent& evt);
        void OnButton(wxCommandEvent& evt);
        void OnPopupMenuSelectDesc(wxCommandEvent& evt);
        void OnPopupMenuSelectTheme(wxCommandEvent& evt);
        void OnPopupMenuSelectMarkers(wxCommandEvent& evt);
        void OnPopupMenuSelectSymbol(wxCommandEvent& evt);
        void OnPopupMenu(wxCommandEvent& evt);
        void OnEnter(wxMouseEvent& evt);
        void OnLeave(wxMouseEvent& evt);
        void OnRefreshTimer(wxTimerEvent& evt);
        void OnSearchResultBox(GeneralEvent& evt);

        bool IsSelectModKey( wxKeyEvent &evt );

        void SetVariables();
        void Init();
        void InitStatusBar();
        void SetStatusBarLayout();
        void SetScrollBar();
        void InitMarkerDesc();
        void InitToolTipper();
        void CreatePopupMenu(wxMenu& parent);
        void SetThemeVars();
        void SearchDlg(const wxString& searchfor = wxEmptyString);

        void KlappeToolbar();
        void ClearNumValueBoxes();
        void RedrawGraph(); /// redraw a full-size graph
        void ClearTitleWidths();
        void ClearDescExtraFields();
        void RegisterSymbols();
        void UnRegisterSymbols();
        void GaugeRefresh();
        void SetGraphCursor(int cursorid = -1);

        void DrawBackgroundFaceAgg(const wxString& underlinedsymbol = wxEmptyString);
        void DrawGraph(DoubleHashArray& datas, GraphDataDescriptorArray* outerarr = NULL);
        void DrawCursor();
        void DrawNumValues(bool repos = false, bool forcepaint = false);
        void DrawNumValues(DoubleHash* data, bool repos = false, bool forcepaint = false);
        void DrawHelpPanel();
        void DrawTimeRuler();
        void DrawCurrentTime(DoubleHash * data);
        void DrawSelectArea();
        void DrawSummaryBar(int highlightMarkerIdx = -1);
        void DrawMarkerLines();
        void DrawMarkerToolTip(int idx);
        GraphDataDescriptor* DrawData(wxAgg& aggCanvas, DoubleHash * currData, DoubleHash * prevData);

        void PaintMarkers(wxDC& dc, int w, int h, int li, int ri, int step);
        void PaintCursor(wxDC& dc,int w, int li, int ri, int step);
        void PaintNumValues(wxDC& dc);
        void PaintSelectedArea(wxDC& dc, int li, int ri, int step);
        void PaintToolTip(wxDC& dc, int h);


        bool MagnifyGraph(int delta);
        bool MagnifyGraphTo(int mag_target, int res_target);
        bool MagnifyGraphBetween(size_t start_idx, size_t end_idx);
        void MagnifySymbol(short direction);
        void MoveCursor(int delta, bool isSelect = false);
        void MoveCursorTo(int idx);
        bool MoveScreenTo(int scrollpos);
        void TabToNextSymbol();
        void TabToPrevSymbol();
        void JumpCursorToMarker(bool isNext = true);
        void OffsetSymbol(short dir);

        GraphDataDescriptor* NormalizeCursor();
        void RefreshGUILayout();
        void RefreshSelectedTimeInfoBar();
        void RefreshSelectionByMousePos(int xpos, bool isStartSelection = false, bool isRefreshGauges = true);
        void RefreshMagnifyVal();
        void RefreshGraph();
        void RefreshHistory();
        void FitGraphToScreen();
        void SetZoomBox();
        void SetToolBarZoomSelect();
        void UpdateToolbarButton();


        void CalculateGraphDataPositions();
        void RemoveUnneededGraphData();


        size_t  getHistoryLen();
        int     getStep();
        int     getScreenDataCount();
        int     getScreenDataMaxLen();
        int     getMaxResolution();
        bool    IsSlimData();
        bool    IsInRect(const wxRect& rect, const wxPoint& point );
        bool    IsNullGraph();
        bool    IsECUOnline();


        int GetLeftIdx();
        int GetRightIdx();
        int GetDataLen();
        int GetHeaderSize();
        int GetFooterSize();

        GraphDataDescriptor* RenderBitmapIdx( wxAgg& aggCanvas, int idx = -1, DoubleHash * rtdata = NULL );

        GraphDataDescriptor* SearchGraphDataByIdx(int idx);
        GraphDataDescriptor* SearchNearlyGraphDataByIdx(int idx);
        GraphDataDescriptor* SearchNearlyGraphDataByPosition(int x_pos);

        GraphDataDescriptor* SearchBGraphDataByIdx(int idx);
        GraphDataDescriptor* SearchBNearlyGraphDataByIdx(int idx);
        GraphDataDescriptor* SearchBNearlyGraphDataByPosition(int x_pos);

        wxString SearchNextSymbol();
        wxString SearchPrevSymbol();

        wxVariantHash* SearchSymbolByName(const wxString& name);

        wxColor     GetOverColor(wxVariantHash &hash);
        wxColor     GetSymbolColor(wxVariantHash &hash);
        void        ClearGraphDataArray();
        IntArray    GetNeededIdxs(int from, int to);
        wxString    FormatTimeSpan(int time_millisec, bool isminsize=true);
        int         getTimeMs(DoubleHash * data);
        double      GetRelativePos(int top, double value);
        wxRealPoint GetRelativePos(int top, const wxRealPoint& pos);
        wxVariantHash& GetPropHash(wxVariantHash& descHash);
        wxString    GetHoverSymbolByPos(const wxPoint& pos);

        wxMenu* GetSymbolChooserMenu();


        void    SetSymbolMagnify(const wxString& symbolname = wxEmptyString, double value = 1.0);
        double  GetSymbolMagnify(const wxString& symbolname);

        void MakeGraphBitmap(bool isCalcPositions = true);


        void    MouseCursor(int mouse_state, wxMouseEvent &evt);

        MultiGraphDesc* GetDesc() { return GetDescMgr()->GetMultiGraphDesc(m_profileId);  };
        MultiGraphDescManager * GetDescMgr() { return m_gaugeDescMgr->GetMultiGraphDescMgr(); }

        MultiGraphTheme* GetTheme() { return GetDescMgr()->GetMultiGraphTheme(m_themeId);  };

        static int CompareInts(int* n1, int* n2);

        bool CheckData(DoubleHash * data, const wxString& name);


        void OpenDescEditor(long flags = 0);
        void OpenThemeEditor();

        enum ODE_Flags {
            ODE_NODE = 1,
            ODE_SYMBOL = 2
        };

    private:
        ///for tests
        DoubleArray m_dataHeights;
        double m_cellHeight;

        wxString CursorStateToStr();
        void SetCursorState(const wxPoint& pos);

        double GetSymbolYPos(double value, double minV, double maxV, double cellMin, double cellMax);

        enum{
            idScrollBar,
            idTbButton,
            idRefreshTimer,
            idTestTimer,
            idPopupmenuOpenDescEditor,
            idPopupmenuOpenNodeDescEditor,
            idPopupmenuOpenSymbolDescEditor,
            idPopupmenuOpenThemeEditor,
            idPopupmenuSearchFor,
            idPopupMenuEND
        };

        enum CursorState{
            MGCS_DEFAULT = 0,
            MGCS_OVER_SYMBOL_TITLE = 1,
            MGCS_OVER_GRAPH = 2,
            MGCS_OVER_HELPPANEL = 4,
            MGCS_OVER_SELECTION = 8,
            MGCS_OVER_SELECT_RESIZE_START = 16,
            MGCS_OVER_SELECT_RESIZE_CURRENT = 32,
            MGCS_OVER_SUMMARY_BAR = 64,
            MGCS_IN_SELECT = 128,
            MGCS_IN_SELECT_RESIZE_START = 256,
            MGCS_IN_SELECT_RESIZE_CURRENT = 512,
            MGCS_IN_SELECT_MOVE = 1024,
            MGCS_HIGHLIGHT_MARKER = 2048

        };

        enum MouseState{
            MSC_LEFT_DOWN,
            MSC_LEFT_UP,
            MSC_RIGHT_DOWN,
            MSC_RIGHT_UP,
            MSC_MOVE
        };

        enum ATTExtraTypes {
            ATTET_MARKERPOS = 4200
        };

        enum {
            MGBMP_DUMMY_FIRST = 1,

            MGBMP_FACE,
            MGBMP_CURSOR,
            MGBMP_TIMERULER,
            MGBMP_DYNAMIC_CAPTION,
            MGBMP_CURRENT_TIME,
            MGBMP_SELECTED_FIELD,
            MGBMP_GRAPH,
            MGBMP_SUMMARY,
            MGBMP_TOOLTIP,

            MGBMP_DUMMY_LAST
        };

        wxInt2BitmapPtrHash m_layerBitmaps;

        BubbleStatusBar * m_statusBarMgr;

        LogHistoryBase * m_dataMgr;

        bool m_isRegistered;

//        wxBitmap*   m_bgFaceBmp;
//        wxBitmap*   m_cursorBmp;
//        wxBitmap*   m_timeRulerBmp;
//        wxBitmap*   m_dynamicCaptionBmp;
//        wxBitmap*   m_currentTimeBmp;
//        wxBitmap*   m_selectedFieldBmp;
//        wxBitmap*   m_graphBitmap;
//        wxBitmap*   m_summaryBarBmp;
//        wxBitmap*   m_toolTipBmp;

        //DoubleHashArray     m_dataHistory;



        DoubleHash*         m_prevValue;

        //wxButton    *   m_tbButton;
        bcButton*       m_tbButton;

        //wxBitmapIdxArray    m_bmpHistory;

        bool    m_inited;

        IntArrayHash    m_markerList;

        GraphDataDescriptorArray    m_graphData;

        int     m_magnifyRatio;  /// pixel / data   ( m_magnifyRatio = "data width" )    if data == 1
        int     m_resolution;    /// every n-th data on 1 pixel  ( n = m_resolution )

        int     m_maxMagnify;

        int     m_cellsepar;

        int     m_toolBarSize;
        int     m_captionSize;
        int     m_scrollBarSize;
        int     m_timeRulerSize;
        int     m_statusBarSize;
        int     m_summaryBarSize;
        int     m_resStep;
        int     m_magStep;


        double  m_numValueStroke;
        double  m_numValueHeight;

        double  m_titleStroke;
        double  m_titleHeight;

        double  m_limitValStroke;
        double  m_limitValHeight;

        double  m_cursorWidth;

        int     m_cursorDelta;
        int     m_cursorAccel;

        int     m_dataWidthLimitVert;

        int     m_minDataSize;

        wxHotKeyManager *   m_hotkeyMgr;

        NumValueBoxesHash   m_numBoxes;

        wxRegionHash        m_titleRegions;
        wxRectHash          m_markerSummaryRects;

        //DoubleHash          m_symbolMagnifyRatios;
        Short2StringHash    m_symbolOffsets;

        wxString            m_selectedMSymbol;



        wxRect              m_graphRect;

        int     m_highlightedMarkerIdx;


        int     m_currentSelectedDataIdx;

        int     m_startSelectedDataIdx;

        bool    m_isLocked;

        size_t  m_historyLockLen;


        wxScrollBar *   m_scrollBar;

        Int2wxVariantHashHash   m_symbolTmpProps;

        wxStopWatch sp;

        wxSize      m_lastSize;

        //wxPoint     m_startPosToSelect;

        bool        m_isInSelect;
        bool        m_isNeedCalcBmpPositions;


        size_t      m_minDataLimit;
        int         m_helptextlen;

        bool        m_isCursorMoved;

        long        m_cursorState;
        long        m_prevCursorState;

        long        m_profileId;
        long        m_themeId;

        int         m_lockedHistoryLen;

        wxPoint     m_lastMousePos;

        wxString    m_hoverSymbol;
        wxString    m_tmphoverSymbol;

        MVG_ToolbarPanel * m_toolbar;

        Int2wxVariantHashHash   m_markerTypes;


        AggToolTip  *  m_toolTipMgr;

        Int2IntHash m_menuID2profID;
        Int2IntHash m_menuID2themeID;
        Int2StringHash m_menuID2symbolname;

        int         m_sizeCursorWidth;
        int         m_currWinCursor;

        wxTimer     m_refreshTimer;
        bool        m_isRefresh;
        size_t      m_refreshTimeBase;

        bcLabel*    m_helpPanel;


        SectionArray        m_searchResultSections;

        void    OnSetFocus(wxFocusEvent& evt);

        wxTimer     m_testTimer;



        DECLARE_EVENT_TABLE();

};

class GraphDataDescriptor
{
    public:
        GraphDataDescriptor(DoubleHash* data, int idx, int width, int xpos=0);
        ~GraphDataDescriptor();

        void AddImage(const wxString& name, wxImage * image, int y_position);

        bool MergeToImage(wxImage& targetImg);

        int GetIdx() { return m_logIdx; };
        int GetXPos() { return m_xPos; };

        void SetXPos(int p) { m_xPos = p; } ;

        DoubleHash* GetRTData() { return m_rtData; };

        wxImageStrHash& GetImages() { return m_imgs; };

    private:

        int m_xPos;
        int m_logIdx;
        int m_width;

        String2IntHash  m_yPos;
        wxImageStrHash  m_imgs;
        DoubleHash*     m_rtData;

};

class NumValueBoxes
{
    public:
        NumValueBoxes();
        virtual ~NumValueBoxes();

        bool IsChangedValue(double currentval);

        void SetBitmap(wxBitmap * bmp);

        wxBitmap *  BmpBox;
        wxRealPoint Position;
        wxString    Name;

        double      m_lastValue;


};

class MarkerSettingsDialog: public bcParentDialog
{
    public:
        MarkerSettingsDialog(wxWindow* parent, Int2wxVariantHashHash& markerTypes);

        void SetToHash(Int2wxVariantHashHash& markerTypes);

    private:


        Int2bcCheckBoxArray m_cbs;


};

#endif // MULTIVALUEGRAPH_H
