
/**
 VEMSTableEditor.cpp - multifunctional table for VEMS

 Author: Roma 'TURBO'
*/

// For compilers that support precompilation, include "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif


#include "VEMSTableEditor.h"
#include "../events/VEMSTableEditorEvents.h"
#include "../gaugedesc/GaugeProperties.h"
#include "wx/scrolwin.h"
#include "wx/generic/scrolwin.h"
#include "wx/dcbuffer.h"
#include "../logger.h"
#include "../DrawHelper.h"
#include "../Math.h"
#include "../DirHelper.h"
#include <wx/arrimpl.cpp> // this is a magic incantation which must be done!

#include "../gaugedesc/GaugeDescManager.h"
#include "../hotkeys/wxHotKeyManager.h"


#include "../common/VemsMatrixHelper.h"
#include "GaugeGroupManager.h"
#include "../WindowHelper.h"
#include "TableEditorPanel.h"
#include "../wxPointEx.h"
#include "../agg/wxAgg.h"

#include "FocusParentSinkerBase.h"



WX_DEFINE_OBJARRAY(GradientColorArray);
WX_DEFINE_OBJARRAY(PointArray);

//
// VEMSTableEditor event table
//
BEGIN_EVENT_TABLE(VEMSTableEditor, wxControl)
    EVT_PAINT(VEMSTableEditor::OnPaint)
    EVT_ERASE_BACKGROUND(VEMSTableEditor::OnEraseBackground)
	EVT_KEY_DOWN(VEMSTableEditor::OnKeyDown)
	//EVT_KEY_UP( VEMSTableEditor::OnKeyUp )
    EVT_LEFT_DOWN(VEMSTableEditor::OnMouseLeftClick)

	EVT_LEFT_DCLICK(VEMSTableEditor::OnMouseLeftDoubleClick)
	EVT_RIGHT_DOWN(VEMSTableEditor::OnMouseRightClick)

	EVT_MOTION(VEMSTableEditor::OnMouseMove)
	EVT_LEFT_UP(VEMSTableEditor::OnMouseLeftUp)

	EVT_SET_FOCUS(VEMSTableEditor::OnSetFocus)
	EVT_KILL_FOCUS(VEMSTableEditor::OnKillFocus)


	EVT_MENU(VEMS_TABLE_EDITOR_PM_MULTIPLY_CELLS, VEMSTableEditor::OnPopupMenuMultiply)
	EVT_MENU(VEMS_TABLE_EDITOR_PM_DIVIDE_CELLS , VEMSTableEditor::OnPopupMenuDivide)
	EVT_MENU(VEMS_TABLE_EDITOR_PM_INC_CELL_LARGE, VEMSTableEditor::OnPopupMenuIncCellLarge)
    EVT_MENU(VEMS_TABLE_EDITOR_PM_DEC_CELL_LARGE, VEMSTableEditor::OnPopupMenuDecCellLarge)
	EVT_MENU(VEMS_TABLE_EDITOR_PM_INC_CELL_SMALL, VEMSTableEditor::OnPopupMenuIncCellSmall)
    EVT_MENU(VEMS_TABLE_EDITOR_PM_DEC_CELL_SMALL, VEMSTableEditor::OnPopupMenuDecCellSmall)
	EVT_MENU(VEMS_TABLE_EDITOR_PM_LOCK_CELL, VEMSTableEditor::OnPopupMenuLockCell)
	EVT_MENU(VEMS_TABLE_EDITOR_PM_UNLOCK_CELL, VEMSTableEditor::OnPopupMenuUnlockCell)
	EVT_MENU(VEMS_TABLE_EDITOR_PM_UNDO_CELL_MODIFICATION, VEMSTableEditor::OnPopupMenuUndoCellModification)
	EVT_MENU(VEMS_TABLE_EDITOR_PM_UNDO_ALL_CELLS_MODIFICATION, VEMSTableEditor::OnPopupMenuUndoAllCellsModification)
	EVT_MENU(VEMS_TABLE_EDITOR_PM_LOCK_ALL_CELLS, VEMSTableEditor::OnPopupMenuLockAllCells)
	EVT_MENU(VEMS_TABLE_EDITOR_PM_UNLOCK_ALL_CELLS, VEMSTableEditor::OnPopupMenuUnlockAllCells)
	EVT_MENU(VEMS_TABLE_EDITOR_PM_ACCEPT_ALL_CHANGES,  VEMSTableEditor::OnPopupMenuAcceptAllChanges)
	EVT_MENU(VEMS_TABLE_EDITOR_PM_EDIT_HEADER,  VEMSTableEditor::OnPopupMenuEditHeader)
	EVT_MENU(VEMS_TABLE_EDITOR_PM_MOVE_HEADER,  VEMSTableEditor::OnPopupMenuEditHeader)
	EVT_MENU(VEMS_TABLE_EDITOR_PM_INSERT_HEADER,  VEMSTableEditor::OnPopupMenuEditHeader)
	EVT_MENU(VEMS_TABLE_EDITOR_PM_IMPORT,  VEMSTableEditor::OnPopupMenuImport)
	EVT_MENU(VEMS_TABLE_EDITOR_PM_REMAP,  VEMSTableEditor::OnPopupMenuRemap)
	EVT_SIZE(VEMSTableEditor::OnSize)
END_EVENT_TABLE()

//
// VEMSTableEditor CTOR
//
IMPLEMENT_DYNAMIC_CLASS(VEMSTableEditor, wxControl);

VEMSTableEditor::VEMSTableEditor( wxWindow *parent, GaugeDescManager * dmgr, GaugeProperties &prop)
                //const wxPoint &point, const wxSize &size, GaugeGroupManager* groupManager, const wxString& groupName )
                                  : wxControl( parent, wxID_ANY, prop.getPos(), prop.getSize(), wxNO_BORDER | wxWANTS_CHARS ),
                                  GroupMember( parent->GetParent(), prop, dmgr->GetGaugeGrpMgr(), parent->GetParent()->GetId() )

{
    //Logger::logf(LOG_MSG,_T("VEMSTableEditor::VEMSTableEditor ctor START"));
    m_helpPanelBmp = NULL;

    m_parent = NULL;

    m_focusSinker = NULL;

    m_delta = wxDefaultPosition;
    m_refreshAllCells = true;
    m_isChange = true;
    m_isLoadChange = true;
    m_isSizeChanging = false;

    m_currentRefreshCells.Clear();
    m_oldRefreshCells.Clear();

    m_currentRefreshCellsforLoad.Clear();
    m_oldRefreshCellsforLoad.Clear();

    m_face = NULL; //new wxBitmap(800,400);

	m_numRows = 0;
	m_numCols = 0;

    //CalcWidgetSize(prop.getSize());

	SetDefaults();
/*
	RecalcSizes();

	CreateCells(0, 0);
*/
	m_popupMenuTitle = wxEmptyString;


    m_clientAR = (double)GetClientSize().x / (double)GetClientSize().y;
    m_parentAR = (double)parent->GetSize().x / (double)parent->GetSize().y;

    m_parent = parent;

    m_lastSize = parent->GetSize();

  //  SetInitialSize(wxSize(300,300));
    //SetSize(m_widgetSize);

    m_selectedColLabel  = -1;
    m_selectedRowLabel  = -1;

    m_minRowHeaderValue = 0.0;
    m_maxRowHeaderValue = 0.0;
    m_minColHeaderValue = 0.0;
    m_maxColHeaderValue = 0.0;



    //m_hotkeyMgr = new wxHotKeyManager(DirHelper::GetPath(dHOTKEYS) + _T("2dtableeditorhkeys.db"));

    m_hotkeyMgr = new wxHotKeyManager(this);

    m_isAutoFollowLoad = false;
    m_isShowHelp = false;
    m_isResized = false;
}

//
// VEMSTableEditor DTOR
//

VEMSTableEditor::~VEMSTableEditor()
{
    if ( m_hotkeyMgr )
    {
        delete m_hotkeyMgr;
        m_hotkeyMgr = NULL;
}
    if ( m_face )
    {
        delete m_face;
        m_face = NULL;
    }
    if ( m_helpPanelBmp )
    {
        delete m_helpPanelBmp;
        m_helpPanelBmp = NULL;
    }
}

//
// VEMSTableEditor class methods
//

void VEMSTableEditor::UpdateSize()
{
    m_isResized = true;
    if(GetSize().x < 150) return;
    //if(m_isSizeChanging) return;
    m_isSizeChanging = true;

    RecalcSizes();

    CalcWidgetSize(GetSize());

    m_refreshAllCells = true;

    delete m_face;
    m_face = NULL;

    SelectableWidgetBase* selw = GetSelectableWidgetBase();
    if(selw) selw->DrawSelectionBitmap(this);

    Refresh(false);

//    m_lastSize = size;

    m_isSizeChanging = false;
}

void VEMSTableEditor::OnKillFocus(wxFocusEvent &evt)
{
    if(m_focusSinker) m_focusSinker->OnFocusMsg( FPST_KILLFOCUS );
}

void VEMSTableEditor::OnSetFocus(wxFocusEvent &evt)
{
    if(m_focusSinker) m_focusSinker->OnFocusMsg( FPST_SETFOCUS );
}

void VEMSTableEditor::OnSize(wxSizeEvent &evt)
{
    UpdateSize();
}

void VEMSTableEditor::DeleteAllCells()
{
	m_numRows = 0;
	m_numCols = 0;

	//memset(m_cellValues, 0, sizeof(m_cellValues));
	//memset(m_cellBakValues, 0, sizeof(m_cellBakValues));
	m_cellValues.fill( 0.0 );
	m_cellBakValues.fill( 0.0 );
}

bool VEMSTableEditor::CreateCells(int numrows, int numcols)
{
	wxASSERT(numcols >= 0 && numrows >= 0);
	wxASSERT(numcols <= VEMS_TABLE_EDITOR_MAX_COLS);
	wxASSERT(numrows <= VEMS_TABLE_EDITOR_MAX_ROWS);

	m_currentMarkerLoad = 0;
	m_currentMarkerRpm = 0;
	m_showLoadMarker = false;

	m_numRows = numrows;
	m_numCols = numcols;

	//vemsMemset(m_cellValues, 0, sizeof(m_cellValues));
	//vemsMemset(m_cellBakValues, 0, sizeof(m_cellBakValues));
	m_cellValues.setSize( numrows, numcols );
	m_cellValues.fill( 0.0 );

	m_cellBakValues.setSize( numrows, numcols );
	m_cellBakValues.fill( 0.0 );

	vemsMemset(m_cellLockFlag, 0, sizeof(m_cellLockFlag));
	vemsMemset(m_cellHighlightFlag, 0, sizeof(m_cellHighlightFlag));

	/// set dimensions of headers
	m_rowLabels.setSize( 1, numrows );
	m_rowLabelsBak.setSize( 1, numrows );
	m_colLabels.setSize( 1, numcols );
	m_colLabelsBak.setSize( 1, numcols );

	m_drawOriginRow = 0, m_drawOriginCol = 0;
	m_highlightOriginRow = -1, m_highlightOriginCol = -1;

	RecalcSizes();

	SetInitialSize(m_widgetSize);

	Refresh();

    //m_refreshAllCells = false;

	return true;
}

bool VEMSTableEditor::SetColLabelValue(int col, double value)
{
	wxASSERT(col >= 0 && col < GetNumberCols());
	if (col < 0 || col >= GetNumberCols())
	{
		return false;
	}

    double d = 0.5;
    if(value < 0){
        d =0.0;
    }
    m_colLabels[col] = (int)(value + d);
	AddNewRefreshColLabelIndex( col );
	return true;
}


void VEMSTableEditor::AcceptModifiedColLabelValue( int col )
{
    m_colLabelsBak.set( 0, col, m_colLabels.get( 0, col ) );
}


int VEMSTableEditor::GetColLabelValue(int col)
{
	wxASSERT(col >= 0 && col < GetNumberCols());
	return m_colLabels[col];
}

bool VEMSTableEditor::SetRowLabelValue(int row, double value)
{
	wxASSERT(row >= 0 && row < GetNumberRows());
	if (row < 0 || row >= GetNumberRows())
	{
		return false;
	}
    //Logger::logf(LOG_MSG,_T("VEMSTableEditor::SetRowLabelValue %f"), value);
     double d = 0.5;
    if(value < 0){
        d =0.0;
    }
	m_rowLabels[row] = (int)(value + d);
	AddNewRefreshRowLabelIndex( row );
	return true;
}


void VEMSTableEditor::AcceptModifiedRowLabelValue( int row )
{
    m_rowLabelsBak.set( 0, row, m_rowLabels.get( 0, row ) );
}


int VEMSTableEditor::GetRowLabelValue(int row)
{
	wxASSERT(row >= 0 && row < GetNumberRows());
	return m_rowLabels[row];
}

void VEMSTableEditor::SetCellsMinValue(float v)
{
	m_minCellValue = v;
}

float VEMSTableEditor::GetCellsMinValue() const
{
	return m_minCellValue;
}

void VEMSTableEditor::SetCellsMaxValue(float v)
{
	m_maxCellValue = v;
}

float VEMSTableEditor::GetCellsMaxValue() const
{
	return m_maxCellValue;
}

float VEMSTableEditor::GetCellValue(int row, int col)
{
	wxASSERT(col >= 0 && col < GetNumberCols());
	wxASSERT(row >= 0 && row < GetNumberRows());

	//return m_cellValues[row][col];
	return m_cellValues.get( row, col );
}

bool VEMSTableEditor::SetCellValue(int row, int col, float v)
{
//    Logger::logf( LOG_MSG, _T("VEMSTableEditor::SetCellValue() - row: %d, col: %d, value: %f"), row, col, v );

	wxASSERT(col >= 0 && col < GetNumberCols());
	wxASSERT(row >= 0 && row < GetNumberRows());

	if (v < m_minCellValue || v > m_maxCellValue)
	{
		return false;
	}

	wxString conv;
	double conv_double = 0;
	conv.sprintf(GetValueFormatMask(), v);
	conv.ToDouble(&conv_double);

	//m_cellValues[row][col] = float(conv_double);
	m_cellValues.set( row, col , conv_double );

    AddNewRefreshCell(row, col);


	return true;
}

bool VEMSTableEditor::WasCellModified(int row, int col) const
{
	wxASSERT(col >= 0 && col < GetNumberCols());
	wxASSERT(row >= 0 && row < GetNumberRows());

	wxString val1, val2;

	//val1.sprintf(GetValueFormatMask(), m_cellValues[row][col]);
	//val2.sprintf(GetValueFormatMask(), m_cellBakValues[row][col]);
	val1.sprintf(GetValueFormatMask(), m_cellValues.get( row, col ) );
	val2.sprintf(GetValueFormatMask(), m_cellBakValues.get( row, col ) );

	return val1 == val2 ? false : true;
}


bool VEMSTableEditor::WasRowLabelModified( int row )
{
    int currentValue =  m_rowLabels.get( 0, row );
    int backupValue  = m_rowLabelsBak.get( 0, row );

    return currentValue != backupValue;
}


bool VEMSTableEditor::WasColLabelModified( int col )
{
    int currentValue = m_colLabels.get( 0, col );
    int backupValue = m_colLabelsBak.get( 0, col );

    return currentValue != backupValue;
}


void VEMSTableEditor::AcceptAllModifiedCells()
{
	//memcpy(m_cellBakValues, m_cellValues, sizeof(m_cellBakValues));
	//Logger::logf( LOG_MSG, _T("VEMSTableEditor::AcceptAllModifiedCells()") );

	VemsMatrixHelper::resize( m_cellValues, m_cellBakValues );
	m_refreshAllCells = true;
}


void VEMSTableEditor::AcceptSingleModifiedCell( int row, int col )
{
    //m_cellBakValues[ row ][ col ] = m_cellValues[ row ][ col ];
    //Logger::logf( LOG_MSG, _T("VEMSTableEditor::AcceptSingleModifiedCell()") );
    m_cellBakValues.set( row, col, m_cellValues.get( row, col ) );
}

void VEMSTableEditor::SetCurrentLoad(float load, float rpm)  /// set cell new value
{
    /// old value
    const double EPSILON = 0.000001;

    if ( ( fabs( m_currentMarkerLoad - load ) < EPSILON ) && ( fabs( m_currentMarkerRpm - rpm ) < EPSILON ) ) return;

	m_currentMarkerLoad = load;
    m_currentMarkerRpm = rpm;
	m_showLoadMarker = true;


    //Logger::logf(LOG_MSG,_T("m_currentMarkerRpm = %.2f, m_currentMarkerLoad = %.2f"),m_currentMarkerRpm,m_currentMarkerLoad );

    /// new value

    CalculateLoadCellPoints();

	Refresh();


}

void VEMSTableEditor::ClearCurrentLoad()
{
	m_currentMarkerLoad = 0;
	m_currentMarkerRpm = 0;
	m_showLoadMarker = false;
	Refresh();
}

int VEMSTableEditor::GetNumberRows() const
{
	return m_numRows;
}

int VEMSTableEditor::GetNumberCols() const
{
	return m_numCols;
}

bool VEMSTableEditor::SetValuePrecision(int width, int precision)
{
	wxASSERT(width > 0);
	//wxASSERT(precision > 0);

	if (width > 0 && precision >= 0)
	{
		m_valueWidth = width;
		m_valuePrecision = precision;
		RecalcSizes();
		return true;
	}

	return false;
}

void VEMSTableEditor::GetValuePrecision(int *width, int *precision)
{
	if (width != 0)
	{
		*width = m_valueWidth;
	}

	if (precision >= 0)
	{
		*precision = m_valuePrecision;
	}
}

bool VEMSTableEditor::SetSmallStepValue(float small_step)
{
	if (small_step > 0)
	{
		m_smallStepValue = small_step;
		return true;
	}

	return false;
}

float VEMSTableEditor::GetSmallStepValue() const
{
	return m_smallStepValue;
}

bool VEMSTableEditor::SetLargeStepValue(float large_step)
{
	if (large_step > 0)
	{
		m_largeStepValue = large_step;
		return true;
	}

	return false;
}

float VEMSTableEditor::GetLargeStepValue() const
{
	return m_largeStepValue;
}

bool VEMSTableEditor::SetKey(unsigned int func, int keycode, int modifiers)
{
	wxASSERT(func >= 0 && func < sizeof(m_keyMap) / sizeof(m_keyMap[0]));

	if (func >= 0 && func < sizeof(m_keyMap) / sizeof(m_keyMap[0]))
	{
		m_keyMap[func].keycode = keycode;
		m_keyMap[func].modifiers = modifiers;
		return true;
	}

	return false;
}

int VEMSTableEditor::GetKeycode(unsigned int func) const
{
	wxASSERT(func >= 0 && func < sizeof(m_keyMap) / sizeof(m_keyMap[0]));

	if (func >= 0 && func < sizeof(m_keyMap) / sizeof(m_keyMap[0]))
	{
		return m_keyMap[func].keycode;
	}

	return 0;
}

int VEMSTableEditor::GetKeyModifiers(unsigned int func) const
{
	wxASSERT(func >= 0 && func < sizeof(m_keyMap) / sizeof(m_keyMap[0]));

	if (func >= 0 && func < sizeof(m_keyMap) / sizeof(m_keyMap[0]))
	{
		return m_keyMap[func].modifiers;
	}

	return 0;
}

bool VEMSTableEditor::SetCellFont(const wxFont& font)
{
	wxASSERT(font.IsOk());

	if (font.IsOk())
	{
		m_cellFont = font;
		RecalcSizes();
		return true;
	}

	return false;
}

wxFont VEMSTableEditor::GetCellFont() const
{
	return m_cellFont;
}

void VEMSTableEditor::SetCellTextColor(const wxColor& clr)
{
	m_cellTextColor = clr;
}

wxColor VEMSTableEditor::GetCellTextColor() const
{
	return m_cellTextColor;
}

bool VEMSTableEditor::SetLabelFont(const wxFont& font)
{
	wxASSERT(font.IsOk());

	if (font.IsOk())
	{
		m_labelFont = font;
		RecalcSizes();
		return true;
	}

	return false;
}

wxFont VEMSTableEditor::GetLabelFont() const
{
	return m_labelFont;
}

void VEMSTableEditor::SetLabelTextColor(const wxColor& clr)
{
	m_labelTextColor = clr;
}

wxColor VEMSTableEditor::GetLabelTextColor() const
{
	return m_labelTextColor;
}

bool VEMSTableEditor::SetZoomboxFont(const wxFont& font)
{
	wxASSERT(font.IsOk());

	if (font.IsOk())
	{
		m_zoomboxFont = font;
		RecalcSizes();
		return true;
	}

	return false;
}

wxFont VEMSTableEditor::GetZoomboxFont() const
{
	return m_zoomboxFont;
}

void VEMSTableEditor::SetZoomboxTextColor(const wxColor& clr)
{
	m_zoomboxTextColor = clr;
}

wxColor VEMSTableEditor::GetZoomboxTextColor() const
{
	return m_zoomboxTextColor;
}

wxString VEMSTableEditor::GetValueFormatMask() const
{

	wxString val_mask;
	val_mask.sprintf(_T("%%%d.%df"), m_valueWidth, m_valuePrecision);
	return val_mask;

/*
    wxString val_mask;

    if( m_valuePrecision == 0 )
    {
        val_mask.sprintf(_T("%%%df"), m_valueWidth );
    }
    else
    {
        val_mask.sprintf(_T("%%%d.%df"), m_valueWidth, m_valuePrecision );
    }

    return val_mask;
*/
}



bool VEMSTableEditor::IsKeycodesEqual(int keycode1, int keycode2)
{
	if (wxIsprint(keycode1))
	{
		keycode1 = wxTolower(keycode1);
	}

	if (wxIsprint(keycode2))
	{
		keycode2 = wxTolower(keycode2);
	}

	return (keycode1 == keycode2) ? true : false;

}

bool VEMSTableEditor::IsKeyModifiersEqual(int wx_modifiers, int vems_table_modifiers)
{
	const bool is_ctrl_match = ((((wx_modifiers & wxMOD_CONTROL) != 0) && ((vems_table_modifiers & wxMOD_CONTROL) != 0)) ||
                               (((wx_modifiers & wxMOD_CONTROL) == 0) && ((vems_table_modifiers & wxMOD_CONTROL) == 0))) ? true : false;

	const bool is_shift_match = ((((wx_modifiers & wxMOD_SHIFT) != 0) && ((vems_table_modifiers & wxMOD_SHIFT) != 0)) ||
                               (((wx_modifiers & wxMOD_SHIFT) == 0) && ((vems_table_modifiers & wxMOD_SHIFT) == 0))) ? true : false;

	const bool is_alt_match = ((((wx_modifiers & wxMOD_ALT) != 0) && ((vems_table_modifiers & wxMOD_ALT) != 0)) ||
                               (((wx_modifiers & wxMOD_ALT) == 0) && ((vems_table_modifiers & wxMOD_ALT) == 0))) ? true : false;

	return is_ctrl_match && is_shift_match && is_alt_match;

}

int VEMSTableEditor::GetFunctionByKey(int keycode, int modifiers)
{
	for (size_t i = 0; i < sizeof(m_keyMap) / sizeof(m_keyMap[0]); ++i)
	{
		if (IsKeycodesEqual(keycode, m_keyMap[i].keycode) &&
			IsKeyModifiersEqual(modifiers, m_keyMap[i].modifiers))
		{
			return i;
		}
	}

	return -1;
}

wxString VEMSTableEditor::GetKeyTextualForFunction(int func) const
{
	wxASSERT(func >= 0 && func < VEMS_TABLE_EDITOR_KEY_MAX_INDEX);

	wxString keyStr;

	if (func >= 0 && func < VEMS_TABLE_EDITOR_KEY_MAX_INDEX)
	{
		const int keycode = m_keyMap[func].keycode;
		const int modifiers = m_keyMap[func].modifiers;

		if (modifiers & wxMOD_CONTROL)
		{
			keyStr += _T("Ctrl");
		}

		if (modifiers & wxMOD_ALT)
		{
			if (keyStr.Len() > 0)
			{
				keyStr += _T('+');
			}
			keyStr += _T("Alt");
		}
		if (modifiers & wxMOD_SHIFT)
		{
			if (keyStr.Len() > 0)
			{
				keyStr += _T('+');
			}
			keyStr += _T("Shift");
		}
		if (keyStr.Len() > 0)
		{
			keyStr += _T('+');
		}

	    switch (keycode)
	    {
	        case WXK_BACK: keyStr += _T("Backspace"); break;
			case WXK_TAB: keyStr += _T("Tab"); break;
			case WXK_RETURN: keyStr += _T("Enter"); break;
			case WXK_ESCAPE: keyStr += _T("Esc"); break;
			case WXK_SPACE: keyStr += _T("Space"); break;
			case WXK_DELETE: keyStr += _T("Del"); break;
			case WXK_START: keyStr += _T("Start"); break;
			case WXK_CLEAR: keyStr += _T("Clear"); break;
			case WXK_MENU: keyStr += _T("Menu"); break;
			case WXK_PAUSE: keyStr += _T("Pause"); break;
			case WXK_CAPITAL: keyStr += _T("CapsLock"); break;
			case WXK_END: keyStr += _T("End"); break;
			case WXK_HOME: keyStr += _T("Home"); break;
			case WXK_LEFT: keyStr += _T("Left"); break;
			case WXK_UP: keyStr += _T("Up"); break;
			case WXK_RIGHT: keyStr += _T("Right"); break;
			case WXK_DOWN: keyStr += _T("Down"); break;
			case WXK_SELECT: keyStr += _T("Select"); break;
			case WXK_PRINT: keyStr += _T("Print"); break;
			case WXK_EXECUTE: keyStr += _T("Executa"); break;
			case WXK_SNAPSHOT: keyStr += _T("Snapshot"); break;
			case WXK_INSERT: keyStr += _T("Insert"); break;
			case WXK_HELP: keyStr += _T("Help"); break;
			case WXK_NUMPAD0: keyStr += _T("NUMPAD0"); break;
			case WXK_NUMPAD1: keyStr += _T("NUMPAD1"); break;
			case WXK_NUMPAD2: keyStr += _T("NUMPAD2"); break;
			case WXK_NUMPAD3: keyStr += _T("NUMPAD3"); break;
			case WXK_NUMPAD4: keyStr += _T("NUMPAD4"); break;
			case WXK_NUMPAD5: keyStr += _T("NUMPAD5"); break;
			case WXK_NUMPAD6: keyStr += _T("NUMPAD6"); break;
			case WXK_NUMPAD7: keyStr += _T("NUMPAD7"); break;
			case WXK_NUMPAD8: keyStr += _T("NUMPAD8"); break;
			case WXK_NUMPAD9: keyStr += _T("NUMPAD9"); break;
			case WXK_MULTIPLY: keyStr += _T("Multiply"); break;
			case WXK_ADD: keyStr += _T("Add"); break;
			case WXK_SEPARATOR: keyStr += _T("Separator"); break;
			case WXK_SUBTRACT: keyStr += _T("Substract"); break;
			case WXK_DECIMAL: keyStr += _T("Decimal"); break;
			case WXK_DIVIDE: keyStr += _T("Divide"); break;
			case WXK_F1: keyStr += _T("F1"); break;
			case WXK_F2: keyStr += _T("F2"); break;
			case WXK_F3: keyStr += _T("F3"); break;
			case WXK_F4: keyStr += _T("F4"); break;
			case WXK_F5: keyStr += _T("F5"); break;
			case WXK_F6: keyStr += _T("F6"); break;
			case WXK_F7: keyStr += _T("F7"); break;
			case WXK_F8: keyStr += _T("F8"); break;
			case WXK_F9: keyStr += _T("F9"); break;
			case WXK_F10: keyStr += _T("F10"); break;
			case WXK_F11: keyStr += _T("F11"); break;
			case WXK_F12: keyStr += _T("F12"); break;
			case WXK_F13: keyStr += _T("F13"); break;
			case WXK_F14: keyStr += _T("F14"); break;
			case WXK_F15: keyStr += _T("F15"); break;
			case WXK_F16: keyStr += _T("F16"); break;
			case WXK_F17: keyStr += _T("F17"); break;
			case WXK_F18: keyStr += _T("F18"); break;
			case WXK_F19: keyStr += _T("F19"); break;
			case WXK_F20: keyStr += _T("F20"); break;
			case WXK_F21: keyStr += _T("F21"); break;
			case WXK_F22: keyStr += _T("F22"); break;
			case WXK_F23: keyStr += _T("F23"); break;
			case WXK_F24: keyStr += _T("F24"); break;
			case WXK_NUMLOCK: keyStr += _T("NumLock"); break;
			case WXK_SCROLL: keyStr += _T("Scroll"); break;
			case WXK_PAGEUP: keyStr += _T("PageUp"); break;
			case WXK_PAGEDOWN: keyStr += _T("PageDown"); break;
			case WXK_NUMPAD_SPACE: keyStr += _T("NUMPAD_SPACE"); break;
			case WXK_NUMPAD_TAB: keyStr += _T("NUMPAD_TAB"); break;
			case WXK_NUMPAD_ENTER: keyStr += _T("NUMPAD_ENTER"); break;
			case WXK_NUMPAD_F1: keyStr += _T("NUMPAD_F1"); break;
			case WXK_NUMPAD_F2: keyStr += _T("NUMPAD_F2"); break;
			case WXK_NUMPAD_F3: keyStr += _T("NUMPAD_F3"); break;
			case WXK_NUMPAD_F4: keyStr += _T("NUMPAD_F4"); break;
			case WXK_NUMPAD_HOME: keyStr += _T("NUMPAD_HOME"); break;
			case WXK_NUMPAD_LEFT: keyStr += _T("NUMPAD_LEFT"); break;
			case WXK_NUMPAD_UP: keyStr += _T("NUMPAD_UP"); break;
			case WXK_NUMPAD_RIGHT: keyStr += _T("NUMPAD_RIGHT"); break;
			case WXK_NUMPAD_DOWN: keyStr += _T("NUMPAD_DOWN"); break;
			case WXK_NUMPAD_PAGEUP: keyStr += _T("NUMPAD_PAGEUP"); break;
			case WXK_NUMPAD_PAGEDOWN: keyStr += _T("NUMPAD_PAGEDOWN"); break;
			case WXK_NUMPAD_END: keyStr += _T("NUMPAD_END"); break;
			case WXK_NUMPAD_BEGIN: keyStr += _T("NUMPAD_BEGIN"); break;
			case WXK_NUMPAD_INSERT: keyStr += _T("NUMPAD_INSERT"); break;
			case WXK_NUMPAD_DELETE: keyStr += _T("NUMPAD_DELETE"); break;
			case WXK_NUMPAD_EQUAL: keyStr += _T("NUMPAD_EQUAL"); break;
			case WXK_NUMPAD_MULTIPLY: keyStr += _T("NUMPAD_MULTIPLY"); break;
			case WXK_NUMPAD_ADD: keyStr += _T("NUMPAD_ADD"); break;
			case WXK_NUMPAD_SEPARATOR: keyStr += _T("NUMPAD_SEPARATOR"); break;
			case WXK_NUMPAD_SUBTRACT: keyStr += _T("NUMPAD_SUBTRACT"); break;
			case WXK_NUMPAD_DECIMAL: keyStr += _T("NUMPAD_DECIMAL"); break;
			case WXK_NUMPAD_DIVIDE: keyStr += _T("NUMPAD_DIVIDE"); break;

			default:
				if (wxIsprint(keycode))
				{
					keyStr += char(keycode);
				}
				else
				{
					keyStr += _T("???");
				}
			break;
		}
	}

	return keyStr;
}


void VEMSTableEditor::SetCellBackgroundColor(const wxColor& clr)
{
	//m_gcellBkColor = m_cellBkColorGradient = clr;
	m_gradientColorArray.Insert(clr,0);
}

wxColor VEMSTableEditor::GetCellBackgroundColor() const
{
	return m_gradientColorArray.Item(0);
}

void VEMSTableEditor::SetCellBackgroundColorGradient(const wxColor& clr1, const wxColor& clr2)
{
    m_gradientColorArray.RemoveAt(0);   //remove the first element
    m_gradientColorArray.Insert(clr1, 0); // put the first position

    m_gradientColorArray.RemoveAt(m_gradientColorArray.Count()-1);
    m_gradientColorArray.Add(clr2);

}

void VEMSTableEditor::AddCellBackgroundColorGradient(const wxColor& clr1)
{
    m_gradientColorArray.Add(clr1);
}

bool VEMSTableEditor::GetCellBackgroundColorGradient(wxColor* clr1, wxColor* clr2) const
{
	if (clr1 != 0)
	{
		*clr1 = m_gradientColorArray.Item(0);
	}

	if (clr2 != 0)
	{
		*clr2 =  m_gradientColorArray.Last();
	}

	return (clr1 == clr2) ? false : true;        // Gradient mode is NOT active when two colors are equal
}

void VEMSTableEditor::SetLockedCellBackgroundColor(const wxColor& clr)
{
	m_lockedCellBkColor = clr;
}

wxColor VEMSTableEditor::GetLockedCellBackgroundColor() const
{
	return m_lockedCellBkColor;
}

void VEMSTableEditor::SetModifiedCellTextColor(const wxColor& clr)
{
	m_modifiedCellTextColor = clr;
}

wxColor VEMSTableEditor::GetModifiedCellTextColor() const
{
	return m_modifiedCellTextColor;
}

void VEMSTableEditor::SetGridLinesColor(const wxColor& clr)
{
	m_gridLinesColor = clr;
}

wxColor VEMSTableEditor::GetGridLinesColor() const
{
	return m_gridLinesColor;
}

void VEMSTableEditor::SetCurrentLoadMarkerColor(const wxColor& clr)
{
	m_currentLoadMarkerColor = clr;
}

wxColor VEMSTableEditor::GetCurrentLoadMarkerColor() const
{
	return m_currentLoadMarkerColor;
}

void VEMSTableEditor::SetWidgetBackgroundColor(const wxColor& clr)
{
	m_bkColor = clr;
}

wxColor VEMSTableEditor::GetWidgetBackgroundColor() const
{
	return m_bkColor;
}

void VEMSTableEditor::SetCursorColor(const wxColor& clr)
{
	m_cursorColor = clr;
}

wxColor VEMSTableEditor::GetCursorColor() const
{
	return m_cursorColor;
}

void VEMSTableEditor::SetLabel3DColors(const wxColor& inner, const wxColor& outer)
{
	m_inner3DColor = inner;
	m_outer3DColor = outer;
}

void VEMSTableEditor::GetLabel3DColors(wxColor* inner, wxColor* outer)
{
	if (inner != 0)
	{
		*inner = m_inner3DColor;
	}

	if (outer != 0)
	{
		*outer = m_outer3DColor;
	}
}

wxSize VEMSTableEditor::GetBestSize()
{
	return m_widgetSize;
}

void VEMSTableEditor::CalcWidgetSize(const wxSize &size)
{
    if(GetNumberCols() < 1) return;
    if(GetNumberRows() < 1) return;


    /// there are some magic and mystic constans... be carefull

    #define TABLE_WIDTH_REF             780.0
    #define TABLE_HEIGHT_REF            350.0
    #define TABLE_CELL_FONTSIZE_REF     15.5
    #define TABLE_LABEL_FONTSIZE_REF    0.75   /// 11/16
    #define TABLE_SIZE_RATIO_REF        24.0 / 11.0  /// for 12x12

/*
    #define TABLE_WIDTH_REF             757.0
    #define TABLE_CELL_FONTSIZE_REF     16.0
    #define TABLE_LABEL_FONTSIZE_REF    0.6875   /// 11/16
*/

    /// if (size.x == 757) m_cellFontSize = 16;

    bool isCalcForHeight = false;

    if( (double)size.x / (double)size.y > ( TABLE_SIZE_RATIO_REF * GetSideRatio() ) )
        isCalcForHeight = true;

    if(size == wxDefaultSize) {
        m_cellFontSize = 16;
        m_labelFontSize = 11;
        //m_rowColCellExtraWidth = 16;

    } else {
        int refsize;


        if( isCalcForHeight  ) {
            m_cellFontSize = (int)((size.y / TABLE_HEIGHT_REF) * TABLE_CELL_FONTSIZE_REF * 15 / GetNumberCols());
            m_labelFontSize = (int)(m_cellFontSize * TABLE_LABEL_FONTSIZE_REF);
            refsize = (int)m_cellSize.y;
        } else {
            m_cellFontSize = (int)((size.x / TABLE_WIDTH_REF) * TABLE_CELL_FONTSIZE_REF * 15 / GetNumberCols());
            m_labelFontSize = (int)(m_cellFontSize * TABLE_LABEL_FONTSIZE_REF);
            refsize = (int)m_cellSize.x;
        }

            m_cellFont.SetPointSize(m_cellFontSize);
            int i, w = 0, h = 0;

            wxString valStr;
            for (i = 0; i < m_valueWidth; ++i)
            {
                valStr << _T("9");
            }
            valStr << _T(".");
            for (i = 0; i < m_valuePrecision; ++i)
            {
                valStr << _T("9");
            }

            GetTextExtent(valStr, &w, &h, 0, 0, &m_cellFont);

            int refh;
            if( isCalcForHeight  )
                refh = h;
            else
                refh = (int)(w*1.2);

           // Logger::logf(LOG_MSG,_T("refsize=%i,refh=%i"),refsize,refh);

            m_cellFontSize = (int)(m_cellFontSize * ((double)(refsize) / (double)refh));

    }

	m_cellFont.SetPointSize(m_cellFontSize);
	m_labelFont.SetPointSize(m_labelFontSize);
	m_zoomboxFont.SetPointSize(m_cellFontSize);

    NormalizeFont(m_cellFont);
    NormalizeFont(m_labelFont);

}

void VEMSTableEditor::NormalizeFont(wxFont &font, int limit1, int limit2)
{
    int fs = font.GetPointSize();

    if( fs < limit1 ) {
        font.SetWeight(wxFONTWEIGHT_BOLD);
        return;
    }

    if( fs >= limit1 && fs < limit2 ) {
        font.SetWeight(wxFONTWEIGHT_NORMAL);
        return;
    }

    if( fs >= limit2 ) {
        font.SetWeight(wxFONTWEIGHT_LIGHT);
        return;
    }

}

double VEMSTableEditor::GetSideRatio()
{
    if( !GetNumberCols() || !GetNumberRows() ) return 0;

    return (double)GetNumberCols()/(double)GetNumberRows();
}

void VEMSTableEditor::SetDefaults()
{
	m_valueWidth = 3;
	m_valuePrecision = 1;

	m_minCellValue = 0;
	m_maxCellValue = 200.0f;

	m_smallStepValue = 0.1f;
	m_largeStepValue = 0.1f;




	m_cellFont.Create(m_cellFontSize, wxSWISS, wxNORMAL, wxNORMAL, false);
	m_labelFont.Create(m_labelFontSize, wxSWISS, wxNORMAL, wxNORMAL, false);
	m_zoomboxFont.Create(m_cellFontSize, wxSWISS, wxNORMAL, wxBOLD, false);

	m_cellTextColor = *wxBLACK;
	m_labelTextColor = *wxBLACK;
	m_labelBkColor = wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE);
	m_zoomboxTextColor = *wxBLACK;
//	m_cellBkColor = *wxWHITE;
//	m_cellBkColorGradient = wxColor(191, 191, 191);
    m_gradientColorArray.Add(wxColor(32,64,255));
    m_gradientColorArray.Add(wxColor(32, 128, 32));

     m_gradientColorArray.Add(wxColor(224, 224, 32));

    m_gradientColorArray.Add(wxColor(128,64,0));
    m_gradientColorArray.Add(wxColor(224, 128, 64));

    //m_gradientColorArray.Add(wxColor(224, 209, 31));
    //m_gradientColorArray.Add(wxColor(224, 32, 64));

	m_modifiedCellTextColor = *wxRED;
	m_lockedCellBkColor = wxSystemSettings::GetColour(wxSYS_COLOUR_INFOBK);
	m_gridLinesColor = wxSystemSettings::GetColour(wxSYS_COLOUR_3DSHADOW);
	m_cellHighlightBkColor = wxSystemSettings::GetColour(wxSYS_COLOUR_HIGHLIGHT);

	m_cellHighlightTextColor = wxSystemSettings::GetColour(wxSYS_COLOUR_HIGHLIGHTTEXT);

	m_currentLoadMarkerColor = wxColor(255, 128, 128);

	m_bkColor = wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE);

	m_cursorColor = *wxBLACK;

	// Set default keys
	SetKey(VEMS_TABLE_EDITOR_KEY_DEC_LARGE, _T('Q'));
	SetKey(VEMS_TABLE_EDITOR_KEY_DEC_SMALL, _T('W'));
	SetKey(VEMS_TABLE_EDITOR_KEY_INC_SMALL, _T('E'));
	SetKey(VEMS_TABLE_EDITOR_KEY_INC_LARGE, _T('R'));

	SetKey(VEMS_TABLE_EDITOR_KEY_NAVIGATE_LEFT, WXK_LEFT);
	SetKey(VEMS_TABLE_EDITOR_KEY_NAVIGATE_RIGHT, WXK_RIGHT);
	SetKey(VEMS_TABLE_EDITOR_KEY_NAVIGATE_UP, WXK_UP);
	SetKey(VEMS_TABLE_EDITOR_KEY_NAVIGATE_DOWN, WXK_DOWN);
	SetKey(VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_COPY_LEFT, WXK_LEFT, wxMOD_SHIFT);
	SetKey(VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_COPY_RIGHT, WXK_RIGHT, wxMOD_SHIFT);
	SetKey(VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_COPY_UP, WXK_UP, wxMOD_SHIFT);
	SetKey(VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_COPY_DOWN, WXK_DOWN, wxMOD_SHIFT);

	SetKey(VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_HIGHLIGHT_LEFT, WXK_LEFT, wxMOD_CONTROL);
	SetKey(VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_HIGHLIGHT_RIGHT, WXK_RIGHT, wxMOD_CONTROL);
	SetKey(VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_HIGHLIGHT_UP, WXK_UP, wxMOD_CONTROL);
	SetKey(VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_HIGHLIGHT_DOWN, WXK_DOWN, wxMOD_CONTROL);

	SetKey(VEMS_TABLE_EDITOR_KEY_NAVIGATE_GRID_LEFTTOP_CORNER, WXK_HOME);
	SetKey(VEMS_TABLE_EDITOR_KEY_NAVIGATE_GRID_RIGHTBOTTOM_CORNER, WXK_END);

	SetKey(VEMS_TABLE_EDITOR_KEY_LOCK_CELLS, _T('L'));
	SetKey(VEMS_TABLE_EDITOR_KEY_LOCK_ALL_CELLS, _T('L'), wxMOD_CONTROL);
	SetKey(VEMS_TABLE_EDITOR_KEY_UNLOCK_ALL_CELLS, _T('L'), wxMOD_SHIFT);

	SetKey(VEMS_TABLE_EDITOR_KEY_INTERPOLATE_CELLS, _T('I'));
	SetKey(VEMS_TABLE_EDITOR_KEY_EDIT_VALUE, WXK_RETURN);
	SetKey(VEMS_TABLE_EDITOR_KEY_HIGHLIGHT_ALL_CELLS, _T('A'), wxMOD_CONTROL);

	SetKey(VEMS_TABLE_EDITOR_KEY_UNDO_CELL_MODIFICATION, _T('U'));
	SetKey(VEMS_TABLE_EDITOR_KEY_UNDO_ALL_CELLS_MODIFICATION,  _T('U'), wxMOD_CONTROL);

	SetKey( VEMS_TABLE_EDITOR_KEY_MULTIPLY_CELLS, WXK_NUMPAD_MULTIPLY );
	SetKey( VEMS_TABLE_EDITOR_KEY_DIVIDE_CELLS, WXK_NUMPAD_DIVIDE );

	SetKey( VEMS_TABLE_EDITOR_KEY_SELECT_LOAD, WXK_SPACE );


    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_INC_SMALL")] = VEMS_TABLE_EDITOR_KEY_INC_SMALL;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_INC_LARGE")] = VEMS_TABLE_EDITOR_KEY_INC_LARGE;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_DEC_SMALL")] = VEMS_TABLE_EDITOR_KEY_DEC_SMALL;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_DEC_LARGE")] = VEMS_TABLE_EDITOR_KEY_DEC_LARGE;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_NAVIGATE_LEFT")] = VEMS_TABLE_EDITOR_KEY_NAVIGATE_LEFT;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_NAVIGATE_RIGHT")] = VEMS_TABLE_EDITOR_KEY_NAVIGATE_RIGHT;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_NAVIGATE_UP")] = VEMS_TABLE_EDITOR_KEY_NAVIGATE_UP;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_NAVIGATE_DOWN")] = VEMS_TABLE_EDITOR_KEY_NAVIGATE_DOWN;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_COPY_LEFT")] = VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_COPY_LEFT;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_COPY_RIGHT")] = VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_COPY_RIGHT;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_COPY_UP")] = VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_COPY_UP;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_COPY_DOWN")] = VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_COPY_DOWN;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_HIGHLIGHT_LEFT")] = VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_HIGHLIGHT_LEFT;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_HIGHLIGHT_RIGHT")] = VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_HIGHLIGHT_RIGHT;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_HIGHLIGHT_UP")] = VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_HIGHLIGHT_UP;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_HIGHLIGHT_DOWN")] = VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_HIGHLIGHT_DOWN;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_NAVIGATE_GRID_LEFTTOP_CORNER")] = VEMS_TABLE_EDITOR_KEY_NAVIGATE_GRID_LEFTTOP_CORNER;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_NAVIGATE_GRID_RIGHTBOTTOM_CORNER")] = VEMS_TABLE_EDITOR_KEY_NAVIGATE_GRID_RIGHTBOTTOM_CORNER;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_LOCK_CELLS")] = VEMS_TABLE_EDITOR_KEY_LOCK_CELLS;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_LOCK_ALL_CELLS")] = VEMS_TABLE_EDITOR_KEY_LOCK_ALL_CELLS;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_UNLOCK_ALL_CELLS")] = VEMS_TABLE_EDITOR_KEY_UNLOCK_ALL_CELLS;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_INTERPOLATE_CELLS")] = VEMS_TABLE_EDITOR_KEY_INTERPOLATE_CELLS;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_EDIT_VALUE")] = VEMS_TABLE_EDITOR_KEY_EDIT_VALUE;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_HIGHLIGHT_ALL_CELLS")] = VEMS_TABLE_EDITOR_KEY_HIGHLIGHT_ALL_CELLS;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_UNDO_CELL_MODIFICATION")] = VEMS_TABLE_EDITOR_KEY_UNDO_CELL_MODIFICATION;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_UNDO_ALL_CELLS_MODIFICATION")] = VEMS_TABLE_EDITOR_KEY_UNDO_ALL_CELLS_MODIFICATION;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_MULTIPLY_CELLS")] = VEMS_TABLE_EDITOR_KEY_MULTIPLY_CELLS;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_DIVIDE_CELLS")] = VEMS_TABLE_EDITOR_KEY_DIVIDE_CELLS;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_SELECT_LOAD")] = VEMS_TABLE_EDITOR_KEY_SELECT_LOAD;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_CHANGE_TABLE")] = VEMS_TABLE_EDITOR_CHANGE_TABLE;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_WRITE_FW_CONFIG")] = VEMS_TABLE_EDITOR_WRITE_FW_CONFIG;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_BURN_FW_CONFIG")] = VEMS_TABLE_EDITOR_BURN_FW_CONFIG;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_REFRESH_FW_CONFIG")] = VEMS_TABLE_EDITOR_REFRESH_FW_CONFIG;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_AUTO_FOLLOW_LOAD")] = VEMS_TABLE_EDITOR_KEY_AUTO_FOLLOW_LOAD;
    m_hotkeySymbolToEnum[_T("VEMS_TABLE_EDITOR_KEY_DISPLAY_HELP")] = VEMS_TABLE_EDITOR_KEY_DISPLAY_HELP;
}

void VEMSTableEditor::RecalcSizes()
{
    if(GetNumberRows()<1) return;
    if(GetNumberCols()<1) return;

	double w = 0, h = 0;


    //double cw = GetClientSize().x - ((16-GetNumberCols())*2);
    double cw = GetClientSize().x;
    double ch = GetClientSize().y;

    w = cw / (double)(GetNumberCols()+1);
    h = ch / (double)(GetNumberRows()+1);

/*
	if (col_w > w)
	{
		w = col_w;
	}

	if (row_h > h)
	{
		h = row_h;
	}

    h *= 0.9;

    // cells size fine-sizing
*/
    if(ch > 100 && GetNumberRows() > 5)
        h = ch / (GetNumberRows()+1);


    double ewd = 0.0;

    if(GetNumberCols() > 15)
        ewd = 1;

    m_rowColCellExtraWidth = 0.0;
    //(cw - ( w * (double)(GetNumberCols()+1))) / (double)(GetNumberCols()+1) - ewd;

    w += m_rowColCellExtraWidth;

	m_cellSize.x = w;
	m_cellSize.y = h;

	m_rowLabelWidth = w;
	m_colLabelHeight = h;

	m_gridRect.x = (int)m_rowLabelWidth;
	m_gridRect.y = (int)m_colLabelHeight;
	m_gridRect.width  = (int)(m_numCols * m_cellSize.x);
	m_gridRect.height = (int)(m_numRows * m_cellSize.y);

	m_widgetSize.x = (int)(m_rowLabelWidth + m_gridRect.width);
	m_widgetSize.y = (int)(m_colLabelHeight + m_gridRect.height);

	m_cursorRow = m_cursorCol = 0;

    m_widgetSize.IncBy(2);





	//Logger::logf(LOG_MSG,_T("m_widgetSize.x = %i, m_widgetSize.y = %i"),m_widgetSize.x,m_widgetSize.y);

	//m_face->SetWidth(m_widgetSize.x);
	//m_face->SetHeight(m_widgetSize.y);

    double r = (h - 2)/2.0;

    if( w < h )
        r =  (w - 2)/2.0 ;

    SetLoadMarkerRadius( r );

	//m_refreshAllCells =true;


}

bool VEMSTableEditor::IsScrollNeededToViewCell(int row, int col, int *scrollRow, int *scrollCol) const
{
	wxASSERT(col >= 0 && col < GetNumberCols());
	wxASSERT(row >= 0 && row < GetNumberRows());

	wxRect rc = GetCellRectPix(row, col);
	wxRect clientRc = this->GetClientRect();

	clientRc.x = (int)(clientRc.x + m_rowLabelWidth);
	clientRc.y = (int)(clientRc.y - m_colLabelHeight);
	clientRc.width = (int)(clientRc.width - m_rowLabelWidth);
	clientRc.height = (int)(clientRc.height - m_colLabelHeight);

	if (rc.IsEmpty() || clientRc.IsEmpty())
	{
		return false;
	}

	int newDrawRow = m_drawOriginRow, newDrawCol = m_drawOriginCol;

	if (col == 0)
	{
		newDrawCol = 0;
	}
	else if (col == GetNumberCols() - 1)
	{
		newDrawCol = (int)(wxMax (0, GetNumberCols() - clientRc.GetWidth() / m_cellSize.x));
	}
	else if (rc.x < clientRc.x)
	{
		const int diff = clientRc.x - rc.x;
		newDrawCol = (int)(wxMax (0, m_drawOriginCol - (diff + m_cellSize.x - 1) / m_cellSize.x));
	}
	else if (rc.GetRight() > clientRc.GetRight())
	{
		const int diff = rc.GetRight() - clientRc.GetRight();
		newDrawCol = (int)(wxMin(GetNumberCols() - clientRc.GetWidth() / m_cellSize.x, m_drawOriginCol + (diff + m_cellSize.x - 1) / m_cellSize.x));
	}

	if (row == 0)
	{
		newDrawRow = 0;
	}
	else if (row == GetNumberRows() - 1)
	{
		newDrawRow = (int)(wxMax (0, GetNumberRows() - clientRc.GetHeight() / m_cellSize.y));
	}
	else if (rc.y < clientRc.y)
	{
		const int diff = clientRc.y - rc.y;
		newDrawRow = (int)(wxMax (0, m_drawOriginRow - (diff + m_cellSize.y - 1) / m_cellSize.y));
	}
	else if (rc.GetBottom() > clientRc.GetBottom())
	{
		const int diff = rc.GetBottom() - clientRc.GetBottom();
		newDrawRow = (int)(wxMin(GetNumberRows() - clientRc.GetHeight() / m_cellSize.y, m_drawOriginRow + (diff + m_cellSize.y - 1) / m_cellSize.y));
	}

	if (scrollRow != 0)
	{
		*scrollRow = newDrawRow;
	}

	if (scrollCol != 0)
	{
		*scrollCol = newDrawCol;
	}

	return ((newDrawRow != m_drawOriginRow) || (newDrawCol != m_drawOriginCol)) ? true : false;
}

bool VEMSTableEditor::EnsureCellVisible(int row, int col)
{
	wxASSERT(col >= 0 && col < GetNumberCols());
	wxASSERT(row >= 0 && row < GetNumberRows());


	int newScrollRow = 0, newScrollCol = 0;
	if (IsScrollNeededToViewCell(row, col, &newScrollRow, &newScrollCol))
	{
	    //Logger::logf(LOG_MSG,_T("VEMSTableEditor::EnsureCellVisible IsScrollNeededToViewCell is true"));
		m_drawOriginRow = newScrollRow;
		m_drawOriginCol = newScrollCol;
		return true;
	}

	return false;
}

void VEMSTableEditor::LockCell(int row, int col)
{
	wxASSERT(col >= 0 && col < GetNumberCols());
	wxASSERT(row >= 0 && row < GetNumberRows());

	m_cellLockFlag[row][col] = 1;
}


void VEMSTableEditor::UnlockCell(int row, int col)
{
	wxASSERT(col >= 0 && col < GetNumberCols());
	wxASSERT(row >= 0 && row < GetNumberRows());

	m_cellLockFlag[row][col] = 0;
}


bool VEMSTableEditor::IsCellLocked(int row, int col) const
{
	wxASSERT(col >= 0 && col < GetNumberCols());
	wxASSERT(row >= 0 && row < GetNumberRows());

	return (m_cellLockFlag[row][col] != 0) ? true : false;
}

bool VEMSTableEditor::IsCellHighlighted(int row, int col) const
{
	wxASSERT(col >= 0 && col < GetNumberCols());
	wxASSERT(row >= 0 && row < GetNumberRows());

	return (m_cellHighlightFlag[row][col] != 0) ? true : false;
}

void VEMSTableEditor::LockHighlightedCells()
{
	for (int i = 0; i < GetNumberRows(); ++i)
	{
		for (int j = 0; j < GetNumberCols(); ++j)
		{
			if (IsCellHighlighted(i, j))
			{
				LockCell(i, j);
			}
		}
	}

	UnhighlightAllCells();
}

void VEMSTableEditor::LockAllCells()
{
	for (int i = 0; i < GetNumberRows(); ++i)
	{
		for (int j = 0; j < GetNumberCols(); ++j)
		{
			LockCell(i, j);
		}
	}
}

void VEMSTableEditor::UnlockAllCells()
{
	for (int i = 0; i < GetNumberRows(); ++i)
	{
		for (int j = 0; j < GetNumberCols(); ++j)
		{
			UnlockCell(i, j);
		}
	}
}


wxRect VEMSTableEditor::GetCellRectPix(int row, int col) const
{
	wxASSERT(col >= 0 && col < GetNumberCols());
	wxASSERT(row >= 0 && row < GetNumberRows());

	wxRect rc;
	rc.x = (int)(m_rowLabelWidth + (col - m_drawOriginCol) * m_cellSize.x);
	rc.y = (int)(m_colLabelHeight + (row - m_drawOriginRow) * m_cellSize.y);
	rc.width = (int)(m_cellSize.x);
	rc.height = (int)(m_cellSize.y);
	return rc;
}

wxRect VEMSTableEditor::GetCellRectVal(int row, int col) const
{
	wxASSERT(col >= 0 && col < GetNumberCols());
	wxASSERT(row >= 0 && row < GetNumberRows());

	wxRect rc;

/*
	rc.SetLeft( m_colLabels[col]
	rc.SetRight(


    rc.SetTop
    rc.SetBottom

*/

	return rc;
}

wxRect VEMSTableEditor::GetLabelRowRect(int row) const
{
	wxASSERT(row >= 0 && row < GetNumberRows());

	wxRect rc;
	rc.x = 0;
	rc.y = (int)(m_colLabelHeight + (row - m_drawOriginRow) * m_cellSize.y);
	rc.width = (int)(m_rowLabelWidth);
	rc.height = (int)(m_cellSize.y);

	return rc;
}

wxRect VEMSTableEditor::GetLabelColRect(int col) const
{
	wxASSERT(col >= 0 && col < GetNumberCols());

	wxRect rc;
	rc.x = (int)(m_rowLabelWidth + (col - m_drawOriginCol) * m_cellSize.x);
	rc.y = 0;
	rc.width = (int)(m_cellSize.x);
	rc.height = (int)(m_colLabelHeight);

	return rc;
}

bool VEMSTableEditor::HitTestCell(const wxPoint& pos, int *row, int *col) const
{
	for (int i = 0; i < GetNumberRows(); ++i)
	{
		for (int j = 0; j < GetNumberCols(); ++j)
		{
			wxRect rc = GetCellRectPix(i, j);

			if (rc.x < m_rowLabelWidth)
			{
				rc.width = (int)(rc.width - (m_rowLabelWidth - rc.x));
				rc.x = (int)(m_rowLabelWidth);
			}

			if (rc.y < m_colLabelHeight)
			{
				rc.height = (int)(rc.height - (m_colLabelHeight - rc.y));
				rc.y = (int)(m_colLabelHeight);
			}

			if (rc.Contains(pos))
			{
				if (row != 0)
				{
					*row = i;
				}

				if (col != 0)
				{
					*col = j;
				}

				return true;
			}
		}
	}

	return false;
}

bool VEMSTableEditor::HitTestLabelRow(const wxPoint& pos, int *row) const
{
	for (int i = 0; i < GetNumberRows(); ++i)
	{
		const wxRect rc = GetLabelRowRect(i);

		if (rc.Contains(pos))
		{
			if (row != 0)
			{
				*row = i;
			}

			return true;
		}

	}

	return false;
}

bool VEMSTableEditor::HitTestLabelCol(const wxPoint& pos, int *col) const
{
	for (int i = 0; i < GetNumberCols(); ++i)
	{
		const wxRect rc = GetLabelColRect(i);

		if (rc.Contains(pos))
		{
			if (col != 0)
			{
				*col = i;
			}

			return true;
		}

	}

	return false;
}

bool VEMSTableEditor::HitTestLeftTopCorner(const wxPoint& pos) const
{
	wxRect rc;
	rc.x = rc.y = 0;
	rc.width = (int)(m_rowLabelWidth);
	rc.height = (int)(m_colLabelHeight);

	return (rc.Contains(pos)) ? true : false;
}

void VEMSTableEditor::HighlightRow(int row)
{
	wxASSERT(row >= 0 && row < GetNumberRows());

	if (row >= 0 && row < GetNumberRows())
	{
		for (int i = 0; i < GetNumberCols(); ++i)
		{
			if (!IsCellLocked(row, i))
			{
				m_cellHighlightFlag[row][i] = 1;
			}
		}
	}
}

void VEMSTableEditor::UnhighlightRow(int row)
{
	wxASSERT(row >= 0 && row < GetNumberRows());

	if (row >= 0 && row < GetNumberRows())
	{
		for (int i = 0; i < GetNumberCols(); ++i)
		{
			m_cellHighlightFlag[row][i] = 0;
		}
	}
}

bool VEMSTableEditor::IsRowHighlighted(int row) const
{
	wxASSERT(row >= 0 && row < GetNumberRows());

	if (row >= 0 && row < GetNumberRows())
	{
		for (int i = 0; i < GetNumberCols(); ++i)
		{
			if (!IsCellHighlighted(row, i) && !IsCellLocked(row, i))
			{
				return false;
			}
		}
	}

	return true;
}

void VEMSTableEditor::HighlightColumn(int col)
{
	wxASSERT(col >= 0 && col < GetNumberCols());

	if (col >= 0 && col < GetNumberCols())
	{
		for (int i = 0; i < GetNumberRows(); ++i)
		{
			if (!IsCellLocked(i, col))
			{
				m_cellHighlightFlag[i][col] = 1;
			}
		}
	}
}

void VEMSTableEditor::UnhighlightColumn(int col)
{
	wxASSERT(col >= 0 && col < GetNumberCols());

	if (col >= 0 && col < GetNumberCols())
	{
		for (int i = 0; i < GetNumberRows(); ++i)
		{
			m_cellHighlightFlag[i][col] = 0;
		}
	}
}

bool VEMSTableEditor::IsColumnHighlighted(int col) const
{
	wxASSERT(col >= 0 && col < GetNumberCols());

	if (col >= 0 && col < GetNumberCols())
	{
		for (int i = 0; i < GetNumberRows(); ++i)
		{
			if (!IsCellHighlighted(i, col) && !IsCellLocked(i, col))
			{
				return false;
			}
		}
	}

	return true;
}

void VEMSTableEditor::HighlightCell(int row, int col)
{
	wxASSERT(col >= 0 && col < GetNumberCols());
	wxASSERT(row >= 0 && row < GetNumberRows());

	if (col >= 0 && col < GetNumberCols() && row >= 0 && row < GetNumberRows())
	{
		if (!IsCellLocked(row, col))
		{
			m_cellHighlightFlag[row][col] = 1;
		}
	}

	AddNewRefreshCell(row,col);

}

void VEMSTableEditor::UnhighlightCell(int row, int col)
{
	wxASSERT(col >= 0 && col < GetNumberCols());
	wxASSERT(row >= 0 && row < GetNumberRows());

	if (col >= 0 && col < GetNumberCols() && row >= 0 && row < GetNumberRows())
	{
		m_cellHighlightFlag[row][col] = 0;
	}

	AddNewRefreshCell(row,col);
}

void VEMSTableEditor::HighlightAllCells()
{
	for (int i = 0; i < GetNumberRows(); ++i)
	{
		for (int j = 0; j < GetNumberCols(); ++j)
		{
			if (!IsCellLocked(i, j))
			{
				HighlightCell(i, j);
			}
		}
	}
}

void VEMSTableEditor::UnhighlightAllCells()
{
	for (int i = 0; i < GetNumberRows(); ++i)
	{
		for (int j = 0; j < GetNumberCols(); ++j)
		{
			UnhighlightCell(i, j);
		}
	}
}

bool VEMSTableEditor::AreAllCellsHighlighted() const
{
	for (int i = 0; i < GetNumberRows(); ++i)
	{
		for (int j = 0; j < GetNumberCols(); ++j)
		{
			if (!IsCellHighlighted(i, j) && !IsCellLocked(i, j))
			{
				return false;
			}
		}
	}

	return true;
}

bool VEMSTableEditor::AreCellsHighlighted() const
{
	for (int i = 0; i < GetNumberRows(); ++i)
	{
		for (int j = 0; j < GetNumberCols(); ++j)
		{
			if (IsCellHighlighted(i, j))
			{
				return true;
			}
		}
	}

	return false;
}


bool VEMSTableEditor::AreCellsModified() const
{
	for (int i = 0; i < GetNumberRows(); ++i)
	{
		for (int j = 0; j < GetNumberCols(); ++j)
		{
			if (WasCellModified(i, j))
			{
				return true;
			}
		}
	}

	return false;
}


bool VEMSTableEditor::HighlightRectangularAreaOfCells(int start_row, int start_col, int end_row, int end_col)
{
	wxASSERT(start_row >= 0 && start_row < GetNumberRows());
	wxASSERT(start_col >= 0 && start_col < GetNumberCols());
	wxASSERT(end_row >= 0 && end_row < GetNumberRows());
	wxASSERT(end_col >= 0 && end_col < GetNumberCols());

	if (start_row < 0 || start_row >= GetNumberRows() ||
		end_row < 0 || end_row >= GetNumberCols() ||
		start_col < 0 || start_col >= GetNumberCols() ||
		end_col < 0 || end_col >= GetNumberCols())
	{
		return false;
	}

	if (start_row > end_row)
	{
		// Swap values
		int tmp = start_row;
		start_row = end_row;
		end_row = tmp;
	}

	if (start_col > end_col)
	{
		// Swap values
		int tmp = start_col;
		start_col = end_col;
		end_col = tmp;
	}

	for (int i = start_row; i <= end_row; ++i)
	{
		for (int j = start_col; j <= end_col; ++j)
		{
			if (!IsCellLocked(i, j))
			{
				HighlightCell(i, j);
			}
		}
	}

	return true;
}

bool VEMSTableEditor::IsRectangularAreaOfCellsHighlighted() const
{
	return false;
}

void VEMSTableEditor::ResetHighlightOrigin()
{
	m_highlightOriginCol = m_highlightOriginRow = -1;
}


void VEMSTableEditor::InterpolateCell(int row, int col)
{
	wxASSERT(col > 0 && col < GetNumberCols() - 1);
	wxASSERT(row > 0 && row < GetNumberRows() - 1);

	//const float lefttop = m_cellValues[row - 1][col - 1];
	//const float righttop = m_cellValues[row - 1][col + 1];
	//const float leftbottom = m_cellValues[row + 1][col - 1];
	//const float rightbottom = m_cellValues[row + 1][col + 1];

	const float lefttop = m_cellValues.get( row - 1,col - 1 );
	const float righttop = m_cellValues.get( row - 1,col + 1 );
	const float leftbottom = m_cellValues.get( row + 1, col - 1 );
	const float rightbottom = m_cellValues.get( row + 1, col + 1 );

	float new_value = ((lefttop + righttop) / 2.0 + (leftbottom + rightbottom) / 2.0) / 2.0;

	if (new_value < m_minCellValue)
	{
		new_value = m_minCellValue;
	}
	else if (new_value > m_maxCellValue)
	{
		new_value = m_maxCellValue;
	}

	SetCellValue(row, col, new_value);
}

void VEMSTableEditor::InterpolateSelection()
{
	int left_cell = GetNumberCols(), right_cell = -1;
	int top_cell = GetNumberRows(), bottom_cell = -1;

	for (int i = 0; i < GetNumberRows(); ++i)
	{
		for (int j = 0; j < GetNumberCols(); ++j)
		{
			if (IsCellHighlighted(i, j))
			{
				left_cell = wxMin(j, left_cell);
				right_cell = wxMax(j, right_cell);
				top_cell = wxMin(i, top_cell);
				bottom_cell = wxMax(i, bottom_cell);
			}
		}
	}

//    const double topleft = m_cellValues[top_cell][left_cell]; /// z0
//    const double topright = m_cellValues[top_cell][right_cell]; ///z1
//    const double bottomleft = m_cellValues[bottom_cell][left_cell]; /// z2
//    const double bottomright = m_cellValues[bottom_cell][right_cell]; ///z3

    const double topleft = m_cellValues.get( top_cell, left_cell ); /// z0
    const double topright = m_cellValues.get( top_cell, right_cell ); ///z1
    const double bottomleft = m_cellValues.get( bottom_cell, left_cell ); /// z2
    const double bottomright = m_cellValues.get( bottom_cell, right_cell ); ///z3


    //Logger::logf(LOG_MSG, _T("InterpolateSelection:() leftCell:%i, rightCell:%i"), left_cell, right_cell);
    //Logger::logf(LOG_MSG, _T("InterpolateSelection:() top_cell:%i, bottom_cell:%i"), top_cell, bottom_cell);

    double p0[3];
    double p1[3];
    double p2[3];
    double p3[3];
    double px[3];
    p0[0] = p2[0] = m_colLabels[left_cell]; ///x0 x2
    p0[1] = p1[1] = m_rowLabels[top_cell]; ///y0 y1
    p0[2] = topleft;

    p1[0] = p3[0] = m_colLabels[right_cell]; ///x1 x3
    ///p1[1] = p1[1] = m_rowLabels[]; ///y0 y1
    p1[2] = topright;

    ///p0[0] = p2[0] = m_colLabels[left_cell]; ///x0 x2
    p2[1] = p3[1] = m_rowLabels[bottom_cell]; ///y2 y3
    p2[2] = bottomleft;

    p3[2] = bottomright;

    //Logger::logf(LOG_MSG, _T("InterpolateSelection: p0:(%f,%f,%f)"), p0[0], p0[1], p0[2]);
    //Logger::logf(LOG_MSG, _T("InterpolateSelection: p1:(%f,%f,%f)"), p1[0], p1[1], p1[2]);
    //Logger::logf(LOG_MSG, _T("InterpolateSelection: p2:(%f,%f,%f)"), p2[0], p2[1], p2[2]);
    //Logger::logf(LOG_MSG, _T("InterpolateSelection: p3:(%f,%f,%f)"), p3[0], p3[1], p3[2]);

    for (int i = top_cell; i <= bottom_cell; ++i)
    {
        for (int j = left_cell; j <= right_cell; ++j)
        {
            px[0] = m_colLabels[j]; ///x
            px[1] = m_rowLabels[i]; ///y
            px[2] = -1; /// invalid value

            if (IsCellHighlighted(i, j))
            {
                bool res = Math::linearInterpolation3D(p0, p1, p2, p3, px);
                //Logger::logf(LOG_MSG, _T("InterpolateSelection:(%i,%i) px:(%f,%f,%f)"), j,i, px[0], px[1], px[2]);

                if(res){ SetCellValue(i, j, px[2]); }
            }
        }
    }

    Send( TableEditorEvent::CELLS_INTERPOLATED );
}

void VEMSTableEditor::UndoCellModification(int row, int col)
{
	wxASSERT(col >= 0 && col < GetNumberCols());
	wxASSERT(row >= 0 && row < GetNumberRows());

	//m_cellValues[row][col] = m_cellBakValues[row][col];
	m_cellValues.set( row, col, m_cellBakValues.get( row, col ) );

}

void VEMSTableEditor::UndoHighlightedCellsModification()
{
	for (int i = 0; i < GetNumberRows(); ++i)
	{
		for (int j = 0; j < GetNumberCols(); ++j)
		{
			if (IsCellHighlighted(i, j))
			{
				UndoCellModification(i, j);
			}
		}
	}
}

void VEMSTableEditor::UndoAllCellsModification()
{
	for (int i = 0; i < GetNumberRows(); ++i)
	{
		for (int j = 0; j < GetNumberCols(); ++j)
		{
			if (!IsCellLocked(i, j))
			{
				UndoCellModification(i, j);
			}
		}
	}
}

void VEMSTableEditor::IncCellLarge()
{
	if (m_cursorRow >= 0 && m_cursorCol >= 0)
	{
		if (IsCellLocked(m_cursorRow, m_cursorCol))
		{
			wxBell();
		}
		else
		{
			float d = GetCellValue(m_cursorRow, m_cursorCol);
			if (d < m_maxCellValue)
			{
				Freeze();
				d = d + m_largeStepValue;
				if (d > m_maxCellValue)
				{
					d = m_maxCellValue;
				}
				SetCellValue(m_cursorRow, m_cursorCol, d);
				Thaw();
			}
		}
	}
}

void VEMSTableEditor::DecCellLarge()
{
	if (m_cursorRow >= 0 && m_cursorCol >= 0)
	{
		if (IsCellLocked(m_cursorRow, m_cursorCol))
		{
			wxBell();
		}
		else
		{
			float d = GetCellValue(m_cursorRow, m_cursorCol);
			if (d > m_minCellValue)
			{
				Freeze();
				d = d - m_largeStepValue;
				if (d < m_minCellValue)
				{
					d = m_minCellValue;
				}
				SetCellValue(m_cursorRow, m_cursorCol, d);
				Thaw();
			}
		}
	}
}

/*
int left_cell = GetNumberCols(), right_cell = -1;
	int top_cell = GetNumberRows(), bottom_cell = -1;

	for (int i = 0; i < GetNumberRows(); ++i)
	{
		for (int j = 0; j < GetNumberCols(); ++j)
		{
			if (IsCellHighlighted(i, j))
			{
				left_cell = wxMin(j, left_cell);
				right_cell = wxMax(j, right_cell);
				top_cell = wxMin(i, top_cell);
				bottom_cell = wxMax(i, bottom_cell);
			}
		}
	}

    const double topleft = m_cellValues[top_cell][left_cell]; /// z0
    const double topright = m_cellValues[top_cell][right_cell]; ///z1
    const double bottomleft = m_cellValues[bottom_cell][left_cell]; /// z2
    const double bottomright = m_cellValues[bottom_cell][right_cell]; ///z3
*/
/*
void VEMSTableEditor::ChangeSelectedCells( double delta )
{
    if ( !m_bHighlighted )
    {
        ChangeCellValue( m_selectedRow, m_selectedCol, inc );
        return;
    }

    int left    = wxMin( m_selectedCol, m_highlightOriginCol );
    int right   = wxMax( m_selectedCol, m_highlightOriginCol );
    int top     = wxMax( m_selectedRow, m_highlightOriginRow );
    int bottom  = wxMin( m_selectedRow, m_highlightOriginRow );
    //int top     = m_rows - wxMax( m_selectedRow, m_highlightOriginRow ) - 1;
    //int bottom  = m_rows - wxMin( m_selectedRow, m_highlightOriginRow ) - 1;

    for ( int i = bottom; i <= top; ++i )
    {
        for ( int j = left; j <= right; ++j )
        {
            ChangeCellValue( i, j, inc );
        }
    }
}
*/

void VEMSTableEditor::ChangeCells( int start_row, int start_col, int end_row, int end_col, double d )
{
    wxASSERT(start_row >= 0 && start_row < GetNumberRows());
	wxASSERT(start_col >= 0 && start_col < GetNumberCols());
	wxASSERT(end_row >= 0 && end_row < GetNumberRows());
	wxASSERT(end_col >= 0 && end_col < GetNumberCols());

    if (start_row < 0 || start_row >= GetNumberRows() ||
    end_row < 0 || end_row >= GetNumberCols() ||
    start_col < 0 || start_col >= GetNumberCols() ||
    end_col < 0 || end_col >= GetNumberCols())
	{
		return;
	}

	if (start_row > end_row)
	{
		// Swap values
		int tmp = start_row;
		start_row = end_row;
		end_row = tmp;
	}

	if (start_col > end_col)
	{
		// Swap values
		int tmp = start_col;
		start_col = end_col;
		end_col = tmp;
	}

	for (int i = start_row; i <= end_row; ++i)
	{
		for (int j = start_col; j <= end_col; ++j)
		{
			if (!IsCellLocked(i, j))
			{
				ChangeCellValue( i, j ,d );
			}
		}
	}

    //Logger::logf( LOG_MSG, _T("VEMSTableEditor::ChangeCells()") );
    TableEditorEvent e( TableEditorEvent::CELLS_CHANGED );
    if ( GetParent() ) { GetParent()->AddPendingEvent( e ); }
}


void VEMSTableEditor::ChangeSelectedCells( double delta, bool readAfterWrite )
{
//    Logger::logf( LOG_MSG, _T("VEMSTableEditor::ChangeSelectedCells( %f, %d ) - m_highlightOriginRow: %d"),
//        delta, readAfterWrite, m_highlightOriginRow );

    if ( AreAllCellsHighlighted() )
    {
        ChangeCells( 0, 0, GetNumberRows() - 1, GetNumberCols() - 1, delta );
    }
    else
    {

        //if ( m_highlightOriginRow == -1 )
        if ( !IsCellHighlighted( m_cursorRow, m_cursorCol ) )
        {
            if ( !ChangeCellValue( m_cursorRow, m_cursorCol, delta ) ) { return; }


            //if( WasCellModified( m_cursorRow, m_cursorCol ) || readAfterWrite )
            //if( readAfterWrite )

            {
                //OnSingleCellChanged( m_cursorRow, m_cursorCol, d );
                TableEditorEvent e( TableEditorEvent::CELL_CHANGED );
                e.row = m_cursorRow;
                e.col = m_cursorCol;
                e.value = GetCellValue( m_cursorRow, m_cursorCol );
                e.readAfterWrite = readAfterWrite;

                if ( GetParent() )
                {
                    GetParent()->AddPendingEvent( e );
                }
            }

            //return;
        }
        else
        {
            ChangeCells(    m_highlightOriginRow, m_highlightOriginCol,
                            m_cursorRow, m_cursorCol, delta  );
        }
    }

    Refresh();
}


bool VEMSTableEditor::ChangeCellValue( int row, int col, double delta )
{
    double d = GetCellValue( row, col );
    double old = d;

    d += delta;

    if ( d > m_maxCellValue ) { d = m_maxCellValue; }
    if ( d < m_minCellValue ) { d = m_minCellValue; }

    if ( old == d ) { return false; }

    if ( !IsCellLocked( row, col ) )
    {
        SetCellValue( row, col, d );
        return true;
    }

    return false;
}


void VEMSTableEditor::IncCellSmall()
{
	if (m_cursorRow >= 0 && m_cursorCol >= 0)
	{
		if (IsCellLocked(m_cursorRow, m_cursorCol))
		{
			wxBell();
		}
		else
		{
			float d = GetCellValue(m_cursorRow, m_cursorCol);
			if (d < m_maxCellValue)
			{
				Freeze();
				d = d + m_smallStepValue;
				if (d > m_maxCellValue )
				{
					d = m_maxCellValue;
				}
				SetCellValue(m_cursorRow, m_cursorCol, d);
				Thaw();
			}
		}
	}
}

void VEMSTableEditor::DecCellSmall()
{
	if (m_cursorRow >= 0 && m_cursorCol >= 0)
	{
		if (IsCellLocked(m_cursorRow, m_cursorCol))
		{
			wxBell();
		}
		else
		{
			float d = GetCellValue(m_cursorRow, m_cursorCol);
			if (d > m_minCellValue)
			{
				Freeze();
				d = d - m_smallStepValue;
				if (d < m_minCellValue)
				{
					d = m_minCellValue;
				}
				SetCellValue(m_cursorRow, m_cursorCol, d);
				Thaw();
			}
		}
	}
}

void VEMSTableEditor::LockUnlockCells()
{
	Freeze();

	if (AreCellsHighlighted())
	{
		LockHighlightedCells();
	}
	else if (IsCellLocked(m_cursorRow, m_cursorCol))
	{
		UnlockCell(m_cursorRow, m_cursorCol);
	}
	else
	{
		LockCell(m_cursorRow, m_cursorCol);
	}

	Thaw();
}

bool VEMSTableEditor::isCursorPositionValid() const
{
    return ( m_cursorRow >= 0 && m_cursorRow < GetNumberRows() && m_cursorCol >= 0 && m_cursorCol < GetNumberCols() );
}

void VEMSTableEditor::showChangeValueDialog( const wxString& initialValue )
{
    const float cur_val = GetCellValue( m_cursorRow, m_cursorCol );

    wxRect cellRect = GetCellRectPix(m_cursorRow, m_cursorCol);
    cellRect.Offset(ClientToScreen(wxPoint(0, 0)));

    wxString val_mask;
    val_mask.sprintf(_T("%%.%df"), m_valuePrecision );

    VEMSCellEditorDialog dlg(wxGetActiveWindow(), cur_val, m_minCellValue, m_maxCellValue, cellRect, m_zoomboxFont, m_zoomboxTextColor, val_mask, initialValue);
    dlg.ShowModal();

    if ( dlg.GetReturnCode() != wxID_OK ) return;

    if ( !dlg.IsValueAccepted() )
    {
        wxMessageBox(_T("Value is incorrect or out of range"), _T("Error"), wxOK | wxICON_ERROR);
        return;
    }

    wxString valStr;
    const float newValue = dlg.GetResultValue();

    if ( AreCellsHighlighted() )
    {
        changeSelection( newValue  );
        Send( TableEditorEvent::CELLS_CHANGED );
    }
    else
    {
        SetCellValue( m_cursorRow, m_cursorCol, newValue );
        if( !WasCellModified( m_cursorRow, m_cursorCol ) ) return;
        Send( TableEditorEvent::CELL_CHANGED, m_cursorRow, m_cursorCol, newValue );
    }

    ResetHighlightOrigin();
    UnhighlightAllCells();
    Refresh();
}

void VEMSTableEditor::changeSelection( double d )
{
	for (int i = 0; i < GetNumberRows(); ++i)
	{
		for (int j = 0; j < GetNumberCols(); ++j)
		{
			if ( IsCellHighlighted(i, j) && !IsCellLocked(i, j) )
			{
				SetCellValue( i, j, d );
			}
		}
	}
}

void VEMSTableEditor::RequestValueForCursorCell( const wxString& initialValue )
{
	if ( !isCursorPositionValid() ) return;

    if ( IsCellLocked( m_cursorRow, m_cursorCol ) )
    {
        wxBell();
    }
    else
    {
        showChangeValueDialog( initialValue );
    }

    SetFocus();
}

void VEMSTableEditor::EnsureCellVisible( int dir )
{
    int row = m_cursorRow;
    int col = m_cursorCol;

    switch ( dir )
    {
        case LEFT   :
            if ( col - 1 >= 0 ) { --col; }
            break;
        case RIGHT  :
            if ( col + 1 < GetNumberCols() ) { ++col; }
            break;
        case UP     :
            if ( row - 1 >= 0) { --row; }
            break;
        case DOWN   :
            if ( row + 1 < GetNumberRows() ) { ++row; }
            break;
    }

    EnsureCellVisible( row, col );
}


void VEMSTableEditor::MoveCursor( int dir, int style )
{
    if ( !MoveCursorIsPossible( dir) ) { return; }

    const int prevCursorRow = m_cursorRow;
    const int prevCursorCol = m_cursorCol;

    Freeze();

    UnhighlightAllCells();

    switch ( dir )
    {
        case LEFT   :
            --m_cursorCol;
            break;
        case RIGHT  :
            ++m_cursorCol;
            break;
        case UP     :
            --m_cursorRow;
            break;
        case DOWN   :
            ++m_cursorRow;
            break;
    }

    switch ( style )
    {
        case COPY:
//            Logger::logf( LOG_MSG, _T("-----VEMSTableEditor::MoveCursor() - COPY") );
            ResetHighlightOrigin();
            CopyFrom( prevCursorRow, prevCursorCol );
            break;
        case HIGHLIGHT:
//            Logger::logf( LOG_MSG, _T("-----VEMSTableEditor::MoveCursor() - HIGHLIGHT") );
            HighlightFrom( prevCursorRow, prevCursorCol );
            break;
        default:
//            Logger::logf( LOG_MSG, _T("-----VEMSTableEditor::MoveCursor() - MOVE") );
            Send( TableEditorEvent::CURSOR_MOVED, m_cursorRow, m_cursorCol );
            ResetHighlightOrigin();
            break;
    }

    AddNewRefreshCell( m_cursorRow, m_cursorCol );


    EnsureCellVisible( dir );

    Thaw();
}


bool VEMSTableEditor::MoveCursorIsPossible( int dir )
{
    bool result =   ( ( dir == LEFT ) && ( m_cursorCol > 0 ) ) ||
                    ( ( dir == RIGHT ) && ( m_cursorCol < GetNumberCols() - 1 ) ) ||
                    ( ( dir == UP ) && ( m_cursorRow > 0 ) ) ||
                    ( ( dir == DOWN ) && ( m_cursorRow < GetNumberRows() - 1 ) );
    return result;
}


void VEMSTableEditor::CopyFrom( int row, int col )
{
    if (!IsCellLocked(m_cursorRow, m_cursorCol))
    {
        SetCellValue(m_cursorRow, m_cursorCol, GetCellValue( row, col ) );
        if ( WasCellModified( m_cursorRow, m_cursorCol ) )
        {
            //Logger::logf( LOG_MSG, _T("VEMSTableEditor::CopyFrom") );
            VEMSTableEditor::Send( TableEditorEvent::CELL_COPIED, m_cursorRow, m_cursorCol, GetCellValue( row, col ) );
        }
    }
    else
    {
        wxBell();
    }
}


void VEMSTableEditor::HighlightFrom( int row, int col )
{
    if (m_highlightOriginRow < 0 || m_highlightOriginRow >= GetNumberRows() ||
        m_highlightOriginCol < 0 || m_highlightOriginCol >= GetNumberCols())
    {
        m_highlightOriginRow = row;
        m_highlightOriginCol = col;
    }

    HighlightRectangularAreaOfCells(m_highlightOriginRow, m_highlightOriginCol, m_cursorRow, m_cursorCol);
}


void VEMSTableEditor::OnKeyDown(wxKeyEvent& event)
{
    int kk = event.GetKeyCode();
    //Logger::logf( LOG_MSG, _T("VEMSTableEditor::OnKeyDown() - %c"), kk );


    int mod = 1;
    if( event.ControlDown() )
        mod = 10;


    switch(kk){

        case WXK_NUMPAD_UP:

                m_currentMarkerLoad += mod;
                SetCurrentLoad(m_currentMarkerLoad, m_currentMarkerRpm);
              //  return;
            break;

        case WXK_NUMPAD_DOWN:
                m_currentMarkerLoad -= mod;
                SetCurrentLoad(m_currentMarkerLoad, m_currentMarkerRpm);
            //    return;
            break;

        case WXK_NUMPAD_RIGHT:
                m_currentMarkerRpm += mod * 10;
                SetCurrentLoad(m_currentMarkerLoad, m_currentMarkerRpm);
          //      return;
            break;

        case WXK_NUMPAD_LEFT:
                m_currentMarkerRpm -= mod * 10;
                SetCurrentLoad(m_currentMarkerLoad, m_currentMarkerRpm);
             //   return;
            break;

        case WXK_NUMPAD_PAGEUP:
                m_currentMarkerLoad += mod;
                m_currentMarkerRpm  += mod * 10;
                SetCurrentLoad(m_currentMarkerLoad, m_currentMarkerRpm);
            break;

        case WXK_NUMPAD_PAGEDOWN:
                m_currentMarkerLoad -= mod;
                m_currentMarkerRpm  += mod * 10;
                SetCurrentLoad(m_currentMarkerLoad, m_currentMarkerRpm);
            break;

        case WXK_NUMPAD_HOME:
                m_currentMarkerLoad += mod;
                m_currentMarkerRpm  -= mod * 10;
                SetCurrentLoad(m_currentMarkerLoad, m_currentMarkerRpm);
            break;

        case WXK_NUMPAD_END:
                m_currentMarkerLoad -= mod;
                m_currentMarkerRpm  -= mod * 10;
                SetCurrentLoad(m_currentMarkerLoad, m_currentMarkerRpm);
            break;


        case WXK_ESCAPE:{
                wxWindow * gwin = GetGrandParent();
                if(gwin) gwin->SetFocus();
                }break;

        case WXK_F12:
            if(m_groupManager)
                m_groupManager->ResetGaugesMinMax( m_props.groupName );

            break;
    }

    if( m_groupManager->IsEditMode() ) {

        WindowHelper whe(GetGrandParent());
        int res = whe.Action(event);

        whe.GroupAction(m_groupManager, res, m_props.groupName, true);

        event.Skip();
        return;

    }


//	const int prevCursorRow = m_cursorRow, prevCursorCol = m_cursorCol;

//	int func = GetFunctionByKey(event.GetKeyCode(), event.GetModifiers());

    int func = -1;

    wxString evtstr = m_hotkeyMgr->GetEvent(event);

    if( m_hotkeySymbolToEnum.find(evtstr) == m_hotkeySymbolToEnum.end() )
        func = -1;
    else
        func = m_hotkeySymbolToEnum[evtstr];


	switch ( func )
	{
		case VEMS_TABLE_EDITOR_KEY_NAVIGATE_LEFT:
            MoveCursor( LEFT );
            break;
		case VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_COPY_LEFT:
            MoveCursor( LEFT, COPY );
            break;
		case VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_HIGHLIGHT_LEFT:
            MoveCursor( LEFT, HIGHLIGHT );
            break;
		case VEMS_TABLE_EDITOR_KEY_NAVIGATE_RIGHT:
            MoveCursor( RIGHT );
            break;
		case VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_COPY_RIGHT:
            MoveCursor( RIGHT, COPY );
            break;
		case VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_HIGHLIGHT_RIGHT:
            MoveCursor( RIGHT, HIGHLIGHT );
            break;
		case VEMS_TABLE_EDITOR_KEY_NAVIGATE_UP:
            MoveCursor( UP );
            break;
		case VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_COPY_UP:
            MoveCursor( UP, COPY );
            break;
		case VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_HIGHLIGHT_UP:
            MoveCursor( UP, HIGHLIGHT );
            break;
		case VEMS_TABLE_EDITOR_KEY_NAVIGATE_DOWN:
            MoveCursor( DOWN );
            break;
		case VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_COPY_DOWN:
            MoveCursor( DOWN, COPY );
            break;
		case VEMS_TABLE_EDITOR_KEY_NAVIGATE_AND_HIGHLIGHT_DOWN:
            MoveCursor( DOWN, HIGHLIGHT );
            break;
		case VEMS_TABLE_EDITOR_KEY_NAVIGATE_GRID_LEFTTOP_CORNER:
		{
			Freeze();

			UnhighlightAllCells();

			m_cursorRow = m_cursorCol = 0;
			EnsureCellVisible(m_cursorRow, m_cursorCol);

			Thaw();
		}
		break;

		case VEMS_TABLE_EDITOR_KEY_NAVIGATE_GRID_RIGHTBOTTOM_CORNER:
		{
			Freeze();

			UnhighlightAllCells();

			m_cursorRow = GetNumberRows() - 1;
			m_cursorCol = GetNumberCols() - 1;

			EnsureCellVisible(m_cursorRow, m_cursorCol);

			Thaw();
		}
		break;

		case VEMS_TABLE_EDITOR_KEY_INC_SMALL:
		{
			//IncCellSmall();
			ChangeSelectedCells( m_smallStepValue );
		}
		break;

		case VEMS_TABLE_EDITOR_KEY_INC_LARGE:
		{
			//IncCellLarge();
			ChangeSelectedCells( m_largeStepValue );
		}
		break;

		case VEMS_TABLE_EDITOR_KEY_DEC_SMALL:
		{
			//DecCellSmall();
			ChangeSelectedCells( -m_smallStepValue );
		}
		break;

		case VEMS_TABLE_EDITOR_KEY_DEC_LARGE:
		{
			//DecCellLarge();
            ChangeSelectedCells( -m_largeStepValue );
            //ChangeCellValue( m_cursorRow, m_cursorCol,  -m_largeStepValue );
            //SendCursorCellChangedEvent();
		}
		break;

		case VEMS_TABLE_EDITOR_KEY_LOCK_CELLS:
		{
			LockUnlockCells();
		}
		break;

		case VEMS_TABLE_EDITOR_KEY_LOCK_ALL_CELLS:
		{
			UnhighlightAllCells();
			LockAllCells();
			SetFullRefresh();
			Refresh();
		}
		break;

		case VEMS_TABLE_EDITOR_KEY_UNLOCK_ALL_CELLS:
		{
			UnhighlightAllCells();
			UnlockAllCells();
			SetFullRefresh();
			Refresh();
		}
		break;


		case VEMS_TABLE_EDITOR_KEY_INTERPOLATE_CELLS:
		{
			if (AreCellsHighlighted())
			{
				InterpolateSelection();
			}
			else if ((m_cursorRow > 0) && (m_cursorRow < GetNumberRows() - 1) && (m_cursorCol > 0) && (m_cursorCol < GetNumberCols() - 1) && !IsCellLocked(m_cursorRow, m_cursorCol))
			{
				InterpolateCell(m_cursorRow, m_cursorCol);
			}
			else
			{
				wxBell();
			}
			UnhighlightAllCells();

			Refresh();
		}
		break;

        case VEMS_TABLE_EDITOR_KEY_MULTIPLY_CELLS:
		{
            ShowMultiplyDialog();
		} break;

        case VEMS_TABLE_EDITOR_KEY_DIVIDE_CELLS:
		{
            ShowDivideDialog();
		} break;

		case VEMS_TABLE_EDITOR_KEY_EDIT_VALUE:
		{
			RequestValueForCursorCell();
		}
		break;

        case VEMS_TABLE_EDITOR_KEY_SELECT_LOAD:
		{
			SelectLoad();
		} break;

        case VEMS_TABLE_EDITOR_KEY_AUTO_FOLLOW_LOAD:
		{
			m_isAutoFollowLoad = !m_isAutoFollowLoad;
		} break;

        case VEMS_TABLE_EDITOR_KEY_DISPLAY_HELP:
		{
			m_isShowHelp = !m_isShowHelp;
            SetFullRefresh();
			Refresh();
		} break;


		case VEMS_TABLE_EDITOR_KEY_HIGHLIGHT_ALL_CELLS:
		{
			ResetHighlightOrigin();

			if (AreAllCellsHighlighted())
			{
				UnhighlightAllCells();
			}
			else
			{
				HighlightAllCells();
			}
			SetFullRefresh();
			Refresh();
		}
		break;

		case VEMS_TABLE_EDITOR_KEY_UNDO_CELL_MODIFICATION:
		{
			if (AreCellsHighlighted())
			{
				UndoHighlightedCellsModification();
				UnhighlightAllCells();
				Refresh();
			}
			else if ((m_cursorRow >= 0) && (m_cursorCol >= 0) && WasCellModified(m_cursorRow, m_cursorCol))
			{
				UndoCellModification(m_cursorRow, m_cursorCol);
				AddNewRefreshCell(m_cursorRow, m_cursorCol);

				Refresh();
			}
		}
		break;

		case VEMS_TABLE_EDITOR_KEY_UNDO_ALL_CELLS_MODIFICATION:
		{
			UnhighlightAllCells();
			UndoAllCellsModification();
			SetFullRefresh();
			Refresh();
			Send( TableEditorEvent::UNDO_ALL_CELLS_MODIFICATION );
		}
		break;

		case VEMS_TABLE_EDITOR_WRITE_FW_CONFIG:
		{
            Send( TableEditorEvent::WRITE_FW_CONFIG );
		} break;

		case VEMS_TABLE_EDITOR_BURN_FW_CONFIG:
		{
            Send( TableEditorEvent::BURN_FW_CONFIG );
		} break;

		case VEMS_TABLE_EDITOR_REFRESH_FW_CONFIG:
		{
		    Logger::logf( LOG_MSG, _T("VEMS_TABLE_EDITOR_REFRESH_FW_CONFIG") );
            Send( TableEditorEvent::REFRESH_FW_CONFIG );
		} break;

		default:
		{
			if ( event.HasModifiers() || event.ShiftDown() ) break;
			const int keycode = event.GetKeyCode();
			wxString key = wxEmptyString;

            switch( keycode )
            {
                case WXK_NUMPAD0: key = _T('0'); break;
                case WXK_NUMPAD1: key = _T('1'); break;
                case WXK_NUMPAD2: key = _T('2'); break;
                case WXK_NUMPAD3: key = _T('3'); break;
                case WXK_NUMPAD4: key = _T('4'); break;
                case WXK_NUMPAD5: key = _T('5'); break;
                case WXK_NUMPAD6: key = _T('6'); break;
                case WXK_NUMPAD7: key = _T('7'); break;
                case WXK_NUMPAD8: key = _T('8'); break;
                case WXK_NUMPAD9: key = _T('9'); break;
                case WXK_NUMPAD_DECIMAL: key = _T('.'); break;
                default:
                    if( wxIsdigit(keycode) || keycode == _T('.') )
                    {
                        key.sprintf( _T("%c"), keycode );
                    }
            }

			if( key != wxEmptyString )
			{
				RequestValueForCursorCell( key );
			}
			else
			{
				event.Skip();
			}

		}
		break;
	}



}


void VEMSTableEditor::OnKeyUp( wxKeyEvent& event )
{
	const int func = GetFunctionByKey( event.GetKeyCode(), event.GetModifiers() );

	switch ( func )
	{
		case VEMS_TABLE_EDITOR_KEY_INC_SMALL:
		{
			//IncCellSmall();
            ChangeSelectedCells( 0.0, true );
			Logger::logf( LOG_MSG, _T("VEMSTableEditor::OnKeyUp() - VEMS_TABLE_EDITOR_KEY_INC_SMALL") );
		} break;
		case VEMS_TABLE_EDITOR_KEY_INC_LARGE:
		{
			//IncCellLarge();
			ChangeSelectedCells( 0.0, true );
			Logger::logf( LOG_MSG, _T("VEMSTableEditor::OnKeyUp() - VEMS_TABLE_EDITOR_KEY_INC_LARGE") );
		} break;
		case VEMS_TABLE_EDITOR_KEY_DEC_SMALL:
		{
			//DecCellSmall();
			ChangeSelectedCells( 0.0, true );
			Logger::logf( LOG_MSG, _T("VEMSTableEditor::OnKeyUp() - VEMS_TABLE_EDITOR_KEY_DEC_SMALL") );
		} break;
		case VEMS_TABLE_EDITOR_KEY_DEC_LARGE:
		{
			//DecCellLarge();
            ChangeSelectedCells( 0.0, true );
            Logger::logf( LOG_MSG, _T("VEMSTableEditor::OnKeyUp() - VEMS_TABLE_EDITOR_KEY_DEC_LARGE") );
		} break;
	}
}


/*
			VEMSCellEditorDialog dlg(wxGetActiveWindow(), cur_val, m_minCellValue, m_maxCellValue, cellRect, m_zoomboxFont, m_zoomboxTextColor, val_mask, initialValue);
			dlg.ShowModal();
			if (dlg.GetReturnCode() == wxID_OK)
			{
				if (dlg.IsValueAccepted())
				{
					wxString valStr;
					const float d = dlg.GetResultValue();
					SetCellValue(m_cursorRow, m_cursorCol, d);
					//AddNewRefreshCell(m_cursorRow, m_cursorCol);
					//m_isChange = true;
					Refresh();

					if( WasCellModified( m_cursorRow, m_cursorCol ) )
					{
                        //OnSingleCellChanged( m_cursorRow, m_cursorCol, d );
                        SingleCellChangedEvent e( SingleCellChangedEvent::TABLE );
                        e.row = m_cursorRow;
                        e.col = m_cursorCol;
                        e.value = d;
                        if ( GetParent() )
                        {
                            GetParent()->AddPendingEvent( e );
                        }
					}
				}
				else
				{
					wxMessageBox(_T("Value is incorrect or out of range"), _T("Error"), wxOK | wxICON_ERROR);
				}
			}
*/

SelectableWidgetBase* VEMSTableEditor::GetSelectableWidgetBase()
{
    wxWindow* win = GetGrandParent();

    if(!win) return NULL;

    TableEditorPanel* tep = wxDynamicCast(win,TableEditorPanel);

    if(!tep) return NULL;

    return tep;
}

wxWindow* VEMSTableEditor::GetSelectFaceParent()
{
    wxWindow* gp = GetGrandParent();

    if(!gp) return NULL;

    wxWindow* gpp = gp->GetParent();

    return gpp;
}

void VEMSTableEditor::OnMouseLeftUp(wxMouseEvent& evt)
{
    m_delta = wxDefaultPosition;

    if(!m_groupManager->IsEditMode()) return;

    wxWindow* gpp = GetSelectFaceParent();
    if(gpp) gpp->AddPendingEvent(evt);

    if(HasCapture())
        ReleaseMouse();

    evt.Skip();
}

void VEMSTableEditor::OnMouseMove(wxMouseEvent& evt)
{
    if(!m_groupManager->IsEditMode()) return;

    if(m_delta == wxDefaultPosition) {
        wxWindow* gpp = GetSelectFaceParent();
        if(gpp) gpp->AddPendingEvent(evt);
        evt.Skip();
        return;
    }

    if (evt.Dragging())
    {

        wxPoint pos = ClientToScreen(evt.GetPosition());

        wxPoint cpos = GetGrandParent()->GetPosition();

        GetGrandParent()->Move(wxPoint(pos.x - m_delta.x, pos.y - m_delta.y));

        //if(m_winTip) m_winTip->ShowMsg(wxString::Format(_T("xpos=%i\nypos=%i"),GetPosition().x,GetPosition().y));

        wxPointEx rdelta = GetGrandParent()->GetPosition() - cpos;

        wxVariantHash hash;
        hash[_T("groupname")] = m_props.groupName;
        hash[_T("cmd")] =  _T("gauge_move");
        hash[_T("sender_id")] = GetGrandParent() == NULL ? 0L : (long)GetGrandParent()->GetId();
        hash[_T("target_id")] = _T("0");
        hash[_T("deltapos")] << rdelta;

        m_groupManager->ProcessWidgetCommand(hash);

    }



    evt.Skip();
}

void VEMSTableEditor::OnMouseLeftClick(wxMouseEvent& event)
{


    if(m_groupManager->IsEditMode()) {


        CaptureMouse();

        wxPoint pos = ClientToScreen(event.GetPosition());

        wxPoint origin = GetGrandParent()->GetPosition();
        int dx =  pos.x - origin.x;
        int dy = pos.y - origin.y;
        m_delta = wxPoint(dx, dy);

        Refresh(false);

        SelectableWidgetBase* selw = GetSelectableWidgetBase();

        if( !selw->IsSelected() ) {

            if( !event.ShiftDown() ) {

                wxVariantHash hash;
                long id = GetGrandParent()->GetId();
                hash[_T("groupname")] = m_props.groupName;
                hash[_T("cmd")] =  _T("gauge_click");
                hash[_T("sender_id")] = id;
                hash[_T("target_id")] = _T("0");

                m_groupManager->ProcessWidgetCommand(hash);

            }

            selw->SetSelected(true);
            selw->DrawSelectionBitmap(this);
            Refresh(false);

        } else {
            if( event.ShiftDown() ) {
                selw->SetSelected(false);
                Refresh(false);
        }
        }

        event.Skip();
        return;
    }


	if (FindFocus() != this)
	{
		SetFocus();
	}

	const wxPoint& pos = event.GetPosition();

	int hit_row = 0, hit_col = 0;

	if (HitTestLeftTopCorner(pos))
	{
		// Do nothing at this time
	}
	else if (HitTestLabelRow(pos, &hit_row))
	{
		ResetHighlightOrigin();

		UnhighlightAllCells();

		if (IsRowHighlighted(hit_row))
		{
			UnhighlightRow(hit_row);
		}
		else
		{
			HighlightRow(hit_row);
		}
		//Logger::logf(LOG_MSG,_T("HitTestLabelRow hit_row = %i"),hit_row);

		for(short a = 0; a < GetNumberCols(); a++)
            AddNewRefreshCell( hit_row,a );

		Refresh();
	}
	else if (HitTestLabelCol(pos, &hit_col))
	{
		ResetHighlightOrigin();

		UnhighlightAllCells();

		if (IsColumnHighlighted(hit_col))
		{
			UnhighlightColumn(hit_col);
		}
		else
		{
			HighlightColumn(hit_col);
		}
		//Logger::logf(LOG_MSG,_T("HitTestLabelCol hit_row = %i"),hit_col);

		for(short a = 0; a < GetNumberRows(); a++)
            AddNewRefreshCell(a, hit_col);


		Refresh();
	}
	else if (HitTestCell(pos, &hit_row, &hit_col))
	{
	    AddOldRefreshCell(m_cursorRow, m_cursorCol );

		UnhighlightAllCells();

		m_cursorRow = hit_row;
		m_cursorCol = hit_col;

		if (event.ControlDown() &&
			m_highlightOriginRow >= 0 && m_highlightOriginRow < GetNumberRows() &&
			m_highlightOriginCol >= 0 && m_highlightOriginCol < GetNumberCols())
		{
			HighlightRectangularAreaOfCells(m_highlightOriginRow, m_highlightOriginCol, m_cursorRow, m_cursorCol);
		}
		else
		{
			m_highlightOriginRow = hit_row;
			m_highlightOriginCol = hit_col;
		}

		EnsureCellVisible(m_cursorRow, m_cursorCol);

        AddNewRefreshCell(hit_row, hit_col);

		Refresh();
	}

	event.Skip();
}

void VEMSTableEditor::OnMouseLeftDoubleClick(wxMouseEvent& event)
{
    if(m_groupManager->IsEditMode()) {
        event.Skip();
        return;
    }

	ResetHighlightOrigin();

	const wxPoint& pos = event.GetPosition();

	int hit_row = 0, hit_col = 0;
	if (HitTestCell(pos, &hit_row, &hit_col))
	{
		m_cursorRow = hit_row;
		m_cursorCol = hit_col;

		EnsureCellVisible(m_cursorRow, m_cursorCol);

		Refresh();

		RequestValueForCursorCell();
	}
    else if ( HitTestLabelRow( pos, &hit_row ) ) // edit row label
	{
        m_selectedRowLabel =  hit_row;
        UnhighlightAllCells();
        ShowRowHeaderDialog( VEMS_TABLE_EDITOR_PM_EDIT_HEADER );
	}
	else if ( HitTestLabelCol( pos, &hit_col ) )// edit col label
	{
        m_selectedColLabel =  hit_col;
        UnhighlightAllCells();
        ShowColHeaderDialog( VEMS_TABLE_EDITOR_PM_EDIT_HEADER );
	}

	event.Skip();
}


void VEMSTableEditor::AppendPopupMenu( wxMenu& menu, int popupMenuID, const wxString& eventSymbol, const wxString& miscStr )
{
    wxString titleStr = m_hotkeyMgr->GetHotKeyTitleBySymbol( eventSymbol );
    wxString hotkeyStr = m_hotkeyMgr->GetHotKeyStringBySymbol( eventSymbol );

    wxString label = titleStr + miscStr + _T("\t") + hotkeyStr;
    menu.Append( popupMenuID, label );
}


void VEMSTableEditor::OnMouseRightClick(wxMouseEvent& event)
{
	if (FindFocus() != this)
	{
        SetFocus();
	}

	ResetHighlightOrigin();

	const wxPoint& pos = event.GetPosition();

	int hit_row = 0, hit_col = 0;
	if (HitTestCell(pos, &hit_row, &hit_col))
	{
	    m_oldRefreshCells.Add( wxPoint( m_cursorRow,m_cursorCol));

		m_cursorRow = hit_row;
		m_cursorCol = hit_col;
		EnsureCellVisible(m_cursorRow, m_cursorCol);

		m_currentRefreshCells.Add( wxPoint(m_cursorRow,m_cursorCol) );

		///m_isChange = true;

		Refresh();

		wxMenu menu;

		AppendPopupMenu( menu, VEMS_TABLE_EDITOR_PM_MULTIPLY_CELLS, _T("VEMS_TABLE_EDITOR_KEY_MULTIPLY_CELLS") );
		AppendPopupMenu( menu, VEMS_TABLE_EDITOR_PM_DIVIDE_CELLS, _T("VEMS_TABLE_EDITOR_KEY_DIVIDE_CELLS") );
		menu.AppendSeparator();

		if (!IsCellLocked(m_cursorRow, m_cursorCol))
		{
			wxString smallStepStr, largeStepStr;
			smallStepStr.sprintf(GetValueFormatMask(), m_smallStepValue);
			largeStepStr.sprintf(GetValueFormatMask(), m_largeStepValue);

			AppendPopupMenu( menu, VEMS_TABLE_EDITOR_PM_DEC_CELL_LARGE, _T("VEMS_TABLE_EDITOR_KEY_DEC_LARGE"), largeStepStr );
			AppendPopupMenu( menu, VEMS_TABLE_EDITOR_PM_DEC_CELL_SMALL, _T("VEMS_TABLE_EDITOR_KEY_DEC_SMALL"), smallStepStr );
			AppendPopupMenu( menu, VEMS_TABLE_EDITOR_PM_INC_CELL_SMALL, _T("VEMS_TABLE_EDITOR_KEY_INC_SMALL"), smallStepStr );
			AppendPopupMenu( menu, VEMS_TABLE_EDITOR_PM_INC_CELL_LARGE, _T("VEMS_TABLE_EDITOR_KEY_INC_LARGE"), largeStepStr );

			menu.AppendSeparator();
		}

		if (AreCellsHighlighted())
		{
			//menu.Append(VEMS_TABLE_EDITOR_PM_LOCK_CELL, _T("Lock Selected Cells\t") + GetKeyTextualForFunction(VEMS_TABLE_EDITOR_KEY_LOCK_CELLS));
            AppendPopupMenu( menu, VEMS_TABLE_EDITOR_PM_LOCK_CELL, _T("VEMS_TABLE_EDITOR_KEY_LOCK_CELLS") );
		}
		else
		{
		    if (!IsCellLocked(m_cursorRow, m_cursorCol))
            {
                //menu.Append(VEMS_TABLE_EDITOR_PM_LOCK_CELL, _T("Lock Cell\t") + GetKeyTextualForFunction(VEMS_TABLE_EDITOR_KEY_LOCK_CELLS));
                AppendPopupMenu( menu, VEMS_TABLE_EDITOR_PM_LOCK_CELL, _T("VEMS_TABLE_EDITOR_KEY_LOCK_CELLS") );
            }
            else
            {
                //menu.Append(VEMS_TABLE_EDITOR_PM_UNLOCK_CELL, _T("Unlock Cell\t") + GetKeyTextualForFunction(VEMS_TABLE_EDITOR_KEY_LOCK_CELLS));
                AppendPopupMenu( menu, VEMS_TABLE_EDITOR_PM_UNLOCK_CELL, _T("VEMS_TABLE_EDITOR_KEY_LOCK_CELLS") );
            }
		}

		//menu.Append(VEMS_TABLE_EDITOR_PM_LOCK_ALL_CELLS, _T("Lock All Cells\t") + GetKeyTextualForFunction(VEMS_TABLE_EDITOR_KEY_LOCK_ALL_CELLS));
		//menu.Append(VEMS_TABLE_EDITOR_PM_UNLOCK_ALL_CELLS, _T("Unlock All Cells\t") + GetKeyTextualForFunction(VEMS_TABLE_EDITOR_KEY_UNLOCK_ALL_CELLS));
        AppendPopupMenu( menu, VEMS_TABLE_EDITOR_PM_LOCK_ALL_CELLS, _T("VEMS_TABLE_EDITOR_KEY_LOCK_ALL_CELLS") );
        AppendPopupMenu( menu, VEMS_TABLE_EDITOR_PM_UNLOCK_ALL_CELLS, _T("VEMS_TABLE_EDITOR_KEY_UNLOCK_ALL_CELLS") );
		menu.AppendSeparator();

		if (!IsCellLocked(m_cursorRow, m_cursorCol) && WasCellModified(m_cursorRow, m_cursorCol))
		{
			//menu.Append(VEMS_TABLE_EDITOR_PM_UNDO_CELL_MODIFICATION, _T("Undo Cell Modification\t") + GetKeyTextualForFunction(VEMS_TABLE_EDITOR_KEY_UNDO_CELL_MODIFICATION));
            AppendPopupMenu( menu, VEMS_TABLE_EDITOR_PM_UNDO_CELL_MODIFICATION, _T("VEMS_TABLE_EDITOR_KEY_UNDO_CELL_MODIFICATION") );
		}
		//menu.Append(VEMS_TABLE_EDITOR_PM_UNDO_ALL_CELLS_MODIFICATION,  _T("Undo All Modifications\t") + GetKeyTextualForFunction(VEMS_TABLE_EDITOR_KEY_UNDO_ALL_CELLS_MODIFICATION));
		AppendPopupMenu( menu, VEMS_TABLE_EDITOR_PM_UNDO_ALL_CELLS_MODIFICATION, _T("VEMS_TABLE_EDITOR_KEY_UNDO_ALL_CELLS_MODIFICATION") );
		menu.AppendSeparator();

		menu.Append(VEMS_TABLE_EDITOR_PM_ACCEPT_ALL_CHANGES, _T("Accept All Changes"));

		menu.AppendSeparator();
		menu.Append( VEMS_TABLE_EDITOR_PM_IMPORT, _T("Import") );
		menu.Append( VEMS_TABLE_EDITOR_PM_REMAP, _T("Remap") );

		//gauge  group functions
		//if( m_groupManager )
		if( GetItemID() != -1 )
		{
		    // tables from ini file
		    wxMenu* tables = new wxMenu();
		    for ( size_t i = 0; i < m_tables.GetCount(); ++i )
		    {
                int id = wxNewId();
                tables->Append( id, m_tables[i] );
                m_menuIDtoTableName[id] = m_tables[i];
                Connect( id, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(VEMSTableEditor::OnPopupMenuChangeTable) );
		    }

            menu.SetTitle( m_props.groupName );
            menu.AppendSeparator();
            menu.AppendSubMenu( tables, _T("Change Table To") );
		    menu.AppendSeparator();
		    menu.AppendSubMenu( createAddSubMenu(), _T("add") );
		    menu.Append( createRemoveMenu(&menu) );
		    menu.Append( createSaveMenu(&menu) );
		}

		/*if( m_popupMenuTitle != wxEmptyString )
		{
            menu.SetTitle( m_popupMenuTitle );
		}

		//menu.AppendSeparator();
		AppendPopupMenuItem( wxID_ANY, _T("Save all") );
		AppendPopupMenuItem( wxID_ANY, _T("Add all") );

		wxMenuItemList& list = m_popupMenu.GetMenuItems();

        wxMenuItemList::iterator iter;

        for( iter = list.begin(); iter != list.end(); ++iter )
        {
            wxMenuItem* current = *iter;

            menu.Append( current );
        }
*/
		PopupMenu(&menu, pos.x, pos.y);
	}
	else if ( HitTestLabelRow( pos, &hit_row ) ) // edit row label
	{
        m_selectedRowLabel =  hit_row;
        wxMenu menu;
        menu.Append( VEMS_TABLE_EDITOR_PM_EDIT_HEADER, _T("Edit") );
        menu.Append( VEMS_TABLE_EDITOR_PM_MOVE_HEADER, _T("Move") );
        wxMenuItem* insertMenu = menu.Append( VEMS_TABLE_EDITOR_PM_INSERT_HEADER, _T("Insert") );

        if ( m_selectedRowLabel == 0 )
        {
            insertMenu->Enable( false );
        }


        PopupMenu( &menu, pos.x, pos.y );
	}
	else if ( HitTestLabelCol( pos, &hit_col ) )
	{
        m_selectedColLabel =  hit_col;
        wxMenu menu;
        menu.Append( VEMS_TABLE_EDITOR_PM_EDIT_HEADER, _T("Edit") );
        menu.Append( VEMS_TABLE_EDITOR_PM_MOVE_HEADER, _T("Move") );
        wxMenuItem* insertMenu = menu.Append( VEMS_TABLE_EDITOR_PM_INSERT_HEADER, _T("Insert") );

        if ( m_selectedColLabel > m_numCols - 2 )
        {
            insertMenu->Enable( false );
        }

        PopupMenu( &menu, pos.x, pos.y );
	}

	event.Skip();
}

bool VEMSTableEditor::SearchInPointArray(const PointArray &parr, const wxPoint &point)
{

    for(uint16_t a = 0; a < parr.GetCount(); a++)
        if( parr[a] == point ) return true;

    return false;

}

void VEMSTableEditor::DrawBackgroundCustomCells(wxDC &dc, const wxPoint &point)
{

    if(!m_refreshAllCells) {

        if( !SearchInPointArray(m_refreshedCells, point) )
            m_refreshedCells.Add(point);
        else
            return;

    }

    int i = point.x;
    int j = point.y;

    wxRect rc = GetCellRectPix(i, j);
    if(j>0) rc.x--;
    if(i>0) rc.y--;  // header flicking

    if (IsCellHighlighted(i, j))
    {
        dc.SetBrush(m_cellHighlightBkColor);
        dc.SetPen(m_cellHighlightBkColor);
    }
    else if (IsCellLocked(i, j))
    {
        dc.SetBrush(m_lockedCellBkColor);
        dc.SetPen(m_lockedCellBkColor);
    }
    else
    {
        //Logger::logf(LOG_MSG,_T("VEMSTableEditor::DrawBackgroundCustomCells  row=%i, col=%i"),i,j);

        int maxIndex = m_gradientColorArray.Count()-1;
        if(maxIndex < 0){ //
            m_gradientColorArray.Add(wxWHITE);
        }

        //0-->1;


        wxColor color = CalculateCellBgColor(point);

        dc.SetPen(color);
        //dc.SetPen(wxColor(0, 0, 0));
        dc.SetBrush(color);

    }

    dc.DrawRectangle(rc);

}


wxColor VEMSTableEditor::CalculateCellBgColor(const wxPoint &point)
{
    if( m_gradientColorArray.IsEmpty() ) { return *wxRED; }

    int maxIndex = m_gradientColorArray.Count()-1;

    const float clr_factor = (GetCellValue( point.x , point.y ) - m_minCellValue) / (m_maxCellValue - m_minCellValue);

    if( clr_factor < 0 ) { return *wxRED; }

    float baseIndex = clr_factor * maxIndex; // size:3// 0-->0; 0.5-->1; 1-->2
    int startIndex = (int)baseIndex;
    int endIndex = startIndex + 1;
    if(endIndex >= maxIndex ){  endIndex = maxIndex;}

    wxColor startColor = m_gradientColorArray.Item(startIndex);
    wxColor endColor = m_gradientColorArray.Item(endIndex);
    //fprintf(stderr, "i:%d j:%d (%d:%d)",i,j, startIndex, endIndex);
    //Logger::logf(LOG_MSG,_T("VEMSTableEditor::DrawBackgroundCustomCells i:%d j:%d (%d:%d)"),i,j, startIndex, endIndex);

    float reminder = baseIndex - startIndex; // 0-->1

    const int r = startColor.Red() - int((int(startColor.Red()) - int(endColor.Red())) * reminder);
    const int g = startColor.Green() - int((int(startColor.Green()) - int(endColor.Green())) * reminder);
    const int b = startColor.Blue() - int((int(startColor.Blue()) - int(endColor.Blue())) * reminder);

    return wxColor(r,g,b);
}

void VEMSTableEditor::DrawCellLabel(wxDC &dc, const wxPoint &point)
{


    if(!m_refreshAllCells) {

        if( !SearchInPointArray(m_refreshedCells, point) )
            m_refreshedCells.Add(point);
        else
            return;

    }

    int i = point.x;
    int j = point.y;


    dc.SetFont(m_cellFont);
    dc.SetBrush(*wxTRANSPARENT_BRUSH);
    dc.SetPen(m_gradientColorArray.Item(0));
    dc.SetBackgroundMode(wxTRANSPARENT);

    wxString str;
    //str.sprintf(GetValueFormatMask(), m_cellValues[i][j]);
    str.sprintf(GetValueFormatMask(), m_cellValues.get( i, j ) );

    wxRect rc = GetCellRectPix(i, j);

    if (WasCellModified(i, j))
    {
        dc.SetTextForeground(m_modifiedCellTextColor);
    }
    else if (IsCellHighlighted(i, j))
    {
        dc.SetTextForeground(m_cellHighlightTextColor);
    }
    else
    {
        dc.SetTextForeground(m_cellTextColor);
    }

    dc.DrawLabel(str, rc, wxALIGN_CENTER);


    // Draw cursor
    if ((i == m_cursorRow) && (j == m_cursorCol))
    {
        wxPen oldPen = dc.GetPen();
        dc.SetPen(wxPen(m_cursorColor, 1, wxSOLID));
        rc.width--, rc.height--;
        dc.DrawRectangle(rc);
        rc.Deflate(1, 1);
        dc.DrawRectangle(rc);
        dc.SetPen(oldPen);
    }

}

void VEMSTableEditor::ClearRefreshArrays()
{

    m_currentRefreshCells.Clear();
    m_oldRefreshCells.Clear();


    m_currentRefreshCellsforLoad.Clear();
    m_oldRefreshCellsforLoad.Clear();


    m_currentRefresRowLabel.Clear();
    m_oldRefresRowLabel.Clear();

    m_currentRefresColLabel.Clear();
    m_oldRefresColLabel.Clear();


}

void VEMSTableEditor::OnPaint(wxPaintEvent& event)
{
	wxSize sz = GetClientSize();

//Logger::logf(LOG_MSG,_T("%i,%i"),sz.x,sz.y);

	wxBufferedPaintDC dco(this);


    wxMemoryDC dc;

    //dc.SetBackground( *wxBLACK_BRUSH );

    if(dc.IsOk())
        dc.Clear();

    if( !m_face ){ m_face = new wxBitmap(sz.x, sz.y); }
    dc.SelectObjectAsSource(*m_face);

    //dc.SelectObject(*m_face);


    wxRect rect(0, 0, sz.x, sz.y);

    int i, j;


    if(m_refreshAllCells) {
        CalculateGridValueLimits();

        // Clear main background
        dc.SetBrush(m_bkColor);
        dc.SetPen(m_bkColor);
        dc.DrawRectangle(rect);

        ClearRefreshArrays();

    }
    // Draw background of custom cells (locked, highlighted etc.)

    if(m_refreshAllCells) {
        for (i = m_drawOriginRow; i < GetNumberRows(); ++i)
            for (j = m_drawOriginCol; j < GetNumberCols(); ++j)
                DrawBackgroundCustomCells(dc, wxPoint(i,j));
    }
    else {
        if(m_isChange || m_isLoadChange)
            RefreshBackgroundCustomCells(dc);

    }


    if(m_isChange || m_isLoadChange || m_refreshAllCells) {
    // Draw grid lines
        if (GetNumberCols() > 0 && GetNumberRows() > 0)
        {
            dc.SetPen(m_gridLinesColor);
            const wxRect leftTopCell = GetCellRectPix(0, 0);
            const wxRect rightBottomCell = GetCellRectPix(GetNumberRows() - 1, GetNumberCols() - 1);

            for (i = m_drawOriginRow + 1; i <= GetNumberRows(); ++i)
            {
                const int y = leftTopCell.y + (int)(i * m_cellSize.y - 1);
                dc.DrawLine(leftTopCell.x, y, rightBottomCell.GetRight() + 1, y);
            }

            for (i = m_drawOriginCol + 1; i <= GetNumberCols(); ++i)
            {
                const int x = leftTopCell.x + (int)(i * m_cellSize.x - 1);
                dc.DrawLine(x, leftTopCell.y, x, rightBottomCell.GetBottom());
            }
        }

    }


    // Draw current load
    if (m_showLoadMarker && (m_isLoadChange || m_isChange || m_refreshAllCells) )
    {
        DrawCurrentLoadMarker(dc, rect);

    }

    if(m_refreshAllCells) {


        dc.SetFont(m_labelFont);
        dc.SetBrush(m_labelBkColor);
        dc.SetPen(m_labelBkColor);

        /// draw right side and bottom "corner"
        dc.DrawRectangle( 0, m_gridRect.GetBottom()+1, m_gridRect.GetRight()+5, 5);
        dc.DrawRectangle( m_gridRect.GetRight()+1, 0, 5, m_gridRect.GetBottom()+5 );


        /// Draw upper top corner
        wxRect upperTopCorner(0, 0, (int)(m_rowLabelWidth), (int)(m_colLabelHeight));
        dc.DrawRectangle(upperTopCorner);

        dc.SetBackgroundMode(wxTRANSPARENT);
        dc.SetTextForeground(m_labelTextColor);

        /// Draw column labels
        for (i = m_drawOriginCol; i < GetNumberCols(); ++i)
        {
            DrawColLabel(dc, i);
        }

        /// Draw row labels
        for (i = m_drawOriginRow; i < GetNumberRows(); ++i)
        {
            DrawRowLabel(dc, i);
        }

    }

    if(m_isLoadChange || m_isChange) {

        dc.SetFont(m_labelFont);
        dc.SetBrush(m_labelBkColor);
        dc.SetPen(m_labelBkColor);

        /// draw right side and bottom "corner"
        dc.DrawRectangle( 0, m_gridRect.GetBottom()+1, m_gridRect.GetRight()+5, 5);
        dc.DrawRectangle( m_gridRect.GetRight()+1, 0, 5, m_gridRect.GetBottom()+5 );


        /// Draw upper top corner
        wxRect upperTopCorner(0, 0, (int)(m_rowLabelWidth), (int)(m_colLabelHeight));
        dc.DrawRectangle(upperTopCorner);

        dc.SetBackgroundMode(wxTRANSPARENT);
        dc.SetTextForeground(m_labelTextColor);

        DrawRefreshLabels(dc);

    }



    if(m_refreshAllCells) {
        for (i = m_drawOriginRow; i < GetNumberRows(); ++i)
            for (j = m_drawOriginCol; j < GetNumberCols(); ++j)
                DrawCellLabel(dc, wxPoint(i,j) );
    }
    else {
        if(m_isChange || m_isLoadChange)
            RefreshCellLabels(dc);
    }

    if ( m_isShowHelp ) {
        drawHelpPanel( dc );
    }

    dc.SelectObject(wxNullBitmap);

    dco.Clear();
    dco.DrawBitmap(*m_face, 0, 0, 1);


    if(m_refreshAllCells)
        m_refreshAllCells = false;

    if(m_isChange) {

        m_oldRefreshCells.Clear();
        WX_APPEND_ARRAY(m_oldRefreshCells,m_currentRefreshCells);
        m_currentRefreshCells.Clear();

        m_oldRefresRowLabel.Clear();
        WX_APPEND_ARRAY(m_oldRefresRowLabel,m_currentRefresRowLabel);
        m_currentRefresRowLabel.Clear();

        m_oldRefresColLabel.Clear();
        WX_APPEND_ARRAY(m_oldRefresColLabel,m_currentRefresColLabel);
        m_currentRefresColLabel.Clear();

        m_isChange = false;
    }


    if(m_isLoadChange) {

        m_oldRefreshCellsforLoad.Clear();
        WX_APPEND_ARRAY(m_oldRefreshCellsforLoad,m_currentRefreshCellsforLoad);
        m_currentRefreshCellsforLoad.Clear();

        m_oldRefresRowLabel.Clear();
        WX_APPEND_ARRAY(m_oldRefresRowLabel,m_currentRefresRowLabel);
        m_currentRefresRowLabel.Clear();

        m_oldRefresColLabel.Clear();
        WX_APPEND_ARRAY(m_oldRefresColLabel,m_currentRefresColLabel);
        m_currentRefresColLabel.Clear();

        m_isLoadChange = false;
    }

    m_oldRefreshCells.Clear();

    SelectableWidgetBase* selw = GetSelectableWidgetBase();

    if(selw && selw->IsSelected()) {
        wxBitmap* selbmp = selw->GetSelectionBitmap();
        if(selbmp) dco.DrawBitmap( *selbmp, 0,0 );
    }

    m_isResized = false;

}

void VEMSTableEditor::DrawRefreshLabels(wxDC &dc)
{
    size_t i;

    /// Draw column labels
    for (i = 0; i < m_currentRefresColLabel.GetCount(); i++)
        DrawColLabel(dc, m_currentRefresColLabel[i]);

    for (i = 0; i < m_oldRefresColLabel.GetCount(); i++)
        DrawColLabel(dc, m_oldRefresColLabel[i]);



    /// Draw row labels
    for (i = 0; i < m_currentRefresRowLabel.GetCount(); i++)
        DrawRowLabel(dc, m_currentRefresRowLabel[i]);

    for (i = 0; i < m_oldRefresRowLabel.GetCount(); i++)
        DrawRowLabel(dc, m_oldRefresRowLabel[i]);


}

void VEMSTableEditor::DrawRowLabel(wxDC &dc, int row)
{
    const wxRect rc = GetLabelRowRect(row);

    wxString labelStr;
    //labelStr.sprintf(_T("%d %%"), GetRowLabelValue(row));
    labelStr.sprintf(_T("%d"), GetRowLabelValue(row));

    dc.DrawRectangle(rc);
    Draw3DBorder(dc, rc);

    if ( WasRowLabelModified( row ) )
    {
        dc.SetTextForeground(m_modifiedCellTextColor);
    }
    else
    {
        dc.SetTextForeground(m_cellTextColor);
    }

    dc.DrawLabel(labelStr, rc, wxALIGN_CENTER);
}

void VEMSTableEditor::DrawColLabel(wxDC &dc, int col)
{
    const wxRect rc = GetLabelColRect(col);

    wxString labelStr;
    labelStr.sprintf(_T("%d"), GetColLabelValue(col));

    dc.DrawRectangle(rc);
    Draw3DBorder(dc, rc);

    if ( WasColLabelModified( col ) )
    {
        dc.SetTextForeground(m_modifiedCellTextColor);
    }
    else
    {
        dc.SetTextForeground(m_cellTextColor);
    }

    dc.DrawLabel(labelStr, rc, wxALIGN_CENTER);
}

void VEMSTableEditor::RefreshBackgroundCustomCells(wxDC &dc)
{
    /*
    Logger::logf(LOG_MSG,_T("RefreshBackgroundCustomCells c = %i, o = %i, cL = %i, oL = %i")
    ,m_currentRefreshCells.GetCount()
    ,m_oldRefreshCells.GetCount()
    ,m_currentRefreshCellsforLoad.GetCount()
    ,m_oldRefreshCellsforLoad.GetCount()
    );
*/
    m_refreshedCells.Clear();

	size_t a;

    for(a = 0; a < m_currentRefreshCells.GetCount(); a++)
        DrawBackgroundCustomCells(dc, m_currentRefreshCells[a]);

    for(a = 0; a < m_oldRefreshCells.GetCount(); a++)
        DrawBackgroundCustomCells(dc, m_oldRefreshCells[a]);


    for(a = 0; a < m_currentRefreshCellsforLoad.GetCount(); a++)
        DrawBackgroundCustomCells(dc, m_currentRefreshCellsforLoad[a]);

    for(a = 0; a < m_oldRefreshCellsforLoad.GetCount(); a++)
        DrawBackgroundCustomCells(dc, m_oldRefreshCellsforLoad[a]);

}

void VEMSTableEditor::RefreshCellLabels(wxDC &dc)
{
    m_refreshedCells.Clear();

	size_t a;

    for(a = 0; a < m_currentRefreshCells.GetCount(); a++)
        DrawCellLabel(dc, m_currentRefreshCells[a]);

    for(a = 0; a < m_oldRefreshCells.GetCount(); a++)
        DrawCellLabel(dc, m_oldRefreshCells[a]);


    for(a = 0; a < m_currentRefreshCellsforLoad.GetCount(); a++)
        DrawCellLabel(dc, m_currentRefreshCellsforLoad[a]);

    for(a = 0; a < m_oldRefreshCellsforLoad.GetCount(); a++)
        DrawCellLabel(dc, m_oldRefreshCellsforLoad[a]);

}

void VEMSTableEditor::OnEraseBackground(wxEraseEvent& ev)
{
	// Do nothing on erase
}

bool VEMSTableEditor::AddNewRefreshColLabelIndex(short idx)
{
    if( idx < m_drawOriginCol || idx >= m_numCols ) return false;

    m_currentRefresColLabel.Add(idx);

    return true;
}

bool VEMSTableEditor::AddNewRefreshRowLabelIndex(short idx)
{
    if( idx < m_drawOriginRow || idx >= m_numRows ) return false;

    m_currentRefresRowLabel.Add(idx);

    return true;
}

bool VEMSTableEditor::AddNewRefreshCell(const wxPoint& point)
{
    if(point.x < 0 || point.y < 0 || point.x >= m_numRows || point.y >= m_numCols) return false;

    m_currentRefreshCells.Add(point);
    m_isChange = true;
    return true;

}

bool VEMSTableEditor::AddNewRefreshCell(short x, short y)
{

    if(x < 0 || y < 0 || x >= m_numRows || y >= m_numCols) return false;

    m_currentRefreshCells.Add(wxPoint(x,y));
    m_isChange = true;
    return true;
}

bool VEMSTableEditor::AddOldRefreshCell(const wxPoint& point)
{
    if(point.x < 0 || point.y < 0 || point.x >= m_numRows || point.y >= m_numCols) return false;

    m_oldRefreshCells.Add(point);
    m_isChange = true;
    return true;
}

bool VEMSTableEditor::AddOldRefreshCell(short x, short y)
{

    if(x < 0 || y < 0 || x >= m_numRows || y >= m_numCols) return false;

    m_oldRefreshCells.Add(wxPoint(x,y));
    m_isChange = true;
    return true;
}

bool VEMSTableEditor::AddNewRefreshCellforLoad(const wxPoint& point)
{
    if(point.x < 0 || point.y < 0 || point.x >= m_numRows || point.y >= m_numCols) return false;

    m_currentRefreshCellsforLoad.Add(point);
    m_isLoadChange = true;
    return true;
}

bool VEMSTableEditor::AddNewRefreshCellforLoad(short x, short y)
{

    if(x < 0 || y < 0 || x >= m_numRows || y >= m_numCols) return false;

    m_currentRefreshCellsforLoad.Add(wxPoint(x,y));
    m_isLoadChange = true;
    return true;
}

void VEMSTableEditor::CalculateGridValueLimits()
{

    m_minGridRpm  = m_colLabels[0] - ( m_colLabels[1] - m_colLabels[0] )/2.0;
    m_maxGridRpm  = ( m_colLabels[m_numCols - 2] - m_colLabels[m_numCols - 3] )/2.0 + m_colLabels[m_numCols - 1];

    m_maxGridLoad = m_rowLabels[0] - ( m_rowLabels[1] - m_rowLabels[0] )/2.0;
    m_minGridLoad = ( m_rowLabels[m_numRows - 1] - m_rowLabels[m_numRows - 2] )/2.0 + m_rowLabels[m_numRows - 1];

}

void VEMSTableEditor::SetLoadMarkerRadius(double r)
{
    /// temporarly radius <= cell_height/2 !!!! (tip. is equal)

    m_loadMarkerRadius = r;
    m_gridWarningRect = m_gridRect;
    m_gridWarningRect.Deflate( (int)m_loadMarkerRadius );

}

void VEMSTableEditor::CalculateLoadCellPoints()
{
    if(m_maxGridRpm == 0.0) return;

	int col = -1, row = -1, i;

    double min_Cval = 0;
    double max_Cval = 0;
    double min_Rval = 0;
    double max_Rval = 0;
    double prev_val = 0;
    double next_val = 0;
    double tmp = 0;


    if(m_currentMarkerRpm < m_minGridRpm){
        col = 0;

    }else if(m_currentMarkerRpm > m_maxGridRpm){
        col = m_numCols - 1;

    }else {

        for (i = 0; i < m_numCols; ++i)
        {

            if(!i)
                prev_val = m_colLabels[i] - ( m_colLabels[i+1] - m_colLabels[i] );
            else
                prev_val = m_colLabels[i-1];

            if(i == m_numCols - 1)
                next_val = ( m_colLabels[i] - m_colLabels[i-1] ) + m_colLabels[i];
            else
                next_val = m_colLabels[i+1];

            min_Cval = (prev_val + m_colLabels[i]) / 2.0;
            max_Cval = (next_val + m_colLabels[i]) / 2.0;

            if (min_Cval > max_Cval)
            {
                tmp = min_Cval;
                min_Cval = max_Cval;
                max_Cval = tmp;
            }

            if (m_currentMarkerRpm >= min_Cval && m_currentMarkerRpm <= max_Cval)
            {
                col = i;
                break;
            }

        }
    }

    if(m_currentMarkerLoad < m_minGridLoad){
        row = m_numRows - 1;

    }else if(m_currentMarkerLoad > m_maxGridLoad){
        row = 0;

    }else {

        for (i = 0; i < m_numRows; ++i)
        {
            if(!i)
                prev_val = m_rowLabels[i] - ( m_rowLabels[i+1] - m_rowLabels[i] );
            else
                prev_val = m_rowLabels[i-1];

            if(i == GetNumberRows() - 1)
                next_val = ( m_rowLabels[i] - m_rowLabels[i-1] ) + m_rowLabels[i];
            else
                next_val = m_rowLabels[i+1];

            min_Rval = (prev_val + m_rowLabels[i]) / 2.0;
            max_Rval = (next_val + m_rowLabels[i]) / 2.0;


            if( min_Rval > max_Rval )
            {
                tmp = min_Rval;
                min_Rval = max_Rval;
                max_Rval = tmp;
            }

            if( m_currentMarkerLoad >= min_Rval && m_currentMarkerLoad <= max_Rval)
            {
                row = i;
                break;
            }

        }
    }

  //  Logger::logf(LOG_MSG,_T("CalculateLoadCellPoints min_Cval=%.1f, max_Cval=%.1f, min_Rval=%.1f, max_Rval=%.1f"),min_Cval, max_Cval, min_Rval, max_Rval);


//    Logger::logf(LOG_MSG,_T("CalculateLoadCellPoints row[%i]=%i, col[%i]=%i"),row,m_rowLabels[row],col,m_colLabels[col]);

   // Logger::logf(LOG_MSG,_T("CalculateLoadCellPoints m_minGridRpm=%.1f, m_maxGridRpm=%.1f, m_minGridLoad=%.1f, m_maxGridLoad=%.1f"),m_minGridRpm, m_maxGridRpm, m_minGridLoad, m_maxGridLoad);

    m_currentLoadCoord = wxPoint(row, col);

    SetCurrentLoadMarkerColor( DrawHelper::InvertColor( CalculateCellBgColor(m_currentLoadCoord) ) );

    const wxRect cellRect = GetCellRectPix(row, col);

   // Logger::logf(LOG_MSG,_T("CalculateLoadCellPoints cellRect x=%i, y=%i, w=%i, h=%i"),cellRect.x,cellRect.y,cellRect.width,cellRect.height);

    //Logger::logf(LOG_MSG,_T("CalculateLoadCellPoints m_gridRect.width = %i, m_rowLabelWidth = %i"),m_gridRect.width,m_rowLabelWidth);


    /// calc x coord

    //Logger::logf(LOG_MSG,_T("m_currentMarkerRpm=%0.2f, m_maxGridRpm=%0.2f"),m_currentMarkerRpm, m_maxGridRpm);

    if( m_currentMarkerRpm < m_minGridRpm )
        m_currentLoadCoordPix.x = m_gridRect.GetLeft();

    else if( m_currentMarkerRpm > m_maxGridRpm )
        m_currentLoadCoordPix.x = m_gridRect.GetRight()-0;

    else {
        double colMidValue      =  m_colLabels[ col ];
        double cellHalfWidth    = cellRect.width / 2;

        if ( m_currentMarkerRpm <  colMidValue ) {
            m_currentLoadCoordPix.x = (int)(cellRect.x + ( ( (double)m_currentMarkerRpm  - min_Cval) / (colMidValue - min_Cval) * cellHalfWidth ));
        } else {
            m_currentLoadCoordPix.x = (int)(cellRect.x + cellHalfWidth +
                ( ( (double)m_currentMarkerRpm  - colMidValue) / (max_Cval - colMidValue ) * cellHalfWidth ));
        }
    }


    /// calc y coord

    if( m_currentMarkerLoad < m_minGridLoad )
        m_currentLoadCoordPix.y = m_gridRect.GetBottom()-0;

    else if( m_currentMarkerLoad > m_maxGridLoad )
        m_currentLoadCoordPix.y = m_gridRect.GetTop();

    else {
        double rowMidValue      =  m_rowLabels[ row ];
        double cellHalfHeight    = cellRect.height / 2;
        double midY = cellRect.y + cellHalfHeight;

        if ( m_currentMarkerLoad <  rowMidValue ) {
            m_currentLoadCoordPix.y = (int)(midY + ( ( rowMidValue - (double)m_currentMarkerLoad ) / ( rowMidValue - min_Rval ) * cellHalfHeight ));
            //m_currentLoadCoordPix.y = (int)(cellRect.y - ( ( (double)m_currentMarkerLoad - min_Rval) / (rowMidValue - min_Rval) * cellRect.height) + cellRect.height);
        } else {
            m_currentLoadCoordPix.y = (int)(midY - ( ( (double)m_currentMarkerLoad - rowMidValue ) / ( max_Rval - rowMidValue ) * cellHalfHeight ));
        }
    }



    //Logger::logf(LOG_MSG,_T("CalculateLoadCellPoints pix x=%i, y=%i"),m_currentLoadCoordPix.x,m_currentLoadCoordPix.y);

    /******************************************
            Calculate refresh cells

    */

    /* cell areas explain

    if the marker radius <= cell_height/2  !!!!

           limit_left  limit_right
                │           │
            ╔═══════════════════╗
            ║ 1 │     3     │ 5 ║
         ───║───┼───────────┼───║───limit_vert
            ║ 2 │     4     │ 6 ║
            ╚═══════════════════╝
                │           │

    else
           limit_left  limit_right
                │           │
            ╔═══════════════════╗
            ║ 1 │     3     │ 5 ║
         ───║───┼───────────┼───║───limit_top
            ║   │           │   ║
            ║   │           │   ║
            ║ 7 │           │ 8 ║
            ║   │           │   ║
            ║   │           │   ║
         ───║───┼───────────┼───║───limit_bottom
            ║ 2 │     4     │ 6 ║
            ╚═══════════════════╝
                │           │

    */

    short load_x = m_currentLoadCoordPix.x;             // load x coord by pixel
    short load_y = m_currentLoadCoordPix.y;             // load y coord by pixel
    /*
    short limit_left = cellRect.x + cellRect.width/4;       // cell left limit
    short limit_right = cellRect.x + cellRect.width * 0.75;  // cell right limit
    */
    short limit_left = cellRect.x + (int)(m_loadMarkerRadius);                   // cell left limit
    short limit_right = cellRect.x + cellRect.width - (int)(m_loadMarkerRadius); // cell right limit

    short limit_top = cellRect.y + (int)(m_loadMarkerRadius);                      // cell top limit
    short limit_bottom = cellRect.y + cellRect.height - (int)(m_loadMarkerRadius); // cell bottom limit


    if( load_x <= limit_left && load_y <= limit_top) {  //area 1

        AddNewRefreshCellforLoad(row-1, col  );
        AddNewRefreshCellforLoad(row-1, col-1);
        AddNewRefreshCellforLoad(row  , col-1);

        if( m_currentLoadCoordPix.x < m_gridWarningRect.x) {
            AddNewRefreshRowLabelIndex(row);
            AddNewRefreshRowLabelIndex(row-1);
        }

        if( m_currentLoadCoordPix.y < m_gridWarningRect.y){
            AddNewRefreshColLabelIndex(col);
            AddNewRefreshColLabelIndex(col-1);
        }

    }
    else if( load_x <= limit_left && load_y >= limit_bottom ) {  //area 2

        AddNewRefreshCellforLoad(row+1, col  );
        AddNewRefreshCellforLoad(row+1, col-1);
        AddNewRefreshCellforLoad(row  , col-1);

        if( m_currentLoadCoordPix.x < m_gridWarningRect.x) {
            AddNewRefreshRowLabelIndex(row);
            AddNewRefreshRowLabelIndex(row+1);
        }

    }
    else if( load_x >= limit_left && load_x <= limit_right && load_y <= limit_top) {  //area 3

        AddNewRefreshCellforLoad(row-1,col);

        if( m_currentLoadCoordPix.y < m_gridWarningRect.y)
            AddNewRefreshColLabelIndex(col);

    }
    else if( load_x >= limit_left && load_x <= limit_right && load_y >= limit_bottom) {  //area 4

        AddNewRefreshCellforLoad(row+1,col);

    }
    else if( load_x >= limit_right && load_y <= limit_top) {  //area 5


        AddNewRefreshCellforLoad(row-1, col  );
        AddNewRefreshCellforLoad(row-1, col+1);
        AddNewRefreshCellforLoad(row  , col+1);


        if( m_currentLoadCoordPix.y < m_gridWarningRect.y){
            AddNewRefreshColLabelIndex(col+1);
            AddNewRefreshColLabelIndex(col);
        }


    }
    else if( load_x >= limit_right && load_y >= limit_bottom ) {  //area 6

        AddNewRefreshCellforLoad(row+1, col  );
        AddNewRefreshCellforLoad(row+1, col+1);
        AddNewRefreshCellforLoad(row  , col+1);

    }
    else if( load_x <= limit_left && load_y >= limit_top && load_y <= limit_bottom ) {  //area 7

        AddNewRefreshCellforLoad(row,col-1);

        if( m_currentLoadCoordPix.x < m_gridWarningRect.x)
            AddNewRefreshRowLabelIndex(row);

    }
    else if( load_x >= limit_right && load_y >= limit_top && load_y <= limit_bottom ) {  //area 8

        AddNewRefreshCellforLoad(row,col+1);

    }


    AddNewRefreshCellforLoad(m_currentLoadCoord);

}

void VEMSTableEditor::DrawCurrentLoadMarker(wxDC& dc, const wxRect& clientRc)
{

    dc.SetPen(m_currentLoadMarkerColor);
    dc.SetBrush(m_currentLoadMarkerColor);


    dc.DrawCircle(m_currentLoadCoordPix, (int)(m_loadMarkerRadius) );

    //Logger::logf(LOG_MSG,_T("m_currentLoadCoordPix.x=%i, y=%i"),m_currentLoadCoordPix.x, m_currentLoadCoordPix.y);


        /// lol
/*
        double tmp1, tmp2;

        if( m_currentLoadCoordPix.x < m_gridWarningRect.GetLeft() ) {

            tmp1 = sqrt(pow(m_loadMarkerRadius,2) - pow( m_gridRect.x - m_currentLoadCoordPix.x, 2));
            tmp2 = -tmp1 + m_currentLoadCoordPix.y;
            tmp1 += m_currentLoadCoordPix.y;

            Logger::logf(LOG_MSG,_T("DrawCurrentLoadMarker tmp1=%.2f, tmp2=%.2f"), tmp1, tmp2);

            wxPoint up(m_gridRect.x, tmp1);
            wxPoint down(m_gridRect.x, tmp2);

            dc.DrawLine(up, down);
            dc.DrawArc(up, down,  m_currentLoadCoordPix);

        }
*/



	//if (m_currentLoadCoord.y >= 0 && m_currentLoadCoord.x >= 0)

}

void VEMSTableEditor::Draw3DBorder(wxDC& dc, const wxRect& rc)
{
	wxPen oldPen = dc.GetPen();

	dc.SetPen( wxPen(wxSystemSettings::GetColour(wxSYS_COLOUR_3DSHADOW), 1, wxSOLID));
	dc.DrawLine(rc.x, rc.y, rc.GetRight() + 1, rc.y);
	dc.DrawLine(rc.x, rc.y, rc.x, rc.GetBottom() + 1);
	dc.DrawLine(rc.GetRight(), rc.GetBottom(), rc.GetRight(), rc.y);
	dc.DrawLine(rc.GetRight(), rc.GetBottom(), rc.x, rc.GetBottom());

	dc.SetPen(*wxWHITE_PEN);
	dc.DrawLine(rc.x + 1, rc.y + 1, rc.GetRight(), rc.y + 1);
	dc.DrawLine(rc.x + 1, rc.y + 1, rc.x + 1, rc.GetBottom());

	dc.SetPen(oldPen);
}


void VEMSTableEditor::OnPopupMenuMultiply( wxCommandEvent& event )
{
	ShowMultiplyDialog();
}


void VEMSTableEditor::OnPopupMenuDivide( wxCommandEvent& event )
{
	ShowDivideDialog();
}


void VEMSTableEditor::OnPopupMenuIncCellLarge(wxCommandEvent& event)
{
	IncCellLarge();
}

void VEMSTableEditor::OnPopupMenuDecCellLarge(wxCommandEvent& event)
{
	DecCellLarge();
}

void VEMSTableEditor::OnPopupMenuIncCellSmall(wxCommandEvent& event)
{
	IncCellSmall();
}

void VEMSTableEditor::OnPopupMenuDecCellSmall(wxCommandEvent& event)
{
	DecCellSmall();
}

void VEMSTableEditor::OnPopupMenuLockCell(wxCommandEvent& event)
{
	LockUnlockCells();
}

void VEMSTableEditor::OnPopupMenuUnlockCell(wxCommandEvent& event)
{
	LockUnlockCells();
}

void VEMSTableEditor::OnPopupMenuUndoCellModification(wxCommandEvent& event)
{
	if (AreCellsHighlighted())
	{
		UndoHighlightedCellsModification();
		UnhighlightAllCells();
		Refresh();
	}
	else if ((m_cursorRow >= 0) && (m_cursorCol >= 0) && WasCellModified(m_cursorRow, m_cursorCol))
	{
		UndoCellModification(m_cursorRow, m_cursorCol);
		Refresh();
	}
}

void VEMSTableEditor::OnPopupMenuUndoAllCellsModification(wxCommandEvent& event)
{
	UnhighlightAllCells();
	UndoAllCellsModification();
	SetFullRefresh();
	Refresh();
}

void VEMSTableEditor::OnPopupMenuLockAllCells(wxCommandEvent& event)
{
	UnhighlightAllCells();
	LockAllCells();
	SetFullRefresh();
	Refresh();
}

void VEMSTableEditor::OnPopupMenuUnlockAllCells(wxCommandEvent& event)
{
	UnhighlightAllCells();
	UnlockAllCells();
	SetFullRefresh();
	Refresh();
}

void VEMSTableEditor::OnPopupMenuAcceptAllChanges(wxCommandEvent& event)
{
	AcceptAllModifiedCells();
	UnhighlightAllCells();
	SetFullRefresh();
	Refresh();
}


void VEMSTableEditor::OnPopupMenuEditHeader( wxCommandEvent& event )
{
    int id = event.GetId();

    if ( m_selectedColLabel != -1 )
    {
        ShowColHeaderDialog( id );
    }
    else
    {
        ShowRowHeaderDialog( id );
    }

    m_selectedRowLabel = -1;
    m_selectedColLabel = -1;
}

void VEMSTableEditor::OnPopupMenuChangeTable( wxCommandEvent& event )
{
    TableEditorEvent e( TableEditorEvent::TABLE_NAME_CHANGED );

    e.name = m_menuIDtoTableName[ event.GetId() ];

    if ( GetParent() ) { GetParent()->AddPendingEvent( e ); }
}


void VEMSTableEditor::OnPopupMenuImport( wxCommandEvent& event )
{
    Send( TableEditorEvent::IMPORT );
}


void VEMSTableEditor::OnPopupMenuRemap( wxCommandEvent& event )
{
    Send( TableEditorEvent::REMAP );
}


void VEMSTableEditor::MultiplySelection( double d )
{
	for (int i = 0; i < GetNumberRows(); ++i)
	{
		for (int j = 0; j < GetNumberCols(); ++j)
		{
			if ( IsCellHighlighted(i, j) && !IsCellLocked(i, j) )
			{
				MultiplyCell( i, j, d );
			}
		}
	}
}

void VEMSTableEditor::MultiplyCell( int row, int col, double d )
{
    double oldValue = GetCellValue( row, col );
    double newValue = oldValue * d;

    if ( newValue >= m_minCellValue && newValue <= m_maxCellValue )
    {
        SetCellValue( row, col, newValue );
    }

    // Logger::logf( LOG_MSG, _T("VEMSTableEditor::MultiplyCell() - r: %d, c:%d, v:%f, n:%f"), row, col, d, newValue );
}

 void VEMSTableEditor::ShowMultiplyDialog()
 {
    if (m_cursorRow >= 0 && m_cursorRow < GetNumberRows() && m_cursorCol >= 0 && m_cursorCol < GetNumberCols())
	{
		if (!IsCellLocked(m_cursorRow, m_cursorCol))
		{
			//const float cur_val = GetCellValue(m_cursorRow, m_cursorCol);

			wxRect cellRect = GetCellRectPix(m_cursorRow, m_cursorCol);
			cellRect.Offset(ClientToScreen(wxPoint(0, 0)));

			wxString val_mask( _T("%.2f") );

			VEMSCellEditorDialog dlg(wxGetActiveWindow(), 1.0, -1000, 1000, cellRect, m_zoomboxFont, m_zoomboxTextColor, val_mask, _T("") );
			dlg.SetTitle( _T("Multiply") );
			dlg.ShowModal();

			if (dlg.GetReturnCode() == wxID_OK)
			{
                if ( AreCellsHighlighted() )
                {
                    MultiplySelection( dlg.GetResultValue() );
                    Send( TableEditorEvent::CELLS_CHANGED );
                }
                else
                {
                    MultiplyCell( m_cursorRow, m_cursorCol, dlg.GetResultValue() );
                    Send( TableEditorEvent::CELL_CHANGED, m_cursorRow, m_cursorCol, dlg.GetResultValue() );
                }
			}

		}

        ResetHighlightOrigin();
        UnhighlightAllCells();

		Refresh();
	}
 }

void VEMSTableEditor::DivideSelection( double d )
{
	for (int i = 0; i < GetNumberRows(); ++i)
	{
		for (int j = 0; j < GetNumberCols(); ++j)
		{
			if ( IsCellHighlighted(i, j) && !IsCellLocked(i, j) )
			{
				DivideCell( i, j, d );
			}
		}
	}
}

void VEMSTableEditor::DivideCell( int row, int col, double d )
{
    if ( d == 0.0 ) { return; }

    double oldValue = GetCellValue( row, col );
    double newValue = oldValue / d;

    if ( newValue >= m_minCellValue && newValue <= m_maxCellValue )
    {
        SetCellValue( row, col, newValue );
    }
}

void VEMSTableEditor::ShowDivideDialog()
{
    if (m_cursorRow >= 0 && m_cursorRow < GetNumberRows() && m_cursorCol >= 0 && m_cursorCol < GetNumberCols())
	{
		if (!IsCellLocked(m_cursorRow, m_cursorCol))
		{
			//const float cur_val = GetCellValue(m_cursorRow, m_cursorCol);

			wxRect cellRect = GetCellRectPix(m_cursorRow, m_cursorCol);
			cellRect.Offset(ClientToScreen(wxPoint(0, 0)));

			wxString val_mask( _T("%.2f") );

			VEMSCellEditorDialog dlg(wxGetActiveWindow(), 1.0, -1000, 1000, cellRect, m_zoomboxFont, m_zoomboxTextColor, val_mask, _T("") );
			dlg.SetTitle( _T("Divide") );
			dlg.ShowModal();

			if (dlg.GetReturnCode() == wxID_OK)
			{
                if ( AreCellsHighlighted() )
                {
                    DivideSelection( dlg.GetResultValue() );
                    Send( TableEditorEvent::CELLS_CHANGED );
                }
                else
                {
                    DivideCell( m_cursorRow, m_cursorCol, dlg.GetResultValue() );
                    Send( TableEditorEvent::CELL_CHANGED, m_cursorRow, m_cursorCol, dlg.GetResultValue() );
                }
			}

		}

        ResetHighlightOrigin();
        UnhighlightAllCells();

		Refresh();
	}
}


void VEMSTableEditor::ShowRowHeaderDialog( int type )
{
    wxRect cellRect = GetLabelRowRect( m_selectedRowLabel );
    int cur_val     = GetRowLabelValue( m_selectedRowLabel );
    double  min     = m_minRowHeaderValue;
    double  max     = m_maxRowHeaderValue;

    if ( type == VEMS_TABLE_EDITOR_PM_MOVE_HEADER )
    {
        if ( m_selectedRowLabel > 0 )
        {
            max = GetRowLabelValue( m_selectedRowLabel - 1 );
        }

        if ( m_selectedRowLabel < ( m_numRows -  1 ) )
        {
            min = GetRowLabelValue( m_selectedRowLabel + 1 );
        }
    }
    else if ( type == VEMS_TABLE_EDITOR_PM_INSERT_HEADER )
    {
        min = GetRowLabelValue( m_selectedRowLabel );

        if ( m_selectedRowLabel > 0 )
        {
            max = GetRowLabelValue( m_selectedRowLabel - 1 );
        }
    }

    wxString val_mask( _T("%.0f") );

    cellRect.Offset( ClientToScreen( wxPoint(0, 0) ) );
    VEMSCellEditorDialog dlg(wxGetActiveWindow(), cur_val, (int)min, (int)max, cellRect, m_zoomboxFont, m_zoomboxTextColor, val_mask, _T("") );
    dlg.ShowModal();

    if ( dlg.GetReturnCode() != wxID_OK ) { return; }

    if ( !dlg.IsValueAccepted() )
    {
        wxMessageBox( _T("Value is incorrect or out of range!"), _T("Error"), wxOK | wxICON_ERROR );
        return;
    }

    double d = dlg.GetResultValue();

    switch ( type )
    {
        case VEMS_TABLE_EDITOR_PM_EDIT_HEADER:
            SetRowLabelValue( m_selectedRowLabel, d );
            Send( TableEditorEvent::ROW_HEADER_CHANGED, m_selectedRowLabel, -1, d );
            break;
        case VEMS_TABLE_EDITOR_PM_MOVE_HEADER:
            SetRowLabelValue( m_selectedRowLabel, d );
            VemsMatrixHelper::interpolate_row( &m_cellValues, m_rowLabels, m_selectedRowLabel );
            Send( TableEditorEvent::ROW_HEADER_MOVED, m_selectedRowLabel, -1, d );
            break;
        case VEMS_TABLE_EDITOR_PM_INSERT_HEADER:
            //if ( !m_cellValues.insert_row_before( (size_t)m_selectedRowLabel ) ) { return; }
            //if ( !m_rowLabels.insert_col_before( (size_t)m_selectedRowLabel ) ) { return; }

            //SetRowLabelValue( m_selectedRowLabel - 1, d );
            //VemsMatrixHelper::interpolate_row( &m_cellValues, m_rowLabels, m_selectedRowLabel - 1 );

            //m_cellValues.resize( m_numRows, m_numCols );
            //m_rowLabels.resize( 1, m_numRows );

            if ( !m_cellValues.insert_row( (size_t)m_selectedRowLabel - 1 ) ) { return; }
            if ( !m_rowLabels.insert_col( (size_t)m_selectedRowLabel - 1 ) ) { return; }

            SetRowLabelValue( m_selectedRowLabel, d );

            VemsMatrixHelper::interpolate_row( &m_cellValues, m_rowLabels, m_selectedRowLabel );

            m_cellValues.shift_up();
            m_cellValues.resize( m_numRows, m_numCols );

            m_rowLabels.shift_left();
            m_rowLabels.resize( 1, m_numRows );

            Send( TableEditorEvent::ROW_HEADER_INSERTED );

            break;
    }

    SetFullRefresh();
    Refresh();
}


void VEMSTableEditor::ShowColHeaderDialog( int type )
{
    wxRect cellRect = GetLabelColRect( m_selectedColLabel );
    int cur_val     = GetColLabelValue( m_selectedColLabel );
    double  min     = m_minColHeaderValue;
    double  max     = m_maxColHeaderValue;

    if ( type == VEMS_TABLE_EDITOR_PM_MOVE_HEADER )
    {
        if ( m_selectedColLabel > 0 )
        {
            min = GetColLabelValue( m_selectedColLabel - 1 );
        }

        if ( m_selectedColLabel < ( m_numCols -  1 ) )
        {
            max = GetColLabelValue( m_selectedColLabel + 1 );
        }
    }
    else if ( type == VEMS_TABLE_EDITOR_PM_INSERT_HEADER )
    {
        min = GetColLabelValue( m_selectedColLabel );

        if ( m_selectedColLabel < ( m_numCols -  1 ) )
        {
            max = GetColLabelValue( m_selectedColLabel + 1 );
        }
    }

    wxString val_mask( _T("%.0f") );

    cellRect.Offset( ClientToScreen( wxPoint(0, 0) ) );
    VEMSCellEditorDialog dlg(wxGetActiveWindow(), cur_val, (int)min, (int)max, cellRect, m_zoomboxFont, m_zoomboxTextColor, val_mask, _T("") );
    dlg.ShowModal();

    if ( dlg.GetReturnCode() != wxID_OK ) { return; }

    if ( !dlg.IsValueAccepted() )
    {
        wxMessageBox( _T("Value is incorrect or out of range!"), _T("Error"), wxOK | wxICON_ERROR );
        return;
    }

    const int d = (int)dlg.GetResultValue();

    switch ( type )
    {
        case VEMS_TABLE_EDITOR_PM_EDIT_HEADER:
            SetColLabelValue( m_selectedColLabel, d );
            Send( TableEditorEvent::COL_HEADER_CHANGED, -1, m_selectedColLabel, d );
            break;
        case VEMS_TABLE_EDITOR_PM_MOVE_HEADER:
            SetColLabelValue( m_selectedColLabel, d );
            VemsMatrixHelper::interpolate_col( &m_cellValues, m_colLabels, m_selectedColLabel );
            Send( TableEditorEvent::COL_HEADER_MOVED, -1, m_selectedColLabel, d );
            break;
        case VEMS_TABLE_EDITOR_PM_INSERT_HEADER:
            Logger::logf( LOG_MSG, _T("VEMSTableEditor::ShowColHeaderDialog() - VEMS_TABLE_EDITOR_PM_INSERT_HEADER") );
            if ( !m_cellValues.insert_col( (size_t)m_selectedColLabel ) ) { return; }
            if ( !m_colLabels.insert_col( (size_t)m_selectedColLabel ) ) { return; }

            SetColLabelValue( (size_t)m_selectedColLabel + 1, d );

            Logger::logf( LOG_MSG, _T("m_selectedColLabel: %d"), m_selectedColLabel );
            Logger::logf( LOG_MSG, _T("m_cellValues.cols(): %d"), m_cellValues.cols() );

            VemsMatrixHelper::interpolate_col( &m_cellValues, m_colLabels, m_selectedColLabel + 1 );

            m_cellValues.resize( m_numRows, m_numCols );
            m_colLabels.resize( 1, m_numCols );

            Send( TableEditorEvent::COL_HEADER_INSERTED, -1, m_selectedColLabel, d );

            break;
    }

    SetFullRefresh();
    Refresh();
}


void VEMSTableEditor::Send( int type, int row, int col, double v )
{
    TableEditorEvent e( type );

    e.row = row;
    e.col = col;
    e.value = v;

    if ( GetParent() ) { GetParent()->AddPendingEvent( e ); }
}

void VEMSTableEditor::SelectLoad()
{
    int row = -1;
    int col = -1;

    if ( !HitTestCell( m_currentLoadCoordPix, &row, &col ) ) {
        return;
    }

    if ( ( m_cursorRow == row ) && ( m_cursorCol == col ) ) {
        return;
    }

        AddNewRefreshCell( m_cursorRow, m_cursorCol );
        m_cursorRow = row;
        m_cursorCol = col;
        UnhighlightAllCells();
        Refresh();
    }

void VEMSTableEditor::loadFromVemsMatrix( const VemsMatrix<double>& m )
{
    for ( size_t i = 0; i < m.rows(); ++i )
    {
        for ( size_t j = 0; j < m.cols(); ++j )
        {
            SetCellValue( i, j, (float)m.get( i, j ) );
        }
    }
}

void VEMSTableEditor::loadRowHeaderFromVemsMatrix( const VemsMatrix<double>& m )
{
    for ( size_t i = 0; i < m.cols(); ++i )
    {
//        Logger::logf(   LOG_MSG, _T("VEMSTableEditor::loadRowHeaderFromVemsMatrix() - idx: %d, v: %f"),
//                        i, m.get( i ) );
        SetRowLabelValue( i, m.get( i ) );
    }

    Refresh();
    SetFullRefresh();
}

void VEMSTableEditor::loadColHeaderFromVemsMatrix( const VemsMatrix<double>& m )
{
    for ( size_t i = 0; i < m.cols(); ++i )
    {
        SetColLabelValue( i, m.get( i ) );
    }
    Refresh();
    SetFullRefresh();
}
//
// VEMSCellEditorDialog event table
//
//
BEGIN_EVENT_TABLE(VEMSCellEditorDialog, wxDialog)
    EVT_BUTTON(wxID_OK, VEMSCellEditorDialog::OnButton)
END_EVENT_TABLE()

//
// VEMSCellEditorDialog CTOR
//
VEMSCellEditorDialog::VEMSCellEditorDialog(wxWindow *parent, float cur_val, float min_value, float max_value, const wxRect& cellRect, const wxFont& zoomboxFont, const wxColor& zoomboxTextColor, const wxString& formatMask, const wxString& initialValue)
             : wxDialog(parent, wxID_ANY, wxString(_T("")), cellRect.GetLeftTop(), cellRect.GetSize())
{
	m_resultVal = 0;
	m_isValAccepted = false;
	m_minVal = min_value;
	m_maxVal = max_value;

	wxString hintStr, valStr;

	wxString hintFormatStr;
	//hintFormatStr = _T("Enter value between ") + formatMask + _T(" and ") + formatMask;
    hintFormatStr = formatMask + _T(" and ") + formatMask;

	hintStr.sprintf(hintFormatStr, min_value, max_value);

	SetTitle(hintStr);

	if (initialValue.Length() > 0)
	{
		valStr = initialValue;
	}
	else
	{
		valStr.sprintf(formatMask, float(cur_val));
	}

	m_valueText = new wxTextCtrl(this, wxID_ANY, valStr,
                            wxPoint(55, 10), wxSize(140, wxDefaultCoord), wxTE_CENTRE);

	wxTextValidator validator(wxFILTER_NUMERIC);
	m_valueText->SetValidator(validator);
	m_valueText->SetMaxLength(6);

	m_okButton = new wxButton(this, wxID_OK, _T("&OK"), wxPoint(20, 40), wxSize(60, wxDefaultCoord));
	m_cancelButton = new wxButton(this, wxID_CANCEL, _T("&Cancel"), wxPoint(1024, 1024), wxSize(1, 1));

	m_valueText->SetFont(zoomboxFont);
	m_valueText->SetForegroundColour(zoomboxTextColor);

	m_valueText->SetFocus();

	if (initialValue.Length() > 0)
	{
		m_valueText->SetSelection(-1, 0);
	}

	m_okButton->SetDefault();

	// Create vertical sizer
	wxBoxSizer *sizerTop = new wxBoxSizer(wxHORIZONTAL);

	sizerTop->Add(m_valueText, 0, wxALIGN_CENTER | wxALL, 10);
	sizerTop->Add(m_okButton, 0, wxALIGN_CENTER | wxALL, 10);

	SetSizer(sizerTop);

	sizerTop->SetSizeHints(this);
	sizerTop->Fit(this);

	wxRect parentRect = GetParent()->GetRect();

	const wxSize newSize = GetSize();
	int newX = cellRect.x + (cellRect.width / 2) - newSize.GetWidth() / 2;
	int newY = cellRect.GetBottom() + 1;

	if (newX < parentRect.x)
	{
		newX = parentRect.x;
	}
	else if (newX + newSize.GetWidth() > parentRect.GetRight())
	{
		newX = parentRect.GetRight() - newSize.GetWidth();
	}

	if (newY < parentRect.y)
	{
		newY = parentRect.y;
	}
	else if (newY + newSize.GetHeight() > parentRect.GetBottom())
	{
		newY = parentRect.GetBottom() - newSize.GetHeight();
	}

	SetSize(wxRect(newX, newY, newSize.x, newSize.y));
}

//
// VEMSCellEditorDialog DTOR
//

VEMSCellEditorDialog::~VEMSCellEditorDialog()
{
}

void VEMSCellEditorDialog::OnButton(wxCommandEvent& event)
{
	if (event.GetEventObject() == m_okButton)
	{
		wxString valStr = m_valueText->GetValue();
		double val = 0;
		if (valStr.ToDouble(&val) && float(val) >= m_minVal && float(val) <= m_maxVal)
		{
			m_resultVal = float(val);
			m_isValAccepted = true;
		}
	}

	event.Skip();
}

//
// VEMSCellEditorDialog class methods
//

bool VEMSCellEditorDialog::IsValueAccepted() const
{
	return m_isValAccepted;
}

float VEMSCellEditorDialog::GetResultValue() const
{
	wxASSERT(m_isValAccepted);
	return m_resultVal;
}


void VEMSTableEditor::SendCursorCellChangedEvent()
{
    TableEditorEvent e( TableEditorEvent::CELL_CHANGED );

    e.row = m_cursorRow;
    e.col = m_cursorCol;
    e.value = GetCellValue( m_cursorRow, m_cursorCol );
    e.readAfterWrite = false;

    if ( GetParent() )
    {
        GetParent()->AddPendingEvent( e );
    }
}


void VEMSTableEditor::moveCursorToPosition( const wxPoint& position )
{
    Logger::logf( LOG_MSG, _T("VEMSTableEditor::moveCursorToPosition") );
    if  ( wxWindow::FindFocus() == this ) return;

    int row = position.y;
    int col = position.x;

    if ( row < 0 || row > GetNumberRows() - 1 || col < 0 || col > GetNumberCols() - 1 ) return;
    if ( row == m_cursorRow && col == m_cursorCol ) return;

    Freeze();

    m_cursorRow = row;
    m_cursorCol = col;

    UnhighlightAllCells();
    ResetHighlightOrigin();
    AddNewRefreshCell( m_cursorRow, m_cursorCol );

    Thaw();

    Refresh();
}

VemsMatrix<double>*  VEMSTableEditor::GetRowLabels()
{
    size_t cols = m_rowLabels.cols();

    VemsMatrix<double>* rowLabels = new VemsMatrix<double>( 1, cols );

    for ( size_t col = 0; col < cols; ++col )
    {
        rowLabels->set( 0, col, m_rowLabels.get( 0, col ) );
    }

    return rowLabels;
}

VemsMatrix<double>*  VEMSTableEditor::GetColLabels()
{
    size_t cols = m_colLabels.cols();

    VemsMatrix<double>* colLabels = new VemsMatrix<double>( 1, cols );

    for ( size_t col = 0; col < cols; ++col )
    {
        colLabels->set( 0, col, m_colLabels.get( 0, col ) );
    }

    return colLabels;
}

void VEMSTableEditor::AutoFollowLoad()
{
    if ( m_isAutoFollowLoad ) {
        SelectLoad();
    }
}

void VEMSTableEditor::drawHelpPanel( wxDC& dc )
{
    wxSize size = GetClientSize();

    if ( !m_helpPanelBmp || m_isResized ) { createHelpPanel( size.GetHeight() ); }

    int helpBmpWidth    = m_helpPanelBmp->GetWidth();
    int helpBmpHeight   = m_helpPanelBmp->GetHeight();

    int helpBmtLeft = ( size.GetWidth() - helpBmpWidth ) / 2;
    int helpBmtTop = ( size.GetHeight() - helpBmpHeight ) / 2;

    dc.DrawBitmap( *m_helpPanelBmp, helpBmtLeft, helpBmtTop );
}

double VEMSTableEditor::getHelpTextHeight( int clientHeight, size_t lineCount, double margin, double rowScale )
{
    const double MAX_TEXT_HEIGHT = 9.5;
    const double MIN_TEXT_HEIGHT = 5.0;

    double textAreaHeight = (double)clientHeight - ( margin * 2 ) - 2;

    double rowHeight = textAreaHeight / (double)lineCount;

    double textHeight = rowHeight / rowScale;

    if ( textHeight > MAX_TEXT_HEIGHT ) { textHeight = MAX_TEXT_HEIGHT; }
    else if ( textHeight < MIN_TEXT_HEIGHT ) { textHeight = MIN_TEXT_HEIGHT ;}

    return textHeight;
}

void VEMSTableEditor::createHelpPanel( int clientHeight )
{
    HotKeyDescArray arr;

    double m = 10.0;
    double rowScale = 1.6;

    m_hotkeyMgr->GetHotKeyList(arr);

    double maxTitleWidth = 0.0, maxHKWidth = 0.0;

    size_t arrc = arr.GetCount();

    double th = getHelpTextHeight( clientHeight, arrc, m, rowScale );

    for( size_t a=0; a<arrc; a++ ) {
        double tw = wxAgg::getTextWidth( arr[a].Title, th );
        double hw = wxAgg::getTextWidth( arr[a].HotKey, th );

        if( maxTitleWidth < tw ) maxTitleWidth = tw;
        if( maxHKWidth < hw ) maxHKWidth = hw;
    }

    double rowheight = th * rowScale;

    double pw = 2*m + maxTitleWidth + 2*m + maxHKWidth;
    double ph = arrc * rowheight + 2*m + 2;

    wxAgg aggCanvas( (int)pw, (int)ph );

    aggCanvas.clear(wxColor(0,0,0,0));

    aggCanvas.setPen( wxColor(0,0,0,255) );

    aggCanvas.setBrush(wxColor( 255,255,255,200 ));

    double bw = 2;

    aggCanvas.drawFilledRoundedRectangle(bw/2, bw/2, pw-2*bw, ph-2*bw, m, bw);

    aggCanvas.setPen( wxColor(0,0,0,255) );

    for( size_t a=0; a<arrc; a++ ) {

        double ty = (a+1) * rowheight + m;

        aggCanvas.drawText( arr[a].Title, m, ty, -1, th, 1.0);
        aggCanvas.drawText( arr[a].HotKey, 2*m + maxTitleWidth, ty, -1, th, 1.0);

    }

    if(m_helpPanelBmp)
    {
        ptrdel(m_helpPanelBmp);
    }

    m_helpPanelBmp = aggCanvas.renderToBitmap();
}
