#include "VemsGaugeAs.h"
#include <math.h>
//#include <iostream>

#include <wx/dcclient.h>
#include <wx/brush.h>

#include <wx/image.h>

#include "../logger.h"
#include "gaugeframe.h"

#include "../MTConfigHelper.h"

#include "../DrawHelper.h"

#define PDA_ARC_OVERLAP 1.3


IMPLEMENT_DYNAMIC_CLASS(VemsGaugeAs,VemsWidgetBase);

BEGIN_EVENT_TABLE(VemsGaugeAs, VemsWidgetBase)
     EVT_PAINT(VemsGaugeAs::OnPaint)
	 EVT_SIZE(VemsGaugeAs::OnSize)

END_EVENT_TABLE()

VemsGaugeAs::VemsGaugeAs(){};

VemsGaugeAs::VemsGaugeAs(wxWindow* parent, wxWindowID id, wxPoint CustomPosition, wxSize CustomSize, bool DefaultPosSize,
    long style, const VEMSGaugeDescArray * gaugeDescA)
: VemsWidgetBase(parent, id,CustomPosition,CustomSize,style, _T("VemsGaugeAs"), gaugeDescA)
, VemsGaugeAsBgBitmap (NULL)

{
	 Logger::logf(LOG_MSG,_T("VemsGaugeAs CTOR START"));

    GBuffer = new TMGraphics();
    GCover = new TMGraphics();

    //Logger::logf(LOG_MSG,_T("VemsGaugeAs::VemsGaugeAs isvalid = %i"),GCover->IsValid());

    GBuffer->Reset(GetSize().x,GetSize().y);
    GCover->Reset(GetSize().x,GetSize().y);

    //Create(parent, id,CustomPosition,CustomSize,style);

    ////Logger::logf(LOG_MSG,wxString::Format(_T("VemsGaugeAs=%p"),this));

    m_width = CustomSize.x;

    m_parent = parent;

    (m_parent)->GetSize(&width,&height);

    setSize(width,height);


    SetFaceColor(new wxColor(152,12,12));
    SetPointerColor(new wxColor(253,204,0));
	SetMarkingsColor(new wxColor(220,230,230));
	SetCenterColor(new wxColor(231,30,31));

    DefPosSize = DefaultPosSize;
    gsize = CustomSize;

    // SetValue(0);
    m_hasShape = false;

    m_parent->GetClientSize(&width,&height);

    if(width < height)
      W = width;
    else
      W = height;

    CP.x = CP.y = /*floor*/ (W / 2);


    RG = CP.x;

    RG -= 2;




    CreateBoundMask();

    SetBackgroundStyle(wxBG_STYLE_CUSTOM);  // MJ ~ Very important, avoid the flickering!!!



	SetValue(0);

	VemsGaugeAsBgBitmap       = new wxBitmap(DisplayHelper::GetDisplaySize().x, DisplayHelper::GetDisplaySize().y);

	VemsGaugeAsBmpBezel       = new wxBitmap(DisplayHelper::GetDisplaySize().x, DisplayHelper::GetDisplaySize().y);/*
	VemsGaugeAsBmpDialFace    = new wxBitmap(DisplayHelper::GetDisplaySize().x, DisplayHelper::GetDisplaySize().y);
	VemsGaugeAsBmpColArea     = new wxBitmap(DisplayHelper::GetDisplaySize().x, DisplayHelper::GetDisplaySize().y);
	VemsGaugeAsBmpTickMarks   = new wxBitmap(DisplayHelper::GetDisplaySize().x, DisplayHelper::GetDisplaySize().y);
	VemsGaugeAsBmpGaugeNums   = new wxBitmap(DisplayHelper::GetDisplaySize().x, DisplayHelper::GetDisplaySize().y);
	VemsGaugeAsBmpInnerCirc   = new wxBitmap(DisplayHelper::GetDisplaySize().x, DisplayHelper::GetDisplaySize().y);
*/


#if defined (__WINCE_STANDARDSDK__)
    SetBackgroundColour(*wxWHITE);
    SetWindowStyle(wxNO_BORDER);
#endif


    m_isExtendedResize = true;

	Logger::logf(LOG_MSG,_T("VemsGaugeAs::VemsGaugeAs() END"));



}

void VemsGaugeAs::SetGaugeDesc(VEMSGaugeDesc* gaugeDesc)                            ///to VemsWidgetBase
{
    m_gd = gaugeDesc;

    SetDefaultGaugeDesc(0);
}


VemsGaugeAs::~VemsGaugeAs(void)
{
    // TODO: one of them is null!!!
    Logger::logf(LOG_MSG,_T("VemsGaugeAs DTOR START"));

	delete faceBrush;
	delete faceColor;
	delete pointerBrush;
	delete pointerPen;
	delete markingsPen;
	delete markingsColor;
	delete centerPen;
	delete centerBrush;
	//delete wood;
	//delete image;

    delete GBuffer;
    delete GCover;

    delete VemsGaugeAsBgBitmap;
	delete VemsGaugeAsBmpBezel;

    //UnRegisterViewer();

    Logger::logf(LOG_MSG,_T("VemsGaugeAs DTOR END"));
}

void VemsGaugeAs::SetValue(double value)
{
    if( value == this->m_value) return;

    if(m_gd == NULL)
      {
        fprintf( stdout, "m_gd is NULL! \n");
        m_value = 0;
        return;
      }


    this->m_value = value;

    if(value > m_gd->GetMaxValue() ){
        m_value = m_gd->GetMaxValue();
    }

    if(m_value < m_gd->GetMinValue() ){
        m_value = m_gd->GetMinValue();
    }




    UpdatePointer();
    UpdateGaugeValue();


    Refresh(false);

}

void VemsGaugeAs::UpdatePointer() {

    //Logger::logf(LOG_MSG,_T("VemsGauge::UpdatePointer() START"));
    wxRect rect1 = GetPointerRect();

    double maxV = m_gd->GetMaxValue();
    double minV = m_gd->GetMinValue();
    double anglePerRange = (double)m_gd->GetGaugeAngle() / (maxV-minV);

    double sweepAngle =  anglePerRange * m_value;
    double offsetAngle = anglePerRange * minV;
    pointerAngle = (sweepAngle + m_gd->GetStartAngle() - offsetAngle) * RADIAN_FACTOR;



    pointerPoints[0].x = (int)(cos(pointerAngle - (10*RADIAN_FACTOR))*pointerInnerRadius);
    pointerPoints[0].y = (int)(sin(pointerAngle - (10*RADIAN_FACTOR))*pointerInnerRadius);
    pointerPoints[1].x = (int)(cos(pointerAngle)*pointerOuterRadius);
    pointerPoints[1].y = (int)(sin(pointerAngle)*pointerOuterRadius);
    pointerPoints[2].x = (int)(cos(pointerAngle + (10*RADIAN_FACTOR))*pointerInnerRadius);
    pointerPoints[2].y = (int)(sin(pointerAngle + (10*RADIAN_FACTOR))*pointerInnerRadius);


    PPoint[0].x = pointerPoints[0].x;
    PPoint[0].y = pointerPoints[0].y;
    PPoint[1].x = pointerPoints[1].x;
    PPoint[1].y = pointerPoints[1].y;
    PPoint[2].x = pointerPoints[2].x;
    PPoint[2].y = pointerPoints[2].y;


/*
    PPoint[0].x = (cos(pointerAngle - (10*RADIAN_FACTOR))*pointerInnerRadius);
    PPoint[0].y = (sin(pointerAngle - (10*RADIAN_FACTOR))*pointerInnerRadius);
    PPoint[1].x = (cos(pointerAngle)*pointerOuterRadius);
    PPoint[1].y = (sin(pointerAngle)*pointerOuterRadius);
    PPoint[2].x = (cos(pointerAngle + (10*RADIAN_FACTOR))*pointerInnerRadius);
    PPoint[2].y = (sin(pointerAngle + (10*RADIAN_FACTOR))*pointerInnerRadius);
*/



	 /// wxRect rect2 = GetPointerRect();

	  ///RefreshRect(rect1.Union(rect2).Inflate(1,1),false);



	  //Logger::logf(LOG_MSG,_T("VemsGauge::UpdatePointer() END"));
}

wxRect VemsGaugeAs::GetPointerRect()
{

	  wxCoord x1=pointerPoints[0].x;
	  wxCoord y1=pointerPoints[0].y;
	  wxCoord x2=pointerPoints[0].x;
	  wxCoord y2=pointerPoints[0].y;

	  for (int i=1; i<3; i++)
      {
		    if (pointerPoints[i].x < x1)
          {
			      x1 = pointerPoints[i].x;
		      }
		    if (pointerPoints[i].y < y1)
          {
			      y1 = pointerPoints[i].y;
		      }
		    if (pointerPoints[i].x > x2)
          {
			      x2 = pointerPoints[i].x;
		      }
		    if (pointerPoints[i].y > y2)
          {
			      y2 = pointerPoints[i].y;
		      }
	    }

	  wxRect rect1(x1+centerPoint.x,y1+centerPoint.y,(x2-x1),(y2-y1));

	  return rect1;

}




void VemsGaugeAs::UpdateGaugeValue() {

    //Logger::logf(LOG_MSG,_T("VemsGaugeAs::UpdateGaugeValue() START"));
	wxClientDC dc(this);

    DigitalFont = wxFont((int)(gaugeRadius*0.2),wxSWISS,wxFONTSTYLE_NORMAL,0,false,_T("7 Segment"),wxFONTENCODING_DEFAULT);

    dc.SetFont(DigitalFont);
	wxCoord textw, texth;

	// get the area under the old value
	dc.GetTextExtent(gaugeValue,&textw,&texth);
	wxRect rect1(gaugeValuePoint.x,gaugeValuePoint.y,textw,texth);

	// update value

    //Logger::logf(LOG_MSG,_T("VemsGaugeAs::UpdateGaugeValue() format: ") + m_gd->GetFormat() +  wxString::Format(_T(" Value %f"), m_value) );

    wxString symbol = GetSelectedSymbol();
    if(  (symbol != wxEmptyString) && isExist(symbol)  ){
        //Logger::logf(LOG_MSG, _T("VemsGaugeAs::UpdateGaugeValue(): %") + m_gd->GetGaugeNumberFormat());
        gaugeValue.Printf(m_gd->GetGaugeNumberFormat(),m_value);
    }else{
        gaugeValue = _T("---");
    }

	// get the area under the new value
	//dc.GetTextExtent(gaugeValue,&textw,&texth);

	//gaugeValuePoint.x = centerPoint.x-(textw/2);
	//gaugeValuePoint.y = centerPoint.y + valueRadius;//-(texth);

	//wxRect rect2(gaugeValuePoint.x,gaugeValuePoint.y,textw,texth);

	// redraw the area under the combined old and new values
	///RefreshRect(rect1.Union(rect2),false);

	 //Logger::logf(LOG_MSG,_T("VemsGaugeAs::UpdateGaugeValue() END"));
}

/* Paint the bezel */
void VemsGaugeAs::paintBezel(wxDC& dc)
{
    //wxBufferedPaintDC dc(this);
	double color = 255;
	wxCoord rx1,ry1,rx2,ry2;
	int d = 2;
	int count;

	for (count=0; count<360; count = count + d) {
		wxBrush brush(*wxWHITE);
		wxPen pen(*wxWHITE);
		brush.SetColour(abs((int)color),abs((int)color),abs((int)color));
		pen.SetColour(abs((int)color),abs((int)color),abs((int)color));
		//dc.SetBrush(*wxTRANSPARENT_BRUSH);
		dc.SetBrush(brush);
		//dc.SetPen(*wxTRANSPARENT_PEN);
		dc.SetPen(pen);
		rx1 = (wxCoord) (centerPoint.x + (cos(count*RADIAN_FACTOR)*(gaugeRadius+15)));
		ry1 = (wxCoord) (centerPoint.y + (sin(count*RADIAN_FACTOR)*(gaugeRadius+15)));
		rx2 = (wxCoord) (centerPoint.x + (cos((count-d-PDA_ARC_OVERLAP)*RADIAN_FACTOR)*(gaugeRadius+15)));
		ry2 = (wxCoord) (centerPoint.y + (sin((count-d-PDA_ARC_OVERLAP)*RADIAN_FACTOR)*(gaugeRadius+15)));
		dc.DrawArc(rx1, ry1, rx2, ry2, centerPoint.x, centerPoint.y);

//		//Logger::logf(LOG_MSG,wxString::Format(_T("rx1:%i, ry1:%i, rx2:%i, ry2:%i, color:%f"),rx1,ry1,rx2,ry2,color));

		//color = ((int)(count*1.416666666)%255);
		color = color-(1.416666666*(double)d);
	}
    //fprintf( stdout, "VemsGaugeAs::OnPaint 40 \n");
	/*
	dc.SetPen(*wxBLACK_PEN);
	dc.SetBrush(*wxBLACK_BRUSH);
	dc.DrawCircle(centerPoint, gaugeRadius+12);
	*/

	color = 0;
	//wxCoord rx1,ry1,rx2,ry2;
	for (count=0; count<360; count = count + 2) {
		wxBrush brush(*wxWHITE);
		wxPen pen(*wxWHITE);
		brush.SetColour(abs((int)color),abs((int)color),abs((int)color));
		pen.SetColour(abs((int)color),abs((int)color),abs((int)color));
		//dc.SetBrush(*wxTRANSPARENT_BRUSH);
		dc.SetBrush(brush);
		//dc.SetPen(*wxTRANSPARENT_PEN);
		dc.SetPen(pen);
		rx1 = (wxCoord) (centerPoint.x + (cos(count*RADIAN_FACTOR)*(gaugeRadius+7)));
		ry1 = (wxCoord) (centerPoint.y + (sin(count*RADIAN_FACTOR)*(gaugeRadius+7)));
		rx2 = (wxCoord) (centerPoint.x + (cos((count-2-PDA_ARC_OVERLAP)*RADIAN_FACTOR)*(gaugeRadius+7)));
		ry2 = (wxCoord) (centerPoint.y + (sin((count-2-PDA_ARC_OVERLAP)*RADIAN_FACTOR)*(gaugeRadius+7)));
		dc.DrawArc(rx1, ry1, rx2, ry2, centerPoint.x, centerPoint.y);
		//color = ((int)(count*1.416666666)%255);
		if (count < 180) {
			color = color-(1.416666666*2.0);
		} else {
			color = color+(1.416666666*2.0);
		}
	}
    //fprintf( stdout, "VemsGaugeAs::OnPaint 50 \n");
	//dc.SetPen(*wxBLACK_PEN);
	//dc.SetBrush(*wxTRANSPARENT_BRUSH);
	//dc.DrawCircle(centerPoint, gaugeRadius+15);
}

 /* Paint the dial face */
void VemsGaugeAs::paintDialFace(wxDC& dc)
{
	dc.SetBrush(*faceBrush);
	dc.DrawCircle(centerPoint, gaugeRadius);
	dc.SetBrush(wxNullBrush);
}

/* Paint the redline arc, then overpaint the part we don't want in black */
void VemsGaugeAs::paintColoredArea(wxDC& dc)
{
    double  Alpha;
    double  SinA1, CosA1, SinA2, CosA2;
    double  R1, R2;


	//double sweepAngle =  anglePerRange * value;
	//double offsetAngle = anglePerRange * minV;
	//pointerAngle = (sweepAngle + m_gd.GetStartAngle() - offsetAngle) * RADIAN_FACTOR;

    wxCoord rx1,ry1,rx2,ry2;
    double maxV = m_gd->GetMaxValue();
    double minV = m_gd->GetMinValue();
    double gaugeStartAngle = (double) m_gd->GetStartAngle();
    double gaugeAngle = (double) m_gd->GetGaugeAngle();
    double anglePerRange = (double)m_gd->GetGaugeAngle() / (maxV-minV);
    double offsetAngle = anglePerRange * minV;
    //double maxValue = (double) m_gd.GetMaxValue();

    R1 = tickInnerRadius - 4;
    R2 = tickOuterRadius + 5;

    for(uint16_t i= 0; i <  m_gd->coloredA.Count(); i++){
        ColoredArea * ca = m_gd->coloredA.Item(i);


        if( ca->minValue >  ca->maxValue) continue;
        if( ca->minValue < minV) continue; //ca->minValue = minV;
        if( ca->maxValue > maxV) continue; //ca->maxValue = maxV;
        if( ca->minValue == ca->maxValue) continue;

        double maxAngle = (gaugeStartAngle + (anglePerRange * ca->maxValue)) - offsetAngle;
        double minAngle = (gaugeStartAngle + (anglePerRange * ca->minValue)) - offsetAngle;

        Alpha = maxAngle * RADIAN_FACTOR;
        SinA1 = sin(Alpha);
        CosA1 = cos(Alpha);

        Alpha = minAngle * RADIAN_FACTOR;
        SinA2 = sin(Alpha);
        CosA2 = cos(Alpha);

        rx1 = (wxCoord) (centerPoint.x + (CosA1 * R2));
        ry1 = (wxCoord) (centerPoint.y + (SinA1 * R2));
        rx2 = (wxCoord) (centerPoint.x + (CosA2 * R2));
        ry2 = (wxCoord) (centerPoint.y + (SinA2 * R2));
        dc.SetBrush(*ca->color);
        dc.DrawArc(rx1, ry1, rx2, ry2, centerPoint.x, centerPoint.y);

/*
        rx1 = (wxCoord) (centerPoint.x + (CosA1 * R1));
        ry1 = (wxCoord) (centerPoint.y + (SinA1 * R1));
        rx2 = (wxCoord) (centerPoint.x + (CosA2 * R1));
        ry2 = (wxCoord) (centerPoint.y + (SinA2 * R1));
        // dc.SetBrush(*faceBrush);
        dc.SetBrush(*wxBLACK_BRUSH);
        dc.DrawArc(rx1, ry1, rx2, ry2, centerPoint.x, centerPoint.y);
*/
    }
     //dc.Circle(rx1, ry1, rx2, ry2, centerPoint.x, centerPoint.y);
}

/* Paint the tick marks */
void VemsGaugeAs::paintTickMarks(wxDC& dc)
{
	dc.SetPen(*markingsPen);
	int totalDivisions = m_gd->GetTotalDivisions();
	int startAngle = m_gd->GetStartAngle();
	int majorDivisionInterval = m_gd->GetMinorDivisions();

	double incrementAngle = ((double)m_gd->GetGaugeAngle() / (double)totalDivisions) * RADIAN_FACTOR;

    //fprintf( stdout, "VemsGaugeAs::OnPaint 61 \n");
	for (int i=0; i < (totalDivisions+1); i++) {
        //fprintf( stdout, "VemsGaugeAs::OnPaint 62 \n");
		wxCoord x1,y1,x2,y2;

		x1 = (wxCoord) (centerPoint.x + ((cos((startAngle*RADIAN_FACTOR) + (i*incrementAngle))*tickInnerRadius)));
		y1 = (wxCoord) (centerPoint.y + ((sin((startAngle*RADIAN_FACTOR) + (i*incrementAngle))*tickInnerRadius)));

		x2 = (wxCoord) (centerPoint.x + ((cos((startAngle*RADIAN_FACTOR) + (i*incrementAngle))*tickOuterRadius)));
		y2 = (wxCoord) (centerPoint.y + ((sin((startAngle*RADIAN_FACTOR) + (i*incrementAngle))*tickOuterRadius)));

        //fprintf( stdout, "VemsGaugeAs::OnPaint 63 %d\n", majorDivisionInterval);
		// make every nth line thicker
		if ((i % majorDivisionInterval) == 0) {
		    //fprintf( stdout, "VemsGaugeAs::OnPaint 64 \n");
			wxPen p = dc.GetPen();
			p.SetWidth(3);
			dc.SetPen(p);
			//fprintf( stdout, "VemsGaugeAs::OnPaint 64-2 \n");
		} else {
		    //fprintf( stdout, "VemsGaugeAs::OnPaint 65 \n");
			wxPen p = dc.GetPen();
			p.SetWidth(1);
			dc.SetPen(p);
			//fprintf( stdout, "VemsGaugeAs::OnPaint 65-2 \n");
		}
        //fprintf( stdout, "VemsGaugeAs::OnPaint 68 \n");
		// draw the line
		dc.DrawLine(x1,y1,x2,y2);

	};
}

/* Paint the numbers */
void VemsGaugeAs::paintGaugeNumbers(wxDC& dc)
{
   // fprintf( stdout, "VemsGaugeAs::paintGaugeNumbers(): 1\n");
	double textAngle = (double)m_gd->GetStartAngle();

	double gaugeAngle = (double)m_gd->GetGaugeAngle();
	double totalDivisions = (double)m_gd->GetTotalDivisions();
	double majorDivisionInterval = (double)m_gd->GetMinorDivisions();

    double textIncrement =  gaugeAngle / (totalDivisions /majorDivisionInterval);
    int textX = 0;
    int textY = 0;

	dc.SetFont(font);
	dc.SetTextForeground(*markingsColor);
	dc.SetTextBackground(*faceColor);



	for (uint16_t i=0; i< m_gd->m_gaugeNumbers.GetCount(); /*((totalDivisions/majorDivisionInterval)+1);*/ i++) {
		wxCoord textw, texth; //, offsetx, offsety;
		//wxString s;
		//s.Printf(_T("%i"), i*((maximumValue/(totalDivisions/majorDivisionInterval))/scaleFactor));
	    //dc.GetTextExtent(s,&textw,&texth);
	    wxString number = m_gd->m_gaugeNumbers.Item(i);
		dc.GetTextExtent(number,&textw, &texth);
		double offsetx = textw/2.0;
		double offsety = texth/2.0;
		textX = (int)((wxCoord) ((double)centerPoint.x + (double)(cos(textAngle*RADIAN_FACTOR)*(double)textRadius)) - offsetx);
		textY = (int)((wxCoord) ((double)centerPoint.y + (double)(sin(textAngle*RADIAN_FACTOR)*(double)textRadius)) - offsety);
		dc.DrawText(number, textX, textY); //gaugeNumberPoints[i].x,gaugeNumberPoints[i].y);
		//dc.DrawText(s,textX,textY);
		textAngle=textAngle+textIncrement;
	//	 fprintf( stdout, "VemsGaugeAs::paintGaugeNumbers():2- %i\n",i);
	};

    paintGaugeTitle(dc);
}

void VemsGaugeAs::paintGaugeTitle(wxDC &dc)
{
    /// draw Gauge Title Text

    wxSize ws = dc.GetSize();
    ws.Scale(0.5,0.5);

    wxFont tmpFont = dc.GetFont();

    tmpFont.SetPointSize( fontSize*1.5);

    dc.SetFont(tmpFont);

    Logger::logf(LOG_MSG,_T("m_width = %i"),m_width);

    if( m_width > 240 ){

        DrawHelper::DrawArcText(&dc,m_gd->GetTitle(), wxPoint(ws.x,ws.y), textRadius*1.1, 270, 85, fontSize*1.7 );
    }
    else {
        wxString gTitle = m_gd->GetTitle();

        gTitle.Replace(_T(" "), _T("\n"),false);

        wxSize tSize = dc.GetMultiLineTextExtent(gTitle);

        wxPoint titlePos;

        titlePos.x = centerPoint.x - tSize.x/2;
        titlePos.y = (int)(centerPoint.y * 1.45) ;

        DrawHelper::DrawMultiLineText(&dc,gTitle,titlePos,wxALIGN_CENTER);

    }

    dc.SetFont(font);
}

void VemsGaugeAs::OnPaint(wxPaintEvent& WXUNUSED(event))
{
//    Logger::logf(LOG_MSG,_T("VemsGaugeAs::OnPaint"));


	wxBufferedPaintDC dc(this);

    dc.Clear();

    if(!GBuffer->IsValid() || !GCover->IsValid()){
        Logger::logf(LOG_MSG, _T("VemsGaugeAs::OnPaint() invalid AS-BUFFER !!"));
        return;
    }

    DrawPointer();


    wxMemoryDC Cover_DC;

    Cover_DC.SetBackground(*wxWHITE_BRUSH);
    Cover_DC.Clear();

    GCover->GetMemoryDC(&Cover_DC);


    dc.Blit(wxPoint(0,0),wxSize(W,W), &Cover_DC, wxPoint(0,0));


}

void VemsGaugeAs::paintGaugeBg(int whatpaint)
{
//	Logger::logf(LOG_MSG,_T("paintGaugeBg() START"));

	//wxMemoryDC memDC(*VemsGaugeAsBgBitmap);

	wxMemoryDC memDC;

	//memDC.SetBackgroundMode(wxTRANSPARENT);

	//memDC.SetBackground(*wxTRANSPARENT_BRUSH);

	memDC.SetBackground(*wxWHITE_BRUSH);

    if(whatpaint & BGP_BEZEL ) {
        memDC.SelectObject(*VemsGaugeAsBmpBezel);
        memDC.Clear();
        paintBezel(memDC);
        //memDC.SelectObject(wxNullBitmap);
    }

    memDC.SelectObject(*VemsGaugeAsBgBitmap);

    memDC.DrawBitmap(*VemsGaugeAsBmpBezel,0,0,1);

    paintDialFace(memDC);
    paintColoredArea(memDC);
    paintTickMarks(memDC);
    paintGaugeNumbers(memDC);

    memDC.SetPen(*centerPen);
    memDC.SetBrush(*centerBrush);
    memDC.DrawCircle(centerPoint, (wxCoord)(pointerInnerRadius));


	memDC.SelectObject(wxNullBitmap);



/*
    if(whatpaint & BGP_DIALFACE ){
        memDC.SelectObject(*VemsGaugeAsBmpDialFace);
        memDC.Clear();
        paintDialFace(memDC);
        //memDC.SelectObject(wxNullBitmap);
    }

    if(whatpaint & BGP_TICKMARKS ){
        memDC.SelectObject(*VemsGaugeAsBmpTickMarks);
        memDC.Clear();
        paintTickMarks(memDC);
        //memDC.SelectObject(wxNullBitmap);
    }

    //Paint internal circle
    if(whatpaint & BGP_INNERCIRCLE ) {
        memDC.SelectObject(*VemsGaugeAsBmpInnerCirc);
        memDC.Clear();
        memDC.SetPen(*centerPen);
        memDC.SetBrush(*centerBrush);
        memDC.DrawCircle(centerPoint, pointerInnerRadius);
        //memDC.SelectObject(wxNullBitmap);
    }

    if(whatpaint & BGP_GAUGENUMBERS ) {
        memDC.SelectObject(*VemsGaugeAsBmpGaugeNums);
        memDC.Clear();
        //memDC.SetPen(wxNullPen);
        paintGaugeNumbers(memDC);
        //memDC.SelectObject(wxNullBitmap);
    }

    if(whatpaint & BGP_COLOREDAREA ){
        memDC.SelectObject(*VemsGaugeAsBmpColArea);
        memDC.Clear();
        paintColoredArea(memDC);
        //memDC.SelectObject(wxNullBitmap);
    }

    memDC.SelectObject(*VemsGaugeAsBgBitmap);

     memDC.Clear();

     memDC.DrawBitmap(*VemsGaugeAsBmpBezel,0,0,1);
     memDC.DrawBitmap(*VemsGaugeAsBmpDialFace,0,0,1);
     memDC.DrawBitmap(*VemsGaugeAsBmpTickMarks,0,0,1);
     memDC.DrawBitmap(*VemsGaugeAsBmpInnerCirc,0,0,1);
     memDC.DrawBitmap(*VemsGaugeAsBmpColArea,0,0,1);

    memDC.SelectObject(wxNullBitmap);
*/


//	Logger::logf(LOG_MSG,_T("paintGaugeBg() END"));
}


void VemsGaugeAs::OnSize(wxSizeEvent& WXUNUSED(event))
{
//	Logger::logf(LOG_MSG,_T("VemsGaugeAs::OnSize START"));


#if defined (__WINCE_STANDARDSDK__)

//	Refresh();

//	//Logger::logf(LOG_MSG,wxString::Format(_T("VemsGaugeAs::OnSize WINCE:  x=%i, y=%i"), GetClientSize().x, GetClientSize().y));

#else

/*    if(m_drawInProgress){
        Logger::logf(LOG_MSG, _T("VemsGaugeAs::OnSize() m_drawInProgress!!"));
        return;
    }
    m_drawInProgress = true;
*/
    UpdateSize();
    Refresh(false);

//  m_drawInProgress = false;
#endif

}

void VemsGaugeAs::UpdateSize()
{
    int x,y;

    GetSize(&x,&y);
    SetSize(x, y);
    CalculateSize();

    UpdateGaugeValue();
    UpdatePointer();

    if(x == y)
      {
        CreateBoundMask();
        ReCreateFace();
        DrawPointer();
      }
}

void VemsGaugeAs::CalculateLegentPoint(wxDC& dc)
{
    wxCoord textw, texth;

    gaugeLegend = m_gd->GetUnit();

    if( m_gd->GetUnitsScaleFactor() != 1)
        gaugeLegend += wxString::Format(_T(" x %i"), m_gd->GetUnitsScaleFactor() );

	dc.GetTextExtent(gaugeLegend,&textw,&texth);
	gaugeLegendPoint.x = centerPoint.x-(textw/2);
	gaugeLegendPoint.y = (int)(centerPoint.y-legendRadius-(texth/2));
}

void VemsGaugeAs::CalculateSize()
{
 //   Logger::logf(LOG_MSG,_T(" VemsGaugeAs::CalculateSize() START"));

//    fprintf(stdout, "VemsGaugeAs::CalculateSize()\n");
	  wxClientDC dc(this);
	  dc.GetSize(&width,&height);

    //fprintf(stdout,  "VemsGaugeAs::CalculateSize x=%i, y=%i!!\n", width, height );


    if(DefPosSize)
        m_parent->GetSize(&width,&height);
    else{
        width = gsize.x;
        height = gsize.y;
    }

	  if(height > width)
        height = width;
	  if(width > height)
        width = height;

    SetSize(width,height);
	  dc.GetSize(&width,&height);

    //fprintf(stdout,  "VemsGaugeAs::CalculateSize x=%i, y=%i!!\n", width, height );
	  centerPoint.x = width/2;
	  centerPoint.y = height/2;

	  if (width > height)
      {
		    gaugeRadius = (height/2) - 15;
	    }
    else
      {
		    gaugeRadius = (width/2) - 15;
	    }

	  tickInnerRadius = gaugeRadius*0.85;

//	  Logger::logf(LOG_MSG,_T("tickInnerRadius = %f"),tickInnerRadius);

	  tickOuterRadius = gaugeRadius*0.95;
	  textRadius = gaugeRadius*0.70;
	  fontSize = (int)(gaugeRadius*0.09);
	  font = wxFont(fontSize,wxFONTFAMILY_DEFAULT,wxFONTSTYLE_NORMAL,wxFONTWEIGHT_NORMAL);
	  dc.SetFont(font);

	  brandRadius = gaugeRadius*0.7;
	  legendRadius = gaugeRadius*0.4;
	  valueRadius = gaugeRadius*0.4;
	  pointerInnerRadius = gaugeRadius*0.2;
	  pointerOuterRadius = gaugeRadius*0.9;

	  wxCoord textw, texth;
	  dc.GetTextExtent(gaugeValue,&textw,&texth);
	  gaugeValuePoint.x = centerPoint.x-(textw/2);
	  gaugeValuePoint.y = (int)(centerPoint.y+valueRadius-(texth/2));

    CalculateLegentPoint(dc);

	  dc.GetTextExtent(gaugeBrand,&textw,&texth);
	  gaugeBrandPoint.x = centerPoint.x-(textw/2);
	  gaugeBrandPoint.y = (int)(centerPoint.y+brandRadius-(texth/2));

    //-----------------------------------------------------------------------
    // MJ variables
   // Logger::logf(LOG_MSG,_T("VemsGaugeAs::CalculateSize()") );
    if(width < height)
      W = width;
    else\
      W = height;

    CP.y = CP.x = /*floor*/ (W / 2);

    RG = CP.x;
    RS = RG - 4;

    m_width = width;

 //   Logger::logf(LOG_MSG,_T("VemsGaugeAs::CalculateSize() END!") );
}

void VemsGaugeAs::OnMouseWheel(wxMouseEvent& event)                                 ///to VemsWidgetBase ????
{
    /*
 //   Logger::logf(LOG_MSG,_T("VemsGaugeAs::OnMouseWheel"));

	bool forwards = (event.GetWheelRotation() > 0);

    ////Logger::logf(LOG_MSG,_T("VemsGaugeAs::OnMouseWheel"));

	if (forwards) {
		SetValue(m_value + 100);
	} else {
		SetValue(m_value - 100);
	}
	//Refresh();
	*/
}

void VemsGaugeAs::SetFaceColor(wxColor *color)
{
	faceColor = color;
	faceBrush = new wxBrush(*color);
}

void VemsGaugeAs::SetPointerColor(wxColor *color)
{
	pointerBrush = new wxBrush(*color);
	pointerPen = new wxPen(*color);
}

void VemsGaugeAs::SetMarkingsColor(wxColor *color)
{
	markingsColor = color;
	markingsPen = new wxPen(*color);
}

void VemsGaugeAs::SetCenterColor(wxColor *color)
{
	centerBrush = new wxBrush(*color);
	centerPen = new wxPen(*color);
}

void VemsGaugeAs::setSize(int x , int y)
{
    //fprintf(stdout, "setSize: %d\n", x);

    m_bmp = wxBitmap(x, x, -1);
    wxMemoryDC temp_dc;
    temp_dc.SelectObject(m_bmp);

    temp_dc.SetBackground(*wxWHITE);
    //temp_dc.SetPen(*wxBLACK);
    // Set a black pen and black brush
    //temp_dc.SetPen(wxPen(*wxBLACK, 2, wxSOLID));
    //temp_dc.SetBrush(*wxBLACK);

    //temp_dc.DrawCircle(x/2, y/2, x/2);
    temp_dc.DrawCircle(x/2, x/2, x/2);

}

wxRegion VemsGaugeAs::GetShape()
{
    int size;

    if(GetSize().x < GetSize().y)
        size = GetSize().x;
    else
        size = GetSize().y;

//    //Logger::logf(LOG_MSG,wxString::Format(_T("size=%i"),size));

    wxBitmap tempbmp = wxBitmap(size,size);

    wxMemoryDC temp_dc;

    temp_dc.SelectObject(tempbmp);
    temp_dc.SetBackground(*wxWHITE);
    temp_dc.DrawCircle(size/2, size/2, (size/2)-3);

    wxRegion region(tempbmp, *wxBLACK);

    return region;

}


/*
  -----------------------------------------------------------------------
    Methods of VemsGauge by Majer János
  -----------------------------------------------------------------------
    Majer János
    GÖD
    2007.07.11
  -----------------------------------------------------------------------
*/


void VemsGaugeAs::CreateBoundMask()
  {
    wxMemoryDC temp_dc;

    m_bmp = wxBitmap(W, W, 1);
    temp_dc.SelectObject(m_bmp);

    temp_dc.SetBackground(*wxWHITE);
    //temp_dc.DrawCircle(w/2, w/2, w/2);

    temp_dc.DrawCircle((wxCoord)(CP.x-1), (wxCoord)(CP.y-0), (wxCoord)(RG - 2));

  }//VemsGauge::CreateBoundMask()


void VemsGaugeAs::DrawPointer()
{
    //UpdatePointer();

    wxMemoryDC  DC;




    int         Ct;
    TDPoint     *P;
    TRGB        RGB;
    int         i;

    int         w, h;


    // wxMessageBox("M DrawPointer 1");


    // TMGraphics::CopyBitmap(TMP_BMP, BCK_BMP);



    if(!GBuffer->IsValid())
      return;

    GCover->CopyFrom(GBuffer);

    GCover->GetMemoryDC(&DC);

    font.SetPointSize((int)(gaugeRadius * 0.17));

    DC.SetFont(font);

	DC.SetTextForeground(*markingsColor);
	DC.SetTextBackground(*faceColor);
    DC.GetTextExtent(gaugeValue, &w, &h);

    gaugeValuePoint.x = (W / 2)   - (w / 2);
    //gaugeValuePoint.y = (0.80 * W) - h;
    gaugeValuePoint.y = (int)(centerPoint.y + valueRadius - h/2);



    DC.SetTextForeground(*wxWHITE);
    DC.DrawText(gaugeValue,gaugeValuePoint);


    if(m_graphQual < 2) {

        DC.SetPen(*pointerPen);
        DC.SetBrush(*pointerBrush);
        DC.DrawPolygon(3,pointerPoints, centerPoint.x, centerPoint.y);

    } else {


        Ct = 4;
        P = new TDPoint[Ct];

        P[0].x = PPoint[0].x;
        P[0].y = PPoint[0].y;
        P[1].x = PPoint[1].x;
        P[1].y = PPoint[1].y;
        P[2].x = PPoint[2].x;
        P[2].y = PPoint[2].y;
        P[3].x = PPoint[0].x;
        P[3].y = PPoint[0].y;

        for(i=0; i<Ct; i++)
          {
            P[i].x = (P[i].x + CP.x);
            P[i].y = (P[i].y + CP.y);
          }


        // RGB = TRGB(255, 0, 0);
        RGB = TRGB(253,204, 0);


        GCover->DrawPolygon_1(RGB, P, Ct);
        delete P;
    }

    /// Shadow and Highlight

    //RGB = TRGB(255, 96, 96);
    //GCover->DrawLine_1(RGB, P[2].x, P[2].y, P[1].x, P[1].y);

    //RGB = TRGB(96, 0, 0);
    //GCover->DrawLine_1(RGB, P[0].x, P[0].y, P[1].x, P[1].y);


//    //Logger::logf(LOG_MSG,wxString::Format(_T("VemsGaugeAs::DrawPointer 6.  CP x=%f, y=%f , tickInnerRadius=%f "),CP.x, CP.y, tickInnerRadius ));





    GCover->DrawGradientCircle(TRGB(90,90,90), TRGB(180,180,180), CP.x, CP.y, 0.25*tickInnerRadius, 0);

    GCover->DrawFillCircle(TRGB(135,135,135), CP.x, CP.y, 0.22*tickInnerRadius);


    // TestRobust();

}

//-----------------------------------------------------------------------

void VemsGaugeAs::DrawNumbers()
{
    wxMemoryDC DC;

    GBuffer->GetMemoryDC(&DC);
    paintGaugeNumbers(DC);

	  // Paint the logo
	  DC.DrawText(gaugeBrand,gaugeBrandPoint);

	  // Paint the legend
    CalculateLegentPoint(DC);
	  DC.DrawText(gaugeLegend, gaugeLegendPoint);

	  // Paint the value
}//VemsGauge::DrawNumbers()

/*
void VemsGaugeAs::TestRobust()
  {
    double  x1, x2, y1, y2;
    double  Alpha;
    double  dA;
    double  R1, R2;
    double  SinA, CosA;
    int     i, Ct;
    TRGB    RGB;


    //-----------------------------------------------------------------------
    // TMGraphics::DrawALine_1
    R1 = W/4;
    R2 = W;
    Ct = 20;
    Alpha = 0;
    dA = 2 * M_PI / Ct;

    RGB.R = 255;
    RGB.G = 255;
    RGB.B = 0;

    // Half-Cross Lines
    for(i=0; i<Ct; i++)
      {
        SinA = sin(Alpha);
        CosA = cos(Alpha);

        x1 = CP.x + CosA * R1 + SinA * R1;
        y1 = CP.x + SinA * R1 - CosA * R1;

        x2 = CP.x + CosA * R2 + SinA * R2;
        y2 = CP.x + SinA * R2 - CosA * R2;

        // x2 = CP.x;
        // y2 = CP.y;

        GCover->DrawLine_1(RGB, x1, y1, x2, y2);

        Alpha += dA;
      }

    // Full outside lines
    GCover->DrawLine_1(RGB, 0, -10, W + 10, -10);

    GCover->DrawLine_1(RGB, -10, W + 10, W + 10, W + 10);

    GCover->DrawLine_1(RGB, -10, -10, -10, W + 10);

    GCover->DrawLine_1(RGB, W + 10, -10, W + 10, W + 10);

    // Full-Cross Lines
    GCover->DrawLine_1(RGB, CP.x, -10, CP.x, W + 10);
    GCover->DrawLine_1(RGB, -10, CP.y, W + 10, CP.y);


    //-----------------------------------------------------------------------
    //TMGraphics::FillACircle()

    GCover->DrawFillCircle(TRGB(0,255,128), 0, 0, 40);
    GCover->DrawFillCircle(TRGB(0,255,128), W, 0, 40);
    GCover->DrawFillCircle(TRGB(0,255,128), W, W, 40);
    GCover->DrawFillCircle(TRGB(0,255,128), 0, W, 40);

  }//VemsGauge::TestRobust()

*/

//-----------------------------------------------------------------------
// Redraw the face, without pointer and value
void VemsGaugeAs::ReCreateFace()
  {
    Logger::logf(LOG_MSG,_T(" VemsGaugeAs::ReCreateFace() START"));
    wxMemoryDC DC;

    DC.SetBackground(*wxWHITE_BRUSH);
    DC.Clear();

//GBuffer->GetMemoryDC(&DC);

    GBuffer->Reset(W, W);

   //   Logger::logf(LOG_MSG,_T(" VemsGaugeAs::ReCreateFace() 1"));
    // Calculate sizes

    // Create DC
//GBuffer->GetMemoryDC(&DC);
  //   Logger::logf(LOG_MSG,_T(" VemsGaugeAs::ReCreateFace() 2"));
    // Background paint
    // DC.SetBackground(*wxGREY_BRUSH);
    // DC.Clear();

    // Bezel
    DrawBezel();
//     Logger::logf(LOG_MSG,_T(" VemsGaugeAs::ReCreateFace() 3"));

    // Sectors
    // paintDialFace(DC);
    paintColoredArea(DC);
//     Logger::logf(LOG_MSG,_T(" VemsGaugeAs::ReCreateFace() 4"));
    // ThickMarks
    DrawTickMarks();
 //    Logger::logf(LOG_MSG,_T(" VemsGaugeAs::ReCreateFace() 5"));
    // Captions
    paintGaugeNumbers(DC);
 //    Logger::logf(LOG_MSG,_T(" VemsGaugeAs::ReCreateFace() 6"));
	  // Paint the logo
    // DC.SetTextForeground(wxColour(255,255,0));
	  DC.DrawText(gaugeBrand,gaugeBrandPoint);
  //   Logger::logf(LOG_MSG,_T(" VemsGaugeAs::ReCreateFace() 7"));
	  // Paint the legend
     CalculateLegentPoint(DC);
  //    Logger::logf(LOG_MSG,_T(" VemsGaugeAs::ReCreateFace() 8"));
	 DC.DrawText(gaugeLegend, gaugeLegendPoint);
 //    Logger::logf(LOG_MSG,_T(" VemsGaugeAs::ReCreateFace() 9"));
    UpdateGaugeValue();

  }//VemsGauge::ReCreateface()


//-----------------------------------------------------------------------
void VemsGaugeAs::DrawBezel()
  {
    double   R;
    TRGB  RGB_A,  RGB_B;



    R = RG - 2;

    RGB_A = TRGB(40,40,40);
    RGB_B = TRGB(230,230,230);
 //   Logger::logf(LOG_MSG,_T(" VemsGaugeAs::DrawBezel() 1"));

    GBuffer->DrawGradientCircle(RGB_A, RGB_B, CP.x-1, CP.y-0, R, 0);

 //    Logger::logf(LOG_MSG,_T(" VemsGaugeAs::DrawBezel() 2"));
    R = (R  + (tickOuterRadius + 3)) / 2;
    GBuffer->DrawGradientCircle(RGB_B, RGB_A, CP.x-1, CP.y-0, R, 0);


    R = tickOuterRadius + 4;
    GBuffer->DrawFillCircle(TRGB(0,0,0), CP.x-1, CP.y-0, R);

  }//VemsGauge::DrawBezel()

//-----------------------------------------------------------------------
void VemsGaugeAs::DrawTickMarks()
  {
    TRGB    RGB;
    double  x1,y1,x2,y2;
    double  R1, R2;
    double  Alpha;
    double  SinA, CosA;

    double   X = CP.x - 1;
    double   Y = CP.y - 0;


    int totalDivisions = m_gd->GetTotalDivisions();
	  int startAngle = m_gd->GetStartAngle();
	  int majorDivisionInterval = m_gd->GetMinorDivisions();

	  double incrementAngle = ((double)m_gd->GetGaugeAngle() / (double)totalDivisions) * RADIAN_FACTOR;

    RGB = TRGB(255,255,255);

    /**/
    // R1 = 0.66 * tickInnerRadius - 3;
    R1 = tickInnerRadius - 3;
    R2 = tickOuterRadius + 2;

	  for (int i=0; i < (totalDivisions+1); i++)
      {
        Alpha = (startAngle * RADIAN_FACTOR) + (i * incrementAngle);

        SinA = sin(Alpha);
        CosA = cos(Alpha);
		    x1 = X + (CosA * R2);
		    y1 = Y + (SinA * R2);

		    x2 = X + (CosA * R1);
		    y2 = Y + (SinA * R1);

        if ((i % majorDivisionInterval) == 0)
          {
            // Major Division lines are widen
            // W = 3;

            if(W < 250)
              GBuffer->DrawLine(RGB, 3, x1, y1, x2, y2);
            else if(W < 400)
              GBuffer->DrawLine(RGB, 4, x1, y1, x2, y2);
            else
              GBuffer->DrawLine(RGB, 5, x1, y1, x2, y2);

          }
        else
        {
            // W = 1;

            // if(i < ((totalDivisions+1)/2))
              {
                RGB = TRGB(255,255,255);
                int XF = (int)(1.2 * RG);
                RGB = TRGB(XF, XF, XF);
                GBuffer->DrawLine_1(RGB, x1, y1, x2, y2);
                RGB = TRGB(255,255,255);
                GBuffer->DrawLine_1(RGB, x1, y1, x2, y2);
              }
            /**
            else
              {
                // "pontosabb" kirajzolás
                RGB = TRGB(128,255,255);
                GBuffer->SSDrawLine(RGB, TDPoint(x1, y1), TDPoint(x2, y2), 15);
              }
            **/
          }
      }

    RGB = TRGB(0, 0, 0);
    // RGB = TRGB(96, 230, 255);
    // RGB = TRGB(90, 90, 90);
    //GBuffer->DrawDonut(RGB, X, Y, R1 + 1, R1 - 3);
    GBuffer->DrawDonut(RGB, X, Y, R1 + 1, 5);
    GBuffer->DrawDonut(RGB, X, Y, R2 + 2, R2 - 1);


  }//VemsGauge::PaintTickMarks()

