#include "VemsGaugeDescDialog.h"

#include <wx/statline.h>
#include "../ThemesHelper.h"
#include "../common/VemsMessageHashHelper.h"
#include "GaugeGroupManager.h"

VemsWidgetBaseEditor::VemsWidgetBaseEditor(VemsWidgetBase * pw)
{
    m_parentWidget = pw;

}


int VemsWidgetBaseEditor::ShowEditor(VEMSGaugeDesc* gd)
{
    if(!m_parentWidget) return -1;

    VemsGaugeDescDialog* dlg  = new VemsGaugeDescDialog(m_parentWidget,gd);

    dlg->Show();

    return 1;

}

/**********************************************************************************************************************************
 *                                                  ColoredAreaEdit
 */

void ColoredAreaEdit::Update()
{

    double min = 0.0, max = 0.0;

    if(MinCtrl)
        MinCtrl->GetValue().ToDouble( &min );

    if(MaxCtrl)
        MaxCtrl->GetValue().ToDouble( &max );

    if(Ca){
        Ca->minValue = min;
        Ca->maxValue = max;
        Ca->ColorName = ColorNameList->GetValue();
    }

}

void ColoredAreaEdit::RefreshCtrl(const wxString &numformat)
{
    MinCtrl->SetValue( wxString::Format(numformat,Ca->minValue) );
    MaxCtrl->SetValue( wxString::Format(numformat,Ca->maxValue) );
    ColorNameList->SetStringSelection( Ca->ColorName );
}

/**********************************************************************************************************************************
 *                                                VemsGaugeDescDialog
 */




VemsGaugeDescDialog::VemsGaugeDescDialog(VemsWidgetBase * pw, VEMSGaugeDesc* gd)
{
    //ctor
    m_parentWidget = pw;

    m_outerGaugeDesc = gd;

    CreateDialog();

}

VemsGaugeDescDialog::~VemsGaugeDescDialog()
{
    //dtor
}

void VemsGaugeDescDialog::RefreshParent()
{
    GetDesc()->SetGaugeNumbers();
    m_parentWidget->RedrawWidget();
    m_parentWidget->Refresh(false);
}

void VemsGaugeDescDialog::OnSlider( wxScrollEvent &evt )
{
    int id = evt.GetId();

    wxSlider * sl = m_items[id]->m_slider;
    GDDesc  * gdd = m_items[id]->m_gdd;

    if( !sl || !gdd ) return;

    //Logger::logf(LOG_MSG,gdd->Name+_T("=%i"),sl->GetValue());

    if( gdd->Factor > 1)
        GetDesc()->Properties[ gdd->Name ] =  (double)sl->GetValue() / (double)gdd->Factor ;
    else
        GetDesc()->Properties[ gdd->Name ] = (long)sl->GetValue();

    RefreshParent();

}

void VemsGaugeDescDialog::RefreshByDesc(DescItemDesc * desc, bool refresparent)
{
    if(desc == NULL) return;

    wxTextCtrl * ctrl = desc->m_textCtrl;
    GDDesc  * gdd = desc->m_gdd;
    ColoredAreaEdit * cae = desc->m_cae;
    bool isnum = desc->m_isNum;

    if( !gdd ) return;

    if( ctrl ) {
        if(isnum) {
            double val = 0.0;
            ctrl->GetValue().ToDouble( &val );
            GetDesc()->Properties[ gdd->Name ] = val;
        } else {
            wxString val = ctrl->GetValue();
            GetDesc()->Properties[ gdd->Name ] = val;
        }
    } else if( cae ){
        cae->Update();
    }

    if(refresparent)
        RefreshParent();
}

void VemsGaugeDescDialog::OnRefreshButton(wxCommandEvent &evt)
{
    int id = evt.GetId();

    if( m_items.find(id) == m_items.end() ) return;

    RefreshByDesc( m_items[id], true );
}

void VemsGaugeDescDialog::OnEnter(wxCommandEvent &evt)
{
    int id = evt.GetId();

    if( m_itemsToEnter.find(id) == m_itemsToEnter.end() ) return;

    RefreshByDesc( m_itemsToEnter[id], true );
}

void VemsGaugeDescDialog::AddSlider( GDDesc * gdd, wxWindow * parent, wxGridBagSizer * sizer )
{
    //Logger::logf(LOG_MSG,_T("p=%p,s=%i"),parent,sizer);

    int id = wxNewId();

    wxStaticText *st = new wxStaticText(parent, wxID_ANY, gdd->Title);

    wxFont font = st->GetFont();
    font.SetWeight(wxFONTWEIGHT_BOLD);
    st->SetFont( font );

    long val;

    if( gdd->Factor > 1)
        val = (long)(GetDesc()->Properties[gdd->Name].GetDouble() * gdd->Factor);
    else
        val = GetDesc()->Properties[gdd->Name].GetLong();

    wxSlider * sl = new wxSlider(parent, id, val , gdd->Min, gdd->Max, wxDefaultPosition, wxSize(200,-1), wxSL_LABELS | wxSL_HORIZONTAL);

    //sl->SetName( gdd->Name );

    DescItemDesc * did = new DescItemDesc(sl, gdd);

    m_items[id] = did;

    Connect(id, wxEVT_SCROLL_THUMBTRACK, wxScrollEventHandler(VemsGaugeDescDialog::OnSlider), NULL, this);

    Connect(id, wxEVT_SCROLL_THUMBRELEASE, wxScrollEventHandler(VemsGaugeDescDialog::OnSlider), NULL, this);
    Connect(id, wxEVT_SCROLL_PAGEDOWN, wxScrollEventHandler(VemsGaugeDescDialog::OnSlider), NULL, this);
    Connect(id, wxEVT_SCROLL_PAGEUP, wxScrollEventHandler(VemsGaugeDescDialog::OnSlider), NULL, this);
    Connect(id, wxEVT_SCROLL_LINEDOWN, wxScrollEventHandler(VemsGaugeDescDialog::OnSlider), NULL, this);
    Connect(id, wxEVT_SCROLL_LINEUP, wxScrollEventHandler(VemsGaugeDescDialog::OnSlider), NULL, this);
    Connect(id, wxEVT_SCROLL_BOTTOM, wxScrollEventHandler(VemsGaugeDescDialog::OnSlider), NULL, this);
    Connect(id, wxEVT_SCROLL_TOP, wxScrollEventHandler(VemsGaugeDescDialog::OnSlider), NULL, this);


    int rows = sizer->GetRows();

    sizer->Add(st, wxGBPosition(rows,0),wxDefaultSpan,wxALIGN_CENTER_VERTICAL | wxALL,3);
    sizer->Add(sl, wxGBPosition(rows,1),wxDefaultSpan,wxEXPAND|wxALL,3);

    //m_mainSizer->Add(st,0,wxALL , 3);
    //m_mainSizer->Add(sl,1,wxALL | wxEXPAND, 5);

    sizer->Layout();

}

bool VemsGaugeDescDialog::IsRefreshButton()
{
    return m_outerGaugeDesc == NULL ? true : false;
}

void VemsGaugeDescDialog::AddString(GDDesc * gdd, wxWindow * parent, wxGridBagSizer * sizer)
{

    int id = wxNewId();

    wxStaticText *st = new wxStaticText(parent, wxID_ANY, gdd->Title);

    wxFont font = st->GetFont();
    font.SetWeight( wxFONTWEIGHT_BOLD );
    st->SetFont( font );

    wxString val = GetDesc()->Properties[gdd->Name].GetString();

    wxTextCtrl * tc = new wxTextCtrl(parent, id, val, wxDefaultPosition, wxSize(150, wxDefaultCoord), wxTE_LEFT | wxTE_PROCESS_ENTER);
    Connect(id, wxEVT_COMMAND_TEXT_ENTER, wxCommandEventHandler(VemsGaugeDescDialog::OnEnter));

    DescItemDesc * did = new DescItemDesc(tc, gdd, false);

    m_itemsToEnter[id] = did;

    id = wxNewId();

    m_items[id] = did;

    wxButton * butt = NULL;

    if(IsRefreshButton()) {
        butt = new wxButton(parent , id, _T("RefreshGUI"));
        Connect(id, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(VemsGaugeDescDialog::OnRefreshButton), NULL, this);
    }

    int rows = sizer->GetRows();
    wxBoxSizer * tsizer = new wxBoxSizer(wxHORIZONTAL);
    sizer->Add(st, wxGBPosition(rows,0),wxDefaultSpan,wxALIGN_CENTER_VERTICAL | wxALL,3);
    tsizer->Add(tc, 1, wxEXPAND|wxALL,3);
    if(IsRefreshButton() && butt != NULL) tsizer->Add(butt, 0, wxALL,3);
    sizer->Add(tsizer, wxGBPosition(rows,1),wxDefaultSpan,wxEXPAND);
    sizer->Layout();

}

void VemsGaugeDescDialog::AddNumeric(GDDesc * gdd, wxWindow * parent, wxGridBagSizer * sizer)
{

    int id = wxNewId();

    wxStaticText *st = new wxStaticText(parent, wxID_ANY, gdd->Title);

    wxFont font = st->GetFont();
    font.SetWeight( wxFONTWEIGHT_BOLD );
    st->SetFont( font );

    double val = GetDesc()->Properties[gdd->Name].GetDouble() * gdd->Factor;

    wxString tf = GetDesc()->Properties[_T("numformat")].GetString();

    wxTextCtrl * tc = new wxTextCtrl(parent, id, wxString::Format(tf,val), wxDefaultPosition, wxSize(140, wxDefaultCoord), wxTE_LEFT | wxTE_PROCESS_ENTER);
    Connect(id, wxEVT_COMMAND_TEXT_ENTER, wxCommandEventHandler(VemsGaugeDescDialog::OnEnter));

	wxTextValidator validator( wxFILTER_NUMERIC );
	tc->SetValidator(validator);

    DescItemDesc * did = new DescItemDesc(tc, gdd);

    m_itemsToEnter[id] = did;

    id = wxNewId();

    m_items[id] = did;

    wxButton * butt = NULL;

    if(IsRefreshButton()){
        butt = new wxButton(parent , id, _T("RefreshGUI"));
        Connect(id, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(VemsGaugeDescDialog::OnRefreshButton), NULL, this);
    }

    int rows = sizer->GetRows();
    wxBoxSizer * tsizer = new wxBoxSizer(wxHORIZONTAL);
    sizer->Add(st, wxGBPosition(rows,0),wxDefaultSpan,wxALIGN_CENTER_VERTICAL | wxALL,3);
    tsizer->Add(tc, 1, wxEXPAND|wxALL,3);
    if(IsRefreshButton() && butt != NULL ) tsizer->Add(butt, 0, wxALL,3);
    sizer->Add(tsizer, wxGBPosition(rows,1),wxDefaultSpan,wxEXPAND);
    sizer->Layout();

}

VEMSGaugeDesc* VemsGaugeDescDialog::GetDesc()
{
    if(m_outerGaugeDesc == NULL)
        return m_parentWidget->m_gd;
    else
        return m_outerGaugeDesc;
}

void VemsGaugeDescDialog::AddColoredArea(GDDesc * gdd, ColoredArea * ca, wxWindow * parent, wxGridBagSizer * sizer)
{

    int id = wxNewId();

    //wxStaticText *st = new wxStaticText(parent, wxID_ANY, wxString::Format(_T("#%i"),ca->itemID) );
    wxStaticText *st = new wxStaticText(parent, wxID_ANY, _T("#") );
    wxFont font = st->GetFont();
    font.SetWeight( wxFONTWEIGHT_BOLD );
    st->SetFont( font );

    double min = ca->minValue;
    double max = ca->maxValue;

    wxString tf = GetDesc()->Properties[_T("numformat")].GetString();

    int idtcmin = wxNewId();
    int idtcmax = wxNewId();

    wxTextCtrl * tcmin = new wxTextCtrl(parent, idtcmin, wxString::Format(tf,min), wxDefaultPosition, wxSize(60, -1), wxTE_LEFT | wxTE_PROCESS_ENTER);
    wxTextCtrl * tcmax = new wxTextCtrl(parent, idtcmin, wxString::Format(tf,max), wxDefaultPosition, wxSize(60, -1), wxTE_LEFT | wxTE_PROCESS_ENTER);

    Connect(idtcmax, wxEVT_COMMAND_TEXT_ENTER, wxCommandEventHandler(VemsGaugeDescDialog::OnEnter));
    Connect(idtcmin, wxEVT_COMMAND_TEXT_ENTER, wxCommandEventHandler(VemsGaugeDescDialog::OnEnter));

    wxArrayString colornames;

    ThemesHelper::GetDefinedColornames( m_parentWidget->GetGaugeDescMgr()->GetWidgetThemes(), m_parentWidget->GetThemeName(), colornames);

    int cbid = wxNewId();
    wxComboBox * cbcolor = new wxComboBox(parent, cbid, ca->ColorName, wxDefaultPosition, wxSize(65, -1), colornames,wxCB_READONLY|wxTE_PROCESS_ENTER);

    Connect(cbid,wxEVT_COMMAND_TEXT_UPDATED,  wxCommandEventHandler( VemsGaugeDescDialog::OnEnter ) );
    Connect(cbid,wxEVT_COMMAND_TEXT_ENTER,  wxCommandEventHandler( VemsGaugeDescDialog::OnEnter ) );

	wxTextValidator validator( wxFILTER_NUMERIC );
	tcmin->SetValidator(validator);

	tcmax->SetValidator(validator);

    ColoredAreaEdit * cae = new ColoredAreaEdit(tcmin, tcmax,cbcolor, ca);

    DescItemDesc * did = new DescItemDesc(cae, gdd);

    m_itemsToEnter[cbid] = did;

    m_itemsToEnter[idtcmin] = did;
    m_itemsToEnter[idtcmax] = did;

    m_items[id] = did;

    wxButton * butt = NULL;
    if(IsRefreshButton()) {
        butt = new wxButton(parent , id, _T("RefreshGUI"), wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT);
        Connect(id, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(VemsGaugeDescDialog::OnRefreshButton), NULL, this);
    }

    int id2 = wxNewId();

    wxVariant * var = new wxVariant((long)id);

    wxButton * dbutt = new wxButton(parent , id2, _T("X"), wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT);
    Connect(id2, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(VemsGaugeDescDialog::OnDelCaItem), var, this);

    int rows = sizer->GetRows();

    sizer->Add(st, wxGBPosition(rows,0),wxDefaultSpan,wxALIGN_CENTER_VERTICAL | wxALL,3);

    wxBoxSizer * tsizer = new wxBoxSizer(wxHORIZONTAL);
    tsizer->Add( new wxStaticText(parent, -1, _T("Min:")), 0, wxALL | wxALIGN_CENTER_VERTICAL , 3);
    tsizer->Add(tcmin, 1, wxEXPAND|wxALL,3);
    tsizer->Add( new wxStaticText(parent, -1, _T("Max:")), 0, wxALL | wxALIGN_CENTER_VERTICAL , 3);
    tsizer->Add(tcmax, 1, wxEXPAND|wxALL,3);
    tsizer->Add(cbcolor, 1, wxEXPAND|wxALL,3);
    if(IsRefreshButton() && butt != NULL ) tsizer->Add(butt, 0, wxALL,3);
    tsizer->Add(dbutt, 0, wxALL,3);

//    dbutt->Enable( false );

    sizer->Add(tsizer, wxGBPosition(rows,1),wxDefaultSpan,wxEXPAND);
    sizer->Layout();

    cae->Tsizer = tsizer;
    cae->GBSizer = sizer;


}

void VemsGaugeDescDialog::OnDelCaItem(wxCommandEvent &evt)
{
    long id = ((wxVariant*)(evt.m_callbackUserData))->GetLong();

    ColoredAreaEdit * cae = m_items[id]->m_cae;

    ColoredArea * ca = cae->Ca;

    wxSizerItem * item = NULL;

    for(short i = 0; 1 ; i++) {
        item = cae->Tsizer->GetItem( (size_t) 0);
        if(!item)
            break;

        wxWindow * win = item->GetWindow();

        if(!win)
            continue;

        cae->Tsizer->Remove(0);
        delete win;

    }

    wxGridBagSizer * gbs = cae->GBSizer;

    wxGBSizerItem * sitem = gbs->FindItem( cae->Tsizer );

    wxGBPosition gbpos = sitem->GetPos();

    gbpos.SetCol(0);

    wxWindow * wins = gbs->FindItemAtPosition( gbpos )->GetWindow();

    gbs->Remove( wins );
    gbs->Remove( cae->Tsizer );
    delete wins;

    for(uint16_t a=gbpos.GetRow()+1; a < gbs->GetRows(); a++) {
        for(int i = 0; i < gbs->GetCols(); i++){
            wxGBSizerItem * sItem = gbs->FindItemAtPosition(wxGBPosition(a, i));
            if(sItem)
                sItem->SetPos(wxGBPosition(a-1,i));
        }
    }

    gbs->RecalcSizes();
    gbs->Layout();

    m_items.erase( id );
    delete cae;

    m_parentWidget->GetGaugeDescMgr()->ManageColoredArea( wxID_DELETE, ca );

    GetDesc()->coloredA.Remove( ca );
    delete ca;

    RefreshParent();
}

void VemsGaugeDescDialog::OnAddCaItem(wxCommandEvent &evt)
{
    Logger::logf(LOG_MSG,_T("%i"),evt.GetId());

    DescItemDesc * did = m_items[evt.GetId()];

    if(!did) return;

    ColoredArea * ca = new ColoredArea(0.0,0.0,_T(""));

    m_parentWidget->GetGaugeDescMgr()->ManageColoredArea( wxID_ADD, ca, GetDesc()->Properties[_T("id")].GetLong() );

    AddColoredArea( did->m_gdd , ca, did->m_cae->Parent, did->m_cae->GBSizer);

    GetDesc()->coloredA.Add( ca );

}

void VemsGaugeDescDialog::AddColoredAreas(GDDesc * gdd, wxWindow * parent, wxGridBagSizer * sizer)
{
    ColoredAreaArray &caa = GetDesc()->coloredA;

    int id = wxNewId();

    wxButton * butt = new wxButton(parent, id, _T("Add new area"));
    Connect(id, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(VemsGaugeDescDialog::OnAddCaItem), NULL, this);

    sizer->Add(butt, wxGBPosition(0,1), wxDefaultSpan, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
    sizer->Layout();

    m_items[id] = new DescItemDesc(new ColoredAreaEdit(sizer,parent), gdd);

    for(uint16_t i= 0; i <  caa.Count(); i++)
    {
        AddColoredArea(gdd, caa.Item(i), parent, sizer);
    }

}

void VemsGaugeDescDialog::AddLines()
{

    m_panelBook = new wxBookCtrl(this, wxID_ANY);

    GDDescArray* gdds = m_parentWidget->GetGaugeDescMgr()->GetGDDescArray();

    wxPanel * panel = NULL;
    wxGridBagSizer * sizer = NULL;

    wxString pagename = _T("");

    int gType = m_parentWidget->GetGaugeType();

    for(uint8_t a=0; a < gdds->GetCount();a++) {
        GDDesc * gdd = gdds->Item(a);

//        Logger::logf(LOG_MSG,_T("gdd->ValidGauges.GetCount=%i"),gdd->ValidGauges.GetCount());

        if( gdd->ValidGauges.GetCount() > 0 && gdd->ValidGauges.Index(gType) == wxNOT_FOUND ) continue;

        if( pagename != gdd->PageName ) {
            pagename = gdd->PageName;
            panel = new wxPanel( m_panelBook );

            m_panelBook->AddPage(panel, pagename );

            sizer = new wxGridBagSizer();
            sizer->AddGrowableCol(1,1);

            panel->SetSizer(sizer);
            sizer->Fit(panel);
            sizer->SetSizeHints(panel);
        }



        if(panel) {
            if( gdd->Type == _T("slider") )
                AddSlider( gdd , panel, sizer );
            else if( gdd->Type == _T("numeric") )
                AddNumeric( gdd , panel, sizer );
            else if( gdd->Type == _T("coloredareas") )
                AddColoredAreas( gdd , panel, sizer );
            else if( gdd->Type == _T("string") )
                AddString( gdd , panel, sizer );
        }
    }


    m_mainSizer->Add( m_panelBook, 1, wxALL | wxEXPAND, 5);

}

void VemsGaugeDescDialog::CreateDialog()
{

    Create(m_parentWidget, wxID_ANY, _T("Gauge descriptor editor"));

    m_defaultDesc = new VEMSGaugeDesc( GetDesc() );

    m_mainSizer  = new wxBoxSizer(wxVERTICAL);

    VEMSGaugeDesc * gd = GetDesc();

    m_titlePage = new wxHtmlWindow(this, wxID_ANY, wxDefaultPosition, wxSize(200,30), wxHW_SCROLLBAR_NEVER);

    SetTitleText(gd->GetName());

    m_mainSizer->Add(m_titlePage, 0, wxALL | wxEXPAND , 3);

    m_mainSizer->Add(new wxStaticLine(this),0,wxEXPAND|wxALL,2);

    AddLines();

    m_mainSizer->Add(new wxStaticLine(this),0,wxEXPAND|wxALL,2);

    VemsMessage vmsg = VemsMessageHashHelper::GetVemsMsg(_T("VemsGaugeDescDialogNote"));

    wxString msg = vmsg.getHtmlMsg();

    //delete vmsg;

    m_mainSizer->Add(new wxStaticText(this, wxID_ANY,msg),0, wxALL, 5 );

    CreateButtons();

	SetSizer(m_mainSizer);
	m_mainSizer->Fit(this);
	m_mainSizer->SetSizeHints(this);

    Connect(wxEVT_CLOSE_WINDOW, wxCloseEventHandler(VemsGaugeDescDialog::OnClose));
}

void VemsGaugeDescDialog::OnClose(wxCloseEvent& evt)
{
    ResetGauge();
    CloseEditor(wxID_CANCEL);
}

void VemsGaugeDescDialog::SetTitleText(const wxString &title)
{
    wxString bgcolor = wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE).GetAsString(wxC2S_HTML_SYNTAX);

    wxString mm = _T("<html><body leftmargin=0 topmargin=0 rightmargin=0 bottommargin=0 marginwidth=0 marginheight=0 bgcolor=") + bgcolor + _T(" >");

    m_titlePage->SetPage(mm + _T("<font size=-1>Current gauge descriptor name: </font><b><font size=+1>") + title + _T("</font></b></body></html>") );

    //m_titlePage->SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
}

void VemsGaugeDescDialog::CreateButtons()
{

    wxBoxSizer * butts = new wxBoxSizer(wxHORIZONTAL);


    CreateButton(_T("Ok"), idOkButton, butts);  /// save and exit
    CreateButton(_T("Cancel"), idCancelButton, butts); /// reset and exit
    CreateButton(_T("Reset"), idResetButton, butts); /// reset
    //CreateButton(_T("Apply"), idApplyButton, butts);  /// apply
    CreateButton(_T("Save As"), idSaveAsButton, butts);


    m_mainSizer->Add(butts,0,wxALIGN_RIGHT);

}

void VemsGaugeDescDialog::CreateButton(const wxString &title, int id, wxSizer * sizer)
{

    wxButton * b = new wxButton(this, id, title);

    Connect(id, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(VemsGaugeDescDialog::OnButton),NULL,this);

    sizer->Add(b, 0, wxALL, 2);

  //  if(id == idOkButton || id == idSaveAsButton) b->Enable(false);

}

void VemsGaugeDescDialog::CloseEditor(int retcode)
{
    if(IsModal()) {
        EndModal(retcode);
    } else {
        SetReturnCode(retcode);
        Destroy();
    }
}

void VemsGaugeDescDialog::OnButton(wxCommandEvent &evt)
{
    int id = evt.GetId();

    switch(id){

        case idApplyButton:
            UpdateModell();
        break;

        case idCancelButton:
            ResetGauge();
            CloseEditor(wxID_CANCEL);
        break;

        case idResetButton:
            ResetGauge();
        break;

        case idOkButton:
            UpdateModell(false);
            m_parentWidget->GetGaugeDescMgr()->SaveGaugeDescriptor( GetDesc() );
            RefreshParent();
            CloseEditor(wxID_OK);
        break;

        case idSaveAsButton:
            UpdateModell();
            bool m = false;
            if( GetDesc()->GetType() == _T("null") )
                m = true;

            NameDialog dlg(this, GetDesc()->GetName(), m, m_parentWidget->GetGaugeDescMgr());
            if( dlg.ShowModal() == wxID_OK) {
                SaveGaugeDesc(dlg.GetStr(),dlg.GetType());
                CloseEditor(wxID_OK);
            }

        break;

    }


}

void VemsGaugeDescDialog::SaveGaugeDesc(const wxString &name, const wxString &type)
{
    VEMSGaugeDesc * gd = new VEMSGaugeDesc( GetDesc() );

    gd->SetName( name );

    if(type != wxEmptyString) {
        gd->SetType( type );
    }

    bool res = m_parentWidget->GetGaugeDescMgr()->AddGaugeDescriptor( gd );

    if(!res) {
        delete gd;
        return;
    }

    GetDesc()->Set( m_defaultDesc );

    m_parentWidget->m_gd = gd;

    SetTitleText( gd->GetName() );


    m_parentWidget->SetGaugeDescByName( name );


    m_parentWidget->GetGaugeDescMgr()->GetGaugeGrpMgr()->SaveDesc( m_parentWidget->GetItemID(), GetDesc()->GetName() );
}

void VemsGaugeDescDialog::UpdateModell(bool refreshParent)
{

    for(IntToDescItemDesc::iterator it = m_items.begin(); it != m_items.end(); it++){
        RefreshByDesc( it->second );
    }

    if(refreshParent)
        RefreshParent();
}

void VemsGaugeDescDialog::ResetGauge()
{

    GetDesc()->Set( m_defaultDesc );
    RefreshParent();

    for(IntToDescItemDesc::iterator it = m_items.begin(); it != m_items.end(); it++){

        if( it->second->m_slider ) {

            if(it->second->m_gdd->Factor > 1)
                it->second->m_slider->SetValue( (int)(GetDesc()->Properties[it->second->m_gdd->Name].GetDouble() * it->second->m_gdd->Factor) );
            else
                it->second->m_slider->SetValue( GetDesc()->Properties[it->second->m_gdd->Name].GetLong() );

        } else if( it->second->m_textCtrl ) {

            if( it->second->m_isNum ) {
                double val = GetDesc()->Properties[it->second->m_gdd->Name].GetDouble() * it->second->m_gdd->Factor;
                wxString tf = GetDesc()->Properties[_T("numformat")].GetString();
                it->second->m_textCtrl->SetValue( wxString::Format(tf,val) );
            } else {
                wxString val = GetDesc()->Properties[it->second->m_gdd->Name].GetString();
                it->second->m_textCtrl->SetValue( val );
            }

        } else if( it->second->m_cae ) {

            if( !(it->second->m_cae->Ca) ) continue;

            ColoredArea * ca = GetDesc()->GetColoredArea( it->second->m_cae->Ca->itemID );

            if(!ca) continue;

            wxString tf = GetDesc()->Properties[_T("numformat")].GetString();

            it->second->m_cae->Ca = ca;
            it->second->m_cae->RefreshCtrl( tf );


        }

    }

}


void VemsGaugeDescDialog::ClearColoredAreasPanel()
{


}

NameDialog::NameDialog(wxWindow *parent,const wxString &str, bool isshowtypelist, GaugeDescManager * gdmgr)
{
    Create(parent, wxID_ANY, _T("Name:"));
    m_typeList = NULL;

    m_text = new wxTextCtrl(this, wxID_ANY, str);

    wxBoxSizer  * mainsizer = new wxBoxSizer(wxVERTICAL);

    mainsizer->Add(m_text,0, wxALL | wxEXPAND, 5);

    if( isshowtypelist && gdmgr ) {
        wxArrayString arr;
        gdmgr->GetTypeList(arr);
        m_typeList = new wxComboBox(this, -1, _T(""),wxDefaultPosition, wxDefaultSize, 0,0, wxCB_SORT|wxCB_READONLY );
        m_typeList->Append(arr);
        m_typeList->Select(0);
        mainsizer->AddSpacer(10);
        mainsizer->Add( new wxStaticText(this,-1,_T("The descriptor type is null!\nPlease select a type:")), 0, wxLEFT|wxRIGHT ,5 );
        mainsizer->Add( m_typeList, 0, wxALL|wxEXPAND, 5);
    }

    mainsizer->Add(CreateButtonSizer(wxOK | wxCANCEL), 0, wxALL, 5);

	SetSizer(mainsizer);
	mainsizer->Fit(this);
	mainsizer->SetSizeHints(this);

}

wxString NameDialog::GetStr()
{
    return m_text->GetValue();
}

wxString NameDialog::GetType()
{
    if(m_typeList)
        return m_typeList->GetValue();
    else
        return wxEmptyString;
}
