#include "VemsTextWidget.h"

#include <wx/dcbuffer.h>

#include "../wxStaticTextEx.h"
#include "../logger.h"
#include "../WidgetThemeTable.h"


BEGIN_EVENT_TABLE( VemsTextWidget, VemsWidgetBase )

  //  EVT_SIZE ( VemsTextWidget::OnSize )

    EVT_PAINT(VemsTextWidget::OnPaint)

END_EVENT_TABLE()

/** todo:   refactor this paint
            refactor GPSRouteGauge
            releasing apache-based list processing on webinterface panel
*/

VemsTextWidget::VemsTextWidget( wxWindow* parent, wxWindowID id, GaugeDescManager * gdmgr, GaugeProperties &prop)
{
    //ctor
    m_sideRatio = 0.3;
    m_sideRatioRestriction = 1 / m_sideRatio;

    m_face = NULL;


    CreateWidget(parent,id, gdmgr, prop);

#if !defined (__WINCE_STANDARDSDK__)
    Connect(wxEVT_SIZE, wxSizeEventHandler(VemsTextWidget::OnSize));
#endif

#if defined (__WINCE_STANDARDSDK__)
    SetWindowStyle(wxBORDER_RAISED);
#else
    SetWindowStyle(wxBORDER_SUNKEN);
#endif

//    Logger::logf(LOG_MSG,_T("prop.width=%i"),prop.width);

}

VemsTextWidget::~VemsTextWidget()
{
    //dtor
}

void VemsTextWidget::OnPaint(wxPaintEvent& evt)
{
    wxBufferedPaintDC dc(this);

    if(m_face) dc.DrawBitmap(*m_face, 0,0);

    DrawExtraLayer(dc);
}

wxRegion VemsTextWidget::GetShape()
{

    wxRect rect(0,0,GetSize().x,GetSize().y);

    rect.Inflate(1,1);

    wxRegion region(rect);

    region.Offset(-1,-2);

    return region;

}

void VemsTextWidget::DrawFace()
{
    ptrdel(m_face);
    m_face = new wxBitmap( GetClientSize().x, GetClientSize().y );

    wxMemoryDC dc(*m_face);

    dc.SetBackground( wxBrush( m_theme->BackgroundColor ) );
    dc.Clear();

    wxString title = GetSymbolTitle();
    wxString unit = m_gd->GetUnit();
    wxString value = wxString::Format(m_gd->GetGaugeNumberFormat(),m_value);

    wxSize size = GetClientSize();

    int fsize = (int)((size.x*0.1)-1);

    int fsize_limit = size.y/3;

    if(fsize > fsize_limit) {
        fsize = fsize_limit;
    }

    wxFont font = dc.GetFont();
    font.SetPointSize( fsize );

    dc.SetFont(font);

    wxSize unitsize = dc.GetTextExtent(unit);
    wxSize valuesize = dc.GetTextExtent(value);

    dc.SetTextForeground( m_theme->TitleColor );
    dc.DrawText( title, 1, 1 );

    dc.SetTextForeground( m_theme->ValueColor );
    dc.DrawText( value, size.x - valuesize.x - unitsize.x - 2, size.y - valuesize.y - 1 );

    dc.SetTextForeground( m_theme->SuffixColor );
    dc.DrawText( unit, size.x - unitsize.x - 1, size.y - unitsize.y - 1 );

    dc.SelectObject(wxNullBitmap);
}

void VemsTextWidget::UpdateSize()
{
    SetWindowStyle(wxBORDER_SUNKEN);

    DrawFace();

    Refresh(false);
}

void VemsTextWidget::setSize(int x , int y)
{

}

void VemsTextWidget::SetValue(double value)
{
//    Logger::logf(LOG_MSG,_T("VemsTextWidget::SetValue() name=") + m_currSymbolName + _T(" value=%f"),value);
    m_value = value;


    wxString tmp = _T("N/A");

    if ( isDataAvailable() ) { tmp = wxString::Format(m_gd->GetGaugeNumberFormat()+_T(""),m_value); }

    if(tmp != m_prevValue) {
        UpdateSize();
    }

    m_prevValue = tmp;

}

void VemsTextWidget::OnSize(wxSizeEvent &event)
{
    //Logger::logf(LOG_MSG,_T("VemsTextWidget::OnSize"));

    int width = event.GetSize().x;
    int height = event.GetSize().y;


    if( height > width*m_sideRatio )
        height = (int)(width*m_sideRatio);
    if( width > height/m_sideRatio )
        width = (int)(height/m_sideRatio);


    UpdateSize();

    event.Skip();
}
