#include "vemsgaugedesc.h"
#include "../logger.h"
#include "../common/typedef.h"

//WX_DEFINE_OBJARRAY(ColoredAreaArray)

#define RAD_CONST 0.01745329

/** \class VEMSGaugeDesc
  *
  * \~english
  * \brief Gauge descriptor class.
  * \details
  * Contains the all skin data for the VemsGauge, and other VemsWidgetBase-child classes use some data from this.
  *
  * \~hungarian
  * \brief Gauge leíró osztály.
  * \details
  * Tartalmazza a VemsGauge megjelenítéséhez szükséges adatokat és a többi VemsWidgetBaseből származó osztály is használ(hat) innen.
  *
  */

#include <wx/arrimpl.cpp>
WX_DEFINE_OBJARRAY(ColoredAreaRefArray);


void ColoredArea::Set( ColoredArea * ca )
{
    minValue = ca->minValue;
    maxValue = ca->maxValue;
    color = ca->color;
    ColorName = ca->ColorName;
    itemID = ca->itemID;
}

VEMSGaugeDesc::VEMSGaugeDesc()/*
    :m_title (_T("VEMS"))
    ,m_gaugeNumberFormat(_T("%.0f") )
    ,m_minValue (0)
    ,m_maxValue (1000)
    ,m_scaleFactor (1)
    ,m_majorDivisions (10)
    ,m_minorDivisions (5)
    ,m_startAngle (45+90)
    ,m_gaugeAngle (270)
    ,m_endAngle (m_startAngle + m_gaugeAngle)
    ,m_lDecimals(0)
    */
{
    SetTitle(_T("VEMS"));
    SetGaugeNumberFormat(_T("%.0f") );
    SetMinValue(0);
    SetMaxValue(1000);
    SetUnitsScaleFactor(1);
    SetMajorDivisions(10);
    SetMinorDivisions(5);
    SetStartAngle(135);
    SetEndAngle(405);
    SetLDecimals(0);
}

void VEMSGaugeDesc::Set( VEMSGaugeDesc * vgd )
{

    WX_CLEAR_ARRAY( coloredA );//coloredA.Clear();

    for(uint8_t a=0; a<vgd->coloredA.GetCount();a++)
        coloredA.Add( new ColoredArea( vgd->coloredA[a] ) );


    Properties  = vgd->Properties;

}

VEMSGaugeDesc::VEMSGaugeDesc( VEMSGaugeDesc * vgd )
{
    Set(vgd);
}

VEMSGaugeDesc::~VEMSGaugeDesc()
{
    WX_CLEAR_ARRAY( coloredA );
}

void VEMSGaugeDesc::SetSymbolName(const wxString& s)
{
    m_symbolName = s;
}

void VEMSGaugeDesc::SetTitle(const wxString& s)
{
    m_title = s;
}

void VEMSGaugeDesc::SetType(const wxString& type)
{
    Properties[_T("type")] = type;
    //m_name = name;
}


void VEMSGaugeDesc::SetName(const wxString& name)
{
    Properties[_T("name")] = name;
    //m_name = name;
}

void VEMSGaugeDesc::SetUnit(const wxString& unit)
{
    Properties[_T("unit")] = unit;
    //m_unit = unit;
}

void VEMSGaugeDesc::SetGaugeNumberFormat(const wxString& s)
{
    Properties[_T("numformat")] = s;
    //m_gaugeNumberFormat = s;
}

void VEMSGaugeDesc::SetMinValue(double v)
{
    Properties[_T("Lo")] = v;
    //m_minValue = v;
}
void VEMSGaugeDesc::SetMaxValue(double v)
{
    Properties[_T("Hi")] = v;
        //m_maxValue = v;
}

void VEMSGaugeDesc::SetMajorDivisions(long v)
{
    Properties[_T("MajorDiv")] = v;
        //m_majorDivisions = v;
}

void VEMSGaugeDesc::SetMinorDivisions(long v)
{
    Properties[_T("MinorDiv")] = v;
        //m_minorDivisions = v;
}

void VEMSGaugeDesc::SetUnitsScaleFactor(long v)
{
    if(v > 0) Properties[_T("scale")] = v;
     //m_scaleFactor = v;
}

void VEMSGaugeDesc::SetStartAngle(long v)
{
    Properties[_T("StartAngle")] = v;
    //m_startAngle = v;
  //  m_startAngleRad   = ( v ) * rad_const;
//    m_angleRad = GetGaugeAngle() * rad_const;
}

void VEMSGaugeDesc::SetEndAngle(long v)
{
    Properties[_T("EndAngle")] = v;
  //  m_angleRad = GetGaugeAngle() * rad_const;
    //m_endAngle = v;
}

void VEMSGaugeDesc::SetVDecimals(long vd)
{
    Properties[_T("VD")] = vd;
    //m_vDecimals = vd;
}

void VEMSGaugeDesc::SetLDecimals(long ld)
{
    Properties[_T("LD")] = ld;
    //m_lDecimals = ld;
}

///DESIGN
/*
void VEMSGaugeDesc::SetFaceColor(wxColor* c)
{
    m_faceColor = c;
}
void VEMSGaugeDesc::SetPointerColor(wxColor* c){
    m_pointerColor = c;
}
void VEMSGaugeDesc::SetMarkingsColor(wxColor* c){
    m_markingsColor = c;
}
void VEMSGaugeDesc::SetCenterColor(wxColor* c){
    m_centerColor= c;
}
*/

///GETTERS
wxString VEMSGaugeDesc::GetType()
{
    return Properties[_T("type")].GetString();
}

wxString VEMSGaugeDesc::GetName()
{
    //return m_name;
    return Properties[_T("name")].GetString();
}

wxString VEMSGaugeDesc::GetSymbolName()
{
    return m_symbolName;
}

wxString VEMSGaugeDesc::GetUnit(){
    //return m_unit;
    return Properties[_T("unit")].GetString();
}
wxString VEMSGaugeDesc::GetGaugeNumberFormat(){
    //return m_gaugeNumberFormat;
    return Properties[_T("numformat")].GetString();
}
double VEMSGaugeDesc::GetMinValue(){
    //return m_minValue;
    return Properties[_T("Lo")].GetDouble();
}
double VEMSGaugeDesc::GetMaxValue(){
    //return m_maxValue;
    return Properties[_T("Hi")].GetDouble();
}
int VEMSGaugeDesc::GetMajorDivisions(){
    //return m_majorDivisions;
    return Properties[_T("MajorDiv")].GetLong();
}
int VEMSGaugeDesc::GetMinorDivisions(){
    //return m_minorDivisions;
    return Properties[_T("MinorDiv")].GetLong();
}
int VEMSGaugeDesc::GetUnitsScaleFactor(){
    //return m_scaleFactor;
    return Properties[_T("scale")].GetLong();
}
int VEMSGaugeDesc::GetStartAngle(){
    //return m_startAngle;
    return Properties[_T("StartAngle")].GetLong();
}

double VEMSGaugeDesc::GetStartAngleRad(){
    //return m_startAngle;
    return GetStartAngle() * RAD_CONST;

}
int VEMSGaugeDesc::GetGaugeAngle(){
    //return m_gaugeAngle;
    //return m_endAngle - m_startAngle;
    return Properties[_T("EndAngle")].GetLong() - Properties[_T("StartAngle")].GetLong();
}
double VEMSGaugeDesc::GetGaugeAngleRad(){
    return GetGaugeAngle() * RAD_CONST;
}
int VEMSGaugeDesc::GetEndAngle(){
    //return m_endAngle;
    return Properties[_T("EndAngle")].GetLong();
}
int VEMSGaugeDesc::GetTotalDivisions(){

    //return m_majorDivisions * m_minorDivisions;

    return Properties[_T("MajorDiv")].GetLong() * Properties[_T("MinorDiv")].GetLong();
}

int VEMSGaugeDesc::GetVDecimals()
{
    //return m_vDecimals;
    return Properties[_T("VD")].GetLong();
}

int VEMSGaugeDesc::GetLDecimals()
{
    //return m_lDecimals;
    return Properties[_T("LD")].GetLong();
}


int VEMSGaugeDesc::GetTitleAngle()
{
    return Properties[_T("TitleAngle")].GetLong();
}

int VEMSGaugeDesc::GetTitlePos()
{
    return Properties[_T("TitlePos")].GetLong();
}

int VEMSGaugeDesc::GetUnitAngle()
{
    return Properties[_T("UnitAngle")].GetLong();
}

int VEMSGaugeDesc::GetUnitPos()
{
    return Properties[_T("UnitPos")].GetLong();
}

int VEMSGaugeDesc::GetValAngle()
{
    return Properties[_T("ValueAngle")].GetLong();
}

int VEMSGaugeDesc::GetValPos()
{
    return Properties[_T("ValuePos")].GetLong();
}


void VEMSGaugeDesc::SetGaugeNumbers()
{
    //(7000-1000)/100
    m_gaugeNumbers.Clear();
    double range = (GetMaxValue()-GetMinValue()) / (double)GetUnitsScaleFactor();
    double numberInterval = range/(double)GetMajorDivisions();

    double i = GetMinValue()/(double) GetUnitsScaleFactor();
    double max = GetMaxValue()/GetUnitsScaleFactor();
    wxString format = wxString::Format( _T("%if"),GetLDecimals());
	while(i <= max){
        wxString s = wxString::Format(_T("%.")+format, i);
		m_gaugeNumbers.Add(s);
//		fprintf(stdout, "VEMSGaugeDesc::SetGaugeNumbers() %f\n", i);
//        Logger::logf(LOG_MSG, _T("VEMSGaugeDesc::SetGaugeNumbers(): format: ")+ format);
        i += numberInterval;

    }
    /*
	for (int i=0; i < range+1; i=i+numberInterval) {
		wxString s;
		s.Printf(_T("%i"),i);
		m_gaugeNumbers.Add(s);
	}*/
}

wxString VEMSGaugeDesc::toString()
{
    wxString str = wxString::Format(_T("Name : %s \n")
                            _T("Symbol Name : %s \n")
                            _T("Min Value : %f \n")
                            _T("Max Value : %f \n")
                            _T("Scale factor: %i \n")
                            _T("Major divisions: %i \n")
                            ,GetName().c_str()
                            ,GetSymbolName().c_str()
                            ,GetMinValue()
                            ,GetMaxValue()
                            ,GetUnitsScaleFactor()
                            ,GetMajorDivisions());

    for(size_t i=0; i<coloredA.Count(); i++){
        ColoredArea* ca = coloredA[i];
        str += ca->toString() + _T("\n");
    }

    return str;
}

wxString VEMSGaugeDesc::GetFormat()
{

	wxString format = _T("%.0f");

	double maxV = GetMaxValue();
	double minV = GetMinValue();

	double range = maxV - minV;

	if(range > 1000){
            format = _T("%.0f");
    }else if(range > 100){
            format = _T("%3.1f");
    }else if(range > 10){
            format = _T("%2.2f");
    }else if(range > 1){
            format = _T("%1.2f");
    }else if(range > 0.1){
            format = _T("%1.2f");
    }

    return format;
}


int VEMSGaugeDesc::compare_function(VEMSGaugeDesc **A,VEMSGaugeDesc **B)
{
    wxString nameA = (*A)->GetName();
    wxString nameB = (*B)->GetName();

    return nameA.CmpNoCase(nameB);
}

ColoredArea * VEMSGaugeDesc::GetColoredArea(int itemID)
{
    for( size_t a=0; a<coloredA.GetCount(); a++ ) {
        if( coloredA[a]->itemID != -1 && coloredA[a]->itemID == itemID )
            return coloredA[a];
    }

    return NULL;
}
