#include "vemsprotocol.h"

#include <wx/arrstr.h>
#include <wx/datetime.h>

#include "CommunicationBase.h"

#include "vemsprotocol/ECUStateMgrBase.h"
#include "vemsprotocol/ECUInfo.h"
#include "vemsprotocol/SerialNumber.h"

#include "logger.h"
#include "UnicodeHelper.h"
#include "RealTimeDesc.h"
#include "events/AIMEvent.h"
#include "SerialException.h"
//#include "vemsprotocol/Sipr.h"

VemsProtocol::VemsProtocol( ECUStateMgrBase* mgrBase )
{

#if !defined (__WINCE_STANDARDSDK__)
    Logger::logf( LOG_MSG, _T("START VemsProtocol ctor!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! \n"));
#endif

    m_AimChannelOffsetHash[1] = 0;  /// RPM
    m_AimChannelOffsetHash[17] = 8; ///
    m_AimChannelOffsetHash[33] = 12;
    m_AimChannelOffsetHash[45] = 14; /// TPS

    m_AimChannelOffsetHash[69] = 16;
    m_AimChannelOffsetHash[101] = 20;
    m_AimChannelOffsetHash[105] = 22;

/*
    m_AimChannelOffsetHash[97] = 8;
    m_AimChannelOffsetHash[17] = 8;
    m_AimChannelOffsetHash[105] = 8;
*/
    m_commBase = mgrBase->getCommunicationBase();
    //m_sipr = new Sipr( NULL, SiprSensorType::SIPR_RT_MT_EXT, NULL );

    m_ECUStateMgr = mgrBase;
    Logger::logf( LOG_MSG, _T("END VemsProtocol ctor!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! \n"));
}


VemsProtocol::~VemsProtocol()
{
//     fprintf( stdout, "START VemsProtocol dtor!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! \n");
    //dtor
//    close();
//    fprintf(stdout, "VemsProtocol dtor 2\n");
    //delete sp;
   // fprintf(stdout, "VemsProtocol dtor 3\n");
//delete m_ECUStateMgr;
}


void VemsProtocol::close()
{
    m_state = -1;
    if(m_commBase != NULL){
//        fprintf(stdout, "VemsProtocol close\n");
        m_commBase->close();
        //delete sp;
    }else{
//        fprintf(stdout, "VemsProtocol::close() sp is NULL!!!!!!!!!!!!!!\n");
    }
}


int VemsProtocol::setState( int state )
{
    return m_ECUStateMgr->setState( state );
}


int VemsProtocol::getDevice()
{
     return m_ECUStateMgr->getDevice();
}

int VemsProtocol::getState()
{
    return m_ECUStateMgr->getState();
}

int VemsProtocol::getStoredState()
{
    return m_ECUStateMgr->getStoredState();
}

int VemsProtocol::clear()
{
    return m_commBase->clear();
}


int VemsProtocol::sendString(const wxString& str)
{
    return m_commBase->SendString( (uint8_t*) UnicodeHelper::U2C(str) );
}

int VemsProtocol::sendString(const char * value)
{
    return m_commBase->SendString((uint8_t*) value);
}

int VemsProtocol::sendString(uint8_t* value, size_t len)
{
    return m_commBase->SendString(value, len);
}

int VemsProtocol::sendStringBlock(uint8_t* value, size_t len, size_t blockLen)
{
    return m_commBase->sendStringBlock(value, len, blockLen);
}

void VemsProtocol::setBaudRate( int baudRate )
{
    m_commBase->setBaudRate( baudRate );
}


int VemsProtocol::sendArray(ByteArray& array, int blockCount)
{
    size_t size = array.Count();
    if(size < 1) return 0;

    uint8_t *block = new uint8_t[blockCount];
    uint8_t bufferIndex=0;
    int res=0;

    for(uint32_t i = 0; i< array.Count(); i++){
       // vp->sendString((char *)line.c_str());
        uint8_t value = array.Item(i);
        bufferIndex = i % blockCount;  //15%16->15, 16%16->0, 17%16->1
        block[bufferIndex] = value;
        if( bufferIndex == blockCount-1){
            res = sendString( block, blockCount );
            if(res < 0){
				delete[] block;
				return res;
			}
        }
     }

     if(bufferIndex != blockCount-1){
        res = sendString( block, bufferIndex + 1 );
        if(res < 0){
				delete[] block;
				return res;
			}
     }

/*
    for(uint32_t i = 0; i< array.Count(); i++){
       // vp->sendString((char *)line.c_str());
        uint8_t value = array.Item(i);
        sendString(  (const char *) &value, 1 );

    }
*/
	delete[] block;
    return 0;
}

/*
int VemsProtocol::sendPageBuff(struct pagebuffer_t * pageBuff)
{
    unsigned char * buf;
    buf =  pageBuff->buf;

    sendString(buf, PAGESIZEBYTES);



}
*/
int VemsProtocol::readString(uint8_t* str, size_t len )
{
    return m_commBase->ReadString(str, len);
}

int VemsProtocol::readString(uint8_t* str, size_t len, int tryCount )
{
    return m_commBase->ReadString(str, len, tryCount);
}

int VemsProtocol::sendNumber(uint32_t number, int length)
{
    return m_commBase->sendNumber(number, length);
}

int VemsProtocol::sendNumber(uint16_t number)
{
    return m_commBase->sendNumber(number);
}

/*
int VemsProtocol::send(uint32_t number)
{
    uint8_t str[4];

    srt[0]


}
*/

/*
int VemsProtocol::ReadUntilEOS(char ** readbuf, char * eosString , long timeout_in_ms)

{
   // char* receivedBytes = NULL;
   // return m_commBase->ReadUntilEOS(readbuf, eosString, timeout_in_ms);

}
*/
/*
int VemsProtocol::getRealTimeValue(SymbolTest* symbol)
{
   int offset = symbol->GetOffset();
   int size = symbol->GetSize();

  if( getRuntimeData(realTimeBuff) < 0){
        Helper::Message(_T("Runtimedata disabled"));
        RealTimeDesc::SetDisabled(true);
  }
    //realTimeBuff[offset] = 10;
    //realTimeBuff[offset+1] = 1;

    wxString log = wxString::Format(_T("VemsProtocol::getRealTimeValue() size=%d"), size);
    Logger::log(LOG_MSG, log);
  switch(size){
        case 1:
            return (unsigned char)realTimeBuff[offset];
        case 2:
            return 256*(unsigned char)realTimeBuff[offset] +  (unsigned char)realTimeBuff[offset+1];
        default:
            return -1;
  }
}
*/

bool VemsProtocol::isAimData()
{
    int len = 10;
    uint8_t buff[16];
//    if(m_commBase->getBaudRate() != 9600){m_commBase->setBaudRate( 9600 );}

    //int result = m_commBase->ReadString(buff, len);
    m_commBase->ReadString(buff, len);

    return isAimData(buff, len);
}

bool VemsProtocol::isAimData(uint8_t* buff, int len)
{
    int result;
    for(int i=0; i< len; i++){
        result = m_aimParser.feed( buff[i] );
        //Logger::logf(LOG_MSG, _T("VemsProtocol::isAimData() feed result2: %02X"), buff[i]);
        if(result == AIMParser::END){
            return true;
        }
    }
    return false;
}

int VemsProtocol::getAimOffset(uint8_t channel)
{
    if( m_AimChannelOffsetHash.find(channel) == m_AimChannelOffsetHash.end() ) return 128;
    return m_AimChannelOffsetHash[channel];
}

/*
int VemsProtocol::fillAimBuff(char* buff, uint8_t channel)
{
    int offset = getAimOffset(channel);
    buff[offset] = m_aimParser.data.high;
    buff[offset+1] = m_aimParser.data.low;

    m_AimChannelHash[channel] = 256 * m_aimParser.data.high + m_aimParser.data.low;
}
*/

int VemsProtocol::fillAimBuff(uint8_t* buff)
{
    Int2IntHash::iterator it;

    for(it = m_AimChannelHash.begin(); it != m_AimChannelHash.end(); ++it){
        int channel = it->first;
        int offset = getAimOffset(channel);

        buff[offset] = (m_AimChannelHash[channel] >> 8) & 255;
        buff[offset+1] = m_AimChannelHash[channel] & 255;

        //Logger::logf(LOG_MSG, _T("VemsProtocol::fillAimBuff() channel=%i, offset=%i, buff=%i"), channel, offset, 256*  buff[offset] + buff[offset+1]);
    }

	return 0;
}

int VemsProtocol::fillAimChannelHash(uint8_t channel)
{
    //int offset = getAimOffset(channel);
    m_AimChannelHash[channel] = 256 * m_aimParser.data.high + m_aimParser.data.low;
	return 0;
}

int VemsProtocol::getAimRecord(uint8_t* buff, int len)
{
    int tryCount = 0;
    int MAX_COUNT = 16;
    int aimOffset = 0;
    int byteRead=0;

    do{
        int result = m_commBase->ReadString(&(buff[aimOffset]), 5-aimOffset);
        if(result <0) return result;

        byteRead+= result;
        if(byteRead != 5){
            aimOffset += result;
            tryCount++;
            //Logger::logf(LOG_MSG, _T("VemsProtocol::getAimData2() len=%i"), result);
        }
    }while(byteRead != 5 && (tryCount < MAX_COUNT));

    return 0;
}

int VemsProtocol::getAimData2(uint8_t* buff)
{
    uint8_t c[5];
    int tryCount = 0;
    int MAX_COUNT = 4;

     //Logger::logf(LOG_MSG, _T("VemsProtocol::getAimData2() START"));
    do{
        getAimRecord(c, 5);

        int result;
        result = m_aimParser.syncRecord(c);
        if(result == AIMParser::SYNC_OK){
            int channel = m_aimParser.data.channel;
            //Logger::logf(LOG_MSG, _T("VemsProtocol::getAimData2() SYN OK channel=%i data=%i"), channel, 256 * m_aimParser.data.high + m_aimParser.data.low );
            if( m_AimChannelHash.find(channel) ==  m_AimChannelHash.end() ){
                fillAimChannelHash(channel);
                tryCount = 0;
            }else{
                fillAimBuff(buff);
                m_AimChannelHash.clear();
                fillAimChannelHash(channel); // save the last
                //Logger::logf(LOG_MSG, _T("VemsProtocol::getAimData2() -----"));
                return ECUInfo::AIM;
            }

        }else{
            //Logger::logf(LOG_MSG, _T("VemsProtocol::getAimData2() result=%i"), result);
            result = m_commBase->ReadString(c, result);
            tryCount++;
        }
    }while(tryCount < MAX_COUNT);

    return ECUInfo::UNKNOWN;
}

//CHANNEL, A3,HIGH DATA, LOW DATA, SUMM

int VemsProtocol::getAimData(char* buff)
{
//    if(m_commBase->getBaudRate() != 9600){m_commBase->setBaudRate( 9600 );}
    uint8_t c;
    uint8_t offset;
    int result = m_commBase->ReadString(&c, 1);
    if( result < 0){ //comm error
        return AIMEvent::NOT_CONNECTED;
    }else if(result == 0){
        return AIMEvent::NO_AIM_DATA;
    }else if(result == 1){
         //Logger::logf(LOG_MSG, _T("VemsProtocol::getAimData() feed result1: %i, %02X"), result, (uint8_t)c);
        result = m_aimParser.feed((uint8_t) c);
        //Logger::logf(LOG_MSG, _T("VemsProtocol::getAimData() feed result2: %i, %X"), result, (uint8_t)c);
        if(result == AIMParser::END){
            switch( m_aimParser.data.channel){
                case 1:
                    offset = 0;
                    break;
                case 33:
                    Logger::logf(LOG_MSG, _T("VemsProtocol::getAimData() BAT"));
                    offset = 6;
                    break;
                case 45:
                    Logger::logf(LOG_MSG, _T("VemsProtocol::getAimData() TPS %i"), 256 * m_aimParser.data.high + m_aimParser.data.low);
                    offset = 8;
                    break;
                default:
                    offset = 8;
                    break;
            }
            buff[offset] = m_aimParser.data.high;
            buff[offset+1] = m_aimParser.data.low;
            return AIMEvent::CONNECTED;   //DATA_FINISH   step the ringbuffer
        }else if(result == AIMParser::CHECKSUM_ERROR)
        {
            Logger::logf(LOG_MSG, _T("VemsProtocol::getAimData() CHECKSUM_ERROR"));
        }

        return AIMEvent::AIM_DATA;   //DATA_RECIEVED
    }

    return AIMEvent::NOT_CONNECTED;
}

/**


*/

int VemsProtocol::getECUData(uint8_t* buff, bool sync)
{
    if(sync){
        int MAX_TRY_COUNT = 1;
        int try_count = 0;
        do{
            //Logger::logf(LOG_MSG, _T("----VemsProtocol::getECUData()") );
            m_state = getState();
            //clearSerialBufferCircle(4,64);
            Logger::logf(LOG_MSG, _T("VemsProtocol::getECUData(): SYNC state: ")+ ECUInfo::getStateAsString( m_state) );
            switch(m_state){
                case ECUInfo::BOOT:
                    return ECUInfo::BOOT;
                /* case ECUInfo::MEGATUNE:
                    //setState(ECUInfo::MAN);
                    //m_state = m_ECUStateMgr->setState( ECUInfo::MEGATUNE, ECUInfo::MAN );
                    m_state = m_ECUStateMgr->setState( ECUInfo::MAN );
                    //clearSerialBufferCircle(4,64);
                */
                case ECUInfo::AIM:
                     //m_state = m_ECUStateMgr->setState( ECUInfo::AIM, ECUInfo::MEGATUNE );
                     //m_state = getState();
                    //syncRTDataLength(buff);
                    break;
                case ECUInfo::MAN:
                    m_state = m_ECUStateMgr->setState( ECUInfo::MAN, ECUInfo::MEGATUNE );
                case ECUInfo::MEGATUNE:
                    syncRTDataLength(buff);
                    break;
                case ECUInfo::SIPR:
                    Logger::logf(LOG_MSG, _T("VemsProtocol::getECUData() SIPR!!!!!"));
            }
            Logger::logf(LOG_MSG, _T("VemsProtocol::getECUData(): speed: %i"), m_ECUStateMgr->getBaudRate() );
            try_count++;
        }while(try_count < MAX_TRY_COUNT);

        if( (m_state != ECUInfo::MEGATUNE) && (m_state != ECUInfo::AIM) && (m_state != ECUInfo::SIPR)) return m_state;
    }

    //Logger::logf(LOG_MSG, _T("VemsProtocol::getECUData() -------------") );
    int state;
    switch( m_state){
        case ECUInfo::MEGATUNE:
            state = getRTData(buff);
            break;
        case ECUInfo::AIM:
            state = getAimData2(buff);
            break;
        case ECUInfo::SIPR:
            state = getSIPR(buff);
            break;
        default:
            state = ECUInfo::UNKNOWN;
    }
    if(m_state != state){
         m_state = getState();
    }
    //Logger::logf(LOG_MSG, _T("VemsProtocol::getECUData() m_state =%i, state = %i"),m_state, state );
    return m_state;
}

void VemsProtocol::syncRTDataLength(uint8_t* buff)
{
    //clearSerialBufferCircle(4,64);
    int tryCount =0;
    int maxTryCount = 10;
    uint8_t* command = RealTimeDesc::getCommand();
   // int MAX_TRY_COUNT = RealTimeDesc::MAX_TRY_COUNT;
   int RUNTIME_LEN;
   int result;
   do{
        RUNTIME_LEN = RealTimeDesc::getDataSize();
        result = getRTData(command, RUNTIME_LEN+10, buff);
        RealTimeDesc::SetDataSize(result);
        tryCount++;
   }while((RUNTIME_LEN != result) && (tryCount < maxTryCount) );
    Logger::logf(LOG_MSG, _T(" VemsProtocol::syncRTDataLength(): length=%i"), result );
}

int VemsProtocol::getRTData(uint8_t* command, int len, uint8_t* buff)
{
//      Logger::logf(LOG_MSG, _T(" VemsProtocol::getRTData(): start read=%i"), len );
//      Logger::logMilli(LOG_MSG, _T(" VemsProtocol::getRTData(): end write") );
//      Logger::logMilli(LOG_MSG, _T(" VemsProtocol::getRTData(): end write") );
        int result = m_commBase->SendString(command, strlen((const char*)command) );
//      Logger::logMilli(LOG_MSG, _T(" VemsProtocol::getRTData(): read 1") );
        result = m_commBase->ReadString(buff, len, 16);
//      Logger::logMilli(LOG_MSG, _T(" VemsProtocol::getRTData(): end read") );
//      fprintf(stdout, "VemsProtocol::getRTData() Received: %d, expected: %d\n", result, RUNTIME_LEN);
//      Logger::logf(LOG_MSG, _T(" VemsProtocol::getRTData(): START %i, %i"), bRate, result );
        return result;
}

int VemsProtocol::getRTData(uint8_t* buff)
{
    uint8_t* command = RealTimeDesc::getCommand();
    int MAX_TRY_COUNT = 10;//RealTimeDesc::MAX_TRY_COUNT;
    int RUNTIME_LEN = RealTimeDesc::getDataSize();
    //wxDateTime now;
    int try_count = 0;
    int result = getRTData(command, RUNTIME_LEN, buff);
    //Logger::logf(LOG_MSG, _T("VemsProtocol::getRTData(): speed: %i"), m_ECUStateMgr->getBaudRate() );
    do{
        if( result == RUNTIME_LEN){ //GOOD
             // runtimecommand returned expected number of bytes
//            Helper::setOffLine(false);
            if( getDevice() == ECUInfo::DEVICE_ROUND){ m_commBase->ReadString(m_buffer, 4, 2); } /// ROUND BUG!!!!!!!!!!!
            return ECUInfo::MEGATUNE;
        }else if(result == 0){
            Logger::logf(LOG_MSG, _T("VemsProtocol::getRTData(): NOT SYNC (RUNTIME_LEN=%i,%i)"),RUNTIME_LEN, result );
            try_count ++;
        }else if( result < 0){ //comm error
//... give som error message and go to offline
//            Helper::setOffLine(true);
            return result;
        }else{ // if(result > RUNTIME_LEN || result < RUNTIME_LEN){ //SYNC OUT
            //... try to make syn
            //int state = getState();
            //if(state != ECUInfo::MEGATUNE)
            Logger::logf(LOG_MSG, _T("VemsProtocol::getRTData(): NOT SYNC (RUNTIME_LEN=%i,%i)"),RUNTIME_LEN, result );
            //fprintf(stderr, "BUFF: %s\n", buff);
            try_count ++;
            //clear();
        }

    }while(try_count < MAX_TRY_COUNT);
    // try to reset the ecu
    throw SerialException(SP_CANNOT_INIT_ECU);

}

int VemsProtocol::getSIPR(uint8_t* buff)
{
/*    uint8_t dataCount = 1;
    uint8_t delay =0;
    uint8_t pageType = 0;*/
    //bool res = m_sipr->readSensors(dataCount, delay, pageType, buff);
//    if(res) return ECUInfo::SIPR;
    return ECUInfo::UNKNOWN;
}

/*
int VemsProtocol::getAimData(uint8_t* buff)
{
    m_AimChannelHash.Clear();
    uint16_t data;
    uint8_t channel;
    do{
        channel = getAimChannelData(&data);
        if(channel  >= 0){
            m_AimChannelHash[channel] = 1;
        }
    }while(m_AimChannelHash.find(channel) == m_AimChannelHash.end());

}
*/
int VemsProtocol::getRuntimeData(uint8_t* buff, bool syncDataSize)
{

    if(RealTimeDesc::isDisabled() ) return -1; // in case verify, upload, etc

    //char a[1];
    //a[0] = 'A';
     //char runtimeData[100];

    uint8_t* command = RealTimeDesc::getCommand();
    int MAX_TRY_COUNT = RealTimeDesc::MAX_TRY_COUNT;
    int RUNTIME_LEN = RealTimeDesc::getDataSize();
     wxDateTime now;
    int try_count = 0;
    do{

        Logger::logf(LOG_MSG, _T(" VemsProtocol::getRuntimeData(): start write") );

  //       Logger::logMilli(LOG_MSG, _T(" VemsProtocol::getRuntimeData(): end write") );
//        Logger::logMilli(LOG_MSG, _T(" VemsProtocol::getRuntimeData(): end write") );
        int len = RUNTIME_LEN;
        if(syncDataSize){
//             if(m_commBase->getBaudRate() != 9600){m_commBase->setBaudRate( 9600 );}
            len += 10; //try to read more then expected in case of sync!
            //clearSerialBufferCircle(4,5);
        }
        int bRate = m_commBase->getBaudRate();

        int result = m_commBase->SendString(command, strlen((const char*)command) );
    //     Logger::logMilli(LOG_MSG, _T(" VemsProtocol::getRuntimeData(): read 1") );
        result = m_commBase->ReadString(buff, len +1);
 //       Logger::logMilli(LOG_MSG, _T(" VemsProtocol::getRuntimeData(): end read") );
        //fprintf(stdout, "VemsProtocol::getRuntimeData() Received: %d, expected: %d\n", result, RUNTIME_LEN);
        Logger::logf(LOG_MSG, _T(" VemsProtocol::getRuntimeData(): START %i, %i"), bRate, result );
        if( result == RUNTIME_LEN){ //GOOD
             // runtimecommand returned expected number of bytes
//            Helper::setOffLine(false);
            return 0; //OK
        }else if(result == 0){

            int currState = getState();
            if(currState == ECUInfo::BOOT){
                return ECUInfo::BOOT;
            }else{
                setState(ECUInfo::MEGATUNE);
                try_count ++;
            }
        }else if( result < 0){ //comm error
            //... give som error message and go to offline
//            Helper::setOffLine(true);
            return result;
        }else{ // if(result > RUNTIME_LEN || result < RUNTIME_LEN){ //SYNC OUT
            //... try to make syn
            //int state = getState();
            //if(state != ECUInfo::MEGATUNE)
            Logger::logf(LOG_MSG, _T("VemsProtocol::getRuntimeData(): NOT SYNC") );
            if( (result > 20) && isAimData((uint8_t*)buff, 20) ){
                Logger::logf(LOG_MSG, _T("VemsProtocol::getRuntimeData(): AIM") );
                return ECUInfo::AIM;
            }

            try_count ++;
            if(syncDataSize){
                //setState(ECUInfo::MEGATUNE);
                //try to guess the data size in
                //if((result > 40) && (result < 75)){
                    RealTimeDesc::SetDataSize(result);
                //}
            }else{
               clear();
            }
        }

    }while(try_count < MAX_TRY_COUNT);

    //if(try_count == MAX_TRY_COUNT){
        // try to reset the ecu
    throw SerialException(SP_CANNOT_INIT_ECU);
    //}

}

wxString VemsProtocol::getErrorMessage(int errorCode)
{
    // return sp->getErrorMessage(errorCode);
    return wxEmptyString;
}

/*
void VemsProtocol::fetchRoundFirmwareInfo()
{

 try{
        wxString rawResponse;
        getAnswer(command_serNum, 12, 12, 5, &rawResponse);
        SerialNumber roundSerialNumer(rawResponse);
        Logger::logf(LOG_MSG,_T("VemsProtocol::fetchRoundFirmwareInfo(): ") + rawResponse);
        Logger::logf(LOG_MSG,_T("VemsProtocol::fetchRoundFirmwareInfo(): ") + roundSerialNumer.getserialNumberHex() );
        m_fwInfo.setSerialNumber(roundSerialNumer.getSerialNumberStr());
    }catch(SerialException& se){
        //return &m_fwInfo;
    }

}
*/




