#ifndef VEMSPROTOCOL_H
#define VEMSPROTOCOL_H

#include "common/types/ByteArray.h"
#include "common/typedef.h"

#include "AIMParser.h"


class CommunicationBase;
class ECUStateMgrBase;
//class Sipr;

WX_DECLARE_HASH_MAP( int, wxString, wxIntegerHash, wxIntegerEqual, IntHash );

class VemsProtocol //: public CommunicationBase
{
    public:
        VemsProtocol( ECUStateMgrBase* mgrBase );
        VemsProtocol();
        virtual ~VemsProtocol();
        /// new functions
        wxString getErrorMessage(int errorCode);
        int getECUData(uint8_t* buff, bool sync);


        ///directly delegated to ECUStateMgrBase
        int getStoredState();
        int getDevice();
        int setState(int state);
        int getState(void);

        //int generalRead(char *buf, int len, const char * command, int state= ECUInfo::MAN ) throw (SerialException);
        ///directly delegated to CommunicationBase
        void close();
        int sendNumber(uint16_t number);
        int sendNumber(uint32_t number, int length);
        int sendString(const char* value);
        int sendString(uint8_t* value, size_t len);
        int sendString(const wxString& str);
        int sendStringBlock(uint8_t* value, size_t len, size_t blockLen);
        //int sendArray(wxArrayString& array);
        int sendArray(ByteArray& array, int blockCount = 16);
        //int sendConfig(wxArrayString& array);
        int readString(uint8_t* str, size_t len );
        int readString(uint8_t* str, size_t len, int tryCount );
//        int ReadUntilEOS(char** readbuf, char* eosString , long timeout_in_ms);
        int clear();
        void setBaudRate( int baudRate );
        CommunicationBase* getCommunicationBase(){ return m_commBase; }
        //void print_buffer(int cnt, char *buf);
        //MISC
    protected:
        CommunicationBase* m_commBase;
        //int  stateBrate[4];
        //char realTimeBuff[255];
        uint8_t m_buffer[255];

        int fillAimBuff(uint8_t* buff);
        int fillAimChannelHash(uint8_t channel);
        int getAimData2(uint8_t* buff);
        int getAimOffset(uint8_t channel);
        int getAimRecord(uint8_t* buff, int len);
        int getAimData(char* buff);
        int getRuntimeData(uint8_t* buff, bool syncDataSize);
        int getSIPR(uint8_t* buff);
    private:
        Int2IntHash         m_AimChannelHash;
        Int2IntHash         m_AimChannelOffsetHash;
        ECUStateMgrBase*    m_ECUStateMgr;
        AIMParser           m_aimParser;
        //Sipr*               m_sipr;
        int                 m_state;

        int getRTData(uint8_t* command, int len, uint8_t* buff);
        int getRTData(uint8_t* buff);
        bool isAimData();
        bool isAimData(uint8_t* buff, int len);
        void syncRTDataLength(uint8_t* buff);

};

#endif // VEMSPROTOCOL_H

