#include "AIMDeviceController.h"

#include "../events/InputEvent.h"
#include "../events/CommunicationEvent.h"
#include "../communication/SerialPortHandlerFactory.h"
#include "../CommunicationBase.h"
#include "../communication/CommunicationController2.h"
//#include "../communication/CommunicationControllerEvent.h"
#include "../communication/clients/AIMDetector.h"

#include "../logger.h"

AIMDeviceController::AIMDeviceController( const VemsDeviceControllerConfig& config, wxEvtHandler* handler )
{
    //ctor


    m_communicationClient       = NULL;
    m_prevCommunicationClient   = NULL;
    m_config                    = config;
    m_commBase                  = NULL;
    m_clientHandler             = NULL;


    m_handler = handler;

    m_isStopped = true;

}

AIMDeviceController::~AIMDeviceController()
{
    //m_controller->stop();
    if ( m_isStopped == false )
        stop();
}

void AIMDeviceController::start()
{
    initCommunication();
}

void AIMDeviceController::stop()
{
    m_isStopped = true;
    m_controller->stop();
    clearCommBase();
}

void AIMDeviceController::reset(VemsDeviceControllerConfig& config)
{
    m_config = config;
    start();
}

void AIMDeviceController::clearCommBase()
{
    if ( m_commBase ) {
        m_commBase->clear();
        m_commBase->close();
        delete m_commBase;
        m_commBase = NULL;
    }
}

void AIMDeviceController::SendSIPRPacket(SIPRPacket& packet)
{
    if(!m_clientHandler) return;

    InputEvent evt;
    evt.setType(SIPR_EVENT_TYPE_OUTPUT);
    evt.setPacket(packet);

    m_clientHandler->ProcessEvent(evt);
}

void AIMDeviceController::initCommunication()
{
    clearCommBase();

    if ( m_config.mode.IsSameAs( _T("SERIAL") ) )
    {

		m_deviceInfo.serialConfig.setPortName( m_config.port );
		m_deviceInfo.serialConfig.setBaudRate( m_config.baudrate );

        m_commBase = SerialPortHandlerFactory::create(&m_deviceInfo.serialConfig);
        LOGP( wxString::Format( _T("Create SerialPort! - port: ") + m_config.port + _T(", baudrate: %d"), m_config.baudrate ) );
    }
//    else
//    {
//        wxString host = m_config.ip.BeforeFirst( _T(':') );
//        long port = 0;
//        m_config.ip.AfterLast( _T(':') ).ToLong( &port );
//
//        m_commBase =  SerialPortHandlerFactory::create( host, port );
//
//        LOGP( wxString::Format( _T("Create TCPClientHandler! - ip: ") + host + _T(", port: %d"), port ) );
//    }

    m_isStopped = false;

    AIMDetector* aimdec = new AIMDetector( m_handler ? m_handler : this );

    m_clientHandler = aimdec;

    m_communicationClient = aimdec;

    m_controller = new CommunicationController2( m_handler ? m_handler : this, m_communicationClient, m_commBase );

    m_controller->start();
}
