#include "BootModeRound.h"

#include <wx/thread.h>

#include "../logger.h"
#include "../CommunicationBase.h"


BootModeRound::BootModeRound(CommunicationBase*  commBase)
:BootModeBase(commBase)
{
    //ctor
    m_type = FirmwareData::CHEX;
    m_device = ECUInfo::DEVICE_ROUND;
}

BootModeRound::~BootModeRound()
{
    //dtor
}

uint32_t BootModeRound::getPageID(uint32_t pageIdx)
{
    uint32_t addressW = m_firmwareData->getPageAddressW(pageIdx);  ///!! WORD ADDRESS
    return addressW;
}

int BootModeRound::selectPageTMP(uint32_t idx)
{
    selectPage("A", idx);
    return 0;
}

int BootModeRound::verifyPage(uint32_t idx)
{
    selectPageTMP(idx);
    return verifyPageCommon("DCC", idx);
}

bool BootModeRound::checksumVerify(uint32_t ecu_checksum, uint32_t file_checksum ){

    Logger::logf(LOG_MSG, _T("BootModeRound::checksumVerify() START"));

	uint16_t received_chk_high = ecu_checksum >> 16;
	uint16_t chk_hexfile_high = file_checksum >> 16;

	return ( received_chk_high == chk_hexfile_high );
//	if( received_chk_high != chk_hexfile_high){
//		return false;
//	}
//	return true;
}

int BootModeRound::uploadPage(uint32_t idx)
{
    selectPageTMP(idx);
    return uploadPageCommon("Dcc", idx);
}

int BootModeRound::verifyFlash()
{
   int error_cnt = verifyFlashCommon();
    // NTC ?????
    if(error_cnt == 0){
        m_commBase->SendString((uint8_t*)"E");
    }
    return error_cnt;
}

int BootModeRound::uploadFlash()
{
    Logger::logf(LOG_MSG, _T("BootModeRound::uploadFlash() START"));
    int error_cnt = uploadFlashCommon(0);
    if(error_cnt == 0){
        m_commBase->SendString((uint8_t*)"E");
    }else{
          observerNotify(_T("Error detected ECU left in BOOT Mode!"));
    }

    return error_cnt;
}

bool BootModeRound::failSafeVerifyPage(uint32_t idx)
{
    return failSafePageHandle(idx, &BootModeBase::verifyPage );
}

bool BootModeRound::failSafeUploadPage(uint32_t idx)
{
    return failSafePageHandle(idx, &BootModeBase::uploadPage );
}

////////////////////////

////////////////////////
int BootModeRound::eraseFlash()
{
    return 0;
}
