#ifndef DEVICEINFO_H
#define DEVICEINFO_H


#include "ECUInfo.h"
#include "../FirmwareInfo.h"
#include "../serialconfig.h"


struct DeviceInfo
{
    DeviceInfo()
    {
        device      = ECUInfo::DEVICE_UNKNOWN;
        deviceMode  = ECUInfo::UNKNOWN;
    }

    void setPortName( const wxString& portName ) { serialConfig.setPortName( portName ); }
    void setBaudrate( size_t baudrate ) { serialConfig.setBaudRate( baudrate ); }

    wxString    getPortName() const { return serialConfig.getPortName(); }
    size_t      getBaudrate() const { return serialConfig.getBaudRate(); }
    wxString    getBaudrateAsString() const { return serialConfig.getBaudRateStr(); };

    bool isV3() const { return ( device == ECUInfo::DEVICE_V3 ); }
    bool isRound() const { return ( device == ECUInfo::DEVICE_ROUND ); }
    bool isUnknown() const { return ( device == ECUInfo::DEVICE_UNKNOWN ); }

    bool isBootMode() const { return ( deviceMode == ECUInfo::BOOT ); }
    bool isManMode() const { return ( deviceMode == ECUInfo::MAN ); }
    bool isMegatuneMode() const { return ( deviceMode == ECUInfo::MEGATUNE ); }
    bool isUnknownMode() const { return ( deviceMode == ECUInfo::UNKNOWN ); }

    wxString toString() const
    {
        wxString infoStr = _T("Port: ") + getPortName() + _T(", Baudrate: ") + getBaudrateAsString() + _T("\n");
        wxString deviceStr = ECUInfo::getDeviceAsString( device );
        wxString deviceModeStr = ECUInfo::getStateAsString( deviceMode );
        infoStr += _T("Device: ") + deviceStr + _T(", Mode: ") + deviceModeStr;
        return infoStr;
    }

    ECUInfo::Device     device;
    ECUInfo::EcuMode    deviceMode;

    FirmwareInfo        firmwareInfo;
    SerialConfig        serialConfig;
};

#endif // DEVICEINFO_H
