#include "ECUStateMgr.h"

#include "ECUInfo.h"
#include "ECUState.h"
#include "ECUStateSIPR.h"
#include "EcuStateGraph.h"
#include "../logger.h"

ECUStateMgr::ECUStateMgr( CommunicationBase* commBase )
:ECUStateMgrBase(commBase)
{
    reset();
    m_device    = ECUInfo::DEVICE_INIT;
    //m_state     = ECUInfo::UNKNOWN;

    setV3();
    setRound();
    //setARM();

    m_currStateGraph =  m_ecuStateGraphH[ ECUInfo::DEVICE_V3 ];
    //m_currStateGraph =  m_ecuStateGraphH[ ECUInfo::DEVICE_ARM ];

    m_baudRateA.Add( 19200 );
    m_baudRateA.Add( 9600 );
    //m_baudRateA.Add( 115200 );

}

ECUStateMgr::~ECUStateMgr()
{
    //dtor
}

void ECUStateMgr::reset()
{
    m_device = ECUInfo::DEVICE_INIT;
    m_state = ECUInfo::UNKNOWN;
}

void ECUStateMgr::setV3()
{
    EcuStateGraph*  ecuStateGraph = new EcuStateGraph();
    m_ecuStateGraphH[ ECUInfo::DEVICE_V3 ] = ecuStateGraph;

    ECUStateBaseArray* ecuStates = new ECUStateBaseArray();
    m_statesH[ ECUInfo::DEVICE_V3 ] = ecuStates;

    ECUStateBase* ecuStateAIM = new ECUStateAIM( m_commBase, 9600 );
    ecuStateAIM->addChangeState( ECUInfo::MAN, _T("ManbyebyeMan") );
    ecuStateAIM->addBaudRate( 19200 );
    ecuStateGraph->add( ecuStateAIM );
    ecuStates->Add( ecuStateAIM );


/*
    ECUStateAIM* ecuStateAIM = new ECUStateAIM( m_commBase, 19200 );
    ecuStateAIM->addChangeState( ECUInfo::MAN2, _T("Manmx5") );
    ecuStateGraph->add( ecuStateAIM );
    ecuStates->Add( ecuStateAIM );
*/

    ECUState* ecuState;
    ecuState = new ECUState( ECUInfo::MEGATUNE, _T("Megatune"), m_commBase, _T("S."), _T("VEMS Genboard v3"), 9600 );
    ecuState->addChangeState( ECUInfo::MAN, _T("Man") );
    ecuState->addChangeState( ECUInfo::BOOT, _T("Mancb33cb") );
    ecuState->addChangeState( ECUInfo::SIPR, _T("Mas") );
    ecuState->addBaudRate( 19200 );
    ecuStateGraph->add( ecuState );
    ecuStates->Add( ecuState );
/*
    ecuState = new ECUState( ECUInfo::MEGATUNE2, _T("Megatune"), m_commBase, _T("S"), _T("VEMS Genboard v3"), 19200 );
    ecuState->addChangeState( ECUInfo::MAN2, _T("Man") );
    ecuState->addChangeState( ECUInfo::BOOT, _T("ManMancb33cb") );
    ecuStateGraph->add( ecuState );
    ecuStates->Add( ecuState );
*/

    ecuState = new ECUState( ECUInfo::MAN, _T("Man"), m_commBase, _T("mxc"), _T("Yes, menu and comm worx"), 9600 );
    ecuState->addChangeState( ECUInfo::MEGATUNE, _T("mmbye") );
    ecuState->addChangeState( ECUInfo::BOOT, _T("cb33cb") );
    ecuState->addChangeState( ECUInfo::AIM, _T("Manmx5") );
    //ecuState->addChangeState( ECUInfo::MAN2, _T("ManmcB34") );
    ecuState->addBaudRate( 19200 );
    ecuStateGraph->add( ecuState );
    ecuStates->Add( ecuState );
/*
    ecuState = new ECUState( ECUInfo::MAN2, _T("Man"), m_commBase, _T("mxc"), _T("Yes, menu and comm worx"), 19200 );
    ecuState->addChangeState( ECUInfo::MEGATUNE2, _T("mmbye") );
    ecuState->addChangeState( ECUInfo::BOOT, _T("Mancb33cb") );
    ecuState->addChangeState( ECUInfo::AIM2, _T("Manmx5") );
    ecuState->addChangeState( ECUInfo::MAN, _T("ManmcB68") );
    ecuStateGraph->add( ecuState );
    ecuStates->Add( ecuState );
*/

    ecuState = new ECUState( ECUInfo::BOOT, _T("Boot"), m_commBase, _T("S"), _T("AVREFI1"), 19200 );
    ecuState->addChangeState( ECUInfo::MEGATUNE, _T("E") );
    ecuStateGraph->add( ecuState );
    ecuStates->Add( ecuState );
/*
    ECUStateBase* ecuStateSIPR = new ECUStateSIPR( m_commBase, 9600, ECUInfo::DEVICE_V3 );
    //ecuStateSIPR->addChangeState( ECUInfo::MAN, _T("Man") );
    ecuStateSIPR->addChangeState( ECUInfo::MEGATUNE, wxEmptyString );
    ecuStateSIPR->addBaudRate( 19200 );
    ecuStateGraph->add( ecuStateSIPR );
    ecuStates->Add( ecuStateSIPR );
*/
}


void ECUStateMgr::setRound()
{
    EcuStateGraph*  ecuStateGraph = new EcuStateGraph();
    m_ecuStateGraphH[ ECUInfo::DEVICE_ROUND ] = ecuStateGraph;

    ECUStateBaseArray* ecuStates = new ECUStateBaseArray();
    m_statesH[ ECUInfo::DEVICE_ROUND ] = ecuStates;

    ECUState* ecuState;
    ecuState = new ECUState( ECUInfo::MEGATUNE, _T("Megatune"), m_commBase, _T("S."), _T("VEMS Round WBO2 v"), 19200 );
    ecuState->addChangeState( ECUInfo::MAN, _T("ManMan.") );
    ecuState->addChangeState( ECUInfo::BOOT, _T("ManMancb33cb.") );
    ecuStateGraph->add( ecuState );
    ecuStates->Add( ecuState );

    ecuState = new ECUState( ECUInfo::MAN, _T("Man"), m_commBase, _T("mmxc."), _T("MenuPingx"), 19200 );
    ecuState->addChangeState( ECUInfo::MEGATUNE, _T("mmmbye.") );
    ecuState->addChangeState( ECUInfo::BOOT, _T("mManMancb33cb.") );
    ecuStateGraph->add( ecuState );
    ecuStates->Add( ecuState );

    ecuState = new ECUState( ECUInfo::BOOT, _T("Boot"), m_commBase, _T("S"), _T("AVR2"), 19200 );
    ecuState->addChangeState( ECUInfo::MEGATUNE, _T("E") );
    ecuStateGraph->add( ecuState );
    ecuStates->Add( ecuState );
}

void ECUStateMgr::setARM()
{
    EcuStateGraph*  ecuStateGraph = new EcuStateGraph();
    m_ecuStateGraphH[ ECUInfo::DEVICE_ARM ] = ecuStateGraph;

    ECUStateBaseArray* ecuStates = new ECUStateBaseArray();
    m_statesH[ ECUInfo::DEVICE_ARM ] = ecuStates;

    ECUStateBase* ecuState;
/*
    ecuState = new ECUState( ECUInfo::MEGATUNE, _T("Megatune"), m_commBase, _T("S"), _T("VEMS ARMboard v3"), 9600 );
    ecuState->addChangeState( ECUInfo::MAN, _T("Man") );
    ecuStateGraph->add( ecuState );
    ecuStates->Add( ecuState );

    ecuState = new ECUState( ECUInfo::MAN, _T("Man"), m_commBase, _T("mxc"), _T("MenuPingx"), 9600 );
    ecuState->addChangeState( ECUInfo::MEGATUNE, _T("mmbye") );
    ecuStateGraph->add( ecuState );
    ecuStates->Add( ecuState );
*/
    ecuState = new ECUStateSIPR( m_commBase, 19200, ECUInfo::DEVICE_ARM );
    ecuStateGraph->add( ecuState );
    ecuStates->Add( ecuState );

    ecuState = new ECUState( ECUInfo::BOOT, _T("Boot"), m_commBase, _T("S"), _T("ARMEFI1"), 19200 );
    //ecuState->addChangeState( ECUInfo::SIPR, _T("M") );
    ecuStateGraph->add( ecuState );
    ecuStates->Add( ecuState );
}

const ECUStateBaseArray* ECUStateMgr::getStates( int device )
{
    if( m_statesH.find( device ) == m_statesH.end() )
        return NULL;

    return m_statesH[ device ];
}


//use getState() before call this function
int ECUStateMgr::getDevice()
{
    return m_device;
}


int ECUStateMgr::getStateAtGivenBaudrate( const ECUStateBaseArray* states )
{
    for( size_t i = 0;  i <  states->Count(); ++i )
    {
        ECUStateBase* ecuState = states->Item( i );
        bool res = states->Item( i )->check();

        if( res ){
            return ecuState->getID();
        }
    }
    return ECUInfo::UNKNOWN;
}

int ECUStateMgr::getState( const ECUStateBaseArray* states )
{
    ECUInfo::EcuMode ecuMode = ( ECUInfo::EcuMode) getStateAtGivenBaudrate( states );
    if( ecuMode != ECUInfo::UNKNOWN){return ecuMode; }
        for(size_t i=0; i < m_baudRateA.GetCount(); i++){
            m_commBase->setBaudRate(m_baudRateA[i]);
            ecuMode = ( ECUInfo::EcuMode) getStateAtGivenBaudrate( states );
            if(ecuMode != ECUInfo::UNKNOWN){return ecuMode; }
        }

    return ECUInfo::UNKNOWN;
}

int ECUStateMgr::getStoredState()
{
    return m_state;
}

//set current device type and return device status if it is exist
int ECUStateMgr::getState()
{
    //Logger::logf( LOG_MSG, _T("ECUStateMgr::getState() -  START") );
    if( m_device != ECUInfo::DEVICE_UNKNOWN && m_device != ECUInfo::DEVICE_INIT )
    {
        ECUStateBaseArray* states = m_statesH[ m_device ];
        int result = getState( states );
        if( result != ECUInfo::UNKNOWN ){
            m_state = result;
            //Logger::logf(LOG_MSG, _T("ECUStateMgr::getState() ???"));
            return result;
        }
    }
    //Logger::logf( LOG_MSG, _T("ECUStateMgr::getState() -  100") );
    ECUStateBaseArrayH::const_iterator it;
    for( it = m_statesH.begin(); it != m_statesH.end(); ++it )
    {
        if( it->first == m_device ) continue;
        int result = getState( it->second );
        if( result != ECUInfo::UNKNOWN )
        {
            m_device = it->first;
            m_currStateGraph = m_ecuStateGraphH[ m_device ];
            m_state = result;
            //Logger::logf(LOG_MSG, _T("ECUStateMgr::getState() FOUND"));
            return result;
        }
    }

    //Logger::logf(LOG_MSG, _T("ECUStateMgr::getState() END"));
    m_device = ECUInfo::DEVICE_UNKNOWN;
    m_state = ECUInfo::UNKNOWN;
    return ECUInfo::UNKNOWN;
}


int ECUStateMgr::setState( int state )
{
    //Logger::logf(LOG_MSG, _T("ECUStateMgr::setState()"));

    int currentState = getState();
    return setState( currentState, state );
}

int ECUStateMgr::setState( int fromState, int toState )
{
     //Logger::logf(LOG_MSG, _T("ECUStateMgr::setState() START"));
    if(fromState == ECUInfo::UNKNOWN || toState == ECUInfo::UNKNOWN ){
        //Logger::logf(LOG_MSG, _T("ECUStateMgr::setState() 1"));
        return ECUInfo::UNKNOWN;
    }

    if( fromState == toState)
    {
         //Logger::logf(LOG_MSG, _T("ECUStateMgr::setState() 2"));
        return toState;
    }

    //Logger::logf(LOG_MSG, _T("ECUStateMgr::setState() 3"));
    Int2ECUStateBaseHash& stateH = m_currStateGraph->getStateH();
    //Logger::logf(LOG_MSG, _T("ECUStateMgr::setState() 4"));
    ECUStateBase* srcState = stateH[ fromState ];
    ECUStateBase* descState = stateH[ toState ];

    //if(srcState == NULL)  Logger::logf(LOG_MSG, _T("ECUStateMgr::setState() src NULL!"));
    //if(descState == NULL)  Logger::logf(LOG_MSG, _T("ECUStateMgr::setState() desc NULL!"));

    //Logger::logf(LOG_MSG, _T("ECUStateMgr::setState() 10, ")+ srcState->getName()+_T(" ")+ descState->getName() );

    if( srcState && descState )
    {
         //Logger::logf(LOG_MSG, _T("ECUStateMgr::setState() 20"));
        return setState( srcState, descState );
    }

    return toState;
}


int ECUStateMgr::setState( const ECUStateBase* fromState, const ECUStateBase* toState )
{
    const int TRY_COUNT = 3;

    const ECUStateBaseArray* orderArray = m_currStateGraph->getRoute( fromState->getID(), toState->getID() );

    int dest = ECUInfo::UNKNOWN;

    if(orderArray->GetCount() < 1) return dest;

    for( uint32_t i =0; i< orderArray->GetCount() - 1; ++i )
    {
        ECUStateBase* currState = orderArray->Item( i );
        ECUStateBase* nextState = orderArray->Item( i + 1 );

        //Logger::logf(LOG_MSG, _T("ECUStateMgr::setState() %i,%i"),currState->getID(), nextState->getID() );
        int next = nextState->getID();
        int count = 0;

        do{
            currState->goToState( next );
            dest = getState();
            ++count;
        }while( ( count < TRY_COUNT ) && ( next != dest ) );

        if(  next != dest )
        {
            return dest;
        }
    }

    return dest;
}

void ECUStateMgr::setBaudRate( int baudRate )
{
    int state = getState();

    setState( ECUInfo::MAN );

    switch( baudRate )
    {
        case 9600  :
            //m_commBase->SendString( (uint8_t*)"ManmcB68" );
            m_commBase->SendString( _T("ManmcB68") );
            break;
        case 19200 :
            m_commBase->SendString( (uint8_t*)"ManmcB34" );
            break;
    }

    setState( state );
}

int ECUStateMgr::getBaudRate()
{
   return m_commBase->getBaudRate();
}
