#include "EcuStateGraph.h"

#include "../logger.h"
#include "ECUInfo.h"
#include "ECUState.h"


EcuStateGraph::EcuStateGraph()
{
    //ctor
}


EcuStateGraph::~EcuStateGraph()
{
    //dtor
}


void EcuStateGraph::add( ECUStateBase* ecuState )
{
    m_stateH[ ecuState->getID() ] = ecuState;
}


const ECUStateBaseArray* EcuStateGraph::getRoute( int fromID, int toID )
{
    initParents( fromID );

    ECUStateBaseArray searchQueue;

    searchQueue.Add( m_stateH[ fromID ] );

    while( !searchQueue.IsEmpty() )
    {
        ECUStateBase* state = searchQueue.Item( 0 );
        searchQueue.RemoveAt( 0 );

        IntArray*  targetA = state->getChangeStateA();

        uint32_t i;
        for( i = 0; i < targetA->GetCount(); ++i )
        {
            ECUStateBase* targetState = m_stateH[ targetA->Item( i ) ];

            if( targetState->getID() == toID )
            {
                targetState->setParentID( state->getID() );
                searchQueue.Clear();
                break;
            }

            if( targetState->getParentID() == 0 )
            {
                targetState->setParentID( state->getID() );
                searchQueue.Add( targetState );
            }
        }
    }

    setRoute( fromID, toID );

    return &m_stateOrderPathA;
}


void EcuStateGraph::initParents( int fromID )
{
    Int2ECUStateBaseHash::iterator it;

    for( it = m_stateH.begin(); it != m_stateH.end(); ++it )
    {
        ECUStateBase* ecuState = it->second;
        ecuState->setParentID( 0 );
    }

    m_stateH[ fromID ]->setParentID( -1 );
}


void EcuStateGraph::setRoute( int fromID, int toID )
{
    m_stateOrderPathA.clear();

    if( m_stateH[ toID ]->getParentID() == 0 )
    {
        //no route to target
        return;
    }

    int id = toID;

    while( id != fromID )
    {
        m_stateOrderPathA.Insert( m_stateH[ id ], 0u );
        //Logger::logf( LOG_MSG, _T(" EcuStateGraph::setRoute() - id = %i"), id );
        id = m_stateH[ id ]->getParentID();

    }

    m_stateOrderPathA.Insert( m_stateH[ id ], 0u );

}


/*TEST*/
void EcuStateGraph::printParents()
{
    Int2ECUStateBaseHash::iterator it;

    for( it = m_stateH.begin(); it != m_stateH.end(); ++it )
    {
        ECUStateBase* ecuState = it->second;
        Logger::logf( LOG_MSG, _T("state %s parent = %i" )
                    ,ECUInfo::getStateAsString( ecuState->getID() ).c_str()
                    ,ecuState->getParentID() );
    }
}


/*TEST*/
wxString EcuStateGraph::getOrderAsString()
{
    wxString tmp;

    for( uint32_t i = 0; i < m_stateOrderPathA.GetCount() ; ++i )
    {
        ECUStateBase* state = m_stateOrderPathA.Item( i );
        tmp += _T(" ") + state->getName();
        //Logger::log( LOG_MSG, _T("EcuStateGraph::getOrderAsString() - Path : ") + state->getName() );
    }

    return tmp;
}
