#ifndef SIPRDESCRIPTOR_H
#define SIPRDESCRIPTOR_H

#include "../common/types/IntArray.h"
#include "../common/types/String2IntHash.h"

class SIPRPageDescriptor
{
    public:
        SIPRPageDescriptor();
        SIPRPageDescriptor( int page,
                            const wxString& readCommand,
                            const wxString& writeCommand,
                            int type,
                            const IntArray tables );

        void        setPageNo( int pageNo ) { m_iPageNumber = pageNo; }
        int         getPageNo() const       { return m_iPageNumber; }
        void        setType( int type )     { m_iCommandType = type; }
        int         getType() const         { return m_iCommandType; }
        void        addTable( int type)     { m_tables.Add( type ); }
        IntArray    getTables() const       { return m_tables; }
        void        setPageReadCommand( const wxString& cmd )   { m_strReadCommand = cmd; }
        wxString    getPageReadCommand() const                  { return m_strReadCommand; }
        void        setPageWriteCommand( const wxString& cmd )  { m_strWriteCommand = cmd; }
        wxString    getPageWriteCommand() const                 { return m_strWriteCommand; }

    private:
        IntArray    m_tables;
        wxString    m_strReadCommand;
        wxString    m_strWriteCommand;
        int         m_iPageNumber;
        int         m_iCommandType;
};

WX_DECLARE_HASH_MAP( int, SIPRPageDescriptor*, wxIntegerHash, wxIntegerEqual, IntToSIPRPageDescriptor );

class SIPRTableDescriptor
{
    public:
        void        setType( int type )     { m_iType = type; }
        int         getType() const         { return m_iType; }
        void        setSize( int size )     { m_iSize = size; }
        int         getSize() const         { return m_iSize; }
        void        setOffset( int offset ) { m_iOffset = offset; }
        int         getOffset() const       { return m_iOffset; }
        void        addPage( int pageNo )   { m_pages.Add( pageNo ); }
        IntArray    getPages() const        { return m_pages; }
        void        setConfigFileID( const wxString& id ) { m_ConfigFileID = id; }
        wxString    getConfigFileID() const { return m_ConfigFileID; }

    private:
        IntArray    m_pages;
        wxString    m_ConfigFileID;
        int         m_iType;
        int         m_iOffset;
        int         m_iSize;

};

WX_DECLARE_HASH_MAP( int, SIPRTableDescriptor*, wxIntegerHash, wxIntegerEqual, IntToSIPRTableDescriptor );

class RealTimePageDesc
{
    public:
        wxString 	MegatuneReadCommand;
        int 		SiprPageType;

    public:
                    RealTimePageDesc( const wxString& mtReadCmd, int siprPageType )
                    {
                        MegatuneReadCommand     = mtReadCmd;
                        SiprPageType            = siprPageType;
                    }

};

/// key = SiprPageType
WX_DECLARE_HASH_MAP( int, RealTimePageDesc*, wxIntegerHash, wxIntegerEqual, RealTimePageDescHash );

class SIPRDescriptor
{
    public:
		SIPRDescriptor();
		virtual ~SIPRDescriptor();


        // parser methods
    	//void        addPage( int pageNo, const wxString& type );
    	///add Megatune page to Descriptor( m_pages ) ( type = SIPR_GET_TABLEDATA, SIPR_GET_CONFIG_1, 2 )
    	void                addPage( int pageNo, const wxString& readCommand, const wxString& writeCommand, const wxString& type );

        ///add SIPR table types to a megatune page / create SIPRTable descriptor ( m_tables )
        void                addSIPRTableToPage( int pageNo, const wxString& table );

        ///add SIPR table to Descriptor ( m_tables )
        void                addSIPRTable( const wxString& table, int offset, int size, const wxString& id );

        ///add SIPR config to Descriptor ( m_configs )
        void                addSIPRConfig( const wxString& config, int offset, int size, const wxString& id );

        void                addRealTimePage( const wxString& mtReadCmd, const wxString& siprPageType );

        // getter methods
        bool                isPageExist( int pageNo ) const;
        int                 getPageType( int pageNo ) const;
        IntArray            getTables( int pageNo ) const;
        int                 getOffset( int pageType, int table ) const;
        int                 getSize( int pageType, int table ) const;
        wxString            getConfigFileID(  int pageType, int table ) const;
        IntArray            getPages( int pageType, int table ) const;
        wxString            getPageReadCommand( int page ) const;
        wxString            getPageWriteCommand( int page ) const;
        int                 getPageSize(  int pageNo ) const;
        size_t              getPageCount() const { return m_pages.size(); }                     /// return megatune pages count
        void                setBurnCommand( const wxString& cmd ) { m_strBurnCommand = cmd; }
        wxString            getBurnCommand() const { return m_strBurnCommand; }
        const IntArray&     getPagesOrder() const { return m_pagesOrder; }                      /// return megatune page number at idx
        int                 getPageNumber( int idx ) const;
        void                setOchGetCommand( const wxString& command ) { m_strOchGetCommand = command; }
        wxString            getOchGetCommand() const { return m_strOchGetCommand; }
        wxString            getMTReadRealTimeCommand( int type ) const;
        wxString            getMTReadRealTimeCommand( const wxString& type ) const;
        size_t              getSiprTableCount() const;
        //
        void                clear();

    private:
        IntToSIPRPageDescriptor     m_pages;
        IntArray                    m_pagesOrder;
        IntToSIPRTableDescriptor    m_tables;
        IntToSIPRTableDescriptor    m_configs;
        RealTimePageDescHash        m_RealTimePages;
        wxString                    m_strBurnCommand;
        wxString                    m_strOchGetCommand;

        String2IntHash              m_RealTimeLookUpTable;  // text to sipr ids

    private:
        int getTableType( const wxString& table );
        int getConfigType( const wxString& config );
        int getConfigPageSize(  int pageNo ) const;
        int getTablePageSize(  int pageNo ) const;
};

#endif // SIPRDESCRIPTOR_H

