#ifndef SIPRPACKET_H
#define SIPRPACKET_H

//#include "cstring"
#include "wx/wx.h"
#include "../common/typedef.h"
#include "../common/types/IntArray.h"

class timer;

class SIPRPacket : public wxObject
{
    public:
        static const uint8_t    SIPR_SIGN;
        static const int        HEADER_SIZE;
        static const int        SIPR_FRAME_SIZE;
        static const int        SIPR_MAX_DATA_SIZE;

    public:
                        SIPRPacket();

                        SIPRPacket( uint8_t sequence,
                                    uint8_t type,
                                    uint8_t size = 0,
                                    const uint8_t* data = NULL,
                                    uint16_t crc = 0X0000 );        /// TODO - add bool flag to sign crc usage
                        SIPRPacket(uint8_t sequence, uint8_t type, uint8_t size, const IntArray& data);

                        SIPRPacket( const SIPRPacket& packet );

                        ~SIPRPacket();

        void            operator=( const SIPRPacket& packet );

        // methods for transmit data
        const uint8_t*  packet_data() const { return m_pData; }
        int             packet_size() const { return SIPR_FRAME_SIZE + size(); }

        // getter methods
        uint8_t         sequence() const    { return m_pData[1]; }
        uint8_t         type() const        { return m_pData[2]; }
        uint8_t         size() const        { return m_pData[3]; }
        uint8_t*        data() const        { return size() ? &m_pData[4] : NULL; }
        uint16_t        crc() const         { return getCRC(); }

        // helper methods
        static uint16_t calculateCRC( uint8_t* data, int size );

        void            set_timeout( int timeout );
        void            start_timer();
        bool            is_timed_out() const;
        int             getSubType() const;
        wxString        toString();
    private:
        uint8_t*    m_pData;            // the packet
//        timer*      m_pTimer;           // timer for timeout
//        int         m_timeout_flag;     // timer set this flag
        wxDateTime  m_startTime;
        int         m_timeout;

    private:
        void        init(   uint8_t sequence,
                            uint8_t type,
                            uint8_t size = 0,
                            const uint8_t* data = NULL,
                            uint16_t crc = 0X0000 );

        void        setCRC( uint16_t crc );
        uint16_t    getCRC() const;
};

WX_DECLARE_LIST( SIPRPacket, SIPRPacketList );      /// store pointers

/*
class SIPRPacket
{
    public:
        SIPRPacket()
        {
            size = 0;
            data = NULL;
        }

        SIPRPacket(int s)
        {
            size = s;
            data = new uint8_t[s];
        }

        SIPRPacket(const SIPRPacket& packet)
        {
            //wxLogMessage( _("SIPRPacket::CopyConstructor()") );
            data = new uint8_t[packet.size];
            seq = packet.seq;
            type = packet.type;
            size = packet.size;
            if ( size ) { memcpy(data,packet.data,size); }
            crc = packet.crc;
        }

        ~SIPRPacket() { delete[] data; }//{ wxLogMessage( _T("~SIPRPacket()") ); delete[] data; }

        void alocateData() { delete[] data; data = new uint8_t[size]; }

       void operator=(const SIPRPacket& packet)
        {
            delete[] data;
            data = new uint8_t[packet.size];
            seq = packet.seq;
            type = packet.type;
            size = packet.size;
            memcpy(data,packet.data,size);
            crc = packet.crc;
            //return *this;
        };

        uint8_t seq;
        uint8_t type;
        uint8_t size;
        uint8_t* data;
        uint16_t crc;
    private:
};
*/

#endif // SIPRPACKET_H
