#include "VEMSLogBuffer.h"
#include "VemsLogReader.h"
#include "SIPRTypes.h"
#include "../logger.h"
#include "../RawLogHelper.h"


VEMSLogBuffer::VEMSLogBuffer()
{
    clearBuffer();
/*    m_index = 0;
    m_counter = 0;
    m_blockLen = m_prevblockLen = 0;
    m_prevvalue = 0;
    m_blockNr = 0;
    m_RTtimeIndex.Clear();
    m_stCfgIdxHash.clear();
    m_stTableIdxHash.clear(); */
}

VEMSLogBuffer::~VEMSLogBuffer()
{
    //dtor
    Logger::logf(LOG_MSG, _T("VEMSLogBuffer dtor"));
    clearBuffer();
}

bool VEMSLogBuffer::loadLogFromFile(const wxString& filename, StringHash& headerHash, long & logversion)
{
    VemsLogReader * logreader = new VemsLogReader();
    if ( ! wxFileExists(filename) )
    {
        Logger::logf(LOG_MSG, _T("VEMSLogBuffer::loadLogFromFile - file doesnot exists") + filename);
        return false;
    }

    clearBuffer();

    if ( logreader->SetInputFile( filename ) < 0 )
    {
        Logger::logf(LOG_MSG, _T("VEMSLogBuffer::loadLogFromFile - logreader->SetInputFile() failed") );
        return false;
    }

    if ( logreader->ParseHeader() < 0 )
    {
        Logger::logf(LOG_MSG, _T("VEMSLogBuffer::loadLogFromFile - logreader->ParseHeader() failed") );
        return false;
    }

    logreader->reset();

    headerHash = logreader->getHeaderHash();
    logreader->getHeaderValue( RawLogHelper::logVersion ).ToLong( &logversion );

    Logger::logf(LOG_MSG, _T("VEMSLogBuffer::loadLogFromFile - packet reading started"));
    //logreader->start();

    SIPRPacket sp;

    while (logreader->getNext(sp) != -2)
    {
        addPacket(sp);
    }
    Logger::logf(LOG_MSG, _T("VEMSLogBuffer::loadLogFromFile - packet reading finished"));
    Logger::logf(LOG_MSG, _T("VEMSLogBuffer::loadLogFromFile - %d packet added"), m_siprpackets.GetCount() );

    delete logreader;

    Logger::logf( LOG_MSG, _T("rt count=%d"),getHistoryCount(IDX_RTDATA_NORMAL) );

    return true;
}

void VEMSLogBuffer::clearBuffer()
{
    Logger::logf(LOG_MSG, _T("clearbuffer()"));
    if ( m_siprpackets.GetCount() > 0 )
    {
        WX_CLEAR_ARRAY ( m_siprpackets );
        m_indexHash.clear();
    }

    m_index = 0; m_counter = 0;
    m_blockLen = m_prevblockLen = 0; m_prevvalue = 0;
    m_blockNr = 0;
    m_RTtimeIndex.Clear();
    m_stCfgIdxHash.clear();
    m_stTableIdxHash.clear();
    m_gpsIdxHash.clear();
    m_RealRTtime.clear();
}

size_t VEMSLogBuffer::addPacket( SIPRPacket packet ) /// need to discuss!! ( GetCount()-1 problem )
{
    SIPRPacket* p = new SIPRPacket( packet );
    m_siprpackets.Add( p );

    size_t retIdx = 0;

    switch ( p->type() )
    {
        case SIPR_RT_TIME:
        {
            m_indexHash[IDX_RT_TIME].Add( m_index );
            retIdx = m_indexHash[IDX_RT_TIME].GetCount() - 1;
            convertRTTime(*p);

            break;
        }
        case SIPR_READ_SENSORS:
        case SIPR_READ_MEMORY_32:
        {
            int subType = 0;

            if ( p->type() == SIPR_READ_SENSORS ) subType = p->getSubType();
            else if ( p->type() == SIPR_READ_MEMORY_32 ) subType = SiprSensorType::SIPR_READ_MEMORY_32;

            switch ( subType )
            {
                case SiprSensorType::SIPR_RT_MT_NORMAL:
                case SiprSensorType::SIPR_RT_MT_EXT:
                case SiprSensorType::SIPR_READ_MEMORY_32:
                {
                    uint8_t counter_value = (p->data())[3];

                    if (m_RTtimeIndex.GetCount() == 0)
                        m_prevvalue = counter_value;

                    m_indexHash[IDX_RTDATA_NORMAL].Add( m_index );

//                    Logger::logf(LOG_MSG, _T("addPacket: %d packetidx => %d. rtidx"), m_index, m_indexHash[IDX_RTDATA_NORMAL].GetCount()-1 );

                    if ( counter_value == m_prevvalue )
                    {
                        if ( m_blockNr == 0 || m_blockNr == 1 )
                        {
                            m_RTtimeIndex.Add(0.0); // will be recalculated

                            if ( m_blockNr == 1) m_blockLen++;
                            m_prevvalue = counter_value;
                            retIdx = m_RTtimeIndex.GetCount()-1;
                            if ( m_gpsIdxHash.find(retIdx) == m_gpsIdxHash.end() )
                                m_gpsIdxHash[retIdx] = m_indexHash[IDX_RTDATA_GPS].GetCount() - 1;
                            break;
                        }
                        else
                        {
                            double _time = m_RTtimeIndex[m_RTtimeIndex.GetCount()-m_blockLen] + 1.0*(double)(m_blockLen+1)/(double)m_prevblockLen;
                            m_RTtimeIndex.Add(_time);

                            m_blockLen++;
                            m_prevvalue = counter_value;
                            retIdx = m_RTtimeIndex.GetCount()-1;
                            if ( m_gpsIdxHash.find(retIdx) == m_gpsIdxHash.end() )
                                m_gpsIdxHash[retIdx] = m_indexHash[IDX_RTDATA_GPS].GetCount() - 1;
                            break;
                        }
                    }
                    else
                    {
                        if ( m_blockNr == 0 )
                        {
                            m_blockNr = 1;
                            m_RTtimeIndex.Add(0.0); // will be recalculated

                            m_prevvalue = counter_value;
                            m_blockLen++;
                            retIdx = m_RTtimeIndex.GetCount()-1;
                            if ( m_gpsIdxHash.find(retIdx) == m_gpsIdxHash.end() )
                                m_gpsIdxHash[retIdx] = m_indexHash[IDX_RTDATA_GPS].GetCount() - 1;

                            break;
                        }
                        else if ( m_blockNr == 1 )
                        {
                            // recalculate the timestamp of the very first block
                            size_t end_of_firstblock = m_RTtimeIndex.GetCount() - m_blockLen;
                            for ( size_t i = 0; i < end_of_firstblock; ++i )
                            {
                                m_RTtimeIndex[i] = (double)i * 1.0 / m_blockLen;
                            }
                            // recalculate the 2nd block
                            for ( size_t i = 0; i < m_blockLen; ++i )
                            {
                                m_RTtimeIndex[end_of_firstblock+i] = m_RTtimeIndex[end_of_firstblock-1] + (double)(i+1) * 1.0 / (double) m_blockLen;
                            }

                            // calculate forward
                            double _time = m_RTtimeIndex[m_RTtimeIndex.GetCount()-1] + 1.0/(double)m_blockLen;
                            m_RTtimeIndex.Add(_time);

                            m_prevblockLen = m_blockLen;
                            m_prevvalue = counter_value;
                            m_blockLen = 1;
                            m_blockNr = 2;
                            retIdx = m_RTtimeIndex.GetCount()-1;
                            if ( m_gpsIdxHash.find(retIdx) == m_gpsIdxHash.end() )
                                m_gpsIdxHash[retIdx] = m_indexHash[IDX_RTDATA_GPS].GetCount() - 1;
                            break;
                        }
                        else
                        {
                            size_t end_of_prev_block = m_RTtimeIndex.GetCount() - m_blockLen;
                            for ( size_t i = 0; i < m_blockLen; ++i )
                            {
                                m_RTtimeIndex[end_of_prev_block+i] = m_RTtimeIndex[end_of_prev_block-1] + (double)(i+1) * 1.0 / (double) m_blockLen;
                            }
                            // calculate forward
                            double _time = m_RTtimeIndex[m_RTtimeIndex.GetCount()-1] + 1.0/(double)m_blockLen;
                            m_RTtimeIndex.Add(_time);

                            m_prevblockLen = m_blockLen;
                            m_blockLen = 1;
                            m_prevvalue = counter_value;

                            retIdx = m_RTtimeIndex.GetCount()-1;
                            if ( m_gpsIdxHash.find(retIdx) == m_gpsIdxHash.end() )
                                m_gpsIdxHash[retIdx] = m_indexHash[IDX_RTDATA_GPS].GetCount() - 1;
                            break;
                        }
                    }

                    break;
                }
                case SiprSensorType::SIPR_RT_ROUND:
                    m_indexHash[IDX_RTDATA_NORMAL].Add( m_index );

                    retIdx = m_indexHash[IDX_RTDATA_NORMAL].GetCount()-1;

                    if ( m_gpsIdxHash.find(retIdx) == m_gpsIdxHash.end() )
                        m_gpsIdxHash[retIdx] = m_indexHash[IDX_RTDATA_GPS].GetCount() - 1;

                    break;

                default:
                    m_indexHash[IDX_RTDATA_GPS].Add( m_index );

                    retIdx = m_indexHash[IDX_RTDATA_GPS].GetCount() - 1;
                    break;
            }
        }
        break;

        case SIPR_GET_TABLEDATA:
        case SIPR_GET_CONFIG_1:
        case SIPR_GET_CONFIG_2:
        /*case SIPR_READ_MEMORY:
        case SIPR_READ_MEMORY_32:*/
        {
            m_indexHash[IDX_STDATA].Add( m_index );

            retIdx = m_indexHash[IDX_STDATA].GetCount() - 1;

            // add idx to stdata idx hash
            if ( p->type() == SIPR_GET_CONFIG_1 || p->type() == SIPR_GET_CONFIG_2 )
            {
                m_stCfgIdxHash[p->type()].Add( m_index );
            }

            else if ( p->type() == SIPR_GET_TABLEDATA )
            {
                unsigned int subType = p->getSubType();
                m_stTableIdxHash[subType].Add( m_index );
            }

        }
        break;

        // LOG Marker
        case SIPR_LOG_MARKER:
        {
            m_indexHash[IDX_LOGMARKER].Add( m_index );

            retIdx = m_indexHash[IDX_LOGMARKER].GetCount() - 1;
            break;
        }

        case SIPR_MDX_FRAMES_END:
        {
            m_indexHash[IDX_FRAME_END].Add( m_index );

            retIdx = m_indexHash[IDX_FRAME_END].GetCount() - 1;
        }
    }

    m_index++;

    return retIdx;
}

double VEMSLogBuffer::getPacketTime( size_t idx )
{
    if ( m_RealRTtime.GetCount() == 0 )
    {
        // use calculated values
        if ( idx < 0 || idx >= m_RTtimeIndex.GetCount() )
        {
            return -1.0;
        }
        else return m_RTtimeIndex[idx];
    }
    else
    {
        if ( idx < 0 || idx >= m_RealRTtime.GetCount() )
        {
            return -1.0;
        }
        else
        {
            return ( m_RealRTtime[idx] / 1000.0 );
        }
    }

}

long VEMSLogBuffer::getPosition()
 {
     if ( m_siprpackets.GetCount() < 1)
        return -1;
    else return m_counter;
}

void VEMSLogBuffer::setPosition(size_t position)
{
    m_counter = position;
}

SIPRPacket* VEMSLogBuffer::getFirstPacket()
{
    m_counter = 0;
    if ( m_siprpackets.GetCount() == 0)
        return NULL;
    else return (SIPRPacket*)(m_siprpackets[m_counter]);
}

SIPRPacket* VEMSLogBuffer::getNextPacket()
{
    if ( m_counter >= m_siprpackets.GetCount() || m_siprpackets.GetCount() == 0 )
        return NULL;
    else
    {
        //Logger::logf(LOG_MSG, _T("Pos=%d, Len=%d"), m_counter, m_siprpackets.GetCount());
        return (SIPRPacket*)m_siprpackets[m_counter++];
    }
}

SIPRPacket* VEMSLogBuffer::getPacket()
{
    if ( m_counter >= m_siprpackets.GetCount() || m_siprpackets.GetCount() == 0 )
        return NULL;
    else
        return (SIPRPacket*)m_siprpackets[m_counter];
}

SIPRPacket* VEMSLogBuffer::getPacket( size_t idx )
{
    if ( idx >= m_siprpackets.GetCount() )
        return NULL;
    else
        return (SIPRPacket*)m_siprpackets[idx];
}

bool VEMSLogBuffer::hasNext()
{
    if ( (m_counter+1) < m_siprpackets.GetCount() )
        return true;
    else return false;
}

size_t VEMSLogBuffer::getHistoryCount(size_t packetType)
{
    if ( packetType == IDX_ALL )
    {
        return m_siprpackets.GetCount();
    }

    size_t result = 0;
    if ( packetType & IDX_RTDATA_NORMAL )
        result += m_indexHash[IDX_RTDATA_NORMAL].GetCount();

    if ( packetType & IDX_RTDATA_GPS )
        result += m_indexHash[IDX_RTDATA_GPS].GetCount();

    if ( packetType & IDX_STDATA )
        result += m_indexHash[IDX_STDATA].GetCount();

    if ( packetType & IDX_LOGMARKER )
        result += m_indexHash[IDX_LOGMARKER].GetCount();

    if ( packetType & IDX_RT_TIME )
        result += m_indexHash[IDX_RT_TIME].GetCount();

    if ( packetType & IDX_FRAME_END )
        result += m_indexHash[IDX_FRAME_END].GetCount();

    return result;
}

SIPRPacket* VEMSLogBuffer::getPacketByType(size_t idx, size_t packetType)
{
/*
    long packetIdx = getIdxByDIdx(idx, packetType );
    if(packetIdx == -1) return NULL;
    return (SIPRPacket*)(m_siprpackets[packetIdx] );
*/

    switch ( packetType )
    {
        case IDX_RTDATA_GPS:
            if ( idx >= m_indexHash[IDX_RTDATA_GPS].GetCount() )
                return NULL;
            return (SIPRPacket*)(m_siprpackets[m_indexHash[IDX_RTDATA_GPS][idx]]);

        case IDX_STDATA:
            if ( idx >= m_indexHash[IDX_STDATA].GetCount() )
                return NULL;
            return (SIPRPacket*)(m_siprpackets[m_indexHash[IDX_STDATA][idx]]);

        case IDX_LOGMARKER:
            if ( idx >= m_indexHash[IDX_LOGMARKER].GetCount() )
                return NULL;

            return (SIPRPacket*)( m_siprpackets[m_indexHash[IDX_LOGMARKER][idx]]);

        case IDX_RT_TIME:
            if ( idx >= m_indexHash[IDX_RT_TIME].GetCount() )
                return NULL;

            return (SIPRPacket*)( m_siprpackets[m_indexHash[IDX_RT_TIME][idx]]);

        case IDX_FRAME_END:
            if ( idx >= m_indexHash[IDX_FRAME_END].GetCount() )
                return NULL;

            return (SIPRPacket*)( m_siprpackets[m_indexHash[IDX_FRAME_END][idx]]);

        case IDX_RTDATA_NORMAL:
        default:
            if ( idx < 0 || idx >= m_indexHash[IDX_RTDATA_NORMAL].GetCount() )
                return NULL;
            return (SIPRPacket*)(m_siprpackets[m_indexHash[IDX_RTDATA_NORMAL][idx]]);
    }
}

long VEMSLogBuffer::findRTidxByIdx(size_t idx, size_t searchFrom)
{
    for ( size_t i = searchFrom; i < m_indexHash[IDX_RTDATA_NORMAL].GetCount(); ++i )
    {
        if ( m_indexHash[IDX_RTDATA_NORMAL][i] == (int)idx )
            return i;
    }
    return -1;
}

long VEMSLogBuffer::getSTIndexByRTIdx( size_t idx )
{
    if ( idx < 0 || idx >= m_indexHash[IDX_RTDATA_NORMAL].GetCount() )
        return -1;

    int rtidx = m_indexHash[IDX_RTDATA_NORMAL][idx];

    long stidx = -1;
    long oldstidx = 0;

    for ( size_t i = 0; i < m_indexHash[IDX_STDATA].GetCount(); ++i )
    {
        stidx = m_indexHash[IDX_STDATA][i];
        if ( stidx > rtidx )
            return oldstidx;

        oldstidx = stidx;
    }

    return oldstidx;
}

//void VEMSLogBuffer::updateSlider(size_t position)
// {
//    int i = findRTidxByIdx(position), idx = getHistoryCount(IDX_RTDATA_NORMAL);
//    if ( i == -1 ) return;
//    observerNotify( i , idx );
//    observerNotify(1);
//
//    double t = m_RTtimeIndex[i];
//    int ms = (int)((t - (int)t)*100);
//    int t1 = (int)t;
//    int hh=0,mm=0,ss=0;
//    if ( t1 > 3600) { hh = (int)(t1/3600); t1 = t1 % 3600; }
//    if ( t1 > 60 ) { mm = (int)(t1/60); ss = t1 % 60; }
//    else ss = t1;
//    wxString timestr = wxString::Format(_T("%02d:%02d:%02d.%02d"), hh, mm, ss,ms);
//    observerNotify(timestr);
// }

long VEMSLogBuffer::getIdxByDIdx(size_t idx, size_t packetType)
{
/*
    IntToIntArrayHash::const_iterator IT = m_indexHash.find(packetType);
    if(IT == m_indexHash.end() ) return -1;
    return m_indexHash[packetType][idx];
*/
    switch ( packetType )
    {
        case IDX_RTDATA_GPS:
            if ( idx>= m_indexHash[IDX_RTDATA_GPS].GetCount())
                return -1;
            return m_indexHash[IDX_RTDATA_GPS][idx];

        case IDX_STDATA:
            if (idx>= m_indexHash[IDX_STDATA].GetCount())
                return -1;
            return m_indexHash[IDX_STDATA][idx];

        case IDX_LOGMARKER:
            if ( idx >= m_indexHash[IDX_LOGMARKER].GetCount() )
                return -1;
            return m_indexHash[IDX_LOGMARKER][idx];

        case IDX_RT_TIME:
            if ( idx >= m_indexHash[IDX_RT_TIME].GetCount() )
                return -1;
            return m_indexHash[IDX_RT_TIME][idx];

        case IDX_FRAME_END:
            if ( idx >= m_indexHash[IDX_FRAME_END].GetCount() )
                return -1;
            return m_indexHash[IDX_FRAME_END][idx];

        case IDX_RTDATA_NORMAL:
        default:
            if (idx>= m_indexHash[IDX_RTDATA_NORMAL].GetCount())
                return -1;
            return m_indexHash[IDX_RTDATA_NORMAL][idx];
    }
}

SIPRPacket* VEMSLogBuffer::getGPSData(size_t rtIdx, int* gpsIdx)
{
    SIPRPacket* gpsdata = NULL;

    if ( gpsIdx != NULL )
    {
        *gpsIdx = -1;
    }

    if ( rtIdx < 0 || rtIdx >= m_indexHash[IDX_RTDATA_NORMAL].GetCount() )
        return gpsdata;

    int gpsHashidx = m_gpsIdxHash[rtIdx];

    if ( gpsHashidx > -1 )
    {
        gpsdata = (SIPRPacket*)(m_siprpackets[m_indexHash[IDX_RTDATA_GPS][gpsHashidx]]);

        if ( gpsIdx != NULL )
            *gpsIdx = gpsHashidx;
    }

    return gpsdata;

}

void VEMSLogBuffer::getSTData(size_t rtIdx, wxObjectArray& stpackets)
{
    int packetIdx = getIdxByDIdx( rtIdx );

    // st cfg
    IntToIntArrayHash::iterator it = m_stCfgIdxHash.begin();
    for ( ; it != m_stCfgIdxHash.end(); ++it )
    {
        int idx = -1;
        for ( size_t i = 0; i < ( it->second ).GetCount(); ++i )
        {
            int tmp = (it->second)[i];
            if ( ( tmp > idx ) && ( tmp < packetIdx) )
                idx = tmp;
        }
        if ( idx != -1 )
        {
            SIPRPacket * p = (SIPRPacket*)(m_siprpackets[idx]);
            stpackets.Add(p);
        }
    }

    // st tables
    it = m_stTableIdxHash.begin();
    for ( ; it != m_stTableIdxHash.end(); ++it )
    {
        int idx = -1;
        for ( size_t i = 0; i < ( it->second ).GetCount(); ++i )
        {
            int tmp = (it->second)[i];
            if ( ( tmp > idx ) && ( tmp < packetIdx) )
                idx = tmp;
        }
        if ( idx != -1 )
        {
            SIPRPacket * p = (SIPRPacket*)(m_siprpackets[idx]);
            stpackets.Add(p);
        }
    }
}

long VEMSLogBuffer::getNextRTidxByIdx( int index )
{
    if ( index < 0 || (size_t)index >= m_siprpackets.GetCount() )
        return -1;

    for ( size_t i = (size_t)index; i < m_siprpackets.GetCount(); ++i )
    {
        SIPRPacket* p = (SIPRPacket*) m_siprpackets[i];

        if ( p->type() == SIPR_READ_SENSORS || p->type() == SIPR_READ_MEMORY_32 )
        {
            int subType;

        if ( p->type() == SIPR_READ_SENSORS )
                subType = p->getSubType();
            else if ( p->type() == SIPR_READ_MEMORY_32 )
                subType = SiprSensorType::SIPR_READ_MEMORY_32;

            if ( subType == SiprSensorType::SIPR_RT_MT_NORMAL || subType == SiprSensorType::SIPR_RT_MT_EXT
                || subType == SiprSensorType::SIPR_RT_ROUND || subType == SiprSensorType::SIPR_READ_MEMORY_32 )
            {
                IntArray& arr = m_indexHash[IDX_RTDATA_NORMAL];
                int idx_ = arr.Index( i );

                if ( idx_ != wxNOT_FOUND )
                    return idx_;
            }
        }
    }
    return -1;
}

void VEMSLogBuffer::convertRTTime( const SIPRPacket& p )
{
    uint8_t data[4];

    for ( int i = 0; i < 4; data[i++] = 0 );

    memcpy( &data, p.data(), 4 );

    size_t rtime = data[0] * 256 * 256 * 256 + data[1] * 256 * 256 + data[2] * 256 + data[3];

    if ( m_RealRTtime.GetCount() == 0 )
    {
        m_refTime = rtime;
    }

    rtime -= m_refTime;

    m_RealRTtime.Add( rtime );
}
