#ifndef VEMSLOGBUFFER_H
#define VEMSLOGBUFFER_H


#include "../common/types/wxObjectArray.h"
#include "../common/types/IntArray.h"
#include "../common/types/StringHash.h"
#include "../common/types/DoubleArray.h"
#include "../common/types/Int2IntHash.h"
#include "SIPRPacket.h"


enum {  IDX_RTDATA_NORMAL = 1,
        IDX_RTDATA_GPS = 2,
        IDX_STDATA = 4,
        IDX_LOGMARKER = 8,
        IDX_RT_TIME = 16,
        IDX_FRAME_END = 32,

        IDX_ALL = IDX_RTDATA_NORMAL | IDX_RTDATA_GPS | IDX_STDATA | IDX_LOGMARKER | IDX_RT_TIME | IDX_FRAME_END

};

WX_DECLARE_HASH_MAP( int, IntArray, wxIntegerHash, wxIntegerEqual, IntToIntArrayHash );

class VEMSLogBuffer
{
    public:
        VEMSLogBuffer();
        ~VEMSLogBuffer();

        bool                loadLogFromFile(const wxString& filename, StringHash& headerHash, long& logversion);
        void                clearBuffer();

        size_t              addPacket(SIPRPacket packet);

        size_t              getBufferLength() { return m_siprpackets.GetCount(); }
        SIPRPacket*         getFirstPacket();
        SIPRPacket*         getNextPacket();
        SIPRPacket*         getPacket(); // get packet from current position without stepping to the next pos
        SIPRPacket*         getPacket( size_t idx ); // get packet from idx position without stepping to the next pos
//        SIPRPacket*         getPacket(unsigned long packet);
        bool                hasNext();
        long                getPosition();
        size_t              getHistoryCount(size_t packetType);
        SIPRPacket*         getPacketByType(size_t idx, size_t packetType = IDX_RTDATA_NORMAL);
        long                findRTidxByIdx(size_t id, size_t searchFrom = 0);
        long                getNextRTidxByIdx( int index );
        double              getPacketTime( size_t idx );

        long                getSTIndexByRTIdx( size_t idx );

        // eg 10th. RT => buffer[20];
        long                getIdxByDIdx( size_t idx, size_t packetType = IDX_RTDATA_NORMAL);

        void                setPosition(size_t position);
        void                getSTData(size_t rtIdx, wxObjectArray& stpackets);
        SIPRPacket*         getGPSData(size_t rtIdx, int* gpsIdx = NULL);

    protected:

    private:

        wxObjectArray       m_siprpackets;
        size_t              m_counter;
        size_t              m_index;
        IntToIntArrayHash   m_indexHash;

        // ST Datas
        IntToIntArrayHash   m_stTableIdxHash;
        IntToIntArrayHash   m_stCfgIdxHash;

        // RTData time idx
        DoubleArray         m_RTtimeIndex;

        // GPS data: RT idx -> GPS idx
        Int2IntHash         m_gpsIdxHash;

        IntArray            m_RealRTtime;
        size_t              m_refTime;


        /// for computing time
        uint8_t             m_prevvalue;
        uint8_t             m_blockLen;
        uint8_t             m_prevblockLen;
        uint8_t             m_blockNr; // only 0,1,2 is important

        void        convertRTTime( const SIPRPacket& p );

};

#endif // VEMSLOGBUFFER_H
