#ifndef VEMSCOMMAND_H
#define VEMSCOMMAND_H

#include "../common/typedef.h"
#include "../common/TxData.h"
#include <wx/list.h>

class VemsCommand
{
    public:
        virtual ~VemsCommand() {}
};

WX_DECLARE_LIST( VemsCommand, VemsCommandList ); //store pointers

class ReadPageCommand : public VemsCommand
{
    public:
        ReadPageCommand( int pageId )
        {
            this->pageId = pageId;
        }

        ReadPageCommand( int pageId, int pageSize )
        {
            this->pageId    = pageId;
            this->pageSize  = pageSize;
        }

        int pageId;
        int pageSize;
};

class ReadAllPageCommand : public VemsCommand
{
    public:
};

class WritePageCommand : public VemsCommand
{
    public:
        WritePageCommand( int pageId, int pageOffset, uint8_t value )
        {
            this->pageId        = pageId;
            this->pageOffset    = pageOffset;
            this->value         = value;
        }

        int     pageId;
        int     pageOffset;
        size_t  value;

};

class BurnCommand : public VemsCommand
{
    public:
//        BurnCommand() { setType( VemsCommand::BURN ); }
//        virtual ~BurnCommand() {}
};


class StringCommand : public VemsCommand
{
    public:
        StringCommand(const wxString& str, size_t sleepAfterWrite = 0, bool sendAck = false ) {
            m_str = str;
            this->sleepAfterWrite = sleepAfterWrite;
            this->sendAck = sendAck;
//        setType(VemsCommand::GENERIC_STRING);
        }
    wxString m_str;
    size_t  sleepAfterWrite;
    bool    sendAck;
};

class RawCommand : public VemsCommand
{
    public:
        RawCommand( const TxData& rawData )
        {
            this->rawData = rawData;
        }

        TxData rawData;
};
//class VemsCommand
//{
//    public:
//        /// if add new type don't forget to add to getTypeAsString()
//        enum VemsCommandType
//        {
//            UNKNOWN         = -1,
//
//            READ_PAGE       = 0,
//            READ_ALL_PAGE,
//            WRITE_PAGE,
//            BURN,
//            SET_BAUDRATE,
//            PING,
//            SET_STATE,
//            READ_REALTIME,
//            ENABLE_READ_REALTIME,
//            GET_COMPILE_TIME,
//            GET_FIRMWARE_VERSION,
//            GET_MT_SIGNATURE,
//            GET_HWSIG,
//            SYNC,
//
//            SD_LOG_START,
//            SD_LOG_STOP,
//            SD_LOG_ERASE,
//            SD_LOG_SAVE,
//            SD_LOG_SAVE_A_BLOCK,
//            SD_LOG_SAVE_N_BLOCKS,
//            SD_LOG_GET_STATUS,
//            SD_LOG_CANCEL,
//            SD_LOG_CHECK_EMPTY,     // check SD empty or not
//
//            DOWNLOAD_ALL,
//            DOWNLOAD_TABLES,
//            DOWNLOAD_CONFIG,
//            WBO2,
//            MEMORY_READ,
//            MEMORY_WRITE,
//            GENERIC_STRING
//
//        };
//    public:
//        VemsCommand()
//        {
//            m_iType                     = UNKNOWN;
//            m_iPageID                   = -1;
//            m_iPageOffset               = -1;
//            m_iPageSize                 = -1;    // for Megatune sync
//            /// for SD_LOG  0 - save from SD begin, > 0 - save last m_iBackTime minute from SD
//            m_iBackTime                 = -1;
//            m_iStartIndex               = -1;
//            m_iBaudRate                 = -1;
//            m_iState                    = -1;
//            m_uValue                    = 0;
//            m_bIsReadRealTimeEnabled    = false;
//        }
//
//        virtual ~VemsCommand() {}
//
//        int     getType() const                 { return m_iType; }
//        int     getPageID() const               { return m_iPageID; }
//        int     getPageOffset() const           { return m_iPageOffset; }
//        uint8_t getValue() const                { return m_uValue; }
//        int     getBackTime() const             { return m_iBackTime; }     // for SD_LOG
//        int     getStartIndex() const           { return m_iStartIndex; }   // for SD_LOG
//        int     getBaudRate() const             { return m_iBaudRate; }
//        int     getPageSize() const             { return m_iPageSize; }
//        int     getState() const                { return m_iState; }
//
//        bool    isReadRealTimeEnabled() const   { return m_bIsReadRealTimeEnabled; }
//
//        //bool    getReadRealTime() const         { return m_bReadRealTime; }
//        //int     getReadRealTimeDelay() const    { return m_iReadRealTimeDelay; }
//        wxString toString() const { return getTypeAsString( m_iType ); }
//        bool    getWBO2() const { return m_wbo2Enable; }
//
//        uint8_t count;
//        uint8_t delay;
//        uint8_t page;
//
//    protected:
//        void        setType( int type )             { m_iType = type; }
//        void        setPageID( int id )             { m_iPageID = id; }
//        void        setPageOffset( int offset )     { m_iPageOffset = offset; }
//        void        setValue( uint8_t value )       { m_uValue = value; }
//        void        setBackTime( int time )         { m_iBackTime = time; }     // for SD_LOG
//        void        setStartIndex( int idx )        { m_iStartIndex = idx; }    // for SD_LOG
//        void        setBaudRate( int baud )         { m_iBaudRate = baud; }
//        void        setPageSize( int size )         { m_iPageSize = size; }
//        void        enableReadRealTime( bool b )    { m_bIsReadRealTimeEnabled = b; }
//        void        setState( int state )           { m_iState = state; }
//        //void    setReadRealTime( bool read )    { m_bReadRealTime = read; }
//        //int     setReadRealTimeDelay( int d )   { m_iReadRealTimeDelay = d; }
//        void        setWBO2( bool enable )           { m_wbo2Enable = enable; }
//
//    private:
//        int         m_iType;
//        int         m_iPageID;
//        int         m_iPageOffset;
//        int         m_iPageSize;    // for Megatune sync
//        /// for SD_LOG  0 - save from SD begin, > 0 - save last m_iBackTime minute from SD
//        int         m_iBackTime;
//        int         m_iStartIndex;
//        int         m_iBaudRate;
//        int         m_iState;
//        uint8_t     m_uValue;
//        bool        m_bIsReadRealTimeEnabled;
//        bool        m_wbo2Enable;
//
//    private:
//        wxString    getTypeAsString( int type ) const;
//        //int         m_iReadRealTimeDelay; // if 0 read
//};
//
//WX_DECLARE_LIST( VemsCommand, VemsCommandList ); //store pointers
//
/////---------------------------------------------------------COMMON COMMANDS
//class ReadPageCommand : public VemsCommand
//{
//    public:
//        ReadPageCommand( int pageID )
//        {
//            setType( VemsCommand::READ_PAGE );
//            setPageID( pageID );
//        }
//
//        ReadPageCommand( int pageID, int pageSize )
//        {
//            setType( VemsCommand::READ_PAGE );
//            setPageID( pageID );
//            setPageSize( pageSize );
//        }
//
//        virtual ~ReadPageCommand() {}
//};
//
//class ReadAllPageCommand : public VemsCommand
//{
//    public:
//        ReadAllPageCommand() { setType( VemsCommand::READ_ALL_PAGE ); }
//        virtual ~ReadAllPageCommand() {}
//};
//
//class WritePageCommand : public VemsCommand
//{
//    public:
//        WritePageCommand( int pageID, int pageOffset, uint8_t value )
//        {
//            setType( VemsCommand::WRITE_PAGE );
//            setPageID( pageID );
//            setPageOffset( pageOffset );
//            setValue( value );
//        }
//
//        virtual ~WritePageCommand() {}
//
//};
//
//class BurnCommand : public VemsCommand
//{
//    public:
//        BurnCommand() { setType( VemsCommand::BURN ); }
//        virtual ~BurnCommand() {}
//};
//
//class SetBaudCommand : public VemsCommand
//{
//    public:
//        SetBaudCommand( int baudRate )
//        {
//            setType( VemsCommand::SET_BAUDRATE );
//            setBaudRate( baudRate );
//        }
//        virtual ~SetBaudCommand() {}
//};
//
//class PingCommand : public VemsCommand
//{
//    public:
//        PingCommand() { setType( VemsCommand::PING ); }
//        virtual ~PingCommand() {}
//};
//
//class SetStateCommand : public VemsCommand
//{
//    public:
//        SetStateCommand( int state )
//        {
//            setType( VemsCommand::SET_STATE );
//            setState( state );
//        }
//};
//
//class ReadRealTimeCommand : public VemsCommand
//{
//    public:
//        ReadRealTimeCommand( uint8_t count = 255, uint8_t delay = 0, uint8_t page = 0 )
//        {
//            this->count = count;
//            this->delay = delay;
//            this->page  = page;
//            setType( VemsCommand::READ_REALTIME );
//        }
//};
//
//class EnableReadRealTimeCommand : public VemsCommand
//{
//    public:
//        EnableReadRealTimeCommand( bool enable )
//        {
//            setType( VemsCommand::ENABLE_READ_REALTIME );
//            enableReadRealTime( enable );
//        }
//};
//
//class GetCompileTimeCommand : public VemsCommand
//{
//    public:
//        GetCompileTimeCommand()
//        {
//            setType( VemsCommand::GET_COMPILE_TIME );
//        }
//};
//
//class GetFirmwareVersionCommand : public VemsCommand
//{
//    public:
//        GetFirmwareVersionCommand()
//        {
//            setType( VemsCommand::GET_FIRMWARE_VERSION );
//        }
//};
//
//class GetMtsigCommand : public VemsCommand
//{
//    public:
//        GetMtsigCommand()
//        {
//            setType( VemsCommand::GET_MT_SIGNATURE );
//        }
//};
//
//class GetHwsigCommand : public VemsCommand
//{
//    public:
//        GetHwsigCommand()
//        {
//            setType( VemsCommand::GET_HWSIG );
//        }
//};
//
//class SyncCommand : public VemsCommand
//{
//    public:
//        SyncCommand()
//        {
//            setType( VemsCommand::SYNC );
//        }
//};
//
/////---------------------------------------------------------SD LOG COMMANDS
//class SDLogStartCommand : public VemsCommand
//{
//    public:
//        SDLogStartCommand() { setType( VemsCommand::SD_LOG_START ); }
//        virtual ~SDLogStartCommand() {}
//};
//
//class SDLogStopCommand : public VemsCommand
//{
//    public:
//        SDLogStopCommand() { setType( VemsCommand::SD_LOG_STOP ); }
//        virtual ~SDLogStopCommand() {}
//};
//
//class SDLogEraseCommand : public VemsCommand
//{
//    public:
//        SDLogEraseCommand() { setType( VemsCommand::SD_LOG_ERASE ); }
//        virtual ~SDLogEraseCommand() {}
//};
//
//class SDLogSaveCommand : public VemsCommand
//{
//    public:
//        SDLogSaveCommand( int backTime = 0 )
//        {
//            setType( VemsCommand::SD_LOG_SAVE );
//            setBackTime( backTime );
//        }
//        virtual ~SDLogSaveCommand() {}
//};
//
//class SDLogGetStatusCommand : public VemsCommand
//{
//    public:
//        SDLogGetStatusCommand() { setType( VemsCommand::SD_LOG_GET_STATUS ); }
//        virtual ~SDLogGetStatusCommand() {}
//};
//
//class SDLogSaveABlockCommand : public VemsCommand
//{
//    public:
//        SDLogSaveABlockCommand( int startIndex )
//        {
//            setType( VemsCommand::SD_LOG_SAVE_A_BLOCK );
//            setStartIndex( startIndex );
//        }
//        virtual ~SDLogSaveABlockCommand() {}
//};
//
//class SDLogSaveNBlocksCommand : public VemsCommand
//{
//    public:
//        SDLogSaveNBlocksCommand( int startIndex )
//        {
//            setType( VemsCommand::SD_LOG_SAVE_N_BLOCKS );
//            setStartIndex( startIndex );
//        }
//        virtual ~SDLogSaveNBlocksCommand() {}
//};
//
//class SDLogCancelCommand : public VemsCommand
//{
//    public:
//        SDLogCancelCommand() { setType( VemsCommand::SD_LOG_CANCEL ); }
//        virtual ~SDLogCancelCommand() {}
//};
//
//class SDLogCheckEmptyCommand : public VemsCommand
//{
//    public:
//        SDLogCheckEmptyCommand() { setType( VemsCommand::SD_LOG_CHECK_EMPTY ); }
//        virtual ~SDLogCheckEmptyCommand() {}
//};
//
//
/////---------------------------------------------------------DOWNLOAD COMMANDS
//class DownloadAllCommand : public VemsCommand
//{
//    public:
//        DownloadAllCommand() { setType( VemsCommand::DOWNLOAD_ALL ); }
//        virtual ~DownloadAllCommand() {}
//};
//
//class DownloadTablesCommand : public VemsCommand
//{
//    public:
//        DownloadTablesCommand() { setType( VemsCommand::DOWNLOAD_TABLES ); }
//        virtual ~DownloadTablesCommand() {}
//};
//
//class DownloadConfigCommand : public VemsCommand
//{
//    public:
//        DownloadConfigCommand() { setType( VemsCommand::DOWNLOAD_CONFIG ); }
//};
//
//class WBO2Command : public VemsCommand
//{
//    public:
//        WBO2Command(bool enable) {
//            setWBO2(enable);
//            setType( VemsCommand::WBO2 );
//        }
//
//    virtual ~WBO2Command() {}
//
//};
//
//class MemoryReadCommand : public VemsCommand
//{
//    public: MemoryReadCommand(size_t address, uint8_t length){
//        m_address = address;
//        m_length = length;
//        setType(VemsCommand::MEMORY_READ);
//    }
//    size_t m_address;
//    uint8_t m_length;
//};
//
//class MemoryWriteCommand : public VemsCommand
//{
//    public: MemoryWriteCommand(size_t address, uint8_t* data_ptr, uint8_t length){
//        m_address = address;
//        m_length = length;
//        setType(VemsCommand::MEMORY_WRITE);
//        memcpy(m_data_ptr,data_ptr,length);
//        }
//    size_t m_address;
//    uint8_t m_length;
//    uint8_t m_data_ptr[253];
//};
//
//class StringCommand : public VemsCommand
//{
//    public: StringCommand(const wxString& str){
//        m_str = str;
//        setType(VemsCommand::GENERIC_STRING);
//        }
//    wxString m_str;
//};
#endif // VEMSCOMMAND_H
