#include "VemsDeviceControllerBase.h"
#include "../CommunicationBase.h"
#include "../communication/CommunicationController2.h"
#include "../communication/CommunicationControllerEvent.h"
#include "../communication/CommunicationControllerClient.h"


VemsDeviceControllerBase::VemsDeviceControllerBase()
{
    m_eventHandler      = NULL;

    m_commController    = NULL;
    m_commBase          = NULL;

    m_commClient        = NULL;
    m_prevCommClient    = NULL;

    m_interWriteDelay   = 0;
    m_timeout           = 0;
    m_baseTimeout       = 0;
}

VemsDeviceControllerBase::~VemsDeviceControllerBase()
{
    //dtor
}

void VemsDeviceControllerBase::setInterWriteDelay( size_t interWriteDelay )
{
    m_interWriteDelay = interWriteDelay;
}

size_t VemsDeviceControllerBase::getInterWriteDelay() const
{
    return m_interWriteDelay;
}

void VemsDeviceControllerBase::setTimeout( size_t timeout )
{
    m_timeout = timeout;
}

void VemsDeviceControllerBase::setBaseTimeout( size_t timeout )
{
    m_baseTimeout = timeout;
}

size_t VemsDeviceControllerBase::getTimeout() const
{
    return m_timeout;
}

size_t VemsDeviceControllerBase::getBaseTimeout() const
{
    return m_baseTimeout;
}

const IntArray& VemsDeviceControllerBase::getSupportedBaudrates() const
{
    return m_supportedBaudrates;
}

ECUInfo::Device VemsDeviceControllerBase::getDevice() const
{
    return m_deviceInfo.device;
}

ECUInfo::EcuMode VemsDeviceControllerBase::getDeviceMode() const
{
    return m_deviceInfo.deviceMode;
}

wxString VemsDeviceControllerBase::getDeviceAsString() const
{
    return ECUInfo::getDeviceAsString( m_deviceInfo.device );
}

wxString VemsDeviceControllerBase::getDeviceModeAsString() const
{
    return ECUInfo::getStateAsString( m_deviceInfo.deviceMode );
}

const FirmwareInfo& VemsDeviceControllerBase::getFirmwareInfo() const
{
    return m_deviceInfo.firmwareInfo;
}

bool VemsDeviceControllerBase::isV3() const
{
    return m_deviceInfo.isV3();
}

bool VemsDeviceControllerBase::isRound() const
{
    return m_deviceInfo.isRound();
}

bool VemsDeviceControllerBase::isBootMode() const
{
    return m_deviceInfo.isBootMode();
}

bool VemsDeviceControllerBase::isManMode() const
{
    return m_deviceInfo.isManMode();
}

bool VemsDeviceControllerBase::isMegatuneMode() const
{
    return m_deviceInfo.isMegatuneMode();
}

void VemsDeviceControllerBase::initCommunication( CommunicationBase* commBase )
{
    m_commBase = commBase;

    m_commController = new CommunicationController2( this, m_commClient, m_commBase );
}

void VemsDeviceControllerBase::clearCommunication()
{
    if ( m_commController ) {
        m_commController->stop();
        delete m_commController;
        m_commController = NULL;
    }

    if ( m_commBase ) {
        m_commBase->close();
        delete m_commBase;
        m_commBase = NULL;
    }
}

void VemsDeviceControllerBase::registerClient( CommunicationControllerClient* client )
{
    delete m_commClient;
    m_commClient = client;

    ///TODO
    m_commClient->setBaseTimeout( m_baseTimeout );
    m_commClient->setTimeout( m_timeout );
    m_commClient->setInterWriteDelay( m_interWriteDelay );

    m_commController->registerClient( m_commClient );
}

void VemsDeviceControllerBase::startCommControllerNow()
{
    m_commController->start();
}

void VemsDeviceControllerBase::setEventHandler( wxEvtHandler* eventHandler )
{
    m_eventHandler = eventHandler;
}
