#ifndef VEMSDEVICECONTROLLERBASE_H
#define VEMSDEVICECONTROLLERBASE_H


#include "DeviceInfo.h"
#include "CommandRequestBase.h"
#include "../common/types/IntArray.h"


class FirmwareData;
class MMCFilesDesc;
class CommunicationController2;
class CommunicationBase;
class CommunicationControllerClient;


enum
{
    VDC_FLAG_DEFAULT    = 0,
    VDC_FLAG_NOEVENT    = 1,
    VDC_FLAG_AUTO_LEAVE_BOOT_MODE = 2,
    VDC_FLAG_NO_RECONNECT = 4,
    VDC_FLAG_GET_PAGES = 8,
};


class VemsDeviceControllerBase : public wxEvtHandler, public CommandRequestBase
{
    public:
        VemsDeviceControllerBase();
        virtual ~VemsDeviceControllerBase();

        ///void setController( DeviceController )

        DeviceInfo getDeviceInfo() const { return m_deviceInfo; }
        const FirmwareInfo& getFirmwareInfo() const;

        CommunicationControllerClient* getCommClient() { return m_commClient; }
        CommunicationBase* getCommBase() { return m_commBase; }
        const IntArray& getSupportedBaudrates() const;

        void setInterWriteDelay( size_t interWriteDelay );
        size_t getInterWriteDelay() const;

        void setTimeout( size_t timeout );
        void setBaseTimeout( size_t timeout );
        size_t getTimeout() const;
        size_t getBaseTimeout() const;

        void setEventHandler( wxEvtHandler* eventHandler );

        ECUInfo::Device     getDevice() const;
        ECUInfo::EcuMode    getDeviceMode() const;

        wxString getDeviceAsString() const;
        wxString getDeviceModeAsString() const;

        bool isV3() const;
        bool isRound() const;
        bool isBootMode() const;
        bool isManMode() const;
        bool isMegatuneMode() const;

        virtual void start( int flags = VDC_FLAG_DEFAULT ) = 0;
        virtual void stop( int flag = VDC_FLAG_DEFAULT ) = 0;
        virtual void resume( int flag = VDC_FLAG_DEFAULT ) = 0;
        virtual void timeout() {}

        virtual bool isConnected() const = 0;

        /// TODO - remove these methods

        /// ??? - remove these methods
        virtual void changeBaudRate( size_t baudrate ) = 0;
        void setCommBaudrate( size_t baudrate ) { m_deviceInfo.serialConfig.setBaudRate( baudrate); }

        SerialConfig getSerialConfig() const { return m_deviceInfo.serialConfig; }

        void initCommunication( CommunicationBase* commBase );
        void clearCommunication();
        void registerClient( CommunicationControllerClient* client );

        void startCommControllerNow();

    protected:
        DeviceInfo m_deviceInfo;

        CommunicationController2*       m_commController;
        CommunicationBase*              m_commBase;

        CommunicationControllerClient*  m_commClient;
        CommunicationControllerClient*  m_prevCommClient;

        IntArray                        m_supportedBaudrates;

        wxEvtHandler*                   m_eventHandler;

        size_t                          m_interWriteDelay;
        size_t                          m_timeout;
        size_t                          m_baseTimeout;

};

#endif // VEMSDEVICECONTROLLERBASE_H
