#include "VemsDeviceControllerStates.h"
#include "VemsDeviceControllerFSM.h"
#include "../logger.h"
#include "../communication/clients/PortScannerEvent.h"


///VemsDeviceControllerState--------------------------------------------------------------------------
VemsDeviceControllerState::VemsDeviceControllerState( VemsDeviceControllerFSM* fsm )
{
    m_fsm               = fsm;
}

/**
    Detecting
        OnDeviceFound

            ++m_connectTryCount;

    if timeout
            if ( m_connectTryCount >= MAX_CONNECT_TRY_COUNT )
                stop connecting
                show message
                m_connectTryCount = 0

    if connected
            m_connectTryCount = 0
*/

///Detecting--------------------------------------------------------------------------
/// TODO - OnCannotOpenPort
void Detecting::OnDeviceFound( PortScannerEvent& event )
{
    wxString logMsg = _T("device: ") + ECUInfo::getDeviceAsString( event.device );
    logMsg += _T(", deviceMode: ") + ECUInfo::getStateAsString(event.mode);
    logMsg += wxString::Format( _T(", baudrate: %d"), event.baudrate );

    LOGP( logMsg );

    LOGP( wxString::Format( _T("m_fsm: %p"), m_fsm ) );

    m_fsm->setDevice( event.device );
    m_fsm->setDeviceMode( event.mode );
    m_fsm->setBaudrate( event.baudrate );
    m_fsm->deviceFound();

    switch ( event.mode )
    {
        case ECUInfo::BOOT:
        {
            if ( m_fsm->isLeaveBootModeEnabled() ) {
                m_fsm->selectMegatuneMode();
            } else {
                m_fsm->bootModeFound();
                m_fsm->setState( new BootMode( m_fsm ) );
            }
        } break;

        case ECUInfo::MAN:
        {
            m_fsm->selectMegatuneMode();
        } break;

        case ECUInfo::MEGATUNE:
        {
            OnMegatuneMode();
        } break;

        default:
            LOGP( _T("UNHANDLED DEVICE_MODE: ") + ECUInfo::getStateAsString( event.mode ) );
    }
}

void Detecting::OnDeviceNotFound()
{
    m_fsm->deviceNotFound();
}

void Detecting::OnDeviceModeSelected( ECUInfo::EcuMode deviceMode )
{
    m_fsm->setDeviceMode( deviceMode );
    OnMegatuneMode();
}

void Detecting::OnBaudrateChanged( size_t baudrate )
{
    m_fsm->setBaudrate(baudrate);
    OnMegatuneMode();
}

void Detecting::OnMegatuneMode()
{
    if ( m_fsm->hasPreferredBaudrate() && m_fsm->isUseSerialComm() ) {
        m_fsm->changeToPreferredBaudrate();
    } else  if ( m_fsm->getDeviceInfo().isV3() ) {
        m_fsm->checkPassword();
        m_fsm->setState( new CheckingPassword( m_fsm ) );
    } else {
        m_fsm->fetchFirmwareInfo();
        m_fsm->setState( new FetchingFirmwareInfo( m_fsm ) );
    }
}


///BootMode--------------------------------------------------------------------------
void BootMode::OnDeviceModeSelected( ECUInfo::EcuMode deviceMode )
{
    ///m_controller->sendDeviceControllerEvent( VDC_EVENT_DEVICE_FOUND );
    m_fsm->setDeviceMode( deviceMode );
    m_fsm->fetchFirmwareInfo();
    m_fsm->setState( new FetchingFirmwareInfo( m_fsm ) );
}

///TODO
//void BootMode::OnTimeout()
//{
//            m_state = DETECTING;
//            m_controller->timeout();
//}


///--------------------------------------------------------------------------
/**
void VemsDeviceControllerFSM::checkingPassword( CommunicationClientEvent::Type eventType )
{
    switch ( eventType )
    {
        case DEVICE_PROTECTED:
            LOGP( _T("DEVICE_PROTECTED") );
            m_controller->sendDeviceControllerEvent( VDC_EVENT_DEVICE_PROTECTED );
            m_controller->stop( VDC_FLAG_NOEVENT );
            m_state = DETECTING;
            break;

        case DEVICE_NOT_PROTECTED:
            m_state = FETCHING_FIRMWARE_INFO;
            m_controller->sendDeviceControllerEvent( VDC_EVENT_DEVICE_NOT_PROTECTED );
            m_controller->fetchFirmwareInfo();
            break;

        case TIMEOUT:
            m_state = DETECTING;
//            m_controller->detect();
            break;

        default:
            break;
    }
}
*/
void CheckingPassword::OnDeviceProtected()
{
    ///TODO
//    m_fsm->setState( new ProtectedMode( m_fsm ) );
//    m_fsm->fetchFirmwareInfo();
    m_fsm->sendDeviceProtectedEvent();
}

void CheckingPassword::OnDeviceNotProtected()
{
    LOG
    m_fsm->fetchFirmwareInfo();
    m_fsm->setState( new FetchingFirmwareInfo( m_fsm ) );
}
///TODO timeout

///FetchingFirmwareInfo--------------------------------------------------------------------------
void FetchingFirmwareInfo::OnFirmwareInfoFetched(  const FirmwareInfo& fwInfo )
{
    m_fsm->setFirmwareInfo( fwInfo );
    m_fsm->setFirmwareVersion( fwInfo.getFirmwareVersion() );

    if ( m_fsm->isDeviceReconnected() ) {
        LOG
        m_fsm->getAllPages();
        //m_fsm->setState( new Connected( m_fsm ) );
    } else {
        LOG
        m_fsm->sync();
        m_fsm->setState( new Syncing( m_fsm ) );
    }
}

void FetchingFirmwareInfo::OnAllPageReceived()
{
    m_fsm->setState( new Connected( m_fsm ) );
    m_fsm->sendDeviceReconnectedEvent();
}

void FetchingFirmwareInfo::OnTimeout()
{
    m_fsm->timeout();
}


void Syncing::OnFirmwareDataSynced()
{
    m_fsm->connect();
    m_fsm->setState( new Connected( m_fsm ) );
}

void Syncing::OnTimeout()
{
    m_fsm->timeout();
}

///Connected--------------------------------------------------------------------------
void Connected::OnTimeout()
{
    m_fsm->timeout();
    //m_fsm->setState( new Detecting( m_fsm ) );
}

/// leave boot mode
void Connected::OnDeviceModeSelected( ECUInfo::EcuMode deviceMode )
{
    /// TODO
    m_fsm->setDeviceMode( ECUInfo::MEGATUNE );

    if ( m_fsm->hasPreferredBaudrate() ) {
        m_fsm->changeToPreferredBaudrate();
//    } else  if ( device == ECUInfo::DEVICE_V3 ) { /// TODO - round
    } else {
        m_fsm->checkPassword();
        m_fsm->setState( new Detecting( m_fsm ) );
    }
//    } else {
//        m_fsm->setState( new FetchingFirmwareInfo( m_fsm ) );
//        m_fsm->fetchFirmwareInfo();
//    }
}

void Connected::OnBaudrateChanged( size_t baudrate )
{
    m_fsm->setBaudrate(baudrate);
    m_fsm->baudrateChanged();
}



///Resuming--------------------------------------------------------------------------
///TODO - OnCannotOpenPort
///TODO - Timeout
void Resuming::OnDeviceNotFound()
{
    LOG
    m_fsm->timeout(); /// TODO
//    m_fsm->setState( new Detecting( m_fsm ) );
}
void Resuming::OnDeviceFound( PortScannerEvent& event )
{
    m_fsm->setDevice( event.device );
    m_fsm->setDeviceMode( event.mode );
    m_fsm->setBaudrate( event.baudrate );

    LOGP( event.toString() );

    if ( m_fsm->isDeviceResumed() ) {
        if ( m_fsm->isBaudrateResumed() ) {
            if ( m_getPages ) {
                m_fsm->getAllPages();
            } else {
                m_fsm->deviceResumed();
                m_fsm->setState( new Connected( m_fsm ) );
            }
        } else {
            m_fsm->resumeBaudrate();
        }
    } else {
        m_fsm->selectMegatuneMode();
    }
}

void Resuming::OnDeviceModeSelected( ECUInfo::EcuMode deviceMode )
{
    m_fsm->setDeviceMode( deviceMode );

    if ( m_fsm->isBaudrateResumed() ) {
        if ( m_getPages ) {
            m_fsm->getAllPages();
        } else {
            m_fsm->deviceResumed();
            m_fsm->setState( new Connected( m_fsm ) );
        }
    } else {
        m_fsm->resumeBaudrate();
    }
}

void Resuming::OnBaudrateChanged( size_t baudrate )
{
    m_fsm->setBaudrate(baudrate);

    if ( m_getPages ) {
        m_fsm->getAllPages();
    } else {
        m_fsm->deviceResumed();
        m_fsm->setState( new Connected( m_fsm ) );
    }
    //m_fsm->deviceResumed();
    //m_fsm->setState( new Detecting( m_fsm ) );
}

void Resuming::OnTimeout()
{
    m_fsm->timeout(); /// TODO
//    m_fsm->setState( new Detecting( m_fsm ) );
}

void Resuming::OnAllPageReceived()
{
    //m_fsm->deviceResumed();
    m_fsm->setState( new Connected( m_fsm ) );
    m_fsm->sendDeviceResumedEvent();
}
