#include "DevicesIcon.h"

#include "../common/ProcessBase.h"

#include "../logger.h"
#include "../DrawHelper.h"

#include "../events/VemsDeviceControllerEvent.h"
#include "../DirHelper.h"


BEGIN_EVENT_TABLE(DevicesIcon, VemsToolIconBase)

    EVT_LEFT_DOWN(DevicesIcon::OnLeftDown)
    EVT_LEFT_UP(DevicesIcon::OnLeftUp)

END_EVENT_TABLE()


DevicesIcon::DevicesIcon(wxWindow *parent, wxWindowID id,  const wxString& icondir, const wxPoint &pos,
    wxEvtHandler* eventHandler, ProcessBase* commCtrlState)
         :VemsToolIconBase(parent,id,icondir,pos,_T("state-white.png"))
{
    if(eventHandler){
        eventHandler->Connect(  VDC_EVENT_CANNOT_OPEN_PORT,
                                wxEVT_VEMS_DEVICE_CONTROLLER,
                                VemsDeviceControllerEventHandler( DevicesIcon::OnCannotOpenPort ),
                                NULL, this );

        eventHandler->Connect(   VDC_EVENT_DEVICE_NOT_FOUND,
                                wxEVT_VEMS_DEVICE_CONTROLLER,
                                VemsDeviceControllerEventHandler( DevicesIcon::OnDeviceNotFound ),
                                NULL, this );

        eventHandler->Connect(  VDC_EVENT_DEVICE_CONNECTED,
                                wxEVT_VEMS_DEVICE_CONTROLLER,
                                VemsDeviceControllerEventHandler( DevicesIcon::OnDeviceConnected ),
                                NULL, this );

        eventHandler->Connect(    VDC_EVENT_DEVICE_DISCONNECTED,
                                        wxEVT_VEMS_DEVICE_CONTROLLER,
                                        VemsDeviceControllerEventHandler( DevicesIcon::OnDeviceDisconnected ),
                                        NULL, this );

        eventHandler->Connect(  VDC_EVENT_STOPPED,
                                wxEVT_VEMS_DEVICE_CONTROLLER,
                                VemsDeviceControllerEventHandler( DevicesIcon::OnDeviceControllerStopped ),
                                NULL, this );

        eventHandler->Connect(  VDC_EVENT_RESUMED,
                                wxEVT_VEMS_DEVICE_CONTROLLER,
                                VemsDeviceControllerEventHandler( DevicesIcon::OnDeviceResumed ),
                                NULL, this );
    }

    m_commCtrlState = commCtrlState;

    m_upBmps[DI_NA_RED] = new wxBitmapLayer(0, icondir +  _T("state-red.png"),_T("N/A"));
    m_upBmps[DI_NA_WHITE] = new wxBitmapLayer(0, icondir +  _T("state-white.png"),_T("N/A"));

    m_upBmps[DI_ARM] = new wxBitmapLayer(0, icondir +  _T("state-green.png"),_T("ARM"));
    m_upBmps[DI_V3] = new wxBitmapLayer(0, icondir +  _T("state-green.png"),_T("V3"));
    m_upBmps[DI_ROUND] = new wxBitmapLayer(0, icondir +  _T("state-green.png"),_T("RND"));


    m_downBmps[DI_NA_RED] = new wxBitmapLayer(0, icondir +  _T("state-red-down.png"),_T("N/A"));
    m_downBmps[DI_NA_WHITE] = new wxBitmapLayer(0, icondir +  _T("state-white-down.png"),_T("N/A"));

    m_downBmps[DI_ARM] = new wxBitmapLayer(0, icondir +  _T("state-green-down.png"),_T("ARM"));
    m_downBmps[DI_V3] = new wxBitmapLayer(0, icondir +  _T("state-green-down.png"),_T("V3"));
    m_downBmps[DI_ROUND] = new wxBitmapLayer(0, icondir +  _T("state-green-down.png"),_T("RND"));


    wxBitmap tempbmp(GetSize().GetWidth(),GetSize().GetHeight());
    wxMemoryDC memdc;
    memdc.SelectObject(tempbmp);

    memdc.Clear();

    memdc.SetPen(wxColor(255,255,255));

    wxBitmapLayerHash::iterator it;
    for(it = m_upBmps.begin(); it != m_upBmps.end(); it++){
        InitBmpTinyChar(memdc,it);
    }

    for(it = m_downBmps.begin(); it != m_downBmps.end(); it++){
        InitBmpTinyChar(memdc,it,wxSize(1,1));
    }

}

DevicesIcon::~DevicesIcon()
{
}

void DevicesIcon::IconInit()
{

    IconInitBase(m_upBmps[DI_NA_RED]);
    m_state = DI_NA_RED;

}

void DevicesIcon::OnLeftDown(wxMouseEvent& evt)
{

    m_bmpLayers->AddBitmap(m_downBmps[m_state]);

    Refresh(false);


}

void DevicesIcon::OnLeftUp(wxMouseEvent& evt)
{

    m_bmpLayers->AddBitmap(m_upBmps[m_state]);

    Refresh(false);

    if(m_commCtrlState == NULL) return;

    bool state = m_commCtrlState->isRunning();

    if(state){
        m_commCtrlState->stop();
    }else{
        m_commCtrlState->start();
    }
}


void DevicesIcon::refreshDevice( ECUInfo::Device device )
{
    switch( device )
    {
        case ECUInfo::DEVICE_V3:
            refresh( DI_V3 );
            break;

        case ECUInfo::DEVICE_ROUND:
            refresh( DI_ROUND );
            break;

        case ECUInfo::DEVICE_ARM:
            refresh( DI_ARM );
            break;

        default:
            refresh( DI_NA_RED );
            break;
    }
}

void DevicesIcon::refresh( State state )
{
    m_state = state;
    m_bmpLayers->AddBitmap(m_upBmps[m_state]);
    Refresh(false);
}

void DevicesIcon::OnCannotOpenPort( VemsDeviceControllerEvent& event )
{
    refresh( DI_NA_WHITE );
}

void DevicesIcon::OnDeviceNotFound( VemsDeviceControllerEvent& event )
{
    refresh( DI_NA_RED );
}

void DevicesIcon::OnDeviceConnected( VemsDeviceControllerEvent& event )
{
    refreshDevice( event.getDevice() );
}

void DevicesIcon::OnDeviceDisconnected( VemsDeviceControllerEvent& event )
{
    refresh( DI_NA_RED );
}

void DevicesIcon::OnDeviceControllerStopped( VemsDeviceControllerEvent& event )
{
    refresh( DI_NA_WHITE );
}

void DevicesIcon::OnDeviceResumed( VemsDeviceControllerEvent& event )
{
    refreshDevice( event.getDevice() );
}
