#include "StateIcon.h"

#include "../VemsTuneController.h"

#include "../logger.h"

#include "../events/ECUEvent.h"
#include "../events/EcuCommEvent.h"

#include "../events/GPSEvent.h"
#include "../events/GPSCommEvent.h"
#include "../events/ECULogEvent.h"

#include "../DirHelper.h"

BEGIN_EVENT_TABLE(StateIcon, VemsToolIconBase)

    EVT_LEFT_DOWN(StateIcon::OnLeftDown)
    EVT_LEFT_UP(StateIcon::OnLeftUp)

END_EVENT_TABLE()

StateIcon::StateIcon(wxWindow *parent, wxWindowID id, const wxString& icondir, const wxString& name, const wxPoint &pos)
         :VemsToolIconBase(parent,id,icondir,pos,_T("state-white.png"))
{
    //ctor
    m_vemsTuneController = NULL;
    m_EcuState = ECUEvent::NOT_CONNECTED;
    m_EcuCommState = ECUCommEvent::OFFLINE;

    m_gpsState = GPSInfo::NOT_CONNECTED;
    m_gpsCommState = GPSInfo::ONLINE;
    m_state = SI_RED;

    m_stateName = name.Mid(name.Find(_T('.'), true)+1);

    Logger::logf(LOG_MSG,_T("StateIcon ctor name = ")+m_stateName);

    m_upBmps[SI_RED] = new wxBitmapLayer(0, icondir +  _T("state-red.png"),m_stateName);
    m_upBmps[SI_GREEN] = new wxBitmapLayer(0, icondir +  _T("state-green.png"),m_stateName);
    m_upBmps[SI_YELLOW] = new wxBitmapLayer(0, icondir +  _T("state-yellow.png"),m_stateName);
    m_upBmps[SI_WHITE] = new wxBitmapLayer(0, icondir +  _T("state-white.png"),m_stateName);

    m_downBmps[SI_RED] = new wxBitmapLayer(0, icondir +  _T("state-red-down.png"),m_stateName);
    m_downBmps[SI_GREEN] = new wxBitmapLayer(0, icondir +  _T("state-green-down.png"),m_stateName);
    m_downBmps[SI_YELLOW] = new wxBitmapLayer(0, icondir +  _T("state-yellow-down.png"),m_stateName);
    m_downBmps[SI_WHITE] = new wxBitmapLayer(0, icondir +  _T("state-white-down.png"),m_stateName);


    wxBitmap tempbmp(GetSize().GetWidth(),GetSize().GetHeight());
    wxMemoryDC memdc;
    memdc.SelectObject(tempbmp);

    memdc.Clear();

    memdc.SetPen(wxColor(255,255,255));


    wxBitmapLayerHash::iterator it;
    for(it = m_upBmps.begin(); it != m_upBmps.end(); it++){
        InitBmpTinyChar(memdc,it);
    }

    for(it = m_downBmps.begin(); it != m_downBmps.end(); it++){
        InitBmpTinyChar(memdc,it,wxSize(1,1));
    }




}

StateIcon::~StateIcon()
{
    //dtor
}



void StateIcon::IconInit()
{

    IconInitBase(m_upBmps[SI_RED]);
    m_state = SI_RED;

}

void StateIcon::SetVemsTuneController(VemsTuneController *ctrl)
{
    m_vemsTuneController = ctrl;

    if(m_stateName == _T("GPS")){

        m_vemsTuneController->Connect( wxEVT_GPS, GPSEventHandler( StateIcon::OnGPSEvent ),NULL, this);
        m_vemsTuneController->Connect( wxEVT_GPS_COMM, GPSCommEventHandler( StateIcon::OnGPSCommEvent ),NULL, this);

    }else if(m_stateName == _T("ECU") || m_stateName == _T("AIM") ){

        m_vemsTuneController->Connect( wxEVT_ECU, ECUEventHandler( StateIcon::OnECUEvent ),NULL, this);
        m_vemsTuneController->Connect( wxEVT_ECU_COMM, ECUCommEventHandler( StateIcon::OnECUCommState ),NULL, this);



    }else if(m_stateName == _T("LOG")){



    }

}


void StateIcon::OnECUEvent(ECUEvent& event)
{

    m_EcuState = event.state;
    showEcuState();
    event.Skip();

}

void StateIcon::OnECUCommState(ECUCommEvent& event)
{

    m_EcuCommState = event.state;
    showEcuState();
    event.Skip();

}

void StateIcon::showEcuState()
{

    if(m_EcuCommState == ECUCommEvent::OFFLINE){
        m_state = SI_WHITE;

    }else{
        if(m_EcuState == ECUEvent::CONNECTED){
            m_state = SI_GREEN;
        }
        else if(m_EcuState == ECUEvent::AIM_MODE){
            m_state = SI_GREEN;

        }else{
            m_state = SI_RED;

        }

    }


    m_bmpLayers->AddBitmap(m_upBmps[m_state]);
    Refresh(false);

}


void StateIcon::OnGPSEvent( GPSEvent& event )
{

    //Logger::logf( LOG_MSG, _T("TbTbGPS::OnGPSEvent() - state : %i"), event.state );
    m_gpsState = event.state;
    showGPSState();
    event.Skip();

}

void StateIcon::OnGPSCommEvent( GPSCommEvent& event )
{

    m_gpsCommState = event.state;
    showGPSState();
    event.Skip();

}

void StateIcon::showGPSState()
{

    if( m_gpsCommState == GPSInfo::OFFLINE )
    {
        m_state = SI_WHITE;
    }
    else
    {
        switch( m_gpsState )
        {
            case GPSInfo::CONNECTED:
                m_state = SI_GREEN;
                break;

            case GPSInfo::NO_GPS_DATA:
             case GPSInfo::DATA_RECEIVED:
                m_state = SI_YELLOW;
                break;

            case GPSInfo::NOT_CONNECTED:
                m_state = SI_RED;
                break;
        }
    }


    m_bmpLayers->AddBitmap(m_upBmps[m_state]);

    Refresh(false);

}


void StateIcon::OnLeftDown(wxMouseEvent& evt)
{

    m_bmpLayers->AddBitmap(m_downBmps[m_state]);

    Refresh(false);

}

void StateIcon::OnLeftUp(wxMouseEvent& evt)
{

    m_bmpLayers->AddBitmap(m_upBmps[m_state]);

    Refresh(false);

    if(m_vemsTuneController == NULL) return;

    bool state;

    if(m_stateName == _T("GPS")){

        state = m_vemsTuneController->isGPSOnline();
        m_vemsTuneController->setGPSOnline( !state );

    }else if(m_stateName == _T("ECU")){

        state = m_vemsTuneController->isECUOnline();
        m_vemsTuneController->setECUOnline( !state );

    }



}

