#include "WIPAccessories.h"

#include <wx/wx.h>
#include "../DirHelper.h"
#include "../logger.h"
#include "WebHelper.h"
#include "FileUploaderProviderBase.h"
#include <wx/url.h>
#include <wx/file.h>
#include <wx/zipstrm.h>
#include <wx/wfstream.h>
#include <memory>


/****************************************************
                WIPDesc
*/

bool WIPDesc::IsOk(bool createLocalPath) const
{

    if( LocalPath.Len() > 0 && !(wxFileName::DirExists(DirHelper::GetPath(dWEBCACHE) + LocalPath)) ){
        wxFileName::Mkdir( DirHelper::GetPath(dWEBCACHE) + LocalPath );
    }

    if(
           Title.Len() >= 0
        && BaseServer.Len() > 4
      //  && Handler.Len() > 2 //when Handler is empty switch on the no-php based dir listing, parsing
        && LocalPath.Len() > 0
        && RemotePathTypeList.size() > 0
        && wxFileName::DirExists(DirHelper::GetPath(dWEBCACHE) + LocalPath)
        )
        return true;
    else
        return false;
}

/****************************************************
              DataTransferMgr
*/


bool DataTransferMgr::DownloadFile(uint16_t idx)
{

//    DownloadThread * dt = new DownloadThread( m_remoteArray[idx], DirHelper::GetPath(dWEBCACHE) + m_localArray[idx] );
//
//    if ( dt->Create() != wxTHREAD_NO_ERROR ){
//        return false;
//    }
//
//    dt->registerObserver( this );
//
//    dt->Run();

    DownloadProcess* dp = new DownloadProcess(m_remoteArray[idx], DirHelper::GetPath(dWEBCACHE) + m_localArray[idx], true);
    dp->registerObserver( this );
    dp->start();

    return true;
}

bool DataTransferMgr::UploadFile(uint16_t idx)
{
    //Logger::logf(LOG_MSG,_T("DataTransferMgr::UploadFile()"));

    if(!m_uploaderProvider) return false;

    wxString localfile = DirHelper::GetPath(dWEBCACHE) +  m_localArray[idx];
    wxString remotepath = m_remoteArray[idx];

    //Logger::logf(LOG_MSG,_T("remote=")+ remotepath + _T(", local=")+ localfile );

    //return m_uploaderProvider->UploadFile(  m_linkArray[idx], DirHelper::GetPath(dWEBCACHE) + m_destArray[idx] );

    m_uploaderProvider->registerObserver( this );

    //return m_uploaderProvider->UploadFile( localfile, remotepath );

    wxVariantHash hash;
    hash[_T("file")] = localfile;

    return m_uploaderProvider->UploadFiles( hash, remotepath );

    //return true;
}

//void DataTransferMgr::AddItem(const wxString& link, const wxString& dest)
void DataTransferMgr::AddItem(const wxString& remote, const wxString& local)
{
    m_remoteArray.Add( remote );
    m_localArray.Add( local );
}

void DataTransferMgr::Clear()
{
    m_remoteArray.Clear();
    m_localArray.Clear();
}

bool DataTransferMgr::Start(int direction)
{
    Logger::logf(LOG_MSG,_T("dir=%i"),direction);

    m_transferedSize = 0;
    m_fullSize = 0;

    for(uint16_t a=0;a<m_remoteArray.GetCount();a++) {
        //Logger::logf(LOG_MSG,_T("rem=") + m_remoteArray[a] + _T(", loc=") + m_localArray[a] );

        if( direction == WIPDesc::WIP_DOWNLOAD )
            DownloadFile( a );

        if( direction == WIPDesc::WIP_UPLOAD )
            UploadFile( a );

    }
    return true;
}

void DataTransferMgr::notify( int i )
{
}

void DataTransferMgr::notify(const wxString& str)
{
    observerNotify(str);
}

void DataTransferMgr::notify(int i, int idx)
{
    if(idx == WEB_FILESIZE ) {
        m_fullSize += i;

    } else if(idx == WEB_ENDSESSION ) {

        m_remoteArray.RemoveAt( i );
        m_localArray.RemoveAt( i );

        if( m_remoteArray.GetCount() == 0 )
            observerNotify( WEB_ENDSESSION );

    } else if(idx == WEB_TRANSMITTEDDATASIZE ) {

        m_transferedSize += i;

        observerNotify( m_transferedSize, m_fullSize );

    }

    //Logger::logf(LOG_MSG,_T("DownloadMgr fullsize=%i"),m_fullSize);
}


/****************************************************
                DownloadProcess
*/

DownloadProcess::DownloadProcess(const wxString& link, const wxString& dest, bool usetmpfile)
{
    m_link = link;
    m_dest = dest;
    idx = 0;
    m_isUseTmpFile = usetmpfile;
    m_shouldNotify = true;
}

DownloadProcess::~DownloadProcess()
{
    // dtor
}

void DownloadProcess::EndedDownload()
{
    if ( m_shouldNotify == false )
        return;

    if(m_isUseTmpFile)
        wxRemoveFile(m_tmpFileName);

    observerNotify( idx, WEB_ENDSESSION );
    observerNotify( m_dest );

    m_shouldNotify = false;

}

void DownloadProcess::finished()
{
    EndedDownload();
}

bool DownloadProcess::process()
{

    wxURL url( m_link );

    char *data;

    Logger::logf(LOG_MSG,_T("void* DownloadProcess process link=")+m_link);

    if (url.GetError() == wxURL_NOERR)
    {
        wxInputStream *ins;

        ins = url.GetInputStream();

        if(!ins) {
            EndedDownload();
            return false;
        }

        int fsize = ins->GetSize();

        wxFile file;

        m_tmpFileName = m_dest + (m_isUseTmpFile ? _T(".tmp") : _T(""));


        if( !file.Open( m_tmpFileName ,wxFile::write) ){
            EndedDownload();
            return false;
        }

        int lr=0;

        size_t bufsize = 1024;

        data = new char[bufsize+4];

        observerNotify( fsize, WEB_FILESIZE );

        while(1){

            ins->Read(data,bufsize);
            lr = ins->LastRead();

            if(ins->LastRead() < 1) break;

            file.Write(data,lr);

            observerNotify( lr, WEB_TRANSMITTEDDATASIZE );

        }

        delete[] data;
        //Logger::logf(LOG_MSG,_T("")+tmp);

        file.Close();

        if(m_isUseTmpFile) {
            wxRenameFile(m_tmpFileName,m_dest,true);
        }

        EndedDownload();

    } else {

        EndedDownload();

    }

    return false;

}


/****************************************************
                DownloadThread
*/

DownloadThread::DownloadThread(const wxString& link, const wxString& dest)
{
    m_link = link;
    m_dest = dest;
    idx = 0;
}

DownloadThread::~DownloadThread()
{
    // dtor
}

void* DownloadThread::Entry()
{

    wxURL url( m_link );

    char *data;

    Logger::logf(LOG_MSG,_T("void* DownloadThread::Entry() link=")+m_link);


    if (url.GetError() == wxURL_NOERR)
    {
        wxInputStream *ins;

        ins = url.GetInputStream();

        if(!ins) {
            observerNotify( idx, WEB_ENDSESSION );
            return 0;
        }

        int fsize = ins->GetSize();

        wxFile file;

        if( !file.Open( m_dest ,wxFile::write) ){
            observerNotify( idx, WEB_ENDSESSION );
            return NULL;
        }

        int lr=0;
        data = new char[1024];

        observerNotify( fsize, WEB_FILESIZE );

        while(1){

            ins->Read(data,1024);
            lr = ins->LastRead();

            if(ins->LastRead() < 1) break;

            file.Write(data,lr);

            observerNotify( lr, WEB_TRANSMITTEDDATASIZE );

        }

        delete[] data;
        //Logger::logf(LOG_MSG,_T("")+tmp);

        file.Close();

        observerNotify( idx, WEB_ENDSESSION );

        Exit();

    } else {
        observerNotify( idx, WEB_ENDSESSION );
    }

    return 0;

}


wxString FirmwarePackChecker::GetStatus(const wxString &item, WIPDesc * wd)
{
    if( !(wd->IsOk()) ) return _T("-error-");

    wxString packfile = DirHelper::GetPath(dWEBCACHE) + wd->LocalPath + DirHelper::Separ() + item;

    if( !wxFileName::FileExists( packfile ) ) return _T("file not found");

    //Logger::logf(LOG_MSG,_T("packfile=")+packfile);

    //return _T("gizmo");
    wxString fwpath;

    std::auto_ptr<wxZipEntry> entry;

    wxFFileInputStream in( packfile );

    wxZipInputStream zip(in);

    wxZipEntry* ze = zip.GetNextEntry();

    if(!ze) return _T("Zip archive error");

    entry.reset(ze);

    fwpath = entry->GetName();


    bool res = wxFileName::DirExists( DirHelper::GetPath(dPROJECTS) + fwpath );


    if( res )
        return _T("Installed");
    else
        return _T("Not installed");

}

short FirmwarePackChecker::StatusChecker(const wxString &statusstring)
{
    if(statusstring == _T("Not installed"))
        return 0;
    if(statusstring == _T("Installed"))
        return 1;
    if(statusstring == _T("-error-"))
        return -1;

    return -1;
}

int FirmwareInstallService::DoInstall(const wxString &filename, WIPDesc * wd)
{
    if( !(wd->IsOk()) ) return -1;

    wxString packfile = DirHelper::GetPath(dWEBCACHE) + wd->LocalPath + DirHelper::Separ() + filename;

    std::auto_ptr<wxZipEntry> entry;

    wxFFileInputStream in( packfile );
    wxZipInputStream zip(in);

    char *data;
    unsigned long fs = 0;

    wxString outpath = DirHelper::GetPath(dPROJECTS);

    while (entry.reset(zip.GetNextEntry()), entry.get() != NULL)
    {
        // access meta-data
        wxString name = entry->GetName();
        fs = entry->GetSize();

        //Logger::logf(LOG_MSG,_T("s=%i,name=")+name,fs);
        // read 'zip' to access the entry's data

        if(entry->IsDir()) {
            wxFileName::Mkdir(outpath + name);
            continue;
        }

        wxFile file;

        if( !file.Open( outpath + name ,wxFile::write) )
            continue;

        data = new char[fs];

        zip.Read(data,fs);

        //Logger::logf(LOG_MSG,_T("size=%i"),fs);

        file.Write(data,fs);

        file.Close();

        delete[] data;

    }

    return 1;

}


