#ifndef WIPACCESSORIES_H
#define WIPACCESSORIES_H

#include <wx/wx.h>
#include "../ObserverBase.h"
#include "../Observable.h"
#include "../common/types/StringHash.h"
#include "../common/types/Int2StringHash.h"
#include "WebHelper.h"
#include "../common/ProcessBase.h"
#include <wx/filename.h>
#include <wx/thread.h>


//class WebInterfacePanel;
class ItemStatusCheckerBase;
class PackageInstallServiceBase;
class FileUploaderProviderBase;

class WIPDesc
{
    public:
        enum {
            WIP_DOWNLOAD = 1,
            WIP_UPLOAD = 2,
            WIP_BOTH = 3
        };

        WIPDesc(const wxString& title = _T("")) {
            Title = title;
            FileFilter = _T("*");
            LocalChecker = NULL;
            TransferDirection = WIP_DOWNLOAD;
            UploaderProvider = NULL;
        };


        wxString GetServer() { return BaseServer + _T("/") + Handler; };
        bool IsOk(bool createLocalPath = true) const;

        wxString        Title;
        wxString        BaseServer;
        wxString        Handler;
        wxString        LocalPath; ///relative!

        StringHash      RemotePathTypeList;  ///key:title, value: remote path type

        wxString        FileFilter;

        wxString        PopupMenuStr;
        Int2StringHash  PopupMenuStrings;


        ItemStatusCheckerBase * LocalChecker;
        PackageInstallServiceBase * LocalInstaller;

        FileUploaderProviderBase  * UploaderProvider;


        int             TransferDirection;
        long             Timeout;

};


class DataTransferMgr: public ObserverBase, public Observable
{
    public:

        void AddItem(const wxString& remote, const wxString& local);

        void Clear();

        bool Start(int direction = WIPDesc::WIP_DOWNLOAD);

        virtual void notify( int i );
        virtual void notify(int i, int idx);
        virtual void notify(const wxString& str);
        virtual void Init(const wxString& name, const wxString& msg, const int max) {};

        void SetUploadProvider(FileUploaderProviderBase * prov) { m_uploaderProvider = prov; };

    protected:

        bool DownloadFile(uint16_t idx);
        bool UploadFile(uint16_t idx);

        wxArrayString   m_remoteArray;
        wxArrayString   m_localArray;

        long            m_fullSize;
        long            m_transferedSize;

        FileUploaderProviderBase  * m_uploaderProvider;

};

class DownloadProcess: public ProcessBase, public Observable
{
    public:
        DownloadProcess(const wxString& link, const wxString& dest, bool usetmpfile = false);
        virtual ~DownloadProcess();

        int idx;

        virtual bool process();
        virtual void reset(){};

        wxString GetDestination() { return m_dest; };

    private:

        bool m_shouldNotify;
        void EndedDownload();

        virtual void finished();

        wxString m_link;
        wxString m_dest;
        wxString m_tmpFileName;

        bool    m_isUseTmpFile;

};


class DownloadThread: public wxThread, public Observable
{
    public:
        DownloadThread(const wxString& link, const wxString& dest);
        virtual ~DownloadThread();
        virtual void* Entry();

        int idx;

    private:

        wxString m_link;
        wxString m_dest;

};

class ItemStatusCheckerBase
{
    public:
        ItemStatusCheckerBase() {};

        virtual wxString GetStatus(const wxString &item, WIPDesc * wd) = 0;
        virtual short StatusChecker(const wxString &statusstring) = 0;

};

class FirmwarePackChecker: public ItemStatusCheckerBase
{
    public:
        FirmwarePackChecker() {};

        virtual wxString GetStatus(const wxString &item, WIPDesc * wd);
        virtual short StatusChecker(const wxString &statusstring);

};

class PackageInstallServiceBase
{
    public:
        virtual int DoInstall(const wxString &filename, WIPDesc * wd) = 0;
};

class FirmwareInstallService: public PackageInstallServiceBase
{
    public:
        virtual int DoInstall(const wxString &filename, WIPDesc * wd);

};

#endif // WIPACCESSORIES_H
