#ifdef WXACTIVEX_HTML_RENDERER

#include "wxWebBrowserActiveX.h"

#include "../logger.h"

/**
wxActiveX property list
    http://msdn.microsoft.com/en-us/library/aa768398%28v=VS.85%29.aspx

wxActiveX event list
    http://msdn.microsoft.com/en-us/library/aa768400%28v=VS.85%29.aspx
*/

BEGIN_EVENT_TABLE(wxWebBrowserActiveX,wxWebBrowserBase)

    EVT_ACTIVEX(idWebCtrl, "TitleChange", wxWebBrowserActiveX::OnTitleChange)
    EVT_ACTIVEX(idWebCtrl, "BeforeNavigate2", wxWebBrowserActiveX::OnBeforeNavigate2)
    EVT_ACTIVEX(idWebCtrl, "NavigateComplete2", wxWebBrowserActiveX::OnNavigateComplete2)

END_EVENT_TABLE()

wxWebBrowserActiveX::wxWebBrowserActiveX(wxWindow* parent)
    :wxWebBrowserBase(parent)
{
    //ctor
    m_webCtrl = NULL;

    m_webCtrl = new wxWebControl(this,idWebCtrl);
    SetHome(_T("about:blank"));

    BuildMainGUI(m_webCtrl);
}

wxWebBrowserActiveX::~wxWebBrowserActiveX()
{
    //dtor
}

/** wxActiveXEvent: fetching received params

    for(size_t a=0;a<evt.ParamCount();a++) {
        wxVariant param = evt[a];
        Logger::logf(LOG_MSG,_T("param[%i]=")+param.MakeString(),a);
    }
*/

void wxWebBrowserActiveX::OnNavigateComplete2(wxActiveXEvent& evt)
{
    if(evt.ParamCount() < 2) return;

    Logger::logf(LOG_MSG,_T("OnNavigateComplete2"));

    AddHistory(m_webCtrl->GetPropAsString(_T("LocationURL")));

    ContentLoaded(evt[1].MakeString());
}

void wxWebBrowserActiveX::OnBeforeNavigate2(wxActiveXEvent& evt)
{
    if(!m_webCtrl) return;

    if(evt.ParamCount() < 5) return;

    Logger::logf(LOG_MSG,_T("OnBeforeNavigate2"));

    wxString url = m_webCtrl->GetPropAsString(_T("LocationURL"));

    SetLoadingContentURL(evt[1].MakeString());

    SetLoadingURL(url);
}

void wxWebBrowserActiveX::OnTitleChange(wxActiveXEvent& evt)
{
    if(evt.ParamCount()<1) return;

    SetTitle( evt[0].MakeString() );
}

bool wxWebBrowserActiveX::LoadURL(const wxString& url)
{
    wxString nurl = GetRealURL(url);

    m_webCtrl->LoadUrl(nurl);

    SetFocusToWeb();

    return true;
}

#endif
