#include "IACControl.h"

#include "../vemsprotocol/CommandRequestBase.h"
#include "../vemsprotocol/VemsCommand.h"


IACHistory::IACHistory()
{
    m_nextIdx = 0;
    m_sequence = 0;
    m_timer.Start();
}

int IACHistory::getNextIdx(int idx)
{
    if(idx == 3){
        idx = 0;
        return idx;
    }
    idx++;
    return idx;
}

int IACHistory::getPrevIdx(int idx)
{
    if(idx == 0){
        idx = 3;
        return idx;
    }
    idx--;
    return idx;
}

void IACHistory::Add(int rpm)
{
    m_rpmBuff[m_nextIdx] = rpm;
    m_timeBuff[m_nextIdx] =  m_timer.Time();

    m_nextIdx = getNextIdx(m_nextIdx);
    m_sequence++;
}

int IACHistory::getDelta()
{
    if(m_sequence < 4) return 0;

    // delta [t] [t-2]; hope more precise timediff;
    int currIdx = getPrevIdx(m_nextIdx);
    int prevIdx = getPrevIdx( getPrevIdx(currIdx)); //

    int currRPM = m_rpmBuff[currIdx];
    int prevRPM = m_rpmBuff[prevIdx];

    int currTime = m_timeBuff[currIdx];
    int prevTime = m_timeBuff[prevIdx];

    if(currTime == prevTime){
        // wxDateTime::GetTimeNow() resolution is around 16ms so it could happen
        return 0;
    }
    double dt = (currTime - prevTime);
    //Logger::logf(LOG_MSG,_T("IACControl::getDelta() dt: %f"), dt);
    double drpm = currRPM - prevRPM;
    int delta = (int)(1000* drpm / dt);

    return delta; // unit: 1/min/s
}



//////////////////
IACControl::IACControl(RTDataMgrBase* mgr, CommandRequestBase* requestMgr, bool doregister)
{
    //ctor

    m_rpm = _T("rpm");
    m_tps = _T("throttle");
    m_iacPos = _T("iacPos");
    m_map =_T("map");

    m_isSymbolChanged = true;
    m_symbolA.Add(m_rpm);
    m_symbolA.Add(m_tps);
    m_symbolA.Add(m_iacPos);

    m_targetRPM = 1000;
    m_prevRPM = -1;
    m_integral = 0.01;
    m_D     = 0.005;
    m_remained = 0.0;

    SetDataMgr(mgr);

    if(doregister) {
        RegisterViewer();
        require();
    }

    m_requestMgr = requestMgr;

    Logger::logf(LOG_MSG,_T("IACControl() ctor END"));
}

IACControl::~IACControl()
{
    //dtor
    unregisterSymbols();
    UnRegisterViewer();
}

void IACControl::Start()
{
    RegisterViewer();
    require();
}

void IACControl::Stop()
{
    unregisterSymbols();
    UnRegisterViewer();
}

void IACControl::SetValueIdx(int idx)
{
/*
    Logger::logf(LOG_MSG,_T("VemsWidgetBase::SetValueIdx recived index: %i"), idx );

    m_defGaugeId = choices2gaugeDescA.Item(idx);

    Logger::logf(LOG_MSG, _T("VemsWidgetBase m_defGaugeId: %i"), m_defGaugeId );


    SetGaugeBySymbolName( m_symbolDescArr[idx]->Name );


    requireChanged();

    UpdateSize();
	Refresh(false);
*/
}

void IACControl::require()
{
    if( !m_isSymbolChanged ) return;

    for( unsigned int i= 0; i < m_symbolA.GetCount(); ++i){
        m_rtDataMgr->registerSymbol( m_symbolA.Item(i) );
    }
    m_isSymbolChanged = false;

}

void IACControl::unregisterSymbols()
{
    for( unsigned int i= 0; i < m_symbolA.GetCount(); ++i){
        m_rtDataMgr->unregisterSymbol( m_symbolA.Item(i) );
    }

}

void IACControl::SetIntegral(double i)
{
    m_integral = i;
}

void IACControl::SetDifferential(double d)
{
    m_D = d;
}

void IACControl::refresh(DoubleHash* table)
{
     //Logger::logf(LOG_MSG,_T("IACControl::refresh() START"));
    //Logger::logf(LOG_MSG,_T("m_currSymbolName=")+GetCurrSymbolName()+_T(", itemid=%i"),GetItemID() );

    for( unsigned int i= 0; i < m_symbolA.GetCount(); ++i){
        if(table->find(m_symbolA.Item(i)) == table->end()){
            Logger::logf(LOG_MSG,_T("symbol not found: ") + m_symbolA.Item(i) );
            // symbol not found
            return;
        }
    }

    int m_rpmV = (int)( (*table)[m_rpm]);
    double m_tpsV = (*table)[m_tps];
    double m_mapV = (*table)[m_map];
    int m_iacPosV = (int)((*table)[m_iacPos]);

    m_iacHistory.Add(m_rpmV);

    //Saver rpmSaver((int)m_rpmV, &m_prevRPM); // save rpm to prevRPM at return;

    if(m_rpmV < 300){
        Logger::logf(LOG_MSG,_T("IACControl::refresh() no rpm"));
        return;
    }

    if(m_tpsV > 1.0){
        // not in IDLE mode
         Logger::logf(LOG_MSG,_T("IACControl::refresh() not ide mode"));
        return;
    }

    int diffRPM = m_rpmV - m_targetRPM;
    int dRpmPerDt = m_iacHistory.getDelta(); // dRPM/dt unit: 1/min/sec

     if( (diffRPM > 0) && (m_mapV < 33.0) ){
        // seems engine break
        return;
     }

 //   Logger::logf(LOG_MSG,_T("IACControl::refresh() drpmPerdt: %i"), dRpmPerDt );
    if(diffRPM > 0) diffRPM /= 2;
    double iPart = m_integral * (double)diffRPM;
    if(dRpmPerDt > 0) dRpmPerDt /= 2;
    double dPart =  m_D * (double)dRpmPerDt;

    int adj =  (int)(iPart + dPart + m_remained);
    m_remained = iPart+dPart+m_remained - adj;
    //if(dPart > 0) dPart = 0;
    //int adj =  iPart + dPart;
    if( adj < -2 ) adj = -2;
    if( adj > 1 ) adj = 1;
    Logger::logf(LOG_MSG,_T("IACControl::refresh() diff:%i, drpm/dt:%i, i:%.2f d:%.2f r:%.4f"), diffRPM, dRpmPerDt, iPart, dPart, m_remained );
    if( adj == 0 ) return;

    int currIACValue = m_iacPosV - adj;
    if( currIACValue == 0) return; // 0 is disable IAC control


    Logger::logf(LOG_MSG,_T("IACControl::refresh()IAC: %i"), currIACValue);
    commandRequest(currIACValue);
/*
    if( diffRPM > 100 ){
        if(m_mapV < 30.0){
            // RPM is high, but map is low = ENGINE break consider check pulsewidth as well;
            return;
        }

        // decrease -1
        Logger::logf(LOG_MSG,_T("IACControl::refresh()HI %i"), currIACValue);
        commandRequest(currIACValue);
    }

    if( diffRPM < -50 ){
        // increase +1
        Logger::logf(LOG_MSG,_T("IACControl::refresh()LO %i"), currIACValue);
        commandRequest(currIACValue);
    }
    */
    // do nothing
}

void IACControl::commandRequest(int value)
{
    if( (value < 0) || (value > 255) ) return;
    wxString command = wxString::Format(_T("Manmdi%02Xmbye"), value);

    //Logger::logf( LOG_MSG, _T("IACLockPanel::commandRequest(): ")+ command);

    VemsCommand* vemsCommand = new StringCommand(command);
    m_requestMgr->request(vemsCommand);

}
