#include "IgnitionLockPanel.h"

#include "../logger.h"

#include "../vemsprotocol/VemsCommand.h"
#include "../vemsprotocol/CommandRequestBase.h"


BEGIN_EVENT_TABLE( IgnitionLockPanel, wxPanel )
    EVT_TEXT_ENTER(wxID_ANY, IgnitionLockPanel::OnEnter)
    EVT_KILL_FOCUS( IgnitionLockPanel::OnKillFocus )
    EVT_KEY_DOWN(IgnitionLockPanel::OnKeyDown)
    EVT_BUTTON( wxID_APPLY, IgnitionLockPanel::OnEnter )
    EVT_BUTTON( wxID_CLOSE, IgnitionLockPanel::OnCloseButton )
END_EVENT_TABLE()

IgnitionLockPanel::IgnitionLockPanel(wxWindow * parent, CommandRequestBase* requestMgr)
{
    //ctor

    m_requestMgr = requestMgr;
    buildGUI(parent);
}

IgnitionLockPanel::~IgnitionLockPanel()
{
    //dtor

    ignitionRequest(0);
}


void IgnitionLockPanel::buildGUI(wxWindow* parent)
{
    Create(parent, wxID_ANY);

    SetBackgroundColour( wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE ) );
    ClearBackground();

    wxBoxSizer* m_mainsizer = new wxBoxSizer( wxVERTICAL );
    //m_mainsizer  = new wxBoxSizer( wxVERTICAL );

    wxStaticText* text2 = new wxStaticText( this, -1,
    _T("Ignition Lock Dialog\n\n To activate set 0-63 deg\n To disable set 0 or close") );
    //text2->SetFont( font );

    m_mainsizer->Add( text2, 0, wxALL | wxEXPAND, 5 );


    wxFlexGridSizer* grid = new wxFlexGridSizer( 3, 1, 5 );
    m_mainsizer->Add( grid, 1, wxEXPAND | wxALL, 0 );

    wxStaticText* textName= new wxStaticText( this, -1, _T("Ignition advance:"));
    grid->Add( textName, 0, wxALL | wxEXPAND, 5 );

    m_textctrl = new wxTextCtrl(this, -1, _T(""), wxDefaultPosition,
    wxDefaultSize, wxTE_PROCESS_ENTER | wxTE_BESTWRAP );
    grid->Add( m_textctrl, 0, wxALL | wxEXPAND, 5 );

    wxStaticText* textUnit = new wxStaticText( this, -1, _T("Deg."));
    grid->Add( textUnit, 0, wxALL | wxEXPAND, 5 );

    wxButton* sendButton = new wxButton( this, wxID_APPLY , _T("Send") );
    wxButton* closeButton = new wxButton( this, wxID_CLOSE , _T("Close") );

    grid->AddGrowableRow( 1 );
    grid->AddStretchSpacer();
    grid->AddStretchSpacer();
    grid->AddStretchSpacer();

    grid->Add( sendButton, 0, wxALL, 5 );
    grid->Add( closeButton, 0, wxALL, 5 );

    SetSizer( m_mainsizer );
    m_mainsizer->Fit( this );
    m_mainsizer->SetSizeHints( this );
}


void IgnitionLockPanel::OnEnter( wxCommandEvent& event )
{
    Logger::logf( LOG_MSG, _T("IgnitionLockPanel::OnEnter()") );
    wxString valueStr = m_textctrl->GetValue();
    double dValue;
    bool result = valueStr.ToDouble(&dValue);
    if(!result) return;

    dValue *= 4;    /// deg * 4;
    int binaryValue = (int)dValue;
    Logger::logf( LOG_MSG, _T("IgnitionLockPanel::OnEnter() spark*4=%d"), binaryValue);
    ignitionRequest(binaryValue);
}

/// O means desable ing lock
void IgnitionLockPanel::ignitionRequest(int ignition)
{
    if( (ignition < 0) || (ignition > 255) ) return;
    wxString command = wxString::Format(_T("Manmda%02Xmbye"), ignition);

    Logger::logf( LOG_MSG, _T("IgnitionLockPanel::OnEnter() ign. lock command: ")+ command);

    VemsCommand* vemsCommand = new StringCommand(command);
    m_requestMgr->request(vemsCommand);

}

void IgnitionLockPanel::OnKillFocus( wxFocusEvent& event )
{
     Logger::logf( LOG_MSG, _T("IgnitionLockPanel::OnKillFocus()") );
}

void IgnitionLockPanel::OnKeyDown( wxKeyEvent& event )
{
    int key = event.GetKeyCode();

    if ( key == WXK_TAB )
    {
        if ( event.ShiftDown() )
        {

            return;
        }

        return;
    }

    event.Skip();
}

void IgnitionLockPanel::OnCloseButton( wxCommandEvent& event )
{
    wxWindow* parent = GetParent();

    if ( parent ) {
        parent->Destroy();
    }
}
