#include "TachometerLockPanel.h"

#include "../logger.h"

#include "../vemsprotocol/VemsCommand.h"
#include "../vemsprotocol/CommandRequestBase.h"


BEGIN_EVENT_TABLE( TachometerLockPanel, wxPanel )
    EVT_TEXT_ENTER(wxID_ANY, TachometerLockPanel::OnEnter)
    EVT_KILL_FOCUS( TachometerLockPanel::OnKillFocus )
    EVT_KEY_DOWN(TachometerLockPanel::OnKeyDown)
    EVT_BUTTON( wxID_APPLY, TachometerLockPanel::OnEnter )
    EVT_BUTTON( wxID_CLOSE, TachometerLockPanel::OnCloseButton )
END_EVENT_TABLE()

static int CompareStringNum(const wxString& first, const wxString& second)
{
    double a=0,b=0;

    first.ToDouble(&a);
    second.ToDouble(&b);

    return a < b ? -1 : 1;
}

TachometerLockPanel::TachometerLockPanel(wxWindow * parent, CommandRequestBase* requestMgr)
{
    //ctor

    m_requestMgr = requestMgr;
    buildGUI(parent);
}

TachometerLockPanel::~TachometerLockPanel()
{
    //dtor

    commandRequest(-1);
}


void TachometerLockPanel::buildGUI(wxWindow* parent)
{
    Create(parent, wxID_ANY);

    SetBackgroundColour( wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE ) );
    ClearBackground();

    wxBoxSizer* m_mainsizer = new wxBoxSizer( wxVERTICAL );
    //m_mainsizer  = new wxBoxSizer( wxVERTICAL );

    wxString msg = _T("Tachometer Lock Dialog\n\n To activate set 0-3900\n To disable set -1 or close");
    wxStaticText* text2 = new wxStaticText( this, -1, msg);

    //wxString  warningMsg = _T("\nmax value limited by MaxSteps To Extend");
    //wxString  warningMsg = _T("The following settings could disable Tachometer lock functionality:\n\n");

    //warningMsg += _T("Menu: Outputs/Tach output\n\n");

//    wxStaticText* warningST = new wxStaticText( this, -1, warningMsg);
//    warningST->SetForegroundColour(wxColour(255,0,0) );
    //text2->SetFont( font );

    m_mainsizer->Add( text2, 0, wxALL | wxEXPAND, 5 );


    wxFlexGridSizer* grid = new wxFlexGridSizer( 3, 1, 5 );
    m_mainsizer->Add( grid, 0, wxEXPAND | wxALL, 0 );

    wxStaticText* textName= new wxStaticText( this, -1, _T("Tachometer frequency:"));
    grid->Add( textName, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5 );

    wxArrayString values;

    for(int a = 1; a <= 256; a++) {
        int freq=3900/a;
        if (freq>15){
            wxString val = wxString::Format(_T("%i"), freq );
            if(values.Index(val) == wxNOT_FOUND )
                values.Add(val);
        }
    }

    values.Sort(CompareStringNum);

    wxString firstval = values[0];

    values.Insert(_T("-1"),0);

    m_values = new wxComboBox(this, wxID_ANY, firstval, wxDefaultPosition, wxDefaultSize, values, wxCB_READONLY | wxTE_PROCESS_ENTER);

    //m_textctrl = new wxTextCtrl(this, -1, _T(""), wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER | wxTE_BESTWRAP );
    grid->Add( m_values, 0, wxALL | wxEXPAND, 5 );

    wxStaticText* textUnit = new wxStaticText( this, -1, _T(""));
    grid->Add( textUnit, 0, wxALL | wxEXPAND, 5 );

//    m_mainsizer->Add( warningST, 0, wxALL | wxEXPAND, 5 );


    wxButton* sendButton = new wxButton( this, wxID_APPLY , _T("Send") );
    wxButton* closeButton = new wxButton( this, wxID_CLOSE , _T("Close") );

    wxBoxSizer* buttonSizer = new wxBoxSizer( wxHORIZONTAL );

    buttonSizer->Add( sendButton, 0, wxALL, 5 );
    buttonSizer->AddStretchSpacer();
    buttonSizer->Add( closeButton, 0, wxALL, 5 );

    m_mainsizer->Add( buttonSizer, 1, wxALL | wxEXPAND | wxALIGN_BOTTOM , 5 );


    SetSizer( m_mainsizer );
    m_mainsizer->Fit( this );
    m_mainsizer->SetSizeHints( this );
}


void TachometerLockPanel::OnEnter( wxCommandEvent& event )
{
    Logger::logf( LOG_MSG, _T("TachometerLockPanel::OnEnter()") );
    wxString valueStr = m_values->GetValue();

    long val;
    bool result = valueStr.ToLong(&val);
    if(!result) return;

    commandRequest(val);
}

/// -1 means desable tacho lock
void TachometerLockPanel::commandRequest(int freq)
{
    uint8_t datacmd[3];

    if( (freq < -1) || (freq > 3900) ) return;

    //wxString command = wxString::Format(_T("Manmdi%02Xbye"), freq);
    int val;
    if (freq) val = 3900 / freq -1;
    else val=255;

   // wxString command = wxString::Format(_T("X%02X%02X"), freq == -1 ? 1 : 0 , val );
    datacmd[0]='X';
    datacmd[1]=freq == -1 ? 0 : 1;
    datacmd[2]=val;
    Logger::logf( LOG_MSG, _T("TachometerLockPanel::commandRequest() tacho. val=%d"),val);
    TxData command( datacmd, 3);
    VemsCommand* vemsCommand = new RawCommand(command);
    m_requestMgr->request(vemsCommand);
}

void TachometerLockPanel::OnKillFocus( wxFocusEvent& event )
{
     Logger::logf( LOG_MSG, _T("TachometerLockPanel::OnKillFocus()") );
}

void TachometerLockPanel::OnKeyDown( wxKeyEvent& event )
{
    int key = event.GetKeyCode();

    if ( key == WXK_TAB )
    {
        if ( event.ShiftDown() )
        {

            return;
        }

        return;
    }

    event.Skip();
}

void TachometerLockPanel::OnCloseButton( wxCommandEvent& event )
{
    wxWindow* parent = GetParent();

    if ( parent ) {
        parent->Destroy();
    }
}

