#include "wxRectEx.h"

#include <wx/arrimpl.cpp>
#include "common/types/IntArray.h"

WX_DEFINE_OBJARRAY(wxRectExArray);

wxRectEx::~wxRectEx()
{
    //dtor
}


void wxRectEx::Init(int xx, int yy, int ww, int hh)
{
    x = xx;
    y = yy;
    width = ww;
    height = hh;
}

/** @brief operator!=
  *
  * @todo: document this function
  */
bool wxRectEx::operator!=(const wxRectEx& rect) const
{
    if( (*this) == rect )
        return false;
    else
        return true;
}

/** @brief operator==
  *
  * @todo: document this function
  */
bool wxRectEx::operator==(const wxRectEx& rect) const
{
    if(  x - rect.x == 0 &&
         y - rect.y == 0 &&
         width - rect.width == 0 &&
         height - rect.height == 0   )
        return true;
    else
        return false;
}

/** @brief operator=
  *
  * @todo: document this function
  */
void wxRectEx::operator=(const wxRectEx& rect)
{
    x = rect.x;
    y = rect.y;
    width = rect.width;
    height = rect.height;

}

void wxRectEx::operator=(const wxRect& rect)
{
    x = rect.x;
    y = rect.y;
    width = rect.width;
    height = rect.height;

}

wxRectEx& wxRectEx::operator+=(const wxRectEx& rect)
{
    SetLeft( wxMin(x, rect.x) );
    SetTop( wxMin(y, rect.y) );

    SetRight( wxMax( GetRight(), rect.GetRight() ) );
    SetBottom( wxMax( GetBottom(), rect.GetBottom() ) );

    return *this;
}

wxRectEx wxRectEx::operator+(const wxRectEx& rect) const
{
    wxRectEx r;

    r.SetLeft( wxMin(x, rect.x) );
    r.SetTop( wxMin(y, rect.y) );

    r.SetRight( wxMax( GetRight(), rect.GetRight() ) );
    r.SetBottom( wxMax( GetBottom(), rect.GetBottom() ) );

    return r;
}


wxRectExArray wxRectEx::Subtract(const wxRectEx& rect) const
{
    wxRectExArray rects;
    rects.Add(rect);
    return Subtract(rects);
}


int int_sort_func(int * a, int * b)
{
    return (*a) > (*b) ? 1 : ( (*b) > (*a) ? -1 : 0 );
}

wxRectExArray wxRectEx::Subtract(const wxRectExArray& rects, bool removeNullRects) const
{
    wxRectExArray result;

    IntArray slice_x;
    IntArray slice_y;

    for(size_t a=0; a<rects.GetCount(); a++){

        wxRectEx& crect = rects[a];

        if( !Intersects(crect) ) continue;

        slice_x.Add( crect.GetLeft() );
        slice_y.Add( crect.GetTop() );
        slice_x.Add( crect.GetRight() );
        slice_y.Add( crect.GetBottom() );
    }

    if( !slice_x.GetCount() || !slice_y.GetCount() ) {
        result.Add(*this);
        return result;
    }

    slice_x.Add( GetLeft() );
    slice_y.Add( GetTop() );
    slice_x.Add( GetRight() );
    slice_y.Add( GetBottom() );

    slice_x.Sort(int_sort_func);
    slice_y.Sort(int_sort_func);

    for(size_t sy=0; sy<slice_y.GetCount()-1; sy++){

        int py = slice_y[sy];
        int ny = slice_y[sy+1];

        for(size_t sx=0; sx<slice_x.GetCount()-1; sx++){

            int px = slice_x[sx];
            int nx = slice_x[sx+1];

            wxRectEx resrect( wxPoint(px,py), wxPoint(nx,ny) );

            if( (!resrect.width || !resrect.height) && removeNullRects ) continue;

            bool isFound = false;

            for(size_t a=0; a<rects.GetCount(); a++){
                if( rects[a].Contains( resrect ) ) { isFound = true; break; }
            }

            if(isFound) continue;

            result.Add(resrect);
        }
    }

    return result;
}

wxString wxRectEx::GetAsString(){

    return wxString::Format(_T("x=%i, y=%i, w=%i, h=%i"),x,y,width,height);
}

wxString wxRectEx::GetAsStringBorders()
{
    return wxString::Format(_T("top=%i, left=%i, bottom=%i, right=%i"),GetTop(),GetLeft(),GetBottom(),GetRight());
}
