#include "wxSystemHelper.h"

#include "logger.h"

#include <wx/jsonreader.h>

wxString wxSystemHelper::GetSysPropString(int id)
{
    switch(id) {
        case SYSPROP_OSNAME: return _T("OsName");
        case SYSPROP_OSVER: return _T("OsVer");
        case SYSPROP_OSMANUFACTURER: return _T("OsManufacturer");
        case SYSPROP_INSTALLDATE: return _T("Istalled");
        case SYSPROP_UPTIME: return _T("Uptime");
        case SYSPROP_TYPE: return _T("SystemType");
        case SYSPROP_CPUS: return _T("CPUs");
        case SYSPROP_BIOS: return _T("Bios");
        case SYSPROP_TOTAL_PHYSICAL_MEMORY: return _T("TotalPhysicalMemory");
        default: return _T("Unkown system property");
    }
}

void wxSystemHelper::LaunchDeviceManager()
{
#if defined __WXMSW__
    wxShell(_T("devmgmt.msc"));
#endif
}

void wxSystemHelper::GetSystemSummary(Int2StringHash& data)
{
#if defined __WXMSW__
    StringHash mswdata;
    GetSystemSummaryMSW(mswdata);
    data[SYSPROP_OSNAME] = mswdata[_T("OS Name")];
    data[SYSPROP_OSVER] = mswdata[_T("OS Version")];
    data[SYSPROP_OSMANUFACTURER] = mswdata[_T("OS Manufacturer")];
    data[SYSPROP_INSTALLDATE] = mswdata[_T("Original Install Date")];
    data[SYSPROP_UPTIME] = mswdata[_T("System Up Time")];
    data[SYSPROP_TYPE] = mswdata[_T("System type")];
    data[SYSPROP_CPUS] = mswdata[_T("Processor(s)")];
    data[SYSPROP_BIOS] = mswdata[_T("BIOS Version")];
    data[SYSPROP_TOTAL_PHYSICAL_MEMORY] = mswdata[_T("Total Physical Memory")];
#endif
}


size_t wxSystemHelper::GetMemoryUsage(long processid)
{
    long procid = processid < 1 ? wxGetProcessId() : processid;

#if defined __WXMSW__
    PROCESS_MEMORY_COUNTERS pmc;
    GetMemoryInfoMSW(procid,pmc);

    return (size_t)pmc.WorkingSetSize;
#endif

///under linux: http://linux.die.net/man/2/getrusage


    return 0;
}

#if defined __WXMSW__
void wxSystemHelper::GetSystemSummaryMSW(StringHash& data)
{
    wxArrayString lines;

    wxExecute(_T("systeminfo /fo csv"),lines);

    //Logger::logf(LOG_MSG,_T("lines c=%i"),lines.GetCount());

    if(lines.GetCount() < 3) return;

    wxJSONReader reader;
    wxJSONValue header,line_data;

    reader.Parse(_T("[")+lines[1]+_T("]"),&header);
    reader.Parse(_T("[")+lines[2]+_T("]"),&line_data);

    const wxJSONInternalArray * header_arr = header.AsArray();
    const wxJSONInternalArray * data_arr = line_data.AsArray();

    //Logger::logf(LOG_MSG,_T("header_arr=%p data_arr=%p"),header_arr,data_arr);

    if(!header_arr || !data_arr) return;
    //Logger::logf(LOG_MSG,_T("c1=%i,c2=%i"),header_arr->GetCount(),data_arr->GetCount());
    if(header_arr->GetCount() != data_arr->GetCount()) return;

    for(size_t a=0;a<header_arr->GetCount();a++) {
        wxString key = header_arr->Item(a).AsString();
        wxString value = data_arr->Item(a).AsString();
        data[key] = value;
    }

    //Logger::logf(LOG_MSG,_T("dc=%i"),data.size());
}


bool wxSystemHelper::GetMemoryInfoMSW( DWORD processID, PROCESS_MEMORY_COUNTERS& pmc )
{
    HANDLE hProcess;

    hProcess = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, processID );

    if(hProcess == NULL)
        return false;

    bool res = GetProcessMemoryInfo( hProcess, &pmc, sizeof(pmc));

    CloseHandle( hProcess );

    return res;
}

#endif
